﻿/**
 * Copyright (C) 2003-2018, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit Mobile PDF SDK to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */


#include "pch.h"
#include "MainPage.xaml.h"
#include "Common\VisualTreeHelperEx.h"
#include "StartPage.xaml.h"
#include <Robuffer.h>
#include <wrl.h>
#include <sstream>  

using namespace CompletePDFViewer;
using namespace foxit::rdk;
using namespace foxit::rdk::uiextensions;
using namespace Microsoft::WRL;

using namespace Platform;
using namespace Windows::Foundation;
using namespace Windows::Foundation::Collections;
using namespace Windows::UI::Xaml;
using namespace Windows::UI::Xaml::Controls;
using namespace Windows::UI::Xaml::Controls::Primitives;
using namespace Windows::UI::Xaml::Data;
using namespace Windows::UI::Xaml::Input;
using namespace Windows::UI::Xaml::Media;
using namespace Windows::UI::Xaml::Navigation;
using namespace Windows::Storage;

MainPage::MainPage()
{
	InitializeComponent();

	OnInitializeControl();
}

void MainPage::OnInitializeControl()
{
	m_pViewCtrl = PDFViewControl;

	m_pViewCtrl->OpenPdfFileEvent += ref new foxit::rdk::OpenDocEvent(this, &CompletePDFViewer::MainPage::OnOpenPdfFileEvent);
	m_pViewCtrl->ClosePdfFileEvent += ref new foxit::rdk::CloseDocEvent(this, &CompletePDFViewer::MainPage::OnClosePdfFileEvent);
	
	m_uiextensions = ref new foxit::rdk::uiextensions::UIExtensionsManager(m_pViewCtrl);
	m_pViewCtrl->ExtensionsManager = m_uiextensions;
	
	m_uiextensions->SetFlyoutPlacement(FlyoutGird);
	
	this->Loaded += ref new Windows::UI::Xaml::RoutedEventHandler(this, &CompletePDFViewer::MainPage::OnLoaded);
	this->Unloaded += ref new Windows::UI::Xaml::RoutedEventHandler(this, &CompletePDFViewer::MainPage::OnUnloaded);
}

void MainPage::OpenPdfFile(Windows::Storage::StorageFile^ pFile)
{
	if (pFile)
	{
		auto pDocument = m_pViewCtrl->GetDoc();
		if (pDocument == nullptr || !pFile->IsEqual(m_pViewCtrl->GetStorageFile()))
		{
			m_pPDFFile = pFile;
			m_pViewCtrl->CloseDoc();
			m_pViewCtrl->OpenDocAsync(pFile, L"");
		}
	}
}

void CompletePDFViewer::MainPage::OnLoaded(Platform::Object ^sender, Windows::UI::Xaml::RoutedEventArgs ^e)
{
	OpenPdfFile(m_pPDFFile);
}


void CompletePDFViewer::MainPage::OnUnloaded(Platform::Object ^sender, Windows::UI::Xaml::RoutedEventArgs ^e)
{
	
}

void MainPage::OnNavigatedTo(Windows::UI::Xaml::Navigation::NavigationEventArgs^ e)
{
	m_pPDFFile = safe_cast<StorageFile^>(e->Parameter);

	auto _DataTransferManager = Windows::ApplicationModel::DataTransfer::DataTransferManager::GetForCurrentView();
	m_DataRequestedEventToken = _DataTransferManager->DataRequested += ref new Windows::Foundation::TypedEventHandler<Windows::ApplicationModel::DataTransfer::DataTransferManager ^, Windows::ApplicationModel::DataTransfer::DataRequestedEventArgs ^>(this, &CompletePDFViewer::MainPage::OnDataRequested);
}

void MainPage::OnNavigatedFrom(NavigationEventArgs^ e)
{
	auto _DataTransferManager = Windows::ApplicationModel::DataTransfer::DataTransferManager::GetForCurrentView();
	if (m_DataRequestedEventToken.Value)
	{
		_DataTransferManager->DataRequested -= m_DataRequestedEventToken;
		m_DataRequestedEventToken.Value = 0;
	}
}

void CompletePDFViewer::MainPage::OpenFileButton_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{
	auto _FileOpenPicker = ref new Windows::Storage::Pickers::FileOpenPicker();
	_FileOpenPicker->ViewMode = Windows::Storage::Pickers::PickerViewMode::List;
	_FileOpenPicker->SuggestedStartLocation = Windows::Storage::Pickers::PickerLocationId::DocumentsLibrary;
	_FileOpenPicker->FileTypeFilter->Append(".pdf");

	//用户选择单个文件
	concurrency::create_task(_FileOpenPicker->PickSingleFileAsync())
		.then([this](Windows::Storage::StorageFile^ openedFile)
	{
		if (openedFile == nullptr)
			throw ref new Platform::NullReferenceException(L"File is null.");
		
		return m_pViewCtrl->OpenDocAsync(openedFile, L"");
	})
		.then([=](concurrency::task<foxit::common::ErrorCode> t)
	{
		try
		{
			t.get();
		}
		catch (...)
		{
			
		}
	});
}

void CompletePDFViewer::MainPage::SplitViewToggle_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{
	PageSplitView->IsPaneOpen = !PageSplitView->IsPaneOpen;

	//AppSample::PrintVisualTree(HostContent);
}

void CompletePDFViewer::MainPage::OnOpenPdfFileEvent(PDFViewCtrl ^sender, DocEventArgs^ e)
{
	if (e->ErroCode == foxit::common::ErrorCode::e_ErrSuccess)
	{
		OutputDebugString((L"OnOpenPdfFileEvent: " + m_pViewCtrl->GetStorageFile()->Path + L"\n")->Data());
	}
}

static Windows::UI::Xaml::Media::Imaging::WriteableBitmap^ GetThumbImageSync(foxit::pdf::PDFDoc^ doc, int pageIndex, int width, int height, int nRotate)
{
	Windows::UI::Xaml::Media::Imaging::WriteableBitmap^ bitmap = nullptr;

	auto page = doc->GetPage(pageIndex);
	if (page == nullptr)
		return bitmap;

	Windows::Foundation::Size PageSize((float)width, (float)height);
	concurrency::cancellation_token_source pCts;
	int _bitmapwidth = (int)(width * 1.0);
	int _bitmapheight = (int)(height * 1.0);

	foxit::common::Bitmap^ m_bitmap = ref new foxit::common::Bitmap(_bitmapwidth, _bitmapheight, foxit::common::DIBFormat::e_DIBRgb32, nullptr, 0);
	if (m_bitmap == nullptr)
		return bitmap;

	//Set rect area and fill the color of bitmap.
	foxit::common::fxcrt::RectI^ rect = ref new foxit::common::fxcrt::RectI(0, 0, _bitmapwidth, _bitmapheight);
	m_bitmap->FillRect(0xffffffff, rect);

	//Get the page's matrix.
	foxit::common::fxcrt::Matrix2D^ mt = page->GetDisplayMatrix(0, 0, _bitmapwidth, _bitmapheight, (foxit::common::Rotation)nRotate);

	float pageWidth = page->GetWidth(), pageHeight = page->GetHeight();
	int index = page->GetIndex();

	//Create a renderer based on a given bitmap, and page will be rendered to this bitmap.
	foxit::common::Renderer^ renderer = ref new foxit::common::Renderer(m_bitmap, false);
	renderer->SetRenderContentFlags((unsigned int)(foxit::common::ContentFlag::e_RenderAnnot | foxit::common::ContentFlag::e_RenderPage));
	BOOL bIsParsed = page->IsParsed();
	if (!bIsParsed)
	{
		page->StartParse(0, nullptr, false);
	}
	renderer->StartRender(page, mt, nullptr);

	auto buffer = m_bitmap->GetBuffer();
	Platform::Object^ _obj = buffer;
	/*ComPtr<IInspectable> _insp(reinterpret_cast<IInspectable*>(_obj));
	ComPtr<Windows::Storage::Streams::IBufferByteAccess> _bufferByteAccess;
	_insp.As(&_bufferByteAccess);*/
	byte* pixels = nullptr;
	//_bufferByteAccess->Buffer(&pixels);
	void* lpBmpBuf = buffer->Data;

	width = m_bitmap->GetWidth();
	height = m_bitmap->GetHeight();
	unsigned int stride = m_bitmap->GetPitch();
	unsigned int size = stride * height;
	bitmap = ref new Windows::UI::Xaml::Media::Imaging::WriteableBitmap(width, height);
	_obj = bitmap->PixelBuffer;
	ComPtr<IInspectable> _insp2(reinterpret_cast<IInspectable*>(_obj));
	ComPtr<Windows::Storage::Streams::IBufferByteAccess> _bufferByteAccess2;
	_insp2.As(&_bufferByteAccess2);
	_bufferByteAccess2->Buffer(&pixels);
	memcpy(pixels, lpBmpBuf, size);

	return bitmap;
}

void CompletePDFViewer::MainPage::OnClosePdfFileEvent(PDFViewCtrl ^sender, DocEventArgs^ e)
{
	OutputDebugString((L"OnClosePdfFileEvent: " + m_pViewCtrl->GetStorageFile()->Path + L"\n")->Data());

	auto doc = m_pViewCtrl->GetDoc();
	if (doc && doc->GetEncryptionType() == foxit::pdf::EncryptType::e_EncryptNone && doc->GetPageCount() > 0)
	{
		auto pStorgeFile = m_pViewCtrl->GetStorageFile();
		auto nPageIndex = m_pViewCtrl->GetCurrentPage();
		auto _GeneralSetting = CompletePDFViewer::GeneralSetting::Current;
		auto image = GetThumbImageSync(doc, nPageIndex, (int)_GeneralSetting->MRUThumbnailWidth, (int)_GeneralSetting->MRUThumbnailHeight, 0);
		if (image)
		{
			concurrency::create_task(CompletePDFViewer::MRUManager::SaveMRUItemThumbnailToFileByStorageFile(pStorgeFile, image))
				.then([=](concurrency::task<void> t)
			{
				try
				{
					t.get();
					//LogHelper::LogFileOut("RequestAppTirggerAsync: ");
				}
				catch (...)
				{
					//LogHelper::LogFileOut("RequestAppTirggerAsync Failed ");
				}
			});
		}
	}

	if (m_pPDFFile == nullptr || m_pViewCtrl->GetStorageFile()->Equals(m_pPDFFile))
	{
		this->Frame->Navigate(Windows::UI::Xaml::Interop::TypeName(StartPage::typeid), nullptr);
	}
}

void CompletePDFViewer::MainPage::OnDataRequested(Windows::ApplicationModel::DataTransfer::DataTransferManager ^sender, Windows::ApplicationModel::DataTransfer::DataRequestedEventArgs ^args)
{
	
}
