/**
 * Copyright (C) 2003-2018, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit Mobile PDF SDK to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */


#include "pch.h"
#include "RecentlyFileItem.h"
#include "MRUManager.h"
#include "GeneralSetting.h"

using namespace CompletePDFViewer;

using namespace Platform;
using namespace concurrency;
using namespace Windows::Storage;
using namespace Windows::Storage::Streams;
using namespace Windows::Storage::Pickers;
using namespace Windows::Storage::AccessCache;
using namespace Windows::Storage::FileProperties;
using namespace Windows::ApplicationModel::Resources;
using namespace Windows::ApplicationModel::Resources::Core;

using namespace Windows::Foundation::Collections;
using namespace Windows::UI::Xaml;
using namespace Windows::UI::Xaml::Controls;
using namespace Windows::UI::Xaml::Controls::Primitives;
using namespace Windows::UI::Xaml::Data;
using namespace Windows::UI::Xaml::Input;
using namespace Windows::UI::Xaml::Media;
using namespace Windows::UI::Xaml::Navigation;
using namespace Windows::UI::Core;
using namespace Windows::System;

using namespace Windows::UI::Xaml::Interop;
using namespace Windows::ApplicationModel::DataTransfer;
using namespace Windows::Foundation;

RecentlyFileItem::RecentlyFileItem()
{
	_Thumbnail = ref new Windows::UI::Xaml::Media::ImageBrush();

	auto _GeneralSetting = GeneralSetting::Current;
	auto _bitmapImage = ref new Windows::UI::Xaml::Media::Imaging::BitmapImage(ref new Windows::Foundation::Uri(L"ms-appx:///Assets/RecentThumbViewItemBk.png"));
	_Thumbnail->ImageSource = _bitmapImage;
	_ThumbnailWidth = _GeneralSetting->MRUThumbnailWidth;
	_ThumbnailHeight = _GeneralSetting->MRUThumbnailHeight;
	_Thumb = ref new Windows::UI::Xaml::Media::Imaging::BitmapImage(ref new Windows::Foundation::Uri(L"ms-appx:///Assets/StartPageFileLogo.png"));
	_IsPasswordProtected = Windows::UI::Xaml::Visibility::Collapsed;

	m_needGetThumbnail = true;
	m_receivedChangeWhileRunningGetThumbnail = false;
	m_runningGetThumbnailAsync = false;
	_FileSizeVisibility = Windows::UI::Xaml::Visibility::Visible;
	_BorderThinknessValue = 1;
	_BorderColor = ref new SolidColorBrush(foxit::rdk::Utility::ConvertFromString("#CECECE"));
	_IsNotSkyDriveFile = true;
}

RecentlyFileItem::RecentlyFileItem(Platform::String^ str)
{
	_Thumbnail = ref new Windows::UI::Xaml::Media::ImageBrush();

	auto _GeneralSetting = GeneralSetting::Current;
	auto _bitmapImage = ref new Windows::UI::Xaml::Media::Imaging::BitmapImage(ref new Windows::Foundation::Uri(L"ms-appx:///Assets/openfileThumbil.png"));
	_Thumbnail->ImageSource = _bitmapImage;
	_ThumbnailWidth = _GeneralSetting->MRUThumbnailWidth;
	_ThumbnailHeight = _GeneralSetting->MRUThumbnailHeight;
	_Thumb = ref new Windows::UI::Xaml::Media::Imaging::BitmapImage(ref new Windows::Foundation::Uri(L"ms-appx:///Assets/openfile.png"));
	_IsPasswordProtected = Windows::UI::Xaml::Visibility::Collapsed;

	m_needGetThumbnail = true;
	m_receivedChangeWhileRunningGetThumbnail = false;
	m_runningGetThumbnailAsync = false;
	_FileSizeVisibility = Windows::UI::Xaml::Visibility::Collapsed;
	//_BorderThinknessValue = 0;
	_BorderColor = ref new SolidColorBrush(Windows::UI::Colors::Transparent);
	_IsNotSkyDriveFile = true;
}


RecentlyFileItem::~RecentlyFileItem()
{

}

Windows::UI::Xaml::Media::SolidColorBrush^ RecentlyFileItem::BorderColor::get()
{
	return _BorderColor;
}

Windows::UI::Xaml::Media::ImageSource^ RecentlyFileItem::ThumbImage::get()
{
	return _Thumb;
}


Windows::UI::Xaml::Media::ImageBrush^ RecentlyFileItem::Thumbnail::get()
{
	return _Thumbnail;
}

void RecentlyFileItem::Thumbnail::set(Windows::UI::Xaml::Media::ImageBrush^ value)
{
	if (_Thumbnail != value)
	{
		_Thumbnail = value;
		OnPropertyChanged("Thumbnail");
	}
}

Platform::String^ RecentlyFileItem::AutomationId::get()
{
	return _AutomationId;
}

void RecentlyFileItem::AutomationId::set(Platform::String^ value)
{
	if (_AutomationId != value)
	{
		_AutomationId = value;
		InitRecentlyFileImage(_AutomationId);

		OnPropertyChanged("AutomationId");
	}
}

Platform::String^ RecentlyFileItem::FileName::get()
{
	return _FileName;
}

void RecentlyFileItem::FileName::set(Platform::String^ value)
{
	if (_FileName != value)
	{
		_FileName = value;
		OnPropertyChanged("FileName");
	}
}

Platform::String^ RecentlyFileItem::FilePath::get()
{
	return _FilePath;
}

void RecentlyFileItem::FilePath::set(Platform::String^ value)
{
	if (_FilePath != value)
	{
		if (_FilePath == nullptr && value->Length() > 0)
		{
			Platform::String^ filePath = ref new Platform::String(value->Data());

			const wchar_t *seperator = wcsrchr(filePath->Data(), '\\');
			_FileName = seperator ? ref new Platform::String(seperator + 1) : filePath;
			OnPropertyChanged("FileName");
		}

		_FilePath = value;
		OnPropertyChanged("FilePath");
	}
}

Platform::String^ RecentlyFileItem::FileTime::get()
{
	return _FileTime;
}

void RecentlyFileItem::FileTime::set(Platform::String^ value)
{
	if (_FileTime != value)
	{
		_FileTime = value;
		OnPropertyChanged("FileTime");
	}
}

uint64 RecentlyFileItem::FileSize::get()
{
	return _FileSize;
}

void RecentlyFileItem::FileSize::set(uint64 value)
{
	if (_FileSize != value)
	{
		_FileSize = value;
		OnPropertyChanged("FileTime");
	}
}

Platform::String^ RecentlyFileItem::MRUToolTip::get()
{
	return _MRUToolTip;
}

void RecentlyFileItem::MRUToolTip::set(Platform::String^ value)
{
	if (_MRUToolTip != value)
	{
		_MRUToolTip = value;
		OnPropertyChanged("MRUToolTip");
	}
}

double RecentlyFileItem::ThumbnailWidth::get()
{
	return _ThumbnailWidth;
}

void RecentlyFileItem::ThumbnailWidth::set(double value)
{
	if (_ThumbnailWidth != value)
	{
		_ThumbnailWidth = value;
		OnPropertyChanged("ThumbnailWidth");
	}
}

double RecentlyFileItem::ThumbnailHeight::get()
{
	return _ThumbnailHeight;
}

void RecentlyFileItem::ThumbnailHeight::set(double value)
{
	if (_ThumbnailHeight != value)
	{
		_ThumbnailHeight = value;
		OnPropertyChanged("ThumbnailHeight");
	}
}

Windows::UI::Xaml::Visibility RecentlyFileItem::IsPasswordProtected::get()
{
	return _IsPasswordProtected;
}

void RecentlyFileItem::IsPasswordProtected::set(Windows::UI::Xaml::Visibility value)
{
	if (_IsPasswordProtected != value)
	{
		_IsPasswordProtected = value;
		OnPropertyChanged("IsPasswordProtected");
	}
}

Windows::UI::Xaml::Visibility RecentlyFileItem::FileSizeVisibility::get()
{
	return _FileSizeVisibility;
}

void RecentlyFileItem::FileSizeVisibility::set(Windows::UI::Xaml::Visibility value)
{
	if (_FileSizeVisibility != value)
	{
		_FileSizeVisibility = value;
		OnPropertyChanged("FileSizeVisibility");
	}
}

double RecentlyFileItem::BorderThinknessValue::get()
{
	return _BorderThinknessValue;
}

void RecentlyFileItem::BorderThinknessValue::set(double value)
{
	if (_BorderThinknessValue != value)
	{
		_BorderThinknessValue = value;
		OnPropertyChanged("BorderThinknessValue");
	}
}

bool RecentlyFileItem::IsNotSkyDriveFile::get()
{
	return _IsNotSkyDriveFile;
}

void RecentlyFileItem::IsNotSkyDriveFile::set(bool value)
{
	if (_IsNotSkyDriveFile != value)
	{
		_IsNotSkyDriveFile = value;
		OnPropertyChanged("IsNotSkyDriveFile");
	}
}

void RecentlyFileItem::InitRecentlyFileImage(Platform::String^ guid)
{
	Windows::Storage::ApplicationDataContainer^ localSettings = Windows::Storage::ApplicationData::Current->LocalSettings;
	auto values = localSettings->Values;

	if (values->HasKey(_AutomationId))
	{
		Windows::Storage::ApplicationDataCompositeValue^ composite = dynamic_cast<Windows::Storage::ApplicationDataCompositeValue^>(localSettings->Values->Lookup(_AutomationId));
		if (composite)
		{
			Platform::String^ isEncryptFile = nullptr;
			
			if (composite->HasKey(L"FileTime"))
			{
				_FileDateTime.UniversalTime = dynamic_cast<Platform::IBox<long long>^>(composite->Lookup(L"FileTime"))->Value;
			}

			auto _LongDateDateTimeFormatter = ref new Windows::Globalization::DateTimeFormatting::DateTimeFormatter(L"{year.full}-{month.integer(2)}-{day.integer(2)} {hour.integer(2)}:{minute.integer(2)}");
			_FileTime = _LongDateDateTimeFormatter->Format(_FileDateTime);
			OnPropertyChanged(L"FileTime");

			if (composite->HasKey(L"PasswordProtected"))
			{
				isEncryptFile = dynamic_cast<Platform::String^>(composite->Lookup(L"PasswordProtected"));
				if (isEncryptFile != nullptr && isEncryptFile == "1")
				{
					_IsPasswordProtected = Windows::UI::Xaml::Visibility::Visible;
					OnPropertyChanged(L"IsPasswordProtected");
					auto _bitmapImage = ref new Windows::UI::Xaml::Media::Imaging::BitmapImage(ref new Windows::Foundation::Uri(L"ms-appx:///Assets/StartPageLockFileLogo.png"));
					_Thumbnail->ImageSource = _bitmapImage;
					OnPropertyChanged("Thumbnail");
				}
				else
				{
					ReGetThumbnailImage();
				}
			}
			else
			{
				ReGetThumbnailImage();
			}
		}
	}
	ReGetFileInfo();
}

void RecentlyFileItem::ReGetThumbnailImage()
{
	if (!m_needGetThumbnail)
		return;

	if (m_runningGetThumbnailAsync)
	{
		m_receivedChangeWhileRunningGetThumbnail = true;
	}

	if (!m_runningGetThumbnailAsync)
	{
		m_runningGetThumbnailAsync = true;
		create_task(MRUManager::LoadWriteableBitmapFromMRUThumbnailFileAsync(AutomationId))
			.then([this](concurrency::task<Windows::UI::Xaml::Media::Imaging::WriteableBitmap^> t)
		{
			try
			{
				if (!m_receivedChangeWhileRunningGetThumbnail)
				{
					OutputDebugString(L"OnItemsSizeChanged...OK.....\r\n");
				}
				m_runningGetThumbnailAsync = false;
				if (m_receivedChangeWhileRunningGetThumbnail)
				{
					m_receivedChangeWhileRunningGetThumbnail = false;
					ReGetThumbnailImage();
				}

				Windows::UI::Xaml::Media::Imaging::WriteableBitmap^ _bitmap = t.get();
				_ThumbnailWidth = _bitmap?_bitmap->PixelWidth:0;
				_ThumbnailHeight = _bitmap?_bitmap->PixelHeight:0;
				_Thumbnail->ImageSource = _bitmap;

				OnPropertyChanged("ThumbnailWidth");
				OnPropertyChanged("ThumbnailHeight");
				OnPropertyChanged("Thumbnail");
#if _DEBUG
				OutputDebugString(L"Load Thumbnail OK...");
				OutputDebugString(this->AutomationId->Data());
				OutputDebugString(L", ");
				OutputDebugString(this->FileName->Data());
				OutputDebugString(L"...\n");
#endif
			}
			catch (Platform::COMException^ e)
			{
				OutputDebugString(L"Load Thumbnail Exception...");
				OutputDebugString(this->AutomationId->Data());
				OutputDebugString(L", ");
				OutputDebugString(this->FileName->Data());
				OutputDebugString(L", ");
				OutputDebugString(e->Message->Data());
				OutputDebugString(L"...\n");
			}
		});
	}
}


void RecentlyFileItem::ReGetFileInfo()
{
	task<StorageFile^> OpenRecently(Windows::Storage::AccessCache::StorageApplicationPermissions::MostRecentlyUsedList->GetFileAsync(AutomationId, Windows::Storage::AccessCache::AccessCacheOptions::SuppressAccessTimeUpdate));
	OpenRecently.then([this](task<Windows::Storage::StorageFile^> res)
	{
		Windows::Storage::StorageFile^ file = nullptr;
		try {
				file = res.get();
				if (file == nullptr)
					throw ref new Platform::NullReferenceException();
				}

		catch (Platform::Exception^ e)
		{
			IAsyncOperation<BasicProperties^>^ op = concurrency::create_async([]() {
						return create_task([]() { 
							return (Windows::Storage::FileProperties::BasicProperties^)nullptr;
						});
				}); 
			return op;
		}

		_FileName = file->Name;
		OnPropertyChanged(L"FileName");

		_FilePath = file->Path;
		OnPropertyChanged(L"FilePath");
		
		return file->GetBasicPropertiesAsync();
	}, concurrency::task_continuation_context::use_current())
		.then([this](Windows::Storage::FileProperties::BasicProperties^ fileProperties)
	{
		if (fileProperties)
		{
			_FileSize = fileProperties->Size;
			OnPropertyChanged(L"FileSize");

			if (_FileTime == nullptr || _FileTime->Length() == 0)
			{
				auto _LongDateDateTimeFormatter = ref new Windows::Globalization::DateTimeFormatting::DateTimeFormatter(L"{year.full}-{month.integer(2)}-{day.integer(2)} {hour.integer(2)}:{minute.integer(2)}");
				_FileTime = _LongDateDateTimeFormatter->Format(fileProperties->DateModified);
				OnPropertyChanged(L"FileTime");
			}
		}
	}, concurrency::task_continuation_context::use_current())
		.then([this](concurrency::task<void> t)
	{
		try
		{
			t.get();
			std::wstringstream ws;
			ws << L"Get RecentlyFileItem MostRecentlyUsedList OK...";
			ws << _FileName->Data();
			ws << L"...\n";
			OutputDebugString(ws.str().c_str());
		}
		catch (Platform::COMException^ e)
		{
			std::wstringstream ws;
			ws << L"Get RecentlyFileItem MostRecentlyUsedList Exception...";
			ws << _FileName->Data();
			ws << L", HResult: " << e->HResult;
			ws << L", Message: " << e->Message->Data();
			ws << L"...\n";
			OutputDebugString(ws.str().c_str());

			ResourceLoader^ resourceLoader = ref new ResourceLoader();
			auto unknow = resourceLoader->GetString(L"UNKNOWN");
			_FileTime = unknow;
			OnPropertyChanged(L"FileTime");
		}
	});
}

void RecentlyFileItem::RefreshRecentlyFileImage()
{
	Windows::Storage::ApplicationDataContainer^ localSettings = Windows::Storage::ApplicationData::Current->LocalSettings;
	auto values = localSettings->Values;
	Platform::String^ isEncryptFile = nullptr;
	if (values->HasKey(_AutomationId))
	{
		Windows::Storage::ApplicationDataCompositeValue^ composite = dynamic_cast<Windows::Storage::ApplicationDataCompositeValue^>(localSettings->Values->Lookup(_AutomationId));
		if (composite)
		{
			if (composite->HasKey(L"PasswordProtected"))
			{
				isEncryptFile = dynamic_cast<Platform::String^>(composite->Lookup(L"PasswordProtected"));
				if (isEncryptFile != nullptr && isEncryptFile == "1")
				{
					_IsPasswordProtected = Windows::UI::Xaml::Visibility::Visible;
					OnPropertyChanged(L"IsPasswordProtected");
					auto _bitmapImage = ref new Windows::UI::Xaml::Media::Imaging::BitmapImage(ref new Windows::Foundation::Uri(L"ms-appx:///Assets/StartPageLockFileLogo.png"));
					_Thumbnail->ImageSource = _bitmapImage;
					OnPropertyChanged("Thumbnail");
				}
				else
				{
					ReGetThumbnailImage();
				}
			}
			else
			{
				ReGetThumbnailImage();
			}
		}
	}
	ReGetFileInfo();
}

bool RecentlyFileItem::operator < (const RecentlyFileItem^ m)
{
	return _FileDateTime.UniversalTime > m->_FileDateTime.UniversalTime;
}


PinFolderItem::PinFolderItem()
{
	_Thumbnail = ref new Windows::UI::Xaml::Media::ImageBrush();

	auto _GeneralSetting = GeneralSetting::Current;
	auto _bitmapImage = ref new Windows::UI::Xaml::Media::Imaging::BitmapImage(ref new Windows::Foundation::Uri(L"ms-appx:///Assets/openfileThumbil.png"));
	_Thumbnail->ImageSource = _bitmapImage;
	_ThumbnailWidth = _GeneralSetting->MRUThumbnailWidth;
	_ThumbnailHeight = _GeneralSetting->MRUThumbnailHeight;
	_NextArrowVisibility = Windows::UI::Xaml::Visibility::Visible;

	_Thumb = ref new Windows::UI::Xaml::Media::Imaging::BitmapImage(ref new Windows::Foundation::Uri(L"ms-appx:///Assets/openfile.png"));

	_IsPasswordProtected = Windows::UI::Xaml::Visibility::Collapsed;

	m_needGetThumbnail = true;
	m_receivedChangeWhileRunningGetThumbnail = false;
	m_runningGetThumbnailAsync = false;

	_BorderColor = ref new SolidColorBrush(foxit::rdk::Utility::ConvertFromString("#CECECE"));
}


PinFolderItem::~PinFolderItem()
{

}

Windows::UI::Xaml::Media::ImageSource^ PinFolderItem::ThumbImage::get()
{
	return _Thumb;
}

Windows::UI::Xaml::Media::SolidColorBrush^ PinFolderItem::BorderColor::get()
{
	return _BorderColor;
}

void PinFolderItem::BorderColor::set(Windows::UI::Xaml::Media::SolidColorBrush^ value)
{
	if (_BorderColor != value)
	{
		_BorderColor = value;
		OnPropertyChanged("BorderColor");
	}
}

Windows::UI::Xaml::Media::ImageBrush^ PinFolderItem::Thumbnail::get()
{
	return _Thumbnail;
}

void PinFolderItem::Thumbnail::set(Windows::UI::Xaml::Media::ImageBrush^ value)
{
	if (_Thumbnail != value)
	{
		_Thumbnail = value;
		OnPropertyChanged("Thumbnail");
	}
}

Platform::String^ PinFolderItem::AutomationId::get()
{
	return _AutomationId;
}

void PinFolderItem::AutomationId::set(Platform::String^ value)
{
	if (_AutomationId != value)
	{
		_AutomationId = value;

		concurrency::create_task(Windows::Storage::AccessCache::StorageApplicationPermissions::FutureAccessList->GetItemAsync(_AutomationId))
			.then([this](concurrency::task<Windows::Storage::IStorageItem^> result)
		{
			try
			{
				Windows::Storage::IStorageItem^ folder = result.get();

				_StorageObj = folder;
			}
			catch (...)
			{
				OutputDebugString(L"Folder is deleted or moved...\n");
			}
		});

		OnPropertyChanged("AutomationId");
	}
}

Platform::String^ PinFolderItem::FolderName::get()
{
	return _FileName;
}

void PinFolderItem::FolderName::set(Platform::String^ value)
{
	if (_FileName != value)
	{
		if (value == "Pin Folder")
		{
			value = Windows::ApplicationModel::Resources::ResourceLoader::GetForCurrentView()->GetString(L"StartPinFolderStr");
		}
		else if (value == "Folder")
		{
			value = Windows::ApplicationModel::Resources::ResourceLoader::GetForCurrentView()->GetString(L"FolderStr");
		}

		_FileName = value;
		OnPropertyChanged("FolderName");
	}
}

Platform::String^ PinFolderItem::FilePath::get()
{
	return _FilePath;
}

void PinFolderItem::FilePath::set(Platform::String^ value)
{
	if (_FilePath != value)
	{
		if (_FilePath == nullptr && value->Length() > 0)
		{
			Platform::String^ filePath = ref new Platform::String(value->Data());
			const wchar_t *seperator = wcsrchr(filePath->Data(), '\\');
			_FileName = seperator ? ref new Platform::String(seperator + 1) : filePath;
			OnPropertyChanged("FolderName");

			if (!_IsFolder)
			{
				_Thumb = ref new Windows::UI::Xaml::Media::Imaging::BitmapImage(ref new Windows::Foundation::Uri(L"ms-appx:///Assets/StartPageFileLogo.png"));

				auto _bitmapImage = ref new Windows::UI::Xaml::Media::Imaging::BitmapImage(ref new Windows::Foundation::Uri(L"ms-appx:///Assets/RecentThumbViewItemBk.png"));
				_Thumbnail->ImageSource = _bitmapImage;

				_FilePathId = MRUManager::GetStorageFileMRUToken(value);
				if (_FilePathId)
					InitRecentlyFileImage(_FilePathId);
			}
			else
			{

			}
		}

		_FilePath = value;
		OnPropertyChanged("FilePath");
	}
	else
	{
		if (value->Length() == 0 && !_IsFolder)
		{
			_Thumb = ref new Windows::UI::Xaml::Media::Imaging::BitmapImage(ref new Windows::Foundation::Uri(L"ms-appx:///Assets/pinfolderlist.png"));

			auto _bitmapImage = ref new Windows::UI::Xaml::Media::Imaging::BitmapImage(ref new Windows::Foundation::Uri(L"ms-appx:///Assets/pinfolderthumb.png"));
			_Thumbnail->ImageSource = _bitmapImage;
		}
	}
}

double PinFolderItem::ThumbnailWidth::get()
{
	return _ThumbnailWidth;
}

void PinFolderItem::ThumbnailWidth::set(double value)
{
	if (_ThumbnailWidth != value)
	{
		_ThumbnailWidth = value;
		OnPropertyChanged("ThumbnailWidth");
	}
}

double PinFolderItem::ThumbnailHeight::get()
{
	return _ThumbnailHeight;
}

void PinFolderItem::ThumbnailHeight::set(double value)
{
	if (_ThumbnailHeight != value)
	{
		_ThumbnailHeight = value;
		OnPropertyChanged("ThumbnailHeight");
	}
}

Windows::UI::Xaml::Visibility PinFolderItem::IsPasswordProtected::get()
{
	return _IsPasswordProtected;
}

void PinFolderItem::IsPasswordProtected::set(Windows::UI::Xaml::Visibility value)
{
	if (_IsPasswordProtected != value)
	{
		_IsPasswordProtected = value;
		OnPropertyChanged("IsPasswordProtected");
	}
}

IStorageItem^ PinFolderItem::StorageObj::get()
{
	return _StorageObj;
}

void PinFolderItem::StorageObj::set(IStorageItem^ value)
{
	if (_StorageObj != value)
	{
		_StorageObj = value;
	}
}

void PinFolderItem::InitRecentlyFileImage(Platform::String^ guid)
{
	Windows::Storage::ApplicationDataContainer^ localSettings = Windows::Storage::ApplicationData::Current->LocalSettings;
	auto values = localSettings->Values;

	if (values->HasKey(_FilePathId))
	{
		Windows::Storage::ApplicationDataCompositeValue^ composite = dynamic_cast<Windows::Storage::ApplicationDataCompositeValue^>(localSettings->Values->Lookup(_FilePathId));
		if (composite)
		{
			Platform::String^ isEncryptFile = nullptr;

			if (composite->HasKey(L"PasswordProtected"))
			{
				isEncryptFile = dynamic_cast<Platform::String^>(composite->Lookup(L"PasswordProtected"));
				if (isEncryptFile != nullptr && isEncryptFile == "1")
				{
					_IsPasswordProtected = Windows::UI::Xaml::Visibility::Visible;
					OnPropertyChanged(L"IsPasswordProtected");
					auto _bitmapImage = ref new Windows::UI::Xaml::Media::Imaging::BitmapImage(ref new Windows::Foundation::Uri(L"ms-appx:///Assets/StartPageLockFileLogo.png"));
					_Thumbnail->ImageSource = _bitmapImage;
					OnPropertyChanged("Thumbnail");
				}
				else
				{
					ReGetThumbnailImage();
				}
			}
			else
			{
				ReGetThumbnailImage();
			}
		}
	}
}

void PinFolderItem::ReGetThumbnailImage()
{
	if (!m_needGetThumbnail)
		return;

	if (m_runningGetThumbnailAsync)
	{
		m_receivedChangeWhileRunningGetThumbnail = true;
	}

	if (!m_runningGetThumbnailAsync)
	{
		m_runningGetThumbnailAsync = true;
		create_task(MRUManager::LoadWriteableBitmapFromMRUThumbnailFileAsync(FilePathId))
			.then([this](concurrency::task<Windows::UI::Xaml::Media::Imaging::WriteableBitmap^> t)
		{
			try
			{
				if (!m_receivedChangeWhileRunningGetThumbnail)
				{
					OutputDebugString(L"OnItemsSizeChanged...OK.....\r\n");
				}
				m_runningGetThumbnailAsync = false;
				if (m_receivedChangeWhileRunningGetThumbnail)
				{
					m_receivedChangeWhileRunningGetThumbnail = false;
					ReGetThumbnailImage();
				}

				Windows::UI::Xaml::Media::Imaging::WriteableBitmap^ _bitmap = t.get();
				_ThumbnailWidth = _bitmap->PixelWidth;
				_ThumbnailHeight = _bitmap->PixelHeight;
				_Thumbnail->ImageSource = _bitmap;

				OnPropertyChanged("ThumbnailWidth");
				OnPropertyChanged("ThumbnailHeight");
				OnPropertyChanged("Thumbnail");
			}
			catch (Platform::COMException^ e)
			{

			}
		});
	}
}

void PinFolderItem::ReGetFileInfo()
{
	task<StorageFile^> OpenRecently(Windows::Storage::AccessCache::StorageApplicationPermissions::MostRecentlyUsedList->GetFileAsync(AutomationId, Windows::Storage::AccessCache::AccessCacheOptions::SuppressAccessTimeUpdate));
	OpenRecently.then([this](task<Windows::Storage::StorageFile^> res)
	{
		Windows::Storage::StorageFile^ file = res.get();
		if (file == nullptr)
			throw ref new Platform::NullReferenceException();

		_FileName = file->Name;
		OnPropertyChanged(L"FolderName");

		_FilePath = file->Path;
		OnPropertyChanged(L"FilePath");

		return file->GetBasicPropertiesAsync();
	}, concurrency::task_continuation_context::use_current())
		.then([this](Windows::Storage::FileProperties::BasicProperties^ fileProperties)
	{
		if (fileProperties)
		{

		}
	}, concurrency::task_continuation_context::use_current())
		.then([this](concurrency::task<void> t)
	{
		try
		{
			t.get();
			std::wstringstream ws;
			ws << L"Get RecentlyFileItem MostRecentlyUsedList OK...";
			ws << _FileName->Data();
			ws << L"...\n";
			OutputDebugString(ws.str().c_str());
		}
		catch (Platform::COMException^ e)
		{
			std::wstringstream ws;
			ws << L"Get RecentlyFileItem MostRecentlyUsedList Exception...";
			ws << _FileName->Data();
			ws << L", HResult: " << e->HResult;
			ws << L", Message: " << e->Message->Data();
			ws << L"...\n";
			OutputDebugString(ws.str().c_str());

		}
	});
}

void PinFolderItem::RefreshRecentlyFileImage()
{
	Windows::Storage::ApplicationDataContainer^ localSettings = Windows::Storage::ApplicationData::Current->LocalSettings;
	auto values = localSettings->Values;
	Platform::String^ isEncryptFile = nullptr;
	if (values->HasKey(_AutomationId))
	{
		Windows::Storage::ApplicationDataCompositeValue^ composite = dynamic_cast<Windows::Storage::ApplicationDataCompositeValue^>(localSettings->Values->Lookup(_AutomationId));
		if (composite)
		{
			if (composite->HasKey(L"PasswordProtected"))
			{
				isEncryptFile = dynamic_cast<Platform::String^>(composite->Lookup(L"PasswordProtected"));
				if (isEncryptFile != nullptr && isEncryptFile == "1")
				{
					_IsPasswordProtected = Windows::UI::Xaml::Visibility::Visible;
					OnPropertyChanged(L"IsPasswordProtected");
					auto _bitmapImage = ref new Windows::UI::Xaml::Media::Imaging::BitmapImage(ref new Windows::Foundation::Uri(L"ms-appx:///Assets/StartPageLockFileLogo.png"));
					_Thumbnail->ImageSource = _bitmapImage;
					OnPropertyChanged("Thumbnail");
				}
				else
				{
					ReGetThumbnailImage();
				}
			}
			else
			{
				ReGetThumbnailImage();
			}
		}
	}
}

bool PinFolderItem::operator < (const PinFolderItem^ m)
{
	return true;
}

bool PinFolderItem::IsFolder::get()
{
	return _IsFolder;
}

void PinFolderItem::IsFolder::set(bool value)
{
	if (_IsFolder != value)
	{
		_IsFolder = value;
	}
}

Platform::String^ PinFolderItem::FilePathId::get()
{
	return _FilePathId;
}

void PinFolderItem::FilePathId::set(Platform::String^ value)
{
	if (_FilePathId != value)
	{
		_FilePathId = value;
	}
}

Windows::UI::Xaml::Visibility PinFolderItem::NextArrowVisibility::get()
{
	return _NextArrowVisibility;
}

void PinFolderItem::NextArrowVisibility::set(Windows::UI::Xaml::Visibility value)
{
	if (_NextArrowVisibility != value)
	{
		_NextArrowVisibility = value;
		OnPropertyChanged("NextArrowVisibility");
	}
}