/**
 * Copyright (C) 2003-2018, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit Mobile PDF SDK to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */


#pragma once

namespace CompletePDFViewer {
		ref class BindableBase;
}

namespace CompletePDFViewer
{
	public enum class FileViewMode
	{
		List,
		Thumb
	};

	[Windows::UI::Xaml::Data::Bindable]
	[Windows::Foundation::Metadata::WebHostHidden]
	public ref class RecentlyFileItem sealed : public BindableBase
	{
	public:
		RecentlyFileItem();
		RecentlyFileItem(Platform::String^ str);

		virtual ~RecentlyFileItem();

		property Windows::UI::Xaml::Media::ImageSource^ ThumbImage
		{
			Windows::UI::Xaml::Media::ImageSource^ get();
		}

		property Windows::UI::Xaml::Media::ImageBrush^ Thumbnail
		{
			Windows::UI::Xaml::Media::ImageBrush^ get();
			void set(Windows::UI::Xaml::Media::ImageBrush^ value);
		}
		property Platform::String^ AutomationId
		{
			Platform::String^ get();
			void set(Platform::String^ value);
		}
		property Platform::String^ FileName
		{
			Platform::String^ get();
			void set(Platform::String^ value);
		}
		property Platform::String^ FilePath
		{
			Platform::String^ get();
			void set(Platform::String^ value);
		}
		property Platform::String^ FileTime
		{
			Platform::String^ get();
			void set(Platform::String^ value);
		}
		property uint64 FileSize
		{
			uint64 get();
			void set(uint64 value);
		}
		property Platform::String^ MRUToolTip
		{
			Platform::String^ get();
			void set(Platform::String^ value);
		}
		property double ThumbnailWidth
		{
			double get();
			void set(double value);
		}
		property double ThumbnailHeight
		{
			double get();
			void set(double value);
		}
		property Windows::UI::Xaml::Visibility IsPasswordProtected
		{
			Windows::UI::Xaml::Visibility get();
			void set(Windows::UI::Xaml::Visibility value);
		}
		property Windows::UI::Xaml::Visibility FileSizeVisibility
		{
			Windows::UI::Xaml::Visibility get();
			void set(Windows::UI::Xaml::Visibility value);
		}
		property double BorderThinknessValue
		{
			double get();
			void set(double value);
		}

		property bool IsNotSkyDriveFile
		{
			bool get();
			void set(bool value);
		}

		property Windows::UI::Xaml::Media::SolidColorBrush^ BorderColor {Windows::UI::Xaml::Media::SolidColorBrush^ get(); }

		void RefreshRecentlyFileImage();

	internal:
		void InitRecentlyFileImage(Platform::String^ guid);
		
	internal:
		void ReGetThumbnailImage();
		void ReGetFileInfo();
		bool operator < (const RecentlyFileItem^ m);

	private:
		bool m_needGetThumbnail;
		bool m_runningGetThumbnailAsync;
		bool m_receivedChangeWhileRunningGetThumbnail;

	private:
		Windows::UI::Xaml::Media::Imaging::BitmapImage^ _Thumb;
		Windows::UI::Xaml::Media::ImageBrush^ _Thumbnail;
		Platform::String^ _AutomationId;
		Platform::String^ _FileName;
		Platform::String^ _FilePath;
		Platform::String^ _FileTime;
		uint64 _FileSize;
		Platform::String^ _MRUToolTip;
		double _ThumbnailWidth;
		double _ThumbnailHeight;
		Windows::UI::Xaml::Visibility _IsPasswordProtected;
		Windows::Foundation::DateTime _FileDateTime;
		Windows::UI::Xaml::Visibility _FileSizeVisibility;
		double _BorderThinknessValue;
		Windows::UI::Xaml::Media::SolidColorBrush^ _BorderColor;
		bool _IsNotSkyDriveFile;
	};

	[Windows::UI::Xaml::Data::Bindable]
	public ref class PinFolderItem sealed : public BindableBase
	{
	public:
		PinFolderItem();
		virtual ~PinFolderItem();

		property Windows::UI::Xaml::Media::SolidColorBrush^ BorderColor
		{
			Windows::UI::Xaml::Media::SolidColorBrush^ get();
			void set(Windows::UI::Xaml::Media::SolidColorBrush^ value);
		}
		property Windows::UI::Xaml::Media::ImageBrush^ Thumbnail
		{
			Windows::UI::Xaml::Media::ImageBrush^ get();
			void set(Windows::UI::Xaml::Media::ImageBrush^ value);
		}
		property Windows::UI::Xaml::Media::ImageSource^ ThumbImage
		{
			Windows::UI::Xaml::Media::ImageSource^ get();
		}
		property Platform::String^ AutomationId
		{
			Platform::String^ get();
			void set(Platform::String^ value);
		}
		property Platform::String^ FolderName
		{
			Platform::String^ get();
			void set(Platform::String^ value);
		}
		property Platform::String^ FilePath
		{
			Platform::String^ get();
			void set(Platform::String^ value);
		}		
		property double ThumbnailWidth
		{
			double get();
			void set(double value);
		}
		property double ThumbnailHeight
		{
			double get();
			void set(double value);
		}
		property Windows::UI::Xaml::Visibility IsPasswordProtected
		{
			Windows::UI::Xaml::Visibility get();
			void set(Windows::UI::Xaml::Visibility value);
		}		
		property bool IsFolder
		{
			bool get();
			void set(bool value);
		}
		property Windows::Storage::IStorageItem^ StorageObj
		{
			Windows::Storage::IStorageItem^ get();
			void set(Windows::Storage::IStorageItem^ value);
		}
		property Platform::String^ FilePathId
		{
			Platform::String^ get();
			void set(Platform::String^ value);
		}

		void RefreshRecentlyFileImage();

		property Windows::UI::Xaml::Visibility NextArrowVisibility
		{
			Windows::UI::Xaml::Visibility get();
			void set(Windows::UI::Xaml::Visibility value);
		}

	internal:
		void InitRecentlyFileImage(Platform::String^ guid);

	internal:
		void ReGetThumbnailImage();
		void ReGetFileInfo();
		bool operator < (const PinFolderItem^ m);

	private:
		bool m_needGetThumbnail;
		bool m_runningGetThumbnailAsync;
		bool m_receivedChangeWhileRunningGetThumbnail;

	private:
		Windows::UI::Xaml::Media::Imaging::BitmapImage^ _Thumb;
		Windows::UI::Xaml::Media::ImageBrush^ _Thumbnail;
		Platform::String^ _AutomationId;
		Platform::String^ _FileName;
		Platform::String^ _FilePath;		
		double _ThumbnailWidth;
		double _ThumbnailHeight;
		Windows::UI::Xaml::Visibility _IsPasswordProtected;
		Windows::UI::Xaml::Media::SolidColorBrush^ _BorderColor;
		bool _IsFolder;
		Windows::Storage::IStorageItem^ _StorageObj;
		Platform::String^ _FilePathId;
		Windows::UI::Xaml::Visibility _NextArrowVisibility;
	};
}

