﻿/**
 * Copyright (C) 2003-2018, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit Mobile PDF SDK to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */


#include "pch.h"
#include "RecentlyListPage.xaml.h"
#include "MainPage.xaml.h"
#include "RecentlyFileItem.h"
#include "MRUManager.h"
#include "StartPage.xaml.h"
#include <regex>

using namespace CompletePDFViewer;

using namespace Platform;
using namespace Windows::Foundation;
using namespace Windows::Foundation::Collections;
using namespace Windows::UI::Xaml;
using namespace Windows::UI::Xaml::Controls;
using namespace Windows::UI::Xaml::Controls::Primitives;
using namespace Windows::UI::Xaml::Data;
using namespace Windows::UI::Xaml::Input;
using namespace Windows::UI::Xaml::Media;
using namespace Windows::UI::Xaml::Navigation;
using namespace Windows::Storage;
using namespace Windows::Storage::Streams;
using namespace Windows::Storage::FileProperties;
using namespace Windows::Storage::AccessCache;
using namespace Windows::Storage::Pickers;
using namespace Windows::Security::Cryptography;
using namespace Windows::Security::Cryptography::Core;


RecentlyListPage::RecentlyListPage()
{
	InitializeComponent();

	OnInitializePage();
}

void RecentlyListPage::OnInitializePage()
{
	m_ParentPage = nullptr;

	Windows::Storage::ApplicationDataContainer^ _localSettings = Windows::Storage::ApplicationData::Current->LocalSettings;
	auto _localSettingsValues = _localSettings->Values;
	auto _SettingDataCompositeValue = _localSettingsValues->HasKey(APPSETTINGKEY) ? dynamic_cast<Windows::Storage::ApplicationDataCompositeValue^>(_localSettingsValues->Lookup(APPSETTINGKEY)) : ref new Windows::Storage::ApplicationDataCompositeValue();

	int iViewMode = 0;
	if (_SettingDataCompositeValue->HasKey(L"RecentDocumentViewMode"))
	{
		auto _obValue = _SettingDataCompositeValue->Lookup(L"RecentDocumentViewMode");
		iViewMode = (_obValue->GetType()->FullName == int::typeid->FullName) ? ((Platform::IBox<int>^)(_obValue))->Value : 0;
	}

	_SettingDataCompositeValue->Insert(L"RecentDocumentViewMode", iViewMode);
	_localSettingsValues->Insert(APPSETTINGKEY, _SettingDataCompositeValue);

	m_ViewMode = GetEnumViewMode(iViewMode);
	SetViewMode(GetStringViewMode(m_ViewMode));

	bool bIsFirstEnterApp = true;
	if (_SettingDataCompositeValue->HasKey(L"IsFirstEnterApp"))
	{
		auto _obValue = _SettingDataCompositeValue->Lookup(L"IsFirstEnterApp");
		bIsFirstEnterApp = (_obValue->GetType()->FullName == bool::typeid->FullName) ? ((Platform::IBox<bool>^)(_obValue))->Value : true;
	}
	
	if (bIsFirstEnterApp)
	{
		_SettingDataCompositeValue->Insert(L"IsFirstEnterApp", false);
		_localSettingsValues->Insert(APPSETTINGKEY, _SettingDataCompositeValue);
	}

	Windows::Storage::StorageFolder^ documentFolder = Windows::Storage::KnownFolders::DocumentsLibrary;
	concurrency::create_task(documentFolder->TryGetItemAsync("developer_guide_uwp.pdf"))
		.then([this, bIsFirstEnterApp, documentFolder](IStorageItem^ item)
	{
		if (item == nullptr || bIsFirstEnterApp)
		{
			Windows::Storage::StorageFolder^ installedLocation = Windows::ApplicationModel::Package::Current->InstalledLocation;
			return concurrency::create_task(installedLocation->GetFileAsync("developer_guide_uwp.pdf"))
				.then([this, documentFolder](Windows::Storage::StorageFile^ file)
			{
				return file->CopyAsync(documentFolder, file->Name, NameCollisionOption::ReplaceExisting);
			});
		}

		return concurrency::create_task([item]()
		{
			auto bIsFolder = item->IsOfType(Windows::Storage::StorageItemTypes::Folder);
			if (bIsFolder)
				MRUManager::DeleteFromMRUListByFilePath(item->Path);

			return dynamic_cast<Windows::Storage::StorageFile^>(item);
		});
	})
		.then([=](Windows::Storage::StorageFile^ file)
	{
		if (file != nullptr)
		{
			if (!MRUManager::GetStorageFileMRUToken(file->Path))
				MRUManager::AddStorageFileToMRUList(file);
		}

		m_MRUVectorItems = ref new Platform::Collections::Vector<RecentlyFileItem^>();
		InitMruFileItems();
		RecentlyListViewSource->Source = m_MRUVectorItems;
	}, concurrency::task_continuation_context::use_current())
		.then([=](concurrency::task<void> t)
	{
		try
		{
			t.get();
		}
		catch (...)
		{

		}
	});

	this->Loaded += ref new Windows::UI::Xaml::RoutedEventHandler(this, &CompletePDFViewer::RecentlyListPage::OnLoaded);
	this->Unloaded += ref new Windows::UI::Xaml::RoutedEventHandler(this, &CompletePDFViewer::RecentlyListPage::OnUnloaded);
}

int RecentlyListPage::GetViewMode(FileViewMode viewmode)
{
	return (int)viewmode;
}

FileViewMode RecentlyListPage::GetEnumViewMode(int viewmode)
{
	return (viewmode >= 0 && viewmode <= 1) ? (FileViewMode)viewmode : FileViewMode::List;
}

Platform::String^ RecentlyListPage::GetStringViewMode(FileViewMode viewmode)
{
	Platform::String^ viewstr;
	if (viewmode == FileViewMode::List)
	{
		viewstr = L"List";
	}
	else if (viewmode == FileViewMode::Thumb)
	{
		viewstr = L"Thumb";
	}

	return viewstr;
}

void RecentlyListPage::SetViewMode(Platform::String^ viewMode)
{
	if (viewMode == L"List")
	{
		m_ViewMode = FileViewMode::List;
	}
	else if (viewMode == L"Thumb")
	{
		m_ViewMode = FileViewMode::Thumb;
	}

	Windows::Storage::ApplicationDataContainer^ _localSettings = Windows::Storage::ApplicationData::Current->LocalSettings;
	auto _localSettingsValues = _localSettings->Values;
	auto _SettingDataCompositeValue = _localSettingsValues->HasKey(APPSETTINGKEY) ? dynamic_cast<Windows::Storage::ApplicationDataCompositeValue^>(_localSettingsValues->Lookup(APPSETTINGKEY)) : ref new Windows::Storage::ApplicationDataCompositeValue();
	_SettingDataCompositeValue->Insert(L"RecentDocumentViewMode", GetViewMode(m_ViewMode));
	_localSettingsValues->Insert(APPSETTINGKEY, _SettingDataCompositeValue);

}

void RecentlyListPage::ClearMruList()
{
	Windows::Storage::AccessCache::StorageApplicationPermissions::MostRecentlyUsedList->Clear();

	if (m_MRUVectorItems->Size > 0)
	{
		for (int i = m_MRUVectorItems->Size - 1; i > 0; i--)
		{
			m_MRUVectorItems->RemoveAt(i);
		}
	}
}

FileViewMode RecentlyListPage::LayoutMode::get()
{
	return m_ViewMode;
}

void RecentlyListPage::InitMruFileItems()
{
	ApplicationDataContainer^ localSettings = ApplicationData::Current->LocalSettings;
	auto values = localSettings->Values;
	ApplicationDataCompositeValue^ composite;

	auto _SettingDataCompositeValue = values->HasKey(APPSETTINGKEY) ? dynamic_cast<Windows::Storage::ApplicationDataCompositeValue^>(values->Lookup(APPSETTINGKEY)) : ref new Windows::Storage::ApplicationDataCompositeValue();

	auto resourceLoader = Windows::ApplicationModel::Resources::ResourceLoader::GetForCurrentView(L"Resources");
	Platform::String^ filenameStr = "";
	if (Windows::System::Profile::AnalyticsInfo::VersionInfo->DeviceFamily == "Windows.Desktop")
	{
		filenameStr = resourceLoader->GetString(L"QuickGuideText");
	}
	else
	{
		filenameStr = resourceLoader->GetString(L"QuickGuideMobileText");
	}

	LoadMruFileItems();
}

void RecentlyListPage::LoadMruFileItems()
{
	if (m_MRUVectorItems->Size == 0)
	{
		Windows::Storage::AccessCache::AccessListEntryView^ _entries = Windows::Storage::AccessCache::StorageApplicationPermissions::MostRecentlyUsedList->Entries;

		int count = _entries->Size;

		auto _iter = _entries->First();
		while (_iter->HasCurrent)
		{
			auto _value = _iter->Current;
			auto _item = ref new RecentlyFileItem();

			String^ filePath = _value.Metadata;
			_item->AutomationId = _value.Token;
			_item->FilePath = _value.Metadata;
			m_MRUVectorItems->Append(_item);
			_iter->MoveNext();
		}

		RecentlyFileItem^ openfileItem = ref new RecentlyFileItem("open file");
		auto resourceLoader = Windows::ApplicationModel::Resources::ResourceLoader::GetForCurrentView();
		String^ title = resourceLoader->GetString(L"OpenFileTitleText");
		openfileItem->FileName = title;
		openfileItem->ThumbnailWidth = 110;
		openfileItem->ThumbnailHeight = 92;
		openfileItem->FilePath = title;
		openfileItem->BorderThinknessValue = 0;
		m_MRUVectorItems->InsertAt(0, openfileItem);
	}

	int size = m_MRUVectorItems->Size;
	UpdateMRUControlViewState();
}

bool RecentlyListPage::JudgeIsSkyDriveFile(Platform::String^ filePath)
{
	bool isSkyDriveFile = false;

	std::wregex _pattern = std::wregex(L"^C:\\\\.+");

	auto _bIsMatch = std::regex_match(filePath->Data(), _pattern);
	if (_bIsMatch)
	{
		const wchar_t *seperator = wcsrchr(filePath->Data(), '\\');
		String^ fileName = seperator ? ref new Platform::String(seperator + 1) : filePath;

		Windows::Foundation::Collections::IVector<Platform::String^>^ pathList = ref new Platform::Collections::Vector<Platform::String^>();

		const wchar_t* logoData = fileName->Data();
		auto length = wcslen(logoData) + 1;
		wchar_t* data = new wchar_t[length];

		wcscpy_s(data, length, logoData);
		wchar_t* pszResult = NULL;
		wchar_t* splitData = wcstok_s(data, L";", &pszResult);

		String^ filepath = ref new String((wchar_t*)splitData);
		pathList->Append(filepath);
		while (splitData != NULL)
		{
			splitData = wcstok_s(pszResult, L";", &pszResult);
			if (splitData != NULL && wcslen(splitData) > 0)
			{
				filepath = ref new String((wchar_t*)splitData);
				pathList->Append(filepath);
			}
		}
		delete[] data;

		if (pathList && pathList->Size == 6)
		{
			isSkyDriveFile = true;
		}
	}

	return isSkyDriveFile;
}

void RecentlyListPage::UpdateMRUControlViewState()
{
	if (m_MRUVectorItems->Size > 0)
	{
		HasNotDocumentsPanel->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
		DocumentsPanel->Visibility = Windows::UI::Xaml::Visibility::Visible;
		if (m_ViewMode == FileViewMode::List)
		{
			ListModeMRUButtonPanel->Visibility = Windows::UI::Xaml::Visibility::Visible;
			ThumbModeMRUButtonPanel->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
		}
		else if (m_ViewMode == FileViewMode::Thumb)
		{
			ListModeMRUButtonPanel->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
			ThumbModeMRUButtonPanel->Visibility = Windows::UI::Xaml::Visibility::Visible;
		}
	}
}

void RecentlyListPage::OnNavigatedTo(Windows::UI::Xaml::Navigation::NavigationEventArgs^ e)
{
	auto _typeName = e->SourcePageType.Name;

	m_ParentPage = dynamic_cast<Windows::UI::Xaml::Controls::Page^>(e->Parameter);

	m_ThumbnailFileSaveCompletionToken = MRUManager::Current->ThumbnailFileSaveCompletion += ref new CompletePDFViewer::ThumbnailFileSaveCompletionEventHandler(this, &CompletePDFViewer::RecentlyListPage::OnThumbnailFileSaveCompletion);
}

void RecentlyListPage::OnNavigatedFrom(Windows::UI::Xaml::Navigation::NavigationEventArgs^ e)
{
	if (m_FileOpenPickerCts.get_token().is_cancelable() && !m_FileOpenPickerCts.get_token().is_canceled())
		m_FileOpenPickerCts.cancel();
}

void CompletePDFViewer::RecentlyListPage::MRUButtonPanel_ItemClick(Platform::Object^ sender, Windows::UI::Xaml::Controls::ItemClickEventArgs^ e)
{
	auto item = dynamic_cast<RecentlyFileItem^>(e->ClickedItem);
	curFileItem = item;
	if (item->BorderThinknessValue == 0)
	{
		if (m_bFileOpenPickerOpened)
			return;

		Windows::Storage::Pickers::FileOpenPicker^ openPicker = ref new Windows::Storage::Pickers::FileOpenPicker();
		openPicker->ViewMode = Windows::Storage::Pickers::PickerViewMode::Thumbnail;
		openPicker->SuggestedStartLocation = Windows::Storage::Pickers::PickerLocationId::DocumentsLibrary;
		openPicker->FileTypeFilter->Append(".pdf");

		m_bFileOpenPickerOpened = true;
		concurrency::create_task(openPicker->PickSingleFileAsync(), m_FileOpenPickerCts.get_token())
			.then([this](Windows::Storage::StorageFile^ file)
		{
			if (file)
				m_ParentPage->Frame->Navigate(Windows::UI::Xaml::Interop::TypeName(MainPage::typeid), file);
			m_bFileOpenPickerOpened = false;
		});
	}
	else
	{
		concurrency::create_task(Windows::Storage::AccessCache::StorageApplicationPermissions::MostRecentlyUsedList->GetFileAsync(item->AutomationId))
			.then([this, item](concurrency::task<Windows::Storage::StorageFile^> result)
		{
			try
			{
				Windows::Storage::StorageFile^ file = result.get();
				String^ filename = file->Name;
				String^ oldfilename = item->FileName;
				auto nAttributes = file->Attributes;
				auto pFileType = file->FileType;
				auto bIsFile = file->IsOfType(Windows::Storage::StorageItemTypes::File);
				auto bIsFolder = file->IsOfType(Windows::Storage::StorageItemTypes::Folder);
				
				if (filename != oldfilename || bIsFolder)
				{
					RemoveDeleteFile(item);
				}
				else
				{
					if (file && m_ParentPage)
						m_ParentPage->Frame->Navigate(Windows::UI::Xaml::Interop::TypeName(MainPage::typeid), file);
				}
			}
			catch (...)
			{
				OutputDebugString(L"File is deleted or moved...\n");
				RemoveDeleteFile(item);
			}
		});

	}
}

void CompletePDFViewer::RecentlyListPage::RemoveDeleteFile(RecentlyFileItem^ item)
{
	MRUManager::DeleteFromMRUListByToken(item->AutomationId);
	unsigned int index = -1;
	if (m_MRUVectorItems->IndexOf(item, &index))
		m_MRUVectorItems->RemoveAt(index);

	auto resouceLoader = Windows::ApplicationModel::Resources::ResourceLoader::GetForCurrentView(L"foxit.rdk.uiextensions/Resources");
	Platform::String^ dlgTips = resouceLoader->GetString(L"OpenFileErrorTips");
	Platform::String^ dlgTitle = resouceLoader->GetString(L"OpenFileErrorTitle");

	auto dialog = ref new Windows::UI::Popups::MessageDialog(dlgTips, dlgTitle);
	if (dialog != nullptr)
	{
		concurrency::create_task(dialog->ShowAsync(), m_TipsDlgCts.get_token());
	}
}

void CompletePDFViewer::RecentlyListPage::DownLoadDriveFile()
{
	if (!curFileItem)
		return;

}

void CompletePDFViewer::RecentlyListPage::OnThumbnailFileSaveCompletion(Platform::Object ^sender, CompletePDFViewer::ThumbnailFileSaveCompletionEventArgs ^args)
{
	auto _findIter = std::find_if(begin(m_MRUVectorItems), end(m_MRUVectorItems), [args](RecentlyFileItem^ item)
	{
		return item && item->AutomationId == args->Token;
	});

	if (_findIter != end(m_MRUVectorItems))
	{
		RecentlyFileItem^ _item = (*_findIter);
		(*_findIter)->ReGetThumbnailImage();

		concurrency::create_task(Windows::Storage::AccessCache::StorageApplicationPermissions::MostRecentlyUsedList->GetFileAsync(_item->AutomationId))
			.then([=](concurrency::task<Windows::Storage::StorageFile^> result)
		{
			try
			{
				result.get();

				(*_findIter)->ReGetThumbnailImage();
			}
			catch (...)
			{
				MRUManager::DeleteFromMRUListByToken(_item->AutomationId);
				unsigned int index = -1;
				if (m_MRUVectorItems->IndexOf(_item, &index))
					m_MRUVectorItems->RemoveAt(index);
			}
		});
	}
}


void CompletePDFViewer::RecentlyListPage::OnLoaded(Platform::Object ^sender, Windows::UI::Xaml::RoutedEventArgs ^e)
{
}


void CompletePDFViewer::RecentlyListPage::OnUnloaded(Platform::Object ^sender, Windows::UI::Xaml::RoutedEventArgs ^e)
{
	if (m_ThumbnailFileSaveCompletionToken.Value)
	{
		MRUManager::Current->ThumbnailFileSaveCompletion -= m_ThumbnailFileSaveCompletionToken;
		m_ThumbnailFileSaveCompletionToken.Value = 0;
	}

	if (m_TipsDlgCts.get_token().is_cancelable() && !m_TipsDlgCts.get_token().is_canceled())
	{
		m_TipsDlgCts.cancel();
	}
}


void CompletePDFViewer::RecentlyListPage::ThumbModeMRUButtonPanel_Loaded(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{
}

