﻿/**
 * Copyright (C) 2003-2018, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit Mobile PDF SDK to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */


#include "pch.h"
#include "StartPage.xaml.h"
#include "RecentlyListPage.xaml.h"

using namespace CompletePDFViewer;

using namespace Platform;
using namespace Windows::Foundation;
using namespace Windows::Foundation::Collections;
using namespace Windows::UI::Xaml;
using namespace Windows::UI::Xaml::Controls;
using namespace Windows::UI::Xaml::Controls::Primitives;
using namespace Windows::UI::Xaml::Data;
using namespace Windows::UI::Xaml::Input;
using namespace Windows::UI::Xaml::Media;
using namespace Windows::UI::Xaml::Navigation;
using namespace Windows::Storage;
using namespace Windows::Storage::Pickers;


StartPage::StartPage()
{
	InitializeComponent();

	OnInitializePage();
}

void StartPage::OnInitializePage()
{
	m_CommandBarIsCompact = true;
	m_PinCommandBarIsCompact = true;
	m_bFileOpenPickerOpened = false;

	Windows::Foundation::Collections::IVectorView<String^>^ langus = Windows::Globalization::ApplicationLanguages::Languages;

	ApplicationDataContainer^ localSettings = ApplicationData::Current->LocalSettings;
	auto values = localSettings->Values;
	ApplicationDataCompositeValue^ composite;

	auto _AppSettingDataCompositeValue = values->HasKey(APPSETTINGKEY) ? dynamic_cast<Windows::Storage::ApplicationDataCompositeValue^>(values->Lookup(APPSETTINGKEY)) : ref new Windows::Storage::ApplicationDataCompositeValue();
	int  m_recentlyPageFlag = 0;
	if (_AppSettingDataCompositeValue->HasKey(L"RecentlyOrPinPage"))
	{
		auto _obValue = _AppSettingDataCompositeValue->Lookup(L"RecentlyOrPinPage");
		if (_obValue)
			m_recentlyPageFlag = (_obValue->GetType()->FullName == int::typeid->FullName) ? ((Platform::IBox<int>^)(_obValue))->Value : 0;
	}

	if (m_recentlyPageFlag == 0)
	{
		RecentRadioButton->IsChecked = true;
		OpenPage(RecentlyListPage::typeid->FullName, RecentRadioButton->Label);
	}

	this->Loaded += ref new Windows::UI::Xaml::RoutedEventHandler(this, &CompletePDFViewer::StartPage::OnLoaded);
	this->Unloaded += ref new Windows::UI::Xaml::RoutedEventHandler(this, &CompletePDFViewer::StartPage::OnUnloaded);
}

void StartPage::UpdateCommandBarControl()
{
	auto pageClassName = SplitViewRootFrame->SourcePageType.Name;

	if (pageClassName == RecentlyListPage::typeid->FullName)
	{
		CommandBarContentPresenter->Visibility = Windows::UI::Xaml::Visibility::Visible;
		RecentCommandBarPanel->Visibility = Windows::UI::Xaml::Visibility::Visible;

		RecentlyListPage^ recentlyListPage = dynamic_cast<RecentlyListPage^>(SplitViewRootFrame->Content);

		ListViewBtn->Visibility = recentlyListPage->LayoutMode == FileViewMode::List ? Windows::UI::Xaml::Visibility::Collapsed : Windows::UI::Xaml::Visibility::Visible;
		ThumbViewBtn->Visibility = recentlyListPage->LayoutMode == FileViewMode::List ? Windows::UI::Xaml::Visibility::Visible : Windows::UI::Xaml::Visibility::Collapsed;

		SetCommandBarControl(m_CommandBarIsCompact);
		HeaderRoot->Height = m_CommandBarIsCompact ? 48 : 60;
	}
	else
	{
		CommandBarContentPresenter->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
		HeaderRoot->Height = 48;
	}
}

void StartPage::UpdatePageTitle()
{
	auto pageClassName = SplitViewRootFrame->SourcePageType.Name;

	Platform::String^ title = nullptr;
	if (pageClassName == RecentlyListPage::typeid->FullName)
	{
		title = RecentRadioButton->Label;
		RecentRadioButton->IsChecked = true;
	}

	if (title)
		PageTitleTextBlock->Text = title;
}

void StartPage::UpdatePageState()
{
	auto pageClassName = SplitViewRootFrame->SourcePageType.Name;

	PageBackButton->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
	Windows::UI::Xaml::Thickness _PageTitleTextBlockMargin = Windows::UI::Xaml::ThicknessHelper::FromLengths(0, 0, 0, 0);
	if (Windows::System::Profile::AnalyticsInfo::VersionInfo->DeviceFamily == L"Windows.Desktop")
	{
		_PageTitleTextBlockMargin = Windows::UI::Xaml::ThicknessHelper::FromLengths(24, 11, 0, 0);
	}
	else if (Windows::System::Profile::AnalyticsInfo::VersionInfo->DeviceFamily == L"Windows.Mobile")
	{
		_PageTitleTextBlockMargin = Windows::UI::Xaml::ThicknessHelper::FromLengths(10, 11, 0, 0);
	}

	PageTitleTextBlock->Margin = _PageTitleTextBlockMargin;
	UpdateCommandBarControl();
	UpdatePageTitle();
}

void StartPage::OpenPage(Platform::String^ pageClassName, Platform::String^ title)
{
	m_titleStr = pageClassName;

	HeaderRoot->Visibility = Windows::UI::Xaml::Visibility::Visible;
	if (PageTitleTextBlock->Text != title)
		PageTitleTextBlock->Text = title;

	if (SplitViewRootFrame->SourcePageType.Name != pageClassName)
	{
		Windows::UI::Xaml::Interop::TypeName pageType = { pageClassName, Windows::UI::Xaml::Interop::TypeKind::Custom };
		SplitViewRootFrame->Navigate(pageType, this);
	}

	UpdatePageState();
}

double StartPage::GetRelativePanelLength()
{
	if (PageSplitView->IsPaneOpen)
		return PageSplitView->OpenPaneLength;
	else
		return PageSplitView->CompactPaneLength;
}

void StartPage::SetCommandBarControl(bool bIsCompact)
{
	auto size = RecentCommandBarPanel->Children->Size;
	bool isCompact = bIsCompact;
	for (unsigned int i = 0; i < size; i++)
	{
		auto _item = RecentCommandBarPanel->Children->GetAt(i);
		auto _ICommandBarElement = dynamic_cast<Windows::UI::Xaml::Controls::ICommandBarElement^>(_item);
		if (_ICommandBarElement)
		{
			_ICommandBarElement->IsCompact = isCompact;
		}
	}

	m_CommandBarIsCompact = isCompact;
	HeaderRoot->Height = m_CommandBarIsCompact ? 44 : 60;
	CommandBarContentPresenter->Height = m_CommandBarIsCompact ? 44 : 60;
}

void StartPage::SplitViewToggle_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{
	PageSplitView->IsPaneOpen = !PageSplitView->IsPaneOpen;
}

void StartPage::RecentRadioButton_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{
	m_CommandBarIsCompact = true;

	OpenPage(RecentlyListPage::typeid->FullName, RecentRadioButton->Label);

	if (Windows::System::Profile::AnalyticsInfo::VersionInfo->DeviceFamily == L"Windows.Mobile")
		PageSplitView->IsPaneOpen = false;
}

void StartPage::ListViewBtn_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{
	if (SplitViewRootFrame->SourcePageType.Name == RecentlyListPage::typeid->FullName)
	{
		RecentlyListPage^ recentlyListPage = dynamic_cast<RecentlyListPage^>(SplitViewRootFrame->Content);
		if (recentlyListPage)
		{
			recentlyListPage->SetViewMode(L"List");
			recentlyListPage->UpdateMRUControlViewState();
		}

		UpdateCommandBarControl();
	}
}


void StartPage::ThumbViewBtn_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{
	if (SplitViewRootFrame->SourcePageType.Name == RecentlyListPage::typeid->FullName)
	{
		RecentlyListPage^ recentlyListPage = dynamic_cast<RecentlyListPage^>(SplitViewRootFrame->Content);
		if (recentlyListPage)
		{
			recentlyListPage->SetViewMode(L"Thumb");
			recentlyListPage->UpdateMRUControlViewState();
		}

		UpdateCommandBarControl();
	}
}


void StartPage::ClearListBtn_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{
	if (SplitViewRootFrame->SourcePageType.Name == RecentlyListPage::typeid->FullName)
	{
		RecentlyListPage^ recentlyListPage = dynamic_cast<RecentlyListPage^>(SplitViewRootFrame->Content);
		if (recentlyListPage)
			recentlyListPage->ClearMruList();
	}
}


void StartPage::RecentMoreBtn_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{
	bool isCompact = !m_CommandBarIsCompact;

	SetCommandBarControl(isCompact);
}


void StartPage::PageBackButton_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{
	if (SplitViewRootFrame->CanGoBack)
	{
		SplitViewRootFrame->GoBack();
		HeaderRoot->Visibility = Windows::UI::Xaml::Visibility::Visible;
		UpdatePageTitle();
		UpdatePageState();
	}
}

void StartPage::OnLoaded(Platform::Object ^sender, Windows::UI::Xaml::RoutedEventArgs ^e)
{
	Windows::Storage::ApplicationDataContainer^ _localSettings = Windows::Storage::ApplicationData::Current->LocalSettings;
	auto _localSettingsValues = _localSettings->Values;
	auto _SettingDataCompositeValue = _localSettingsValues->HasKey(APPSETTINGKEY) ? dynamic_cast<Windows::Storage::ApplicationDataCompositeValue^>(_localSettingsValues->Lookup(APPSETTINGKEY)) : ref new Windows::Storage::ApplicationDataCompositeValue();

	if (Windows::System::Profile::AnalyticsInfo::VersionInfo->DeviceFamily == L"Windows.Desktop")
	{
		bool bIsPaneOpen = true;
		if (_SettingDataCompositeValue->HasKey(L"IsSplitViewPanelOpen"))
		{
			auto _obValue = _SettingDataCompositeValue->Lookup(L"IsSplitViewPanelOpen");
			bIsPaneOpen = (_obValue->GetType()->FullName == bool::typeid->FullName) ? ((Platform::IBox<bool>^)(_obValue))->Value : false;
		}

		auto winRc = Windows::UI::Xaml::Window::Current->Bounds;
		if (winRc.Width <= 600)
		{
			if (bIsPaneOpen)
				bIsPaneOpen = false;
		}
		else
		{
			bIsPaneOpen = true;
		}

		PageSplitView->IsPaneOpen = bIsPaneOpen;
		_localSettingsValues->Insert(APPSETTINGKEY, _SettingDataCompositeValue);
	}

	m_SuspendingEventToken = Windows::UI::Xaml::Application::Current->Suspending += ref new Windows::UI::Xaml::SuspendingEventHandler(this, &CompletePDFViewer::StartPage::OnSuspending);

	if (Windows::Foundation::Metadata::ApiInformation::IsTypePresent("Windows.UI.Core.SystemNavigationManager"))
	{
		m_SystemNavigationBackRequestedEventToken = Windows::UI::Core::SystemNavigationManager::GetForCurrentView()->BackRequested += ref new Windows::Foundation::EventHandler<Windows::UI::Core::BackRequestedEventArgs ^>(this, &CompletePDFViewer::StartPage::OnBackRequested);
	}
}

void StartPage::OnUnloaded(Platform::Object ^sender, Windows::UI::Xaml::RoutedEventArgs ^e)
{
	Windows::Storage::ApplicationDataContainer^ _localSettings = Windows::Storage::ApplicationData::Current->LocalSettings;
	auto _localSettingsValues = _localSettings->Values;
	auto _SettingDataCompositeValue = _localSettingsValues->HasKey(APPSETTINGKEY) ? dynamic_cast<Windows::Storage::ApplicationDataCompositeValue^>(_localSettingsValues->Lookup(APPSETTINGKEY)) : ref new Windows::Storage::ApplicationDataCompositeValue();
	_SettingDataCompositeValue->Insert(L"IsSplitViewPanelOpen", PageSplitView->IsPaneOpen);
	_localSettingsValues->Insert(APPSETTINGKEY, _SettingDataCompositeValue);

	if (m_SuspendingEventToken.Value)
	{
		Windows::UI::Xaml::Application::Current->Suspending -= m_SuspendingEventToken;
		m_SuspendingEventToken.Value = 0;
	}

	if (Windows::Foundation::Metadata::ApiInformation::IsTypePresent("Windows.UI.Core.SystemNavigationManager"))
	{
		Windows::UI::Core::SystemNavigationManager::GetForCurrentView()->BackRequested -= m_SystemNavigationBackRequestedEventToken;
		m_SystemNavigationBackRequestedEventToken.Value = 0;
	}

	if (m_FileOpenPickerCts.get_token().is_cancelable() && !m_FileOpenPickerCts.get_token().is_canceled())
		m_FileOpenPickerCts.cancel();
}


void StartPage::OnSuspending(Platform::Object ^sender, Windows::ApplicationModel::SuspendingEventArgs ^e)
{
	Windows::Storage::ApplicationDataContainer^ _localSettings = Windows::Storage::ApplicationData::Current->LocalSettings;
	auto _localSettingsValues = _localSettings->Values;
	auto _SettingDataCompositeValue = _localSettingsValues->HasKey(APPSETTINGKEY) ? dynamic_cast<Windows::Storage::ApplicationDataCompositeValue^>(_localSettingsValues->Lookup(APPSETTINGKEY)) : ref new Windows::Storage::ApplicationDataCompositeValue();
	_SettingDataCompositeValue->Insert(L"IsSplitViewPanelOpen", PageSplitView->IsPaneOpen);
	_localSettingsValues->Insert(APPSETTINGKEY, _SettingDataCompositeValue);
}


void StartPage::OnBackRequested(Platform::Object ^sender, Windows::UI::Core::BackRequestedEventArgs ^args)
{
	bool bIsNeedGoBack = true;

	if (bIsNeedGoBack && SplitViewRootFrame->CanGoBack)
	{
		SplitViewRootFrame->GoBack();
		HeaderRoot->Visibility = Windows::UI::Xaml::Visibility::Visible;
		UpdatePageTitle();
		UpdatePageState();
		args->Handled = true;
	}
}

