﻿/**
 * Copyright (C) 2003-2018, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit Mobile PDF SDK to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */


#include "pch.h"
#include "MainPage.xaml.h"
#include <fstream>


using namespace FunctionDemo;

using namespace Platform;
using namespace Windows::Foundation;
using namespace Windows::Foundation::Collections;
using namespace Windows::UI::Xaml;
using namespace Windows::UI::Xaml::Controls;
using namespace Windows::UI::Xaml::Controls::Primitives;
using namespace Windows::UI::Xaml::Data;
using namespace Windows::UI::Xaml::Input;
using namespace Windows::UI::Xaml::Media;
using namespace Windows::UI::Xaml::Navigation;

using namespace Windows::Storage;
using namespace Windows::Storage::Streams;
using namespace concurrency;
using namespace Windows::UI::Xaml::Media::Imaging;
using namespace Windows::Graphics::Imaging;

using namespace foxit;

MainPage::MainPage()
{
	InitializeComponent();

	//Copy the test files from installed location to document library.
	concurrency::create_task(Windows::Storage::KnownFolders::DocumentsLibrary->CreateFolderAsync(L"input_files", Windows::Storage::CreationCollisionOption::OpenIfExists))
		.then([=](Windows::Storage::StorageFolder^ folder)
	{
		if (folder)
			m_InputFilesFolder = folder;
		return  Windows::Storage::KnownFolders::DocumentsLibrary->CreateFolderAsync(L"output_files", Windows::Storage::CreationCollisionOption::OpenIfExists);
	})
		.then([=](Windows::Storage::StorageFolder^ folder)
	{
		if (folder)
			m_OutputFilesFolder = folder;
		return Windows::ApplicationModel::Package::Current->InstalledLocation->GetFolderAsync(L"TestFiles");
	})
		.then([this](Windows::Storage::StorageFolder^ folder)
	{
		return folder->GetFilesAsync();
	})
		.then([this](IVectorView<StorageFile^>^ files)
	{
		for each (StorageFile^ file  in files)
		{
			concurrency::create_task(file->CopyAsync(m_InputFilesFolder, file->Name, NameCollisionOption::ReplaceExisting));
		}
	});
}

//Extract the texts on PDF to a text file.
void FunctionDemo::MainPage::pdf2txt_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{
	create_task(m_InputFilesFolder->GetFileAsync("FoxitBigPreview.pdf")).then([=](StorageFile^ file) {
		try
		{
			m_doc = ref new foxit::pdf::PDFDoc(file, false, 0, 0);
			if (!m_doc)
			{
				return;
			}
			foxit::common::ErrorCode code = m_doc->LoadW("");
			if (code != foxit::common::ErrorCode::e_ErrSuccess) {
				return;
			}
		}
		catch (Platform::Exception^ e)
		{
			OutputDebugString(e->Message->Data());
		}
	}).then([=]()
	{
		return m_OutputFilesFolder->CreateFileAsync("pdf2txt.txt", CreationCollisionOption::ReplaceExisting);
	}).then([=](StorageFile^ file)
	{
		try
		{
			String^ text = nullptr;
			for (int i = 0; i < m_doc->GetPageCount(); i++)
			{
				foxit::pdf::PDFPage^ page = m_doc->GetPage(i);
				page->StartParse((int)foxit::pdf::ParseFlags::e_ParsePageTextOnly, nullptr, false);
				foxit::pdf::TextPage^ textSelect = ref new foxit::pdf::TextPage(page, 0);
				if (!textSelect)
				{
					continue;
				}
				text += textSelect->GetChars(0, -1);
				if (!text)
				{
					continue;
				}
			}
			//save
			create_task(FileIO::WriteTextAsync(file, text));
			this->showMesseageDialog("", "The output file is saved to:" + file->Path);
			
		}
		catch (Platform::Exception^ e)
		{
			OutputDebugString(e->Message->Data());
		}
	});
}


//Edit the bookmarks title, color, style... then save them to a new pdf.
void FunctionDemo::MainPage::outline_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{
	create_task(m_InputFilesFolder->GetFileAsync("Outline.pdf")).then([=](StorageFile^ file) {
		try
		{
			m_doc = ref new foxit::pdf::PDFDoc(file, false, 0, 0);
			if (!m_doc)
			{
				return;
			}
			foxit::common::ErrorCode code = m_doc->LoadW("");
			if (code != foxit::common::ErrorCode::e_ErrSuccess) {
				return;
			}

			foxit::pdf::Bookmark^ root = m_doc->GetRootBookmark();
			this->setProperties(root);
		}
		catch (Platform::Exception^ e)
		{
			OutputDebugString(e->Message->Data());
		}
	}).then([=]()
	{
		return m_OutputFilesFolder->CreateFileAsync("Output_Outline.pdf", CreationCollisionOption::ReplaceExisting);
	}).then([=](StorageFile^ file)
	{
		try
		{
			foxit::common::Progressive^ progress = m_doc->StartSaveAs(file, (int)foxit::pdf::SaveFlags::e_SaveFlagNormal, nullptr);
			if (progress)
			{
				foxit::common::State state = progress->Continue();
				while(state == foxit::common::State::e_ToBeContinued) 
					state = progress->Continue();
			}
			this->showMesseageDialog("", "The output file is saved to:" + file->Path);
		}
		catch (Platform::Exception^ e)
		{
		}
	});
}

void FunctionDemo::MainPage::setProperties(foxit::pdf::Bookmark^ bookmark)
{
	if(!bookmark || bookmark->IsEmpty()) return;
	foxit::pdf::Bookmark^ bm = nullptr;
	for (bm = bookmark->GetFirstChild(); bm != nullptr && !bm->IsEmpty(); bm = bm->GetNextSibling())
	{
		String^ title = bm->GetTitle();
		bm->SetTitle(title + "_renamed");
		bm->SetColor(0x0000ffff);
		bm->SetStyle((int)foxit::pdf::Style::e_StyleBold);

		foxit::pdf::Bookmark^ child = bm->GetFirstChild();
		if (child && !child->IsEmpty())
		{
			this->setProperties(bm);
		}
	}
}

void FunctionDemo::MainPage::showMesseageDialog(Platform::String^ title, Platform::String^ content)
{
	auto messageDialog = ref new Windows::UI::Popups::MessageDialog(content, title);
	messageDialog->Commands->Append(ref new Windows::UI::Popups::UICommand("OK", nullptr, PropertyValue::CreateInt32(0)));
	messageDialog->DefaultCommandIndex = 0;
	messageDialog->ShowAsync();
}

//Add some text markup annoations to a PDF file.
void FunctionDemo::MainPage::annotation_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{
	create_task(m_InputFilesFolder->GetFileAsync("Annotations.pdf")).then([=](StorageFile^ file) {
		try
		{
			m_doc = ref new foxit::pdf::PDFDoc(file, false, 0, 0);
			if (!m_doc)
			{
				return;
			}
			foxit::common::ErrorCode code = m_doc->LoadW("");
			if (code != foxit::common::ErrorCode::e_ErrSuccess) {
				return;
			}

			foxit::pdf::PDFPage^ page = m_doc->GetPage(0);
			foxit::common::Progressive^ state = page->StartParse(0, nullptr, false);

			foxit::pdf::annots::AnnotType annotType[] = {
				foxit::pdf::annots::AnnotType::e_Highlight,
				foxit::pdf::annots::AnnotType::e_Underline,
				foxit::pdf::annots::AnnotType::e_StrikeOut,
				foxit::pdf::annots::AnnotType::e_Squiggly
			};

			float opacity[] = { 0.3f, 0.6f, 0.9f };
			unsigned int color[] = { 0xffff0000, 0xff00ff00, 0xff0000ff };
			foxit::common::fxcrt::RectF^ rect = nullptr;
			for (int i = 0; i < 3; i++)
			{
				rect = ref new foxit::common::fxcrt::RectF(100 + (float) i * 160, 180, 120 + (float)i * 160, 200);
				foxit::pdf::annots::Annot^ annot = page->AddAnnot(foxit::pdf::annots::AnnotType::e_Note, rect);
				if (!annot)
				{
					continue;
				}
				foxit::pdf::annots::Note^ note = ref new foxit::pdf::annots::Note(annot);
				note->SetIconName("Comment");
				note->SetBorderColor(color[i]);
				note->ResetAppearanceStream();
			}

			const wchar_t* charArray[] = { L"Highlight_1", L"Highlight_2", L"Highlight_3",
				L"Underline_1", L"Underline_2", L"Underline_3",
				L"Strikeout_1", L"Strikeout_2", L"Strikeout_3",
				L"Squiggly_1", L"Squiggly_2", L"Squiggly_3" };
			int arraySize = sizeof(charArray)/sizeof(wchar_t*);
			Array<String^>^ searchText = ref new Array<String^>(arraySize);
			for (int i = 0; i < 12; i++)
			{
				String^ tmp = ref new String(charArray[i]);
				searchText->set(i, tmp);
			}

			int cntOfTexts = searchText->Length;
			for (int i = 0; i < cntOfTexts; i++) {
				foxit::pdf::TextSearch^ textSearch = ref new foxit::pdf::TextSearch(page->GetDocument(), nullptr);
				if (!textSearch) {
					continue;
				}

				textSearch->SetPattern(searchText->get(i));
				bool bMatched = textSearch->FindNext();
				if (bMatched) {
					auto pRects = textSearch->GetMatchRects();
					int cntOfMatchRect = pRects->GetSize();
					
					foxit::pdf::annots::QuadPointsArray^ quadArray = ref new foxit::pdf::annots::QuadPointsArray();

					for (int j = 0; j < cntOfMatchRect; j++) {
						foxit::common::fxcrt::RectF^ rect = pRects->GetAt(j);
						foxit::common::fxcrt::PointF^ point1 = ref new foxit::common::fxcrt::PointF(rect->left, rect->top);
						foxit::common::fxcrt::PointF^ point2 = ref new foxit::common::fxcrt::PointF(rect->right, rect->top);
						foxit::common::fxcrt::PointF^ point3 = ref new foxit::common::fxcrt::PointF(rect->left, rect->bottom);
						foxit::common::fxcrt::PointF^ point4 = ref new foxit::common::fxcrt::PointF(rect->right, rect->bottom);
				
						foxit::pdf::annots::QuadPoints^ quadPoints = ref new foxit::pdf::annots::QuadPoints(point1, point2, point3, point4);
						quadArray->Add(quadPoints);
					}

					foxit::common::fxcrt::RectF^ rect = ref new foxit::common::fxcrt::RectF();
					foxit::pdf::annots::Annot^ annot = page->AddAnnot(annotType[i / 3], rect);
					foxit::pdf::annots::TextMarkup^ markup = ref new foxit::pdf::annots::TextMarkup(annot);
					foxit::pdf::annots::ITextMarkup^ textMarkup = markup;
					textMarkup->SetQuadPoints(quadArray);
					textMarkup->SetBorderColor(color[i % 3]);
					textMarkup->SetOpacity(opacity[i % 3]);
					textMarkup->ResetAppearanceStream();
				}
			}
		}
		catch (Platform::Exception^ e)
		{
			OutputDebugString(e->Message->Data());
		}
	}).then([=]()
	{
		return m_OutputFilesFolder->CreateFileAsync("Output_Annotations.pdf", CreationCollisionOption::ReplaceExisting);
	}).then([=](StorageFile^ file)
	{
		try
		{
			m_doc->StartSaveAs(file, 0, nullptr);
			this->showMesseageDialog("", "The output file is saved to:" + file->Path);
		}
		catch (Platform::Exception^ e)
		{
		}
	});
}

//Extract the document metadata info to a text file.
void FunctionDemo::MainPage::docinfo_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{
	create_task(m_InputFilesFolder->GetFileAsync("FoxitBigPreview.pdf")).then([=](StorageFile^ file) {
		try
		{
			m_doc = ref new foxit::pdf::PDFDoc(file, false, 0, 0);
			if (!m_doc)
			{
				return;
			}
			foxit::common::ErrorCode code = m_doc->LoadW("");
			if (code != foxit::common::ErrorCode::e_ErrSuccess) {
				return;
			}

			int nPage = m_doc->GetPageCount();
			if (nPage < 1) {
				return;
			}
			String^ str = "PageCount" + nPage + "pages";
			Array<String^>^ itemArray = ref new Array<String^>(5);
			foxit::pdf::Metadata^ metadata = ref new foxit::pdf::Metadata(m_doc);
			itemArray->set(0, "Title: " + metadata->GetValues("Title")->GetAt(0));
			itemArray->set(1, "Author: " + metadata->GetValues("Author")->GetAt(0));
			itemArray->set(2, "Subject: " + metadata->GetValues("Subject")->GetAt(0));
			itemArray->set(3, "Keywords: " + metadata->GetValues("Keywords")->GetAt(0));
			itemArray->set(4, "PageCount: " + nPage.ToString());
			create_task(m_OutputFilesFolder->CreateFileAsync("Output_Docinfo.txt", CreationCollisionOption::ReplaceExisting)).then([=](StorageFile^ newFile) {
				String^ content = nullptr;
				for (int i = 0; i < 5; i++)
				{
					content += itemArray->get(i) + "\r\n";
				}
				this->showMesseageDialog("", "The output file is saved to:" + newFile->Path);
				return create_task(FileIO::WriteTextAsync(newFile, content));
			});
		}
		catch (Platform::Exception^ e)
		{
			OutputDebugString(e->Message->Data());
		}
	});
}

//Render the pdf page, then save it to a bitmap file.
void FunctionDemo::MainPage::render_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{
	std::shared_ptr<foxit::common::Bitmap^> spBitmap = std::make_shared<foxit::common::Bitmap^>(nullptr);
	create_task(m_InputFilesFolder->GetFileAsync("FoxitBigPreview.pdf")).then([=](StorageFile^ file) {
		try
		{
			m_doc = ref new foxit::pdf::PDFDoc(file, false, 0, 0);
			if (!m_doc)
			{
				return;
			}
			
			foxit::common::ErrorCode code = m_doc->LoadW("");
			if (code != foxit::common::ErrorCode::e_ErrSuccess) {
				return;
			}

			foxit::pdf::PDFPage^ page = m_doc->GetPage(0);
			if (!page->IsParsed())
			{
				page->StartParse(0, nullptr, false);
			}
			//create bitmap
			int pageWidth = (int)page->GetWidth();
			int pageHeight = (int)page->GetHeight();
			*spBitmap = ref new foxit::common::Bitmap(pageWidth, pageHeight, foxit::common::DIBFormat::e_DIBRgb32, nullptr, 0);
			if (!(*spBitmap)) {
				return;
			}
			(*spBitmap)->FillRect(0xffffffff, nullptr);

			//init renderer
			foxit::common::Renderer^ render = ref new foxit::common::Renderer((*spBitmap), false);
			if (!render) {
				return;
			}

			//set different kinds of properties
			render->SetRenderContentFlags((int)foxit::common::ContentFlag::e_RenderPage);
			render->SetTransformAnnotIcon(true);
			render->SetColorMode(foxit::common::ColorMode::e_ColorModeNormal);
			render->SetForceHalftone(true);

			foxit::common::fxcrt::Matrix2D^ matrix = page->GetDisplayMatrix(0, 0, pageWidth, pageHeight, foxit::common::Rotation::e_Rotation0);
			render->StartRender(page, matrix, nullptr);

		}
		catch (Platform::Exception^ e)
		{
			OutputDebugString(e->Message->Data());
		}
	}).then([=]()
	{
		return m_OutputFilesFolder->CreateFileAsync("Output_Render.bmp", CreationCollisionOption::ReplaceExisting);
	}).then([=](StorageFile^ file)
	{
		foxit::common::Image^ img = ref new foxit::common::Image();
		img->AddFrame((*spBitmap));
		if(img->SaveAs(file, "bmp"))
			this->showMesseageDialog("", "The output file is saved to:" + file->Path);
		else
			this->showMesseageDialog("", "error!");
	});
	
} 