/**
 * Copyright (C) 2003-2018, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit Mobile PDF SDK to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */


#include "pch.h"
#include "AnnotBase.h"
#include "AnnotCreaterFactory.h"
#include "Annot_ReplaceText.h"
#include "Annot_FreeText.h"

using namespace foxit::rdk::uiextensions;

using namespace Platform;
using namespace Windows::UI::Xaml;


BaseAnnot::BaseAnnot()
{
	_PageIndex = -1;
	_AnnoIndex = -1;
	_Opacity = 1.0;
	_Color = Windows::UI::Colors::Transparent;
	_AnnotBorder = ref new AnnotBorderWrap();
	_PageHandleWrap = ref new PDFPageWrapper();
}

bool  BaseAnnot::IsModifyProperty::get()
{
	return _IsModify;
}

void   BaseAnnot::IsModifyProperty::set(bool value)
{
	if (_IsModify != value)
	{
		_IsModify = value;
	}
}

int BaseAnnot::PageIndex::get()
{
	return _PageIndex;
}

int BaseAnnot::AnnoIndex::get()
{
	return _AnnoIndex;
}

Platform::String^ BaseAnnot::Name::get()
{
	return _Name;
}

void BaseAnnot::Name::set(Platform::String^ value)
{
	if (_Name != value)
	{
		_Name = value;
		if (_PdfAnnot && _PdfAnnot->PDF_Annot)
		{
			_PdfAnnot->PDF_Annot->SetUniqueID(value);
			NotifyDocumentChanged();
		}
	}
}

Platform::String^ BaseAnnot::Title::get()
{
	return _Title;
}

void BaseAnnot::Title::set(Platform::String^ value)
{
	if (_Title != value)
	{
		_Title = value;
		if (_PdfAnnot && _PdfAnnot->PDF_Annot)
		{
			foxit::pdf::annots::IMarkup^ markup = dynamic_cast<foxit::pdf::annots::IMarkup^>(FXDocUtil::DynamicCastBaseAnnot(_PdfAnnot->PDF_Annot));
			if (markup)
			{
				markup->SetTitle(value);
			}
			NotifyDocumentChanged();
		}
	}
}

Platform::String^ BaseAnnot::Contents::get()
{
	return _Contents;
}

void BaseAnnot::Contents::set(Platform::String^ value)
{
	if (_Contents != value)
	{
		_Contents = value;
		if (_PdfAnnot && _PdfAnnot->PDF_Annot)
		{
			_PdfAnnot->PDF_Annot->SetContent(value);
			NotifyDocumentChanged();
		}
	}
}

Windows::UI::Color BaseAnnot::Color::get()
{
	return _Color;
}

void BaseAnnot::Color::set(Windows::UI::Color value)
{
	if (!(_Color.A == value.A && _Color.R == value.R && _Color.G == value.G && _Color.B == value.B))
	{
		_Color = value;
		if (_PdfAnnot && _PdfAnnot->PDF_Annot)
		{
			FS_ARGB color = (_Color.A << 24) | (_Color.R << 16) | (_Color.G << 8) | _Color.B;
			_PdfAnnot->PDF_Annot->SetBorderColor(color);
			_PdfAnnot->PDF_Annot->ResetAppearanceStream();

			if (_AnnotType == AnnotationType::FreeText)
			{
				foxit::common::fxcrt::RectF^ rc = _PdfAnnot->PDF_Annot->GetRect();
				{
					_MinRect = FoxitRectConverter::ConvertFromFoxitRectF(rc);
					FreeTextAnnot^ replaceTextAnnot = dynamic_cast<FreeTextAnnot^>(this);
					replaceTextAnnot->Page_HightLightRect = _MinRect;
				}
			}
			NotifyDocumentChanged();
		}
	}
}

double BaseAnnot::Opacity::get()
{
	return _Opacity;
}

void BaseAnnot::Opacity::set(double value)
{
	if (_Opacity != value)
	{
		_Opacity = value;
		if (_PdfAnnot && _PdfAnnot->PDF_Annot)
		{
			foxit::pdf::annots::IMarkup^ markup = dynamic_cast<foxit::pdf::annots::IMarkup^>(FXDocUtil::DynamicCastBaseAnnot(_PdfAnnot->PDF_Annot));
			markup->SetOpacity((FS_FLOAT)value);
			markup->ResetAppearanceStream();

			if (_AnnotType == AnnotationType::FreeText)
			{
				foxit::common::fxcrt::RectF^ rc = markup->GetRect();
				_MinRect = FoxitRectConverter::ConvertFromFoxitRectF(rc);
				FreeTextAnnot^ replaceTextAnnot = dynamic_cast<FreeTextAnnot^>(this);
				replaceTextAnnot->Page_HightLightRect = _MinRect;
			}
			NotifyDocumentChanged();
		}
	}
}

Windows::Foundation::Rect BaseAnnot::MinRect::get()
{
	return _MinRect;
}

void BaseAnnot::MinRect::set(Windows::Foundation::Rect value)
{
	if (_MinRect != value)
	{
		_MinRect = value;
		foxit::common::fxcrt::RectF^ fsrect;
		fsrect = FoxitRectConverter::ConvertToFoxitRectF(_MinRect);
		_PdfAnnot->PDF_Annot->Move(fsrect);
		_PdfAnnot->PDF_Annot->ResetAppearanceStream();

		if (_AnnotType == AnnotationType::Ink)
		{
			PencilAnnot^ inkAnnot = dynamic_cast<PencilAnnot^>(this);
			if (inkAnnot)
				inkAnnot->UpdatePathData();
		}
		if (_AnnotType == AnnotationType::FreeText)
		{
			foxit::common::fxcrt::RectF^ rc = _PdfAnnot->PDF_Annot->GetRect();
			_MinRect = FoxitRectConverter::ConvertFromFoxitRectF(rc);
			FreeTextAnnot^ replaceTextAnnot = dynamic_cast<FreeTextAnnot^>(this);
			replaceTextAnnot->Page_HightLightRect = _MinRect;
		}

		NotifyDocumentChanged();
	}
}

AnnotBorderWrap^ BaseAnnot::AnnotBorder::get()
{
	return _AnnotBorder;
}

void BaseAnnot::AnnotBorder::set(AnnotBorderWrap^ value)
{
	_AnnotBorder = value;
}

double BaseAnnot::BorderWidth::get()
{
	return _AnnotBorder ? _AnnotBorder->BorderWidth : 0.0;
}

Windows::Foundation::DateTime BaseAnnot::CreationDateTime::get()
{
	return _CreationDateTime;
}

void BaseAnnot::CreationDateTime::set(Windows::Foundation::DateTime value)
{
	if (_CreationDateTime.UniversalTime != value.UniversalTime)
	{
		_CreationDateTime = value;
		if (_PdfAnnot && _PdfAnnot->PDF_Annot)
		{
			auto _datetime = FoxitTypeConvert::ConvertToFoxitDateTime(value);
			foxit::pdf::annots::IMarkup^ markup = dynamic_cast<foxit::pdf::annots::IMarkup^>(FXDocUtil::DynamicCastBaseAnnot(_PdfAnnot->PDF_Annot));
			markup->SetCreationDateTime(_datetime);
			NotifyDocumentChanged();
		}
	}
}

Windows::Foundation::DateTime BaseAnnot::ModifiedDateTime::get()
{
	return _ModifiedDateTime;
}

void BaseAnnot::ModifiedDateTime::set(Windows::Foundation::DateTime value)
{
	if (_ModifiedDateTime.UniversalTime != value.UniversalTime)
	{
		_ModifiedDateTime = value;
		if (_PdfAnnot && _PdfAnnot->PDF_Annot)
		{
			auto _datetime = FoxitTypeConvert::ConvertToFoxitDateTime(value);
			foxit::pdf::annots::IMarkup^ markup = dynamic_cast<foxit::pdf::annots::IMarkup^>(FXDocUtil::DynamicCastBaseAnnot(_PdfAnnot->PDF_Annot));
			markup->SetCreationDateTime(_datetime);
			NotifyDocumentChanged();
		}
	}
}

void BaseAnnot::BorderWidth::set(double value)
{
	if (_AnnotBorder && _AnnotBorder->BorderWidth != (float)value)
	{
		_AnnotBorder->BorderWidth = (float)value;
		foxit::common::fxcrt::FloatArray^ dashes = ref new foxit::common::fxcrt::FloatArray();
		for (int i = 0; i < _AnnotBorder->DashCount; i++)
		{
			dashes->Add(_AnnotBorder->DashPattern[i]);
		}
		foxit::pdf::annots::BorderInfo^ borderInfo = ref new foxit::pdf::annots::BorderInfo(
			(float)value,
			FoxitTypeConvert::ConvertToFoxitAnnotBorderStyle(_AnnotBorder->BorderStyle),
			_AnnotBorder->CloudIntensity,
			_AnnotBorder->DashPhase,
			dashes
		);

		if (_PdfAnnot && _PdfAnnot->PDF_Annot)
		{

			_PdfAnnot->PDF_Annot->SetBorderInfo(borderInfo);
			_PdfAnnot->PDF_Annot->ResetAppearanceStream();
			NotifyDocumentChanged();
		}
	}
}

AnnotationType BaseAnnot::AnnotType::get()
{
	return _AnnotType;
}

IAnnotManager^ BaseAnnot::AnnotMgr::get()
{
	return _AnnotMgrWeakRef.Resolve<IAnnotManager>();
}

PDFAnnotHandle^ BaseAnnot::AnnotHandleWrap::get()
{
	return _PdfAnnot;
}

Platform::Object^ BaseAnnot::PageHandleWrap::get()
{
	return _PageHandleWrap;
}

Platform::String^ BaseAnnot::GetAnnotTypeString(AnnotationType type)
{
	Platform::String^ typestr = nullptr;
	switch (type)
	{
	case AnnotationType::NonSurport:
		break;
	case AnnotationType::HightLight:
		typestr = FSPDF_ANNOTTYPE_HIGHLIGHT;
		break;
	case AnnotationType::Link:
		typestr = FSPDF_ANNOTTYPE_LINK;
		break;
	case AnnotationType::UnderLine:
		typestr = FSPDF_ANNOTTYPE_UNDERLINE;
		break;
	case AnnotationType::StrikeOut:
		typestr = FSPDF_ANNOTTYPE_STRIKEOUT;
		break;
	case AnnotationType::Square:
		typestr = FSPDF_ANNOTTYPE_SQUARE;
		break;
	case AnnotationType::Circle:
		typestr = FSPDF_ANNOTTYPE_CIRCLE;
		break;
	case AnnotationType::Line:
		typestr = FSPDF_ANNOTTYPE_LINE;
		break;
	case AnnotationType::Ink:
		typestr = FSPDF_ANNOTTYPE_INK;
		break;
	case AnnotationType::Squiggly:
		typestr = FSPDF_ANNOTTYPE_SQUIGGLY;
		break;
	case AnnotationType::Text:
		typestr = FSPDF_ANNOTTYPE_TEXT;
		break;
	case AnnotationType::FreeText:
		typestr = FSPDF_ANNOTTYPE_FREETEXT;
		break;
	case AnnotationType::Caret:
		typestr = FSPDF_ANNOTTYPE_CARET;
		break;
	case AnnotationType::Stamp:
		typestr = FSPDF_ANNOTTYPE_STAMP;
		break;
	case AnnotationType::Popup:
		typestr = FSPDF_ANNOTTYPE_POPUP;
		break;
	case AnnotationType::Widget:
		typestr = FSPDF_ANNOTTYPE_WIDGET;
		break;
	case AnnotationType::None:
		break;
	default:
		break;
	}

	return typestr;
}

void BaseAnnot::NotifyDocumentChanged()
{
	if (AnnotMgr)
	{
		AnnotMgr->OnNotifyAnnotModify(this);
		AnnotMgr->NotifyDocumentChanged();
		AnnotMgr->UpdatePageView(_PageIndex);
	}
}

bool BaseAnnot::IsVisibity()
{
	return false;
}

bool BaseAnnot::IsHitTest(Windows::Foundation::Point pagePoint)
{
	return false;
}

bool BaseAnnot::IsHitTestRange(Windows::Foundation::Point pagePoint)
{
	return _MinRect.Contains(pagePoint);
}

bool BaseAnnot::OnLoadAnnot(IAnnotManager^ annotmgr, Platform::Object^ pageObj, int annotIndex)
{
	foxit::pdf::PDFPage^ page = (foxit::pdf::PDFPage^)pageObj;

	foxit::pdf::annots::Annot^ annot = page->GetAnnot(annotIndex);

	if (page == nullptr || annot == nullptr)
		return false;

	_PageHandleWrap->Pointer = page;
	_AnnotMgrWeakRef = annotmgr;
	_PdfAnnot = ref new PDFAnnotHandle(annot);
	_PageIndex = annotIndex;

	FS_INT32 index = -1;
	_Name = annot->GetUniqueID();
	foxit::pdf::annots::IMarkup^ pIMarkup = annot->IsMarkup() ? ref new foxit::pdf::annots::Markup(annot) : nullptr;
	_Title = pIMarkup ? pIMarkup->GetTitle() : "";

	_Contents = annot->GetContent();

	foxit::common::fxcrt::RectF^ rc = annot->GetRect();
	_MinRect = FoxitRectConverter::ConvertFromFoxitRectF(rc);

	FS_ARGB color = annot->GetBorderColor();
	_Color = Windows::UI::ColorHelper::FromArgb(color >> 24 & 0xff, color >> 16 & 0xff, color >> 8 & 0xff, color & 0xff);

	FS_FLOAT opacity = (FS_FLOAT)(pIMarkup ? pIMarkup->GetOpacity() : 1.0);
	_Opacity = opacity;

	if (_Opacity < 0.0 || _Opacity > 1.0)
		_Opacity = 1.0;

	foxit::pdf::annots::BorderInfo^ border = annot->GetBorderInfo();
	_AnnotBorder->BorderWidth = border->width;
	_AnnotBorder->BorderStyle = FoxitTypeConvert::ConvertFromFoxitAnnotBorderStyle(border->style);
	_AnnotBorder->CloudIntensity = border->cloud_intensity;
	_AnnotBorder->DashPhase = border->dash_phase;
	_AnnotBorder->DashCount = border->dashes->GetSize();

	auto dashes = border->dashes;
	auto dashesSize = dashes->GetSize();
	for (int i = 0; i < dashesSize; i++)
	{
		_AnnotBorder->DashPattern[i] = dashes->GetAt(i);
	}

	foxit::common::DateTime^ _dateTime = pIMarkup ? pIMarkup->GetCreationDateTime() : nullptr;
	_CreationDateTime = FoxitTypeConvert::ConvertFromFoxitDateTime(_dateTime);

	_dateTime = annot->GetModifiedDateTime();
	_ModifiedDateTime = FoxitTypeConvert::ConvertFromFoxitDateTime(_dateTime);

	return true;
}

bool BaseAnnot::OnLoadAnnot(IAnnotManager^ annotmgr, Platform::Object^ pageObj, PDFAnnotHandle^ annot)
{
	foxit::pdf::PDFPage^ page = dynamic_cast<foxit::pdf::PDFPage^>(pageObj);
	if (page == nullptr || annot == nullptr)
		return false;

	_AnnotMgrWeakRef = annotmgr;
	_PdfAnnot = annot;
	_PageHandleWrap->Pointer = page;

	FS_INT32 index = page->GetIndex();

	_PageIndex = index;
	foxit::pdf::annots::IAnnot^ fsAnnot = FXDocUtil::DynamicCastBaseAnnot(annot->PDF_Annot);

	index = fsAnnot->GetIndex();
	_AnnoIndex = index;
	_Name = fsAnnot->GetUniqueID();
	
	foxit::pdf::annots::IMarkup^ markup = annot->PDF_Annot->IsMarkup()?ref new foxit::pdf::annots::Markup(annot->PDF_Annot):nullptr;
	if (markup)
	{
		_Title = markup->GetTitle();
		_Contents = markup->GetContent();
	}

	foxit::common::fxcrt::RectF^ rc = fsAnnot->GetRect();
	_MinRect = FoxitRectConverter::ConvertFromFoxitRectF(rc);
	

	FS_ARGB color = fsAnnot->GetBorderColor();
	_Color = Windows::UI::ColorHelper::FromArgb(color >> 24 & 0xff, color >> 16 & 0xff, color >> 8 & 0xff, color & 0xff);

	if (markup)
	{
		FS_FLOAT opacity = markup->GetOpacity();
		_Opacity = opacity;

		if (_Opacity < 0.0 || _Opacity > 1.0)
			_Opacity = 1.0;
	}

	foxit::pdf::annots::BorderInfo^ border = fsAnnot->GetBorderInfo();
	if (border)
	{
		_AnnotBorder->BorderWidth = border->width;
		_AnnotBorder->BorderStyle = FoxitTypeConvert::ConvertFromFoxitAnnotBorderStyle(border->style);
		_AnnotBorder->CloudIntensity = border->cloud_intensity;
		_AnnotBorder->DashPhase = border->dash_phase;
		_AnnotBorder->DashCount = border->dashes->GetSize();
		for (int i = 0; i < _AnnotBorder->DashCount; i++)
		{
			_AnnotBorder->DashPattern[i] = border->dashes->GetAt(i);
		}
	}

	if (markup)
	{
		try
		{
			foxit::common::DateTime^ _dateTime = markup->GetCreationDateTime();
			if (_dateTime->year != 0)
				_CreationDateTime = FoxitTypeConvert::ConvertFromFoxitDateTime(_dateTime);
		}
		catch (Platform::Exception^ e)
		{

		}
		
	}
	foxit::common::DateTime^ _dateTime = fsAnnot->GetModifiedDateTime();
	if(_dateTime->year != 0)
		_ModifiedDateTime = FoxitTypeConvert::ConvertFromFoxitDateTime(_dateTime);

	return true;
}

bool BaseAnnot::OnCreate(IAnnotManager^ annotmgr, Platform::Object^ pageObj)
{
	_AnnotMgrWeakRef = annotmgr;
	foxit::pdf::PDFPage^ _pageHandle = dynamic_cast<foxit::pdf::PDFPage^>(pageObj);
	if (annotmgr == nullptr || _pageHandle == nullptr)
		return false;

	FS_INT32 count = _pageHandle->GetAnnotCount();
	FS_INT32 pageindex = _pageHandle->GetIndex();
	_PageIndex = pageindex;
	_PageHandleWrap->Pointer = _pageHandle;

	foxit::pdf::annots::Annot^ _annotHandler = nullptr;
	auto bsAnnotType = GetAnnotTypeString(_AnnotType);
	foxit::common::fxcrt::RectF^ rect = FoxitRectConverter::ConvertToFoxitRectF(_MinRect);

	_annotHandler = _pageHandle->AddAnnot(FoxitTypeConvert::ConvertFromAnnotType(_AnnotType), rect);
	if (_annotHandler == nullptr)
		return false;

	FS_INT32 annotindex = _annotHandler->GetIndex();
	_AnnoIndex = annotindex;

	if (_AnnotType == AnnotationType::FreeText)
	{
		foxit::pdf::annots::FreeText^ freeText = ref new foxit::pdf::annots::FreeText(_annotHandler);
		if (freeText)
			freeText->SetIntent(FSPDF_ANNOT_INTENTNAME_FREETEXT_TYPEWRITER);
	}	
	

	if (_Name && !_Name->IsEmpty())
	{
		_annotHandler->SetUniqueID(_Name);
	}
	

	if (_Title && !_Title->IsEmpty())
	{
		foxit::pdf::annots::IMarkup^ markup = _annotHandler->IsMarkup()?ref new foxit::pdf::annots::Markup(_annotHandler):nullptr;
		if(markup)
			markup->SetTitle(_Title);
	}

	if (_Contents && !_Contents->IsEmpty())
	{
		_annotHandler->SetContent(_Contents);
	}

	FS_ARGB color = (_Color.A << 24) | (_Color.R << 16) | (_Color.G << 8) | _Color.B;
	_annotHandler->SetBorderColor(color);
	FS_FLOAT opacity = (FS_FLOAT)_Opacity;
	foxit::pdf::annots::IMarkup^ markup = _annotHandler->IsMarkup()?ref new foxit::pdf::annots::Markup(_annotHandler):nullptr;
	if (markup)
		markup->SetOpacity(opacity);

	if (_AnnotType != AnnotationType::FreeText)
	{
		foxit::common::fxcrt::FloatArray^ dashes = ref new  foxit::common::fxcrt::FloatArray();
		for (int i = 0; i < _AnnotBorder->DashCount; i++)
		{
			dashes->Add(_AnnotBorder->DashPattern[i]);
		}

		foxit::pdf::annots::BorderInfo^ border = ref new foxit::pdf::annots::BorderInfo(
			_AnnotBorder->BorderWidth,
			FoxitTypeConvert::ConvertToFoxitAnnotBorderStyle(_AnnotBorder->BorderStyle),
			_AnnotBorder->CloudIntensity,
			_AnnotBorder->DashPhase,
			dashes
		);

		_annotHandler->SetBorderInfo(border);
	}

	foxit::common::DateTime^ _dateTime;
	if (_CreationDateTime.UniversalTime != 0)
	{
		_dateTime = FoxitTypeConvert::ConvertToFoxitDateTime(_CreationDateTime);
		
		foxit::pdf::annots::IMarkup^ markup = _annotHandler->IsMarkup()?ref new foxit::pdf::annots::Markup(_annotHandler):nullptr;
		if (markup)
			markup->SetCreationDateTime(_dateTime);
	}

	if (_ModifiedDateTime.UniversalTime != 0)
	{
		_dateTime = FoxitTypeConvert::ConvertToFoxitDateTime(_ModifiedDateTime);
		_annotHandler->SetModifiedDateTime(_dateTime);
	}

	_PdfAnnot = ref new PDFAnnotHandle(_annotHandler);
	return true;
}

bool BaseAnnot::OnDelete()
{
	FS_RESULT iRet = -1;
	if (AnnotMgr && _PdfAnnot)
	{
		AnnotMgr->AnnotCollections_CritsecLock();
		OutputDebugString(L"AnnotCollections_CritsecLock\n");

		foxit::pdf::PDFPage^ page = dynamic_cast<foxit::pdf::PDFPage^>(_PageHandleWrap->Pointer);
		page->RemoveAnnot(_PdfAnnot->PDF_Annot);
		OutputDebugString(L"FSPDF_Annot_Remove\n");
		AnnotMgr->DeleteAnnot(this);

		_PdfAnnot = nullptr;
		AnnotMgr->AnnotCollections_CritsecUnLock();
		OutputDebugString(L"AnnotCollections_CritsecUnLock\n");
	}
	return iRet == FSCRT_ERRCODE_SUCCESS ? true : false;
}

bool BaseAnnot::OnDeleteNoReflesh()
{
	if (AnnotMgr && _PdfAnnot)
	{
		foxit::pdf::PDFPage^ page = dynamic_cast<foxit::pdf::PDFPage^>(_PageHandleWrap->Pointer);
		page->RemoveAnnot(_PdfAnnot->PDF_Annot);
		AnnotMgr->DeleteAnnot(this);
		_PdfAnnot = nullptr;
	}
	return true;
}

bool BaseAnnot::SetSelectedState(bool bIsShow)
{
	if (AnnotMgr == nullptr)
		return false;

	return true;
}

void BaseAnnot::SetAnnotVisible(bool isVisible)
{
	FS_DWORD flag = _PdfAnnot->PDF_Annot->GetFlags();
	if (!_PdfAnnot)
		return;

	if (!isVisible && flag != (FS_DWORD)foxit::pdf::annots::Flags::e_FlagHidden)
	{
		annotflag = flag;
		_PdfAnnot->PDF_Annot->SetFlags((unsigned int)foxit::pdf::annots::Flags::e_FlagHidden);
	}
	else
	{
		if (flag != annotflag)
			_PdfAnnot->PDF_Annot->SetFlags(annotflag);
	}
	_PdfAnnot->PDF_Annot->ResetAppearanceStream();
	if (AnnotMgr)
	{
		AnnotMgr->RefleshOnePageAnnots(_PageIndex);
	}
}

void BaseAnnot::CheckPageIndex()
{
	foxit::pdf::PDFPage^ pSdkPage = (foxit::pdf::PDFPage^)_PageHandleWrap->Pointer;
	FS_INT32 index = pSdkPage->GetIndex();
	_PageIndex = index;
}


namespace foxit
{
	namespace rdk
	{
		namespace uiextensions
		{
			ref class AnnotDefaultEventPeer : IBaseAnnotEventPeer
			{
			internal:
				AnnotDefaultEventPeer()
				{

				}

			public:
				virtual void OnPointerPressed(Platform::Object^ sender, Windows::UI::Xaml::Input::PointerRoutedEventArgs^ e)
				{

				}
				virtual void OnPointerMoved(Platform::Object^ sender, Windows::UI::Xaml::Input::PointerRoutedEventArgs^ e)
				{

				}
				virtual void OnPointerReleased(Platform::Object^ sender, Windows::UI::Xaml::Input::PointerRoutedEventArgs^ e)
				{

				}
				virtual void OnTapped(Platform::Object^ sender, Windows::UI::Xaml::Input::TappedRoutedEventArgs^ e)
				{

				}
				virtual void OnRightTapped(Platform::Object^ sender, Windows::UI::Xaml::Input::RightTappedRoutedEventArgs^ e)
				{

				}
				virtual void OnManipulationStarted(Platform::Object^ sender, Windows::UI::Xaml::Input::ManipulationStartedRoutedEventArgs^ e)
				{

				}
				virtual void OnManipulationDelta(Platform::Object^ sender, Windows::UI::Xaml::Input::ManipulationDeltaRoutedEventArgs^ e)
				{

				}
				virtual void OnManipulationCompleted(Platform::Object^ sender, Windows::UI::Xaml::Input::ManipulationCompletedRoutedEventArgs^ e)
				{

				}

				virtual void OnCreateAnnotAfterSelectText()
				{}

				virtual void OnManipulationInertiaStarting(Platform::Object^ sender, Windows::UI::Xaml::Input::ManipulationInertiaStartingRoutedEventArgs^ e)
				{

				}
			};
		}
	}
}

BaseAnnotHandler::BaseAnnotHandler()
{

}

IAnnotManager^ BaseAnnotHandler::AnnotMgr::get()
{
	return _pAnnotMgrWeakRef.Resolve<IAnnotManager>();
}

AnnotationType BaseAnnotHandler::AnnotHandlerType::get()
{
	return _AnnotHandlerType;
}

IBaseAnnotEventPeer^ BaseAnnotHandler::AnnotEvent::get()
{
	if (m_AnnotEventPeer == nullptr)
		m_AnnotEventPeer = OnCreateEventPeer();

	return m_AnnotEventPeer;
}

IBaseAnnotEventPeer^ BaseAnnotHandler::OnCreateEventPeer()
{
	return ref new AnnotDefaultEventPeer();
}

void BaseAnnotHandler::SetEventPeer(IBaseAnnotEventPeer^ peer)
{
	if (peer)
		m_AnnotEventPeer = peer;
}

void BaseAnnotHandler::SetAnnotMgr(IAnnotManager^ manager)
{
	if (manager)
	{
		if (_pAnnotMgrWeakRef)
		{
			if (!AnnotMgr->Equals(manager))
			{
				_pAnnotMgrWeakRef = manager;
			}
		}
		else
		{
			_pAnnotMgrWeakRef = manager;
		}
	}
}

bool BaseAnnotHandler::IsHitTest(IBaseAnnot^ pAnnot, Windows::Foundation::Point ptDevice)
{
	return false;
}

void BaseAnnotHandler::ClearSelectedBorder()
{
	auto _canvas = m_CanvasWeakRef.Resolve<Windows::UI::Xaml::Controls::Canvas>();
	if (_canvas)
		_canvas->Children->Clear();
}


void BaseAnnotHandler::CancelDrawingOneStep()
{

}

void BaseAnnotHandler::EndConsistentDrawing()
{

}

void BaseAnnotHandler::EndDrawing()
{
	EndConsistentDrawing();
}

void BaseAnnotHandler::Reset()
{

}

void BaseAnnotHandler::OnCreateNewAnnot(Platform::Object^ pageObj, int annotIndex)
{
	auto _annot = AnnotCreaterFactory::GetInstance()->CreateNewAnnot(_AnnotHandlerType);

	if (_annot)
		_annot->OnCreate(AnnotMgr, annotIndex);
}

void BaseAnnotHandler::OnCreateNewAnnot(Platform::Object^ pageObj, IBaseAnnot^ newannot)
{
	if (newannot)
		newannot->OnCreate(AnnotMgr, pageObj);
}

IBaseAnnot^ BaseAnnotHandler::OnLoadAnnot(Platform::Object^ pageObj, PDFAnnotHandle^ annot)
{
	auto _annot = AnnotCreaterFactory::GetInstance()->CreateNewAnnot(_AnnotHandlerType);

	if (_annot)
		_annot->OnLoadAnnot(AnnotMgr, pageObj, annot);
	return _annot;
}

bool BaseAnnotHandler::OnDelete(IBaseAnnot^ pAnnot)
{
	return true;
}

bool BaseAnnotHandler::OnRelease(IBaseAnnot^ pAnnot)
{
	return true;
}

bool BaseAnnotHandler::OnVisible(IBaseAnnot^ pAnnot)
{
	return true;
}

bool BaseAnnotHandler::OnHidden(IBaseAnnot^ pAnnot)
{
	return true;
}

bool BaseAnnotHandler::OnPrint(IBaseAnnot^ pAnnot)
{
	return true;
}

void BaseAnnotHandler::OnMoving(IBaseAnnot^ pAnnot)
{

}

void BaseAnnotHandler::OnMoved(IBaseAnnot^ pAnnot)
{

}

void BaseAnnotHandler::CanecelMoving(IBaseAnnot^ pAnnot)
{

}

void BaseAnnotHandler::OnMoveOutOfPage(IBaseAnnot^ pAnnot)
{

}

bool BaseAnnotHandler::OnResizing(IBaseAnnot^ pAnnot)
{
	return false;
}

void BaseAnnotHandler::OnPopupMenu(IBaseAnnot^ pAnnot, Windows::Foundation::Point point)
{

}

void BaseAnnotHandler::OnDoAnnotAction(IBaseAnnot^ pAnnot, Windows::Foundation::Point curPoint)
{

}

void BaseAnnotHandler::OnPointerPressed(Platform::Object^ sender, Windows::UI::Xaml::Input::PointerRoutedEventArgs^ e)
{
	if (m_AnnotEventPeer)
		m_AnnotEventPeer->OnPointerPressed(sender, e);
}

void BaseAnnotHandler::OnPointerMoved(Platform::Object^ sender, Windows::UI::Xaml::Input::PointerRoutedEventArgs^ e)
{
	if (m_AnnotEventPeer)
		m_AnnotEventPeer->OnPointerMoved(sender, e);
}

void BaseAnnotHandler::OnPointerReleased(Platform::Object^ sender, Windows::UI::Xaml::Input::PointerRoutedEventArgs^ e)
{
	if (m_AnnotEventPeer)
		m_AnnotEventPeer->OnPointerReleased(sender, e);
}

void BaseAnnotHandler::OnTapped(Platform::Object^ sender, Windows::UI::Xaml::Input::TappedRoutedEventArgs^ e)
{
	if (m_AnnotEventPeer)
		m_AnnotEventPeer->OnTapped(sender, e);
}

void BaseAnnotHandler::OnRightTapped(Platform::Object^ sender, Windows::UI::Xaml::Input::RightTappedRoutedEventArgs^ e)
{
	if (m_AnnotEventPeer)
		m_AnnotEventPeer->OnRightTapped(sender, e);
}

void BaseAnnotHandler::OnManipulationStarted(Platform::Object^ sender, Windows::UI::Xaml::Input::ManipulationStartedRoutedEventArgs^ e)
{
	if (m_AnnotEventPeer)
		m_AnnotEventPeer->OnManipulationStarted(sender, e);
}

void BaseAnnotHandler::OnManipulationDelta(Platform::Object^ sender, Windows::UI::Xaml::Input::ManipulationDeltaRoutedEventArgs^ e)
{
	if (m_AnnotEventPeer)
		m_AnnotEventPeer->OnManipulationDelta(sender, e);
}

void BaseAnnotHandler::OnManipulationCompleted(Platform::Object^ sender, Windows::UI::Xaml::Input::ManipulationCompletedRoutedEventArgs^ e)
{
	if (m_AnnotEventPeer)
		m_AnnotEventPeer->OnManipulationCompleted(sender, e);
}

void BaseAnnotHandler::OnManipulationInertiaStarting(Platform::Object^ sender, Windows::UI::Xaml::Input::ManipulationInertiaStartingRoutedEventArgs^ e)
{
	if (m_AnnotEventPeer)
		m_AnnotEventPeer->OnManipulationInertiaStarting(sender, e);
}

void BaseAnnotHandler::CreateAnnotAfterSelectText()
{
	if (m_AnnotEventPeer)
		m_AnnotEventPeer->OnCreateAnnotAfterSelectText();
}

void BaseAnnotHandler::OnCreateNewAnnotGroup(Platform::Object^ pageObj, Windows::Foundation::Collections::IVector<IBaseAnnot^>^ newannots)
{
	auto _annot = AnnotCreaterFactory::GetInstance()->CreateNewAnnot(AnnotationType::ReplaceText);

	if (_annot)
	{
		ReplaceTextAnnot^ replaceTextAnnot = dynamic_cast<ReplaceTextAnnot^>(_annot);
		replaceTextAnnot->OnCreateNewAnnotGroup(pageObj, newannots);
	}
}