/**
 * Copyright (C) 2003-2018, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit Mobile PDF SDK to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */


#pragma once


namespace foxit
{
	namespace rdk
	{
		namespace uiextensions
		{
			interface class IBaseAnnot;
			interface class IBaseAnnotHandler;
		}
	}
}

namespace foxit
{
	namespace rdk
	{
		namespace uiextensions
		{
			[Windows::Foundation::Metadata::WebHostHidden]
			public ref class BaseAnnot : public Windows::UI::Xaml::DependencyObject, IBaseAnnot
			{
			internal:
				BaseAnnot();

			public:
				virtual property int PageIndex
				{
					int get();
				}

				virtual property int AnnoIndex
				{
					int get();
				}

				virtual property Platform::String^ Name
				{
					Platform::String^ get();
					void set(Platform::String^ value);
				}

				virtual property Platform::String^ Title
				{
					Platform::String^ get();
					void set(Platform::String^ value);
				}

				virtual property Platform::String^ Contents
				{
					Platform::String^ get();
					void set(Platform::String^ value);
				}

				virtual property Windows::UI::Color Color
				{
					Windows::UI::Color get();
					void set(Windows::UI::Color value);
				}

				virtual property double Opacity
				{
					double get();
					void set(double value);
				}

				virtual property Windows::Foundation::Rect MinRect
				{
					Windows::Foundation::Rect get();
					void set(Windows::Foundation::Rect value);
				}

				virtual property AnnotBorderWrap^ AnnotBorder
				{
					AnnotBorderWrap^ get();
					void set(AnnotBorderWrap^ value);
				}

				virtual property double BorderWidth
				{
					double get();
					void set(double value);
				}

				virtual property Windows::Foundation::DateTime CreationDateTime
				{
					Windows::Foundation::DateTime get();
					void set(Windows::Foundation::DateTime value);
				}

				virtual property Windows::Foundation::DateTime ModifiedDateTime
				{
					Windows::Foundation::DateTime get();
					void set(Windows::Foundation::DateTime value);
				}

				virtual property PDFAnnotHandle^ AnnotHandleWrap
				{
					PDFAnnotHandle^ get();
				}

				virtual property AnnotationType AnnotType
				{
					AnnotationType get();
				}

				virtual property IAnnotManager^ AnnotMgr
				{
					IAnnotManager^ get();
				}

				virtual property bool IsModifyProperty
				{
					bool get();
					void set(bool isModify);
				}

				virtual property Platform::Object^ PageHandleWrap
				{
					Platform::Object^ get();
				}

			public:
				/// set Annotation visible
				virtual bool IsVisibity();
				/// Click on the precise
				virtual bool IsHitTest(Windows::Foundation::Point pagePoint);
				/// Click on the range
				virtual bool IsHitTestRange(Windows::Foundation::Point pagePoint);
				/// initialize
				virtual bool OnLoadAnnot(IAnnotManager^ annotmgr, Platform::Object^ pageObj, int annotIndex);
				virtual bool OnLoadAnnot(IAnnotManager^ annotmgr, Platform::Object^ pageObj, PDFAnnotHandle^ annot);
				/// create
				virtual bool OnCreate(IAnnotManager^ annotmgr, Platform::Object^ pageObj);
				/// delete
				virtual bool OnDelete();
				virtual bool OnDeleteNoReflesh();
				/// set the selected state
				virtual bool SetSelectedState(bool bIsShow);
				virtual void SetAnnotVisible(bool isVisible);
				virtual void CheckPageIndex();

			protected private:
				Platform::String^ GetAnnotTypeString(AnnotationType type);
				void NotifyDocumentChanged();

			protected private:
				int									_PageIndex;
				int									_AnnoIndex;
				Platform::String^					_Name;
				Platform::String^					_Title;
				Platform::String^					_Contents;
				Windows::UI::Color					_Color;
				double								_Opacity;
				Windows::Foundation::Rect			_MinRect;
				AnnotBorderWrap^ _AnnotBorder;
				AnnotationType _AnnotType;
				PDFAnnotHandle^ _PdfAnnot;
				PDFPageWrapper^ _PageHandleWrap;
				Windows::Foundation::DateTime _CreationDateTime;
				Windows::Foundation::DateTime _ModifiedDateTime;
				Platform::WeakReference	_AnnotMgrWeakRef;
				Platform::WeakReference _PageViewWeakRef;
				FS_DWORD  annotflag;
				bool   _IsModify = false;

			};

			[Windows::Foundation::Metadata::WebHostHidden]
			public ref class BaseAnnotHandler : public Windows::UI::Xaml::DependencyObject, public IBaseAnnotHandler
			{
			internal:
				BaseAnnotHandler();

			public:
				virtual property AnnotationType AnnotHandlerType
				{
					AnnotationType get();
				}
				virtual property IAnnotManager^ AnnotMgr
				{
					IAnnotManager^ get();
				}
				virtual property IBaseAnnotEventPeer^ AnnotEvent
				{
					IBaseAnnotEventPeer^ get();
				}

			protected private:
				AnnotationType	_AnnotHandlerType;
				Platform::WeakReference _pAnnotMgrWeakRef;
				Platform::WeakReference m_CanvasWeakRef;
				IBaseAnnotEventPeer^ m_AnnotEventPeer;

			public:
				virtual IBaseAnnotEventPeer^ OnCreateEventPeer();
				virtual void SetEventPeer(IBaseAnnotEventPeer^ peer);
				virtual void SetAnnotMgr(IAnnotManager^ manager);
				virtual bool IsHitTest(IBaseAnnot^ pAnnot, Windows::Foundation::Point ptDevice);
				virtual void ClearSelectedBorder();
				virtual void CancelDrawingOneStep();
				virtual void EndConsistentDrawing();
				virtual void EndDrawing();
				virtual void Reset();

			public:
				/// create new Annotation
				virtual void OnCreateNewAnnot(Platform::Object^ pageObj, int annotIndex);
				virtual void OnCreateNewAnnot(Platform::Object^ pageObj, IBaseAnnot^ newannot);
				/// load Annotation
				virtual IBaseAnnot^ OnLoadAnnot(Platform::Object^ pageObj, PDFAnnotHandle^ annot);
				/// delete Annotation
				virtual bool OnDelete(IBaseAnnot^ pAnnot);
				/// release Annotation
				virtual bool OnRelease(IBaseAnnot^ pAnnot);

				/// show Annotation
				virtual bool OnVisible(IBaseAnnot^ pAnnot);
				/// hidden Annotation
				virtual bool OnHidden(IBaseAnnot^ pAnnot);
				/// print Annotation
				virtual bool OnPrint(IBaseAnnot^ pAnnot);

				virtual void OnMoving(IBaseAnnot^ pAnnot);
				virtual void OnMoved(IBaseAnnot^ pAnnot);
				virtual void CanecelMoving(IBaseAnnot^ pAnnot);
				virtual void OnMoveOutOfPage(IBaseAnnot^ pAnnot);
				virtual bool OnResizing(IBaseAnnot^ pAnnot);
				virtual void OnPopupMenu(IBaseAnnot^ pAnnot, Windows::Foundation::Point point);
				virtual void OnDoAnnotAction(IBaseAnnot^ pAnnot, Windows::Foundation::Point curPoint);

				virtual void CreateAnnotAfterSelectText();
				virtual void OnCreateNewAnnotGroup(Platform::Object^ pageObj, Windows::Foundation::Collections::IVector<IBaseAnnot^>^ newannots);

			public:
				virtual void OnPointerPressed(Platform::Object^ sender, Windows::UI::Xaml::Input::PointerRoutedEventArgs^ e);
				virtual void OnPointerMoved(Platform::Object^ sender, Windows::UI::Xaml::Input::PointerRoutedEventArgs^ e);
				virtual void OnPointerReleased(Platform::Object^ sender, Windows::UI::Xaml::Input::PointerRoutedEventArgs^ e);
				virtual void OnTapped(Platform::Object^ sender, Windows::UI::Xaml::Input::TappedRoutedEventArgs^ e);
				virtual void OnRightTapped(Platform::Object^ sender, Windows::UI::Xaml::Input::RightTappedRoutedEventArgs^ e);
				virtual void OnManipulationStarted(Platform::Object^ sender, Windows::UI::Xaml::Input::ManipulationStartedRoutedEventArgs^ e);
				virtual void OnManipulationDelta(Platform::Object^ sender, Windows::UI::Xaml::Input::ManipulationDeltaRoutedEventArgs^ e);
				virtual void OnManipulationCompleted(Platform::Object^ sender, Windows::UI::Xaml::Input::ManipulationCompletedRoutedEventArgs^ e);
				virtual void OnManipulationInertiaStarting(Platform::Object^ sender, Windows::UI::Xaml::Input::ManipulationInertiaStartingRoutedEventArgs^ e);
			};
		}
	}
}

