/**
 * Copyright (C) 2003-2018, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit Mobile PDF SDK to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */


#include "pch.h"
#include "AnnotCreaterFactory.h"
#include "Annot_Highlight.h"
#include "Annot_UnderLine.h"
#include "Annot_StrikeOut.h"
#include "Annot_Squiggly.h"
#include "Annot_Link.h"
#include "Annot_Text.h"
#include "Annot_Square.h"
#include "Annot_Circle.h"
#include "Annot_Line.h"
#include "Annot_Ink.h"
#include "Annot_Unknow.h"
#include "Annot_Stamp.h"
#include "Annot_FreeText.h"
#include "Annot_Caret.h"
#include "Annot_ReplaceText.h"
#include "Annot_Widget.h"

using namespace foxit::rdk::uiextensions;

using namespace Platform;

static AnnotCreaterFactory^ _Instance = nullptr;

AnnotCreaterFactory^ AnnotCreaterFactory::GetInstance()
{
	if (_Instance == nullptr)
	{
		_Instance = ref new AnnotCreaterFactory();
	}
	return _Instance;
}

IBaseAnnot^ AnnotCreaterFactory::CreateNewAnnot(AnnotationType type)
{
	IBaseAnnot^ _baseannot = nullptr;
	
	switch (type)
	{
	case AnnotationType::NonSurport:
		_baseannot = ref new NonSurportAnnot();
		break;
	case AnnotationType::HightLight:
		_baseannot = ref new HighLightAnnot();
		break;
	case AnnotationType::Link:
		_baseannot = ref new LinkAnnot();
		break;
	case AnnotationType::UnderLine:
		_baseannot = ref new UnderLineAnnot();
		break;
	case AnnotationType::StrikeOut:
		_baseannot = ref new StrikeOutAnnot();
		break;
	case AnnotationType::Text:
		_baseannot = ref new TextAnnot();
		break;
	case AnnotationType::Square:
		_baseannot = ref new SquareAnnot();
		break;
	case AnnotationType::Circle:
		_baseannot = ref new CircleAnnot();
		break;
	case AnnotationType::Line:
		_baseannot = ref new LineAnnot();
		break;
	case AnnotationType::Ink:
		_baseannot = ref new PencilAnnot();
		break;
	case AnnotationType::Squiggly:
		_baseannot = ref new SquigglyAnnot();
		break;
	case AnnotationType::Stamp:
		_baseannot = ref new StampAnnot();
		break;
	case AnnotationType::FreeText:
		_baseannot = ref new FreeTextAnnot();
		break;
	case AnnotationType::Caret:
		_baseannot = ref new CaretAnnot();
		break;
	case AnnotationType::ReplaceText:
		_baseannot = ref new ReplaceTextAnnot();
		break;
	case AnnotationType::Widget:
		_baseannot = ref new WidgetAnnot();
		break;
	case AnnotationType::None:
		break;
	default:
		break;
	}

	if (_baseannot == nullptr)
		throw ref new Platform::NotImplementedException();
	
	return _baseannot;
}
