/**
 * Copyright (C) 2003-2018, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit Mobile PDF SDK to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */


#include "pch.h"
#include "AnnotManager.h"
#include "AnnotationHandlerFactory.h"
#include "Annot_Ink.h"
#include "Annot_ReplaceText.h"
#include "AnnotCreaterFactory.h"
#include "Annot_StrikeOut.h"
#include "Annot_Caret.h"

using namespace foxit::rdk::uiextensions;

using namespace Platform;
using namespace Windows::Foundation;
using namespace Concurrency;
using namespace Windows::UI::Xaml::Controls;
using namespace Windows::UI::Xaml;
using namespace Windows::ApplicationModel::Resources::Core;
using namespace Windows::UI::Xaml::Input;
using namespace Windows::UI::Xaml::Media;

namespace foxit
{
	namespace rdk
	{
		namespace uiextensions
		{
			ref class DefaultAnnotAutomationPeer : IAnnotManagerPeer
			{
			internal:
				DefaultAnnotAutomationPeer()
				{

				}

			public:
				virtual int GetPageCount()
				{
					return 0;
				}
				virtual Platform::Object^ GetPdfPageObject(int pageIndex)
				{
					return nullptr;
				}
				virtual void NotifyDocumentChanged()
				{

				}
				virtual void UpdatePageView(int pageindex)
				{

				}

				virtual void SaveAnonotModifyBeforeProperty()
				{

				}

				virtual void InsertAnnotModifyUndo()
				{

				}

				virtual void OnAnnotPointerEntered(Platform::Object^ sender, Windows::UI::Xaml::Input::PointerRoutedEventArgs^ e)
				{

				}
				virtual void OnAnnotPointerExited(Platform::Object^ sender, Windows::UI::Xaml::Input::PointerRoutedEventArgs^ e)
				{

				}
				virtual void OnAnnotPointerMoved(Platform::Object^ sender, Windows::UI::Xaml::Input::PointerRoutedEventArgs^ e)
				{

				}
				virtual void OnAnnotPointerPressed(Platform::Object^ sender, Windows::UI::Xaml::Input::PointerRoutedEventArgs^ e)
				{

				}
				virtual void OnAnnotPointerReleased(Platform::Object^ sender, Windows::UI::Xaml::Input::PointerRoutedEventArgs^ e)
				{

				}
				virtual void OnAnnotTapped(Platform::Object^ sender, Windows::UI::Xaml::Input::TappedRoutedEventArgs^ e)
				{

				}
				virtual void OnAnnotRightTapped(Platform::Object^ sender, Windows::UI::Xaml::Input::RightTappedRoutedEventArgs^ e)
				{

				}
				virtual void OnAnnotManipulationStarted(Platform::Object^ sender, Windows::UI::Xaml::Input::ManipulationStartedRoutedEventArgs^ e)
				{

				}
				virtual void OnAnnotManipulationDelta(Platform::Object^ sender, Windows::UI::Xaml::Input::ManipulationDeltaRoutedEventArgs^ e)
				{

				}
				virtual void OnAnnotManipulationCompleted(Platform::Object^ sender, Windows::UI::Xaml::Input::ManipulationCompletedRoutedEventArgs^ e)
				{

				}

				virtual void OnManipulationInertiaStarting(Platform::Object^ sender, Windows::UI::Xaml::Input::ManipulationInertiaStartingRoutedEventArgs^ e)
				{

				}
			};
		}
	}
}

LoadPageAnnotsContext::LoadPageAnnotsContext(int pageindex) : _pageindex(pageindex)
{
	_bIsLoading = false;
	_bIsCompletion = false;
}

LoadPageAnnotsContext::~LoadPageAnnotsContext()
{
	if (!_completionEvent._IsTriggered())
	{
		_completionEvent._Cancel();
	}
}

int LoadPageAnnotsContext::PageIndex::get()
{
	return _pageindex;
}

bool LoadPageAnnotsContext::IsLoadPageAnnotsCompletion()
{
	return !_bIsLoading && _bIsCompletion;
}

bool LoadPageAnnotsContext::IsLoadingPageAnnots()
{
	return _bIsLoading;
}

void LoadPageAnnotsContext::SetIsLoading()
{
	_bIsLoading = true;
}

void LoadPageAnnotsContext::SetCompletion()
{
	_bIsLoading = false;
	_bIsCompletion = true;
	_completionEvent.set();
}

void LoadPageAnnotsContext::SetPageIndex(int pageIndex)
{
	_pageindex = pageIndex;
}

AnnotationManager::AnnotationManager()
{
	AnnotCollections_Critsec = std::make_shared<CritSec>();
	m_LoadAnnotsContexts = ref new Platform::Collections::Map<int, LoadPageAnnotsContext^>();
	m_AnnotCollections = ref new Platform::Collections::Map<int, Windows::Foundation::Collections::IVector<IBaseAnnot^>^>();
	m_MainPageDispatcher = Windows::UI::Core::CoreWindow::GetForCurrentThread()->Dispatcher;
	m_AnnotHanderGroup = ref new Platform::Collections::Vector<IBaseAnnotHandler^>();
	m_AnnoMgrPeer = ref new DefaultAnnotAutomationPeer();
	m_pSettingPageview = ref new AnnotSettingManager();
	m_PageNoteCollections = ref new Platform::Collections::Map<int, Windows::Foundation::Collections::IVector<IBaseAnnot^>^>();
	m_FreeTextAnnotCollections = ref new Platform::Collections::Map<int, Windows::Foundation::Collections::IVector<IBaseAnnot^>^>();

	InitializeSurportedAnnotHandler();
	isExitPageView = false;
	createAnnotRect = Rect(0, 0, 0, 0);
}

AnnotationManager::~AnnotationManager()
{
	if (m_LoadAnnotsContexts && m_LoadAnnotsContexts->Size > 0)
		m_LoadAnnotsContexts->Clear();
	m_LoadAnnotsContexts = nullptr;

	if (m_AnnotCollections && m_AnnotCollections->Size > 0)
		m_AnnotCollections->Clear();
	m_AnnotCollections = nullptr;

	m_pSettingPageview = nullptr;
}

Windows::Foundation::Collections::IMap<int, Windows::Foundation::Collections::IVector<IBaseAnnot^>^>^  AnnotationManager::AnnotCollections::get()
{
	return m_AnnotCollections;
}

Windows::Foundation::Collections::IVector<IBaseAnnotHandler^>^ AnnotationManager::AnnotHanderGroup::get()
{
	return m_AnnotHanderGroup;
}

Windows::UI::Core::CoreDispatcher^ AnnotationManager::GetmainPageDispatcher()
{
	return m_MainPageDispatcher;
}

IAnnotSettingImplementor^ AnnotationManager::GetAnnotSetting()
{
	return m_pSettingPageview;
}

void AnnotationManager::SetAnnotSetting(IAnnotSettingImplementor^ pSettingPageview)
{
	m_pSettingPageview = pSettingPageview;
}

void AnnotationManager::InitializeSurportedAnnotHandler()
{
	IBaseAnnotHandler^ _handler = nullptr;
	auto _annotHandlerFactory = AnnotationHandlerFactory::GetInstance();

	_handler = _annotHandlerFactory->CreateHandler(AnnotationType::HightLight, this);
	RegisterAnnotHandler(_handler);
	_handler = _annotHandlerFactory->CreateHandler(AnnotationType::Link, this);
	RegisterAnnotHandler(_handler);
	_handler = _annotHandlerFactory->CreateHandler(AnnotationType::UnderLine, this);
	RegisterAnnotHandler(_handler);
	_handler = _annotHandlerFactory->CreateHandler(AnnotationType::StrikeOut, this);
	RegisterAnnotHandler(_handler);
	_handler = _annotHandlerFactory->CreateHandler(AnnotationType::Square, this);
	RegisterAnnotHandler(_handler);
	_handler = _annotHandlerFactory->CreateHandler(AnnotationType::Circle, this);
	RegisterAnnotHandler(_handler);
	_handler = _annotHandlerFactory->CreateHandler(AnnotationType::Line, this);
	RegisterAnnotHandler(_handler);
	_handler = _annotHandlerFactory->CreateHandler(AnnotationType::Ink, this);
	RegisterAnnotHandler(_handler);
	_handler = _annotHandlerFactory->CreateHandler(AnnotationType::Squiggly, this);
	RegisterAnnotHandler(_handler);
	_handler = _annotHandlerFactory->CreateHandler(AnnotationType::Text, this);
	RegisterAnnotHandler(_handler);
	_handler = _annotHandlerFactory->CreateHandler(AnnotationType::Stamp, this);
	RegisterAnnotHandler(_handler);
	_handler = _annotHandlerFactory->CreateHandler(AnnotationType::FreeText, this);
	RegisterAnnotHandler(_handler);
	_handler = _annotHandlerFactory->CreateHandler(AnnotationType::NonSurport, this);
	RegisterAnnotHandler(_handler);
	_handler = _annotHandlerFactory->CreateHandler(AnnotationType::Caret, this);
	RegisterAnnotHandler(_handler);
	_handler = _annotHandlerFactory->CreateHandler(AnnotationType::ReplaceText, this);
	RegisterAnnotHandler(_handler);
	_handler = _annotHandlerFactory->CreateHandler(AnnotationType::Widget, this);
	RegisterAnnotHandler(_handler);
}

void AnnotationManager::OnCreate(IAnnotManagerPeer^ annotPeer)
{
	m_AnnoMgrPeer = annotPeer;
}

bool AnnotationManager::OnLoaded()
{

	return true;
}

void AnnotationManager::OnDelete()
{
	if (m_AnnotHanderGroup && m_AnnotHanderGroup->Size > 0)
	{
		auto _AnnotHanderGroupSize = m_AnnotHanderGroup->Size;
		for (unsigned int i = 0; i < _AnnotHanderGroupSize; i++)
		{
			auto _item = m_AnnotHanderGroup->GetAt(i);
			if (_item)
			{
				_item->EndDrawing();
			}
		}
		m_AnnotHanderGroup->Clear();
	}
	m_AnnotHanderGroup = nullptr;
}

void AnnotationManager::AddAnnot(IBaseAnnot^ annot)
{
	if (annot == nullptr || annot->AnnotMgr != this)
		return;

	auto pageIndex = annot->PageIndex;

	if (!m_AnnotCollections->HasKey(pageIndex))
	{
		AnnotCollections_Critsec->Lock();
		m_AnnotCollections->Insert(pageIndex, ref new Platform::Collections::Vector<IBaseAnnot^>());
		AnnotCollections_Critsec->Unlock();
	}
	AnnotCollections_Critsec->Lock();
	Windows::Foundation::Collections::IVector<IBaseAnnot^>^ pageCollections = m_AnnotCollections->Lookup(pageIndex);
	AnnotCollections_Critsec->Unlock();
	pageCollections->Append(annot);
	AddAnnotEvent(this, annot);

	if (annot->AnnotType == AnnotationType::Text)
	{
		if (!m_PageNoteCollections->HasKey(pageIndex))
		{
			AnnotCollections_Critsec->Lock();
			m_PageNoteCollections->Insert(pageIndex, ref new Platform::Collections::Vector<IBaseAnnot^>());
			AnnotCollections_Critsec->Unlock();
		}
		AnnotCollections_Critsec->Lock();
		Windows::Foundation::Collections::IVector<IBaseAnnot^>^ noteCollections = m_PageNoteCollections->Lookup(pageIndex);
		AnnotCollections_Critsec->Unlock();
		noteCollections->Append(annot);
	}
	NotifyDocumentChanged();
	UpdatePageView(pageIndex);
}

void AnnotationManager::ClearAnnots(int nPageIndex)
{
	if (m_AnnotCollections && m_AnnotCollections->HasKey(nPageIndex))
	{
		AnnotCollections_Critsec->Lock();
		Windows::Foundation::Collections::IVector<IBaseAnnot^>^ pageCollections = m_AnnotCollections->Lookup(nPageIndex);
		AnnotCollections_Critsec->Unlock();
		if (pageCollections->Size > 0)
		{
			m_AnnotCollections->Remove(nPageIndex);
			pageCollections->Clear();
			pageCollections = nullptr;
		}
	}
}

void AnnotationManager::DeleteAnnot(IBaseAnnot^ annot)
{
	if (annot == nullptr || annot->AnnotMgr != this)
		return;

	auto pageIndex = annot->PageIndex;


	if (!m_AnnotCollections->HasKey(pageIndex))
		return;

	unsigned int index = 0;
	Windows::Foundation::Collections::IVector<IBaseAnnot^>^ pageCollections = m_AnnotCollections->Lookup(pageIndex);
	if (!pageCollections->IndexOf(annot, &index))
		return;


	pageCollections->RemoveAt(index);
	OutputDebugString(L"AnnotationManager DeleteAnnot\n");
	DeleteAnnotEvent(this, annot);

	if (annot->AnnotType == AnnotationType::Text)
	{
		Windows::Foundation::Collections::IVector<IBaseAnnot^>^ noteCollections = m_PageNoteCollections->Lookup(pageIndex);

		index = 0;
		if (!noteCollections->IndexOf(annot, &index))
			return;
		noteCollections->RemoveAt(index);
	}


	if (annot == m_pSelectedAnnot)
		SetSelectedAnnot(nullptr);

	NotifyDocumentChanged();
	UpdatePageView(pageIndex);
}

void AnnotationManager::OnCreateNewAnnot(Platform::Object^ pageObj, IBaseAnnot^ newannot)
{
	OutputDebugString(L"OnCreateNewAnnot\n");

	auto _basehandler = GetAnnotHandlerByType(newannot->AnnotType);
	if (_basehandler)
	{
		_basehandler->OnCreateNewAnnot(pageObj, newannot);
	}
}

void AnnotationManager::OnCreateNewAnnotGroup(Platform::Object^ pageObj, Windows::Foundation::Collections::IVector<IBaseAnnot^>^ newannots)
{
	int size = newannots ? newannots->Size : 0;
	if (size > 0)
	{
		for (int i = 0; i < size; i++)
		{
			auto _basehandler = GetAnnotHandlerByType(newannots->GetAt(i)->AnnotType);
			if (_basehandler)
			{
				_basehandler->OnCreateNewAnnot(pageObj, newannots->GetAt(i));
			}
		}

		StrikeOutAnnot^ strikeoutAnnot = dynamic_cast<StrikeOutAnnot^>(newannots->GetAt(0));
		if (strikeoutAnnot)
		{
			strikeoutAnnot->IsHasGroup = true;
			strikeoutAnnot->GroupAnnotMinRect = newannots->GetAt(1)->MinRect;
			strikeoutAnnot->GroupAnnotIndex = newannots->GetAt(1)->AnnoIndex;
		}

		CaretAnnot^ caretAnnot = dynamic_cast<CaretAnnot^>(newannots->GetAt(1));
		if (caretAnnot)
		{
			caretAnnot->IsHasGroup = true;
			caretAnnot->GroupAnnotMinRect = newannots->GetAt(0)->MinRect;
			caretAnnot->GroupAnnotIndex = newannots->GetAt(0)->AnnoIndex;
		}
	}

	auto _basehandler = GetAnnotHandlerByType(AnnotationType::ReplaceText);
	if (_basehandler)
	{
		_basehandler->OnCreateNewAnnotGroup(pageObj, newannots);
	}
}


IBaseAnnot^ AnnotationManager::OnLoadAnnot(Platform::Object^ pageObj, PDFAnnotHandle^ annot)
{
	IBaseAnnot^ iBaseAnnot = nullptr;

	String^ csType = FoxitTypeConvert::ConvertFromFoxitAnnotType(annot->PDF_Annot->GetType());

	auto _annothandler = GetAnnotHandlerByTypeString(csType);
	IBaseAnnot^ _annot = nullptr;
	if (_annothandler)
	{
		_annot = _annothandler->OnLoadAnnot(pageObj, annot);
	}
	return iBaseAnnot;
}

IBaseAnnot^ AnnotationManager::GetAnnotFromMaps(int pageIndex, int index)
{
	if (m_AnnotCollections && m_AnnotCollections->HasKey(pageIndex))
	{
		AnnotCollections_Critsec->Lock();
		Windows::Foundation::Collections::IVector<IBaseAnnot^>^ pageCollections = m_AnnotCollections->Lookup(pageIndex);
		AnnotCollections_Critsec->Unlock();
		if (pageCollections->Size > (unsigned int)index)
		{
			return pageCollections->GetAt(index);
		}
	}
	return nullptr;
}

void AnnotationManager::RegisterAnnotHandler(IBaseAnnotHandler^ AnnotHandler)
{
	if (AnnotHandler != nullptr && m_AnnotHanderGroup != nullptr)
		m_AnnotHanderGroup->Append(AnnotHandler);
}

IBaseAnnotHandler^ AnnotationManager::GetAnnotHandlerByType(AnnotationType type)
{
	if (m_AnnotHanderGroup)
	{
		int nHandlerCount = m_AnnotHanderGroup->Size;
		for (int i = 0; i < nHandlerCount; i++)
		{
			IBaseAnnotHandler^ pAnnotHandler = m_AnnotHanderGroup->GetAt(i);
			if (pAnnotHandler != nullptr)
			{
				AnnotationType type11 = pAnnotHandler->AnnotHandlerType;
				if (pAnnotHandler->AnnotHandlerType == type)
				{
					return pAnnotHandler;
				}
			}
		}
	}
	return nullptr;
}

IBaseAnnotHandler^ AnnotationManager::GetAnnotHandlerByTypeString(Platform::String^ type)
{
	AnnotationType _annottype = AnnotationType::NonSurport;

	if (type == "HighLight")
	{
		_annottype = AnnotationType::HightLight;
	}
	else if (type == "Squiggly")
	{
		_annottype = AnnotationType::Squiggly;
	}
	else if (type == "Link")
	{
		_annottype = AnnotationType::Link;
	}
	else if (type == "UnderLine")
	{
		_annottype = AnnotationType::UnderLine;
	}
	else if (type == "StrikeOut")
	{
		_annottype = AnnotationType::StrikeOut;
	}
	else if (type == "Square")
	{
		_annottype = AnnotationType::Square;
	}
	else if (type == "Circle")
	{
		_annottype = AnnotationType::Circle;
	}
	else if (type == "Line")
	{
		_annottype = AnnotationType::Line;
	}
	else if (type == "Ink")
	{
		_annottype = AnnotationType::Ink;
	}
	else if (type == "Text")
	{
		_annottype = AnnotationType::Text;
	}
	else if (type == "Stamp")
	{
		_annottype = AnnotationType::Stamp;
	}
	else if (type == "FreeText")
	{
		_annottype = AnnotationType::FreeText;
	}
	else if (type == "Caret")
	{
		_annottype = AnnotationType::Caret;
	}
	else if (type == "Widget")
	{
		_annottype = AnnotationType::Widget;
	}

	return GetAnnotHandlerByType(_annottype);
}

IBaseAnnot^ AnnotationManager::IsHitAnnot(int index, Windows::Foundation::Point ptDevice)
{
	if (m_AnnotCollections && m_AnnotCollections->HasKey(index))
	{
		AnnotCollections_Critsec->Lock();
		Windows::Foundation::Collections::IVector<IBaseAnnot^>^ pageCollections = m_AnnotCollections->Lookup(index);
		AnnotCollections_Critsec->Unlock();
		int nAnnotCount = pageCollections->Size;
		IBaseAnnot^ pAnnotSelect = nullptr;;
		for (int nAnnotIndex = 0; nAnnotIndex < nAnnotCount; nAnnotIndex++)
		{
			IBaseAnnot^ pAnnot = pageCollections->GetAt(nAnnotIndex);
			if (pAnnot->AnnotType == AnnotationType::NonSurport)
				continue;
			if (pAnnot->IsHitTestRange(ptDevice))
			{
				if (pAnnot->AnnotType == AnnotationType::Link)
				{
					pAnnotSelect = pAnnot;
				}
				else
				{
					return pAnnot;
				}
			}
		}
		return pAnnotSelect;
	}
	return nullptr;
}

Windows::Foundation::Collections::IVector<IBaseAnnot^>^ AnnotationManager::IsHitInkAnnot(int index, Windows::Foundation::Point ptDevice)
{
	if (m_AnnotCollections && m_AnnotCollections->HasKey(index))
	{
		Windows::Foundation::Collections::IVector<IBaseAnnot^>^ inkAnnotList = ref new  Platform::Collections::Vector<IBaseAnnot^>();
		AnnotCollections_Critsec->Lock();
		Windows::Foundation::Collections::IVector<IBaseAnnot^>^ pageCollections = m_AnnotCollections->Lookup(index);
		AnnotCollections_Critsec->Unlock();
		int nAnnotCount = pageCollections->Size;
		IBaseAnnot^ pAnnotSelect = nullptr;;
		for (int nAnnotIndex = 0; nAnnotIndex < nAnnotCount; nAnnotIndex++)
		{
			IBaseAnnot^ pAnnot = pageCollections->GetAt(nAnnotIndex);
			if (pAnnot->AnnotType == AnnotationType::Ink && pAnnot->IsHitTestRange(ptDevice))
			{
				inkAnnotList->Append(pAnnot);
			}
		}
		return inkAnnotList;
	}
	return nullptr;
}

IBaseAnnot^ AnnotationManager::GetAnnotHitPoint(int index, Windows::Foundation::Point pagePt)
{
	return nullptr;
}

Windows::Foundation::Collections::IVector<IBaseAnnot^>^ AnnotationManager::GetAnnotsHitPoint(int index, Windows::Foundation::Point pagePt)
{
	return nullptr;
}

bool AnnotationManager::IsInitAnnot(int pageindex)
{
	if (m_AnnoMgrPeer == nullptr)
		return false;

	if (pageindex >= 0 && pageindex < m_AnnoMgrPeer->GetPageCount())
	{
		if (m_LoadAnnotsContexts->HasKey(pageindex))
		{
			auto _context = m_LoadAnnotsContexts->Lookup(pageindex);
			return _context->IsLoadPageAnnotsCompletion();
		}
	}

	return false;
}

LoadPageAnnotsContext^ AnnotationManager::GetLoadPageAnnotsContext(int pageindex)
{
	if (pageindex >= 0 && m_AnnoMgrPeer && pageindex < m_AnnoMgrPeer->GetPageCount())
	{
		if (m_LoadAnnotsContexts->HasKey(pageindex))
		{
			auto _context = m_LoadAnnotsContexts->Lookup(pageindex);
			return _context;
		}
	}
	return nullptr;
}

void AnnotationManager::OnInitAnnot(int nPageindex)
{
	if (isExitPageView)
	{
		return;
	}
	auto _loadcontext = GetLoadPageAnnotsContext(nPageindex);
	if (_loadcontext && (_loadcontext->IsLoadingPageAnnots() || _loadcontext->IsLoadPageAnnotsCompletion()))
		return;

	InitOnePageAnnots(nPageindex);

	if (isExitPageView)
	{
		return;
	}
}

void AnnotationManager::ReInitAnnot(int nPageinex)
{
	if (isExitPageView)
	{
		return;
	}
	auto _loadcontext = GetLoadPageAnnotsContext(nPageinex);
	if (_loadcontext && (_loadcontext->IsLoadingPageAnnots() || !_loadcontext->IsLoadPageAnnotsCompletion()))
		return;

	if (m_AnnotCollections && m_AnnotCollections->HasKey(nPageinex))
	{
		AnnotCollections_Critsec->Lock();
		m_AnnotCollections->Remove(nPageinex);
		AnnotCollections_Critsec->Unlock();
	}

	if (m_PageNoteCollections && m_PageNoteCollections->HasKey(nPageinex))
	{
		AnnotCollections_Critsec->Lock();
		m_PageNoteCollections->Remove(nPageinex);
		AnnotCollections_Critsec->Unlock();
	}


	InitOnePageAnnots(nPageinex);

	if (isExitPageView)
	{
		return;
	}
}

void foxit::rdk::uiextensions::AnnotationManager::FinalizeAnnots(int nPageIndex)
{
	if (isExitPageView)
	{
		return;
	}
	auto _loadcontext = GetLoadPageAnnotsContext(nPageIndex);
	if (_loadcontext && (_loadcontext->IsLoadingPageAnnots() || !_loadcontext->IsLoadPageAnnotsCompletion()))
		return;

	if (m_AnnotCollections && m_AnnotCollections->HasKey(nPageIndex))
	{
		AnnotCollections_Critsec->Lock();
		m_AnnotCollections->Remove(nPageIndex);
		AnnotCollections_Critsec->Unlock();
	}

	if (m_PageNoteCollections && m_PageNoteCollections->HasKey(nPageIndex))
	{
		AnnotCollections_Critsec->Lock();
		m_PageNoteCollections->Remove(nPageIndex);
		AnnotCollections_Critsec->Unlock();
	}
}

void AnnotationManager::DeleteAnnotCollection(int pageIndex)
{
	if (isExitPageView)
	{
		return;
	}
	auto _loadcontext = GetLoadPageAnnotsContext(pageIndex);
	if (_loadcontext && (_loadcontext->IsLoadingPageAnnots() || !_loadcontext->IsLoadPageAnnotsCompletion()))
		return;

	if (m_AnnotCollections && m_AnnotCollections->HasKey(pageIndex))
	{
		AnnotCollections_Critsec->Lock();
		m_AnnotCollections->Remove(pageIndex);
		AnnotCollections_Critsec->Unlock();
	}

	if (m_PageNoteCollections && m_PageNoteCollections->HasKey(pageIndex))
	{
		AnnotCollections_Critsec->Lock();
		m_PageNoteCollections->Remove(pageIndex);
		AnnotCollections_Critsec->Unlock();
	}

	NotifyDocumentChanged();
	
	UpdatePageView(pageIndex);
}

void AnnotationManager::RefleshOnePageAnnots(int nPageIndex)
{
	if (m_AnnotCollections && m_AnnotCollections->HasKey(nPageIndex))
	{
		UpdatePageView(nPageIndex);
	}
	else
	{
		OnInitAnnot(nPageIndex);
	}
}

void AnnotationManager::InitOnePageAnnots(int pageindex)
{
	if (isExitPageView)
		return;

	if (m_AnnoMgrPeer == nullptr)
	{
		return;
	}
	m_AnnoMgrPeer->GetPageCount();
	auto _pdfPageObj = dynamic_cast<foxit::pdf::PDFPage^>(m_AnnoMgrPeer->GetPdfPageObject(pageindex));
	if (_pdfPageObj == nullptr)
	{
		return;
	}

	auto _loadcontext = ref new LoadPageAnnotsContext(pageindex);
	if (m_LoadAnnotsContexts)
		m_LoadAnnotsContexts->Insert(pageindex, _loadcontext);
	_loadcontext->SetIsLoading();

	foxit::pdf::PDFPage^ pagehandle = _pdfPageObj;
	IAnnotSettingImplementor^ SettingPageview = m_pSettingPageview;
	Windows::Foundation::Collections::IVector<IBaseAnnot^>^ annotCollections = ref new Platform::Collections::Vector<IBaseAnnot^>();
	Windows::Foundation::Collections::IVector<IBaseAnnot^>^ noteCollections = ref new Platform::Collections::Vector<IBaseAnnot^>();
	Windows::Foundation::Collections::IVector<IBaseAnnot^>^ freetextCollections = ref new Platform::Collections::Vector<IBaseAnnot^>();
	if (m_AnnotCollections && !m_AnnotCollections->HasKey(pageindex))
	{
		foxit::pdf::annots::Annot^ annotHandle;
		int count = pagehandle->GetAnnotCount();

		for (int i = 0; i < count; i++)
		{
			annotHandle = pagehandle->GetAnnot(i);

			PDFAnnotHandle^ pdfAnnot = ref new PDFAnnotHandle(annotHandle);
			String^ csType = nullptr;
			foxit::pdf::annots::AnnotType annotType = annotHandle->GetType();

			csType = FoxitTypeConvert::ConvertFromFoxitAnnotType(annotType);

			auto _annothandler = GetAnnotHandlerByTypeString(csType);
			IBaseAnnot^ _annot = nullptr;
			if (_annothandler)
			{
				_annot = _annothandler->OnLoadAnnot(_pdfPageObj, pdfAnnot);
			}
			if (_annot && (_annot->AnnotType == AnnotationType::Text 
				|| _annot->AnnotType == AnnotationType::Link || _annot->AnnotType == AnnotationType::FreeText))
			{
				noteCollections->Append(_annot);
			}

			if (_annot)
				annotCollections->Append(_annot);
		}
		AnnotCollections_Critsec->Lock();
		m_AnnotCollections->Insert(pageindex, annotCollections);
		AnnotCollections_Critsec->Unlock();

		AnnotCollections_Critsec->Lock();
		m_PageNoteCollections->Insert(pageindex, noteCollections);
		AnnotCollections_Critsec->Unlock();
	}
	_loadcontext->SetCompletion();
}

void AnnotationManager::NotifyDocumentChanged()
{
	if (m_AnnoMgrPeer)
	{
		m_AnnoMgrPeer->NotifyDocumentChanged();
	}
}

void AnnotationManager::UpdatePageView(int pageindex)
{
	if (m_AnnotCollections && !m_AnnotCollections->HasKey(pageindex))
		return;

	if (m_AnnoMgrPeer)
	{
		m_AnnoMgrPeer->UpdatePageView(pageindex);
	}
}

bool AnnotationManager::IsNeedDrawAnnot(IBaseAnnot^ annot)
{
	if (!annot)
		return false;

	auto _annottype = annot->AnnotType;
	return (_annottype == AnnotationType::HightLight || _annottype == AnnotationType::UnderLine || _annottype == AnnotationType::NonSurport
		|| _annottype == AnnotationType::StrikeOut || _annottype == AnnotationType::Squiggly
		|| _annottype == AnnotationType::Circle || _annottype == AnnotationType::Square
		|| _annottype == AnnotationType::Line || _annottype == AnnotationType::PolyLine
		|| _annottype == AnnotationType::Ink || _annottype == AnnotationType::Polygon
		|| _annottype == AnnotationType::Stamp || _annottype == AnnotationType::FreeText
		|| _annottype == AnnotationType::Caret /* || _annottype == AnnotationType::Widget*/) ? true : false;
}

Windows::Foundation::Collections::IVector<PDFAnnotHandle^>^ AnnotationManager::GetDrawingAnnots(int pageindex)
{
	Windows::Foundation::Collections::IVector<PDFAnnotHandle^>^ _pageCollections = ref new Platform::Collections::Vector<PDFAnnotHandle^>();

	OutputDebugString(L"GetDrawingAnnots\n");
	if (m_AnnotCollections && m_AnnotCollections->Size > 0 && m_AnnotCollections->HasKey(pageindex))
	{
		OutputDebugString(L"GetDrawingAnnots Locked\n");
		auto _annotsCollection = m_AnnotCollections->Lookup(pageindex);
		OutputDebugString(L"GetDrawingAnnots UnLocked\n");
		if (_annotsCollection)
		{
			auto size = _annotsCollection->Size;
			for (unsigned int i = 0; i < size; i++)
			{
				if (i < _annotsCollection->Size)
				{
					auto _annot = _annotsCollection->GetAt(i);
					if (_annot && IsNeedDrawAnnot(_annot))
					{
						_pageCollections->Append(_annot->AnnotHandleWrap);
					}
				}
			}
		}
	}
	return _pageCollections;
}

void AnnotationManager::OnPointerEntered(Platform::Object^ sender, Windows::UI::Xaml::Input::PointerRoutedEventArgs^ e)
{

}

void AnnotationManager::OnPointerExited(Platform::Object^ sender, Windows::UI::Xaml::Input::PointerRoutedEventArgs^ e)
{

}

void AnnotationManager::OnPointerMoved(Platform::Object^ sender, Windows::UI::Xaml::Input::PointerRoutedEventArgs^ e)
{
	if (m_AnnoMgrPeer)
	{
		m_AnnoMgrPeer->OnAnnotPointerMoved(sender, e);
	}
}

void AnnotationManager::OnPointerPressed(Platform::Object^ sender, Windows::UI::Xaml::Input::PointerRoutedEventArgs^ e)
{
	if (m_AnnoMgrPeer)
	{
		m_AnnoMgrPeer->OnAnnotPointerPressed(sender, e);
	}
}

void AnnotationManager::OnPointerReleased(Platform::Object^ sender, Windows::UI::Xaml::Input::PointerRoutedEventArgs^ e)
{
	if (m_AnnoMgrPeer)
	{
		m_AnnoMgrPeer->OnAnnotPointerReleased(sender, e);
	}
}

void AnnotationManager::OnTapped(Platform::Object^ sender, Windows::UI::Xaml::Input::TappedRoutedEventArgs^ e)
{
	if (m_AnnoMgrPeer)
	{
		m_AnnoMgrPeer->OnAnnotTapped(sender, e);
	}
}

void AnnotationManager::OnRightTapped(Platform::Object^ sender, Windows::UI::Xaml::Input::RightTappedRoutedEventArgs^ e)
{
	if (m_AnnoMgrPeer)
	{
		m_AnnoMgrPeer->OnAnnotRightTapped(sender, e);
	}
}

Windows::Foundation::Collections::IVector<IBaseAnnot^>^ AnnotationManager::GetPageAnnotCollection(int pageindex)
{
	Platform::Collections::Vector<IBaseAnnot^>^ _annotsvect = ref new Platform::Collections::Vector<IBaseAnnot^>();
	if (m_pSelectedAnnot)
	{
		_annotsvect->Append(m_pSelectedAnnot);
	}

	return _annotsvect;
}

void AnnotationManager::OnManipulationStarted(Platform::Object^ sender, Windows::UI::Xaml::Input::ManipulationStartedRoutedEventArgs^ e)
{
	m_AnnoMgrPeer->OnAnnotManipulationStarted(sender, e);
}

void AnnotationManager::OnManipulationDelta(Platform::Object^ sender, Windows::UI::Xaml::Input::ManipulationDeltaRoutedEventArgs^ e)
{
	m_AnnoMgrPeer->OnAnnotManipulationDelta(sender, e);
}

void AnnotationManager::OnManipulationCompleted(Platform::Object^ sender, Windows::UI::Xaml::Input::ManipulationCompletedRoutedEventArgs^ e)
{
	m_AnnoMgrPeer->OnAnnotManipulationCompleted(sender, e);
}

void AnnotationManager::OnManipulationInertiaStarting(Platform::Object^ sender, Windows::UI::Xaml::Input::ManipulationInertiaStartingRoutedEventArgs^ e)
{
	m_AnnoMgrPeer->OnManipulationInertiaStarting(sender, e);
}

Windows::Foundation::Rect AnnotationManager::InitMinRectFroTextAnnot(Windows::Foundation::Collections::IVector<Windows::Foundation::Rect>^ DeviceRectGroup)
{
	Rect minrectDevice;
	if (DeviceRectGroup == nullptr)
		return Windows::UI::Xaml::RectHelper::Empty;

	if (DeviceRectGroup->Size == 1)
	{
		minrectDevice = DeviceRectGroup->GetAt(0);
	}
	else
	{
		Rect rect = DeviceRectGroup->GetAt(0);
		float left = rect.Left;
		float top = rect.Top;
		float bottom = rect.Bottom;
		float right = rect.Right;
		for (unsigned int i = 1; i < DeviceRectGroup->Size; i++)
		{
			Windows::Foundation::Rect rectCur = DeviceRectGroup->GetAt(i);
			float bottomcur = rectCur.Bottom;
			if (rectCur.Left == 0 || rectCur.Top == 0 || rect.Width == 0 || rect.Height == 0)
			{
				continue;
			}
			if (left > rectCur.Left)
			{
				left = rectCur.Left;
			}
			if (top > rectCur.Top)
			{
				top = rectCur.Top;
			}
			if (bottom < rectCur.Bottom)
			{
				bottom = rectCur.Bottom;
			}
			if (right < rectCur.Right)
			{
				right = rectCur.Right;
			}
		}
		Windows::Foundation::Rect minrect;
		minrect.X = left;
		minrect.Y = top;
		minrect.Width = right - left + 2;
		minrect.Height = bottom - top + 2;
		minrectDevice = minrect;
	}
	return minrectDevice;
}

void AnnotationManager::SetPdfSelectAnnotNull()
{
	if (m_AnnoMgrPeer)
	{
		m_AnnoMgrPeer->InsertAnnotModifyUndo();
	}

	m_pSelectedAnnot = nullptr;
	SetGroupAnnotNullptr();
}

void AnnotationManager::InsertModifyAnnotBeforeDelete()
{
	if (m_AnnoMgrPeer)
	{
		m_AnnoMgrPeer->InsertAnnotModifyUndo();
	}
}

IBaseAnnot^ AnnotationManager::GetSelectedAnnot()
{
	return m_pSelectedAnnot;
}

void AnnotationManager::SetSelectedAnnot(IBaseAnnot^ annot)
{
	if (annot && m_pSelectedAnnot == annot)
		return;

	if (m_pSelectedAnnot)
	{
		m_pSelectedAnnot->SetSelectedState(false);
		auto _annothandler = GetAnnotHandlerByType(m_pSelectedAnnot->AnnotType);
		if (_annothandler)
			_annothandler->Reset();
	}

	if (annot)
		annot->SetSelectedState(true);

	m_pSelectedAnnot = annot;
	if (m_pSelectedAnnot)
	{
		if (m_pSelectedAnnot->AnnotType == AnnotationType::StrikeOut)
		{
			StrikeOutAnnot^ strikeoutannot = dynamic_cast<StrikeOutAnnot^>(annot);
			if (strikeoutannot->IsHasGroup)
			{
				Windows::Foundation::Rect groupminRect = strikeoutannot->GroupAnnotMinRect;
				bool isGetGroupAnnot = IsGetGroupAnnotElement(annot->PageIndex, groupminRect, strikeoutannot->GroupAnnotIndex, AnnotationType::Caret);
			}
		}
		else if (m_pSelectedAnnot->AnnotType == AnnotationType::Caret)
		{
			CaretAnnot^ caretannot = dynamic_cast<CaretAnnot^>(annot);
			if (caretannot->IsHasGroup)
			{
				Windows::Foundation::Rect groupminRect = caretannot->GroupAnnotMinRect;
				bool isGetGroupAnnot = IsGetGroupAnnotElement(annot->PageIndex, groupminRect, caretannot->GroupAnnotIndex, AnnotationType::StrikeOut);
				m_pSelectedAnnot = GetGroupAnnot();
				m_pSelectedGroupAnnot = caretannot;
			}
		}
	}
	else
	{
		SetGroupAnnotNullptr();
	}

	if (m_AnnoMgrPeer)
	{
		m_AnnoMgrPeer->SaveAnonotModifyBeforeProperty();
	}
}

void AnnotationManager::EndDrawing()
{

}

Windows::Foundation::Collections::IMap<int, Windows::Foundation::Collections::IVector<IBaseAnnot^>^>^  AnnotationManager::PageNoteCollections::get()
{
	return m_PageNoteCollections;
}

Windows::Foundation::Collections::IVector<IBaseAnnot^>^ AnnotationManager::GetPageNotes(int pageIndex)
{
	if (m_PageNoteCollections && m_PageNoteCollections->HasKey(pageIndex))
	{
		return m_PageNoteCollections->Lookup(pageIndex);
	}
	else
	{
		return nullptr;
	}
}

Windows::Foundation::Collections::IVector<IBaseAnnot^>^ AnnotationManager::GetPageWidgetAnnots(int pageIndex)
{
	Platform::Collections::Vector<IBaseAnnot^>^ _WidgetAnnots = ref new Platform::Collections::Vector<IBaseAnnot^>();

	if (m_AnnotCollections && m_AnnotCollections->HasKey(pageIndex))
	{
		auto _AllAnnots = m_AnnotCollections->Lookup(pageIndex);
		
		for each (IBaseAnnot^ var in _AllAnnots)
		{
			if (var->AnnotType ==  AnnotationType::Widget)
			{
				_WidgetAnnots->Append(var);
			}
		}
	}

	return _WidgetAnnots;
}

void AnnotationManager::UpdateNoteRect(int pageIndex, Windows::Foundation::Rect rect)
{
	if (m_PageNoteCollections && m_PageNoteCollections->HasKey(pageIndex))
	{
		Windows::Foundation::Collections::IVector<IBaseAnnot^>^ annots = ref new Platform::Collections::Vector<IBaseAnnot^>();
		if (m_AnnotCollections && m_AnnotCollections->Size > 0 && m_AnnotCollections->HasKey(pageIndex))
		{
			AnnotCollections_Critsec->Lock();
			Windows::Foundation::Collections::IVector<IBaseAnnot^>^ pageCollections = m_AnnotCollections->Lookup(pageIndex);
			AnnotCollections_Critsec->Unlock();
			for (unsigned int i = 0; i < pageCollections->Size; i++)
			{
				if (pageCollections->GetAt(i)->AnnotType == AnnotationType::Text || pageCollections->GetAt(i)->AnnotType == AnnotationType::Link)
				{
					Windows::Foundation::Rect rect = pageCollections->GetAt(i)->MinRect;
					annots->Append(pageCollections->GetAt(i));
				}
			}
		}
		AnnotCollections_Critsec->Lock();
		m_PageNoteCollections->Remove(pageIndex);
		m_PageNoteCollections->Insert(pageIndex, annots);
		AnnotCollections_Critsec->Unlock();
	}
}

void AnnotationManager::SetNewCreateAnnot(Windows::Foundation::Rect rect)
{
	createAnnotRect = rect;
}

Windows::Foundation::Rect  AnnotationManager::GetNewCreateAnnotRect()
{
	return createAnnotRect;
}

void AnnotationManager::ClearResource()
{
	if (m_LoadAnnotsContexts && m_LoadAnnotsContexts->Size > 0)
		m_LoadAnnotsContexts->Clear();
	m_LoadAnnotsContexts = nullptr;

	if (m_AnnotCollections && m_AnnotCollections->Size > 0)
		m_AnnotCollections->Clear();
	m_AnnotCollections = nullptr;

	if (m_PageNoteCollections && m_PageNoteCollections->Size > 0)
		m_PageNoteCollections->Clear();
	m_PageNoteCollections = nullptr;

	m_pSettingPageview = nullptr;
	isExitPageView = true;
}

void AnnotationManager::UpdateAnnotMinRect(int pageIndex, AnnotationType type, Windows::Foundation::Rect oldrect, Windows::Foundation::Rect minrect)
{
	if (m_PageNoteCollections && m_PageNoteCollections->HasKey(pageIndex))
	{
		if (m_AnnotCollections && m_AnnotCollections->Size > 0 && m_AnnotCollections->HasKey(pageIndex))
		{
			AnnotCollections_Critsec->Lock();
			Windows::Foundation::Collections::IVector<IBaseAnnot^>^ pageCollections = m_AnnotCollections->Lookup(pageIndex);
			AnnotCollections_Critsec->Unlock();
			for (unsigned int i = 0; i < pageCollections->Size; i++)
			{
				if (pageCollections->GetAt(i)->AnnotType == type && pageCollections->GetAt(i)->MinRect == oldrect)
				{
					pageCollections->GetAt(i)->MinRect = minrect;
				}
			}
		}
	}
}

void AnnotationManager::AnnotCollections_CritsecLock()
{
	annotCollectIsLock = true;
	AnnotCollections_Critsec->Lock();
}

void AnnotationManager::AnnotCollections_CritsecUnLock()
{
	annotCollectIsLock = false;
	AnnotCollections_Critsec->Unlock();
}

bool AnnotationManager::IsGetGroupAnnotElement(int pageIndex, Windows::Foundation::Rect minrect, int annotIndex, AnnotationType groupAnnotType)
{
	bool isGetGroup = false;
	if (m_AnnotCollections && m_AnnotCollections->HasKey(pageIndex))
	{
		AnnotCollections_Critsec->Lock();
		Windows::Foundation::Collections::IVector<IBaseAnnot^>^ pageCollections = m_AnnotCollections->Lookup(pageIndex);
		AnnotCollections_Critsec->Unlock();
		for (unsigned int i = 0; i < pageCollections->Size; i++)
		{
			if (pageCollections->GetAt(i)->AnnotType == groupAnnotType
				&& pageCollections->GetAt(i)->MinRect == minrect
				&& pageCollections->GetAt(i)->AnnoIndex == annotIndex)
			{
				isGetGroup = true;
				m_pSelectedGroupAnnot = pageCollections->GetAt(i);
				break;
			}
		}
	}
	return isGetGroup;
}

IBaseAnnot^ AnnotationManager::GetGroupAnnot()
{
	return m_pSelectedGroupAnnot;
}

void AnnotationManager::SetGroupAnnotNullptr()
{
	m_pSelectedGroupAnnot = nullptr;
}

void AnnotationManager::ChangePageIndex(int srcIndex, int dstIndex)
{
	std::vector<std::tuple<LoadPageAnnotsContext^, Windows::Foundation::Collections::IVector<IBaseAnnot^>^, Windows::Foundation::Collections::IVector<IBaseAnnot^>^>> _TmpList;
	
	int nStart = srcIndex < dstIndex ? srcIndex : dstIndex;
	int nEnd = srcIndex < dstIndex ? dstIndex : srcIndex;

	for (int i = nStart; i <= nEnd; i++)
	{
		if (m_LoadAnnotsContexts->HasKey(i))
		{
			auto _SwapItem = m_LoadAnnotsContexts->Lookup(i);
			Windows::Foundation::Collections::IVector<IBaseAnnot^>^ _SwapAnnotsItem = nullptr;
			Windows::Foundation::Collections::IVector<IBaseAnnot^>^ _SwapNoteAnnotsItem = nullptr;

			if (m_AnnotCollections->HasKey(i))
			{
				_SwapAnnotsItem = m_AnnotCollections->Lookup(i);
				m_AnnotCollections->Remove(i);
			}
			
			if (m_PageNoteCollections->HasKey(i))
			{
				_SwapNoteAnnotsItem = m_PageNoteCollections->Lookup(i);
				m_PageNoteCollections->Remove(i);
			}

			_TmpList.push_back(std::make_tuple<LoadPageAnnotsContext^, Windows::Foundation::Collections::IVector<IBaseAnnot^>^>(std::move(_SwapItem), std::move(_SwapAnnotsItem), std::move(_SwapNoteAnnotsItem)));
		}
	}

	auto _TmpListSize = _TmpList.size();
	for (unsigned int i = 0; i < _TmpListSize; i++)
	{
		auto item = _TmpList[i];
		FS_INT32 index = -1;
		FS_RESULT iRet = -1;
		auto pLoadHandle = std::get<0>(item);
		auto pAnnots = std::get<1>(item);
		auto pNoteAnnots = std::get<2>(item);
		
		if (pAnnots)
		{
			if (pAnnots->Size > 0)
			{
				for each (IBaseAnnot^ var in pAnnots)
				{
					var->CheckPageIndex();
					if (index == -1)
						index = var->PageIndex;
				}
			}
		}

		if (index != -1)
		{
			pLoadHandle->SetPageIndex(index);
			m_AnnotCollections->Insert(index, pAnnots);
			if (pNoteAnnots)
				m_PageNoteCollections->Insert(index, pNoteAnnots);
		}
	}
}

void AnnotationManager::ChangePageIndex(Windows::Foundation::Collections::IVector<int>^ srcIndexs, int dstIndex)
{
	std::vector<std::tuple<LoadPageAnnotsContext^, Windows::Foundation::Collections::IVector<IBaseAnnot^>^, Windows::Foundation::Collections::IVector<IBaseAnnot^>^>> _TmpList;

	int nStart = dstIndex;
	int nEnd = dstIndex;
	auto srcIndexSize = srcIndexs->Size;
	for (unsigned int i = 0; i < srcIndexSize; i++)
	{
		auto nItem = srcIndexs->GetAt(i);
		if (nStart > nItem)
			nStart = nItem;
		if (nEnd < nItem)
			nEnd = nItem;
	}

	for (int i = nStart; i <= nEnd; i++)
	{
		if (m_LoadAnnotsContexts->HasKey(i))
		{
			auto _SwapItem = m_LoadAnnotsContexts->Lookup(i);
			Windows::Foundation::Collections::IVector<IBaseAnnot^>^ _SwapAnnotsItem = nullptr;
			Windows::Foundation::Collections::IVector<IBaseAnnot^>^ _SwapNoteAnnotsItem = nullptr;

			if (m_AnnotCollections->HasKey(i))
			{
				_SwapAnnotsItem = m_AnnotCollections->Lookup(i);
				m_AnnotCollections->Remove(i);
			}

			if (m_PageNoteCollections->HasKey(i))
			{
				_SwapNoteAnnotsItem = m_PageNoteCollections->Lookup(i);
				m_PageNoteCollections->Remove(i);
			}
			m_LoadAnnotsContexts->Remove(i);

			_TmpList.push_back(std::make_tuple<LoadPageAnnotsContext^, Windows::Foundation::Collections::IVector<IBaseAnnot^>^>(std::move(_SwapItem), std::move(_SwapAnnotsItem), std::move(_SwapNoteAnnotsItem)));
		}
	}

	auto _TmpListSize = _TmpList.size();
	for (unsigned int i = 0; i < _TmpListSize; i++)
	{
		auto item = _TmpList[i];
		FS_INT32 index = -1;
		FS_RESULT iRet = -1;
		auto pLoadHandle = std::get<0>(item);
		auto pAnnots = std::get<1>(item);
		auto pNoteAnnots = std::get<2>(item);

		if (pAnnots)
		{
			if (pAnnots->Size > 0)
			{
				for each (IBaseAnnot^ var in pAnnots)
				{
					var->CheckPageIndex();
					if (index == -1)
						index = var->PageIndex;
				}
			}
		}

		auto _PrevIndex = pLoadHandle->PageIndex;
		if (index != -1)
		{
			pLoadHandle->SetPageIndex(index);
			
			m_LoadAnnotsContexts->Insert(index, pLoadHandle);
			m_AnnotCollections->Insert(index, pAnnots);
			if (pNoteAnnots)
				m_PageNoteCollections->Insert(index, pNoteAnnots);
		}
		else
		{
			if (!m_LoadAnnotsContexts->HasKey(_PrevIndex))
			{
				m_LoadAnnotsContexts->Insert(_PrevIndex, pLoadHandle);
				m_AnnotCollections->Insert(_PrevIndex, pAnnots);
				if (pNoteAnnots)
					m_PageNoteCollections->Insert(_PrevIndex, pNoteAnnots);
			}
		}
	}

	for (int i = nStart; i <= nEnd; i++)
	{
		if (!m_LoadAnnotsContexts->HasKey(i))
		{
			Windows::Foundation::Collections::IVector<IBaseAnnot^>^ annotCollections = ref new Platform::Collections::Vector<IBaseAnnot^>();
			Windows::Foundation::Collections::IVector<IBaseAnnot^>^ noteAnnotCollections = ref new Platform::Collections::Vector<IBaseAnnot^>();
			
			auto _loadcontext = ref new LoadPageAnnotsContext(i);
			m_LoadAnnotsContexts->Insert(i, _loadcontext);
			m_AnnotCollections->Insert(i, annotCollections);
			m_PageNoteCollections->Insert(i, noteAnnotCollections);
			_loadcontext->SetCompletion();
		}
	}
}

void AnnotationManager::PageDeleteBefore(Windows::Foundation::Collections::IVector<int>^ pageIndexs)
{
	auto _PageCount = m_AnnoMgrPeer->GetPageCount();

	Platform::Array<int>^ _TmpArray = ref new Platform::Array<int>(pageIndexs->Size);
	pageIndexs->GetMany(0, _TmpArray);

	std::vector<int> _pageIndexVect(_TmpArray->begin(), _TmpArray->end());
	std::sort(_pageIndexVect.begin(), _pageIndexVect.end(), std::less<int>());
	_pageIndexVect.push_back(_PageCount);
	auto srcIndexSize = _pageIndexVect.size();
	
	std::vector<std::tuple<LoadPageAnnotsContext^, Windows::Foundation::Collections::IVector<IBaseAnnot^>^, Windows::Foundation::Collections::IVector<IBaseAnnot^>^>> _TmpList;
	int nDelete = 1;
	for (size_t i = 0; i < srcIndexSize; i++)
	{
		if (i == srcIndexSize - 1)
		{
			break;
		}

		auto iStart = _pageIndexVect[i];
		auto iEnd = _pageIndexVect[i+1];

		for (int j = iStart; j < iEnd; j++)
		{
			auto pageIndex = j;
			if (m_LoadAnnotsContexts->HasKey(pageIndex))
			{
				auto _SwapItem = m_LoadAnnotsContexts->Lookup(pageIndex);
				Windows::Foundation::Collections::IVector<IBaseAnnot^>^ _SwapAnnotsItem = nullptr;
				Windows::Foundation::Collections::IVector<IBaseAnnot^>^ _SwapNoteAnnotsItem = nullptr;

				if (m_AnnotCollections->HasKey(pageIndex))
				{
					_SwapAnnotsItem = m_AnnotCollections->Lookup(pageIndex);
					m_AnnotCollections->Remove(pageIndex);
				}

				if (m_PageNoteCollections->HasKey(pageIndex))
				{
					_SwapNoteAnnotsItem = m_PageNoteCollections->Lookup(pageIndex);
					m_PageNoteCollections->Remove(pageIndex);
				}

				m_LoadAnnotsContexts->Remove(pageIndex);
				if (_SwapItem)
				{
					_SwapItem->SetPageIndex(pageIndex - nDelete);
				}

				if (j != iStart)
					_TmpList.push_back(std::make_tuple<LoadPageAnnotsContext^, Windows::Foundation::Collections::IVector<IBaseAnnot^>^, Windows::Foundation::Collections::IVector<IBaseAnnot^>^>(std::move(_SwapItem), std::move(_SwapAnnotsItem), std::move(_SwapNoteAnnotsItem)));
			}
		}

		nDelete++;
	}

	auto _TmpListSize = _TmpList.size();
	for (unsigned int i = 0; i < _TmpListSize; i++)
	{
		auto item = _TmpList[i];
		FS_INT32 index = -1;
		FS_RESULT iRet = -1;
		auto pLoadHandle = std::get<0>(item);
		auto pAnnots = std::get<1>(item);
		auto pNoteAnnots = std::get<2>(item);

		auto _PrevIndex = pLoadHandle->PageIndex;

		m_LoadAnnotsContexts->Insert(_PrevIndex, pLoadHandle);
		m_AnnotCollections->Insert(_PrevIndex, pAnnots);
		if (pNoteAnnots)
			m_PageNoteCollections->Insert(_PrevIndex, pNoteAnnots);
	}
}

void AnnotationManager::PageDeleteAfter(Windows::Foundation::Collections::IVector<int>^ pageIndexs)
{
	std::vector<std::tuple<LoadPageAnnotsContext^, Windows::Foundation::Collections::IVector<IBaseAnnot^>^, Windows::Foundation::Collections::IVector<IBaseAnnot^>^>> _TmpList;

	auto _PageCount = m_AnnoMgrPeer->GetPageCount();
	int nStart = _PageCount;
	int nEnd = _PageCount;

	auto srcIndexSize = pageIndexs->Size;
	for (unsigned int i = 0; i < srcIndexSize; i++)
	{
		auto nItem = pageIndexs->GetAt(i);
		if (nStart > nItem)
			nStart = nItem;
	}

	for (int i = nStart; i <= nEnd; i++)
	{
		if (m_LoadAnnotsContexts->HasKey(i))
		{
			auto _SwapItem = m_LoadAnnotsContexts->Lookup(i);
			Windows::Foundation::Collections::IVector<IBaseAnnot^>^ _SwapAnnotsItem = nullptr;
			Windows::Foundation::Collections::IVector<IBaseAnnot^>^ _SwapNoteAnnotsItem = nullptr;
			
			if (m_AnnotCollections->HasKey(i))
			{
				_SwapAnnotsItem = m_AnnotCollections->Lookup(i);
				m_AnnotCollections->Remove(i);
			}

			if (m_PageNoteCollections->HasKey(i))
			{
				_SwapNoteAnnotsItem = m_PageNoteCollections->Lookup(i);
				m_PageNoteCollections->Remove(i);
			}
			m_LoadAnnotsContexts->Remove(i);

			_TmpList.push_back(std::make_tuple<LoadPageAnnotsContext^, Windows::Foundation::Collections::IVector<IBaseAnnot^>^, Windows::Foundation::Collections::IVector<IBaseAnnot^>^>(std::move(_SwapItem), std::move(_SwapAnnotsItem), std::move(_SwapNoteAnnotsItem)));
		}
	}

	unsigned int _CurIndex = nStart;
	auto _TmpListSize = _TmpList.size();
	for (unsigned int i = 0; i < _TmpListSize; i++)
	{
		auto item = _TmpList[i];
		FS_INT32 index = -1;
		FS_RESULT iRet = -1;
		auto pLoadHandle = std::get<0>(item);
		auto pAnnots = std::get<1>(item);
		auto pNoteAnnots = std::get<2>(item);

		if (pAnnots)
		{
			if (pAnnots->Size > 0)
			{
				for each (IBaseAnnot^ var in pAnnots)
				{
					var->CheckPageIndex();
					if (index == -1)
						index = var->PageIndex;
				}
			}
		}

		auto _PrevIndex = pLoadHandle->PageIndex;
		if (index != -1)
		{
			pLoadHandle->SetPageIndex(index);

			m_AnnotCollections->Insert(index, pAnnots);
			if (pNoteAnnots)
				m_PageNoteCollections->Insert(index, pNoteAnnots);

			if (_CurIndex == index)
			{
				m_LoadAnnotsContexts->Insert(index, pLoadHandle);
				_CurIndex++;
			}
			
		}
		else
		{
			pLoadHandle->SetPageIndex(_CurIndex);

			m_LoadAnnotsContexts->Insert(_CurIndex, pLoadHandle);
			_CurIndex++;
		}
	}
}

void AnnotationManager::CheckPageIndex(int startIndex, int increasingCount)
{
	std::vector<std::tuple<LoadPageAnnotsContext^, Windows::Foundation::Collections::IVector<IBaseAnnot^>^, Windows::Foundation::Collections::IVector<IBaseAnnot^>^>> _TmpList;

	int nStart = startIndex;
	int nEnd = m_AnnoMgrPeer->GetPageCount();

	for (int i = nStart; i < nEnd; i++)
	{
		if (m_LoadAnnotsContexts->HasKey(i))
		{
			auto _SwapItem = m_LoadAnnotsContexts->Lookup(i);
			Windows::Foundation::Collections::IVector<IBaseAnnot^>^ _SwapAnnotsItem = nullptr;
			Windows::Foundation::Collections::IVector<IBaseAnnot^>^ _SwapNoteAnnotsItem = nullptr;

			if (m_AnnotCollections->HasKey(i))
			{
				_SwapAnnotsItem = m_AnnotCollections->Lookup(i);
				m_AnnotCollections->Remove(i);
			}

			if (m_PageNoteCollections->HasKey(i))
			{
				_SwapNoteAnnotsItem = m_PageNoteCollections->Lookup(i);
				m_PageNoteCollections->Remove(i);
			}

			m_LoadAnnotsContexts->Remove(i);

			_TmpList.push_back(std::make_tuple<LoadPageAnnotsContext^, Windows::Foundation::Collections::IVector<IBaseAnnot^>^>(std::move(_SwapItem), std::move(_SwapAnnotsItem), std::move(_SwapNoteAnnotsItem)));
		}
	}

	auto _TmpListSize = _TmpList.size();
	for (unsigned int i = 0; i < _TmpListSize; i++)
	{
		auto item = _TmpList[i];
		FS_INT32 index = -1;
		FS_RESULT iRet = -1;
		auto pLoadHandle = std::get<0>(item);
		auto pAnnots = std::get<1>(item);
		auto pNoteAnnots = std::get<2>(item);

		if (pAnnots)
		{
			if (pAnnots->Size > 0)
			{
				for each (IBaseAnnot^ var in pAnnots)
				{
					var->CheckPageIndex();
					if (index == -1)
						index = var->PageIndex;
				}
			}
		}

		auto _PrevIndex = pLoadHandle->PageIndex;
		if (index != -1)
		{
			pLoadHandle->SetPageIndex(index);
			m_LoadAnnotsContexts->Insert(index, pLoadHandle);
			m_AnnotCollections->Insert(index, pAnnots);
			if (pNoteAnnots)
				m_PageNoteCollections->Insert(index, pNoteAnnots);
		}
		else
		{
			if (_PrevIndex != startIndex)
			{
				_PrevIndex = _PrevIndex + increasingCount;
				pLoadHandle->SetPageIndex(_PrevIndex);
			}
				
			m_LoadAnnotsContexts->Insert(_PrevIndex, pLoadHandle);
			m_AnnotCollections->Insert(_PrevIndex, pAnnots);
			if (pNoteAnnots)
				m_PageNoteCollections->Insert(_PrevIndex, pNoteAnnots);
		}
	}
}

void AnnotationManager::OnNotifyAnnotModify(IBaseAnnot^ pAnnot)
{
	this->ModifyAnnotEvent(this, pAnnot);
}
