/**
 * Copyright (C) 2003-2018, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit Mobile PDF SDK to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */


#pragma once


namespace foxit {
	namespace rdk {
		namespace uiextensions {
			interface class IBaseAnnot;
			interface class IAnnotManager;
		}
	}
}

namespace foxit
{
	namespace rdk
	{
		namespace uiextensions
		{
			ref class LoadPageAnnotsContext
			{
			public:
				virtual ~LoadPageAnnotsContext();
				property int PageIndex
				{
					int get();
				}
				bool IsLoadPageAnnotsCompletion();
				bool IsLoadingPageAnnots();
				void SetIsLoading();
				void SetCompletion();

			internal:
				LoadPageAnnotsContext(int pageindex);

				void SetPageIndex(int pageIndex);

			internal:
				int _pageindex;
				bool _bIsLoading;
				bool _bIsCompletion;
				concurrency::task_completion_event<void> _completionEvent;
			};

			
			[Windows::Foundation::Metadata::WebHostHidden]
			public ref class AnnotationManager sealed : public IAnnotManager
			{
			public:
				AnnotationManager();
				virtual ~AnnotationManager();

				property Windows::Foundation::Collections::IMap<int, Windows::Foundation::Collections::IVector<IBaseAnnot^>^>^ AnnotCollections
				{
					Windows::Foundation::Collections::IMap<int, Windows::Foundation::Collections::IVector<IBaseAnnot^>^>^  get();
				}

				property Windows::Foundation::Collections::IVector<IBaseAnnotHandler^>^ AnnotHanderGroup
				{
					Windows::Foundation::Collections::IVector<IBaseAnnotHandler^>^  get();
				}

				property Windows::Foundation::Collections::IMap<int, Windows::Foundation::Collections::IVector<IBaseAnnot^>^>^ PageNoteCollections
				{
					Windows::Foundation::Collections::IMap<int, Windows::Foundation::Collections::IVector<IBaseAnnot^>^>^  get();
				}

				Windows::UI::Core::CoreDispatcher^ GetmainPageDispatcher();

				virtual void OnCreate(IAnnotManagerPeer^ annotPeer);
				virtual bool OnLoaded();
				virtual void OnDelete();
				virtual void AddAnnot(IBaseAnnot^ annot);
				virtual void DeleteAnnot(IBaseAnnot^ annot);
				virtual void ClearAnnots(int nPageIndex);
				/// create new Annotation
				virtual void OnCreateNewAnnot(Platform::Object^ pageObj, IBaseAnnot^ newannot);
				/// load annotation
				virtual IBaseAnnot^ OnLoadAnnot(Platform::Object^ pageObj, PDFAnnotHandle^ annot);

				Windows::Foundation::Collections::IVector<IBaseAnnot^>^ GetPageAnnotCollection(int pageindex);

				virtual void ClearResource();
				virtual void AnnotCollections_CritsecLock();
				virtual void AnnotCollections_CritsecUnLock();
				virtual void OnCreateNewAnnotGroup(Platform::Object^ pageObj, Windows::Foundation::Collections::IVector<IBaseAnnot^>^ newannots);
				virtual bool IsGetGroupAnnotElement(int pageIndex, Windows::Foundation::Rect minrect, int annotIndex, AnnotationType groupAnnotType);
				virtual IBaseAnnot^ GetGroupAnnot();
				virtual void DeleteAnnotCollection(int pageIndex);

				virtual void OnNotifyAnnotModify(IBaseAnnot^ pAnnot);

				virtual event Windows::Foundation::EventHandler<IBaseAnnot^>^ AddAnnotEvent;
				virtual event Windows::Foundation::EventHandler<IBaseAnnot^>^ DeleteAnnotEvent;
				virtual event Windows::Foundation::EventHandler<IBaseAnnot^>^ ModifyAnnotEvent;

			public:
				IBaseAnnot^ GetAnnotFromMaps(int pageIndex, int index);
				void RegisterAnnotHandler(IBaseAnnotHandler^ AnnotHandler);
				virtual IBaseAnnot^ IsHitAnnot(int index, Windows::Foundation::Point ptDevice);
				virtual Windows::Foundation::Collections::IVector<IBaseAnnot^>^ IsHitInkAnnot(int index, Windows::Foundation::Point ptDevice);
				virtual IBaseAnnot^ GetAnnotHitPoint(int index, Windows::Foundation::Point pagePt);
				virtual Windows::Foundation::Collections::IVector<IBaseAnnot^>^ GetAnnotsHitPoint(int index, Windows::Foundation::Point pagePt);
				virtual IBaseAnnotHandler^ GetAnnotHandlerByType(AnnotationType type);
				IBaseAnnotHandler^ GetAnnotHandlerByTypeString(Platform::String^ type);
				bool IsNeedDrawAnnot(IBaseAnnot^ annot);
				virtual Windows::Foundation::Collections::IVector<PDFAnnotHandle^>^ GetDrawingAnnots(int pageindex);

				Windows::Foundation::Rect InitMinRectFroTextAnnot(Windows::Foundation::Collections::IVector<Windows::Foundation::Rect>^ DeviceRectGroup);

				virtual void NotifyDocumentChanged();
				virtual void UpdatePageView(int pageindex);
				virtual void SetGroupAnnotNullptr();
				virtual void InsertModifyAnnotBeforeDelete();

				bool IsInitAnnot(int pageindex);

			private:
				void InitializeSurportedAnnotHandler();
				void InitOnePageAnnots(int pageindex);
				LoadPageAnnotsContext^ GetLoadPageAnnotsContext(int pageindex);

			public:
				virtual IAnnotSettingImplementor^ GetAnnotSetting();
				virtual void SetAnnotSetting(IAnnotSettingImplementor^ pSettingPageview);

				virtual void OnInitAnnot(int nPageindex);
				virtual void ReInitAnnot(int nPageinex);
				virtual void FinalizeAnnots(int nPageIndex);

				virtual void OnPointerEntered(Platform::Object^ sender, Windows::UI::Xaml::Input::PointerRoutedEventArgs^ e);
				virtual void OnPointerExited(Platform::Object^ sender, Windows::UI::Xaml::Input::PointerRoutedEventArgs^ e);
				virtual void OnPointerMoved(Platform::Object^ sender, Windows::UI::Xaml::Input::PointerRoutedEventArgs^ e);
				virtual void OnPointerPressed(Platform::Object^ sender, Windows::UI::Xaml::Input::PointerRoutedEventArgs^ e);
				virtual void OnPointerReleased(Platform::Object^ sender, Windows::UI::Xaml::Input::PointerRoutedEventArgs^ e);
				virtual void OnTapped(Platform::Object^ sender, Windows::UI::Xaml::Input::TappedRoutedEventArgs^ e);
				virtual void OnRightTapped(Platform::Object^ sender, Windows::UI::Xaml::Input::RightTappedRoutedEventArgs^ e);

				virtual void OnManipulationStarted(Platform::Object^ sender, Windows::UI::Xaml::Input::ManipulationStartedRoutedEventArgs^ e);
				virtual void OnManipulationDelta(Platform::Object^ sender, Windows::UI::Xaml::Input::ManipulationDeltaRoutedEventArgs^ e);
				virtual void OnManipulationCompleted(Platform::Object^ sender, Windows::UI::Xaml::Input::ManipulationCompletedRoutedEventArgs^ e);
				virtual void OnManipulationInertiaStarting(Platform::Object^ sender, Windows::UI::Xaml::Input::ManipulationInertiaStartingRoutedEventArgs^ e);
				virtual void RefleshOnePageAnnots(int nPageIndex);

				virtual void SetPdfSelectAnnotNull();
				virtual IBaseAnnot^ GetSelectedAnnot();
				virtual void SetSelectedAnnot(IBaseAnnot^ annot);
				virtual void EndDrawing();
				virtual Windows::Foundation::Collections::IVector<IBaseAnnot^>^ GetPageNotes(int pageIndex);
				virtual Windows::Foundation::Collections::IVector<IBaseAnnot^>^ GetPageWidgetAnnots(int pageIndex);
				virtual void UpdateNoteRect(int pageIndex, Windows::Foundation::Rect rect);
				virtual void SetNewCreateAnnot(Windows::Foundation::Rect rect);
				virtual Windows::Foundation::Rect  GetNewCreateAnnotRect();
				virtual void UpdateAnnotMinRect(int pageIndex, AnnotationType type, Windows::Foundation::Rect oldrect, Windows::Foundation::Rect minrect);

				virtual void ChangePageIndex(int srcIndex, int dstIndex);
				virtual void ChangePageIndex(Windows::Foundation::Collections::IVector<int>^ srcIndexs, int dstIndex);
				virtual void PageDeleteBefore(Windows::Foundation::Collections::IVector<int>^ pageIndexs);
				virtual void PageDeleteAfter(Windows::Foundation::Collections::IVector<int>^ pageIndexs);
				virtual void CheckPageIndex(int startIndex, int increasingCount);

			private:
				std::shared_ptr<CritSec> AnnotCollections_Critsec;
				Platform::Collections::Map<int, Windows::Foundation::Collections::IVector<IBaseAnnot^>^>^ m_AnnotCollections;
				Platform::Collections::Map<int, LoadPageAnnotsContext^>^ m_LoadAnnotsContexts;
				Windows::Foundation::Collections::IVector<IBaseAnnotHandler^>^	m_AnnotHanderGroup;
				Windows::UI::Core::CoreDispatcher^	m_MainPageDispatcher;
				Platform::WeakReference _PageView;
				IAnnotSettingImplementor^ m_pSettingPageview;
				IAnnotManagerPeer^ m_AnnoMgrPeer;
				Platform::Collections::Map<int, Windows::Foundation::Collections::IVector<IBaseAnnot^>^>^ m_PageNoteCollections;
				bool annotCollectIsLock;
				Platform::Collections::Map<int, Windows::Foundation::Collections::IVector<IBaseAnnot^>^>^ m_FreeTextAnnotCollections;

			private:
				IBaseAnnotHandler^ m_pSelectedAnnotHandler;
				IBaseAnnot^ m_pSelectedAnnot;
				Platform::WeakReference m_PrevTappedFrameworkElementWeakRef;
				bool isExitPageView;
				Windows::Foundation::Rect  createAnnotRect;
				IBaseAnnot^ m_pSelectedGroupAnnot;
			};
		}
	}
}
