/**
 * Copyright (C) 2003-2018, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit Mobile PDF SDK to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */


#include "pch.h"
#include "AnnotSettingManager.h"

using namespace foxit::rdk::uiextensions;

using namespace Windows::Foundation;
using namespace Windows::Storage;
using namespace Windows::Storage::AccessCache;
using namespace Windows::ApplicationModel::Resources;
using namespace Windows::System;
using namespace concurrency;
using namespace Platform;
using namespace Platform::Collections;
using namespace Windows::Foundation::Collections;

AnnotSettingManager::AnnotSettingManager()
{
	ApplicationDataContainer^ localSettings = ApplicationData::Current->LocalSettings;
	auto values = localSettings->Values;
	_pencilopacity = 1;
	if (values->HasKey("AnnotProperty"))
	{
		composite = dynamic_cast<ApplicationDataCompositeValue^>(values->Lookup("AnnotProperty"));
	}
	else
	{
		composite = ref new ApplicationDataCompositeValue();
	}

	if (composite->HasKey("HightLightAnnotOpacity"))
	{
		_HighlightOpacity = (double)composite->Lookup("HightLightAnnotOpacity");
	}
	else
	{
		_HighlightOpacity = 1;
		SetHightLightAnnotOpacity(_HighlightOpacity);
	}

	if (composite->HasKey("HightLightAnnotColor"))
	{
		Platform::String^ strColor = safe_cast<Platform::String^>(composite->Lookup("HightLightAnnotColor"));
		_HighlightColor = FoxitTypeConvert::ConvertFromColorString(strColor);
	}
	else
	{
		_HighlightColor = FoxitTypeConvert::ConvertFromColorString("#FFFF00");
		SetHightLightAnnotColor(_HighlightColor);
	}

	if (composite->HasKey("UnderLineAnnotOpacity"))
	{
		_UnderLineOpacity = (double)composite->Lookup("UnderLineAnnotOpacity");
	}
	else
	{
		_UnderLineOpacity = 1;
		SetUnderLineAnnotOpacity(_UnderLineOpacity);
	}

	if (composite->HasKey("UnderLineAnnotColor"))
	{
		Platform::String^ strColor = safe_cast<Platform::String^>(composite->Lookup("UnderLineAnnotColor"));
		_UnderLineColor = FoxitTypeConvert::ConvertFromColorString(strColor);
	}
	else
	{
		_UnderLineColor = FoxitTypeConvert::ConvertFromColorString("#33CC00");
		SetUnderLineAnnotColor(_UnderLineColor);
	}

	if (composite->HasKey("StrikeOutAnnotOpacity"))
	{
		_StrikeOutOpacity = (double)composite->Lookup("StrikeOutAnnotOpacity");
	}
	else
	{
		_StrikeOutOpacity = 1;
		SetStrikeOutAnnotOpacity(_StrikeOutOpacity);
	}

	if (composite->HasKey("StrikeOutAnnotColor"))
	{
		Platform::String^ strColor = safe_cast<Platform::String^>(composite->Lookup("StrikeOutAnnotColor"));
		_StrikeOutColor = FoxitTypeConvert::ConvertFromColorString(strColor);
	}
	else
	{
		_StrikeOutColor = FoxitTypeConvert::ConvertFromColorString("#FF3333");
		SetStrikeOutAnnotColor(_StrikeOutColor);
	}

	if (composite->HasKey("SquigglyAnnotOpacity"))
	{
		_SquigglyOpacity = (double)composite->Lookup("SquigglyAnnotOpacity");
	}
	else
	{
		_SquigglyOpacity = 1;
		SetSquigglyAnnotOpacity(_SquigglyOpacity);
	}

	if (composite->HasKey("SquigglyAnnotColor"))
	{
		Platform::String^ strColor = safe_cast<Platform::String^>(composite->Lookup("SquigglyAnnotColor"));
		_SquigglyColor = FoxitTypeConvert::ConvertFromColorString(strColor);
	}
	else
	{
		_SquigglyColor = FoxitTypeConvert::ConvertFromColorString("#33CC00");
		SetSquigglyAnnotColor(_SquigglyColor);
	}	

	if (composite->HasKey("LineAnnotColor"))
	{
		Platform::String^ strColor = safe_cast<Platform::String^>(composite->Lookup("LineAnnotColor"));
		_PencilColor = FoxitTypeConvert::ConvertFromColorString(strColor);
	}
	else
	{
		_PencilColor = FoxitTypeConvert::ConvertFromColorString("#FFFF9F40");
		SetLineAnnotColor(_PencilColor);
	}

	if (composite->HasKey("AnnotDrawBorderWidth"))
	{
		_linewidth = (int)(composite->Lookup("AnnotDrawBorderWidth"));
	}
	else
	{
		_linewidth = 2;
		SetAnnotDrawBorderWidth(_linewidth);
	}

	if (composite->HasKey("AnnotOpacity"))
	{
		_opacity = (double)(composite->Lookup("AnnotOpacity"));
	}
	else
	{
		_opacity = 1;
		SetAnnotOpacity(_opacity);
	}

	if (composite->HasKey("AnnotColor"))
	{
		Platform::String^ strColor = safe_cast<Platform::String^>(composite->Lookup("AnnotColor"));
		_PencilColor = FoxitTypeConvert::ConvertFromColorString(strColor);
	}
	else
	{
		_PencilColor = FoxitTypeConvert::ConvertFromColorString("#8080FF");
		SetAnnotColor(_PencilColor);
	}

	if (composite->HasKey("NoteLogo"))
	{
		noteLogo = safe_cast<Platform::String^>(composite->Lookup("NoteLogo"));
	}
	else
	{
		noteLogo = "Comment";
		SetNoteLogoUrl(noteLogo);
	}

	if (composite->HasKey("AuthorName"))
	{
		_authorName = safe_cast<Platform::String^>(composite->Lookup("AuthorName"));
	}
	else
	{			
		_authorName = "";
	}
	if (composite->HasKey("IsLinkHighlight"))
	{
		_isLinkHightlight = safe_cast<bool>(composite->Lookup("IsLinkHighlight"));
	}
	else
	{
		_isLinkHightlight = false;
		SetIsLinkHighlight(_isLinkHightlight);
	}

	if (composite->HasKey("EraserWidth"))
	{
		_EraserWidth = safe_cast<int>(composite->Lookup("EraserWidth"));
	}
	else
	{
		_EraserWidth = 10;
		SetEraserWidth(_EraserWidth);
	}
	
	if (composite->HasKey("StampIndex"))
	{
		_StamIndex = safe_cast<int>(composite->Lookup("StampIndex"));
	}
	else
	{
		_StamIndex = 0;
		SetStampIndex(_StamIndex);
	}

	if (composite->HasKey("IsDarkMode"))
	{
		_isDarkMode = safe_cast<Platform::String^>(composite->Lookup("IsDarkMode"));
	}
	else
	{
		_isDarkMode = "0";
		SetViewIsDarkMode(_isDarkMode);
	}

	if (composite->HasKey("IsFirstUserAnnot"))
	{
		_isFirstUserAnnot = safe_cast<bool>(composite->Lookup("IsFirstUserAnnot"));
	}
	else
	{
		_isFirstUserAnnot = true;
		SetIsFirstUserAnnot(_isFirstUserAnnot);
	}

	if (composite->HasKey("HightLightIsContinueOpera"))
	{
		_HightLightIsContinueOpera = safe_cast<bool>(composite->Lookup("HightLightIsContinueOpera"));
	}
	else
	{
		_HightLightIsContinueOpera = false;
		SetHightLightIsContinueOpera(_HightLightIsContinueOpera);
	}

	if (composite->HasKey("AreaHightLightIsContinueOpera"))
	{
		_AreaHightLightIsContinueOpera = safe_cast<bool>(composite->Lookup("AreaHightLightIsContinueOpera"));
	}
	else
	{
		_AreaHightLightIsContinueOpera = false;
		SetAreaHightLightIsContinueOpera(_AreaHightLightIsContinueOpera);
	}

	if (composite->HasKey("StrikeOutIsContinueOpera"))
	{
		_StrikeOutIsContinueOpera = safe_cast<bool>(composite->Lookup("StrikeOutIsContinueOpera"));
	}
	else
	{
		_StrikeOutIsContinueOpera = false;
		SetStrikeOutIsContinueOpera(_StrikeOutIsContinueOpera);
	}

	if (composite->HasKey("UnderLineIsContinueOpera"))
	{
		_UnderLineIsContinueOpera = safe_cast<bool>(composite->Lookup("UnderLineIsContinueOpera"));
	}
	else
	{
		_UnderLineIsContinueOpera = false;
		SetUnderLineIsContinueOpera(_UnderLineIsContinueOpera);
	}

	if (composite->HasKey("SquigglyIsContinueOpera"))
	{
		_SquigglyIsContinueOpera = safe_cast<bool>(composite->Lookup("SquigglyIsContinueOpera"));
	}
	else
	{
		_SquigglyIsContinueOpera = false;
		SetSquigglyIsContinueOpera(_SquigglyIsContinueOpera);
	}

	if (composite->HasKey("TextIsContinueOpera"))
	{
		_TextIsContinueOpera = safe_cast<bool>(composite->Lookup("TextIsContinueOpera"));
	}
	else
	{
		_TextIsContinueOpera = false;
		SetTextIsContinueOpera(_TextIsContinueOpera);
	}

	if (composite->HasKey("LineIsContinueOpera"))
	{
		_LineIsContinueOpera = safe_cast<bool>(composite->Lookup("LineIsContinueOpera"));
	}
	else
	{
		_LineIsContinueOpera = false;
		SetLineIsContinueOpera(_LineIsContinueOpera);
	}

	if (composite->HasKey("ArrowIsContinueOpera"))
	{
		_ArrowIsContinueOpera = safe_cast<bool>(composite->Lookup("ArrowIsContinueOpera"));
	}
	else
	{
		_ArrowIsContinueOpera = false;
		SetArrowIsContinueOpera(_ArrowIsContinueOpera);
	}

	if (composite->HasKey("SquareIsContinueOpera"))
	{
		_SquareIsContinueOpera = safe_cast<bool>(composite->Lookup("SquareIsContinueOpera"));
	}
	else
	{
		_SquareIsContinueOpera = false;
		SetSquareIsContinueOpera(_SquareIsContinueOpera);
	}

	if (composite->HasKey("CircleIsContinueOpera"))
	{
		_CircleIsContinueOpera = safe_cast<bool>(composite->Lookup("CircleIsContinueOpera"));
	}
	else
	{
		_CircleIsContinueOpera = false;
		SetCircleIsContinueOpera(_CircleIsContinueOpera);
	}

	if (composite->HasKey("StampIsContinueOpera"))
	{
		_StampIsContinueOpera = safe_cast<bool>(composite->Lookup("StampIsContinueOpera"));
	}
	else
	{
		_StampIsContinueOpera = false;
		SetStampeIsContinueOpera(_StampIsContinueOpera);
	}

	if (composite->HasKey("FreeTextIsContinueOpera"))
	{
		_FreeTextIsContinueOpera = safe_cast<bool>(composite->Lookup("FreeTextIsContinueOpera"));
	}
	else
	{
		_FreeTextIsContinueOpera = false;
		SetStampeIsContinueOpera(_FreeTextIsContinueOpera);
	}

	if (composite->HasKey("FreeTextAnnotColor"))
	{
		Platform::String^ strColor = safe_cast<Platform::String^>(composite->Lookup("FreeTextAnnotColor"));
		_FreeTextColor = FoxitTypeConvert::ConvertFromColorString(strColor);
	}
	else
	{
		_FreeTextColor = FoxitTypeConvert::ConvertFromColorString("#3366CC");
		SetFreeTextColor(_FreeTextColor);
	}

	if (composite->HasKey("FreeTextAnnotOpacity"))
	{
		_FreeTextOpacity = (double)composite->Lookup("FreeTextAnnotOpacity");
	}
	else
	{
		_FreeTextOpacity = 1;
		SetFreeTextAnnotOpacity(_FreeTextOpacity);
	}

	if (composite->HasKey("FreeTextAnnotFontSize"))
	{
		_FreeTextFontSize = (int)composite->Lookup("FreeTextAnnotFontSize");
	}
	else
	{
		_FreeTextFontSize = 12;
		SetFreeTextFontSize(_FreeTextFontSize);
	}

	if (composite->HasKey("FreeTextAnnotFontName"))
	{
		_FreeTextFontName = safe_cast<Platform::String^>(composite->Lookup("FreeTextAnnotFontName"));
	}
	else
	{
		_FreeTextFontName = "Times-Roman";
		SetFreeTextFontName(_FreeTextFontName);
	}

	if (composite->HasKey("IsFirstUserTip"))
	{
		_IsFirstUserTip = safe_cast<bool>(composite->Lookup("IsFirstUserTip"));
	}
	else
	{
		_IsFirstUserTip = true;
		SetIsFirstUserTip(_IsFirstUserTip);
	}


	if (composite->HasKey("InsertTextAnnotColor"))
	{
		Platform::String^ strColor = safe_cast<Platform::String^>(composite->Lookup("InsertTextAnnotColor"));
		_InsertTextColor = FoxitTypeConvert::ConvertFromColorString(strColor);
	}
	else
	{
		_InsertTextColor = FoxitTypeConvert::ConvertFromColorString("#FF3333");
		SetInsertTextColor(_InsertTextColor);
	}

	if (composite->HasKey("InsertTextAnnotOpacity"))
	{
		_InsertTextOpacity = (double)composite->Lookup("InsertTextAnnotOpacity");
	}
	else
	{
		_InsertTextOpacity = 1;
		SetInsertTextAnnotOpacity(_InsertTextOpacity);
	}

	if (composite->HasKey("CaretIsContinueOpera"))
	{
		_InsertTextIsContinueOpera = safe_cast<bool>(composite->Lookup("CaretIsContinueOpera"));
	}
	else
	{
		_InsertTextIsContinueOpera = false;
		SetInsertTextIsContinueOpera(_InsertTextIsContinueOpera);
	}


	if (composite->HasKey("ReplaceTextAnnotColor"))
	{
		Platform::String^ strColor = safe_cast<Platform::String^>(composite->Lookup("ReplaceTextAnnotColor"));
		_ReplaceTextColor = FoxitTypeConvert::ConvertFromColorString(strColor);
	}
	else
	{
		_ReplaceTextColor = FoxitTypeConvert::ConvertFromColorString("#FF3333");
		SetReplaceTextColor(_ReplaceTextColor);
	}

	if (composite->HasKey("ReplaceTextAnnotOpacity"))
	{
		_ReplaceTextOpacity = (double)composite->Lookup("ReplaceTextAnnotOpacity");
	}
	else
	{
		_ReplaceTextOpacity = 1;
		SetReplaceTextAnnotOpacity(_ReplaceTextOpacity);
	}


	if (composite->HasKey("ReplaceTextIsContinueOpera"))
	{
		_ReplaceTextIsContinueOpera = safe_cast<bool>(composite->Lookup("ReplaceTextIsContinueOpera"));
	}
	else
	{
		_ReplaceTextIsContinueOpera = false;
		SetReplaceTextIsContinueOpera(_ReplaceTextIsContinueOpera);
	}


	if (values->HasKey("AnnotProperty"))
	{
		values->Remove("AnnotProperty");
	}
	values->Insert("AnnotProperty", composite);
}

void AnnotSettingManager::SavePropertySetting()
{
	ApplicationDataContainer^ localSettings = ApplicationData::Current->LocalSettings;
	auto values = localSettings->Values;
	if (values->HasKey("AnnotProperty"))
	{
		values->Remove("AnnotProperty");
	}
	values->Insert("AnnotProperty", composite);
}

void AnnotSettingManager::SetAnnotOpacity(double value)
{
	if (composite->HasKey("AnnotOpacity"))
	{
		composite->Remove("AnnotOpacity");
	}
	composite->Insert("AnnotOpacity", dynamic_cast<PropertyValue^>(PropertyValue::CreateDouble(value)));
	_opacity = value;
}

void AnnotSettingManager::UpdateSaveCommandVisible()
{

}

void AnnotSettingManager::SetAnnotColor(Windows::UI::Color value)
{
	if (composite->HasKey("AnnotColor"))
	{
		composite->Remove("AnnotColor");;
	}
	composite->Insert("AnnotColor", FoxitTypeConvert::ConvertToColorString(value));
	_PencilColor = value;
}

double AnnotSettingManager::GetAnnotOpacity()
{
	if (composite->HasKey("AnnotOpacity"))
	{
		_pencilopacity = (double)composite->Lookup("AnnotOpacity");
	}
	return _pencilopacity;
}

Windows::UI::Color AnnotSettingManager::GetAnnotColor()
{
	Platform::String^ colorstr;
	if (composite->HasKey("AnnotColor"))
	{
		colorstr = safe_cast<Platform::String^>(composite->Lookup("AnnotColor"));
		_PencilColor = FoxitTypeConvert::ConvertFromColorString(colorstr);
	}
	return _PencilColor;
}

void AnnotSettingManager::SetAnnotDrawBorderWidth(int linewidth)
{
	if (composite->HasKey("AnnotDrawBorderWidth"))
	{
		composite->Remove("AnnotDrawBorderWidth");;
	}
	composite->Insert("AnnotDrawBorderWidth", dynamic_cast<PropertyValue^>(PropertyValue::CreateInt32(linewidth)));
	_linewidth = linewidth;
}

int AnnotSettingManager::GetAnnotDrawBorderWidth()
{
	if (composite->HasKey("AnnotDrawBorderWidth"))
	{
		_pencilopacity = (int)composite->Lookup("AnnotDrawBorderWidth");
	}
	return _linewidth;
}

void AnnotSettingManager::SetNoteLogoUrl(Platform::String^ url)
{
	if (composite->HasKey("NoteLogo"))
	{
		composite->Remove("NoteLogo");;
	}
	composite->Insert("NoteLogo", url);
	noteLogo = url;
}

Platform::String^ AnnotSettingManager::GetNoteLogoUrl()
{
	if (composite->HasKey("NoteLogo"))
	{
		noteLogo = safe_cast<Platform::String^>(composite->Lookup("NoteLogo"));
	}
	return noteLogo;
}

void AnnotSettingManager::SetNoteAuthor(Platform::String^ authorName)
{
	if (composite->HasKey("AuthorName"))
	{
		composite->Remove("AuthorName");;
	}
	composite->Insert("AuthorName", authorName);
	_authorName = authorName;
}

Platform::String^ AnnotSettingManager::GetNoteAuthor()
{
	if (composite->HasKey("AuthorName"))
	{
		_authorName = safe_cast<Platform::String^>(composite->Lookup("AuthorName"));
	}
	return _authorName;
}

void foxit::rdk::uiextensions::AnnotSettingManager::SetIsLinkHighlight(bool isHighlight)
{
	if (composite->HasKey("IsLinkHighlight"))
	{
		composite->Remove("IsLinkHighlight");;
	}
	composite->Insert("IsLinkHighlight", isHighlight);
	_isLinkHightlight = isHighlight;
}

bool foxit::rdk::uiextensions::AnnotSettingManager::GetIsLinkHighlight()
{
	if (composite->HasKey("IsLinkHighlight"))
	{
		_isLinkHightlight = safe_cast<bool>(composite->Lookup("IsLinkHighlight"));
	}
	return _isLinkHightlight;
}

void AnnotSettingManager::SetViewIsDarkMode(Platform::String^ isDarkMode)
{
	if (composite->HasKey("IsDarkMode"))
	{
		composite->Remove("IsDarkMode");;
	}
	composite->Insert("IsDarkMode", isDarkMode);
	_isDarkMode = isDarkMode;
}

Platform::String^ AnnotSettingManager::GetViewIsDarkMode()
{
	if (composite->HasKey("IsDarkMode"))
	{
		_isDarkMode = safe_cast<Platform::String^>(composite->Lookup("IsDarkMode"));
	}
	return _isDarkMode;
}

void AnnotSettingManager::SetIsFirstUserAnnot(bool isFirstUserAnnot)
{
	if (composite->HasKey("IsFirstUserAnnot"))
	{
		composite->Remove("IsFirstUserAnnot");;
	}
	composite->Insert("IsFirstUserAnnot", isFirstUserAnnot);
	_isFirstUserAnnot = isFirstUserAnnot;
}

bool AnnotSettingManager::GetIsFirstUserAnnot()
{
	if (composite->HasKey("IsFirstUserAnnot"))
	{
		_isFirstUserAnnot = safe_cast<bool>(composite->Lookup("IsFirstUserAnnot"));
	}
	return _isFirstUserAnnot;
}

Windows::UI::Color AnnotSettingManager::GetHightLightAnnotColor()
{
	Platform::String^ colorstr;
	if (composite->HasKey("HightLightAnnotColor"))
	{
		colorstr = safe_cast<Platform::String^>(composite->Lookup("HightLightAnnotColor"));
		_HighlightColor = FoxitTypeConvert::ConvertFromColorString(colorstr);
	}
	return _HighlightColor;
}

void AnnotSettingManager::SetHightLightAnnotColor(Windows::UI::Color value)
{
	if (composite->HasKey("HightLightAnnotColor"))
	{
		composite->Remove("HightLightAnnotColor");;
	}
	composite->Insert("HightLightAnnotColor", FoxitTypeConvert::ConvertToColorString(value));
	_HighlightColor = value;
}

double AnnotSettingManager::GetHightLightAnnotOpacity()
{
	if (composite->HasKey("HightLightAnnotOpacity"))
	{
		_HighlightOpacity = (double)composite->Lookup("HightLightAnnotOpacity");
	}
	return _HighlightOpacity;
}

void AnnotSettingManager::SetHightLightAnnotOpacity(double value)
{
	if (composite->HasKey("HightLightAnnotOpacity"))
	{
		composite->Remove("HightLightAnnotOpacity");
	}
	composite->Insert("HightLightAnnotOpacity", dynamic_cast<PropertyValue^>(PropertyValue::CreateDouble(value)));
	_HighlightOpacity = value;
}

Windows::UI::Color AnnotSettingManager::GetUnderLineAnnotColor()
{
	Platform::String^ colorstr;
	if (composite->HasKey("UnderLineAnnotColor"))
	{
		colorstr = safe_cast<Platform::String^>(composite->Lookup("UnderLineAnnotColor"));
		_UnderLineColor = FoxitTypeConvert::ConvertFromColorString(colorstr);
	}
	return _UnderLineColor;
}

void AnnotSettingManager::SetUnderLineAnnotColor(Windows::UI::Color value)
{
	if (composite->HasKey("UnderLineAnnotColor"))
	{
		composite->Remove("UnderLineAnnotColor");;
	}
	composite->Insert("UnderLineAnnotColor", FoxitTypeConvert::ConvertToColorString(value));
	_UnderLineColor = value;
}

double AnnotSettingManager::GetUnderLineAnnotOpacity()
{
	if (composite->HasKey("UnderLineAnnotOpacity"))
	{
		_UnderLineOpacity = (double)composite->Lookup("UnderLineAnnotOpacity");
	}
	return _UnderLineOpacity;
}

void AnnotSettingManager::SetUnderLineAnnotOpacity(double value)
{
	if (composite->HasKey("UnderLineAnnotOpacity"))
	{
		composite->Remove("UnderLineAnnotOpacity");
	}
	composite->Insert("UnderLineAnnotOpacity", dynamic_cast<PropertyValue^>(PropertyValue::CreateDouble(value)));
	_UnderLineOpacity = value;
}

Windows::UI::Color AnnotSettingManager::GetStrikeOutAnnotColor()
{
	Platform::String^ colorstr;
	if (composite->HasKey("StrikeOutAnnotColor"))
	{
		colorstr = safe_cast<Platform::String^>(composite->Lookup("StrikeOutAnnotColor"));
		_StrikeOutColor = FoxitTypeConvert::ConvertFromColorString(colorstr);
	}
	return _StrikeOutColor;
}

void AnnotSettingManager::SetStrikeOutAnnotColor(Windows::UI::Color value)
{
	if (composite->HasKey("StrikeOutAnnotColor"))
	{
		composite->Remove("StrikeOutAnnotColor");;
	}
	composite->Insert("StrikeOutAnnotColor", FoxitTypeConvert::ConvertToColorString(value));
	_StrikeOutColor = value;
}

double AnnotSettingManager::GetStrikeOutAnnotOpacity()
{
	if (composite->HasKey("StrikeOutAnnotOpacity"))
	{
		_StrikeOutOpacity = (double)composite->Lookup("StrikeOutAnnotOpacity");
	}
	return _StrikeOutOpacity;
}

void AnnotSettingManager::SetStrikeOutAnnotOpacity(double value)
{
	if (composite->HasKey("StrikeOutAnnotOpacity"))
	{
		composite->Remove("StrikeOutAnnotOpacity");
	}
	composite->Insert("StrikeOutAnnotOpacity", dynamic_cast<PropertyValue^>(PropertyValue::CreateDouble(value)));
	_StrikeOutOpacity = value;
}

Windows::UI::Color AnnotSettingManager::GetSquigglyAnnotColor()
{
	Platform::String^ colorstr;
	if (composite->HasKey("SquigglyAnnotColor"))
	{
		colorstr = safe_cast<Platform::String^>(composite->Lookup("SquigglyAnnotColor"));
		_SquigglyColor = FoxitTypeConvert::ConvertFromColorString(colorstr);
	}
	return _SquigglyColor;
}

Windows::UI::Color AnnotSettingManager::GetLineAnnotColor()
{
	Platform::String^ colorstr;
	if (composite->HasKey("LineAnnotColor"))
	{
		colorstr = safe_cast<Platform::String^>(composite->Lookup("LineAnnotColor"));
		_LineColor = FoxitTypeConvert::ConvertFromColorString(colorstr);
	}
	return _LineColor;
}

void AnnotSettingManager::SetLineAnnotColor(Windows::UI::Color value)
{
	if (composite->HasKey("LineAnnotColor"))
	{
		composite->Remove("LineAnnotColor");;
	}
	composite->Insert("LineAnnotColor", FoxitTypeConvert::ConvertToColorString(value));
	_LineColor = value;
}

void AnnotSettingManager::SetSquigglyAnnotColor(Windows::UI::Color value)
{
	if (composite->HasKey("SquigglyAnnotColor"))
	{
		composite->Remove("SquigglyAnnotColor");;
	}
	composite->Insert("SquigglyAnnotColor", FoxitTypeConvert::ConvertToColorString(value));
	_SquigglyColor = value;
}

double AnnotSettingManager::GetSquigglyAnnotOpacity()
{
	if (composite->HasKey("SquigglyAnnotOpacity"))
	{
		_SquigglyOpacity = (double)composite->Lookup("SquigglyAnnotOpacity");
	}
	return _SquigglyOpacity;
}

void AnnotSettingManager::SetSquigglyAnnotOpacity(double value)
{
	if (composite->HasKey("SquigglyAnnotOpacity"))
	{
		composite->Remove("SquigglyAnnotOpacity");
	}
	composite->Insert("SquigglyAnnotOpacity", dynamic_cast<PropertyValue^>(PropertyValue::CreateDouble(value)));
	_SquigglyOpacity = value;
}

int AnnotSettingManager::GetEraserWidth()
{
	if (composite->HasKey("EraserWidth"))
	{
		_EraserWidth = (int)composite->Lookup("EraserWidth");
	}
	return _EraserWidth;
}

void AnnotSettingManager::SetEraserWidth(int value)
{
	if (composite->HasKey("EraserWidth"))
	{
		composite->Remove("EraserWidth");
	}
	composite->Insert("EraserWidth", dynamic_cast<PropertyValue^>(PropertyValue::CreateInt32(value)));
	_EraserWidth = value;
}

int AnnotSettingManager::GetStampIndex()
{
	if (composite->HasKey("StampIndex"))
	{
		_StamIndex = (int)composite->Lookup("StampIndex");
	}
	return _StamIndex;
}

void AnnotSettingManager::SetStampIndex(int value)
{
	if (composite->HasKey("StampIndex"))
	{
		composite->Remove("StampIndex");
	}
	composite->Insert("StampIndex", dynamic_cast<PropertyValue^>(PropertyValue::CreateInt32(value)));
	_StamIndex = value;
}

bool AnnotSettingManager::GetHightLightIsContinueOpera()
{
	if (composite->HasKey("HightLightIsContinueOpera"))
	{
		_HightLightIsContinueOpera = safe_cast<bool>(composite->Lookup("HightLightIsContinueOpera"));
	}
	return _HightLightIsContinueOpera;
}

void AnnotSettingManager::SetHightLightIsContinueOpera(bool value)
{
	if (composite->HasKey("HightLightIsContinueOpera"))
	{
		composite->Remove("HightLightIsContinueOpera");
	}
	composite->Insert("HightLightIsContinueOpera", value);
	_HightLightIsContinueOpera = value;
}

bool AnnotSettingManager::GetAreaHightLightIsContinueOpera()
{
	if (composite->HasKey("AreaHightLightIsContinueOpera"))
	{
		_AreaHightLightIsContinueOpera = safe_cast<bool>(composite->Lookup("AreaHightLightIsContinueOpera"));
	}
	return _AreaHightLightIsContinueOpera;
}

void AnnotSettingManager::SetAreaHightLightIsContinueOpera(bool value)
{
	if (composite->HasKey("AreaHightLightIsContinueOpera"))
	{
		composite->Remove("AreaHightLightIsContinueOpera");
	}
	composite->Insert("AreaHightLightIsContinueOpera", value);
	_AreaHightLightIsContinueOpera = value;
}

bool AnnotSettingManager::GetStrikeOutIsContinueOpera()
{
	if (composite->HasKey("StrikeOutIsContinueOpera"))
	{
		_StrikeOutIsContinueOpera = safe_cast<bool>(composite->Lookup("StrikeOutIsContinueOpera"));
	}
	return _StrikeOutIsContinueOpera;
}

void AnnotSettingManager::SetStrikeOutIsContinueOpera(bool value)
{
	if (composite->HasKey("StrikeOutIsContinueOpera"))
	{
		composite->Remove("StrikeOutIsContinueOpera");
	}
	composite->Insert("StrikeOutIsContinueOpera", value);
	_StrikeOutIsContinueOpera = value;
}

bool AnnotSettingManager::GetUnderLineIsContinueOpera()
{
	if (composite->HasKey("UnderLineIsContinueOpera"))
	{
		_UnderLineIsContinueOpera = safe_cast<bool>(composite->Lookup("UnderLineIsContinueOpera"));
	}
	return _UnderLineIsContinueOpera;
}

void AnnotSettingManager::SetUnderLineIsContinueOpera(bool value)
{
	if (composite->HasKey("UnderLineIsContinueOpera"))
	{
		composite->Remove("UnderLineIsContinueOpera");
	}
	composite->Insert("UnderLineIsContinueOpera", value);
	_UnderLineIsContinueOpera = value;
}

bool AnnotSettingManager::GetSquigglyIsContinueOpera()
{
	if (composite->HasKey("SquigglyIsContinueOpera"))
	{
		_SquigglyIsContinueOpera = safe_cast<bool>(composite->Lookup("SquigglyIsContinueOpera"));
	}
	return _SquigglyIsContinueOpera;
}

void AnnotSettingManager::SetSquigglyIsContinueOpera(bool value)
{
	if (composite->HasKey("SquigglyIsContinueOpera"))
	{
		composite->Remove("SquigglyIsContinueOpera");
	}
	composite->Insert("SquigglyIsContinueOpera", value);
	_SquigglyIsContinueOpera = value;
}

bool AnnotSettingManager::GetTextIsContinueOpera()
{
	if (composite->HasKey("TextIsContinueOpera"))
	{
		_TextIsContinueOpera = safe_cast<bool>(composite->Lookup("TextIsContinueOpera"));
	}
	return _TextIsContinueOpera;
}

void AnnotSettingManager::SetTextIsContinueOpera(bool value)
{
	if (composite->HasKey("TextIsContinueOpera"))
	{
		composite->Remove("TextIsContinueOpera");
	}
	composite->Insert("TextIsContinueOpera", value);
	_TextIsContinueOpera = value;
}

bool AnnotSettingManager::GetLineIsContinueOpera()
{
	if (composite->HasKey("LineIsContinueOpera"))
	{
		_LineIsContinueOpera = safe_cast<bool>(composite->Lookup("LineIsContinueOpera"));
	}
	return _LineIsContinueOpera;
}

void AnnotSettingManager::SetLineIsContinueOpera(bool value)
{
	if (composite->HasKey("LineIsContinueOpera"))
	{
		composite->Remove("LineIsContinueOpera");
	}
	composite->Insert("LineIsContinueOpera", value);
	_LineIsContinueOpera = value;
}

bool AnnotSettingManager::GetArrowIsContinueOpera()
{
	if (composite->HasKey("ArrowIsContinueOpera"))
	{
		_ArrowIsContinueOpera = safe_cast<bool>(composite->Lookup("ArrowIsContinueOpera"));
	}
	return _ArrowIsContinueOpera;
}

void AnnotSettingManager::SetArrowIsContinueOpera(bool value)
{
	if (composite->HasKey("ArrowIsContinueOpera"))
	{
		composite->Remove("ArrowIsContinueOpera");
	}
	composite->Insert("ArrowIsContinueOpera", value);
	_ArrowIsContinueOpera = value;
}

bool AnnotSettingManager::GetSquareIsContinueOpera()
{
	if (composite->HasKey("SquareIsContinueOpera"))
	{
		_SquareIsContinueOpera = safe_cast<bool>(composite->Lookup("SquareIsContinueOpera"));
	}
	return _SquareIsContinueOpera;
}

void AnnotSettingManager::SetSquareIsContinueOpera(bool value)
{
	if (composite->HasKey("SquareIsContinueOpera"))
	{
		composite->Remove("SquareIsContinueOpera");
	}
	composite->Insert("SquareIsContinueOpera", value);
	_SquareIsContinueOpera = value;
}

bool AnnotSettingManager::GetCircleIsContinueOpera()
{
	if (composite->HasKey("CircleIsContinueOpera"))
	{
		_CircleIsContinueOpera = safe_cast<bool>(composite->Lookup("CircleIsContinueOpera"));
	}
	return _CircleIsContinueOpera;
}

void AnnotSettingManager::SetCircleIsContinueOpera(bool value)
{
	if (composite->HasKey("CircleIsContinueOpera"))
	{
		composite->Remove("CircleIsContinueOpera");
	}
	composite->Insert("CircleIsContinueOpera", value);
	_CircleIsContinueOpera = value;
}

bool AnnotSettingManager::GetStampIsContinueOpera()
{
	if (composite->HasKey("StampIsContinueOpera"))
	{
		_StampIsContinueOpera = safe_cast<bool>(composite->Lookup("StampIsContinueOpera"));
	}
	return _StampIsContinueOpera;
}

void AnnotSettingManager::SetStampeIsContinueOpera(bool value)
{
	if (composite->HasKey("StampIsContinueOpera"))
	{
		composite->Remove("StampIsContinueOpera");
	}
	composite->Insert("StampIsContinueOpera", value);
	_StampIsContinueOpera = value;
}

bool AnnotSettingManager::GetFreeTextIsContinueOpera()
{
	if (composite->HasKey("FreeTextIsContinueOpera"))
	{
		_FreeTextIsContinueOpera = safe_cast<bool>(composite->Lookup("FreeTextIsContinueOpera"));
	}
	return _FreeTextIsContinueOpera;
}

void AnnotSettingManager::SetFreeTextIsContinueOpera(bool value)
{
	if (composite->HasKey("FreeTextIsContinueOpera"))
	{
		composite->Remove("FreeTextIsContinueOpera");
	}
	composite->Insert("FreeTextIsContinueOpera", value);
	_FreeTextIsContinueOpera = value;
}

Windows::UI::Color AnnotSettingManager::GetFreeTextColor()
{
	Platform::String^ colorstr;
	if (composite->HasKey("FreeTextAnnotColor"))
	{
		colorstr = safe_cast<Platform::String^>(composite->Lookup("FreeTextAnnotColor"));
		_FreeTextColor = FoxitTypeConvert::ConvertFromColorString(colorstr);
	}
	return _FreeTextColor;
}

void AnnotSettingManager::SetFreeTextColor(Windows::UI::Color value)
{
	if (composite->HasKey("FreeTextAnnotColor"))
	{
		composite->Remove("FreeTextAnnotColor");;
	}
	composite->Insert("FreeTextAnnotColor", FoxitTypeConvert::ConvertToColorString(value));
	_FreeTextColor = value;
}

double AnnotSettingManager::GetFreeTextAnnotOpacity()
{
	if (composite->HasKey("FreeTextAnnotOpacity"))
	{
		_FreeTextOpacity = (double)composite->Lookup("FreeTextAnnotOpacity");
	}
	return _FreeTextOpacity;
}

void AnnotSettingManager::SetFreeTextAnnotOpacity(double value)
{
	if (composite->HasKey("FreeTextAnnotOpacity"))
	{
		composite->Remove("FreeTextAnnotOpacity");
	}
	composite->Insert("FreeTextAnnotOpacity", dynamic_cast<PropertyValue^>(PropertyValue::CreateDouble(value)));
	_FreeTextOpacity = value;
}

int AnnotSettingManager::GetFreeTextFontSize()
{
	if (composite->HasKey("FreeTextAnnotFontSize"))
	{
		_FreeTextFontSize = (int)composite->Lookup("FreeTextAnnotFontSize");
	}
	return _FreeTextFontSize;
}

void AnnotSettingManager::SetFreeTextFontSize(int value)
{
	if (composite->HasKey("FreeTextAnnotFontSize"))
	{
		composite->Remove("FreeTextAnnotFontSize");
	}
	composite->Insert("FreeTextAnnotFontSize", dynamic_cast<PropertyValue^>(PropertyValue::CreateInt32(value)));
	_FreeTextFontSize = value;
}

Platform::String^ AnnotSettingManager::GetFreeTextFontName()
{
	if (composite->HasKey("FreeTextAnnotFontName"))
	{
		_FreeTextFontName = safe_cast<Platform::String^>(composite->Lookup("FreeTextAnnotFontName"));
	}
	return _FreeTextFontName;
}

void AnnotSettingManager::SetFreeTextFontName(Platform::String^ value)
{
	if (composite->HasKey("FreeTextAnnotFontName"))
	{
		composite->Remove("FreeTextAnnotFontName");
	}
	composite->Insert("FreeTextAnnotFontName", value);
	_FreeTextFontName = value;
}

void AnnotSettingManager::SetIsFirstUserTip(bool isFirstUserTip)
{
	if (composite->HasKey("IsFirstUserTip"))
	{
		composite->Remove("IsFirstUserTip");;
	}
	composite->Insert("IsFirstUserTip", isFirstUserTip);
	_IsFirstUserTip = isFirstUserTip;
}

bool AnnotSettingManager::GetIsFirstUserTip()
{
	if (composite->HasKey("IsFirstUserTip"))
	{
		_IsFirstUserTip = safe_cast<bool>(composite->Lookup("IsFirstUserTip"));
	}
	return _IsFirstUserTip;
}

bool AnnotSettingManager::GetInsertTextIsContinueOpera()
{
	if (composite->HasKey("CaretIsContinueOpera"))
	{
		_InsertTextIsContinueOpera = safe_cast<bool>(composite->Lookup("CaretIsContinueOpera"));
	}
	return _InsertTextIsContinueOpera;
}

void AnnotSettingManager::SetInsertTextIsContinueOpera(bool value)
{
	if (composite->HasKey("CaretIsContinueOpera"))
	{
		composite->Remove("CaretIsContinueOpera");
	}
	composite->Insert("CaretIsContinueOpera", value);
	_InsertTextIsContinueOpera = value;
}

Windows::UI::Color AnnotSettingManager::GetInsertTextColor()
{
	Platform::String^ colorstr;
	if (composite->HasKey("InsertTextAnnotColor"))
	{
		colorstr = safe_cast<Platform::String^>(composite->Lookup("InsertTextAnnotColor"));
		_InsertTextColor = FoxitTypeConvert::ConvertFromColorString(colorstr);
	}
	return _InsertTextColor;
}

void AnnotSettingManager::SetInsertTextColor(Windows::UI::Color value)
{
	if (composite->HasKey("InsertTextAnnotColor"))
	{
		composite->Remove("InsertTextAnnotColor");;
	}
	composite->Insert("InsertTextAnnotColor", FoxitTypeConvert::ConvertToColorString(value));
	_InsertTextColor = value;
}

double AnnotSettingManager::GetInsertTextAnnotOpacity()
{
	if (composite->HasKey("InsertTextAnnotOpacity"))
	{
		_InsertTextOpacity = (double)composite->Lookup("InsertTextAnnotOpacity");
	}
	return _InsertTextOpacity;
}

void AnnotSettingManager::SetInsertTextAnnotOpacity(double value)
{
	if (composite->HasKey("InsertTextAnnotOpacity"))
	{
		composite->Remove("InsertTextAnnotOpacity");
	}
	composite->Insert("InsertTextAnnotOpacity", dynamic_cast<PropertyValue^>(PropertyValue::CreateDouble(value)));
	_InsertTextOpacity = value;
}

bool AnnotSettingManager::GetReplaceTextIsContinueOpera()
{
	if (composite->HasKey("ReplaceTextIsContinueOpera"))
	{
		_ReplaceTextIsContinueOpera = safe_cast<bool>(composite->Lookup("ReplaceTextIsContinueOpera"));
	}
	return _ReplaceTextIsContinueOpera;
}

void AnnotSettingManager::SetReplaceTextIsContinueOpera(bool value)
{
	if (composite->HasKey("ReplaceTextIsContinueOpera"))
	{
		composite->Remove("ReplaceTextIsContinueOpera");
	}
	composite->Insert("ReplaceTextIsContinueOpera", value);
	_ReplaceTextIsContinueOpera = value;
}

Windows::UI::Color AnnotSettingManager::GetReplaceTextColor()
{
	Platform::String^ colorstr;
	if (composite->HasKey("ReplaceTextAnnotColor"))
	{
		colorstr = safe_cast<Platform::String^>(composite->Lookup("ReplaceTextAnnotColor"));
		_ReplaceTextColor = FoxitTypeConvert::ConvertFromColorString(colorstr);
	}
	return _ReplaceTextColor;
}

void AnnotSettingManager::SetReplaceTextColor(Windows::UI::Color value)
{
	if (composite->HasKey("ReplaceTextAnnotColor"))
	{
		composite->Remove("ReplaceTextAnnotColor");;
	}
	composite->Insert("ReplaceTextAnnotColor", FoxitTypeConvert::ConvertToColorString(value));
	_ReplaceTextColor = value;
}

double AnnotSettingManager::GetReplaceTextAnnotOpacity()
{
	if (composite->HasKey("ReplaceTextAnnotOpacity"))
	{
		_ReplaceTextOpacity = (double)composite->Lookup("ReplaceTextAnnotOpacity");
	}
	return _ReplaceTextOpacity;
}

void AnnotSettingManager::SetReplaceTextAnnotOpacity(double value)
{
	if (composite->HasKey("ReplaceTextAnnotOpacity"))
	{
		composite->Remove("ReplaceTextAnnotOpacity");
	}
	composite->Insert("ReplaceTextAnnotOpacity", dynamic_cast<PropertyValue^>(PropertyValue::CreateDouble(value)));
	_ReplaceTextOpacity = value;
}