/**
 * Copyright (C) 2003-2018, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit Mobile PDF SDK to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */


#pragma once


namespace foxit
{
	namespace rdk
	{
		namespace uiextensions
		{
/** @brief	Text annotation, markup type. */
#define FSPDF_ANNOTTYPE_TEXT				"Text"
/** @brief	Link annotation. */
#define FSPDF_ANNOTTYPE_LINK				"Link"
/** @brief	Free text annotation, markup type. */
#define FSPDF_ANNOTTYPE_FREETEXT			"FreeText"
/** @brief	Line annotation, markup type. */
#define FSPDF_ANNOTTYPE_LINE				"Line"
/** @brief	Square annotation, markup type. */
#define FSPDF_ANNOTTYPE_SQUARE				"Square"
/** @brief	Circle annotation, markup type. */
#define FSPDF_ANNOTTYPE_CIRCLE				"Circle"
/** @brief	Polygon annotation, markup type. */
#define FSPDF_ANNOTTYPE_POLYGON				"Polygon"
/** @brief	Polyline annotation, markup type. */
#define FSPDF_ANNOTTYPE_POLYLINE			"PolyLine"
/** @brief	Highlight annotation, markup type. */
#define FSPDF_ANNOTTYPE_HIGHLIGHT			"Highlight"
/** @brief	Underline annotation, markup type. */
#define FSPDF_ANNOTTYPE_UNDERLINE			"Underline"
/** @brief	Squiggly-underline annotation, markup type. */
#define FSPDF_ANNOTTYPE_SQUIGGLY			"Squiggly"
/** @brief	Strikeout annotation, markup type. */
#define FSPDF_ANNOTTYPE_STRIKEOUT			"StrikeOut"
/** @brief	Rubber stamp annotation, markup type. */
#define FSPDF_ANNOTTYPE_STAMP				"Stamp"
/** @brief	Caret annotation, markup type. */
#define FSPDF_ANNOTTYPE_CARET				"Caret"
/** @brief	Ink annotation, markup type. */
#define FSPDF_ANNOTTYPE_INK					"Ink"
/** @brief	Pop-up annotation. */
#define FSPDF_ANNOTTYPE_POPUP				"Popup"
/** @brief	File attachment annotation, markup type. */
#define FSPDF_ANNOTTYPE_FILEATTACHMENT		"FileAttachment"
/**
* @brief	Sound annotation, markup type.
* @note	Currently, Foxit PDF SDK only provides functions for common access to sound annotation.
*/
#define FSPDF_ANNOTTYPE_SOUND				"Sound"
/**
* @brief	Movie annotation.
* @note	Currently, Foxit PDF SDK only provides functions for common access to movie annotation.
*/
#define FSPDF_ANNOTTYPE_MOVIE				"Movie"
/**
* @brief	Widget annotation.
* @note	In current module, Foxit PDF SDK only provides functions for common access to widget annotation.<br>
*			For widget related to form object, please refer to \ref FPDFFORM "PDF Form" module.<br>
*/
#define FSPDF_ANNOTTYPE_WIDGET				"Widget"
/**
* @brief	Screen annotation.
* @note	Currently, Foxit PDF SDK only provides functions for common access to screen annotation.
*/
#define FSPDF_ANNOTTYPE_SCREEN				"Screen"
/**
* @brief	Printer's mark annotation.
* @note	Currently, Foxit PDF SDK only provides functions for common access to printer's mark annotation.
*/
#define FSPDF_ANNOTTYPE_PRINTERMARK			"PrinterMark"
/**
* @brief	Trap network annotation.
* @note	Currently, Foxit PDF SDK only provides functions for common access to trap network annotation.
*/
#define FSPDF_ANNOTTYPE_TRAPNET				"TrapNet"
/**
* @brief	Watermark annotation.
* @note	In current module, Foxit PDF SDK only provides functions for common access to watermark annotation.<br>
*			For watermark annotation, please refer to \ref FPDFWATERMARK "PDF Watermark" module.
*/
#define FSPDF_ANNOTTYPE_WATERMARK			"Watermark"
/**
* @brief	Pressure Sensitive Ink annotation. Currently not support.
* @note	In current module, Foxit PDF SDK only provides functions for common access to pressure sensitive ink annotation.<br>
*			For pressure sensitive ink annotation, please refer to \ref FPDFWATERMARK "PDF Watermark" module.
*/
#define FSPDF_ANNOTTYPE_PSI					"PSInk"
/**
* @brief	3D annotation.
* @note	Currently, Foxit PDF SDK only provides functions for common access to 3D annotation.
*/
#define FSPDF_ANNOTTYPE_3D					"3D"


#ifndef _FSPDF_DEF_MACRO_ANNOT_INTENTNAME_
#define _FSPDF_DEF_MACRO_ANNOT_INTENTNAME_
/**
* @name	Macro Definitions for Intent Names
*/
/**@{*/

/** @brief	FreeText annotation intent name: FreeTextCallout. */
#define FSPDF_ANNOT_INTENTNAME_FREETEXT_CALLOUT			"FreeTextCallout"
/** @brief	FreeText annotation intent name: FreeTextTypewriter. */
#define FSPDF_ANNOT_INTENTNAME_FREETEXT_TYPEWRITER		"FreeTextTypewriter"

/** @brief	Line annotation intent name: LineArrow. */
#define FSPDF_ANNOT_INTENTNAME_LINE_ARROW				"LineArrow"
/** @brief	Line annotation intent name: LineDimension. */
#define FSPDF_ANNOT_INTENTNAME_LINE_DIMENSION			"LineDimension"

/** @brief	Polygon annotation intent name: PolygonCloud. */
#define FSPDF_ANNOT_INTENTNAME_POLYGON_CLOUD			"PolygonCloud"
/** @brief	Polygon annotation intent name: PolygonDimension. */
#define FSPDF_ANNOT_INTENTNAME_POLYGON_DIMENSION		"PolygonDimension"

/** @brief	PolyLine annotation intent name: PolyLineDimension. */
#define FSPDF_ANNOT_INTENTNAME_POLYLINE_DIMENSION		"PolyLineDimension"

/**@}*/
#endif /* _FSPDF_DEF_MACRO_ANNOT_INTENTNAME_ */

			public enum class AnnotationType
			{
				None,
				NonSurport,
				HightLight,
				Link,
				UnderLine,
				StrikeOut,
				Squiggly,
				Text,
				FreeText,
				Line,
				Circle,
				Square,
				Ink,
				PolyLine,
				Polygon,
				Stamp,
				Caret,
				Popup,
				FileAttachment,
				Sound,
				Movie,
				Widget,
				Screen,
				PrinterMark,
				TrapNet,
				Watermark,
				PSInk,
				ThreeD,
				ReplaceText
			};

			public enum class AnnotationFlag
			{
				INVISIBLE = 0x0001,
				HIDDEN = 0x0002,
				PRINT = 0x0004,
				NOZOOM = 0x0008,
				NOROTATE = 0x0010,
				NOVIEW = 0x0020,
				READONLY = 0x0040,
				LOCKED = 0x0080,
				TOGGLENOVIEW = 0x0100,
				LOCKEDCONTENTS = 0x0200,
			};

			public enum class AnnotIconNameTextType
			{
				Checkmark,
				Circle,
				Comment,
				Cross,
				Help,
				Insert,
				Key,
				NewParagraph,
				Note,
				Paragraph,
				RightArrow,
				RightPointer,
				Star,
				UpArrow,
				UpleftArrow,
			};

			public enum class AnnotIconNameFileattachType
			{
				Graph,
				PaperClip,
				PushPin,
				Tag,
			};

			public enum class AnnotIconNameStampType
			{
				Approved,
				AsIs,
				Confidential,
				Departmental,
				Draft,
				Experimental,
				Expired,
				Final,
				ForComment,
				ForPublicRelease,
				NotApproved,
				NotForPublicRelease,
				Sold,
				TopSecret,
			};

			public enum class AnnotIconNameSoundType
			{
				Speaker,
				Mic,
			};

			public enum class AnnotBorderStyle
			{
				/** @brief	Border style: Solid */
				SOLID = 0,
				/** @brief	Border style: Dashed */
				DASHED = 1,
				/** @brief	Border style: Underline */
				UNDERLINE = 2,
				/** @brief	Border style: Beveled */
				BEVELED = 3,
				/** @brief	Border style: Inset */
				INSET = 4,
				/** @brief	Border style: Cloudy */
				CLOUDY = 5,
			};

			public ref class AnnotBorderWrap sealed
			{
			public:
				/**
				* @brief	Width of border, in points.
				*
				* @details	If this value is 0, no border is drawn. This value should not be negative.
				*/
				property float BorderWidth
				{
					float get()
					{
						return _BorderWidth;
					}
					void set(float value)
					{
						_BorderWidth = value;
					}
				}
				/**
				* @brief	Border style.
				*
				* @details	Please refer to macro definitions <b>FSPDF_ANNOT_BORDERSTYLE_XXX</b> and this should be one of these macros.
				*/
				property AnnotBorderStyle BorderStyle
				{
					AnnotBorderStyle get()
					{
						return _BorderStyle;
					}
					void set(AnnotBorderStyle value)
					{
						_BorderStyle = value;
					}
				}
				/**
				* @brief	Intensity of the cloudy effect.
				*
				* @details	Only free text, square, circle, polygon, and polyline annotations can have cloud intensity property.<br>
				*			Suggested values range from 0 to 2. -1 mean no cloudy border effect.<br>
				*			Only useful when <i>borderStyle</i> is FSPDF_ANNOT_BORDERSTYLE_CLOUDY; otherwise -1.
				*/
				property float CloudIntensity
				{
					float get()
					{
						return _CloudIntensity;
					}
					void set(float value)
					{
						_CloudIntensity = value;
					}
				}
				/**
				* @brief	Dash phase.
				*
				* @details	Only useful when <i>borderStyle</i> is FSPDF_ANNOT_BORDERSTYLE_DASHED.
				*/
				property float DashPhase
				{
					float get()
					{
						return _DashPhase;
					}
					void set(float value)
					{
						_DashPhase = value;
					}
				}
				/**
				* @brief	Count of useful elements in array <i>dashPattern</i>.
				*
				* @details	Only useful when <i>borderStyle</i> is FSPDF_ANNOT_BORDERSTYLE_DASHED.
				*/
				property int DashCount
				{
					int get()
					{
						return _DashCount;
					}
					void set(int value)
					{
						_DashCount = value;
					}
				}
				/**
				* @brief	Dash array.
				*
				* @details	The value of elements in this array should not be negative.<br>
				*			Only be useful when <i>borderStyle</i> is FSPDF_ANNOT_BORDERSTYLE_DASHED.
				*/
				property Platform::Array<float>^ DashPattern
				{
					Platform::Array<float>^ get()
					{
						return _DashPattern;
					}
				}

			public:
				AnnotBorderWrap()
				{
					_BorderWidth = 0.0;
					_BorderStyle = AnnotBorderStyle::SOLID;
					_CloudIntensity = -1;
					_DashPhase = 0.0;
					_DashCount = 0;
					_DashPattern = ref new Platform::Array<float>(16);
				}

			private:
				float _BorderWidth;
				AnnotBorderStyle _BorderStyle;
				float _CloudIntensity;
				float _DashPhase;
				int _DashCount;
				Platform::Array<float>^ _DashPattern;
			};

			public ref class TypeWriterRange sealed
			{
			internal:
				TypeWriterRange(Platform::String^ content, float actualheight, float actualwidth) : _ContentStr(content), _ActualHeight(actualheight), _ActualWidth(actualwidth)
				{

				}

			public:
				property Platform::String^ ContentStr
				{
					Platform::String^ get()
					{
						return _ContentStr;
					}
				}
				property float ActualHeight
				{
					float get()
					{
						return _ActualHeight;
					}
				}
				property float ActualWidth
				{
					float get()
					{
						return _ActualWidth;
					}
				}

			private:
				Platform::String^ _ContentStr;
				float _ActualHeight;
				float _ActualWidth;
			};
		}
	}
}

