/**
 * Copyright (C) 2003-2018, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit Mobile PDF SDK to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */


#include "pch.h"
#include "Annot_Caret.h"


using namespace foxit::rdk::uiextensions;

using namespace Platform;
using namespace std;
using namespace Windows::Foundation;
using namespace Windows::Foundation::Collections;
using namespace Windows::UI::Xaml;
using namespace Windows::UI::Xaml::Controls;
using namespace Windows::UI::Xaml::Controls::Primitives;
using namespace Windows::UI::Xaml::Data;
using namespace Windows::UI::Xaml::Input;
using namespace Windows::UI::Xaml::Media;
using namespace Windows::UI::Xaml::Navigation;
using namespace Windows::UI::Popups;
using namespace Windows::UI::Xaml::Shapes;

CaretAnnot::CaretAnnot()
{
	_AnnotType = AnnotationType::Caret;
	_IsHasGroup = false;
	_GroupAnnotMinRect = Windows::Foundation::Rect::Empty;
}

Windows::Foundation::Rect CaretAnnot::Page_HightLightRect::get()
{
	return _Page_HightLightRect;
}

void CaretAnnot::Page_HightLightRect::set(Windows::Foundation::Rect value)
{
	if (_Page_HightLightRect != value)
	{
		_Page_HightLightRect = value;
		if (_PdfAnnot && _PdfAnnot->PDF_Annot)
		{
			foxit::common::fxcrt::RectF^ rect = FoxitRectConverter::ConvertToFoxitRectF(_Page_HightLightRect);
			_PdfAnnot->PDF_Annot->Move(rect);
			_PdfAnnot->PDF_Annot->ResetAppearanceStream();
			NotifyDocumentChanged();
		}
	}
}

Windows::Foundation::Rect CaretAnnot::Device_HightLightRect::get()
{
	return _Device_HightLightRect;
}

void CaretAnnot::Device_HightLightRect::set(Windows::Foundation::Rect value)
{
	if (_Device_HightLightRect != value)
	{
		_Device_HightLightRect = value;
	}
}

Windows::Foundation::Rect CaretAnnot::GroupAnnotMinRect::get()
{
	return _GroupAnnotMinRect;
}

void CaretAnnot::GroupAnnotMinRect::set(Windows::Foundation::Rect value)
{
	if (_GroupAnnotMinRect != value)
	{
		_GroupAnnotMinRect = value;
	}
}

bool CaretAnnot::IsHasGroup::get()
{
	return _IsHasGroup;
}

void CaretAnnot::IsHasGroup::set(bool value)
{
	if (_IsHasGroup != value)
	{
		_IsHasGroup = value;
	}
}

int CaretAnnot::GroupAnnotIndex::get()
{
	return _GroupAnnotIndex;
}

void CaretAnnot::GroupAnnotIndex::set(int value)
{
	if (_GroupAnnotIndex != value)
	{
		_GroupAnnotIndex = value;
	}
}

int CaretAnnot::TextRotate::get()
{
	return _TextRotate;
}

void CaretAnnot::TextRotate::set(int value)
{
	if (_TextRotate != value)
	{
		_TextRotate = value;
	}
}

double CaretAnnot::InnerMargin::get()
{
	return _InnerMargin;
}

void CaretAnnot::InnerMargin::set(double value)
{
	if (_InnerMargin != value)
	{
		_InnerMargin = value;
	}
}

Platform::String^ CaretAnnot::Intent::get()
{
	if (_Intent == "")
	{
		auto annot = dynamic_cast<foxit::pdf::annots::IMarkup^>(FXDocUtil::DynamicCastBaseAnnot(_PdfAnnot->PDF_Annot));
		auto pIntent = annot->GetIntent();

		_Intent = pIntent;
	}
	return _Intent;
}

void CaretAnnot::Intent::set(Platform::String^ value)
{
	if (_Intent != value)
	{
		auto annot = dynamic_cast<foxit::pdf::annots::IMarkup^>(FXDocUtil::DynamicCastBaseAnnot(_PdfAnnot->PDF_Annot));
		annot->SetIntent(value);

		_Intent = value;
	}
}

bool CaretAnnot::IsHitTest(Windows::Foundation::Point pagePoint)
{
	return false;
}

bool CaretAnnot::IsHitTestRange(Windows::Foundation::Point pagePoint)
{
	return _Page_HightLightRect.Contains(pagePoint);
	return false;
}

bool CaretAnnot::OnLoadAnnot(IAnnotManager^ annotmgr, Platform::Object^ pageObj, PDFAnnotHandle^ annot)
{
	if (!BaseAnnot::OnLoadAnnot(annotmgr, pageObj, annot))
		return false;

	_Page_HightLightRect = _MinRect;

	FS_FLOAT left;
	FS_FLOAT top;
	FS_FLOAT right;
	FS_FLOAT bottom;
	
	foxit::pdf::annots::Caret^ caret = dynamic_cast<foxit::pdf::annots::Caret^>(FXDocUtil::DynamicCastBaseAnnot(_PdfAnnot->PDF_Annot));
	foxit::common::fxcrt::RectF^ innerRect = caret->GetInnerRect();
	foxit::common::fxcrt::RectF^ annotRect = caret->GetRect();
	left = innerRect->left - annotRect->left;
	bottom = innerRect->bottom - annotRect->bottom;
	right = annotRect->right - innerRect->right ;
	top = annotRect->top - innerRect->top ;

	_InnerMargin = left;

	FS_BOOL isGrouped = caret->IsGrouped();
	if (isGrouped)
	{	
		_IsHasGroup = true;
		auto pGroupElements = caret->GetGroupElements();
		unsigned int nCount = pGroupElements->GetSize();
		String^ typeStr;

		if (nCount > 0)
		{
			for (unsigned int i = 0; i < nCount; i++)
			{
				foxit::pdf::annots::Markup^ markup = pGroupElements->GetAt(i);
				foxit::pdf::annots::AnnotType type = markup->GetType();
					
				String^ typeStr = FoxitTypeConvert::ConvertFromFoxitAnnotType(type);
				if (typeStr != AnnotationType::Caret.ToString())
				{
					_GroupAnnotIndex = markup->GetIndex();

					foxit::common::fxcrt::RectF^ rc = markup->GetRect();
					_GroupAnnotMinRect = FoxitRectConverter::ConvertFromFoxitRectF(rc);
					break;
				}
					
			}
		}
		
	}

	return true;
}

bool CaretAnnot::OnCreate(IAnnotManager^ annotmgr, Platform::Object^ pageObj)
{
	_MinRect = _Page_HightLightRect;
	if (!BaseAnnot::OnCreate(annotmgr, pageObj))
		return false;

	_PdfAnnot->PDF_Annot->SetFlags((unsigned int)foxit::pdf::annots::Flags::e_FlagNoRotate);
	foxit::common::fxcrt::RectF^ rect = _PdfAnnot->PDF_Annot->GetRect();
	foxit::pdf::annots::Caret^ caret = dynamic_cast<foxit::pdf::annots::Caret^>(FXDocUtil::DynamicCastBaseAnnot(_PdfAnnot->PDF_Annot));
	if (!caret) return false;

	caret->SetInnerRect(ref new foxit::common::fxcrt::RectF(rect->left + (float)_InnerMargin, rect->bottom + (float)_InnerMargin,
		rect->right - (float)_InnerMargin, rect->top - (float)_InnerMargin));

	_PdfAnnot->PDF_Annot->ResetAppearanceStream();
	AnnotMgr->AddAnnot(this);

	return true;
}

CaretHandler::CaretHandler()
{
	_AnnotHandlerType = AnnotationType::Caret;
}
