/**
 * Copyright (C) 2003-2018, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit Mobile PDF SDK to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */


#include "pch.h"
#include "Annot_FreeText.h"

using namespace foxit::rdk::uiextensions;

using namespace Platform;
using namespace std;
using namespace Windows::Foundation;
using namespace Windows::Foundation::Collections;
using namespace Windows::UI::Xaml;
using namespace Windows::UI::Xaml::Controls;
using namespace Windows::UI::Xaml::Controls::Primitives;
using namespace Windows::UI::Xaml::Data;
using namespace Windows::UI::Xaml::Input;
using namespace Windows::UI::Xaml::Media;
using namespace Windows::UI::Xaml::Navigation;
using namespace Windows::UI::Popups;
using namespace Windows::UI::Xaml::Shapes;

FreeTextAnnot::FreeTextAnnot()
{
	_AnnotType = AnnotationType::FreeText;
	_Color = Windows::UI::Colors::Black;
	_FontSize = 0;
	_TextMatrix.m11 = 1;
	_TextMatrix.m12 = 0;
	_TextMatrix.m21 = 0;
	_TextMatrix.m22 = 1;
	_TextMatrix.m31 = 0;
	_TextMatrix.m32 = 0;
}

Windows::Foundation::Rect FreeTextAnnot::Page_HightLightRect::get()
{
	return _Page_HightLightRect;
}

void FreeTextAnnot::Page_HightLightRect::set(Windows::Foundation::Rect value)
{
	if (_Page_HightLightRect != value)
	{
		_Page_HightLightRect = value;
		if (_PdfAnnot && _PdfAnnot->PDF_Annot)
		{
			foxit::common::fxcrt::RectF^ rect = FoxitRectConverter::ConvertToFoxitRectF(_Page_HightLightRect);
			_PdfAnnot->PDF_Annot->Move(rect);
			_MinRect = _Page_HightLightRect;			
			NotifyDocumentChanged();
		}
	}
}

Platform::String^ FreeTextAnnot::Intent::get()
{
	if (_Intent == "")
	{
		auto annot = dynamic_cast<foxit::pdf::annots::IMarkup^>(FXDocUtil::DynamicCastBaseAnnot(_PdfAnnot->PDF_Annot));
		auto pIntent = annot->GetIntent();
		
		_Intent = pIntent;
	}
	return _Intent;
}

void FreeTextAnnot::Intent::set(Platform::String^ value)
{
	if (_Intent != value)
	{
		auto annot = dynamic_cast<foxit::pdf::annots::IMarkup^>(FXDocUtil::DynamicCastBaseAnnot(_PdfAnnot->PDF_Annot));
		annot->SetIntent(value);

		_Intent = value;
	}
}

double FreeTextAnnot::FontSize::get()
{
	if (_FontSize <= 0)
	{
		foxit::pdf::annots::FreeText^ annot = dynamic_cast<foxit::pdf::annots::FreeText^>(FXDocUtil::DynamicCastBaseAnnot(_PdfAnnot->PDF_Annot));
		foxit::pdf::annots::DefaultAppearance^ deA = annot->GetDefaultAppearance();
		_FontSize = deA->text_size;
	}
	return _FontSize;
}

void FreeTextAnnot::FontSize::set(double value)
{
	if (_FontSize != value)
	{
		_FontSize = value;
		if (_PdfAnnot && _PdfAnnot->PDF_Annot)
		{
			foxit::pdf::annots::FreeText^ annot = dynamic_cast<foxit::pdf::annots::FreeText^>(FXDocUtil::DynamicCastBaseAnnot(_PdfAnnot->PDF_Annot));
			foxit::pdf::annots::DefaultAppearance^ deA = annot->GetDefaultAppearance();
			deA->text_size = (FS_FLOAT)_FontSize;
			annot->SetDefaultAppearance(deA);
			annot->ResetAppearanceStream();

			foxit::common::fxcrt::RectF^ rc = annot->GetRect();
			if (rc)
			{
				_MinRect = FoxitRectConverter::ConvertFromFoxitRectF(rc);
				_Page_HightLightRect = _MinRect;
			}
			NotifyDocumentChanged();
		}
	}
}

Platform::String^ FreeTextAnnot::FontName::get()
{
	if (_FontName == "")
	{
		foxit::pdf::annots::FreeText^ annot = dynamic_cast<foxit::pdf::annots::FreeText^>(FXDocUtil::DynamicCastBaseAnnot(_PdfAnnot->PDF_Annot));
		foxit::pdf::annots::DefaultAppearance^ deA = annot->GetDefaultAppearance();
		if (deA->font && !deA->font->IsEmpty()) {
			return deA->font->GetName();
		}
	}
	return _FontName;
}

void FreeTextAnnot::FontName::set(Platform::String^ value)
{
	if (_FontName != value)
	{
		_FontName = value;
		if (_PdfAnnot && _PdfAnnot->PDF_Annot)
		{
			foxit::pdf::annots::FreeText^ annot = dynamic_cast<foxit::pdf::annots::FreeText^>(FXDocUtil::DynamicCastBaseAnnot(_PdfAnnot->PDF_Annot));
			foxit::pdf::annots::DefaultAppearance^ deA = annot->GetDefaultAppearance();

			if ((this->FontName == "Helvetica") || (this->FontName == ""))
			{
				deA->font = ref new foxit::common::Font(foxit::common::StandardID::e_StdIDHelvetica);
			}
			else if (this->FontName == "Helvetica-Bold")
			{
				deA->font = ref new foxit::common::Font(foxit::common::StandardID::e_StdIDHelveticaB);
			}
			else if (this->FontName == "Helvetica-BoldOblique")
			{
				deA->font = ref new foxit::common::Font(foxit::common::StandardID::e_StdIDHelveticaBI);
			}
			else if (this->FontName == "Helvetica-Oblique")
			{
				deA->font = ref new foxit::common::Font(foxit::common::StandardID::e_StdIDHelveticaI);
			}
			else if (this->FontName == "Courier")
			{
				deA->font = ref new foxit::common::Font(foxit::common::StandardID::e_StdIDCourier);
			}
			else if (this->FontName == "Courier-Bold")
			{
				deA->font = ref new foxit::common::Font(foxit::common::StandardID::e_StdIDCourierB);
			}
			else if (this->FontName == "Courier-BoldOblique")
			{
				deA->font = ref new foxit::common::Font(foxit::common::StandardID::e_StdIDCourierBI);
			}
			else if (this->FontName == "Courier-Oblique")
			{
				deA->font = ref new foxit::common::Font(foxit::common::StandardID::e_StdIDCourierI);
			}
			else if (this->FontName == "Times-Roman")
			{
				deA->font = ref new foxit::common::Font(foxit::common::StandardID::e_StdIDTimes);
			}
			else if (this->FontName == "Times-Bold")
			{
				deA->font = ref new foxit::common::Font(foxit::common::StandardID::e_StdIDTimesB);
			}
			else if (this->FontName == "Times-Italic")
			{
				deA->font = ref new foxit::common::Font(foxit::common::StandardID::e_StdIDTimesI);
			}
			else if (this->FontName == "Times-BoldItalic")
			{
				deA->font = ref new foxit::common::Font(foxit::common::StandardID::e_StdIDTimesBI);
			}
			else
			{
				deA->font = ref new foxit::common::Font(this->FontName, 0, foxit::common::Charset::e_CharsetDefault, 0);
			}
			deA->text_size = (FS_FLOAT)_FontSize;
			annot->SetDefaultAppearance(deA);
			annot->ResetAppearanceStream();


			foxit::common::fxcrt::RectF^ rc = _PdfAnnot->PDF_Annot->GetRect();
			{
				_MinRect = FoxitRectConverter::ConvertFromFoxitRectF(rc);
				_Page_HightLightRect = _MinRect;
			}

			NotifyDocumentChanged();
		}
	}
}

Windows::UI::Color FreeTextAnnot::TextColor::get()
{
	return _TextColor;
}

void FreeTextAnnot::TextColor::set(Windows::UI::Color value)
{
	if (!(_TextColor.A == value.A && _TextColor.R == value.R && _TextColor.G == value.G && _TextColor.B == value.B))
	{
		_TextColor = value;
		if (_PdfAnnot && _PdfAnnot->PDF_Annot)
		{
			foxit::pdf::annots::FreeText^ annot = dynamic_cast<foxit::pdf::annots::FreeText^>(FXDocUtil::DynamicCastBaseAnnot(_PdfAnnot->PDF_Annot));
			foxit::pdf::annots::DefaultAppearance^ defAppearance = annot->GetDefaultAppearance();

			FS_ARGB color = (_TextColor.A << 24) | (_TextColor.R << 16) | (_TextColor.G << 8) | _TextColor.B;
			annot->SetBorderColor(color);
			if (!defAppearance->flags || defAppearance->flags != (unsigned int)(foxit::pdf::annots::APFlags::e_FlagTextColor | foxit::pdf::annots::APFlags::e_FlagFont | foxit::pdf::annots::APFlags::e_FlagFontSize))
			{
				defAppearance->flags = (unsigned int)(foxit::pdf::annots::APFlags::e_FlagTextColor | foxit::pdf::annots::APFlags::e_FlagFont | foxit::pdf::annots::APFlags::e_FlagFontSize);
			}
			defAppearance->text_color = color;
			if (!defAppearance->font)
			{
				defAppearance->font = ref new foxit::common::Font(foxit::common::StandardID::e_StdIDHelvetica);
			}
			
			annot->SetDefaultAppearance(defAppearance);
			annot->ResetAppearanceStream();
			NotifyDocumentChanged();
			AnnotMgr->RefleshOnePageAnnots(_PageIndex);

		}
	}
}

Windows::Foundation::Numerics::float3x2 FreeTextAnnot::TextMatrix::get()
{
	return _TextMatrix;
}

void FreeTextAnnot::TextMatrix::set(Windows::Foundation::Numerics::float3x2 value)
{
	if (!(_TextMatrix.m11 == value.m11 && _TextMatrix.m12 == value.m12 && _TextMatrix.m21 == value.m21 && _TextMatrix.m22 == value.m22 && _TextMatrix.m31 == value.m31 && _TextMatrix.m32 == value.m32))
	{
		_TextMatrix = value;
		if (_PdfAnnot && _PdfAnnot->PDF_Annot)
		{
			foxit::pdf::annots::FreeText^ annot = dynamic_cast<foxit::pdf::annots::FreeText^>(FXDocUtil::DynamicCastBaseAnnot(_PdfAnnot->PDF_Annot));
			foxit::pdf::annots::DefaultAppearance^ defAppearance = annot->GetDefaultAppearance();

			if (!defAppearance->flags || defAppearance->flags != (unsigned int)(foxit::pdf::annots::APFlags::e_FlagTextColor | foxit::pdf::annots::APFlags::e_FlagFont | foxit::pdf::annots::APFlags::e_FlagFontSize))
			{
				defAppearance->flags = (unsigned int)(foxit::pdf::annots::APFlags::e_FlagTextColor | foxit::pdf::annots::APFlags::e_FlagFont | foxit::pdf::annots::APFlags::e_FlagFontSize);
			}

			if (!defAppearance->font)
			{
				defAppearance->font = ref new foxit::common::Font(foxit::common::StandardID::e_StdIDHelvetica);
			}
			
			annot->SetDefaultAppearance(defAppearance);
			annot->ResetAppearanceStream();
			NotifyDocumentChanged();
			AnnotMgr->RefleshOnePageAnnots(_PageIndex);
		}
	}
}

bool FreeTextAnnot::IsHitTest(Windows::Foundation::Point pagePoint)
{
	return false;
}

bool FreeTextAnnot::IsHitTestRange(Windows::Foundation::Point pagePoint)
{
	return _Page_HightLightRect.Contains(pagePoint);
}

bool FreeTextAnnot::OnLoadAnnot(IAnnotManager^ annotmgr, Platform::Object^ pageObj, PDFAnnotHandle^ annot)
{
	if (!BaseAnnot::OnLoadAnnot(annotmgr, pageObj, annot))
		return false;

	FS_ARGB color;

	foxit::pdf::annots::FreeText^ freetext = dynamic_cast<foxit::pdf::annots::FreeText^>(FXDocUtil::DynamicCastBaseAnnot(_PdfAnnot->PDF_Annot));
	foxit::pdf::annots::DefaultAppearance^ defAppearance = freetext->GetDefaultAppearance();

	color = defAppearance->text_color;
	_FontSize = defAppearance->text_size;
	foxit::common::Font^ font = defAppearance->font;
	if (font && !font->IsEmpty()) {
		_FontName = font->GetName();
	}
	else
	{
		_FontName = "";
	}

	if (_FontName == "CourierNewPSMT")
	{
		_FontName = "Courier";
	}
	else if (_FontName == "CourierNewPS-BoldMT")
	{
		_FontName = "Courier-Bold";
	}
	else if (_FontName == "CourierNewPS-BoldItalicMT")
	{
		_FontName = "Courier-BoldOblique";
	}
	else if (_FontName == "CourierNewPS-ItalicMT")
	{
		_FontName = "Courier-Oblique";
	}
	else if (_FontName == "ArialMT")
	{
		_FontName = "Helvetica";
	}
	else if (_FontName == "Arial-BoldMT")
	{
		_FontName = "Helvetica-Bold";
	}
	else if (_FontName == "Arial-BoldItalicMT")
	{
		_FontName = "Helvetica-BoldOblique";
	}
	else if (_FontName == "Arial-ItalicMT")
	{
		_FontName = "Courier-Oblique";
	}
	else if (_FontName == "TimesNewRomanPSMT")
	{		
		_FontName = "Times-Roman";
	}
	else if (_FontName == "TimesNewRomanPS-BoldMT")
	{
		_FontName = "Times-Bold";
	}
	else if (_FontName == "TimesNewRomanPS-ItalicMT")
	{
		_FontName = "Times-Italic";
	}
	else if (_FontName == "TimesNewRomanPS-BoldItalicMT")
	{
		_FontName = "Times-BoldItalic";
	}

	_TextColor = Windows::UI::ColorHelper::FromArgb(color >> 24 & 0xff, color >> 16 & 0xff, color >> 8 & 0xff, color & 0xff);

	_Page_HightLightRect = _MinRect;

	return true;
}

bool FreeTextAnnot::OnCreate(IAnnotManager^ annotmgr, Platform::Object^ pageObj)
{
	_MinRect = _Page_HightLightRect;
	if (!BaseAnnot::OnCreate(annotmgr, pageObj))
		return false;
	
	foxit::pdf::annots::DefaultAppearance^ defAppearance = ref new foxit::pdf::annots::DefaultAppearance();
	defAppearance->flags = (unsigned int)(foxit::pdf::annots::APFlags::e_FlagFont | foxit::pdf::annots::APFlags::e_FlagTextColor | foxit::pdf::annots::APFlags::e_FlagFontSize);


	if ((this->FontName == "Helvetica") || (this->FontName == ""))
	{
		defAppearance->font = ref new foxit::common::Font(foxit::common::StandardID::e_StdIDHelvetica);
	}
	else if (this->FontName == "Helvetica-Bold")
	{
		defAppearance->font = ref new foxit::common::Font(foxit::common::StandardID::e_StdIDHelveticaB);
	}
	else if (this->FontName == "Helvetica-BoldOblique")
	{
		defAppearance->font = ref new foxit::common::Font(foxit::common::StandardID::e_StdIDHelveticaBI);
	}
	else if (this->FontName == "Helvetica-Oblique")
	{
		defAppearance->font = ref new foxit::common::Font(foxit::common::StandardID::e_StdIDHelveticaI);
	}
	else if (this->FontName == "Courier")
	{
		defAppearance->font = ref new foxit::common::Font(foxit::common::StandardID::e_StdIDCourier);
	}
	else if (this->FontName == "Courier-Bold")
	{
		defAppearance->font = ref new foxit::common::Font(foxit::common::StandardID::e_StdIDCourierB);
	}
	else if (this->FontName == "Courier-BoldOblique")
	{
		defAppearance->font = ref new foxit::common::Font(foxit::common::StandardID::e_StdIDCourierBI);
	}
	else if (this->FontName == "Courier-Oblique")
	{
		defAppearance->font = ref new foxit::common::Font(foxit::common::StandardID::e_StdIDCourierI);
	}
	else if (this->FontName == "Times-Roman")
	{
		defAppearance->font = ref new foxit::common::Font(foxit::common::StandardID::e_StdIDTimes);
	}
	else if (this->FontName == "Times-Bold")
	{
		defAppearance->font = ref new foxit::common::Font(foxit::common::StandardID::e_StdIDTimesB);
	}
	else if (this->FontName == "Times-Italic")
	{
		defAppearance->font = ref new foxit::common::Font(foxit::common::StandardID::e_StdIDTimesI);
	}
	else if (this->FontName == "Times-BoldItalic")
	{
		defAppearance->font = ref new foxit::common::Font(foxit::common::StandardID::e_StdIDTimesBI);
	}
	else
	{
		defAppearance->font = ref new foxit::common::Font(this->FontName, 0, foxit::common::Charset::e_CharsetDefault, 0);
	}
	defAppearance->text_color = (_TextColor.A << 24) | (_TextColor.R << 16) | (_TextColor.G << 8) | _TextColor.B;
	defAppearance->text_size = (FS_FLOAT)_FontSize;
	
	foxit::pdf::annots::FreeText^ freetext = dynamic_cast<foxit::pdf::annots::FreeText^>(FXDocUtil::DynamicCastBaseAnnot(_PdfAnnot->PDF_Annot));
	if (freetext)
		freetext->SetDefaultAppearance(defAppearance);
	
	_PdfAnnot->PDF_Annot->SetFlags((unsigned int)foxit::pdf::annots::Flags::e_FlagNoRotate);
	_PdfAnnot->PDF_Annot->ResetAppearanceStream();

	foxit::common::fxcrt::RectF^ rc = _PdfAnnot->PDF_Annot->GetRect();
	_MinRect = FoxitRectConverter::ConvertFromFoxitRectF(rc);
	_Page_HightLightRect = _MinRect;

	AnnotMgr->AddAnnot(this);

	return true;
}

void FreeTextAnnot::ResetAppearence()
{
	_PdfAnnot->PDF_Annot->ResetAppearanceStream();

	foxit::common::fxcrt::RectF^ rc = _PdfAnnot->PDF_Annot->GetRect();
	{
		_MinRect = FoxitRectConverter::ConvertFromFoxitRectF(rc);
		_Page_HightLightRect = _MinRect;
	}
}


FreeTextAnnotHandler::FreeTextAnnotHandler()
{
	_AnnotHandlerType = AnnotationType::FreeText;
}

void FreeTextAnnotHandler::OnManipulationStarted(Platform::Object^ sender, Windows::UI::Xaml::Input::ManipulationStartedRoutedEventArgs^ e)
{
	FrameworkElement^ _targetElement = dynamic_cast<FrameworkElement^>(sender);
	if (_targetElement == nullptr || AnnotMgr == nullptr)
		return;
}

void FreeTextAnnotHandler::OnManipulationDelta(Platform::Object^ sender, Windows::UI::Xaml::Input::ManipulationDeltaRoutedEventArgs^ e)
{

}

void FreeTextAnnotHandler::OnManipulationCompleted(Platform::Object^ sender, Windows::UI::Xaml::Input::ManipulationCompletedRoutedEventArgs^ e)
{

}