/**
 * Copyright (C) 2003-2018, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit Mobile PDF SDK to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */


#include "pch.h"
#include "Annot_Highlight.h"

using namespace foxit::rdk::uiextensions;

using namespace Platform;
using namespace std;
using namespace Windows::Foundation;
using namespace Windows::Foundation::Collections;
using namespace Windows::UI::Xaml;
using namespace Windows::UI::Xaml::Controls;
using namespace Windows::UI::Xaml::Controls::Primitives;
using namespace Windows::UI::Xaml::Data;
using namespace Windows::UI::Xaml::Input;
using namespace Windows::UI::Xaml::Media;
using namespace Windows::UI::Xaml::Navigation;
using namespace Windows::UI::Popups;
using namespace Windows::UI::Xaml::Shapes;

HighLightAnnot::HighLightAnnot()
{
	_AnnotType = AnnotationType::HightLight;
}

bool HighLightAnnot::IsHitTest(Windows::Foundation::Point pagePoint)
{
	if (_Page_HightLightRectGroup)
	{
		auto _size = _Page_HightLightRectGroup->Size;
		for (unsigned int i = 0; i < _size; i++)
		{
			Windows::Foundation::Rect rc = _Page_HightLightRectGroup->GetAt(i);
			if (rc.Contains(pagePoint))
			{
				return true;
			}
			else
			{
				continue;
			}
		}
	}
	return false;
}

bool HighLightAnnot::IsHitTestRange(Windows::Foundation::Point pagePoint)
{
	if (_Page_HightLightRectGroup)
	{
		auto _size = _Page_HightLightRectGroup->Size;
		for (unsigned int i = 0; i < _size; i++)
		{
			Windows::Foundation::Rect rc = _Page_HightLightRectGroup->GetAt(i);
			if (rc.Contains(pagePoint))
			{
				return true;
			}
			else
			{
				continue;
			}
		}
	}
	return false;
}

bool HighLightAnnot::OnLoadAnnot(IAnnotManager^ annotmgr, Platform::Object^ pageObj, PDFAnnotHandle^ annot)
{
	if (!BaseAnnot::OnLoadAnnot(annotmgr, pageObj, annot))
		return false;
	
	foxit::pdf::annots::ITextMarkup^ markup = dynamic_cast<foxit::pdf::annots::ITextMarkup^>(FXDocUtil::DynamicCastBaseAnnot(_PdfAnnot->PDF_Annot));
	if (!markup)
		return false;

	auto pQuadPoints = markup->GetQuadPoints();
	unsigned int nCount = pQuadPoints->GetSize();

	_Page_HightLightRectGroup = ref new Platform::Collections::Vector<Windows::Foundation::Rect>();

	for (unsigned int i = 0; i < nCount; i++)
	{
		foxit::pdf::annots::QuadPoints^ quard = pQuadPoints->GetAt(i);
		auto rc = FoxitRectConverter::ConvertFromQuardsPoint(quard);
		_Page_HightLightRectGroup->Append(rc);
	}

	return true;
}

bool HighLightAnnot::OnCreate(IAnnotManager^ annotmgr, Platform::Object^ pageObj)
{
	if (!BaseAnnot::OnCreate(annotmgr, pageObj))
		return false;

	auto annot_quad_size = _Page_HightLightRectGroup->Size;

	foxit::pdf::annots::ITextMarkup^ markup = dynamic_cast<foxit::pdf::annots::ITextMarkup^>(FXDocUtil::DynamicCastBaseAnnot(_PdfAnnot->PDF_Annot));
	if (!markup)
		return false;

	foxit::pdf::annots::QuadPointsArray^ quards = ref new foxit::pdf::annots::QuadPointsArray();

	for (unsigned int i = 0; i < annot_quad_size; i++)
	{
		auto gmtrc = _Page_HightLightRectGroup->GetAt(i);

		foxit::common::fxcrt::RectF^ rect = FoxitRectConverter::ConvertToFoxitRectF(gmtrc);

		foxit::common::fxcrt::PointF^ first = ref new foxit::common::fxcrt::PointF(rect->left, rect->bottom);
		foxit::common::fxcrt::PointF^ second = ref new foxit::common::fxcrt::PointF(rect->right, rect->bottom);
		foxit::common::fxcrt::PointF^ third = ref new foxit::common::fxcrt::PointF(rect->left, rect->top);
		foxit::common::fxcrt::PointF^ fourth = ref new foxit::common::fxcrt::PointF(rect->right, rect->top);

		foxit::pdf::annots::QuadPoints^ quard = ref new foxit::pdf::annots::QuadPoints(first, second, third, fourth);
		if(quards)
			quards->Add(quard);
	}

	if (quards)
	{
		markup->SetQuadPoints(quards);
	}

	foxit::common::fxcrt::RectF^ fsrect = markup->GetRect();
	_MinRect = FoxitRectConverter::ConvertFromFoxitRectF(fsrect);
	markup->ResetAppearanceStream();
	AnnotMgr->AddAnnot(this);

	return true;
}

HighLightAnnotHandler::HighLightAnnotHandler()
{
	_AnnotHandlerType = AnnotationType::HightLight;

	InitCommentPopup();
}

void HighLightAnnotHandler::ClearSelectedBorder()
{
	Windows::UI::Xaml::Controls::Canvas^ _Canvas = m_CanvasWeakRef.Resolve<Windows::UI::Xaml::Controls::Canvas>();
	if (_Canvas)
	{
		_Canvas->Children->Clear();
	}
}

void HighLightAnnotHandler::EndDrawing()
{
	ResetCommentPopup();

	BaseAnnotHandler::EndDrawing();
}

void HighLightAnnotHandler::Reset()
{
	ResetCommentPopup();
}

void HighLightAnnotHandler::InitCommentPopup()
{
	m_PopUp_Annotation = ref new Windows::UI::Xaml::Controls::Primitives::Popup();

	__TB = ref new TextBox();
	{
		__TB->BorderBrush = ref new SolidColorBrush(Windows::UI::ColorHelper::FromArgb(0xFF, 0x2B, 0x5F, 0x8F));
		__TB->BorderThickness = Thickness(3);
		__TB->AcceptsReturn = true;
		__TB->TextWrapping = TextWrapping::Wrap;
		__TB->IsSpellCheckEnabled = true;
		__TB->IsTextPredictionEnabled = true;
		__TB->Width = 320;
		__TB->Height = 150;
	}

	Windows::UI::Xaml::Automation::AutomationProperties::SetName(m_PopUp_Annotation, L"StickyTextBoxPopup");
	m_PopUp_Annotation->Child = __TB;
	m_PopUp_Annotation->IsLightDismissEnabled = true;

	__TB->LostFocus += ref new Windows::UI::Xaml::RoutedEventHandler(this, &HighLightAnnotHandler::OnRichEditBoxLostFocus);
	m_PopUp_Annotation->Opened += ref new EventHandler<Platform::Object^>(this, &HighLightAnnotHandler::CommentPopupOpened);
	m_PopUp_Annotation->Closed += ref new EventHandler<Platform::Object^>(this, &HighLightAnnotHandler::CommentPopupClosed);
}

void HighLightAnnotHandler::CommentPopupOpened(Platform::Object^ sender, Platform::Object^ e)
{
	Windows::UI::Xaml::Controls::Primitives::Popup^ _pop = safe_cast<Windows::UI::Xaml::Controls::Primitives::Popup^>(sender);
	if (!_pop)
		return;

	__TB->Focus(Windows::UI::Xaml::FocusState::Programmatic);

	auto _TextStr = __TB->Text;
	if (_TextStr && _TextStr->Length() > 0)
		__TB->Select(_TextStr->Length(), 0);

	auto inputPane = Windows::UI::ViewManagement::InputPane::GetForCurrentView();
	__inputPaneShowingEventRegistrationToken = inputPane->Showing += ref new TypedEventHandler<Windows::UI::ViewManagement::InputPane^, Windows::UI::ViewManagement::InputPaneVisibilityEventArgs^>([=](Windows::UI::ViewManagement::InputPane^ sender, Windows::UI::ViewManagement::InputPaneVisibilityEventArgs^ args)
	{
		args->EnsuredFocusedElementInView = false;
		m_PopUp_Annotation->IsLightDismissEnabled = false;
		if (args->OccludedRect.IntersectsWith(Rect((float)m_PopUp_Annotation->HorizontalOffset, (float)m_PopUp_Annotation->VerticalOffset, 320, 150)))
		{
			auto winrc = Windows::UI::Xaml::Window::Current->Bounds;
			auto dbVertocalOffset = m_PopUp_Annotation->VerticalOffset - (150 - (args->OccludedRect.Y - m_PopUp_Annotation->VerticalOffset));
			m_PopUp_Annotation->VerticalOffset = dbVertocalOffset;
			m_PopUp_Annotation->IsLightDismissEnabled = true;
		}
	});
	__inputPaneHidingEventRegistrationToken = inputPane->Hiding += ref new TypedEventHandler<Windows::UI::ViewManagement::InputPane^, Windows::UI::ViewManagement::InputPaneVisibilityEventArgs^>([=](Windows::UI::ViewManagement::InputPane^ sender, Windows::UI::ViewManagement::InputPaneVisibilityEventArgs^ args)
	{
		if (m_PopUp_Annotation->IsOpen)
		{
			if (m_PopUp_Annotation->VerticalOffset != m_PopUp_AnnotationVertocalOffset)
				m_PopUp_Annotation->VerticalOffset = m_PopUp_AnnotationVertocalOffset;
		}
	});
}

void HighLightAnnotHandler::CommentPopupClosed(Platform::Object^ sender, Platform::Object^ e)
{
	__TB->Text = nullptr;
	m_PopUp_Annotation->Tag = nullptr;
}

void HighLightAnnotHandler::OpenCommentPopupMarkup(Windows::Foundation::Point pt, Platform::String^ contents, Platform::Object^ param)
{
	auto winrc = Windows::UI::Xaml::Window::Current->Bounds;

	__TB->IsReadOnly = false;
	__TB->Text = contents;
	m_PopUp_Annotation->HorizontalOffset = pt.X + 320 < winrc.Right ? pt.X : winrc.Right - 320;
	m_PopUp_Annotation->VerticalOffset = pt.Y + 150 < winrc.Bottom ? pt.Y : winrc.Bottom - 150;
	m_PopUp_AnnotationVertocalOffset = m_PopUp_Annotation->VerticalOffset;
	m_PopUp_Annotation->Tag = param;
	m_PopUp_Annotation->IsOpen = true;
}

void HighLightAnnotHandler::ResetCommentPopup()
{
	if (m_PopUp_Annotation->IsOpen)
	{
		m_PopUp_Annotation->IsOpen = false;
	}
}

void HighLightAnnotHandler::OnRichEditBoxLostFocus(Platform::Object ^sender, Windows::UI::Xaml::RoutedEventArgs ^e)
{
	m_PopUp_Annotation->IsLightDismissEnabled = false;
}

