/**
 * Copyright (C) 2003-2018, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit Mobile PDF SDK to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */


#include "pch.h"
#include "Annot_Ink.h"

using namespace foxit::rdk::uiextensions;

using namespace Platform;
using namespace std;
using namespace Windows::Foundation;
using namespace Windows::Foundation::Collections;
using namespace Windows::UI::Xaml;
using namespace Windows::UI::Xaml::Controls;
using namespace Windows::UI::Xaml::Controls::Primitives;
using namespace Windows::UI::Xaml::Data;
using namespace Windows::UI::Xaml::Input;
using namespace Windows::UI::Xaml::Media;
using namespace Windows::UI::Xaml::Navigation;
using namespace Windows::UI::Popups;
using namespace Windows::UI::Xaml::Shapes;

PencilAnnot::PencilAnnot()
{
	_AnnotType = AnnotationType::Ink;

	_Page_VectPoint = ref new Platform::Collections::Vector<Windows::Foundation::Collections::IVector<Windows::Foundation::Point>^>();
	m_isUpdateView = false;
}

Windows::Foundation::Rect PencilAnnot::PageMinRect::get()
{
	return _PageMinRect;
}

void PencilAnnot::PageMinRect::set(Windows::Foundation::Rect value)
{
	if (_PageMinRect != value)
	{
		auto _tramsx = value.X - _PageMinRect.X;
		auto _transy = value.Y - _PageMinRect.Y;
		_PageMinRect = value;

		if (_PdfAnnot && _PdfAnnot->PDF_Annot)
		{
			foxit::common::fxcrt::RectF^ rect = FoxitRectConverter::ConvertToFoxitRectF(_PageMinRect);
			_PdfAnnot->PDF_Annot->Move(rect);
			
			_PdfAnnot->PDF_Annot->ResetAppearanceStream();
			rect = _PdfAnnot->PDF_Annot->GetRect();
			_MinRect = FoxitRectConverter::ConvertFromFoxitRectF(rect);

			UpdatePathData();

			NotifyDocumentChanged();
		}
	}
}

Windows::Foundation::Rect PencilAnnot::DeviceMinRect::get()
{
	return _DeviceMinRect;
}

void PencilAnnot::DeviceMinRect::set(Windows::Foundation::Rect value)
{
	if (_DeviceMinRect != value)
	{
		_DeviceMinRect = value;
	}
}


Windows::Foundation::Collections::IVector<Windows::Foundation::Collections::IVector<Windows::Foundation::Point>^>^ PencilAnnot::UpdatePage_VectPoint::get()
{
	return _Update_VectPoint;
}

void PencilAnnot::UpdatePage_VectPoint::set(Windows::Foundation::Collections::IVector<Windows::Foundation::Collections::IVector<Windows::Foundation::Point>^>^ value)
{
	if (!AnnotHandle)
		return;

	_Update_VectPoint = value;

	auto itemsize = _Update_VectPoint->Size;
	foxit::common::Path^ pathData = ref new foxit::common::Path();
	FS_INT32 index = 0;
	for (unsigned int k = 0; k < itemsize; k++)
	{
		auto pointsize = _Update_VectPoint->GetAt(k)->Size;
		for (unsigned int p = 0; p < pointsize; p++)
		{
			index = index + 1;
		}
	}
	index = 0;
	for (unsigned int k = 0; k < itemsize; k++)
	{
		auto pointsize = _Update_VectPoint->GetAt(k)->Size;
		for (unsigned int p = 0; p < pointsize; p++)
		{
			Point tmpPoint = _Update_VectPoint->GetAt(k)->GetAt(p);
			foxit::common::fxcrt::PointF^ fsPoint = ref new foxit::common::fxcrt::PointF(tmpPoint.X, tmpPoint.Y);
			if (p == 0)
			{
				pathData->MoveTo(fsPoint);
			}
			else
			{
				pathData->LineTo(fsPoint);
			}
			index = index + 1;
		}
	}

	foxit::pdf::annots::Ink^ ink = ref new foxit::pdf::annots::Ink(AnnotHandle);
	ink->SetInkList(pathData);
	
	AnnotHandle->ResetAppearanceStream();
	foxit::common::fxcrt::RectF^ fsrect = AnnotHandle->GetRect();
	_MinRect = FoxitRectConverter::ConvertFromFoxitRectF(fsrect);
	_PageMinRect = _MinRect;

	_Page_VectPoint = _Update_VectPoint;

	_PdfAnnot->PDF_Annot->ResetAppearanceStream();
	AnnotMgr->NotifyDocumentChanged();
}


bool PencilAnnot::IsHitTest(Windows::Foundation::Point pagePoint)
{
	if (!_PageMinRect.IsEmpty)
	{
		if (_PageMinRect.Contains(pagePoint))
		{
			return true;
		}
	}
	return false;
}

bool PencilAnnot::IsHitTestRange(Windows::Foundation::Point pagePoint)
{
	if (!_PageMinRect.IsEmpty)
	{
		if (_PageMinRect.Contains(pagePoint))
		{
			return true;
		}
	}

	return false;
}

bool PencilAnnot::EraserPencilPoint(Windows::Foundation::Rect eraserRect, bool isUpdateView)
{
	bool isEraser = false;
	auto _pagevectsize = _Page_VectPoint->Size;
	m_isUpdateView = isUpdateView;

	Windows::Foundation::Collections::IVector<Windows::Foundation::Collections::IVector<Windows::Foundation::Point>^>^ eraserPoints =
		ref new Platform::Collections::Vector<Windows::Foundation::Collections::IVector<Windows::Foundation::Point>^>();
	for (unsigned int i = 0; i < _pagevectsize; i++)
	{
		Windows::Foundation::Collections::IVector<Windows::Foundation::Point>^ points = ref new Platform::Collections::Vector<Windows::Foundation::Point>();

		auto _pointVect = _Page_VectPoint->GetAt(i);
		auto _pointVectSize = _pointVect->Size;
		for (unsigned int j = 0; j < _pointVectSize; j++)
		{
			auto _point = _pointVect->GetAt(j);
			if (eraserRect.Contains(_point))
			{ 
				isEraser = true;
				if (points->Size > 0)
					eraserPoints->Append(points);
				points = ref new Platform::Collections::Vector<Windows::Foundation::Point>();
				continue;
			}
			else
			{
				points->Append(_point);
			}
		}
		if (points->Size > 0)
			eraserPoints->Append(points);
	}

	if (isEraser && eraserPoints->Size > 0)
	{
		UpdatePage_VectPoint = eraserPoints;
	}

	if (_MinRect.X < 0 && _MinRect.Y < 0)
	{
		return true;
	}
		
	if (eraserPoints->Size > 0)
		return false;
	else
	{
		return true;
	}	
}

bool PencilAnnot::OnLoadAnnot(IAnnotManager^ annotmgr, Platform::Object^ pageObj, PDFAnnotHandle^ annot)
{
	if (!BaseAnnot::OnLoadAnnot(annotmgr, pageObj, annot))
		return false;
	if(!UpdatePathData())
		return false;
	return true;
}

bool PencilAnnot::OnCreate(IAnnotManager^ annotmgr, Platform::Object^ pageObj)
{
	if (!BaseAnnot::OnCreate(annotmgr, pageObj))
		return false;

	_PageMinRect = _MinRect;

	auto pencilgroupitem = _Page_VectPoint;
	auto itemsize = pencilgroupitem->Size;

	foxit::common::Path^ pathData = ref new foxit::common::Path();

	FS_INT32 index = 0;
	for (unsigned int k = 0; k < itemsize; k++)
	{
		auto pointsize = _Page_VectPoint->GetAt(k)->Size;
		for (unsigned int p = 0; p < pointsize; p++)
		{
			index = index + 1;
		}
	}
	index = 0;
	for (unsigned int k = 0; k < itemsize; k++)
	{
		auto pointsize = _Page_VectPoint->GetAt(k)->Size;
		for (unsigned int p = 0; p < pointsize; p++)
		{
			Point tmpPoint = pencilgroupitem->GetAt(k)->GetAt(p);
			if (p == 0)
			{
				pathData->MoveTo(ref new foxit::common::fxcrt::PointF(tmpPoint.X, tmpPoint.Y));
			}
			else
			{
				pathData->LineTo(ref new foxit::common::fxcrt::PointF(tmpPoint.X, tmpPoint.Y));
			}
			index = index + 1;
		}
	}

	foxit::pdf::annots::Ink^ inkAnnot = ref new foxit::pdf::annots::Ink(AnnotHandle);
	if (inkAnnot->IsEmpty())
		return false;

	inkAnnot->SetInkList(pathData);
		
	inkAnnot->ResetAppearanceStream();
	foxit::common::fxcrt::RectF^ fsrect = inkAnnot->GetRect();
	_MinRect = FoxitRectConverter::ConvertFromFoxitRectF(fsrect);
	_PageMinRect = _MinRect;

	_PdfAnnot->PDF_Annot->ResetAppearanceStream();
	AnnotMgr->AddAnnot(this);

	return true;
}

bool PencilAnnot::UpdatePathData()
{
	_PageMinRect = _MinRect;
	
	foxit::pdf::annots::Ink^ inkAnnot = ref new foxit::pdf::annots::Ink(AnnotHandle);
	if (inkAnnot->IsEmpty())
		return false;

	foxit::common::Path^ dataPath = inkAnnot->GetInkList();
	if (dataPath)
	{
		FS_INT32 count = dataPath->GetPointCount();	
		_Page_VectPoint = ref new Platform::Collections::Vector<Windows::Foundation::Collections::IVector<Windows::Foundation::Point>^>();
		Windows::Foundation::Collections::IVector<Windows::Foundation::Point>^ points = nullptr;
		for (int i = 0; i < count; i++)
		{
			FS_FLOAT x;
			FS_FLOAT y;
			foxit::common::PointType pointType = dataPath->GetPointType(i);

			foxit::common::fxcrt::PointF^ point = dataPath->GetPoint(i);
			x = point->x;
			y = point->y;
			
			if (pointType == foxit::common::PointType::e_TypeMoveTo)
			{
				if (points && points->Size > 0)
				{
					_Page_VectPoint->Append(points);
				}
				points = ref new Platform::Collections::Vector<Windows::Foundation::Point>();
			}
			Point pt = Point(x, y);
			points->Append(pt);
		}

		if (points && points->Size > 0)
		{
			_Page_VectPoint->Append(points);
		}
	}
	return true;
}

bool PencilAnnot::IsHitInkPoint(Windows::Foundation::Rect eraserRect)
{
	bool isEraser = false;
	auto _pagevectsize = _Page_VectPoint->Size;

	Windows::Foundation::Collections::IVector<Windows::Foundation::Collections::IVector<Windows::Foundation::Point>^>^ eraserPoints =
		ref new Platform::Collections::Vector<Windows::Foundation::Collections::IVector<Windows::Foundation::Point>^>();
	for (unsigned int i = 0; i < _pagevectsize; i++)
	{
		auto _pointVect = _Page_VectPoint->GetAt(i);
		auto _pointVectSize = _pointVect->Size;
		for (unsigned int j = 0; j < _pointVectSize; j++)
		{
			auto _point = _pointVect->GetAt(j);
			if (eraserRect.Contains(_point))
			{
				isEraser = true;
				break;
			}
		}
	}

	return isEraser;
}

Windows::Foundation::Collections::IVector<Windows::Foundation::Collections::IVector<Windows::Foundation::Point>^>^ PencilAnnot::GetNewVector(Windows::Foundation::Collections::IVector<Windows::Foundation::Collections::IVector<Windows::Foundation::Point>^>^ pVect)
{
	Windows::Foundation::Collections::IVector<Windows::Foundation::Collections::IVector<Windows::Foundation::Point>^>^ eraserPoints =
		ref new Platform::Collections::Vector<Windows::Foundation::Collections::IVector<Windows::Foundation::Point>^>();

	if (pVect == nullptr)
		return eraserPoints;
	
	auto _pagevectsize = pVect->Size;
	for (unsigned int i = 0; i < _pagevectsize; i++)
	{
		Windows::Foundation::Collections::IVector<Windows::Foundation::Point>^ points = ref new Platform::Collections::Vector<Windows::Foundation::Point>();

		auto _pointVect = pVect->GetAt(i);
		auto _pointVectSize = _pointVect->Size;
		for (unsigned int j = 0; j < _pointVectSize; j++)
		{
			auto _point = _pointVect->GetAt(j);
			points->Append(Windows::Foundation::Point(_point));
		}

		eraserPoints->Append(points);
	}

	return eraserPoints;
}


PencilHandler::PencilHandler()
{
	_AnnotHandlerType = AnnotationType::Ink;
}
