/**
 * Copyright (C) 2003-2018, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit Mobile PDF SDK to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */


#include "pch.h"
#include "Annot_Line.h"

using namespace foxit::rdk::uiextensions;

using namespace Platform;
using namespace std;
using namespace Windows::Foundation;
using namespace Windows::Foundation::Collections;
using namespace Windows::UI::Xaml;
using namespace Windows::UI::Xaml::Controls;
using namespace Windows::UI::Xaml::Controls::Primitives;
using namespace Windows::UI::Xaml::Data;
using namespace Windows::UI::Xaml::Input;
using namespace Windows::UI::Xaml::Media;
using namespace Windows::UI::Xaml::Navigation;
using namespace Windows::UI::Popups;
using namespace Windows::UI::Xaml::Shapes;

LineAnnot::LineAnnot()
{
	_AnnotType = AnnotationType::Line;
}

Windows::Foundation::Rect LineAnnot::PageMinRect::get()
{
	return _PageMinRect;
}

void LineAnnot::PageMinRect::set(Windows::Foundation::Rect value)
{
	if (_PageMinRect != value)
	{
		_PageMinRect = value;
		if (_PdfAnnot && _PdfAnnot->PDF_Annot)
		{
			foxit::common::fxcrt::RectF^ rect = FoxitRectConverter::ConvertToFoxitRectF(_PageMinRect);
			_PdfAnnot->PDF_Annot->Move(rect);
			_PdfAnnot->PDF_Annot->ResetAppearanceStream();
			rect = _PdfAnnot->PDF_Annot->GetRect();
			_MinRect = FoxitRectConverter::ConvertFromFoxitRectF(rect);

			if (_Page_StartPoint.X < _Page_EndPoint.X)
			{
				_Page_StartPoint = Point(_PageMinRect.X, _PageMinRect.Y);
				_Page_EndPoint = Point(_PageMinRect.X + _PageMinRect.Width, _PageMinRect.Y + _PageMinRect.Height);
			}
			else
			{
				_Page_EndPoint = Point(_PageMinRect.X, _PageMinRect.Y);
				_Page_StartPoint = Point(_PageMinRect.X + _PageMinRect.Width, _PageMinRect.Y + _PageMinRect.Height);
			}
			NotifyDocumentChanged();
		}
	}
}

Windows::Foundation::Point LineAnnot::Page_StartPoint::get()
{
	return _Page_StartPoint;
}

void LineAnnot::Page_StartPoint::set(Windows::Foundation::Point value)
{
	if (value != _Page_StartPoint)
	{
		_Page_StartPoint = value;
		if (_PdfAnnot && _PdfAnnot->PDF_Annot)
		{
			foxit::pdf::annots::Line^ lineAnnot = dynamic_cast<foxit::pdf::annots::Line^>(FXDocUtil::DynamicCastBaseAnnot(_PdfAnnot->PDF_Annot));
			lineAnnot->SetStartPoint(ref new foxit::common::fxcrt::PointF(_Page_StartPoint.X, _Page_StartPoint.Y));
			lineAnnot->SetEndPoint(ref new foxit::common::fxcrt::PointF(_Page_EndPoint.X, _Page_EndPoint.Y));
			lineAnnot->ResetAppearanceStream();
			NotifyDocumentChanged();
		}
	}
}

Windows::Foundation::Point LineAnnot::Page_EndPoint::get()
{
	return _Page_EndPoint;
}

void LineAnnot::Page_EndPoint::set(Windows::Foundation::Point value)
{
	if (value != _Page_EndPoint)
	{
		_Page_EndPoint = value;
		if (_PdfAnnot && _PdfAnnot->PDF_Annot)
		{
			foxit::pdf::annots::Line^ lineAnnot = dynamic_cast<foxit::pdf::annots::Line^>(FXDocUtil::DynamicCastBaseAnnot(_PdfAnnot->PDF_Annot));
			lineAnnot->SetStartPoint(ref new foxit::common::fxcrt::PointF(_Page_StartPoint.X, _Page_StartPoint.Y));
			lineAnnot->SetEndPoint(ref new foxit::common::fxcrt::PointF(_Page_EndPoint.X, _Page_EndPoint.Y));
			lineAnnot->ResetAppearanceStream();
			NotifyDocumentChanged();
		}
	}
}

bool LineAnnot::IsArrow::get()
{
	return m_IsArrow;
}

void LineAnnot::IsArrow :: set(bool value)
{
	m_IsArrow = value;
}

bool LineAnnot::IsHitTest(Windows::Foundation::Point pagePoint)
{
	return false;
}

bool LineAnnot::IsHitTestRange(Windows::Foundation::Point pagePoint)
{
	return _PageMinRect.Contains(pagePoint);
}

bool LineAnnot::OnLoadAnnot(IAnnotManager^ annotmgr, Platform::Object^ pageObj, PDFAnnotHandle^ annot)
{
	if (!BaseAnnot::OnLoadAnnot(annotmgr, pageObj, annot))
		return false;

	_PageMinRect = _MinRect;

	FS_FLOAT x1, y1, x2, y2;
	foxit::pdf::annots::Line^ lineAnnot = dynamic_cast<foxit::pdf::annots::Line^>(FXDocUtil::DynamicCastBaseAnnot(_PdfAnnot->PDF_Annot));
	foxit::common::fxcrt::PointF^ pt1 = lineAnnot->GetStartPoint();
	foxit::common::fxcrt::PointF^ pt2 = lineAnnot->GetEndPoint();
	x1 = pt1->x;
	y1 = pt1->y;
	x2 = pt2->x;
	y2 = pt2->y;
	_Page_StartPoint = Point(x1, y1);
	_Page_EndPoint = Point(x2, y2);
	
	auto startStyle = lineAnnot->GetLineStartStyle();
	auto endStyle = lineAnnot->GetLineEndStyle();
	
	if (startStyle == foxit::pdf::annots::EndingStyle::e_EndingStyleNone && endStyle != foxit::pdf::annots::EndingStyle::e_EndingStyleNone
		|| startStyle != foxit::pdf::annots::EndingStyle::e_EndingStyleNone && endStyle == foxit::pdf::annots::EndingStyle::e_EndingStyleNone)
		m_IsArrow = true;
	else if (startStyle != foxit::pdf::annots::EndingStyle::e_EndingStyleNone && endStyle != foxit::pdf::annots::EndingStyle::e_EndingStyleNone)
	{
		m_IsDistance = true;
	}

	return true;
}

bool LineAnnot::OnCreate(IAnnotManager^ annotmgr, Platform::Object^ pageObj)
{
	_MinRect = _PageMinRect;

	if (_MinRect.Width<= 0)
		return false;

	if (!BaseAnnot::OnCreate(annotmgr, pageObj))
		return false;

	foxit::pdf::annots::Line^ lineAnnot = dynamic_cast<foxit::pdf::annots::Line^>(FXDocUtil::DynamicCastBaseAnnot(_PdfAnnot->PDF_Annot));
	lineAnnot->SetStartPoint(ref new foxit::common::fxcrt::PointF(_Page_StartPoint.X, _Page_StartPoint.Y));
	lineAnnot->SetEndPoint(ref new foxit::common::fxcrt::PointF(_Page_EndPoint.X, _Page_EndPoint.Y));
	if (m_IsArrow)
	{
		auto startStyle = foxit::pdf::annots::EndingStyle::e_EndingStyleNone;
		auto endStyle = foxit::pdf::annots::EndingStyle::e_EndingStyleOpenArrow;
		lineAnnot->SetLineStartStyle(startStyle);
		lineAnnot->SetLineEndStyle(endStyle);
	}
	else if (m_IsDistance)
	{
		auto startStyle = foxit::pdf::annots::EndingStyle::e_EndingStyleClosedArrow;
		auto endStyle = foxit::pdf::annots::EndingStyle::e_EndingStyleClosedArrow;
		lineAnnot->SetLineStartStyle(startStyle);
		lineAnnot->SetLineEndStyle(endStyle);
	}

	lineAnnot->ResetAppearanceStream();
	AnnotMgr->AddAnnot(this);

	return true;
}


LineHandler::LineHandler()
{
	_AnnotHandlerType = AnnotationType::Line;
}
