/**
 * Copyright (C) 2003-2018, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit Mobile PDF SDK to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */


#include "pch.h"
#include "Annot_Link.h"

using namespace foxit::rdk::uiextensions;

using namespace Platform;
using namespace std;
using namespace Windows::Foundation;
using namespace Windows::Foundation::Collections;
using namespace Windows::UI::Xaml;
using namespace Windows::UI::Xaml::Controls;
using namespace Windows::UI::Xaml::Controls::Primitives;
using namespace Windows::UI::Xaml::Data;
using namespace Windows::UI::Xaml::Input;
using namespace Windows::UI::Xaml::Media;
using namespace Windows::UI::Xaml::Navigation;
using namespace Windows::UI::Popups;
using namespace Windows::UI::Xaml::Shapes;

LinkAnnot::LinkAnnot()
{
	_AnnotType = AnnotationType::Link;
}

bool LinkAnnot::IsHitTest(Windows::Foundation::Point pagePoint)
{
	if (_Page_HightLightRectGroup)
	{
		auto _size = _Page_HightLightRectGroup->Size;
		for (unsigned int i = 0; i < _size; i++)
		{
			auto _DevicePolygon = _Page_HightLightRectGroup->GetAt(i);
			if (pointInPolygon(_DevicePolygon, pagePoint))
			{
				return true;
			}
			else
			{
				continue;
			}
		}
	}
	return false;
}

bool LinkAnnot::IsHitTestRange(Windows::Foundation::Point pagePoint)
{
	if (_Page_HightLightRectGroup)
	{
		auto _size = _Page_HightLightRectGroup->Size;
		for (unsigned int i = 0; i < _size; i++)
		{
			auto _DevicePolygon = _Page_HightLightRectGroup->GetAt(i);
			if (pointInPolygon(_DevicePolygon, pagePoint))
			{
				return true;
			}
			else
			{
				continue;
			}
		}
	}
	return false;
}

bool LinkAnnot::OnLoadAnnot(IAnnotManager^ annotmgr, Platform::Object^ pageObj, PDFAnnotHandle^ annot)
{
	if (!BaseAnnot::OnLoadAnnot(annotmgr, pageObj, annot))
		return false;
	
	foxit::pdf::annots::Link^ linkAnnot = dynamic_cast<foxit::pdf::annots::Link^>(FXDocUtil::DynamicCastBaseAnnot(_PdfAnnot->PDF_Annot));
	if (!linkAnnot)
		return false;

	auto pQuadPoints = linkAnnot->GetQuadPoints();
	unsigned int nCount = pQuadPoints->GetSize();
	_Page_HightLightRectGroup = ref new Platform::Collections::Vector<LinkAnnotPolygon^>();
	if (nCount > 0)
	{
		for (unsigned int i = 0; i < nCount; i++)
		{
			foxit::pdf::annots::QuadPoints^ quadPoints = pQuadPoints->GetAt(i);

			LinkAnnotPolygon^ pPointData = ref new LinkAnnotPolygon();
			pPointData->x1 = quadPoints->first->x;
			pPointData->y1 = quadPoints->first->y;
			pPointData->x2 = quadPoints->second->x;
			pPointData->y2 = quadPoints->second->y;
			pPointData->x3 = quadPoints->third->x;
			pPointData->y3 = quadPoints->third->y;
			pPointData->x4 = quadPoints->fourth->x;
			pPointData->y4 = quadPoints->fourth->y;
			_Page_HightLightRectGroup->Append(pPointData);
		}
	}
	else
	{
		foxit::common::fxcrt::RectF^ rect = _PdfAnnot->PDF_Annot->GetRect();

		LinkAnnotPolygon^ pPointData = ref new LinkAnnotPolygon();
		pPointData->x1 = rect->left;
		pPointData->y1 = rect->top;

		pPointData->x2 = rect->right;
		pPointData->y2 = rect->top;

		pPointData->x3 = rect->left;
		pPointData->y3 = rect->bottom;

		pPointData->x4 = rect->right;
		pPointData->y4 = rect->bottom;

		_Page_HightLightRectGroup->Append(pPointData);
		
	}

	return true;
}

bool LinkAnnot::OnCreate(IAnnotManager^ annotmgr, Platform::Object^ pageObj)
{
	if (!BaseAnnot::OnCreate(annotmgr, pageObj))
		return false;

	auto annot_quad_size = _Page_HightLightRectGroup->Size;

	foxit::pdf::annots::QuadPointsArray^ annot_quad = ref new foxit::pdf::annots::QuadPointsArray();

	for (unsigned int i = 0; i < annot_quad_size; i++)
	{
		auto gmtrc = _Page_HightLightRectGroup->GetAt(i);
		auto item = annot_quad->GetAt(i);

		item->first->x = (FS_FLOAT)gmtrc->x1;
		item->first->y = (FS_FLOAT)gmtrc->x1;

		item->second->x = (FS_FLOAT)gmtrc->x2;
		item->second->y = (FS_FLOAT)gmtrc->x3;

		item->third->x = (FS_FLOAT)gmtrc->x3;
		item->third->y = (FS_FLOAT)gmtrc->x3;

		item->fourth->x = (FS_FLOAT)gmtrc->x4;
		item->fourth->y = (FS_FLOAT)gmtrc->x4;
	}

	if (annot_quad)
	{
		foxit::pdf::annots::Link^ linkAnnot = dynamic_cast<foxit::pdf::annots::Link^>(FXDocUtil::DynamicCastBaseAnnot(_PdfAnnot->PDF_Annot));
		linkAnnot->SetQuadPoints(annot_quad);
	}

	foxit::common::fxcrt::RectF^ fsrect = _PdfAnnot->PDF_Annot->GetRect();
	_MinRect = FoxitRectConverter::ConvertFromFoxitRectF(fsrect);
	_PdfAnnot->PDF_Annot->ResetAppearanceStream();
	AnnotMgr->AddAnnot(this);

	return true;
}

bool LinkAnnot::pointInPolygon(LinkAnnotPolygon^ pLinkPoint, Windows::Foundation::Point pagePt)
{
	int polySides = 4;
	int i, j = polySides - 1;
	bool  oddNodes = false;
	double polyX[4];
	double polyY[4];
	double x = pagePt.X;
	double y = pagePt.Y;
	int nCross = 0;
	/*(x1, y1)				(x2,y2)
	---------------------
	|					|
	|					|
	---------------------
	(x4, y4)			(x3,y3)*/
	polyX[0] = (float)pLinkPoint->x1; polyX[1] = (float)pLinkPoint->x2;  polyX[2] = (float)pLinkPoint->x4;  polyX[3] = (float)pLinkPoint->x3;
	polyY[0] = (float)pLinkPoint->y1; polyY[1] = (float)pLinkPoint->y2;  polyY[2] = (float)pLinkPoint->y4;  polyY[3] = (float)pLinkPoint->y3;
	for (i = 0; i<polySides; i++)
	{
		Point p1;
		p1.X = (float)polyX[i];
		p1.Y = (float)polyY[i];

		Point p2;
		p2.X = (float)polyX[(i + 1) % polySides];
		p2.Y = (float)polyY[(i + 1) % polySides];

		
		if (p1.Y == p2.Y)
			continue;
		if (pagePt.Y < min(p1.Y, p2.Y)) 
			continue;
		if (pagePt.Y >= max(p1.Y, p2.Y))
			continue;

		double x = (double)(pagePt.Y - p1.Y) * (double)(p2.X - p1.X) / (double)(p2.Y - p1.Y) + p1.X;
		if (x > pagePt.X)
			nCross++; 
	}

	bool ret = (nCross % 2 == 1);
	return ret;
}

LinkAnnotHandler::LinkAnnotHandler()
{
	_AnnotHandlerType = AnnotationType::Link;

}

void LinkAnnotHandler::ClearSelectedBorder()
{
	
}

void LinkAnnotHandler::EndDrawing()
{
	BaseAnnotHandler::EndDrawing();
}

void LinkAnnotHandler::Reset()
{

}

