/**
 * Copyright (C) 2003-2018, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit Mobile PDF SDK to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */


#include "pch.h"
#include "Annot_Squiggly.h"

using namespace foxit::rdk::uiextensions;

using namespace Platform;
using namespace std;
using namespace Windows::Foundation;
using namespace Windows::Foundation::Collections;
using namespace Windows::UI::Xaml;
using namespace Windows::UI::Xaml::Controls;
using namespace Windows::UI::Xaml::Controls::Primitives;
using namespace Windows::UI::Xaml::Data;
using namespace Windows::UI::Xaml::Input;
using namespace Windows::UI::Xaml::Media;
using namespace Windows::UI::Xaml::Navigation;
using namespace Windows::UI::Popups;
using namespace Windows::UI::Xaml::Shapes;


SquigglyAnnot::SquigglyAnnot()
{
	_AnnotType = AnnotationType::Squiggly;
}

bool SquigglyAnnot::IsHitTest(Windows::Foundation::Point pagePoint)
{
	if (_Page_HightLightRectGroup)
	{
		auto _size = _Page_HightLightRectGroup->Size;
		for (unsigned int i = 0; i < _size; i++)
		{
			Windows::Foundation::Rect rc = _Page_HightLightRectGroup->GetAt(i);
			if (rc.Contains(pagePoint))
			{
				return true;
			}
			else
			{
				continue;
			}
		}
	}
	return false;
}

bool SquigglyAnnot::IsHitTestRange(Windows::Foundation::Point pagePoint)
{
	if (_Page_HightLightRectGroup)
	{
		auto _size = _Page_HightLightRectGroup->Size;
		for (unsigned int i = 0; i < _size; i++)
		{
			Windows::Foundation::Rect rc = _Page_HightLightRectGroup->GetAt(i);
			if (rc.Contains(pagePoint))
			{
				return true;
			}
			else
			{
				continue;
			}
		}
	}
	return false;
}

bool SquigglyAnnot::OnLoadAnnot(IAnnotManager^ annotmgr, Platform::Object^ pageObj, PDFAnnotHandle^ annot)
{
	if (!BaseAnnot::OnLoadAnnot(annotmgr, pageObj, annot))
		return false;

	foxit::pdf::annots::Squiggly^ squiggly = dynamic_cast<foxit::pdf::annots::Squiggly^>(FXDocUtil::DynamicCastBaseAnnot(_PdfAnnot->PDF_Annot));

	auto pQuadPoints = squiggly->GetQuadPoints();
	unsigned int nCount = pQuadPoints->GetSize();
	_Page_HightLightRectGroup = ref new Platform::Collections::Vector<Windows::Foundation::Rect>();

	for (unsigned int i = 0; i < nCount; i++)
	{
		foxit::pdf::annots::QuadPoints^ quardPoints = pQuadPoints->GetAt(i);
		auto rc = FoxitRectConverter::ConvertFromQuardsPoint(quardPoints);
		_Page_HightLightRectGroup->Append(rc);
	}
	return true;
}

bool SquigglyAnnot::OnCreate(IAnnotManager^ annotmgr, Platform::Object^ pageObj)
{
	if (!BaseAnnot::OnCreate(annotmgr, pageObj))
		return false;

	auto annot_quad_size = _Page_HightLightRectGroup->Size;

	foxit::pdf::annots::QuadPointsArray^ annot_quad = ref new foxit::pdf::annots::QuadPointsArray();

	for (unsigned int i = 0; i < annot_quad_size; i++)
	{
		auto gmtrc = _Page_HightLightRectGroup->GetAt(i);
		int textRotate = 0;
		if (_TextRotateVect  && _TextRotateVect->Size>0)
			textRotate = _TextRotateVect->GetAt(i);

		foxit::common::fxcrt::RectF^ rect = FoxitRectConverter::ConvertToFoxitRectF(gmtrc);
		bool vert = textRotate == 1 || textRotate == 3;
		auto item = ref new foxit::pdf::annots::QuadPoints();
		if (vert) {
			item->first->x = rect->left;
			item->first->y = rect->top;

			item->second->x = rect->left;
			item->second->y = rect->bottom;

			item->third->x = rect->right;
			item->third->y = rect->top;

			item->fourth->x = rect->right;
			item->fourth->y = rect->bottom;
		}
		else
		{
			item->first->x = rect->left;
			item->first->y = rect->top;

			item->second->x = rect->right;
			item->second->y = rect->top;

			item->third->x = rect->left;
			item->third->y = rect->bottom;

			item->fourth->x = rect->right;
			item->fourth->y = rect->bottom;
		}
		annot_quad->Add(item);
	}

	foxit::pdf::annots::Squiggly^ squiggly = dynamic_cast<foxit::pdf::annots::Squiggly^>(FXDocUtil::DynamicCastBaseAnnot(_PdfAnnot->PDF_Annot));
	squiggly->SetQuadPoints(annot_quad);


	foxit::common::fxcrt::RectF^ fsrect = _PdfAnnot->PDF_Annot->GetRect();
	_MinRect = FoxitRectConverter::ConvertFromFoxitRectF(fsrect);
	_PdfAnnot->PDF_Annot->ResetAppearanceStream();
	AnnotMgr->AddAnnot(this);

	return true;
}

SquigglyAnnotHandler::SquigglyAnnotHandler()
{
	_AnnotHandlerType = AnnotationType::Squiggly;

}

void SquigglyAnnotHandler::ClearSelectedBorder()
{
	Windows::UI::Xaml::Controls::Canvas^ _Canvas = m_CanvasWeakRef.Resolve<Windows::UI::Xaml::Controls::Canvas>();
	if (_Canvas)
	{
		_Canvas->Children->Clear();
	}
}

void SquigglyAnnotHandler::EndDrawing()
{

	BaseAnnotHandler::EndDrawing();
}

void SquigglyAnnotHandler::Reset()
{

}

