/**
 * Copyright (C) 2003-2018, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit Mobile PDF SDK to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */


#include "pch.h"
#include "Annot_Stamp.h"
#include "ImageFileIteor.h"

using namespace foxit::rdk::uiextensions;

using namespace Platform;
using namespace std;
using namespace Windows::Foundation;
using namespace Windows::Foundation::Collections;
using namespace Windows::UI::Xaml;
using namespace Windows::UI::Xaml::Controls;
using namespace Windows::UI::Xaml::Controls::Primitives;
using namespace Windows::UI::Xaml::Data;
using namespace Windows::UI::Xaml::Input;
using namespace Windows::UI::Xaml::Media;
using namespace Windows::UI::Xaml::Navigation;
using namespace Windows::UI::Popups;
using namespace Windows::UI::Xaml::Shapes;
using namespace concurrency;

StampAnnot::StampAnnot()
{
	_AnnotType = AnnotationType::Stamp;
	_Color = Windows::UI::Colors::Black;
	_FontSize = 0;
	m_DefaultStampIndex = 0;
}

Windows::Foundation::Rect StampAnnot::Page_HightLightRect::get()
{
	return _Page_HightLightRect;
}

void StampAnnot::Page_HightLightRect::set(Windows::Foundation::Rect value)
{
	if (_Page_HightLightRect != value)
	{
		_Page_HightLightRect = value;
		if (_PdfAnnot && _PdfAnnot->PDF_Annot)
		{
			foxit::common::fxcrt::RectF^ rect = FoxitRectConverter::ConvertToFoxitRectF(_Page_HightLightRect);
			_PdfAnnot->PDF_Annot->Move(rect);
			_PdfAnnot->PDF_Annot->ResetAppearanceStream();
			NotifyDocumentChanged();
		}
	}
}

bool StampAnnot::IsHitTest(Windows::Foundation::Point pagePoint)
{
	return false;
}

bool StampAnnot::IsHitTestRange(Windows::Foundation::Point pagePoint)
{
	return _Page_HightLightRect.Contains(pagePoint);
}

bool StampAnnot::OnLoadAnnot(IAnnotManager^ annotmgr, Platform::Object^ pageObj, PDFAnnotHandle^ annot)
{
	if (!BaseAnnot::OnLoadAnnot(annotmgr, pageObj, annot))
		return false;

	_Page_HightLightRect = _MinRect;

	auto pSourceAnnot = dynamic_cast<foxit::pdf::annots::Stamp^>(FXDocUtil::DynamicCastBaseAnnot(_PdfAnnot->PDF_Annot));
	auto pIconName = pSourceAnnot->GetIconName();

	return true;
}

bool StampAnnot::OnCreate(IAnnotManager^ annotmgr, Platform::Object^ pageObj)
{
	_MinRect = _Page_HightLightRect;
	if (!BaseAnnot::OnCreate(annotmgr, pageObj))
		return false;

	Uri^ uri = nullptr;
	Windows::Storage::StorageFile^ storageFile;
	if (DefaultStampIndex != -1)
	{
		Platform::String^ strindex = DefaultStampIndex.ToString();
		if (strindex->Length() == 1)
			strindex = "0" + strindex;
		uri = ref new Uri("ms-appx:///foxit.rdk.uiextensions/Assets/Annot/stamp_default_" + strindex + ".png");

		auto storageFileAction = Windows::Storage::StorageFile::GetFileFromApplicationUriAsync(uri);
		while (storageFileAction->Status != Windows::Foundation::AsyncStatus::Completed)
		{
			if ((storageFileAction->Status == Windows::Foundation::AsyncStatus::Error || storageFileAction->Status == Windows::Foundation::AsyncStatus::Canceled))
				return false;
		}
		storageFile = storageFileAction->GetResults();
	}
	else
	{
		storageFile = m_CustomStampResourceFile;
	}

	foxit::common::Image^ image = ref new foxit::common::Image(storageFile->Path);
	auto stamp = dynamic_cast<foxit::pdf::annots::Stamp^>(FXDocUtil::DynamicCastBaseAnnot(_PdfAnnot->PDF_Annot));
	stamp->SetImage(image, 0, 0);

	_PdfAnnot->PDF_Annot->ResetAppearanceStream();
	AnnotMgr->AddAnnot(this);
	AnnotMgr->RefleshOnePageAnnots(_PageIndex);
	NotifyDocumentChanged();

	return true;
}


StampAnnotHandler::StampAnnotHandler()
{
	_AnnotHandlerType = AnnotationType::Stamp;
}

void StampAnnotHandler::OnManipulationStarted(Platform::Object^ sender, Windows::UI::Xaml::Input::ManipulationStartedRoutedEventArgs^ e)
{

}

void StampAnnotHandler::OnManipulationDelta(Platform::Object^ sender, Windows::UI::Xaml::Input::ManipulationDeltaRoutedEventArgs^ e)
{
	
}

void StampAnnotHandler::OnManipulationCompleted(Platform::Object^ sender, Windows::UI::Xaml::Input::ManipulationCompletedRoutedEventArgs^ e)
{
	
}


