/**
 * Copyright (C) 2003-2018, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit Mobile PDF SDK to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */


#include "pch.h"
#include "Annot_StrikeOut.h"

using namespace foxit::rdk::uiextensions;

using namespace Platform;
using namespace std;
using namespace Windows::Foundation;
using namespace Windows::Foundation::Collections;
using namespace Windows::UI::Xaml;
using namespace Windows::UI::Xaml::Controls;
using namespace Windows::UI::Xaml::Controls::Primitives;
using namespace Windows::UI::Xaml::Data;
using namespace Windows::UI::Xaml::Input;
using namespace Windows::UI::Xaml::Media;
using namespace Windows::UI::Xaml::Navigation;
using namespace Windows::UI::Popups;
using namespace Windows::UI::Xaml::Shapes;

StrikeOutAnnot::StrikeOutAnnot()
{
	_AnnotType = AnnotationType::StrikeOut;
}

bool StrikeOutAnnot::IsHitTest(Windows::Foundation::Point pagePoint)
{
	if (_Page_HightLightRectGroup)
	{
		auto _size = _Page_HightLightRectGroup->Size;
		for (unsigned int i = 0; i < _size; i++)
		{
			Windows::Foundation::Rect rc = _Page_HightLightRectGroup->GetAt(i);
			if (rc.Contains(pagePoint))
			{
				return true;
			}
			else
			{
				continue;
			}
		}
	}
	return false;
}

bool StrikeOutAnnot::IsHitTestRange(Windows::Foundation::Point pagePoint)
{
	if (_Page_HightLightRectGroup)
	{
		if (MinRect.Contains(pagePoint))
		{
			return true;
		}
		else
		{
			return false;
		}
	}
	return false;
}


Windows::Foundation::Rect StrikeOutAnnot::GroupAnnotMinRect::get()
{
	return _GroupAnnotMinRect;
}

void StrikeOutAnnot::GroupAnnotMinRect::set(Windows::Foundation::Rect value)
{
	if (_GroupAnnotMinRect != value)
	{
		_GroupAnnotMinRect = value;
	}
}

bool StrikeOutAnnot::IsHasGroup::get()
{
	return _IsHasGroup;
}

void StrikeOutAnnot::IsHasGroup::set(bool value)
{
	if (_IsHasGroup != value)
	{
		_IsHasGroup = value;
	}
}

int StrikeOutAnnot::GroupAnnotIndex::get()
{
	return _GroupAnnotIndex;
}

void StrikeOutAnnot::GroupAnnotIndex::set(int value)
{
	if (_GroupAnnotIndex != value)
	{
		_GroupAnnotIndex = value;
	}
}


bool StrikeOutAnnot::OnLoadAnnot(IAnnotManager^ annotmgr, Platform::Object^ pageObj, PDFAnnotHandle^ annot)
{
	if (!BaseAnnot::OnLoadAnnot(annotmgr, pageObj, annot))
		return false;

	auto strikeOut = dynamic_cast<foxit::pdf::annots::StrikeOut^>(FXDocUtil::DynamicCastBaseAnnot(_PdfAnnot->PDF_Annot));
	auto pQuadPoints = strikeOut->GetQuadPoints();
	unsigned int nCount = pQuadPoints->GetSize();
	_Page_HightLightRectGroup = ref new Platform::Collections::Vector<Windows::Foundation::Rect>();
	for (unsigned int i = 0; i < nCount; i++)
	{
	 	auto quadPoint = pQuadPoints->GetAt(i);
		foxit::common::fxcrt::RectF^ rect = ref new foxit::common::fxcrt::RectF();
		rect->left = quadPoint->first->x;
		rect->right = quadPoint->second->x;
		rect->top = quadPoint->second->y;
		rect->bottom = quadPoint->third->y;
		auto rc = FoxitRectConverter::ConvertFromFoxitRectF(rect);
		_Page_HightLightRectGroup->Append(rc);
	}

	FS_BOOL isGrouped = strikeOut->IsGrouped();
	if (isGrouped) {
		_IsHasGroup = true;
		auto pGroupElements = strikeOut->GetGroupElements();
		unsigned int nCount = pGroupElements->GetSize();
		for (unsigned int i = 0; i < nCount; i++)
		{
			auto _annot = pGroupElements->GetAt(i);
			auto _type = _annot->GetType();
			if (_type != foxit::pdf::annots::AnnotType::e_StrikeOut) {
				_GroupAnnotIndex = _annot->GetIndex();
				auto rc = _annot->GetRect();
				_GroupAnnotMinRect = FoxitRectConverter::ConvertFromFoxitRectF(rc);
				break;
			}
		}
	}

	return true;
}

bool StrikeOutAnnot::OnCreate(IAnnotManager^ annotmgr, Platform::Object^ pageObj)
{
	if (!BaseAnnot::OnCreate(annotmgr, pageObj))
		return false;

	auto annot_quad_size = _Page_HightLightRectGroup->Size;

	auto _strikeOut = dynamic_cast<foxit::pdf::annots::StrikeOut^>(FXDocUtil::DynamicCastBaseAnnot(_PdfAnnot->PDF_Annot));
	foxit::pdf::annots::QuadPointsArray^ annot_quad = nullptr;
	if (annot_quad_size > 0)
		annot_quad = ref new foxit::pdf::annots::QuadPointsArray();

	for (unsigned int i = 0; i < annot_quad_size; i++)
	{
		auto gmtrc = _Page_HightLightRectGroup->GetAt(i);
		int textRotate = 0;
		if (_TextRotateVect && _TextRotateVect->Size>0)
			textRotate = _TextRotateVect->GetAt(i);
		foxit::common::fxcrt::RectF^ rect = FoxitRectConverter::ConvertToFoxitRectF(gmtrc);
		bool vert = textRotate == 1 || textRotate == 3;
		auto item = ref new foxit::pdf::annots::QuadPoints();
		if (vert) {
			item->first->x = rect->left;
			item->first->y = rect->top;

			item->second->x = rect->left;
			item->second->y = rect->bottom;

			item->third->x = rect->right;
			item->third->y = rect->top;

			item->fourth->x = rect->right;
			item->fourth->y = rect->bottom;
		}
		else
		{
			item->first->x = rect->left;
			item->first->y = rect->top;

			item->second->x = rect->right;
			item->second->y = rect->top;

			item->third->x = rect->left;
			item->third->y = rect->bottom;

			item->fourth->x = rect->right;
			item->fourth->y = rect->bottom;
		}
		annot_quad->Add(item);
	}

	if (annot_quad)
		_strikeOut->SetQuadPoints(annot_quad);

	foxit::common::fxcrt::RectF^ fsrect = _strikeOut->GetRect();
	_MinRect = FoxitRectConverter::ConvertFromFoxitRectF(fsrect);
	_strikeOut->ResetAppearanceStream();
	AnnotMgr->AddAnnot(this);

	return true;
}

StrikeOutAnnotHandler::StrikeOutAnnotHandler()
{
	_AnnotHandlerType = AnnotationType::StrikeOut;

}

void StrikeOutAnnotHandler::ClearSelectedBorder()
{
	Windows::UI::Xaml::Controls::Canvas^ _Canvas = m_CanvasWeakRef.Resolve<Windows::UI::Xaml::Controls::Canvas>();
	if (_Canvas)
	{
		_Canvas->Children->Clear();
	}
}

void StrikeOutAnnotHandler::EndDrawing()
{

	BaseAnnotHandler::EndDrawing();
}

void StrikeOutAnnotHandler::Reset()
{

}

