/**
 * Copyright (C) 2003-2018, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit Mobile PDF SDK to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */


#include "pch.h"
#include "Annot_Text.h"

using namespace foxit::rdk::uiextensions;

using namespace Platform;
using namespace std;
using namespace Windows::Foundation;
using namespace Windows::Foundation::Collections;
using namespace Windows::UI::Xaml;
using namespace Windows::UI::Xaml::Controls;
using namespace Windows::UI::Xaml::Controls::Primitives;
using namespace Windows::UI::Xaml::Data;
using namespace Windows::UI::Xaml::Input;
using namespace Windows::UI::Xaml::Media;
using namespace Windows::UI::Xaml::Navigation;
using namespace Windows::UI::Popups;
using namespace Windows::UI::Xaml::Shapes;

TextAnnot::TextAnnot()
{
	_AnnotType = AnnotationType::Text;

	_IconType = AnnotIconNameTextType::Comment;
	_IconSize = Windows::Foundation::Size(20, 20);
	_ReplyAnnotGroup = ref new Platform::Collections::Vector<ReplyAnnot^>();
	_ReplyCount = 0;
	_DrawRect = Windows::Foundation::Rect(-1, -1, 0, 0);
}

AnnotIconNameTextType TextAnnot::IconType::get()
{
	return _IconType;
}

void TextAnnot::IconType::set(AnnotIconNameTextType value)
{
	if (_IconType != value)
	{
		_IconType = value;
		if (_PdfAnnot && _PdfAnnot->PDF_Annot)
		{
			auto _note = dynamic_cast<foxit::pdf::annots::Note^>(FXDocUtil::DynamicCastBaseAnnot(_PdfAnnot->PDF_Annot));
			auto _str = FoxitTypeConvert::ConvertToFoxitAnnotIconNameTextTypeString(value);
			_note->SetIconName(_str);
			_note->ResetAppearanceStream();
			AnnotMgr->NotifyDocumentChanged();
		}
	}
}

Windows::Foundation::Point TextAnnot::PosPoint::get()
{
	return _PosPoint;
}

void TextAnnot::PosPoint::set(Windows::Foundation::Point value)
{
	if (_PosPoint != value)
	{
		_PosPoint = value;
	}
}

Windows::Foundation::Size TextAnnot::IconSize::get()
{
	return _IconSize;
}

void TextAnnot::IconSize::set(Windows::Foundation::Size value)
{
	if (_IconSize != value)
	{
		_IconSize = value;
		if (_PdfAnnot && _PdfAnnot->PDF_Annot)
		{
			NotifyDocumentChanged();
		}
	}
}

bool TextAnnot::OpenStatus::get()
{
	return _OpenStatus;
}

void TextAnnot::OpenStatus::set(bool value)
{
	if (_OpenStatus != value)
	{
		_OpenStatus = value;
		if (_PdfAnnot && _PdfAnnot->PDF_Annot)
		{
			(dynamic_cast<foxit::pdf::annots::Note^>(FXDocUtil::DynamicCastBaseAnnot(_PdfAnnot->PDF_Annot)))->SetOpenStatus(value);
			NotifyDocumentChanged();
		}
	}
}

bool TextAnnot::IsHitTest(Windows::Foundation::Point pagePoint)
{
	if (_MinRect.Contains(pagePoint))
	{
		return true;
	}
	return false;
}

bool TextAnnot::IsHitTestRange(Windows::Foundation::Point pagePoint)
{
	if (_DrawRect.Contains(pagePoint))
	{
		return true;
	}
	return false;
}

int TextAnnot::ReplyCount::get()
{
	return _ReplyCount;
}

void TextAnnot::ReplyCount::set(int value)
{
	_ReplyCount = value;
}

Windows::Foundation::Rect TextAnnot::DrawRect::get()
{
	return _DrawRect;
}

void TextAnnot::DrawRect::set(Windows::Foundation::Rect value)
{
	_DrawRect = value;
}


bool TextAnnot::OnLoadAnnot(IAnnotManager^ annotmgr, Platform::Object^ pageObj, PDFAnnotHandle^ annot)
{
	if (!BaseAnnot::OnLoadAnnot(annotmgr, pageObj, annot))
		return false;

	auto _note = dynamic_cast<foxit::pdf::annots::Note^>(FXDocUtil::DynamicCastBaseAnnot(_PdfAnnot->PDF_Annot));
	String^ bsStr = _note->GetIconName();
	_IconType = FoxitTypeConvert::ConvertFromFoxitAnnotIconNameTextTypeString(bsStr);

	auto _Subject = _note->GetSubject();

	FS_INT32 count = _note->GetReplyCount();
	if (count > 0)
	{
		LoadSubReplyAnnot(_note, count, 0);
	}
	_OpenStatus = _note->GetOpenStatus();

	return true;
}

void TextAnnot::LoadSubReplyAnnot(foxit::pdf::annots::Note^ parentannot, int count, int series)
{
	auto _parentAnnot = parentannot;
	for (int i = 0; i < count; i++)
	{
		int seriesIndex = series;

		auto replyAnnot = _parentAnnot->GetReply(i);
		if (replyAnnot)
		{
			ReplyAnnot^ annot = ref new ReplyAnnot();

			annot->ReplyTitle = replyAnnot->GetTitle();
			annot->ReplyContents = replyAnnot->GetContent();

			foxit::common::DateTime^ _dateTime = replyAnnot->GetCreationDateTime();
			auto _LongDateDateTimeFormatter = ref new Windows::Globalization::DateTimeFormatting::DateTimeFormatter(L"shortdate");
			if (_dateTime)
				annot->ReplyDateTime = _LongDateDateTimeFormatter->Format(FoxitTypeConvert::ConvertFromFoxitDateTime(_dateTime));
			else
			{
				Windows::Foundation::DateTime creationDateTime;
				creationDateTime.UniversalTime = 0;
				annot->ReplyDateTime = _LongDateDateTimeFormatter->Format(creationDateTime);
			}
			annot->ReplySeries = series;
			_ReplyAnnotGroup->Append(annot);
			_ReplyCount++;

			int counts = replyAnnot->GetReplyCount();
			if (counts > 0)
			{
				seriesIndex++;
				LoadSubReplyAnnot(replyAnnot, counts, seriesIndex);
			}
		}
	}
}

bool TextAnnot::OnCreate(IAnnotManager^ annotmgr, Platform::Object^ pageObj)
{
	_MinRect = Windows::Foundation::Rect(_PosPoint.X,_PosPoint.Y-20,20,20);

	if (!BaseAnnot::OnCreate(annotmgr, pageObj))
		return false;
	auto _note = dynamic_cast<foxit::pdf::annots::Note^>(FXDocUtil::DynamicCastBaseAnnot(_PdfAnnot->PDF_Annot));
	foxit::common::fxcrt::RectF^ fsrect = _note->GetRect();
	_MinRect = FoxitRectConverter::ConvertFromFoxitRectF(fsrect);

	auto _str = FoxitTypeConvert::ConvertToFoxitAnnotIconNameTextTypeString(_IconType);
	_note->SetIconName(_str);

	auto _Subject = _note->GetSubject();

	_note->ResetAppearanceStream();
	AnnotMgr->AddAnnot(this);

	return true;
}

TextAnnotHandler::TextAnnotHandler()
{
	_AnnotHandlerType = AnnotationType::Text;
}

void TextAnnotHandler::ClearSelectedBorder()
{
	Windows::UI::Xaml::Controls::Canvas^ _Canvas = m_CanvasWeakRef.Resolve<Windows::UI::Xaml::Controls::Canvas>();
	if (_Canvas)
	{
		_Canvas->Children->Clear();
	}
}

void TextAnnotHandler::EndDrawing()
{

	BaseAnnotHandler::EndDrawing();
}

void TextAnnotHandler::Reset()
{

}


ReplyAnnot::ReplyAnnot()
{

}

Platform::String^ ReplyAnnot::ReplyTitle::get()
{
	return _ReplyTitle;
}

void ReplyAnnot::ReplyTitle::set(Platform::String^ value)
{
	_ReplyTitle = value;
	OnPropertyChanged("ReplyTitle");
}

Platform::String^ ReplyAnnot::ReplyContents::get()
{
	return _ReplyContents;
}

void ReplyAnnot::ReplyContents::set(Platform::String^ value)
{
	_ReplyContents = value;
	OnPropertyChanged("ReplyContents");
}

Platform::String^ ReplyAnnot::ReplyDateTime::get()
{
	return _ReplyDateTime;
}

void ReplyAnnot::ReplyDateTime::set(Platform::String^ value)
{
	_ReplyDateTime = value;
	OnPropertyChanged("ReplyDateTime");
}

int ReplyAnnot::ReplySeries::get()
{
	return _ReplySeries;
}

void ReplyAnnot::ReplySeries::set(int value)
{
	_ReplySeries = value;
	OnPropertyChanged("ReplySeries");
}
