/**
 * Copyright (C) 2003-2018, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit Mobile PDF SDK to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */


#include "pch.h"
#include "AnnotationHandlerFactory.h"
#include "Annot_Highlight.h"
#include "Annot_UnderLine.h"
#include "Annot_StrikeOut.h"
#include "Annot_Squiggly.h"
#include "Annot_Link.h"
#include "Annot_Text.h"
#include "Annot_Square.h"
#include "Annot_Circle.h"
#include "Annot_Line.h"
#include "Annot_Ink.h"
#include "Annot_Unknow.h"
#include "Annot_Stamp.h"
#include "Annot_FreeText.h"
#include "Annot_Caret.h"
#include "Annot_ReplaceText.h"
#include "Annot_Widget.h"

using namespace foxit::rdk::uiextensions;

using namespace Platform;

static AnnotationHandlerFactory^ _AnnotationHandlerFactory;

AnnotationHandlerFactory::AnnotationHandlerFactory()
{

}

AnnotationHandlerFactory^ AnnotationHandlerFactory::GetInstance()
{
	if (_AnnotationHandlerFactory == nullptr)
	{
		_AnnotationHandlerFactory = ref new AnnotationHandlerFactory();
	}
	
	return _AnnotationHandlerFactory;
}

IBaseAnnotHandler^ AnnotationHandlerFactory::CreateHandler(AnnotationType type, IAnnotManager^ manager)
{
	IBaseAnnotHandler^ _handler = nullptr;
	switch (type)
	{
	case AnnotationType::NonSurport:
		_handler = ref new NonSurportAnnotHandler();
		break;
	case AnnotationType::HightLight:
		_handler = ref new HighLightAnnotHandler();
		break;
	case AnnotationType::Link:
		_handler = ref new LinkAnnotHandler();
		break;
	case AnnotationType::UnderLine:
		_handler = ref new UnderLineAnnotHandler();
		break;
	case AnnotationType::StrikeOut:
		_handler = ref new StrikeOutAnnotHandler();
		break;
	case AnnotationType::Text:
		_handler = ref new TextAnnotHandler();
		break;
	case AnnotationType::Square:
		_handler = ref new SquareHandler();
		break;
	case AnnotationType::Circle:
		_handler = ref new CircleHandler();
		break;
	case AnnotationType::Line:
		_handler = ref new LineHandler();
		break;
	case AnnotationType::Ink:
	    _handler = ref new PencilHandler();
		break;
	case AnnotationType::Squiggly:
		_handler = ref new SquigglyAnnotHandler();
		break;
	case AnnotationType::Stamp:
		_handler = ref new StampAnnotHandler();
		break;
	case AnnotationType::FreeText:
		_handler = ref new FreeTextAnnotHandler();
		break;
	case AnnotationType::Caret:
		_handler = ref new CaretHandler();
		break;
	case AnnotationType::ReplaceText:
		_handler = ref new ReplaceTextHandler();
		break;
	case AnnotationType::Widget:
		_handler = ref new WidgetAnnotHandler();
		break;
	case AnnotationType::None:
		break;
	default:
		break;
	}
	
	if (_handler && manager)
		_handler->SetAnnotMgr(manager);

	return _handler;
}
