/**
 * Copyright (C) 2003-2018, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit Mobile PDF SDK to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */


#pragma once
#include "AnnotTypeDefine.h"


namespace foxit
{
	namespace rdk
	{
		namespace uiextensions
		{
			//Annot property
			public interface class IAnnotSettingImplementor
			{
				int GetAnnotDrawBorderWidth();

				void UpdateSaveCommandVisible();
				void SavePropertySetting();

				void SetAnnotDrawBorderWidth(int linewidth);

				void SetNoteLogoUrl(Platform::String^ url);
				Platform::String^ GetNoteLogoUrl();

				void SetNoteAuthor(Platform::String^ authorName);
				Platform::String^ GetNoteAuthor();

				void SetIsLinkHighlight(bool isHighlight);
				bool GetIsLinkHighlight();

				void SetViewIsDarkMode(Platform::String^ isDarkMode);
				Platform::String^ GetViewIsDarkMode();

				void SetIsFirstUserAnnot(bool isFirstUserAnnot);
				bool GetIsFirstUserAnnot();

				Windows::UI::Color GetAnnotColor();
				void SetAnnotColor(Windows::UI::Color value);

				double GetAnnotOpacity();
				void SetAnnotOpacity(double value);

				Windows::UI::Color GetHightLightAnnotColor();
				void SetHightLightAnnotColor(Windows::UI::Color value);

				double GetHightLightAnnotOpacity();
				void SetHightLightAnnotOpacity(double value);

				Windows::UI::Color GetUnderLineAnnotColor();
				void SetUnderLineAnnotColor(Windows::UI::Color value);

				double GetUnderLineAnnotOpacity();
				void SetUnderLineAnnotOpacity(double value);

				Windows::UI::Color GetStrikeOutAnnotColor();
				void SetStrikeOutAnnotColor(Windows::UI::Color value);

				double GetStrikeOutAnnotOpacity();
				void SetStrikeOutAnnotOpacity(double value);

				Windows::UI::Color GetSquigglyAnnotColor();
				void SetSquigglyAnnotColor(Windows::UI::Color value);

				double GetSquigglyAnnotOpacity();
				void SetSquigglyAnnotOpacity(double value);

				Windows::UI::Color GetLineAnnotColor();
				void SetLineAnnotColor(Windows::UI::Color value);

				int GetEraserWidth();
				void SetEraserWidth(int value);

				int GetStampIndex();
				void SetStampIndex(int value);

				bool GetHightLightIsContinueOpera();
				void SetHightLightIsContinueOpera(bool value);

				bool GetAreaHightLightIsContinueOpera();
				void SetAreaHightLightIsContinueOpera(bool value);

				bool GetStrikeOutIsContinueOpera();
				void SetStrikeOutIsContinueOpera(bool value);

				bool GetUnderLineIsContinueOpera();
				void SetUnderLineIsContinueOpera(bool value);

				bool GetSquigglyIsContinueOpera();
				void SetSquigglyIsContinueOpera(bool value);

				bool GetTextIsContinueOpera();
				void SetTextIsContinueOpera(bool value);

				bool GetLineIsContinueOpera();
				void SetLineIsContinueOpera(bool value);

				bool GetArrowIsContinueOpera();
				void SetArrowIsContinueOpera(bool value);

				bool GetSquareIsContinueOpera();
				void SetSquareIsContinueOpera(bool value);

				bool GetCircleIsContinueOpera();
				void SetCircleIsContinueOpera(bool value);

				bool GetStampIsContinueOpera();
				void SetStampeIsContinueOpera(bool value);

				bool GetFreeTextIsContinueOpera();
				void SetFreeTextIsContinueOpera(bool value);

				Windows::UI::Color GetFreeTextColor();
				void SetFreeTextColor(Windows::UI::Color value);

				double GetFreeTextAnnotOpacity();
				void SetFreeTextAnnotOpacity(double value);

				int GetFreeTextFontSize();
				void SetFreeTextFontSize(int value);

				Platform::String^ GetFreeTextFontName();
				void SetFreeTextFontName(Platform::String^ value);

				void SetIsFirstUserTip(bool isFirstUserTip);
				bool GetIsFirstUserTip();

				bool GetInsertTextIsContinueOpera();
				void SetInsertTextIsContinueOpera(bool value);

				Windows::UI::Color GetInsertTextColor();
				void SetInsertTextColor(Windows::UI::Color value);

				double GetInsertTextAnnotOpacity();
				void SetInsertTextAnnotOpacity(double value);

				Windows::UI::Color GetReplaceTextColor();
				void SetReplaceTextColor(Windows::UI::Color value);

				double GetReplaceTextAnnotOpacity();
				void SetReplaceTextAnnotOpacity(double value);

				bool GetReplaceTextIsContinueOpera();
				void SetReplaceTextIsContinueOpera(bool value);
			};

			ref class PDFPageWrapper
			{
			internal:
				Platform::Object^ Pointer;
			};

			ref class PDFAnnotWrapper
			{
			internal:
				Platform::Object^ Pointer;
			};

			ref class PDFFormControlWrapper
			{
			internal:
				Platform::Object^ Pointer;
			};

			public ref class PDFAnnotHandle sealed
			{
			public:
				property Platform::Object^ PDFAnnot
				{
					Platform::Object^ get()
					{
						PDFAnnotWrapper^ wrap = ref new PDFAnnotWrapper;
						wrap->Pointer = _PdfAnnot;
						return wrap;
					}
				}

			internal:
				PDFAnnotHandle()
				{

				}
				PDFAnnotHandle(foxit::pdf::annots::Annot^ pdfannot)
				{
					_PdfAnnot = pdfannot;
				}

				property foxit::pdf::annots::Annot^ PDF_Annot
				{
					foxit::pdf::annots::Annot^ get()
					{
						return _PdfAnnot;
					}
					void set(foxit::pdf::annots::Annot^ value)
					{
						_PdfAnnot = value;
					}
				}

			private:
				foxit::pdf::annots::Annot^ _PdfAnnot;
			};

			interface class IAnnotManager;
			[Windows::Foundation::Metadata::WebHostHidden]
			public interface class IBaseAnnot
			{
			public:
				property int PageIndex
				{
					int get();
				}

				property int AnnoIndex
				{
					int get();
				}

				property Platform::String^ Name
				{
					Platform::String^ get();
					void set(Platform::String^ value);
				}

				property Platform::String^ Title
				{
					Platform::String^ get();
					void set(Platform::String^ value);
				}

				property Platform::String^ Contents
				{
					Platform::String^ get();
					void set(Platform::String^ value);
				}

				property Windows::UI::Color Color
				{
					Windows::UI::Color get();
					void set(Windows::UI::Color value);
				}

				property double Opacity
				{
					double get();
					void set(double value);
				}

				property Windows::Foundation::Rect MinRect
				{
					Windows::Foundation::Rect get();
					void set(Windows::Foundation::Rect value);
				}

				property AnnotBorderWrap^ AnnotBorder
				{
					AnnotBorderWrap^ get();
					void set(AnnotBorderWrap^ value);
				}

				property double BorderWidth
				{
					double  get();
					void set(double value);
				}

				property Windows::Foundation::DateTime CreationDateTime
				{
					Windows::Foundation::DateTime get();
					void set(Windows::Foundation::DateTime value);
				}

				property Windows::Foundation::DateTime ModifiedDateTime
				{
					Windows::Foundation::DateTime get();
					void set(Windows::Foundation::DateTime value);
				}

				property AnnotationType AnnotType
				{
					AnnotationType get();
				}

				property IAnnotManager^ AnnotMgr
				{
					IAnnotManager^ get();
				}

				property bool IsModifyProperty
				{
					bool get();
					void set(bool isModify);
				}


				property Platform::Object^ PageHandleWrap
				{
					Platform::Object^ get();
				}

				property PDFAnnotHandle^ AnnotHandleWrap
				{
					PDFAnnotHandle^ get();
				}

				/// set Annotation visible
				virtual bool IsVisibity();
				/// Click on the precise
				virtual bool IsHitTest(Windows::Foundation::Point pagePoint);
				/// Click on the range
				bool IsHitTestRange(Windows::Foundation::Point pagePoint);
				/// initialize
				[Windows::Foundation::Metadata::DefaultOverload]
				bool OnLoadAnnot(IAnnotManager^ annotmgr, Platform::Object^ pageObj, int annotIndex);
				bool OnLoadAnnot(IAnnotManager^ annotmgr, Platform::Object^ pageObj, PDFAnnotHandle^ annot);
				/// create
				bool OnCreate(IAnnotManager^ annotmgr, Platform::Object^ pageObj);
				/// delete
				bool OnDelete();
				bool OnDeleteNoReflesh();
				/// set the selected state
				bool SetSelectedState(bool bIsShow);

				void SetAnnotVisible(bool isVisible);
				void CheckPageIndex();
			};


			[Windows::Foundation::Metadata::WebHostHidden]
			public interface class IBaseAnnotEventPeer
			{
				void OnPointerPressed(Platform::Object^ sender, Windows::UI::Xaml::Input::PointerRoutedEventArgs^ e);
				void OnPointerMoved(Platform::Object^ sender, Windows::UI::Xaml::Input::PointerRoutedEventArgs^ e);
				void OnPointerReleased(Platform::Object^ sender, Windows::UI::Xaml::Input::PointerRoutedEventArgs^ e);
				void OnTapped(Platform::Object^ sender, Windows::UI::Xaml::Input::TappedRoutedEventArgs^ e);
				void OnRightTapped(Platform::Object^ sender, Windows::UI::Xaml::Input::RightTappedRoutedEventArgs^ e);
				void OnManipulationStarted(Platform::Object^ sender, Windows::UI::Xaml::Input::ManipulationStartedRoutedEventArgs^ e);
				void OnManipulationDelta(Platform::Object^ sender, Windows::UI::Xaml::Input::ManipulationDeltaRoutedEventArgs^ e);
				void OnManipulationCompleted(Platform::Object^ sender, Windows::UI::Xaml::Input::ManipulationCompletedRoutedEventArgs^ e);
				void OnManipulationInertiaStarting(Platform::Object^ sender, Windows::UI::Xaml::Input::ManipulationInertiaStartingRoutedEventArgs^ e);

				void OnCreateAnnotAfterSelectText();
			};

			[Windows::Foundation::Metadata::WebHostHidden]
			public interface class IBaseAnnotHandler
			{
			public:
				property AnnotationType AnnotHandlerType
				{
					AnnotationType get();
				}
				property IAnnotManager^ AnnotMgr
				{
					IAnnotManager^ get();
				}
				property IBaseAnnotEventPeer^ AnnotEvent
				{
					IBaseAnnotEventPeer^ get();
				}

			public:
				IBaseAnnotEventPeer^ OnCreateEventPeer();
				void SetEventPeer(IBaseAnnotEventPeer^ peer);
				void SetAnnotMgr(IAnnotManager^ manager);
				bool IsHitTest(IBaseAnnot^ pAnnot, Windows::Foundation::Point ptDevice);
				void ClearSelectedBorder();
				void CancelDrawingOneStep();
				void EndConsistentDrawing();
				void EndDrawing();
				void Reset();

			public:
				/// create new Annotation
				[Windows::Foundation::Metadata::DefaultOverload]
				void OnCreateNewAnnot(Platform::Object^ pageObj, int annotIndex);
				void OnCreateNewAnnot(Platform::Object^ pageObj, IBaseAnnot^ newannot);
				/// load Annotation
				IBaseAnnot^ OnLoadAnnot(Platform::Object^ pageObj, PDFAnnotHandle^ annot);
				/// delete Annotation
				bool OnDelete(IBaseAnnot^ pAnnot);
				/// release Annotation
				bool OnRelease(IBaseAnnot^ pAnnot);

				/// show Annotation
				bool OnVisible(IBaseAnnot^ pAnnot);
				/// hidden Annotation
				bool OnHidden(IBaseAnnot^ pAnnot);
				/// print Annotation
				bool OnPrint(IBaseAnnot^ pAnnot);

				void OnMoving(IBaseAnnot^ pAnnot);
				void OnMoved(IBaseAnnot^ pAnnot);
				void CanecelMoving(IBaseAnnot^ pAnnot);
				void OnMoveOutOfPage(IBaseAnnot^ pAnnot);
				bool OnResizing(IBaseAnnot^ pAnnot);
				void OnPopupMenu(IBaseAnnot^ pAnnot, Windows::Foundation::Point point);
				void OnDoAnnotAction(IBaseAnnot^ pAnnot, Windows::Foundation::Point curPoint);

				void CreateAnnotAfterSelectText();
				void OnCreateNewAnnotGroup(Platform::Object^ pageObj, Windows::Foundation::Collections::IVector<IBaseAnnot^>^ newannots);

			public:
				void OnPointerPressed(Platform::Object^ sender, Windows::UI::Xaml::Input::PointerRoutedEventArgs^ e);
				void OnPointerMoved(Platform::Object^ sender, Windows::UI::Xaml::Input::PointerRoutedEventArgs^ e);
				void OnPointerReleased(Platform::Object^ sender, Windows::UI::Xaml::Input::PointerRoutedEventArgs^ e);
				void OnTapped(Platform::Object^ sender, Windows::UI::Xaml::Input::TappedRoutedEventArgs^ e);
				void OnRightTapped(Platform::Object^ sender, Windows::UI::Xaml::Input::RightTappedRoutedEventArgs^ e);
				void OnManipulationStarted(Platform::Object^ sender, Windows::UI::Xaml::Input::ManipulationStartedRoutedEventArgs^ e);
				void OnManipulationDelta(Platform::Object^ sender, Windows::UI::Xaml::Input::ManipulationDeltaRoutedEventArgs^ e);
				void OnManipulationCompleted(Platform::Object^ sender, Windows::UI::Xaml::Input::ManipulationCompletedRoutedEventArgs^ e);
				void OnManipulationInertiaStarting(Platform::Object^ sender, Windows::UI::Xaml::Input::ManipulationInertiaStartingRoutedEventArgs^ e);
			};

			[Windows::Foundation::Metadata::WebHostHidden]
			public interface class IAnnotManagerPeer
			{
				int GetPageCount();
				Platform::Object^ GetPdfPageObject(int pageIndex);
				void NotifyDocumentChanged();
				void UpdatePageView(int pageIndex);

				void SaveAnonotModifyBeforeProperty();
				void InsertAnnotModifyUndo();

				void OnAnnotPointerEntered(Platform::Object^ sender, Windows::UI::Xaml::Input::PointerRoutedEventArgs^ e);
				void OnAnnotPointerExited(Platform::Object^ sender, Windows::UI::Xaml::Input::PointerRoutedEventArgs^ e);
				void OnAnnotPointerMoved(Platform::Object^ sender, Windows::UI::Xaml::Input::PointerRoutedEventArgs^ e);
				void OnAnnotPointerPressed(Platform::Object^ sender, Windows::UI::Xaml::Input::PointerRoutedEventArgs^ e);
				void OnAnnotPointerReleased(Platform::Object^ sender, Windows::UI::Xaml::Input::PointerRoutedEventArgs^ e);
				void OnAnnotTapped(Platform::Object^ sender, Windows::UI::Xaml::Input::TappedRoutedEventArgs^ e);
				void OnAnnotRightTapped(Platform::Object^ sender, Windows::UI::Xaml::Input::RightTappedRoutedEventArgs^ e);

				void OnAnnotManipulationStarted(Platform::Object^ sender, Windows::UI::Xaml::Input::ManipulationStartedRoutedEventArgs^ e);
				void OnAnnotManipulationDelta(Platform::Object^ sender, Windows::UI::Xaml::Input::ManipulationDeltaRoutedEventArgs^ e);
				void OnAnnotManipulationCompleted(Platform::Object^ sender, Windows::UI::Xaml::Input::ManipulationCompletedRoutedEventArgs^ e);
				void OnManipulationInertiaStarting(Platform::Object^ sender, Windows::UI::Xaml::Input::ManipulationInertiaStartingRoutedEventArgs^ e);
			};


			[Windows::Foundation::Metadata::WebHostHidden]
			public interface class IAnnotManager
			{
				void OnCreate(IAnnotManagerPeer^ annotPeer);
				bool OnLoaded();
				void OnDelete();
				void AddAnnot(IBaseAnnot^ annot);
				void DeleteAnnot(IBaseAnnot^ annot);
				void ClearAnnots(int nPageIndex);
				/// create new Annotation
				void OnCreateNewAnnot(Platform::Object^ pageObj, IBaseAnnot^ newannot);
				/// load annotation
				IBaseAnnot^ OnLoadAnnot(Platform::Object^ pageObj, PDFAnnotHandle^ annot);

				void NotifyDocumentChanged();
				void UpdatePageView(int pageindex);
				void OnInitAnnot(int nPageindex);
				void ReInitAnnot(int nPageinex);
				void FinalizeAnnots(int nPageIndex);
				IBaseAnnotHandler^ GetAnnotHandlerByType(AnnotationType type);

				void OnPointerEntered(Platform::Object^ sender, Windows::UI::Xaml::Input::PointerRoutedEventArgs^ e);
				void OnPointerExited(Platform::Object^ sender, Windows::UI::Xaml::Input::PointerRoutedEventArgs^ e);
				void OnPointerMoved(Platform::Object^ sender, Windows::UI::Xaml::Input::PointerRoutedEventArgs^ e);
				void OnPointerPressed(Platform::Object^ sender, Windows::UI::Xaml::Input::PointerRoutedEventArgs^ e);
				void OnPointerReleased(Platform::Object^ sender, Windows::UI::Xaml::Input::PointerRoutedEventArgs^ e);
				void OnTapped(Platform::Object^ sender, Windows::UI::Xaml::Input::TappedRoutedEventArgs^ e);
				void OnRightTapped(Platform::Object^ sender, Windows::UI::Xaml::Input::RightTappedRoutedEventArgs^ e);

				void OnManipulationStarted(Platform::Object^ sender, Windows::UI::Xaml::Input::ManipulationStartedRoutedEventArgs^ e);
				void OnManipulationDelta(Platform::Object^ sender, Windows::UI::Xaml::Input::ManipulationDeltaRoutedEventArgs^ e);
				void OnManipulationCompleted(Platform::Object^ sender, Windows::UI::Xaml::Input::ManipulationCompletedRoutedEventArgs^ e);
				void OnManipulationInertiaStarting(Platform::Object^ sender, Windows::UI::Xaml::Input::ManipulationInertiaStartingRoutedEventArgs^ e);

				void RefleshOnePageAnnots(int nPageIndex);
				IBaseAnnot^ IsHitAnnot(int index, Windows::Foundation::Point ptDevice);
				Windows::Foundation::Collections::IVector<IBaseAnnot^>^ IsHitInkAnnot(int index, Windows::Foundation::Point ptDevice);
				IBaseAnnot^ GetAnnotHitPoint(int index, Windows::Foundation::Point pagePt);
				Windows::Foundation::Collections::IVector<IBaseAnnot^>^ GetAnnotsHitPoint(int index, Windows::Foundation::Point pagePt);
				Windows::Foundation::Collections::IVector<PDFAnnotHandle^>^ GetDrawingAnnots(int pageindex);

				IAnnotSettingImplementor^ GetAnnotSetting();
				void SetAnnotSetting(IAnnotSettingImplementor^ pSettingPageview);
				IBaseAnnot^ GetSelectedAnnot();
				void SetSelectedAnnot(IBaseAnnot^ annot);
				void SetPdfSelectAnnotNull();
				void EndDrawing();

				Windows::Foundation::Collections::IVector<IBaseAnnot^>^ GetPageNotes(int pageIndex);
				Windows::Foundation::Collections::IVector<IBaseAnnot^>^ GetPageWidgetAnnots(int pageIndex);
				void UpdateNoteRect(int pageIndex, Windows::Foundation::Rect rect);
				void SetNewCreateAnnot(Windows::Foundation::Rect rect);
				Windows::Foundation::Rect  GetNewCreateAnnotRect();
				void ClearResource();

				void UpdateAnnotMinRect(int pageIndex, AnnotationType type, Windows::Foundation::Rect oldrect, Windows::Foundation::Rect minrect);

				void AnnotCollections_CritsecLock();
				void AnnotCollections_CritsecUnLock();

				void OnCreateNewAnnotGroup(Platform::Object^ pageObj, Windows::Foundation::Collections::IVector<IBaseAnnot^>^ newannots);

				bool IsGetGroupAnnotElement(int pageIndex, Windows::Foundation::Rect minrect, int annotIndex, AnnotationType groupAnnotType);
				IBaseAnnot^ GetGroupAnnot();
				void SetGroupAnnotNullptr();

				void DeleteAnnotCollection(int pageIndex);
				void InsertModifyAnnotBeforeDelete();

				[Windows::Foundation::Metadata::DefaultOverload]
				void ChangePageIndex(int srcIndex, int dstIndex);
				void ChangePageIndex(Windows::Foundation::Collections::IVector<int>^ srcIndexs, int dstIndex);
				void PageDeleteBefore(Windows::Foundation::Collections::IVector<int>^ pageIndexs);
				void PageDeleteAfter(Windows::Foundation::Collections::IVector<int>^ pageIndexs);
				void CheckPageIndex(int startIndex, int increasingCount);

				void OnNotifyAnnotModify(IBaseAnnot^ pAnnot);

				event Windows::Foundation::EventHandler<IBaseAnnot^>^ AddAnnotEvent;
				event Windows::Foundation::EventHandler<IBaseAnnot^>^ DeleteAnnotEvent;
				event Windows::Foundation::EventHandler<IBaseAnnot^>^ ModifyAnnotEvent;
			};
		}
	}
}
