/**
 * Copyright (C) 2003-2018, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit Mobile PDF SDK to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */


#include "pch.h"
#include "ImageFileIteor.h"


using namespace foxit::rdk::uiextensions;

using namespace Platform;
using namespace Platform::Collections;
using namespace std;
using namespace Windows::Foundation;
using namespace Windows::Foundation::Collections;
using namespace Windows::UI::Xaml;
using namespace Windows::UI::Xaml::Controls;
using namespace Windows::UI::Xaml::Controls::Primitives;
using namespace Windows::UI::Xaml::Data;
using namespace Windows::UI::Xaml::Input;
using namespace Windows::UI::Xaml::Media;
using namespace Windows::UI::Xaml::Navigation;
using namespace Windows::UI::Popups;
using namespace Windows::UI::Xaml::Shapes;
using namespace Windows::Storage;
using namespace Windows::Storage::Streams;
using namespace Windows::Storage::FileProperties;
using namespace Windows::UI::Xaml::Media::Imaging;

#define MAX_FILECHACHE 30*1024*1024

FSSDK_FileHandlerReadWriteAnnotCom::FSSDK_FileHandlerReadWriteAnnotCom()
{
	m_pFile = nullptr;
	m_pFileCacheBuffer = nullptr;
}

FSSDK_FileHandlerReadWriteAnnotCom::~FSSDK_FileHandlerReadWriteAnnotCom()
{
	Release();
}

void FSSDK_FileHandlerReadWriteAnnotCom::SetSize(int nSize)
{
	m_iFileSize = nSize;
}

void FSSDK_FileHandlerReadWriteAnnotCom::CreateFileRead(Windows::Storage::StorageFile^ pFile, int nFileSize)
{
	m_pFile = pFile;
	m_iFileSize = nFileSize;
	LoadSpecifyFileCache();
}

void FSSDK_FileHandlerReadWriteAnnotCom::CreateFileToRead(Windows::Storage::StorageFile^ pFile, int nFileSize)
{
	m_pFile = pFile;
	m_iFileSize = nFileSize;
}

void FSSDK_FileHandlerReadWriteAnnotCom::Release()
{
	m_pFile = nullptr;
	if (m_Buffer && m_Buffer->Size > 0)
	{
		m_Buffer->Clear();
	}
	if (m_pFileCacheBuffer)
	{
		free(m_pFileCacheBuffer);
	}
}

int	FSSDK_FileHandlerReadWriteAnnotCom::GetSize()
{
	return m_iFileSize;
}

bool FSSDK_FileHandlerReadWriteAnnotCom::ReadBlock(void* buffer, int offset, size_t size)
{
	if (m_pFile == nullptr)
		return FALSE;

	if (m_pFileCacheBuffer && (offset + size) * sizeof(unsigned char) <= MAX_FILECHACHE)
	{
		int ID = GetCurrentThreadId();
		void* scr = (unsigned char *)m_pFileCacheBuffer + offset * sizeof(unsigned char);
		memcpy(buffer, scr, size * sizeof(unsigned char));
		return TRUE;
	}
	else
	{
		bool _bIsReadOk = false;
		HANDLE processingHandle = CreateEventEx(
			NULL,
			NULL,
			false,
			EVENT_ALL_ACCESS
			);

		if (processingHandle == NULL)
			return FALSE;

		auto  OpenOp = m_pFile->OpenAsync(Windows::Storage::FileAccessMode::Read);
		OpenOp->Completed = ref new AsyncOperationCompletedHandler<IRandomAccessStream^>([buffer, offset, size, processingHandle, &_bIsReadOk](IAsyncOperation<IRandomAccessStream^>^ operation, Windows::Foundation::AsyncStatus status)
		{

			if (status == Windows::Foundation::AsyncStatus::Completed)
			{
				IRandomAccessStream^ readStream = operation->GetResults();
				readStream->Seek(offset);
				DataReader^ dataReader = ref new DataReader(readStream);
				auto LoadOp = dataReader->LoadAsync((unsigned int) size);
				LoadOp->Completed = ref new AsyncOperationCompletedHandler<unsigned int>([buffer, dataReader, processingHandle, &_bIsReadOk](IAsyncOperation<unsigned int>^ operation, Windows::Foundation::AsyncStatus status)
				{
					if (status == Windows::Foundation::AsyncStatus::Completed)
					{
						int numBytesLoaded = operation->GetResults();
						if (numBytesLoaded > 0)
						{
							Array<unsigned char, 1>^ fileContent = ref new Array<unsigned char, 1>(numBytesLoaded);
							dataReader->ReadBytes(fileContent);
							memcpy(buffer, fileContent->Data, numBytesLoaded);
							dataReader->DetachStream();
						}
						_bIsReadOk = true;
						SetEvent(processingHandle);
					}
					else
					{
						_bIsReadOk = false;
						OutputDebugString(L"LoadAsync status ERROR!!!!!\n");
						SetEvent(processingHandle);
					}
				});
			}
			else
			{
				_bIsReadOk = false;
				OutputDebugString(L"status ERROR!!!!!\n");
				SetEvent(processingHandle);
			}
		});
		WaitForSingleObjectEx(processingHandle, INFINITE, true);
		CloseHandle(processingHandle);
		return _bIsReadOk ? TRUE : FALSE;
	}
}

bool FSSDK_FileHandlerReadWriteAnnotCom::WriteBlock(const void* buffer, int offset, size_t size)
{
	if (nullptr == m_Buffer)
	{
		m_Buffer = ref new Vector<SaveFileBufferAnnotCom^>();
	}

	Platform::Array<unsigned char>^ fileContent = ref new Platform::Array<unsigned char>((unsigned char*)buffer, (unsigned int)size);
	SaveFileBufferAnnotCom^ savebuffer = ref new SaveFileBufferAnnotCom();
	savebuffer->fileBuffer = fileContent;
	savebuffer->offsetvalue = offset;
	savebuffer->size = (int) size;

	m_Buffer->Append(savebuffer);
	return TRUE;
}

void FSSDK_FileHandlerReadWriteAnnotCom::ReleaseTempWriteBuffer()
{
	if (m_Buffer->Size > 0)
	{
		m_Buffer->Clear();
		m_Buffer = nullptr;
	}
}

Platform::Collections::Vector<SaveFileBufferAnnotCom^>^ FSSDK_FileHandlerReadWriteAnnotCom::GetFileBuffer()
{
	return m_Buffer;
}

BOOL FSSDK_FileHandlerReadWriteAnnotCom::LoadSpecifyFileCache()
{
	if (m_pFile == nullptr)
		return FALSE;
	int bufferSize = 0;
	if (m_iFileSize <= MAX_FILECHACHE)
		bufferSize = m_iFileSize;
	else
		bufferSize = MAX_FILECHACHE;
	BYTE* pFileCacheBuffer = (unsigned char *)malloc(sizeof(unsigned char) * bufferSize);
	bool _bIsReadOk = false;
	HANDLE processingHandle = CreateEventEx(
		NULL,
		NULL,
		false,
		EVENT_ALL_ACCESS
		);
	if (processingHandle == NULL)
		return FALSE;

	auto  OpenOp = m_pFile->OpenAsync(Windows::Storage::FileAccessMode::Read);
	OpenOp->Completed = ref new AsyncOperationCompletedHandler<IRandomAccessStream^>([=, &_bIsReadOk](IAsyncOperation<IRandomAccessStream^>^ operation, Windows::Foundation::AsyncStatus status)
	{
		if (status == Windows::Foundation::AsyncStatus::Completed)
		{
			IRandomAccessStream^ readStream = operation->GetResults();
			readStream->Seek(0);
			DataReader^ dataReader = ref new DataReader(readStream);
			auto LoadOp = dataReader->LoadAsync(bufferSize);
			LoadOp->Completed = ref new AsyncOperationCompletedHandler<unsigned int>([=, &_bIsReadOk](IAsyncOperation<unsigned int>^ operation, Windows::Foundation::AsyncStatus status)
			{
				if (status == Windows::Foundation::AsyncStatus::Completed)
				{
					int numBytesLoaded = operation->GetResults();
					if (numBytesLoaded > 0)
					{
						Array<unsigned char, 1>^ fileContent = ref new Array<unsigned char, 1>(numBytesLoaded);
						dataReader->ReadBytes(fileContent);
						memcpy(pFileCacheBuffer, fileContent->Data, numBytesLoaded);
						dataReader->DetachStream();
					}
					_bIsReadOk = true;
					SetEvent(processingHandle);
				}
				else
				{
					_bIsReadOk = false;
					OutputDebugString(L"LoadAsync status ERROR!!!!!\n");
					SetEvent(processingHandle);
				}
			});
		}
		else
		{
			_bIsReadOk = false;
			OutputDebugString(L"status ERROR!!!!!\n");
			SetEvent(processingHandle);
		}
	});
	WaitForSingleObjectEx(processingHandle, INFINITE, true);
	CloseHandle(processingHandle);
	m_pFileCacheBuffer = pFileCacheBuffer;
	return _bIsReadOk ? TRUE : FALSE;
}

