﻿/**
 * Copyright (C) 2003-2018, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit Mobile PDF SDK to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */


#include "pch.h"
#include "AnnotNoteUserControl.xaml.h"
#include "EditAnnotUnDoAction.h"


using namespace foxit::rdk;
using namespace foxit::rdk::uiextensions;
using namespace Platform;
using namespace Windows::Foundation;
using namespace Windows::Foundation::Collections;
using namespace Windows::UI::Xaml;
using namespace Windows::UI::Xaml::Controls;
using namespace Windows::UI::Xaml::Controls::Primitives;
using namespace Windows::UI::Xaml::Data;
using namespace Windows::UI::Xaml::Input;
using namespace Windows::UI::Xaml::Media;
using namespace Windows::UI::Xaml::Navigation;

// The User Control item template is documented at http://go.microsoft.com/fwlink/?LinkId=234236

namespace foxit
{
	namespace rdk
	{
		namespace uiextensions
		{
			[Windows::Foundation::Metadata::WebHostHidden]
			public ref class CreateTextAnnotUndoAction sealed : IUndoItem
			{
			public:
				CreateTextAnnotUndoAction(foxit::rdk::PDFViewCtrl^ view, TextAnnot^ annot);

				virtual void Undo();
				virtual void Redo();
				virtual Platform::String^ GetDescr();
				virtual void ReleaseItem();

			internal:
				property foxit::rdk::PDFViewCtrl^ View
				{
					foxit::rdk::PDFViewCtrl^ get();
				}

				property TextAnnot^ AnnotHandle
				{
					TextAnnot^ get();
				}

			private:
				Platform::WeakReference m_ViewWeakRef;
				Platform::WeakReference m_AnnotWeakRef;
				BaseAnnot^ m_RedoAnnot;
			};
		}
	}
}

CreateTextAnnotUndoAction::CreateTextAnnotUndoAction(foxit::rdk::PDFViewCtrl^ view, TextAnnot^ annot)
{
	m_ViewWeakRef = view;
	m_AnnotWeakRef = annot;
}

foxit::rdk::PDFViewCtrl^ CreateTextAnnotUndoAction::View::get()
{
	return m_ViewWeakRef.Resolve<foxit::rdk::PDFViewCtrl>();
}

TextAnnot^ CreateTextAnnotUndoAction::AnnotHandle::get()
{
	return m_AnnotWeakRef.Resolve<TextAnnot>();
}

void CreateTextAnnotUndoAction::Undo()
{
	if (View && AnnotHandle)
	{
		m_RedoAnnot = AnnotHandle;
		View->CancelRenderTask();
		AnnotHandle->OnDelete();
	}
}

void CreateTextAnnotUndoAction::Redo()
{
	UIExtensionsManager^ uiMgr = (UIExtensionsManager^)View->ExtensionsManager;
	if (View && m_RedoAnnot)
	{
		auto _page = View->GetDoc()->GetPage(m_RedoAnnot->PageIndex);

		View->CancelRenderTask();
		uiMgr->getAnnotMgr()->OnCreateNewAnnot(_page, m_RedoAnnot);
	}
}

Platform::String^ CreateTextAnnotUndoAction::GetDescr()
{
	return ref new Platform::String(L"");
}

void CreateTextAnnotUndoAction::ReleaseItem()
{
	m_ViewWeakRef = nullptr;
	m_AnnotWeakRef = nullptr;
	m_RedoAnnot = nullptr;
}


namespace foxit
{
	namespace rdk
	{
		namespace uiextensions
		{
			[Windows::Foundation::Metadata::WebHostHidden]
			public ref class CreateCaretAnnotUndoAction sealed : IUndoItem
			{
			public:
				CreateCaretAnnotUndoAction(PDFViewCtrl^ view, CaretAnnot^ annot);

				virtual void Undo();
				virtual void Redo();
				virtual Platform::String^ GetDescr();
				virtual void ReleaseItem();

			internal:
				property PDFViewCtrl^ View
				{
					PDFViewCtrl^ get();
				}

				property CaretAnnot^ AnnotHandle
				{
					CaretAnnot^ get();
				}

			private:
				Platform::WeakReference m_ViewWeakRef;
				Platform::WeakReference m_AnnotWeakRef;
				BaseAnnot^ m_RedoAnnot;
			};
		}
	}
}

CreateCaretAnnotUndoAction::CreateCaretAnnotUndoAction(PDFViewCtrl^ view, CaretAnnot^ annot)
{
	m_ViewWeakRef = view;
	m_AnnotWeakRef = annot;
}

PDFViewCtrl^ CreateCaretAnnotUndoAction::View::get()
{
	return m_ViewWeakRef.Resolve<PDFViewCtrl>();
}

CaretAnnot^ CreateCaretAnnotUndoAction::AnnotHandle::get()
{
	return m_AnnotWeakRef.Resolve<CaretAnnot>();
}

void CreateCaretAnnotUndoAction::Undo()
{
	if (View && AnnotHandle)
	{
		m_RedoAnnot = AnnotHandle;
		View->CancelRenderTask();
		AnnotHandle->OnDelete();
	}
}

void CreateCaretAnnotUndoAction::Redo()
{
	UIExtensionsManager^ uiMgr = (UIExtensionsManager^)View->ExtensionsManager;
	if (View && m_RedoAnnot)
	{
		auto _page = View->GetDoc()->GetPage(m_RedoAnnot->PageIndex);
		View->CancelRenderTask();
		uiMgr->getAnnotMgr()->OnCreateNewAnnot(_page, m_RedoAnnot);
	}
}

Platform::String^ CreateCaretAnnotUndoAction::GetDescr()
{
	return ref new Platform::String(L"");
}

void CreateCaretAnnotUndoAction::ReleaseItem()
{
	m_ViewWeakRef = nullptr;
	m_AnnotWeakRef = nullptr;
	m_RedoAnnot = nullptr;
}

AnnotNoteUserControl::AnnotNoteUserControl(PDFViewCtrl^ page, Windows::Foundation::Point pt)
{
	InitializeComponent();

	m_ViewWeakRef = page;

	UIDeviceFamilyTrigger^ trigger = ref new UIDeviceFamilyTrigger();
	OnInitializeControl();
	m_isTextAnnotDelete = false;
	m_isCreateNewNote = false;
	isCreatePopupContent = true;
	notePt = pt;
	selectLength = 0;

	contentLenghtTimer = ref new Windows::UI::Xaml::DispatcherTimer();
	auto _UISettings = ref new Windows::UI::ViewManagement::UISettings();
	auto _DoubleClickTime = _UISettings->DoubleClickTime;
	Windows::Foundation::TimeSpan _HideTimerTimeSpan;
	_HideTimerTimeSpan.Duration = _DoubleClickTime * 10000000 / 1000; 
	contentLenghtTimer->Interval = _HideTimerTimeSpan;
	contentLenghtTimer->Tick += ref new Windows::Foundation::EventHandler<Platform::Object ^>(this, &AnnotNoteUserControl::OnTappedTick);
}

void AnnotNoteUserControl::OnTappedTick(Platform::Object ^sender, Platform::Object ^args)
{
	UIExtensionsManager^ uiMgr = (UIExtensionsManager^)View->ExtensionsManager;
	if (isCreatePopupContent)
	{
		if (!isPhoneHorizontalMode)
		{
			if (ContentTextBlock->Text->Length() > 0 && ContentTextBlock->Text != " ")
			{
				CreateBtn->IsEnabled = true;
			}
			else
			{
				CreateBtn->IsEnabled = false;
			}
		}
		else
		{
			if (ContentTextBlockPhoneHor->Text->Length() > 0 && ContentTextBlockPhoneHor->Text != " ")
			{
				CreateBtnPhoneHor->IsEnabled = true;
			}
			else
			{
				CreateBtnPhoneHor->IsEnabled = false;
			}
		}
	}
	else
	{
		auto _selectAnnot = uiMgr->getAnnotMgr()->GetSelectedAnnot();

		if (_selectAnnot)
		{
			if (!isPhoneHorizontalMode)
			{
				if (ContentTextBlock->Text->Length() > 0 && ContentTextBlock->Text != " "
					&& (_selectAnnot->AnnotType == AnnotationType::Text 
						|| _selectAnnot->AnnotType == AnnotationType::Caret
						|| _selectAnnot->AnnotType == AnnotationType::StrikeOut))
				{
					SaveBtn->IsEnabled = true;
				}
				else if (_selectAnnot->AnnotType != AnnotationType::Text 
					&& _selectAnnot->AnnotType != AnnotationType::Caret  && _selectAnnot->AnnotType != AnnotationType::StrikeOut)
				{
					SaveBtn->IsEnabled = true;
				}
				else
				{
					SaveBtn->IsEnabled = false;
				}
			}
			else
			{
				if (ContentTextBlockPhoneHor->Text->Length() > 0 && ContentTextBlockPhoneHor->Text != " "
					&& (_selectAnnot->AnnotType == AnnotationType::Text || _selectAnnot->AnnotType == AnnotationType::Caret || _selectAnnot->AnnotType == AnnotationType::StrikeOut))
				{
					SaveBtnPhoneHor->IsEnabled = true;
				}
				else if (_selectAnnot->AnnotType != AnnotationType::Text && _selectAnnot->AnnotType != AnnotationType::Caret && _selectAnnot->AnnotType != AnnotationType::StrikeOut)
				{
					SaveBtnPhoneHor->IsEnabled = true;
				}
				else
				{
					SaveBtnPhoneHor->IsEnabled = false;
				}
			}
		}
	}
}

void AnnotNoteUserControl::OnInitializeControl()
{
	this->Loaded += ref new Windows::UI::Xaml::RoutedEventHandler(this, &AnnotNoteUserControl::OnLoaded);
	this->Unloaded += ref new Windows::UI::Xaml::RoutedEventHandler(this, &AnnotNoteUserControl::OnUnloaded);
}


Platform::String^ AnnotNoteUserControl::GetDateTimeStr(Windows::Foundation::DateTime datetime)
{
	auto _LongDateDateTimeFormatter = ref new Windows::Globalization::DateTimeFormatting::DateTimeFormatter(L"{year.full}-{month.integer}-{day.integer}");
	return _LongDateDateTimeFormatter->Format(datetime);
}

PDFViewCtrl^ AnnotNoteUserControl::View::get()
{
	return m_ViewWeakRef.Resolve<PDFViewCtrl>();
}

void AnnotNoteUserControl::DeleteBtn_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{
	UIExtensionsManager^ uiMgr = (UIExtensionsManager^)View->ExtensionsManager;
	if (View && uiMgr->getAnnotMgr())
	{
		auto _annot = uiMgr->getAnnotMgr()->GetSelectedAnnot();

		if (_annot && (_annot->AnnotType == AnnotationType::Text || _annot->AnnotType == AnnotationType::Caret))
		{
			auto pUndoItem = ref new DeleteAnnotUndoAction(View, _annot);
			UndoFactory::GetUndoInstance()->AddUndoItem(uiMgr, pUndoItem);

			Windows::Foundation::Rect rc = _annot->MinRect;
			foxit::common::fxcrt::RectF^ minRect = FoxitRectConverter::ConvertToFoxitRectF(_annot->MinRect);
			String^ rectStr = "[" + minRect->left.ToString() + " " + minRect->top.ToString() + " " + minRect->right.ToString() + " " + minRect->bottom.ToString() + "]";
			if (_annot->AnnotType == AnnotationType::Text)
				uiMgr->CollectAnnotOperation("Note", "Delete", _annot->PageIndex.ToString(), rectStr);
			else
				uiMgr->CollectAnnotOperation("Insert Text", "Delete", _annot->PageIndex.ToString(), rectStr);

			_annot->OnDelete();
		}
		else if (_annot->AnnotType == AnnotationType::StrikeOut)
		{
			StrikeOutAnnot^ strikeoutannot = dynamic_cast<StrikeOutAnnot^>(_annot);
			if (strikeoutannot->IsHasGroup)
			{
				Windows::Foundation::Rect groupminRect = strikeoutannot->GroupAnnotMinRect;
				bool isGetGroupAnnot = uiMgr->getAnnotMgr()->IsGetGroupAnnotElement(_annot->PageIndex, groupminRect, strikeoutannot->GroupAnnotIndex, AnnotationType::Caret);
				if (isGetGroupAnnot)
				{
					IBaseAnnot^ groupAnnot = uiMgr->getAnnotMgr()->GetGroupAnnot();

					auto pUndoItem = ref new DeleteReplaceAnnotUndoAction(View, groupAnnot, _annot);
					UndoFactory::GetUndoInstance()->AddUndoItem(uiMgr, pUndoItem);

					if (groupAnnot)
						groupAnnot->OnDelete();
				}
				_annot->OnDelete();
			}
			else
			{
				m_isTextAnnotDelete = true;
				if (_annot)
				{
					_annot->Contents = "";
				}
				_annot->IsModifyProperty = true;

				uiMgr->getAnnotMgr()->SetPdfSelectAnnotNull();
				uiMgr->SetAnnotCanvasVisible(false);
				View->Update();
			}
		}
		else
		{
			m_isTextAnnotDelete = true;
			if (_annot)
			{
				_annot->Contents = "";
			}
			_annot->IsModifyProperty = true;

			uiMgr->getAnnotMgr()->SetPdfSelectAnnotNull();
			uiMgr->SetAnnotCanvasVisible(false);
			View->Update();
		}
		uiMgr->SetTextMenuPopupHide();
		uiMgr->SetAnnotCanvasVisible(false);
		uiMgr->CloseAddNoteWindows();
	}
}


void AnnotNoteUserControl::CloseBtn_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{
	UIExtensionsManager^ uiMgr = (UIExtensionsManager^)View->ExtensionsManager;
	IsSelectText();

	if (contentLenghtTimer->IsEnabled)
		contentLenghtTimer->Stop();
	
	uiMgr->CloseAddNoteWindows();

	if (!FSPDFDocUtil::GetIsDocumentCanModifyAnnotation(View->GetDoc()) && View->GetDoc()->GetPasswordType() != foxit::pdf::PasswordType::e_PwdOwner)
	{
		if (uiMgr->getAnnotMgr()->GetSelectedAnnot() && uiMgr->getAnnotMgr()->GetSelectedAnnot()->AnnotType == AnnotationType::Text)
		{
			uiMgr->getAnnotMgr()->SetPdfSelectAnnotNull();
			View->Update();
		}
	}
	else if (uiMgr->getAnnotMgr()->GetSelectedAnnot() &&
		(uiMgr->getAnnotMgr()->GetSelectedAnnot()->AnnotType != AnnotationType::Text && uiMgr->getAnnotMgr()->GetSelectedAnnot()->AnnotType != AnnotationType::Caret))
	{
		if (uiMgr->getAnnotMgr()->GetSelectedAnnot()->AnnotType == AnnotationType::StrikeOut)
		{
			StrikeOutAnnot^ strikeoutannot = dynamic_cast<StrikeOutAnnot^>(uiMgr->getAnnotMgr()->GetSelectedAnnot());
			if (!strikeoutannot->IsHasGroup)
			{
				uiMgr->getAnnotMgr()->SetPdfSelectAnnotNull();
				View->Update();
			}
		}
		else
		{
			uiMgr->SetAnnotCanvasVisible(false);
			uiMgr->SetTextMenuPopupHide();
			uiMgr->getAnnotMgr()->SetPdfSelectAnnotNull();
			View->Update();
		}
	}

	if (View->GetDoc() && !FSPDFDocUtil::GetIsDocumentCanModifyAnnotation(View->GetDoc()) && View->GetDoc()->GetPasswordType() != foxit::pdf::PasswordType::e_PwdOwner)
	{
	}
	else
	{
		if (uiMgr->getAnnotMgr()->GetSelectedAnnot())
		{
			IBaseAnnot^ annot = uiMgr->getAnnotMgr()->GetSelectedAnnot();
			Windows::Foundation::Rect minRect = annot->MinRect;
			if (annot->AnnotType == AnnotationType::Text)
			{
				TextAnnot^ textAnnot = dynamic_cast<TextAnnot^>(annot);
				minRect = textAnnot->DrawRect;
			}
			auto deviceRect = View->ConvertPageRectToDeviceRect(annot->PageIndex, minRect);
			if (deviceRect != Windows::Foundation::Rect::Empty)
			{
				if (annot->AnnotType == AnnotationType::Text)
				{
					Point pt = Point(deviceRect.X + deviceRect.Width / 2, deviceRect.Y);
					uiMgr->ShowTextmarkContextmenu(AnnotContextMenuType::NotePro, pt);
				}
				else if (annot->AnnotType == AnnotationType::Caret || annot->AnnotType == AnnotationType::StrikeOut)
				{
					Point pt = Point(deviceRect.X + deviceRect.Width / 2, deviceRect.Y);
					uiMgr->ShowTextmarkContextmenu(AnnotContextMenuType::NotePro, pt);
				}
			}
		}
	}
	
	if (uiMgr->CreateTypeOfAnnot == AnnotCreateType::Caret || uiMgr->CreateTypeOfAnnot == AnnotCreateType::ReplaceText)
	{
		auto _selectedtext = uiMgr->getTextMgr()->SelectedText;
		int size = _selectedtext->Size;

		if (uiMgr->getTextMgr()->SelectedText && uiMgr->getTextMgr()->SelectedText->Size > 0)
		{
			uiMgr->getTextMgr()->ClearTextSelect();
			View->Update();
		}
	}

	if (uiMgr->CreateTypeOfAnnot != AnnotCreateType::None
		&& !uiMgr->getAnnotMgr()->GetAnnotSetting()->GetTextIsContinueOpera()
		&& !uiMgr->getAnnotMgr()->GetAnnotSetting()->GetInsertTextIsContinueOpera()
		&& !uiMgr->getAnnotMgr()->GetAnnotSetting()->GetReplaceTextIsContinueOpera())
	{
		uiMgr->CreateTypeOfAnnot = AnnotCreateType::None;
		uiMgr->ResetAnnotToolAppBar();
	}
	uiMgr->SetManipulationStartFlagFalse();
}


void AnnotNoteUserControl::OnLoaded(Platform::Object ^sender, Windows::UI::Xaml::RoutedEventArgs ^e)
{
	__windowSizeChangedEventRegistrationToken = Window::Current->SizeChanged += ref new Windows::UI::Xaml::WindowSizeChangedEventHandler(this, &AnnotNoteUserControl::OnSizeChanged);

	isPhoneHorizontalMode = false;
	if (View && !GetIsDesktopPlatform())
	{
		auto winrc = Windows::UI::Xaml::Window::Current->Bounds;
		mainGrid->Width = winrc.Width;
		if (winrc.Width > winrc.Height)
		{
			isPhoneHorizontalMode = true;

			GridHasNoReplyPhoneHor->Height = winrc.Height*0.41;

			ContentTextBlockPhoneHor->Width = winrc.Width*0.635;
			ContentTBPhoneHor->Width = winrc.Width*0.635;
			ContentTextBlockPhoneHor->Height = winrc.Height*0.41 - winrc.Height*0.045 * 3;
			ContentTBPhoneHor->Height = winrc.Height*0.41 - winrc.Height*0.045 * 3;

			CreateBtnPhoneHor->Width = winrc.Width*0.2;
			DeleteBtnPhoneHor->Width = winrc.Width*0.2;
			CancelBtnPhoneHor->Width = winrc.Width*0.2;
			CloseBtnPhoneHor->Width = winrc.Width*0.2;
			SaveBtnPhoneHor->Width = winrc.Width*0.2;

			CreateBtnPhoneHor->Height = winrc.Height*0.075;
			DeleteBtnPhoneHor->Height = winrc.Height*0.075;
			CancelBtnPhoneHor->Height = winrc.Height*0.075;
			CloseBtnPhoneHor->Height = winrc.Height*0.075;
			SaveBtnPhoneHor->Height = winrc.Height*0.075;
		}
		else
		{
			CreateBtn->Width = winrc.Width*0.43;
			DeleteBtn->Width = winrc.Width*0.43;
			CancelBtn->Width = winrc.Width*0.43;
			CloseBtn->Width = winrc.Width*0.43;
			SaveBtn->Width = winrc.Width*0.43;
		}
	}
	else
	{
		this->ManipulationDelta += ref new Windows::UI::Xaml::Input::ManipulationDeltaEventHandler(this, &AnnotNoteUserControl::OnManipulationDelta);
		this->ManipulationMode = ::ManipulationModes::All;
		this->RenderTransform = m_LayoutGridTransform = ref new CompositeTransform();
		m_LayoutGridTransform->TranslateX = 0;
		m_LayoutGridTransform->TranslateY = 0;
	}

	if (View)
	{
		SetUserControlUpdate();

		Popup^ m_Popup = safe_cast<Popup^>(this->Parent);
		if (m_Popup && GetIsDesktopPlatform())
		{
			auto inputPane = Windows::UI::ViewManagement::InputPane::GetForCurrentView();
			__inputPaneShowingEventRegistrationToken = inputPane->Showing += ref new TypedEventHandler<Windows::UI::ViewManagement::InputPane^, Windows::UI::ViewManagement::InputPaneVisibilityEventArgs^>([=](Windows::UI::ViewManagement::InputPane^ sender, Windows::UI::ViewManagement::InputPaneVisibilityEventArgs^ args)
			{
				args->EnsuredFocusedElementInView = false;

				if (args->OccludedRect.IntersectsWith(Rect((float)m_Popup->HorizontalOffset, (float)m_Popup->VerticalOffset, 320, 150)))
				{
					auto winrc = Windows::UI::Xaml::Window::Current->Bounds;
					auto dbVertocalOffset = m_Popup->VerticalOffset - (234 - (args->OccludedRect.Y - m_Popup->VerticalOffset));
					m_Popup->VerticalOffset = dbVertocalOffset;
					m_Popup->IsLightDismissEnabled = false;
				}
			});
			__inputPaneHidingEventRegistrationToken = inputPane->Hiding += ref new TypedEventHandler<Windows::UI::ViewManagement::InputPane^, Windows::UI::ViewManagement::InputPaneVisibilityEventArgs^>([=](Windows::UI::ViewManagement::InputPane^ sender, Windows::UI::ViewManagement::InputPaneVisibilityEventArgs^ args)
			{
				auto winrc = Windows::UI::Xaml::Window::Current->Bounds;
				double verticalOffset = notePt.Y + 50;
				if (verticalOffset + 234 > winrc.Height)
				{
					verticalOffset = winrc.Height - 234;
				}
				m_Popup->VerticalOffset = verticalOffset;
			});
		}
	}
}


void AnnotNoteUserControl::OnUnloaded(Platform::Object ^sender, Windows::UI::Xaml::RoutedEventArgs ^e)
{
	if (contentLenghtTimer->IsEnabled)
		contentLenghtTimer->Stop();

	auto __inputPane = Windows::UI::ViewManagement::InputPane::GetForCurrentView();
	__inputPane->Showing -= __inputPaneShowingEventRegistrationToken;
	__inputPane->Hiding -= __inputPaneHidingEventRegistrationToken;

	if (__windowSizeChangedEventRegistrationToken.Value)
	{
		Window::Current->SizeChanged -= __windowSizeChangedEventRegistrationToken;
		__windowSizeChangedEventRegistrationToken.Value = 0;
	}
}


void AnnotNoteUserControl::Grid_Tapped(Platform::Object^ sender, Windows::UI::Xaml::Input::TappedRoutedEventArgs^ e)
{
	
}


void AnnotNoteUserControl::CreateBtn_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{
	CreateAnnotNote();
}

void AnnotNoteUserControl::ContentTextBlock_GotFocus(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{
	UIExtensionsManager^ uiMgr = (UIExtensionsManager^)View->ExtensionsManager;
	OutputDebugString(L"ContentTextBlock_GotFocus...\n");
	if (contentLenghtTimer->IsEnabled)
		contentLenghtTimer->Stop();
	contentLenghtTimer->Start();

	auto _AnnotMgr = uiMgr->getAnnotMgr();
	if (_AnnotMgr)
	{
		auto _selectAnnot = _AnnotMgr->GetSelectedAnnot();
		AnnotCommandToken^ _Token = dynamic_cast<AnnotCommandToken^>(this->Tag);
		BaseAnnot^ _baseAnnot = nullptr;

		if ((_Token && (_baseAnnot = dynamic_cast<BaseAnnot^>(_Token->Param)) && _baseAnnot->AnnotMgr)
			|| (_selectAnnot && (_selectAnnot->AnnotType == AnnotationType::Text
				|| _selectAnnot->AnnotType == AnnotationType::Caret)))
		{
			isCreatePopupContent = false;

			ContentTextBlock->SelectionStart = ContentTextBlock->Text->Length();
			CreateBtn->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
			DeleteBtn->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
			CloseBtn->Visibility = Windows::UI::Xaml::Visibility::Collapsed;

			SaveBtn->Visibility = Windows::UI::Xaml::Visibility::Visible;
			CancelBtn->Visibility = Windows::UI::Xaml::Visibility::Visible;
		}
		else
		{
			if (!m_isTextAnnotDelete)
			{
				ContentTextBlock->SelectionStart = ContentTextBlock->Text ? ContentTextBlock->Text->Length() : 0;
				if (_selectAnnot && _selectAnnot->Contents->Length() > 0)
				{
					ContentTextBlock->SelectionStart = ContentTextBlock->Text->Length();
					CreateBtn->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
					DeleteBtn->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
					CloseBtn->Visibility = Windows::UI::Xaml::Visibility::Collapsed;

					SaveBtn->Visibility = Windows::UI::Xaml::Visibility::Visible;
					CancelBtn->Visibility = Windows::UI::Xaml::Visibility::Visible;
				}
				else
				{
					CreateBtn->Visibility = Windows::UI::Xaml::Visibility::Visible;
					DeleteBtn->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
					CloseBtn->Visibility = Windows::UI::Xaml::Visibility::Visible;

					SaveBtn->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
					CancelBtn->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
				}
			}
		}
	}
}


void AnnotNoteUserControl::CancelBtn_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{
	UIExtensionsManager^ uiMgr = (UIExtensionsManager^)View->ExtensionsManager;
	if (!isPhoneHorizontalMode)
	{
		DeleteBtn->Visibility = Windows::UI::Xaml::Visibility::Visible;
		CloseBtn->Visibility = Windows::UI::Xaml::Visibility::Visible;

		SaveBtn->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
		CancelBtn->Visibility = Windows::UI::Xaml::Visibility::Collapsed;

		DeleteBtn->Focus(Windows::UI::Xaml::FocusState::Programmatic);

		if (View && uiMgr->getAnnotMgr())
		{
			auto _annot = uiMgr->getAnnotMgr()->GetSelectedAnnot();
			if (_annot)
				ContentTextBlock->Text = _annot->Contents;
		}
	}
	else
	{
		DeleteBtnPhoneHor->Visibility = Windows::UI::Xaml::Visibility::Visible;
		CloseBtnPhoneHor->Visibility = Windows::UI::Xaml::Visibility::Visible;

		SaveBtnPhoneHor->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
		CancelBtnPhoneHor->Visibility = Windows::UI::Xaml::Visibility::Collapsed;

		DeleteBtnPhoneHor->Focus(Windows::UI::Xaml::FocusState::Programmatic);

		if (View && uiMgr->getAnnotMgr())
		{
			auto _annot = uiMgr->getAnnotMgr()->GetSelectedAnnot();
			if (_annot)
				ContentTextBlockPhoneHor->Text = _annot->Contents;
		}
	}
}


void AnnotNoteUserControl::SaveBtn_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{
	UIExtensionsManager^ uiMgr = (UIExtensionsManager^)View->ExtensionsManager;
	SaveAnnotNote();
	if (View)
	{
		uiMgr->getAnnotMgr()->SetPdfSelectAnnotNull();
		uiMgr->SetAnnotCanvasVisible(false);
		View->Update();

		uiMgr->SetTextMenuPopupHide();
		uiMgr->CloseAddNoteWindows();
	}
}

void AnnotNoteUserControl::ContentTextBlock_LostFocus(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{
	OutputDebugString(L"ContentTextBlock_LostFocus\n");
	auto inputPane = Windows::UI::ViewManagement::InputPane::GetForCurrentView();
	inputPane->TryHide();
	selectLength = 0;

	ContentTB->Text = ContentTextBlock->Text;
	ContentTB->Visibility = Windows::UI::Xaml::Visibility::Visible;
	ContentTextBlock->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
}


void AnnotNoteUserControl::ContentTB_Tapped(Platform::Object^ sender, Windows::UI::Xaml::Input::TappedRoutedEventArgs^ e)
{
	if (!FSPDFDocUtil::GetIsDocumentCanModifyAnnotation(View->GetDoc()) && View->GetDoc()->GetPasswordType() != foxit::pdf::PasswordType::e_PwdOwner)
	{

	}
	else
	{
		if (!isPhoneHorizontalMode)
		{
			ContentTextBlock->Text = ContentTB->Text;
			ContentTB->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
			ContentTextBlock->Visibility = Windows::UI::Xaml::Visibility::Visible;
			ContentTextBlock->Focus(Windows::UI::Xaml::FocusState::Programmatic);
		}
		else
		{
			ContentTextBlockPhoneHor->Text = ContentTBPhoneHor->Text;
			ContentTBPhoneHor->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
			ContentTextBlockPhoneHor->Visibility = Windows::UI::Xaml::Visibility::Visible;
			ContentTextBlockPhoneHor->Focus(Windows::UI::Xaml::FocusState::Programmatic);
		}
	}
}


void AnnotNoteUserControl::OnManipulationDelta(Platform::Object ^sender, Windows::UI::Xaml::Input::ManipulationDeltaRoutedEventArgs ^e)
{
	UIExtensionsManager^ uiMgr = (UIExtensionsManager^)View->ExtensionsManager;
	auto winrc = Windows::UI::Xaml::Window::Current->Bounds;
	double verticalOffset = notePt.Y + 50;
	double deltaX = e->Delta.Translation.X;
	double translateX = m_LayoutGridTransform->TranslateX + deltaX;
	double deltaY = e->Delta.Translation.Y;
	double translateY = m_LayoutGridTransform->TranslateY + deltaY;

	if (View)
	{
		Point pt = uiMgr->GetAnnotPopupOffset();
		if (deltaX != 0)
		{
			if (deltaX > 0 && pt.X + translateX + 405 > winrc.Width)
			{
				return;
			}
			if (deltaX < 0 && pt.X + translateX < 0)
			{
				return;
			}
		}
		if (deltaY != 0)
		{
			if (deltaY > 0 && pt.Y + translateY + 234> winrc.Height)
			{
				return;
			}
			if (deltaY < 0 && pt.Y + translateY < 0)
			{
				return;
			}
		}
	}

	m_LayoutGridTransform->TranslateX += e->Delta.Translation.X;
	m_LayoutGridTransform->TranslateY += e->Delta.Translation.Y;
}


void AnnotNoteUserControl::CreateBtn_PointerEntered(Platform::Object^ sender, Windows::UI::Xaml::Input::PointerRoutedEventArgs^ e)
{
	if (View && GetIsDesktopPlatform())
	{
		this->ManipulationMode = ::ManipulationModes::None;
	}
}


void AnnotNoteUserControl::CreateBtn_PointerExited(Platform::Object^ sender, Windows::UI::Xaml::Input::PointerRoutedEventArgs^ e)
{
	if (View && GetIsDesktopPlatform())
	{
		this->ManipulationMode = ::ManipulationModes::All;
	}
}


void AnnotNoteUserControl::ContentTextBlockPhoneHor_LostFocus(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{
	OutputDebugString(L"ContentTextBlock_LostFocus\n");
	auto inputPane = Windows::UI::ViewManagement::InputPane::GetForCurrentView();
	inputPane->TryHide();

	ContentTBPhoneHor->Text = ContentTextBlockPhoneHor->Text;
	ContentTBPhoneHor->Visibility = Windows::UI::Xaml::Visibility::Visible;
	ContentTextBlockPhoneHor->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
	selectLength = 0;
}


void AnnotNoteUserControl::ContentTextBlockPhoneHor_GotFocus(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{
	if (contentLenghtTimer->IsEnabled)
		contentLenghtTimer->Stop();
	contentLenghtTimer->Start();

	UIExtensionsManager^ uiMgr = (UIExtensionsManager^)View->ExtensionsManager;
	auto _AnnotMgr = uiMgr->getAnnotMgr();
	if (_AnnotMgr)
	{
		auto _selectAnnot = _AnnotMgr->GetSelectedAnnot();
		if (ContentTextBlockPhoneHor->Text->Length() > 0 && ContentTextBlockPhoneHor->Text != " "
			|| (_selectAnnot && (_selectAnnot->AnnotType == AnnotationType::Text || _selectAnnot->AnnotType == AnnotationType::Caret)))
		{
			isCreatePopupContent = false;

			ContentTextBlockPhoneHor->SelectionStart = ContentTextBlockPhoneHor->Text->Length();
			CreateBtnPhoneHor->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
			DeleteBtnPhoneHor->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
			CloseBtnPhoneHor->Visibility = Windows::UI::Xaml::Visibility::Collapsed;

			SaveBtnPhoneHor->Visibility = Windows::UI::Xaml::Visibility::Visible;
			CancelBtnPhoneHor->Visibility = Windows::UI::Xaml::Visibility::Visible;
		}
		else
		{
			if (!m_isTextAnnotDelete)
			{
				ContentTextBlockPhoneHor->SelectionStart = 0;
				CreateBtnPhoneHor->Visibility = Windows::UI::Xaml::Visibility::Visible;
				DeleteBtnPhoneHor->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
				CloseBtnPhoneHor->Visibility = Windows::UI::Xaml::Visibility::Visible;

				SaveBtnPhoneHor->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
				CancelBtnPhoneHor->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
			}
		}
	}
}


void AnnotNoteUserControl::OnSizeChanged(Platform::Object ^sender, Windows::UI::Core::WindowSizeChangedEventArgs ^e)
{
}

void AnnotNoteUserControl::SetUserControlUpdate()
{
	UIExtensionsManager^ uiMgr = (UIExtensionsManager^)View->ExtensionsManager;
	if (View)
	{
		auto _AnnotMgr = uiMgr->getAnnotMgr();
		if (_AnnotMgr)
		{
			auto _selectAnnot = _AnnotMgr->GetSelectedAnnot();
			if (_selectAnnot)
			{

				TextAnnot^ textAnnot = nullptr;
				if (_selectAnnot->AnnotType == AnnotationType::Text)
				{
					textAnnot = dynamic_cast<TextAnnot^>(_selectAnnot);
				}
				else if (_selectAnnot->AnnotType == AnnotationType::StrikeOut)
				{
					StrikeOutAnnot^ strikeoutannot = dynamic_cast<StrikeOutAnnot^>(uiMgr->getAnnotMgr()->GetSelectedAnnot());
					if (strikeoutannot->IsHasGroup)
					{
						if (uiMgr->getAnnotMgr()->IsGetGroupAnnotElement(strikeoutannot->PageIndex, strikeoutannot->GroupAnnotMinRect,
							strikeoutannot->GroupAnnotIndex, AnnotationType::Caret))
						{
							_selectAnnot = uiMgr->getAnnotMgr()->GetGroupAnnot();
						}
					}
				}

				if (textAnnot && textAnnot->ReplyCount > 0)
				{
					TextAnnot^ textAnnot = dynamic_cast<TextAnnot^>(_selectAnnot);
					replyListView->ItemsSource = textAnnot->ReplyAnnotGroup;

					AuthorTextBlock1->Text = _selectAnnot->Title;
					ContentTB1->Text = _selectAnnot->Contents;
					TimeTextBlock1->Text = GetDateTimeStr(_selectAnnot->CreationDateTime);
					GridHasReply->Visibility = Windows::UI::Xaml::Visibility::Visible;
					GridHasNoReply->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
					ReplyCountTextBlock->Text = textAnnot->ReplyCount.ToString() + " " + "Replies";
				}
				else
				{
					if (!isPhoneHorizontalMode)
					{
						GridHasReply->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
						GridHasNoReplyPhoneHor->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
						GridHasNoReply->Visibility = Windows::UI::Xaml::Visibility::Visible;
						AuthorTextBlock->Text = _selectAnnot->Title;
						ContentTextBlock->Text = _selectAnnot->Contents;
						TimeTextBlock->Text = GetDateTimeStr(_selectAnnot->CreationDateTime);
						ContentTB->Text = _selectAnnot->Contents;
						if (!FSPDFDocUtil::GetIsDocumentCanModifyAnnotation(View->GetDoc()) && View->GetDoc()->GetPasswordType() != foxit::pdf::PasswordType::e_PwdOwner)
						{
							CreateBtn->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
							CloseBtn->Visibility = Windows::UI::Xaml::Visibility::Visible;
							ContentTB->Visibility = Windows::UI::Xaml::Visibility::Visible;
							ContentTextBlock->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
						}
						else
						{
							if (_selectAnnot->Contents->Length() > 0 && _selectAnnot->Contents != " "
								|| _selectAnnot->AnnotType == AnnotationType::Text || _selectAnnot->AnnotType == AnnotationType::Caret)
							{
								CreateBtn->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
								DeleteBtn->Visibility = Windows::UI::Xaml::Visibility::Visible;
							}
							else
							{
								CreateBtn->Visibility = Windows::UI::Xaml::Visibility::Visible;
								DeleteBtn->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
								ContentTextBlock->Focus(Windows::UI::Xaml::FocusState::Programmatic);
							}
						}
					}
					else
					{
						GridHasReply->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
						GridHasNoReplyPhoneHor->Visibility = Windows::UI::Xaml::Visibility::Visible;
						GridHasNoReply->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
						AuthorTextBlockPhoneHor->Text = _selectAnnot->Title;
						ContentTextBlockPhoneHor->Text = _selectAnnot->Contents;
						TimeTextBlockPhoneHor->Text = GetDateTimeStr(_selectAnnot->CreationDateTime);
						ContentTBPhoneHor->Text = _selectAnnot->Contents;
						if (!FSPDFDocUtil::GetIsDocumentCanModifyAnnotation(View->GetDoc()) && View->GetDoc()->GetPasswordType() != foxit::pdf::PasswordType::e_PwdOwner)
						{
							CreateBtnPhoneHor->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
							CloseBtnPhoneHor->Visibility = Windows::UI::Xaml::Visibility::Visible;
							ContentTBPhoneHor->Visibility = Windows::UI::Xaml::Visibility::Visible;
							ContentTextBlockPhoneHor->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
						}
						else
						{
							if (_selectAnnot->Contents->Length() > 0 && _selectAnnot->Contents != " "
								|| _selectAnnot->AnnotType == AnnotationType::Text || _selectAnnot->AnnotType == AnnotationType::Caret)
							{
								CreateBtnPhoneHor->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
								DeleteBtnPhoneHor->Visibility = Windows::UI::Xaml::Visibility::Visible;
							}
							else
							{
								CreateBtnPhoneHor->Visibility = Windows::UI::Xaml::Visibility::Visible;
								DeleteBtnPhoneHor->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
								ContentTextBlockPhoneHor->Focus(Windows::UI::Xaml::FocusState::Programmatic);
							}
						}
					}
				}
			}
			else
			{
				if (!isPhoneHorizontalMode)
				{
					AnnotCommandToken^ _Token = dynamic_cast<AnnotCommandToken^>(this->Tag);
					if (_Token && _Token->Type == AnnotCommandType::Add)
					{
						auto _pCreateAnnot = dynamic_cast<foxit::rdk::uiextensions::IBaseAnnot^>(_Token->Param);
						AuthorTextBlock->Text = _pCreateAnnot->Title;
						auto _Calendar = ref new Windows::Globalization::Calendar();
						TimeTextBlock->Text = GetDateTimeStr(_Calendar->GetDateTime());
					}
					CreateBtn->Visibility = Windows::UI::Xaml::Visibility::Visible;
					DeleteBtn->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
					ContentTextBlock->Visibility = Windows::UI::Xaml::Visibility::Visible;
					ContentTextBlock->Focus(Windows::UI::Xaml::FocusState::Programmatic);
				}
				else
				{
					GridHasNoReplyPhoneHor->Visibility = Windows::UI::Xaml::Visibility::Visible;
					GridHasNoReply->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
					GridHasReply->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
					AnnotCommandToken^ _Token = dynamic_cast<AnnotCommandToken^>(this->Tag);
					if (_Token && _Token->Type == AnnotCommandType::Add)
					{
						auto _pCreateAnnot = dynamic_cast<foxit::rdk::uiextensions::IBaseAnnot^>(_Token->Param);
						AuthorTextBlockPhoneHor->Text = _pCreateAnnot->Title;
						auto _Calendar = ref new Windows::Globalization::Calendar();
						TimeTextBlockPhoneHor->Text = GetDateTimeStr(_Calendar->GetDateTime());
					}
					CreateBtnPhoneHor->Visibility = Windows::UI::Xaml::Visibility::Visible;
					DeleteBtnPhoneHor->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
					ContentTextBlockPhoneHor->Visibility = Windows::UI::Xaml::Visibility::Visible;
					ContentTextBlockPhoneHor->Focus(Windows::UI::Xaml::FocusState::Programmatic);
				}
			}
		}
	}
}

Windows::Foundation::Point AnnotNoteUserControl::CreateAnnotNote()
{
	UIExtensionsManager^ uiMgr = (UIExtensionsManager^)View->ExtensionsManager;
	Point noteRectPt(0, 0);

	if (contentLenghtTimer->IsEnabled)
		contentLenghtTimer->Stop();

	m_isCreateNewNote = true;

	auto _popup = GetVisualParent<Windows::UI::Xaml::Controls::Primitives::Popup^>(this);
	auto _selectAnnot = uiMgr->getAnnotMgr()->GetSelectedAnnot();
	Platform::String^ noteContent = "";
	if (isPhoneHorizontalMode)
	{
		noteContent = ContentTextBlockPhoneHor->Text;
	}
	else
	{
		noteContent = ContentTextBlock->Text;
	}

	if (_selectAnnot && (_selectAnnot->AnnotType == AnnotationType::HightLight
		|| _selectAnnot->AnnotType == AnnotationType::StrikeOut || _selectAnnot->AnnotType == AnnotationType::UnderLine
		|| _selectAnnot->AnnotType == AnnotationType::Squiggly || _selectAnnot->AnnotType == AnnotationType::Ink || _selectAnnot->AnnotType == AnnotationType::Square
		|| _selectAnnot->AnnotType == AnnotationType::Circle
		|| _selectAnnot->AnnotType == AnnotationType::Line))
	{
		_selectAnnot->Contents = noteContent;
		_selectAnnot->IsModifyProperty = true;
		uiMgr->getAnnotMgr()->SetPdfSelectAnnotNull();
		uiMgr->SetAnnotCanvasVisible(false);
		View->Update();
	}
	else
	{
		AnnotCommandToken^ _Token = dynamic_cast<AnnotCommandToken^>(this->Tag);
		if (_Token == nullptr)
			return noteRectPt;

		if (_Token->Type != AnnotCommandType::Add)
			return noteRectPt;

		auto _page = View->GetDoc()->GetPage(_Token->Pageindex);
		auto _createtype = uiMgr->CreateTypeOfAnnot;

		if (_createtype == AnnotCreateType::Text)
		{
			auto _pCreateAnnot = dynamic_cast<foxit::rdk::uiextensions::TextAnnot^>(_Token->Param);

			if (noteContent->Length() == 0)
			{
				if (_popup)
					_popup->IsOpen = false;
				uiMgr->CloseAddNoteWindows();
				return noteRectPt;
			}

			_pCreateAnnot->Contents = noteContent;
			_pCreateAnnot->CreationDateTime = (ref new Windows::Globalization::Calendar())->GetDateTime();

			uiMgr->getAnnotMgr()->OnCreateNewAnnot(_page, _pCreateAnnot);
			auto pUndoItem = ref new CreateTextAnnotUndoAction(View, _pCreateAnnot);
			UndoFactory::GetUndoInstance()->AddUndoItem(uiMgr, pUndoItem);

			Windows::Foundation::Rect rc = _pCreateAnnot->MinRect;
			foxit::common::fxcrt::RectF^ minRect = FoxitRectConverter::ConvertToFoxitRectF(_pCreateAnnot->MinRect);
			String^ rectStr = "[" + minRect->left.ToString() + " " + minRect->top.ToString() + " " + minRect->right.ToString() + " " + minRect->bottom.ToString() + "]";
			uiMgr->CollectAnnotOperation("Note", "Add", _Token->Pageindex.ToString(), rectStr);

			noteRectPt = Point(_pCreateAnnot->MinRect.X, _pCreateAnnot->MinRect.Y);

			if (!uiMgr->getAnnotMgr()->GetAnnotSetting()->GetTextIsContinueOpera() || uiMgr->IsRightMenuAdd)
			{
				uiMgr->CreateTypeOfAnnot = AnnotCreateType::None;
				uiMgr->ExitAnnotPropertyMode();
				uiMgr->IsRightMenuAdd = false;
			}
		}
		else if (_createtype == AnnotCreateType::Caret)
		{
			auto _pCreateAnnot = dynamic_cast<foxit::rdk::uiextensions::CaretAnnot^>(_Token->Param);

			if (noteContent->Length() == 0)
			{
				if (_popup)
					_popup->IsOpen = false;
				uiMgr->CloseAddNoteWindows();
				return noteRectPt;
			}
			_pCreateAnnot->Contents = noteContent;
			_pCreateAnnot->CreationDateTime = (ref new Windows::Globalization::Calendar())->GetDateTime();

			uiMgr->getAnnotMgr()->OnCreateNewAnnot(_page, _pCreateAnnot);
			auto pUndoItem = ref new CreateCaretAnnotUndoAction(View, _pCreateAnnot);
			UndoFactory::GetUndoInstance()->AddUndoItem(uiMgr, pUndoItem);

			Windows::Foundation::Rect rc = _pCreateAnnot->MinRect;
			foxit::common::fxcrt::RectF^ minRect = FoxitRectConverter::ConvertToFoxitRectF(_pCreateAnnot->MinRect);
			String^ rectStr = "[" + minRect->left.ToString() + " " + minRect->top.ToString() + " " + minRect->right.ToString() + " " + minRect->bottom.ToString() + "]";
			uiMgr->CollectAnnotOperation("Insert Text", "Add", _Token->Pageindex.ToString(), rectStr);

			noteRectPt = Point(_pCreateAnnot->MinRect.X, _pCreateAnnot->MinRect.Y);

			if (!uiMgr->getAnnotMgr()->GetAnnotSetting()->GetInsertTextIsContinueOpera())
			{
				uiMgr->CreateTypeOfAnnot = AnnotCreateType::None;
				uiMgr->ExitAnnotPropertyMode();
			}
		}
		else if (_createtype == AnnotCreateType::ReplaceText)
		{
			auto _pCreateAnnot = dynamic_cast<foxit::rdk::uiextensions::CaretAnnot^>(_Token->Param);
			auto _pCreateStrikeoutAnnot = dynamic_cast<foxit::rdk::uiextensions::StrikeOutAnnot^>(_Token->StrickoutParam);

			if (noteContent->Length() == 0)
			{
				if (_popup)
					_popup->IsOpen = false;
				uiMgr->CloseAddNoteWindows();
				return noteRectPt;
			}
			_pCreateStrikeoutAnnot->Contents = noteContent;
			_pCreateAnnot->Contents = noteContent;
			_pCreateAnnot->CreationDateTime = (ref new Windows::Globalization::Calendar())->GetDateTime();

			Windows::Foundation::Collections::IVector<IBaseAnnot^>^ newannots = ref new Platform::Collections::Vector<IBaseAnnot^>();
			newannots->Append(_pCreateStrikeoutAnnot);
			newannots->Append(_pCreateAnnot);
			uiMgr->getAnnotMgr()->OnCreateNewAnnotGroup(_page, newannots);

			auto pUndoItem = ref new CreateReplaceTextAnnotUndoAction(View, newannots);
			UndoFactory::GetUndoInstance()->AddUndoItem(uiMgr, pUndoItem);

			Windows::Foundation::Rect rc = _pCreateAnnot->MinRect;
			foxit::common::fxcrt::RectF^ minRect = FoxitRectConverter::ConvertToFoxitRectF(_pCreateAnnot->MinRect);
			String^ rectStr = "[" + minRect->left.ToString() + " " + minRect->top.ToString() + " " + minRect->right.ToString() + " " + minRect->bottom.ToString() + "]";
			uiMgr->CollectAnnotOperation("Replace Text", "Add", _Token->Pageindex.ToString(), rectStr);

			noteRectPt = Point(_pCreateAnnot->MinRect.X, _pCreateAnnot->MinRect.Y);
			
			if (!uiMgr->getAnnotMgr()->GetAnnotSetting()->GetReplaceTextIsContinueOpera())
			{
				uiMgr->CreateTypeOfAnnot = AnnotCreateType::None;
				uiMgr->ExitAnnotPropertyMode();
			}
		}

		auto _selectedtext = uiMgr->getTextMgr()->SelectedText;
		if (_selectedtext && _selectedtext->Size > 0)
		{
			uiMgr->getTextMgr()->ClearTextSelect();
		}
	}
	if (_popup)
		_popup->IsOpen = false;
	uiMgr->CloseAddNoteWindows();


	return noteRectPt;
}

void AnnotNoteUserControl::SaveAnnotNote()
{
	UIExtensionsManager^ uiMgr = (UIExtensionsManager^)View->ExtensionsManager;
	if (!isPhoneHorizontalMode)
	{
		auto _AnnotMgr = uiMgr->getAnnotMgr();
		if (_AnnotMgr)
		{
			auto _selectAnnot = _AnnotMgr->GetSelectedAnnot();
			if (_selectAnnot)
			{
				String^ contentStr = ContentTextBlock->Text;

				if (_selectAnnot->AnnotType == AnnotationType::StrikeOut)
				{
					StrikeOutAnnot^ strikeoutannot = dynamic_cast<StrikeOutAnnot^>(uiMgr->getAnnotMgr()->GetSelectedAnnot());
					if (strikeoutannot->IsHasGroup)
					{
						if (uiMgr->getAnnotMgr()->IsGetGroupAnnotElement(strikeoutannot->PageIndex, strikeoutannot->GroupAnnotMinRect,
							strikeoutannot->GroupAnnotIndex, AnnotationType::Caret))
						{
							IBaseAnnot^ annot = uiMgr->getAnnotMgr()->GetGroupAnnot();
							if (contentStr != annot->Contents)
							{
								annot->Contents = ContentTextBlock->Text;
								_selectAnnot->IsModifyProperty = true;
							}
						}
					}
					else
					{
						if (ContentTextBlock->Text != _selectAnnot->Contents)
						{
							_selectAnnot->Contents = ContentTextBlock->Text;
							_selectAnnot->IsModifyProperty = true;
						}
					}
				}
				else
				{
					if (ContentTextBlock->Text != _selectAnnot->Contents)
					{
						_selectAnnot->Contents = ContentTextBlock->Text;
						_selectAnnot->IsModifyProperty = true;
					}
				}
			}
		}
		DeleteBtn->Visibility = Windows::UI::Xaml::Visibility::Visible;
		CloseBtn->Visibility = Windows::UI::Xaml::Visibility::Visible;

		SaveBtn->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
		CancelBtn->Visibility = Windows::UI::Xaml::Visibility::Collapsed;


		AuthorTextBlock->Focus(Windows::UI::Xaml::FocusState::Programmatic);
	}
	else
	{
		auto _AnnotMgr = uiMgr->getAnnotMgr();
		if (_AnnotMgr)
		{
			auto _selectAnnot = _AnnotMgr->GetSelectedAnnot();
			if (_selectAnnot)
			{
				if (_selectAnnot->AnnotType == AnnotationType::StrikeOut)
				{
					StrikeOutAnnot^ strikeoutannot = dynamic_cast<StrikeOutAnnot^>(uiMgr->getAnnotMgr()->GetSelectedAnnot());
					if (strikeoutannot->IsHasGroup)
					{
						if (uiMgr->getAnnotMgr()->IsGetGroupAnnotElement(strikeoutannot->PageIndex, strikeoutannot->GroupAnnotMinRect,
							strikeoutannot->GroupAnnotIndex, AnnotationType::Caret))
						{
							IBaseAnnot^ annot = uiMgr->getAnnotMgr()->GetGroupAnnot();
							if (ContentTextBlockPhoneHor->Text != annot->Contents)
							{
								annot->Contents = ContentTextBlockPhoneHor->Text;
								_selectAnnot->IsModifyProperty = true;
							}
						}
					}
					else
					{
						if (ContentTextBlockPhoneHor->Text != _selectAnnot->Contents)
						{
							_selectAnnot->Contents = ContentTextBlockPhoneHor->Text;
							_selectAnnot->IsModifyProperty = true;
						}
					}
				}
				else
				{
					if (ContentTextBlockPhoneHor->Text != _selectAnnot->Contents)
					{
						_selectAnnot->Contents = ContentTextBlockPhoneHor->Text;
						_selectAnnot->IsModifyProperty = true;
					}
				}
			}
		}
		DeleteBtnPhoneHor->Visibility = Windows::UI::Xaml::Visibility::Visible;
		CloseBtnPhoneHor->Visibility = Windows::UI::Xaml::Visibility::Visible;

		SaveBtnPhoneHor->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
		CancelBtnPhoneHor->Visibility = Windows::UI::Xaml::Visibility::Collapsed;

		ContentTextBlockPhoneHor->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
		ContentTBPhoneHor->Visibility = Windows::UI::Xaml::Visibility::Visible;
		TimeTextBlockPhoneHor->Focus(Windows::UI::Xaml::FocusState::Programmatic);
	}
}

void AnnotNoteUserControl::CreateNewNote()
{
	UIExtensionsManager^ uiMgr = (UIExtensionsManager^)View->ExtensionsManager;
	if (contentLenghtTimer->IsEnabled)
		contentLenghtTimer->Stop();

	m_isCreateNewNote = true;

	auto _popup = GetVisualParent<Windows::UI::Xaml::Controls::Primitives::Popup^>(this);
	auto _selectAnnot = uiMgr->getAnnotMgr()->GetSelectedAnnot();
	Platform::String^ noteContent = "";
	if (isPhoneHorizontalMode)
	{
		noteContent = ContentTextBlockPhoneHor->Text;
	}
	else
	{
		noteContent = ContentTextBlock->Text;
	}

	if (_selectAnnot && (_selectAnnot->AnnotType == AnnotationType::HightLight
		|| _selectAnnot->AnnotType == AnnotationType::StrikeOut || _selectAnnot->AnnotType == AnnotationType::UnderLine
		|| _selectAnnot->AnnotType == AnnotationType::Squiggly || _selectAnnot->AnnotType == AnnotationType::Ink || _selectAnnot->AnnotType == AnnotationType::Square
		|| _selectAnnot->AnnotType == AnnotationType::Circle
		|| _selectAnnot->AnnotType == AnnotationType::Line))
	{
		_selectAnnot->Contents = noteContent;
		_selectAnnot->IsModifyProperty = true;
		uiMgr->getAnnotMgr()->SetPdfSelectAnnotNull();
		uiMgr->SetAnnotCanvasVisible(false);
		View->Update();
	}
	else
	{
		AnnotCommandToken^ _Token = dynamic_cast<AnnotCommandToken^>(this->Tag);
		if (_Token == nullptr)
			return;

		if (_Token->Type != AnnotCommandType::Add)
			return;

		auto _page = View->GetDoc()->GetPage(_Token->Pageindex);
		auto _createtype = uiMgr->CreateTypeOfAnnot;

		if (_createtype == AnnotCreateType::Text)
		{
			auto _pCreateAnnot = dynamic_cast<foxit::rdk::uiextensions::TextAnnot^>(_Token->Param);

			if (noteContent->Length() == 0)
			{
				if (_popup)
					_popup->IsOpen = false;
				uiMgr->CloseAddNoteWindows();
				return;
			}

			_pCreateAnnot->Contents = noteContent;
			_pCreateAnnot->CreationDateTime = (ref new Windows::Globalization::Calendar())->GetDateTime();

			uiMgr->getAnnotMgr()->OnCreateNewAnnot(_page, _pCreateAnnot);
			auto pUndoItem = ref new CreateTextAnnotUndoAction(View, _pCreateAnnot);
			UndoFactory::GetUndoInstance()->AddUndoItem(uiMgr, pUndoItem);

			Windows::Foundation::Rect rc = _pCreateAnnot->MinRect;
			foxit::common::fxcrt::RectF^ minRect = FoxitRectConverter::ConvertToFoxitRectF(_pCreateAnnot->MinRect);
			String^ rectStr = "[" + minRect->left.ToString() + " " + minRect->top.ToString() + " " + minRect->right.ToString() + " " + minRect->bottom.ToString() + "]";
			uiMgr->CollectAnnotOperation("Note", "Add", _Token->Pageindex.ToString(), rectStr);

			if (!uiMgr->getAnnotMgr()->GetAnnotSetting()->GetTextIsContinueOpera())
			{
				uiMgr->CreateTypeOfAnnot = AnnotCreateType::None;
				uiMgr->ExitAnnotPropertyMode();
			}
		}
		else if (_createtype == AnnotCreateType::Caret)
		{
			if (noteContent->Length() > 0)
			{
				auto _pCreateAnnot = dynamic_cast<foxit::rdk::uiextensions::CaretAnnot^>(_Token->Param);

				_pCreateAnnot->Contents = noteContent;
				_pCreateAnnot->CreationDateTime = (ref new Windows::Globalization::Calendar())->GetDateTime();

				uiMgr->getAnnotMgr()->OnCreateNewAnnot(_page, _pCreateAnnot);
				auto pUndoItem = ref new CreateCaretAnnotUndoAction(View, _pCreateAnnot);
				UndoFactory::GetUndoInstance()->AddUndoItem(uiMgr, pUndoItem);

				Windows::Foundation::Rect rc = _pCreateAnnot->MinRect;
				foxit::common::fxcrt::RectF^ minRect = FoxitRectConverter::ConvertToFoxitRectF(_pCreateAnnot->MinRect);
				String^ rectStr = "[" + minRect->left.ToString() + " " + minRect->top.ToString() + " " + minRect->right.ToString() + " " + minRect->bottom.ToString() + "]";
				uiMgr->CollectAnnotOperation("Insert Text", "Add", _Token->Pageindex.ToString(), rectStr);
			}

			if (!uiMgr->getAnnotMgr()->GetAnnotSetting()->GetInsertTextIsContinueOpera())
			{
				uiMgr->CreateTypeOfAnnot = AnnotCreateType::None;
				uiMgr->ExitAnnotPropertyMode();
			}
		}
		else if (_createtype == AnnotCreateType::ReplaceText)
		{
			auto _pCreateAnnot = dynamic_cast<foxit::rdk::uiextensions::CaretAnnot^>(_Token->Param);
			auto _pCreateStrikeoutAnnot = dynamic_cast<foxit::rdk::uiextensions::StrikeOutAnnot^>(_Token->StrickoutParam);

			if (noteContent->Length() == 0)
			{
				if (_popup)
					_popup->IsOpen = false;
				uiMgr->CloseAddNoteWindows();
				return;
			}
			_pCreateAnnot->Contents = noteContent;
			_pCreateAnnot->CreationDateTime = (ref new Windows::Globalization::Calendar())->GetDateTime();


			Windows::Foundation::Collections::IVector<IBaseAnnot^>^ newannots = ref new Platform::Collections::Vector<IBaseAnnot^>();
			newannots->Append(_pCreateStrikeoutAnnot);
			newannots->Append(_pCreateAnnot);
			uiMgr->getAnnotMgr()->OnCreateNewAnnotGroup(_page, newannots);

			auto pUndoItem = ref new CreateReplaceTextAnnotUndoAction(View, newannots);
			UndoFactory::GetUndoInstance()->AddUndoItem(uiMgr, pUndoItem);

			Windows::Foundation::Rect rc = _pCreateAnnot->MinRect;
			foxit::common::fxcrt::RectF^ minRect = FoxitRectConverter::ConvertToFoxitRectF(_pCreateAnnot->MinRect);
			String^ rectStr = "[" + minRect->left.ToString() + " " + minRect->top.ToString() + " " + minRect->right.ToString() + " " + minRect->bottom.ToString() + "]";
			uiMgr->CollectAnnotOperation("Replace Text", "Add", _Token->Pageindex.ToString(), rectStr);

			if (!uiMgr->getAnnotMgr()->GetAnnotSetting()->GetReplaceTextIsContinueOpera())
			{
				uiMgr->CreateTypeOfAnnot = AnnotCreateType::None;
				uiMgr->ExitAnnotPropertyMode();
			}
		}


		auto _selectedtext = uiMgr->getTextMgr()->SelectedText;
		if (_selectedtext && _selectedtext->Size > 0)
		{
			uiMgr->getTextMgr()->ClearTextSelect();
		}
	}
	if (_popup)
		_popup->IsOpen = false;
	uiMgr->CloseAddNoteWindows();
}

bool AnnotNoteUserControl::IsSelectText()
{
	bool isSelect = false;
	if (!isPhoneHorizontalMode)
	{
		if (selectLength > 0)
		{
			isSelect = true;
		}
	}
	return isSelect;
}

void foxit::rdk::uiextensions::AnnotNoteUserControl::ContentTextBlock_SelectionChanged(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{
	selectLength = ContentTextBlock->SelectionLength;
	String^ length = ContentTextBlock->SelectionLength.ToString();
	OutputDebugString(length->Data());
	OutputDebugString(L"\n");
}

void foxit::rdk::uiextensions::AnnotNoteUserControl::CreateInsertAnnot()
{
	UIExtensionsManager^ uiMgr = (UIExtensionsManager^)View->ExtensionsManager;
	Platform::String^ noteContent = "";
	if (isPhoneHorizontalMode)
	{
		noteContent = ContentTextBlockPhoneHor->Text;
	}
	else
	{
		noteContent = ContentTextBlock->Text;
	}

	AnnotCommandToken^ _Token = dynamic_cast<AnnotCommandToken^>(this->Tag);
	if (_Token == nullptr)
		return;

	if (_Token->Type != AnnotCommandType::Add)
		return;

	auto _page = View->GetDoc()->GetPage(_Token->Pageindex);
	auto _createtype = uiMgr->CreateTypeOfAnnot;

	if (_createtype == AnnotCreateType::Caret)
	{
		auto _pCreateAnnot = dynamic_cast<foxit::rdk::uiextensions::CaretAnnot^>(_Token->Param);

		_pCreateAnnot->Contents = noteContent;
		_pCreateAnnot->CreationDateTime = (ref new Windows::Globalization::Calendar())->GetDateTime();

		uiMgr->getAnnotMgr()->OnCreateNewAnnot(_page, _pCreateAnnot);
		auto pUndoItem = ref new CreateCaretAnnotUndoAction(View, _pCreateAnnot);
		UndoFactory::GetUndoInstance()->AddUndoItem(uiMgr, pUndoItem);

		Windows::Foundation::Rect rc = _pCreateAnnot->MinRect;
		foxit::common::fxcrt::RectF^ minRect = FoxitRectConverter::ConvertToFoxitRectF(_pCreateAnnot->MinRect);
		String^ rectStr = "[" + minRect->left.ToString() + " " + minRect->top.ToString() + " " + minRect->right.ToString() + " " + minRect->bottom.ToString() + "]";
		uiMgr->CollectAnnotOperation("Insert Text", "Add", _Token->Pageindex.ToString(), rectStr);

		if (!uiMgr->getAnnotMgr()->GetAnnotSetting()->GetInsertTextIsContinueOpera())
		{
			uiMgr->CreateTypeOfAnnot = AnnotCreateType::None;
			uiMgr->ExitAnnotPropertyMode();
		}
	}
	else if (_createtype == AnnotCreateType::ReplaceText)
	{
		auto _pCreateAnnot = dynamic_cast<foxit::rdk::uiextensions::CaretAnnot^>(_Token->Param);
		auto _pCreateStrikeoutAnnot = dynamic_cast<foxit::rdk::uiextensions::StrikeOutAnnot^>(_Token->StrickoutParam);

		_pCreateAnnot->Contents = noteContent;
		_pCreateAnnot->CreationDateTime = (ref new Windows::Globalization::Calendar())->GetDateTime();


		Windows::Foundation::Collections::IVector<IBaseAnnot^>^ newannots = ref new Platform::Collections::Vector<IBaseAnnot^>();
		newannots->Append(_pCreateStrikeoutAnnot);
		newannots->Append(_pCreateAnnot);
		uiMgr->getAnnotMgr()->OnCreateNewAnnotGroup(_page, newannots);

		auto pUndoItem = ref new CreateReplaceTextAnnotUndoAction(View, newannots);
		UndoFactory::GetUndoInstance()->AddUndoItem(uiMgr, pUndoItem);

		Windows::Foundation::Rect rc = _pCreateAnnot->MinRect;
		foxit::common::fxcrt::RectF^ minRect = FoxitRectConverter::ConvertToFoxitRectF(_pCreateAnnot->MinRect);
		String^ rectStr = "[" + minRect->left.ToString() + " " + minRect->top.ToString() + " " + minRect->right.ToString() + " " + minRect->bottom.ToString() + "]";
		uiMgr->CollectAnnotOperation("Replace Text", "Add", _Token->Pageindex.ToString(), rectStr);

		uiMgr->CreateTypeOfAnnot = AnnotCreateType::None;
		uiMgr->ExitAnnotPropertyMode();
	}

	auto _selectedtext = uiMgr->getTextMgr()->SelectedText;
	if (_selectedtext && _selectedtext->Size > 0)
	{
		uiMgr->getTextMgr()->ClearTextSelect();
	}
}