﻿/**
 * Copyright (C) 2003-2018, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit Mobile PDF SDK to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */


#include "pch.h"
#include "AnnotOperatingOverlayControl.xaml.h"
#include "AnnotImp/AnnotManager.h"

using namespace foxit::rdk;
using namespace foxit::rdk::uiextensions;

using namespace Platform;
using namespace Windows::Foundation;
using namespace Windows::Foundation::Collections;
using namespace Windows::UI::Xaml;
using namespace Windows::UI::Xaml::Controls;
using namespace Windows::UI::Xaml::Controls::Primitives;
using namespace Windows::UI::Xaml::Data;
using namespace Windows::UI::Xaml::Input;
using namespace Windows::UI::Xaml::Media;
using namespace Windows::UI::Xaml::Navigation;

AnnotOperatingBindData::AnnotOperatingBindData()
{

}

float64 AnnotOperatingBindData::CropOverlayLeft::get()
{
	return m_cropOverlayLeft;
}

float64 AnnotOperatingBindData::CropOverlayTop::get()
{
	return m_cropOverlayTop;
}

float64 AnnotOperatingBindData::CropOverlayHeight::get()
{
	return m_cropOverlayHeight;
}

float64 AnnotOperatingBindData::CropOverlayWidth::get()
{
	return m_cropOverlayWidth;
}

Windows::UI::Color AnnotOperatingBindData::CropOverlayColor::get()
{
	return m_cropOverlayColor;
}

bool AnnotOperatingBindData::IsAnnotOperatingOverlayVisible::get()
{
	return m_isOperatingOverlayVisible;
}

void AnnotOperatingBindData::CalculateInitialOperatingOverlayPosition(Windows::UI::Xaml::Media::GeneralTransform^ transform, float width, float height)
{
	Point topLeft;
	topLeft.X = 0;
	topLeft.Y = 0;
	if (transform != nullptr)
	{
		topLeft = transform->TransformPoint(topLeft);
	}

	m_left = topLeft.X;
	m_top = topLeft.Y;
	m_bottom = m_top + height;
	m_right = m_left + width;
	m_cropOverlayTop = m_top;
	m_cropOverlayLeft = m_left;
	m_cropOverlayHeight = height;
	m_cropOverlayWidth = width;
	m_actualHeight = height;
	m_actualWidth = width;
	m_isOperatingOverlayVisible = true;

	OnPropertyChanged("CropOverlayLeft");
	OnPropertyChanged("CropOverlayTop");
	OnPropertyChanged("CropOverlayHeight");
	OnPropertyChanged("CropOverlayWidth");
	OnPropertyChanged("IsAnnotOperatingOverlayVisible");
}

void AnnotOperatingBindData::CalculateInitialOperatingOverlayPosition(Windows::Foundation::Rect originalRect, Windows::Foundation::Rect overlayRect, Windows::UI::Color color, bool isFreeTextAnnot)
{
	if (!isFreeTextAnnot)
	{
		m_left = overlayRect.X - 5;
		m_top = overlayRect.Y - 5;
		m_bottom = overlayRect.Bottom + 5;
		m_right = overlayRect.Right + 5;
	}


	m_cropOverlayLeft = (originalRect.X > overlayRect.Right) ? overlayRect.Right - 5 : originalRect.X - 5;
	m_cropOverlayTop = (originalRect.Y > overlayRect.Bottom) ? overlayRect.Bottom - 5 : originalRect.Y - 5;
	m_cropOverlayWidth = (originalRect.X > overlayRect.Right) ? 10 : originalRect.Width + 10;
	m_cropOverlayHeight = (originalRect.Y > overlayRect.Bottom) ? 10 : originalRect.Height + 10;

	if (m_cropOverlayTop < overlayRect.Y)
	{
		m_cropOverlayTop = overlayRect.Y;
	}

	if (m_cropOverlayTop + m_cropOverlayHeight > overlayRect.Y + overlayRect.Height)
	{
		double offset = m_cropOverlayTop + m_cropOverlayHeight - (overlayRect.Y + overlayRect.Height);
		m_cropOverlayHeight = m_cropOverlayHeight - offset;
	}
	if (m_cropOverlayLeft < overlayRect.X)
	{
		m_cropOverlayLeft = overlayRect.X;
	}
	if (m_cropOverlayLeft + m_cropOverlayWidth > overlayRect.X + overlayRect.Width)
	{
		double xoffset = m_cropOverlayLeft + m_cropOverlayWidth - (overlayRect.X + overlayRect.Width);
		m_cropOverlayWidth = m_cropOverlayWidth - xoffset;
	}

	m_actualWidth = overlayRect.Width;
	m_actualHeight = overlayRect.Height;
	m_cropOverlayColor = color;

	m_isOperatingOverlayVisible = true;

	OnPropertyChanged("CropOverlayLeft");
	OnPropertyChanged("CropOverlayTop");
	OnPropertyChanged("CropOverlayHeight");
	OnPropertyChanged("CropOverlayWidth");
	OnPropertyChanged("IsAnnotOperatingOverlayVisible");
	OnPropertyChanged("CropOverlayColor");
}

void AnnotOperatingBindData::UpdateOperatingOverlayPostion(Windows::UI::Xaml::Controls::Primitives::Thumb^ thumb, float64 verticalChange, float64 horizontalChange, float64 minWidth, float64 minHeight)
{
	if (thumb != nullptr)
	{
		float64 deltaH;
		float64 deltaV;
		float64 left;
		float64 top;

		switch (thumb->VerticalAlignment)
		{
		case Windows::UI::Xaml::VerticalAlignment::Bottom:
			deltaV = min(-verticalChange, m_cropOverlayHeight - minHeight);
			m_cropOverlayHeight -= deltaV;
			if (m_cropOverlayHeight + m_cropOverlayTop > m_bottom)
			{
				m_cropOverlayHeight = m_bottom - m_cropOverlayTop;
			}
			OnPropertyChanged("CropOverlayHeight");
			break;
		case Windows::UI::Xaml::VerticalAlignment::Top:
			deltaV = min(verticalChange, m_cropOverlayHeight - minHeight);
			top = m_cropOverlayTop + deltaV;
			if (m_top < top)
			{
				m_cropOverlayTop = top;
				m_cropOverlayHeight -= deltaV;
				if (m_cropOverlayHeight > m_actualHeight)
				{
					m_cropOverlayHeight = m_actualHeight;
				}
				OnPropertyChanged("CropOverlayTop");
				OnPropertyChanged("CropOverlayHeight");
			}
			break;
		default:
			break;
		}

		switch (thumb->HorizontalAlignment)
		{
		case Windows::UI::Xaml::HorizontalAlignment::Left:
			deltaH = min(horizontalChange, m_cropOverlayWidth - minWidth);
			left = m_cropOverlayLeft + deltaH;
			if (m_left < left)
			{
				m_cropOverlayLeft = left;
				m_cropOverlayWidth -= deltaH;
				if (m_cropOverlayWidth > m_actualWidth)
				{
					m_cropOverlayWidth = m_actualWidth;
				}
				OnPropertyChanged("CropOverlayLeft");
				OnPropertyChanged("CropOverlayWidth");
			}
			break;
		case Windows::UI::Xaml::HorizontalAlignment::Right:
			deltaH = min(-horizontalChange, m_cropOverlayWidth - minWidth);
			m_cropOverlayWidth -= deltaH;
			if (m_cropOverlayWidth + m_cropOverlayLeft > m_right)
			{
				m_cropOverlayWidth = m_right - m_cropOverlayLeft;
			}
			OnPropertyChanged("CropOverlayWidth");
			break;

		default:
			break;
		}
	}
}

void AnnotOperatingBindData::MoveOperatingOverlayPostion(float64 horizontalChange, float64 verticalChange)
{
	float64 deltaH = horizontalChange;
	float64 deltaV = verticalChange;
	float64 left;
	float64 top;

	left = m_cropOverlayLeft + deltaH;
	if (m_left < left)
	{
		m_cropOverlayLeft = left + m_cropOverlayWidth > m_right ? m_right - m_cropOverlayWidth : left;
	}
	else
	{
		m_cropOverlayLeft = m_left;
	}

	top = m_cropOverlayTop + deltaV;
	if (m_top < top)
	{
		m_cropOverlayTop = top + m_cropOverlayHeight > m_bottom ? m_bottom - m_cropOverlayHeight : top;
	}
	else
	{
		m_cropOverlayTop = m_top;
	}

	OnPropertyChanged("CropOverlayLeft");
	OnPropertyChanged("CropOverlayTop");
}

void AnnotOperatingBindData::ShowControlView()
{
	m_isOperatingOverlayVisible = true;

	OnPropertyChanged("IsAnnotOperatingOverlayVisible");
}

void AnnotOperatingBindData::HideControlView()
{
	m_isOperatingOverlayVisible = false;

	OnPropertyChanged("IsAnnotOperatingOverlayVisible");
}

// The User Control item template is documented at http://go.microsoft.com/fwlink/?LinkId=234236

AnnotOperatingOverlayControl::AnnotOperatingOverlayControl()
{
	InitializeComponent();

	OnInitializeControl();
}

void AnnotOperatingOverlayControl::OnInitializeControl()
{
	this->Loaded += ref new Windows::UI::Xaml::RoutedEventHandler(this, &foxit::rdk::uiextensions::AnnotOperatingOverlayControl::OnLoaded);
	this->Unloaded += ref new Windows::UI::Xaml::RoutedEventHandler(this, &foxit::rdk::uiextensions::AnnotOperatingOverlayControl::OnUnloaded);
}

void foxit::rdk::uiextensions::AnnotOperatingOverlayControl::OnLoaded(Platform::Object ^sender, Windows::UI::Xaml::RoutedEventArgs ^e)
{
	if (m_bindData == nullptr)
	{
		m_bindData = ref new AnnotOperatingBindData();
		this->DataContext = m_bindData;
	}

	m_LayoutGridTransform = ref new Windows::UI::Xaml::Media::CompositeTransform();
	m_LayoutGridTransform->TranslateX = 0;
	m_LayoutGridTransform->TranslateY = 0;
	gridCanvas->RenderTransform = m_LayoutGridTransform;

	gridNoResizeCanvas->RenderTransform = m_LayoutGridTransform;

	m_ismoveOutPage = false;
}


void foxit::rdk::uiextensions::AnnotOperatingOverlayControl::OnUnloaded(Platform::Object ^sender, Windows::UI::Xaml::RoutedEventArgs ^e)
{
	m_bindData = nullptr;
}

void foxit::rdk::uiextensions::AnnotOperatingOverlayControl::OnThumbDragDelta(Platform::Object^ sender, Windows::UI::Xaml::Controls::Primitives::DragDeltaEventArgs^ e)
{
	UIExtensionsManager^ uiMgr = (UIExtensionsManager^)View->ExtensionsManager;
	if (View->GetDoc() && !FSPDFDocUtil::GetIsDocumentCanModifyAnnotation(View->GetDoc()) && View->GetDoc()->GetPasswordType() != foxit::pdf::PasswordType::e_PwdOwner)
		return;
	uiMgr->SetPropertyPoupHide();

	gridCanvas->ManipulationMode = Windows::UI::Xaml::Input::ManipulationModes::None;
	gridCanvas->Background = ref new SolidColorBrush(Windows::UI::Colors::Gray);
	gridCanvas->Opacity = 0.5;

	if (m_bindData)
	{
		m_bindData->UpdateOperatingOverlayPostion(safe_cast<Windows::UI::Xaml::Controls::Primitives::Thumb^>(sender),
			e->VerticalChange,
			e->HorizontalChange,
			AnnotOperatingOverlay->MinWidth, AnnotOperatingOverlay->MinHeight);
	}
}


void foxit::rdk::uiextensions::AnnotOperatingOverlayControl::OnOperatingRectangleTapped(Platform::Object^ sender, Windows::UI::Xaml::Input::TappedRoutedEventArgs^ e)
{
	OutputDebugString(L"AnnotOperatingOverlayControl::OnOperatingRectangleTapped...\n");

}



void foxit::rdk::uiextensions::AnnotOperatingOverlayControl::OnOperatingManipulationDelta(Platform::Object^ sender, Windows::UI::Xaml::Input::ManipulationDeltaRoutedEventArgs^ e)
{
	UIExtensionsManager^ uiMgr = (UIExtensionsManager^)View->ExtensionsManager;
	if (uiMgr->getAnnotMgr()->GetSelectedAnnot() && uiMgr->getAnnotMgr()->GetSelectedAnnot()->AnnotType == AnnotationType::Text)
	{
		if (!uiMgr->ExitNotePopupEdit())
			return;
	}

	if (View->GetDoc() && !FSPDFDocUtil::GetIsDocumentCanModifyAnnotation(View->GetDoc()) && View->GetDoc()->GetPasswordType() != foxit::pdf::PasswordType::e_PwdOwner)
		return;

	if (e->IsInertial)
	{
		e->Complete();
		return;
	}
	if (m_isAnnotMoveEnd)
		return;

	Point pt = e->Position;

	double xvalue = e->Delta.Translation.X;
	double yvalue = e->Delta.Translation.Y;

	deltaX = deltaX + e->Delta.Translation.X;
	deltaY = deltaY + e->Delta.Translation.Y;

	auto _Delta = e->Delta;

	double culativeX = e->Cumulative.Translation.X;
	double culativeY = e->Cumulative.Translation.Y;

	Windows::Foundation::Rect deviceRect(annotMinRect.X + (float)culativeX, annotMinRect.Y + (float)culativeY, annotMinRect.Width, annotMinRect.Height);

	if (annotInPageRect.Contains(Point(deviceRect.X, deviceRect.Y)) && annotInPageRect.Contains(Point(deviceRect.X + deviceRect.Width, deviceRect.Y + deviceRect.Height)))
	{
		m_LayoutGridTransform->TranslateY = culativeY;
		m_LayoutGridTransform->TranslateX = culativeX;
	}
	else
	{
		deviceRect = Windows::Foundation::Rect(annotMinRect.X + (float)culativeX, annotMinRect.Y, annotMinRect.Width, annotMinRect.Height);
		bool iscontain1 = annotInPageRect.Contains(Point(deviceRect.X, deviceRect.Y));
		bool iscontain2 = annotInPageRect.Contains(Point(deviceRect.X + deviceRect.Width, deviceRect.Y + deviceRect.Height));
		if (iscontain1 && iscontain2)
		{
			m_LayoutGridTransform->TranslateX = culativeX;

			deviceRect = Windows::Foundation::Rect(annotMinRect.X, annotMinRect.Y + (float)culativeY, annotMinRect.Width, annotMinRect.Height);
			iscontain1 = annotInPageRect.Contains(Point(deviceRect.X, deviceRect.Y));
			iscontain2 = annotInPageRect.Contains(Point(deviceRect.X + deviceRect.Width, deviceRect.Y + deviceRect.Height));
			double yvalue = annotMinRect.Y + culativeY;
			if (!iscontain1 && iscontain2)
			{
				if (deviceRect.Y < annotInPageRect.Y)
				{
					double offset = annotInPageRect.Y - deviceRect.Y;
					culativeY = culativeY + offset;

					deviceRect = Windows::Foundation::Rect(annotMinRect.X, annotMinRect.Y + (float)culativeY, annotMinRect.Width, annotMinRect.Height);
					iscontain1 = annotInPageRect.Contains(Point(deviceRect.X, deviceRect.Y));
					iscontain2 = annotInPageRect.Contains(Point(deviceRect.X + deviceRect.Width, deviceRect.Y + deviceRect.Height));
					if (iscontain1 && iscontain2)
					{
						m_LayoutGridTransform->TranslateY = culativeY;
					}
				}
			}
			else
			{
				if (deviceRect.Y + deviceRect.Height > annotInPageRect.Y + annotInPageRect.Height)
				{
					double offset = deviceRect.Y + deviceRect.Height - (annotInPageRect.Y + annotInPageRect.Height);
					culativeY = culativeY - offset;

					deviceRect = Windows::Foundation::Rect(annotMinRect.X, annotMinRect.Y + (float)culativeY, annotMinRect.Width, annotMinRect.Height);
					iscontain1 = annotInPageRect.Contains(Point(deviceRect.X, deviceRect.Y));
					iscontain2 = annotInPageRect.Contains(Point(deviceRect.X + deviceRect.Width, deviceRect.Y + deviceRect.Height));
					if (iscontain1 && iscontain2)
					{
						m_LayoutGridTransform->TranslateY = culativeY;
					}
				}
			}
		}
		else
		{
			deviceRect = Windows::Foundation::Rect(annotMinRect.X, annotMinRect.Y + (float)culativeY, annotMinRect.Width, annotMinRect.Height);
			iscontain1 = annotInPageRect.Contains(Point(deviceRect.X, deviceRect.Y));
			iscontain2 = annotInPageRect.Contains(Point(((float)((int)deviceRect.X + (int)deviceRect.Width)), deviceRect.Y + deviceRect.Height));
			if (iscontain1 && iscontain2)
			{
				m_LayoutGridTransform->TranslateY = culativeY;

				deviceRect = Windows::Foundation::Rect(annotMinRect.X + (float)culativeX, annotMinRect.Y, annotMinRect.Width, annotMinRect.Height);
				iscontain1 = annotInPageRect.Contains(Point(deviceRect.X, deviceRect.Y));
				iscontain2 = annotInPageRect.Contains(Point(deviceRect.X + deviceRect.Width, deviceRect.Y + deviceRect.Height));
				double yvalue = annotMinRect.Y + culativeY;
				if (!iscontain1 && iscontain2)
				{
					if (deviceRect.X < annotInPageRect.X)
					{
						double offset = annotInPageRect.X - deviceRect.X;
						culativeX = culativeX + offset;

						deviceRect = Windows::Foundation::Rect(annotMinRect.X + (float)culativeX, annotMinRect.Y, annotMinRect.Width, annotMinRect.Height);
						iscontain1 = annotInPageRect.Contains(Point(deviceRect.X, deviceRect.Y));
						iscontain2 = annotInPageRect.Contains(Point(deviceRect.X + deviceRect.Width, deviceRect.Y + deviceRect.Height));
						if (iscontain1 && iscontain2)
						{
							m_LayoutGridTransform->TranslateX = culativeX;
						}
					}
				}
				else
				{
					if (deviceRect.X + deviceRect.Width > annotInPageRect.X + annotInPageRect.Width)
					{
						double offset = deviceRect.X + deviceRect.Width - (annotInPageRect.X + annotInPageRect.Width);
						culativeX = culativeX - offset;

						deviceRect = Windows::Foundation::Rect(annotMinRect.X + (float)culativeX, annotMinRect.Y, annotMinRect.Width, annotMinRect.Height);
						iscontain1 = annotInPageRect.Contains(Point(deviceRect.X, deviceRect.Y));
						iscontain2 = annotInPageRect.Contains(Point(deviceRect.X + deviceRect.Width, deviceRect.Y + deviceRect.Height));
						if (iscontain1 && iscontain2)
						{
							m_LayoutGridTransform->TranslateX = culativeX;
						}
					}
				}
			}
		}
	}
}


void AnnotOperatingOverlayControl::CalculateInitialOperatingOverlayPosition(Windows::Foundation::Rect originalRect, Windows::Foundation::Rect overlayRect)
{
	UIExtensionsManager^ uiMgr = (UIExtensionsManager^)View->ExtensionsManager;
	if (m_bindData)
	{
		bool isFreeTextAnnot = false;
		IBaseAnnot^ annot = uiMgr->getAnnotMgr()->GetSelectedAnnot();
		if (annot)
		{
			Windows::UI::Color color = annot->Color;
			if (annot->AnnotType == AnnotationType::FreeText)
			{
				FreeTextAnnot^ freetextAnnot = dynamic_cast<FreeTextAnnot^>(annot);
				color = freetextAnnot->TextColor;
				isFreeTextAnnot = true;
			}
			m_bindData->CalculateInitialOperatingOverlayPosition(originalRect, overlayRect, color, isFreeTextAnnot);
		}
		else
		{
			Windows::UI::Color colorTra = Windows::UI::Colors::Transparent;
			m_bindData->CalculateInitialOperatingOverlayPosition(originalRect, overlayRect, colorTra, false);
		}
	}
	annotMinRect = originalRect;
	annotInPageRect = overlayRect;
	if (annotMinRect.Y < annotInPageRect.Y)
	{
		annotMinRect.Y = annotInPageRect.Y;
		annotMinRect.Height = annotMinRect.Height - (annotInPageRect.Y - annotMinRect.Y);
	}
	m_isMoved = false;
}

void AnnotOperatingOverlayControl::ShowControlView()
{
	UIExtensionsManager^ uiMgr = (UIExtensionsManager^)View->ExtensionsManager;
	if (m_bindData)
	{
		m_bindData->ShowControlView();
	}
	if (uiMgr->getAnnotMgr()->GetSelectedAnnot())
	{
		if (uiMgr->getAnnotMgr()->GetSelectedAnnot()->AnnotType == AnnotationType::FreeText)
		{
			auto annot = dynamic_cast<FreeTextAnnot^>(uiMgr->getAnnotMgr()->GetSelectedAnnot());
			if (annot)
			{
				m_freeTextInitRect = annot->Page_HightLightRect;
				m_freeTextInitMatrix = annot->TextMatrix;
			}
		}
	}


	if (uiMgr->getAnnotMgr()->GetSelectedAnnot())
	{
		if (uiMgr->getAnnotMgr()->GetSelectedAnnot()->AnnotType == AnnotationType::FreeText
			|| uiMgr->getAnnotMgr()->GetSelectedAnnot()->AnnotType == AnnotationType::Text)
		{
			auto annot = dynamic_cast<FreeTextAnnot^>(uiMgr->getAnnotMgr()->GetSelectedAnnot());
			if (annot)
			{
				m_freeTextInitRect = annot->Page_HightLightRect;
				m_freeTextInitFontSize = annot->FontSize;
				m_freeTextInitMatrix = annot->TextMatrix;
			}
			AnnotOperatingOverlay->Template = dynamic_cast<ControlTemplate^>(this->Resources->Lookup("AnnotOperatingNoResizeTemplate"));

			auto _CompositionScaleX = View->CompositionScaleX;
			auto _CompositionScaleY = View->CompositionScaleY;
			Point compositionPT(_CompositionScaleX, _CompositionScaleY);
			border->BorderThickness = 2;
			border->Visibility = Windows::UI::Xaml::Visibility::Visible;
			gridCanvas->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
		}
		else
		{
			AnnotOperatingOverlay->Template = dynamic_cast<ControlTemplate^>(this->Resources->Lookup("AnnotOperatingOverlayTemplate"));
			border->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
			gridCanvas->Visibility = Windows::UI::Xaml::Visibility::Visible;
		}
	}
	else
	{
		AnnotOperatingOverlay->Template = dynamic_cast<ControlTemplate^>(this->Resources->Lookup("AnnotOperatingOverlayTemplate"));
		border->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
		gridCanvas->Visibility = Windows::UI::Xaml::Visibility::Visible;
	}
}

void AnnotOperatingOverlayControl::HideControlView()
{
	if (m_bindData)
	{
		m_bindData->HideControlView();
	}
}


void AnnotOperatingOverlayControl::Grid_ManipulationCompleted(Platform::Object^ sender, Windows::UI::Xaml::Input::ManipulationCompletedRoutedEventArgs^ e)
{
	UIExtensionsManager^ uiMgr = (UIExtensionsManager^)View->ExtensionsManager;
	if (View->GetDoc() && !FSPDFDocUtil::GetIsDocumentCanModifyAnnotation(View->GetDoc()) && View->GetDoc()->GetPasswordType() != foxit::pdf::PasswordType::e_PwdOwner)
		return;
	if (m_isAnnotMoveEnd)
		return;

	annotMinRect.X = annotMinRect.X + (float)m_LayoutGridTransform->TranslateX;
	annotMinRect.Y = annotMinRect.Y + (float)m_LayoutGridTransform->TranslateY;

	if (uiMgr->getAnnotMgr()->GetSelectedAnnot() && uiMgr->getAnnotMgr()->GetSelectedAnnot()->AnnotType == AnnotationType::Line)
	{
		deviceLineStartPt.X += (float)m_LayoutGridTransform->TranslateX;
		deviceLineStartPt.Y += (float)m_LayoutGridTransform->TranslateY;
		deviceLineEndPt.X += (float)m_LayoutGridTransform->TranslateX;
		deviceLineEndPt.Y += (float)m_LayoutGridTransform->TranslateY;;
	}


	if (uiMgr->getAnnotMgr()->GetSelectedAnnot() && uiMgr->getAnnotMgr()->GetSelectedAnnot()->AnnotType == AnnotationType::Text)
	{
		if (!uiMgr->ExitNotePopupEdit())
			return;
	}

	UpdateAnnotRangeAfterMove();
}

void AnnotOperatingOverlayControl::gridCanvas_ManipulationStarted(Platform::Object^ sender, Windows::UI::Xaml::Input::ManipulationStartedRoutedEventArgs^ e)
{
	UIExtensionsManager^ uiMgr = (UIExtensionsManager^)View->ExtensionsManager;
	if (uiMgr->getAnnotMgr()->GetSelectedAnnot() && uiMgr->getAnnotMgr()->GetSelectedAnnot()->AnnotType == AnnotationType::Text)
	{
		if (!uiMgr->ExitNotePopupEdit())
			return;
	}

	uiMgr->SetPropertyPoupHide();
	gridCanvas->Background = ref new SolidColorBrush(Windows::UI::Colors::Gray);
	gridCanvas->Opacity = 0.5;

	gridNoResizeCanvas->Background = ref new SolidColorBrush(Windows::UI::Colors::Gray);
	gridNoResizeCanvas->Opacity = 0.5;
	m_isAnnotMoveEnd = false;
	m_isMoved = false;

	deltaX = 0;
	deltaY = 0;
}


void AnnotOperatingOverlayControl::RelatePageView(foxit::rdk::PDFViewCtrl^ view)
{
	m_PageViewWeakRef = view;
}

foxit::rdk::PDFViewCtrl^ AnnotOperatingOverlayControl::View::get()
{
	return m_PageViewWeakRef.Resolve<foxit::rdk::PDFViewCtrl>();
}

void AnnotOperatingOverlayControl::gridCanvas_ManipulationInertiaStarting(Platform::Object^ sender, Windows::UI::Xaml::Input::ManipulationInertiaStartingRoutedEventArgs^ e)
{
	
}

void AnnotOperatingOverlayControl::UpdateAnnotRangeAfterMove()
{
	UIExtensionsManager^ uiMgr = (UIExtensionsManager^)View->ExtensionsManager;
	bool isFreeTextAnnot = false;
	gridCanvas->Background = ref new SolidColorBrush(Windows::UI::Colors::Transparent);
	gridNoResizeCanvas->Background = ref new SolidColorBrush(Windows::UI::Colors::Transparent);
	m_LayoutGridTransform->TranslateX = 0;
	m_LayoutGridTransform->TranslateY = 0;

	if (uiMgr->CreateTypeOfAnnot == AnnotCreateType::Signature)
	{
		uiMgr->ReLocateImageSignature(annotMinRect);
		m_bindData->CalculateInitialOperatingOverlayPosition(annotMinRect, annotInPageRect, Windows::UI::Colors::Transparent, isFreeTextAnnot);
		m_isMoved = false;
		return;
	}

	if (!uiMgr->getAnnotMgr()->GetSelectedAnnot())
		return;

	Windows::Foundation::Rect minpageRect = View->ConvertDeviceRectToPageRect(uiMgr->getAnnotMgr()->GetSelectedAnnot()->PageIndex, annotMinRect);

	if (uiMgr->getAnnotMgr()->GetSelectedAnnot())
	{
		uiMgr->getAnnotMgr()->GetSelectedAnnot()->IsModifyProperty = true;

		if (uiMgr->getAnnotMgr()->GetSelectedAnnot()->AnnotType != AnnotationType::FreeText)
		{
			uiMgr->getAnnotMgr()->GetSelectedAnnot()->MinRect = minpageRect;
		}
		if (uiMgr->getAnnotMgr()->GetSelectedAnnot()->AnnotType == AnnotationType::Text)
		{
			auto annot = dynamic_cast<TextAnnot^>(uiMgr->getAnnotMgr()->GetSelectedAnnot());
			if (annot)
				annot->DrawRect = minpageRect;
		}
		if (uiMgr->getAnnotMgr()->GetSelectedAnnot()->AnnotType == AnnotationType::Ink)
		{
			auto annot = dynamic_cast<PencilAnnot^>(uiMgr->getAnnotMgr()->GetSelectedAnnot());
			if (annot)
				annot->PageMinRect = minpageRect;
		}
		else if (uiMgr->getAnnotMgr()->GetSelectedAnnot()->AnnotType == AnnotationType::Square)
		{
			auto annot = dynamic_cast<SquareAnnot^>(uiMgr->getAnnotMgr()->GetSelectedAnnot());
			if (annot)
				annot->Page_HightLightRect = minpageRect;
		}
		else if (uiMgr->getAnnotMgr()->GetSelectedAnnot()->AnnotType == AnnotationType::Circle)
		{
			auto annot = dynamic_cast<CircleAnnot^>(uiMgr->getAnnotMgr()->GetSelectedAnnot());
			if (annot)
				annot->Page_HightLightRect = minpageRect;
		}
		else if (uiMgr->getAnnotMgr()->GetSelectedAnnot()->AnnotType == AnnotationType::Line)
		{
			auto annot = dynamic_cast<LineAnnot^>(uiMgr->getAnnotMgr()->GetSelectedAnnot());
			if (annot)
			{
				annot->PageMinRect = minpageRect;
			}
		}
		else if (uiMgr->getAnnotMgr()->GetSelectedAnnot()->AnnotType == AnnotationType::Stamp)
		{
			auto annot = dynamic_cast<StampAnnot^>(uiMgr->getAnnotMgr()->GetSelectedAnnot());			
			if (annot)
			{				
				annot->Page_HightLightRect = minpageRect;
			}				
		}
		else if (uiMgr->getAnnotMgr()->GetSelectedAnnot()->AnnotType == AnnotationType::FreeText)
		{
			auto annot = dynamic_cast<FreeTextAnnot^>(uiMgr->getAnnotMgr()->GetSelectedAnnot());
			if (annot)
			{
				isFreeTextAnnot = true;
				annot->Page_HightLightRect = minpageRect;
			}
		}
	}

	Point pt = Point(annotMinRect.X + annotMinRect.Width / 2, annotMinRect.Y);
	if (uiMgr->getAnnotMgr()->GetSelectedAnnot()->AnnotType == AnnotationType::FreeText)
	{
		if (annotMinRect.X < annotInPageRect.X)
		{
			pt = Point(annotInPageRect.X + annotInPageRect.Width / 2, annotMinRect.Y);
		}
	}

	if (View->GetDoc() && !FSPDFDocUtil::GetIsDocumentCanModifyAnnotation(View->GetDoc()) && View->GetDoc()->GetPasswordType() != foxit::pdf::PasswordType::e_PwdOwner)
	{
		uiMgr->ShowTextmarkContextmenu(AnnotContextMenuType::Note, pt);
	}
	else
	{
		if (uiMgr->getAnnotMgr()->GetSelectedAnnot()->AnnotType == AnnotationType::Stamp)
		{
			uiMgr->ShowTextmarkContextmenu(AnnotContextMenuType::Delete, pt);
		}
		else if (uiMgr->getAnnotMgr()->GetSelectedAnnot()->AnnotType == AnnotationType::Text)
		{
			uiMgr->ShowTextmarkContextmenu(AnnotContextMenuType::NotePro, pt);
		}
		else
		{
			uiMgr->ShowTextmarkContextmenu(AnnotContextMenuType::Edit, pt);
		}
	}

	m_bindData->CalculateInitialOperatingOverlayPosition(annotMinRect, annotInPageRect, uiMgr->getAnnotMgr()->GetSelectedAnnot()->Color, isFreeTextAnnot);
	m_isMoved = false;
}

void AnnotOperatingOverlayControl::SetLineAnnotPoint(Windows::Foundation::Point startPoint, Windows::Foundation::Point endPoint)
{
	deviceLineStartPt = startPoint;
	deviceLineEndPt = endPoint;
}


void foxit::rdk::uiextensions::AnnotOperatingOverlayControl::OnThumbDragCompleted(Platform::Object^ sender, Windows::UI::Xaml::Controls::Primitives::DragCompletedEventArgs^ e)
{
	if (View->GetDoc() && !FSPDFDocUtil::GetIsDocumentCanModifyAnnotation(View->GetDoc()) && View->GetDoc()->GetPasswordType() != foxit::pdf::PasswordType::e_PwdOwner)
		return;

	UIExtensionsManager^ uiMgr = (UIExtensionsManager^)View->ExtensionsManager;
	Windows::Foundation::Rect rect((float)(m_bindData->CropOverlayLeft + 5), (float)(m_bindData->CropOverlayTop + 5), (float)(m_bindData->CropOverlayWidth > 10 ? m_bindData->CropOverlayWidth - 10 : 0), (float)(m_bindData->CropOverlayHeight > 10 ? m_bindData->CropOverlayHeight - 10 : 0));

	if (rect.Width < 5)
	{
		rect.Width = 5;
	}
	if (rect.Height < 5)
	{
		rect.Height = 5;
	}

	annotMinRect = rect;
	uiMgr->SetPropertyPoupHide();
	UpdateAnnotRangeAfterMove();
	gridCanvas->ManipulationMode = Windows::UI::Xaml::Input::ManipulationModes::All;
}
