﻿/**
 * Copyright (C) 2003-2018, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit Mobile PDF SDK to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */


#pragma once

#include "Annotations\AnnotOperatingOverlayControl.g.h"

namespace foxit
{
	namespace rdk
	{
		namespace uiextensions
		{
			[Windows::UI::Xaml::Data::Bindable]
			[Windows::Foundation::Metadata::WebHostHidden]
			public ref class AnnotOperatingBindData sealed : public BindableBase
			{
			internal:
				AnnotOperatingBindData();

				void CalculateInitialOperatingOverlayPosition(Windows::UI::Xaml::Media::GeneralTransform^ transform, float width, float height);
				void CalculateInitialOperatingOverlayPosition(Windows::Foundation::Rect originalRect, Windows::Foundation::Rect overlayRect, Windows::UI::Color color, bool isFreeTextAnnot);
				void UpdateOperatingOverlayPostion(Windows::UI::Xaml::Controls::Primitives::Thumb^ thumb, float64 verticalChange, float64 horizontalChange, float64 minWidth, float64 minHeight);
				void MoveOperatingOverlayPostion(float64 horizontalChange, float64 verticalChange);

				void ShowControlView();
				void HideControlView();

			public:
				property float64 CropOverlayLeft { float64 get(); }
				property float64 CropOverlayTop { float64 get(); }
				property float64 CropOverlayHeight { float64 get(); }
				property float64 CropOverlayWidth { float64 get(); }
				property bool IsAnnotOperatingOverlayVisible { bool get(); }
				property Windows::UI::Color CropOverlayColor { Windows::UI::Color get(); }

			private:
				bool m_isOperatingOverlayVisible;

				float64 m_left;
				float64 m_top;
				float64 m_right;
				float64 m_bottom;

				float64 m_cropOverlayLeft;
				float64 m_cropOverlayTop;
				float64 m_cropOverlayHeight;
				float64 m_cropOverlayWidth;
				float64 m_actualHeight;
				float64 m_actualWidth;
				Windows::UI::Color m_cropOverlayColor;
			};

			[Windows::Foundation::Metadata::WebHostHidden]
			public ref class AnnotOperatingOverlayControl sealed
			{
			public:
				AnnotOperatingOverlayControl();

				void RelatePageView(foxit::rdk::PDFViewCtrl^ view);
				void CalculateInitialOperatingOverlayPosition(Windows::Foundation::Rect originalRect, Windows::Foundation::Rect overlayRect);
				void ShowControlView();
				void HideControlView();
				void SetLineAnnotPoint(Windows::Foundation::Point startPoint, Windows::Foundation::Point endPoint);

			internal:
				property foxit::rdk::PDFViewCtrl^ View
				{
					foxit::rdk::PDFViewCtrl^ get();
				}

			private:
				AnnotOperatingBindData^ m_bindData;
				Windows::UI::Xaml::Media::CompositeTransform^ m_LayoutGridTransform;
				Platform::WeakReference m_PageViewWeakRef;
				Windows::Foundation::Rect annotMinRect;
				Windows::Foundation::Rect annotInPageRect;
				Windows::Foundation::Point deviceLineStartPt;
				Windows::Foundation::Point deviceLineEndPt;
				bool m_isAnnotMoveEnd;
				Windows::Foundation::Rect m_freeTextInitRect;
				double m_freeTextInitFontSize;
				Windows::Foundation::Numerics::float3x2 m_freeTextInitMatrix;
				bool m_isMoved;
				bool m_ismoveOutPage;
				double deltaX;
				double deltaY;

			private:
				void OnInitializeControl();
				void UpdateAnnotRangeAfterMove();

			private:
				void OnLoaded(Platform::Object ^sender, Windows::UI::Xaml::RoutedEventArgs ^e);
				void OnUnloaded(Platform::Object ^sender, Windows::UI::Xaml::RoutedEventArgs ^e);
				void OnThumbDragDelta(Platform::Object^ sender, Windows::UI::Xaml::Controls::Primitives::DragDeltaEventArgs^ e);
				void OnOperatingRectangleTapped(Platform::Object^ sender, Windows::UI::Xaml::Input::TappedRoutedEventArgs^ e);
				void OnOperatingManipulationDelta(Platform::Object^ sender, Windows::UI::Xaml::Input::ManipulationDeltaRoutedEventArgs^ e);
				void Grid_ManipulationCompleted(Platform::Object^ sender, Windows::UI::Xaml::Input::ManipulationCompletedRoutedEventArgs^ e);
				void gridCanvas_ManipulationStarted(Platform::Object^ sender, Windows::UI::Xaml::Input::ManipulationStartedRoutedEventArgs^ e);
				void gridCanvas_ManipulationInertiaStarting(Platform::Object^ sender, Windows::UI::Xaml::Input::ManipulationInertiaStartingRoutedEventArgs^ e);
				void OnThumbDragCompleted(Platform::Object^ sender, Windows::UI::Xaml::Controls::Primitives::DragCompletedEventArgs^ e);
			};
		}
	}
}
