﻿/**
 * Copyright (C) 2003-2018, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit Mobile PDF SDK to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */


#include "pch.h"
#include "AnnotPropertyAppBar.xaml.h"
#include "../FloatWindow/NoteStyleUserControl.xaml.h"
#include "../FloatWindow/AnnotStampUserControl.xaml.h"
#include "FreeTextProperty.xaml.h"
#include "FreeTextEventPeer.h"

using namespace foxit::rdk;
using namespace foxit::rdk::uiextensions;
using namespace Platform;
using namespace Windows::Foundation;
using namespace Windows::Foundation::Collections;
using namespace Windows::UI::Xaml;
using namespace Windows::UI::Xaml::Controls;
using namespace Windows::UI::Xaml::Controls::Primitives;
using namespace Windows::UI::Xaml::Data;
using namespace Windows::UI::Xaml::Input;
using namespace Windows::UI::Xaml::Media;
using namespace Windows::UI::Xaml::Navigation;



// The User Control item template is documented at http://go.microsoft.com/fwlink/?LinkId=234236
AnnotPropertyAppBar::AnnotPropertyAppBar(foxit::rdk::PDFViewCtrl^ view)
{
	InitializeComponent();

	m_PageViewWeakRef = view;
	noteStylePanel->RelateToView(view);
	InitPropertyPanelVisible();


	this->Loaded += ref new Windows::UI::Xaml::RoutedEventHandler(this, &foxit::rdk::uiextensions::AnnotPropertyAppBar::OnLoaded);
	this->Unloaded += ref new Windows::UI::Xaml::RoutedEventHandler(this, &foxit::rdk::uiextensions::AnnotPropertyAppBar::OnUnloaded);
}


void foxit::rdk::uiextensions::AnnotPropertyAppBar::OnLoaded(Platform::Object ^sender, Windows::UI::Xaml::RoutedEventArgs ^e)
{
	double width = this->ActualWidth;
	freeTextPanel->RelateToView(View, width, this->ActualHeight);
	sliderLiness->ValueChanged += ref new Windows::UI::Xaml::Controls::Primitives::RangeBaseValueChangedEventHandler(this, &foxit::rdk::uiextensions::AnnotPropertyAppBar::OnValueChanged);
	stampIconPopup = ref new Windows::UI::Xaml::Controls::Primitives::Popup();
}


void foxit::rdk::uiextensions::AnnotPropertyAppBar::OnUnloaded(Platform::Object ^sender, Windows::UI::Xaml::RoutedEventArgs ^e)
{

}

void AnnotPropertyAppBar::InitPropertyPanelVisible()
{
	UIExtensionsManager^ uiMgr = (UIExtensionsManager^)View->ExtensionsManager;
	IBaseAnnot^ annot = uiMgr->getAnnotMgr()->GetSelectedAnnot();
	if (!annot)
	{
		colorPanel->Visibility = Windows::UI::Xaml::Visibility::Visible;
		opacityPanel->Visibility = Windows::UI::Xaml::Visibility::Visible;
		LinessPanel->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
		noteStylePanel->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
		if (uiMgr->CreateTypeOfAnnot == AnnotCreateType::FreeText)
		{
			freeTextPanel->Visibility = Windows::UI::Xaml::Visibility::Visible;
		}
		else
		{
			freeTextPanel->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
		}
	}
	else
	{
		colorPanel->Visibility = Windows::UI::Xaml::Visibility::Visible;
		opacityPanel->Visibility = Windows::UI::Xaml::Visibility::Visible;
		LinessPanel->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
		noteStylePanel->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
		freeTextPanel->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
		AnnotationType type = annot->AnnotType;

		if (type == AnnotationType::Circle || type == AnnotationType::Square
			|| type == AnnotationType::Line || type == AnnotationType::Ink)
		{
			LinessPanel->Visibility = Windows::UI::Xaml::Visibility::Visible;
		}
		else if (type == AnnotationType::Text)
		{
			noteStylePanel->Visibility = Windows::UI::Xaml::Visibility::Visible;
			colorPanel->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
			opacityPanel->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
		}
		else if (type == AnnotationType::FreeText)
		{
			freeTextPanel->Visibility = Windows::UI::Xaml::Visibility::Visible;
		}
	}
	InitAppBarProperty();
}


foxit::rdk::PDFViewCtrl^ AnnotPropertyAppBar::View::get()
{
	return m_PageViewWeakRef.Resolve<foxit::rdk::PDFViewCtrl>();
}

void AnnotPropertyAppBar::InitAppBarProperty()
{
	Windows::UI::Color colorstr;
	double opacity = 0;
	AnnotationType type;
	double borderWidth = 0;
	UIExtensionsManager^ uiMgr = (UIExtensionsManager^)View->ExtensionsManager;
	if (uiMgr->getAnnotMgr()->GetSelectedAnnot())
	{
		type = uiMgr->getAnnotMgr()->GetSelectedAnnot()->AnnotType;
		InitAnnotColorList(type);

		if (uiMgr->getAnnotMgr()->GetSelectedAnnot()->AnnotType == AnnotationType::FreeText)
		{
			FreeTextAnnot^ freetextannot = dynamic_cast<FreeTextAnnot^>(uiMgr->getAnnotMgr()->GetSelectedAnnot());
			if (freetextannot)
				colorstr = freetextannot->TextColor;
		}
		else
		{
			colorstr = uiMgr->getAnnotMgr()->GetSelectedAnnot()->Color;
		}
		opacity = uiMgr->getAnnotMgr()->GetSelectedAnnot()->Opacity;

		borderWidth = uiMgr->getAnnotMgr()->GetSelectedAnnot()->BorderWidth;
	}
	else
	{
		type = GetAnnotType(uiMgr->CreateTypeOfAnnot);
		InitAnnotColorList(type);

		auto _AnnotSetting = uiMgr->getAnnotMgr()->GetAnnotSetting();
		if (uiMgr->CreateTypeOfAnnot == AnnotCreateType::HightLight
			|| uiMgr->CreateTypeOfAnnot == AnnotCreateType::AreaHighLight)
		{
			colorstr = _AnnotSetting->GetHightLightAnnotColor();
			opacity = _AnnotSetting->GetHightLightAnnotOpacity();
		}
		else if (uiMgr->CreateTypeOfAnnot == AnnotCreateType::UnderLine)
		{
			colorstr = _AnnotSetting->GetUnderLineAnnotColor();
			opacity = _AnnotSetting->GetUnderLineAnnotOpacity();
		}
		else if (uiMgr->CreateTypeOfAnnot == AnnotCreateType::StrikeOut)
		{
			colorstr = _AnnotSetting->GetStrikeOutAnnotColor();
			opacity = _AnnotSetting->GetStrikeOutAnnotOpacity();
		}
		else if (uiMgr->CreateTypeOfAnnot == AnnotCreateType::Squiggly)
		{
			colorstr = _AnnotSetting->GetSquigglyAnnotColor();
			opacity = _AnnotSetting->GetSquigglyAnnotOpacity();
		}
		else if (uiMgr->CreateTypeOfAnnot == AnnotCreateType::FreeText)
		{
			colorstr = _AnnotSetting->GetFreeTextColor();
			opacity = _AnnotSetting->GetFreeTextAnnotOpacity();
		}
		else if (uiMgr->CreateTypeOfAnnot == AnnotCreateType::Line || uiMgr->CreateTypeOfAnnot == AnnotCreateType::Arrow)
		{
			colorstr = _AnnotSetting->GetLineAnnotColor();
			opacity = _AnnotSetting->GetAnnotOpacity();
			borderWidth = _AnnotSetting->GetAnnotDrawBorderWidth();
		}
		else if (uiMgr->CreateTypeOfAnnot == AnnotCreateType::Caret)
		{
			colorstr = _AnnotSetting->GetInsertTextColor();
			opacity = _AnnotSetting->GetInsertTextAnnotOpacity();
		}
		else if (uiMgr->CreateTypeOfAnnot == AnnotCreateType::ReplaceText)
		{
			colorstr = _AnnotSetting->GetReplaceTextColor();
			opacity = _AnnotSetting->GetReplaceTextAnnotOpacity();
		}
		else
		{
			colorstr = _AnnotSetting->GetAnnotColor();
			opacity = _AnnotSetting->GetAnnotOpacity();
			borderWidth = _AnnotSetting->GetAnnotDrawBorderWidth();
		}
	}

	if (opacity == 0.25)
	{
		OpacityPageBtn1->IsChecked = true;
	}
	else if (opacity == 0.5)
	{
		OpacityPageBtn2->IsChecked = true;
	}
	else if (opacity == 0.75)
	{
		OpacityPageBtn3->IsChecked = true;
	}
	else if (opacity == 1.0)
	{
		OpacityPageBtn4->IsChecked = true;
	}

	Platform::String^ colorRGBValue = Utility::ConvertToColorString(colorstr);
	SetRadioColorChecked(colorRGBValue, type);

	sliderLiness->Value = borderWidth;
	tbLiness->Text = borderWidth.ToString() + "pt";
}


void AnnotPropertyAppBar::NoteLogoSetting_Tapped(Platform::Object^ sender, Windows::UI::Xaml::Input::TappedRoutedEventArgs^ e)
{
	UIExtensionsManager^ uiMgr = (UIExtensionsManager^)View->ExtensionsManager;
	uiMgr->LoadNoteStylePage();
}


void AnnotPropertyAppBar::ColorPageBtnSelect_Tapped(Platform::Object^ sender, Windows::UI::Xaml::Input::TappedRoutedEventArgs^ e)
{
	UIExtensionsManager^ uiMgr = (UIExtensionsManager^)View->ExtensionsManager;
	IBaseAnnot^ annot = uiMgr->getAnnotMgr()->GetSelectedAnnot();
	RadioButton^ button = dynamic_cast<RadioButton^>(sender);
	Platform::String^ content = button->Content->ToString();
	Windows::UI::Color color = Utility::ConvertFromString(button->Content->ToString());
	if (annot)
	{
		annot->IsModifyProperty = true;
		annot->Color = color;
		if (annot->AnnotType == AnnotationType::HightLight)
		{
			uiMgr->getAnnotMgr()->GetAnnotSetting()->SetHightLightAnnotColor(color);
		}
		else if (annot->AnnotType == AnnotationType::UnderLine)
		{
			uiMgr->getAnnotMgr()->GetAnnotSetting()->SetUnderLineAnnotColor(color);
		}
		else if (annot->AnnotType == AnnotationType::StrikeOut)
		{
			uiMgr->getAnnotMgr()->GetAnnotSetting()->SetStrikeOutAnnotColor(color);
			StrikeOutAnnot^ strikeoutAnnot = dynamic_cast<StrikeOutAnnot^>(annot);
			if (strikeoutAnnot->IsHasGroup)
			{
				if (uiMgr->getAnnotMgr()->IsGetGroupAnnotElement(strikeoutAnnot->PageIndex, strikeoutAnnot->GroupAnnotMinRect,
					strikeoutAnnot->GroupAnnotIndex, AnnotationType::Caret))
				{
					IBaseAnnot^ groupAnnot = uiMgr->getAnnotMgr()->GetGroupAnnot();
					groupAnnot->Color = color;
				}
				uiMgr->getAnnotMgr()->GetAnnotSetting()->SetReplaceTextColor(color);
			}
		}
		else if (annot->AnnotType == AnnotationType::Squiggly)
		{
			uiMgr->getAnnotMgr()->GetAnnotSetting()->SetSquigglyAnnotColor(color);
		}
		else if (annot->AnnotType == AnnotationType::Line)
		{
			uiMgr->getAnnotMgr()->GetAnnotSetting()->SetLineAnnotColor(color);
		}
		else if (annot->AnnotType == AnnotationType::Ink || annot->AnnotType == AnnotationType::Circle
			|| annot->AnnotType == AnnotationType::Square)
		{
			uiMgr->getAnnotMgr()->GetAnnotSetting()->SetAnnotColor(color);
		}
		else if (annot->AnnotType == AnnotationType::FreeText)
		{
			FreeTextAnnot^ freetextannot = dynamic_cast<FreeTextAnnot^>(annot);
			if (freetextannot)
			{
				freetextannot->Contents = freetextannot->Contents;
				freetextannot->TextColor = color;				
				uiMgr->UpdateFreeTextAnnotOverlayControl(annot->PageIndex, freetextannot->Page_HightLightRect);
			}
				

			uiMgr->getAnnotMgr()->GetAnnotSetting()->SetFreeTextColor(color);
		}
		else if (annot->AnnotType == AnnotationType::Caret)
		{
			uiMgr->getAnnotMgr()->GetAnnotSetting()->SetInsertTextColor(color);
			CaretAnnot^ caretAnnot = dynamic_cast<CaretAnnot^>(annot);
			if (caretAnnot->IsHasGroup)
			{
				if (uiMgr->getAnnotMgr()->IsGetGroupAnnotElement(caretAnnot->PageIndex, caretAnnot->GroupAnnotMinRect,
					caretAnnot->GroupAnnotIndex, AnnotationType::StrikeOut))
				{
					IBaseAnnot^ groupAnnot = uiMgr->getAnnotMgr()->GetGroupAnnot();
					groupAnnot->Color = color;
				}
			}
		}
	}
	else
	{
		if (uiMgr->CreateTypeOfAnnot != AnnotCreateType::None && uiMgr->CreateTypeOfAnnot != AnnotCreateType::FreeText)
			uiMgr->CreateGraphAnnotBySliderPage();
		if (uiMgr->CreateTypeOfAnnot == AnnotCreateType::HightLight || uiMgr->CreateTypeOfAnnot == AnnotCreateType::AreaHighLight)
		{
			uiMgr->getAnnotMgr()->GetAnnotSetting()->SetHightLightAnnotColor(color);
		}
		else if (uiMgr->CreateTypeOfAnnot == AnnotCreateType::UnderLine)
		{
			uiMgr->getAnnotMgr()->GetAnnotSetting()->SetUnderLineAnnotColor(color);
		}
		else if (uiMgr->CreateTypeOfAnnot == AnnotCreateType::StrikeOut)
		{
			uiMgr->getAnnotMgr()->GetAnnotSetting()->SetStrikeOutAnnotColor(color);
		}
		else if (uiMgr->CreateTypeOfAnnot == AnnotCreateType::Squiggly)
		{
			uiMgr->getAnnotMgr()->GetAnnotSetting()->SetSquigglyAnnotColor(color);
		}
		else if (uiMgr->CreateTypeOfAnnot == AnnotCreateType::Line || uiMgr->CreateTypeOfAnnot == AnnotCreateType::Arrow)
		{
			uiMgr->getAnnotMgr()->GetAnnotSetting()->SetLineAnnotColor(color);
		}
		else if (uiMgr->CreateTypeOfAnnot == AnnotCreateType::Pencil || uiMgr->CreateTypeOfAnnot == AnnotCreateType::Circle
			|| uiMgr->CreateTypeOfAnnot == AnnotCreateType::Square)
		{
			uiMgr->getAnnotMgr()->GetAnnotSetting()->SetAnnotColor(color);
		}
		else if (uiMgr->CreateTypeOfAnnot == AnnotCreateType::FreeText)
		{
			uiMgr->getAnnotMgr()->GetAnnotSetting()->SetFreeTextColor(color);
			IBaseAnnotEventPeer^ annotEventPeer = uiMgr->getAnnotMgr()->GetAnnotHandlerByType(AnnotationType::FreeText)->AnnotEvent;
			FreeTextEventPeer^ freetextEventPeer = dynamic_cast<FreeTextEventPeer^>(annotEventPeer);
			if (freetextEventPeer)
				freetextEventPeer->UpdatFreeTextBoxProperty();
		}
		else if (uiMgr->CreateTypeOfAnnot == AnnotCreateType::Caret)
		{
			uiMgr->getAnnotMgr()->GetAnnotSetting()->SetInsertTextColor(color);
		}
		else if (uiMgr->CreateTypeOfAnnot == AnnotCreateType::ReplaceText)
		{
			uiMgr->getAnnotMgr()->GetAnnotSetting()->SetReplaceTextColor(color);
		}
		uiMgr->SetPropertyCurColor(color);
	}
}


void AnnotPropertyAppBar::OpacityPageBtnSelect_Tapped(Platform::Object^ sender, Windows::UI::Xaml::Input::TappedRoutedEventArgs^ e)
{
	UIExtensionsManager^ uiMgr = (UIExtensionsManager^)View->ExtensionsManager;
	IBaseAnnot^ annot = uiMgr->getAnnotMgr()->GetSelectedAnnot();
	RadioButton^ button = dynamic_cast<RadioButton^>(sender);
	String^ opacityvalue = button->Tag->ToString();
	double opacity = 0;
	if (opacityvalue == "25")
	{
		opacity = 25;
	}
	else if (opacityvalue == "50")
	{
		opacity = 50;
	}
	else if (opacityvalue == "75")
	{
		opacity = 75;
	}
	else
	{
		opacity = 100;
	}
	opacity = opacity / 100;
	if (annot)
	{
		annot->IsModifyProperty = true;
		annot->Opacity = opacity;

		if (annot->AnnotType == AnnotationType::HightLight)
		{
			uiMgr->getAnnotMgr()->GetAnnotSetting()->SetHightLightAnnotOpacity(opacity);
		}
		else if (annot->AnnotType == AnnotationType::UnderLine)
		{
			uiMgr->getAnnotMgr()->GetAnnotSetting()->SetUnderLineAnnotOpacity(opacity);
		}
		else if (annot->AnnotType == AnnotationType::StrikeOut)
		{
			uiMgr->getAnnotMgr()->GetAnnotSetting()->SetStrikeOutAnnotOpacity(opacity);
			StrikeOutAnnot^ strikeoutAnnot = dynamic_cast<StrikeOutAnnot^>(annot);
			if (strikeoutAnnot->IsHasGroup)
			{
				if (uiMgr->getAnnotMgr()->IsGetGroupAnnotElement(strikeoutAnnot->PageIndex, strikeoutAnnot->GroupAnnotMinRect,
					strikeoutAnnot->GroupAnnotIndex, AnnotationType::Caret))
				{
					IBaseAnnot^ groupAnnot = uiMgr->getAnnotMgr()->GetGroupAnnot();
					groupAnnot->Opacity = opacity;
				}
				uiMgr->getAnnotMgr()->GetAnnotSetting()->SetReplaceTextAnnotOpacity(opacity);
			}
		}
		else if (annot->AnnotType == AnnotationType::Squiggly)
		{
			uiMgr->getAnnotMgr()->GetAnnotSetting()->SetSquigglyAnnotOpacity(opacity);
		}
		else if (annot->AnnotType == AnnotationType::Ink || annot->AnnotType == AnnotationType::Circle
			|| annot->AnnotType == AnnotationType::Square || annot->AnnotType == AnnotationType::Line)
		{
			uiMgr->getAnnotMgr()->GetAnnotSetting()->SetAnnotOpacity(opacity);
		}
		else if (annot->AnnotType == AnnotationType::FreeText)
		{
			uiMgr->getAnnotMgr()->GetAnnotSetting()->SetFreeTextAnnotOpacity(opacity);
			FreeTextAnnot^ freetextannot = dynamic_cast<FreeTextAnnot^>(annot);
			if (freetextannot)
			{
				uiMgr->UpdateFreeTextAnnotOverlayControl(annot->PageIndex, freetextannot->Page_HightLightRect);
			}
		}
		else if (annot->AnnotType == AnnotationType::Caret)
		{
			uiMgr->getAnnotMgr()->GetAnnotSetting()->SetInsertTextAnnotOpacity(opacity);
			CaretAnnot^ caretAnnot = dynamic_cast<CaretAnnot^>(annot);
			if (caretAnnot->IsHasGroup)
			{
				if (uiMgr->getAnnotMgr()->IsGetGroupAnnotElement(caretAnnot->PageIndex, caretAnnot->GroupAnnotMinRect,
					caretAnnot->GroupAnnotIndex, AnnotationType::StrikeOut))
				{
					IBaseAnnot^ groupAnnot = uiMgr->getAnnotMgr()->GetGroupAnnot();
					groupAnnot->Opacity = opacity;
				}
			}
		}
	}
	else
	{
		if (uiMgr->CreateTypeOfAnnot != AnnotCreateType::None && uiMgr->CreateTypeOfAnnot != AnnotCreateType::FreeText)
			uiMgr->CreateGraphAnnotBySliderPage();
		if (uiMgr->CreateTypeOfAnnot == AnnotCreateType::HightLight || uiMgr->CreateTypeOfAnnot == AnnotCreateType::AreaHighLight)
		{
			uiMgr->getAnnotMgr()->GetAnnotSetting()->SetHightLightAnnotOpacity(opacity);
		}
		else if (uiMgr->CreateTypeOfAnnot == AnnotCreateType::UnderLine)
		{
			uiMgr->getAnnotMgr()->GetAnnotSetting()->SetUnderLineAnnotOpacity(opacity);
		}
		else if (uiMgr->CreateTypeOfAnnot == AnnotCreateType::StrikeOut)
		{
			uiMgr->getAnnotMgr()->GetAnnotSetting()->SetStrikeOutAnnotOpacity(opacity);
		}
		else if (uiMgr->CreateTypeOfAnnot == AnnotCreateType::Squiggly)
		{
			uiMgr->getAnnotMgr()->GetAnnotSetting()->SetSquigglyAnnotOpacity(opacity);
		}
		else if (uiMgr->CreateTypeOfAnnot == AnnotCreateType::Pencil || uiMgr->CreateTypeOfAnnot == AnnotCreateType::Circle
			|| uiMgr->CreateTypeOfAnnot == AnnotCreateType::Square || uiMgr->CreateTypeOfAnnot == AnnotCreateType::Line || uiMgr->CreateTypeOfAnnot == AnnotCreateType::Arrow)
		{
			uiMgr->getAnnotMgr()->GetAnnotSetting()->SetAnnotOpacity(opacity);
		}
		else if (uiMgr->CreateTypeOfAnnot == AnnotCreateType::FreeText)
		{
			uiMgr->getAnnotMgr()->GetAnnotSetting()->SetFreeTextAnnotOpacity(opacity);
			IBaseAnnotEventPeer^ annotEventPeer = uiMgr->getAnnotMgr()->GetAnnotHandlerByType(AnnotationType::FreeText)->AnnotEvent;
			FreeTextEventPeer^ freetextEventPeer = dynamic_cast<FreeTextEventPeer^>(annotEventPeer);
			if (freetextEventPeer)
				freetextEventPeer->UpdatFreeTextBoxProperty();
		}
		else if (uiMgr->CreateTypeOfAnnot == AnnotCreateType::Caret)
		{
			uiMgr->getAnnotMgr()->GetAnnotSetting()->SetInsertTextAnnotOpacity(opacity);
		}
		else if (uiMgr->CreateTypeOfAnnot == AnnotCreateType::ReplaceText)
		{
			uiMgr->getAnnotMgr()->GetAnnotSetting()->SetReplaceTextAnnotOpacity(opacity);
		}
	}
}


void AnnotPropertyAppBar::ExitProperty_Tapped(Platform::Object^ sender, Windows::UI::Xaml::Input::TappedRoutedEventArgs^ e)
{
	if (!View)
		return;
	UIExtensionsManager^ uiMgr = (UIExtensionsManager^)View->ExtensionsManager;
	if (uiMgr->CreateTypeOfAnnot == AnnotCreateType::Pencil)
	{
		auto _pAnnotHandler = uiMgr->getAnnotMgr()->GetAnnotHandlerByType(GetAnnotType(AnnotCreateType::Pencil));
		if (_pAnnotHandler)
			_pAnnotHandler->CreateAnnotAfterSelectText();
	}

	if (uiMgr->CreateTypeOfAnnot == AnnotCreateType::Circle || uiMgr->CreateTypeOfAnnot == AnnotCreateType::Square
		|| uiMgr->CreateTypeOfAnnot == AnnotCreateType::Line)
	{
		View->Redraw(View->GetCurrentPage());
	}

	uiMgr->HideNoteStylePopup();
	uiMgr->ExitAnnotPropertyMode();
	uiMgr->SetAnnotCanvasVisible(false);
	uiMgr->ShowDirecControl(false);

	if (!uiMgr->getAnnotMgr())
		return;
	if (uiMgr->getAnnotMgr()->GetSelectedAnnot())
	{
		uiMgr->getAnnotMgr()->SetPdfSelectAnnotNull();
		View->Update();
	}


	stampIconPopup->IsOpen = false;
}

bool AnnotPropertyAppBar::IsNoteStylePopupIsOpen()
{
	return false;
}

void AnnotPropertyAppBar::HideNoteStylePopup()
{

}


void AnnotPropertyAppBar::OnSizeChanged(Platform::Object ^sender, Windows::UI::Core::WindowSizeChangedEventArgs ^e)
{

}

foxit::rdk::uiextensions::AnnotationType AnnotPropertyAppBar::GetAnnotType(foxit::rdk::uiextensions::AnnotCreateType optType)
{
	AnnotationType _annotType;
	switch (optType)
	{
	case AnnotCreateType::None:
		_annotType = AnnotationType::None;
		break;
	case AnnotCreateType::HightLight:
		_annotType = AnnotationType::HightLight;
		break;
	case AnnotCreateType::AreaHighLight:
		_annotType = AnnotationType::HightLight;
		break;
	case AnnotCreateType::Comment:
		_annotType = AnnotationType::HightLight;
		break;
	case AnnotCreateType::UnderLine:
		_annotType = AnnotationType::UnderLine;
		break;
	case AnnotCreateType::StrikeOut:
		_annotType = AnnotationType::StrikeOut;
		break;
	case AnnotCreateType::Square:
		_annotType = AnnotationType::Square;
		break;
	case AnnotCreateType::Circle:
		_annotType = AnnotationType::Circle;
		break;
	case AnnotCreateType::Line:
		_annotType = AnnotationType::Line;
		break;
	case AnnotCreateType::Arrow:
		_annotType = AnnotationType::Line;
		break;
	case AnnotCreateType::Pencil:
		_annotType = AnnotationType::Ink;
		break;
	case AnnotCreateType::Squiggly:
		_annotType = AnnotationType::Squiggly;
		break;
	case AnnotCreateType::Text:
		_annotType = AnnotationType::Text;
		break;
	case AnnotCreateType::FreeText:
		_annotType = AnnotationType::FreeText;
		break;
	case AnnotCreateType::Caret:
	case AnnotCreateType::ReplaceText:
		_annotType = AnnotationType::Caret;
		break;
	default:
		_annotType = AnnotationType::None;
		break;
	}
	return _annotType;
}

void AnnotPropertyAppBar::InitAnnotColorList(foxit::rdk::uiextensions::AnnotationType type)
{
	if (type == foxit::rdk::uiextensions::AnnotationType::HightLight)
	{
		ColorPageBtn1->Content = "#FFFFFF00";
		ColorPageBtn2->Content = "#FFCCFF66";
		ColorPageBtn3->Content = "#FF00FFFF";
		ColorPageBtn4->Content = "#FF99CCFF";
		ColorPageBtn5->Content = "#FF7480FC";
		ColorPageBtn6->Content = "#FFFF9999";
		ColorPageBtn7->Content = "#FF00CC66";
		ColorPageBtn8->Content = "#FF22F3B1";
	}
	else if (type == foxit::rdk::uiextensions::AnnotationType::FreeText)
	{
		ColorPageBtn1->Content = "#FF3366CC";
		ColorPageBtn2->Content = "#FF669933";
		ColorPageBtn3->Content = "#FFCC6600";
		ColorPageBtn4->Content = "#FFCC9900";
		ColorPageBtn5->Content = "#FFCC0000";
		ColorPageBtn6->Content = "#FF336666";
		ColorPageBtn7->Content = "#FF660066";
		ColorPageBtn8->Content = "#FF000000";
	}
	else if (type == foxit::rdk::uiextensions::AnnotationType::UnderLine || type == foxit::rdk::uiextensions::AnnotationType::Squiggly)
	{
		ColorPageBtn1->Content = "#FF33CC00";
		ColorPageBtn2->Content = "#FFCCCC00";
		ColorPageBtn3->Content = "#FFFF9933";
		ColorPageBtn4->Content = "#FF0099CC";
		ColorPageBtn5->Content = "#FF3366FF";
		ColorPageBtn6->Content = "#FFCC33FF";
		ColorPageBtn7->Content = "#FFCC0099";
		ColorPageBtn8->Content = "#FFFF0000";
	}
	else if (type == foxit::rdk::uiextensions::AnnotationType::StrikeOut || type == foxit::rdk::uiextensions::AnnotationType::Caret)
	{
		ColorPageBtn1->Content = "#FFFF3333";
		ColorPageBtn2->Content = "#FFFF00FF";
		ColorPageBtn3->Content = "#FF9966FF";
		ColorPageBtn4->Content = "#FF66CC33";
		ColorPageBtn5->Content = "#FFCCCC00";
		ColorPageBtn6->Content = "#FFFF9900";
		ColorPageBtn7->Content = "#FF00CCFF";
		ColorPageBtn8->Content = "#FF00CCCC";
	}
	else if (type == foxit::rdk::uiextensions::AnnotationType::Circle || type == foxit::rdk::uiextensions::AnnotationType::Square
		|| type == foxit::rdk::uiextensions::AnnotationType::Line || type == foxit::rdk::uiextensions::AnnotationType::Ink)
	{
		ColorPageBtn1->Content = "#FFFF9F40";
		ColorPageBtn2->Content = "#FF8080FF";
		ColorPageBtn3->Content = "#FFBAE94C";
		ColorPageBtn4->Content = "#FFFFF160";
		ColorPageBtn5->Content = "#FFFF4C4C";
		ColorPageBtn6->Content = "#FF669999";
		ColorPageBtn7->Content = "#FFC72DA1";
		ColorPageBtn8->Content = "#FF996666";
	}
}

void AnnotPropertyAppBar::SetRadioColorChecked(Platform::String^ colorRGBValue, foxit::rdk::uiextensions::AnnotationType type)
{
	if (type == AnnotationType::HightLight)
	{
		if (colorRGBValue == "#FFFFFF00")
		{
			ColorPageBtn1->IsChecked = true;
		}
		else if (colorRGBValue == "#FFCCFF66")
		{
			ColorPageBtn2->IsChecked = true;
		}
		else if (colorRGBValue == "#FF00FFFF")
		{
			ColorPageBtn3->IsChecked = true;
		}
		else if (colorRGBValue == "#FF99CCFF")
		{
			ColorPageBtn4->IsChecked = true;
		}
		else if (colorRGBValue == "#FF7480FC")
		{
			ColorPageBtn5->IsChecked = true;
		}
		else if (colorRGBValue == "#FFFF9999")
		{
			ColorPageBtn6->IsChecked = true;
		}
		else if (colorRGBValue == "#FF00CC66")
		{
			ColorPageBtn7->IsChecked = true;
		}
		else if (colorRGBValue == "#FF22F3B1")
		{
			ColorPageBtn8->IsChecked = true;
		}
	}
	else if (type == AnnotationType::FreeText)
	{
		if (colorRGBValue == "#FF3366CC")
		{
			ColorPageBtn1->IsChecked = true;
		}
		else if (colorRGBValue == "#FF669933")
		{
			ColorPageBtn2->IsChecked = true;
		}
		else if (colorRGBValue == "#FFCC6600")
		{
			ColorPageBtn3->IsChecked = true;
		}
		else if (colorRGBValue == "#FFCC9900")
		{
			ColorPageBtn4->IsChecked = true;
		}
		else if (colorRGBValue == "#FFCC0000")
		{
			ColorPageBtn5->IsChecked = true;
		}
		else if (colorRGBValue == "#FF336666")
		{
			ColorPageBtn6->IsChecked = true;
		}
		else if (colorRGBValue == "#FF660066")
		{
			ColorPageBtn7->IsChecked = true;
		}
		else if (colorRGBValue == "#FF000000")
		{
			ColorPageBtn8->IsChecked = true;
		}
	}
	else if (type == foxit::rdk::uiextensions::AnnotationType::UnderLine || type == foxit::rdk::uiextensions::AnnotationType::Squiggly)
	{
		if (colorRGBValue == "#FF33CC00")
		{
			ColorPageBtn1->IsChecked = true;
		}
		else if (colorRGBValue == "#FFCCCC00")
		{
			ColorPageBtn2->IsChecked = true;
		}
		else if (colorRGBValue == "#FFFF9933")
		{
			ColorPageBtn3->IsChecked = true;
		}
		else if (colorRGBValue == "#FF0099CC")
		{
			ColorPageBtn4->IsChecked = true;
		}
		else if (colorRGBValue == "#FF3366FF")
		{
			ColorPageBtn5->IsChecked = true;
		}
		else if (colorRGBValue == "#FFCC33FF")
		{
			ColorPageBtn6->IsChecked = true;
		}
		else if (colorRGBValue == "#FFCC0099")
		{
			ColorPageBtn7->IsChecked = true;
		}
		else if (colorRGBValue == "#FFFF0000")
		{
			ColorPageBtn8->IsChecked = true;
		}
	}
	else if (type == foxit::rdk::uiextensions::AnnotationType::StrikeOut || type == foxit::rdk::uiextensions::AnnotationType::Caret)
	{
		if (colorRGBValue == "#FFFF3333")
		{
			ColorPageBtn1->IsChecked = true;
		}
		else if (colorRGBValue == "#FFFF00FF")
		{
			ColorPageBtn2->IsChecked = true;
		}
		else if (colorRGBValue == "#FF9966FF")
		{
			ColorPageBtn3->IsChecked = true;
		}
		else if (colorRGBValue == "#FF66CC33")
		{
			ColorPageBtn4->IsChecked = true;
		}
		else if (colorRGBValue == "#FFCCCC00")
		{
			ColorPageBtn5->IsChecked = true;
		}
		else if (colorRGBValue == "#FFFF9900")
		{
			ColorPageBtn6->IsChecked = true;
		}
		else if (colorRGBValue == "#FF00CCFF")
		{
			ColorPageBtn7->IsChecked = true;
		}
		else if (colorRGBValue == "#FF00CCCC")
		{
			ColorPageBtn8->IsChecked = true;
		}
	}
	else if (type == foxit::rdk::uiextensions::AnnotationType::Circle || type == foxit::rdk::uiextensions::AnnotationType::Square || type == foxit::rdk::uiextensions::AnnotationType::Line || type == foxit::rdk::uiextensions::AnnotationType::Ink)
	{
		if (colorRGBValue == "#FFFF9F40")
		{
			ColorPageBtn1->IsChecked = true;
		}
		else if (colorRGBValue == "#FF8080FF")
		{
			ColorPageBtn2->IsChecked = true;
		}
		else if (colorRGBValue == "#FFBAE94C")
		{
			ColorPageBtn3->IsChecked = true;
		}
		else if (colorRGBValue == "#FFFFF160")
		{
			ColorPageBtn4->IsChecked = true;
		}
		else if (colorRGBValue == "#FFFF4C4C")
		{
			ColorPageBtn5->IsChecked = true;
		}
		else if (colorRGBValue == "#FF669999")
		{
			ColorPageBtn6->IsChecked = true;
		}
		else if (colorRGBValue == "#FFC72DA1")
		{
			ColorPageBtn7->IsChecked = true;
		}
		else if (colorRGBValue == "#FF996666")
		{
			ColorPageBtn8->IsChecked = true;
		}
	}
}


void AnnotPropertyAppBar::AppBarButton_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{
	UIExtensionsManager^ uiMgr = (UIExtensionsManager^)View->ExtensionsManager;
	if (View && uiMgr->CreateTypeOfAnnot == AnnotCreateType::Pencil)
	{
		auto _pAnnotHandler = uiMgr->getAnnotMgr()->GetAnnotHandlerByType(GetAnnotType(AnnotCreateType::Pencil));
		if (_pAnnotHandler)
			_pAnnotHandler->CreateAnnotAfterSelectText();
	}
}


void AnnotPropertyAppBar::SetPropertyToolIsEnabled(bool isEnable)
{
	ColorPageBtn1->IsEnabled = isEnable;
	ColorPageBtn2->IsEnabled = isEnable;
	ColorPageBtn3->IsEnabled = isEnable;
	ColorPageBtn4->IsEnabled = isEnable;
	ColorPageBtn5->IsEnabled = isEnable;
	ColorPageBtn6->IsEnabled = isEnable;
	ColorPageBtn7->IsEnabled = isEnable;
	ColorPageBtn8->IsEnabled = isEnable;

	OpacityPageBtn1->IsEnabled = isEnable;
	OpacityPageBtn2->IsEnabled = isEnable;
	OpacityPageBtn3->IsEnabled = isEnable;
	OpacityPageBtn4->IsEnabled = isEnable;
}

void AnnotPropertyAppBar::CloseBorderListPopup()
{

}


void AnnotPropertyAppBar::SetStampPopupClosed()
{
	stampIconPopup->IsOpen = false;
}


void foxit::rdk::uiextensions::AnnotPropertyAppBar::OnValueChanged(Platform::Object ^sender, Windows::UI::Xaml::Controls::Primitives::RangeBaseValueChangedEventArgs ^e)
{
	UIExtensionsManager^ uiMgr = (UIExtensionsManager^)View->ExtensionsManager;
	tbLiness->Text = sliderLiness->Value.ToString() + "pt";

	IBaseAnnot^ annot = uiMgr->getAnnotMgr()->GetSelectedAnnot();
	double borderWidth = sliderLiness->Value;

	if (annot)
	{
		annot->IsModifyProperty = true;
		annot->BorderWidth = borderWidth;
	}
	uiMgr->getAnnotMgr()->GetAnnotSetting()->SetAnnotDrawBorderWidth((int)borderWidth);
}

void AnnotPropertyAppBar::InitControlSize()
{
	Windows::Foundation::Rect winRect = Windows::UI::Xaml::Window::Current->Bounds;
	mainGrid->Width = winRect.Width;
}

void AnnotPropertyAppBar::SetFreeTextPropertyShow(bool isShow)
{
	if (isShow)
	{
		colorPanel->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
		opacityPanel->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
	}
	else
	{
		colorPanel->Visibility = Windows::UI::Xaml::Visibility::Visible;
		opacityPanel->Visibility = Windows::UI::Xaml::Visibility::Visible;
	}
}
