﻿/**
 * Copyright (C) 2003-2018, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit Mobile PDF SDK to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */


#include "pch.h"
#include "AnnotToolAppBar.xaml.h"
#include "../MainPageAppBar.xaml.h"
#include "../FloatWindow/AnnotStampUserControl.xaml.h"
#include "../FloatWindow/NoteStyleUserControl.xaml.h"
#include "AnnotPropertyAppBar.xaml.h"
#include "../FloatWindow/PhoneAnnotTipControl.xaml.h"

using namespace foxit::rdk::uiextensions;

using namespace Platform;
using namespace Windows::Foundation;
using namespace Windows::Foundation::Collections;
using namespace Windows::UI::Xaml;
using namespace Windows::UI::Xaml::Controls;
using namespace Windows::UI::Xaml::Controls::Primitives;
using namespace Windows::UI::Xaml::Data;
using namespace Windows::UI::Xaml::Input;
using namespace Windows::UI::Xaml::Media;
using namespace Windows::UI::Xaml::Navigation;

// The User Control item template is documented at http://go.microsoft.com/fwlink/?LinkId=234236

AnnotToolAppBar::AnnotToolAppBar()
{
	InitializeComponent();

	this->Loaded += ref new Windows::UI::Xaml::RoutedEventHandler(this, &AnnotToolAppBar::OnLoaded);
	this->Unloaded += ref new Windows::UI::Xaml::RoutedEventHandler(this, &AnnotToolAppBar::OnUnloaded);

	this->SizeChanged += ref new Windows::UI::Xaml::SizeChangedEventHandler(this, &foxit::rdk::uiextensions::AnnotToolAppBar::OnSizeChanged);
}

MainPageAppBar^ AnnotToolAppBar::MainAppBar::get()
{
	return m_MainAppBarWeakRef.Resolve<MainPageAppBar>();
}

void AnnotToolAppBar::OnLoaded(Platform::Object ^sender, Windows::UI::Xaml::RoutedEventArgs ^e)
{
	proprtyPopup = ref new Windows::UI::Xaml::Controls::Primitives::Popup();
	selectcolor = ref new Windows::UI::Xaml::Media::SolidColorBrush(Utility::ConvertFromString("#FF14A6F0"));
	normalcolor = ref new Windows::UI::Xaml::Media::SolidColorBrush(Windows::UI::Colors::Transparent);

	auto _mainAppbar = GetVisualParent<MainPageAppBar^>(this);
	if (_mainAppbar)
		m_MainAppBarWeakRef = _mainAppbar;

	auto _winrect = Windows::UI::Xaml::Window::Current->Bounds;
	LandMobileMoreCommans->Width = _winrect.Width;

	noteStyleControl->RelateToView(MainAppBar->View);
	annotStampControl->RelativePageView(MainAppBar->View);

	UpdateUndoControlState();
	InitNoteIcon();

	m_HideLableTimer = ref new Windows::UI::Xaml::DispatcherTimer();
	Windows::Foundation::TimeSpan _HideTimerTimeSpan;
	_HideTimerTimeSpan.Duration = 3 * 10000000; 
	m_HideLableTimer->Interval = _HideTimerTimeSpan;
	m_HideLableTimer->Tick += ref new Windows::Foundation::EventHandler<Platform::Object ^>(this, &foxit::rdk::uiextensions::AnnotToolAppBar::OnTick);

	m_HideMobileAnnotTipTimer = ref new Windows::UI::Xaml::DispatcherTimer();
	Windows::Foundation::TimeSpan _HideAnnotTipTimerTimeSpan;
	_HideAnnotTipTimerTimeSpan.Duration = 2 * 10000000; 
	m_HideMobileAnnotTipTimer->Interval = _HideAnnotTipTimerTimeSpan;
	m_HideMobileAnnotTipTimer->Tick += ref new Windows::Foundation::EventHandler<Platform::Object ^>(this, &foxit::rdk::uiextensions::AnnotToolAppBar::OnHideTipTick);

	auto pUndo = UndoFactory::GetUndoInstance();
	m_AfterUndoEventToken = pUndo->AfterUndo += ref new foxit::rdk::uiextensions::UndoEventHandler(this, &foxit::rdk::uiextensions::AnnotToolAppBar::OnAfterUndo);
	m_AfterRedoEventToken = pUndo->AfterRedo += ref new foxit::rdk::uiextensions::UndoEventHandler(this, &foxit::rdk::uiextensions::AnnotToolAppBar::OnAfterRedo);
	m_OperaListChangedUndoRedoEventToken = pUndo->OperaListChanged += ref new foxit::rdk::uiextensions::UndoOperaListChangedEventHandler(this, &foxit::rdk::uiextensions::AnnotToolAppBar::OnOperaListChanged);

	phoneAnnotTipControl->Loaded += ref new Windows::UI::Xaml::RoutedEventHandler(this, &foxit::rdk::uiextensions::AnnotToolAppBar::OnPhoneTipLoaded);

	if (Windows::System::Profile::AnalyticsInfo::VersionInfo->DeviceFamily == "Windows.Desktop")
	{
		scrollTollbar->ManipulationMode = Windows::UI::Xaml::Input::ManipulationModes::All;
		scrollTollbar->ManipulationDelta += ref new Windows::UI::Xaml::Input::ManipulationDeltaEventHandler(this, &foxit::rdk::uiextensions::AnnotToolAppBar::scrollTollbar_ManipulationDelta);
	}
}

void AnnotToolAppBar::StartHideLabelTick()
{
	if (m_HideLableTimer)
	{
		m_HideLableTimer->Stop();
		m_HideLableTimer->Start();
	}
}

void AnnotToolAppBar::OnUnloaded(Platform::Object ^sender, Windows::UI::Xaml::RoutedEventArgs ^e)
{
	auto pUndo = UndoFactory::GetUndoInstance();
	if (m_AfterUndoEventToken.Value)
	{
		pUndo->AfterUndo -= m_AfterUndoEventToken;
		m_AfterUndoEventToken.Value = 0;
	}
	if (m_AfterRedoEventToken.Value)
	{
		pUndo->AfterRedo -= m_AfterRedoEventToken;
		m_AfterRedoEventToken.Value = 0;
	}
	if (m_OperaListChangedUndoRedoEventToken.Value)
	{
		pUndo->OperaListChanged -= m_OperaListChangedUndoRedoEventToken;
		m_OperaListChangedUndoRedoEventToken.Value = 0;
	}

	m_MainAppBarWeakRef = nullptr;

	if (m_HideLableTimer && m_HideLableTimer->IsEnabled)
	{
		m_HideLableTimer->Stop();
	}
	if (m_HideMobileAnnotTipTimer && m_HideMobileAnnotTipTimer->IsEnabled)
	{
		m_HideMobileAnnotTipTimer->Stop();
	}
}

void AnnotToolAppBar::CreateAnnot(foxit::rdk::uiextensions::AnnotCreateType type)
{
	if (!MainAppBar->View)
		return;
	UIExtensionsManager^ uiMgr = (UIExtensionsManager^)MainAppBar->View->ExtensionsManager;
	uiMgr->SetAnnotStampClosed();
	uiMgr->AddAnnotByType(type);

	SetAnnotPropertButtonEnable(true);

	if (!uiMgr->IsShowAnnot)
	{
		uiMgr->IsShowAnnot = true;
		MainAppBar->View->Redraw();
	}
}

void AnnotToolAppBar::SetAppBarIsCompact(bool isCompact)
{
	if (isCompact)
	{
		gridBar->Height = 44;
		MainAppBar->SetMainAppBarCompact(true);
	}
	if (Windows::System::Profile::AnalyticsInfo::VersionInfo->DeviceFamily == L"Windows.Mobile")
	{
		auto size = WindowsMobileCommandBar->Children->Size;
		for (unsigned int i = 0; i < size; i++)
		{
			auto _item = WindowsMobileCommandBar->Children->GetAt(i);
			auto _ICommandBarElement = dynamic_cast<Windows::UI::Xaml::Controls::ICommandBarElement^>(_item);
			if (_ICommandBarElement)
			{
				_ICommandBarElement->IsCompact = true;
			}
		}
		SecondaryMobileCommandsPopup->IsOpen = false;
		SecondaryMobileHorCommandsPopup->IsOpen = false;
	}
	else
	{
		auto size = DesktopCommandBarPanel->Children->Size;
		for (unsigned int i = 0; i < size; i++)
		{
			auto _item = DesktopCommandBarPanel->Children->GetAt(i);
			auto _ICommandBarElement = dynamic_cast<Windows::UI::Xaml::Controls::ICommandBarElement^>(_item);
			if (_ICommandBarElement)
			{
				_ICommandBarElement->IsCompact = isCompact;
			}
		}
		size = DesktopCommandBarNarrowPanel->Children->Size;
		for (unsigned int i = 0; i < size; i++)
		{
			auto _item = DesktopCommandBarNarrowPanel->Children->GetAt(i);
			auto _ICommandBarElement = dynamic_cast<Windows::UI::Xaml::Controls::ICommandBarElement^>(_item);
			if (_ICommandBarElement)
			{
				_ICommandBarElement->IsCompact = isCompact;
			}
		}

		size = DesktopBarNarrowAnnotToolPanel->Children->Size;
		for (unsigned int i = 0; i < size; i++)
		{
			auto _item = DesktopBarNarrowAnnotToolPanel->Children->GetAt(i);
			auto _ICommandBarElement = dynamic_cast<Windows::UI::Xaml::Controls::ICommandBarElement^>(_item);
			if (_ICommandBarElement)
			{
				_ICommandBarElement->IsCompact = isCompact;
			}
		}

		size = DesktopBarNarrowPropertyPanel->Children->Size;
		for (unsigned int i = 0; i < size; i++)
		{
			auto _item = DesktopBarNarrowPropertyPanel->Children->GetAt(i);
			auto _ICommandBarElement = dynamic_cast<Windows::UI::Xaml::Controls::ICommandBarElement^>(_item);
			if (_ICommandBarElement)
			{
				_ICommandBarElement->IsCompact = isCompact;
			}
		}
	}
}

void AnnotToolAppBar::UpdateUndoControlState()
{
	auto pUndo = UndoFactory::GetUndoInstance();
	if (MainAppBar && MainAppBar->View->GetDoc())
	{
		UIExtensionsManager^ uiMgr = (UIExtensionsManager^)MainAppBar->View->ExtensionsManager;
		RadioUndo->IsEnabled = pUndo->CanUndo(uiMgr);
		RadioRedo->IsEnabled = pUndo->CanRedo(uiMgr);
		RadioUndoNarrow->IsEnabled = pUndo->CanUndo(uiMgr);
		RadioRedoNarrow->IsEnabled = pUndo->CanRedo(uiMgr);
	}
	else
	{
		RadioUndo->IsEnabled = false;
		RadioRedo->IsEnabled = false;
		RadioUndoNarrow->IsEnabled = false;
		RadioRedoNarrow->IsEnabled = false;
	}
}

void AnnotToolAppBar::RadioHightLight_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{
	AnnotToolClick(sender, foxit::rdk::uiextensions::AnnotCreateType::HightLight);
}


void AnnotToolAppBar::RadioUnderLine_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{
	AnnotToolClick(sender, foxit::rdk::uiextensions::AnnotCreateType::UnderLine);
}


void AnnotToolAppBar::RadioStrikeOut_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{
	AnnotToolClick(sender, foxit::rdk::uiextensions::AnnotCreateType::StrikeOut);
}


void AnnotToolAppBar::RadioNote_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{
	SetAllPopupClosed();
	AnnotToolClick(sender, foxit::rdk::uiextensions::AnnotCreateType::Text);
}

void AnnotToolAppBar::exitAnnotTool_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{
	UIExtensionsManager^ uiMgr = (UIExtensionsManager^)MainAppBar->View->ExtensionsManager;
	if (uiMgr->IsPopupShow())
	{
		return;
	}

	ResetAppBarBG();
	SetAllPopupClosed();
	if (MainAppBar)
	{
		if (uiMgr->CreateTypeOfAnnot == foxit::rdk::uiextensions::AnnotCreateType::Eraser)
		{
			MainAppBar->RestartTimeTick();
			uiMgr->CollectEraserOpration();
		}

		uiMgr->SaveOrCreateNoteAnnot();
		uiMgr->CreateGraphAnnotBySliderPage();

		uiMgr->CreateTypeOfAnnot = foxit::rdk::uiextensions::AnnotCreateType::None;
		uiMgr->ResetDefaultAppBarButton();
		uiMgr->ExitAnnotSelect();
		uiMgr->SetAnnotCanvasVisible(false);
	}
	SetAppBarIsCompact(true);
	MainAppBar->SetMainAppBarCompact(true);
	MainAppBar->RestartTimeTick();
	HideAllLabel->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
	ShowAllLabel->Visibility = Windows::UI::Xaml::Visibility::Visible;
	SetProertyBtnEnabled();
	if (m_HideMobileAnnotTipTimer)
	{
		m_HideMobileAnnotTipTimer->Stop();
	}
	if (m_HideLableTimer)
	{
		m_HideLableTimer->Stop();
	}
}

void AnnotToolAppBar::RadioSquiggly_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{
	AnnotToolClick(sender, foxit::rdk::uiextensions::AnnotCreateType::Squiggly);
}

void AnnotToolAppBar::RadioAreaHightLight_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{
	AnnotToolClick(sender, foxit::rdk::uiextensions::AnnotCreateType::AreaHighLight);
}

void AnnotToolAppBar::RadioInk_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{
	UIExtensionsManager^ uiMgr = (UIExtensionsManager^)MainAppBar->View->ExtensionsManager;
	if (MainAppBar->View && uiMgr->IsPopupShow())
	{
		AppBarToggleButton^ button = dynamic_cast<AppBarToggleButton^>(sender);
		if (button)
			button->IsChecked = false;
		return;
	}
	uiMgr->SaveOrCreateNoteAnnot();
	uiMgr->ExitAnnotSelect();

	uiMgr->CreateGraphAnnotBySliderPage();
	AnnotToolClick(sender, foxit::rdk::uiextensions::AnnotCreateType::Pencil);
}

void AnnotToolAppBar::RadioSquare_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{
	UIExtensionsManager^ uiMgr = (UIExtensionsManager^)MainAppBar->View->ExtensionsManager;
	if (Windows::System::Profile::AnalyticsInfo::VersionInfo->DeviceFamily == L"Windows.Desktop")
	{
		ResetAppBarBG();
		uiMgr->CreateGraphAnnotBySliderPage();
		uiMgr->ExitAnnotSelect();
		CreateAnnot(foxit::rdk::uiextensions::AnnotCreateType::Square);
		if (!MainAppBar->View)
			return;

		SetAnnotToolPopupClosed();
		InitCurGraphAnnot(foxit::rdk::uiextensions::AnnotCreateType::Square);
	}
	else
	{
		AnnotToolClick(sender, foxit::rdk::uiextensions::AnnotCreateType::Square);
	}
}


void AnnotToolAppBar::RadioCircle_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{
	UIExtensionsManager^ uiMgr = (UIExtensionsManager^)MainAppBar->View->ExtensionsManager;

	if (Windows::System::Profile::AnalyticsInfo::VersionInfo->DeviceFamily == L"Windows.Desktop")
	{
		uiMgr->CreateGraphAnnotBySliderPage();
		ResetAppBarBG();
		uiMgr->ExitAnnotSelect();
		CreateAnnot(foxit::rdk::uiextensions::AnnotCreateType::Circle);
		if (!MainAppBar->View)
			return;

		SetAnnotToolPopupClosed();
		InitCurGraphAnnot(foxit::rdk::uiextensions::AnnotCreateType::Circle);
	}
	else
	{
		AnnotToolClick(sender, foxit::rdk::uiextensions::AnnotCreateType::Circle);
	}
}


void AnnotToolAppBar::RadioLine_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{
	UIExtensionsManager^ uiMgr = (UIExtensionsManager^)MainAppBar->View->ExtensionsManager;

	if (Windows::System::Profile::AnalyticsInfo::VersionInfo->DeviceFamily == L"Windows.Desktop")
	{
		uiMgr->CreateGraphAnnotBySliderPage();
		ResetAppBarBG();
		uiMgr->ExitAnnotSelect();
		CreateAnnot(foxit::rdk::uiextensions::AnnotCreateType::Line);
		if (!MainAppBar->View)
			return;

		SetAnnotToolPopupClosed();
		InitCurGraphAnnot(foxit::rdk::uiextensions::AnnotCreateType::Line);
	}
	else
	{
		AnnotToolClick(sender, foxit::rdk::uiextensions::AnnotCreateType::Line);
	}
}

void AnnotToolAppBar::OnSizeChanged(Platform::Object ^sender, Windows::UI::Xaml::SizeChangedEventArgs ^e)
{
	auto _winrect = Windows::UI::Xaml::Window::Current->Bounds;

	if (Windows::System::Profile::AnalyticsInfo::VersionInfo->DeviceFamily == "Windows.Mobile")
	{
		InitAnnotMobileToolLayout();
		ResetAppBarBG();
		SetAnnotMobileMoreBtnSelected();
		if (SecondaryMobileCommandsPopup->IsOpen || SecondaryMobileHorCommandsPopup->IsOpen)
		{
			if (_winrect.Width > _winrect.Height)
			{
				SecondaryMobileCommandsPopup->IsOpen = false;
				SecondaryMobileHorCommandsPopup->HorizontalOffset = 0;
				SecondaryMobileHorCommandsPopup->VerticalOffset = -140 + this->ActualHeight;
				SecondaryMobileHorCommandsPopup->IsOpen = true;
			}
			else
			{
				SecondaryMobileCommandsPopup->IsOpen = true;
				double height = 280;
				Windows::Foundation::Collections::IVectorView<String^>^ langus = Windows::Globalization::ApplicationLanguages::Languages;
				if (langus->Size > 0 && langus->GetAt(0) == "en-US")
				{
					height = 295;
				}
				LandMobileMoreCommans->Height = height;
				SecondaryMobileCommandsPopup->VerticalOffset = -height + this->ActualHeight;
				SecondaryMobileCommandsPopup->HorizontalOffset = 0;
				SecondaryMobileHorCommandsPopup->IsOpen = false;
			}
		}
	}
}


void AnnotToolAppBar::InitAnnotMobileToolLayout()
{
	auto _winrect = Windows::UI::Xaml::Window::Current->Bounds;
	LandMobileMoreCommans->Width = _winrect.Width;
	PivotMobileMoreCommans->Width = _winrect.Width;

	if (SecondaryMobileCommandsPopup->IsOpen)
	{
		SecondaryMobileCommandsPopup->VerticalOffset = -LandMobileMoreCommans->Height + this->ActualHeight;
	}
	else if (SecondaryMobileHorCommandsPopup->IsOpen)
	{
		SecondaryMobileHorCommandsPopup->VerticalOffset = -PivotMobileMoreCommans->Height + this->ActualHeight;
	}
}

void AnnotToolAppBar::imgMoreMain_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{
	UIExtensionsManager^ uiMgr = (UIExtensionsManager^)MainAppBar->View->ExtensionsManager;
	if (uiMgr->IsPopupShow())
		return;
	uiMgr->SaveOrCreateNoteAnnot();
	auto _winrect = Windows::UI::Xaml::Window::Current->Bounds;
	if (_winrect.Width < _winrect.Height)
	{
		SecondaryMobileHorCommandsPopup->IsOpen = false;
		if (SecondaryMobileCommandsPopup->IsOpen)
		{
			SecondaryMobileCommandsPopup->IsOpen = false;
		}
		else
		{
			InitAnnotMobileToolLayout();

			Windows::Foundation::Collections::IVectorView<String^>^ langus = Windows::Globalization::ApplicationLanguages::Languages;
			String^ lang = langus->GetAt(0);
			double height = 280;
			if (langus->Size > 0 && langus->GetAt(0) == "en-US")
			{
				height = 295;
			}

			LandMobileMoreCommans->Height = height;
			SecondaryMobileCommandsPopup->HorizontalOffset = 0;
			double voffset = -LandMobileMoreCommans->ActualHeight + this->ActualHeight;
			SecondaryMobileCommandsPopup->VerticalOffset = -height + this->ActualHeight;
			SecondaryMobileCommandsPopup->IsOpen = true;

			SetAnnotMobileMoreBtnSelected();
		}
	}
	else
	{
		SecondaryMobileCommandsPopup->IsOpen = false;
		if (SecondaryMobileHorCommandsPopup->IsOpen)
		{
			SecondaryMobileHorCommandsPopup->IsOpen = false;
		}
		else
		{
			InitAnnotMobileToolLayout();
			SecondaryMobileHorCommandsPopup->HorizontalOffset = 0;
			SecondaryMobileHorCommandsPopup->VerticalOffset = -140 + this->ActualHeight;
			SecondaryMobileHorCommandsPopup->IsOpen = true;
			SetAnnotMobileMoreBtnSelected();
		}

		if (uiMgr->CreateTypeOfAnnot != foxit::rdk::uiextensions::AnnotCreateType::None)
			SetAnnotBtnClick(uiMgr->CreateTypeOfAnnot, true);
	}
	uiMgr->SetUnRedoPhoneAppBarVisible(false);
}


void AnnotToolAppBar::WindowsDesktopCommandBar_Opening(Platform::Object^ sender, Platform::Object^ e)
{
	if (AnnotToolListPopup->IsOpen)
	{
		AnnotToolListPopup->IsOpen = false;
	}
}


void foxit::rdk::uiextensions::AnnotToolAppBar::RadioArrow_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{
	UIExtensionsManager^ uiMgr = (UIExtensionsManager^)MainAppBar->View->ExtensionsManager;
	if (Windows::System::Profile::AnalyticsInfo::VersionInfo->DeviceFamily == L"Windows.Desktop")
	{
		uiMgr->CreateGraphAnnotBySliderPage();
		ResetAppBarBG();
		uiMgr->ExitAnnotSelect();
		CreateAnnot(foxit::rdk::uiextensions::AnnotCreateType::Arrow);
		if (!MainAppBar->View)
			return;

		SetAnnotToolPopupClosed();
		InitCurGraphAnnot(foxit::rdk::uiextensions::AnnotCreateType::Arrow);
	}
	else
	{
		AnnotToolClick(sender, foxit::rdk::uiextensions::AnnotCreateType::Arrow);
	}
}


void foxit::rdk::uiextensions::AnnotToolAppBar::RadioEraser_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{
	UIExtensionsManager^ uiMgr = (UIExtensionsManager^)MainAppBar->View->ExtensionsManager;
	if (uiMgr->IsPopupShow())
	{
		AppBarToggleButton^ button = dynamic_cast<AppBarToggleButton^>(sender);
		if (button)
			button->IsChecked = false;
		return;
	}
	ResetAppBarBG();
	SetAllPopupClosed();

	if (Windows::System::Profile::AnalyticsInfo::VersionInfo->DeviceFamily == L"Windows.Mobile")
	{
		SecondaryMobileCommandsPopup->IsOpen = false;
		SecondaryMobileHorCommandsPopup->IsOpen = false;
		uiMgr->SetUnRedoPhoneAppBarVisible(true);
	}
	
	uiMgr->CreateGraphAnnotBySliderPage();
	if (uiMgr->CreateTypeOfAnnot == foxit::rdk::uiextensions::AnnotCreateType::Eraser)
	{
		MainAppBar->RestartTimeTick();
		if (Windows::System::Profile::AnalyticsInfo::VersionInfo->DeviceFamily == L"Windows.Desktop")
		{
			SetAnnotBtnClick(foxit::rdk::uiextensions::AnnotCreateType::Eraser, false);
		}
		uiMgr->CreateTypeOfAnnot = foxit::rdk::uiextensions::AnnotCreateType::None;
		uiMgr->CollectEraserOpration();
	}
	else
	{
		uiMgr->ExitAnnotSelect();
		CreateAnnot(foxit::rdk::uiextensions::AnnotCreateType::Eraser);
		if (Windows::System::Profile::AnalyticsInfo::VersionInfo->DeviceFamily == L"Windows.Desktop")
		{
			SetAnnotBtnClick(foxit::rdk::uiextensions::AnnotCreateType::Eraser, true);
		}
		else
		{
			uiMgr->ShowPhonAnnotPropertyIcon(false);
		}
		ResetSetMobileAnnotBarCompact();

		if (!MainAppBar->View)
			return;

		MainAppBar->StopHideTimeTick();
	}
}


void foxit::rdk::uiextensions::AnnotToolAppBar::OnAfterUndo(Platform::Object ^sender, Platform::Object ^args)
{
	UpdateUndoControlState();
}


void foxit::rdk::uiextensions::AnnotToolAppBar::OnAfterRedo(Platform::Object ^sender, Platform::Object ^args)
{
	UpdateUndoControlState();
}

void foxit::rdk::uiextensions::AnnotToolAppBar::OnOperaListChanged(Platform::Object ^sender, Platform::Object ^args)
{
	UpdateUndoControlState();
}

void foxit::rdk::uiextensions::AnnotToolAppBar::RadioUndo_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{
	UIExtensionsManager^ uiMgr = (UIExtensionsManager^)MainAppBar->View->ExtensionsManager;
	if (uiMgr->IsPopupShow())
	{
		return;
	}


	SetAllPopupClosed();
	if (uiMgr->CreateTypeOfAnnot == foxit::rdk::uiextensions::AnnotCreateType::Eraser)
	{
		uiMgr->CreateTypeOfAnnot = foxit::rdk::uiextensions::AnnotCreateType::None;
		RadioEraser->IsChecked = false;
	}

	uiMgr->ExitAnnotSelect();
	if (uiMgr->CreateTypeOfAnnot == foxit::rdk::uiextensions::AnnotCreateType::FreeText)
	{
		uiMgr->CreateGraphAnnotBySliderPage();
		if (!uiMgr->getAnnotMgr()->GetAnnotSetting()->GetFreeTextIsContinueOpera())
			ResetAppBarBG();
	}

	auto pUndo = UndoFactory::GetUndoInstance();

	if (MainAppBar && MainAppBar->View->GetDoc())
		pUndo->Undo((UIExtensionsManager^)MainAppBar->View->ExtensionsManager);
}


void foxit::rdk::uiextensions::AnnotToolAppBar::RadioRedo_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{
	UIExtensionsManager^ uiMgr = (UIExtensionsManager^)MainAppBar->View->ExtensionsManager;
	if (uiMgr->IsPopupShow())
		return;


	SetAllPopupClosed();
	if (uiMgr->CreateTypeOfAnnot == foxit::rdk::uiextensions::AnnotCreateType::Eraser)
	{
		uiMgr->CreateTypeOfAnnot = foxit::rdk::uiextensions::AnnotCreateType::None;
		RadioEraser->IsChecked = false;
	}
	uiMgr->ExitAnnotSelect();
	if (uiMgr->CreateTypeOfAnnot == foxit::rdk::uiextensions::AnnotCreateType::FreeText)
	{
		uiMgr->CreateGraphAnnotBySliderPage();
		if (!uiMgr->getAnnotMgr()->GetAnnotSetting()->GetFreeTextIsContinueOpera())
			ResetAppBarBG();
	}
	auto pUndo = UndoFactory::GetUndoInstance();

	if (MainAppBar && MainAppBar->View->GetDoc())
		pUndo->Redo((UIExtensionsManager^)MainAppBar->View->ExtensionsManager);
}

void foxit::rdk::uiextensions::AnnotToolAppBar::ResetAppBarBG()
{
	if (Windows::System::Profile::AnalyticsInfo::VersionInfo->DeviceFamily == L"Windows.Desktop")
	{
		RadioHightLight->IsChecked = false;
		RadioUnderLine->IsChecked = false;
		RadioSquiggly->IsChecked = false;
		RadioStrikeOut->IsChecked = false;
		RadioAreaHightLight->IsChecked = false;
		RadioNote->IsChecked = false;
		RadioInk->IsChecked = false;
		RadioEraser->IsChecked = false;
		RadioStamp->IsChecked = false;
		appBarBorder->IsChecked = false;
		appBarProerty->IsChecked = false;
		appBarContinueOpera->IsChecked = false;
		RadioShapeGroup->IsChecked = false;
		appBarBorder->IsEnabled = false;
		appBarProerty->IsEnabled = false;
		appBarContinueOpera->IsEnabled = false;
		RadioNoteDown->IsChecked = false;
		RadioGroupDown->IsChecked = false;
		RadioStampDown->IsChecked = false;
		RadioFreeText->IsChecked = false;
		RadioInsertText->IsChecked = false;
		RadioReplaceText->IsChecked = false;

		RadioHightLightNarrow->IsChecked = false;
		RadioUnderLineNarrow->IsChecked = false;
		RadioSquigglyNarrow->IsChecked = false;
		RadioStrikeOutNarrow->IsChecked = false;
		RadioAreaHightLightNarrow->IsChecked = false;
		RadioNoteNarrow->IsChecked = false;
		RadioInkNarrow->IsChecked = false;
		RadioEraserNarrow->IsChecked = false;
		RadioStampNarrow->IsChecked = false;
		appBarBorderNarrow->IsChecked = false;
		appBarProertyNarrow->IsChecked = false;
		appBarContinueOperaNarrow->IsChecked = false;
		RadioShapeGroupNarrow->IsChecked = false;
		appBarBorderNarrow->IsEnabled = false;
		appBarProertyNarrow->IsEnabled = false;
		appBarContinueOperaNarrow->IsEnabled = false;
		RadioNoteDownNarrow->IsChecked = false;
		RadioGroupDownNarrow->IsChecked = false;
		RadioStampDownNarrow->IsChecked = false;
		RadioFreeTextNarrow->IsChecked = false;
		RadioInsertTextNarrow->IsChecked = false;
		RadioReplaceTextNarrow->IsChecked = false;

		appbarSetting->IsChecked = false;
		appbarSettingNarrow->IsChecked = false;
	}
	else
	{
		RadioHightLightMobile->IsChecked = false;
		RadioStrikeOutMobile->IsChecked = false;
		RadioNoteMobile->IsChecked = false;
		RadioHightLightMobileMore->IsChecked = false;
		RadioUnderLineMobile->IsChecked = false;
		RadioUnderLineMobileMore->IsChecked = false;
		RadioAreaHightLightMobileMore->IsChecked = false;
		RadioSquigglyMobile->IsChecked = false;
		RadioStrikeOutMobileMore->IsChecked = false;
		RadioAreaHightLightMobile->IsChecked = false;
		RadioNoteMobileMore->IsChecked = false;
		RadioPencilMobileMore->IsChecked = false;
		RadioEraserMobile->IsChecked = false;
		RadioSquareMobile->IsChecked = false;
		RadioCircleMobile->IsChecked = false;
		RadioLineMobile->IsChecked = false;
		RadioArrowMobile->IsChecked = false;
		RadioStampMobile->IsChecked = false;
		RadioFreeTextMobile->IsChecked = false;
		RadioInsertTextMobile->IsChecked = false;
		RadioReplaceTextMobile->IsChecked = false;

		RadioHightLightMobileMorePor->IsChecked = false;
		RadioUnderLineMobileMorePor->IsChecked = false;;
		RadioAreaHightLightMobileMorePor->IsChecked = false;
		RadioSquigglyMobilePor->IsChecked = false;
		RadioStrikeOutMobileMorePor->IsChecked = false;
		RadioNoteMobileMorePor->IsChecked = false;
		RadioPencilMobileMorePor->IsChecked = false;
		RadioEraserMobilePor->IsChecked = false;
		RadioSquareMobilePor->IsChecked = false;
		RadioCircleMobilePor->IsChecked = false;
		RadioLineMobilePor->IsChecked = false;
		RadioArrowMobilePor->IsChecked = false;
		RadioStampMobilePor->IsChecked = false;
		RadioFreeTextMobilePor->IsChecked = false;
		RadioInsertTextMobilePor->IsChecked = false;
		RadioReplaceTextMobilePor->IsChecked = false;
	}
}

void AnnotToolAppBar::SetStampPopupClosed()
{
	stampIconPopup->IsOpen = false;
	if (stampPhonePopup)
		stampPhonePopup->IsOpen = false;
	if (WindowsDesktopCommandBar->Visibility == Windows::UI::Xaml::Visibility::Visible)
	{
		RadioStampDown->IsChecked = false;
	}
	else
	{
		RadioStampDownNarrow->IsChecked = false;
	}
}

void AnnotToolAppBar::SetAnnotToolPopupClosed()
{
	if (AnnotToolListPopup->IsOpen)
		AnnotToolListPopup->IsOpen = false;

	if (WindowsDesktopCommandBar->Visibility == Windows::UI::Xaml::Visibility::Visible)
	{
		RadioGroupDown->IsChecked = false;
	}
	else
	{
		RadioGroupDownNarrow->IsChecked = false;
	}
}


void AnnotToolAppBar::SetAnnotPropertButtonEnable(bool isEnable)
{
	UIExtensionsManager^ uiMgr = (UIExtensionsManager^)MainAppBar->View->ExtensionsManager;
	foxit::rdk::uiextensions::AnnotCreateType type = uiMgr->CreateTypeOfAnnot;
	
	if (type == foxit::rdk::uiextensions::AnnotCreateType::Circle || type == foxit::rdk::uiextensions::AnnotCreateType::Square
		|| type == foxit::rdk::uiextensions::AnnotCreateType::Line || type == foxit::rdk::uiextensions::AnnotCreateType::Arrow
		|| type == foxit::rdk::uiextensions::AnnotCreateType::Pencil)
	{
		appBarBorder->IsEnabled = isEnable;
		appBarBorderNarrow->IsEnabled = isEnable;
	}
	else
	{
		appBarBorder->IsEnabled = false;
		appBarBorderNarrow->IsEnabled = false;
	}

	if (type == foxit::rdk::uiextensions::AnnotCreateType::Eraser || type == foxit::rdk::uiextensions::AnnotCreateType::Text
		|| type == foxit::rdk::uiextensions::AnnotCreateType::Stamp)
	{
		appBarProerty->IsEnabled = false;
		appBarProertyNarrow->IsEnabled = false;
	}
	else
	{
		appBarProerty->IsEnabled = isEnable;
		appBarProertyNarrow->IsEnabled = isEnable;
		InitCurCreateAnnotTypeColor();
	}

	if (type != foxit::rdk::uiextensions::AnnotCreateType::Eraser && type != foxit::rdk::uiextensions::AnnotCreateType::Pencil)
	{
		appBarContinueOpera->IsEnabled = isEnable;
		appBarContinueOperaNarrow->IsEnabled = isEnable;
	}
	else
	{
		appBarContinueOpera->IsEnabled = false;
		appBarContinueOperaNarrow->IsEnabled = false;
	}
	if (appBarContinueOpera->IsEnabled)
		InitAnnotIsContinueOpera(type);
}


void foxit::rdk::uiextensions::AnnotToolAppBar::ShowAllLabel_Tapped(Platform::Object^ sender, Windows::UI::Xaml::Input::TappedRoutedEventArgs^ e)
{
	UIExtensionsManager^ uiMgr = (UIExtensionsManager^)MainAppBar->View->ExtensionsManager;
	Windows::Foundation::Collections::IVectorView<String^>^ langus = Windows::Globalization::ApplicationLanguages::Languages;
	String^ lang = langus->GetAt(0);
	if (langus->Size > 0 && langus->GetAt(0) == "en-US" || langus->Size > 0 && langus->GetAt(0) == "zh-Hant-TW")
	{
		gridBar->Height = 75;
	}
	else
	{
		gridBar->Height = 60;
	}
	MainAppBar->SetMainAppBarCompact(false);
	ResetAppBarBG();
	if (uiMgr->CreateTypeOfAnnot == foxit::rdk::uiextensions::AnnotCreateType::Eraser)
	{
		MainAppBar->RestartTimeTick();
		uiMgr->CollectEraserOpration();
	}
	
	uiMgr->CreateGraphAnnotBySliderPage();
	uiMgr->ExitAnnotSelect();

	uiMgr->CreateTypeOfAnnot = foxit::rdk::uiextensions::AnnotCreateType::None;
	SettingPopup->IsOpen = false;
	SetAppBarIsCompact(false);
	HideAllLabel->Visibility = Windows::UI::Xaml::Visibility::Visible;
	ShowAllLabel->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
	if (m_HideLableTimer)
		m_HideLableTimer->Start();
	if (uiMgr->CreateTypeOfAnnot == foxit::rdk::uiextensions::AnnotCreateType::None && !IsPopupIsOpen())
	{
		MainAppBar->RestartTimeTick();
	}
}


void foxit::rdk::uiextensions::AnnotToolAppBar::HideAllLabel_Tapped(Platform::Object^ sender, Windows::UI::Xaml::Input::TappedRoutedEventArgs^ e)
{
	UIExtensionsManager^ uiMgr = (UIExtensionsManager^)MainAppBar->View->ExtensionsManager;
	gridBar->Height = 44;
	MainAppBar->SetMainAppBarCompact(true);
	ResetAppBarBG();
	if (uiMgr->CreateTypeOfAnnot == foxit::rdk::uiextensions::AnnotCreateType::Eraser)
	{
		MainAppBar->RestartTimeTick();
		uiMgr->CollectEraserOpration();
	}

	uiMgr->CreateGraphAnnotBySliderPage();

	uiMgr->ExitAnnotSelect();
	uiMgr->CreateTypeOfAnnot = foxit::rdk::uiextensions::AnnotCreateType::None;
	SettingPopup->IsOpen = false;
	SetAppBarIsCompact(true);
	HideAllLabel->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
	ShowAllLabel->Visibility = Windows::UI::Xaml::Visibility::Visible;

	if (uiMgr->CreateTypeOfAnnot == foxit::rdk::uiextensions::AnnotCreateType::None && !IsPopupIsOpen())
	{
		MainAppBar->RestartTimeTick();
	}
}


void foxit::rdk::uiextensions::AnnotToolAppBar::BorderPanel_Tapped(Platform::Object^ sender, Windows::UI::Xaml::Input::TappedRoutedEventArgs^ e)
{
	UIExtensionsManager^ uiMgr = (UIExtensionsManager^)MainAppBar->View->ExtensionsManager;
	uiMgr->CreateGraphAnnotBySliderPage();

	ResetBorderWidthList();
	StackPanel^ stackPanel = dynamic_cast<StackPanel^>(sender);
	Windows::UI::Xaml::Media::SolidColorBrush^ brush = ref new Windows::UI::Xaml::Media::SolidColorBrush(Utility::ConvertFromString("#FFCCCCCC"));
	String^ borderwidthvalue = stackPanel->Tag->ToString();
	double borderWidth = 0;
	if (borderwidthvalue == "1")
	{
		borderWidth = 1;
		Border1->Background = brush;
	}
	else if (borderwidthvalue == "2")
	{
		borderWidth = 2;
		Border2->Background = brush;
	}
	else if (borderwidthvalue == "3")
	{
		borderWidth = 3;
		Border3->Background = brush;
	}
	else if (borderwidthvalue == "4")
	{
		borderWidth = 4;
		Border4->Background = brush;
	}
	else if (borderwidthvalue == "5")
	{
		borderWidth = 5;
		Border5->Background = brush;
	}
	else
	{
		borderWidth = 6;
		Border6->Background = brush;
	}

	uiMgr->getAnnotMgr()->GetAnnotSetting()->SetAnnotDrawBorderWidth((int)borderWidth);
}


void foxit::rdk::uiextensions::AnnotToolAppBar::appBarBorder_Tapped(Platform::Object^ sender, Windows::UI::Xaml::Input::TappedRoutedEventArgs^ e)
{
	UIExtensionsManager^ uiMgr = (UIExtensionsManager^)MainAppBar->View->ExtensionsManager;
	if (uiMgr->IsPopupShow())
	{
		AppBarToggleButton^ button = dynamic_cast<AppBarToggleButton^>(sender);
		if (button)
			button->IsChecked = false;
		return;
	}
	ResetPropertyBG();

	proprtyPopup->IsOpen = false;
	SettingPopup->IsOpen = false;
	appbarSettingNarrow->IsChecked = false;
	appbarSetting->IsChecked = false;
	foxit::rdk::uiextensions::AnnotCreateType type = uiMgr->CreateTypeOfAnnot;

	if (type == foxit::rdk::uiextensions::AnnotCreateType::Circle || type == foxit::rdk::uiextensions::AnnotCreateType::Square
		|| type == foxit::rdk::uiextensions::AnnotCreateType::Line || type == foxit::rdk::uiextensions::AnnotCreateType::Arrow
		|| type == foxit::rdk::uiextensions::AnnotCreateType::Pencil)
	{
		if (!borderGroupPopup->IsOpen)
		{
			InitBorderWidth();
			Windows::Foundation::Point curPt = e->GetPosition(nullptr);
			borderGroupPopup->HorizontalOffset = curPt.X - 75;
			borderGroupPopup->VerticalOffset = this->ActualHeight;
			borderGroupPopup->IsOpen = true;

			appBarBorder->IsChecked = true;
			appBarBorderNarrow->IsChecked = true;
		}
		else
		{
			ResetBorderWidthList();
			borderGroupPopup->IsOpen = false;
			appBarBorder->IsChecked = false;
			appBarBorderNarrow->IsChecked = false;
		}
	}
}


void foxit::rdk::uiextensions::AnnotToolAppBar::appBarProerty_Tapped(Platform::Object^ sender, Windows::UI::Xaml::Input::TappedRoutedEventArgs^ e)
{
	UIExtensionsManager^ uiMgr = (UIExtensionsManager^)MainAppBar->View->ExtensionsManager;
	if (uiMgr->IsPopupShow())
	{
		AppBarToggleButton^ button = dynamic_cast<AppBarToggleButton^>(sender);
		if (button)
			button->IsChecked = false;
		return;
	}
	ResetPropertyBG();
	if (borderGroupPopup->IsOpen)
	{
		borderGroupPopup->IsOpen = false;
		ResetBorderWidthList();
	}

	stampIconPopup->IsOpen = false;
	noteIconPopup->IsOpen = false;
	SettingPopup->IsOpen = false;
	appbarSettingNarrow->IsChecked = false;
	appbarSetting->IsChecked = false;
	if (!proprtyPopup->IsOpen)
	{
		Windows::Foundation::Point curPt = e->GetPosition(nullptr);

		Windows::Foundation::Rect winRect = Windows::UI::Xaml::Window::Current->Bounds;

		AnnotPropertyAppBar^ control = ref new AnnotPropertyAppBar(MainAppBar->View);
		proprtyPopup->Child = control;
		proprtyPopup->HorizontalOffset = curPt.X - 50;
		proprtyPopup->VerticalOffset = this->ActualHeight;
		proprtyPopup->IsOpen = true;


		appBarProerty->IsChecked = true;
		appBarProertyNarrow->IsChecked = true;

	}
	else
	{
		proprtyPopup->IsOpen = false;

		appBarProerty->IsChecked = false;
		appBarProertyNarrow->IsChecked = false;
	}
}

void AnnotToolAppBar::HideNoteStylePopup()
{
	if (noteIconPopup->IsOpen)
	{
		noteIconPopup->IsOpen = false;

		RadioNoteDown->IsChecked = false;
		RadioNoteDownNarrow->IsChecked = false;
	}
}

void AnnotToolAppBar::InitCurGraphAnnot(foxit::rdk::uiextensions::AnnotCreateType type)
{
	BitmapIcon^ icon = ref new BitmapIcon();
	BitmapIcon^ iconNarrow = ref new BitmapIcon();
	Uri^ url;
	if (type == foxit::rdk::uiextensions::AnnotCreateType::Line)
	{
		url = ref new Uri("ms-appx:///foxit.rdk.uiextensions/Assets/Annot/line.png");
	}
	else if (type == foxit::rdk::uiextensions::AnnotCreateType::Circle)
	{
		url = ref new Uri("ms-appx:///foxit.rdk.uiextensions/Assets/Annot/oval.png");
	}
	else if (type == foxit::rdk::uiextensions::AnnotCreateType::Arrow)
	{
		url = ref new Uri("ms-appx:///foxit.rdk.uiextensions/Assets/Annot/arrow.png");
	}
	else
	{
		url = ref new Uri("ms-appx:///foxit.rdk.uiextensions/Assets/Annot/square.png");
	}
	icon->UriSource = url;
	iconNarrow->UriSource = url;

	RadioShapeGroup->Icon = icon;
	RadioShapeGroup->IsChecked = true;

	RadioShapeGroupNarrow->Icon = iconNarrow;
	RadioShapeGroupNarrow->IsChecked = true;

}

void foxit::rdk::uiextensions::AnnotToolAppBar::appBarContinueOpera_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{
	UIExtensionsManager^ uiMgr = (UIExtensionsManager^)MainAppBar->View->ExtensionsManager;
	if (uiMgr->IsPopupShow())
	{
		AppBarToggleButton^ button = dynamic_cast<AppBarToggleButton^>(sender);
		if (button)
			button->IsChecked = false;
		return;
	}
	ResetPropertyBG();
	proprtyPopup->IsOpen = false;
	borderGroupPopup->IsOpen = false;
	foxit::rdk::uiextensions::AnnotCreateType type = uiMgr->CreateTypeOfAnnot;
	auto _AnnotSetting = uiMgr->getAnnotMgr()->GetAnnotSetting();

	bool isContinue = false;
	switch (type)
	{
	case foxit::rdk::uiextensions::AnnotCreateType::HightLight:
		isContinue = _AnnotSetting->GetHightLightIsContinueOpera();
		_AnnotSetting->SetHightLightIsContinueOpera(!isContinue);
		break;
	case foxit::rdk::uiextensions::AnnotCreateType::AreaHighLight:
		isContinue = _AnnotSetting->GetAreaHightLightIsContinueOpera();
		_AnnotSetting->SetAreaHightLightIsContinueOpera(!isContinue);
		break;
	case foxit::rdk::uiextensions::AnnotCreateType::StrikeOut:
		isContinue = _AnnotSetting->GetStrikeOutIsContinueOpera();
		_AnnotSetting->SetStrikeOutIsContinueOpera(!isContinue);
		break;
	case foxit::rdk::uiextensions::AnnotCreateType::UnderLine:
		isContinue = _AnnotSetting->GetUnderLineIsContinueOpera();
		_AnnotSetting->SetUnderLineIsContinueOpera(!isContinue);
		break;
	case foxit::rdk::uiextensions::AnnotCreateType::Squiggly:
		isContinue = _AnnotSetting->GetSquigglyIsContinueOpera();
		_AnnotSetting->SetSquigglyIsContinueOpera(!isContinue);
		break;
	case foxit::rdk::uiextensions::AnnotCreateType::Square:
		isContinue = _AnnotSetting->GetSquareIsContinueOpera();
		_AnnotSetting->SetSquareIsContinueOpera(!isContinue);
		break;
	case foxit::rdk::uiextensions::AnnotCreateType::Circle:
		isContinue = _AnnotSetting->GetCircleIsContinueOpera();
		_AnnotSetting->SetCircleIsContinueOpera(!isContinue);
		break;
	case foxit::rdk::uiextensions::AnnotCreateType::Line:
		isContinue = _AnnotSetting->GetLineIsContinueOpera();
		_AnnotSetting->SetLineIsContinueOpera(!isContinue);
		break;
	case foxit::rdk::uiextensions::AnnotCreateType::Arrow:
		isContinue = _AnnotSetting->GetArrowIsContinueOpera();
		_AnnotSetting->SetArrowIsContinueOpera(!isContinue);
		break;
	case foxit::rdk::uiextensions::AnnotCreateType::Stamp:
		isContinue = _AnnotSetting->GetStampIsContinueOpera();
		_AnnotSetting->SetStampeIsContinueOpera(!isContinue);
		break;
	case foxit::rdk::uiextensions::AnnotCreateType::Text:
		isContinue = _AnnotSetting->GetTextIsContinueOpera();
		_AnnotSetting->SetTextIsContinueOpera(!isContinue);
		break;
	case foxit::rdk::uiextensions::AnnotCreateType::FreeText:
		isContinue = _AnnotSetting->GetFreeTextIsContinueOpera();
		_AnnotSetting->SetFreeTextIsContinueOpera(!isContinue);
		break;
	case foxit::rdk::uiextensions::AnnotCreateType::Caret:
		isContinue = _AnnotSetting->GetInsertTextIsContinueOpera();
		_AnnotSetting->SetInsertTextIsContinueOpera(!isContinue);
		break;
	case foxit::rdk::uiextensions::AnnotCreateType::ReplaceText:
		isContinue = _AnnotSetting->GetReplaceTextIsContinueOpera();
		_AnnotSetting->SetReplaceTextIsContinueOpera(!isContinue);
		break;
	default:
		break;
	}


	if (isContinue)
	{
		appBarContinueOpera->IsChecked = false;
		appBarContinueOperaNarrow->IsChecked = false;
	}
	else
	{
		appBarContinueOpera->IsChecked = true;
		appBarContinueOperaNarrow->IsChecked = true;
	}
}

bool AnnotToolAppBar::IsMobileAnnnotMoreToolOpen()
{
	if (SecondaryMobileCommandsPopup->IsOpen || SecondaryMobileHorCommandsPopup->IsOpen)
	{
		return true;
	}
	return false;
}

void AnnotToolAppBar::SetAllPopupClosed()
{
	UIExtensionsManager^ uiMgr = (UIExtensionsManager^)MainAppBar->View->ExtensionsManager;
	if (Windows::System::Profile::AnalyticsInfo::VersionInfo->DeviceFamily == L"Windows.Mobile")
	{
		if (stampPhonePopup && stampPhonePopup->IsOpen)
			stampPhonePopup->IsOpen = false;

		if (SecondaryMobileCommandsPopup->IsOpen)
		{
			SecondaryMobileCommandsPopup->IsOpen = false;
			uiMgr->SetUnRedoPhoneAppBarVisible(true);
		}

		if (SecondaryMobileHorCommandsPopup->IsOpen)
		{
			SecondaryMobileHorCommandsPopup->IsOpen = false;
			uiMgr->SetUnRedoPhoneAppBarVisible(true);
		}
	}
	else
	{
		AnnotToolListPopup->IsOpen = false;
		SettingPopup->IsOpen = false;
		if (borderGroupPopup->IsOpen)
		{
			ResetBorderWidthList();
			borderGroupPopup->IsOpen = false;
		}
		noteIconPopup->IsOpen = false;
		stampIconPopup->IsOpen = false;
		proprtyPopup->IsOpen = false;

		RadioNoteDown->IsChecked = false;
		RadioGroupDown->IsChecked = false;
		RadioStampDown->IsChecked = false;
		appBarProerty->IsChecked = false;
		appBarBorder->IsChecked = false;
		appbarSetting->IsChecked = false;

		RadioNoteDownNarrow->IsChecked = false;
		RadioStampDownNarrow->IsChecked = false;
		RadioGroupDownNarrow->IsChecked = false;
		appBarProertyNarrow->IsChecked = false;
		appBarBorderNarrow->IsChecked = false;
		appbarSettingNarrow->IsChecked = false;
	}
}

void AnnotToolAppBar::AnnotToolClick(Platform::Object^ sender, foxit::rdk::uiextensions::AnnotCreateType type)
{
	UIExtensionsManager^ uiMgr = (UIExtensionsManager^)MainAppBar->View->ExtensionsManager;
	AppBarToggleButton^ button = dynamic_cast<AppBarToggleButton^>(sender);
	if (uiMgr->IsPopupShow())
	{
		button->IsChecked = false;
		return;
	}

	uiMgr->ExitAnnotSelect();
	SetAllPopupClosed();

	uiMgr->CreateGraphAnnotBySliderPage();

	if (button->IsChecked->Value)
	{
		MainAppBar->StopHideTimeTick();
		ResetAppBarBG();
		CreateAnnot(type);
		if (Windows::System::Profile::AnalyticsInfo::VersionInfo->DeviceFamily == L"Windows.Mobile")
		{
			button->IsChecked = true;

			uiMgr->ShowPhonAnnotPropertyIcon(true);
			ResetSetMobileAnnotBarCompact();
		}
		else
		{
			SetAnnotBtnClick(type, true);
		}
	}
	else
	{
		MainAppBar->RestartTimeTick();
		ResetAnnotToolBarButton(false);
		uiMgr->CreateTypeOfAnnot = foxit::rdk::uiextensions::AnnotCreateType::None;
		SetProertyBtnEnabled();

		if (Windows::System::Profile::AnalyticsInfo::VersionInfo->DeviceFamily == L"Windows.Mobile")
		{
			uiMgr->ShowPhonAnnotPropertyIcon(false);
			SecondaryMobileCommandsPopup->IsOpen = false;
			SecondaryMobileHorCommandsPopup->IsOpen = false;
		}
		else
		{
			SetAnnotBtnClick(type, false);
		}
	}
	if (Windows::System::Profile::AnalyticsInfo::VersionInfo->DeviceFamily == L"Windows.Mobile")
	{
		uiMgr->SetUnRedoPhoneAppBarVisible(true);
	}
}

void AnnotToolAppBar::ResetPropertyBG()
{
	appBarBorder->IsChecked = false;
	appBarProerty->IsChecked = false;
	appBarBorderNarrow->IsChecked = false;
	appBarProertyNarrow->IsChecked = false;
}

void foxit::rdk::uiextensions::AnnotToolAppBar::RadioGroupDown_Tapped(Platform::Object^ sender, Windows::UI::Xaml::Input::TappedRoutedEventArgs^ e)
{
	UIExtensionsManager^ uiMgr = (UIExtensionsManager^)MainAppBar->View->ExtensionsManager;
	if (uiMgr->IsPopupShow())
	{
		AppBarToggleButton^ button = dynamic_cast<AppBarToggleButton^>(sender);
		if (button)
			button->IsChecked = false;
		return;
	}

	uiMgr->SaveOrCreateNoteAnnot();

	RadioGroupDown->IsChecked = false;
	RadioNoteDown->IsChecked = false;
	RadioStampDown->IsChecked = false;

	RadioGroupDownNarrow->IsChecked = false;
	RadioNoteDownNarrow->IsChecked = false;
	RadioStampDownNarrow->IsChecked = false;

	if (AnnotToolListPopup->IsOpen)
	{
		AnnotToolListPopup->IsOpen = false;
		RadioGroupDown->IsChecked = false;
		RadioGroupDownNarrow->IsChecked = false;
	}
	else
	{
		SetAllPopupClosed();
		MainAppBar->StopHideTimeTick();
		Windows::UI::Xaml::FrameworkElement^ _FrameworkElement = dynamic_cast<Windows::UI::Xaml::FrameworkElement^>(sender);
		Windows::Foundation::Point _Pt;
		_Pt = e->GetPosition(_FrameworkElement);
		Windows::Foundation::Point curPt = e->GetPosition(nullptr);

		auto _winrect = Windows::UI::Xaml::Window::Current->Bounds;
		AnnotToolListPopup->HorizontalOffset = curPt.X - 65;
		AnnotToolListPopup->VerticalOffset = this->ActualHeight;
		AnnotToolListPopup->IsOpen = true;

		noteIconPopup->IsOpen = false;
		stampIconPopup->IsOpen = false;
		RadioGroupDown->IsChecked = true;
		RadioNoteDown->IsChecked = false;
		RadioStampDown->IsChecked = false;

		RadioGroupDownNarrow->IsChecked = true;
		RadioNoteDownNarrow->IsChecked = false;
		RadioStampDownNarrow->IsChecked = false;
	}
}

void AnnotToolAppBar::SetProertyBtnEnabled()
{
	appBarBorder->IsEnabled = false;
	appBarProerty->IsEnabled = false;
	appBarContinueOpera->IsEnabled = false;
	appBarContinueOpera->IsChecked = false;
	appBarBorder->IsChecked = false;
	appBarProerty->IsChecked = false;
	appBarBorderNarrow->IsEnabled = false;
	appBarProertyNarrow->IsEnabled = false;
	appBarContinueOperaNarrow->IsEnabled = false;
	appBarContinueOperaNarrow->IsChecked = false;
	appBarBorderNarrow->IsChecked = false;
	appBarProertyNarrow->IsChecked = false;
}


void foxit::rdk::uiextensions::AnnotToolAppBar::RadioNoteDown_Tapped(Platform::Object^ sender, Windows::UI::Xaml::Input::TappedRoutedEventArgs^ e)
{
	UIExtensionsManager^ uiMgr = (UIExtensionsManager^)MainAppBar->View->ExtensionsManager;
	if (uiMgr->IsPopupShow())
	{
		AppBarToggleButton^ button = dynamic_cast<AppBarToggleButton^>(sender);
		if (button)
			button->IsChecked = false;
		return;
	}

	uiMgr->SaveOrCreateNoteAnnot();

	RadioGroupDown->IsChecked = false;
	RadioNoteDown->IsChecked = false;
	RadioStampDown->IsChecked = false;

	RadioGroupDownNarrow->IsChecked = false;
	RadioNoteDownNarrow->IsChecked = false;
	RadioStampDownNarrow->IsChecked = false;

	if (!noteIconPopup->IsOpen)
	{
		SetAllPopupClosed();
		MainAppBar->StopHideTimeTick();
		Windows::Foundation::Point curPt = e->GetPosition(nullptr);
		noteIconPopup->HorizontalOffset = curPt.X - 75;
		noteIconPopup->VerticalOffset = this->ActualHeight;
		noteIconPopup->IsOpen = true;


		AnnotToolListPopup->IsOpen = false;
		stampIconPopup->IsOpen = false;

		RadioNoteDown->IsChecked = true;
		RadioGroupDown->IsChecked = false;
		RadioStampDown->IsChecked = false;
		RadioNoteDownNarrow->IsChecked = true;
		RadioGroupDownNarrow->IsChecked = false;
		RadioStampDownNarrow->IsChecked = false;
	}
	else
	{
		noteIconPopup->IsOpen = false;
		RadioNoteDown->IsChecked = false;
		RadioNoteDownNarrow->IsChecked = false;
	}
}


void foxit::rdk::uiextensions::AnnotToolAppBar::RadioStampDown_Tapped(Platform::Object^ sender, Windows::UI::Xaml::Input::TappedRoutedEventArgs^ e)
{
	UIExtensionsManager^ uiMgr = (UIExtensionsManager^)MainAppBar->View->ExtensionsManager;
	if (uiMgr->IsPopupShow())
	{
		AppBarToggleButton^ button = dynamic_cast<AppBarToggleButton^>(sender);
		if (button)
			button->IsChecked = false;
		return;
	}

	uiMgr->SaveOrCreateNoteAnnot();

	RadioGroupDown->IsChecked = false;
	RadioNoteDown->IsChecked = false;
	RadioStampDown->IsChecked = false;

	RadioGroupDownNarrow->IsChecked = false;
	RadioNoteDownNarrow->IsChecked = false;
	RadioStampDownNarrow->IsChecked = false;

	if (!stampIconPopup->IsOpen)
	{
		SetAllPopupClosed();
		MainAppBar->StopHideTimeTick();
		Windows::Foundation::Point curPt = e->GetPosition(nullptr);

		Windows::Foundation::Rect winRect = Windows::UI::Xaml::Window::Current->Bounds;
		stampIconPopup->HorizontalOffset = curPt.X - 200;
		stampIconPopup->VerticalOffset = this->ActualHeight;
		stampIconPopup->IsOpen = true;

		AnnotToolListPopup->IsOpen = false;
		noteIconPopup->IsOpen = false;
		RadioStampDown->IsChecked = true;
		RadioNoteDown->IsChecked = false;
		RadioGroupDown->IsChecked = false;
		RadioStampDownNarrow->IsChecked = true;
		RadioNoteDownNarrow->IsChecked = false;
		RadioGroupDownNarrow->IsChecked = false;
	}
	else
	{
		stampIconPopup->IsOpen = false;
		RadioStampDown->IsChecked = false;
		RadioStampDownNarrow->IsChecked = false;
	}
}

void AnnotToolAppBar::InitCurNoteIcon()
{
	UIExtensionsManager^ uiMgr = (UIExtensionsManager^)MainAppBar->View->ExtensionsManager;
	uiMgr->ExitAnnotSelect();
	uiMgr->CreateGraphAnnotBySliderPage();

	Platform::String^ noteIocnName = uiMgr->getAnnotMgr()->GetAnnotSetting()->GetNoteLogoUrl();
	Uri^ url = ref new Uri("ms-appx:///foxit.rdk.uiextensions/Assets/Annot/" + noteIocnName + "White.png");
	BitmapIcon^ icon = ref new BitmapIcon();
	icon->UriSource = url;

	BitmapIcon^ iconNarrow = ref new BitmapIcon();
	iconNarrow->UriSource = url;

	MainAppBar->StopHideTimeTick();
	ResetAppBarBG();
	CreateAnnot(foxit::rdk::uiextensions::AnnotCreateType::Text);
	RadioNote->Icon = icon;
	RadioNote->IsChecked = true;
	RadioNoteNarrow->Icon = iconNarrow;
	RadioNoteNarrow->IsChecked = true;
}

void  AnnotToolAppBar::InitStampAppBarSelect()
{
	UIExtensionsManager^ uiMgr = (UIExtensionsManager^)MainAppBar->View->ExtensionsManager;
	uiMgr->ExitAnnotSelect();
	uiMgr->CreateGraphAnnotBySliderPage();

	MainAppBar->StopHideTimeTick();
	ResetAppBarBG();
	CreateAnnot(foxit::rdk::uiextensions::AnnotCreateType::Stamp);

	RadioStamp->IsChecked = true;

	RadioStampNarrow->IsChecked = true;
}

bool  AnnotToolAppBar::IsPopupIsOpen()
{
	if (Windows::System::Profile::AnalyticsInfo::VersionInfo->DeviceFamily == L"Windows.Mobile")
	{
		if (stampPhonePopup && stampPhonePopup->IsOpen)
			return true;
		if (SecondaryMobileCommandsPopup->IsOpen || SecondaryMobileHorCommandsPopup->IsOpen)
			return true;
	}
	else
	{
		if (AnnotToolListPopup->IsOpen)
			return true;
		if (borderGroupPopup->IsOpen)
			return true;
		if (proprtyPopup->IsOpen)
			return true;
		if (noteIconPopup->IsOpen)
			return true;
		if (stampIconPopup->IsOpen)
			return true;
		if (SettingPopup->IsOpen)
			return true;
	}

	return false;
}

void AnnotToolAppBar::SetPropertyCurColor(Windows::UI::Color color)
{
	
}

void AnnotToolAppBar::InitCurCreateAnnotTypeColor()
{
	UIExtensionsManager^ uiMgr = (UIExtensionsManager^)MainAppBar->View->ExtensionsManager;
	Windows::UI::Color color = Windows::UI::Colors::Yellow;
	foxit::rdk::uiextensions::AnnotCreateType type = uiMgr->CreateTypeOfAnnot;
	auto _AnnotSetting = uiMgr->getAnnotMgr()->GetAnnotSetting();

	if (type == foxit::rdk::uiextensions::AnnotCreateType::HightLight
		|| type == foxit::rdk::uiextensions::AnnotCreateType::AreaHighLight)
	{
		color = _AnnotSetting->GetHightLightAnnotColor();
	}
	else if (type == foxit::rdk::uiextensions::AnnotCreateType::UnderLine)
	{
		color = _AnnotSetting->GetUnderLineAnnotColor();
	}
	else if (type == foxit::rdk::uiextensions::AnnotCreateType::StrikeOut)
	{
		color = _AnnotSetting->GetStrikeOutAnnotColor();
	}
	else if (type == foxit::rdk::uiextensions::AnnotCreateType::Squiggly)
	{
		color = _AnnotSetting->GetSquigglyAnnotColor();
	}
	else if (type == foxit::rdk::uiextensions::AnnotCreateType::Line || type == foxit::rdk::uiextensions::AnnotCreateType::Arrow)
	{
		color = _AnnotSetting->GetLineAnnotColor();
	}
	else if (type == foxit::rdk::uiextensions::AnnotCreateType::Pencil || type == foxit::rdk::uiextensions::AnnotCreateType::Circle
		|| type == foxit::rdk::uiextensions::AnnotCreateType::Square)
	{
		color = _AnnotSetting->GetAnnotColor();
	}

}

void foxit::rdk::uiextensions::AnnotToolAppBar::OnTick(Platform::Object ^sender, Platform::Object ^args)
{
	gridBar->Height = 44;
	MainAppBar->SetMainAppBarCompact(true);
	if (ShowAllLabel->Visibility == Windows::UI::Xaml::Visibility::Collapsed)
	{
		SetAppBarIsCompact(true);
		HideAllLabel->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
		ShowAllLabel->Visibility = Windows::UI::Xaml::Visibility::Visible;
		if (SettingPopup->IsOpen)
		{
			SettingPopup->VerticalOffset = 44;
		}
	}
	if (AnnotToolListPopup->IsOpen)
	{ 
		AnnotToolListPopup->VerticalOffset = 44;
	}
	else if (noteIconPopup->IsOpen)
	{
		noteIconPopup->VerticalOffset = 44;
	}
	else if (stampIconPopup->IsOpen)
	{
		stampIconPopup->VerticalOffset = 44;
	}

	if (m_HideLableTimer)
		m_HideLableTimer->Stop();
}

void AnnotToolAppBar::InitBorderWidth()
{
	UIExtensionsManager^ uiMgr = (UIExtensionsManager^)MainAppBar->View->ExtensionsManager;
	int borderwidthvalue = uiMgr->getAnnotMgr()->GetAnnotSetting()->GetAnnotDrawBorderWidth();
	Windows::UI::Xaml::Media::SolidColorBrush^ brush = ref new Windows::UI::Xaml::Media::SolidColorBrush(Utility::ConvertFromString("#FFCCCCCC"));
	if (borderwidthvalue == 1)
	{
		Border1->Background = brush;
	}
	else if (borderwidthvalue == 2)
	{
		Border2->Background = brush;
	}
	else if (borderwidthvalue == 3)
	{
		Border3->Background = brush;
	}
	else if (borderwidthvalue == 4)
	{
		Border4->Background = brush;
	}
	else if (borderwidthvalue == 5)
	{
		Border5->Background = brush;
	}
	else
	{
		Border6->Background = brush;
	}
}

void AnnotToolAppBar::ResetBorderWidthList()
{
	Border1->Background = ref new Windows::UI::Xaml::Media::SolidColorBrush(Windows::UI::Colors::Transparent);
	Border2->Background = ref new Windows::UI::Xaml::Media::SolidColorBrush(Windows::UI::Colors::Transparent);
	Border3->Background = ref new Windows::UI::Xaml::Media::SolidColorBrush(Windows::UI::Colors::Transparent);
	Border4->Background = ref new Windows::UI::Xaml::Media::SolidColorBrush(Windows::UI::Colors::Transparent);
	Border5->Background = ref new Windows::UI::Xaml::Media::SolidColorBrush(Windows::UI::Colors::Transparent);
	Border6->Background = ref new Windows::UI::Xaml::Media::SolidColorBrush(Windows::UI::Colors::Transparent);
}

void AnnotToolAppBar::InitMobileCurAnnotTip(foxit::rdk::uiextensions::AnnotCreateType type)
{
	
}

void AnnotToolAppBar::OnHideTipTick(Platform::Object ^sender, Platform::Object ^args)
{
	mobileNoteTip->IsOpen = false;

	if (m_HideMobileAnnotTipTimer)
		m_HideMobileAnnotTipTimer->Stop();
}

void AnnotToolAppBar::InitAnnotIsContinueOpera(foxit::rdk::uiextensions::AnnotCreateType type)
{
	UIExtensionsManager^ uiMgr = (UIExtensionsManager^)MainAppBar->View->ExtensionsManager;
	bool isContinue = false;
	auto _AnnotSetting = uiMgr->getAnnotMgr()->GetAnnotSetting();
	switch (type)
	{
	case foxit::rdk::uiextensions::AnnotCreateType::HightLight:
		isContinue = _AnnotSetting->GetHightLightIsContinueOpera();
		break;
	case foxit::rdk::uiextensions::AnnotCreateType::AreaHighLight:
		isContinue = _AnnotSetting->GetAreaHightLightIsContinueOpera();
		break;
	case foxit::rdk::uiextensions::AnnotCreateType::StrikeOut:
		isContinue = _AnnotSetting->GetStrikeOutIsContinueOpera();
		break;
	case foxit::rdk::uiextensions::AnnotCreateType::UnderLine:
		isContinue = _AnnotSetting->GetUnderLineIsContinueOpera();
		break;
	case foxit::rdk::uiextensions::AnnotCreateType::Squiggly:
		isContinue = _AnnotSetting->GetSquigglyIsContinueOpera();
		break;
	case foxit::rdk::uiextensions::AnnotCreateType::Square:
		isContinue = _AnnotSetting->GetSquareIsContinueOpera();
		break;
	case foxit::rdk::uiextensions::AnnotCreateType::Circle:
		isContinue = _AnnotSetting->GetCircleIsContinueOpera();
		break;
	case foxit::rdk::uiextensions::AnnotCreateType::Line:
		isContinue = _AnnotSetting->GetLineIsContinueOpera();
		break;
	case foxit::rdk::uiextensions::AnnotCreateType::Arrow:
		isContinue = _AnnotSetting->GetArrowIsContinueOpera();
		break;
	case foxit::rdk::uiextensions::AnnotCreateType::Stamp:
		isContinue = _AnnotSetting->GetStampIsContinueOpera();
		break;
	case foxit::rdk::uiextensions::AnnotCreateType::Text:
		isContinue = _AnnotSetting->GetTextIsContinueOpera();
		break;
	case foxit::rdk::uiextensions::AnnotCreateType::FreeText:
		isContinue = _AnnotSetting->GetFreeTextIsContinueOpera();
		break;
	case foxit::rdk::uiextensions::AnnotCreateType::Caret:
		isContinue = _AnnotSetting->GetInsertTextIsContinueOpera();
		break;
	case foxit::rdk::uiextensions::AnnotCreateType::ReplaceText:
		isContinue = _AnnotSetting->GetReplaceTextIsContinueOpera();
		break;
	default:
		break;
	}

	if (isContinue)
	{
		appBarContinueOpera->IsChecked = true;
		appBarContinueOperaNarrow->IsChecked = true;
	}
	else
	{
		appBarContinueOpera->IsChecked = false;
		appBarContinueOperaNarrow->IsChecked = false;
	}

}

void AnnotToolAppBar::ResetSetMobileAnnotBarCompact()
{
	UIExtensionsManager^ uiMgr = (UIExtensionsManager^)MainAppBar->View->ExtensionsManager;
	if (SecondaryMobileCommandsPopup->IsOpen || SecondaryMobileHorCommandsPopup->IsOpen)
	{
		SecondaryMobileCommandsPopup->IsOpen = false;
		SecondaryMobileHorCommandsPopup->IsOpen = false;
	}

	ResetAnnotToolBarButton(true);

	phoneAnnotTipControl->InitMobileCurAnnotTip(uiMgr->CreateTypeOfAnnot);
	mobileNoteTip->IsOpen = true;

	m_HideMobileAnnotTipTimer->Start();
}


void foxit::rdk::uiextensions::AnnotToolAppBar::RadioFreeText_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{
	AnnotToolClick(sender, foxit::rdk::uiextensions::AnnotCreateType::FreeText);
}

void AnnotToolAppBar::SetUnRedoUnEnabled()
{
	RadioUndo->IsEnabled = false;
	RadioRedo->IsEnabled = false;
	RadioUndoNarrow->IsEnabled = false;
	RadioRedoNarrow->IsEnabled = false;
}

void AnnotToolAppBar::SetAnnotBtnClick(foxit::rdk::uiextensions::AnnotCreateType type, bool m_isCreate)
{
	if (m_isCreate)
	{
		switch (type)
		{
		case foxit::rdk::uiextensions::AnnotCreateType::HightLight:
			RadioHightLight->IsChecked = true;
			RadioHightLightNarrow->IsChecked = true;
			break;
		case foxit::rdk::uiextensions::AnnotCreateType::AreaHighLight:
			RadioAreaHightLight->IsChecked = true;
			RadioAreaHightLightNarrow->IsChecked = true;
			break;
		case foxit::rdk::uiextensions::AnnotCreateType::StrikeOut:
			RadioStrikeOut->IsChecked = true;
			RadioStrikeOutNarrow->IsChecked = true;
			break;
		case foxit::rdk::uiextensions::AnnotCreateType::UnderLine:
			RadioUnderLine->IsChecked = true;
			RadioUnderLineNarrow->IsChecked = true;
			break;
		case foxit::rdk::uiextensions::AnnotCreateType::Squiggly:
			RadioSquiggly->IsChecked = true;
			RadioSquigglyNarrow->IsChecked = true;
			break;
		case foxit::rdk::uiextensions::AnnotCreateType::Stamp:
			RadioStamp->IsChecked = true;
			RadioStampNarrow->IsChecked = true;
			break;
		case foxit::rdk::uiextensions::AnnotCreateType::Text:
			RadioNote->IsChecked = true;
			RadioNoteNarrow->IsChecked = true;
			break;
		case foxit::rdk::uiextensions::AnnotCreateType::Eraser:
			RadioEraser->IsChecked = true;
			RadioEraserNarrow->IsChecked = true;
			break;
		case foxit::rdk::uiextensions::AnnotCreateType::Pencil:
			RadioInk->IsChecked = true;
			RadioInkNarrow->IsChecked = true;
			break;
		case foxit::rdk::uiextensions::AnnotCreateType::FreeText:
			RadioFreeText->IsChecked = true;
			RadioFreeTextNarrow->IsChecked = true;
			break;
		case foxit::rdk::uiextensions::AnnotCreateType::Caret:
			RadioInsertText->IsChecked = true;
			RadioInsertTextNarrow->IsChecked = true;
			break;
		case foxit::rdk::uiextensions::AnnotCreateType::ReplaceText:
			RadioReplaceText->IsChecked = true;
			RadioReplaceTextNarrow->IsChecked = true;
			break;
		case foxit::rdk::uiextensions::AnnotCreateType::Line:
		case foxit::rdk::uiextensions::AnnotCreateType::Arrow:
		case foxit::rdk::uiextensions::AnnotCreateType::Square:
		case foxit::rdk::uiextensions::AnnotCreateType::Circle:
			RadioShapeGroup->IsChecked = true;
			RadioShapeGroupNarrow->IsChecked = true;
			break;
		default:
			break;
		}
	}
	else
	{
		switch (type)
		{
		case foxit::rdk::uiextensions::AnnotCreateType::HightLight:
			RadioHightLight->IsChecked = false;
			RadioHightLightNarrow->IsChecked = false;
			break;
		case foxit::rdk::uiextensions::AnnotCreateType::AreaHighLight:
			RadioAreaHightLight->IsChecked = false;
			RadioAreaHightLightNarrow->IsChecked = false;
			break;
		case foxit::rdk::uiextensions::AnnotCreateType::StrikeOut:
			RadioStrikeOut->IsChecked = false;
			RadioStrikeOutNarrow->IsChecked = false;
			break;
		case foxit::rdk::uiextensions::AnnotCreateType::UnderLine:
			RadioUnderLine->IsChecked = false;
			RadioUnderLineNarrow->IsChecked = false;
			break;
		case foxit::rdk::uiextensions::AnnotCreateType::Squiggly:
			RadioSquiggly->IsChecked = false;
			RadioSquigglyNarrow->IsChecked = false;
			break;
		case foxit::rdk::uiextensions::AnnotCreateType::Stamp:
			RadioStamp->IsChecked = false;
			RadioStampNarrow->IsChecked = false;
			break;
		case foxit::rdk::uiextensions::AnnotCreateType::Text:
			RadioNote->IsChecked = false;
			RadioNoteNarrow->IsChecked = false;
			break;
		case foxit::rdk::uiextensions::AnnotCreateType::Eraser:
			RadioEraser->IsChecked = false;
			RadioEraserNarrow->IsChecked = false;
			break;
		case foxit::rdk::uiextensions::AnnotCreateType::Pencil:
			RadioInk->IsChecked = false;
			RadioInkNarrow->IsChecked = false;
			break;
		case foxit::rdk::uiextensions::AnnotCreateType::FreeText:
			RadioFreeText->IsChecked = false;
			RadioFreeTextNarrow->IsChecked = false;
			break;
		case foxit::rdk::uiextensions::AnnotCreateType::Caret:
			RadioInsertText->IsChecked = false;
			RadioInsertTextNarrow->IsChecked = false;
			break;
		case foxit::rdk::uiextensions::AnnotCreateType::ReplaceText:
			RadioReplaceText->IsChecked = false;
			RadioReplaceTextNarrow->IsChecked = false;
			break;
		case foxit::rdk::uiextensions::AnnotCreateType::Line:
		case foxit::rdk::uiextensions::AnnotCreateType::Arrow:
		case foxit::rdk::uiextensions::AnnotCreateType::Square:
		case foxit::rdk::uiextensions::AnnotCreateType::Circle:
			RadioShapeGroup->IsChecked = false;
			RadioShapeGroupNarrow->IsChecked = false;
			break;
		default:
			break;
		}
	}
}

void AnnotToolAppBar::SetAnnotMobileMoreBtnSelected()
{
	UIExtensionsManager^ uiMgr = (UIExtensionsManager^)MainAppBar->View->ExtensionsManager;
	auto _winrect = Windows::UI::Xaml::Window::Current->Bounds;
	if (uiMgr->CreateTypeOfAnnot != foxit::rdk::uiextensions::AnnotCreateType::None)
	{
		foxit::rdk::uiextensions::AnnotCreateType type = uiMgr->CreateTypeOfAnnot;
		if (_winrect.Width > _winrect.Height)
		{
			switch (type)
			{
			case foxit::rdk::uiextensions::AnnotCreateType::HightLight:
				RadioHightLightMobileMorePor->IsChecked = true;
				RadioHightLightMobile->IsChecked = true;
				break;
			case foxit::rdk::uiextensions::AnnotCreateType::AreaHighLight:
				RadioAreaHightLightMobileMorePor->IsChecked = true;
				RadioAreaHightLightMobile->IsChecked = true;
				break;
			case foxit::rdk::uiextensions::AnnotCreateType::StrikeOut:
				RadioStrikeOutMobileMorePor->IsChecked = true;
				RadioStrikeOutMobile->IsChecked = true;
				break;
			case foxit::rdk::uiextensions::AnnotCreateType::UnderLine:
				RadioUnderLineMobileMorePor->IsChecked = true;
				RadioUnderLineMobile->IsChecked = true;
				break;
			case foxit::rdk::uiextensions::AnnotCreateType::Squiggly:
				RadioSquigglyMobilePor->IsChecked = true;
				break;
			case foxit::rdk::uiextensions::AnnotCreateType::Stamp:
				RadioStampMobilePor->IsChecked = true;
				break;
			case foxit::rdk::uiextensions::AnnotCreateType::Text:
				RadioNoteMobileMorePor->IsChecked = true;
				RadioNoteMobile->IsChecked = true;
				break;
			case foxit::rdk::uiextensions::AnnotCreateType::Eraser:
				RadioEraserMobilePor->IsChecked = true;
				break;
			case foxit::rdk::uiextensions::AnnotCreateType::Pencil:
				RadioPencilMobileMorePor->IsChecked = true;
				break;
			case foxit::rdk::uiextensions::AnnotCreateType::Square:
				RadioSquareMobilePor->IsChecked = true;
				break;
			case foxit::rdk::uiextensions::AnnotCreateType::Circle:
				RadioCircleMobilePor->IsChecked = true;
				break;
			case foxit::rdk::uiextensions::AnnotCreateType::Line:
				RadioLineMobilePor->IsChecked = true;
				break;
			case foxit::rdk::uiextensions::AnnotCreateType::Arrow:
				RadioArrowMobilePor->IsChecked = true;
				break;
			case foxit::rdk::uiextensions::AnnotCreateType::FreeText:
				RadioFreeTextMobilePor->IsChecked = true;
				break;
			case foxit::rdk::uiextensions::AnnotCreateType::Caret:
				RadioInsertTextMobilePor->IsChecked = true;
				break;
			case foxit::rdk::uiextensions::AnnotCreateType::ReplaceText:
				RadioReplaceTextMobilePor->IsChecked = true;
				break;
			}
		}
		else
		{
			switch (type)
			{
			case foxit::rdk::uiextensions::AnnotCreateType::HightLight:
				RadioHightLightMobileMore->IsChecked = true;
				RadioHightLightMobile->IsChecked = true;
				break;
			case foxit::rdk::uiextensions::AnnotCreateType::AreaHighLight:
				RadioAreaHightLightMobileMore->IsChecked = true;
				RadioAreaHightLightMobile->IsChecked = true;
				break;
			case foxit::rdk::uiextensions::AnnotCreateType::StrikeOut:
				RadioStrikeOutMobileMore->IsChecked = true;
				RadioStrikeOutMobile->IsChecked = true;
				break;
			case foxit::rdk::uiextensions::AnnotCreateType::UnderLine:
				RadioUnderLineMobileMore->IsChecked = true;
				RadioUnderLineMobile->IsChecked = true;
				break;
			case foxit::rdk::uiextensions::AnnotCreateType::Squiggly:
				RadioSquigglyMobile->IsChecked = true;
				break;
			case foxit::rdk::uiextensions::AnnotCreateType::Stamp:
				RadioStampMobile->IsChecked = true;
				break;
			case foxit::rdk::uiextensions::AnnotCreateType::Text:
				RadioNoteMobileMore->IsChecked = true;
				RadioNoteMobile->IsChecked = true;
				break;
			case foxit::rdk::uiextensions::AnnotCreateType::Eraser:
				RadioEraserMobile->IsChecked = true;
				break;
			case foxit::rdk::uiextensions::AnnotCreateType::Pencil:
				RadioPencilMobileMore->IsChecked = true;
				break;
			case foxit::rdk::uiextensions::AnnotCreateType::Square:
				RadioSquareMobile->IsChecked = true;
				break;
			case foxit::rdk::uiextensions::AnnotCreateType::Circle:
				RadioCircleMobile->IsChecked = true;
				break;
			case foxit::rdk::uiextensions::AnnotCreateType::Line:
				RadioLineMobile->IsChecked = true;
				break;
			case foxit::rdk::uiextensions::AnnotCreateType::Arrow:
				RadioArrowMobile->IsChecked = true;
				break;
			case foxit::rdk::uiextensions::AnnotCreateType::FreeText:
				RadioFreeTextMobile->IsChecked = true;
				break;
			case foxit::rdk::uiextensions::AnnotCreateType::Caret:
				RadioInsertTextMobile->IsChecked = true;
				break;
			case foxit::rdk::uiextensions::AnnotCreateType::ReplaceText:
				RadioReplaceTextMobile->IsChecked = true;
				break;
			}
		}
	}
}


void foxit::rdk::uiextensions::AnnotToolAppBar::Border1_PointerEntered(Platform::Object^ sender, Windows::UI::Xaml::Input::PointerRoutedEventArgs^ e)
{
	Border^ border = dynamic_cast<Border^>(sender);
	if (border)
		border->Background = ref new Windows::UI::Xaml::Media::SolidColorBrush(Utility::ConvertFromString("#FFCCCCCC"));
}


void foxit::rdk::uiextensions::AnnotToolAppBar::Border1_PointerExited(Platform::Object^ sender, Windows::UI::Xaml::Input::PointerRoutedEventArgs^ e)
{
	UIExtensionsManager^ uiMgr = (UIExtensionsManager^)MainAppBar->View->ExtensionsManager;
	Border^ border = dynamic_cast<Border^>(sender);
	int borderwidthvalue = uiMgr->getAnnotMgr()->GetAnnotSetting()->GetAnnotDrawBorderWidth();
	String^ tag = border->Tag->ToString();
	String^ borderwidth = borderwidthvalue.ToString();

	if (border && border->Tag->ToString() != borderwidthvalue.ToString())
	{
		border->Background = ref new Windows::UI::Xaml::Media::SolidColorBrush(Windows::UI::Colors::Transparent);
	}
}

void foxit::rdk::uiextensions::AnnotToolAppBar::InitNoteIcon()
{
	UIExtensionsManager^ uiMgr = (UIExtensionsManager^)MainAppBar->View->ExtensionsManager;
	Platform::String^ noteIocnName = uiMgr->getAnnotMgr()->GetAnnotSetting()->GetNoteLogoUrl();
	Uri^ url = ref new Uri("ms-appx:///foxit.rdk.uiextensions/Assets/Annot/" + noteIocnName + "White.png");
	BitmapIcon^ icon = ref new BitmapIcon();
	icon->UriSource = url;
	BitmapIcon^ iconNarrow = ref new BitmapIcon();
	iconNarrow->UriSource = url;
	RadioNote->Icon = icon;
	RadioNoteNarrow->Icon = iconNarrow;
}

void foxit::rdk::uiextensions::AnnotToolAppBar::ResetAnnotToolBarButton(bool isChecked)
{
	UIExtensionsManager^ uiMgr = (UIExtensionsManager^)MainAppBar->View->ExtensionsManager;
	foxit::rdk::uiextensions::AnnotCreateType type = uiMgr->CreateTypeOfAnnot;
	if (isChecked)
	{
		if (type == foxit::rdk::uiextensions::AnnotCreateType::HightLight)
		{
			RadioHightLightMobile->IsChecked = true;
			RadioHightLightMobileMore->IsChecked = true;
			RadioHightLightMobileMorePor->IsChecked = true;
		}
		else if (type == foxit::rdk::uiextensions::AnnotCreateType::UnderLine)
		{
			RadioUnderLineMobile->IsChecked = true;
			RadioUnderLineMobileMore->IsChecked = true;
			RadioUnderLineMobileMorePor->IsChecked = true;
		}
		else if (type == foxit::rdk::uiextensions::AnnotCreateType::StrikeOut)
		{
			RadioStrikeOutMobile->IsChecked = true;
			RadioStrikeOutMobileMore->IsChecked = true;
			RadioStrikeOutMobileMorePor->IsChecked = true;
		}
		else if (type == foxit::rdk::uiextensions::AnnotCreateType::AreaHighLight)
		{
			RadioAreaHightLightMobile->IsChecked = true;
			RadioAreaHightLightMobileMore->IsChecked = true;
			RadioAreaHightLightMobileMorePor->IsChecked = true;
		}
		else if (type == foxit::rdk::uiextensions::AnnotCreateType::Text)
		{
			RadioNoteMobile->IsChecked = true;
			RadioNoteMobileMore->IsChecked = true;
			RadioNoteMobileMorePor->IsChecked = true;
		}
	}
	else
	{
		if (type == foxit::rdk::uiextensions::AnnotCreateType::HightLight)
		{
			RadioHightLightMobile->IsChecked = false;
			RadioHightLightMobileMore->IsChecked = false;
			RadioHightLightMobileMorePor->IsChecked = false;
		}
		else if (type == foxit::rdk::uiextensions::AnnotCreateType::UnderLine)
		{
			RadioUnderLineMobile->IsChecked = false;
			RadioUnderLineMobileMore->IsChecked = false;
			RadioUnderLineMobileMorePor->IsChecked = false;
		}
		else if (type == foxit::rdk::uiextensions::AnnotCreateType::StrikeOut)
		{
			RadioStrikeOutMobile->IsChecked = false;
			RadioStrikeOutMobileMore->IsChecked = false;
			RadioStrikeOutMobileMorePor->IsChecked = false;
		}
		else if (type == foxit::rdk::uiextensions::AnnotCreateType::AreaHighLight)
		{
			RadioAreaHightLightMobile->IsChecked = false;
			RadioAreaHightLightMobileMore->IsChecked = false;
			RadioAreaHightLightMobileMorePor->IsChecked = false;
		}
		else if (type == foxit::rdk::uiextensions::AnnotCreateType::Text)
		{
			RadioNoteMobile->IsChecked = false;
			RadioNoteMobileMore->IsChecked = false;
			RadioNoteMobileMorePor->IsChecked = false;
		}
	}
}


void foxit::rdk::uiextensions::AnnotToolAppBar::LandMobileMoreCommans_Tapped(Platform::Object^ sender, Windows::UI::Xaml::Input::TappedRoutedEventArgs^ e)
{
}


void foxit::rdk::uiextensions::AnnotToolAppBar::Grid_PointerReleased(Platform::Object^ sender, Windows::UI::Xaml::Input::PointerRoutedEventArgs^ e)
{
	UIExtensionsManager^ uiMgr = (UIExtensionsManager^)MainAppBar->View->ExtensionsManager;
	if (MainAppBar->View)
		uiMgr->CreateGraphAnnotBySliderPage();
}

void AnnotToolAppBar::SetFreeTextPropertyShow(bool isShow)
{
	if (proprtyPopup->IsOpen)
	{
		AnnotPropertyAppBar^ control = dynamic_cast<AnnotPropertyAppBar^>(proprtyPopup->Child);
		if (control)
			control->SetFreeTextPropertyShow(isShow);
	}
}

void AnnotToolAppBar::SetUnRedoIsEnabled(bool isEnable)
{
	if (!isEnable)
	{
		RadioUndo->IsEnabled = isEnable;
		RadioRedo->IsEnabled = isEnable;
		RadioUndoNarrow->IsEnabled = isEnable;
		RadioRedoNarrow->IsEnabled = isEnable;
	}
	else
	{
		UpdateUndoControlState();
	}
}

void AnnotToolAppBar::OnPhoneTipLoaded(Platform::Object ^sender, Windows::UI::Xaml::RoutedEventArgs ^e)
{
	auto _winrect = Windows::UI::Xaml::Window::Current->Bounds;
	double actualWidth = phoneAnnotTipControl->ActualWidth;
	mobileNoteTip->HorizontalOffset = (_winrect.Width - actualWidth) / 2;
	mobileNoteTip->VerticalOffset = -30;
}

void foxit::rdk::uiextensions::AnnotToolAppBar::RadioInsertText_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{
	AnnotToolClick(sender, foxit::rdk::uiextensions::AnnotCreateType::Caret);
}


void foxit::rdk::uiextensions::AnnotToolAppBar::RadioReplaceText_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{
	AnnotToolClick(sender, foxit::rdk::uiextensions::AnnotCreateType::ReplaceText);
}


void foxit::rdk::uiextensions::AnnotToolAppBar::scrollTollbar_ManipulationDelta(Platform::Object^ sender, Windows::UI::Xaml::Input::ManipulationDeltaRoutedEventArgs^ e)
{
	double scrollWidth = scrollTollbar->ActualWidth;
	double toolWidth = DesktopBarNarrowAnnotToolPanel->ActualWidth;

	double xvalue = e->Delta.Translation.X;
	offsetWidth = scrollTollbar->HorizontalOffset;
	offsetWidth = offsetWidth + (-xvalue);

	scrollTollbar->ChangeView(offsetWidth, 0.0, (float)1.0);
}


void foxit::rdk::uiextensions::AnnotToolAppBar::appbarSetting_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{
	UIExtensionsManager^ uiMgr = (UIExtensionsManager^)MainAppBar->View->ExtensionsManager;
	if (uiMgr->IsPopupShow())
	{
		AppBarToggleButton^ button = dynamic_cast<AppBarToggleButton^>(sender);
		if (button)
			button->IsChecked = false;
		return;
	}

	uiMgr->SaveOrCreateNoteAnnot();

	if (!SettingPopup->IsOpen)
	{
		SetAllPopupClosed();

		auto _winrect = Windows::UI::Xaml::Window::Current->Bounds;

		SettingPopup->HorizontalOffset = _winrect.Width - 150;
		SettingPopup->VerticalOffset = this->ActualHeight;
		SettingPopup->IsOpen = true;

		appbarSetting->IsChecked = true;
		appbarSettingNarrow->IsChecked = true;
	}
	else
	{
		SettingPopup->IsOpen = false;

		appbarSetting->IsChecked = false;
		appbarSettingNarrow->IsChecked = false;
	}
}


void foxit::rdk::uiextensions::AnnotToolAppBar::RadioStampMobile_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{
	UIExtensionsManager^ uiMgr = (UIExtensionsManager^)MainAppBar->View->ExtensionsManager;
	uiMgr->CreateGraphAnnotBySliderPage();
	if (uiMgr->IsPopupShow())
	{
		AppBarToggleButton^ button = dynamic_cast<AppBarToggleButton^>(sender);
		if (button)
			button->IsChecked = false;
		return;
	}
	SetAllPopupClosed();

	AnnotToolClick(sender, foxit::rdk::uiextensions::AnnotCreateType::Stamp);

	if (Windows::System::Profile::AnalyticsInfo::VersionInfo->DeviceFamily == L"Windows.Mobile"
		&& uiMgr->CreateTypeOfAnnot == foxit::rdk::uiextensions::AnnotCreateType::Stamp)
	{
		stampPhonePopup = ref new Popup();
		AnnotStampUserControl^ stampcontrol = ref new AnnotStampUserControl();
		stampcontrol->RelativePageView(MainAppBar->View);
		stampPhonePopup->Child = stampcontrol;
		stampPhonePopup->HorizontalOffset = 0;
		stampPhonePopup->VerticalOffset = 0;
		stampPhonePopup->IsOpen = true;
	}
}


void foxit::rdk::uiextensions::AnnotToolAppBar::RadioShapeGroup_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{
	UIExtensionsManager^ uiMgr = (UIExtensionsManager^)MainAppBar->View->ExtensionsManager;
	AppBarToggleButton^ button = dynamic_cast<AppBarToggleButton^>(sender);
	if (uiMgr->IsPopupShow())
	{
		if (button)
			button->IsChecked = false;
		return;
	}

	SetAllPopupClosed();

	if (button && !button->IsChecked->Value)
	{
		MainAppBar->RestartTimeTick();
		uiMgr->CreateTypeOfAnnot = foxit::rdk::uiextensions::AnnotCreateType::None;
		ResetAppBarBG();
	}
	else
	{
		ResetAppBarBG();
		MainAppBar->StopHideTimeTick();
		BitmapIcon^ icon = dynamic_cast<BitmapIcon^>(button->Icon);
		if (icon)
		{
			Uri^ url = icon->UriSource;

			Platform::String^ urlPath = url->AbsoluteUri;;
			const wchar_t *seperator = wcsrchr(urlPath->Data(), '/');
			Platform::String^ iconName = seperator ? ref new Platform::String(seperator + 1) : urlPath;

			RadioShapeGroup->IsChecked = true;
			RadioShapeGroupNarrow->IsChecked = true;
			if (iconName == "line.png")
			{
				AnnotToolClick(sender, foxit::rdk::uiextensions::AnnotCreateType::Line);
			}
			else if (iconName == "oval.png")
			{
				AnnotToolClick(sender, foxit::rdk::uiextensions::AnnotCreateType::Circle);
			}
			else if (iconName == "arrow.png")
			{
				AnnotToolClick(sender, foxit::rdk::uiextensions::AnnotCreateType::Arrow);
			}
			else
			{
				AnnotToolClick(sender, foxit::rdk::uiextensions::AnnotCreateType::Square);
			}
		}
		uiMgr->SetAnnotCanvasVisible(false);
	}
}


void foxit::rdk::uiextensions::AnnotToolAppBar::RadioStamp_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{
	UIExtensionsManager^ uiMgr = (UIExtensionsManager^)MainAppBar->View->ExtensionsManager;
	uiMgr->CreateGraphAnnotBySliderPage();
	if (uiMgr->IsPopupShow())
	{
		AppBarToggleButton^ button = dynamic_cast<AppBarToggleButton^>(sender);
		if (button)
			button->IsChecked = false;
		return;
	}
	SetAllPopupClosed();

	AnnotToolClick(sender, foxit::rdk::uiextensions::AnnotCreateType::Stamp);

	if (Windows::System::Profile::AnalyticsInfo::VersionInfo->DeviceFamily == L"Windows.Mobile"
		&& uiMgr->CreateTypeOfAnnot == foxit::rdk::uiextensions::AnnotCreateType::Stamp)
	{
		stampPhonePopup = ref new Popup();
		AnnotStampUserControl^ stampcontrol = ref new AnnotStampUserControl();
		stampcontrol->RelativePageView(MainAppBar->View);
		stampPhonePopup->Child = stampcontrol;
		stampPhonePopup->HorizontalOffset = 0;
		stampPhonePopup->VerticalOffset = 0;
		stampPhonePopup->IsOpen = true;
	}
}
