﻿/**
* Copyright (C) 2003-2018, Foxit Software Inc..
* All Rights Reserved.
*
* http://www.foxitsoftware.com
*
* The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
* distribute any parts of Foxit Mobile PDF SDK to third party or public without permission unless an agreement
* is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
* Review legal.txt for additional license and legal information.
*/


#include "pch.h"
#include "AnnotationsPage.xaml.h"
#include "MainPageAppBar.xaml.h"

using namespace foxit::rdk::uiextensions;

using namespace Platform;
using namespace Windows::Foundation;
using namespace Windows::Foundation::Collections;
using namespace Windows::UI::Xaml;
using namespace Windows::UI::Xaml::Controls;
using namespace Windows::UI::Xaml::Controls::Primitives;
using namespace Windows::UI::Xaml::Data;
using namespace Windows::UI::Xaml::Input;
using namespace Windows::UI::Xaml::Media;
using namespace Windows::UI::Xaml::Navigation;


AnnotationItem::AnnotationItem(IBaseAnnot^ pAnnot)
{
	if (!pAnnot)
	{
		_ItemTitle = L"";
		_ItemIcon = Utility::GetAnnotBitmap(L"Default");
	}
	else
	{
		_pAnnot = pAnnot;
		
		InitializeIcon();

		InitializeTitle();
	}
}

void AnnotationItem::InitializeTitle()
{
	if (_pAnnot == nullptr)
	{
		_ItemTitle = L"";
		return;
	}

	Platform::String^ pTitle = _pAnnot->Title;
	Platform::String^ pContents = _pAnnot->Contents;

	auto _pPageHandleWrap = (PDFPageWrapper^)(_pAnnot->PageHandleWrap);
	foxit::pdf::PDFPage^ pPage = dynamic_cast<foxit::pdf::PDFPage^>(_pPageHandleWrap->Pointer);
	FSPDFDocUtil::PageParse(pPage);

	Windows::Foundation::Collections::IVector<Windows::Foundation::Rect>^ pQuids = nullptr;

	auto eType = _pAnnot->AnnotType;
	if (eType == AnnotationType::HightLight)
	{
		auto pSourceAnnot = dynamic_cast<HighLightAnnot^>(_pAnnot);
		pQuids = pSourceAnnot->Page_HightLightRectGroup;
	}
	else if (eType == AnnotationType::UnderLine)
	{
		auto pSourceAnnot = dynamic_cast<UnderLineAnnot^>(_pAnnot);
		pQuids = pSourceAnnot->Page_HightLightRectGroup;
	}
	else if (eType == AnnotationType::Squiggly)
	{
		auto pSourceAnnot = dynamic_cast<SquigglyAnnot^>(_pAnnot);
		pQuids = pSourceAnnot->Page_HightLightRectGroup;
	}
	else if (eType == AnnotationType::StrikeOut)
	{
		auto pSourceAnnot = dynamic_cast<StrikeOutAnnot^>(_pAnnot);
		pQuids = pSourceAnnot->Page_HightLightRectGroup;
	}

	Platform::String^ pTextMarkContents = L"";
	auto pTextSelect = ref new foxit::pdf::TextPage(pPage, (int)foxit::pdf::TextParseFlags::e_ParseTextNormal);
	if (pQuids && pQuids->Size > 0)
	{
		auto pQuidsSize = pQuids->Size;
		for (unsigned int i = 0; i < pQuidsSize; i++)
		{
			auto item = pQuids->GetAt(i);

			pTextMarkContents += pTextSelect->GetTextInRect(FoxitRectConverter::ConvertToFoxitRectF(item));
		}
	}

	if (pTitle->Length() == 0)
	{
		ItemTitle = pContents->Length() > 0 ? pContents : pTextMarkContents;
	}
	else
	{
		ItemTitle = pTitle + L": " + (pContents->Length() > 0 ? pContents : pTextMarkContents);
	}
}

void AnnotationItem::InitializeIcon()
{
	if (_pAnnot == nullptr)
		return;

	auto eType = _pAnnot->AnnotType;
	if (eType == AnnotationType::Line)
	{
		auto pSourceAnnot = dynamic_cast<LineAnnot^>(_pAnnot);
		_ItemIcon = Utility::GetAnnotBitmap(pSourceAnnot->IsArrow ? L"Arrow" : L"Line");
	}
	else if (eType == AnnotationType::FreeText)
	{
		auto pSourceAnnot = dynamic_cast<FreeTextAnnot^>(_pAnnot);
		auto pIntent = pSourceAnnot->Intent;
		_ItemIcon = Utility::GetAnnotBitmap(pIntent->Length() > 0 ? pIntent : L"FreeText");
	}
	/*else if (eType == AnnotationType::Caret)
	{
		auto pSourceAnnot = dynamic_cast<CaretAnnot^>(_pAnnot);
		auto pIntent = pSourceAnnot->Intent;
		_ItemIcon = Utility::GetAnnotBitmap(pIntent->Length() > 0 ? pIntent : L"Caret");
	}*/
	else
	{
		_ItemIcon = Utility::GetAnnotBitmap(FoxitTypeConvert::ConvertFromFoxitAnnotType(_pAnnot->AnnotHandleWrap->PDF_Annot->GetType()));
	}
}

Platform::String^ AnnotationItem::ItemTitle::get()
{
	return _ItemTitle;
}

void AnnotationItem::ItemTitle::set(Platform::String^ value)
{
	if (_ItemTitle != value)
	{
		_ItemTitle = value;
		OnPropertyChanged("ItemTitle");
	}
}

Windows::UI::Xaml::Media::ImageSource^ AnnotationItem::ItemIcon::get()
{
	return _ItemIcon;
}

foxit::rdk::uiextensions::IBaseAnnot^ AnnotationItem::AnnotHandle::get()
{
	return _pAnnot;
}

AnnotationItemsGroup::AnnotationItemsGroup(int nPageIndex) : _PageIndex(nPageIndex)
{
	_Title = L"Page " + (nPageIndex + 1);
	_ItemsList = ref new Platform::Collections::Vector<AnnotationItem^>();
}

int AnnotationItemsGroup::PageIndex::get()
{
	return _PageIndex;
}

Platform::String^ AnnotationItemsGroup::Title::get()
{
	return _Title;
}

void AnnotationItemsGroup::Title::set(Platform::String^ value)
{
	if (_Title != value)
	{
		_Title = value;
		OnPropertyChanged("Title");
	}
}

Windows::Foundation::Collections::IObservableVector<AnnotationItem^>^ AnnotationItemsGroup::ItemsList::get()
{
	return _ItemsList;
}


// https://go.microsoft.com/fwlink/?LinkId=234238 上介绍了“空白页”项模板

AnnotationsPage::AnnotationsPage()
{
	InitializeComponent();

	OnInitialize();
}

AnnotationsPage::AnnotationsPage(UIExtensionsManager^ pMgr)
{
	InitializeComponent();

	OnInitialize();

	SetUIExtensions(pMgr);
}

UIExtensionsManager^ AnnotationsPage::UIMgr::get()
{
	return m_pUIExtensionsManagerWeakRef.Resolve<UIExtensionsManager>();
}

void AnnotationsPage::SetUIExtensions(UIExtensionsManager^ pMgr)
{
	m_pUIExtensionsManagerWeakRef = pMgr;

	if (m_FlyoutHosted == nullptr)
	{
		auto _ResourceLoader = Windows::ApplicationModel::Resources::ResourceLoader::GetForCurrentView("foxit.rdk.uiextensions/Resources");

		m_FlyoutHosted = ref new FlyoutHostedControl();
		m_FlyoutHosted->Title = _ResourceLoader->GetString(L"AnnotationsPageTitleText");
		m_FlyoutHosted->Content = this;
	}
}

bool AnnotationsPage::IsValidUIExtensions()
{
	return m_pUIExtensionsManagerWeakRef && m_pUIExtensionsManagerWeakRef.Resolve<UIExtensionsManager>() != nullptr ? true : false;
}

void AnnotationsPage::ShowFlyoutPage()
{
	if (!IsValidUIExtensions())
		return;

	auto pMainAppBar = UIMgr->GetMainAppBar();
	pMainAppBar->ActivateAppBar();
	
	m_FlyoutHosted->ShowFlyoutPage(UIMgr->FlyoutPlacement);

	pMainAppBar->HideSecondaryPopup();
	pMainAppBar->SetMainAppBarCompact(true);
}

void AnnotationsPage::CloseFlyoutPage()
{
	m_FlyoutHosted->CloseFlyout();
}

bool AnnotationsPage::IsPin::get()
{
	return m_bIsPin;
}

void AnnotationsPage::IsPin::set(bool value)
{
	if (m_bIsPin == value)
		return;

	if (this->Parent)
	{
		m_FlyoutHosted->CloseFlyout();

		m_bIsPin = value;
		m_FlyoutHosted->IsPinEnabled = value;

		ShowFlyoutPage();
	}
	else
	{
		m_bIsPin = value;
		m_FlyoutHosted->IsPinEnabled = value;
	}
}

bool AnnotationsPage::IsShow::get()
{
	return this->Parent ? true : false;
}

void AnnotationsPage::GetAnnotationsList()
{
	if (m_bIsGetAnnots)
		return;

	m_bIsGetAnnots = true;
	ShowProgressRing();

	GetAnnotationsListAsync()
		.then([=](Platform::Collections::Vector<AnnotationItemsGroup^>^ pAnnotsList)
	{
		HideProgressRing();

		if (pAnnotsList->Size == 0)
			return;

		Platform::Array<AnnotationItemsGroup^>^ pArray = ref new Platform::Array<AnnotationItemsGroup^>(pAnnotsList->Size);
		pAnnotsList->GetMany(0, pArray);
		m_AnnotationsList->ReplaceAll(pArray);
	}, concurrency::task_continuation_context::use_current())
		.then([=](concurrency::task<void> t)
	{
		try
		{
			t.get();
		}
		catch (...)
		{
			HideProgressRing();
		}
	});
}

concurrency::task<Platform::Collections::Vector<AnnotationItemsGroup^>^> AnnotationsPage::GetAnnotationsListAsync()
{
	auto spAnnotsList = std::make_shared<Platform::Collections::Vector<AnnotationItemsGroup^>^>(ref new Platform::Collections::Vector<AnnotationItemsGroup^>());
	auto pSdkDoc = UIMgr->viewControl->GetDoc();
	auto pAnnotMgr = UIMgr->getAnnotMgr();
	int nPageCount = pSdkDoc->GetPageCount();

	int nMinIterCount = 15;
	int nPerMinIterCount = nPageCount / nMinIterCount;
	int nSplit = 5;
	if (nPerMinIterCount <= 5)
	{
		nSplit = 5;
	}
	else if (nPerMinIterCount <= 50)
	{
		nSplit = 10;
	}
	else if (nPerMinIterCount <= 100)
	{
		nSplit = 20;
	}
	else
	{
		nSplit = 50;
	}

	std::vector<concurrency::task<Platform::Collections::Vector<AnnotationItemsGroup^>^>> _Tasks;
	int nCount = nPageCount / nSplit + 1;
	for (int i = 0; i < nCount; i++)
	{
		int nPerCount = (i + 1)*nSplit > nPageCount ? nPageCount - i*nSplit : nSplit;
		_Tasks.push_back(GetAnnotationsListAsync(i*nSplit, nPerCount));
	}

	return concurrency::when_all(std::begin(_Tasks), std::end(_Tasks))
		.then([=](std::vector<Platform::Collections::Vector<AnnotationItemsGroup^>^> returnValues)
	{
		auto retSize = returnValues.size();

		for (size_t i = 0; i < retSize; i++)
		{
			auto item = returnValues[i];
			auto _itemSize = item->Size;
			for (unsigned int j = 0; j < _itemSize; j++)
			{
				(*spAnnotsList)->Append(item->GetAt(j));
			}
		}
		auto pAnnotsList = *spAnnotsList;

		std::sort(begin(pAnnotsList), end(pAnnotsList), [](AnnotationItemsGroup^ a, AnnotationItemsGroup^ b)
		{
			return a->PageIndex < b->PageIndex;
		});

		return *spAnnotsList;
	});
}

concurrency::task<Platform::Collections::Vector<AnnotationItemsGroup^>^> AnnotationsPage::GetAnnotationsListAsync(int startIndex, int count)
{
	auto spAnnotsList = std::make_shared<Platform::Collections::Vector<AnnotationItemsGroup^>^>(nullptr);

	auto uiDelegate = [=]()
	{
		auto pSdkDoc = UIMgr->viewControl->GetDoc();
		auto pAnnotMgr = UIMgr->getAnnotMgr();
		int nPageCount = pSdkDoc->GetPageCount();
		auto pAnnotsList = ref new Platform::Collections::Vector<AnnotationItemsGroup^>();
		*spAnnotsList = pAnnotsList;

		//OutputDebugString((L"GetAnnotationsListAsync " + startIndex + L", " + count + L"\n")->Data());
		for (int i = startIndex; i < startIndex + count; i++)
		{
			AnnotationItemsGroup^ pGroup = ref new AnnotationItemsGroup(i);

			if (!pAnnotMgr->IsInitAnnot(i))
				pAnnotMgr->OnInitAnnot(i);

			if (!pAnnotMgr->AnnotCollections->HasKey(i))
				continue;

			auto pPageAnnots = pAnnotMgr->AnnotCollections->Lookup(i);
			auto pPageAnnotsSize = pPageAnnots->Size;
			for (unsigned int j = 0; j < pPageAnnotsSize; j++)
			{
				auto pAnnot = pPageAnnots->GetAt(j);
				auto sdkType = pAnnot->AnnotHandleWrap->PDF_Annot->GetType();
				auto eType = pAnnot->AnnotType;
				if (eType == AnnotationType::Widget || eType == AnnotationType::NonSurport || eType == AnnotationType::Link)
					continue;

				foxit::rdk::uiextensions::PDFAnnotWrapper^ wrapper = (foxit::rdk::uiextensions::PDFAnnotWrapper^)(pAnnot->AnnotHandleWrap->PDFAnnot);
				auto pBaseAnnot = foxit::rdk::FXDocUtil::DynamicCastBaseAnnot((foxit::pdf::annots::Annot^)wrapper->Pointer);
				auto pIMarkupAnnot = dynamic_cast<foxit::pdf::annots::IMarkup^>(pBaseAnnot);
				if (pIMarkupAnnot)
				{
					if (pIMarkupAnnot->IsGrouped())
					{
						auto pIMarkupAnnotIndex = pIMarkupAnnot->GetIndex();
						auto pIMarkupInconnt = pIMarkupAnnot->GetIntent();

						auto pHeaderAnnot = pIMarkupAnnot->GetGroupHeader();
						auto pBaseHeaderAnnotIndex = !pHeaderAnnot->IsEmpty() ? pHeaderAnnot->GetIndex() : -1;
						auto pHeaderAnnotInconnt = pHeaderAnnot->GetIntent();
						if (pIMarkupAnnotIndex != pBaseHeaderAnnotIndex)
						{
							OutputDebugString(L"Markup is not groupheader.\n");
							//continue;
						}
					}
				}

				AnnotationItem^ pItem = ref new AnnotationItem(pAnnot);

				pGroup->ItemsList->Append(pItem);
			}

			if (pGroup->ItemsList->Size > 0)
				pAnnotsList->Append(pGroup);
		}
	};

	return concurrency::create_task(this->Dispatcher->RunAsync(Windows::UI::Core::CoreDispatcherPriority::Low, ref new Windows::UI::Core::DispatchedHandler(uiDelegate)))
		.then([=]()
	{
		return (*spAnnotsList);
	});
}

void AnnotationsPage::ShowProgressRing()
{
	if (ProgressRingPanel->Visibility == Windows::UI::Xaml::Visibility::Collapsed)
		ProgressRingPanel->Visibility = Windows::UI::Xaml::Visibility::Visible;
}

void AnnotationsPage::HideProgressRing()
{
	if (ProgressRingPanel->Visibility == Windows::UI::Xaml::Visibility::Visible)
		ProgressRingPanel->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
}

void AnnotationsPage::RegisterAnnotEvent()
{
	if (IsValidUIExtensions())
	{
		if (m_AddAnnotEventToken.Value == 0)
			m_AddAnnotEventToken = UIMgr->AnnotMgr->AddAnnotEvent += ref new Windows::Foundation::EventHandler<foxit::rdk::uiextensions::IBaseAnnot ^>(this, &foxit::rdk::uiextensions::AnnotationsPage::OnAddAnnotEvent);
		if (m_DeleteAnnotEventToken.Value == 0)
			m_DeleteAnnotEventToken = UIMgr->AnnotMgr->DeleteAnnotEvent += ref new Windows::Foundation::EventHandler<foxit::rdk::uiextensions::IBaseAnnot ^>(this, &foxit::rdk::uiextensions::AnnotationsPage::OnDeleteAnnotEvent);
		if (m_ModifyAnnotEventToken.Value == 0)
			m_ModifyAnnotEventToken = UIMgr->AnnotMgr->ModifyAnnotEvent += ref new Windows::Foundation::EventHandler<foxit::rdk::uiextensions::IBaseAnnot ^>(this, &foxit::rdk::uiextensions::AnnotationsPage::OnModifyAnnotEvent);
	}
}

void AnnotationsPage::UnRegisterAnnotEvent()
{
	if (IsValidUIExtensions())
	{
		if (m_AddAnnotEventToken.Value)
		{
			UIMgr->AnnotMgr->AddAnnotEvent -= m_AddAnnotEventToken;
			m_AddAnnotEventToken.Value = 0;
		}
		if (m_DeleteAnnotEventToken.Value)
		{
			UIMgr->AnnotMgr->DeleteAnnotEvent -= m_DeleteAnnotEventToken;
			m_DeleteAnnotEventToken.Value = 0;
		}
		if (m_ModifyAnnotEventToken.Value)
		{
			UIMgr->AnnotMgr->ModifyAnnotEvent -= m_ModifyAnnotEventToken;
			m_ModifyAnnotEventToken.Value = 0;
		}
	}
}

void AnnotationsPage::OnInitialize()
{
	m_AnnotationsList = ref new Platform::Collections::Vector<AnnotationItemsGroup^>();
	AnnotationsListViewSource->Source = m_AnnotationsList;

	this->Loaded += ref new Windows::UI::Xaml::RoutedEventHandler(this, &foxit::rdk::uiextensions::AnnotationsPage::OnLoaded);
	this->Unloaded += ref new Windows::UI::Xaml::RoutedEventHandler(this, &foxit::rdk::uiextensions::AnnotationsPage::OnUnloaded);
}


void foxit::rdk::uiextensions::AnnotationsPage::OnLoaded(Platform::Object ^sender, Windows::UI::Xaml::RoutedEventArgs ^e)
{
	RegisterAnnotEvent();
	
	if (!IsValidUIExtensions())
		return;

	auto pMainAppBar = UIMgr->GetMainAppBar();
	pMainAppBar->SetFlyoutshowFlag(true);

	GetAnnotationsList();
}


void foxit::rdk::uiextensions::AnnotationsPage::OnUnloaded(Platform::Object ^sender, Windows::UI::Xaml::RoutedEventArgs ^e)
{
	if (!IsValidUIExtensions())
		return;

	auto pMainAppBar = UIMgr->GetMainAppBar();
	pMainAppBar->SetFlyoutshowFlag(false);
}


void foxit::rdk::uiextensions::AnnotationsPage::ItemsList_ItemClick(Platform::Object^ sender, Windows::UI::Xaml::Controls::ItemClickEventArgs^ e)
{
	AnnotationItem^ pItem = dynamic_cast<AnnotationItem^>(e->ClickedItem);
	if (pItem == nullptr)
		return;

	auto pAnnot = pItem->AnnotHandle;
	if (pAnnot == nullptr)
		return;

	auto nCurIndex = UIMgr->GetCurrentPageIndex();
	int nGotoIndex = pAnnot->PageIndex;
	auto annotRc = pAnnot->MinRect;
	auto eType = pAnnot->AnnotType;
	if (eType == AnnotationType::Text)
	{
		auto pSourceAnnot = dynamic_cast<TextAnnot^>(pAnnot);
		annotRc = pSourceAnnot->DrawRect;
	}

	//if (nCurIndex != nGotoIndex)
	UIMgr->viewControl->GotoPagePoint(nGotoIndex, Windows::Foundation::Point(annotRc.X, annotRc.Y));

	Windows::Foundation::Rect deviceRect = UIMgr->viewControl->ConvertPageRectToDeviceRect(nGotoIndex, annotRc);

	UIMgr->AnnotHighlightFlicker(deviceRect);

	if (Utility::IsMobilePlatform())
		m_FlyoutHosted->CloseFlyout();
}


void foxit::rdk::uiextensions::AnnotationsPage::OnAddAnnotEvent(Platform::Object ^sender, foxit::rdk::uiextensions::IBaseAnnot ^args)
{
	if (!args)
		return;

	auto pAnnotMgr = UIMgr->getAnnotMgr();
	auto _findIter = std::find_if(begin(m_AnnotationsList), end(m_AnnotationsList), [args](AnnotationItemsGroup^ item)
	{
		return item && item->PageIndex == args->PageIndex;
	});

	AnnotationItemsGroup^ _item = nullptr;
	if (_findIter == end(m_AnnotationsList))
	{
		_item = ref new AnnotationItemsGroup(args->PageIndex);

		auto _InertIter = std::find_if(begin(m_AnnotationsList), end(m_AnnotationsList), [args](AnnotationItemsGroup^ item)
		{
			return item && item->PageIndex > args->PageIndex;
		});

		if (_InertIter == end(m_AnnotationsList))
			m_AnnotationsList->Append(_item);
		else
			m_AnnotationsList->InsertAt(_InertIter - begin(m_AnnotationsList), _item);
	}
	else
	{
		_item = (*_findIter);
	}

	AnnotationItem^ pItem = ref new AnnotationItem(args);

	_item->ItemsList->Append(pItem);
}


void foxit::rdk::uiextensions::AnnotationsPage::OnDeleteAnnotEvent(Platform::Object ^sender, foxit::rdk::uiextensions::IBaseAnnot ^args)
{
	if (!args)
		return;

	auto pAnnotMgr = UIMgr->getAnnotMgr();
	auto _findIter = std::find_if(begin(m_AnnotationsList), end(m_AnnotationsList), [args](AnnotationItemsGroup^ item)
	{
		return item && item->PageIndex == args->PageIndex;
	});

	AnnotationItemsGroup^ _item = nullptr;
	if (_findIter == end(m_AnnotationsList))
	{
		return;
	}
	else
	{
		_item = (*_findIter);
	}

	auto itemList = _item->ItemsList;

	auto _DeleteIter = std::find_if(begin(itemList), end(itemList), [args](AnnotationItem^ item)
	{
		return item && item->AnnotHandle == args;
	});

	if (_DeleteIter != end(itemList))
	{
		itemList->RemoveAt(_DeleteIter - begin(itemList));

		if (itemList->Size == 0)
			m_AnnotationsList->RemoveAt(_findIter - begin(m_AnnotationsList));
	}
}

void foxit::rdk::uiextensions::AnnotationsPage::OnModifyAnnotEvent(Platform::Object ^sender, foxit::rdk::uiextensions::IBaseAnnot ^args)
{
	if (!args)
		return;

	auto pAnnotMgr = UIMgr->getAnnotMgr();
	auto _findIter = std::find_if(begin(m_AnnotationsList), end(m_AnnotationsList), [args](AnnotationItemsGroup^ item)
	{
		return item && item->PageIndex == args->PageIndex;
	});

	AnnotationItemsGroup^ _item = nullptr;
	if (_findIter == end(m_AnnotationsList))
	{
		return;
	}
	else
	{
		_item = (*_findIter);
	}

	auto itemList = _item->ItemsList;

	auto _DeleteIter = std::find_if(begin(itemList), end(itemList), [args](AnnotationItem^ item)
	{
		return item && item->AnnotHandle == args;
	});

	if (_DeleteIter != end(itemList))
	{
		(*_DeleteIter)->InitializeTitle();
	}
}

