﻿/**
* Copyright (C) 2003-2018, Foxit Software Inc..
* All Rights Reserved.
*
* http://www.foxitsoftware.com
*
* The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
* distribute any parts of Foxit Mobile PDF SDK to third party or public without permission unless an agreement
* is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
* Review legal.txt for additional license and legal information.
*/


#pragma once

#include "Annotations\AnnotationsPage.g.h"

namespace foxit
{
	namespace rdk
	{
		namespace uiextensions
		{
			[Windows::UI::Xaml::Data::Bindable]
			[Windows::Foundation::Metadata::WebHostHidden]
			public ref class AnnotationItem sealed : public BindableBase
			{
			public:
				AnnotationItem(IBaseAnnot^ pAnnot);

				property Platform::String^ ItemTitle
				{
					Platform::String^ get();
					void set(Platform::String^ value);
				}

				property Windows::UI::Xaml::Media::ImageSource^ ItemIcon
				{
					Windows::UI::Xaml::Media::ImageSource^ get();
				}

				property foxit::rdk::uiextensions::IBaseAnnot^ AnnotHandle
				{
					foxit::rdk::uiextensions::IBaseAnnot^ get();
				}

			internal:
				void InitializeTitle();
				void InitializeIcon();

			private:
				Platform::String^ _ItemTitle;
				Windows::UI::Xaml::Media::Imaging::BitmapImage^ _ItemIcon;

				foxit::rdk::uiextensions::IBaseAnnot^ _pAnnot;
			};

			[Windows::UI::Xaml::Data::Bindable]
			[Windows::Foundation::Metadata::WebHostHidden]
			public ref class AnnotationItemsGroup sealed : public BindableBase
			{
			public:
				AnnotationItemsGroup(int nPageIndex);

				property int PageIndex
				{
					int get();
				}

				property Platform::String^ Title
				{
					Platform::String^ get();
					void set(Platform::String^ value);
				}
				property Windows::Foundation::Collections::IObservableVector<AnnotationItem^>^ ItemsList
				{
					Windows::Foundation::Collections::IObservableVector<AnnotationItem^>^ get();
				}

			private:
				int _PageIndex;
				Platform::String^ _Title;
				Platform::Collections::Vector<AnnotationItem^>^ _ItemsList;
			};

			/// <summary>
			/// 可用于自身或导航至 Frame 内部的空白页。
			/// </summary>
			[Windows::Foundation::Metadata::WebHostHidden]
			public ref class AnnotationsPage sealed
			{
			public:
				AnnotationsPage();
				AnnotationsPage(UIExtensionsManager^ pMgr);

				void ShowFlyoutPage();
				void CloseFlyoutPage();

				property bool IsPin
				{
					bool get();
					void set(bool value);
				}

				property bool IsShow
				{
					bool get();
				}

			internal:
				property UIExtensionsManager^ UIMgr
				{
					UIExtensionsManager^ get();
				}

				void RegisterAnnotEvent();
				void UnRegisterAnnotEvent();

			private:
				Platform::WeakReference m_pUIExtensionsManagerWeakRef;
				Windows::Foundation::EventRegistrationToken m_WindowSizeChangedEventToken;
				Windows::Foundation::EventRegistrationToken m_AddAnnotEventToken, m_DeleteAnnotEventToken, m_ModifyAnnotEventToken;
				FlyoutHostedControl^ m_FlyoutHosted;

				Platform::Collections::Vector<AnnotationItemsGroup^>^ m_AnnotationsList;

				bool m_bIsPin;
				bool m_bIsGetAnnots;

			private:
				void SetUIExtensions(UIExtensionsManager^ pMgr);
				bool IsValidUIExtensions();

				void GetAnnotationsList();
				concurrency::task<Platform::Collections::Vector<AnnotationItemsGroup^>^> GetAnnotationsListAsync();
				concurrency::task<Platform::Collections::Vector<AnnotationItemsGroup^>^> GetAnnotationsListAsync(int startIndex, int count);

				void ShowProgressRing();
				void HideProgressRing();

			private:
				void OnInitialize();
				void OnLoaded(Platform::Object ^sender, Windows::UI::Xaml::RoutedEventArgs ^e);
				void OnUnloaded(Platform::Object ^sender, Windows::UI::Xaml::RoutedEventArgs ^e);
				void ItemsList_ItemClick(Platform::Object^ sender, Windows::UI::Xaml::Controls::ItemClickEventArgs^ e);
				void OnAddAnnotEvent(Platform::Object ^sender, foxit::rdk::uiextensions::IBaseAnnot ^args);
				void OnDeleteAnnotEvent(Platform::Object ^sender, foxit::rdk::uiextensions::IBaseAnnot ^args);
				void OnModifyAnnotEvent(Platform::Object ^sender, foxit::rdk::uiextensions::IBaseAnnot ^args);
			};
		}
	}
}
