/**
 * Copyright (C) 2003-2018, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit Mobile PDF SDK to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */


#include "pch.h"
#include "CaretAnnotEventPeer.h"
#include "../Text/PDFTextManager.h"
#include "AnnotNoteUserControl.xaml.h"

using namespace foxit::rdk;
using namespace foxit::rdk::uiextensions;

using namespace Platform;
using namespace Windows::Foundation;


CaretAnnotEventPeer::CaretAnnotEventPeer(PDFViewCtrl^ view)
{
	m_PageViewWeakRef = view;
	m_TextSelectContext = ref new TextSelectContext();
}

PDFViewCtrl^ CaretAnnotEventPeer::View::get()
{
	return m_PageViewWeakRef.Resolve<PDFViewCtrl>();
}

void CaretAnnotEventPeer::OnPointerPressed(Platform::Object^ sender, Windows::UI::Xaml::Input::PointerRoutedEventArgs^ e)
{
	Windows::UI::Xaml::FrameworkElement^ _FrameworkElement = dynamic_cast<Windows::UI::Xaml::FrameworkElement^>(sender);
	Windows::Foundation::Point _Pt;
	auto _PointerPt = e->GetCurrentPoint(_FrameworkElement);
	_Pt = _PointerPt->Position;
	auto _Pageinfo = View->ConvertDevicePtToPDFPagePt(_Pt);
	int index = _Pageinfo->PageIndex;
	auto _PagePoint = _Pageinfo->PagePoint;
	m_TextSelectContext->StartIndex = index;
	m_TextSelectContext->StartPoint = _PagePoint;
	_FrameworkElement->ManipulationMode = Windows::UI::Xaml::Input::ManipulationModes::System;
}

void CaretAnnotEventPeer::OnPointerMoved(Platform::Object^ sender, Windows::UI::Xaml::Input::PointerRoutedEventArgs^ e)
{

}

void CaretAnnotEventPeer::OnPointerReleased(Platform::Object^ sender, Windows::UI::Xaml::Input::PointerRoutedEventArgs^ e)
{
	UIExtensionsManager^ uiMgr = (UIExtensionsManager^)View->ExtensionsManager;
	Windows::UI::Xaml::FrameworkElement^ _FrameworkElement = dynamic_cast<Windows::UI::Xaml::FrameworkElement^>(sender);
	Windows::Foundation::Point _Pt;
	auto _PointerPt = e->GetCurrentPoint(_FrameworkElement);
	_Pt = _PointerPt->Position;
	auto _Pageinfo = View->ConvertDevicePtToPDFPagePt(_Pt);
	int index = _Pageinfo->PageIndex;
	auto _PagePoint = _Pageinfo->PagePoint;
	m_TextSelectContext->EndIndex = index;
	m_TextSelectContext->EndPoint = _PagePoint;

	auto _AnnotSetting = uiMgr->getAnnotMgr()->GetAnnotSetting();
	int pageRotate = View->GetRotate();
	double scale = View->GetScale();
	Point compositePT = Point(View->CompositionScaleX, View->CompositionScaleY);
	double hOffset = 8* scale;
	double vOffset = 12 * scale;

	auto _selectedtext = uiMgr->getTextMgr()->SelectedText;
	if (_selectedtext && _selectedtext->Size > 0 && _selectedtext->GetAt(0)->TextRectVect->Size>0)
	{
		auto size = _selectedtext->Size;
		bool _bIsClearText = true;
		Windows::Foundation::Rect rect = _selectedtext->GetAt(0)->TextRectVect->GetAt(0);
		for (unsigned int i = 0; i < size; i++)
		{
			auto rectGroup = ref new Platform::Collections::Vector<Windows::Foundation::Rect>();
			int count=_selectedtext->GetAt(i)->TextRectVect->Size;
			int textrotate = _selectedtext->GetAt(i)->TextRotateVect->GetAt(0);
			for each (Windows::Foundation::Rect gmt in _selectedtext->GetAt(i)->TextRectVect)
			{
				rectGroup->Append(gmt);
			}
			
			
			auto _selectpageindex = _selectedtext->GetAt(i)->PageIndex;
			auto _page = View->GetDoc()->GetPage(_selectpageindex);

			auto _CaretAnnot = foxit::rdk::uiextensions::AnnotCreaterFactory::GetInstance()->CreateNewAnnot(AnnotationType::Caret);
			auto _pCreateCaretAnnot = dynamic_cast<CaretAnnot^>(_CaretAnnot);
			auto rect = rectGroup->GetAt(rectGroup->Size - 1);

			Windows::Foundation::Rect textRangeRect = rect;
			if ((textrotate>0 && textrotate<=90) || (textrotate>180 && textrotate<=270))
			{
				double width = rect.Width;
				rect.Width = rect.Height;
				rect.Height = (float)width;
			}

			double strHeight = rect.Height;
			double strWidth = strHeight*(2.0 / 3.0);
			if (GetIsDesktopPlatform())
			{
				if (rect.Height > 14)
				{
					strHeight = strHeight*0.7;
				}
				else if (rect.Height < 6.0)
				{
					strHeight = 6.0;
				}
				strWidth= strHeight*(2.0 / 3.0);
			}
			else
			{
				if (rect.Height > 20)
				{
					strHeight = strHeight*0.8;
					strWidth = strHeight*(2.0 / 3.0);
				}				
				else if (rect.Height <= 10.0)
				{
					if (rect.Height < 6.0)
						strHeight = 6.0;
					strWidth = strHeight*(2.0 / 3.0);
				}
				else if (rect.Height >10.0 && rect.Height<20)
				{
					strWidth = strHeight*0.85;
				}
			}			
			
			Windows::Foundation::Rect pagerect = Windows::Foundation::Rect(rect.X+rect.Width- (float)(strWidth/2.0),rect.Y- (float)(strHeight /2.0),(float)strWidth, (float)strHeight);
		
			if ((textrotate > 0 && textrotate <= 90) || (textrotate > 180 && textrotate <= 270))
			{
				pagerect = Windows::Foundation::Rect(rect.X - (float)(strWidth / 2.0), rect.Y - (float)(strHeight / 2.0), (float)strWidth, (float)strHeight);
			}
			
			auto deviceRect = View->ConvertPageRectToDeviceRect(index, textRangeRect);
		
			
			_pCreateCaretAnnot->InnerMargin = (int)(1.3);
			_pCreateCaretAnnot->Color = _AnnotSetting->GetInsertTextColor();
			_pCreateCaretAnnot->Opacity = _AnnotSetting->GetInsertTextAnnotOpacity();
			_pCreateCaretAnnot->Page_HightLightRect = pagerect;
			_pCreateCaretAnnot->Name = uiMgr->getAnnotMgr()->GetAnnotSetting()->GetNoteAuthor();
			_pCreateCaretAnnot->Title = uiMgr->getAnnotMgr()->GetAnnotSetting()->GetNoteAuthor();
			_pCreateCaretAnnot->CreationDateTime = (ref new Windows::Globalization::Calendar())->GetDateTime();		
			_pCreateCaretAnnot->TextRotate = 0;

			auto _createtype = uiMgr->CreateTypeOfAnnot;			
		
			Point curPt = Point(deviceRect.X, deviceRect.Y);
			if (uiMgr->CreateTypeOfAnnot == AnnotCreateType::ReplaceText)
			{
				_pCreateCaretAnnot->Color = _AnnotSetting->GetReplaceTextColor();
				_pCreateCaretAnnot->Opacity = _AnnotSetting->GetReplaceTextAnnotOpacity();

				auto _Annot = foxit::rdk::uiextensions::AnnotCreaterFactory::GetInstance()->CreateNewAnnot(AnnotationType::StrikeOut);
				auto _pCreateAnnot = dynamic_cast<StrikeOutAnnot^>(_Annot);
				Platform::String^ colorRGBValue = Utility::ConvertToColorString(_AnnotSetting->GetStrikeOutAnnotColor());

				_pCreateAnnot->Color = _AnnotSetting->GetReplaceTextColor();
				_pCreateAnnot->Opacity = _AnnotSetting->GetReplaceTextAnnotOpacity();
				_pCreateAnnot->Page_HightLightRectGroup = rectGroup;
				_pCreateAnnot->Name = uiMgr->getAnnotMgr()->GetAnnotSetting()->GetNoteAuthor();
				_pCreateAnnot->Title = _AnnotSetting->GetNoteAuthor();
				_pCreateAnnot->CreationDateTime = (ref new Windows::Globalization::Calendar())->GetDateTime();
				_pCreateAnnot->TextRotateVect = _selectedtext->GetAt(i)->TextRotateVect;

				auto _msgDlg = ref new MessageDlg();
				auto _AnnotNoteUserControl = ref new AnnotNoteUserControl(View, curPt);
				auto _AnnotComandToken = ref new AnnotCommandToken();
				_AnnotComandToken->Pageindex = index;
				_AnnotComandToken->Param = _pCreateCaretAnnot;
				_AnnotComandToken->Type = AnnotCommandType::Add;
				_AnnotComandToken->StrickoutParam = _pCreateAnnot;
				_AnnotNoteUserControl->Tag = _AnnotComandToken;			

				uiMgr->ShowAddNoteWindows(_AnnotNoteUserControl, curPt);
			}
			else
			{
				auto _msgDlg = ref new MessageDlg();
				auto _AnnotNoteUserControl = ref new AnnotNoteUserControl(View, curPt);
				auto _AnnotComandToken = ref new AnnotCommandToken();
				_AnnotComandToken->Pageindex = index;
				_AnnotComandToken->Param = _pCreateCaretAnnot;
				_AnnotComandToken->Type = AnnotCommandType::Add;
				_AnnotNoteUserControl->Tag = _AnnotComandToken;

				uiMgr->ShowAddNoteWindows(_AnnotNoteUserControl, curPt);
			}
		}
	}
	else
	{
		if (uiMgr->CreateTypeOfAnnot == AnnotCreateType::Caret)
		{
			bool  isHitText = uiMgr->getTextMgr()->IsHitText(index, _PagePoint);
			if (isHitText)
			{
				auto _CaretAnnot = foxit::rdk::uiextensions::AnnotCreaterFactory::GetInstance()->CreateNewAnnot(AnnotationType::Caret);
				auto _pCreateCaretAnnot = dynamic_cast<CaretAnnot^>(_CaretAnnot);
				
				IPageSelectText^ selectText= uiMgr->getTextMgr()->GetTextRect(index, _PagePoint);
				if (!selectText)
					return;

				Windows::Foundation::Rect pagerect = selectText->TextRectVect->GetAt(0);
				int rotate = selectText->TextRotate;

				auto deviceRc = View->ConvertPageRectToDeviceRect(index, pagerect);
				Windows::Foundation::Rect rect = deviceRc;				

				_pCreateCaretAnnot->Color = _AnnotSetting->GetInsertTextColor();
				_pCreateCaretAnnot->Opacity = _AnnotSetting->GetInsertTextAnnotOpacity();
				_pCreateCaretAnnot->Page_HightLightRect = pagerect;
				_pCreateCaretAnnot->Name = uiMgr->getAnnotMgr()->GetAnnotSetting()->GetNoteAuthor();
				_pCreateCaretAnnot->Title = uiMgr->getAnnotMgr()->GetAnnotSetting()->GetNoteAuthor();
				_pCreateCaretAnnot->CreationDateTime = (ref new Windows::Globalization::Calendar())->GetDateTime();
				_pCreateCaretAnnot->TextRotate = rotate;
				_pCreateCaretAnnot->InnerMargin = (int)(1.3);

				Point curPt = Point(rect.X, rect.Y);
				double xvalue = rect.X;
				double yvalue = rect.Y;
				if (xvalue == rect.X && yvalue == rect.Y)
				{
					auto _msgDlg = ref new MessageDlg();
					auto _AnnotNoteUserControl = ref new AnnotNoteUserControl(View, curPt);
					auto _AnnotComandToken = ref new AnnotCommandToken();
					_AnnotComandToken->Pageindex = index;
					_AnnotComandToken->Param = _pCreateCaretAnnot;
					_AnnotComandToken->Type = AnnotCommandType::Add;
					_AnnotNoteUserControl->Tag = _AnnotComandToken;

					uiMgr->ShowAddNoteWindows(_AnnotNoteUserControl, curPt);
				}
			}
		}
	}

	m_TextSelectContext->ResetContext();
	_FrameworkElement->ManipulationMode = Windows::UI::Xaml::Input::ManipulationModes::All;
}

void CaretAnnotEventPeer::OnTapped(Platform::Object^ sender, Windows::UI::Xaml::Input::TappedRoutedEventArgs^ e)
{
	UIExtensionsManager^ uiMgr = (UIExtensionsManager^)View->ExtensionsManager;
	auto _SelectedAnnot = uiMgr->getAnnotMgr()->GetSelectedAnnot();
	if (_SelectedAnnot)
	{
		Windows::Foundation::Point curPt = e->GetPosition(nullptr);
		Windows::Foundation::Rect rect = _SelectedAnnot->MinRect;
		auto deviceRect = View->ConvertPageRectToDeviceRect(_SelectedAnnot->PageIndex, rect);

		auto control = ref new AnnotNoteUserControl(View, curPt);
		uiMgr->ShowAddNoteWindows(control, curPt);
	}
}

void CaretAnnotEventPeer::OnRightTapped(Platform::Object^ sender, Windows::UI::Xaml::Input::RightTappedRoutedEventArgs^ e)
{

}

void CaretAnnotEventPeer::OnManipulationStarted(Platform::Object^ sender, Windows::UI::Xaml::Input::ManipulationStartedRoutedEventArgs^ e)
{

}

void CaretAnnotEventPeer::OnManipulationDelta(Platform::Object^ sender, Windows::UI::Xaml::Input::ManipulationDeltaRoutedEventArgs^ e)
{

}

void CaretAnnotEventPeer::OnManipulationCompleted(Platform::Object^ sender, Windows::UI::Xaml::Input::ManipulationCompletedRoutedEventArgs^ e)
{

}

void CaretAnnotEventPeer::OnCreateAnnotAfterSelectText()
{
	UIExtensionsManager^ uiMgr = (UIExtensionsManager^)View->ExtensionsManager;
	auto _AnnotSetting = uiMgr->getAnnotMgr()->GetAnnotSetting();
	auto _selectedtext = uiMgr->getTextMgr()->SelectedText;
	if (_selectedtext && _selectedtext->Size > 0)
	{
		if (!View->GetDoc())
			return;

		auto size = _selectedtext->Size;
		bool _bIsClearText = true;
		Windows::Foundation::Rect rect = _selectedtext->GetAt(0)->TextRectVect->GetAt(0);
		for (unsigned int i = 0; i < size; i++)
		{
			auto rectGroup = ref new Platform::Collections::Vector<Windows::Foundation::Rect>();

			for each (Windows::Foundation::Rect gmt in _selectedtext->GetAt(i)->TextRectVect)
			{
				rectGroup->Append(gmt);
			}

			auto _Annot = foxit::rdk::uiextensions::AnnotCreaterFactory::GetInstance()->CreateNewAnnot(AnnotationType::StrikeOut);
			auto _pCreateAnnot = dynamic_cast<StrikeOutAnnot^>(_Annot);
			_pCreateAnnot->Color = _AnnotSetting->GetStrikeOutAnnotColor();
			_pCreateAnnot->Opacity = _AnnotSetting->GetStrikeOutAnnotOpacity();
			_pCreateAnnot->Page_HightLightRectGroup = rectGroup;
			_pCreateAnnot->Name = uiMgr->getAnnotMgr()->GetAnnotSetting()->GetNoteAuthor();
			_pCreateAnnot->Title = _AnnotSetting->GetNoteAuthor();
			_pCreateAnnot->CreationDateTime = (ref new Windows::Globalization::Calendar())->GetDateTime();
			_pCreateAnnot->TextRotateVect = _selectedtext->GetAt(i)->TextRotateVect;
			auto _selectpageindex = _selectedtext->GetAt(i)->PageIndex;

			auto _page = View->GetDoc()->GetPage(_selectpageindex);

			auto _createtype = uiMgr->CreateTypeOfAnnot;

			if (_createtype == AnnotCreateType::StrikeOut)
			{
				uiMgr->getAnnotMgr()->OnCreateNewAnnot(_page, _pCreateAnnot);

				foxit::common::fxcrt::RectF^ minRect = FoxitRectConverter::ConvertToFoxitRectF(_pCreateAnnot->MinRect);
				String^ rectStr = "[" + minRect->left.ToString() + " " + minRect->top.ToString() + " " + minRect->right.ToString() + " " + minRect->bottom.ToString() + "]";
				uiMgr->CollectAnnotOperation("Strikeout", "Add", _selectpageindex.ToString(), rectStr);
			}
		}
		if (_bIsClearText)
			uiMgr->getTextMgr()->ClearTextSelect();
	}
	m_TextSelectContext->ResetContext();

	uiMgr->CreateTypeOfAnnot = AnnotCreateType::None;
	uiMgr->ExitAnnotPropertyMode();
}

void CaretAnnotEventPeer::OnManipulationInertiaStarting(Platform::Object^ sender, Windows::UI::Xaml::Input::ManipulationInertiaStartingRoutedEventArgs^ e)
{

}
