/**
 * Copyright (C) 2003-2018, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit Mobile PDF SDK to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */


#include "pch.h"
#include "CircleAnnotEventPeer.h"

using namespace foxit::rdk;
using namespace foxit::rdk::uiextensions;

using namespace Platform;
using namespace Windows::Foundation;

namespace foxit
{
	namespace rdk
	{
		namespace uiextensions
		{
			[Windows::Foundation::Metadata::WebHostHidden]
			public ref class CreateCircleAnnotUndoAction sealed : IUndoItem
			{
			public:
				CreateCircleAnnotUndoAction(PDFViewCtrl^ view, CircleAnnot^ annot);

				virtual void Undo();
				virtual void Redo();
				virtual Platform::String^ GetDescr();
				virtual void ReleaseItem();

			internal:
				property PDFViewCtrl^ View
				{
					PDFViewCtrl^ get();
				}

				property CircleAnnot^ AnnotHandle
				{
					CircleAnnot^ get();
				}

			private:
				Platform::WeakReference m_ViewWeakRef;
				Platform::WeakReference m_AnnotWeakRef;
				BaseAnnot^ m_RedoAnnot;
			};
		}
	}
}

CreateCircleAnnotUndoAction::CreateCircleAnnotUndoAction(PDFViewCtrl^ view, CircleAnnot^ annot)
{
	m_ViewWeakRef = view;
	m_AnnotWeakRef = annot;
	m_RedoAnnot = nullptr;
}

PDFViewCtrl^ CreateCircleAnnotUndoAction::View::get()
{
	return m_ViewWeakRef.Resolve<PDFViewCtrl>();
}

CircleAnnot^ CreateCircleAnnotUndoAction::AnnotHandle::get()
{
	return m_AnnotWeakRef.Resolve<CircleAnnot>();
}

void CreateCircleAnnotUndoAction::Undo()
{
	if (View && AnnotHandle)
	{
		m_RedoAnnot = AnnotHandle;
		View->CancelRenderTask();
		AnnotHandle->OnDelete();
	}
}

void CreateCircleAnnotUndoAction::Redo()
{
	if (View)
	{
		UIExtensionsManager^ uiMgr = (UIExtensionsManager^)View->ExtensionsManager;
		auto _page = View->GetDoc()->GetPage(m_RedoAnnot->PageIndex);

		View->CancelRenderTask();
		uiMgr->getAnnotMgr()->OnCreateNewAnnot(_page, m_RedoAnnot);
	}
}

Platform::String^ CreateCircleAnnotUndoAction::GetDescr()
{
	return ref new Platform::String(L"");
}

void CreateCircleAnnotUndoAction::ReleaseItem()
{
	m_ViewWeakRef = nullptr;
	m_AnnotWeakRef = nullptr;
}


CircleAnnotEventPeer::CircleAnnotEventPeer(PDFViewCtrl^ view)
{
	m_PageViewWeakRef = view;
	m_pressStartPoint = Point(-1, -1);
	m_isPressed = false;
}

PDFViewCtrl^ CircleAnnotEventPeer::View::get()
{
	return m_PageViewWeakRef.Resolve<PDFViewCtrl>();
}

void CircleAnnotEventPeer::OnPointerPressed(Platform::Object^ sender, Windows::UI::Xaml::Input::PointerRoutedEventArgs^ e)
{
	if (!View)
		return;

	if (m_isPressed)
		return;

	UIExtensionsManager^ uiMgr = (UIExtensionsManager^)View->ExtensionsManager;
	auto _AnnotSetting = uiMgr->getAnnotMgr()->GetAnnotSetting();
	borderWidth = _AnnotSetting->GetAnnotDrawBorderWidth();

	ellipse = ref new Windows::UI::Xaml::Shapes::Ellipse();

	ellipse->Fill= ref new Windows::UI::Xaml::Media::SolidColorBrush(Windows::UI::Colors::Gray);
	ellipse->StrokeThickness = 1;
	ellipse->Opacity = 0.5;

	uiMgr->CreateNewShapeToCanvas(ellipse);

	Windows::UI::Xaml::FrameworkElement^ _FrameworkElement = dynamic_cast<Windows::UI::Xaml::FrameworkElement^>(sender);
	_FrameworkElement->ManipulationMode = Windows::UI::Xaml::Input::ManipulationModes::System;
	Windows::Foundation::Point _Pt;
	auto _PointerPt = e->GetCurrentPoint(_FrameworkElement);
	_Pt = _PointerPt->Position;

	auto _Pageinfo = View->ConvertDevicePtToPDFPagePt(_Pt);
	m_PageIndex = _Pageinfo->PageIndex;

	if (_Pageinfo->PagePoint.X != -1 && _Pageinfo->PagePoint.Y != -1)
	{
		m_isPressed = true;

		m_pressStartPoint = _Pt;
	}
}

void CircleAnnotEventPeer::OnPointerMoved(Platform::Object^ sender, Windows::UI::Xaml::Input::PointerRoutedEventArgs^ e)
{
	if (!m_isPressed)
		return;


	Windows::UI::Xaml::FrameworkElement^ _FrameworkElement = dynamic_cast<Windows::UI::Xaml::FrameworkElement^>(sender);
	Windows::Foundation::Point currentPoint;
	auto _PointerPt = e->GetCurrentPoint(_FrameworkElement);
	currentPoint = _PointerPt->Position;

	Windows::Foundation::Rect pageViewRect = View->GetPageViewRect(m_PageIndex);
	double width = 0;
	double height = 0;
	Windows::Foundation::Point  startPoint = m_pressStartPoint;


	if (m_pressStartPoint.X != -1 && m_pressStartPoint.Y != -1)
	{
		if (currentPoint.Y > pageViewRect.Y + pageViewRect.Height)
		{
			currentPoint.Y = pageViewRect.Y + pageViewRect.Height;
		}
		else if (currentPoint.Y < pageViewRect.Y)
		{
			currentPoint.Y = pageViewRect.Y;
		}
		if (currentPoint.X > pageViewRect.X + pageViewRect.Width)
		{
			currentPoint.X = pageViewRect.X + pageViewRect.Width;
		}
		else if (currentPoint.X < pageViewRect.X)
		{
			currentPoint.X = pageViewRect.X;
		}


		if (currentPoint.X > m_pressStartPoint.X)
		{
			width = currentPoint.X - m_pressStartPoint.X;
		}
		else
		{
			width = m_pressStartPoint.X - currentPoint.X;
			startPoint.X = currentPoint.X;
		}
		if (currentPoint.Y > m_pressStartPoint.Y)
		{
			height = currentPoint.Y - m_pressStartPoint.Y;
		}
		else
		{
			height = m_pressStartPoint.Y - currentPoint.Y;
			startPoint.Y = currentPoint.Y;
		}

		m_AreaRect = Rect(startPoint.X, startPoint.Y, (float)width, (float)height);


		ellipse->SetValue(Windows::UI::Xaml::Controls::Canvas::TopProperty, m_AreaRect.Y);
		ellipse->SetValue(Windows::UI::Xaml::Controls::Canvas::LeftProperty, m_AreaRect.X);

		ellipse->Width = m_AreaRect.Width;
		ellipse->Height = m_AreaRect.Height;
	}
}

void CircleAnnotEventPeer::OnPointerReleased(Platform::Object^ sender, Windows::UI::Xaml::Input::PointerRoutedEventArgs^ e)
{
	UIExtensionsManager^ uiMgr = (UIExtensionsManager^)View->ExtensionsManager;
	if (!m_isPressed)
		return;
	m_pressStartPoint = Point(-1, -1);
	m_isPressed = false;

	Windows::UI::Xaml::FrameworkElement^ _FrameworkElement = dynamic_cast<Windows::UI::Xaml::FrameworkElement^>(sender);
	_FrameworkElement->ManipulationMode = Windows::UI::Xaml::Input::ManipulationModes::All;

	auto _AnnotSetting = uiMgr->getAnnotMgr()->GetAnnotSetting();
	if (m_AreaRect.Width > 0 && m_AreaRect.Height > 0)
	{		
		auto _Annot = foxit::rdk::uiextensions::AnnotCreaterFactory::GetInstance()->CreateNewAnnot(AnnotationType::Circle);
		auto _pCreateAnnot = dynamic_cast<CircleAnnot^>(_Annot);

		auto deviceRc = View->ConvertDeviceRectToPageRect(m_PageIndex, m_AreaRect);

		_pCreateAnnot->Color = _AnnotSetting->GetAnnotColor();
		_pCreateAnnot->Opacity = _AnnotSetting->GetAnnotOpacity();
		_pCreateAnnot->Page_HightLightRect = deviceRc;
		_pCreateAnnot->BorderWidth = _AnnotSetting->GetAnnotDrawBorderWidth();
		_pCreateAnnot->Name = uiMgr->getAnnotMgr()->GetAnnotSetting()->GetNoteAuthor();
		_pCreateAnnot->Title = _AnnotSetting->GetNoteAuthor();

		_pCreateAnnot->CreationDateTime = (ref new Windows::Globalization::Calendar())->GetDateTime();

		auto _page = View->GetDoc()->GetPage(m_PageIndex);


		uiMgr->getAnnotMgr()->OnCreateNewAnnot(_page, _pCreateAnnot);
		uiMgr->SetAnnotCanvasVisible(false);

		auto pUndoItem = ref new CreateCircleAnnotUndoAction(View, _pCreateAnnot);
		UndoFactory::GetUndoInstance()->AddUndoItem(uiMgr, pUndoItem);

		foxit::common::fxcrt::RectF^ minRect = FoxitRectConverter::ConvertToFoxitRectF(_pCreateAnnot->Page_HightLightRect);
		String^ rectStr = "[" + minRect->left.ToString() + " " + minRect->top.ToString() + " " + minRect->right.ToString() + " " + minRect->bottom.ToString()+"]";
		uiMgr->CollectAnnotOperation("Oval","Add", m_PageIndex.ToString(), rectStr);


		m_AreaRect = Windows::Foundation::Rect::Empty;

		if (!_AnnotSetting->GetCircleIsContinueOpera())
		{
			uiMgr->CreateTypeOfAnnot = AnnotCreateType::None;
			uiMgr->ExitAnnotPropertyMode();
		}
	}	
}

void CircleAnnotEventPeer::OnTapped(Platform::Object^ sender, Windows::UI::Xaml::Input::TappedRoutedEventArgs^ e)
{
	UIExtensionsManager^ uiMgr = (UIExtensionsManager^)View->ExtensionsManager;
	auto _SelectedAnnot = uiMgr->getAnnotMgr()->GetSelectedAnnot();
	if (_SelectedAnnot)
	{
		CircleAnnot^ annot = dynamic_cast<CircleAnnot^>(_SelectedAnnot);
		uiMgr->CreateEllipseAnnotToCanvas(annot->PageIndex, annot->MinRect, annot->Opacity, annot->BorderWidth, annot->Color);

		Windows::Foundation::Rect rect = annot->MinRect;
		rect.X = rect.X - 5;
		rect.Y = rect.Y - 5;
		rect.Width = rect.Width + 5 * 2;
		rect.Height = rect.Height + 5 * 2;
		auto deviceRect = View->ConvertPageRectToDeviceRect(uiMgr->getAnnotMgr()->GetSelectedAnnot()->PageIndex, rect);
		Point pt;

		pt = Point(deviceRect.X + deviceRect.Width / 2, deviceRect.Y);


		if (View->GetDoc() && !FSPDFDocUtil::GetIsDocumentCanModifyAnnotation(View->GetDoc()) && View->GetDoc()->GetPasswordType() != foxit::pdf::PasswordType::e_PwdOwner)
		{
			if (_SelectedAnnot->Contents->Length() > 0)
			{
				uiMgr->ShowTextmarkContextmenu(AnnotContextMenuType::Note, pt);
			}
		}
		else
		{
			uiMgr->ShowTextmarkContextmenu(AnnotContextMenuType::Edit, pt);
		}
	}
}

void CircleAnnotEventPeer::OnRightTapped(Platform::Object^ sender, Windows::UI::Xaml::Input::RightTappedRoutedEventArgs^ e)
{

}

void CircleAnnotEventPeer::OnManipulationStarted(Platform::Object^ sender, Windows::UI::Xaml::Input::ManipulationStartedRoutedEventArgs^ e)
{

}

void CircleAnnotEventPeer::OnManipulationDelta(Platform::Object^ sender, Windows::UI::Xaml::Input::ManipulationDeltaRoutedEventArgs^ e)
{

}

void CircleAnnotEventPeer::OnManipulationCompleted(Platform::Object^ sender, Windows::UI::Xaml::Input::ManipulationCompletedRoutedEventArgs^ e)
{

}

void CircleAnnotEventPeer::OnCreateAnnotAfterSelectText()
{
	if (!m_isPressed)
		return;
	m_pressStartPoint = Point(-1, -1);
	m_isPressed = false;

	UIExtensionsManager^ uiMgr = (UIExtensionsManager^)View->ExtensionsManager;
	auto _AnnotSetting = uiMgr->getAnnotMgr()->GetAnnotSetting();
	if (m_AreaRect.Width > 0 && m_AreaRect.Height > 0)
	{
		auto _Annot = foxit::rdk::uiextensions::AnnotCreaterFactory::GetInstance()->CreateNewAnnot(AnnotationType::Circle);
		auto _pCreateAnnot = dynamic_cast<CircleAnnot^>(_Annot);

		auto deviceRc = View->ConvertDeviceRectToPageRect(m_PageIndex, m_AreaRect);

		_pCreateAnnot->Color = _AnnotSetting->GetAnnotColor();
		_pCreateAnnot->Opacity = _AnnotSetting->GetAnnotOpacity();
		_pCreateAnnot->Page_HightLightRect = deviceRc;
		_pCreateAnnot->BorderWidth = _AnnotSetting->GetAnnotDrawBorderWidth();
		_pCreateAnnot->Name = uiMgr->getAnnotMgr()->GetAnnotSetting()->GetNoteAuthor();
		_pCreateAnnot->Title = _AnnotSetting->GetNoteAuthor();

		_pCreateAnnot->CreationDateTime = (ref new Windows::Globalization::Calendar())->GetDateTime();

		auto _page = View->GetDoc()->GetPage(m_PageIndex);

		uiMgr->getAnnotMgr()->OnCreateNewAnnot(_page, _pCreateAnnot);
		uiMgr->SetAnnotCanvasVisible(false);

		auto pUndoItem = ref new CreateCircleAnnotUndoAction(View, _pCreateAnnot);
		UndoFactory::GetUndoInstance()->AddUndoItem(uiMgr, pUndoItem);

		foxit::common::fxcrt::RectF^ minRect = FoxitRectConverter::ConvertToFoxitRectF(_pCreateAnnot->Page_HightLightRect);
		String^ rectStr = "[" + minRect->left.ToString() + " " + minRect->top.ToString() + " " + minRect->right.ToString() + " " + minRect->bottom.ToString() + "]";
		uiMgr->CollectAnnotOperation("Oval", "Add", m_PageIndex.ToString(), rectStr);


		m_AreaRect = Windows::Foundation::Rect::Empty;
	}
}

void CircleAnnotEventPeer::OnManipulationInertiaStarting(Platform::Object^ sender, Windows::UI::Xaml::Input::ManipulationInertiaStartingRoutedEventArgs^ e)
{

}

