/**
 * Copyright (C) 2003-2018, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit Mobile PDF SDK to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */


#include "pch.h"
#include "EditAnnotUnDoAction.h"


using namespace foxit::rdk;
using namespace foxit::rdk::uiextensions;

using namespace Platform;
using namespace Windows::Foundation;

ModifyAnnotUndoAction::ModifyAnnotUndoAction(foxit::rdk::PDFViewCtrl^ view, IBaseAnnot^ annot)
{
	m_ViewWeakRef = view;
	m_AnnotWeakRef = annot;

	SetAnnotOldColor(annot->Color);
	SetAnnotOpacity(annot->Opacity);
	SetBorderWidth(annot->BorderWidth);

	SetAnnotMinRect(annot->MinRect);
	if (annot->AnnotType == AnnotationType::Text)
	{
		TextAnnot^ textAnnot = dynamic_cast<TextAnnot^>(annot);
		SetTextAnnotIconName(textAnnot->IconType);
	}

	if (annot->AnnotType == AnnotationType::FreeText)
	{
		FreeTextAnnot^ freetextAnnot = dynamic_cast<FreeTextAnnot^>(annot);
		SetFreeTextFontName(freetextAnnot->FontName);
		SetFreeTextFontSize((int)freetextAnnot->FontSize);
		SetFreeTextMatrix(freetextAnnot->TextMatrix);
	}
	contents = annot->Contents;
}

foxit::rdk::PDFViewCtrl^ ModifyAnnotUndoAction::View::get()
{
	return m_ViewWeakRef.Resolve<foxit::rdk::PDFViewCtrl>();
}

IBaseAnnot^ ModifyAnnotUndoAction::AnnotHandle::get()
{
	return m_AnnotWeakRef.Resolve<IBaseAnnot>();
}

void ModifyAnnotUndoAction::Undo()
{
	if (AnnotHandle)
	{
		if (AnnotHandle->Opacity != annotOpacity)
		{
			double opactiy = AnnotHandle->Opacity;
			AnnotHandle->Opacity = annotOpacity;
			annotOpacity = opactiy;
		}


		if (AnnotHandle->AnnotType == AnnotationType::FreeText)
		{
			FreeTextAnnot^ freetextAnnot = dynamic_cast<FreeTextAnnot^>(AnnotHandle);
			if (freetextAnnot)
			{
				if (!(freetextAnnot->TextColor.A == annotcolor.A && freetextAnnot->TextColor.R == annotcolor.R
					&& freetextAnnot->TextColor.G == annotcolor.G && freetextAnnot->TextColor.B == annotcolor.B))
				{
					Windows::UI::Color color = freetextAnnot->TextColor;
					freetextAnnot->TextColor = annotcolor;
					annotcolor = color;
				}
			}
		}
		else
		{
			if (!(AnnotHandle->Color.A == annotcolor.A && AnnotHandle->Color.R == annotcolor.R && AnnotHandle->Color.G == annotcolor.G && AnnotHandle->Color.B == annotcolor.B))
			{
				Windows::UI::Color color = AnnotHandle->Color;
				AnnotHandle->Color = annotcolor;
				annotcolor = color;
			}
		}
		

		if (AnnotHandle->AnnotType == AnnotationType::Circle || AnnotHandle->AnnotType == AnnotationType::Square
			|| AnnotHandle->AnnotType == AnnotationType::Line || AnnotHandle->AnnotType == AnnotationType::Ink
			|| AnnotHandle->AnnotType == AnnotationType::Stamp)
		{
			if (AnnotHandle->Contents != contents)
			{
				Platform::String^ contentValue = AnnotHandle->Contents;
				AnnotHandle->Contents = contents;
				contents = contentValue;				
			}

			if (AnnotHandle->BorderWidth != borderwidth)
			{
				double borderwidthvalue = AnnotHandle->BorderWidth;
				AnnotHandle->BorderWidth = borderwidth;
				borderwidth = borderwidthvalue;
			}

			if (AnnotHandle->MinRect != _MinRect)
			{
				Windows::Foundation::Rect rect = AnnotHandle->MinRect;
				AnnotHandle->MinRect = _MinRect;

				if (AnnotHandle->AnnotType == AnnotationType::Ink)
				{
					auto annot = dynamic_cast<PencilAnnot^>(AnnotHandle);
					if (annot)
						annot->PageMinRect = _MinRect;
				}
				else if (AnnotHandle->AnnotType == AnnotationType::Square)
				{
					auto annot = dynamic_cast<SquareAnnot^>(AnnotHandle);
					if (annot)
						annot->Page_HightLightRect = _MinRect;
				}
				else if (AnnotHandle->AnnotType == AnnotationType::Circle)
				{
					auto annot = dynamic_cast<CircleAnnot^>(AnnotHandle);
					if (annot)
						annot->Page_HightLightRect = _MinRect;
				}
				else if (AnnotHandle->AnnotType == AnnotationType::Line)
				{
					auto annot = dynamic_cast<LineAnnot^>(AnnotHandle);
					if (annot)
					{
						annot->PageMinRect = _MinRect;
					}
				}
				else if (AnnotHandle->AnnotType == AnnotationType::Stamp)
				{
					auto annot = dynamic_cast<StampAnnot^>(AnnotHandle);
					if (annot)
						annot->Page_HightLightRect = _MinRect;
				}

				_MinRect = rect;
			}
		}
		else if (AnnotHandle->AnnotType == AnnotationType::Text)
		{
			if (AnnotHandle->Contents != contents)
			{
				Platform::String^ contentValue = AnnotHandle->Contents;
				AnnotHandle->Contents = contents;
				contents = contentValue;
			}

			if (AnnotHandle->MinRect != _MinRect)
			{
				Windows::Foundation::Rect rect = AnnotHandle->MinRect;
				AnnotHandle->MinRect = _MinRect;
				_MinRect = rect;
			}

			TextAnnot^ textAnnot = dynamic_cast<TextAnnot^>(AnnotHandle);
			if (textAnnot)
			{
				foxit::rdk::uiextensions::AnnotIconNameTextType type = textAnnot->IconType;
				if (textAnnot->IconType != _IconType)
				{
					textAnnot->IconType = _IconType;
					_IconType = type;
					View->Redraw(textAnnot->PageIndex);
				}
			}
		}
		else if (AnnotHandle->AnnotType == AnnotationType::FreeText)
		{
			auto annot = dynamic_cast<FreeTextAnnot^>(AnnotHandle);
			if (annot)
			{
				if (annot->Page_HightLightRect != _MinRect)
				{
					Windows::Foundation::Rect rect = AnnotHandle->MinRect;
					AnnotHandle->MinRect = _MinRect;

					annot->Page_HightLightRect = _MinRect;

					_MinRect = rect;
				}

				if (annot->FontSize != _fontSize)
				{
					int fontsize = (int)annot->FontSize;
					annot->FontSize = _fontSize;
					_fontSize = fontsize;
				}

				if (annot->FontName != _fontName)
				{
					String^ fontname = annot->FontName;
					annot->FontName = _fontName;
					_fontName = fontname;
				}

				if (AnnotHandle->Contents != contents)
				{
					Platform::String^ contentValue = AnnotHandle->Contents;
					AnnotHandle->Contents = contents;
					contents = contentValue;
					if (AnnotHandle->AnnotType == AnnotationType::FreeText)
					{
						FreeTextAnnot^ freetextAnnot = dynamic_cast<FreeTextAnnot^>(AnnotHandle);
						freetextAnnot->ResetAppearence();
					}
				}
				
				View->Redraw(annot->PageIndex);
			}
		}
		else
		{
			if (AnnotHandle->Contents != contents)
			{
				Platform::String^ contentValue = AnnotHandle->Contents;
				AnnotHandle->Contents = contents;
				contents = contentValue;
			}
		}
	}
}

void ModifyAnnotUndoAction::Redo()
{
	if (AnnotHandle)
	{
		if (AnnotHandle->Opacity != annotOpacity)
		{
			double opactiy = AnnotHandle->Opacity;
			AnnotHandle->Opacity = annotOpacity;
			annotOpacity = opactiy;
		}
		if (AnnotHandle->AnnotType == AnnotationType::FreeText)
		{
			FreeTextAnnot^ freetextAnnot = dynamic_cast<FreeTextAnnot^>(AnnotHandle);
			if (freetextAnnot)
			{
				if (!(freetextAnnot->TextColor.A == annotcolor.A && freetextAnnot->TextColor.R == annotcolor.R
					&& freetextAnnot->TextColor.G == annotcolor.G && freetextAnnot->TextColor.B == annotcolor.B))
				{
					Windows::UI::Color color = freetextAnnot->TextColor;
					freetextAnnot->TextColor = annotcolor;
					annotcolor = color;
				}
			}
		}
		else
		{
			if (!(AnnotHandle->Color.A == annotcolor.A && AnnotHandle->Color.R == annotcolor.R && AnnotHandle->Color.G == annotcolor.G && AnnotHandle->Color.B == annotcolor.B))
			{
				Windows::UI::Color color = AnnotHandle->Color;
				AnnotHandle->Color = annotcolor;
				annotcolor = color;
			}
		}

		

		if (AnnotHandle->AnnotType == AnnotationType::Circle || AnnotHandle->AnnotType == AnnotationType::Square
			|| AnnotHandle->AnnotType == AnnotationType::Line || AnnotHandle->AnnotType == AnnotationType::Ink
			|| AnnotHandle->AnnotType == AnnotationType::Stamp)
		{
			if (AnnotHandle->Contents != contents)
			{
				Platform::String^ contentValue = AnnotHandle->Contents;
				AnnotHandle->Contents = contents;
				contents = contentValue;
			}

			if (AnnotHandle->BorderWidth != borderwidth)
			{
				double borderwidthvalue = AnnotHandle->BorderWidth;
				AnnotHandle->BorderWidth = borderwidth;
				borderwidth = borderwidthvalue;
			}

			if (AnnotHandle->MinRect != _MinRect)
			{
				Windows::Foundation::Rect rect = AnnotHandle->MinRect;
				AnnotHandle->MinRect = _MinRect;


				if (AnnotHandle->AnnotType == AnnotationType::Ink)
				{
					auto annot = dynamic_cast<PencilAnnot^>(AnnotHandle);
					if (annot)
						annot->PageMinRect = _MinRect;
				}
				else if (AnnotHandle->AnnotType == AnnotationType::Square)
				{
					auto annot = dynamic_cast<SquareAnnot^>(AnnotHandle);
					if (annot)
						annot->Page_HightLightRect = _MinRect;
				}
				else if (AnnotHandle->AnnotType == AnnotationType::Circle)
				{
					auto annot = dynamic_cast<CircleAnnot^>(AnnotHandle);
					if (annot)
						annot->Page_HightLightRect = _MinRect;
				}
				else if (AnnotHandle->AnnotType == AnnotationType::Line)
				{
					auto annot = dynamic_cast<LineAnnot^>(AnnotHandle);
					if (annot)
					{
						annot->PageMinRect = _MinRect;
					}
				}
				else if (AnnotHandle->AnnotType == AnnotationType::Stamp)
				{
					auto annot = dynamic_cast<StampAnnot^>(AnnotHandle);
					if (annot)
						annot->Page_HightLightRect = _MinRect;
				}
				_MinRect = rect;
			}
		}
		else if (AnnotHandle->AnnotType == AnnotationType::Text)
		{
			if (AnnotHandle->Contents != contents)
			{
				Platform::String^ contentValue = AnnotHandle->Contents;
				AnnotHandle->Contents = contents;
				contents = contentValue;
			}

			if (AnnotHandle->MinRect != _MinRect)
			{
				Windows::Foundation::Rect rect = AnnotHandle->MinRect;
				AnnotHandle->MinRect = _MinRect;
				_MinRect = rect;
			}

			TextAnnot^ textAnnot = dynamic_cast<TextAnnot^>(AnnotHandle);
			if (textAnnot)
			{
				foxit::rdk::uiextensions::AnnotIconNameTextType type = textAnnot->IconType;
				if (textAnnot->IconType != _IconType)
				{
					textAnnot->IconType = _IconType;
					_IconType = type;
					View->Redraw(textAnnot->PageIndex);
				}
			}
		}
		else if (AnnotHandle->AnnotType == AnnotationType::FreeText)
		{
			auto annot = dynamic_cast<FreeTextAnnot^>(AnnotHandle);
			if (annot)
			{
				if (AnnotHandle->MinRect != _MinRect)
				{
					Windows::Foundation::Rect rect = AnnotHandle->MinRect;
					AnnotHandle->MinRect = _MinRect;

					annot->Page_HightLightRect = _MinRect;
					_MinRect = rect;
				}

				if (annot->FontSize != _fontSize)
				{
					int fontsize = (int)annot->FontSize;
					annot->FontSize = _fontSize;
					_fontSize = fontsize;
				}

				if (annot->FontName != _fontName)
				{
					String^ fontname = annot->FontName;
					annot->FontName = _fontName;
					_fontName = fontname;
				}

				if (AnnotHandle->Contents != contents)
				{
					Platform::String^ contentValue = AnnotHandle->Contents;
					AnnotHandle->Contents = contents;
					contents = contentValue;
					if (AnnotHandle->AnnotType == AnnotationType::FreeText)
					{
						FreeTextAnnot^ freetextAnnot = dynamic_cast<FreeTextAnnot^>(AnnotHandle);
						freetextAnnot->ResetAppearence();
					}
				}

				View->Redraw(annot->PageIndex);
			}
		}
		else
		{
			if (AnnotHandle->Contents != contents)
			{
				Platform::String^ contentValue = AnnotHandle->Contents;
				AnnotHandle->Contents = contents;
				contents = contentValue;
			}
		}
	}
}

Platform::String^ ModifyAnnotUndoAction::GetDescr()
{
	return ref new Platform::String(L"");
}

void ModifyAnnotUndoAction::ReleaseItem()
{
	m_ViewWeakRef = nullptr;
	m_AnnotWeakRef = nullptr;
}

void ModifyAnnotUndoAction::SetAnnotOldColor(Windows::UI::Color color)
{
	annotcolor = color;
	if (AnnotHandle->AnnotType == AnnotationType::FreeText)
	{
		FreeTextAnnot^ freetextAnnot = dynamic_cast<FreeTextAnnot^>(AnnotHandle);
		if (freetextAnnot)
		{
			annotcolor = freetextAnnot->TextColor;
			Platform::String^ colorRGBValue = Utility::ConvertToColorString(annotcolor);
			int i = 0;
		}
	}
}

void ModifyAnnotUndoAction::SetAnnotOpacity(double opacity)
{
	annotOpacity = opacity;
}

void ModifyAnnotUndoAction::SetBorderWidth(double borderWidth)
{
	if (AnnotHandle->AnnotType == AnnotationType::Circle || AnnotHandle->AnnotType == AnnotationType::Square
		|| AnnotHandle->AnnotType == AnnotationType::Line || AnnotHandle->AnnotType == AnnotationType::Ink)
	{
		borderwidth = borderWidth;
	}
}

void ModifyAnnotUndoAction::SetAnnotMinRect(Windows::Foundation::Rect minRect)
{
	if (AnnotHandle->AnnotType == AnnotationType::Circle || AnnotHandle->AnnotType == AnnotationType::Square
		|| AnnotHandle->AnnotType == AnnotationType::Line || AnnotHandle->AnnotType == AnnotationType::Ink
		|| AnnotHandle->AnnotType == AnnotationType::Text || AnnotHandle->AnnotType == AnnotationType::Stamp
		|| AnnotHandle->AnnotType == AnnotationType::FreeText)
	{
		_MinRect = minRect;
	}
}

void ModifyAnnotUndoAction::SetTextAnnotIconName(foxit::rdk::uiextensions::AnnotIconNameTextType type)
{
	if (AnnotHandle->AnnotType == AnnotationType::Text)
	{
		_IconType = type;
	}
}

void ModifyAnnotUndoAction::SetFreeTextFontSize(int fontSize)
{
	if (AnnotHandle->AnnotType == AnnotationType::FreeText)
	{
		_fontSize = fontSize;
	}
}

void ModifyAnnotUndoAction::SetFreeTextFontName(Platform::String^ fontName)
{
	if (AnnotHandle->AnnotType == AnnotationType::FreeText)
	{
		_fontName = fontName;
	}
}

void ModifyAnnotUndoAction::SetFreeTextMatrix(Windows::Foundation::Numerics::float3x2 textMatrix)
{
	if (AnnotHandle->AnnotType == AnnotationType::FreeText)
	{
		_textMatrix = textMatrix;
	}
}


DeleteAnnotUndoAction::DeleteAnnotUndoAction(foxit::rdk::PDFViewCtrl^ view, IBaseAnnot^ annot)
{
	m_ViewWeakRef = view;
	m_AnnotWeakRef = annot;
	m_RedoAnnot = annot;
	if (m_RedoAnnot->AnnotType == foxit::rdk::uiextensions::AnnotationType::Text)
	{
		TextAnnot^ textannot = dynamic_cast<TextAnnot^>(m_RedoAnnot);
		textAnnotPosition = Windows::Foundation::Point(textannot->MinRect.X, textannot->MinRect.Y + 20);
	}
}

foxit::rdk::PDFViewCtrl^ DeleteAnnotUndoAction::View::get()
{
	return m_ViewWeakRef.Resolve < foxit::rdk::PDFViewCtrl > ();
}

IBaseAnnot^ DeleteAnnotUndoAction::AnnotHandle::get()
{
	return m_AnnotWeakRef.Resolve<IBaseAnnot>();
}

void DeleteAnnotUndoAction::Undo()
{
	if (View)
	{
		UIExtensionsManager^ uiMgr = (UIExtensionsManager^)View->ExtensionsManager;
		auto _page = View->GetDoc()->GetPage(m_RedoAnnot->PageIndex);

		View->CancelRenderTask();
		if (m_RedoAnnot->AnnotType == foxit::rdk::uiextensions::AnnotationType::Text)
		{
			TextAnnot^ textannot = dynamic_cast<TextAnnot^>(m_RedoAnnot);
			textannot->PosPoint = textAnnotPosition;
			uiMgr->getAnnotMgr()->OnCreateNewAnnot(_page, textannot);
		}
		else
		{
			uiMgr->getAnnotMgr()->OnCreateNewAnnot(_page, m_RedoAnnot);
		}
	}
}

void DeleteAnnotUndoAction::Redo()
{
	if (View && m_RedoAnnot)
	{
		View->CancelRenderTask();
		IBaseAnnot^ deleteAnnot = m_RedoAnnot;
		deleteAnnot->OnDelete();
	}
}

Platform::String^ DeleteAnnotUndoAction::GetDescr()
{
	return ref new Platform::String(L"");
}

void DeleteAnnotUndoAction::ReleaseItem()
{
	m_ViewWeakRef = nullptr;
	m_AnnotWeakRef = nullptr;
	m_RedoAnnot = nullptr;
}


CreateReplaceTextAnnotUndoAction::CreateReplaceTextAnnotUndoAction(foxit::rdk::PDFViewCtrl^ view, Windows::Foundation::Collections::IVector<IBaseAnnot^>^ annots)
{
	m_ViewWeakRef = view;
	int size = annots->Size;
	if (size > 0)
	{
		for (int i = 0; i < size; i++)
		{
			IBaseAnnot^ annot = annots->GetAt(i);
			if (annot->AnnotType == AnnotationType::Caret)
			{
				m_CaretAnnotWeakRef = annot;
			}
			else
			{
				m_StrikeoutAnnotWeakRef = annot;
			}
		}
	}
	m_RedoCaretAnnot = nullptr;
	m_RedoStrikeoutAnnot = nullptr;
}

foxit::rdk::PDFViewCtrl^ CreateReplaceTextAnnotUndoAction::View::get()
{
	return m_ViewWeakRef.Resolve<foxit::rdk::PDFViewCtrl>();
}

StrikeOutAnnot^ CreateReplaceTextAnnotUndoAction::StrikeoutAnnotHandle::get()
{
	return m_StrikeoutAnnotWeakRef.Resolve<StrikeOutAnnot>();
}

CaretAnnot^ CreateReplaceTextAnnotUndoAction::CaretAnnotHandle::get()
{
	return m_CaretAnnotWeakRef.Resolve<CaretAnnot>();
}

void CreateReplaceTextAnnotUndoAction::Undo()
{
	if (View && StrikeoutAnnotHandle && CaretAnnotHandle)
	{
		m_RedoCaretAnnot = CaretAnnotHandle;
		m_RedoStrikeoutAnnot = StrikeoutAnnotHandle;
		View->CancelRenderTask();
		CaretAnnotHandle->OnDelete();
		StrikeoutAnnotHandle->OnDelete();
	}
}

void CreateReplaceTextAnnotUndoAction::Redo()
{
	if (View && m_RedoCaretAnnot && m_RedoStrikeoutAnnot)
	{
		auto _page = View->GetDoc()->GetPage(m_RedoCaretAnnot->PageIndex);

		View->CancelRenderTask();
		Windows::Foundation::Collections::IVector<IBaseAnnot^>^ newannots = ref new Platform::Collections::Vector<IBaseAnnot^>();
		newannots->Append(m_RedoStrikeoutAnnot);
		newannots->Append(m_RedoCaretAnnot);
		UIExtensionsManager^ uiMgr = (UIExtensionsManager^)View->ExtensionsManager;
		uiMgr->getAnnotMgr()->OnCreateNewAnnotGroup(_page, newannots);
	}
}

Platform::String^ CreateReplaceTextAnnotUndoAction::GetDescr()
{
	return ref new Platform::String(L"");
}

void CreateReplaceTextAnnotUndoAction::ReleaseItem()
{
	m_ViewWeakRef = nullptr;
	m_StrikeoutAnnotWeakRef = nullptr;
	m_CaretAnnotWeakRef = nullptr;
	m_RedoStrikeoutAnnot = nullptr;
	m_RedoCaretAnnot = nullptr;
}


DeleteReplaceAnnotUndoAction::DeleteReplaceAnnotUndoAction(foxit::rdk::PDFViewCtrl^ view, foxit::rdk::uiextensions::IBaseAnnot^ caretannot, foxit::rdk::uiextensions::IBaseAnnot^ strikeoutannot)
{
	m_ViewWeakRef = view;
	m_CaretAnnotWeakRef = caretannot;
	m_StrikeOutAnnotWeakRef = strikeoutannot;
	m_RedoStrikeoutAnnot = strikeoutannot;
	m_RedoCaretAnnot = caretannot;	
}

foxit::rdk::PDFViewCtrl^ DeleteReplaceAnnotUndoAction::View::get()
{
	return m_ViewWeakRef.Resolve<foxit::rdk::PDFViewCtrl>();
}

IBaseAnnot^ DeleteReplaceAnnotUndoAction::CaretAnnotHandle::get()
{
	return m_CaretAnnotWeakRef.Resolve<IBaseAnnot>();
}

IBaseAnnot^ DeleteReplaceAnnotUndoAction::StrikeoutAnnotHandle::get()
{
	return m_StrikeOutAnnotWeakRef.Resolve<IBaseAnnot>();
}

void DeleteReplaceAnnotUndoAction::Undo()
{
	if (View && m_RedoCaretAnnot && m_RedoStrikeoutAnnot)
	{
		auto _page = View->GetDoc()->GetPage(m_RedoCaretAnnot->PageIndex);
		View->CancelRenderTask();

		Windows::Foundation::Collections::IVector<IBaseAnnot^>^ newannots = ref new Platform::Collections::Vector<IBaseAnnot^>();
		newannots->Append(m_RedoStrikeoutAnnot);
		newannots->Append(m_RedoCaretAnnot);
		UIExtensionsManager^ uiMgr = (UIExtensionsManager^)View->ExtensionsManager;
		uiMgr->getAnnotMgr()->OnCreateNewAnnotGroup(_page, newannots);
	}
}

void DeleteReplaceAnnotUndoAction::Redo()
{
	if (View &&  m_RedoCaretAnnot && m_RedoStrikeoutAnnot)
	{
		View->CancelRenderTask();
		m_RedoStrikeoutAnnot->OnDelete();
		m_RedoCaretAnnot->OnDelete();
	}
}

Platform::String^ DeleteReplaceAnnotUndoAction::GetDescr()
{
	return ref new Platform::String(L"");
}

void DeleteReplaceAnnotUndoAction::ReleaseItem()
{
	m_ViewWeakRef = nullptr;
	m_StrikeOutAnnotWeakRef = nullptr;
	m_CaretAnnotWeakRef = nullptr;
	m_RedoStrikeoutAnnot = nullptr;
	m_RedoCaretAnnot = nullptr;
}


ModifyReplaceAnnotUndoAction::ModifyReplaceAnnotUndoAction(foxit::rdk::PDFViewCtrl^ view, Windows::Foundation::Collections::IVector<foxit::rdk::uiextensions::IBaseAnnot^>^ annots)
{
	m_ViewWeakRef = view;
	int size = annots->Size;
	if (size > 0)
	{
		for (int i = 0; i < size; i++)
		{
			IBaseAnnot^ annot = annots->GetAt(i);
			if (annot->AnnotType == AnnotationType::Caret)
			{
				m_CaretAnnotWeakRef = annot;
				contents = annot->Contents;
			}
			else
			{
				m_StrikeoutAnnotWeakRef = annot;
			}
			SetAnnotOldColor(annot->Color);
			SetAnnotOpacity(annot->Opacity);
		}
	}
}

foxit::rdk::PDFViewCtrl^ ModifyReplaceAnnotUndoAction::View::get()
{
	return m_ViewWeakRef.Resolve<foxit::rdk::PDFViewCtrl>();
}

IBaseAnnot^ ModifyReplaceAnnotUndoAction::CaretAnnotHandle::get()
{
	return m_CaretAnnotWeakRef.Resolve<IBaseAnnot>();
}

IBaseAnnot^ ModifyReplaceAnnotUndoAction::StrikeoutAnnotHandle::get()
{
	return m_StrikeoutAnnotWeakRef.Resolve<IBaseAnnot>();
}

void ModifyReplaceAnnotUndoAction::Undo()
{
	if (CaretAnnotHandle && StrikeoutAnnotHandle)
	{
		if (CaretAnnotHandle->Opacity != annotOpacity)
		{
			double opactiy = CaretAnnotHandle->Opacity;
			CaretAnnotHandle->Opacity = annotOpacity;
			StrikeoutAnnotHandle->Opacity = annotOpacity;
			annotOpacity = opactiy;
		}		

		if (!(CaretAnnotHandle->Color.A == annotcolor.A && CaretAnnotHandle->Color.R == annotcolor.R 
			&& CaretAnnotHandle->Color.G == annotcolor.G && CaretAnnotHandle->Color.B == annotcolor.B))
		{
			Windows::UI::Color color = CaretAnnotHandle->Color;		
			CaretAnnotHandle->Color = annotcolor;
			StrikeoutAnnotHandle->Color = annotcolor;
			annotcolor = color;
		}

		if (CaretAnnotHandle->Contents != contents)
		{
			Platform::String^ contentValue = CaretAnnotHandle->Contents;
			CaretAnnotHandle->Contents = contents;
			contents = contentValue;
		}
	}
}

void ModifyReplaceAnnotUndoAction::Redo()
{
	if (CaretAnnotHandle && StrikeoutAnnotHandle)
	{
		if (CaretAnnotHandle->Opacity != annotOpacity)
		{
			double opactiy = CaretAnnotHandle->Opacity;
			CaretAnnotHandle->Opacity = annotOpacity;
			StrikeoutAnnotHandle->Opacity = annotOpacity;
			annotOpacity = opactiy;
		}

		if (!(CaretAnnotHandle->Color.A == annotcolor.A && CaretAnnotHandle->Color.R == annotcolor.R
			&& CaretAnnotHandle->Color.G == annotcolor.G && CaretAnnotHandle->Color.B == annotcolor.B))
		{
			Windows::UI::Color color = CaretAnnotHandle->Color;
			CaretAnnotHandle->Color = annotcolor;
			StrikeoutAnnotHandle->Color = annotcolor;
			annotcolor = color;
		}

		if (CaretAnnotHandle->Contents != contents)
		{
			Platform::String^ contentValue = CaretAnnotHandle->Contents;
			CaretAnnotHandle->Contents = contents;
			contents = contentValue;
		}
	}
}

Platform::String^ ModifyReplaceAnnotUndoAction::GetDescr()
{
	return ref new Platform::String(L"");
}

void ModifyReplaceAnnotUndoAction::ReleaseItem()
{
	m_ViewWeakRef = nullptr;
	m_StrikeoutAnnotWeakRef = nullptr;
	m_CaretAnnotWeakRef = nullptr;
}

void ModifyReplaceAnnotUndoAction::SetAnnotOldColor(Windows::UI::Color color)
{
	annotcolor = color;
}

void ModifyReplaceAnnotUndoAction::SetAnnotOpacity(double opacity)
{
	annotOpacity = opacity;
}

