/**
 * Copyright (C) 2003-2018, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit Mobile PDF SDK to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */


#include "../Undo/IUndo.h"


namespace foxit
{
	namespace rdk
	{
		namespace uiextensions
		{
			[Windows::Foundation::Metadata::WebHostHidden]
			public ref class ModifyAnnotUndoAction sealed : IUndoItem
			{
			public:
				ModifyAnnotUndoAction(foxit::rdk::PDFViewCtrl^ view, IBaseAnnot^ annot);

				virtual void Undo();
				virtual void Redo();
				virtual Platform::String^ GetDescr();
				virtual void ReleaseItem();

			internal:
				property foxit::rdk::PDFViewCtrl^ View
				{
					foxit::rdk::PDFViewCtrl^ get();
				}

				property IBaseAnnot^ AnnotHandle
				{
					IBaseAnnot^ get();
				}

				void SetAnnotOldColor(Windows::UI::Color color);
				void SetAnnotOpacity(double opacity);
				void SetBorderWidth(double borderWidth);
				void SetAnnotMinRect(Windows::Foundation::Rect minRect);
				void SetTextAnnotIconName(AnnotIconNameTextType type);
				void SetFreeTextFontSize(int fontSize);
				void SetFreeTextFontName(Platform::String^ fontName);
				void SetFreeTextMatrix(Windows::Foundation::Numerics::float3x2 textMatrix);

			private:
				Platform::WeakReference m_ViewWeakRef;
				Platform::WeakReference m_AnnotWeakRef;

				Windows::UI::Color  annotcolor;
				double  annotOpacity;
				double  borderwidth;

				Windows::Foundation::Rect _MinRect;
				AnnotIconNameTextType _IconType;
				Platform::String^ contents;

				int _fontSize;
				Platform::String^ _fontName;
				Windows::Foundation::Numerics::float3x2 _textMatrix;
			};

			[Windows::Foundation::Metadata::WebHostHidden]
			public ref class DeleteAnnotUndoAction sealed : IUndoItem
			{
			public:
				DeleteAnnotUndoAction(foxit::rdk::PDFViewCtrl^ view, IBaseAnnot^ annot);

				virtual void Undo();
				virtual void Redo();
				virtual Platform::String^ GetDescr();
				virtual void ReleaseItem();

			internal:
				property foxit::rdk::PDFViewCtrl^ View
				{
					foxit::rdk::PDFViewCtrl^ get();
				}

				property IBaseAnnot^ AnnotHandle
				{
					IBaseAnnot^ get();
				}

			private:
				Platform::WeakReference m_ViewWeakRef;
				Platform::WeakReference m_AnnotWeakRef;
				IBaseAnnot^ m_RedoAnnot;
				Windows::Foundation::Point textAnnotPosition;
			};

			[Windows::Foundation::Metadata::WebHostHidden]
			public ref class CreateReplaceTextAnnotUndoAction sealed : IUndoItem
			{
			public:
				CreateReplaceTextAnnotUndoAction(foxit::rdk::PDFViewCtrl^ view, Windows::Foundation::Collections::IVector<IBaseAnnot^>^ annots);

				virtual void Undo();
				virtual void Redo();
				virtual Platform::String^ GetDescr();
				virtual void ReleaseItem();

			internal:
				property foxit::rdk::PDFViewCtrl^ View
				{
					foxit::rdk::PDFViewCtrl^ get();
				}

				property StrikeOutAnnot^ StrikeoutAnnotHandle
				{
					StrikeOutAnnot^ get();
				}

				property CaretAnnot^ CaretAnnotHandle
				{
					CaretAnnot^ get();
				}

			private:
				Platform::WeakReference m_ViewWeakRef;
				Platform::WeakReference m_CaretAnnotWeakRef;
				Platform::WeakReference m_StrikeoutAnnotWeakRef;
				Windows::Foundation::Point textAnnotPosition;
				BaseAnnot^ m_RedoStrikeoutAnnot;
				BaseAnnot^ m_RedoCaretAnnot;
			};
			[Windows::Foundation::Metadata::WebHostHidden]
			public ref class DeleteReplaceAnnotUndoAction sealed : IUndoItem
			{
			public:
				DeleteReplaceAnnotUndoAction(foxit::rdk::PDFViewCtrl^ view, IBaseAnnot^ caretannot, IBaseAnnot^ strikeoutannot);

				virtual void Undo();
				virtual void Redo();
				virtual Platform::String^ GetDescr();
				virtual void ReleaseItem();

			internal:
				property foxit::rdk::PDFViewCtrl^ View
				{
					foxit::rdk::PDFViewCtrl^ get();
				}

				property IBaseAnnot^ CaretAnnotHandle
				{
					IBaseAnnot^ get();
				}

				property IBaseAnnot^ StrikeoutAnnotHandle
				{
					IBaseAnnot^ get();
				}

			private:
				Platform::WeakReference m_ViewWeakRef;
				Platform::WeakReference m_CaretAnnotWeakRef;
				Platform::WeakReference m_StrikeOutAnnotWeakRef;
				IBaseAnnot^ m_RedoStrikeoutAnnot;
				IBaseAnnot^ m_RedoCaretAnnot;
			};
			[Windows::Foundation::Metadata::WebHostHidden]
			public ref class ModifyReplaceAnnotUndoAction sealed : IUndoItem
			{
			public:
				ModifyReplaceAnnotUndoAction(foxit::rdk::PDFViewCtrl^ view, Windows::Foundation::Collections::IVector<IBaseAnnot^>^ annots);

				virtual void Undo();
				virtual void Redo();
				virtual Platform::String^ GetDescr();
				virtual void ReleaseItem();

			internal:
				property foxit::rdk::PDFViewCtrl^ View
				{
					foxit::rdk::PDFViewCtrl^ get();
				}

				property IBaseAnnot^ CaretAnnotHandle
				{
					IBaseAnnot^ get();
				}

				property IBaseAnnot^ StrikeoutAnnotHandle
				{
					IBaseAnnot^ get();
				}

				void SetAnnotOldColor(Windows::UI::Color color);
				void SetAnnotOpacity(double opacity);

			private:
				Platform::WeakReference m_ViewWeakRef;
				Platform::WeakReference m_StrikeoutAnnotWeakRef;
				Platform::WeakReference m_CaretAnnotWeakRef;

				Windows::UI::Color  annotcolor;
				double  annotOpacity;
				Platform::String^ contents;
			};
		}
	}
}
