/**
 * Copyright (C) 2003-2018, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit Mobile PDF SDK to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */


#include "pch.h"
#include "EraserEventPeer.h"
#include "EditAnnotUnDoAction.h"

using namespace foxit::rdk;
using namespace foxit::rdk::uiextensions;

using namespace Platform;
using namespace Windows::Foundation;


namespace foxit
{
	namespace rdk
	{
		namespace uiextensions
		{
			public ref class InkWekRefObj sealed
			{
			internal:
				InkWekRefObj()
				{

				}

			internal:
				property Platform::WeakReference AnnotWeakRef {
					Platform::WeakReference get()
					{
						return 	_AnnotWeakRef;
					}
					void set(Platform::WeakReference value)
					{
						_AnnotWeakRef = value;
					}
				}

			private:
				Platform::WeakReference _AnnotWeakRef;
			};

			public ref class InkVectorPointObj sealed
			{
			internal:
				InkVectorPointObj()
				{
					_InkVectPoint = ref new Platform::Collections::Vector<Windows::Foundation::Collections::IVector<Windows::Foundation::Point>^>();
				}

			public:
				property Windows::Foundation::Collections::IVector<Windows::Foundation::Collections::IVector<Windows::Foundation::Point>^>^ InkVectPoint
				{
					Windows::Foundation::Collections::IVector<Windows::Foundation::Collections::IVector<Windows::Foundation::Point>^>^ get()
					{
						return _InkVectPoint;
					}
					void set(Windows::Foundation::Collections::IVector<Windows::Foundation::Collections::IVector<Windows::Foundation::Point>^>^ value)
					{
						_InkVectPoint = value;
					}
				}

			private:
				Windows::Foundation::Collections::IVector<Windows::Foundation::Collections::IVector<Windows::Foundation::Point>^>^	_InkVectPoint;
			};
			[Windows::Foundation::Metadata::WebHostHidden]
			public ref class EraserAnnotUndoAction sealed : IUndoItem
			{
			public:
				EraserAnnotUndoAction(PDFViewCtrl^ view, Windows::Foundation::Collections::IVector<IBaseAnnot^>^ annotListannot);

				virtual void Undo();
				virtual void Redo();
				virtual Platform::String^ GetDescr();
				virtual void ReleaseItem();

			public:
				void SaveBeforeEraserVectPoint(int index, Windows::Foundation::Collections::IVector<Windows::Foundation::Collections::IVector<Windows::Foundation::Point>^>^ vectPoint);
				void AppendAnnotToAnnotList(IBaseAnnot^ annot);

			internal:
				property PDFViewCtrl^ View
				{
					PDFViewCtrl^ get();
				}

				property PencilAnnot^ AnnotHandle
				{
					PencilAnnot^ get();
				}

				Windows::Foundation::Collections::IVector<Windows::Foundation::Collections::IVector<Windows::Foundation::Point>^>^ GetVectPoints(int nAnnotIndex);

			private:
				Platform::WeakReference m_ViewWeakRef;
				Platform::WeakReference m_AnnotWeakRef;
				Platform::Collections::Map<int, InkWekRefObj^>^ annotWeakRefList;

				Windows::Foundation::Collections::IVector<PencilAnnot^>^ pencilannotList;
				Platform::Collections::Map<int, InkVectorPointObj^>^ pageVectorMap;
			};
		}
	}
}

EraserAnnotUndoAction::EraserAnnotUndoAction(PDFViewCtrl^ view, Windows::Foundation::Collections::IVector<IBaseAnnot^>^ annotList)
{
	m_ViewWeakRef = view;
	pencilannotList = ref new Platform::Collections::Vector<PencilAnnot^>();
	annotWeakRefList = ref new Platform::Collections::Map<int, InkWekRefObj^>();
	for (unsigned int i = 0; i < annotList->Size; i++)
	{
		auto annot = dynamic_cast<PencilAnnot^>(annotList->GetAt(i));
		pencilannotList->Append(annot);

		Platform::WeakReference annotweakRef;
		annotweakRef = annot;
		auto nUniqueId = annot->GetHashCode();
		if (!annotWeakRefList->HasKey(nUniqueId))
		{
			InkWekRefObj^ weakObj = ref new InkWekRefObj();
			weakObj->AnnotWeakRef = annotweakRef;
			annotWeakRefList->Insert(nUniqueId, weakObj);
		}
	}

	pageVectorMap = ref new Platform::Collections::Map<int, InkVectorPointObj^>();
}

PDFViewCtrl^ EraserAnnotUndoAction::View::get()
{
	return m_ViewWeakRef.Resolve<PDFViewCtrl>();
}

PencilAnnot^ EraserAnnotUndoAction::AnnotHandle::get()
{
	return m_AnnotWeakRef.Resolve<PencilAnnot>();
}

void EraserAnnotUndoAction::Undo()
{
	for (unsigned int i = 0; i < pencilannotList->Size; i++)
	{
		PencilAnnot^ pencilannot = nullptr;
		auto nUniqueId = pencilannotList->GetAt(i)->GetHashCode();
		if (annotWeakRefList->HasKey(nUniqueId))
		{
			InkWekRefObj^ weakRef = dynamic_cast<InkWekRefObj^>(annotWeakRefList->Lookup(nUniqueId));
			pencilannot = weakRef->AnnotWeakRef.Resolve<PencilAnnot>();
		}
		if (pencilannot)
		{
			int keyValue = nUniqueId;

			Windows::Foundation::Collections::IVector<Windows::Foundation::Collections::IVector<Windows::Foundation::Point>^>^ inkVectPoints = pencilannot->Page_VectPoint;

			Windows::Foundation::Collections::IVector<Windows::Foundation::Collections::IVector<Windows::Foundation::Point>^>^ inkupdateVectPoints =
				ref new  Platform::Collections::Vector<Windows::Foundation::Collections::IVector<Windows::Foundation::Point>^>();

			if (pageVectorMap->HasKey(keyValue))
			{
				InkVectorPointObj^ inkVectorObj = dynamic_cast<InkVectorPointObj^>(pageVectorMap->Lookup(keyValue));
				inkupdateVectPoints = inkVectorObj->InkVectPoint;
			}


			pencilannot->UpdatePage_VectPoint = inkupdateVectPoints;
			if (pageVectorMap->HasKey(keyValue))
			{
				pageVectorMap->Remove(keyValue);
				InkVectorPointObj^ obj = ref new InkVectorPointObj();
				obj->InkVectPoint = inkVectPoints;

				pageVectorMap->Insert(keyValue, obj);
			}
		}
	}
	View->Redraw(pencilannotList->GetAt(0)->PageIndex);
}

void EraserAnnotUndoAction::Redo()
{
	for (unsigned int i = 0; i < pencilannotList->Size; i++)
	{
		PencilAnnot^ pencilannot = nullptr;
		auto nUniqueId = pencilannotList->GetAt(i)->GetHashCode();
		if (annotWeakRefList->HasKey(nUniqueId))
		{
			InkWekRefObj^ weakRef = dynamic_cast<InkWekRefObj^>(annotWeakRefList->Lookup(nUniqueId));
			pencilannot = weakRef->AnnotWeakRef.Resolve<PencilAnnot>();
		}
		if (pencilannot)
		{
			int keyValue = nUniqueId;

			Windows::Foundation::Collections::IVector<Windows::Foundation::Collections::IVector<Windows::Foundation::Point>^>^ inkVectPoints = pencilannot->Page_VectPoint;

			Windows::Foundation::Collections::IVector<Windows::Foundation::Collections::IVector<Windows::Foundation::Point>^>^ inkupdateVectPoints =
				ref new  Platform::Collections::Vector<Windows::Foundation::Collections::IVector<Windows::Foundation::Point>^>();

			if (pageVectorMap->HasKey(keyValue))
			{
				InkVectorPointObj^ inkVectorObj = dynamic_cast<InkVectorPointObj^>(pageVectorMap->Lookup(keyValue));
				inkupdateVectPoints = inkVectorObj->InkVectPoint;
			}


			pencilannot->UpdatePage_VectPoint = inkupdateVectPoints;
			if (pageVectorMap->HasKey(keyValue))
			{
				pageVectorMap->Remove(keyValue);
				InkVectorPointObj^ obj = ref new InkVectorPointObj();
				obj->InkVectPoint = inkVectPoints;

				pageVectorMap->Insert(keyValue, obj);
			}
		}
	}
	View->Redraw(pencilannotList->GetAt(0)->PageIndex);
}

Platform::String^ EraserAnnotUndoAction::GetDescr()
{
	return ref new Platform::String(L"");
}

void EraserAnnotUndoAction::ReleaseItem()
{
	m_ViewWeakRef = nullptr;
	m_AnnotWeakRef = nullptr;
	for (unsigned int i = 0; i < annotWeakRefList->Size; i++)
	{
		auto nUniqueId = pencilannotList->GetAt(i)->GetHashCode();
		InkWekRefObj^ wekreference = dynamic_cast<InkWekRefObj^>(annotWeakRefList->Lookup(nUniqueId));
		if (wekreference)
			wekreference->AnnotWeakRef = nullptr;
	}
}

void EraserAnnotUndoAction::SaveBeforeEraserVectPoint(int index, Windows::Foundation::Collections::IVector<Windows::Foundation::Collections::IVector<Windows::Foundation::Point>^>^ vectPoint)
{
	if (!pageVectorMap)
		return;

	if (!pageVectorMap->HasKey(index))
	{
		OutputDebugString((L"SaveBeforeEraserVectPoint: " + this->GetHashCode() + L", " + index + L"\n")->Data());
		InkVectorPointObj^ obj = ref new InkVectorPointObj();
		obj->InkVectPoint = vectPoint;
		pageVectorMap->Insert(index, obj);
	}
}

Windows::Foundation::Collections::IVector<Windows::Foundation::Collections::IVector<Windows::Foundation::Point>^>^ EraserAnnotUndoAction::GetVectPoints(int nAnnotIndex)
{
	return pageVectorMap->HasKey(nAnnotIndex) ? pageVectorMap->Lookup(nAnnotIndex)->InkVectPoint : nullptr;
}

void EraserAnnotUndoAction::AppendAnnotToAnnotList(IBaseAnnot^ annot)
{
	bool isExist = false;
	for (unsigned int i = 0; i < pencilannotList->Size; i++)
	{
		if (pencilannotList->GetAt(i)->AnnoIndex == annot->AnnoIndex)
			isExist = true;
	}
	if (!isExist)
	{
		auto annotAppend = dynamic_cast<PencilAnnot^>(annot);
		pencilannotList->Append(annotAppend);

		Platform::WeakReference annotweakRef;
		annotweakRef = annotAppend;
		auto nUniqueId = annot->GetHashCode();
		if (!annotWeakRefList->HasKey(nUniqueId))
		{
			InkWekRefObj^ weakObj = ref new InkWekRefObj();
			weakObj->AnnotWeakRef = annotweakRef;
			annotWeakRefList->Insert(nUniqueId, weakObj);
		}
	}
}


EraserAnnotEventPeer::EraserAnnotEventPeer(PDFViewCtrl^ view)
{
	m_PageViewWeakRef = view;
	m_pressStartPoint = Point(-1, -1);
	m_isPressed = false;
	polyLinePointsVector = ref new  Platform::Collections::Vector<Windows::UI::Xaml::Media::PointCollection^>();
	isDeleteAnnot = false;

	inkAnnotList = ref new Platform::Collections::Map<int, foxit::rdk::uiextensions::IBaseAnnot^>();
	inkDeleteAnnotList = ref new Platform::Collections::Map<int, foxit::rdk::uiextensions::IBaseAnnot^>();
	m_EraserIndex = -1;
}

PDFViewCtrl^ EraserAnnotEventPeer::View::get()
{
	return m_PageViewWeakRef.Resolve<PDFViewCtrl>();
}

void EraserAnnotEventPeer::OnPointerPressed(Platform::Object^ sender, Windows::UI::Xaml::Input::PointerRoutedEventArgs^ e)
{
	if (!View)
		return;

	if (m_isPressed)
		return;

	UIExtensionsManager^ uiMgr = (UIExtensionsManager^)View->ExtensionsManager;
	uiMgr->SetAnnotPropertyToolIsEnable(false);

	auto _AnnotSetting = uiMgr->getAnnotMgr()->GetAnnotSetting();
	borderWidth = 10;

	polyLine = ref new Windows::UI::Xaml::Shapes::Polyline();
	polyLine->Stroke = ref new Windows::UI::Xaml::Media::SolidColorBrush(Windows::UI::Colors::Gray);
	Windows::UI::Xaml::Media::DoubleCollection^ collection = ref new Windows::UI::Xaml::Media::DoubleCollection();
	collection->Append(0.05);
	collection->Append(0.05);
	polyLine->StrokeDashArray = collection;

	polyLine->Opacity = 0.5;
	polyLine->StrokeThickness = borderWidth;
	polyLine->StrokeStartLineCap = Windows::UI::Xaml::Media::PenLineCap::Round;
	polyLine->StrokeEndLineCap = Windows::UI::Xaml::Media::PenLineCap::Round;
	polyLine->StrokeLineJoin = Windows::UI::Xaml::Media::PenLineJoin::Round;
	polyLine->StrokeDashCap = Windows::UI::Xaml::Media::PenLineCap::Round;


	uiMgr->CreateNewShapeToCanvas(polyLine);

	Windows::UI::Xaml::FrameworkElement^ _FrameworkElement = dynamic_cast<Windows::UI::Xaml::FrameworkElement^>(sender);
	_FrameworkElement->ManipulationMode = Windows::UI::Xaml::Input::ManipulationModes::System;
	Windows::Foundation::Point _Pt;
	auto _PointerPt = e->GetCurrentPoint(_FrameworkElement);
	_Pt = _PointerPt->Position;

	auto _Pageinfo = View->ConvertDevicePtToPDFPagePt(_Pt);
	m_PageIndex = _Pageinfo->PageIndex;

	if (_Pageinfo->PagePoint.X != -1 && _Pageinfo->PagePoint.Y != -1)
	{
		m_isPressed = true;

		m_pressStartPoint = _Pt;

		pointVector = ref new Windows::UI::Xaml::Media::PointCollection();
		polyLinePointsVector->Append(pointVector);
		pointVector->Append(_Pt);

		polyLine->Points = pointVector;

		int eraserWidth = 10;
		double xvalue = m_pressStartPoint.X - eraserWidth / 2;
		double yvalue = m_pressStartPoint.Y - eraserWidth / 2;
		m_AreaRect = Windows::Foundation::Rect((float)xvalue, (float)yvalue, (float)eraserWidth, (float)eraserWidth);
	}
}

void EraserAnnotEventPeer::OnPointerMoved(Platform::Object^ sender, Windows::UI::Xaml::Input::PointerRoutedEventArgs^ e)
{
	if (!m_isPressed)
		return;

	UIExtensionsManager^ uiMgr = (UIExtensionsManager^)View->ExtensionsManager;
	Windows::UI::Xaml::FrameworkElement^ _FrameworkElement = dynamic_cast<Windows::UI::Xaml::FrameworkElement^>(sender);
	Windows::Foundation::Point currentPoint;
	auto _PointerPt = e->GetCurrentPoint(_FrameworkElement);
	currentPoint = _PointerPt->Position;
	auto _Pageinfo = View->ConvertDevicePtToPDFPagePt(currentPoint);


	Windows::Foundation::Rect pageViewRect = View->GetPageViewRect(m_PageIndex);

	if (currentPoint.Y > pageViewRect.Y + pageViewRect.Height)
	{
		currentPoint.Y = pageViewRect.Y + pageViewRect.Height;
	}
	else if (currentPoint.Y < pageViewRect.Y)
	{
		currentPoint.Y = pageViewRect.Y;
	}

	if (currentPoint.X > pageViewRect.X + pageViewRect.Width)
	{
		currentPoint.X = pageViewRect.X + pageViewRect.Width;
	}
	else if (currentPoint.X < pageViewRect.X)
	{
		currentPoint.X = pageViewRect.X;
	}
	pointVector->Append(currentPoint);
	polyLine->Points = pointVector;

	int eraserWidth = 10;
	double xvalue = _Pageinfo->PagePoint.X - eraserWidth / 2;
	double yvalue = _Pageinfo->PagePoint.Y - eraserWidth / 2;
	m_AreaRect = Windows::Foundation::Rect((float)xvalue, (float)yvalue, (float)eraserWidth, (float)eraserWidth);

	Windows::Foundation::Collections::IVector<IBaseAnnot^>^ inkRangeAnnotList = uiMgr->getAnnotMgr()->IsHitInkAnnot(_Pageinfo->PageIndex, _Pageinfo->PagePoint);
	if (!pUndoItem && inkAnnotList->Size == 0)
	{
		pUndoItem = ref new EraserAnnotUndoAction(View, inkRangeAnnotList);
		for (unsigned int i = 0; i < inkRangeAnnotList->Size; i++)
		{
			if (!inkAnnotList->HasKey(inkRangeAnnotList->GetAt(i)->AnnoIndex))
			{
				inkAnnotList->Insert(inkRangeAnnotList->GetAt(i)->AnnoIndex, inkRangeAnnotList->GetAt(i));
				auto pencilAnnotRange = dynamic_cast<PencilAnnot^>(inkRangeAnnotList->GetAt(i));
				pUndoItem->SaveBeforeEraserVectPoint(pencilAnnotRange->GetHashCode(), pencilAnnotRange->Page_VectPoint);
			}
		}
	}
	else
	{
		for (unsigned int i = 0; i < inkRangeAnnotList->Size; i++)
		{
			if (!inkAnnotList->HasKey(inkRangeAnnotList->GetAt(i)->AnnoIndex))
			{
				inkAnnotList->Insert(inkRangeAnnotList->GetAt(i)->AnnoIndex, inkRangeAnnotList->GetAt(i));
				pUndoItem->AppendAnnotToAnnotList(inkRangeAnnotList->GetAt(i));
				auto pencilAnnotRange = dynamic_cast<PencilAnnot^>(inkRangeAnnotList->GetAt(i));
				pUndoItem->SaveBeforeEraserVectPoint(pencilAnnotRange->GetHashCode(), pencilAnnotRange->Page_VectPoint);
			}
		}
	}

	for (unsigned int i = 0; i < inkRangeAnnotList->Size; i++)
	{
		annot = inkRangeAnnotList->GetAt(i);
		auto pencilAnnot = dynamic_cast<PencilAnnot^>(annot);
		uiMgr->SetEraserPageIndex(pencilAnnot->PageIndex);
		isDeleteAnnot = pencilAnnot->EraserPencilPoint(m_AreaRect, false);

		if (isDeleteAnnot)
		{
			if (pUndoItem)
				pencilAnnot->UpdatePage_VectPoint = pUndoItem->GetVectPoints(pencilAnnot->GetHashCode());
			inkAnnotList->Remove(annot->AnnoIndex);
			auto pDeleteUndoItem = ref new DeleteAnnotUndoAction(View, annot);
			UndoFactory::GetUndoInstance()->AddUndoItem(uiMgr, pDeleteUndoItem);
			annot->OnDelete();
		}
	}
}

void EraserAnnotEventPeer::OnPointerReleased(Platform::Object^ sender, Windows::UI::Xaml::Input::PointerRoutedEventArgs^ e)
{
	if (!m_isPressed)
		return;
	UIExtensionsManager^ uiMgr = (UIExtensionsManager^)View->ExtensionsManager;
	m_pressStartPoint = Point(-1, -1);
	m_isPressed = false;
	
	Windows::UI::Xaml::FrameworkElement^ _FrameworkElement = dynamic_cast<Windows::UI::Xaml::FrameworkElement^>(sender);
	Windows::Foundation::Point _PtMoved;
	auto _PointerPt = e->GetCurrentPoint(_FrameworkElement);
	_PtMoved = _PointerPt->Position;
	auto _Pageinfo = View->ConvertDevicePtToPDFPagePt(_PtMoved);

	m_EraserIndex = -1;

	if (inkAnnotList->Size > 0)
	{
		if (pUndoItem)
			UndoFactory::GetUndoInstance()->AddUndoItem(uiMgr, pUndoItem);
	}
	else
	{
		OutputDebugString(L"OnPointerReleased...inkAnnotList->Size--0\n");
	}

	View->Redraw(_Pageinfo->PageIndex);

	pointVector->Append(_PtMoved);
	polyLine->Points = pointVector;

	pUndoItem = nullptr;
	inkAnnotList->Clear();
	inkDeleteAnnotList->Clear();
	uiMgr->SetAnnotCanvasVisible(false);
	m_AreaRect = Windows::Foundation::Rect::Empty;
	_FrameworkElement->ManipulationMode = Windows::UI::Xaml::Input::ManipulationModes::All;
}

void EraserAnnotEventPeer::OnTapped(Platform::Object^ sender, Windows::UI::Xaml::Input::TappedRoutedEventArgs^ e)
{

}

void EraserAnnotEventPeer::OnRightTapped(Platform::Object^ sender, Windows::UI::Xaml::Input::RightTappedRoutedEventArgs^ e)
{

}

void EraserAnnotEventPeer::OnManipulationStarted(Platform::Object^ sender, Windows::UI::Xaml::Input::ManipulationStartedRoutedEventArgs^ e)
{

}

void EraserAnnotEventPeer::OnManipulationDelta(Platform::Object^ sender, Windows::UI::Xaml::Input::ManipulationDeltaRoutedEventArgs^ e)
{

}

void EraserAnnotEventPeer::OnManipulationCompleted(Platform::Object^ sender, Windows::UI::Xaml::Input::ManipulationCompletedRoutedEventArgs^ e)
{

}

void EraserAnnotEventPeer::OnCreateAnnotAfterSelectText()
{
	
}

void EraserAnnotEventPeer::OnManipulationInertiaStarting(Platform::Object^ sender, Windows::UI::Xaml::Input::ManipulationInertiaStartingRoutedEventArgs^ e)
{

}
