/**
 * Copyright (C) 2003-2018, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit Mobile PDF SDK to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */


#include "pch.h"
#include "FreeTextEventPeer.h"
#include "FreeTextBoxUC.xaml.h"
#include "EditAnnotUnDoAction.h"


using namespace foxit::rdk;
using namespace foxit::rdk::uiextensions;

using namespace Platform;
using namespace Windows::Foundation;

namespace foxit
{
	namespace rdk
	{
		namespace uiextensions
		{
			[Windows::Foundation::Metadata::WebHostHidden]
			public ref class CreateFreeTextAnnotUndoAction sealed : IUndoItem
			{
			public:
				CreateFreeTextAnnotUndoAction(PDFViewCtrl^ view, FreeTextAnnot^ annot);

				virtual void Undo();
				virtual void Redo();
				virtual Platform::String^ GetDescr();
				virtual void ReleaseItem();

			internal:
				property PDFViewCtrl^ View
				{
					PDFViewCtrl^ get();
				}

				property FreeTextAnnot^ AnnotHandle
				{
					FreeTextAnnot^ get();
				}

			private:
				Platform::WeakReference m_ViewWeakRef;
				Platform::WeakReference m_AnnotWeakRef;
				BaseAnnot^ m_RedoAnnot;
			};
		}
	}
}

CreateFreeTextAnnotUndoAction::CreateFreeTextAnnotUndoAction(PDFViewCtrl^ view, FreeTextAnnot^ annot)
{
	m_ViewWeakRef = view;
	m_AnnotWeakRef = annot;
}

PDFViewCtrl^ CreateFreeTextAnnotUndoAction::View::get()
{
	return m_ViewWeakRef.Resolve<PDFViewCtrl>();
}

FreeTextAnnot^ CreateFreeTextAnnotUndoAction::AnnotHandle::get()
{
	return m_AnnotWeakRef.Resolve<FreeTextAnnot>();
}

void CreateFreeTextAnnotUndoAction::Undo()
{
	if (View && AnnotHandle)
	{
		m_RedoAnnot = AnnotHandle;
		View->CancelRenderTask();
		AnnotHandle->OnDelete();
	}
}

void CreateFreeTextAnnotUndoAction::Redo()
{
	if (View && m_RedoAnnot)
	{
		auto _page = View->GetDoc()->GetPage(m_RedoAnnot->PageIndex);
		View->CancelRenderTask();
		UIExtensionsManager^ uiMgr = (UIExtensionsManager^)View->ExtensionsManager;
		uiMgr->getAnnotMgr()->OnCreateNewAnnot(_page, m_RedoAnnot);
	}
}

Platform::String^ CreateFreeTextAnnotUndoAction::GetDescr()
{
	return ref new Platform::String(L"");
}

void CreateFreeTextAnnotUndoAction::ReleaseItem()
{
	m_ViewWeakRef = nullptr;
	m_AnnotWeakRef = nullptr;
	m_RedoAnnot = nullptr;
}


FreeTextEventPeer::FreeTextEventPeer(PDFViewCtrl^ view)
{
	m_PageViewWeakRef = view;
	m_pressPoint = Point(-1, -1);
	m_isPressed = false;
	RegisterInputPanelEvent();
	m_IsInputPanelShow = false;
}

PDFViewCtrl^ FreeTextEventPeer::View::get()
{
	return m_PageViewWeakRef.Resolve<PDFViewCtrl>();
}

void FreeTextEventPeer::OnPointerPressed(Platform::Object^ sender, Windows::UI::Xaml::Input::PointerRoutedEventArgs^ e)
{
	Windows::UI::Xaml::FrameworkElement^ _FrameworkElement = dynamic_cast<Windows::UI::Xaml::FrameworkElement^>(sender);
	Windows::Foundation::Point _Pt;
	auto _PointerPt = e->GetCurrentPoint(_FrameworkElement);
	_Pt = _PointerPt->Position;

	auto _Pageinfo = View->ConvertDevicePtToPDFPagePt(_Pt);
}

void FreeTextEventPeer::OnPointerMoved(Platform::Object^ sender, Windows::UI::Xaml::Input::PointerRoutedEventArgs^ e)
{

}

void FreeTextEventPeer::OnPointerReleased(Platform::Object^ sender, Windows::UI::Xaml::Input::PointerRoutedEventArgs^ e)
{

}

void FreeTextEventPeer::OnTapped(Platform::Object^ sender, Windows::UI::Xaml::Input::TappedRoutedEventArgs^ e)
{
	UIExtensionsManager^ uiMgr = (UIExtensionsManager^)View->ExtensionsManager;
	Windows::UI::Xaml::FrameworkElement^ _FrameworkElement = dynamic_cast<Windows::UI::Xaml::FrameworkElement^>(sender);
	auto _Pt = e->GetPosition(_FrameworkElement);
	auto _Pageinfo = View->ConvertDevicePtToPDFPagePt(_Pt);
	if (_Pageinfo->PagePoint.X == -1 || _Pageinfo->PagePoint.Y == -1)
	{
		if (uiMgr->GetAnnotCanvasHasChild())
		{
			OnCreateAnnotAfterSelectText();
			if (!uiMgr->getAnnotMgr()->GetAnnotSetting()->GetFreeTextIsContinueOpera())
			{
				uiMgr->CreateTypeOfAnnot = AnnotCreateType::None;
				uiMgr->ExitAnnotPropertyMode();
			}
		}
		return;
	}

	auto _SelectedAnnot = uiMgr->getAnnotMgr()->GetSelectedAnnot();
	if (_SelectedAnnot)
	{
		m_PageIndex = _SelectedAnnot->PageIndex;

		FreeTextAnnot^ annot = dynamic_cast<FreeTextAnnot^>(_SelectedAnnot);
		Windows::Foundation::Rect rectMinRect = annot->MinRect;
		foxit::common::fxcrt::RectF^ rectF = FoxitRectConverter::ConvertToFoxitRectF(rectMinRect);

		uiMgr->CreateRectangelAnnotToCanvas(annot->PageIndex, annot->Page_HightLightRect, annot->Opacity, annot->BorderWidth, annot->Color);

		Windows::Foundation::Rect rect = annot->Page_HightLightRect;
		rect.X = rect.X - 5;
		rect.Y = rect.Y - 5;
		rect.Width = rect.Width + 5 * 2;
		rect.Height = rect.Height + 5 * 2;
		auto deviceRect = View->ConvertPageRectToDeviceRect(uiMgr->getAnnotMgr()->GetSelectedAnnot()->PageIndex, rect);
		Point pt;

		if (View->GetDoc() && !FSPDFDocUtil::GetIsDocumentCanModifyAnnotation(View->GetDoc()) && View->GetDoc()->GetPasswordType() != foxit::pdf::PasswordType::e_PwdOwner)
		{
		}
		else
		{
			Windows::Foundation::Rect pageViewRect = View->GetPageViewRect(m_PageIndex);

			auto deviceMinRect = View->ConvertPageRectToDeviceRect(uiMgr->getAnnotMgr()->GetSelectedAnnot()->PageIndex, annot->Page_HightLightRect);
			Rect pageRect = deviceRect;
			if (deviceMinRect.X < pageViewRect.X && deviceMinRect.Width>pageViewRect.Width)
			{
				pageRect.X = pageViewRect.X;
				pt = Point(pageRect.X + pageViewRect.Width / 2, pageRect.Y);
			}
			else
			{
				pt = Point(pageRect.X + pageRect.Width / 2, pageRect.Y);
			}

			uiMgr->ShowTextmarkContextmenu(AnnotContextMenuType::Edit, pt);
		}
	}
	else
	{
		if (uiMgr->GetAnnotCanvasHasChild() && !uiMgr->getAnnotMgr()->GetAnnotSetting()->GetFreeTextIsContinueOpera())
		{
			OnCreateAnnotAfterSelectText();
			if (!uiMgr->getAnnotMgr()->GetAnnotSetting()->GetFreeTextIsContinueOpera())
			{
				uiMgr->CreateTypeOfAnnot = AnnotCreateType::None;
				uiMgr->ExitAnnotPropertyMode();
			}
		}
		else
		{
			if (uiMgr->GetAnnotCanvasHasChild())
			{
				OnCreateAnnotAfterSelectText();
				return;
			}

			auto _AnnotSetting = uiMgr->getAnnotMgr()->GetAnnotSetting();
			m_needMoveOffset = 0;

			m_pressPoint = _Pt;
			m_PageIndex = _Pageinfo->PageIndex;
			freeTextBox = ref new FreeTextBoxUC();
			uiMgr->CreateFreeTextBoxToCanvas(freeTextBox);
			uiMgr->viewControl->EnableScrollView(false);
			float fontSize =(float) _AnnotSetting->GetFreeTextFontSize();

			m_AreaRect.X = _Pt.X;
			m_AreaRect.Y = _Pt.Y;
			m_AreaRect.Width = (fontSize * 1) * View->GetScale();
			m_AreaRect.Height = (fontSize + 4) * View->GetScale();

			m_AreaRect.Y = m_AreaRect.Y;
			m_AreaRect.X = m_AreaRect.X;

			Windows::Foundation::Rect m_WindowsRect = Windows::UI::Xaml::Window::Current->Bounds;
			double scale = GetRenderScale();
			int fontSizeActual = (int)(fontSize*scale*1.5);
			Windows::Foundation::Rect pageViewRect = View->GetPageViewRect(m_PageIndex);
			double maxHeight = pageViewRect.Y + pageViewRect.Height - _Pt.Y;

			if (m_AreaRect.X < pageViewRect.X)
				m_AreaRect.X = pageViewRect.X;

			if (m_AreaRect.Y > pageViewRect.Y + pageViewRect.Height - fontSizeActual)
			{
				m_AreaRect.Y = pageViewRect.Y + pageViewRect.Height - fontSizeActual;
				maxHeight = pageViewRect.Y + pageViewRect.Height - m_AreaRect.Y;
				freeTextBox->textBox->MaxHeight = pageViewRect.Y + pageViewRect.Height - m_AreaRect.Y;
			}
			else
			{
				freeTextBox->textBox->MaxHeight = pageViewRect.Y + pageViewRect.Height - _Pt.Y;
			}

			freeTextBox->textBox->MinWidth = m_AreaRect.Width;
			double maxwidth = pageViewRect.X + pageViewRect.Width - m_AreaRect.X;
			if (maxwidth < m_AreaRect.Width)
			{
				m_AreaRect.X = m_AreaRect.X - m_AreaRect.Width;
				maxwidth = maxwidth + m_AreaRect.Width;
			}
			if(maxwidth <= 0) return;
			freeTextBox->textBox->MaxWidth = maxwidth;

			freeTextBox->SetValue(Windows::UI::Xaml::Controls::Canvas::TopProperty, m_AreaRect.Y);
			freeTextBox->SetValue(Windows::UI::Xaml::Controls::Canvas::LeftProperty, m_AreaRect.X);

			freeTextBox->textBox->AcceptsReturn = true;
			freeTextBox->textBox->TextWrapping = Windows::UI::Xaml::TextWrapping::Wrap;


			Platform::String^ fontName = _AnnotSetting->GetFreeTextFontName();
			const wchar_t *seperator = wcsrchr(fontName->Data(), '-');
			String^ fontNameHeader = seperator ? ref new Platform::String(seperator - 1) : fontName;


			if (fontName == "Helvetica" || fontName == "Helvetica-BoldOblique" || fontName == "Helvetica-Oblique" || fontName == "Helvetica-Bold")
			{
				freeTextBox->textBox->FontFamily = ref new Windows::UI::Xaml::Media::FontFamily("Arial");
			}
			else if (fontName == "Courier" || fontName == "Courier-BoldOblique" || fontName == "Courier-Oblique" || fontName == "Courier-Bold")
			{
				freeTextBox->textBox->FontFamily = ref new Windows::UI::Xaml::Media::FontFamily("Courier New");
			}
			else if (fontName == "Times-Roman" || fontName == "Times-BoldItalic" || fontName == "Times-Italic" || fontName == "Times-Bold")
			{
				freeTextBox->textBox->FontFamily = ref new Windows::UI::Xaml::Media::FontFamily("Times New Roman");
			}

			if (fontName == "Courier-BoldOblique"
				|| fontName == "Helvetica-BoldOblique"
				|| fontName == "Times-BoldItalic")
			{
				freeTextBox->textBox->FontStyle = Windows::UI::Text::FontStyle::Italic;
				freeTextBox->textBox->FontWeight = Windows::UI::Text::FontWeights::Bold;
			}

			else if (fontName == "Courier-Oblique"
				|| fontName == "Helvetica-Oblique"
				|| fontName == "Times-Italic")
			{
				freeTextBox->textBox->FontStyle = Windows::UI::Text::FontStyle::Italic;
				freeTextBox->textBox->FontWeight = Windows::UI::Text::FontWeights::Thin;
			}

			else if (fontName == "Courier-Bold"
				|| fontName == "Helvetica-Bold"
				|| fontName == "Times-Bold")
			{
				freeTextBox->textBox->FontWeight = Windows::UI::Text::FontWeights::Bold;
				freeTextBox->textBox->FontStyle = Windows::UI::Text::FontStyle::Normal;
			}
			else
			{
				freeTextBox->textBox->FontStyle = Windows::UI::Text::FontStyle::Normal;
				freeTextBox->textBox->FontWeight = Windows::UI::Text::FontWeights::Thin;
			}
			freeTextBox->textBox->FontSize =fontSize*GetRenderScale();
			freeTextBox->textBox->Foreground = ref new Windows::UI::Xaml::Media::SolidColorBrush(_AnnotSetting->GetFreeTextColor());
			freeTextBox->textBox->Focus(Windows::UI::Xaml::FocusState::Programmatic);
			freeTextBox->textBox->Opacity = _AnnotSetting->GetFreeTextAnnotOpacity();
		}
	}
}

Windows::Foundation::Rect FreeTextEventPeer::CalculateRectSize()
{
	Windows::Foundation::Rect rect = freeTextBox->GetAreaRect();

	Windows::UI::Xaml::Controls::TextBlock^ tb = ref new Windows::UI::Xaml::Controls::TextBlock();
	String^ textStr = freeTextBox->textBox->Text;
	tb->Text = freeTextBox->textBox->Text;
	tb->FontSize = freeTextBox->textBox->FontSize;
	tb->FontFamily = freeTextBox->textBox->FontFamily;
	tb->FontStyle = freeTextBox->textBox->FontStyle;
	tb->MaxWidth = freeTextBox->textBox->MaxWidth;
	tb->TextWrapping = Windows::UI::Xaml::TextWrapping::Wrap;

	Windows::Foundation::Rect m_WindowsRect = Windows::UI::Xaml::Window::Current->Bounds;
	Windows::Foundation::Rect pageViewRect = View->GetPageViewRect(m_PageIndex);

	Windows::Foundation::Size size = Windows::Foundation::Size(pageViewRect.Width, pageViewRect.Height);


	tb->Measure(size);
	Size sizeValue = tb->DesiredSize;
	double maxheight = freeTextBox->textBox->MaxHeight;
	if (sizeValue.Height > freeTextBox->textBox->MaxHeight)
	{
		int offsetY = (int)(sizeValue.Height - freeTextBox->textBox->MaxHeight);
		rect.Y = rect.Y - offsetY;
		rect.Height = rect.Height + offsetY;
		if (rect.Y == 0)
		{
			rect.Y = rect.Y + 1;
			rect.Height = rect.Height - 1;
		}
	}
	return rect;
}

void FreeTextEventPeer::OnRightTapped(Platform::Object^ sender, Windows::UI::Xaml::Input::RightTappedRoutedEventArgs^ e)
{

}

void FreeTextEventPeer::OnManipulationStarted(Platform::Object^ sender, Windows::UI::Xaml::Input::ManipulationStartedRoutedEventArgs^ e)
{

}

void FreeTextEventPeer::OnManipulationDelta(Platform::Object^ sender, Windows::UI::Xaml::Input::ManipulationDeltaRoutedEventArgs^ e)
{

}

void FreeTextEventPeer::OnManipulationCompleted(Platform::Object^ sender, Windows::UI::Xaml::Input::ManipulationCompletedRoutedEventArgs^ e)
{

}

void FreeTextEventPeer::OnCreateAnnotAfterSelectText()
{
	if (!freeTextBox)
		return;

	UIExtensionsManager^ uiMgr = (UIExtensionsManager^)View->ExtensionsManager;
	uiMgr->viewControl->EnableScrollView(true);
	if (freeTextBox->textBox->Text->IsEmpty())
	{
		uiMgr->SetAnnotCanvasVisible(false);
		freeTextBox = nullptr;
		return;
	}

	auto _Annot = AnnotCreaterFactory::GetInstance()->CreateNewAnnot(AnnotationType::FreeText);
	auto _pCreateAnnot = dynamic_cast<FreeTextAnnot^>(_Annot);
	auto _AnnotSetting = uiMgr->getAnnotMgr()->GetAnnotSetting();

	Windows::Foundation::Rect rect;
	rect.X = m_pressPoint.X;
	rect.Y = m_pressPoint.Y;
	rect.Width = (float)freeTextBox->textBox->ActualWidth;
	rect.Height = (float)freeTextBox->textBox->ActualHeight;

	double actualWidth = freeTextBox->textBox->ActualWidth;
	double actualHeight = freeTextBox->textBox->ActualHeight;
	if (m_IsInputPanelShow)
	{
		double yValue = freeTextBox->GetAreaRect().Y;
		freeTextBox->SetValue(Windows::UI::Xaml::Controls::Canvas::TopProperty, yValue + m_needMoveOffset);
	}

	auto _Pageinfo = View->ConvertDevicePtToPDFPagePt(m_pressPoint);
	int pageRotate = View->GetRotate();

	Windows::Foundation::Rect pageViewRect = View->GetPageViewRect(m_PageIndex);

	auto areaRect = CalculateRectSize();
	TypeWriterRange^ typewirterRange = GetContentStr(areaRect);
	if (!typewirterRange)
		return;
	_pCreateAnnot->Contents = typewirterRange->ContentStr;
	if (areaRect.Height < typewirterRange->ActualHeight)
		areaRect.Height = typewirterRange->ActualHeight;
	if (areaRect.Width < typewirterRange->ActualWidth)
		areaRect.Width = typewirterRange->ActualWidth;


	auto deviceRc = View->ConvertDeviceRectToPageRect(_Pageinfo->PageIndex, areaRect);

	_pCreateAnnot->Name = uiMgr->getAnnotMgr()->GetAnnotSetting()->GetNoteAuthor();
	_pCreateAnnot->Title = _AnnotSetting->GetNoteAuthor();
	_pCreateAnnot->CreationDateTime = (ref new Windows::Globalization::Calendar())->GetDateTime();

	Windows::Foundation::Rect pageRect = deviceRc;

	_pCreateAnnot->Opacity = _AnnotSetting->GetFreeTextAnnotOpacity();
	_pCreateAnnot->Page_HightLightRect = pageRect;

	auto _page = View->GetDoc()->GetPage(_Pageinfo->PageIndex);

	int fontsize = (int)FSPDFDocUtil::PixelToPDFPoint(View->GetDoc(), m_PageIndex, _AnnotSetting->GetFreeTextFontSize());
	_pCreateAnnot->FontSize = _AnnotSetting->GetFreeTextFontSize();
	_pCreateAnnot->TextColor = _AnnotSetting->GetFreeTextColor();
	_pCreateAnnot->FontName = _AnnotSetting->GetFreeTextFontName();

	uiMgr->getAnnotMgr()->OnCreateNewAnnot(_page, _pCreateAnnot);
	uiMgr->SetAnnotCanvasVisible(false);

	auto pUndoItem = ref new CreateFreeTextAnnotUndoAction(View, _pCreateAnnot);
	UndoFactory::GetUndoInstance()->AddUndoItem(uiMgr, pUndoItem);
	
	freeTextBox = nullptr;
}

void FreeTextEventPeer::OnManipulationInertiaStarting(Platform::Object^ sender, Windows::UI::Xaml::Input::ManipulationInertiaStartingRoutedEventArgs^ e)
{

}

void FreeTextEventPeer::OnEditFreeText()
{
	UIExtensionsManager^ uiMgr = (UIExtensionsManager^)View->ExtensionsManager;
	auto _SelectedAnnot = uiMgr->getAnnotMgr()->GetSelectedAnnot();
	Windows::Foundation::Rect m_WindowsRect = Windows::UI::Xaml::Window::Current->Bounds;
	if (_SelectedAnnot)
	{
		freeTextBox = ref new FreeTextBoxUC();
		uiMgr->CreateFreeTextBoxToCanvas(freeTextBox);

		FreeTextAnnot^ freeAnnot = dynamic_cast<FreeTextAnnot^>(_SelectedAnnot);
		Windows::Foundation::Rect minRect = freeAnnot->Page_HightLightRect;
		auto deviceRect = View->ConvertPageRectToDeviceRect(_SelectedAnnot->PageIndex, minRect);
		m_AreaRect = deviceRect;

		freeTextBox->textBox->AcceptsReturn = true;
		freeTextBox->textBox->TextWrapping = Windows::UI::Xaml::TextWrapping::Wrap;
		freeTextBox->textBox->BorderThickness = 0;


		float fontSize = (float)freeAnnot->FontSize;

		if (freeAnnot->FontName != "" && !freeAnnot->FontName->IsEmpty())
		{
			Platform::String^ fontName = freeAnnot->FontName;
			const wchar_t *seperator = wcsrchr(fontName->Data(), '-');
			String^ fontNameHeader = seperator ? ref new Platform::String(seperator) : fontName;

			if (fontName == "Helvetica" || fontName == "Helvetica-BoldOblique" || fontName == "Helvetica-Oblique" || fontName == "Helvetica-Bold")
			{
				freeTextBox->textBox->FontFamily = ref new Windows::UI::Xaml::Media::FontFamily("Arial");
			}
			else if (fontName == "Courier" || fontName == "Courier-BoldOblique" || fontName == "Courier-Oblique" || fontName == "Courier-Bold")
			{
				freeTextBox->textBox->FontFamily = ref new Windows::UI::Xaml::Media::FontFamily("Courier New");
			}
			else if (fontName == "Times-Roman" || fontName == "Times-BoldItalic" || fontName == "Times-Italic" || fontName == "Times-Bold")
			{
				freeTextBox->textBox->FontFamily = ref new Windows::UI::Xaml::Media::FontFamily("Times New Roman");
			}

			if (fontName == "Courier-BoldOblique"
				|| fontName == "Helvetica-BoldOblique"
				|| fontName == "Times-BoldItalic")
			{
				freeTextBox->textBox->FontStyle = Windows::UI::Text::FontStyle::Italic;
				freeTextBox->textBox->FontWeight = Windows::UI::Text::FontWeights::Bold;
			}

			else if (fontName == "Courier-Oblique"
				|| fontName == "Helvetica-Oblique"
				|| fontName == "Times-Italic")
			{
				freeTextBox->textBox->FontStyle = Windows::UI::Text::FontStyle::Italic;
				freeTextBox->textBox->FontWeight = Windows::UI::Text::FontWeights::Thin;
			}

			else if (fontName == "Courier-Bold"
				|| fontName == "Helvetica-Bold"
				|| fontName == "Times-Bold")
			{
				freeTextBox->textBox->FontWeight = Windows::UI::Text::FontWeights::Bold;
				freeTextBox->textBox->FontStyle = Windows::UI::Text::FontStyle::Normal;
			}
			else
			{
				freeTextBox->textBox->FontStyle = Windows::UI::Text::FontStyle::Normal;
				freeTextBox->textBox->FontWeight = Windows::UI::Text::FontWeights::Thin;
			}
		}
		freeTextBox->textBox->FontSize = fontSize*GetRenderScale();
		freeTextBox->textBox->Foreground = ref new Windows::UI::Xaml::Media::SolidColorBrush(freeAnnot->TextColor);
		freeTextBox->textBox->Opacity = freeAnnot->Opacity;
		freeTextBox->textBox->Text = freeAnnot->Contents;

		Windows::Foundation::Rect pageViewRect = View->GetPageViewRect(_SelectedAnnot->PageIndex);

		if (m_AreaRect.X < pageViewRect.X)
			m_AreaRect.X = pageViewRect.X;

		int pageRotate = View->GetRotate();

		if ((int)foxit::common::Rotation::e_Rotation0 == pageRotate || (int)foxit::common::Rotation::e_Rotation180 == pageRotate)
		{

		}
		else
		{
			double heightvalue = m_AreaRect.Y + m_AreaRect.Height;
			double widthvalue = m_AreaRect.Width;

			if (m_AreaRect.Width > m_AreaRect.Height)
			{
				Windows::UI::Xaml::Controls::TextBlock^ tb = ref new Windows::UI::Xaml::Controls::TextBlock();
				String^ str1 = freeTextBox->textBox->Text;
				double maxwidthvalue = m_WindowsRect.Width - m_AreaRect.X;
				freeTextBox->textBox->MaxWidth = pageViewRect.X + pageViewRect.Width - m_AreaRect.X;

				TypeWriterRange^ typewirterRange = GetContentStr(m_AreaRect);
				if (typewirterRange->ContentStr->Length() == 0)
					return;

				tb->Text = typewirterRange->ContentStr;
				tb->FontSize = freeTextBox->textBox->FontSize;
				tb->FontFamily = freeTextBox->textBox->FontFamily;
				tb->FontStyle = freeTextBox->textBox->FontStyle;

				tb->MaxWidth = maxwidthvalue;
				tb->TextWrapping = Windows::UI::Xaml::TextWrapping::Wrap;

				Windows::Foundation::Size size = Windows::Foundation::Size(pageViewRect.Width, pageViewRect.Height);

				tb->Measure(size);
				Size sizeValue = tb->DesiredSize;
				if (sizeValue.Height > m_AreaRect.Height)
				{
					int offsetY = (int)(sizeValue.Height - m_AreaRect.Height);
					if (m_AreaRect.Y + m_AreaRect.Height + offsetY <= pageViewRect.Y + pageViewRect.Height)
					{
						m_AreaRect.Height = sizeValue.Height;
					}
					else
					{
						offsetY = (int)((m_AreaRect.Y + m_AreaRect.Height + offsetY) - (pageViewRect.Y + pageViewRect.Height));
						m_AreaRect.Y = m_AreaRect.Y - offsetY;
						m_AreaRect.Height = sizeValue.Height;
						if (m_AreaRect.Y == 0)
						{
							m_AreaRect.Y = m_AreaRect.Y + 1;
							m_AreaRect.Height = m_AreaRect.Height - 1;
						}
					}
				}
			}
			else
			{
				m_AreaRect.X = m_AreaRect.X;
				m_AreaRect.Width = m_AreaRect.Height;
				m_AreaRect.Height = (float)widthvalue;

				if (m_AreaRect.X<pageViewRect.X)
					m_AreaRect.X = pageViewRect.X;

				if (m_AreaRect.Y<pageViewRect.Y)
					m_AreaRect.Y = pageViewRect.Y;

				if (m_AreaRect.X + m_AreaRect.Width > pageViewRect.X + pageViewRect.Width)
				{
					m_AreaRect.X = m_AreaRect.X - ((m_AreaRect.X + m_AreaRect.Width) - (pageViewRect.X + pageViewRect.Width));
				}
				if (m_AreaRect.Y + m_AreaRect.Height > pageViewRect.Y + pageViewRect.Height)
				{
					m_AreaRect.Y = m_AreaRect.Y - ((m_AreaRect.Y + m_AreaRect.Height) - (pageViewRect.Y + pageViewRect.Height));
				}
			}
		}

		freeTextBox->SetValue(Windows::UI::Xaml::Controls::Canvas::TopProperty, m_AreaRect.Y);
		freeTextBox->SetValue(Windows::UI::Xaml::Controls::Canvas::LeftProperty, m_AreaRect.X);
		double maxwidth = m_WindowsRect.Width - m_AreaRect.X;
		freeTextBox->textBox->MaxWidth = pageViewRect.X + pageViewRect.Width - m_AreaRect.X;
		double maxHeight = pageViewRect.Y + pageViewRect.Height - m_AreaRect.Y;
		freeTextBox->textBox->MaxHeight = maxHeight;

		if ((int)foxit::common::Rotation::e_Rotation90 == pageRotate || (int)foxit::common::Rotation::e_Rotation270 == pageRotate)
		{
			if (m_AreaRect.Width <= m_AreaRect.Height)
			{
				double maxWidth = pageViewRect.Y + pageViewRect.Height - m_AreaRect.Y;
				double maxHeight = pageViewRect.X + pageViewRect.Width - m_AreaRect.X;
				if (freeTextBox->textBox->MaxWidth > maxWidth)
					freeTextBox->textBox->MaxWidth = maxWidth;

				if (freeTextBox->textBox->MaxHeight > maxHeight)
					freeTextBox->textBox->MaxHeight = maxHeight;
			}
		}

		freeTextBox->textBox->SelectionStart = freeTextBox->textBox->Text->Length();
		freeTextBox->textBox->Focus(Windows::UI::Xaml::FocusState::Programmatic);

		_SelectedAnnot->SetAnnotVisible(false);
	}
}

void FreeTextEventPeer::OnUpdateFreeTextContent()
{
	UIExtensionsManager^ uiMgr = (UIExtensionsManager^)View->ExtensionsManager;
	auto _SelectedAnnot = uiMgr->getAnnotMgr()->GetSelectedAnnot();
	if (freeTextBox && _SelectedAnnot)
	{
		if (freeTextBox->textBox->Text == "")
		{
			_SelectedAnnot->OnDelete();
			uiMgr->getAnnotMgr()->SetPdfSelectAnnotNull();

			auto pUndoItem = ref new DeleteAnnotUndoAction(View, _SelectedAnnot);
			UndoFactory::GetUndoInstance()->AddUndoItem(uiMgr, pUndoItem);
		}
		else
		{
			FreeTextAnnot^ freeTextAnnot = dynamic_cast<FreeTextAnnot^>(_SelectedAnnot);
			if (freeTextAnnot)
			{
				Windows::Foundation::Rect m_freeTextInitRect = freeTextAnnot->Page_HightLightRect;
				Windows::Foundation::Numerics::float3x2 m_freeTextInitMatrix = freeTextAnnot->TextMatrix;

				String^ str1 = freeTextBox->textBox->Text;
				String^ str2 = freeTextAnnot->Contents;

				if (freeTextBox->textBox->Text != freeTextAnnot->Contents)
				{
					int pageRotate = View->GetRotate();
					Windows::Foundation::Rect pageViewRect = View->GetPageViewRect(m_PageIndex);

					Windows::Foundation::Rect textBoxrect = freeTextBox->GetAreaRect();

					auto areaRect = CalculateRectSize();

					TypeWriterRange^ typewirterRange = GetContentStr(areaRect);
					if (typewirterRange->ContentStr->Length() == 0)
						return;
					freeTextAnnot->Contents = typewirterRange->ContentStr;
					freeTextAnnot->IsModifyProperty = true;


					if (m_IsInputPanelShow)
					{
						double yValue = freeTextBox->GetAreaRect().Y;
						freeTextBox->SetValue(Windows::UI::Xaml::Controls::Canvas::TopProperty, yValue + m_needMoveOffset);
					}
					
					if (areaRect.Height != typewirterRange->ActualHeight)
						areaRect.Height = typewirterRange->ActualHeight;

					if (areaRect.Width <= 0)
						return;

					Windows::Foundation::Numerics::float3x2 textMatrix = m_freeTextInitMatrix;
					textMatrix.m11 = 1;
					textMatrix.m22 = 1;

					freeTextAnnot->TextMatrix = textMatrix;
					freeTextAnnot->ResetAppearence();
				}
				else
				{
					freeTextAnnot->ResetAppearence();
				}

				_SelectedAnnot->SetAnnotVisible(true);
			}
		}
		freeTextBox = nullptr;
	}
}

void FreeTextEventPeer::RegisterInputPanelEvent()
{
	auto inputPane = Windows::UI::ViewManagement::InputPane::GetForCurrentView();
	__inputPaneShowingEventRegistrationToken = inputPane->Showing += ref new TypedEventHandler<Windows::UI::ViewManagement::InputPane^, Windows::UI::ViewManagement::InputPaneVisibilityEventArgs^>([=](Windows::UI::ViewManagement::InputPane^ sender, Windows::UI::ViewManagement::InputPaneVisibilityEventArgs^ args)
	{
		m_IsInputPanelShow = true;
		m_needMoveOffset = 0;
		auto winrc = Windows::UI::Xaml::Window::Current->Bounds;
		float top = args->OccludedRect.Top;
		float inputHeight = args->OccludedRect.Height;
		if ((m_AreaRect.Y) > top || m_AreaRect.Y + 30 > top)
		{
			m_needMoveOffset = m_AreaRect.Y + 30 - args->OccludedRect.Y;
			if (freeTextBox)
				freeTextBox->SetValue(Windows::UI::Xaml::Controls::Canvas::TopProperty, m_AreaRect.Y - m_needMoveOffset);
		}
	});

	__inputPaneHideEventRegistrationToken = inputPane->Hiding += ref new TypedEventHandler<Windows::UI::ViewManagement::InputPane^, Windows::UI::ViewManagement::InputPaneVisibilityEventArgs^>([=](Windows::UI::ViewManagement::InputPane^ sender, Windows::UI::ViewManagement::InputPaneVisibilityEventArgs^ args)
	{
		m_IsInputPanelShow = false;
		UIExtensionsManager^ uiMgr = (UIExtensionsManager^)View->ExtensionsManager;
		if (freeTextBox && !IsTextBoxHasContent())
		{			
			uiMgr->SetAnnotCanvasVisible(false);
			freeTextBox = nullptr;
		}
		if (m_needMoveOffset != 0)
		{
			if (isfinite(m_AreaRect.Width) && isfinite(m_AreaRect.Height))
			{
				if (freeTextBox)
				{
					double yValue = freeTextBox->GetAreaRect().Y;
					freeTextBox->SetValue(Windows::UI::Xaml::Controls::Canvas::TopProperty, yValue + m_needMoveOffset);
				}
			}
		}
		m_needMoveOffset = 0;
	});

}

bool FreeTextEventPeer::IsFreeTextSelect()
{
	if (freeTextBox && freeTextBox->GetIsTextBoxSelection())
		return true;
	return false;
}

void FreeTextEventPeer::UpdateEditLayout()
{
	if (freeTextBox)
	{
		freeTextBox->textBox->Text = "";
	}
}

double FreeTextEventPeer::GetRenderScale()
{
	auto pageViewRect = View->ConvertPageRectToDeviceRect(m_PageIndex, Windows::Foundation::Rect(m_pressPoint.X, m_pressPoint.Y, 1, 1));
	return pageViewRect.Width;
}

void FreeTextEventPeer::UpdatFreeTextBoxProperty()
{
	if (freeTextBox)
	{
		UIExtensionsManager^ uiMgr = (UIExtensionsManager^)View->ExtensionsManager;
		auto _AnnotSetting = uiMgr->getAnnotMgr()->GetAnnotSetting();
		freeTextBox->textBox->Foreground = ref new Windows::UI::Xaml::Media::SolidColorBrush(_AnnotSetting->GetFreeTextColor());
		freeTextBox->textBox->Opacity = _AnnotSetting->GetFreeTextAnnotOpacity();
		freeTextBox->textBox->FontSize = _AnnotSetting->GetFreeTextFontSize()*GetRenderScale();

		Platform::String^ fontName = _AnnotSetting->GetFreeTextFontName();
		const wchar_t *seperator = wcsrchr(fontName->Data(), '-');
		String^ fontNameHeader = seperator ? ref new Platform::String(seperator) : fontName;


		if (fontName == "Arial" || fontName == "Helvetica")
		{
			freeTextBox->textBox->FontFamily = ref new Windows::UI::Xaml::Media::FontFamily("Arial");
		}
		else if (fontName == "Courier")
		{
			freeTextBox->textBox->FontFamily = ref new Windows::UI::Xaml::Media::FontFamily("Courier New");
		}
		else if (fontName == "Times-Roman")
		{
			freeTextBox->textBox->FontFamily = ref new Windows::UI::Xaml::Media::FontFamily("Times New Roman");
		}

		if (fontName == "Courier-BoldOblique"
			|| fontName == "Helvetica-BoldOblique"
			|| fontName == "Times-BoldItalic")
		{
			freeTextBox->textBox->FontStyle = Windows::UI::Text::FontStyle::Italic;
			freeTextBox->textBox->FontWeight = Windows::UI::Text::FontWeights::Bold;
		}

		else if (fontName == "Courier-Oblique"
			|| fontName == "Helvetica-Oblique"
			|| fontName == "Times-Italic")
		{
			freeTextBox->textBox->FontStyle = Windows::UI::Text::FontStyle::Italic;
			freeTextBox->textBox->FontWeight = Windows::UI::Text::FontWeights::Normal;
		}

		else if (fontName == "Courier-Bold"
			|| fontName == "Helvetica-Bold"
			|| fontName == "Times-Bold")
		{
			freeTextBox->textBox->FontWeight = Windows::UI::Text::FontWeights::Bold;
			freeTextBox->textBox->FontStyle = Windows::UI::Text::FontStyle::Normal;
		}
		else
		{
			freeTextBox->textBox->FontStyle = Windows::UI::Text::FontStyle::Normal;
			freeTextBox->textBox->FontWeight = Windows::UI::Text::FontWeights::Thin;
		}
	}
}

void FreeTextEventPeer::SetFreeTextBoxGetFocus()
{
	freeTextBox->textBox->SelectionStart = freeTextBox->textBox->Text->Length();
	freeTextBox->textBox->Focus(Windows::UI::Xaml::FocusState::Programmatic);
}

TypeWriterRange^ FreeTextEventPeer::GetContentStr(Windows::Foundation::Rect actualRect)
{
	Platform::String^ contentStr = freeTextBox->textBox->Text;

	double mingwidth = freeTextBox->textBox->MinWidth;
	double maxwidth = freeTextBox->textBox->MaxWidth;
	double actualwidth = freeTextBox->textBox->ActualWidth;
	double actualheight = freeTextBox->textBox->ActualHeight;

	int pageindex = m_PageIndex;
	UIExtensionsManager^ uiMgr = (UIExtensionsManager^)View->ExtensionsManager;
	if (uiMgr->CreateTypeOfAnnot == AnnotCreateType::FreeText)
	{
		pageindex = m_PageIndex;
	}
	else
	{
		if (!uiMgr->getAnnotMgr()->GetSelectedAnnot())
			return nullptr;
		pageindex = uiMgr->getAnnotMgr()->GetSelectedAnnot()->PageIndex;
	}

	int pageRotate = View->GetRotate();
	if ((int)foxit::common::Rotation::e_Rotation90 == pageRotate || (int)foxit::common::Rotation::e_Rotation270 == pageRotate)
	{
		Windows::Foundation::Rect pageViewRect = View->GetPageViewRect(m_PageIndex);
		Windows::Foundation::Rect rect = freeTextBox->GetAreaRect();
		maxwidth = pageViewRect.Y + pageViewRect.Height - rect.Y;
		if (maxwidth < actualRect.Height)
			maxwidth = actualRect.Height;
	}


	TypeWriterRange^ content = uiMgr->GetTypeWriterContent(pageindex, freeTextBox->textBox->Text, freeTextBox->textBox->FontSize, freeTextBox->textBox->FontFamily,
		freeTextBox->textBox->FontStyle, freeTextBox->textBox->FontWeight, freeTextBox->textBox->MinWidth, maxwidth);
	return content;
}

void FreeTextEventPeer::ReCacluateByChangeFontSize()
{
	UIExtensionsManager^ uiMgr = (UIExtensionsManager^)View->ExtensionsManager;
	auto _SelectedAnnot = uiMgr->getAnnotMgr()->GetSelectedAnnot();
	if (_SelectedAnnot)
	{
		FreeTextAnnot^ annot = dynamic_cast<FreeTextAnnot^>(uiMgr->getAnnotMgr()->GetSelectedAnnot());

		freeTextBox = ref new FreeTextBoxUC();
		uiMgr->CreateFreeTextBoxToCanvas(freeTextBox);

		freeTextBox->textBox->Foreground = ref new Windows::UI::Xaml::Media::SolidColorBrush(_SelectedAnnot->Color);
		freeTextBox->textBox->Opacity = _SelectedAnnot->Opacity;
		int fontSize = (int)(annot->FontSize*GetRenderScale());
		freeTextBox->textBox->FontSize = fontSize;

		Platform::String^ fontName = annot->FontName;
		const wchar_t *seperator = wcsrchr(fontName->Data(), '-');
		String^ fontNameHeader = seperator ? ref new Platform::String(seperator) : fontName;

		if (fontName == "Arial" || fontName == "Helvetica")
		{
			freeTextBox->textBox->FontFamily = ref new Windows::UI::Xaml::Media::FontFamily("Arial");
		}
		else if (fontName == "Courier")
		{
			freeTextBox->textBox->FontFamily = ref new Windows::UI::Xaml::Media::FontFamily("Courier New");
		}
		else if (fontName == "Times Roman")
		{
			freeTextBox->textBox->FontFamily = ref new Windows::UI::Xaml::Media::FontFamily("Times New Roman");
		}

		if (fontName == "Courier-BoldOblique"
			|| fontName == "Helvetica-BoldOblique"
			|| fontName == "Times-BoldItalic")
		{
			freeTextBox->textBox->FontStyle = Windows::UI::Text::FontStyle::Italic;
			freeTextBox->textBox->FontWeight = Windows::UI::Text::FontWeights::Bold;
		}

		else if (fontName == "Courier-Oblique"
			|| fontName == "Helvetica-Oblique"
			|| fontName == "Times-Italic")
		{
			freeTextBox->textBox->FontStyle = Windows::UI::Text::FontStyle::Italic;
			freeTextBox->textBox->FontWeight = Windows::UI::Text::FontWeights::Thin;
		}

		else if (fontName == "Courier-Bold"
			|| fontName == "Helvetica-Bold"
			|| fontName == "Times-Bold")
		{
			freeTextBox->textBox->FontWeight = Windows::UI::Text::FontWeights::Bold;
			freeTextBox->textBox->FontStyle = Windows::UI::Text::FontStyle::Normal;
		}
		else
		{
			freeTextBox->textBox->FontStyle = Windows::UI::Text::FontStyle::Normal;
			freeTextBox->textBox->FontWeight = Windows::UI::Text::FontWeights::Thin;
		}

		Windows::Foundation::Rect minRect = annot->Page_HightLightRect;
		auto deviceRect = View->ConvertPageRectToDeviceRect(_SelectedAnnot->PageIndex, minRect);

		freeTextBox->SetValue(Windows::UI::Xaml::Controls::Canvas::TopProperty, deviceRect.Y);
		freeTextBox->SetValue(Windows::UI::Xaml::Controls::Canvas::LeftProperty, deviceRect.X);
		Windows::Foundation::Rect pageViewRect = View->GetPageViewRect(_SelectedAnnot->PageIndex);

		freeTextBox->textBox->MaxWidth = pageViewRect.X + pageViewRect.Width - deviceRect.X;
		double maxHeight = pageViewRect.Y + pageViewRect.Height - deviceRect.Y;
		freeTextBox->textBox->MaxHeight = maxHeight;

		freeTextBox->textBox->AcceptsReturn = true;
		freeTextBox->textBox->TextWrapping = Windows::UI::Xaml::TextWrapping::Wrap;
		freeTextBox->textBox->BorderThickness = 0;
		freeTextBox->textBox->Text = annot->Contents;
		freeTextBox->Opacity = 0;

		auto areaRect = CalculateRectSize();

		TypeWriterRange^ typewirterRange = GetContentStr(areaRect);
		annot->Contents = typewirterRange->ContentStr;

		
		if (areaRect.Height != typewirterRange->ActualHeight)
			areaRect.Height = typewirterRange->ActualHeight;
		auto deviceRc = View->ConvertDeviceRectToPageRect(annot->PageIndex, areaRect);
		annot->MinRect = deviceRc;

		freeTextBox->textBox->Text = "";
		uiMgr->SetAnnotCanvasVisible(false);
	}
}

bool FreeTextEventPeer::IsTextBoxHasContent()
{
	if (freeTextBox && freeTextBox->textBox->Text->Length() > 0)
	{
		return true;
	}
	return false;
}