﻿/**
 * Copyright (C) 2003-2018, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit Mobile PDF SDK to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */


#include "pch.h"
#include "FreeTextProperty.xaml.h"
#include "FreeTextEventPeer.h"

using namespace foxit::rdk;
using namespace foxit::rdk::uiextensions;

using namespace Platform;
using namespace Windows::Foundation;
using namespace Windows::Foundation::Collections;
using namespace Windows::UI::Xaml;
using namespace Windows::UI::Xaml::Controls;
using namespace Windows::UI::Xaml::Controls::Primitives;
using namespace Windows::UI::Xaml::Data;
using namespace Windows::UI::Xaml::Input;
using namespace Windows::UI::Xaml::Media;
using namespace Windows::UI::Xaml::Navigation;
using namespace foxit::rdk::uiextensions;

// The User Control item template is documented at http://go.microsoft.com/fwlink/?LinkId=234236
FontNameItem::FontNameItem()
{

}

Platform::String^ FontNameItem::FontNameValue::get()
{
	return _FontName;
}

void FontNameItem::FontNameValue::set(Platform::String^ value)
{
	if (_FontName != value)
	{
		_FontName = value;
		OnPropertyChanged("FontNameValue");
	}
}



FontSizeItem::FontSizeItem()
{

}

int FontSizeItem::FontSizeValue::get()
{
	return _FontSize;
}

void FontSizeItem::FontSizeValue::set(int value)
{
	if (_FontSize != value)
	{
		_FontSize = value;
		OnPropertyChanged("FontSizeValue");
	}
}


FreeTextProperty::FreeTextProperty()
{
	InitializeComponent();
	this->Loaded += ref new Windows::UI::Xaml::RoutedEventHandler(this, &foxit::rdk::uiextensions::FreeTextProperty::OnLoaded);
	this->Unloaded += ref new Windows::UI::Xaml::RoutedEventHandler(this, &foxit::rdk::uiextensions::FreeTextProperty::OnUnloaded);

	InitData();
}

void FreeTextProperty::InitData()
{
	m_FontVect = ref new Platform::Collections::Vector<FontNameItem^>();
	m_FontSizeVect = ref new Platform::Collections::Vector<FontSizeItem^>();

	FontNameItem^ item1 = ref new FontNameItem();
	item1->FontNameValue = "Courier";
	m_FontVect->Append(item1);

	FontNameItem^ item9 = ref new FontNameItem();
	item9->FontNameValue = "Times-Roman";
	m_FontVect->Append(item9);
	fontNameList->ItemsSource = m_FontVect;



	FontSizeItem^ itemsize1 = ref new FontSizeItem();
	itemsize1->FontSizeValue = 6;
	m_FontSizeVect->Append(itemsize1);

	FontSizeItem^ itemsize2 = ref new FontSizeItem();
	itemsize2->FontSizeValue = 8;
	m_FontSizeVect->Append(itemsize2);

	FontSizeItem^ itemsize3 = ref new FontSizeItem();
	itemsize3->FontSizeValue = 10;
	m_FontSizeVect->Append(itemsize3);

	FontSizeItem^ itemsize4 = ref new FontSizeItem();
	itemsize4->FontSizeValue = 12;
	m_FontSizeVect->Append(itemsize4);

	FontSizeItem^ itemsize5 = ref new FontSizeItem();
	itemsize5->FontSizeValue = 14;
	m_FontSizeVect->Append(itemsize5);

	FontSizeItem^ itemsize6 = ref new FontSizeItem();
	itemsize6->FontSizeValue = 16;
	m_FontSizeVect->Append(itemsize6);

	FontSizeItem^ itemsize7 = ref new FontSizeItem();
	itemsize7->FontSizeValue = 18;
	m_FontSizeVect->Append(itemsize7);

	fontSizeList->ItemsSource = m_FontSizeVect;
}


void FreeTextProperty::RelateToView(PDFViewCtrl^ View, double widthValue, double heightValue)
{
	m_PageViewWeakRef = View;
	fontNameList->Width = widthValue;
	fontSizeList->Width = widthValue;
	gridFontSize->Height = heightValue;

	if (Windows::System::Profile::AnalyticsInfo::VersionInfo->DeviceFamily == "Windows.Mobile")
	{
		auto winrc = Windows::UI::Xaml::Window::Current->Bounds;
		double width = winrc.Width*0.45;
		tbFontName->Width = winrc.Width*0.45;
	}
	UIExtensionsManager^ uiMgr = (UIExtensionsManager^)View->ExtensionsManager;
	IBaseAnnot^ annot = uiMgr->getAnnotMgr()->GetSelectedAnnot();
	if (!annot)
	{
		tbFontName->Content = uiMgr->getAnnotMgr()->GetAnnotSetting()->GetFreeTextFontName();
		tbFontSize->Content = uiMgr->getAnnotMgr()->GetAnnotSetting()->GetFreeTextFontSize().ToString();
	}
	else
	{
		FreeTextAnnot^ freeTextannot = dynamic_cast<FreeTextAnnot^>(annot);
		if (freeTextannot)
		{
			tbFontName->Content = freeTextannot->FontName ? freeTextannot->FontName : uiMgr->getAnnotMgr()->GetAnnotSetting()->GetFreeTextFontName();
			int fontSize = (int)freeTextannot->FontSize;
			tbFontSize->Content = fontSize.ToString();
		}
	}
}

PDFViewCtrl^ FreeTextProperty::View::get()
{
	return m_PageViewWeakRef.Resolve<PDFViewCtrl>();
}

void foxit::rdk::uiextensions::FreeTextProperty::ColorPageBtnSelect_Tapped(Platform::Object^ sender, Windows::UI::Xaml::Input::TappedRoutedEventArgs^ e)
{

}


void foxit::rdk::uiextensions::FreeTextProperty::tbFontName_Tapped(Platform::Object^ sender, Windows::UI::Xaml::Input::TappedRoutedEventArgs^ e)
{
	if (!View)
		return;
	UIExtensionsManager^ uiMgr = (UIExtensionsManager^)View->ExtensionsManager;
	String^ title = uiMgr->GetResourceString(L"FontNameStrTitle");
	uiMgr->SetFreeTextPropertyShow(true, title);

	fontPanel->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
	gridFontName->Visibility = Windows::UI::Xaml::Visibility::Visible;
	gridFontSize->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
	if (Windows::System::Profile::AnalyticsInfo::VersionInfo->DeviceFamily == "Windows.Mobile")
	{
		fontNamePanel->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
	}
}


void foxit::rdk::uiextensions::FreeTextProperty::fontNameList_ItemClick(Platform::Object^ sender, Windows::UI::Xaml::Controls::ItemClickEventArgs^ e)
{
	if (!View)
		return;

	FontNameItem^ item = dynamic_cast<FontNameItem^>(e->ClickedItem);
	String^ fontName = item->FontNameValue;
	UIExtensionsManager^ uiMgr = (UIExtensionsManager^)View->ExtensionsManager;
	uiMgr->getAnnotMgr()->GetAnnotSetting()->SetFreeTextFontName(fontName);

	if (uiMgr->getAnnotMgr()->GetSelectedAnnot())
	{
		FreeTextAnnot^ annot = dynamic_cast<FreeTextAnnot^>(uiMgr->getAnnotMgr()->GetSelectedAnnot());
		if (annot->FontName != fontName)
		{
			annot->FontName = fontName;
			annot->IsModifyProperty = true;

			uiMgr->UpdateFreeTextAnnotOverlayControl(annot->PageIndex, annot->Page_HightLightRect);
		}
	}
	else
	{
		IBaseAnnotEventPeer^ annotEventPeer = uiMgr->getAnnotMgr()->GetAnnotHandlerByType(AnnotationType::FreeText)->AnnotEvent;
		FreeTextEventPeer^ freetextEventPeer = dynamic_cast<FreeTextEventPeer^>(annotEventPeer);
		if (freetextEventPeer)
			freetextEventPeer->UpdatFreeTextBoxProperty();
	}

	tbFontName->Content = fontName;

	
	String^ title = uiMgr->GetResourceString(L"FontNameStrTitle");
	if (GetIsDesktopPlatform())
	{
		uiMgr->SetFreeTextPropertyShow(false, title);
	}
	else
	{
		uiMgr->SetFreeTextPropertyShow(false, "Fill");
	}
	HideFontList();
}


void foxit::rdk::uiextensions::FreeTextProperty::imgBack_Tapped(Platform::Object^ sender, Windows::UI::Xaml::Input::TappedRoutedEventArgs^ e)
{
	UIExtensionsManager^ uiMgr = (UIExtensionsManager^)View->ExtensionsManager;
	String^ title = uiMgr->GetResourceString(L"FontSizeStrTitle");
	if (gridFontName->Visibility == Windows::UI::Xaml::Visibility::Visible)
	{
		title = uiMgr->GetResourceString(L"FontNameStrTitle");
	}
	HideFontList();
	uiMgr->SetFreeTextPropertyShow(false, title);
}

bool foxit::rdk::uiextensions::FreeTextProperty::IsHideFontNameList()
{
	if (gridFontName->Visibility == Windows::UI::Xaml::Visibility::Visible)
	{
		return true;
	}
	return false;
}

void foxit::rdk::uiextensions::FreeTextProperty::tbFontSize_Tapped(Platform::Object^ sender, Windows::UI::Xaml::Input::TappedRoutedEventArgs^ e)
{
	UIExtensionsManager^ uiMgr = (UIExtensionsManager^)View->ExtensionsManager;
	String^ title = uiMgr->GetResourceString(L"FontSizeStrTitle");
	uiMgr->SetFreeTextPropertyShow(true, title);

	fontPanel->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
	gridFontName->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
	gridFontSize->Visibility = Windows::UI::Xaml::Visibility::Visible;

	if (Windows::System::Profile::AnalyticsInfo::VersionInfo->DeviceFamily == "Windows.Mobile")
	{
		fontSizePanel->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
	}
}


void foxit::rdk::uiextensions::FreeTextProperty::fontSizeList_ItemClick(Platform::Object^ sender, Windows::UI::Xaml::Controls::ItemClickEventArgs^ e)
{
	if (!View)
		return;

	FontSizeItem^ item = dynamic_cast<FontSizeItem^>(e->ClickedItem);
	int fontSize = item->FontSizeValue;

	UIExtensionsManager^ uiMgr = (UIExtensionsManager^)View->ExtensionsManager;
	uiMgr->getAnnotMgr()->GetAnnotSetting()->SetFreeTextFontSize(fontSize);
	if (uiMgr->getAnnotMgr()->GetSelectedAnnot())
	{
		FreeTextAnnot^ annot = dynamic_cast<FreeTextAnnot^>(uiMgr->getAnnotMgr()->GetSelectedAnnot());

		if (annot->FontSize != fontSize)
		{
			auto rect = annot->Page_HightLightRect;
			annot->FontSize = fontSize;
			annot->IsModifyProperty = true;
			uiMgr->UpdateFreeTextAnnotOverlayControl(annot->PageIndex, annot->Page_HightLightRect);
		}
	}
	else
	{
		IBaseAnnotEventPeer^ annotEventPeer = uiMgr->getAnnotMgr()->GetAnnotHandlerByType(AnnotationType::FreeText)->AnnotEvent;
		FreeTextEventPeer^ freetextEventPeer = dynamic_cast<FreeTextEventPeer^>(annotEventPeer);
		if (freetextEventPeer)
			freetextEventPeer->UpdatFreeTextBoxProperty();
	}

	tbFontSize->Content = fontSize.ToString();

	HideFontList();
	String^ title = uiMgr->GetResourceString(L"FontSizeStrTitle");
	uiMgr->SetFreeTextPropertyShow(false, title);
}

void FreeTextProperty::HideFontList()
{
	fontPanel->Visibility = Windows::UI::Xaml::Visibility::Visible;
	gridFontName->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
	gridFontSize->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
}


void foxit::rdk::uiextensions::FreeTextProperty::OnLoaded(Platform::Object ^sender, Windows::UI::Xaml::RoutedEventArgs ^e)
{
	__windowSizeChangedEventRegistrationToken = Window::Current->SizeChanged += ref new Windows::UI::Xaml::WindowSizeChangedEventHandler(this, &foxit::rdk::uiextensions::FreeTextProperty::OnSizeChanged);
}


void foxit::rdk::uiextensions::FreeTextProperty::OnUnloaded(Platform::Object ^sender, Windows::UI::Xaml::RoutedEventArgs ^e)
{
	if (__windowSizeChangedEventRegistrationToken.Value)
	{
		Window::Current->SizeChanged -= __windowSizeChangedEventRegistrationToken;
		__windowSizeChangedEventRegistrationToken.Value = 0;
	}
}


void foxit::rdk::uiextensions::FreeTextProperty::OnSizeChanged(Platform::Object ^sender, Windows::UI::Core::WindowSizeChangedEventArgs ^e)
{
	if (Windows::System::Profile::AnalyticsInfo::VersionInfo->DeviceFamily == "Windows.Mobile")
	{
		auto winrc = Windows::UI::Xaml::Window::Current->Bounds;
		if (winrc.Width < winrc.Height)
		{
			tbFontName->Width = winrc.Width*0.45;
		}
		else
		{
			tbFontName->Width = winrc.Width*0.45;
		}

		gridFreeText->Width = winrc.Width;
		fontNameList->Width = winrc.Width;
		fontSizeList->Width = winrc.Width;
		gridFontName->Width = winrc.Width;
		gridFontSize->Width = winrc.Width;
	}
}

void foxit::rdk::uiextensions::FreeTextProperty::ResetFontValue()
{
	UIExtensionsManager^ uiMgr = (UIExtensionsManager^)View->ExtensionsManager;
	IBaseAnnot^ annot = uiMgr->getAnnotMgr()->GetSelectedAnnot();
	if (!annot)
	{
		tbFontName->Content = uiMgr->getAnnotMgr()->GetAnnotSetting()->GetFreeTextFontName();
		tbFontSize->Content = uiMgr->getAnnotMgr()->GetAnnotSetting()->GetFreeTextFontSize().ToString();
	}
	else
	{
		FreeTextAnnot^ freeTextannot = dynamic_cast<FreeTextAnnot^>(annot);
		if (freeTextannot)
		{
			tbFontName->Content = freeTextannot->FontName ? freeTextannot->FontName : uiMgr->getAnnotMgr()->GetAnnotSetting()->GetFreeTextFontName();
			int fontSize = (int)freeTextannot->FontSize;
			tbFontSize->Content = uiMgr->getAnnotMgr()->GetAnnotSetting()->GetFreeTextFontSize().ToString();
		}
	}
}

bool foxit::rdk::uiextensions::FreeTextProperty::IsFontPanelIsVisible()
{
	if (fontPanel->Visibility == Windows::UI::Xaml::Visibility::Visible)
		return true;
	return false;
}

bool foxit::rdk::uiextensions::FreeTextProperty::IsFontNameListIsVisible()
{
	if (gridFontName->Visibility == Windows::UI::Xaml::Visibility::Visible)
		return true;
	return false;
}

bool foxit::rdk::uiextensions::FreeTextProperty::IsFontSizeListIsVisible()
{
	if (gridFontSize->Visibility == Windows::UI::Xaml::Visibility::Visible)
		return true;
	return false;
}

void foxit::rdk::uiextensions::FreeTextProperty::SetFontPanelVisible()
{
	HideFontList();
	UIExtensionsManager^ uiMgr = (UIExtensionsManager^)View->ExtensionsManager;
	uiMgr->SetFreeTextPropertyShow(false, "");
}

void foxit::rdk::uiextensions::FreeTextProperty::SetFontNameListVisible()
{
	fontPanel->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
	gridFontName->Visibility = Windows::UI::Xaml::Visibility::Visible;
	gridFontSize->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
	UIExtensionsManager^ uiMgr = (UIExtensionsManager^)View->ExtensionsManager;
	String^ title = uiMgr->GetResourceString(L"FontNameStrTitle");
	uiMgr->SetFreeTextPropertyShow(true, title);
	if (Windows::System::Profile::AnalyticsInfo::VersionInfo->DeviceFamily == "Windows.Mobile")
	{
		fontNamePanel->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
	}
}

void foxit::rdk::uiextensions::FreeTextProperty::SetFontSizeListVisible()
{
	fontPanel->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
	gridFontName->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
	gridFontSize->Visibility = Windows::UI::Xaml::Visibility::Visible;
	if (Windows::System::Profile::AnalyticsInfo::VersionInfo->DeviceFamily == "Windows.Mobile")
	{
		fontSizePanel->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
	}

	UIExtensionsManager^ uiMgr = (UIExtensionsManager^)View->ExtensionsManager;
	String^ title = uiMgr->GetResourceString(L"FontSizeStrTitle");
	uiMgr->SetFreeTextPropertyShow(true, title);
}

void foxit::rdk::uiextensions::FreeTextProperty::ResetFontNameSizeListWidth()
{
	if (Windows::System::Profile::AnalyticsInfo::VersionInfo->DeviceFamily == "Windows.Mobile")
	{
		auto winrc = Windows::UI::Xaml::Window::Current->Bounds;
		if (winrc.Width < winrc.Height)
		{
			tbFontName->Width = winrc.Width*0.45;
		}
		else
		{
			tbFontName->Width = winrc.Width*0.45;
		}

		gridFreeText->Width = winrc.Width;
		fontNameList->Width = winrc.Width;
		fontSizeList->Width = winrc.Width;
		gridFontName->Width = winrc.Width;
		gridFontSize->Width = winrc.Width;
	}
}