/**
 * Copyright (C) 2003-2018, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit Mobile PDF SDK to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */


#include "pch.h"
#include "HighlightAnnotEventPeer.h"


using namespace foxit::rdk;
using namespace foxit::rdk::uiextensions;

using namespace Platform;
using namespace Windows::Foundation;

namespace foxit
{
	namespace rdk
	{
		namespace uiextensions
		{
			[Windows::Foundation::Metadata::WebHostHidden]
			public ref class CreateHightLightAnnotUndoAction sealed : IUndoItem
			{
			public:
				CreateHightLightAnnotUndoAction(PDFViewCtrl^ view, HighLightAnnot^ annot);

				virtual void Undo();
				virtual void Redo();
				virtual Platform::String^ GetDescr();
				virtual void ReleaseItem();

			internal:
				property PDFViewCtrl^ View
				{
					PDFViewCtrl^ get();
				}

				property HighLightAnnot^ AnnotHandle
				{
					HighLightAnnot^ get();
				}

			private:
				Platform::WeakReference m_ViewWeakRef;
				Platform::WeakReference m_AnnotWeakRef;
				BaseAnnot^ m_RedoAnnot;
			};
		}
	}
}

CreateHightLightAnnotUndoAction::CreateHightLightAnnotUndoAction(PDFViewCtrl^ view, HighLightAnnot^ annot)
{
	m_ViewWeakRef = view;
	m_AnnotWeakRef = annot;
}

PDFViewCtrl^ CreateHightLightAnnotUndoAction::View::get()
{
	return m_ViewWeakRef.Resolve<PDFViewCtrl>();
}

HighLightAnnot^ CreateHightLightAnnotUndoAction::AnnotHandle::get()
{
	return m_AnnotWeakRef.Resolve<HighLightAnnot>();
}

void CreateHightLightAnnotUndoAction::Undo()
{
	if (View && AnnotHandle)
	{
		m_RedoAnnot = AnnotHandle;
		View->CancelRenderTask();
		AnnotHandle->OnDelete();
	}
}

void CreateHightLightAnnotUndoAction::Redo()
{
	if (View && m_RedoAnnot)
	{
		auto _page = View->GetDoc()->GetPage(m_RedoAnnot->PageIndex);
		View->CancelRenderTask();
		UIExtensionsManager^ uiMgr = (UIExtensionsManager^)View->ExtensionsManager;
		uiMgr->getAnnotMgr()->OnCreateNewAnnot(_page, m_RedoAnnot);
	}
}

Platform::String^ CreateHightLightAnnotUndoAction::GetDescr()
{
	return ref new Platform::String(L"");
}

void CreateHightLightAnnotUndoAction::ReleaseItem()
{
	m_ViewWeakRef = nullptr;
	m_AnnotWeakRef = nullptr;
	m_RedoAnnot = nullptr;
}



HighlightAnnotEventPeer::HighlightAnnotEventPeer(PDFViewCtrl^ view)
{
	m_PageViewWeakRef = view;
	m_TextSelectContext = ref new TextSelectContext();
	m_pressStartPoint = Point(-1, -1);
}

PDFViewCtrl^ HighlightAnnotEventPeer::View::get()
{
	return m_PageViewWeakRef.Resolve<PDFViewCtrl>();
}

void HighlightAnnotEventPeer::OnPointerPressed(Platform::Object^ sender, Windows::UI::Xaml::Input::PointerRoutedEventArgs^ e)
{
	Windows::UI::Xaml::FrameworkElement^ _FrameworkElement = dynamic_cast<Windows::UI::Xaml::FrameworkElement^>(sender);
	Windows::Foundation::Point _Pt;
	auto _PointerPt = e->GetCurrentPoint(_FrameworkElement);
	_Pt = _PointerPt->Position;
	auto _Pageinfo = View->ConvertDevicePtToPDFPagePt(_Pt);
	int index = _Pageinfo->PageIndex;
	UIExtensionsManager^ uiMgr = (UIExtensionsManager^)View->ExtensionsManager;
	if (uiMgr->CreateTypeOfAnnot == AnnotCreateType::AreaHighLight)
	{
		if (_Pageinfo->PagePoint.X != -1 && _Pageinfo->PagePoint.Y != -1)
		{
			m_pressStartPoint = _Pt;
			m_StartIndex = index;
		}
	}
	else
	{
		auto _PagePoint = _Pageinfo->PagePoint;
		m_TextSelectContext->StartIndex = index;
		m_TextSelectContext->StartPoint = _PagePoint;
	}
	_FrameworkElement->ManipulationMode = Windows::UI::Xaml::Input::ManipulationModes::System;
}

void HighlightAnnotEventPeer::OnPointerMoved(Platform::Object^ sender, Windows::UI::Xaml::Input::PointerRoutedEventArgs^ e)
{
	UIExtensionsManager^ uiMgr = (UIExtensionsManager^)View->ExtensionsManager;
	if (uiMgr->CreateTypeOfAnnot == AnnotCreateType::AreaHighLight && m_pressStartPoint.X != -1 && m_pressStartPoint.Y != -1)
	{
		Windows::UI::Xaml::FrameworkElement^ _FrameworkElement = dynamic_cast<Windows::UI::Xaml::FrameworkElement^>(sender);
		auto currentPoint = e->GetCurrentPoint(_FrameworkElement)->Position;
		double width = 0;
		double height = 0;
		Windows::Foundation::Point  startPoint = m_pressStartPoint;

		Windows::Foundation::Rect pageViewRect = View->GetPageViewRect(m_StartIndex);
		if (currentPoint.Y > pageViewRect.Y + pageViewRect.Height)
		{
			currentPoint.Y = pageViewRect.Y + pageViewRect.Height;
		}
		else if (currentPoint.Y < pageViewRect.Y)
		{
			currentPoint.Y = pageViewRect.Y;
		}

		if (currentPoint.X > pageViewRect.X + pageViewRect.Width)
		{
			currentPoint.X = pageViewRect.X + pageViewRect.Width;
		}
		else if (currentPoint.X < pageViewRect.X)
		{
			currentPoint.X = pageViewRect.X;
		}


		if (currentPoint.X > m_pressStartPoint.X)
		{
			width = currentPoint.X - m_pressStartPoint.X;
		}
		else
		{
			width = m_pressStartPoint.X - currentPoint.X;
			startPoint.X = currentPoint.X;
		}
		if (currentPoint.Y > m_pressStartPoint.Y)
		{
			height = currentPoint.Y - m_pressStartPoint.Y;
		}
		else
		{
			height = m_pressStartPoint.Y - currentPoint.Y;
			startPoint.Y = currentPoint.Y;
		}

		m_AreaRect = Rect(startPoint.X, startPoint.Y, (float)width, (float)height);
		uiMgr->getAnnotMgr()->SetNewCreateAnnot(m_AreaRect);
		View->Update();
	}
	else
	{
		auto _selectedtext = uiMgr->getTextMgr()->SelectedText;
		if (_selectedtext && _selectedtext->Size > 0)
		{
			int size = _selectedtext->Size;
		}
	}
}

void HighlightAnnotEventPeer::OnPointerReleased(Platform::Object^ sender, Windows::UI::Xaml::Input::PointerRoutedEventArgs^ e)
{
	Windows::UI::Xaml::FrameworkElement^ _FrameworkElement = dynamic_cast<Windows::UI::Xaml::FrameworkElement^>(sender);
	Windows::Foundation::Point _Pt;
	auto _PointerPt = e->GetCurrentPoint(_FrameworkElement);
	_Pt = _PointerPt->Position;
	auto _Pageinfo = View->ConvertDevicePtToPDFPagePt(_Pt);
	int index = _Pageinfo->PageIndex;
	auto _PagePoint = _Pageinfo->PagePoint;
	m_TextSelectContext->EndIndex = index;
	m_TextSelectContext->EndPoint = _PagePoint;

	UIExtensionsManager^ uiMgr = (UIExtensionsManager^)View->ExtensionsManager;
	if (!uiMgr->getAnnotMgr())
		return;

	auto _AnnotSetting = uiMgr->getAnnotMgr()->GetAnnotSetting();

	if (uiMgr->CreateTypeOfAnnot == AnnotCreateType::AreaHighLight && m_pressStartPoint.X != -1 && m_pressStartPoint.Y != -1)
	{
		m_pressStartPoint = Point(-1, -1);
		if (m_AreaRect.Width > 0 && m_AreaRect.Height > 0)
		{
			auto rectGroup = ref new Platform::Collections::Vector<Windows::Foundation::Rect>();
			auto deviceRc = View->ConvertDeviceRectToPageRect(m_StartIndex, m_AreaRect);
			rectGroup->Append(deviceRc);

			auto _Annot = foxit::rdk::uiextensions::AnnotCreaterFactory::GetInstance()->CreateNewAnnot(AnnotationType::HightLight);
			auto _pCreateAnnot = dynamic_cast<HighLightAnnot^>(_Annot);
			_pCreateAnnot->Color = _AnnotSetting->GetHightLightAnnotColor();
			_pCreateAnnot->Opacity = _AnnotSetting->GetHightLightAnnotOpacity();
			_pCreateAnnot->Page_HightLightRectGroup = rectGroup;
			_pCreateAnnot->Name = _AnnotSetting->GetNoteAuthor();
			_pCreateAnnot->Title = _AnnotSetting->GetNoteAuthor();
			_pCreateAnnot->CreationDateTime = (ref new Windows::Globalization::Calendar())->GetDateTime();
			auto _selectpageindex = m_StartIndex;
			auto _page = View->GetDoc()->GetPage(_selectpageindex);

			uiMgr->getAnnotMgr()->OnCreateNewAnnot(_page, _pCreateAnnot);
			auto pUndoItem = ref new CreateHightLightAnnotUndoAction(View, _pCreateAnnot);
			UndoFactory::GetUndoInstance()->AddUndoItem(uiMgr, pUndoItem);

			Windows::Foundation::Rect rc= _pCreateAnnot->MinRect;
			foxit::common::fxcrt::RectF^ minRect = FoxitRectConverter::ConvertToFoxitRectF(_pCreateAnnot->MinRect);
			String^ rectStr = "[" + minRect->left.ToString() + " " + minRect->top.ToString() + " " + minRect->right.ToString() + " " + minRect->bottom.ToString() + "]";
			uiMgr->CollectAnnotOperation("AreaHighlight", "Add", _selectpageindex.ToString(), rectStr);
			

			if (!_AnnotSetting->GetAreaHightLightIsContinueOpera() || uiMgr->IsRightMenuAdd)
			{
				uiMgr->ExitAnnotPropertyMode();
				uiMgr->CreateTypeOfAnnot = AnnotCreateType::None;
				uiMgr->IsRightMenuAdd = false;
			}
			else
			{
				uiMgr->getAnnotMgr()->SetNewCreateAnnot(Windows::Foundation::Rect(-1, -1, 0, 0));
			}
			m_AreaRect =Windows::Foundation::Rect(-1,-1,0,0);
			uiMgr->getAnnotMgr()->SetNewCreateAnnot(m_AreaRect);
		}
	}
	else
	{
		auto _selectedtext = uiMgr->getTextMgr()->SelectedText;
		if (_selectedtext && _selectedtext->Size > 0)
		{
			auto size = _selectedtext->Size;
			bool _bIsClearText = true;

			Windows::Foundation::Rect rect = _selectedtext->GetAt(0)->TextRectVect->GetAt(0);
			for (unsigned int i = 0; i < size; i++)
			{
				auto rectGroup = ref new Platform::Collections::Vector<Windows::Foundation::Rect>();

				for each (Windows::Foundation::Rect gmt in _selectedtext->GetAt(i)->TextRectVect)
				{
					rectGroup->Append(gmt);
				}

				auto _Annot = foxit::rdk::uiextensions::AnnotCreaterFactory::GetInstance()->CreateNewAnnot(AnnotationType::HightLight);
				auto _pCreateAnnot = dynamic_cast<HighLightAnnot^>(_Annot);
				_pCreateAnnot->Color = _AnnotSetting->GetHightLightAnnotColor();
				_pCreateAnnot->Opacity = _AnnotSetting->GetHightLightAnnotOpacity();
				_pCreateAnnot->Page_HightLightRectGroup = rectGroup;
				_pCreateAnnot->Name = _AnnotSetting->GetNoteAuthor();
				_pCreateAnnot->Title = _AnnotSetting->GetNoteAuthor();
				_pCreateAnnot->CreationDateTime = (ref new Windows::Globalization::Calendar())->GetDateTime();
				auto _selectpageindex = _selectedtext->GetAt(i)->PageIndex;
				auto _page = View->GetDoc()->GetPage(_selectpageindex);

				auto _createtype = uiMgr->CreateTypeOfAnnot;

				if (_createtype == AnnotCreateType::Comment)
				{
					if (index == _selectpageindex)
					{
						_bIsClearText = false;
						AnnotCommandToken^ token = ref new AnnotCommandToken();
						token->Pageindex = index;
						token->Param = _pCreateAnnot;
						token->Type = AnnotCommandType::Add;

					}
					else
					{
						uiMgr->getAnnotMgr()->OnCreateNewAnnot(_page, _pCreateAnnot);
					}
				}
				else if (_createtype == AnnotCreateType::HightLight)
				{
					uiMgr->getAnnotMgr()->OnCreateNewAnnot(_page, _pCreateAnnot);
					auto pUndoItem = ref new CreateHightLightAnnotUndoAction(View, _pCreateAnnot);
					UndoFactory::GetUndoInstance()->AddUndoItem(uiMgr, pUndoItem);


					Windows::Foundation::Rect rc = _pCreateAnnot->MinRect;
					foxit::common::fxcrt::RectF^ minRect = FoxitRectConverter::ConvertToFoxitRectF(_pCreateAnnot->MinRect);
					String^ rectStr = "[" + minRect->left.ToString() + " " + minRect->top.ToString() + " " + minRect->right.ToString() + " " + minRect->bottom.ToString() + "]";
					uiMgr->CollectAnnotOperation("Highlight", "Add", _selectpageindex.ToString(), rectStr);
				}
			}

			if (_bIsClearText)
				uiMgr->getTextMgr()->ClearTextSelect();

			if (!_AnnotSetting->GetHightLightIsContinueOpera() || uiMgr->IsRightMenuAdd)
			{
				uiMgr->ExitAnnotPropertyMode();
				uiMgr->CreateTypeOfAnnot = AnnotCreateType::None;
				uiMgr->IsRightMenuAdd = false;
			}
		}
	}

	m_TextSelectContext->ResetContext();

	_FrameworkElement->ManipulationMode = Windows::UI::Xaml::Input::ManipulationModes::All;
}

void HighlightAnnotEventPeer::OnTapped(Platform::Object^ sender, Windows::UI::Xaml::Input::TappedRoutedEventArgs^ e)
{
	UIExtensionsManager^ uiMgr = (UIExtensionsManager^)View->ExtensionsManager;
	auto _SelectedAnnot = uiMgr->getAnnotMgr()->GetSelectedAnnot();
	if (_SelectedAnnot)
	{
		HighLightAnnot^ annot = dynamic_cast<HighLightAnnot^>(_SelectedAnnot);
		Windows::Foundation::Rect rect = annot->MinRect;
		rect.X = rect.X - 5;
		rect.Y = rect.Y - 5;
		rect.Width = rect.Width + 5 * 2;
		rect.Height = rect.Height + 5 * 2;
		auto deviceRect = View->ConvertPageRectToDeviceRect(uiMgr->getAnnotMgr()->GetSelectedAnnot()->PageIndex, rect);
		Point pt;

		pt = Point(deviceRect.X + deviceRect.Width / 2, deviceRect.Y);


		if (View->GetDoc() && !FSPDFDocUtil::GetIsDocumentCanModifyAnnotation(View->GetDoc()) && View->GetDoc()->GetPasswordType() != foxit::pdf::PasswordType::e_PwdOwner)
		{
			if (_SelectedAnnot->Contents->Length() > 0)
			{
				uiMgr->ShowTextmarkContextmenu(AnnotContextMenuType::Note, pt);
			}
		}
		else
		{
			uiMgr->ShowTextmarkContextmenu(AnnotContextMenuType::Edit, pt);
		}
	}
}

void HighlightAnnotEventPeer::OnRightTapped(Platform::Object^ sender, Windows::UI::Xaml::Input::RightTappedRoutedEventArgs^ e)
{

}

void HighlightAnnotEventPeer::OnManipulationStarted(Platform::Object^ sender, Windows::UI::Xaml::Input::ManipulationStartedRoutedEventArgs^ e)
{

}

void HighlightAnnotEventPeer::OnManipulationDelta(Platform::Object^ sender, Windows::UI::Xaml::Input::ManipulationDeltaRoutedEventArgs^ e)
{

}

void HighlightAnnotEventPeer::OnManipulationCompleted(Platform::Object^ sender, Windows::UI::Xaml::Input::ManipulationCompletedRoutedEventArgs^ e)
{

}

void HighlightAnnotEventPeer::OnCreateAnnotAfterSelectText()
{
	UIExtensionsManager^ uiMgr = (UIExtensionsManager^)View->ExtensionsManager;
	auto _AnnotSetting = uiMgr->getAnnotMgr()->GetAnnotSetting();
	auto _selectedtext = uiMgr->getTextMgr()->SelectedText;
	if (_selectedtext && _selectedtext->Size > 0)
	{
		if (!View->GetDoc())
			return;

		auto size = _selectedtext->Size;
		bool _bIsClearText = true;
		Windows::Foundation::Rect rect = _selectedtext->GetAt(0)->TextRectVect->GetAt(0);
		for (unsigned int i = 0; i < size; i++)
		{
			auto rectGroup = ref new Platform::Collections::Vector<Windows::Foundation::Rect>();

			for each (Windows::Foundation::Rect gmt in _selectedtext->GetAt(i)->TextRectVect)
			{
				rectGroup->Append(gmt);
			}

			auto _Annot = foxit::rdk::uiextensions::AnnotCreaterFactory::GetInstance()->CreateNewAnnot(AnnotationType::HightLight);
			auto _pCreateAnnot = dynamic_cast<HighLightAnnot^>(_Annot);
			_pCreateAnnot->Color = _AnnotSetting->GetHightLightAnnotColor();
			_pCreateAnnot->Opacity = _AnnotSetting->GetHightLightAnnotOpacity();
			_pCreateAnnot->Page_HightLightRectGroup = rectGroup;
			_pCreateAnnot->Name = uiMgr->getAnnotMgr()->GetAnnotSetting()->GetNoteAuthor();
			_pCreateAnnot->Title = _AnnotSetting->GetNoteAuthor();
			_pCreateAnnot->CreationDateTime = (ref new Windows::Globalization::Calendar())->GetDateTime();
			auto _selectpageindex = _selectedtext->GetAt(i)->PageIndex;
			auto _page = View->GetDoc()->GetPage(_selectpageindex);

			auto _createtype = uiMgr->CreateTypeOfAnnot;

			if (_createtype == AnnotCreateType::HightLight)
			{
				uiMgr->getAnnotMgr()->OnCreateNewAnnot(_page, _pCreateAnnot);
				auto pUndoItem = ref new CreateHightLightAnnotUndoAction(View, _pCreateAnnot);
				UndoFactory::GetUndoInstance()->AddUndoItem(uiMgr, pUndoItem);
			}
			foxit::common::fxcrt::RectF^ minRect = FoxitRectConverter::ConvertToFoxitRectF(_pCreateAnnot->MinRect);
			String^ rectStr = "[" + minRect->left.ToString() + " " + minRect->top.ToString() + " " + minRect->right.ToString() + " " + minRect->bottom.ToString() + "]";
			uiMgr->CollectAnnotOperation("Highlight", "Add", _selectpageindex.ToString(), rectStr);

		}
		
		if (_bIsClearText)
			uiMgr->getTextMgr()->ClearTextSelect();
		uiMgr->CreateTypeOfAnnot = AnnotCreateType::None;
	}
	else if (uiMgr->CreateTypeOfAnnot == AnnotCreateType::AreaHighLight && m_pressStartPoint.X != -1 && m_pressStartPoint.Y != -1)
	{
		if (m_AreaRect.Width > 0 && m_AreaRect.Height > 0)
		{			
			auto rectGroup = ref new Platform::Collections::Vector<Windows::Foundation::Rect>();
			auto deviceRc = View->ConvertDeviceRectToPageRect(m_StartIndex, m_AreaRect);
			rectGroup->Append(deviceRc);

			auto _Annot = foxit::rdk::uiextensions::AnnotCreaterFactory::GetInstance()->CreateNewAnnot(AnnotationType::HightLight);
			auto _pCreateAnnot = dynamic_cast<HighLightAnnot^>(_Annot);
			_pCreateAnnot->Color = _AnnotSetting->GetHightLightAnnotColor();
			_pCreateAnnot->Opacity = _AnnotSetting->GetHightLightAnnotOpacity();
			_pCreateAnnot->Page_HightLightRectGroup = rectGroup;
			_pCreateAnnot->Name = _AnnotSetting->GetNoteAuthor();
			_pCreateAnnot->Title = _AnnotSetting->GetNoteAuthor();
			_pCreateAnnot->CreationDateTime = (ref new Windows::Globalization::Calendar())->GetDateTime();
			auto _selectpageindex = m_StartIndex;
			auto _page = View->GetDoc()->GetPage(_selectpageindex);

			uiMgr->getAnnotMgr()->OnCreateNewAnnot(_page, _pCreateAnnot);
			auto pUndoItem = ref new CreateHightLightAnnotUndoAction(View, _pCreateAnnot);
			UndoFactory::GetUndoInstance()->AddUndoItem(uiMgr, pUndoItem);

			Windows::Foundation::Rect rc = _pCreateAnnot->MinRect;
			foxit::common::fxcrt::RectF^ minRect = FoxitRectConverter::ConvertToFoxitRectF(_pCreateAnnot->MinRect);
			String^ rectStr = "[" + minRect->left.ToString() + " " + minRect->top.ToString() + " " + minRect->right.ToString() + " " + minRect->bottom.ToString() + "]";
			uiMgr->CollectAnnotOperation("AreaHighlight", "Add", _selectpageindex.ToString(), rectStr);

			if (!_AnnotSetting->GetAreaHightLightIsContinueOpera() || uiMgr->IsRightMenuAdd)
			{
				uiMgr->ExitAnnotPropertyMode();
				uiMgr->CreateTypeOfAnnot = AnnotCreateType::None;
				uiMgr->IsRightMenuAdd = false;
			}
			else
			{
				uiMgr->getAnnotMgr()->SetNewCreateAnnot(Windows::Foundation::Rect(-1, -1, 0, 0));
			}
			m_AreaRect = Windows::Foundation::Rect(-1, -1, 0, 0);
			uiMgr->getAnnotMgr()->SetNewCreateAnnot(m_AreaRect);
		}
	}

	m_TextSelectContext->ResetContext();
}

void HighlightAnnotEventPeer::OnManipulationInertiaStarting(Platform::Object^ sender, Windows::UI::Xaml::Input::ManipulationInertiaStartingRoutedEventArgs^ e)
{

}
