/**
 * Copyright (C) 2003-2018, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit Mobile PDF SDK to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */


#include "pch.h"
#include "InkAnnotEventPeer.h"

using namespace foxit::rdk;
using namespace foxit::rdk::uiextensions;

using namespace Platform;
using namespace Windows::Foundation;

namespace foxit
{
	namespace rdk
	{
		namespace uiextensions
		{
			[Windows::Foundation::Metadata::WebHostHidden]
			public ref class CreateInkAnnotUndoAction sealed : IUndoItem
			{
			public:
				CreateInkAnnotUndoAction(PDFViewCtrl^ view, PencilAnnot^ annot);

				virtual void Undo();
				virtual void Redo();
				virtual Platform::String^ GetDescr();
				virtual void ReleaseItem();

			internal:
				property PDFViewCtrl^ View
				{
					PDFViewCtrl^ get();
				}

				property PencilAnnot^ AnnotHandle
				{
					PencilAnnot^ get();
				}

			private:
				Platform::WeakReference m_ViewWeakRef;
				Platform::WeakReference m_AnnotWeakRef;
				BaseAnnot^ m_RedoAnnot;
			};
			[Windows::Foundation::Metadata::WebHostHidden]
			public ref class DrawInkAnnotUndoAction sealed : IUndoItem
			{
			public:
				DrawInkAnnotUndoAction(PDFViewCtrl^ view, Windows::UI::Xaml::Media::PointCollection^ pointCollection, Windows::Foundation::Rect rect, int pageIndex);

				virtual void Undo();
				virtual void Redo();
				virtual Platform::String^ GetDescr();
				virtual void ReleaseItem();

			internal:
				property PDFViewCtrl^ View
				{
					PDFViewCtrl^ get();
				}

			private:
				Platform::WeakReference m_ViewWeakRef;
				Windows::UI::Xaml::Media::PointCollection^ m_PolyLinePointsVector;
				Windows::Foundation::Rect pageViewRect;
				int m_pageIndex;
			};
		}
	}
}

CreateInkAnnotUndoAction::CreateInkAnnotUndoAction(PDFViewCtrl^ view, PencilAnnot^ annot)
{
	m_ViewWeakRef = view;
	m_AnnotWeakRef = annot;
}

PDFViewCtrl^ CreateInkAnnotUndoAction::View::get()
{
	return m_ViewWeakRef.Resolve<PDFViewCtrl>();
}

PencilAnnot^ CreateInkAnnotUndoAction::AnnotHandle::get()
{
	return m_AnnotWeakRef.Resolve<PencilAnnot>();
}

void CreateInkAnnotUndoAction::Undo()
{
	if (View && AnnotHandle)
	{
		m_RedoAnnot = AnnotHandle;
		View->CancelRenderTask();
		AnnotHandle->OnDelete();
	}
}

void CreateInkAnnotUndoAction::Redo()
{
	if (View && m_RedoAnnot)
	{
		auto _page = View->GetDoc()->GetPage(m_RedoAnnot->PageIndex);

		View->CancelRenderTask();
		UIExtensionsManager^ uiMgr = (UIExtensionsManager^)View->ExtensionsManager;
		uiMgr->getAnnotMgr()->OnCreateNewAnnot(_page, m_RedoAnnot);
	}
}

Platform::String^ CreateInkAnnotUndoAction::GetDescr()
{
	return ref new Platform::String(L"");
}

void CreateInkAnnotUndoAction::ReleaseItem()
{
	m_ViewWeakRef = nullptr;
	m_AnnotWeakRef = nullptr;
	m_RedoAnnot = nullptr;
}


DrawInkAnnotUndoAction::DrawInkAnnotUndoAction(PDFViewCtrl^ view, Windows::UI::Xaml::Media::PointCollection^ pointCollection, Windows::Foundation::Rect rect, int pageIndex)
{
	m_ViewWeakRef = view;
	pageViewRect = rect;
	m_pageIndex = pageIndex;
	m_PolyLinePointsVector = ref new Windows::UI::Xaml::Media::PointCollection();
	for (unsigned int i = 0; i < pointCollection->Size; i++)
	{
		m_PolyLinePointsVector->Append(pointCollection->GetAt(i));
	}
}

PDFViewCtrl^ DrawInkAnnotUndoAction::View::get()
{
	return m_ViewWeakRef.Resolve<PDFViewCtrl>();
}


void DrawInkAnnotUndoAction::Undo()
{
	if (View)
	{	
		UIExtensionsManager^ uiMgr = (UIExtensionsManager^)View->ExtensionsManager;
		Windows::UI::Xaml::Media::PointCollection^ ptColleciton = uiMgr->RemoveDrawPolyLine();
		m_PolyLinePointsVector = ref new Windows::UI::Xaml::Media::PointCollection();

		for (unsigned int i = 0; i < ptColleciton->Size; i++)
		{
			m_PolyLinePointsVector->Append(ptColleciton->GetAt(i));
		}
	}
}

void DrawInkAnnotUndoAction::Redo()
{
	if (View && m_PolyLinePointsVector->Size>0)
	{
		UIExtensionsManager^ uiMgr = (UIExtensionsManager^)View->ExtensionsManager;
		auto _AnnotSetting = uiMgr->getAnnotMgr()->GetAnnotSetting();
		int borderWidth = _AnnotSetting->GetAnnotDrawBorderWidth();

		Windows::UI::Xaml::Shapes::Polyline^ m_PolyLine = ref new Windows::UI::Xaml::Shapes::Polyline();
		m_PolyLine->Stroke = ref new Windows::UI::Xaml::Media::SolidColorBrush(_AnnotSetting->GetAnnotColor());
		m_PolyLine->Opacity = _AnnotSetting->GetAnnotOpacity();
		
		int _borderWidth = (int)FSPDFDocUtil::PixelToPDFPoint(View->GetDoc(), m_pageIndex, borderWidth);
		double pixelThickness = FSPDFDocUtil::PDFPointToPixel(View->GetDoc(), m_pageIndex, (float)_borderWidth, pageViewRect.Width, pageViewRect.Height);

		m_PolyLine->StrokeThickness = pixelThickness;
		m_PolyLine->StrokeStartLineCap = Windows::UI::Xaml::Media::PenLineCap::Round;
		m_PolyLine->StrokeEndLineCap = Windows::UI::Xaml::Media::PenLineCap::Round;
		m_PolyLine->StrokeLineJoin = Windows::UI::Xaml::Media::PenLineJoin::Round;
		m_PolyLine->StrokeDashCap = Windows::UI::Xaml::Media::PenLineCap::Round;

		uiMgr->CreateNewPolyLineToCanvas(m_PolyLine);

		int size = m_PolyLinePointsVector->Size;

		m_PolyLine->Points = m_PolyLinePointsVector;
	}
}

Platform::String^ DrawInkAnnotUndoAction::GetDescr()
{
	return ref new Platform::String(L"");
}

void DrawInkAnnotUndoAction::ReleaseItem()
{
	m_ViewWeakRef = nullptr;
}


InkAnnotEventPeer::InkAnnotEventPeer(PDFViewCtrl^ view)
{
	m_PageViewWeakRef = view;
	m_pressStartPoint = Point(-1, -1);
	polyLinePointsVector = ref new  Platform::Collections::Vector<Windows::UI::Xaml::Media::PointCollection^>();
	m_isPressed = false;
	m_isBeginCreate = true;
	drawInkActionObj= ref new  Platform::Collections::Vector<Platform::Object^>();
}


PDFViewCtrl^ InkAnnotEventPeer::View::get()
{
	return m_PageViewWeakRef.Resolve<PDFViewCtrl>();
}

void InkAnnotEventPeer::OnPointerPressed(Platform::Object^ sender, Windows::UI::Xaml::Input::PointerRoutedEventArgs^ e)
{
	if (!View)
		return;

	if (m_isPressed)
		return;

	UIExtensionsManager^ uiMgr = (UIExtensionsManager^)View->ExtensionsManager;
	uiMgr->SetAnnotPropertyToolIsEnable(false);
	uiMgr->viewControl->EnableScrollView(false);

	Windows::UI::Xaml::FrameworkElement^ _FrameworkElement = dynamic_cast<Windows::UI::Xaml::FrameworkElement^>(sender);
	Windows::Foundation::Point _Pt;
	auto _PointerPt = e->GetCurrentPoint(_FrameworkElement);
	_Pt = _PointerPt->Position;

	auto _Pageinfo = View->ConvertDevicePtToPDFPagePt(_Pt);
	m_PageIndex = _Pageinfo->PageIndex;

	if (polyLinePointsVector->Size > 0)
	{
		Windows::UI::Xaml::Media::PointCollection^ pointVector = polyLinePointsVector->GetAt(polyLinePointsVector->Size-1);
		Windows::Foundation::Point pt = pointVector->GetAt(0);
		auto _PageinfoPt = View->ConvertDevicePtToPDFPagePt(pt);
		if (_PageinfoPt && _PageinfoPt->PageIndex != m_PageIndex)
		{
			OnCreateAnnotAfterSelectText();
		}
	}
	_FrameworkElement->ManipulationMode = Windows::UI::Xaml::Input::ManipulationModes::System;

	auto _AnnotSetting = uiMgr->getAnnotMgr()->GetAnnotSetting();
	borderWidth= _AnnotSetting->GetAnnotDrawBorderWidth();

	Windows::Foundation::Rect pageViewRect = View->GetPageViewRect(m_PageIndex);
	int _borderWidth = (int)FSPDFDocUtil::PixelToPDFPoint(View->GetDoc(), m_PageIndex, (int)borderWidth);
	double pixelThickness = FSPDFDocUtil::PDFPointToPixel(View->GetDoc(), m_PageIndex, (float)_borderWidth, pageViewRect.Width, pageViewRect.Height);

	polyLine = ref new Windows::UI::Xaml::Shapes::Polyline();
	polyLine->Stroke = ref new Windows::UI::Xaml::Media::SolidColorBrush(_AnnotSetting->GetAnnotColor());
	polyLine->Opacity = _AnnotSetting->GetAnnotOpacity();
	polyLine->StrokeThickness = pixelThickness;
	polyLine->StrokeStartLineCap = Windows::UI::Xaml::Media::PenLineCap::Round;
	polyLine->StrokeEndLineCap = Windows::UI::Xaml::Media::PenLineCap::Round;
	polyLine->StrokeLineJoin = Windows::UI::Xaml::Media::PenLineJoin::Round;
	polyLine->StrokeDashCap = Windows::UI::Xaml::Media::PenLineCap::Round;

	uiMgr->CreateNewShapeToCanvas(polyLine);

	if (_Pageinfo->PagePoint.X != -1 && _Pageinfo->PagePoint.Y != -1)
	{
		m_isPressed = true;

		m_pressStartPoint = _Pt;

		pointVector = ref new Windows::UI::Xaml::Media::PointCollection();
		polyLinePointsVector->Append(pointVector);
		pointVector->Append(_Pt);

	    polyLine->Points = pointVector;
	}
}

void InkAnnotEventPeer::OnPointerMoved(Platform::Object^ sender, Windows::UI::Xaml::Input::PointerRoutedEventArgs^ e)
{
	if (!m_isPressed)
		return;


	Windows::UI::Xaml::FrameworkElement^ _FrameworkElement = dynamic_cast<Windows::UI::Xaml::FrameworkElement^>(sender);
	Windows::Foundation::Point currentPoint;
	auto _PointerPt = e->GetCurrentPoint(_FrameworkElement);
	currentPoint = _PointerPt->Position;

	Windows::Foundation::Rect pageViewRect = View->GetPageViewRect(m_PageIndex);

	if (currentPoint.Y > pageViewRect.Y + pageViewRect.Height)
	{
		currentPoint.Y = pageViewRect.Y + pageViewRect.Height;
	}
	else if (currentPoint.Y < pageViewRect.Y)
	{
		currentPoint.Y = pageViewRect.Y;
	}
	
	if (currentPoint.X > pageViewRect.X + pageViewRect.Width)
	{
		currentPoint.X = pageViewRect.X + pageViewRect.Width;
	}
	else if (currentPoint.X < pageViewRect.X)
	{
		currentPoint.X = pageViewRect.X;
	}


	pointVector->Append(currentPoint);
	polyLine->Points = pointVector;
}

void InkAnnotEventPeer::OnPointerReleased(Platform::Object^ sender, Windows::UI::Xaml::Input::PointerRoutedEventArgs^ e)
{
	if (!m_isPressed)
		return;
	UIExtensionsManager^ uiMgr = (UIExtensionsManager^)View->ExtensionsManager;
	OutputDebugString(L"InkAnnotEventPeer OnPointerReleased\n");

	Windows::UI::Xaml::FrameworkElement^ _FrameworkElement = dynamic_cast<Windows::UI::Xaml::FrameworkElement^>(sender);
	Windows::Foundation::Point _PtMoved;
	auto _PointerPt = e->GetCurrentPoint(_FrameworkElement);
	_PtMoved = _PointerPt->Position;

	Windows::Foundation::Rect pageViewRect = View->GetPageViewRect(m_PageIndex);

	if (_PtMoved.Y > pageViewRect.Y + pageViewRect.Height)
	{
		_PtMoved.Y = pageViewRect.Y + pageViewRect.Height;
	}
	else if (_PtMoved.Y < pageViewRect.Y)
	{
		_PtMoved.Y = pageViewRect.Y;
	}

	if (_PtMoved.X > pageViewRect.X + pageViewRect.Width)
	{
		_PtMoved.X = pageViewRect.X + pageViewRect.Width;
	}
	else if (_PtMoved.X < pageViewRect.X)
	{
		_PtMoved.X = pageViewRect.X;
	}

	pointVector->Append(_PtMoved);
	polyLine->Points = pointVector;

	int size= pointVector->Size;
	for (int i = 0; i <size; i++)
	{
		Point pt = pointVector->GetAt(i);
		int k = 0;
	}	
	m_isPressed = false;

	uiMgr->SetAnnotPropertyToolIsEnable(true);

	auto pUndoItem = ref new DrawInkAnnotUndoAction(View, pointVector, pageViewRect, m_PageIndex);
	UndoFactory::GetUndoInstance()->AddUndoItem(uiMgr, pUndoItem);
	drawInkActionObj->Append(pUndoItem);

	uiMgr->UpdateAnnotToolUnReDoControl();
}

void InkAnnotEventPeer::OnTapped(Platform::Object^ sender, Windows::UI::Xaml::Input::TappedRoutedEventArgs^ e)
{
	UIExtensionsManager^ uiMgr = (UIExtensionsManager^)View->ExtensionsManager;
	auto _SelectedAnnot = uiMgr->getAnnotMgr()->GetSelectedAnnot();
	if (_SelectedAnnot)
	{
		PencilAnnot^ annot = dynamic_cast<PencilAnnot^>(_SelectedAnnot);
	 	uiMgr->CreatePolyLineAnnotToCanvas(annot->PageIndex,annot->MinRect, annot->Page_VectPoint, annot->Opacity, annot->BorderWidth,annot->Color);

		Windows::Foundation::Rect rect = annot->MinRect;
		rect.X = rect.X - 5;
		rect.Y = rect.Y - 5;
		rect.Width = rect.Width + 5 * 2;
		rect.Height = rect.Height + 5 * 2;
		auto deviceRect = View->ConvertPageRectToDeviceRect(uiMgr->getAnnotMgr()->GetSelectedAnnot()->PageIndex, rect);
		Point pt = Point(deviceRect.X + deviceRect.Width / 2, deviceRect.Y);		

		if (View->GetDoc() && !FSPDFDocUtil::GetIsDocumentCanModifyAnnotation(View->GetDoc()) && View->GetDoc()->GetPasswordType() != foxit::pdf::PasswordType::e_PwdOwner)
		{
			if (_SelectedAnnot->Contents->Length() > 0)
			{
				uiMgr->ShowTextmarkContextmenu(AnnotContextMenuType::Note, pt);
			}
		}
		else
		{			
			uiMgr->ShowTextmarkContextmenu(AnnotContextMenuType::Edit, pt);
		}
	}
}

void InkAnnotEventPeer::OnRightTapped(Platform::Object^ sender, Windows::UI::Xaml::Input::RightTappedRoutedEventArgs^ e)
{

}

void InkAnnotEventPeer::OnManipulationStarted(Platform::Object^ sender, Windows::UI::Xaml::Input::ManipulationStartedRoutedEventArgs^ e)
{
	UIExtensionsManager^ uiMgr = (UIExtensionsManager^)View->ExtensionsManager;
	auto _SelectedAnnot = uiMgr->getAnnotMgr()->GetSelectedAnnot();
	if (_SelectedAnnot)
	{
		m_isMoveOutPage = false;
		m_isMoveEnd = false;
	}
}

void InkAnnotEventPeer::OnManipulationDelta(Platform::Object^ sender, Windows::UI::Xaml::Input::ManipulationDeltaRoutedEventArgs^ e)
{
	if (!View)
		return;

	if (m_isMoveEnd)
		return;

	UIExtensionsManager^ uiMgr = (UIExtensionsManager^)View->ExtensionsManager;
	auto _SelectedAnnot = uiMgr->getAnnotMgr()->GetSelectedAnnot();
	if (_SelectedAnnot)
	{
		Windows::UI::Xaml::FrameworkElement^ _FrameworkElement = dynamic_cast<Windows::UI::Xaml::FrameworkElement^>(sender);
		auto _Pageinfo = View->ConvertDevicePtToPDFPagePt(e->Position);
		int index = _Pageinfo->PageIndex;
		auto _PagePoint = _Pageinfo->PagePoint;

		Windows::Foundation::Rect deviceMinRect = _SelectedAnnot->MinRect;
	
		if (index != _SelectedAnnot->PageIndex)
		{
			return;
		}
		if (_PagePoint.X == 0 && _PagePoint.Y == 0)
		{
			m_isMoveOutPage = true;
			return;
		}
		else if (m_isMoveOutPage && deviceMinRect.Contains(_PagePoint))
		{
			m_isMoveOutPage = false;
		}

		if (m_isMoveOutPage)
			return;

		auto _Delta = e->Delta;
		auto _actualTranslationX = _Delta.Translation.X;
		auto _actualTranslationY = _Delta.Translation.Y;
		Windows::Foundation::Rect rect = _SelectedAnnot->MinRect;

		auto wrapperDevice = View->ConvertPageRectToDeviceRect(_SelectedAnnot->PageIndex, rect);
		Windows::Foundation::Rect deviceRect = wrapperDevice;

		Windows::Foundation::Rect rangeRect = View->GetPageViewRect(_SelectedAnnot->PageIndex);

		deviceRect.X = deviceRect.X + (float)_actualTranslationX;
		deviceRect.Y = deviceRect.Y + (float)_actualTranslationY;

		if (rangeRect.Contains(Point(deviceRect.X, deviceRect.Y)) && rangeRect.Contains(Point(deviceRect.X + deviceRect.Width, deviceRect.Y + deviceRect.Height)))
		{
			auto deviceRc = View->ConvertDeviceRectToPageRect(_SelectedAnnot->PageIndex, deviceRect);

			Windows::Foundation::Rect endRect = deviceRc;

		
			_SelectedAnnot->MinRect = deviceRc;
			
			View->Update();
		}
		pt = e->Position;
	}
}

void InkAnnotEventPeer::OnManipulationCompleted(Platform::Object^ sender, Windows::UI::Xaml::Input::ManipulationCompletedRoutedEventArgs^ e)
{
	Windows::UI::Xaml::FrameworkElement^ _FrameworkElement = dynamic_cast<Windows::UI::Xaml::FrameworkElement^>(sender);
	_FrameworkElement->ManipulationMode = Windows::UI::Xaml::Input::ManipulationModes::System;

	if (m_isMoveEnd)
		return;
	UIExtensionsManager^ uiMgr = (UIExtensionsManager^)View->ExtensionsManager;
	uiMgr->ShowTextmarkContextmenu(AnnotContextMenuType::Delete, e->Position);
	m_isMoveOutPage = false;
}

void InkAnnotEventPeer::OnCreateAnnotAfterSelectText()
{
	UIExtensionsManager^ uiMgr = (UIExtensionsManager^)View->ExtensionsManager;
	m_isBeginCreate = true;
	polyLinePointsVector = uiMgr->GetPolyLinePoints();
	if (polyLinePointsVector && polyLinePointsVector->Size > 0)
	{
		int size = polyLinePointsVector->Size;

		auto _AnnotSetting = uiMgr->getAnnotMgr()->GetAnnotSetting();
		auto _Annot = foxit::rdk::uiextensions::AnnotCreaterFactory::GetInstance()->CreateNewAnnot(AnnotationType::Ink);
		auto _pCreateAnnot = dynamic_cast<PencilAnnot^>(_Annot);

		Windows::Foundation::Collections::IVector<Windows::Foundation::Collections::IVector<Windows::Foundation::Point>^>^ pointVector=
			ref new Platform::Collections::Vector<Windows::Foundation::Collections::IVector<Windows::Foundation::Point>^>();
		for (int i = 0; i < size; i++)
		{
			Windows::Foundation::Collections::IVector<Windows::Foundation::Point>^ points = ref new Platform::Collections::Vector<Windows::Foundation::Point>();
			pointVector->Append(points);
			for (unsigned int j = 0; j < polyLinePointsVector->GetAt(i)->Size; j++)
			{
				Point devicePt = polyLinePointsVector->GetAt(i)->GetAt(j);
				Point pagePt = View->ConvertDevicePtToPDFPagePt(devicePt)->PagePoint;
				points->Append(pagePt);
			}
		}

		auto _Pageinfo = View->ConvertDevicePtToPDFPagePt(m_pressStartPoint);
		int pageIndex = _Pageinfo->PageIndex;

		_pCreateAnnot->Color = _AnnotSetting->GetAnnotColor();
		_pCreateAnnot->Opacity = _AnnotSetting->GetAnnotOpacity();
		_pCreateAnnot->Page_VectPoint = pointVector;
		int borderWidth = (int)FSPDFDocUtil::PixelToPDFPoint(View->GetDoc(), pageIndex, _AnnotSetting->GetAnnotDrawBorderWidth());
		_pCreateAnnot->BorderWidth = borderWidth;
		_pCreateAnnot->Name = uiMgr->getAnnotMgr()->GetAnnotSetting()->GetNoteAuthor();
		_pCreateAnnot->Title = _AnnotSetting->GetNoteAuthor();

		_pCreateAnnot->CreationDateTime = (ref new Windows::Globalization::Calendar())->GetDateTime();

		auto _page = View->GetDoc()->GetPage(pageIndex);

		uiMgr->getAnnotMgr()->OnCreateNewAnnot(_page, _pCreateAnnot);

		auto pUndoItem = ref new CreateInkAnnotUndoAction(View, _pCreateAnnot);
		UndoFactory::GetUndoInstance()->AddUndoItem(uiMgr, pUndoItem);

		Windows::Foundation::Rect rc = _pCreateAnnot->MinRect;
		foxit::common::fxcrt::RectF^ minRect = FoxitRectConverter::ConvertToFoxitRectF(_pCreateAnnot->MinRect);
		String^ rectStr = "[" + minRect->left.ToString() + " " + minRect->top.ToString() + " " + minRect->right.ToString() + " " + minRect->bottom.ToString() + "]";
		uiMgr->CollectAnnotOperation("Pencil", "Add", pageIndex.ToString(), rectStr);

		if (drawInkActionObj && drawInkActionObj->Size > 0)
		{
			for (unsigned int i = 0; i < drawInkActionObj->Size; i++)
			{
				DrawInkAnnotUndoAction^ action = dynamic_cast<DrawInkAnnotUndoAction^>(drawInkActionObj->GetAt(i));
				if (action)
				{
					UndoFactory::GetUndoInstance()->DeleteUndoItem(uiMgr, action);
				}
			}
			drawInkActionObj->Clear();
		}
		uiMgr->SetAnnotCanvasVisible(false);
	}

	m_pressStartPoint = Point(-1, -1);		
	m_isPressed = false;

	polyLinePointsVector = ref new  Platform::Collections::Vector<Windows::UI::Xaml::Media::PointCollection^>();
	polyLinePointsVector->Clear();

	uiMgr->viewControl->EnableScrollView(true);
}

void InkAnnotEventPeer::OnManipulationInertiaStarting(Platform::Object^ sender, Windows::UI::Xaml::Input::ManipulationInertiaStartingRoutedEventArgs^ e)
{

}

