/**
 * Copyright (C) 2003-2018, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit Mobile PDF SDK to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */


#include "pch.h"
#include "LineAnnotEventPeer.h"


using namespace foxit::rdk;
using namespace foxit::rdk::uiextensions;

using namespace Platform;
using namespace Windows::Foundation;


namespace foxit
{
	namespace rdk
	{
		namespace uiextensions
		{
			[Windows::Foundation::Metadata::WebHostHidden]
			public ref class CreateLineAnnotUndoAction sealed : IUndoItem
			{
			public:
				CreateLineAnnotUndoAction(PDFViewCtrl^ view, LineAnnot^ annot);

				virtual void Undo();
				virtual void Redo();
				virtual Platform::String^ GetDescr();
				virtual void ReleaseItem();

			internal:
				property PDFViewCtrl^ View
				{
					PDFViewCtrl^ get();
				}

				property LineAnnot^ AnnotHandle
				{
					LineAnnot^ get();
				}

			private:
				Platform::WeakReference m_ViewWeakRef;
				Platform::WeakReference m_AnnotWeakRef;
				BaseAnnot^ m_RedoAnnot;
			};
		}
	}
}

CreateLineAnnotUndoAction::CreateLineAnnotUndoAction(PDFViewCtrl^ view, LineAnnot^ annot)
{
	m_ViewWeakRef = view;
	m_AnnotWeakRef = annot;
}

PDFViewCtrl^ CreateLineAnnotUndoAction::View::get()
{
	return m_ViewWeakRef.Resolve<PDFViewCtrl>();
}

LineAnnot^ CreateLineAnnotUndoAction::AnnotHandle::get()
{
	return m_AnnotWeakRef.Resolve<LineAnnot>();
}

void CreateLineAnnotUndoAction::Undo()
{
	if (View && AnnotHandle)
	{
		m_RedoAnnot = AnnotHandle;
		View->CancelRenderTask();
		AnnotHandle->OnDelete();
	}
}

void CreateLineAnnotUndoAction::Redo()
{
	if (View && m_RedoAnnot)
	{
		auto _page = View->GetDoc()->GetPage(m_RedoAnnot->PageIndex);
		View->CancelRenderTask();
		UIExtensionsManager^ uiMgr = (UIExtensionsManager^)View->ExtensionsManager;
		uiMgr->getAnnotMgr()->OnCreateNewAnnot(_page, m_RedoAnnot);
	}
}

Platform::String^ CreateLineAnnotUndoAction::GetDescr()
{
	return ref new Platform::String(L"");
}

void CreateLineAnnotUndoAction::ReleaseItem()
{
	m_ViewWeakRef = nullptr;
	m_AnnotWeakRef = nullptr;
	m_RedoAnnot = nullptr;
}



LineAnnotEventPeer::LineAnnotEventPeer(PDFViewCtrl^ view)
{
	m_PageViewWeakRef = view;
	m_pressStartPoint = Point(-1, -1);
	polyLinePointsVector = ref new  Platform::Collections::Vector<Windows::UI::Xaml::Media::PointCollection^>();
	m_isPressed = false;
}

PDFViewCtrl^ LineAnnotEventPeer::View::get()
{
	return m_PageViewWeakRef.Resolve<PDFViewCtrl>();
}

void LineAnnotEventPeer::OnPointerPressed(Platform::Object^ sender, Windows::UI::Xaml::Input::PointerRoutedEventArgs^ e)
{
	if (!View)
		return;

	if (m_isPressed)
		return;
	UIExtensionsManager^ uiMgr = (UIExtensionsManager^)View->ExtensionsManager;
	auto _AnnotSetting = uiMgr->getAnnotMgr()->GetAnnotSetting();
	borderWidth = _AnnotSetting->GetAnnotDrawBorderWidth();

	line = ref new Windows::UI::Xaml::Shapes::Line();

	line->Stroke = ref new Windows::UI::Xaml::Media::SolidColorBrush(Windows::UI::Colors::Gray);
	line->Opacity = 0.5;
	line->StrokeThickness = 2;

	uiMgr->CreateNewShapeToCanvas(line);

	Windows::UI::Xaml::FrameworkElement^ _FrameworkElement = dynamic_cast<Windows::UI::Xaml::FrameworkElement^>(sender);
	Windows::Foundation::Point _Pt;
	auto _PointerPt = e->GetCurrentPoint(_FrameworkElement);
	_Pt = _PointerPt->Position;

	auto _Pageinfo = View->ConvertDevicePtToPDFPagePt(_Pt);
	m_PageIndex = _Pageinfo->PageIndex;

	if (_Pageinfo->PagePoint.X != -1 && _Pageinfo->PagePoint.Y != -1)
	{
		m_isPressed = true;

		m_pressStartPoint = _Pt;

		pointVector = ref new Windows::UI::Xaml::Media::PointCollection();
		polyLinePointsVector->Append(pointVector);
		pointVector->Append(_Pt);

		line->X1 = m_pressStartPoint.X;
		line->Y1 = m_pressStartPoint.Y;
		line->X2 = m_pressStartPoint.X;
		line->Y2 = m_pressStartPoint.Y;
	}
	_FrameworkElement->ManipulationMode = Windows::UI::Xaml::Input::ManipulationModes::System;
}

void LineAnnotEventPeer::OnPointerMoved(Platform::Object^ sender, Windows::UI::Xaml::Input::PointerRoutedEventArgs^ e)
{
	if (!m_isPressed)
		return;

	Windows::UI::Xaml::FrameworkElement^ _FrameworkElement = dynamic_cast<Windows::UI::Xaml::FrameworkElement^>(sender);
	auto _PointerPt = e->GetCurrentPoint(_FrameworkElement);
	currentPoint = _PointerPt->Position;

	Windows::Foundation::Rect pageViewRect = View->GetPageViewRect(m_PageIndex);

	if (currentPoint.Y > pageViewRect.Y + pageViewRect.Height)
	{
		currentPoint.Y = pageViewRect.Y + pageViewRect.Height;
	}
	else if (currentPoint.Y < pageViewRect.Y)
	{
		currentPoint.Y = pageViewRect.Y;
	}

	if (currentPoint.X > pageViewRect.X + pageViewRect.Width)
	{
		currentPoint.X = pageViewRect.X + pageViewRect.Width;
	}
	else if (currentPoint.X < pageViewRect.X)
	{
		currentPoint.X = pageViewRect.X;
	}

	line->X2 = currentPoint.X;
	line->Y2 = currentPoint.Y;
}

void LineAnnotEventPeer::OnPointerReleased(Platform::Object^ sender, Windows::UI::Xaml::Input::PointerRoutedEventArgs^ e)
{
	if (!m_isPressed)
		return;

	if (m_pressStartPoint.X == -1 || m_pressStartPoint.Y == -1)
		return;

	Windows::UI::Xaml::FrameworkElement^ _FrameworkElement = dynamic_cast<Windows::UI::Xaml::FrameworkElement^>(sender);
	Windows::Foundation::Point currentPoint;
	auto _PointerPt = e->GetCurrentPoint(_FrameworkElement);
	currentPoint = _PointerPt->Position;

	Windows::Foundation::Rect pageViewRect = View->GetPageViewRect(m_PageIndex);

	if (currentPoint.Y > pageViewRect.Y + pageViewRect.Height)
	{
		currentPoint.Y = pageViewRect.Y + pageViewRect.Height;
	}
	else if (currentPoint.Y < pageViewRect.Y)
	{
		currentPoint.Y = pageViewRect.Y;
	}
	if (currentPoint.X > pageViewRect.X + pageViewRect.Width)
	{
		currentPoint.X = pageViewRect.X + pageViewRect.Width;
	}
	else if (currentPoint.X < pageViewRect.X)
	{
		currentPoint.X = pageViewRect.X;
	}
	UIExtensionsManager^ uiMgr = (UIExtensionsManager^)View->ExtensionsManager;
	Windows::Foundation::Rect deviceRect(m_pressStartPoint, currentPoint);
	if (deviceRect.Width < 1 || deviceRect.Height < 1)
	{
		m_pressStartPoint = Point(-1, -1);
		m_isPressed = false;
		uiMgr->SetAnnotCanvasVisible(false);
		return;
	}
	deviceRect.Height = deviceRect.Height + 5;

	Windows::Foundation::Rect pageRect = View->ConvertDeviceRectToPageRect(m_PageIndex, deviceRect);


	auto _AnnotSetting = uiMgr->getAnnotMgr()->GetAnnotSetting();
	auto _Annot = foxit::rdk::uiextensions::AnnotCreaterFactory::GetInstance()->CreateNewAnnot(AnnotationType::Line);
	auto _pCreateAnnot = dynamic_cast<LineAnnot^>(_Annot);

	auto _PageinfoStart = View->ConvertDevicePtToPDFPagePt(m_pressStartPoint);
	auto _PageinfoEnd = View->ConvertDevicePtToPDFPagePt(currentPoint);

	if (uiMgr->CreateTypeOfAnnot == AnnotCreateType::Arrow)
	{
		_pCreateAnnot->IsArrow = true;
	}
	
	_pCreateAnnot->Color = _AnnotSetting->GetLineAnnotColor();
	_pCreateAnnot->Opacity = _AnnotSetting->GetAnnotOpacity();
	_pCreateAnnot->Page_StartPoint = _PageinfoStart->PagePoint;
	_pCreateAnnot->Page_EndPoint = _PageinfoEnd->PagePoint;
	_pCreateAnnot->BorderWidth = _AnnotSetting->GetAnnotDrawBorderWidth();
	_pCreateAnnot->Name = uiMgr->getAnnotMgr()->GetAnnotSetting()->GetNoteAuthor();
	_pCreateAnnot->Title = _AnnotSetting->GetNoteAuthor();
	_pCreateAnnot->PageMinRect = pageRect;

	_pCreateAnnot->CreationDateTime = (ref new Windows::Globalization::Calendar())->GetDateTime();

	auto _page = View->GetDoc()->GetPage(m_PageIndex);

	uiMgr->getAnnotMgr()->OnCreateNewAnnot(_page, _pCreateAnnot);

	auto pUndoItem = ref new CreateLineAnnotUndoAction(View, _pCreateAnnot);
	UndoFactory::GetUndoInstance()->AddUndoItem(uiMgr, pUndoItem);

	Windows::Foundation::Rect rc = _pCreateAnnot->MinRect;
	foxit::common::fxcrt::RectF^ minRect = FoxitRectConverter::ConvertToFoxitRectF(_pCreateAnnot->MinRect);
	String^ rectStr = "[" + minRect->left.ToString() + " " + minRect->top.ToString() + " " + minRect->right.ToString() + " " + minRect->bottom.ToString() + "]";
	if (_pCreateAnnot->IsArrow)
		uiMgr->CollectAnnotOperation("Arrow", "Add", m_PageIndex.ToString(), rectStr);
	else
		uiMgr->CollectAnnotOperation("Line", "Add", m_PageIndex.ToString(), rectStr);

	uiMgr->SetAnnotCanvasVisible(false);

	m_pressStartPoint = Point(-1, -1);
	m_isPressed = false;

	if (uiMgr->CreateTypeOfAnnot == AnnotCreateType::Line && !_AnnotSetting->GetLineIsContinueOpera()
		|| uiMgr->CreateTypeOfAnnot == AnnotCreateType::Arrow && !_AnnotSetting->GetArrowIsContinueOpera())
	{
		uiMgr->CreateTypeOfAnnot = AnnotCreateType::None;
		uiMgr->ExitAnnotPropertyMode();
	}
	_FrameworkElement->ManipulationMode = Windows::UI::Xaml::Input::ManipulationModes::All;
}

void LineAnnotEventPeer::OnTapped(Platform::Object^ sender, Windows::UI::Xaml::Input::TappedRoutedEventArgs^ e)
{
	UIExtensionsManager^ uiMgr = (UIExtensionsManager^)View->ExtensionsManager;
	auto _SelectedAnnot = uiMgr->getAnnotMgr()->GetSelectedAnnot();
	if (_SelectedAnnot)
	{
		LineAnnot^ annot = dynamic_cast<LineAnnot^>(_SelectedAnnot);
		uiMgr->CreateLineAnnotToCanvas(annot->PageIndex, annot->MinRect, annot->Page_StartPoint, annot->Page_EndPoint, annot->Opacity, annot->BorderWidth, annot->Color);

		Windows::Foundation::Rect rect = annot->MinRect;
		rect.X = rect.X - 5;
		rect.Y = rect.Y - 5;
		rect.Width = rect.Width + 5 * 2;
		rect.Height = rect.Height + 5 * 2;
		auto deviceRect = View->ConvertPageRectToDeviceRect(uiMgr->getAnnotMgr()->GetSelectedAnnot()->PageIndex, rect);
		Point pt = Point(deviceRect.X + deviceRect.Width / 2, deviceRect.Y);

		if (View->GetDoc() && !FSPDFDocUtil::GetIsDocumentCanModifyAnnotation(View->GetDoc()) && View->GetDoc()->GetPasswordType() != foxit::pdf::PasswordType::e_PwdOwner)
		{
			if (_SelectedAnnot->Contents->Length() > 0)
			{
				uiMgr->ShowTextmarkContextmenu(AnnotContextMenuType::Note, pt);
			}
		}
		else
		{
			uiMgr->ShowTextmarkContextmenu(AnnotContextMenuType::Edit, pt);
		}
	}
}

void LineAnnotEventPeer::OnRightTapped(Platform::Object^ sender, Windows::UI::Xaml::Input::RightTappedRoutedEventArgs^ e)
{

}

void LineAnnotEventPeer::OnManipulationStarted(Platform::Object^ sender, Windows::UI::Xaml::Input::ManipulationStartedRoutedEventArgs^ e)
{
	UIExtensionsManager^ uiMgr = (UIExtensionsManager^)View->ExtensionsManager;
	auto _SelectedAnnot = uiMgr->getAnnotMgr()->GetSelectedAnnot();
	if (_SelectedAnnot)
	{
		m_isMoveOutPage = false;
		m_isMoveEnd = false;
	}
}

void LineAnnotEventPeer::OnManipulationDelta(Platform::Object^ sender, Windows::UI::Xaml::Input::ManipulationDeltaRoutedEventArgs^ e)
{
	if (!View)
		return;

	if (m_isMoveEnd)
		return;
	UIExtensionsManager^ uiMgr = (UIExtensionsManager^)View->ExtensionsManager;
	auto _SelectedAnnot = uiMgr->getAnnotMgr()->GetSelectedAnnot();
	if (_SelectedAnnot)
	{
		Windows::UI::Xaml::FrameworkElement^ _FrameworkElement = dynamic_cast<Windows::UI::Xaml::FrameworkElement^>(sender);
		auto _Pageinfo = View->ConvertDevicePtToPDFPagePt(e->Position);
		int index = _Pageinfo->PageIndex;
		auto _PagePoint = _Pageinfo->PagePoint;

		Windows::Foundation::Rect deviceMinRect = _SelectedAnnot->MinRect;

		if (index != _SelectedAnnot->PageIndex)
		{
			return;
		}
		if (_PagePoint.X == 0 && _PagePoint.Y == 0)
		{
			m_isMoveOutPage = true;
			return;
		}
		else if (m_isMoveOutPage && deviceMinRect.Contains(_PagePoint))
		{
			m_isMoveOutPage = false;
		}

		if (m_isMoveOutPage)
			return;

		auto _Delta = e->Delta;
		auto _actualTranslationX = _Delta.Translation.X;
		auto _actualTranslationY = _Delta.Translation.Y;
		Windows::Foundation::Rect rect = _SelectedAnnot->MinRect;

		auto wrapperDevice = View->ConvertPageRectToDeviceRect(_SelectedAnnot->PageIndex, rect);
		Windows::Foundation::Rect deviceRect = wrapperDevice;

		Windows::Foundation::Rect rangeRect = View->GetPageViewRect(_SelectedAnnot->PageIndex);
		
		deviceRect.X = deviceRect.X + (float)_actualTranslationX;
		deviceRect.Y = deviceRect.Y + (float)_actualTranslationY;

		if (rangeRect.Contains(Point(deviceRect.X, deviceRect.Y)) && rangeRect.Contains(Point(deviceRect.X + deviceRect.Width, deviceRect.Y + deviceRect.Height)))
		{
			auto deviceRc = View->ConvertDeviceRectToPageRect(_SelectedAnnot->PageIndex, deviceRect);

			Windows::Foundation::Rect endRect = deviceRc;


			_SelectedAnnot->MinRect = deviceRc;

			View->Update();
		}

		pt = e->Position;
	}
}

void LineAnnotEventPeer::OnManipulationCompleted(Platform::Object^ sender, Windows::UI::Xaml::Input::ManipulationCompletedRoutedEventArgs^ e)
{
	if (m_isMoveEnd)
		return;
	UIExtensionsManager^ uiMgr = (UIExtensionsManager^)View->ExtensionsManager;
	uiMgr->ShowTextmarkContextmenu(AnnotContextMenuType::Delete, e->Position);
	m_isMoveOutPage = false;
}

void LineAnnotEventPeer::OnCreateAnnotAfterSelectText()
{
	if (!m_isPressed)
		return;
	UIExtensionsManager^ uiMgr = (UIExtensionsManager^)View->ExtensionsManager;
	Windows::Foundation::Rect pageViewRect = View->GetPageViewRect(m_PageIndex);

	if (currentPoint.Y > pageViewRect.Y + pageViewRect.Height)
	{
		currentPoint.Y = pageViewRect.Y + pageViewRect.Height;
	}
	else if (currentPoint.Y < pageViewRect.Y)
	{
		currentPoint.Y = pageViewRect.Y;
	}
	if (currentPoint.X > pageViewRect.X + pageViewRect.Width)
	{
		currentPoint.X = pageViewRect.X + pageViewRect.Width;
	}
	else if (currentPoint.X < pageViewRect.X)
	{
		currentPoint.X = pageViewRect.X;
	}

	Windows::Foundation::Rect deviceRect(m_pressStartPoint, currentPoint);
	if (deviceRect.Width < 1 || deviceRect.Height < 1)
	{
		m_pressStartPoint = Point(-1, -1);
		m_isPressed = false;
		uiMgr->SetAnnotCanvasVisible(false);
		return;
	}
	deviceRect.Height = deviceRect.Height + 5;

	Windows::Foundation::Rect pageRect = View->ConvertDeviceRectToPageRect(m_PageIndex, deviceRect);

	auto _AnnotSetting = uiMgr->getAnnotMgr()->GetAnnotSetting();
	auto _Annot = foxit::rdk::uiextensions::AnnotCreaterFactory::GetInstance()->CreateNewAnnot(AnnotationType::Line);
	auto _pCreateAnnot = dynamic_cast<LineAnnot^>(_Annot);
	if (uiMgr->CreateTypeOfAnnot == AnnotCreateType::Arrow)
	{
		_pCreateAnnot->IsArrow = true;
	}
	auto _PageinfoStart = View->ConvertDevicePtToPDFPagePt(m_pressStartPoint);
	auto _PageinfoEnd = View->ConvertDevicePtToPDFPagePt(currentPoint);

	_pCreateAnnot->Color = _AnnotSetting->GetLineAnnotColor();
	_pCreateAnnot->Opacity = _AnnotSetting->GetAnnotOpacity();
	_pCreateAnnot->Page_StartPoint = _PageinfoStart->PagePoint;
	_pCreateAnnot->Page_EndPoint = _PageinfoEnd->PagePoint;
	_pCreateAnnot->BorderWidth = _AnnotSetting->GetAnnotDrawBorderWidth();
	_pCreateAnnot->Name = uiMgr->getAnnotMgr()->GetAnnotSetting()->GetNoteAuthor();
	_pCreateAnnot->Title = _AnnotSetting->GetNoteAuthor();
	_pCreateAnnot->PageMinRect = pageRect;

	_pCreateAnnot->CreationDateTime = (ref new Windows::Globalization::Calendar())->GetDateTime();

	auto _page = View->GetDoc()->GetPage(m_PageIndex);

	uiMgr->getAnnotMgr()->OnCreateNewAnnot(_page, _pCreateAnnot);

	auto pUndoItem = ref new CreateLineAnnotUndoAction(View, _pCreateAnnot);
	UndoFactory::GetUndoInstance()->AddUndoItem(uiMgr, pUndoItem);

	Windows::Foundation::Rect rc = _pCreateAnnot->MinRect;
	foxit::common::fxcrt::RectF^ minRect = FoxitRectConverter::ConvertToFoxitRectF(_pCreateAnnot->MinRect);
	String^ rectStr = "[" + minRect->left.ToString() + " " + minRect->top.ToString() + " " + minRect->right.ToString() + " " + minRect->bottom.ToString() + "]";
	if (_pCreateAnnot->IsArrow)
		uiMgr->CollectAnnotOperation("Arrow", "Add", m_PageIndex.ToString(), rectStr);
	else
		uiMgr->CollectAnnotOperation("Line", "Add", m_PageIndex.ToString(), rectStr);

	uiMgr->SetAnnotCanvasVisible(false);

	m_pressStartPoint = Point(-1, -1);
	m_isPressed = false;
}

void LineAnnotEventPeer::OnManipulationInertiaStarting(Platform::Object^ sender, Windows::UI::Xaml::Input::ManipulationInertiaStartingRoutedEventArgs^ e)
{

}

