/**
 * Copyright (C) 2003-2018, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit Mobile PDF SDK to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */


#include "pch.h"
#include "LinkAnnotEventPeer.h"

using namespace foxit::rdk;
using namespace foxit::rdk::uiextensions;

using namespace Platform;
using namespace Windows::Foundation;
using namespace Windows::UI::Popups;

LinkAnnotEventPeer::LinkAnnotEventPeer(PDFViewCtrl^ view)
{
	m_PageViewWeakRef = view;
	m_TextSelectContext = ref new TextSelectContext();		
}

PDFViewCtrl^ LinkAnnotEventPeer::View::get()
{
	return m_PageViewWeakRef.Resolve<PDFViewCtrl>();
}

void LinkAnnotEventPeer::OnPointerPressed(Platform::Object^ sender, Windows::UI::Xaml::Input::PointerRoutedEventArgs^ e)
{
	
}

void LinkAnnotEventPeer::OnPointerMoved(Platform::Object^ sender, Windows::UI::Xaml::Input::PointerRoutedEventArgs^ e)
{

}

void LinkAnnotEventPeer::OnPointerReleased(Platform::Object^ sender, Windows::UI::Xaml::Input::PointerRoutedEventArgs^ e)
{
	
}

void LinkAnnotEventPeer::OnTapped(Platform::Object^ sender, Windows::UI::Xaml::Input::TappedRoutedEventArgs^ e)
{
	UIExtensionsManager^ uiMgr = (UIExtensionsManager^)View->ExtensionsManager;
	auto _SelectedAnnot = uiMgr->getAnnotMgr()->GetSelectedAnnot();
	if (_SelectedAnnot)
	{
		int count = 0;
		foxit::rdk::uiextensions::PDFAnnotWrapper^ wrapper = (foxit::rdk::uiextensions::PDFAnnotWrapper^)(_SelectedAnnot->AnnotHandleWrap->PDFAnnot);
		foxit::pdf::annots::Link^ link = dynamic_cast<foxit::pdf::annots::Link^>(foxit::rdk::FXDocUtil::DynamicCastBaseAnnot((foxit::pdf::annots::Annot^)wrapper->Pointer));
		if (!link)
			return;

		auto action = link->GetAction();
		
		auto uiDelegate = [this, action, uiMgr]()
		{
			if (!action) return;
			auto actionType = action->GetType();
			switch (actionType)
			{
			case foxit::pdf::actions::Type::e_TypeUnknown:
			{
				break;
			}
			case foxit::pdf::actions::Type::e_TypeGoto:
			{					
				foxit::pdf::actions::GotoAction^ gotoAction = ref new foxit::pdf::actions::GotoAction(action);
				auto destion = gotoAction->GetDestination();
				if (!destion || destion->IsEmpty())
					return;
				int pageIndex = destion->GetPageIndex(uiMgr->viewControl->GetDoc());
				int zoommode = (int)destion->GetZoomMode();
				View->GotoPage(pageIndex);

				break;
			}
			case foxit::pdf::actions::Type::e_TypeURI:
			{
				foxit::pdf::actions::URIAction^ uriAction = ref new foxit::pdf::actions::URIAction(action);
				String^ url = uriAction->GetURI();
				String^ base = ref new String(L"http://");

				try
				{
					Windows::Foundation::Uri^ uri;
					int length = url->Length();
					wchar_t *author = (wchar_t*)url->Data();
					wchar_t val = author[0];
					if (val == 'h' || val == 'H')
					{
						uri = ref new Windows::Foundation::Uri(url);
					}
					else
					{
						url = base + url;
						uri = ref new Windows::Foundation::Uri(base, url);
					}

					UIExtensionsManager^ uiMgr = (UIExtensionsManager^)View->ExtensionsManager;
					String^ OpenwebPageTitle = uiMgr->GetResourceString(L"OpenWebPageTipText");
					String^  OpenwebPageContent = uiMgr->GetResourceString(L"OpenWebPageOpenTipText") + " " + uri->DisplayUri + "?";

					String^ OpenwebPageOpen = uiMgr->GetResourceString(L"OpenWebOpenText");
					String^ OpenwebPageCancel = uiMgr->GetResourceString(L"OpenWebCancelText");

					auto messageDialog = ref new MessageDialog(OpenwebPageContent, OpenwebPageTitle);
					messageDialog->Commands->Append(ref new UICommand(OpenwebPageOpen, nullptr, PropertyValue::CreateInt32(0)));
					messageDialog->Commands->Append(ref new UICommand(OpenwebPageCancel, nullptr, PropertyValue::CreateInt32(1)));
					messageDialog->DefaultCommandIndex = 0;
					auto _is_canceled = uiMgr->m_TipsDlgCts.get_token().is_canceled();
					if (uiMgr->m_TipsDlgCts.get_token().is_canceled())
					{
						concurrency::cancellation_token_source _newCts;
						uiMgr->m_TipsDlgCts = _newCts;
					}
					concurrency::create_task(messageDialog->ShowAsync(), uiMgr->m_TipsDlgCts.get_token()).then([=](IUICommand^ command)
					{
						if (command && command->Id != nullptr)
						{
							int const id = safe_cast<IBox<int>^>(command->Id)->Value;
							if (id == 0)
							{
								Windows::System::Launcher::LaunchUriAsync(uri); 
							}
						}
					})
						.then([=](concurrency::task<void> t)
					{
						try
						{
							t.get();
						}
						catch (...)
						{
							OutputDebugString(L"ShowAsync...URL...Exception...\n");
						}
					});
				}
				catch (...)
				{
						
					OutputDebugString(L"LinkAnnotEventPeer...URL.Exception.\n");
				}
			}
			}
		};
		Windows::UI::Core::CoreDispatcher^ _mainPageDispatcher = Windows::UI::Core::CoreWindow::GetForCurrentThread()->Dispatcher;
		_mainPageDispatcher->RunAsync(Windows::UI::Core::CoreDispatcherPriority::High, ref new Windows::UI::Core::DispatchedHandler(uiDelegate));
		uiMgr->getAnnotMgr()->SetPdfSelectAnnotNull();
		uiMgr->SetAnnotCanvasVisible(false);
	}
}

void LinkAnnotEventPeer::OnRightTapped(Platform::Object^ sender, Windows::UI::Xaml::Input::RightTappedRoutedEventArgs^ e)
{

}

void LinkAnnotEventPeer::OnManipulationStarted(Platform::Object^ sender, Windows::UI::Xaml::Input::ManipulationStartedRoutedEventArgs^ e)
{

}

void LinkAnnotEventPeer::OnManipulationDelta(Platform::Object^ sender, Windows::UI::Xaml::Input::ManipulationDeltaRoutedEventArgs^ e)
{

}

void LinkAnnotEventPeer::OnManipulationCompleted(Platform::Object^ sender, Windows::UI::Xaml::Input::ManipulationCompletedRoutedEventArgs^ e)
{

}

void LinkAnnotEventPeer::OnCreateAnnotAfterSelectText()
{

}

void LinkAnnotEventPeer::OnManipulationInertiaStarting(Platform::Object^ sender, Windows::UI::Xaml::Input::ManipulationInertiaStartingRoutedEventArgs^ e)
{

}