/**
 * Copyright (C) 2003-2018, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit Mobile PDF SDK to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */


#include "pch.h"
#include "PageViewAnnotMgrPeer.h"
#include "HighlightAnnotEventPeer.h"
#include "UnderLineAnnotEventPeer.h"
#include "SquigglyAnnotEventPeer.h"
#include "StrikeOutAnnotEventPeer.h"
#include "LinkAnnotEventPeer.h"
#include "TextAnnotEventPeer.h"
#include "InkAnnotEventPeer.h"
#include "SquareAnnotEventPeer.h"
#include "CircleAnnotEventPeer.h"
#include "LineAnnotEventPeer.h"
#include "EraserEventPeer.h"
#include "EditAnnotUnDoAction.h"
#include "StampAnnotEventPeer.h"
#include "FreeTextEventPeer.h"
#include "CaretAnnotEventPeer.h"
#include "WidgetAnnotEventPeer.h"

using namespace foxit::rdk;
using namespace foxit::rdk::uiextensions;

using namespace Platform;
using namespace Windows::Foundation;
using namespace Windows::UI::Popups;

PageViewAnnotMgrPeer::PageViewAnnotMgrPeer(UIExtensionsManager^ pUIViewMgr)
{
	m_UIViewWeakRef = pUIViewMgr;
	auto page = pUIViewMgr->viewControl;
	m_PageViewWeakRef = page;

	UIExtensionsManager^ uiMgr = pUIViewMgr;
	uiMgr->getAnnotMgr()->GetAnnotHandlerByType(AnnotationType::HightLight)->SetEventPeer(ref new HighlightAnnotEventPeer(page));
	uiMgr->getAnnotMgr()->GetAnnotHandlerByType(AnnotationType::UnderLine)->SetEventPeer(ref new UnderLineAnnotEventPeer(page));
	uiMgr->getAnnotMgr()->GetAnnotHandlerByType(AnnotationType::Squiggly)->SetEventPeer(ref new SquigglyAnnotEventPeer(page));
	uiMgr->getAnnotMgr()->GetAnnotHandlerByType(AnnotationType::StrikeOut)->SetEventPeer(ref new StrikeOutAnnotEventPeer(page));
	uiMgr->getAnnotMgr()->GetAnnotHandlerByType(AnnotationType::Link)->SetEventPeer(ref new LinkAnnotEventPeer(page));
	uiMgr->getAnnotMgr()->GetAnnotHandlerByType(AnnotationType::Text)->SetEventPeer(ref new TextAnnotEventPeer(page));
	uiMgr->getAnnotMgr()->GetAnnotHandlerByType(AnnotationType::Ink)->SetEventPeer(ref new InkAnnotEventPeer(page));
	uiMgr->getAnnotMgr()->GetAnnotHandlerByType(AnnotationType::Square)->SetEventPeer(ref new SquareAnnotEventPeer(page));
	uiMgr->getAnnotMgr()->GetAnnotHandlerByType(AnnotationType::Circle)->SetEventPeer(ref new CircleAnnotEventPeer(page));
	uiMgr->getAnnotMgr()->GetAnnotHandlerByType(AnnotationType::Line)->SetEventPeer(ref new LineAnnotEventPeer(page));
	uiMgr->getAnnotMgr()->GetAnnotHandlerByType(AnnotationType::Stamp)->SetEventPeer(ref new StampAnnotEventPeer(page));
	uiMgr->getAnnotMgr()->GetAnnotHandlerByType(AnnotationType::FreeText)->SetEventPeer(ref new FreeTextEventPeer(page));
	uiMgr->getAnnotMgr()->GetAnnotHandlerByType(AnnotationType::Caret)->SetEventPeer(ref new CaretAnnotEventPeer(page));
	uiMgr->getAnnotMgr()->GetAnnotHandlerByType(AnnotationType::Widget)->SetEventPeer(ref new WidgetAnnotEventPeer(page));

	eraserEventPeer = ref new EraserAnnotEventPeer(page);
	isShortTextAnnot = false;

	m_renderToken = page->RenderCanvasEvent += ref new foxit::rdk::RenderCanvasEvent(this, &foxit::rdk::uiextensions::PageViewAnnotMgrPeer::OnRenderEvent);
	m_pageLoadedToken = page->PageLoadedEvent += ref new foxit::rdk::PageLoadedEvent(this, &foxit::rdk::uiextensions::PageViewAnnotMgrPeer::OnPageLoaded);
	m_pageUnloadedToken = page->PageUnloadedEvent += ref new foxit::rdk::PageUnloadedEvent(this, &foxit::rdk::uiextensions::PageViewAnnotMgrPeer::OnPageUnloaded);

}

PageViewAnnotMgrPeer::~PageViewAnnotMgrPeer()
{
	View->RenderCanvasEvent -= m_renderToken;
	View->PageLoadedEvent -= m_pageLoadedToken;
	View->PageUnloadedEvent -= m_pageUnloadedToken;
}

AnnotationType PageViewAnnotMgrPeer::GetAnnotType(AnnotCreateType optType)
{
	AnnotationType _annotType;
	switch (optType)
	{
	case AnnotCreateType::None:
		_annotType = AnnotationType::None;
		break;
	case AnnotCreateType::HightLight:
		_annotType = AnnotationType::HightLight;
		break;
	case AnnotCreateType::AreaHighLight:
		_annotType = AnnotationType::HightLight;
		break;
	case AnnotCreateType::Comment:
		_annotType = AnnotationType::HightLight;
		break;
	case AnnotCreateType::UnderLine:
		_annotType = AnnotationType::UnderLine;
		break;
	case AnnotCreateType::StrikeOut:
		_annotType = AnnotationType::StrikeOut;
		break;
	case AnnotCreateType::Square:
		_annotType = AnnotationType::Square;
		break;
	case AnnotCreateType::Circle:
		_annotType = AnnotationType::Circle;
		break;
	case AnnotCreateType::Line:
		_annotType = AnnotationType::Line;
		break;
	case AnnotCreateType::Arrow:
		_annotType = AnnotationType::Line;
		break;
	case AnnotCreateType::Pencil:
		_annotType = AnnotationType::Ink;
		break;
	case AnnotCreateType::Squiggly:
		_annotType = AnnotationType::Squiggly;
		break;
	case AnnotCreateType::Text:
		_annotType = AnnotationType::Text;
		break;
	case AnnotCreateType::Stamp:
		_annotType = AnnotationType::Stamp;
		break;
	case AnnotCreateType::FreeText:
		_annotType = AnnotationType::FreeText;
		break;
	case AnnotCreateType::Caret:
		_annotType = AnnotationType::Caret;
		break;
	case AnnotCreateType::ReplaceText:
		_annotType = AnnotationType::Caret;
		break;
	default:
		_annotType = AnnotationType::None;
		break;
	}
	return _annotType;
}

PDFViewCtrl^ PageViewAnnotMgrPeer::View::get()
{
	return m_PageViewWeakRef.Resolve<PDFViewCtrl>();
}

UIExtensionsManager^ PageViewAnnotMgrPeer::UIExtMgr::get()
{
	return m_UIViewWeakRef.Resolve<UIExtensionsManager>();
}

int PageViewAnnotMgrPeer::GetPageCount()
{
	if (View && View->GetDoc())
	{
		return View->GetDoc()->GetPageCount();
	}
	else
	{
		return 0;
	}
}

Platform::Object^ PageViewAnnotMgrPeer::GetPdfPageObject(int pageIndex)
{

	if (View && View->GetDoc())
	{
		return View->GetDoc()->GetPage(pageIndex);
	}
	return nullptr;
}

void PageViewAnnotMgrPeer::NotifyDocumentChanged()
{
	if (View && View->GetDoc())
	{
		View->SetModified(true);
	}
}

void PageViewAnnotMgrPeer::UpdatePageView(int pageindex)
{
	View->Redraw(pageindex);
}

void PageViewAnnotMgrPeer::OnAnnotPointerEntered(Platform::Object^ sender, Windows::UI::Xaml::Input::PointerRoutedEventArgs^ e)
{

}

void PageViewAnnotMgrPeer::OnAnnotPointerExited(Platform::Object^ sender, Windows::UI::Xaml::Input::PointerRoutedEventArgs^ e)
{

}

void PageViewAnnotMgrPeer::OnAnnotPointerMoved(Platform::Object^ sender, Windows::UI::Xaml::Input::PointerRoutedEventArgs^ e)
{
	UIExtensionsManager^ uiMgr = (UIExtensionsManager^)View->ExtensionsManager;
	auto _annotMgr = uiMgr->getAnnotMgr();
	if (!_annotMgr)
		return;

	if (uiMgr->CreateTypeOfAnnot == AnnotCreateType::Eraser)
	{
		eraserEventPeer->OnPointerMoved(sender, e);
	}
	else
	{
		auto _pAnnotHandler = _annotMgr->GetAnnotHandlerByType(GetAnnotType(uiMgr->CreateTypeOfAnnot));
		if (_pAnnotHandler)
			_pAnnotHandler->OnPointerMoved(sender, e);
	}
}

void PageViewAnnotMgrPeer::OnAnnotPointerPressed(Platform::Object^ sender, Windows::UI::Xaml::Input::PointerRoutedEventArgs^ e)
{
	UIExtensionsManager^ uiMgr = (UIExtensionsManager^)View->ExtensionsManager;
	auto _annotMgr = uiMgr->getAnnotMgr();
	if (!_annotMgr)
		return;
	isShortTextAnnot = false;

	if ((uiMgr->CreateTypeOfAnnot == AnnotCreateType::Caret || uiMgr->CreateTypeOfAnnot == AnnotCreateType::ReplaceText)  && !uiMgr->ExitNotePopupEdit())
	{
		return;
	}

	if (uiMgr->CreateTypeOfAnnot == AnnotCreateType::Eraser)
	{
		eraserEventPeer->OnPointerPressed(sender, e);
	}
	else
	{
		auto _pAnnotHandler = _annotMgr->GetAnnotHandlerByType(GetAnnotType(uiMgr->CreateTypeOfAnnot));
		if (_pAnnotHandler)
			_pAnnotHandler->OnPointerPressed(sender, e);
	}
}

void PageViewAnnotMgrPeer::OnAnnotPointerReleased(Platform::Object^ sender, Windows::UI::Xaml::Input::PointerRoutedEventArgs^ e)
{
	UIExtensionsManager^ uiMgr = (UIExtensionsManager^)View->ExtensionsManager;
	auto _annotMgr = uiMgr->getAnnotMgr();
	if (!_annotMgr)
		return;

	if ((uiMgr->CreateTypeOfAnnot == AnnotCreateType::Caret || uiMgr->CreateTypeOfAnnot == AnnotCreateType::ReplaceText) && !uiMgr->ExitNotePopupEdit())
	{
		return;
	}

	if (uiMgr->CreateTypeOfAnnot == AnnotCreateType::Eraser)
	{
		eraserEventPeer->OnPointerReleased(sender, e);
	}
	else
	{
		auto _pAnnotHandler = _annotMgr->GetAnnotHandlerByType(GetAnnotType(uiMgr->CreateTypeOfAnnot));


		if (uiMgr->CreateTypeOfAnnot != AnnotCreateType::None && uiMgr->getTextMgr()->SelectedText->Size > 0
			&& uiMgr->getTextMgr()->SelectedText->GetAt(0)->TextRectVect->Size == 1)
		{
			if (uiMgr->getTextMgr()->SelectedText->GetAt(0)->CharStart == uiMgr->getTextMgr()->SelectedText->GetAt(0)->CharEnd)
			{
				isShortTextAnnot = true;
				m_createTypeOfAnnot = uiMgr->CreateTypeOfAnnot;
			}
		}

		if (_pAnnotHandler)
			_pAnnotHandler->OnPointerReleased(sender, e);
	}
}

void PageViewAnnotMgrPeer::OnAnnotTapped(Platform::Object^ sender, Windows::UI::Xaml::Input::TappedRoutedEventArgs^ e)
{
	if (isShortTextAnnot)
		return;

	UIExtensionsManager^ uiMgr = (UIExtensionsManager^)View->ExtensionsManager;
	if (!View || !uiMgr->getAnnotMgr())
		return;

	uiMgr->SetTextMenuPopupHide();
	uiMgr->SetAnnotStampClosed();

	Windows::UI::Xaml::FrameworkElement^ _FrameworkElement = dynamic_cast<Windows::UI::Xaml::FrameworkElement^>(sender);
	Windows::Foundation::Point _Pt;
	_Pt = e->GetPosition(_FrameworkElement);

	auto _annotMgr = uiMgr->getAnnotMgr();
	auto _Pageinfo = View->ConvertDevicePtToPDFPagePt(_Pt);
	int index = _Pageinfo->PageIndex;
	auto _PagePoint = _Pageinfo->PagePoint;
	if (index == -1)
		return;

	IBaseAnnot^ _pSelectedAnnot = nullptr;
	auto _Page = View->GetDoc()->GetPage(index);

	_pSelectedAnnot = _annotMgr->IsHitAnnot(index, _PagePoint);
	auto pSdkAnnot = _Page->GetAnnotAtPoint(ref new foxit::common::fxcrt::PointF(_PagePoint.X, _PagePoint.Y), 10);
	
	if (pSdkAnnot == nullptr)
	{
		if (_pSelectedAnnot && _pSelectedAnnot->AnnotType == AnnotationType::Widget)
		{
			_annotMgr->DeleteAnnot(_pSelectedAnnot);
			_pSelectedAnnot = nullptr;
		}
	}

	if (_pSelectedAnnot && _pSelectedAnnot->AnnotType == AnnotationType::Widget)
	{
		View->EnableScrollView(false);
		uiMgr->SetFormControlTappedPoint(_Pt);
	}
	else
	{
		uiMgr->SetFormControlTappedPoint(Windows::Foundation::Point(0, 0));
	}

	if (uiMgr->CreateTypeOfAnnot == AnnotCreateType::Circle || uiMgr->CreateTypeOfAnnot == AnnotCreateType::Line || uiMgr->CreateTypeOfAnnot == AnnotCreateType::Arrow
		|| uiMgr->CreateTypeOfAnnot == AnnotCreateType::Pencil || uiMgr->CreateTypeOfAnnot == AnnotCreateType::Square
		|| uiMgr->CreateTypeOfAnnot == AnnotCreateType::Eraser || uiMgr->CreateTypeOfAnnot == AnnotCreateType::AreaHighLight)
		return;

	if (uiMgr->getAnnotMgr()->GetSelectedAnnot() && uiMgr->getAnnotMgr()->GetSelectedAnnot()->AnnotType == AnnotationType::FreeText
		&& uiMgr->GetAnnotCanvasHasChild())
	{
		IBaseAnnotEventPeer^ annotEventPeer = uiMgr->getAnnotMgr()->GetAnnotHandlerByType(AnnotationType::FreeText)->AnnotEvent;
		FreeTextEventPeer^ freetextEventPeer = dynamic_cast<FreeTextEventPeer^>(annotEventPeer);
		freetextEventPeer->OnUpdateFreeTextContent();
		uiMgr->SetAnnotCanvasVisible(false);
		uiMgr->getAnnotMgr()->SetPdfSelectAnnotNull();
		return;
	}

	if (uiMgr->CreateTypeOfAnnot == AnnotCreateType::Stamp)
	{		
		auto _pSelectedAnnotHandler = uiMgr->getAnnotMgr()->GetAnnotHandlerByType(AnnotationType::Stamp);
		if (_pSelectedAnnotHandler)
			_pSelectedAnnotHandler->OnTapped(sender, e);		
		return;
	}

	else if (uiMgr->CreateTypeOfAnnot == AnnotCreateType::FreeText)
	{
		auto _pSelectedAnnotHandler = uiMgr->getAnnotMgr()->GetAnnotHandlerByType(AnnotationType::FreeText);
		if (_pSelectedAnnotHandler)
			_pSelectedAnnotHandler->OnTapped(sender, e);
		return;
	}
	else if (uiMgr->CreateTypeOfAnnot == AnnotCreateType::Text && uiMgr->ExitNotePopupEdit())
	{
		if (-1 == View->GetPageAtPoint(e->GetPosition(_FrameworkElement)))
		{
			return;
		}
		auto _pSelectedAnnotHandler = uiMgr->getAnnotMgr()->GetAnnotHandlerByType(AnnotationType::Text);
		if (_pSelectedAnnotHandler)
			_pSelectedAnnotHandler->OnTapped(sender, e);
	}
	
	if ((_pSelectedAnnot && _pSelectedAnnot->AnnotType != AnnotationType::Widget) || (_pSelectedAnnot == nullptr && uiMgr->IsWidgetAnnotPopupOpend()))
	{
		uiMgr->HideWidgetAnnotPopup();
	}

	if ((uiMgr->CreateTypeOfAnnot == AnnotCreateType::Text ||
		_pSelectedAnnot && _pSelectedAnnot->AnnotType != AnnotationType::Text) && !uiMgr->ExitNotePopupEdit())
	{
		return;
	}

	if (!_pSelectedAnnot && !isShortTextAnnot && uiMgr->CreateTypeOfAnnot != AnnotCreateType::Text  &&  uiMgr->CreateTypeOfAnnot != AnnotCreateType::FreeText)
	{
		if (_annotMgr->GetSelectedAnnot())
		{
			uiMgr->ExitAnnotSelect();
		}
		
		else if (uiMgr->CreateTypeOfAnnot == AnnotCreateType::HightLight || uiMgr->CreateTypeOfAnnot == AnnotCreateType::UnderLine
			|| uiMgr->CreateTypeOfAnnot == AnnotCreateType::Squiggly || uiMgr->CreateTypeOfAnnot == AnnotCreateType::StrikeOut)
		{
			if (!isShortTextAnnot)
				return;
		}
		else
		{
			uiMgr->ShowOrHideAppBar();
		}

		uiMgr->ExitNotePopupEdit();
	}
	else if (_pSelectedAnnot && _pSelectedAnnot->AnnotType == AnnotationType::Link)
	{
		if (_annotMgr->GetSelectedAnnot())
		{
			uiMgr->ExitAnnotSelect();
		}
		_annotMgr->SetSelectedAnnot(_pSelectedAnnot);
		auto _pSelectedAnnotHandler = _annotMgr->GetAnnotHandlerByType(AnnotationType::Link);
		if (_pSelectedAnnotHandler)
			_pSelectedAnnotHandler->OnTapped(sender, e);

		e->Handled = true;
	}
	else
	{
		if (_pSelectedAnnot)
		{
			if (uiMgr->IsShowAnnot)
			{
				if (uiMgr->CreateTypeOfAnnot != AnnotCreateType::Text)
				{
					if (_annotMgr->GetSelectedAnnot())
					{
						_annotMgr->SetPdfSelectAnnotNull();
						uiMgr->SetAnnotCanvasVisible(false);
					}
					if (_pSelectedAnnot)
						uiMgr->getAnnotMgr()->SetSelectedAnnot(_pSelectedAnnot);
				}
			}
			else
			{
				if (_pSelectedAnnot->AnnotType == AnnotationType::Widget)
					uiMgr->getAnnotMgr()->SetSelectedAnnot(_pSelectedAnnot);
			}
		}

		OnTapEvent(sender, e);
	}
}

void PageViewAnnotMgrPeer::OnTapEvent(Platform::Object^ sender, Windows::UI::Xaml::Input::TappedRoutedEventArgs^ e)
{
	UIExtensionsManager^ uiMgr = (UIExtensionsManager^)View->ExtensionsManager;
	if (!uiMgr->getAnnotMgr() || !View)
		return;

	Windows::UI::Xaml::FrameworkElement^ _FrameworkElement = dynamic_cast<Windows::UI::Xaml::FrameworkElement^>(sender);
	if (uiMgr->getAnnotMgr()->GetSelectedAnnot())
	{
		IBaseAnnot^ _pSelectedAnnot = uiMgr->getAnnotMgr()->GetSelectedAnnot();

		if(_pSelectedAnnot->AnnotType != AnnotationType::Widget)
			_FrameworkElement->ManipulationMode = Windows::UI::Xaml::Input::ManipulationModes::All;
		uiMgr->CloseAddNoteWindows();
		
		uiMgr->ExitAnnotPropertyMode();

		auto _pSelectedAnnotHandler = uiMgr->getAnnotMgr()->GetAnnotHandlerByType(_pSelectedAnnot->AnnotType);
		if (_pSelectedAnnotHandler)
			_pSelectedAnnotHandler->OnTapped(sender, e);

		e->Handled = true;
		return;
	}

	if (View->GetDoc() && !FSPDFDocUtil::GetIsDocumentCanModifyAnnotation(View->GetDoc()) && View->GetDoc()->GetPasswordType() != foxit::pdf::PasswordType::e_PwdOwner)
	{
		return;
	}

	if (isShortTextAnnot)
	{
		_FrameworkElement->ManipulationMode = Windows::UI::Xaml::Input::ManipulationModes::All;

		auto _pSelectedAnnotHandler = uiMgr->getAnnotMgr()->GetAnnotHandlerByType(GetAnnotType(m_createTypeOfAnnot));
		if (_pSelectedAnnotHandler)
			_pSelectedAnnotHandler->OnTapped(sender, e);
		isShortTextAnnot = false;
	}	
}

void PageViewAnnotMgrPeer::OnAnnotRightTapped(Platform::Object^ sender, Windows::UI::Xaml::Input::RightTappedRoutedEventArgs^ e)
{
	UIExtensionsManager^ uiMgr = (UIExtensionsManager^)View->ExtensionsManager;
	auto _annotMgr = uiMgr->getAnnotMgr();
	if (!_annotMgr)
		return;

	auto _pAnnotHandler = _annotMgr->GetAnnotHandlerByType(GetAnnotType(uiMgr->CreateTypeOfAnnot));
	if (_pAnnotHandler)
		_pAnnotHandler->OnRightTapped(sender, e);
}

void PageViewAnnotMgrPeer::OnAnnotManipulationStarted(Platform::Object^ sender, Windows::UI::Xaml::Input::ManipulationStartedRoutedEventArgs^ e)
{
	if (View->GetDoc() && !FSPDFDocUtil::GetIsDocumentCanModifyAnnotation(View->GetDoc()) && View->GetDoc()->GetPasswordType() != foxit::pdf::PasswordType::e_PwdOwner)
		return;

	UIExtensionsManager^ uiMgr = (UIExtensionsManager^)View->ExtensionsManager;
	auto _annotMgr = uiMgr->getAnnotMgr();
	if (!_annotMgr)
		return;
	IBaseAnnotHandler^ _pAnnotHandler;
	if (uiMgr->CreateTypeOfAnnot != AnnotCreateType::None)
	{
		_pAnnotHandler = _annotMgr->GetAnnotHandlerByType(GetAnnotType(uiMgr->CreateTypeOfAnnot));
	}
	else if (_annotMgr->GetSelectedAnnot() && _annotMgr->GetSelectedAnnot()->AnnotType == AnnotationType::Text)
	{
		if (!uiMgr->ExitNotePopupEdit())
		{
			return;
		}
		TextAnnot^ textAnnot = dynamic_cast<TextAnnot^>(_annotMgr->GetSelectedAnnot());
		auto _Pageinfo = View->ConvertDevicePtToPDFPagePt(e->Position);

		if (textAnnot->DrawRect.Contains(_Pageinfo->PagePoint))
		{
			_pAnnotHandler = _annotMgr->GetAnnotHandlerByType(AnnotationType::Text);
		}
		else
		{
			_annotMgr->SetPdfSelectAnnotNull();
			uiMgr->SetAnnotCanvasVisible(false);
		}
	}

	if (_pAnnotHandler)
		_pAnnotHandler->OnManipulationStarted(sender, e);
}

void PageViewAnnotMgrPeer::OnAnnotManipulationDelta(Platform::Object^ sender, Windows::UI::Xaml::Input::ManipulationDeltaRoutedEventArgs^ e)
{
	if (View->GetDoc() && !FSPDFDocUtil::GetIsDocumentCanModifyAnnotation(View->GetDoc()) && View->GetDoc()->GetPasswordType() != foxit::pdf::PasswordType::e_PwdOwner)
		return;
	UIExtensionsManager^ uiMgr = (UIExtensionsManager^)View->ExtensionsManager;
	auto _annotMgr = uiMgr->getAnnotMgr();
	if (!_annotMgr)
		return;
	IBaseAnnotHandler^ _pAnnotHandler;
	if (uiMgr->CreateTypeOfAnnot != AnnotCreateType::None)
	{
		_pAnnotHandler = _annotMgr->GetAnnotHandlerByType(GetAnnotType(uiMgr->CreateTypeOfAnnot));
	}
	else if (_annotMgr->GetSelectedAnnot() && _annotMgr->GetSelectedAnnot()->AnnotType == AnnotationType::Text)
	{
		if (!uiMgr->ExitNotePopupEdit())
		{
			return;
		}
		_pAnnotHandler = _annotMgr->GetAnnotHandlerByType(AnnotationType::Text);
	}
	
	if (_pAnnotHandler)
		_pAnnotHandler->OnManipulationDelta(sender, e);
}

void PageViewAnnotMgrPeer::OnAnnotManipulationCompleted(Platform::Object^ sender, Windows::UI::Xaml::Input::ManipulationCompletedRoutedEventArgs^ e)
{
	if (View->GetDoc() && !FSPDFDocUtil::GetIsDocumentCanModifyAnnotation(View->GetDoc()) && View->GetDoc()->GetPasswordType() != foxit::pdf::PasswordType::e_PwdOwner)
		return;
	UIExtensionsManager^ uiMgr = (UIExtensionsManager^)View->ExtensionsManager;
	auto _annotMgr = uiMgr->getAnnotMgr();
	if (!_annotMgr)
		return;
	IBaseAnnotHandler^ _pAnnotHandler;
	if (uiMgr->CreateTypeOfAnnot != AnnotCreateType::None)
	{
		_pAnnotHandler = _annotMgr->GetAnnotHandlerByType(GetAnnotType(uiMgr->CreateTypeOfAnnot));
	}
	else if (_annotMgr->GetSelectedAnnot() && _annotMgr->GetSelectedAnnot()->AnnotType == AnnotationType::Text)
	{
		if (!uiMgr->ExitNotePopupEdit())
		{
			return;
		}
		_pAnnotHandler = _annotMgr->GetAnnotHandlerByType(AnnotationType::Text);
	}
	
	if (_pAnnotHandler)
		_pAnnotHandler->OnManipulationCompleted(sender, e);
}

void PageViewAnnotMgrPeer::OnManipulationInertiaStarting(Platform::Object^ sender, Windows::UI::Xaml::Input::ManipulationInertiaStartingRoutedEventArgs^ e)
{
	UIExtensionsManager^ uiMgr = (UIExtensionsManager^)View->ExtensionsManager;
	auto _annotMgr = uiMgr->getAnnotMgr();
	if (!_annotMgr)
		return;

	IBaseAnnotHandler^ _pAnnotHandler;

	if (_annotMgr->GetSelectedAnnot() && _annotMgr->GetSelectedAnnot()->AnnotType == AnnotationType::Text  && uiMgr->ExitNotePopupEdit())
	{
		_pAnnotHandler = _annotMgr->GetAnnotHandlerByType(AnnotationType::Text);
	}
	else if (_annotMgr->GetSelectedAnnot() && _annotMgr->GetSelectedAnnot()->AnnotType == AnnotationType::Ink)
	{
		_pAnnotHandler = _annotMgr->GetAnnotHandlerByType(AnnotationType::Ink);
	}
	if (_pAnnotHandler)
		_pAnnotHandler->OnManipulationInertiaStarting(sender, e);
}

void PageViewAnnotMgrPeer::SaveAnonotModifyBeforeProperty()
{
	UIExtensionsManager^ uiMgr = (UIExtensionsManager^)View->ExtensionsManager;
	auto selectAnnot = uiMgr->getAnnotMgr()->GetSelectedAnnot();
	if (selectAnnot)
	{
		selectAnnot->IsModifyProperty = false;
		if ((selectAnnot->AnnotType == AnnotationType::StrikeOut || selectAnnot->AnnotType == AnnotationType::Caret)
			&& uiMgr->getAnnotMgr()->GetGroupAnnot())
		{
			Windows::Foundation::Collections::IVector<IBaseAnnot^>^ newannots = ref new Platform::Collections::Vector<IBaseAnnot^>();
			newannots->Append(selectAnnot);
			newannots->Append(uiMgr->getAnnotMgr()->GetGroupAnnot());
			modifyReplaceAnnotAction = ref new ModifyReplaceAnnotUndoAction(View, newannots);
		}
		else
		{
			modifyannotAction = ref new ModifyAnnotUndoAction(View, selectAnnot);
		}
	}
}

void PageViewAnnotMgrPeer::InsertAnnotModifyUndo()
{
	if (modifyannotAction || modifyReplaceAnnotAction)
	{
		UIExtensionsManager^ uiMgr = (UIExtensionsManager^)View->ExtensionsManager;
		auto selectAnnot = uiMgr->getAnnotMgr()->GetSelectedAnnot();
		if (selectAnnot && selectAnnot->IsModifyProperty)
		{
			if ((selectAnnot->AnnotType == AnnotationType::StrikeOut || selectAnnot->AnnotType == AnnotationType::Caret)
				&& uiMgr->getAnnotMgr()->GetGroupAnnot())
			{
				UndoFactory::GetUndoInstance()->AddUndoItem(uiMgr, modifyReplaceAnnotAction);
			}
			else
			{
				UndoFactory::GetUndoInstance()->AddUndoItem(uiMgr, modifyannotAction);
			}

			foxit::common::fxcrt::RectF^ minRect = FoxitRectConverter::ConvertToFoxitRectF(selectAnnot->MinRect);
			String^ rectStr = "[" + minRect->left.ToString() + " " + minRect->top.ToString() + " " + minRect->right.ToString() + " " + minRect->bottom.ToString() + "]";

			Platform::String^ annotTypeStr = uiMgr->GetAnnotTypeString(selectAnnot->AnnotType);
			uiMgr->CollectAnnotOperation(annotTypeStr, "Edit", selectAnnot->PageIndex.ToString(), rectStr);
		}
		if (View)
			uiMgr->UpdateAnnotToolUnReDoControl();
	}
}


void foxit::rdk::uiextensions::PageViewAnnotMgrPeer::OnRenderEvent(foxit::rdk::PDFViewCtrl ^sender, RenderCanvasEventArgs^ e)
{
	auto _PageIndex = e->PageIndex;
	
	auto _UIView = UIExtMgr;
	auto _PageView = View;
	if (_UIView)
	{
		Windows::Foundation::Rect pageRect = _PageView->GetPageViewRect(e->PageIndex);

		if (_UIView->IsShowAnnot)
		{
			Windows::Foundation::Collections::IVector<IBaseAnnot^>^ noteAnnots = _UIView->getAnnotMgr()->GetPageNotes(e->PageIndex);
			
			if (noteAnnots && noteAnnots->Size > 0)
			{
				for (unsigned int i = 0; i < noteAnnots->Size; i++)
				{
					if (noteAnnots->GetAt(i)->AnnotType == AnnotationType::Text)
					{
						TextAnnot^ textAnnot = dynamic_cast<TextAnnot^>(noteAnnots->GetAt(i));
						bool isHasDrawRect = true;
						if (textAnnot->DrawRect.Width <= 0 || textAnnot->DrawRect.Height <= 0)
						{
							isHasDrawRect = false;
						}
						Windows::Foundation::Rect rect = noteAnnots->GetAt(i)->MinRect;
						auto _deviceRc = _PageView->ConvertPageRectToDeviceRect(e->PageIndex, rect);
							
						float maxwidth = 30;
						if (Utility::IsMobilePlatform())
						{
							_deviceRc.Width = maxwidth;
							_deviceRc.Height = maxwidth;
						}
						else
						{
							double maxheight = maxwidth;
							maxwidth = maxwidth;

							_deviceRc.Width = (float)(maxwidth);
							_deviceRc.Height = (float)(maxheight);

						}
						if (_deviceRc.X < pageRect.X)
						{
							_deviceRc.X = (float)(pageRect.X);
						}
						if (_deviceRc.X + _deviceRc.Width > pageRect.X + pageRect.Width)
						{
							_deviceRc.X = (float)(pageRect.X + pageRect.Width - _deviceRc.Width);
						}
						if (_deviceRc.Y < pageRect.Y)
						{
							_deviceRc.Y = (float)(pageRect.Y);
						}
						if (_deviceRc.Y + _deviceRc.Height > pageRect.Y + pageRect.Height)
						{
							_deviceRc.Y = (float)(pageRect.Y + pageRect.Height - _deviceRc.Height);
						}

						auto _DevicePageRc = _PageView->ConvertDeviceRectToPageRect(e->PageIndex,_deviceRc); //FSPDFDocUtil::DeviceToPageRect(_PageView->GetDoc(), nPageIndex, pageRect.X, pageRect.Y,

						textAnnot->DrawRect = _DevicePageRc;

						Microsoft::WRL::ComPtr<IWICFormatConverter> image = _UIView->LoadPicSource(textAnnot->IconType.ToString());
						if (image)
						{
							WICPixelFormatGUID format;
							image->GetPixelFormat(&format);
							if (format != GUID_WICPixelFormat32bppPBGRA)
								return;
							UINT width, height;
							image->GetSize(&width, &height);
							UINT stride = width * sizeof(int);
							UINT cbBufferSize = stride*height;
							BYTE* buffer = (BYTE*)malloc(cbBufferSize);
							image->CopyPixels(NULL, stride, cbBufferSize, buffer);

							Windows::Storage::Streams::DataWriter^ writer = ref new Windows::Storage::Streams::DataWriter();
							writer->WriteBytes(Platform::ArrayReference<BYTE>(buffer, cbBufferSize));
							Windows::Storage::Streams::IBuffer^ iBuffer = writer->DetachBuffer();

							foxit::common::Bitmap^ bitmap = ref new foxit::common::Bitmap(width, height, foxit::common::DIBFormat::e_DIBArgb, iBuffer, stride);
							Windows::Foundation::Rect srcRect{
								0, 0, (float)bitmap->GetWidth(), (float)bitmap->GetHeight()
							};
							Windows::Foundation::Rect dstRect{
								_deviceRc.X, _deviceRc.Y,
								_deviceRc.Width, _deviceRc.Height
							};
							e->Device->DrawBitmap(bitmap, srcRect, dstRect);
							free(buffer);
						}
					}
					else if (noteAnnots->GetAt(i)->AnnotType == AnnotationType::Link)
					{
						auto _AnnotSetting = _UIView->AnnotMgr->GetAnnotSetting();
						if (_AnnotSetting->GetIsLinkHighlight())
						{
							float _DrawBorderWidth = 3;

							auto _DeviceRc = _PageView->ConvertPageRectToDeviceRect(noteAnnots->GetAt(i)->PageIndex, noteAnnots->GetAt(i)->MinRect);  //FSPDFDocUtil::PageToDeviceRect(_PageView->GetDoc(), noteAnnots->GetAt(i)->PageIndex, pageRect.X, pageRect.Y,

							double rightvalue = _DeviceRc.Right;
							if (rightvalue > pageRect.X + pageRect.Width)
								rightvalue = pageRect.X + pageRect.Width;
							D2D1_RECT_F const& _Bounds = D2D1::RectF(
								_DeviceRc.Left, _DeviceRc.Top - _DrawBorderWidth,
								(float)rightvalue, _DeviceRc.Bottom + _DrawBorderWidth
							);

							Windows::UI::Color _Color = Windows::UI::ColorHelper::FromArgb(0xff, 0x80, 0x80, 0xff);
							Windows::Foundation::Rect _DrawRect(_Bounds.left, _Bounds.top, _Bounds.right - _Bounds.left, _Bounds.bottom - _Bounds.top);
							e->Device->DrawRectangle(_DrawRect, _Color, _DrawBorderWidth);
							
						}
					}
				}
			}

			auto _selectAnnot = _UIView->AnnotMgr->GetSelectedAnnot();
			if (_selectAnnot)
			{
				if (e->PageIndex == _selectAnnot->PageIndex)
				{
					float _DrawBorderWidth = 2 * View->CompositionScaleX;
					float _MarginValue = 5;
					Windows::Foundation::Rect rect = _selectAnnot->MinRect;

					if ((_selectAnnot->AnnotType == AnnotationType::StrikeOut || _selectAnnot->AnnotType == AnnotationType::Caret)
						&& _UIView->AnnotMgr->GetGroupAnnot())
					{
						if (_selectAnnot->AnnotType == AnnotationType::Caret)
						{
							rect = _UIView->AnnotMgr->GetGroupAnnot()->MinRect;
						}
					}

					if (_selectAnnot->AnnotType == AnnotationType::Text)
					{
						_MarginValue = 3;
						TextAnnot^ textAnnot = dynamic_cast<TextAnnot^>(_selectAnnot);
						rect = textAnnot->DrawRect;
					}
					else if (_selectAnnot->AnnotType == AnnotationType::Widget)
					{

					}
					else
					{
						if (_selectAnnot->AnnotType == AnnotationType::Caret)
							_MarginValue = 3;
						_MarginValue = _MarginValue;
						rect.X = rect.X - _MarginValue;
						rect.Y = rect.Y - _MarginValue;
						rect.Width = rect.Width + _MarginValue * 2;
						rect.Height = rect.Height + _MarginValue * 2;
					}
					auto _DeviceRc = _PageView->ConvertPageRectToDeviceRect(_selectAnnot->PageIndex, rect); //FSPDFDocUtil::PageToDeviceRect(_PageView->GetDoc(), _selectAnnot->PageIndex, pageRect.X, pageRect.Y,

					if (_selectAnnot->AnnotType == AnnotationType::StrikeOut)
					{
						StrikeOutAnnot^ strikeoutannot = dynamic_cast<StrikeOutAnnot^>(_selectAnnot);
						if (strikeoutannot->IsHasGroup)
						{
							if (_UIView->AnnotMgr->IsGetGroupAnnotElement(strikeoutannot->PageIndex, strikeoutannot->GroupAnnotMinRect,
								strikeoutannot->GroupAnnotIndex, AnnotationType::Caret))
							{
								IBaseAnnot^ caretAnnot = _UIView->AnnotMgr->GetGroupAnnot();

								auto _DeviceCaretRc = _PageView->ConvertPageRectToDeviceRect(_selectAnnot->PageIndex, caretAnnot->MinRect); //FSPDFDocUtil::PageToDeviceRect(_PageView->GetDoc(), _selectAnnot->PageIndex, pageRect.X, pageRect.Y,

								if (_DeviceCaretRc.X + _DeviceCaretRc.Width > _DeviceRc.X + _DeviceRc.Width)
								{
									_DeviceRc.Width += _DeviceCaretRc.X + _DeviceCaretRc.Width - (_DeviceRc.X + _DeviceRc.Width) + 5;
								}
								if (_DeviceCaretRc.Y + _DeviceCaretRc.Height > _DeviceRc.Y + _DeviceRc.Height)
								{
									_DeviceRc.Height += _DeviceCaretRc.Y + _DeviceCaretRc.Height - (_DeviceRc.Y + _DeviceRc.Height) + 5;
								}
							}
						}
					}

					if (_selectAnnot->AnnotType == AnnotationType::Text)
					{
						_DeviceRc.X = _DeviceRc.X - _MarginValue,
							_DeviceRc.Y = _DeviceRc.Y - _MarginValue;
						_DeviceRc.Width = _DeviceRc.Width + _MarginValue * 2;
						_DeviceRc.Height = _DeviceRc.Height + _MarginValue * 2;
					}

					D2D1_RECT_F const& _Bounds = D2D1::RectF(
						_DeviceRc.X, _DeviceRc.Y,
						_DeviceRc.X + _DeviceRc.Width, _DeviceRc.Y + _DeviceRc.Height
					);

					Microsoft::WRL::ComPtr<ID2D1SolidColorBrush> _Brush;
					Windows::UI::Color _Color;
					if (_selectAnnot->AnnotType == AnnotationType::HightLight || _selectAnnot->AnnotType == AnnotationType::Text)
					{
						_Color = Windows::UI::ColorHelper::FromArgb(0xff, 0x17, 0x9c, 0xd8);
					}
					else
					{
						_Color = _selectAnnot->Color;
					}

					Windows::Foundation::Rect _DrawRect(_Bounds.left, _Bounds.top, _Bounds.right - _Bounds.left, _Bounds.bottom - _Bounds.top);
					if (_selectAnnot->AnnotType == AnnotationType::HightLight/* || _selectAnnot->AnnotType == AnnotationType::Text*/)
					{
						e->Device->DrawRectangle(_DrawRect, _Color, _DrawBorderWidth);
					}
					else if (_selectAnnot->AnnotType == AnnotationType::UnderLine || _selectAnnot->AnnotType == AnnotationType::StrikeOut
						|| _selectAnnot->AnnotType == AnnotationType::Squiggly || _selectAnnot->AnnotType == AnnotationType::Caret)
					{
						e->Device->DrawRectangle(_DrawRect, _Color);
					}
					else if (_selectAnnot->AnnotType == AnnotationType::Widget)
					{

					}

				}
			}
			else
			{
				if (_UIView->CreateTypeOfAnnot == AnnotCreateType::AreaHighLight)
				{
					Windows::Foundation::Rect  areaAnnotRect = _UIView->AnnotMgr->GetNewCreateAnnotRect();
					if (areaAnnotRect.Width > 0 && areaAnnotRect.Height > 0)
					{
						e->Device->FillRectangle(areaAnnotRect, _UIView->AnnotMgr->GetAnnotSetting()->GetHightLightAnnotColor());
						e->Device->DrawRectangle(areaAnnotRect, _UIView->AnnotMgr->GetAnnotSetting()->GetHightLightAnnotColor());
					}
				}
			}
		}
	}
}

void foxit::rdk::uiextensions::PageViewAnnotMgrPeer::OnPageLoaded(PDFViewCtrl ^ sender, PageLoadedEventArgs ^ e)
{
	UIExtMgr->AnnotMgr->OnInitAnnot(e->PageIndex);
}

void foxit::rdk::uiextensions::PageViewAnnotMgrPeer::OnPageUnloaded(PDFViewCtrl ^ sender, PageUnloadedEventArgs ^ e)
{
	UIExtMgr->AnnotMgr->FinalizeAnnots(e->PageIndex);
}
