/**
 * Copyright (C) 2003-2018, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit Mobile PDF SDK to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */


#include "pch.h"
#include "StampAnnotEventPeer.h"


using namespace Platform;
using namespace Windows::Foundation;

using namespace foxit::rdk;
using namespace foxit::rdk::uiextensions;


namespace foxit
{
	namespace rdk
	{
		namespace uiextensions
		{
			[Windows::Foundation::Metadata::WebHostHidden]
			public ref class CreateStampAnnotUndoAction sealed : IUndoItem
			{
			public:
				CreateStampAnnotUndoAction(PDFViewCtrl^ view, StampAnnot^ annot);

				virtual void Undo();
				virtual void Redo();
				virtual Platform::String^ GetDescr();
				virtual void ReleaseItem();

			internal:
				property PDFViewCtrl^ View
				{
					PDFViewCtrl^ get();
				}

				property StampAnnot^ AnnotHandle
				{
					StampAnnot^ get();
				}

			private:
				Platform::WeakReference m_ViewWeakRef;
				Platform::WeakReference m_AnnotWeakRef;
				BaseAnnot^ m_RedoAnnot;
			};
		}
	}
}

CreateStampAnnotUndoAction::CreateStampAnnotUndoAction(PDFViewCtrl^ view, StampAnnot^ annot)
{
	m_ViewWeakRef = view;
	m_AnnotWeakRef = annot;
}

PDFViewCtrl^ CreateStampAnnotUndoAction::View::get()
{
	return m_ViewWeakRef.Resolve<PDFViewCtrl>();
}

StampAnnot^ CreateStampAnnotUndoAction::AnnotHandle::get()
{
	return m_AnnotWeakRef.Resolve<StampAnnot>();
}

void CreateStampAnnotUndoAction::Undo()
{
	if (View && AnnotHandle)
	{
		m_RedoAnnot = AnnotHandle;
		View->CancelRenderTask();
		AnnotHandle->OnDelete();		
	}
}

void CreateStampAnnotUndoAction::Redo()
{
	if (View && m_RedoAnnot)
	{
		foxit::pdf::PDFPage^ page = View->GetDoc()->GetPage(m_RedoAnnot->PageIndex);

		View->CancelRenderTask();
		UIExtensionsManager^ uiMgr = (UIExtensionsManager^)View->ExtensionsManager;
		uiMgr->getAnnotMgr()->OnCreateNewAnnot(page, m_RedoAnnot);
	}
}

Platform::String^ CreateStampAnnotUndoAction::GetDescr()
{
	return ref new Platform::String(L"");
}

void CreateStampAnnotUndoAction::ReleaseItem()
{
	m_ViewWeakRef = nullptr;
	m_AnnotWeakRef = nullptr;
	m_RedoAnnot = nullptr;
}



StampAnnotEventPeer::StampAnnotEventPeer(PDFViewCtrl^ view)
{
	m_PageViewWeakRef = view;
	m_pressStartPoint = Point(-1, -1);
}

PDFViewCtrl^ StampAnnotEventPeer::View::get()
{
	return m_PageViewWeakRef.Resolve<PDFViewCtrl>();
}

void StampAnnotEventPeer::OnPointerPressed(Platform::Object^ sender, Windows::UI::Xaml::Input::PointerRoutedEventArgs^ e)
{

}

void StampAnnotEventPeer::OnPointerMoved(Platform::Object^ sender, Windows::UI::Xaml::Input::PointerRoutedEventArgs^ e)
{

}

void StampAnnotEventPeer::OnPointerReleased(Platform::Object^ sender, Windows::UI::Xaml::Input::PointerRoutedEventArgs^ e)
{
	m_pressStartPoint = Point(-1, -1);
}

void StampAnnotEventPeer::OnTapped(Platform::Object^ sender, Windows::UI::Xaml::Input::TappedRoutedEventArgs^ e)
{
	View->Update();
	Windows::UI::Xaml::FrameworkElement^ _FrameworkElement = dynamic_cast<Windows::UI::Xaml::FrameworkElement^>(sender);
	Windows::Foundation::Point curPt = e->GetPosition(_FrameworkElement);
	UIExtensionsManager^ uiMgr = (UIExtensionsManager^)View->ExtensionsManager;
	auto _SelectedAnnot = uiMgr->getAnnotMgr()->GetSelectedAnnot();
	if (_SelectedAnnot)
	{
		if (View->GetDoc() && !FSPDFDocUtil::GetIsDocumentCanModifyAnnotation(View->GetDoc()) && View->GetDoc()->GetPasswordType() != foxit::pdf::PasswordType::e_PwdOwner)
		{

		}
		else
		{
			foxit::rdk::uiextensions::StampAnnot^ annot = dynamic_cast<foxit::rdk::uiextensions::StampAnnot^>(_SelectedAnnot);
			uiMgr->CreateRectangelAnnotToCanvas(annot->PageIndex, annot->Page_HightLightRect, annot->Opacity, annot->BorderWidth, annot->Color);

			Windows::Foundation::Rect rect = annot->Page_HightLightRect;
			rect.X = rect.X - 5;
			rect.Y = rect.Y - 5;
			rect.Width = rect.Width + 5 * 2;
			rect.Height = rect.Height + 5 * 2;
			auto deviceRect = View->ConvertPageRectToDeviceRect(uiMgr->getAnnotMgr()->GetSelectedAnnot()->PageIndex, rect);
			Point pt;

			pt = Point(deviceRect.X + deviceRect.Width / 2, deviceRect.Y);
			uiMgr->ShowTextmarkContextmenu(AnnotContextMenuType::Delete, pt);
		}
	}
	else
	{
		auto _Pagepoint = View->ConvertDevicePtToPDFPagePt(curPt);

		if (_Pagepoint->PagePoint.X == -1 || _Pagepoint->PagePoint.Y == -1)
			return;

		auto _Annot = AnnotCreaterFactory::GetInstance()->CreateNewAnnot(AnnotationType::Stamp);
		auto _pCreateAnnot = dynamic_cast<StampAnnot^>(_Annot);

		Windows::Foundation::Rect AnnotRect;
		AnnotRect.X = curPt.X;
		AnnotRect.Y = curPt.Y;

		int pageRotate = View->GetRotate();
		pageRotate = (int) View->GetDoc()->GetPage(_Pagepoint->PageIndex)->GetRotation();


		Windows::Foundation::Rect pageRect= View->GetPageViewRect(_Pagepoint->PageIndex);
	
		Windows::Foundation::Rect stampRect= Windows::Foundation::Rect(_Pagepoint->PagePoint.X, _Pagepoint->PagePoint.Y,78,25);
		if (Windows::System::Profile::AnalyticsInfo::VersionInfo->DeviceFamily == L"Windows.Mobile")
		{
			stampRect = Windows::Foundation::Rect(_Pagepoint->PagePoint.X, _Pagepoint->PagePoint.Y, 122, 38);
		}

		AnnotRect = View->ConvertPageRectToDeviceRect(_Pagepoint->PageIndex, stampRect);

		if (AnnotRect.Y<pageRect.Y)
			AnnotRect.Y = pageRect.Y;
		if (AnnotRect.X + AnnotRect.Width > pageRect.X + pageRect.Width)
			AnnotRect.X = pageRect.X + pageRect.Width - AnnotRect.Width;
		if (AnnotRect.Y + AnnotRect.Height > pageRect.Y + pageRect.Height)
			AnnotRect.Y = pageRect.Y + pageRect.Height - AnnotRect.Height;

		auto deviceRc = View->ConvertDeviceRectToPageRect(_Pagepoint->PageIndex, AnnotRect);
		stampRect = deviceRc;

		if (pageRotate == 1 || pageRotate == 3)
		{
			double widthvalue = stampRect.Width;
			stampRect.Width = stampRect.Height;
			stampRect.Height = (float)widthvalue;
		}

		_pCreateAnnot->Page_HightLightRect = stampRect;
		int index=uiMgr->getAnnotMgr()->GetAnnotSetting()->GetStampIndex();
		
		_pCreateAnnot->Subject = GetSubjectFromDefaultStampIndex((int)index);
		_pCreateAnnot->DefaultStampIndex = (int)index + 1;

		
		auto page = View->GetDoc()->GetPage(_Pagepoint->PageIndex);

		uiMgr->getAnnotMgr()->OnCreateNewAnnot(page, _pCreateAnnot);

		auto pUndoItem = ref new CreateStampAnnotUndoAction(View, _pCreateAnnot);
		UndoFactory::GetUndoInstance()->AddUndoItem(uiMgr, pUndoItem);

		Windows::Foundation::Rect rc = _pCreateAnnot->MinRect;
		foxit::common::fxcrt::RectF^ minRect = FoxitRectConverter::ConvertToFoxitRectF(_pCreateAnnot->MinRect);
		String^ rectStr = "[" + minRect->left.ToString() + " " + minRect->top.ToString() + " " + minRect->right.ToString() + " " + minRect->bottom.ToString() + "]";
		uiMgr->CollectAnnotOperation("Stamp", "Add", _Pagepoint->PageIndex.ToString(), rectStr);

		if (!uiMgr->getAnnotMgr()->GetAnnotSetting()->GetStampIsContinueOpera())
		{			
			uiMgr->CreateTypeOfAnnot = AnnotCreateType::None;
			uiMgr->ExitAnnotPropertyMode();
		}
	}
}

void StampAnnotEventPeer::OnDoubleTapped(Platform::Object^ sender, Windows::UI::Xaml::Input::DoubleTappedRoutedEventArgs^ e)
{


}

void StampAnnotEventPeer::OnRightTapped(Platform::Object^ sender, Windows::UI::Xaml::Input::RightTappedRoutedEventArgs^ e)
{


}

void StampAnnotEventPeer::OnManipulationStarted(Platform::Object^ sender, Windows::UI::Xaml::Input::ManipulationStartedRoutedEventArgs^ e)
{
}

void StampAnnotEventPeer::OnManipulationDelta(Platform::Object^ sender, Windows::UI::Xaml::Input::ManipulationDeltaRoutedEventArgs^ e)
{

}

void StampAnnotEventPeer::OnManipulationCompleted(Platform::Object^ sender, Windows::UI::Xaml::Input::ManipulationCompletedRoutedEventArgs^ e)
{
}

void StampAnnotEventPeer::OnCreateAnnotAfterSelectText()
{

}

void StampAnnotEventPeer::OnManipulationInertiaStarting(Platform::Object^ sender, Windows::UI::Xaml::Input::ManipulationInertiaStartingRoutedEventArgs^ e)
{

}

void StampAnnotEventPeer::OnKeyDown(Platform::Object^ sender, Windows::UI::Core::KeyEventArgs^ e)
{

}

void StampAnnotEventPeer::OnCharacterReceived(Platform::Object^ sender, Windows::UI::Core::CharacterReceivedEventArgs ^e)
{

}

Platform::String^ StampAnnotEventPeer::GetSubjectFromDefaultStampIndex(int index)
{
	switch (index)
	{
	case 0:
		return L"Approved";
	case 1:
		return L"Completed";
	case 2:
		return L"Confidential";
	case 3:
		return L"Draft";
	case 4:
		return L"Emergency";
	case 5:
		return L"Expired";
	case 6:
		return L"Final";
	case 7:
		return L"Received";
	case 8:
		return L"Reviewed";
	case 9:
		return L"Revised";
	case 10:
		return L"Verified";
	case 11:
		return L"Void";
	case 12:
		return L"Accepted";
	case 13:
		return L"Initial";
	case 14:
		return L"Rejected";
	case 15:
		return L"SignHere";
	case 16:
		return L"Witness";
	default:
		break;
	}

	return L"";
}