/**
 * Copyright (C) 2003-2018, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit Mobile PDF SDK to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */


#include "pch.h"
#include "StrikeOutAnnotEventPeer.h"
#include "AnnotNoteUserControl.xaml.h"

using namespace foxit::rdk;
using namespace foxit::rdk::uiextensions;

using namespace Platform;
using namespace Windows::Foundation;

namespace foxit
{
	namespace rdk
	{
		namespace uiextensions
		{
			[Windows::Foundation::Metadata::WebHostHidden]
			public ref class CreateStrikeOutAnnotUndoAction sealed : IUndoItem
			{
			public:
				CreateStrikeOutAnnotUndoAction(PDFViewCtrl^ view, StrikeOutAnnot^ annot);

				virtual void Undo();
				virtual void Redo();
				virtual Platform::String^ GetDescr();
				virtual void ReleaseItem();

			internal:
				property PDFViewCtrl^ View
				{
					PDFViewCtrl^ get();
				}

				property StrikeOutAnnot^ AnnotHandle
				{
					StrikeOutAnnot^ get();
				}

			private:
				Platform::WeakReference m_ViewWeakRef;
				Platform::WeakReference m_AnnotWeakRef;
				BaseAnnot^ m_RedoAnnot;
			};
		}
	}
}

CreateStrikeOutAnnotUndoAction::CreateStrikeOutAnnotUndoAction(PDFViewCtrl^ view, StrikeOutAnnot^ annot)
{
	m_ViewWeakRef = view;
	m_AnnotWeakRef = annot;
}

PDFViewCtrl^ CreateStrikeOutAnnotUndoAction::View::get()
{
	return m_ViewWeakRef.Resolve<PDFViewCtrl>();
}

StrikeOutAnnot^ CreateStrikeOutAnnotUndoAction::AnnotHandle::get()
{
	return m_AnnotWeakRef.Resolve<StrikeOutAnnot>();
}

void CreateStrikeOutAnnotUndoAction::Undo()
{
	if (View && AnnotHandle)
	{
		m_RedoAnnot = AnnotHandle;
		View->CancelRenderTask();
		AnnotHandle->OnDelete();
	}
}

void CreateStrikeOutAnnotUndoAction::Redo()
{
	if (View && m_RedoAnnot)
	{
		auto page = View->GetDoc()->GetPage(m_RedoAnnot->PageIndex);
		View->CancelRenderTask();
		UIExtensionsManager^ uiMgr = (UIExtensionsManager^)View->ExtensionsManager;
		uiMgr->getAnnotMgr()->OnCreateNewAnnot(page, m_RedoAnnot);
	}
}

Platform::String^ CreateStrikeOutAnnotUndoAction::GetDescr()
{
	return ref new Platform::String(L"");
}

void CreateStrikeOutAnnotUndoAction::ReleaseItem()
{
	m_ViewWeakRef = nullptr;
	m_AnnotWeakRef = nullptr;
	m_RedoAnnot = nullptr;
}


StrikeOutAnnotEventPeer::StrikeOutAnnotEventPeer(PDFViewCtrl^ view)
{
	m_PageViewWeakRef = view;
	m_TextSelectContext = ref new TextSelectContext();
}

PDFViewCtrl^ StrikeOutAnnotEventPeer::View::get()
{
	return m_PageViewWeakRef.Resolve<PDFViewCtrl>();
}

void StrikeOutAnnotEventPeer::OnPointerPressed(Platform::Object^ sender, Windows::UI::Xaml::Input::PointerRoutedEventArgs^ e)
{
	Windows::UI::Xaml::FrameworkElement^ _FrameworkElement = dynamic_cast<Windows::UI::Xaml::FrameworkElement^>(sender);
	auto _PointerPt = e->GetCurrentPoint(_FrameworkElement);
	Windows::Foundation::Point _Pt = _PointerPt->Position;
	auto _Pageinfo = View->ConvertDevicePtToPDFPagePt(_Pt);
	int index = _Pageinfo->PageIndex;
	auto _PagePoint = _Pageinfo->PagePoint;
	m_TextSelectContext->StartIndex = index;
	m_TextSelectContext->StartPoint = _PagePoint;
	_FrameworkElement->ManipulationMode = Windows::UI::Xaml::Input::ManipulationModes::System;
}

void StrikeOutAnnotEventPeer::OnPointerMoved(Platform::Object^ sender, Windows::UI::Xaml::Input::PointerRoutedEventArgs^ e)
{

}

void StrikeOutAnnotEventPeer::OnPointerReleased(Platform::Object^ sender, Windows::UI::Xaml::Input::PointerRoutedEventArgs^ e)
{
	Windows::UI::Xaml::FrameworkElement^ _FrameworkElement = dynamic_cast<Windows::UI::Xaml::FrameworkElement^>(sender);
	auto _PointerPt = e->GetCurrentPoint(_FrameworkElement);
	Windows::Foundation::Point _Pt = _PointerPt->Position;
	auto _Pageinfo = View->ConvertDevicePtToPDFPagePt(_Pt);
	int index = _Pageinfo->PageIndex;
	auto _PagePoint = _Pageinfo->PagePoint;
	m_TextSelectContext->EndIndex = index;
	m_TextSelectContext->EndPoint = _PagePoint;

	UIExtensionsManager^ uiMgr = (UIExtensionsManager^)View->ExtensionsManager;
	auto _AnnotSetting = uiMgr->getAnnotMgr()->GetAnnotSetting();

	auto _selectedtext = uiMgr->getTextMgr()->SelectedText;
	if (_selectedtext && _selectedtext->Size > 0)
	{
		auto size = _selectedtext->Size;
		bool _bIsClearText = true;
		Windows::Foundation::Rect rect = _selectedtext->GetAt(0)->TextRectVect->GetAt(0);
		for (unsigned int i = 0; i < size; i++)
		{
			auto rectGroup = ref new Platform::Collections::Vector<Windows::Foundation::Rect>();

			for each (Windows::Foundation::Rect gmt in _selectedtext->GetAt(i)->TextRectVect)
			{
				rectGroup->Append(gmt);
			}

			auto _Annot = foxit::rdk::uiextensions::AnnotCreaterFactory::GetInstance()->CreateNewAnnot(AnnotationType::StrikeOut);
			auto _pCreateAnnot = dynamic_cast<StrikeOutAnnot^>(_Annot);
			Platform::String^ colorRGBValue = Utility::ConvertToColorString(_AnnotSetting->GetStrikeOutAnnotColor());

			_pCreateAnnot->Color = _AnnotSetting->GetStrikeOutAnnotColor();
			_pCreateAnnot->Opacity = _AnnotSetting->GetStrikeOutAnnotOpacity();
			_pCreateAnnot->Page_HightLightRectGroup = rectGroup;
			_pCreateAnnot->Name = uiMgr->getAnnotMgr()->GetAnnotSetting()->GetNoteAuthor();
			_pCreateAnnot->Title = _AnnotSetting->GetNoteAuthor();
			_pCreateAnnot->CreationDateTime = (ref new Windows::Globalization::Calendar())->GetDateTime();
			_pCreateAnnot->TextRotateVect = _selectedtext->GetAt(i)->TextRotateVect;
			auto _selectpageindex = _selectedtext->GetAt(i)->PageIndex;
			auto _page = View->GetDoc()->GetPage(_selectpageindex);

			auto _createtype = uiMgr->CreateTypeOfAnnot;

			if (_createtype == AnnotCreateType::Comment)
			{
				if (index == _selectpageindex)
				{
					_bIsClearText = false;
					AnnotCommandToken^ token = ref new AnnotCommandToken();
					token->Pageindex = index;
					token->Param = _pCreateAnnot;
					token->Type = AnnotCommandType::Add;
				}
				else
				{
					uiMgr->getAnnotMgr()->OnCreateNewAnnot(_page, _pCreateAnnot);
				}
			}
			else if (_createtype == AnnotCreateType::StrikeOut)
			{
				uiMgr->getAnnotMgr()->OnCreateNewAnnot(_page, _pCreateAnnot);

				auto pUndoItem = ref new CreateStrikeOutAnnotUndoAction(View, _pCreateAnnot);
				UndoFactory::GetUndoInstance()->AddUndoItem(uiMgr, pUndoItem);

				Windows::Foundation::Rect rc = _pCreateAnnot->MinRect;
				foxit::common::fxcrt::RectF^ minRect = FoxitRectConverter::ConvertToFoxitRectF(_pCreateAnnot->MinRect);
				String^ rectStr = "[" + minRect->left.ToString() + " " + minRect->top.ToString() + " " + minRect->right.ToString() + " " + minRect->bottom.ToString() + "]";
				uiMgr->CollectAnnotOperation("Strikeout", "Add", _selectpageindex.ToString(), rectStr);
			}			
		}

		if (_bIsClearText)
			uiMgr->getTextMgr()->ClearTextSelect();


		if (!_AnnotSetting->GetStrikeOutIsContinueOpera() || uiMgr->IsRightMenuAdd)
		{
			uiMgr->ExitAnnotPropertyMode();
			uiMgr->CreateTypeOfAnnot = AnnotCreateType::None;
			uiMgr->IsRightMenuAdd = false;
		}
	}

	m_TextSelectContext->ResetContext();
	_FrameworkElement->ManipulationMode = Windows::UI::Xaml::Input::ManipulationModes::All;
}

void StrikeOutAnnotEventPeer::OnTapped(Platform::Object^ sender, Windows::UI::Xaml::Input::TappedRoutedEventArgs^ e)
{
	UIExtensionsManager^ uiMgr = (UIExtensionsManager^)View->ExtensionsManager;
	auto _SelectedAnnot = uiMgr->getAnnotMgr()->GetSelectedAnnot();
	if (_SelectedAnnot)
	{
		StrikeOutAnnot^ annot = dynamic_cast<StrikeOutAnnot^>(_SelectedAnnot);
		if (annot->IsHasGroup)
		{
			Windows::Foundation::Point curPt = e->GetPosition(nullptr);		
			auto control = ref new AnnotNoteUserControl(View, curPt);
			uiMgr->ShowAddNoteWindows(control, curPt);
		}
		else
		{
			Windows::Foundation::Rect rect = annot->MinRect;
			rect.X = rect.X - 5;
			rect.Y = rect.Y - 5;
			rect.Width = rect.Width + 5 * 2;
			rect.Height = rect.Height + 5 * 2;
			auto deviceRect = View->ConvertPageRectToDeviceRect(uiMgr->getAnnotMgr()->GetSelectedAnnot()->PageIndex, rect);
			Point pt = Point(deviceRect.X + deviceRect.Width / 2, deviceRect.Y);

			if (View->GetDoc() && !FSPDFDocUtil::GetIsDocumentCanModifyAnnotation(View->GetDoc()) && View->GetDoc()->GetPasswordType() != foxit::pdf::PasswordType::e_PwdOwner)
			{
				if (_SelectedAnnot->Contents->Length() > 0)
				{
					uiMgr->ShowTextmarkContextmenu(AnnotContextMenuType::Note, pt);
				}
			}
			else
			{
				uiMgr->ShowTextmarkContextmenu(AnnotContextMenuType::Edit, pt);
			}
		}
	}
}

void StrikeOutAnnotEventPeer::OnRightTapped(Platform::Object^ sender, Windows::UI::Xaml::Input::RightTappedRoutedEventArgs^ e)
{

}

void StrikeOutAnnotEventPeer::OnManipulationStarted(Platform::Object^ sender, Windows::UI::Xaml::Input::ManipulationStartedRoutedEventArgs^ e)
{

}

void StrikeOutAnnotEventPeer::OnManipulationDelta(Platform::Object^ sender, Windows::UI::Xaml::Input::ManipulationDeltaRoutedEventArgs^ e)
{

}

void StrikeOutAnnotEventPeer::OnManipulationCompleted(Platform::Object^ sender, Windows::UI::Xaml::Input::ManipulationCompletedRoutedEventArgs^ e)
{

}

void StrikeOutAnnotEventPeer::OnCreateAnnotAfterSelectText()
{
	UIExtensionsManager^ uiMgr = (UIExtensionsManager^)View->ExtensionsManager;
	auto _AnnotSetting = uiMgr->getAnnotMgr()->GetAnnotSetting();
	auto _selectedtext = uiMgr->getTextMgr()->SelectedText;
	if (_selectedtext && _selectedtext->Size > 0)
	{
		if (!View->GetDoc())
			return;

		auto size = _selectedtext->Size;
		bool _bIsClearText = true;
		Windows::Foundation::Rect rect = _selectedtext->GetAt(0)->TextRectVect->GetAt(0);
		for (unsigned int i = 0; i < size; i++)
		{
			auto rectGroup = ref new Platform::Collections::Vector<Windows::Foundation::Rect>();

			for each (Windows::Foundation::Rect gmt in _selectedtext->GetAt(i)->TextRectVect)
			{
				rectGroup->Append(gmt);
			}

			auto _Annot = foxit::rdk::uiextensions::AnnotCreaterFactory::GetInstance()->CreateNewAnnot(AnnotationType::StrikeOut);
			auto _pCreateAnnot = dynamic_cast<StrikeOutAnnot^>(_Annot);
			_pCreateAnnot->Color = _AnnotSetting->GetStrikeOutAnnotColor();
			_pCreateAnnot->Opacity = _AnnotSetting->GetStrikeOutAnnotOpacity();
			_pCreateAnnot->Page_HightLightRectGroup = rectGroup;
			_pCreateAnnot->Name = uiMgr->getAnnotMgr()->GetAnnotSetting()->GetNoteAuthor();
			_pCreateAnnot->Title = _AnnotSetting->GetNoteAuthor();
			_pCreateAnnot->CreationDateTime = (ref new Windows::Globalization::Calendar())->GetDateTime();
			_pCreateAnnot->TextRotateVect = _selectedtext->GetAt(i)->TextRotateVect;
			auto _selectpageindex = _selectedtext->GetAt(i)->PageIndex;

			auto _page = View->GetDoc()->GetPage(_selectpageindex);

			auto _createtype = uiMgr->CreateTypeOfAnnot;

			if (_createtype == AnnotCreateType::StrikeOut)
			{
				uiMgr->getAnnotMgr()->OnCreateNewAnnot(_page, _pCreateAnnot);
				auto pUndoItem = ref new CreateStrikeOutAnnotUndoAction(View, _pCreateAnnot);
				UndoFactory::GetUndoInstance()->AddUndoItem(uiMgr, pUndoItem);

				foxit::common::fxcrt::RectF^ minRect = FoxitRectConverter::ConvertToFoxitRectF(_pCreateAnnot->MinRect);
				String^ rectStr = "[" + minRect->left.ToString() + " " + minRect->top.ToString() + " " + minRect->right.ToString() + " " + minRect->bottom.ToString() + "]";
				uiMgr->CollectAnnotOperation("Strikeout", "Add", _selectpageindex.ToString(), rectStr);
			}
		}
		if (_bIsClearText)
			uiMgr->getTextMgr()->ClearTextSelect();
	}
	m_TextSelectContext->ResetContext();

	uiMgr->CreateTypeOfAnnot = AnnotCreateType::None;
	uiMgr->ExitAnnotPropertyMode();
}

void StrikeOutAnnotEventPeer::OnManipulationInertiaStarting(Platform::Object^ sender, Windows::UI::Xaml::Input::ManipulationInertiaStartingRoutedEventArgs^ e)
{

}
