/**
 * Copyright (C) 2003-2018, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit Mobile PDF SDK to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */


#include "pch.h"
#include "TextAnnotEventPeer.h"
#include "AnnotNoteUserControl.xaml.h"

using namespace foxit::rdk;
using namespace foxit::rdk::uiextensions;

using namespace Platform;
using namespace Windows::Foundation;


TextAnnotEventPeer::TextAnnotEventPeer(PDFViewCtrl^ view)
{
	m_PageViewWeakRef = view;
	m_TextSelectContext = ref new TextSelectContext();
}

PDFViewCtrl^ TextAnnotEventPeer::View::get()
{
	return m_PageViewWeakRef.Resolve<PDFViewCtrl>();
}

void TextAnnotEventPeer::OnPointerPressed(Platform::Object^ sender, Windows::UI::Xaml::Input::PointerRoutedEventArgs^ e)
{

}

void TextAnnotEventPeer::OnPointerMoved(Platform::Object^ sender, Windows::UI::Xaml::Input::PointerRoutedEventArgs^ e)
{

}

void TextAnnotEventPeer::OnPointerReleased(Platform::Object^ sender, Windows::UI::Xaml::Input::PointerRoutedEventArgs^ e)
{

}

void TextAnnotEventPeer::OnTapped(Platform::Object^ sender, Windows::UI::Xaml::Input::TappedRoutedEventArgs^ e)
{
	Windows::UI::Xaml::FrameworkElement^ _FrameworkElement = dynamic_cast<Windows::UI::Xaml::FrameworkElement^>(sender);
	Windows::Foundation::Point curPt = e->GetPosition(_FrameworkElement);
	UIExtensionsManager^ uiMgr = (UIExtensionsManager^)View->ExtensionsManager;
	auto _SelectedAnnot = uiMgr->getAnnotMgr()->GetSelectedAnnot();
	if (_SelectedAnnot)
	{
		Windows::Foundation::Rect rect = _SelectedAnnot->MinRect;
		auto wrapperDevice = View->ConvertPageRectToDeviceRect(_SelectedAnnot->PageIndex, rect);
		Windows::Foundation::Rect deviceRect = wrapperDevice;
				
		auto control = ref new AnnotNoteUserControl(View, curPt);
		uiMgr->ShowAddNoteWindows(control, curPt);

		TextAnnot^ annot = dynamic_cast<TextAnnot^>(_SelectedAnnot);
		uiMgr->CreateRectangelAnnotToCanvas(annot->PageIndex, annot->DrawRect, annot->Opacity, annot->BorderWidth, annot->Color);
	}
	else
	{
		Windows::UI::Xaml::FrameworkElement^ _FrameworkElement = dynamic_cast<Windows::UI::Xaml::FrameworkElement^>(sender);
		auto _AnnotSetting = uiMgr->getAnnotMgr()->GetAnnotSetting();
		Windows::Foundation::Point _Pt = e->GetPosition(_FrameworkElement);

		auto _Pageinfo = View->ConvertDevicePtToPDFPagePt(_Pt);
		int index = _Pageinfo->PageIndex;

		_Pageinfo = View->ConvertDevicePtToPDFPagePt(_Pt);


		auto _Annot = foxit::rdk::uiextensions::AnnotCreaterFactory::GetInstance()->CreateNewAnnot(AnnotationType::Text);
		auto _pCreateAnnot = dynamic_cast<TextAnnot^>(_Annot);
		_pCreateAnnot->Color = Windows::UI::Colors::Yellow;
		_pCreateAnnot->Opacity = 1;
		_pCreateAnnot->Name = uiMgr->getAnnotMgr()->GetAnnotSetting()->GetNoteAuthor();
		_pCreateAnnot->Title = uiMgr->getAnnotMgr()->GetAnnotSetting()->GetNoteAuthor();

		_pCreateAnnot->PosPoint = _Pageinfo->PagePoint;
		_pCreateAnnot->IconType = GetCurIconType();

		auto _msgDlg = ref new MessageDlg();
		auto _AnnotNoteUserControl = ref new AnnotNoteUserControl(View, curPt);
		auto _AnnotComandToken = ref new AnnotCommandToken();
		_AnnotComandToken->Pageindex = index;
		_AnnotComandToken->Param = _pCreateAnnot;
		_AnnotComandToken->Type = AnnotCommandType::Add;
		_AnnotNoteUserControl->Tag = _AnnotComandToken;
		
		uiMgr->ShowAddNoteWindows(_AnnotNoteUserControl, curPt);
	}
}

void TextAnnotEventPeer::OnRightTapped(Platform::Object^ sender, Windows::UI::Xaml::Input::RightTappedRoutedEventArgs^ e)
{

}

void TextAnnotEventPeer::OnManipulationStarted(Platform::Object^ sender, Windows::UI::Xaml::Input::ManipulationStartedRoutedEventArgs^ e)
{
	UIExtensionsManager^ uiMgr = (UIExtensionsManager^)View->ExtensionsManager;
	uiMgr->SetTextMenuPopupHide();
	uiMgr->SetPropertyPoupHide();
	auto _SelectedAnnot = uiMgr->getAnnotMgr()->GetSelectedAnnot();
	if (_SelectedAnnot)
	{
		m_isMoveOutPage = false;
		m_isMoveEnd = false;

		auto page = View->GetDoc()->GetPage(_SelectedAnnot->PageIndex);
		if (page)
		{
			m_PageRotation = (int) page->GetRotation();
		}
	}
}

void TextAnnotEventPeer::OnManipulationDelta(Platform::Object^ sender, Windows::UI::Xaml::Input::ManipulationDeltaRoutedEventArgs^ e)
{
	
}

void TextAnnotEventPeer::OnManipulationCompleted(Platform::Object^ sender, Windows::UI::Xaml::Input::ManipulationCompletedRoutedEventArgs^ e)
{
	if (m_isMoveEnd)
		return;
	UIExtensionsManager^ uiMgr = (UIExtensionsManager^)View->ExtensionsManager;
	uiMgr->ShowTextmarkContextmenu(AnnotContextMenuType::NotePro, e->Position);
	m_isMoveOutPage = false;
}

void TextAnnotEventPeer::OnCreateAnnotAfterSelectText()
{

}

foxit::rdk::uiextensions::AnnotIconNameTextType TextAnnotEventPeer::GetCurIconType()
{
	UIExtensionsManager^ uiMgr = (UIExtensionsManager^)View->ExtensionsManager;
	auto _AnnotSetting = uiMgr->getAnnotMgr()->GetAnnotSetting();
	Platform::String^ iconname = uiMgr->getAnnotMgr()->GetAnnotSetting()->GetNoteLogoUrl();
	AnnotIconNameTextType icontype = AnnotIconNameTextType::Comment;
	if (iconname == "Checkmark")
	{
		icontype = AnnotIconNameTextType::Checkmark;
	}
	else if (iconname == "Circle")
	{
		icontype = AnnotIconNameTextType::Circle;
	}
	else if (iconname == "Comment")
	{
		icontype = AnnotIconNameTextType::Comment;
	}
	else if (iconname == "Cross")
	{
		icontype = AnnotIconNameTextType::Cross;
	}
	else if (iconname == "Help")
	{
		icontype = AnnotIconNameTextType::Help;
	}
	else if (iconname == "Insert")
	{
		icontype = AnnotIconNameTextType::Insert;
	}
	else if (iconname == "Key")
	{
		icontype = AnnotIconNameTextType::Key;
	}
	else if (iconname == "NewParagraph")
	{
		icontype = AnnotIconNameTextType::NewParagraph;
	}
	else if (iconname == "Note")
	{
		icontype = AnnotIconNameTextType::Note;
	}
	else if (iconname == "Paragraph")
	{
		icontype = AnnotIconNameTextType::Paragraph;
	}
	else if (iconname == "RightArrow")
	{
		icontype = AnnotIconNameTextType::RightArrow;
	}
	else if (iconname == "RightPointer")
	{
		icontype = AnnotIconNameTextType::RightPointer;
	}
	else if (iconname == "Star")
	{
		icontype = AnnotIconNameTextType::Star;
	}
	else if (iconname == "UpArrow")
	{
		icontype = AnnotIconNameTextType::UpArrow;
	}
	else if (iconname == "UpLeftArrow")
	{
		icontype = AnnotIconNameTextType::UpleftArrow;
	}
	return icontype;
}

void TextAnnotEventPeer::OnManipulationInertiaStarting(Platform::Object^ sender, Windows::UI::Xaml::Input::ManipulationInertiaStartingRoutedEventArgs^ e)
{
	m_isMoveEnd = true;
	e->Handled = true;
	UIExtensionsManager^ uiMgr = (UIExtensionsManager^)View->ExtensionsManager;
	uiMgr->ShowTextmarkContextmenu(AnnotContextMenuType::NotePro, pt);
	m_isMoveOutPage = false;
}