/**
 * Copyright (C) 2003-2018, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit Mobile PDF SDK to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */


#include "pch.h"
#include "WidgetAnnotEventPeer.h"

using namespace foxit::rdk;
using namespace foxit::rdk::uiextensions;

using namespace Platform;
using namespace Windows::Foundation;


WidgetAnnotEventPeer::WidgetAnnotEventPeer(PDFViewCtrl^ view)
{
	m_PageViewWeakRef = view;
	m_pressStartPoint = Point(-1, -1);
	m_isPressed = false;
}

PDFViewCtrl^ WidgetAnnotEventPeer::View::get()
{
	return m_PageViewWeakRef.Resolve<PDFViewCtrl>();
}

void WidgetAnnotEventPeer::OnPointerPressed(Platform::Object^ sender, Windows::UI::Xaml::Input::PointerRoutedEventArgs^ e)
{
	if (!View)
		return;

	if (m_isPressed)
		return;
	UIExtensionsManager^ uiMgr = (UIExtensionsManager^)View->ExtensionsManager;
	auto _AnnotSetting = uiMgr->getAnnotMgr()->GetAnnotSetting();
	borderWidth = _AnnotSetting->GetAnnotDrawBorderWidth();

	ellipse = ref new Windows::UI::Xaml::Shapes::Ellipse();

	ellipse->Fill = ref new Windows::UI::Xaml::Media::SolidColorBrush(Windows::UI::Colors::Gray);
	ellipse->StrokeThickness = 1;
	ellipse->Opacity = 0.5;

	uiMgr->CreateNewShapeToCanvas(ellipse);

	Windows::UI::Xaml::FrameworkElement^ _FrameworkElement = dynamic_cast<Windows::UI::Xaml::FrameworkElement^>(sender);
	auto _PointerPt = e->GetCurrentPoint(_FrameworkElement);
	Windows::Foundation::Point _Pt = _PointerPt->Position;

	auto _Pageinfo = View->ConvertDevicePtToPDFPagePt(_Pt);
	m_PageIndex = _Pageinfo->PageIndex;

	if (_Pageinfo->PagePoint.X != -1 && _Pageinfo->PagePoint.Y != -1)
	{
		m_isPressed = true;
		m_pressStartPoint = _Pt;
	}
}

void WidgetAnnotEventPeer::OnPointerMoved(Platform::Object^ sender, Windows::UI::Xaml::Input::PointerRoutedEventArgs^ e)
{
	if (!m_isPressed)
		return;

	Windows::UI::Xaml::FrameworkElement^ _FrameworkElement = dynamic_cast<Windows::UI::Xaml::FrameworkElement^>(sender);
	auto _PointerPt = e->GetCurrentPoint(_FrameworkElement);
	Windows::Foundation::Point currentPoint = _PointerPt->Position;
	Windows::Foundation::Rect pageViewRect = View->GetPageViewRect(m_PageIndex);
	double width = 0;
	double height = 0;
	Windows::Foundation::Point  startPoint = m_pressStartPoint;

	if (m_pressStartPoint.X != -1 && m_pressStartPoint.Y != -1)
	{
		if (currentPoint.Y > pageViewRect.Y + pageViewRect.Height)
		{
			currentPoint.Y = pageViewRect.Y + pageViewRect.Height;
		}
		else if (currentPoint.Y < pageViewRect.Y)
		{
			currentPoint.Y = pageViewRect.Y;
		}
		if (currentPoint.X > pageViewRect.X + pageViewRect.Width)
		{
			currentPoint.X = pageViewRect.X + pageViewRect.Width;
		}
		else if (currentPoint.X < pageViewRect.X)
		{
			currentPoint.X = pageViewRect.X;
		}

		if (currentPoint.X > m_pressStartPoint.X)
		{
			width = currentPoint.X - m_pressStartPoint.X;
		}
		else
		{
			width = m_pressStartPoint.X - currentPoint.X;
			startPoint.X = currentPoint.X;
		}
		if (currentPoint.Y > m_pressStartPoint.Y)
		{
			height = currentPoint.Y - m_pressStartPoint.Y;
		}
		else
		{
			height = m_pressStartPoint.Y - currentPoint.Y;
			startPoint.Y = currentPoint.Y;
		}

		m_AreaRect = Rect(startPoint.X, startPoint.Y, (float)width, (float)height);


		ellipse->SetValue(Windows::UI::Xaml::Controls::Canvas::TopProperty, m_AreaRect.Y);
		ellipse->SetValue(Windows::UI::Xaml::Controls::Canvas::LeftProperty, m_AreaRect.X);

		ellipse->Width = m_AreaRect.Width;
		ellipse->Height = m_AreaRect.Height;
	}
}

void WidgetAnnotEventPeer::OnPointerReleased(Platform::Object^ sender, Windows::UI::Xaml::Input::PointerRoutedEventArgs^ e)
{
	
}

void WidgetAnnotEventPeer::OnTapped(Platform::Object^ sender, Windows::UI::Xaml::Input::TappedRoutedEventArgs^ e)
{
	UIExtensionsManager^ uiMgr = (UIExtensionsManager^)View->ExtensionsManager;
	auto _SelectedAnnot = uiMgr->getAnnotMgr()->GetSelectedAnnot();
	if (_SelectedAnnot)
	{
		WidgetAnnot^ annot = dynamic_cast<WidgetAnnot^>(_SelectedAnnot);
		if (annot && uiMgr->IsValidFormControlPtr(annot))
		{
			if (uiMgr->IsFormControlCanSelect(annot))
			{				
				uiMgr->ExitSearchingModel();
				uiMgr->LoadAppBarByType(AppBarType::FormOperationAppbar);
			}
			else
				uiMgr->getAnnotMgr()->DeleteAnnot(annot);
		}
	}
}

void WidgetAnnotEventPeer::OnRightTapped(Platform::Object^ sender, Windows::UI::Xaml::Input::RightTappedRoutedEventArgs^ e)
{

}

void WidgetAnnotEventPeer::OnManipulationStarted(Platform::Object^ sender, Windows::UI::Xaml::Input::ManipulationStartedRoutedEventArgs^ e)
{

}

void WidgetAnnotEventPeer::OnManipulationDelta(Platform::Object^ sender, Windows::UI::Xaml::Input::ManipulationDeltaRoutedEventArgs^ e)
{

}

void WidgetAnnotEventPeer::OnManipulationCompleted(Platform::Object^ sender, Windows::UI::Xaml::Input::ManipulationCompletedRoutedEventArgs^ e)
{

}

void WidgetAnnotEventPeer::OnCreateAnnotAfterSelectText()
{
	
}

void WidgetAnnotEventPeer::OnManipulationInertiaStarting(Platform::Object^ sender, Windows::UI::Xaml::Input::ManipulationInertiaStartingRoutedEventArgs^ e)
{

}

