/**
 * Copyright (C) 2003-2018, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit Mobile PDF SDK to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */


#include "pch.h"
#include "BookmarkHandle.h"

using namespace foxit::rdk::uiextensions;

using namespace Platform;
using namespace Windows::Foundation;
using namespace Concurrency;


BookmarkItem::BookmarkItem(foxit::pdf::Bookmark^ bookmark, foxit::pdf::Bookmark^ parentbookmark, int serials)
{
	if (bookmark)
	{
		m_This = ref new foxit::pdf::Bookmark(bookmark);
	}
	else
	{
		m_This = nullptr;
	}
	if (parentbookmark)
	{
		m_Parent = ref new foxit::pdf::Bookmark(parentbookmark); 
	}
	else
	{
		m_Parent = nullptr;
	}
	_Series = serials;
	m_bIsinitaction = false;
	_IsHasNextList = Windows::UI::Xaml::Visibility::Collapsed;
	InitialBookmark();	
	m_isCancelParserAction = false;
}

int BookmarkItem::Series::get()
{
	return _Series;
}

int BookmarkItem::PageIndex::get()
{
	return _PageIndex;
}

void BookmarkItem::PageIndex::set(int value)
{
	_PageIndex = value;
}

Platform::String^ BookmarkItem::Title::get()
{
	return _Title;
}

void BookmarkItem::Title::set(Platform::String^ value)
{
	_Title = value;	
}

Windows::UI::Xaml::Visibility BookmarkItem::IsHasNextList::get()
{
	return _IsHasNextList;
}

void BookmarkItem::IsHasNextList::set(Windows::UI::Xaml::Visibility value)
{
	_IsHasNextList = value;	
}

Platform::String^ BookmarkItem::Index::get()
{
	return _Index;
}

void BookmarkItem::Index::set(Platform::String^ value)
{	
	_Index = value;
}

foxit::pdf::Bookmark^ BookmarkItem::bookmarkIterator::get()
{
	return _bookmarkIterator;
}

void BookmarkItem::bookmarkIterator::set(foxit::pdf::Bookmark^ value)
{	
	_bookmarkIterator = value;
}

void BookmarkItem::DoAction()
{
	if (!m_bIsinitaction)
		InitialBookmarkDest();
}

IAsyncOperationWithProgress<bool, String^>^ BookmarkItem::InitialBookmarkDestAsync()
{
	return create_async([this](progress_reporter<String^> reporter)->bool
	{
		InitialBookmarkDest();
		return true;
	});
}


void BookmarkItem::InitialBookmarkDest()
{
	if (m_bIsinitaction || m_isCancelParserAction || !bookmarkPos)
		return;
	m_This = bookmarkPos;

	foxit::pdf::actions::Destination^ dest = m_This->GetDestination();
	if (dest && !dest->IsEmpty())
	{
		destVector.push_back(dest);
	}

	m_bIsinitaction = true;
	m_isCancelParserAction = false;
}

void BookmarkItem::InitialBookmark()
{
	foxit::pdf::Bookmark^ bmData = ref new foxit::pdf::Bookmark(m_This);
	_bookmarkIterator = ref new foxit::pdf::Bookmark(m_This); 
	bookmarkPos = ref new foxit::pdf::Bookmark(m_This);

	if (FSPDFDocUtil::IsValidBookmark(m_This) && FSPDFDocUtil::IsValidBookmark(m_This->GetFirstChild()))
	{
		m_This = m_This->GetFirstChild();
		_IsHasNextList = Windows::UI::Xaml::Visibility::Visible;
		if (FSPDFDocUtil::IsValidBookmark(m_This->GetParent()))
		{
			m_This = m_This->GetParent();
		}
	}
	else {
		_IsHasNextList = Windows::UI::Xaml::Visibility::Collapsed;
	}

	_Title = !bmData->IsEmpty() ? bmData->GetTitle() : L"";
}

void BookmarkItem::ReleaseBookmarkPos()
{	
	if (bookmarkPos)
	{
		delete bookmarkPos;
		bookmarkPos = nullptr;
	}
}

void BookmarkItem::CancelParserAction()
{
	m_isCancelParserAction = true;
}
