﻿/**
 * Copyright (C) 2003-2018, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit Mobile PDF SDK to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */


#include "pch.h"
#include "BookMarkPage.xaml.h"

using namespace foxit::rdk::uiextensions;
using namespace foxit::rdk;

using namespace Platform;
using namespace Windows::Foundation;
using namespace Windows::Foundation::Collections;
using namespace Windows::UI::Xaml;
using namespace Windows::UI::Xaml::Controls;
using namespace Windows::UI::Xaml::Controls::Primitives;
using namespace Windows::UI::Xaml::Data;
using namespace Windows::UI::Xaml::Input;
using namespace Windows::UI::Xaml::Media;
using namespace Windows::UI::Xaml::Navigation;
using namespace concurrency;

BookMarkPage::BookMarkPage()
{
	InitializeComponent();

	InitializeBookmarkPage();
}

BookMarkPage::BookMarkPage(foxit::pdf::PDFDoc^ doc, GoToPageIndexDelegate^ GoTodelegate)
{
	InitializeComponent();

	pdfDoc = (foxit::pdf::PDFDoc^)doc;
	goToIndexDelegate = GoTodelegate;
	InitializeBookmarkPage();
	this->SizeChanged += ref new Windows::UI::Xaml::SizeChangedEventHandler(this, &foxit::rdk::uiextensions::BookMarkPage::OnSizeChanged);
}

void BookMarkPage::OnNavigatedTo(NavigationEventArgs^ e)
{
	(void)e;
}


void foxit::rdk::uiextensions::BookMarkPage::OnLoaded(Platform::Object ^sender, Windows::UI::Xaml::RoutedEventArgs ^e)
{
	bookmarkList->Focus(Windows::UI::Xaml::FocusState::Programmatic);

	if (Windows::Foundation::Metadata::ApiInformation::IsTypePresent("Windows.Phone.UI.Input.HardwareButtons"))
	{
		m_HardwareButtonsBackPressedEventToken = Windows::Phone::UI::Input::HardwareButtons::BackPressed += ref new Windows::Foundation::EventHandler<Windows::Phone::UI::Input::BackPressedEventArgs ^>(this, &foxit::rdk::uiextensions::BookMarkPage::OnBackPressed);
	}

	if (Windows::Foundation::Metadata::ApiInformation::IsTypePresent("Windows.UI.Core.SystemNavigationManager"))
	{
		m_SystemNavigationBackRequestedEventToken = Windows::UI::Core::SystemNavigationManager::GetForCurrentView()->BackRequested += ref new Windows::Foundation::EventHandler<Windows::UI::Core::BackRequestedEventArgs ^>(this, &foxit::rdk::uiextensions::BookMarkPage::OnBackRequested);
	}
}

void foxit::rdk::uiextensions::BookMarkPage::OnUnloaded(Platform::Object ^sender, Windows::UI::Xaml::RoutedEventArgs ^e)
{
	if (Windows::Foundation::Metadata::ApiInformation::IsTypePresent("Windows.Phone.UI.Input.HardwareButtons"))
	{
		Windows::Phone::UI::Input::HardwareButtons::BackPressed -= m_HardwareButtonsBackPressedEventToken;
		m_HardwareButtonsBackPressedEventToken.Value = 0;
	}

	if (Windows::Foundation::Metadata::ApiInformation::IsTypePresent("Windows.UI.Core.SystemNavigationManager"))
	{
		Windows::UI::Core::SystemNavigationManager::GetForCurrentView()->BackRequested -= m_SystemNavigationBackRequestedEventToken;
		m_SystemNavigationBackRequestedEventToken.Value = 0;
	}

	if (m_pProgressBar)
		m_pProgressBar->Close();

	ReleaseMarkParser();
}


void BookMarkPage::InitializeBookmarkPage()
{
	titleItems = ref new Platform::Collections::Vector<BookMarkItemData^>();
	items = ref new Platform::Collections::Vector<BookMarkLocalDataList^>();
	markParser = ref new BookMarkParser(pdfDoc);
	CollectBookMarkListDelegate^ delegate = ref new CollectBookMarkListDelegate(this, &foxit::rdk::uiextensions::BookMarkPage::OperateList);
	markParser->SetDelegate(delegate);
	markParser->StartGetAllBookMarkDataTaskWithAsync();

	m_pProgressBar = ref new ProgressBarFactory();
	auto resourceLoader = Windows::ApplicationModel::Resources::ResourceLoader::GetForCurrentView("foxit.rdk.uiextensions/Resources");
	TitleContent = resourceLoader->GetString(L"BookmarkPageOutlineTitleText");
	tbTitle->Text = TitleContent;
	if (markParser)
	{
		bookmarkList->ItemsSource = markParser->m_BookMarkData->Items;
	}
	m_pProgressBar->Show();
	Loaded += ref new Windows::UI::Xaml::RoutedEventHandler(this, &foxit::rdk::uiextensions::BookMarkPage::OnLoaded);
	Unloaded += ref new Windows::UI::Xaml::RoutedEventHandler(this, &foxit::rdk::uiextensions::BookMarkPage::OnUnloaded);
	m_IsParaseBookmarkAction = false;
	m_IsCancelParserAction = false;

}

void foxit::rdk::uiextensions::BookMarkPage::bookmarkList_ItemClick(Platform::Object^ sender, Windows::UI::Xaml::Controls::ItemClickEventArgs^ e)
{
	BookMarkItemData^ item = dynamic_cast<BookMarkItemData^>(e->ClickedItem);
	if (item)
	{
		curClickBookMarkItem = item;
		std::vector<foxit::pdf::actions::Destination^> dests = item->bookmarkItemHandle->destVector;
		if (dests.size() > 0)
		{
			DoBookmarkDestination(item->bookmarkItemHandle->bookmarkIterator, dests);
		}
		else
		{
			m_IsParaseBookmarkAction = true;
			m_pProgressBar->Show();
			Windows::Foundation::IAsyncOperationWithProgress<bool, Platform::String^>^ asynctask = item->bookmarkItemHandle->InitialBookmarkDestAsync();
			asynctask->Progress = ref new AsyncOperationProgressHandler<bool, String^>([this](IAsyncOperationWithProgress<bool, String^>^ asyncInfo, String^ progressInfo)
			{
				if (progressInfo)
					OutputDebugString(progressInfo->Data());
			});

			asynctask->Completed = ref new AsyncOperationWithProgressCompletedHandler<bool, String^>([=](IAsyncOperationWithProgress<bool, String^>^ asyncInfo, AsyncStatus asyncStatus)
			{
				m_pProgressBar->Close();
				if (item->bookmarkItemHandle)
				{
					switch (asyncStatus)
					{
					case Windows::Foundation::AsyncStatus::Canceled:
					{
						break;
					}
					case Windows::Foundation::AsyncStatus::Completed:
					{
						if (!m_IsCancelParserAction)
						{
							DoBookmarkDestination(item->bookmarkItemHandle->bookmarkIterator, item->bookmarkItemHandle->destVector);
						}
						break;
					}
					case Windows::Foundation::AsyncStatus::Error:
					{
						break;
					}

					case Windows::Foundation::AsyncStatus::Started:
						break;
					default:
						break;
					}
				}

				m_IsParaseBookmarkAction = false;
			});
		}
	}
}

void foxit::rdk::uiextensions::BookMarkPage::DoBookmarkDestination(foxit::pdf::Bookmark^ bookmark, std::vector<foxit::pdf::actions::Destination^> dests)
{
	for (size_t i = 0; i < dests.size(); i++)
	{
		foxit::pdf::actions::Destination^ dest = dests.at(i);
		if(dest->IsEmpty()) continue;

		int pageIndex = dest->GetPageIndex(pdfDoc);

		int zoomMode = (int)dest->GetZoomMode();
		float a1 = dest->GetLeft();
		float a2 = dest->GetTop();
		float a3 = dest->GetRight();
		float a4 = dest->GetBottom();
		BookMarkGoToData^ goToData = ref new BookMarkGoToData();
		goToData->bookmark = bookmark;
		goToData->pageIndex = pageIndex;
		goToData->zoomMode = zoomMode;
		goToData->params->Append(a1);
		goToData->params->Append(a2);
		goToData->params->Append(a3);
		goToData->params->Append(a4);
		goToIndexDelegate->Invoke(goToData);
	}
}


void foxit::rdk::uiextensions::BookMarkPage::OnBackPressed(Platform::Object ^sender, Windows::Phone::UI::Input::BackPressedEventArgs ^args)
{
	if (items && items->Size > 1)
	{
		imgBack_Tapped(nullptr, nullptr);
		args->Handled = true;
	}
	else
	{
		auto _FlyoutPresenter = foxit::rdk::uiextensions::GetVisualParent<Windows::UI::Xaml::Controls::FlyoutPresenter^>(this);
		if (_FlyoutPresenter)
		{
			auto _Popup = dynamic_cast<Windows::UI::Xaml::Controls::Primitives::Popup^>(_FlyoutPresenter->Parent);
			if (_Popup)
			{
				_Popup->IsOpen = false;
				args->Handled = true;
			}
		}
	}
}

void  foxit::rdk::uiextensions::BookMarkPage::PushBackList(Platform::String^ key, Windows::Foundation::Collections::IObservableVector<Platform::Object^>^ bookmarkItems)
{
	if (items)
	{
		BookMarkLocalDataList^ data = ref new BookMarkLocalDataList();
		for (int i = 0; i < (int)bookmarkItems->Size; i++)
		{
			data->m_items->Append(bookmarkItems->GetAt(i));
		}
		data->title = key;
		items->Append(data);
	}
}

BookMarkLocalDataList^ foxit::rdk::uiextensions::BookMarkPage::PopListItems()
{
	int size = items->Size;
	if (items && items->Size > 0)
	{
		BookMarkLocalDataList^ itemdata = dynamic_cast<BookMarkLocalDataList^>(items->GetAt(items->Size - 1));
		for (int i = 0; i < (int)itemdata->m_items->Size; i++)
		{
			BookMarkItemData^ data = dynamic_cast<BookMarkItemData^>(itemdata->m_items->GetAt(i));
			data->bookmarkItemHandle->ReleaseBookmarkPos();
		}

		items->RemoveAt(items->Size - 1);
		if (items->Size > 0)
		{
			return items->GetAt(items->Size - 1);
		}
	}
	return nullptr;
}

void foxit::rdk::uiextensions::BookMarkPage::OperateList()
{
	if (m_pProgressBar)
	{
		m_pProgressBar->Close();
	}
	PushBackList(TitleContent, markParser->m_BookMarkData->Items);
}

void foxit::rdk::uiextensions::BookMarkPage::ReleaseAllBookPos()
{
	if (items && items->Size > 0)
	{
		for (int j = 0; j < (int)items->Size; j++)
		{
			BookMarkLocalDataList^ itemlist = dynamic_cast<BookMarkLocalDataList^>(items->GetAt(j));
			for (int i = 0; i < (int)itemlist->m_items->Size; i++)
			{
				BookMarkItemData^ data = dynamic_cast<BookMarkItemData^>(itemlist->m_items->GetAt(i));
				data->bookmarkItemHandle->ReleaseBookmarkPos();
			}
		}
	}
}

bool foxit::rdk::uiextensions::BookMarkPage::isFirstSlibing()
{
	auto resourceLoader = Windows::ApplicationModel::Resources::ResourceLoader::GetForCurrentView("foxit::rdk::uiextensions/Resources");
	auto title = resourceLoader->GetString(L"BookmarkPageOutlineTitleText");
	if (tbTitle && tbTitle->Text == title)
	{
		return true;
	}
	else
	{
		return false;
	}
}

void foxit::rdk::uiextensions::BookMarkPage::ReleaseMarkParser()
{
	if (markParser)
	{
		markParser->ReleaseBookMarkParser();
	}
	ReleaseAllBookPos();

	titleItems->Clear();
	titleItems = nullptr;
	items->Clear();
	items = nullptr;
}

void foxit::rdk::uiextensions::BookMarkPage::CancelMarkParser()
{
	m_pProgressBar->Close();
	if (markParser)
	{
		if (m_IsParaseBookmarkAction)
		{
			m_IsCancelParserAction = true;
			if (curClickBookMarkItem)
			{
				curClickBookMarkItem->bookmarkItemHandle->CancelParserAction();
			}
		}
		markParser->CancelBookMarkParser();
	}

	curClickBookMarkItem = nullptr;
	curTitleBookMarkItem = nullptr;
}

void foxit::rdk::uiextensions::BookMarkPage::ParaseBookmarkTitle(Platform::String^ title)
{
	int length = title->Length();
	const wchar_t *seperator = title->Data();
	wchar_t substr[4] = { 0 };
	wcsncpy_s(substr, seperator, 2);

	Platform::String^ str = ref new Platform::String(substr);
}

void foxit::rdk::uiextensions::BookMarkPage::OnSizeChanged(Platform::Object ^sender, Windows::UI::Xaml::SizeChangedEventArgs ^e)
{
	if (markParser && markParser->m_BookMarkData->Items->Size > 0)
	{
		Windows::Foundation::Collections::IObservableVector<Platform::Object^>^ itemList = dynamic_cast<Windows::Foundation::Collections::IObservableVector<Platform::Object^>^>(bookmarkList->ItemsSource);
		if (itemList && itemList->Size > 0)
		{
			float boundWidth = Windows::UI::Xaml::Window::Current->Bounds.Width;
			for (unsigned int i = 0; i < itemList->Size; i++)
			{
				BookMarkItemData^ data = dynamic_cast<BookMarkItemData^>(itemList->GetAt(i));
				data->pageWidth = boundWidth*0.9;
			}
		}
	}
}

void foxit::rdk::uiextensions::BookMarkPage::TextBlock_Tapped(Platform::Object^ sender, Windows::UI::Xaml::Input::TappedRoutedEventArgs^ e)
{
	Windows::Foundation::Collections::IObservableVector<Platform::Object^>^ itemSecond = dynamic_cast<Windows::Foundation::Collections::IObservableVector<Platform::Object^>^>(bookmarkList->ItemsSource);
	Platform::String^ itemIndex = (dynamic_cast<Windows::UI::Xaml::FrameworkElement^>(sender))->Tag->ToString();
	BookMarkItemData^ itemdata = nullptr;
	for (int i = 0; i < (int)itemSecond->Size; i++)
	{
		BookMarkItemData^ data = dynamic_cast<BookMarkItemData^>(itemSecond->GetAt(i));
		if (data->Index == itemIndex)
		{
			itemdata = data;
			break;
		}
	}
	if (itemdata)
	{
		curClickBookMarkItem = itemdata;
		LoadBookmarkDestination();
		if (Windows::System::Profile::AnalyticsInfo::VersionInfo->DeviceFamily == L"Windows.Mobile")
		{
			auto _FlyoutPresenter = foxit::rdk::uiextensions::GetVisualParent<Windows::UI::Xaml::Controls::FlyoutPresenter^>(this);
			if (_FlyoutPresenter)
			{
				auto _Popup = dynamic_cast<Windows::UI::Xaml::Controls::Primitives::Popup^>(_FlyoutPresenter->Parent);
				if (_Popup)
				{
					_Popup->IsOpen = false;
				}
			}
		}
	}
}


void foxit::rdk::uiextensions::BookMarkPage::tbTitle_Tapped(Platform::Object^ sender, Windows::UI::Xaml::Input::TappedRoutedEventArgs^ e)
{
	if (curTitleBookMarkItem)
	{
		curClickBookMarkItem = curTitleBookMarkItem;
		LoadBookmarkDestination();
	}
}

void foxit::rdk::uiextensions::BookMarkPage::LoadBookmarkDestination()
{
	if (curClickBookMarkItem)
	{
		std::vector<foxit::pdf::actions::Destination^> dest = curClickBookMarkItem->bookmarkItemHandle->destVector;
		if (dest.size() > 0)
		{
			DoBookmarkDestination(curClickBookMarkItem->bookmarkItemHandle->bookmarkIterator, dest);
		}
		else
		{
			m_IsParaseBookmarkAction = true;
			m_pProgressBar->Show();
			Windows::Foundation::IAsyncOperationWithProgress<bool, Platform::String^>^ asynctask = curClickBookMarkItem->bookmarkItemHandle->InitialBookmarkDestAsync();
			asynctask->Progress = ref new AsyncOperationProgressHandler<bool, String^>([this](IAsyncOperationWithProgress<bool, String^>^ asyncInfo, String^ progressInfo)
			{
				if (progressInfo)
					OutputDebugString(progressInfo->Data());
			});

			asynctask->Completed = ref new AsyncOperationWithProgressCompletedHandler<bool, String^>([=](IAsyncOperationWithProgress<bool, String^>^ asyncInfo, AsyncStatus asyncStatus)
			{
				m_pProgressBar->Close();
				if (curClickBookMarkItem && curClickBookMarkItem->bookmarkItemHandle)
				{
					switch (asyncStatus)
					{
					case Windows::Foundation::AsyncStatus::Canceled:
					{
						break;
					}
					case Windows::Foundation::AsyncStatus::Completed:
					{
						if (!m_IsCancelParserAction)
						{
							DoBookmarkDestination(curClickBookMarkItem->bookmarkItemHandle->bookmarkIterator ,curClickBookMarkItem->bookmarkItemHandle->destVector);
						}
						break;
					}
					case Windows::Foundation::AsyncStatus::Error:
					{
						break;
					}

					case Windows::Foundation::AsyncStatus::Started:
						break;
					default:
						break;
					}
				}

				m_IsParaseBookmarkAction = false;
			});
		}
	}
}

void foxit::rdk::uiextensions::BookMarkPage::imgBack_Tapped(Platform::Object^ sender, Windows::UI::Xaml::Input::TappedRoutedEventArgs^ e)
{
	if (m_IsParaseBookmarkAction)
	{
		m_IsCancelParserAction = true;
		if (curClickBookMarkItem)
		{
			curClickBookMarkItem->bookmarkItemHandle->CancelParserAction();
		}
	}
	curClickBookMarkItem = nullptr;
	if (titleItems && titleItems->Size > 0)
	{
		titleItems->RemoveAt(titleItems->Size - 1);
		if (titleItems->Size > 0)
		{
			curTitleBookMarkItem = titleItems->GetAt(titleItems->Size - 1);
		}
		else
		{
			curTitleBookMarkItem = nullptr;
		}
	}
	else
	{
		curTitleBookMarkItem = nullptr;
	}

	BookMarkLocalDataList^ item = PopListItems();
	bookmarkList->ItemsSource = item->m_items;
	if (items && items->Size == 1)
	{
		imgBack->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
		auto resourceLoader = Windows::ApplicationModel::Resources::ResourceLoader::GetForCurrentView("foxit.rdk.uiextensions/Resources");
		TitleContent = resourceLoader->GetString(L"BookmarkPageOutlineTitleText");
		tbTitle->Text = TitleContent;
	}
	else
	{
		tbTitle->Text = item->title;
	}
}


void foxit::rdk::uiextensions::BookMarkPage::Image_Tapped(Platform::Object^ sender, Windows::UI::Xaml::Input::TappedRoutedEventArgs^ e)
{
	m_pProgressBar->Show();
	Windows::Foundation::Collections::IObservableVector<Platform::Object^>^ itemSecond = dynamic_cast<Windows::Foundation::Collections::IObservableVector<Platform::Object^>^>(bookmarkList->ItemsSource);
	Platform::String^ itemIndex = (dynamic_cast<Windows::UI::Xaml::Controls::Grid^>(sender))->Tag->ToString();
	curTitleBookMarkItem = nullptr;
	for (int i = 0; i < (int)itemSecond->Size; i++)
	{
		BookMarkItemData^ data = dynamic_cast<BookMarkItemData^>(itemSecond->GetAt(i));
		if (data->Index == itemIndex)
		{
			curTitleBookMarkItem = data;
			break;
		}
	}
	if (curTitleBookMarkItem)
	{
		titleItems->Append(curTitleBookMarkItem);
		tbTitle->Text = curTitleBookMarkItem->ItemText;

		TitleContent = curTitleBookMarkItem->ItemText;
		markParser->SetCurIterator(curTitleBookMarkItem->bookmarkItemHandle->bookmarkIterator);
		CollectBookMarkListDelegate^ delegate = ref new CollectBookMarkListDelegate(this, &foxit::rdk::uiextensions::BookMarkPage::OperateList);
		markParser->SetDelegate(delegate);
		markParser->StartGetAllBookMarkDataTaskWithAsync();
		bookmarkList->ItemsSource = markParser->m_BookMarkData->Items;
		imgBack->Visibility = Windows::UI::Xaml::Visibility::Visible;
	}
}


void foxit::rdk::uiextensions::BookMarkPage::OnBackRequested(Platform::Object ^sender, Windows::UI::Core::BackRequestedEventArgs ^args)
{
	if (items && items->Size > 1)
	{
		imgBack_Tapped(nullptr, nullptr);
		args->Handled = true;
	}
	else
	{
		auto _FlyoutPresenter = foxit::rdk::uiextensions::GetVisualParent<Windows::UI::Xaml::Controls::FlyoutPresenter^>(this);
		if (_FlyoutPresenter)
		{
			auto _Popup = dynamic_cast<Windows::UI::Xaml::Controls::Primitives::Popup^>(_FlyoutPresenter->Parent);
			if (_Popup)
			{
				_Popup->IsOpen = false;
				args->Handled = true;
			}
		}
	}
}


void BookMarkPage::btnEditBookmark_Tapped(Platform::Object^ sender, Windows::UI::Xaml::Input::TappedRoutedEventArgs^ e)
{
	if (markParser)
	{
		//Add BookmarkItem
		markParser->AddBookmarkItem();
		markParser->StartGetAllBookMarkDataTaskWithAsync();
		bookmarkList->ItemsSource = markParser->m_BookMarkData->Items;
	}
}


void foxit::rdk::uiextensions::BookMarkPage::clearBookMark_Tapped(Platform::Object^ sender, Windows::UI::Xaml::Input::TappedRoutedEventArgs^ e)
{
	int size = (int)bookmarkList->SelectedItems->Size;
	if (size > 0)
	{
		Windows::Foundation::Collections::IObservableVector<Platform::Object^>^ itemSecond = dynamic_cast<Windows::Foundation::Collections::IObservableVector<Platform::Object^>^>(bookmarkList->ItemsSource);


		for (int i = 0; i < (int)bookmarkList->SelectedItems->Size; i++)
		{
			BookMarkItemData^ data = dynamic_cast<BookMarkItemData^>(bookmarkList->SelectedItems->GetAt(0));
			markParser->RemoveBookmarkItem(data->bookmarkItemHandle->bookmarkIterator);
		}
		markParser->SetDelegate(nullptr);
		markParser->StartGetAllBookMarkDataTaskWithAsync();
		bookmarkList->ItemsSource = markParser->m_BookMarkData->Items;

		//pDocument->IsModified = true;
	}
}


void foxit::rdk::uiextensions::BookMarkPage::slelectBookMark_Tapped(Platform::Object^ sender, Windows::UI::Xaml::Input::TappedRoutedEventArgs^ e)
{
	bookmarkList->SelectionMode = ListViewSelectionMode::Multiple;
}
