/**
 * Copyright (C) 2003-2018, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit Mobile PDF SDK to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */


#include "pch.h"
#include "BookMarkParser.h"
#include "BookMarkData.h"
#include "BookMarkHandle.h"

using namespace foxit::rdk::uiextensions;

using namespace Platform;
using namespace Windows::Foundation;
using namespace Concurrency;
using namespace std;

BookMarkParser::BookMarkParser(foxit::pdf::PDFDoc^ doc)
{
	InitializeReset();
	m_IsInitialize = false;
	m_bCancel = false;
	asynctask = nullptr;

	if (doc)
	{
		m_pDoc = doc;

		if (m_pDoc && m_currentBookmark == nullptr)
		{
			m_currentBookmark = m_pDoc->GetRootBookmark();
		}
	}
}

BookMarkParser::~BookMarkParser()
{
	
}

void BookMarkParser::ReleaseBookMarkParser()
{
	CloseHandle(m_EventCancelParserBookmark);
	m_BookMarkData = nullptr;
	if (m_nativeVect.size() > 0)
	{
		m_nativeVect.clear();
	}

	if (m_currentBookmark != nullptr)
	{
		delete m_currentBookmark;
		m_currentBookmark = nullptr;
	}

	m_pDoc = nullptr;
	m_WeakDocument = nullptr;
}

void BookMarkParser::InitializeReset()
{
	m_isFinish = false;
	m_BookMarkData = ref new BookMarkData();
	asynctask = nullptr;
	m_bCancel = false;
	m_EventCancelParserBookmark = CreateEventEx(NULL, L"CancelParser_Event", 0, EVENT_ALL_ACCESS);

	if (m_nativeVect.size() > 0)
	{
		m_nativeVect.clear();
	}
}

void BookMarkParser::Test()
{

}

bool BookMarkParser::ModifyBookMarkText(Object^ item, String^ text)
{
	return true;
}

bool BookMarkParser::AddOneBookMark(Object^ siblingitem, String^ text)
{
	return true;
}

bool  BookMarkParser::BuildBookmark(int series)
{
	if (FSPDFDocUtil::IsValidBookmark(m_currentBookmark) && FSPDFDocUtil::IsValidBookmark(m_currentBookmark->GetFirstChild()))
	{
		m_currentBookmark = m_currentBookmark->GetFirstChild();
	}
	else
	{
		return false;
	}

	ParseOneBookMark(nullptr, nullptr, series);

	while (1)
	{
		if (m_bCancel)
		{
			break;
		}
		DWORD dwResult = WaitForSingleObjectEx(m_EventCancelParserBookmark, 0, false);
		if (WAIT_OBJECT_0 == dwResult || m_bCancel)
		{
			m_bCancel = true;
			break;
		}

		if (FSPDFDocUtil::IsValidBookmark(m_currentBookmark) && FSPDFDocUtil::IsValidBookmark(m_currentBookmark->GetNextSibling()))
		{
			m_currentBookmark = m_currentBookmark->GetNextSibling();
			ParseOneBookMark(nullptr, nullptr, series);
		}
		else
		{
			break;
		}
	}
	if (FSPDFDocUtil::IsValidBookmark(m_currentBookmark->GetParent()))
	{
		m_currentBookmark = m_currentBookmark->GetParent();
	}
	return true;
}

IAsyncOperationWithProgress<bool, String^>^ BookMarkParser::GetAllBookMarkData()
{
	return create_async([this](progress_reporter<String^> reporter)->bool
	{
		BuildBookmark(0);

		std::wstringstream ws;
		return true;
	});
}

void BookMarkParser::CancelBookMarkParser()
{
	if (asynctask && !m_isFinish)
	{
		m_bCancel = true;
		SetEvent(m_EventCancelParserBookmark);
	}
}

bool BookMarkParser::IsCompleted()
{
	return m_isFinish;
}

bool BookMarkParser::IsHaveBookmark()
{
	foxit::pdf::Bookmark^ child = nullptr;
	bool _isHave = false;

	if (m_pDoc == nullptr)
		return _isHave;

	child = m_pDoc->GetRootBookmark();
	child = child->GetFirstChild();
	_isHave = (child != nullptr) ? true : false;

	return _isHave;
}


void BookMarkParser::StartGetAllBookMarkDataTaskWithAsync()
{
	OutputDebugString(L"StartGetAllBookMarkDataTaskWithAsync\n");
	InitializeReset();
	asynctask = GetAllBookMarkData();

	asynctask->Completed = ref new AsyncOperationWithProgressCompletedHandler<bool, String^>([this](IAsyncOperationWithProgress<bool, String^>^ asyncInfo, AsyncStatus asyncStatus)
	{
		if (m_BookMarkData)
		{
			m_BookMarkData->LoadComplete = Windows::UI::Xaml::Visibility::Collapsed;

			switch (asyncStatus)
			{
			case Windows::Foundation::AsyncStatus::Canceled:
			{
				m_isFinish = true;
				asynctask = nullptr;
				m_bCancel = false;
				break;
			}
			case Windows::Foundation::AsyncStatus::Completed:
			{
				m_BookmarkIndex = 0;
				float boundWidth = Windows::UI::Xaml::Window::Current->Bounds.Width;
				for_each(begin(m_nativeVect), end(m_nativeVect), [=](BookmarkItem^ val)
				{
					BookMarkItemData^ data = ref new BookMarkItemData(val);
					data->Index = m_BookmarkIndex.ToString();

					data->pageWidth = boundWidth*0.9;
					m_BookMarkData->Items->Append(data);
					m_BookmarkIndex++;
				});

				asynctask = nullptr;
				m_isFinish = true;
				if (!m_bCancel && collectBookMarkList)
				{
					collectBookMarkList->Invoke();
				}
				m_bCancel = false;
				break;
			}
			case Windows::Foundation::AsyncStatus::Error:
			{
				asynctask = nullptr;
				m_isFinish = true;
				m_bCancel = false;
				break;
			}

			case Windows::Foundation::AsyncStatus::Started:
				break;
			default:
				break;
			}
		}
	});
}


bool BookMarkParser::GetNextChildBookMark(foxit::pdf::Bookmark^ parent, int series)
{
	if (FSPDFDocUtil::IsValidBookmark(m_currentBookmark->GetFirstChild()))
	{
		m_currentBookmark = m_currentBookmark->GetFirstChild();
	}
	else
	{
		m_currentBookmark = m_currentBookmark->GetParent();
		return false;
	}
	ParseOneBookMark(nullptr, nullptr, 0);
	FS_RESULT iRet = FSCRT_ERRCODE_SUCCESS;
	while (iRet == FSCRT_ERRCODE_SUCCESS)
	{
		DWORD dwResult = WaitForSingleObjectEx(m_EventCancelParserBookmark, 0, false);
		if (WAIT_OBJECT_0 == dwResult || m_bCancel)
		{
			m_bCancel = true;
			OutputDebugString(L"Bookmark cancel\n");
			break;
		}
		if (FSPDFDocUtil::IsValidBookmark(m_currentBookmark->GetNextSibling()))
		{
			m_currentBookmark = m_currentBookmark->GetNextSibling();
			ParseOneBookMark(nullptr, nullptr, 0);
			GetNextChildBookMark(nullptr, 0);
		}
		else
		{
			break;
		}
	}
	m_currentBookmark = m_currentBookmark->GetParent();

	return true;
}

bool BookMarkParser::GetNextSiblingBookMark(foxit::pdf::Bookmark^ parent, int series)
{
	FS_RESULT iRet = FSCRT_ERRCODE_SUCCESS;
	while (iRet == FSCRT_ERRCODE_SUCCESS)
	{
		DWORD dwResult = WaitForSingleObjectEx(m_EventCancelParserBookmark, 0, false);
		if (WAIT_OBJECT_0 == dwResult || m_bCancel)
		{
			m_bCancel = true;
			break;
		}
		if (FSPDFDocUtil::IsValidBookmark(m_currentBookmark->GetNextSibling()))
		{
			m_currentBookmark = m_currentBookmark->GetNextSibling();
			ParseOneBookMark(nullptr, nullptr, series);
		}
		else
		{
			break;
		}
	}
	m_currentBookmark = m_currentBookmark->GetParent();
	OutputDebugString(L"GetNextSiblingBookMark - end3\r\n");
	return true;
}

String^ BookMarkParser::GetBookMarkHorizontalTabsBuffer(int series)
{
	if (series == 0)
		return nullptr;
	String^ tmp;
	return tmp;
}

bool BookMarkParser::ParseOneBookMark(foxit::pdf::Bookmark^ parent, foxit::pdf::Bookmark^ bookmark, int series)
{
	DWORD dwResult = WaitForSingleObjectEx(m_EventCancelParserBookmark, 0, false);
	if (WAIT_OBJECT_0 == dwResult || m_bCancel)
	{
		m_bCancel = true;
		return false;
	}
	BookmarkItem^ item = ref new BookmarkItem(m_currentBookmark, parent, series);
	m_nativeVect.push_back(item);

	return true;
}

void BookMarkParser::SetCurIterator(foxit::pdf::Bookmark^ pos)
{
	if (m_currentBookmark)
	{
		m_currentBookmark = pos;
	}
}

void BookMarkParser::SetDelegate(CollectBookMarkListDelegate^ del)
{
	collectBookMarkList = del;
}

void BookMarkParser::AddBookmarkItem()
{
	m_currentBookmark->Insert(L"Add Item", foxit::pdf::Position::e_PosLastSibling);

	m_currentBookmark = m_currentBookmark->GetParent();
}

bool BookMarkParser::RemoveBookmarkItem(foxit::pdf::Bookmark^ pos)
{
	/*if (m_currentBookmark)
	{
		m_currentBookmark = pos;
		m_currentBookmark = m_currentBookmark->GetParent();		
	}*/
	return false;
}
