﻿/**
 * Copyright (C) 2003-2018, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit Mobile PDF SDK to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */


#include "pch.h"
#include "ContentHostedDialog.h"


using namespace foxit::rdk::uiextensions;

using namespace Platform;
using namespace Windows::Foundation;
using namespace Windows::Foundation::Collections;
using namespace Windows::UI::Xaml;
using namespace Windows::UI::Xaml::Controls;
using namespace Windows::UI::Xaml::Data;
using namespace Windows::UI::Xaml::Documents;
using namespace Windows::UI::Xaml::Input;
using namespace Windows::UI::Xaml::Interop;
using namespace Windows::UI::Xaml::Media;

// The Templated Control item template is documented at http://go.microsoft.com/fwlink/?LinkId=234235

UIContentHostedDialog::UIContentHostedDialog()
{
	DefaultStyleKey = "foxit.rdk.uiextensions.ContentHostedDialog";

	if (Windows::ApplicationModel::DesignMode::DesignModeEnabled)
		return;

	OnInitializeControl();
}

static Windows::UI::Xaml::DependencyProperty^ _IsLightDismissEnabledProperty =
Windows::UI::Xaml::DependencyProperty::Register("IsLightDismissEnabled", Windows::UI::Xaml::Interop::TypeName(bool::typeid), Windows::UI::Xaml::Interop::TypeName(UIContentHostedDialog::typeid),
	ref new Windows::UI::Xaml::PropertyMetadata(false));

Windows::UI::Xaml::DependencyProperty^ UIContentHostedDialog::IsLightDismissEnabledProperty::get()
{
	return _IsLightDismissEnabledProperty;
}

bool UIContentHostedDialog::IsLightDismissEnabled::get()
{
	return (bool)GetValue(IsLightDismissEnabledProperty);
}

void UIContentHostedDialog::IsLightDismissEnabled::set(bool value)
{
	SetValue(IsLightDismissEnabledProperty, value);
}

static Windows::UI::Xaml::DependencyProperty^ _AutoCloseProperty =
Windows::UI::Xaml::DependencyProperty::Register("AutoClose", Windows::UI::Xaml::Interop::TypeName(bool::typeid), Windows::UI::Xaml::Interop::TypeName(UIContentHostedDialog::typeid),
	ref new Windows::UI::Xaml::PropertyMetadata(false));

Windows::UI::Xaml::DependencyProperty^ UIContentHostedDialog::AutoCloseProperty::get()
{
	return _AutoCloseProperty;
}

bool UIContentHostedDialog::AutoClose::get()
{
	return (bool)GetValue(AutoCloseProperty);
}

void UIContentHostedDialog::AutoClose::set(bool value)
{
	SetValue(AutoCloseProperty, value);
}

static Windows::UI::Xaml::DependencyProperty^ _IsContentFullScreenProperty =
Windows::UI::Xaml::DependencyProperty::Register("IsContentFullScreen", Windows::UI::Xaml::Interop::TypeName(bool::typeid), Windows::UI::Xaml::Interop::TypeName(UIContentHostedDialog::typeid),
	ref new Windows::UI::Xaml::PropertyMetadata(false));

Windows::UI::Xaml::DependencyProperty^ UIContentHostedDialog::IsContentFullScreenProperty::get()
{
	return _IsContentFullScreenProperty;
}

bool UIContentHostedDialog::IsContentFullScreen::get()
{
	return (bool)GetValue(IsContentFullScreenProperty);
}

void UIContentHostedDialog::IsContentFullScreen::set(bool value)
{
	SetValue(IsContentFullScreenProperty, value);
}

void UIContentHostedDialog::OnInitializeControl()
{
	this->Visibility = Windows::UI::Xaml::Visibility::Collapsed;

	this->Loaded += ref new Windows::UI::Xaml::RoutedEventHandler(this, &foxit::rdk::uiextensions::UIContentHostedDialog::OnLoaded);
	this->Unloaded += ref new Windows::UI::Xaml::RoutedEventHandler(this, &foxit::rdk::uiextensions::UIContentHostedDialog::OnUnloaded);
}

void UIContentHostedDialog::OnApplyTemplate()
{
	ContentControl::OnApplyTemplate();

	UnHookEvents();

	m_LayoutRoot = safe_cast<Windows::UI::Xaml::Controls::Panel^>(GetTemplateChild(L"LayoutRoot"));
	m_ContentPresenter = safe_cast<Windows::UI::Xaml::Controls::ContentPresenter^>(GetTemplateChild(L"ContentPresenter"));
	m_NonVisibilityBtn = safe_cast<foxit::rdk::uiextensions::UINonVisibilityButton^>(GetTemplateChild(L"NonVisibilityBtn"));

	HookUpEvents();
}

void UIContentHostedDialog::HookUpEvents()
{

}

void UIContentHostedDialog::UnHookEvents()
{

}

void UIContentHostedDialog::ShowDialog()
{
	if (m_IsShown)
	{
		return;
	}

	this->Visibility = Windows::UI::Xaml::Visibility::Visible;
	m_IsShown = true;

	m_ParentPanel = dynamic_cast<Windows::UI::Xaml::Controls::Panel^>(this->Parent);
	m_ParentContentControl = dynamic_cast<Windows::UI::Xaml::Controls::ContentControl^>(this->Parent);

	if (m_ParentPanel != nullptr)
	{
		unsigned int index = -1;
		if (m_ParentPanel->Children->IndexOf(this, &index))
		{
			m_ParentPanel->Children->RemoveAt(index);
		}
	}

	if (m_ParentContentControl != nullptr)
	{
		m_ParentContentControl->Content = nullptr;
	}

	if (m_ContentPopup == nullptr)
		m_ContentPopup = ref new Windows::UI::Xaml::Controls::Primitives::Popup();

	m_ContentPopup->Child = this;

	if (m_ParentPanel != nullptr)
	{
		m_ParentPanel->Children->Append(m_ContentPopup);
		m_ParentSizeChangedEventToken = m_ParentPanel->SizeChanged += ref new Windows::UI::Xaml::SizeChangedEventHandler(this, &foxit::rdk::uiextensions::UIContentHostedDialog::OnParentSizeChanged);
	}
	else if (m_ParentContentControl != nullptr)
	{
		m_ParentContentControl->Content = m_ContentPopup;
		m_ParentSizeChangedEventToken = m_ParentContentControl->SizeChanged += ref new Windows::UI::Xaml::SizeChangedEventHandler(this, &foxit::rdk::uiextensions::UIContentHostedDialog::OnParentSizeChanged);
	}
	else
	{
		m_TemporaryParentPanel = foxit::rdk::uiextensions::GetVisualChild<Windows::UI::Xaml::Controls::Panel^>(Windows::UI::Xaml::Window::Current->Content);

		if (m_TemporaryParentPanel)
		{
			m_TemporaryParentPanel->Children->Append(m_ContentPopup);
			m_ParentSizeChangedEventToken = m_TemporaryParentPanel->SizeChanged += ref new Windows::UI::Xaml::SizeChangedEventHandler(this, &foxit::rdk::uiextensions::UIContentHostedDialog::OnParentSizeChanged);
		}
	}

	m_ContentPopup->Opened += ref new Windows::Foundation::EventHandler<Platform::Object ^>(this, &foxit::rdk::uiextensions::UIContentHostedDialog::OnContentPopupOpened);
	m_ContentPopup->Loaded += ref new Windows::UI::Xaml::RoutedEventHandler(this, &foxit::rdk::uiextensions::UIContentHostedDialog::OnPopupLoaded);

	m_ContentPopup->IsOpen = true;
}

void UIContentHostedDialog::Hide()
{
	m_ContentPopup->IsOpen = false;
	m_ContentPopup->Child = nullptr;

	if (m_ParentPanel)
	{
		unsigned int index = -1;
		if (m_ParentPanel->Children->IndexOf(m_ContentPopup, &index))
		{
			m_ParentPanel->Children->RemoveAt(index);
		}

		m_ParentPanel->Children->Append(this);
		m_ParentPanel->SizeChanged -= m_ParentSizeChangedEventToken;
		m_ParentPanel = nullptr;
	}

	if (m_ParentContentControl)
	{
		m_ParentContentControl->Content = this;
		m_ParentContentControl->SizeChanged -= m_ParentSizeChangedEventToken;
		m_ParentContentControl = nullptr;
	}

	if (m_TemporaryParentPanel)
	{
		unsigned int index = -1;
		if (m_TemporaryParentPanel->Children->IndexOf(m_ContentPopup, &index))
		{
			m_TemporaryParentPanel->Children->RemoveAt(index);
			m_TemporaryParentPanel->SizeChanged -= m_ParentSizeChangedEventToken;
		}

		m_TemporaryParentPanel = nullptr;
	}

	m_ContentPopup = nullptr;
	this->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
	m_IsShown = false;
}

void UIContentHostedDialog::ResizeLayoutRoot()
{
	FrameworkElement^ root = m_TemporaryParentPanel;

	if (m_LayoutRoot)
	{
		m_LayoutRoot->Width = root->ActualWidth;
		m_LayoutRoot->Height = root->ActualHeight;

		auto _ContentEle = dynamic_cast<Windows::UI::Xaml::FrameworkElement^>(this->Content);
		if (_ContentEle)
		{
			if (IsContentFullScreen)
			{
				_ContentEle->Width = root->ActualWidth;
				_ContentEle->Height = root->ActualHeight;
			}
			else
			{
				if (_ContentEle->ActualWidth > root->ActualWidth)
				{
					_ContentEle->Width = root->ActualWidth;
				}
			}
		}
	}
}

void foxit::rdk::uiextensions::UIContentHostedDialog::OnLoaded(Platform::Object ^sender, Windows::UI::Xaml::RoutedEventArgs ^e)
{
	HookUpEvents();

	this->ApplyTemplate();

	if (m_LayoutRoot)
	{
		m_LayoutRoot->Tapped += ref new Windows::UI::Xaml::Input::TappedEventHandler(this, &foxit::rdk::uiextensions::UIContentHostedDialog::OnLayoutRootTapped);
	}

	if (AutoClose)
	{
		if (m_AutoCloseTimer == nullptr)
		{
			Windows::Foundation::TimeSpan _HideTimerTimeSpan;
			m_AutoCloseTimer = ref new Windows::UI::Xaml::DispatcherTimer();
			_HideTimerTimeSpan.Duration = 4 * 10000000; 
			m_AutoCloseTimer->Interval = _HideTimerTimeSpan;
			m_AutoCloseTimer->Tick += ref new Windows::Foundation::EventHandler<Platform::Object ^>(this, &foxit::rdk::uiextensions::UIContentHostedDialog::OnAutoCloseTick);
			m_AutoCloseTimer->Start();
		}
	}

	ResizeLayoutRoot();

	if (Windows::Foundation::Metadata::ApiInformation::IsTypePresent("Windows.Phone.UI.Input.HardwareButtons"))
	{
		m_BackPresseEventToken = Windows::Phone::UI::Input::HardwareButtons::BackPressed += ref new Windows::Foundation::EventHandler<Windows::Phone::UI::Input::BackPressedEventArgs ^>(this, &foxit::rdk::uiextensions::UIContentHostedDialog::OnBackPressed);
	}
	if (Windows::Foundation::Metadata::ApiInformation::IsTypePresent("Windows.UI.Core.SystemNavigationManager"))
	{
		m_SystemNavigationBackRequestedEventToken = Windows::UI::Core::SystemNavigationManager::GetForCurrentView()->BackRequested += ref new Windows::Foundation::EventHandler<Windows::UI::Core::BackRequestedEventArgs ^>(this, &foxit::rdk::uiextensions::UIContentHostedDialog::OnBackRequested);
	}
}


void foxit::rdk::uiextensions::UIContentHostedDialog::OnUnloaded(Platform::Object ^sender, Windows::UI::Xaml::RoutedEventArgs ^e)
{
	UnHookEvents();

	if (m_AutoCloseTimer && m_AutoCloseTimer->IsEnabled)
	{
		m_AutoCloseTimer->Stop();
	}

	if (Windows::Foundation::Metadata::ApiInformation::IsTypePresent("Windows.Phone.UI.Input.HardwareButtons"))
	{
		Windows::Phone::UI::Input::HardwareButtons::BackPressed -= m_BackPresseEventToken;
		m_BackPresseEventToken.Value = 0;
	}
	if (Windows::Foundation::Metadata::ApiInformation::IsTypePresent("Windows.UI.Core.SystemNavigationManager"))
	{
		Windows::UI::Core::SystemNavigationManager::GetForCurrentView()->BackRequested -= m_SystemNavigationBackRequestedEventToken;
		m_SystemNavigationBackRequestedEventToken.Value = 0;
	}
}


void foxit::rdk::uiextensions::UIContentHostedDialog::OnLayoutRootTapped(Platform::Object ^sender, Windows::UI::Xaml::Input::TappedRoutedEventArgs ^e)
{
	if (e->OriginalSource == sender)
	{
		auto _InputPane = Windows::UI::ViewManagement::InputPane::GetForCurrentView();
		auto _OccludedRect = _InputPane->OccludedRect;
		if (_OccludedRect.Height != 0)
		{
			//_InputPane->TryHide();
			
			if (m_NonVisibilityBtn)
			{
				m_NonVisibilityBtn->Focus(Windows::UI::Xaml::FocusState::Programmatic);
			}
		}

		if (IsLightDismissEnabled)
			Hide();

		e->Handled = true;
	}
}


void foxit::rdk::uiextensions::UIContentHostedDialog::OnParentSizeChanged(Platform::Object ^sender, Windows::UI::Xaml::SizeChangedEventArgs ^e)
{
	ResizeLayoutRoot();
}


void foxit::rdk::uiextensions::UIContentHostedDialog::OnContentPopupOpened(Platform::Object ^sender, Platform::Object ^args)
{

}


void foxit::rdk::uiextensions::UIContentHostedDialog::OnPopupLoaded(Platform::Object ^sender, Windows::UI::Xaml::RoutedEventArgs ^e)
{
	ResizeLayoutRoot();

	Windows::UI::Xaml::VisualStateManager::GoToState(this, L"OpenPopupState", true);
}


void foxit::rdk::uiextensions::UIContentHostedDialog::OnAutoCloseTick(Platform::Object ^sender, Platform::Object ^args)
{
	m_AutoCloseTimer->Stop();

	Hide();
}

void foxit::rdk::uiextensions::UIContentHostedDialog::OnBackPressed(Platform::Object ^sender, Windows::Phone::UI::Input::BackPressedEventArgs ^args)
{
	args->Handled = true;

	Hide();
}


void foxit::rdk::uiextensions::UIContentHostedDialog::OnBackRequested(Platform::Object ^sender, Windows::UI::Core::BackRequestedEventArgs ^args)
{
	args->Handled = true;

	Hide();
}
