/**
 * Copyright (C) 2003-2018, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit Mobile PDF SDK to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */


#include "pch.h"
#include "ControlSizeTrigger.h"

using namespace foxit::rdk::uiextensions;

using namespace Platform;

UIControlSizeTrigger::UIControlSizeTrigger()
{
	_MinWidth = _MinHeight = -1;
}

Windows::UI::Xaml::FrameworkElement^ UIControlSizeTrigger::TargetElement::get()
{
	return _TargetElement;
}

void UIControlSizeTrigger::TargetElement::set(Windows::UI::Xaml::FrameworkElement^ value)
{
	_TargetElement = value;
	_TargetElement->SizeChanged += ref new Windows::UI::Xaml::SizeChangedEventHandler(this, &foxit::rdk::uiextensions::UIControlSizeTrigger::OnSizeChanged);
}

double UIControlSizeTrigger::MinWidth::get()
{
	return _MinWidth;
}

void UIControlSizeTrigger::MinWidth::set(double value)
{
	_MinWidth = value;
}

double UIControlSizeTrigger::MinHeight::get()
{
	return _MinHeight;
}

void UIControlSizeTrigger::MinHeight::set(double value)
{
	_MinHeight = value;
}

void UIControlSizeTrigger::UpdateTrigger()
{
	if (_TargetElement != nullptr && (_MinWidth > 0 || _MinHeight > 0))
	{
		//if both minHeight and minWidth are set, then both conditions must be satisfied
		if (_MinHeight > 0 && _MinWidth > 0)
		{
			SetActive((_currentHeight >= _MinHeight) && (_currentWidth >= _MinWidth));
		}
		//if only one of them is set, then only that condition needs to be satisfied
		else if (_MinHeight > 0)
		{
			SetActive(_currentHeight >= _MinHeight);
		}
		else
		{
			SetActive(_currentWidth >= _MinWidth);
		}
	}
	else
	{
		SetActive(false);
	}
}

void UIControlSizeTrigger::OnSizeChanged(Platform::Object ^sender, Windows::UI::Xaml::SizeChangedEventArgs ^e)
{
	_currentHeight = e->NewSize.Height;
	_currentWidth = e->NewSize.Width;

	UpdateTrigger();
}