/**
 * Copyright (C) 2003-2018, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit Mobile PDF SDK to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */


#include "pch.h"
#include "FileSizeConverter.h"
#include "LocalResourceLoader.h"

using namespace foxit::rdk::uiextensions;

using namespace Platform;
using namespace Windows::UI::Xaml::Interop;
using namespace Windows::ApplicationModel::Resources;

// See http://go.microsoft.com/fwlink/?LinkId=267279 for info about the FileSizeConverter class
// and other data converters.

FileSizeConverter::FileSizeConverter()
{
    m_loader = ref new LocalResourceLoader();
}

FileSizeConverter::FileSizeConverter(foxit::rdk::uiextensions::IResourceLoader^ loader) : m_loader(loader)
{
}

Platform::Object^ FileSizeConverter::Convert(Platform::Object^ value, Windows::UI::Xaml::Interop::TypeName targetType, Platform::Object^ parameter, Platform::String^ language)
{
    float64 size = static_cast<float64>(safe_cast<uint64>(value));
    std::array<String^, 3> units = 
    { 
        m_loader->GetString("BytesUnit"), 
        m_loader->GetString("KilobytesUnit"), 
        m_loader->GetString("MegabytesUnit") 
    };
    unsigned int index = 0;

    while (size >= 1024)
    {
        size /= 1024;
        index++;
    }

    return ToTwoDecimalPlaces(size) + " " + units[index];
}

float64 FileSizeConverter::ToTwoDecimalPlaces(float64 value)
{
    float64 f;
    float64 intpart;
    float64 fractpart;
    fractpart = modf(value, &intpart);
    f = floor(fractpart * 100 + 0.5) / 100.0;
    return intpart + f;
}

Platform::Object^ FileSizeConverter::ConvertBack(Platform::Object^ value, Windows::UI::Xaml::Interop::TypeName targetType, Platform::Object^ parameter, Platform::String^ language)
{
    throw ref new Platform::NotImplementedException();
}
