﻿/**
* Copyright (C) 2003-2018, Foxit Software Inc..
* All Rights Reserved.
*
* http://www.foxitsoftware.com
*
* The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
* distribute any parts of Foxit Mobile PDF SDK to third party or public without permission unless an agreement
* is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
* Review legal.txt for additional license and legal information.
*/


#include "pch.h"
#include "FlyoutHostedControl.h"

using namespace foxit::rdk::uiextensions;

using namespace Platform;
using namespace Windows::Foundation;
using namespace Windows::Foundation::Collections;
using namespace Windows::UI::Xaml;
using namespace Windows::UI::Xaml::Controls;
using namespace Windows::UI::Xaml::Data;
using namespace Windows::UI::Xaml::Documents;
using namespace Windows::UI::Xaml::Input;
using namespace Windows::UI::Xaml::Interop;
using namespace Windows::UI::Xaml::Media;

// The Templated Control item template is documented at https://go.microsoft.com/fwlink/?LinkId=234235

FlyoutHostedControl::FlyoutHostedControl()
{
	DefaultStyleKey = "foxit.rdk.uiextensions.FlyoutHostedControl";

	if (Windows::ApplicationModel::DesignMode::DesignModeEnabled)
		return;

	OnInitializeControl();
}

static Windows::UI::Xaml::DependencyProperty^ _IsPinEnabledProperty =
Windows::UI::Xaml::DependencyProperty::Register("IsPinEnabled", Windows::UI::Xaml::Interop::TypeName(bool::typeid), Windows::UI::Xaml::Interop::TypeName(FlyoutHostedControl::typeid),
	ref new Windows::UI::Xaml::PropertyMetadata(false, ref new Windows::UI::Xaml::PropertyChangedCallback(FlyoutHostedControl::OnIsPinEnabledChanged)));

Windows::UI::Xaml::DependencyProperty^ FlyoutHostedControl::IsPinEnabledProperty::get()
{
	return _IsPinEnabledProperty;
}

bool FlyoutHostedControl::IsPinEnabled::get()
{
	return (bool)GetValue(IsPinEnabledProperty);
}

void FlyoutHostedControl::IsPinEnabled::set(bool value)
{
	SetValue(IsPinEnabledProperty, value);
}

void FlyoutHostedControl::OnIsPinEnabledChanged(Windows::UI::Xaml::DependencyObject^ obj, Windows::UI::Xaml::DependencyPropertyChangedEventArgs^ args)
{
	FlyoutHostedControl^ _control = static_cast<FlyoutHostedControl^>(obj);

	bool _oldValue = (bool)(args->OldValue);
	bool _newValue = (bool)(args->NewValue);
	bool nowValue = _control->IsPinEnabled;

	_control->OnIsPinEnabledValueChanged(_oldValue, _newValue);
}

void FlyoutHostedControl::OnIsPinEnabledValueChanged(bool oldValue, bool newValue)
{
	if (!this->Parent)
		return;

	if (oldValue)
	{
		auto pFrameEle = m_PlaceTarget.Resolve<Windows::UI::Xaml::Controls::Panel>();
		auto pPanel = dynamic_cast<Windows::UI::Xaml::Controls::Panel^>(pFrameEle);
		if (pPanel)
		{
			unsigned int index = -1;
			if (pPanel->Children->IndexOf(this, &index))
				pPanel->Children->RemoveAt(index);
		}

		auto pContentControl = dynamic_cast<Windows::UI::Xaml::Controls::ContentControl^>(pFrameEle);
		if (pContentControl)
		{
			pContentControl->Content = nullptr;
		}
		Hide(this, nullptr);
	}
	else
	{
		m_Flyout->Content = nullptr;
		m_Flyout->Hide();
	}

	ShowFlyoutPage(m_PlaceTarget.Resolve<Windows::UI::Xaml::FrameworkElement>());
}

static Windows::UI::Xaml::DependencyProperty^ _TitleProperty =
Windows::UI::Xaml::DependencyProperty::Register("Title", Windows::UI::Xaml::Interop::TypeName(Platform::String::typeid), Windows::UI::Xaml::Interop::TypeName(FlyoutHostedControl::typeid),
	ref new Windows::UI::Xaml::PropertyMetadata(L""));

Windows::UI::Xaml::DependencyProperty^ FlyoutHostedControl::TitleProperty::get()
{
	return _TitleProperty;
}

Platform::String^ FlyoutHostedControl::Title::get()
{
	return dynamic_cast<Platform::String^>(GetValue(TitleProperty));
}

void FlyoutHostedControl::Title::set(Platform::String^ value)
{
	SetValue(TitleProperty, value);
}

void FlyoutHostedControl::OnInitializeControl()
{
	if (m_Flyout == nullptr)
	{
		auto _Flyout = ref new Windows::UI::Xaml::Controls::Flyout();
		Windows::UI::Xaml::Style^ _FlyoutPresenterStyle;
		_FlyoutPresenterStyle = this->Resources->HasKey(L"FlyoutPresenterStyle") ? dynamic_cast<Windows::UI::Xaml::Style^>(this->Resources->Lookup(L"FlyoutPresenterStyle")) : nullptr;
		_Flyout->FlyoutPresenterStyle = _FlyoutPresenterStyle;

		_Flyout->Opened += ref new Windows::Foundation::EventHandler<Platform::Object ^>(this, &foxit::rdk::uiextensions::FlyoutHostedControl::OnFlyoutOpened);
		_Flyout->Closed += ref new Windows::Foundation::EventHandler<Platform::Object ^>(this, &foxit::rdk::uiextensions::FlyoutHostedControl::OnFlyoutClosed);

		m_Flyout = _Flyout;
	}

	this->Loaded += ref new Windows::UI::Xaml::RoutedEventHandler(this, &foxit::rdk::uiextensions::FlyoutHostedControl::OnLoaded);
	this->Unloaded += ref new Windows::UI::Xaml::RoutedEventHandler(this, &foxit::rdk::uiextensions::FlyoutHostedControl::OnUnloaded);
}

void FlyoutHostedControl::OnApplyTemplate()
{
	ContentControl::OnApplyTemplate();

	UnHookEvents();

	m_LayoutRoot = safe_cast<Windows::UI::Xaml::Controls::Panel^>(GetTemplateChild(L"LayoutRoot"));
	m_ContentPresenter = safe_cast<Windows::UI::Xaml::Controls::ContentPresenter^>(GetTemplateChild(L"ContentPresenter"));
	m_CloseButton = safe_cast<Windows::UI::Xaml::Controls::Button^>(GetTemplateChild(L"CloseButton"));

	HookUpEvents();
}

void FlyoutHostedControl::HookUpEvents()
{
	if (m_CloseButton)
	{
		if (m_CloseButtonClickEventToken.Value == 0)
			m_CloseButtonClickEventToken = m_CloseButton->Click += ref new Windows::UI::Xaml::RoutedEventHandler(this, &foxit::rdk::uiextensions::FlyoutHostedControl::OnCloseButtonClick);
	}

	if (m_WindowSizeChangedEventToken.Value == 0)
		m_WindowSizeChangedEventToken = Windows::UI::ViewManagement::ApplicationView::GetForCurrentView()->VisibleBoundsChanged += ref new Windows::Foundation::TypedEventHandler<Windows::UI::ViewManagement::ApplicationView ^, Platform::Object ^>(this, &foxit::rdk::uiextensions::FlyoutHostedControl::OnVisibleBoundsChanged);
	if (Windows::Foundation::Metadata::ApiInformation::IsTypePresent("Windows.Phone.UI.Input.HardwareButtons"))
	{
		if (m_BackPresseEventToken.Value == 0)
			m_BackPresseEventToken = Windows::Phone::UI::Input::HardwareButtons::BackPressed += ref new Windows::Foundation::EventHandler<Windows::Phone::UI::Input::BackPressedEventArgs ^>(this, &foxit::rdk::uiextensions::FlyoutHostedControl::OnBackPressed);
	}
	if (Windows::Foundation::Metadata::ApiInformation::IsTypePresent("Windows.UI.Core.SystemNavigationManager"))
	{
		if (m_SystemNavigationBackRequestedEventToken.Value == 0)
			m_SystemNavigationBackRequestedEventToken = Windows::UI::Core::SystemNavigationManager::GetForCurrentView()->BackRequested += ref new Windows::Foundation::EventHandler<Windows::UI::Core::BackRequestedEventArgs ^>(this, &foxit::rdk::uiextensions::FlyoutHostedControl::OnBackRequested);
	}
}

void FlyoutHostedControl::UnHookEvents()
{
	if (m_CloseButtonClickEventToken.Value)
	{
		m_CloseButton->Click -= m_CloseButtonClickEventToken;
		m_CloseButtonClickEventToken.Value = 0;
	}

	if (m_WindowSizeChangedEventToken.Value)
	{
		Windows::UI::ViewManagement::ApplicationView::GetForCurrentView()->VisibleBoundsChanged -= m_WindowSizeChangedEventToken;
		m_WindowSizeChangedEventToken.Value = 0;
	}

	if (Windows::Foundation::Metadata::ApiInformation::IsTypePresent("Windows.Phone.UI.Input.HardwareButtons"))
	{
		if (m_BackPresseEventToken.Value)
		{
			Windows::Phone::UI::Input::HardwareButtons::BackPressed -= m_BackPresseEventToken;
			m_BackPresseEventToken.Value = 0;
		}
	}
	if (Windows::Foundation::Metadata::ApiInformation::IsTypePresent("Windows.UI.Core.SystemNavigationManager"))
	{
		if (m_SystemNavigationBackRequestedEventToken.Value)
		{
			Windows::UI::Core::SystemNavigationManager::GetForCurrentView()->BackRequested -= m_SystemNavigationBackRequestedEventToken;
			m_SystemNavigationBackRequestedEventToken.Value = 0;
		}
	}
}

void foxit::rdk::uiextensions::FlyoutHostedControl::OnLoaded(Platform::Object ^sender, Windows::UI::Xaml::RoutedEventArgs ^e)
{
	HookUpEvents();

	this->ApplyTemplate();

	Windows::UI::Xaml::Style^ _FlyoutPresenterStyle;
	_FlyoutPresenterStyle = m_LayoutRoot->Resources->HasKey(L"FlyoutPresenterStyle") ? dynamic_cast<Windows::UI::Xaml::Style^>(m_LayoutRoot->Resources->Lookup(L"FlyoutPresenterStyle")) : nullptr;
	m_Flyout->FlyoutPresenterStyle = _FlyoutPresenterStyle;

	UpdateFlyoutPosition();
}

void foxit::rdk::uiextensions::FlyoutHostedControl::OnUnloaded(Platform::Object ^sender, Windows::UI::Xaml::RoutedEventArgs ^e)
{
	UnHookEvents();
}

void foxit::rdk::uiextensions::FlyoutHostedControl::OnCloseButtonClick(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{
	CloseFlyout();
}

void FlyoutHostedControl::UpdateFlyoutPosition()
{
	auto _VisibleBounds = Windows::UI::ViewManagement::ApplicationView::GetForCurrentView()->VisibleBounds;
	auto _Bounds = Windows::UI::Xaml::Window::Current->Bounds;
	if (!m_Flyout)
		return;

	auto _FlyoutPresenter = GetVisualParent<Windows::UI::Xaml::Controls::FlyoutPresenter^>(m_Flyout->Content);
	if (_FlyoutPresenter)
	{
#ifdef _DEBUG
		std::wstringstream ws;
		ws << L"Window::Current->SizeChanged--MinHeight: " << _FlyoutPresenter->MinHeight;
		ws << L", MaxHeight: " << _FlyoutPresenter->MaxHeight;
		ws << L", ActualWidth: " << _FlyoutPresenter->ActualWidth;
		ws << L", ActualHeight: " << _FlyoutPresenter->ActualHeight;
		ws << L", VisibleBounds.Width: " << _VisibleBounds.Width;
		ws << L", VisibleBounds.Height: " << _VisibleBounds.Height;
		ws << L", _Bounds.Width: " << _Bounds.Width;
		ws << L", _Bounds.Height: " << _Bounds.Height;
		ws << L"\n";
		OutputDebugString(ws.str().c_str());
#endif

		auto _height = _VisibleBounds.Height;
		_FlyoutPresenter->MinHeight = _height;
		_FlyoutPresenter->MaxHeight = _height;
		_FlyoutPresenter->Height = _height;
		if (_FlyoutPresenter->ActualWidth > _VisibleBounds.Width)
		{
			_FlyoutPresenter->MinWidth = _VisibleBounds.Width;
			_FlyoutPresenter->MaxWidth = _VisibleBounds.Width;
			_FlyoutPresenter->Width = _VisibleBounds.Width;
		}

		Windows::UI::Xaml::Controls::Primitives::Popup^ _Popup = dynamic_cast<Windows::UI::Xaml::Controls::Primitives::Popup^>(_FlyoutPresenter->Parent);
		if (_Popup)
		{
			_Popup->IsLightDismissEnabled = false;
			double _voffset = _Popup->VerticalOffset;
			if (Windows::System::Profile::AnalyticsInfo::VersionInfo->DeviceFamily == L"Windows.Mobile")
			{
				auto _StatusBar = Windows::UI::ViewManagement::StatusBar::GetForCurrentView();
				_voffset = _StatusBar->OccludedRect.Height;
			}
			else if (Windows::System::Profile::AnalyticsInfo::VersionInfo->DeviceFamily == L"Windows.Desktop")
			{
				auto _screenPt = _FlyoutPresenter->TransformToVisual(nullptr)->TransformPoint(Windows::Foundation::Point(0, 0));
				auto pPlaceControl = m_PlaceTarget.Resolve<Windows::UI::Xaml::FrameworkElement>();
				Windows::UI::Xaml::Media::TranslateTransform^ _transform = dynamic_cast<Windows::UI::Xaml::Media::TranslateTransform^>(pPlaceControl->RenderTransform);
				if (_transform == nullptr)
				{
					_transform = ref new Windows::UI::Xaml::Media::TranslateTransform();
					pPlaceControl->RenderTransform = _transform;
				}

				if (_transform->Y != _screenPt.Y)
					_transform->Y = _screenPt.Y;

				_voffset = 0;
			}

			_Popup->VerticalOffset = _voffset;
			_Popup->HorizontalOffset = (_Bounds.Width - _FlyoutPresenter->ActualWidth > 0) ? _Bounds.Width - _FlyoutPresenter->ActualWidth : 0;
		}
	}
}

void foxit::rdk::uiextensions::FlyoutHostedControl::OnFlyoutOpened(Platform::Object ^sender, Platform::Object ^args)
{
	UpdateFlyoutPosition();
}

void foxit::rdk::uiextensions::FlyoutHostedControl::OnFlyoutClosed(Platform::Object ^sender, Platform::Object ^args)
{
	Hide(this, nullptr);
}

void foxit::rdk::uiextensions::FlyoutHostedControl::OnVisibleBoundsChanged(Windows::UI::ViewManagement::ApplicationView ^sender, Platform::Object ^args)
{
	UpdateFlyoutPosition();
}

void FlyoutHostedControl::ShowFlyoutPage(Windows::UI::Xaml::FrameworkElement^ pFrameworkElement)
{
	if (pFrameworkElement == nullptr)
		return;

	if (!pFrameworkElement->Parent)
		return;

	auto pFrameEle = m_PlaceTarget.Resolve<Windows::UI::Xaml::FrameworkElement>();
	if (pFrameEle != pFrameworkElement)
		m_PlaceTarget = pFrameworkElement;

	Show(this, nullptr);
	if (IsPinEnabled)
	{
		unsigned int index = -1;
		Windows::UI::Xaml::Controls::Panel^ pPanel = dynamic_cast<Windows::UI::Xaml::Controls::Panel^>(pFrameworkElement);
		Windows::UI::Xaml::Media::TranslateTransform^ _transform = dynamic_cast<Windows::UI::Xaml::Media::TranslateTransform^>(pPanel->RenderTransform);
		if (_transform)
		{
			if (_transform->Y != 0)
				_transform->Y = 0;
		}
		
		if (!pPanel->Children->IndexOf(this, &index))
			pPanel->Children->Append(this);
	}
	else
	{
		m_Flyout->Content = this;
		Windows::UI::Xaml::Controls::Flyout::SetAttachedFlyout(pFrameworkElement, m_Flyout);
		Windows::UI::Xaml::Controls::Flyout::ShowAttachedFlyout(pFrameworkElement);
	}
}

void FlyoutHostedControl::CloseFlyout()
{
	if (!this->Parent)
		return;

	if (IsPinEnabled)
	{
		auto pFrameEle = m_PlaceTarget.Resolve<Windows::UI::Xaml::Controls::Panel>();
		auto pPanel = dynamic_cast<Windows::UI::Xaml::Controls::Panel^>(pFrameEle);
		if (pPanel)
		{
			unsigned int index = -1;
			if (pPanel->Children->IndexOf(this, &index))
				pPanel->Children->RemoveAt(index);
		}

		auto pContentControl = dynamic_cast<Windows::UI::Xaml::Controls::ContentControl^>(pFrameEle);
		if (pContentControl)
		{
			pContentControl->Content = nullptr;
		}
		Hide(this, nullptr);
	}
	else
	{
		m_Flyout->Hide();
		m_Flyout->Content = nullptr;
	}
}

void foxit::rdk::uiextensions::FlyoutHostedControl::OnBackPressed(Platform::Object ^sender, Windows::Phone::UI::Input::BackPressedEventArgs ^args)
{
	args->Handled = true;

	CloseFlyout();
}


void foxit::rdk::uiextensions::FlyoutHostedControl::OnBackRequested(Platform::Object ^sender, Windows::UI::Core::BackRequestedEventArgs ^args)
{
	args->Handled = true;

	CloseFlyout();
}
