/**
 * Copyright (C) 2003-2018, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit Mobile PDF SDK to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */


#include "pch.h"
#include "NumericTextBox.h"

using namespace foxit::rdk::uiextensions;

using namespace Platform;
using namespace Platform::Collections;
using namespace Windows::Foundation;
using namespace Windows::Foundation::Collections;
using namespace Windows::System;
using namespace Windows::UI::Core;
using namespace Windows::UI::ViewManagement;
using namespace Windows::UI::Xaml;
using namespace Windows::UI::Xaml::Controls;
using namespace Windows::UI::Xaml::Interop;
using namespace Windows::UI::Xaml::Navigation;

NumericTextBox::NumericTextBox()
{
	DefaultStyleKey = L"foxit.rdk.uiextensions.NumericTextBox";

	if (Windows::ApplicationModel::DesignMode::DesignModeEnabled)
		return;

	OnInitializeControl();
}

static DependencyProperty^ _allowSpaceProperty =
	DependencyProperty::Register("AllowSpace",
	TypeName(bool::typeid), TypeName(Page::typeid), ref new PropertyMetadata(false));

DependencyProperty^ NumericTextBox::AllowSpaceProperty::get()
{
	return _allowSpaceProperty;
}

bool NumericTextBox::AllowSpace::get()
{
	return safe_cast<bool>(GetValue(AllowSpaceProperty));
}

void NumericTextBox::AllowSpace::set(bool value)
{
	SetValue(AllowSpaceProperty, value);
}

void NumericTextBox::OnInitializeControl()
{
	this->ApplyTemplate();

	this->TextChanged += ref new Windows::UI::Xaml::Controls::TextChangedEventHandler(this, &foxit::rdk::uiextensions::NumericTextBox::OnTextChanged);
	this->TextCompositionChanged += ref new Windows::Foundation::TypedEventHandler<Windows::UI::Xaml::Controls::TextBox ^, Windows::UI::Xaml::Controls::TextCompositionChangedEventArgs ^>(this, &foxit::rdk::uiextensions::NumericTextBox::OnTextCompositionChanged);
}

void NumericTextBox::OnKeyDown(Windows::UI::Xaml::Input::KeyRoutedEventArgs^ e)
{
	OutputDebugString(L"NumericTextBox::OnKeyDown....\n");

	Control::OnKeyDown(e);
	if (e->Handled)
		return;

	auto _virtualKey = e->Key;
	auto _keyStatus = e->KeyStatus;
	auto coreWindow = Window::Current->CoreWindow;
	auto downState = Windows::UI::Core::CoreVirtualKeyStates::Down;
	bool menuKey = (coreWindow->GetKeyState(VirtualKey::Menu) & downState) == downState;
	bool controlKey = (coreWindow->GetKeyState(VirtualKey::Control) & downState) == downState;
	bool shiftKey = (coreWindow->GetKeyState(VirtualKey::Shift) & downState) == downState;
	bool noModifiers = !menuKey && !controlKey && !shiftKey;

	if (_virtualKey >= Windows::System::VirtualKey::Number0 && _virtualKey <= Windows::System::VirtualKey::Number9 && noModifiers)
	{

	}
	else if (_virtualKey >= Windows::System::VirtualKey::NumberPad0 && _virtualKey <= Windows::System::VirtualKey::NumberPad9 && noModifiers)
	{

	}
	else if (AllowSpace && _virtualKey == Windows::System::VirtualKey::Space)
	{
		
	}
	else if (_virtualKey == Windows::System::VirtualKey::Enter)
	{

	}
	else
	{
		e->Handled = true;
	}
}



void foxit::rdk::uiextensions::NumericTextBox::OnTextChanged(Platform::Object ^sender, Windows::UI::Xaml::Controls::TextChangedEventArgs ^e)
{
	auto _CurString = this->Text;

	OutputDebugString((L"NumericTextBox::OnTextChanged...[" + m_PrevInputString + L"],[" + _CurString + L"]\n")->Data());

	if (m_PrevInputString != _CurString)
	{
		auto _prevStrLength = m_PrevInputString->Length();
		auto _curStrLength = _CurString->Length();
		auto _SelectionStart = this->SelectionStart;
		auto _SelectionLength = this->SelectionLength;
		auto _coreWindow = Window::Current->CoreWindow;
		auto _downState = Windows::UI::Core::CoreVirtualKeyStates::Down;
		bool _controlKey = (_coreWindow->GetKeyState(Windows::System::VirtualKey::Control) & _downState) == _downState;
		bool _shiftKey = (_coreWindow->GetKeyState(Windows::System::VirtualKey::Shift) & _downState) == _downState;
		bool _altKey = (_coreWindow->GetKeyState(Windows::System::VirtualKey::Menu) & _downState) == _downState;
		auto _capitalLockKey = (_coreWindow->GetKeyState(Windows::System::VirtualKey::CapitalLock) & Windows::UI::Core::CoreVirtualKeyStates::Locked) == Windows::UI::Core::CoreVirtualKeyStates::Locked;

		wchar_t* pSrc = (wchar_t*)m_PrevInputString->Data();
		wchar_t* pDes = (wchar_t*)_CurString->Data();
		wchar_t* pSrcStart = pSrc;
		wchar_t* pDesStart = pDes;
		wchar_t* pSrcEnd = NULL;
		wchar_t* pDesEnd = NULL;

		wchar_t* p = (wchar_t*)m_PrevInputString->Data();
		wchar_t* q = (wchar_t*)_CurString->Data();
		while (1)
		{
			if (*p == '\0' || *q == '\0')
			{
				pSrcStart = p;
				pDesStart = q;
				break;
			}

			if (*p == *q)
			{
				p++;
				q++;
			}
			else
			{
				pSrcStart = p;
				pDesStart = q;
				break;
			}
		}

		p = (wchar_t*)m_PrevInputString->Data() + _prevStrLength;
		q = (wchar_t*)_CurString->Data() + _curStrLength;
		while (1)
		{
			if (p - pSrcStart == 0 || q - pDesStart == 0)
			{
				if (*p == *q)
				{
					pSrcEnd = p;
					pDesEnd = q;
				}
				else
				{
					pSrcEnd = ++p;
					pDesEnd = ++q;
				}

				break;
			}
			else
			{
				if (*p == *q)
				{
					p--;
					q--;
				}
				else
				{
					pSrcEnd = ++p;
					pDesEnd = ++q;
					break;
				}
			}
		}

		auto _needDeleteCount = pSrcEnd - pSrcStart;
		auto _iLength = pDesEnd - pDesStart;
		auto _iStr = ref new Platform::String(pDesStart, (unsigned int)_iLength);
		p = (wchar_t*)_iStr->Data();

		OutputDebugString((L"NumericTextBox::OnTextChanged...[Delete: " + _needDeleteCount + L"],[Add: " + _iLength + L"]\n")->Data());

		m_PrevInputString = ref new Platform::String(Text->Data());
	}
}


void foxit::rdk::uiextensions::NumericTextBox::OnTextCompositionChanged(Windows::UI::Xaml::Controls::TextBox ^sender, Windows::UI::Xaml::Controls::TextCompositionChangedEventArgs ^args)
{
	auto _CurString = this->Text;
	auto _StartIndex = args->StartIndex;
	auto _Length = args->Length;

	OutputDebugString((L"NumericTextBox::OnTextCompositionChanged...[" + m_PrevInputString + L"],[" + _CurString + L"], StartIndex: " + args->StartIndex + L", Length: " + args->Length + L"\n")->Data());
}
