﻿/**
 * Copyright (C) 2003-2018, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit Mobile PDF SDK to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */


#include "pch.h"
#include "SearchTextBox.h"

using namespace foxit::rdk::uiextensions;

using namespace Platform;
using namespace Windows::Foundation;
using namespace Windows::Foundation::Collections;
using namespace Windows::UI::Xaml;
using namespace Windows::UI::Xaml::Controls;
using namespace Windows::UI::Xaml::Data;
using namespace Windows::UI::Xaml::Documents;
using namespace Windows::UI::Xaml::Input;
using namespace Windows::UI::Xaml::Interop;
using namespace Windows::UI::Xaml::Media;

// The Templated Control item template is documented at http://go.microsoft.com/fwlink/?LinkId=234235

SearchTextBox::SearchTextBox()
{
	DefaultStyleKey = "foxit.rdk.uiextensions.SearchTextBox";

	if (Windows::ApplicationModel::DesignMode::DesignModeEnabled)
		return;

	OnInitializeControl();
}

static Windows::UI::Xaml::DependencyProperty^ _IsSearchingProperty = 
Windows::UI::Xaml::DependencyProperty::Register("IsSearching", Windows::UI::Xaml::Interop::TypeName(bool::typeid), Windows::UI::Xaml::Interop::TypeName(SearchTextBox::typeid),
	ref new Windows::UI::Xaml::PropertyMetadata(false, ref new Windows::UI::Xaml::PropertyChangedCallback(SearchTextBox::OnIsSearchingChanged)));

Windows::UI::Xaml::DependencyProperty^ SearchTextBox::IsSearchingProperty::get()
{
	return _IsSearchingProperty;
}

bool SearchTextBox::IsSearching::get()
{
	return (bool)GetValue(IsSearchingProperty);
}

void SearchTextBox::IsSearching::set(bool value)
{
	SetValue(IsSearchingProperty, value);
}

void SearchTextBox::OnIsSearchingChanged(Windows::UI::Xaml::DependencyObject^ obj, Windows::UI::Xaml::DependencyPropertyChangedEventArgs^ args)
{
	SearchTextBox^ _control = static_cast<SearchTextBox^>(obj);

	bool _oldValue = (bool)(args->OldValue);
	bool _newValue = (bool)(args->NewValue);
	
}

void SearchTextBox::OnInitializeControl()
{
	this->ApplyTemplate();
	
	this->KeyDown += ref new Windows::UI::Xaml::Input::KeyEventHandler(this, &foxit::rdk::uiextensions::SearchTextBox::OnKeyDown);
	this->TextChanged += ref new Windows::UI::Xaml::Controls::TextChangedEventHandler(this, &foxit::rdk::uiextensions::SearchTextBox::OnTextChanged);
	this->GotFocus += ref new Windows::UI::Xaml::RoutedEventHandler(this, &foxit::rdk::uiextensions::SearchTextBox::OnGotFocus);
	this->LostFocus += ref new Windows::UI::Xaml::RoutedEventHandler(this, &foxit::rdk::uiextensions::SearchTextBox::OnLostFocus);

	this->Loaded += ref new Windows::UI::Xaml::RoutedEventHandler(this, &foxit::rdk::uiextensions::SearchTextBox::OnLoaded);
	this->Unloaded += ref new Windows::UI::Xaml::RoutedEventHandler(this, &foxit::rdk::uiextensions::SearchTextBox::OnUnloaded);
}

void SearchTextBox::OnApplyTemplate()
{
	TextBox::OnApplyTemplate();

	UnHookEvents();

	m_QueryButton = safe_cast<Windows::UI::Xaml::Controls::Button^>(GetTemplateChild(L"QueryButton"));
	m_PlaceholderTextContentPresenter = safe_cast<Windows::UI::Xaml::Controls::ContentControl^>(GetTemplateChild(L"PlaceholderTextContentPresenter"));

	HookUpEvents();
}

void SearchTextBox::HookUpEvents()
{
	if (m_QueryButtonClickEventRegistrationToken.Value == 0)
	{
		if (m_QueryButton)
		{
			m_QueryButtonClickEventRegistrationToken = m_QueryButton->Click += ref new Windows::UI::Xaml::RoutedEventHandler(this, &foxit::rdk::uiextensions::SearchTextBox::OnQueryButtonClick);
		}
	}
}

void SearchTextBox::UnHookEvents()
{
	if (m_QueryButtonClickEventRegistrationToken.Value)
	{
		if (m_QueryButton)
		{
			m_QueryButton->Click -= m_QueryButtonClickEventRegistrationToken;
			m_QueryButtonClickEventRegistrationToken.Value = 0;
		}
	}
}

void foxit::rdk::uiextensions::SearchTextBox::OnLoaded(Platform::Object ^sender, Windows::UI::Xaml::RoutedEventArgs ^e)
{
	//IsSearching = true;

	HookUpEvents();
}


void foxit::rdk::uiextensions::SearchTextBox::OnUnloaded(Platform::Object ^sender, Windows::UI::Xaml::RoutedEventArgs ^e)
{
	UnHookEvents();
}


void foxit::rdk::uiextensions::SearchTextBox::OnKeyDown(Platform::Object ^sender, Windows::UI::Xaml::Input::KeyRoutedEventArgs ^e)
{
	auto _virtualKey = e->Key;
	auto _coreWindow = Window::Current->CoreWindow;
	auto _downState = Windows::UI::Core::CoreVirtualKeyStates::Down;
	bool _menuKey = (_coreWindow->GetKeyState(Windows::System::VirtualKey::Menu) & _downState) == _downState;
	bool _controlKey = (_coreWindow->GetKeyState(Windows::System::VirtualKey::Control) & _downState) == _downState;
	bool _shiftKey = (_coreWindow->GetKeyState(Windows::System::VirtualKey::Shift) & _downState) == _downState;
	bool _numberKeyLocktKey = (_coreWindow->GetAsyncKeyState(_virtualKey) & _downState) == _downState;
	bool _noModifiers = !_menuKey && !_controlKey && !_shiftKey;
	bool _onlyAlt = _menuKey && !_controlKey && !_shiftKey;

	if (_noModifiers && _virtualKey == Windows::System::VirtualKey::Enter)
	{
		auto args = ref new SearchTextBoxQuerySubmittedEventArgs();
		args->QueryText = Text;
		if (m_TextChanged && SearchCheckStringValid(Text))
			IsSearching = true;
		QuerySubmitted(this, args);
		e->Handled = true;
	}
}


void foxit::rdk::uiextensions::SearchTextBox::OnQueryButtonClick(Platform::Object ^sender, Windows::UI::Xaml::RoutedEventArgs ^e)
{
	auto args = ref new SearchTextBoxQuerySubmittedEventArgs();
	args->QueryText = Text;
	if (SearchCheckStringValid(Text))
		IsSearching = true;
	QuerySubmitted(this, args);
}


void foxit::rdk::uiextensions::SearchTextBox::OnTextChanged(Platform::Object ^sender, Windows::UI::Xaml::Controls::TextChangedEventArgs ^e)
{
	m_TextChanged = true;
	IsSearching = false;
}


void foxit::rdk::uiextensions::SearchTextBox::OnGotFocus(Platform::Object ^sender, Windows::UI::Xaml::RoutedEventArgs ^e)
{
	/*if (m_PlaceholderTextContentPresenter)
	{
		m_PlaceholderTextContentPresenter->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
	}*/
}


void foxit::rdk::uiextensions::SearchTextBox::OnLostFocus(Platform::Object ^sender, Windows::UI::Xaml::RoutedEventArgs ^e)
{
	/*if (m_PlaceholderTextContentPresenter)
	{
		if (Text == nullptr || Text->Length() == 0)
		{
			m_PlaceholderTextContentPresenter->Visibility = Windows::UI::Xaml::Visibility::Visible;
		}
	}*/
}

Platform::String^ SearchTextBox::TrimSpace(Platform::String^ str)
{
	Platform::String^ _resultstr = ref new Platform::String(L"");
	if (str == nullptr && str->Length() == 0)
		return _resultstr;

	wchar_t* pstart = (wchar_t*)str->Data();
	while (*pstart == ' ')
		pstart++;

	wchar_t* p = (wchar_t*)str->Data() + str->Length();
	if (p == pstart)
		return _resultstr;
	p--;
	while (*p == ' ' && p != pstart)
		p--;

	auto length = p - pstart + 1;
	_resultstr = ref new Platform::String(pstart, (unsigned int)length);
	return _resultstr;
}

bool SearchTextBox::SearchCheckStringValid(Platform::String^ findstr)
{
	auto _text = TrimSpace(findstr);
	return _text && _text->Length() > 0 ? true : false;
}
