/**
 * Copyright (C) 2003-2018, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit Mobile PDF SDK to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */


#include "pch.h"
#include "Utilities.h"
#include <robuffer.h>
#include <wrl.h>
#include <sstream>  

using namespace foxit::rdk::uiextensions;
using namespace foxit::rdk;

using namespace Platform;
using namespace Windows::Storage::Streams;
using namespace Microsoft::WRL;
using namespace concurrency;

void foxit::rdk::uiextensions::Utility::ThrowIfFailed(HRESULT hr)
{
	if (FAILED(hr))
	{
		throw Exception::CreateException(hr);
	}
}

// Retrieves the raw pixel data from the provided IBuffer object.
byte* foxit::rdk::uiextensions::Utility::GetPointerToPixelData(Windows::Storage::Streams::IBuffer^ buffer, unsigned int* length/* = nullptr*/)
{
	if (buffer && length)
		*length = buffer->Length;

	// Cast to Object^, then to its underlying IInspectable interface.
	Platform::Object^ _obj = buffer;
	ComPtr<IInspectable> _insp(reinterpret_cast<IInspectable*>(_obj));

	// Query the IBufferByteAccess interface.
	ComPtr<Windows::Storage::Streams::IBufferByteAccess> _bufferByteAccess;
	ThrowIfFailed(_insp.As(&_bufferByteAccess));

	// Retrieve the buffer data.
	byte* pixels = nullptr;
	ThrowIfFailed(_bufferByteAccess->Buffer(&pixels));
	return pixels;
}


Platform::String^ foxit::rdk::uiextensions::Utility::GetGUID()
{
	wchar_t buf[64] = { 0 };
	GUID guid;

	if (S_OK == ::CoCreateGuid(&guid))
	{
		_snwprintf_s(buf, 64
			, L"{%08X-%04X-%04x-%02X%02X-%02X%02X%02X%02X%02X%02X}"
			, guid.Data1
			, guid.Data2
			, guid.Data3
			, guid.Data4[0], guid.Data4[1]
			, guid.Data4[2], guid.Data4[3], guid.Data4[4], guid.Data4[5]
			, guid.Data4[6], guid.Data4[7]
			);
	}
	return ref new Platform::String(buf);
}

Platform::String^ foxit::rdk::uiextensions::Utility::GetUuid()
{
	wchar_t buf[64] = { 0 };
	GUID guid;

	if (S_OK == ::CoCreateGuid(&guid))
	{
		_snwprintf_s(buf, 64
			, L"%08X%04X%04X%02X%02X%02X%02X%02X%02X%02X%02X"
			, guid.Data1
			, guid.Data2
			, guid.Data3
			, guid.Data4[0], guid.Data4[1]
			, guid.Data4[2], guid.Data4[3], guid.Data4[4], guid.Data4[5]
			, guid.Data4[6], guid.Data4[7]
			);
	}
	return ref new Platform::String(buf);
}

Platform::String^ foxit::rdk::uiextensions::Utility::GetcPDFUuid()
{
	wchar_t buf[96] = { 0 };
	GUID guid, guid1, guid2;

	if (S_OK == ::CoCreateGuid(&guid) && S_OK == ::CoCreateGuid(&guid1) && S_OK == ::CoCreateGuid(&guid2))
	{
		_snwprintf_s(buf, 96
			, L"%08X%04X%04X%02X%02X%02X%02X%02X%02X%02X%02X%08X%04X%04X-%02X%02X%02X%02X%02X%02X%02X%02X-%08X%04X%04X%02X%02X"
			, guid.Data1
			, guid.Data2
			, guid.Data3
			, guid.Data4[0], guid.Data4[1]
			, guid.Data4[2], guid.Data4[3], guid.Data4[4], guid.Data4[5]
			, guid.Data4[6], guid.Data4[7]
			, guid1.Data1
			, guid1.Data2
			, guid1.Data3
			, guid1.Data4[0], guid1.Data4[1]
			, guid1.Data4[2], guid1.Data4[3], guid1.Data4[4], guid1.Data4[5]
			, guid1.Data4[6], guid1.Data4[7]
			, guid2.Data1
			, guid2.Data2
			, guid2.Data3
			, 0x86, 0x10
			);
	}
	return ref new Platform::String(buf);
}

Windows::UI::Color foxit::rdk::uiextensions::Utility::ConvertFromString(Platform::String^ colorstr)
{
	if (!colorstr)
		return Windows::UI::ColorHelper::FromArgb(255, 255, 255, 255);

	auto colorstrlen = colorstr->Length();
	auto start = colorstr->Begin();

	int a = 0xff;
	int r = 0xff;
	int g = 0xff;
	int b = 0xff;
	int ret = 0;

	if (start[0] == '#' && (colorstrlen == 7 || colorstrlen == 9))
	{
		if (colorstrlen == 7)
		{
			ret = swscanf_s(start, L"#%02X%02X%02X", &r, &g, &b);
			if (ret != 3)
				return Windows::UI::ColorHelper::FromArgb(255, 255, 255, 255);
		}
		else if (colorstrlen == 9)
		{
			ret = swscanf_s(start, L"#%02X%02X%02X%02X", &a, &r, &g, &b);
			if (ret != 4)
				return Windows::UI::ColorHelper::FromArgb(255, 255, 255, 255);
		}
		return Windows::UI::ColorHelper::FromArgb((unsigned char)a, (unsigned char)r, (unsigned char)g, (unsigned char)b);
	}

	return Windows::UI::ColorHelper::FromArgb(255, 255, 255, 255);
}

Platform::String^ foxit::rdk::uiextensions::Utility::ConvertToColorString(Windows::UI::Color color)
{
	wchar_t str[10] = { 0 };

	swprintf_s(str, L"#%02X%02X%02X%02X", color.A, color.R, color.G, color.B);

	return ref new Platform::String(str);
}

foxit::common::fxcrt::Matrix2D^ foxit::rdk::uiextensions::Utility::ConvertToFoxitMatrix(Windows::UI::Xaml::Media::Matrix matrix)
{
	foxit::common::fxcrt::Matrix2D^ tmp = ref new foxit::common::fxcrt::Matrix2D(
		(FS_FLOAT)matrix.M11,
	(FS_FLOAT)matrix.M12,
	(FS_FLOAT)matrix.M21,
	(FS_FLOAT)matrix.M22,
	(FS_FLOAT)matrix.OffsetX,
	(FS_FLOAT)matrix.OffsetY
	);
	
	return tmp;
}

Windows::UI::Xaml::Media::Matrix foxit::rdk::uiextensions::Utility::ConvertFromFoxitMatrix(foxit::common::fxcrt::Matrix2D^ matrix)
{
	return Windows::UI::Xaml::Media::MatrixHelper::FromElements(matrix->a, matrix->b, matrix->c, matrix->d, matrix->e, matrix->f);
}

bool foxit::rdk::uiextensions::Utility::IsEqualColor(Windows::UI::Color a, Windows::UI::Color b)
{
	return (a.A == b.A && a.R == b.R && a.G == b.G && a.B == b.B) ? true : false;
}

Platform::String^ foxit::rdk::uiextensions::Utility::TrimSpace(Platform::String^ str)
{
	Platform::String^ _resultstr = ref new Platform::String(L"");
	if (str == nullptr && str->Length() == 0)
		return _resultstr;

	wchar_t* pstart = (wchar_t*)str->Data();
	while (*pstart == ' ')
		pstart++;

	wchar_t* p = (wchar_t*)str->Data() + str->Length();
	if (p == pstart)
		return _resultstr;
	p--;
	while (*p == ' ' && p != pstart)
		p--;

	auto length = p - pstart + 1;
	_resultstr = ref new Platform::String(pstart, (unsigned int)length);
	return _resultstr;
}

std::string foxit::rdk::uiextensions::Utility::ConvertToStdString(Platform::String^ str)
{	

	const wchar_t* szStr = str->Data();
	int nLen = WideCharToMultiByte(CP_UTF8, 0, szStr, -1, NULL, 0, NULL, NULL);
	if (nLen == 0)
	{
		return NULL;
	}
	char* pResult = new char[nLen];
	WideCharToMultiByte(CP_UTF8, 0, szStr, -1, pResult, nLen, NULL, NULL);

	std::string endStr = pResult;

	return endStr;
}

bool foxit::rdk::uiextensions::Utility::IsDesktopPlatform()
{
	return Windows::System::Profile::AnalyticsInfo::VersionInfo->DeviceFamily == L"Windows.Desktop";
}

bool foxit::rdk::uiextensions::Utility::IsMobilePlatform()
{
	return Windows::System::Profile::AnalyticsInfo::VersionInfo->DeviceFamily == L"Windows.Mobile";
}

Platform::Collections::Map<Platform::String^, Windows::UI::Xaml::Media::Imaging::BitmapImage^>^ foxit::rdk::uiextensions::Utility::m_BitmapMap = ref new Platform::Collections::Map<Platform::String^, Windows::UI::Xaml::Media::Imaging::BitmapImage^>();

void foxit::rdk::uiextensions::Utility::InitAnnotBitmapMap()
{
	m_BitmapMap->Insert(L"Default", ref new Windows::UI::Xaml::Media::Imaging::BitmapImage(ref new Windows::Foundation::Uri(L"ms-appx:///foxit.rdk.uiextensions/Assets/Annot/icon/defaulticon.png")));
	m_BitmapMap->Insert(L"Replace", ref new Windows::UI::Xaml::Media::Imaging::BitmapImage(ref new Windows::Foundation::Uri(L"ms-appx:///foxit.rdk.uiextensions/Assets/Annot/ReplaceText.png")));
	m_BitmapMap->Insert(L"Arrow", ref new Windows::UI::Xaml::Media::Imaging::BitmapImage(ref new Windows::Foundation::Uri(L"ms-appx:///foxit.rdk.uiextensions/Assets/Annot/icon/arrow.png")));
	m_BitmapMap->Insert(L"HighLight", ref new Windows::UI::Xaml::Media::Imaging::BitmapImage(ref new Windows::Foundation::Uri(L"ms-appx:///foxit.rdk.uiextensions/Assets/Annot/icon/highlight.png")));
	m_BitmapMap->Insert(L"InsertText", ref new Windows::UI::Xaml::Media::Imaging::BitmapImage(ref new Windows::Foundation::Uri(L"ms-appx:///foxit.rdk.uiextensions/Assets/Annot/icon/inserttext.png")));
	m_BitmapMap->Insert(L"Line", ref new Windows::UI::Xaml::Media::Imaging::BitmapImage(ref new Windows::Foundation::Uri(L"ms-appx:///foxit.rdk.uiextensions/Assets/Annot/icon/line.png")));
	m_BitmapMap->Insert(L"Text", ref new Windows::UI::Xaml::Media::Imaging::BitmapImage(ref new Windows::Foundation::Uri(L"ms-appx:///foxit.rdk.uiextensions/Assets/Annot/icon/note.png")));
	m_BitmapMap->Insert(L"Circle", ref new Windows::UI::Xaml::Media::Imaging::BitmapImage(ref new Windows::Foundation::Uri(L"ms-appx:///foxit.rdk.uiextensions/Assets/Annot/icon/circle.png")));
	m_BitmapMap->Insert(L"Ink", ref new Windows::UI::Xaml::Media::Imaging::BitmapImage(ref new Windows::Foundation::Uri(L"ms-appx:///foxit.rdk.uiextensions/Assets/Annot/icon/ink.png")));
	m_BitmapMap->Insert(L"Square", ref new Windows::UI::Xaml::Media::Imaging::BitmapImage(ref new Windows::Foundation::Uri(L"ms-appx:///foxit.rdk.uiextensions/Assets/Annot/icon/square.png")));
	m_BitmapMap->Insert(L"Squiggly", ref new Windows::UI::Xaml::Media::Imaging::BitmapImage(ref new Windows::Foundation::Uri(L"ms-appx:///foxit.rdk.uiextensions/Assets/Annot/icon/squiggly.png")));
	m_BitmapMap->Insert(L"Stamp", ref new Windows::UI::Xaml::Media::Imaging::BitmapImage(ref new Windows::Foundation::Uri(L"ms-appx:///foxit.rdk.uiextensions/Assets/Annot/icon/stamp.png")));
	m_BitmapMap->Insert(L"StrikeOut", ref new Windows::UI::Xaml::Media::Imaging::BitmapImage(ref new Windows::Foundation::Uri(L"ms-appx:///foxit.rdk.uiextensions/Assets/Annot/icon/strikeout.png")));
	m_BitmapMap->Insert(L"FreeText", ref new Windows::UI::Xaml::Media::Imaging::BitmapImage(ref new Windows::Foundation::Uri(L"ms-appx:///foxit.rdk.uiextensions/Assets/Annot/icon/textbox.png")));
	m_BitmapMap->Insert(L"FreeTextTypewriter", ref new Windows::UI::Xaml::Media::Imaging::BitmapImage(ref new Windows::Foundation::Uri(L"ms-appx:///foxit.rdk.uiextensions/Assets/Annot/icon/typewriter.png")));
	m_BitmapMap->Insert(L"FreeTextCallout", ref new Windows::UI::Xaml::Media::Imaging::BitmapImage(ref new Windows::Foundation::Uri(L"ms-appx:///foxit.rdk.uiextensions/Assets/Annot/icon/callout.png")));
	m_BitmapMap->Insert(L"UnderLine", ref new Windows::UI::Xaml::Media::Imaging::BitmapImage(ref new Windows::Foundation::Uri(L"ms-appx:///foxit.rdk.uiextensions/Assets/Annot/icon/underline.png")));
	m_BitmapMap->Insert(L"Caret", ref new Windows::UI::Xaml::Media::Imaging::BitmapImage(ref new Windows::Foundation::Uri(L"ms-appx:///foxit.rdk.uiextensions/Assets/Annot/icon/caret.png")));
	m_BitmapMap->Insert(L"Link", ref new Windows::UI::Xaml::Media::Imaging::BitmapImage(ref new Windows::Foundation::Uri(L"ms-appx:///foxit.rdk.uiextensions/Assets/Annot/icon/link.png")));
}

Windows::UI::Xaml::Media::Imaging::BitmapImage^ foxit::rdk::uiextensions::Utility::GetAnnotBitmap(Platform::String^ key)
{
	if (m_BitmapMap->Size == 0)
		InitAnnotBitmapMap();

	if (m_BitmapMap->HasKey(key))
	{
		return m_BitmapMap->Lookup(key);
	}
	
	return m_BitmapMap->Lookup(L"Default");
}

foxit::pdf::PDFPage^ FoxitTypeConvert::ConvertFromWraperClass(Platform::Object^ pageObj)
{
	PDFAnnotWrapper^ wrap = (PDFAnnotWrapper^)pageObj;
	return (foxit::pdf::PDFPage^)wrap->Pointer;
}


Windows::UI::Color FoxitTypeConvert::ConvertFromColorString(Platform::String^ colorstr)
{
	if (!colorstr)
		return Windows::UI::ColorHelper::FromArgb(255, 255, 255, 255);

	auto colorstrlen = colorstr->Length();
	auto start = colorstr->Begin();

	int a = 0xff;
	int r = 0xff;
	int g = 0xff;
	int b = 0xff;
	int ret = 0;

	if (start[0] == '#' && (colorstrlen == 7 || colorstrlen == 9))
	{
		if (colorstrlen == 7)
		{
			ret = swscanf_s(start, L"#%02X%02X%02X", &r, &g, &b);
			if (ret != 3)
				return Windows::UI::ColorHelper::FromArgb(255, 255, 255, 255);
		}
		else if (colorstrlen == 9)
		{
			ret = swscanf_s(start, L"#%02X%02X%02X%02X", &a, &r, &g, &b);
			if (ret != 4)
				return Windows::UI::ColorHelper::FromArgb(255, 255, 255, 255);
		}
		return Windows::UI::ColorHelper::FromArgb((unsigned char)a, (unsigned char)r, (unsigned char)g, (unsigned char)b);
	}

	return Windows::UI::ColorHelper::FromArgb(255, 255, 255, 255);
}

Platform::String^ FoxitTypeConvert::ConvertToColorString(Windows::UI::Color color)
{
	wchar_t str[10] = { 0 };

	swprintf_s(str, L"#%02X%02X%02X%02X", color.A, color.R, color.G, color.B);

	return ref new Platform::String(str);
}

foxit::pdf::annots::Style FoxitTypeConvert::ConvertToFoxitAnnotBorderStyle(AnnotBorderStyle borderstyle)
{
	foxit::pdf::annots::Style style;

	switch (borderstyle)
	{
	case AnnotBorderStyle::SOLID:
		style = foxit::pdf::annots::Style::e_Solid;// FSPDF_ANNOT_BORDERSTYLE_SOLID;
		break;
	case AnnotBorderStyle::DASHED:
		style = foxit::pdf::annots::Style::e_Dashed;// FSPDF_ANNOT_BORDERSTYLE_DASHED;
		break;
	case AnnotBorderStyle::UNDERLINE:
		style = foxit::pdf::annots::Style::e_UnderLine;// FSPDF_ANNOT_BORDERSTYLE_UNDERLINE;
		break;
	case AnnotBorderStyle::BEVELED:
		style = foxit::pdf::annots::Style::e_Beveled;// FSPDF_ANNOT_BORDERSTYLE_BEVELED;
		break;
	case AnnotBorderStyle::INSET:
		style = foxit::pdf::annots::Style::e_Inset;// FSPDF_ANNOT_BORDERSTYLE_INSET;
		break;
	case AnnotBorderStyle::CLOUDY:
		style = foxit::pdf::annots::Style::e_Cloudy;// FSPDF_ANNOT_BORDERSTYLE_CLOUDY;
		break;
	default:
		style = foxit::pdf::annots::Style::e_Solid;// FSPDF_ANNOT_BORDERSTYLE_SOLID;
		break;
	}

	return style;
}

AnnotBorderStyle FoxitTypeConvert::ConvertFromFoxitAnnotBorderStyle(foxit::pdf::annots::Style borderstyle)
{
	AnnotBorderStyle style;

	if (borderstyle == foxit::pdf::annots::Style::e_Solid)
	{
		style = AnnotBorderStyle::SOLID;
	}
	else if (borderstyle == foxit::pdf::annots::Style::e_Dashed)
	{
		style = AnnotBorderStyle::DASHED;
	}
	else if (borderstyle == foxit::pdf::annots::Style::e_UnderLine)
	{
		style = AnnotBorderStyle::UNDERLINE;
	}
	else if (borderstyle == foxit::pdf::annots::Style::e_Beveled)
	{
		style = AnnotBorderStyle::BEVELED;
	}
	else if (borderstyle == foxit::pdf::annots::Style::e_Inset)
	{
		style = AnnotBorderStyle::INSET;
	}
	else if (borderstyle == foxit::pdf::annots::Style::e_Cloudy)
	{
		style = AnnotBorderStyle::CLOUDY;
	}
	else
	{
		style = AnnotBorderStyle::SOLID;
	}
	return style;
}

foxit::common::DateTime^ FoxitTypeConvert::ConvertToFoxitDateTime(Windows::Foundation::DateTime datetime)
{
	auto _Calendar = ref new Windows::Globalization::Calendar();
	if (_Calendar->GetClock() != Windows::Globalization::ClockIdentifiers::TwentyFourHour)
		_Calendar->ChangeClock(Windows::Globalization::ClockIdentifiers::TwentyFourHour);

	_Calendar->SetDateTime(datetime);

	
	int yearValue = _Calendar->Year;
	int monthValue = _Calendar->Month;
	int dayValue = _Calendar->Day;
	int hourValue = _Calendar->Hour;
	int minValue = _Calendar->Minute;
	int secondValue = _Calendar->Second;
	int milliValue = _Calendar->Nanosecond/1000000;

	foxit::common::DateTime^ _DateTime = ref new foxit::common::DateTime();
	_DateTime->year = _Calendar->Year;
	_DateTime->month = _Calendar->Month;
	_DateTime->day = _Calendar->Day;
	_DateTime->hour = _Calendar->Hour;
	_DateTime->minute = _Calendar->Minute;
	_DateTime->second = _Calendar->Second;
	_DateTime->milliseconds = _Calendar->Nanosecond/1000000;

	TIME_ZONE_INFORMATION timezone_info;
	GetTimeZoneInformation(&timezone_info);
	int timezone_hour = (timezone_info.Bias / 60) * -1;
	int timezone_minute = (abs(timezone_info.Bias) % 60);

	_DateTime->utc_hour_offset = timezone_hour;
	_DateTime->utc_minute_offset = timezone_minute;
	return _DateTime;
}

Windows::Foundation::DateTime FoxitTypeConvert::ConvertFromFoxitDateTime(foxit::common::DateTime^ datetime)
{
	auto _Calendar = ref new Windows::Globalization::Calendar();
	if (_Calendar->GetClock() != Windows::Globalization::ClockIdentifiers::TwentyFourHour)
		_Calendar->ChangeClock(Windows::Globalization::ClockIdentifiers::TwentyFourHour);
	Windows::Foundation::DateTime _DateTime;

	_Calendar->Year = datetime->year;
	_Calendar->Month = datetime->month;
	_Calendar->Day = datetime->day;
	_Calendar->Hour = datetime->hour;
	_Calendar->Minute = datetime->minute;
	_Calendar->Second = datetime->second;
	_Calendar->Nanosecond = datetime->milliseconds * 1000000;
	_DateTime = _Calendar->GetDateTime();
	return _DateTime;
}

Platform::String^ FoxitTypeConvert::ConvertToFoxitAnnotIconNameTextTypeString(AnnotIconNameTextType icontype)
{
	Platform::String^ val;
	switch (icontype)
	{
	case AnnotIconNameTextType::Checkmark:
		val = FSPDF_ANNOT_ICONNAME_TEXT_CHECKMARK;
		break;
	case AnnotIconNameTextType::Circle:
		val = FSPDF_ANNOT_ICONNAME_TEXT_CIRCLE;
		break;
	case AnnotIconNameTextType::Comment:
		val = FSPDF_ANNOT_ICONNAME_TEXT_COMMENT;
		break;
	case AnnotIconNameTextType::Cross:
		val = FSPDF_ANNOT_ICONNAME_TEXT_CROSS;
		break;
	case AnnotIconNameTextType::Help:
		val = FSPDF_ANNOT_ICONNAME_TEXT_HELP;
		break;
	case AnnotIconNameTextType::Insert:
		val = FSPDF_ANNOT_ICONNAME_TEXT_INSERT;
		break;
	case AnnotIconNameTextType::Key:
		val = FSPDF_ANNOT_ICONNAME_TEXT_KEY;
		break;
	case AnnotIconNameTextType::NewParagraph:
		val = FSPDF_ANNOT_ICONNAME_TEXT_NEWPARAGRAPH;
		break;
	case AnnotIconNameTextType::Note:
		val = FSPDF_ANNOT_ICONNAME_TEXT_NOTE;
		break;
	case AnnotIconNameTextType::Paragraph:
		val = FSPDF_ANNOT_ICONNAME_TEXT_PARAGRAPH;
		break;
	case AnnotIconNameTextType::RightArrow:
		val = FSPDF_ANNOT_ICONNAME_TEXT_RIGHARROW;
		break;
	case AnnotIconNameTextType::RightPointer:
		val = FSPDF_ANNOT_ICONNAME_TEXT_RIGHPOINTER;
		break;
	case AnnotIconNameTextType::Star:
		val = FSPDF_ANNOT_ICONNAME_TEXT_STAR;
		break;
	case AnnotIconNameTextType::UpArrow:
		val = FSPDF_ANNOT_ICONNAME_TEXT_UPARROW;
		break;
	case AnnotIconNameTextType::UpleftArrow:
		val = FSPDF_ANNOT_ICONNAME_TEXT_UPLEFTARROW;
		break;
	default:
		val = FSPDF_ANNOT_ICONNAME_TEXT_COMMENT;
		break;
	}

	return val;
}

AnnotIconNameTextType FoxitTypeConvert::ConvertFromFoxitAnnotIconNameTextTypeString(Platform::String^ iconname)
{
	AnnotIconNameTextType icontype = AnnotIconNameTextType::Comment;

	if (iconname == FSPDF_ANNOT_ICONNAME_TEXT_CHECKMARK)
	{
		icontype = AnnotIconNameTextType::Checkmark;
	}
	else if (iconname == FSPDF_ANNOT_ICONNAME_TEXT_CIRCLE)
	{
		icontype = AnnotIconNameTextType::Circle;
	}
	else if (iconname == FSPDF_ANNOT_ICONNAME_TEXT_COMMENT)
	{
		icontype = AnnotIconNameTextType::Comment;
	}
	else if (iconname == FSPDF_ANNOT_ICONNAME_TEXT_CROSS)
	{
		icontype = AnnotIconNameTextType::Cross;
	}
	else if (iconname == FSPDF_ANNOT_ICONNAME_TEXT_HELP)
	{
		icontype = AnnotIconNameTextType::Help;
	}
	else if (iconname == FSPDF_ANNOT_ICONNAME_TEXT_INSERT)
	{
		icontype = AnnotIconNameTextType::Insert;
	}
	else if (iconname == FSPDF_ANNOT_ICONNAME_TEXT_KEY)
	{
		icontype = AnnotIconNameTextType::Key;
	}
	else if (iconname == FSPDF_ANNOT_ICONNAME_TEXT_NEWPARAGRAPH)
	{
		icontype = AnnotIconNameTextType::NewParagraph;
	}
	else if (iconname == FSPDF_ANNOT_ICONNAME_TEXT_NOTE)
	{
		icontype = AnnotIconNameTextType::Note;
	}
	else if (iconname == FSPDF_ANNOT_ICONNAME_TEXT_PARAGRAPH)
	{
		icontype = AnnotIconNameTextType::Paragraph;
	}
	else if (iconname == FSPDF_ANNOT_ICONNAME_TEXT_RIGHARROW)
	{
		icontype = AnnotIconNameTextType::RightArrow;
	}
	else if (iconname == FSPDF_ANNOT_ICONNAME_TEXT_RIGHPOINTER)
	{
		icontype = AnnotIconNameTextType::RightPointer;
	}
	else if (iconname == FSPDF_ANNOT_ICONNAME_TEXT_STAR)
	{
		icontype = AnnotIconNameTextType::Star;
	}
	else if (iconname == FSPDF_ANNOT_ICONNAME_TEXT_UPARROW)
	{
		icontype = AnnotIconNameTextType::UpArrow;
	}
	else if (iconname == FSPDF_ANNOT_ICONNAME_TEXT_UPLEFTARROW)
	{
		icontype = AnnotIconNameTextType::UpleftArrow;
	}

	return icontype;
}

Platform::String^ FoxitTypeConvert::ConvertToFoxitAnnotIconNameFileattachTypeString(AnnotIconNameFileattachType icontype)
{
	Platform::String^ val;
	switch (icontype)
	{
	case AnnotIconNameFileattachType::Graph:
		val = FSPDF_ANNOT_ICONNAME_FILEATTACH_GRAPH;
		break;
	case AnnotIconNameFileattachType::PaperClip:
		val = FSPDF_ANNOT_ICONNAME_FILEATTACH_PAPERCLIP;
		break;
	case AnnotIconNameFileattachType::PushPin:
		val = FSPDF_ANNOT_ICONNAME_FILEATTACH_PUSHPIN;
		break;
	case AnnotIconNameFileattachType::Tag:
		val = FSPDF_ANNOT_ICONNAME_FILEATTACH_TAG;
		break;
	default:
		break;
	}

	return val;
}

AnnotIconNameFileattachType FoxitTypeConvert::ConvertFromFoxitAnnotIconNameFileattachTypeString(Platform::String^ iconname)
{
	AnnotIconNameFileattachType icontype = AnnotIconNameFileattachType::Graph;

	if (iconname == FSPDF_ANNOT_ICONNAME_FILEATTACH_GRAPH)
	{
		icontype = AnnotIconNameFileattachType::Graph;
	}
	else if (iconname == FSPDF_ANNOT_ICONNAME_FILEATTACH_PAPERCLIP)
	{
		icontype = AnnotIconNameFileattachType::PaperClip;
	}
	else if (iconname == FSPDF_ANNOT_ICONNAME_FILEATTACH_PUSHPIN)
	{
		icontype = AnnotIconNameFileattachType::PushPin;
	}
	else if (iconname == FSPDF_ANNOT_ICONNAME_FILEATTACH_TAG)
	{
		icontype = AnnotIconNameFileattachType::Tag;
	}

	return icontype;
}

Platform::String^ FoxitTypeConvert::ConvertToFoxitAnnotIconNameStampTypeString(AnnotIconNameStampType icontype)
{
	Platform::String^ val;
	switch (icontype)
	{
	case AnnotIconNameStampType::Approved:
		val = FSPDF_ANNOT_ICONNAME_STAMP_APPROVED;
		break;
	case AnnotIconNameStampType::AsIs:
		val = FSPDF_ANNOT_ICONNAME_STAMP_ASIS;
		break;
	case AnnotIconNameStampType::Confidential:
		val = FSPDF_ANNOT_ICONNAME_STAMP_CONFIDENTIAL;
		break;
	case AnnotIconNameStampType::Departmental:
		val = FSPDF_ANNOT_ICONNAME_STAMP_DEPARTMENTAL;
		break;
	case AnnotIconNameStampType::Draft:
		val = FSPDF_ANNOT_ICONNAME_STAMP_DRAFT;
		break;
	case AnnotIconNameStampType::Experimental:
		val = FSPDF_ANNOT_ICONNAME_STAMP_EXPERIMENTAL;
		break;
	case AnnotIconNameStampType::Expired:
		val = FSPDF_ANNOT_ICONNAME_STAMP_EXPIRED;
		break;
	case AnnotIconNameStampType::Final:
		val = FSPDF_ANNOT_ICONNAME_STAMP_FINAL;
		break;
	case AnnotIconNameStampType::ForComment:
		val = FSPDF_ANNOT_ICONNAME_STAMP_FORCOMMENT;
		break;
	case AnnotIconNameStampType::ForPublicRelease:
		val = FSPDF_ANNOT_ICONNAME_STAMP_FORPUBLICRELEASE;
		break;
	case AnnotIconNameStampType::NotApproved:
		val = FSPDF_ANNOT_ICONNAME_STAMP_NOTAPPROVED;
		break;
	case AnnotIconNameStampType::NotForPublicRelease:
		val = FSPDF_ANNOT_ICONNAME_STAMP_NOTFORPUBLICRELEASE;
		break;
	case AnnotIconNameStampType::Sold:
		val = FSPDF_ANNOT_ICONNAME_STAMP_SOLD;
		break;
	case AnnotIconNameStampType::TopSecret:
		val = FSPDF_ANNOT_ICONNAME_STAMP_TOPSECRET;
		break;
	default:
		val = FSPDF_ANNOT_ICONNAME_STAMP_APPROVED;
		break;
	}

	return val;
}

AnnotIconNameStampType FoxitTypeConvert::ConvertFromFoxitAnnotIconNameStampTypeString(Platform::String^ iconname)
{
	AnnotIconNameStampType icontype = AnnotIconNameStampType::Approved;

	if (iconname == FSPDF_ANNOT_ICONNAME_STAMP_APPROVED)
	{
		icontype = AnnotIconNameStampType::Approved;
	}
	else if (iconname == FSPDF_ANNOT_ICONNAME_STAMP_ASIS)
	{
		icontype = AnnotIconNameStampType::AsIs;
	}
	else if (iconname == FSPDF_ANNOT_ICONNAME_STAMP_CONFIDENTIAL)
	{
		icontype = AnnotIconNameStampType::Confidential;
	}
	else if (iconname == FSPDF_ANNOT_ICONNAME_STAMP_DEPARTMENTAL)
	{
		icontype = AnnotIconNameStampType::Departmental;
	}
	else if (iconname == FSPDF_ANNOT_ICONNAME_STAMP_DRAFT)
	{
		icontype = AnnotIconNameStampType::Draft;
	}
	else if (iconname == FSPDF_ANNOT_ICONNAME_STAMP_EXPERIMENTAL)
	{
		icontype = AnnotIconNameStampType::Experimental;
	}
	else if (iconname == FSPDF_ANNOT_ICONNAME_STAMP_EXPIRED)
	{
		icontype = AnnotIconNameStampType::Expired;
	}
	else if (iconname == FSPDF_ANNOT_ICONNAME_STAMP_FINAL)
	{
		icontype = AnnotIconNameStampType::Final;
	}
	else if (iconname == FSPDF_ANNOT_ICONNAME_STAMP_FORCOMMENT)
	{
		icontype = AnnotIconNameStampType::ForComment;
	}
	else if (iconname == FSPDF_ANNOT_ICONNAME_STAMP_FORPUBLICRELEASE)
	{
		icontype = AnnotIconNameStampType::ForPublicRelease;
	}
	else if (iconname == FSPDF_ANNOT_ICONNAME_STAMP_NOTAPPROVED)
	{
		icontype = AnnotIconNameStampType::NotApproved;
	}
	else if (iconname == FSPDF_ANNOT_ICONNAME_STAMP_NOTFORPUBLICRELEASE)
	{
		icontype = AnnotIconNameStampType::NotForPublicRelease;
	}
	else if (iconname == FSPDF_ANNOT_ICONNAME_STAMP_SOLD)
	{
		icontype = AnnotIconNameStampType::Sold;
	}
	else if (iconname == FSPDF_ANNOT_ICONNAME_STAMP_TOPSECRET)
	{
		icontype = AnnotIconNameStampType::TopSecret;
	}

	return icontype;
}

Platform::String^ FoxitTypeConvert::ConvertToFoxitAnnotIconNameSoundTypeString(AnnotIconNameSoundType icontype)
{
	Platform::String^ val;

	switch (icontype)
	{
	case AnnotIconNameSoundType::Speaker:
		val = FSPDF_ANNOT_ICONNAME_SOUND_SPEAKER;
		break;
	case AnnotIconNameSoundType::Mic:
		val = FSPDF_ANNOT_ICONNAME_SOUND_MIC;
		break;
	default:
		val = FSPDF_ANNOT_ICONNAME_SOUND_SPEAKER;
		break;
	}

	return val;
}

AnnotIconNameSoundType FoxitTypeConvert::ConvertFromFoxitAnnotIconNameSoundTypeString(Platform::String^ iconname)
{
	AnnotIconNameSoundType icontype = AnnotIconNameSoundType::Speaker;

	if (iconname == FSPDF_ANNOT_ICONNAME_SOUND_SPEAKER)
	{
		icontype = AnnotIconNameSoundType::Speaker;
	}
	else if (iconname == FSPDF_ANNOT_ICONNAME_SOUND_MIC)
	{
		icontype = AnnotIconNameSoundType::Mic;
	}

	return icontype;
}

Platform::String^  FoxitTypeConvert::ConvertFromFoxitAnnotType(foxit::pdf::annots::AnnotType annotType)
{
	Platform::String^ type = nullptr;
	switch (annotType)
	{
	case foxit::pdf::annots::AnnotType::e_UnknownType:
		type = "unknown";
		break;
	case foxit::pdf::annots::AnnotType::e_Note:
		type = "Text";
		break;
	case foxit::pdf::annots::AnnotType::e_Link:
		type = "Link";
		break;
	case foxit::pdf::annots::AnnotType::e_FreeText:
		type = "FreeText";
		break;
	case foxit::pdf::annots::AnnotType::e_Line:
		type = "Line";
		break;
	case foxit::pdf::annots::AnnotType::e_Square:
		type = "Square";
		break;
	case foxit::pdf::annots::AnnotType::e_Circle:
		type = "Circle";
		break;
	case foxit::pdf::annots::AnnotType::e_Polygon:
		type = "Polygon";
		break;
	case foxit::pdf::annots::AnnotType::e_PolyLine:
		type = "PolyLine";
		break;
	case foxit::pdf::annots::AnnotType::e_Highlight:
		type = "HighLight";
		break;
	case foxit::pdf::annots::AnnotType::e_Underline:
		type = "UnderLine";
		break;
	case foxit::pdf::annots::AnnotType::e_Squiggly:
		type = "Squiggly";
		break;
	case foxit::pdf::annots::AnnotType::e_StrikeOut:
		type = "StrikeOut";
		break;
	case foxit::pdf::annots::AnnotType::e_Stamp:
		type = "Stamp";
		break;
	case foxit::pdf::annots::AnnotType::e_Caret:
		type = "Caret";
		break;
	case foxit::pdf::annots::AnnotType::e_Ink:
		type = "Ink";
		break;
	case foxit::pdf::annots::AnnotType::e_PSInk:
		type = "PSInk";
		break;
	case foxit::pdf::annots::AnnotType::e_FileAttachment:
		type = "FileAttachment";
		break;
	case foxit::pdf::annots::AnnotType::e_Sound:
		type = "Sound";
		break;
	case foxit::pdf::annots::AnnotType::e_Movie:
		type = "Movie";
		break;
	case foxit::pdf::annots::AnnotType::e_Widget:
		type = "Widget";
		break;
	case foxit::pdf::annots::AnnotType::e_Screen:
		type = "Screen";
		break;
	case foxit::pdf::annots::AnnotType::e_PrinterMark:
		type = "PrinterMark";
		break;
	case foxit::pdf::annots::AnnotType::e_TrapNet:
		type = "TrapNet";
		break;
	case foxit::pdf::annots::AnnotType::e_Watermark:
		type = "Watermark";
		break;
	case foxit::pdf::annots::AnnotType::e_3D:
		type = "ThreeD";
		break;
	case foxit::pdf::annots::AnnotType::e_Popup:
		type = "Popup";
		break;
	default:
		break;
	}
	return type;
}

foxit::pdf::annots::AnnotType FoxitTypeConvert::ConvertFromAnnotType(AnnotationType type)
{
	foxit::pdf::annots::AnnotType fsType = foxit::pdf::annots::AnnotType::e_UnknownType;
	switch (type)
	{
	case foxit::rdk::uiextensions::AnnotationType::None:
		break;
	case foxit::rdk::uiextensions::AnnotationType::NonSurport:
		break;
	case foxit::rdk::uiextensions::AnnotationType::HightLight:
		fsType = foxit::pdf::annots::AnnotType::e_Highlight;
		break;
	case foxit::rdk::uiextensions::AnnotationType::Link:
		fsType = foxit::pdf::annots::AnnotType::e_Link;
		break;
	case foxit::rdk::uiextensions::AnnotationType::UnderLine:
		fsType = foxit::pdf::annots::AnnotType::e_Underline;
		break;
	case foxit::rdk::uiextensions::AnnotationType::StrikeOut:
		fsType = foxit::pdf::annots::AnnotType::e_StrikeOut;
		break;
	case foxit::rdk::uiextensions::AnnotationType::Squiggly:
		fsType = foxit::pdf::annots::AnnotType::e_Squiggly;
		break;
	case foxit::rdk::uiextensions::AnnotationType::Text:
		fsType = foxit::pdf::annots::AnnotType::e_Note;
		break;
	case foxit::rdk::uiextensions::AnnotationType::FreeText:
		fsType = foxit::pdf::annots::AnnotType::e_FreeText;
		break;
	case foxit::rdk::uiextensions::AnnotationType::Line:
		fsType = foxit::pdf::annots::AnnotType::e_Line;
		break;
	case foxit::rdk::uiextensions::AnnotationType::Circle:
		fsType = foxit::pdf::annots::AnnotType::e_Circle;
		break;
	case foxit::rdk::uiextensions::AnnotationType::Square:
		fsType = foxit::pdf::annots::AnnotType::e_Square;
		break;
	case foxit::rdk::uiextensions::AnnotationType::Ink:
		fsType = foxit::pdf::annots::AnnotType::e_Ink;
		break;
	case foxit::rdk::uiextensions::AnnotationType::PolyLine:
		fsType = foxit::pdf::annots::AnnotType::e_PolyLine;
		break;
	case foxit::rdk::uiextensions::AnnotationType::Polygon:
		fsType = foxit::pdf::annots::AnnotType::e_Polygon;
		break;
	case foxit::rdk::uiextensions::AnnotationType::Stamp:
		fsType = foxit::pdf::annots::AnnotType::e_Stamp;
		break;
	case foxit::rdk::uiextensions::AnnotationType::Caret:
		fsType = foxit::pdf::annots::AnnotType::e_Caret;
		break;
	case foxit::rdk::uiextensions::AnnotationType::Popup:
		fsType = foxit::pdf::annots::AnnotType::e_Popup;
		break;
	case foxit::rdk::uiextensions::AnnotationType::FileAttachment:
		fsType = foxit::pdf::annots::AnnotType::e_FileAttachment;
		break;
	case foxit::rdk::uiextensions::AnnotationType::Sound:
		fsType = foxit::pdf::annots::AnnotType::e_Sound;
		break;
	case foxit::rdk::uiextensions::AnnotationType::Movie:
		fsType = foxit::pdf::annots::AnnotType::e_Movie;
		break;
	case foxit::rdk::uiextensions::AnnotationType::Widget:
		fsType = foxit::pdf::annots::AnnotType::e_Widget;
		break;
	case foxit::rdk::uiextensions::AnnotationType::Screen:
		fsType = foxit::pdf::annots::AnnotType::e_Screen;
		break;
	case foxit::rdk::uiextensions::AnnotationType::PrinterMark:
		fsType = foxit::pdf::annots::AnnotType::e_PrinterMark;
		break;
	case foxit::rdk::uiextensions::AnnotationType::TrapNet:
		fsType = foxit::pdf::annots::AnnotType::e_TrapNet;
		break;
	case foxit::rdk::uiextensions::AnnotationType::Watermark:
		fsType = foxit::pdf::annots::AnnotType::e_Watermark;
		break;
	case foxit::rdk::uiextensions::AnnotationType::PSInk:
		fsType = foxit::pdf::annots::AnnotType::e_PSInk;
		break;
	case foxit::rdk::uiextensions::AnnotationType::ThreeD:
		fsType = foxit::pdf::annots::AnnotType::e_3D;
		break;
	case foxit::rdk::uiextensions::AnnotationType::ReplaceText:
		break;
	default:
		break;
	}
	return fsType;
}

int FSPDFDocUtil::GetDocumentSize(Windows::Storage::StorageFile^ file)
{
	HANDLE processingHandle = CreateEventEx(
		NULL,
		NULL,
		false,
		EVENT_ALL_ACCESS
	);
	int fileSize = 0;
	auto op = file->GetBasicPropertiesAsync();
	op->Completed = ref new Windows::Foundation::AsyncOperationCompletedHandler<Windows::Storage::FileProperties::BasicProperties^>([=, &fileSize](Windows::Foundation::IAsyncOperation<Windows::Storage::FileProperties::BasicProperties^>^ operation, Windows::Foundation::AsyncStatus status)
	{
		if (status == Windows::Foundation::AsyncStatus::Completed)
		{
			auto result = operation->GetResults();
			if (result)
				fileSize = (int)result->Size;
		}
		SetEvent(processingHandle);
	});

	WaitForSingleObjectEx(processingHandle, INFINITE, true);
	CloseHandle(processingHandle);
	return fileSize;
}

foxit::rdk::uiextensions::DocumentEncryptType FSPDFDocUtil::GetDocumentEncryptType(foxit::pdf::PDFDoc^ doc)
{
	DocumentEncryptType type;
	FS_INT32 encryptionType = 0;

	foxit::pdf::EncryptType encType = doc->GetEncryptionType();
	switch (encType)
	{
	case foxit::pdf::EncryptType::e_EncryptUnknown:
		type = DocumentEncryptType::CUSTOM;
		break;
	case foxit::pdf::EncryptType::e_EncryptNone:
		type = DocumentEncryptType::NOENCRYPT;
		break;
	case foxit::pdf::EncryptType::e_EncryptPassword:
		type = DocumentEncryptType::PASSWORD;
		break;
	case foxit::pdf::EncryptType::e_EncryptCertificate:
		type = DocumentEncryptType::CERTIFICATE;
		break;
	case foxit::pdf::EncryptType::e_EncryptFoxitDRM:
		type = DocumentEncryptType::FOXITDRM;
		break;
	case foxit::pdf::EncryptType::e_EncryptCustom:
		type = DocumentEncryptType::CUSTOM;
		break;
	case foxit::pdf::EncryptType::e_EncryptRMS:
		type = DocumentEncryptType::RMS;
		break;
	default:
		break;
	}

	return type;
}
bool FSPDFDocUtil::GetIsDocumentSecurity(foxit::pdf::PDFDoc^ doc)
{
	bool bIsEncrypt = false;
	FS_INT32 encryptionType = 0;

	return GetDocumentEncryptType(doc) != DocumentEncryptType::NOENCRYPT;
}
bool FSPDFDocUtil::GetIsDocumentCanExtra(foxit::pdf::PDFDoc^ doc)
{
	unsigned int m_permission = doc->GetUserPermissions();
	return (m_permission & (FS_DWORD)foxit::pdf::UserPermissions::e_PermExtractAccess) ? true : false;
}
bool FSPDFDocUtil::GetIsDocumentCanAssemble(foxit::pdf::PDFDoc^ doc)
{
	unsigned int m_permission = doc->GetUserPermissions();
	return (m_permission & (FS_DWORD)foxit::pdf::UserPermissions::e_PermAssemble) ? true : false;
}
bool FSPDFDocUtil::GetIsDocumentCanBeSigning(foxit::pdf::PDFDoc^ doc)
{
	unsigned int m_permission = doc->GetUserPermissions();
	return (m_permission & (FS_DWORD)foxit::pdf::UserPermissions::e_PermAnnotForm || m_permission & (FS_DWORD)foxit::pdf::UserPermissions::e_PermFillForm) ? true : false;
}
bool FSPDFDocUtil::GetIsDocumentCanModify(foxit::pdf::PDFDoc^ doc)
{
	unsigned int m_permission = doc->GetUserPermissions();
	return (m_permission & (FS_DWORD)foxit::pdf::UserPermissions::e_PermModify) ? true : false;
}
bool FSPDFDocUtil::GetIsDocumentCanBePrint(foxit::pdf::PDFDoc^ doc)
{
	unsigned int m_permission = doc->GetUserPermissions();
	return (m_permission & (FS_DWORD)foxit::pdf::UserPermissions::e_PermPrint) ? true : false;
}
bool FSPDFDocUtil::GetIsDocumentCanBeCopy(foxit::pdf::PDFDoc^ doc)
{
	unsigned int m_permission = doc->GetUserPermissions();
	return (m_permission &  (FS_DWORD)foxit::pdf::UserPermissions::e_PermExtract) ? true : false;
}
bool FSPDFDocUtil::GetIsDocumentCanModifyAnnotation(foxit::pdf::PDFDoc^ doc)
{
	unsigned int m_permission = doc->GetUserPermissions();
	return (/*m_permission & FSPDF_PERMISSION_MODIFY ||*/ m_permission & (FS_DWORD)foxit::pdf::UserPermissions::e_PermAnnotForm) ? true : false;
}
bool FSPDFDocUtil::GetIsDocumentCanFillInForm(foxit::pdf::PDFDoc^ doc)
{
	unsigned int m_permission = doc->GetUserPermissions();
	return (m_permission & (FS_DWORD)foxit::pdf::UserPermissions::e_PermModify || m_permission & (FS_DWORD)foxit::pdf::UserPermissions::e_PermFillForm) ? true : false;
}
bool FSPDFDocUtil::GetIsDocumentCanAddComment(foxit::pdf::PDFDoc^ doc)
{
	unsigned int m_permission = doc->GetUserPermissions();
	return (m_permission & (FS_DWORD)foxit::pdf::UserPermissions::e_PermAnnotForm) ? true : false;
}

FS_FLOAT FSPDFDocUtil::PixelToPDFPoint(foxit::pdf::PDFDoc^ doc, int pageIndex, int pixel)
{
	auto page = doc->GetPage(pageIndex);
	if (page == nullptr)
		return -1;
	float fPageWidth = page->GetWidth();
	float fPageHeight = page->GetHeight();

	Windows::Foundation::Rect rcDevice(0, 0, (float)pixel, (float)pixel);
	foxit::common::fxcrt::RectF^ srcRc = FoxitRectConverter::ConvertToFoxitRectF(rcDevice);

	foxit::common::fxcrt::Matrix2D^ matrix = page->GetDisplayMatrix(0, 0, (FS_INT32)fPageWidth, (FS_INT32)fPageHeight, (foxit::common::Rotation)0);
	foxit::common::fxcrt::Matrix2D^ Dstmatrix = ref new foxit::common::fxcrt::Matrix2D();
	Dstmatrix->SetReverse(matrix);
	Dstmatrix->TransformRect(srcRc);
	Windows::Foundation::Rect rcPDFPoint = FoxitRectConverter::ConvertFromFoxitRectF(srcRc);
	FS_FLOAT ptPDF = rcPDFPoint.Width;

	return ptPDF;
}

double FSPDFDocUtil::PDFPointToPixel(foxit::pdf::PDFDoc^ doc, int pageIndex, float PdfPoint, double dbPageRenderWidth, double dbPageRenderHeight)
{
	auto page = doc->GetPage(pageIndex);
	if (page == nullptr)
		return -1;
	float fPageWidth = page->GetWidth();
	float fPageHeight = page->GetHeight();

	Windows::Foundation::Rect rcPage(0, 0, PdfPoint, PdfPoint);
	foxit::common::fxcrt::RectF^ srcRc = FoxitRectConverter::ConvertToFoxitRectF(rcPage);

	foxit::common::fxcrt::Matrix2D^ matrix = page->GetDisplayMatrix(0, 0, (FS_INT32)fPageWidth, (FS_INT32)fPageHeight, foxit::common::Rotation::e_Rotation0);
	matrix->TransformRect(srcRc);
	Windows::Foundation::Rect rcPixel = FoxitRectConverter::ConvertFromFoxitRectF(srcRc);
	return rcPixel.Width;
}

Windows::UI::Xaml::Media::Imaging::WriteableBitmap^ FSPDFDocUtil::GetThumbImageSync(foxit::pdf::PDFDoc^ doc, int pageIndex, int width, int height, int nRotate)
{
	Windows::UI::Xaml::Media::Imaging::WriteableBitmap^ bitmap = nullptr;

	auto page = doc->GetPage(pageIndex);
	if (page == nullptr)
		return bitmap;

	Windows::Foundation::Size PageSize((float)width, (float)height);
	concurrency::cancellation_token_source pCts;

	FS_INT32 _bitmapwidth = (FS_INT32)(width * 1.0);
	FS_INT32 _bitmapheight = (FS_INT32)(height * 1.0);

	foxit::common::Bitmap^ m_bitmap = ref new foxit::common::Bitmap(_bitmapwidth, _bitmapheight, foxit::common::DIBFormat::e_DIBRgb32, nullptr, 0);
	if (m_bitmap == nullptr)
	{
		OutputDebugString(L"FSCRT_Bitmap_Create error\n");
		return bitmap;
	}

	//Set rect area and fill the color of bitmap.
	foxit::common::fxcrt::RectI^ rect = ref new foxit::common::fxcrt::RectI(0, 0, _bitmapwidth, _bitmapheight);
	m_bitmap->FillRect(0xffffffff, rect);

	//Get the page's matrix.
	foxit::common::fxcrt::Matrix2D^ mt = page->GetDisplayMatrix(0, 0, width, height, (foxit::common::Rotation)nRotate);
	mt->Scale(1.0, 1.0, true);

	FS_FLOAT pageWidth = page->GetWidth(), pageHeight = page->GetHeight();
	FS_INT32 index = page->GetIndex();

	//Create a renderer based on a given bitmap, and page will be rendered to this bitmap.
	foxit::common::Renderer^ renderer = ref new foxit::common::Renderer(m_bitmap, false);
	renderer->SetRenderContentFlags((unsigned int)(foxit::common::ContentFlag::e_RenderAnnot | foxit::common::ContentFlag::e_RenderPage));
	BOOL bIsParsed = page->IsParsed();
	if (!bIsParsed)
	{
		page->StartParse(0, nullptr, false);
	}
	renderer->StartRender(page, mt, nullptr);

	auto byteArray = m_bitmap->GetBuffer();
	auto dataWriter =ref new DataWriter();
    dataWriter->WriteBytes(byteArray);
	
	Platform::Object^ _obj = dataWriter->DetachBuffer();
	ComPtr<IInspectable> _insp(reinterpret_cast<IInspectable*>(_obj));
	ComPtr<IBufferByteAccess> _bufferByteAccess;
	_insp.As(&_bufferByteAccess);
	byte* pixels = nullptr;
	_bufferByteAccess->Buffer(&pixels);
	void* lpBmpBuf = pixels;

	width = m_bitmap->GetWidth();
	height = m_bitmap->GetHeight();
	unsigned int stride = m_bitmap->GetPitch();
	unsigned int size = stride * height;
	bitmap = ref new Windows::UI::Xaml::Media::Imaging::WriteableBitmap(width, height);
	/*_obj = buffer;
	ComPtr<IInspectable> _insp2(reinterpret_cast<IInspectable*>(_obj));
	ComPtr<Windows::Storage::Streams::IBufferByteAccess> _bufferByteAccess2;
	_insp2.As(&_bufferByteAccess2);
	_bufferByteAccess2->Buffer(&pixels);*/
	pixels = Utility::GetPointerToPixelData(bitmap->PixelBuffer);
	memcpy(pixels, lpBmpBuf, size);

	return bitmap;
}

Windows::Foundation::Rect foxit::rdk::uiextensions::FSPDFDocUtil::DeviceToPageRect(foxit::pdf::PDFDoc ^ doc, int index, double dbStartX, double dbStartY, double dbSizeX, double dbSizeY, Windows::Foundation::Rect rcDevice, int nRotate)
{
	foxit::common::fxcrt::RectF^ srcRc = FoxitRectConverter::ConvertToFoxitRectF(rcDevice);
	auto page = doc->GetPage(index);
	if (page == nullptr)
		return Windows::UI::Xaml::RectHelper::Empty;

	foxit::common::fxcrt::Matrix2D^ matrix = page->GetDisplayMatrix((FS_INT32)dbStartX, (FS_INT32)dbStartY, (FS_INT32)dbSizeX, (FS_INT32)dbSizeY, (foxit::common::Rotation)nRotate);
	foxit::common::fxcrt::Matrix2D^ Dstmatrix = ref new foxit::common::fxcrt::Matrix2D();
	Dstmatrix->SetReverse(matrix);
	Dstmatrix->TransformRect(srcRc);

	return FoxitRectConverter::ConvertFromFoxitRectF(srcRc);
}

Windows::Foundation::Rect foxit::rdk::uiextensions::FSPDFDocUtil::PageToDeviceRect(foxit::pdf::PDFDoc ^ doc, int index, double dbSizeX, double dbSizeY, Windows::Foundation::Rect rcPage, int nRotate)
{
	return foxit::rdk::uiextensions::FSPDFDocUtil::PageToDeviceRect(doc, index, 0, 0, dbSizeX, dbSizeY, rcPage, nRotate);
}

Windows::Foundation::Rect foxit::rdk::uiextensions::FSPDFDocUtil::PageToDeviceRect(foxit::pdf::PDFDoc ^ doc, int index, double dbStartX, double dbStartY, double dbSizeX, double dbSizeY, Windows::Foundation::Rect rcPage, int nRotate)
{
	foxit::common::fxcrt::RectF^ srcRc = FoxitRectConverter::ConvertToFoxitRectF(rcPage);
	auto page = doc->GetPage(index);
	if (page == nullptr)
		return Windows::UI::Xaml::RectHelper::Empty;

	foxit::common::fxcrt::Matrix2D^ matrix = page->GetDisplayMatrix((FS_INT32)dbStartX, (FS_INT32)dbStartY, (FS_INT32)dbSizeX, (FS_INT32)dbSizeY, (foxit::common::Rotation)nRotate);
	matrix->TransformRect(srcRc);

	return FoxitRectConverter::ConvertFromFoxitRectF(srcRc);
}

Windows::Foundation::Point foxit::rdk::uiextensions::FSPDFDocUtil::DeviceToPagePoint(foxit::pdf::PDFDoc ^ doc, int index, double dbStartX, double dbStartY, double dbSizeX, double dbSizeY, Windows::Foundation::Point pt, int nRotate)
{
	auto page = doc->GetPage(index);
	if (page == nullptr)
		return pt;

	foxit::common::fxcrt::Matrix2D^ matrix = page->GetDisplayMatrix((FS_INT32)dbStartX, (FS_INT32)dbStartY, (FS_INT32)dbSizeX, (FS_INT32)dbSizeY, (foxit::common::Rotation)nRotate);
	foxit::common::fxcrt::Matrix2D^ Dstmatrix = ref new foxit::common::fxcrt::Matrix2D();
	Dstmatrix->SetReverse(matrix);

	foxit::common::fxcrt::PointF^ pPt = ref new foxit::common::fxcrt::PointF(pt.X, pt.Y);
	Dstmatrix->TransformPoint(pPt);

	return Windows::Foundation::Point(pPt->x, pPt->y);
}

Windows::Foundation::Point foxit::rdk::uiextensions::FSPDFDocUtil::PageToDevicePoint(foxit::pdf::PDFDoc ^ doc, int index, double dbStartX, double dbStartY, double dbSizeX, double dbSizeY, Windows::Foundation::Point pt, int nRotate)
{
	auto page = doc->GetPage(index);
	if (page == nullptr)
		return pt;

	foxit::common::fxcrt::Matrix2D^ matrix = page->GetDisplayMatrix((FS_INT32)dbStartX, (FS_INT32)dbStartY, (FS_INT32)dbSizeX, (FS_INT32)dbSizeY, (foxit::common::Rotation)nRotate);
	foxit::common::fxcrt::PointF^ pPt = ref new foxit::common::fxcrt::PointF(pt.X, pt.Y);
	matrix->TransformPoint(pPt);

	return Windows::Foundation::Point(pPt->x, pPt->y);
}

void foxit::rdk::uiextensions::FSPDFDocUtil::PageParse(foxit::pdf::PDFPage^ page, foxit::pdf::ParseFlags flag/* = foxit::pdf::ParseFlags::e_parsePageNormal*/)
{
	if (!page->IsParsed())
	{
		foxit::common::Progressive^ progress = page->StartParse((unsigned int)flag, nullptr, false);
		if (progress != nullptr)
		{
			foxit::common::State state = progress->Continue();
			while (state == foxit::common::State::e_ToBeContinued)
				state = progress->Continue();
		}
	}
}

bool foxit::rdk::uiextensions::FSPDFDocUtil::IsValidBookmark(foxit::pdf::Bookmark^ pos)
{
	return pos && !pos->IsEmpty();
}

