/**
 * Copyright (C) 2003-2018, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit Mobile PDF SDK to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */


#pragma once

#include <time.h>

namespace foxit
{
	namespace rdk
	{
		namespace uiextensions
		{
			template<typename T>
			T GetVisualChild(Windows::UI::Xaml::DependencyObject^ parent)
			{
				if (parent == nullptr)
					return nullptr;
				T child = nullptr;
				int numVisuals = Windows::UI::Xaml::Media::VisualTreeHelper::GetChildrenCount(parent);
				for (int i = 0; i < numVisuals; i++)
				{
					Windows::UI::Xaml::DependencyObject ^d = (Windows::UI::Xaml::DependencyObject^)Windows::UI::Xaml::Media::VisualTreeHelper::GetChild(parent, i);
					child = dynamic_cast<T>(d);
					if (!child)
						child = GetVisualChild<T>(d);
					if (child)
						break;
				}
				return child;
			};

			template<typename T>
			T GetVisualParent(Windows::UI::Xaml::DependencyObject^ Child)
			{
#define ISNEEDDBGMSG 0
				T parent = nullptr;
				if (Child == nullptr)
					return nullptr;
				DependencyObject^ dependObj = (DependencyObject^)Windows::UI::Xaml::Media::VisualTreeHelper::GetParent(Child);
				if (!dependObj)
				{
					auto _framework = dynamic_cast<Windows::UI::Xaml::FrameworkElement^>(Child);
					if (_framework && _framework->Parent)
					{
						parent = dynamic_cast<T>(_framework->Parent);
#if ISNEEDDBGMSG
						if (parent)
						{
							OutputDebugString(parent->ToString()->Data());
							OutputDebugString(L"\r\n");
						}
#endif
						return parent;
					}
					return nullptr;
				}
#if ISNEEDDBGMSG
				auto frameelement = safe_cast<Windows::UI::Xaml::FrameworkElement^>(dependObj);
				if (frameelement)
				{
					std::wstringstream ws;
					ws << L"Type: " << frameelement->ToString()->Data();
					ws << L", Name: " << frameelement->Name->Data();
					ws << L", ActualWidth: " << frameelement->ActualWidth;
					ws << L", ActualHeight: " << frameelement->ActualHeight;
					ws << L", IsHitTestVisible: " << frameelement->IsHitTestVisible;
					ws << L", Opacity: " << frameelement->Opacity;
					ws << L", Visibility: " << frameelement->Visibility.ToString()->Data();
					ws << L", Margin: " << frameelement->Margin.Left << L"," << frameelement->Margin.Top << L"," << frameelement->Margin.Right << L"," << frameelement->Margin.Bottom;
					ws << L"\n";
					OutputDebugString(ws.str().c_str());
				}
#endif
				parent = dynamic_cast<T>(dependObj);
				if (parent == nullptr)
				{
					parent = GetVisualParent<T>(dependObj);
					if (parent != nullptr)
						return parent;
					else
						return nullptr;
				}
				return parent;
			}

			template<typename T>
			T GetVisualParentByName(Windows::UI::Xaml::DependencyObject^ Child, Platform::String^ name)
			{
				if (Child == nullptr)
					return nullptr;
				T parent = nullptr;
				DependencyObject ^d = (DependencyObject^)Windows::UI::Xaml::Media::VisualTreeHelper::GetParent(Child);
				if (!d)
					return nullptr;

				parent = dynamic_cast<T>(d);
				if (parent == nullptr)
				{
					parent = GetVisualParent<T>(d);
					if (parent != nullptr && parent->Name == name)
						return parent;
					else
						return nullptr;
				}
				return parent;
			}

			template<typename T>
			T GetVisualChildByName(Windows::UI::Xaml::DependencyObject^ parent, Platform::String^ name)
			{
				T child = nullptr;
				if (parent == nullptr)
					return nullptr;
				int numVisuals = Windows::UI::Xaml::Media::VisualTreeHelper::GetChildrenCount(parent);
				for (int i = 0; i < numVisuals; i++)
				{
					Windows::UI::Xaml::DependencyObject ^d = (Windows::UI::Xaml::DependencyObject^)Windows::UI::Xaml::Media::VisualTreeHelper::GetChild(parent, i);
					child = dynamic_cast<T>(d);
					if (child == nullptr || child->Name != name)
					{
						child = GetVisualChildByName<T>(d, name);
						if (child != nullptr)
							return child;
					}
					else
						return child;
				}
				return nullptr;
			}

			template<typename T>
			T GetParent(Windows::UI::Xaml::DependencyObject^ Child)
			{
				T parent = nullptr;

				auto d = dynamic_cast<Windows::UI::Xaml::FrameworkElement^>(Child);
				if (d && d->Parent)
				{
					parent = dynamic_cast<T>(d);
					if (parent == nullptr)
					{
						parent = GetParent<T>(d->Parent);
						if (parent != nullptr)
							return parent;
						else
							return nullptr;
					}
				}

				return parent;
			}

			static Platform::String^ GetTabString(int count)
			{
				if (count == 0)
					return ref new Platform::String();
				Platform::String^ tmp;
				for (int i = 0; i < count; i++)
				{
					tmp = tmp + "\t";
				}
				return tmp;
			}

			static void PrintVisualTreeChildRecursion(Windows::UI::Xaml::DependencyObject ^parent, int depth)
			{
				auto frameelement = safe_cast<Windows::UI::Xaml::FrameworkElement^>(parent);
				if (frameelement)
				{
#ifdef _DEBUG
					std::wstringstream ws;
					ws << GetTabString(depth)->Data() << L"Type: " << frameelement->ToString()->Data();
					ws << L", Name: " << frameelement->Name->Data();
					ws << L", ActualWidth: " << frameelement->ActualWidth;
					ws << L", ActualHeight: " << frameelement->ActualHeight;
					ws << L", IsHitTestVisible: " << frameelement->IsHitTestVisible;
					ws << L", Opacity: " << frameelement->Opacity;
					ws << L", Visibility: " << frameelement->Visibility.ToString()->Data();
					ws << L", Margin: " << frameelement->Margin.Left << L"," << frameelement->Margin.Top << L"," << frameelement->Margin.Right << L"," << frameelement->Margin.Bottom;
					ws << L"\n";
					OutputDebugString(ws.str().c_str());
#endif
				}
				else
				{
					return;
				}
				int numVisuals = Windows::UI::Xaml::Media::VisualTreeHelper::GetChildrenCount(parent);
				for (int i = 0; i < numVisuals; i++) {
					Windows::UI::Xaml::DependencyObject ^d = (Windows::UI::Xaml::DependencyObject^) Windows::UI::Xaml::Media::VisualTreeHelper::GetChild(parent, i);

					if (d)
						PrintVisualTreeChildRecursion(d, depth + 1);
				}
				return;
			};

			static void PrintVisualTree(Windows::UI::Xaml::DependencyObject ^parent)
			{
				PrintVisualTreeChildRecursion(parent, 0);
			}

			inline double random(double start, double end)
			{
				static bool issrand = false;
				if (!issrand)
				{
					std::srand((unsigned int)time(NULL));
					issrand = true;
				}
				return start + (end - start)*rand() / (RAND_MAX + 1.0);
			}

			template <class Function>
			__int64 time_call(Function&& f)
			{
				__int64 begin = GetTickCount64();
				f();
				return GetTickCount64() - begin;
			}
		}
	}
}
