﻿/**
 * Copyright (C) 2003-2018, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit Mobile PDF SDK to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */


#include "pch.h"
#include "AnnotPropertyPhone.xaml.h"
#include "NoteStyleUserControl.xaml.h"
#include "AnnotStampUserControl.xaml.h"
#include "../Annotations/FreeTextProperty.xaml.h"
#include "../Annotations/FreeTextEventPeer.h"

using namespace foxit::rdk::uiextensions;
using namespace foxit::rdk;

using namespace Platform;
using namespace Windows::Foundation;
using namespace Windows::Foundation::Collections;
using namespace Windows::UI::Xaml;
using namespace Windows::UI::Xaml::Controls;
using namespace Windows::UI::Xaml::Controls::Primitives;
using namespace Windows::UI::Xaml::Data;
using namespace Windows::UI::Xaml::Input;
using namespace Windows::UI::Xaml::Media;
using namespace Windows::UI::Xaml::Navigation;

// The User Control item template is documented at http://go.microsoft.com/fwlink/?LinkId=234236

AnnotPropertyPhone::AnnotPropertyPhone(foxit::rdk::PDFViewCtrl^ view)
{
	InitializeComponent();
	m_PageViewWeakRef = view;

	annotStampControl->RelativePageView(view);
	noteStylePanel->RelateToView(view);


	this->Loaded += ref new Windows::UI::Xaml::RoutedEventHandler(this, &foxit::rdk::uiextensions::AnnotPropertyPhone::OnLoaded);
	this->Unloaded += ref new Windows::UI::Xaml::RoutedEventHandler(this, &foxit::rdk::uiextensions::AnnotPropertyPhone::OnUnloaded);
	this->SizeChanged += ref new Windows::UI::Xaml::SizeChangedEventHandler(this, &foxit::rdk::uiextensions::AnnotPropertyPhone::OnSizeChanged);
}

void foxit::rdk::uiextensions::AnnotPropertyPhone::OnLoaded(Platform::Object ^sender, Windows::UI::Xaml::RoutedEventArgs ^e)
{
	__windowSizeChangedEventRegistrationToken = Window::Current->SizeChanged += ref new Windows::UI::Xaml::WindowSizeChangedEventHandler(this, &foxit::rdk::uiextensions::AnnotPropertyPhone::OnWindowsSizeChanged);
	sliderLiness->ValueChanged += ref new Windows::UI::Xaml::Controls::Primitives::RangeBaseValueChangedEventHandler(this, &foxit::rdk::uiextensions::AnnotPropertyPhone::OnValueChanged);
	sliderLinessPor->ValueChanged += ref new Windows::UI::Xaml::Controls::Primitives::RangeBaseValueChangedEventHandler(this, &foxit::rdk::uiextensions::AnnotPropertyPhone::OnValuePorChanged);
	stampIconPopup = ref new Windows::UI::Xaml::Controls::Primitives::Popup();

	Windows::Foundation::Rect winRect = Windows::UI::Xaml::Window::Current->Bounds;
	mainGrid->Width = winRect.Width;

	if (winRect.Width < winRect.Height)
	{
		pivot->Visibility = Windows::UI::Xaml::Visibility::Visible;
		pivotPor->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
	}
	else
	{
		pivot->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
		pivotPor->Visibility = Windows::UI::Xaml::Visibility::Visible;
	}

	InitPropertyPanelVisible();

	freeTextProperty->RelateToView(View, this->ActualWidth, this->ActualHeight);
	freeTextPropertyPor->RelateToView(View, this->ActualWidth, this->ActualHeight);


	gridColorList1->Width = winRect.Width - 30;
	gridColorList2->Width = winRect.Width - 30;
	gridOpacityList1->Width = winRect.Width - 30;

	gridColorList1Por->Width = winRect.Width - 30;
	gridOpacityList1Por->Width = winRect.Width - 30;
	sliderLinessPor->Width = winRect.Width*0.6;
}


void foxit::rdk::uiextensions::AnnotPropertyPhone::OnUnloaded(Platform::Object ^sender, Windows::UI::Xaml::RoutedEventArgs ^e)
{
	UIExtensionsManager^ uiMgr = (UIExtensionsManager^)View->ExtensionsManager;
	if (!GetIsDesktopPlatform())
		uiMgr->SetMainAppBarVisible(true);
	if (__windowSizeChangedEventRegistrationToken.Value)
	{
		Window::Current->SizeChanged -= __windowSizeChangedEventRegistrationToken;
		__windowSizeChangedEventRegistrationToken.Value = 0;
	}
}

void foxit::rdk::uiextensions::AnnotPropertyPhone::OnSizeChanged(Platform::Object ^sender, Windows::UI::Xaml::SizeChangedEventArgs ^e)
{

}


void AnnotPropertyPhone::InitPropertyPanelVisible()
{
	UIExtensionsManager^ uiMgr = (UIExtensionsManager^)View->ExtensionsManager;
	IBaseAnnot^ annot = uiMgr->getAnnotMgr()->GetSelectedAnnot();
	if (!annot)
	{
		colorPanel->Visibility = Windows::UI::Xaml::Visibility::Visible;
		opacityPanel->Visibility = Windows::UI::Xaml::Visibility::Visible;
		LinessPanel->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
		LinessPanelPor->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
		noteStylePanel->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
		annotStampControl->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
		freeTextProperty->Visibility = Windows::UI::Xaml::Visibility::Collapsed;

		AnnotCreateType createtype = uiMgr->CreateTypeOfAnnot;
		if (createtype == AnnotCreateType::Circle || createtype == AnnotCreateType::Square
			|| createtype == AnnotCreateType::Line || createtype == AnnotCreateType::Pencil
			|| createtype == AnnotCreateType::Arrow)
		{
			LinessPanel->Visibility = Windows::UI::Xaml::Visibility::Visible;
			LinessPanelPor->Visibility = Windows::UI::Xaml::Visibility::Visible;
		}
		else if (createtype == AnnotCreateType::Text)
		{
			noteStylePanel->Visibility = Windows::UI::Xaml::Visibility::Visible;
			pivot->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
			pivotPor->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
		}
		else if (createtype == AnnotCreateType::Stamp)
		{
			noteStylePanel->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
			pivot->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
			pivotPor->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
			annotStampControl->Visibility = Windows::UI::Xaml::Visibility::Visible;
		}
		else if (createtype == AnnotCreateType::FreeText)
		{
			freeTextProperty->Visibility = Windows::UI::Xaml::Visibility::Visible;
			freeTextPropertyPor->Visibility = Windows::UI::Xaml::Visibility::Visible;
		}
	}
	else
	{
		colorPanel->Visibility = Windows::UI::Xaml::Visibility::Visible;
		opacityPanel->Visibility = Windows::UI::Xaml::Visibility::Visible;
		LinessPanel->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
		LinessPanelPor->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
		noteStylePanel->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
		freeTextProperty->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
		AnnotationType type = annot->AnnotType;

		if (type == AnnotationType::Circle || type == AnnotationType::Square
			|| type == AnnotationType::Line || type == AnnotationType::Ink)
		{
			LinessPanel->Visibility = Windows::UI::Xaml::Visibility::Visible;
			LinessPanelPor->Visibility = Windows::UI::Xaml::Visibility::Visible;
		}
		else if (type == AnnotationType::Text)
		{
			noteStylePanel->Visibility = Windows::UI::Xaml::Visibility::Visible;
			pivot->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
			pivotPor->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
		}
		else if (type == AnnotationType::FreeText)
		{
			freeTextProperty->Visibility = Windows::UI::Xaml::Visibility::Visible;
			freeTextPropertyPor->Visibility = Windows::UI::Xaml::Visibility::Visible;
		}
	}
	InitAppBarProperty();
}


foxit::rdk::PDFViewCtrl^ AnnotPropertyPhone::View::get()
{
	return m_PageViewWeakRef.Resolve<foxit::rdk::PDFViewCtrl>();
}

void AnnotPropertyPhone::InitAppBarProperty()
{
	Windows::UI::Color colorstr;
	double opacity = 0;
	AnnotationType type;
	double borderWidth = 0;
	UIExtensionsManager^ uiMgr = (UIExtensionsManager^)View->ExtensionsManager;
	if (uiMgr->getAnnotMgr()->GetSelectedAnnot())
	{
		type = uiMgr->getAnnotMgr()->GetSelectedAnnot()->AnnotType;
		InitAnnotColorList(type);
		if (uiMgr->getAnnotMgr()->GetSelectedAnnot()->AnnotType == AnnotationType::FreeText)
		{
			FreeTextAnnot^ freetextannot = dynamic_cast<FreeTextAnnot^>(uiMgr->getAnnotMgr()->GetSelectedAnnot());
			if (freetextannot)
				colorstr = freetextannot->TextColor;
		}
		else
		{
			colorstr = uiMgr->getAnnotMgr()->GetSelectedAnnot()->Color;
		}

		opacity = uiMgr->getAnnotMgr()->GetSelectedAnnot()->Opacity;

		borderWidth = uiMgr->getAnnotMgr()->GetSelectedAnnot()->BorderWidth;
	}
	else
	{
		type = GetAnnotType(uiMgr->CreateTypeOfAnnot);
		InitAnnotColorList(type);

		auto _AnnotSetting = uiMgr->getAnnotMgr()->GetAnnotSetting();
		if (uiMgr->CreateTypeOfAnnot == AnnotCreateType::HightLight
			|| uiMgr->CreateTypeOfAnnot == AnnotCreateType::AreaHighLight)
		{
			colorstr = _AnnotSetting->GetHightLightAnnotColor();
			opacity = _AnnotSetting->GetHightLightAnnotOpacity();
		}
		else if (uiMgr->CreateTypeOfAnnot == AnnotCreateType::UnderLine)
		{
			colorstr = _AnnotSetting->GetUnderLineAnnotColor();
			opacity = _AnnotSetting->GetUnderLineAnnotOpacity();
		}
		else if (uiMgr->CreateTypeOfAnnot == AnnotCreateType::StrikeOut)
		{
			colorstr = _AnnotSetting->GetStrikeOutAnnotColor();
			opacity = _AnnotSetting->GetStrikeOutAnnotOpacity();
		}
		else if (uiMgr->CreateTypeOfAnnot == AnnotCreateType::Squiggly)
		{
			colorstr = _AnnotSetting->GetSquigglyAnnotColor();
			opacity = _AnnotSetting->GetSquigglyAnnotOpacity();
		}
		else if (uiMgr->CreateTypeOfAnnot == AnnotCreateType::Line || uiMgr->CreateTypeOfAnnot == AnnotCreateType::Arrow)
		{
			colorstr = _AnnotSetting->GetLineAnnotColor();
			opacity = _AnnotSetting->GetAnnotOpacity();
			borderWidth = _AnnotSetting->GetAnnotDrawBorderWidth();
		}
		else if (uiMgr->CreateTypeOfAnnot == AnnotCreateType::FreeText)
		{
			colorstr = _AnnotSetting->GetFreeTextColor();
			opacity = _AnnotSetting->GetFreeTextAnnotOpacity();
		}
		else if (uiMgr->CreateTypeOfAnnot == AnnotCreateType::Caret)
		{
			colorstr = _AnnotSetting->GetInsertTextColor();
			opacity = _AnnotSetting->GetInsertTextAnnotOpacity();
		}
		else if (uiMgr->CreateTypeOfAnnot == AnnotCreateType::ReplaceText)
		{
			colorstr = _AnnotSetting->GetReplaceTextColor();
			opacity = _AnnotSetting->GetReplaceTextAnnotOpacity();
		}
		else
		{
			colorstr = _AnnotSetting->GetAnnotColor();
			opacity = _AnnotSetting->GetAnnotOpacity();
			borderWidth = _AnnotSetting->GetAnnotDrawBorderWidth();
		}
	}

	if (opacity == 0.25)
	{
		OpacityPageBtn1->IsChecked = true;
		OpacityPageBtn1Por->IsChecked = true;
	}
	else if (opacity == 0.5)
	{
		OpacityPageBtn2->IsChecked = true;
		OpacityPageBtn2Por->IsChecked = true;
	}
	else if (opacity == 0.75)
	{
		OpacityPageBtn3->IsChecked = true;
		OpacityPageBtn3Por->IsChecked = true;
	}
	else if (opacity == 1.0)
	{
		OpacityPageBtn4->IsChecked = true;
		OpacityPageBtn4Por->IsChecked = true;
	}

	Platform::String^ colorRGBValue = Utility::ConvertToColorString(colorstr);
	SetRadioColorChecked(colorRGBValue, type);

	sliderLiness->Value = borderWidth;
	tbLiness->Text = borderWidth.ToString() + "pt";

	sliderLinessPor->Value = borderWidth;
	tbLinessPor->Text = borderWidth.ToString() + "pt";
}


void AnnotPropertyPhone::NoteLogoSetting_Tapped(Platform::Object^ sender, Windows::UI::Xaml::Input::TappedRoutedEventArgs^ e)
{
	UIExtensionsManager^ uiMgr = (UIExtensionsManager^)View->ExtensionsManager;
	uiMgr->LoadNoteStylePage();
}


void AnnotPropertyPhone::ColorPageBtnSelect_Tapped(Platform::Object^ sender, Windows::UI::Xaml::Input::TappedRoutedEventArgs^ e)
{
	UIExtensionsManager^ uiMgr = (UIExtensionsManager^)View->ExtensionsManager;
	IBaseAnnot^ annot = uiMgr->getAnnotMgr()->GetSelectedAnnot();
	RadioButton^ button = dynamic_cast<RadioButton^>(sender);
	Platform::String^ content = button->Content->ToString();
	Windows::UI::Color color = Utility::ConvertFromString(button->Content->ToString());
	if (annot)
	{

		annot->IsModifyProperty = true;
		annot->Color = color;

		if (annot->AnnotType == AnnotationType::HightLight)
		{
			uiMgr->getAnnotMgr()->GetAnnotSetting()->SetHightLightAnnotColor(color);
		}
		else if (annot->AnnotType == AnnotationType::UnderLine)
		{
			uiMgr->getAnnotMgr()->GetAnnotSetting()->SetUnderLineAnnotColor(color);
		}
		else if (annot->AnnotType == AnnotationType::StrikeOut)
		{
			uiMgr->getAnnotMgr()->GetAnnotSetting()->SetStrikeOutAnnotColor(color);
			StrikeOutAnnot^ strikeoutAnnot = dynamic_cast<StrikeOutAnnot^>(annot);
			if (strikeoutAnnot->IsHasGroup)
			{
				if (uiMgr->getAnnotMgr()->IsGetGroupAnnotElement(strikeoutAnnot->PageIndex, strikeoutAnnot->GroupAnnotMinRect,
					strikeoutAnnot->GroupAnnotIndex, AnnotationType::Caret))
				{
					IBaseAnnot^ groupAnnot = uiMgr->getAnnotMgr()->GetGroupAnnot();
					groupAnnot->Color = color;
				}
				uiMgr->getAnnotMgr()->GetAnnotSetting()->SetReplaceTextColor(color);
			}
		}
		else if (annot->AnnotType == AnnotationType::Squiggly)
		{
			uiMgr->getAnnotMgr()->GetAnnotSetting()->SetSquigglyAnnotColor(color);
		}
		else if (annot->AnnotType == AnnotationType::Line)
		{
			uiMgr->getAnnotMgr()->GetAnnotSetting()->SetLineAnnotColor(color);
		}
		else if (annot->AnnotType == AnnotationType::Ink || annot->AnnotType == AnnotationType::Circle
			|| annot->AnnotType == AnnotationType::Square)
		{
			uiMgr->getAnnotMgr()->GetAnnotSetting()->SetAnnotColor(color);
		}
		else if (annot->AnnotType == AnnotationType::FreeText)
		{
			FreeTextAnnot^ freetextannot = dynamic_cast<FreeTextAnnot^>(annot);
			if (freetextannot)
			{
				freetextannot->TextColor = color;				
				uiMgr->UpdateFreeTextAnnotOverlayControl(annot->PageIndex, freetextannot->Page_HightLightRect);
			}

			uiMgr->getAnnotMgr()->GetAnnotSetting()->SetFreeTextColor(color);
		}
		else if (annot->AnnotType == AnnotationType::Caret)
		{
			uiMgr->getAnnotMgr()->GetAnnotSetting()->SetInsertTextColor(color);
			CaretAnnot^ caretAnnot = dynamic_cast<CaretAnnot^>(annot);
			if (caretAnnot->IsHasGroup)
			{
				if (uiMgr->getAnnotMgr()->IsGetGroupAnnotElement(caretAnnot->PageIndex, caretAnnot->GroupAnnotMinRect,
					caretAnnot->GroupAnnotIndex, AnnotationType::StrikeOut))
				{
					IBaseAnnot^ groupAnnot = uiMgr->getAnnotMgr()->GetGroupAnnot();
					groupAnnot->Color = color;
				}
			}
		}
	}
	else
	{
		if (uiMgr->CreateTypeOfAnnot != AnnotCreateType::None && uiMgr->CreateTypeOfAnnot != AnnotCreateType::FreeText)
			uiMgr->CreateGraphAnnotBySliderPage();
		if (uiMgr->CreateTypeOfAnnot == AnnotCreateType::HightLight || uiMgr->CreateTypeOfAnnot == AnnotCreateType::AreaHighLight)
		{
			uiMgr->getAnnotMgr()->GetAnnotSetting()->SetHightLightAnnotColor(color);
		}
		else if (uiMgr->CreateTypeOfAnnot == AnnotCreateType::UnderLine)
		{
			uiMgr->getAnnotMgr()->GetAnnotSetting()->SetUnderLineAnnotColor(color);
		}
		else if (uiMgr->CreateTypeOfAnnot == AnnotCreateType::StrikeOut)
		{
			uiMgr->getAnnotMgr()->GetAnnotSetting()->SetStrikeOutAnnotColor(color);
		}
		else if (uiMgr->CreateTypeOfAnnot == AnnotCreateType::Squiggly)
		{
			uiMgr->getAnnotMgr()->GetAnnotSetting()->SetSquigglyAnnotColor(color);
		}
		else if (uiMgr->CreateTypeOfAnnot == AnnotCreateType::Line || uiMgr->CreateTypeOfAnnot == AnnotCreateType::Arrow)
		{
			uiMgr->getAnnotMgr()->GetAnnotSetting()->SetLineAnnotColor(color);
		}
		else if (uiMgr->CreateTypeOfAnnot == AnnotCreateType::Pencil || uiMgr->CreateTypeOfAnnot == AnnotCreateType::Circle
			|| uiMgr->CreateTypeOfAnnot == AnnotCreateType::Square || uiMgr->CreateTypeOfAnnot == AnnotCreateType::Arrow)
		{
			uiMgr->getAnnotMgr()->GetAnnotSetting()->SetAnnotColor(color);
		}
		else if (uiMgr->CreateTypeOfAnnot == AnnotCreateType::FreeText)
		{
			uiMgr->getAnnotMgr()->GetAnnotSetting()->SetFreeTextColor(color);

			IBaseAnnotEventPeer^ annotEventPeer = uiMgr->getAnnotMgr()->GetAnnotHandlerByType(AnnotationType::FreeText)->AnnotEvent;
			FreeTextEventPeer^ freetextEventPeer = dynamic_cast<FreeTextEventPeer^>(annotEventPeer);
			if (freetextEventPeer)
				freetextEventPeer->UpdatFreeTextBoxProperty();
		}
		else if (uiMgr->CreateTypeOfAnnot == AnnotCreateType::Caret)
		{
			uiMgr->getAnnotMgr()->GetAnnotSetting()->SetInsertTextColor(color);
		}
		else if (uiMgr->CreateTypeOfAnnot == AnnotCreateType::ReplaceText)
		{
			uiMgr->getAnnotMgr()->GetAnnotSetting()->SetReplaceTextColor(color);
		}
	}
}


void AnnotPropertyPhone::OpacityPageBtnSelect_Tapped(Platform::Object^ sender, Windows::UI::Xaml::Input::TappedRoutedEventArgs^ e)
{
	UIExtensionsManager^ uiMgr = (UIExtensionsManager^)View->ExtensionsManager;
	IBaseAnnot^ annot = uiMgr->getAnnotMgr()->GetSelectedAnnot();
	RadioButton^ button = dynamic_cast<RadioButton^>(sender);
	String^ opacityvalue = button->Tag->ToString();
	double opacity = 0;
	if (opacityvalue == "25")
	{
		opacity = 25;
	}
	else if (opacityvalue == "50")
	{
		opacity = 50;
	}
	else if (opacityvalue == "75")
	{
		opacity = 75;
	}
	else
	{
		opacity = 100;
	}
	opacity = opacity / 100;

	if (annot)
	{
		annot->IsModifyProperty = true;
		annot->Opacity = opacity;

		if (annot->AnnotType == AnnotationType::HightLight)
		{
			uiMgr->getAnnotMgr()->GetAnnotSetting()->SetHightLightAnnotOpacity(opacity);
		}
		else if (annot->AnnotType == AnnotationType::UnderLine)
		{
			uiMgr->getAnnotMgr()->GetAnnotSetting()->SetUnderLineAnnotOpacity(opacity);
		}
		else if (annot->AnnotType == AnnotationType::StrikeOut)
		{
			uiMgr->getAnnotMgr()->GetAnnotSetting()->SetStrikeOutAnnotOpacity(opacity);
			StrikeOutAnnot^ strikeoutAnnot = dynamic_cast<StrikeOutAnnot^>(annot);
			if (strikeoutAnnot->IsHasGroup)
			{
				if (uiMgr->getAnnotMgr()->IsGetGroupAnnotElement(strikeoutAnnot->PageIndex, strikeoutAnnot->GroupAnnotMinRect,
					strikeoutAnnot->GroupAnnotIndex, AnnotationType::Caret))
				{
					IBaseAnnot^ groupAnnot = uiMgr->getAnnotMgr()->GetGroupAnnot();
					groupAnnot->Opacity = opacity;
				}
				uiMgr->getAnnotMgr()->GetAnnotSetting()->SetReplaceTextAnnotOpacity(opacity);
			}
		}
		else if (annot->AnnotType == AnnotationType::Squiggly)
		{
			uiMgr->getAnnotMgr()->GetAnnotSetting()->SetSquigglyAnnotOpacity(opacity);
		}
		else if (annot->AnnotType == AnnotationType::Ink || annot->AnnotType == AnnotationType::Circle
			|| annot->AnnotType == AnnotationType::Square || annot->AnnotType == AnnotationType::Line)
		{
			uiMgr->getAnnotMgr()->GetAnnotSetting()->SetAnnotOpacity(opacity);
		}
		else if (annot->AnnotType == AnnotationType::FreeText)
		{
			uiMgr->getAnnotMgr()->GetAnnotSetting()->SetFreeTextAnnotOpacity(opacity);
			FreeTextAnnot^ freetextannot = dynamic_cast<FreeTextAnnot^>(annot);
			if (freetextannot)
			{				
				uiMgr->UpdateFreeTextAnnotOverlayControl(annot->PageIndex, freetextannot->Page_HightLightRect);
			}
		}
		else if (annot->AnnotType == AnnotationType::Caret)
		{
			uiMgr->getAnnotMgr()->GetAnnotSetting()->SetInsertTextAnnotOpacity(opacity);
			CaretAnnot^ caretAnnot = dynamic_cast<CaretAnnot^>(annot);
			if (caretAnnot->IsHasGroup)
			{
				if (uiMgr->getAnnotMgr()->IsGetGroupAnnotElement(caretAnnot->PageIndex, caretAnnot->GroupAnnotMinRect,
					caretAnnot->GroupAnnotIndex, AnnotationType::StrikeOut))
				{
					IBaseAnnot^ groupAnnot = uiMgr->getAnnotMgr()->GetGroupAnnot();
					groupAnnot->Opacity = opacity;
				}
			}
		}
	}
	else
	{
		if (uiMgr->CreateTypeOfAnnot != AnnotCreateType::None && uiMgr->CreateTypeOfAnnot != AnnotCreateType::FreeText)
			uiMgr->CreateGraphAnnotBySliderPage();
		if (uiMgr->CreateTypeOfAnnot == AnnotCreateType::HightLight || uiMgr->CreateTypeOfAnnot == AnnotCreateType::AreaHighLight)
		{
			uiMgr->getAnnotMgr()->GetAnnotSetting()->SetHightLightAnnotOpacity(opacity);
		}
		else if (uiMgr->CreateTypeOfAnnot == AnnotCreateType::UnderLine)
		{
			uiMgr->getAnnotMgr()->GetAnnotSetting()->SetUnderLineAnnotOpacity(opacity);
		}
		else if (uiMgr->CreateTypeOfAnnot == AnnotCreateType::StrikeOut)
		{
			uiMgr->getAnnotMgr()->GetAnnotSetting()->SetStrikeOutAnnotOpacity(opacity);
		}
		else if (uiMgr->CreateTypeOfAnnot == AnnotCreateType::Squiggly)
		{
			uiMgr->getAnnotMgr()->GetAnnotSetting()->SetSquigglyAnnotOpacity(opacity);
		}
		else if (uiMgr->CreateTypeOfAnnot == AnnotCreateType::Pencil || uiMgr->CreateTypeOfAnnot == AnnotCreateType::Circle
			|| uiMgr->CreateTypeOfAnnot == AnnotCreateType::Square || uiMgr->CreateTypeOfAnnot == AnnotCreateType::Line || uiMgr->CreateTypeOfAnnot == AnnotCreateType::Arrow)
		{
			uiMgr->getAnnotMgr()->GetAnnotSetting()->SetAnnotOpacity(opacity);
		}
		else if (uiMgr->CreateTypeOfAnnot == AnnotCreateType::FreeText)
		{
			uiMgr->getAnnotMgr()->GetAnnotSetting()->SetFreeTextAnnotOpacity(opacity);

			IBaseAnnotEventPeer^ annotEventPeer = uiMgr->getAnnotMgr()->GetAnnotHandlerByType(AnnotationType::FreeText)->AnnotEvent;
			FreeTextEventPeer^ freetextEventPeer = dynamic_cast<FreeTextEventPeer^>(annotEventPeer);
			if (freetextEventPeer)
				freetextEventPeer->UpdatFreeTextBoxProperty();
		}
		else if (uiMgr->CreateTypeOfAnnot == AnnotCreateType::Caret)
		{
			uiMgr->getAnnotMgr()->GetAnnotSetting()->SetInsertTextAnnotOpacity(opacity);
		}
		else if (uiMgr->CreateTypeOfAnnot == AnnotCreateType::ReplaceText)
		{
			uiMgr->getAnnotMgr()->GetAnnotSetting()->SetReplaceTextAnnotOpacity(opacity);
		}
	}
}


void AnnotPropertyPhone::ExitProperty_Tapped(Platform::Object^ sender, Windows::UI::Xaml::Input::TappedRoutedEventArgs^ e)
{
	if (!View)
		return;
	UIExtensionsManager^ uiMgr = (UIExtensionsManager^)View->ExtensionsManager;
	if (uiMgr->CreateTypeOfAnnot == AnnotCreateType::Pencil)
	{
		auto _pAnnotHandler = uiMgr->getAnnotMgr()->GetAnnotHandlerByType(GetAnnotType(AnnotCreateType::Pencil));
		if (_pAnnotHandler)
			_pAnnotHandler->CreateAnnotAfterSelectText();
	}

	if (uiMgr->CreateTypeOfAnnot == AnnotCreateType::Circle || uiMgr->CreateTypeOfAnnot == AnnotCreateType::Square
		|| uiMgr->CreateTypeOfAnnot == AnnotCreateType::Line)
	{
		View->Redraw(View->GetCurrentPage());
	}

	uiMgr->HideNoteStylePopup();
	uiMgr->ExitAnnotPropertyMode();
	uiMgr->SetAnnotCanvasVisible(false);
	uiMgr->ShowDirecControl(false);

	if (!uiMgr->getAnnotMgr())
		return;
	if (uiMgr->getAnnotMgr()->GetSelectedAnnot())
	{
		uiMgr->getAnnotMgr()->SetPdfSelectAnnotNull();
		View->Update();
	}


	stampIconPopup->IsOpen = false;
}

bool AnnotPropertyPhone::IsNoteStylePopupIsOpen()
{
	return false;
}

void AnnotPropertyPhone::HideNoteStylePopup()
{

}


void AnnotPropertyPhone::OnWindowsSizeChanged(Platform::Object ^sender, Windows::UI::Core::WindowSizeChangedEventArgs ^e)
{
	Windows::Foundation::Rect winRect = Windows::UI::Xaml::Window::Current->Bounds;
	mainGrid->Width = winRect.Width;
	gridColorList1->Width = winRect.Width - 60;
	gridColorList2->Width = winRect.Width - 60;
	gridOpacityList1->Width = winRect.Width - 60;

	sliderLinessPor->Width = winRect.Width*0.6;

	UIExtensionsManager^ uiMgr = (UIExtensionsManager^)View->ExtensionsManager;
	IBaseAnnot^ annot = uiMgr->getAnnotMgr()->GetSelectedAnnot();
	if (winRect.Width > winRect.Height)
	{
		gridColorList1Por->Width = winRect.Width - 30;
		gridOpacityList1Por->Width = winRect.Width - 30;

		pivot->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
		if (uiMgr->CreateTypeOfAnnot == AnnotCreateType::Text || (annot && annot->AnnotType == AnnotationType::Text))
		{
			pivotPor->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
		}
		else
		{
			pivotPor->Visibility = Windows::UI::Xaml::Visibility::Visible;
		}
		if (uiMgr->CreateTypeOfAnnot == AnnotCreateType::FreeText || (annot && annot->AnnotType == AnnotationType::FreeText))
		{
			freeTextPropertyPor->ResetFontValue();
			if (freeTextProperty->IsFontPanelIsVisible())
			{
				freeTextPropertyPor->SetFontPanelVisible();
			}
			else if (freeTextProperty->IsFontNameListIsVisible())
			{
				freeTextPropertyPor->SetFontNameListVisible();
			}
			else if (freeTextProperty->IsFontSizeListIsVisible())
			{
				freeTextPropertyPor->SetFontSizeListVisible();
			}
			freeTextProperty->ResetFontNameSizeListWidth();
			freeTextPropertyPor->ResetFontNameSizeListWidth();
		}
	}
	else
	{
		gridColorList1->Width = winRect.Width - 30;
		gridColorList2->Width = winRect.Width - 30;
		gridOpacityList1->Width = winRect.Width - 30;


		if (uiMgr->CreateTypeOfAnnot == AnnotCreateType::Text || (annot && annot->AnnotType == AnnotationType::Text))
		{
			pivot->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
		}
		else
		{
			pivot->Visibility = Windows::UI::Xaml::Visibility::Visible;
		}
		pivotPor->Visibility = Windows::UI::Xaml::Visibility::Collapsed;

		if (uiMgr->CreateTypeOfAnnot == AnnotCreateType::FreeText || (annot && annot->AnnotType == AnnotationType::FreeText))
		{
			freeTextProperty->ResetFontValue();
			if (freeTextPropertyPor->IsFontPanelIsVisible())
			{
				freeTextProperty->SetFontPanelVisible();
			}
			else if (freeTextPropertyPor->IsFontNameListIsVisible())
			{
				freeTextProperty->SetFontNameListVisible();
			}
			else if (freeTextPropertyPor->IsFontSizeListIsVisible())
			{
				freeTextProperty->SetFontSizeListVisible();
			}
			freeTextProperty->ResetFontNameSizeListWidth();
			freeTextPropertyPor->ResetFontNameSizeListWidth();
		}
	}
	InitAppBarProperty();
}


AnnotationType AnnotPropertyPhone::GetAnnotType(AnnotCreateType optType)
{
	AnnotationType _annotType;
	switch (optType)
	{
	case AnnotCreateType::None:
		_annotType = AnnotationType::None;
		break;
	case AnnotCreateType::HightLight:
		_annotType = AnnotationType::HightLight;
		break;
	case AnnotCreateType::AreaHighLight:
		_annotType = AnnotationType::HightLight;
		break;
	case AnnotCreateType::Comment:
		_annotType = AnnotationType::HightLight;
		break;
	case AnnotCreateType::UnderLine:
		_annotType = AnnotationType::UnderLine;
		break;
	case AnnotCreateType::StrikeOut:
		_annotType = AnnotationType::StrikeOut;
		break;
	case AnnotCreateType::Square:
		_annotType = AnnotationType::Square;
		break;
	case AnnotCreateType::Circle:
		_annotType = AnnotationType::Circle;
		break;
	case AnnotCreateType::Line:
		_annotType = AnnotationType::Line;
		break;
	case AnnotCreateType::Arrow:
		_annotType = AnnotationType::Line;
		break;
	case AnnotCreateType::Pencil:
		_annotType = AnnotationType::Ink;
		break;
	case AnnotCreateType::Squiggly:
		_annotType = AnnotationType::Squiggly;
		break;
	case AnnotCreateType::Text:
		_annotType = AnnotationType::Text;
		break;
	case AnnotCreateType::FreeText:
		_annotType = AnnotationType::FreeText;
		break;
	case AnnotCreateType::Caret:
	case AnnotCreateType::ReplaceText:
		_annotType = AnnotationType::Caret;
		break;
	default:
		_annotType = AnnotationType::None;
		break;
	}
	return _annotType;
}

void AnnotPropertyPhone::InitAnnotColorList(AnnotationType type)
{
	if (type == AnnotationType::HightLight)
	{
		ColorPageBtn1->Content = "#FFFFFF00";
		ColorPageBtn2->Content = "#FFCCFF66";
		ColorPageBtn3->Content = "#FF00FFFF";
		ColorPageBtn4->Content = "#FF99CCFF";
		ColorPageBtn5->Content = "#FF7480FC";
		ColorPageBtn6->Content = "#FFFF9999";
		ColorPageBtn7->Content = "#FF00CC66";
		ColorPageBtn8->Content = "#FF22F3B1";

		ColorPageBtn1Por->Content = "#FFFFFF00";
		ColorPageBtn2Por->Content = "#FFCCFF66";
		ColorPageBtn3Por->Content = "#FF00FFFF";
		ColorPageBtn4Por->Content = "#FF99CCFF";
		ColorPageBtn5Por->Content = "#FF7480FC";
		ColorPageBtn6Por->Content = "#FFFF9999";
		ColorPageBtn7Por->Content = "#FF00CC66";
		ColorPageBtn8Por->Content = "#FF22F3B1";
	}
	else if (type == AnnotationType::FreeText)
	{
		ColorPageBtn1->Content = "#FF3366CC";
		ColorPageBtn2->Content = "#FF669933";
		ColorPageBtn3->Content = "#FFCC6600";
		ColorPageBtn4->Content = "#FFCC9900";
		ColorPageBtn5->Content = "#FFCC0000";
		ColorPageBtn6->Content = "#FF336666";
		ColorPageBtn7->Content = "#FF660066";
		ColorPageBtn8->Content = "#FF000000";

		ColorPageBtn1Por->Content = "#FF3366CC";
		ColorPageBtn2Por->Content = "#FF669933";
		ColorPageBtn3Por->Content = "#FFCC6600";
		ColorPageBtn4Por->Content = "#FFCC9900";
		ColorPageBtn5Por->Content = "#FFCC0000";
		ColorPageBtn6Por->Content = "#FF336666";
		ColorPageBtn7Por->Content = "#FF660066";
		ColorPageBtn8Por->Content = "#FF000000";
	}
	else if (type == AnnotationType::UnderLine || type == AnnotationType::Squiggly)
	{
		ColorPageBtn1->Content = "#FF33CC00";
		ColorPageBtn2->Content = "#FFCCCC00";
		ColorPageBtn3->Content = "#FFFF9933";
		ColorPageBtn4->Content = "#FF0099CC";
		ColorPageBtn5->Content = "#FF3366FF";
		ColorPageBtn6->Content = "#FFCC33FF";
		ColorPageBtn7->Content = "#FFCC0099";
		ColorPageBtn8->Content = "#FFFF0000";

		ColorPageBtn1Por->Content = "#FF33CC00";
		ColorPageBtn2Por->Content = "#FFCCCC00";
		ColorPageBtn3Por->Content = "#FFFF9933";
		ColorPageBtn4Por->Content = "#FF0099CC";
		ColorPageBtn5Por->Content = "#FF3366FF";
		ColorPageBtn6Por->Content = "#FFCC33FF";
		ColorPageBtn7Por->Content = "#FFCC0099";
		ColorPageBtn8Por->Content = "#FFFF0000";
	}
	else if (type == AnnotationType::StrikeOut || type == AnnotationType::Caret)
	{
		ColorPageBtn1->Content = "#FFFF3333";
		ColorPageBtn2->Content = "#FFFF00FF";
		ColorPageBtn3->Content = "#FF9966FF";
		ColorPageBtn4->Content = "#FF66CC33";
		ColorPageBtn5->Content = "#FFCCCC00";
		ColorPageBtn6->Content = "#FFFF9900";
		ColorPageBtn7->Content = "#FF00CCFF";
		ColorPageBtn8->Content = "#FF00CCCC";

		ColorPageBtn1Por->Content = "#FFFF3333";
		ColorPageBtn2Por->Content = "#FFFF00FF";
		ColorPageBtn3Por->Content = "#FF9966FF";
		ColorPageBtn4Por->Content = "#FF66CC33";
		ColorPageBtn5Por->Content = "#FFCCCC00";
		ColorPageBtn6Por->Content = "#FFFF9900";
		ColorPageBtn7Por->Content = "#FF00CCFF";
		ColorPageBtn8Por->Content = "#FF00CCCC";
	}
	else if (type == AnnotationType::Circle || type == AnnotationType::Square || type == AnnotationType::Line || type == AnnotationType::Ink)
	{
		ColorPageBtn1->Content = "#FFFF9F40";
		ColorPageBtn2->Content = "#FF8080FF";
		ColorPageBtn3->Content = "#FFBAE94C";
		ColorPageBtn4->Content = "#FFFFF160";
		ColorPageBtn5->Content = "#FFFF4C4C";
		ColorPageBtn6->Content = "#FF669999";
		ColorPageBtn7->Content = "#FFC72DA1";
		ColorPageBtn8->Content = "#FF996666";

		ColorPageBtn1Por->Content = "#FFFF9F40";
		ColorPageBtn2Por->Content = "#FF8080FF";
		ColorPageBtn3Por->Content = "#FFBAE94C";
		ColorPageBtn4Por->Content = "#FFFFF160";
		ColorPageBtn5Por->Content = "#FFFF4C4C";
		ColorPageBtn6Por->Content = "#FF669999";
		ColorPageBtn7Por->Content = "#FFC72DA1";
		ColorPageBtn8Por->Content = "#FF996666";
	}
}

void AnnotPropertyPhone::SetRadioColorChecked(Platform::String^ colorRGBValue, AnnotationType type)
{
	if (type == AnnotationType::HightLight)
	{
		if (colorRGBValue == "#FFFFFF00")
		{
			ColorPageBtn1->IsChecked = true;
			ColorPageBtn1Por->IsChecked = true;
		}
		else if (colorRGBValue == "#FFCCFF66")
		{
			ColorPageBtn2->IsChecked = true;
			ColorPageBtn2Por->IsChecked = true;
		}
		else if (colorRGBValue == "#FF00FFFF")
		{
			ColorPageBtn3->IsChecked = true;
			ColorPageBtn3Por->IsChecked = true;
		}
		else if (colorRGBValue == "#FF99CCFF")
		{
			ColorPageBtn4->IsChecked = true;
			ColorPageBtn4Por->IsChecked = true;
		}
		else if (colorRGBValue == "#FF7480FC")
		{
			ColorPageBtn5->IsChecked = true;
			ColorPageBtn5Por->IsChecked = true;
		}
		else if (colorRGBValue == "#FFFF9999")
		{
			ColorPageBtn6->IsChecked = true;
			ColorPageBtn6Por->IsChecked = true;
		}
		else if (colorRGBValue == "#FF00CC66")
		{
			ColorPageBtn7->IsChecked = true;
			ColorPageBtn7Por->IsChecked = true;
		}
		else if (colorRGBValue == "#FF22F3B1")
		{
			ColorPageBtn8->IsChecked = true;
			ColorPageBtn8Por->IsChecked = true;
		}
	}
	else if (type == AnnotationType::FreeText)
	{
		if (colorRGBValue == "#FF3366CC")
		{
			ColorPageBtn1->IsChecked = true;
			ColorPageBtn1Por->IsChecked = true;
		}
		else if (colorRGBValue == "#FF669933")
		{
			ColorPageBtn2->IsChecked = true;
			ColorPageBtn2Por->IsChecked = true;
		}
		else if (colorRGBValue == "#FFCC6600")
		{
			ColorPageBtn3->IsChecked = true;
			ColorPageBtn3Por->IsChecked = true;
		}
		else if (colorRGBValue == "#FFCC9900")
		{
			ColorPageBtn4->IsChecked = true;
			ColorPageBtn4Por->IsChecked = true;
		}
		else if (colorRGBValue == "#FFCC0000")
		{
			ColorPageBtn5->IsChecked = true;
			ColorPageBtn5Por->IsChecked = true;
		}
		else if (colorRGBValue == "#FF336666")
		{
			ColorPageBtn6->IsChecked = true;
			ColorPageBtn6Por->IsChecked = true;
		}
		else if (colorRGBValue == "#FF660066")
		{
			ColorPageBtn7->IsChecked = true;
			ColorPageBtn7Por->IsChecked = true;
		}
		else if (colorRGBValue == "#FF000000")
		{
			ColorPageBtn8->IsChecked = true;
			ColorPageBtn8Por->IsChecked = true;
		}
	}
	else if (type == AnnotationType::UnderLine || type == AnnotationType::Squiggly)
	{
		if (colorRGBValue == "#FF33CC00")
		{
			ColorPageBtn1->IsChecked = true;
			ColorPageBtn1Por->IsChecked = true;
		}
		else if (colorRGBValue == "#FFCCCC00")
		{
			ColorPageBtn2->IsChecked = true;
			ColorPageBtn2Por->IsChecked = true;
		}
		else if (colorRGBValue == "#FFFF9933")
		{
			ColorPageBtn3->IsChecked = true;
			ColorPageBtn3Por->IsChecked = true;
		}
		else if (colorRGBValue == "#FF0099CC")
		{
			ColorPageBtn4->IsChecked = true;
			ColorPageBtn4Por->IsChecked = true;
		}
		else if (colorRGBValue == "#FF3366FF")
		{
			ColorPageBtn5->IsChecked = true;
			ColorPageBtn5Por->IsChecked = true;
		}
		else if (colorRGBValue == "#FFCC33FF")
		{
			ColorPageBtn6->IsChecked = true;
			ColorPageBtn6Por->IsChecked = true;
		}
		else if (colorRGBValue == "#FFCC0099")
		{
			ColorPageBtn7->IsChecked = true;
			ColorPageBtn7Por->IsChecked = true;
		}
		else if (colorRGBValue == "#FFFF0000")
		{
			ColorPageBtn8->IsChecked = true;
			ColorPageBtn8Por->IsChecked = true;
		}
	}
	else if (type == AnnotationType::StrikeOut || type == AnnotationType::Caret)
	{
		if (colorRGBValue == "#FFFF3333")
		{
			ColorPageBtn1->IsChecked = true;
			ColorPageBtn1Por->IsChecked = true;
		}
		else if (colorRGBValue == "#FFFF00FF")
		{
			ColorPageBtn2->IsChecked = true;
			ColorPageBtn2Por->IsChecked = true;
		}
		else if (colorRGBValue == "#FF9966FF")
		{
			ColorPageBtn3->IsChecked = true;
			ColorPageBtn3Por->IsChecked = true;
		}
		else if (colorRGBValue == "#FF66CC33")
		{
			ColorPageBtn4->IsChecked = true;
			ColorPageBtn4Por->IsChecked = true;
		}
		else if (colorRGBValue == "#FFCCCC00")
		{
			ColorPageBtn5->IsChecked = true;
			ColorPageBtn5Por->IsChecked = true;
		}
		else if (colorRGBValue == "#FFFF9900")
		{
			ColorPageBtn6->IsChecked = true;
			ColorPageBtn6Por->IsChecked = true;
		}
		else if (colorRGBValue == "#FF00CCFF")
		{
			ColorPageBtn7->IsChecked = true;
			ColorPageBtn7Por->IsChecked = true;
		}
		else if (colorRGBValue == "#FF00CCCC")
		{
			ColorPageBtn8->IsChecked = true;
			ColorPageBtn8Por->IsChecked = true;
		}
	}
	else if (type == AnnotationType::Circle || type == AnnotationType::Square || type == AnnotationType::Line || type == AnnotationType::Ink)
	{
		if (colorRGBValue == "#FFFF9F40")
		{
			ColorPageBtn1->IsChecked = true;
			ColorPageBtn1Por->IsChecked = true;
		}
		else if (colorRGBValue == "#FF8080FF")
		{
			ColorPageBtn2->IsChecked = true;
			ColorPageBtn2Por->IsChecked = true;
		}
		else if (colorRGBValue == "#FFBAE94C")
		{
			ColorPageBtn3->IsChecked = true;
			ColorPageBtn3Por->IsChecked = true;
		}
		else if (colorRGBValue == "#FFFFF160")
		{
			ColorPageBtn4->IsChecked = true;
			ColorPageBtn4Por->IsChecked = true;
		}
		else if (colorRGBValue == "#FFFF4C4C")
		{
			ColorPageBtn5->IsChecked = true;
			ColorPageBtn5Por->IsChecked = true;
		}
		else if (colorRGBValue == "#FF669999")
		{
			ColorPageBtn6->IsChecked = true;
			ColorPageBtn6Por->IsChecked = true;
		}
		else if (colorRGBValue == "#FFC72DA1")
		{
			ColorPageBtn7->IsChecked = true;
			ColorPageBtn7Por->IsChecked = true;
		}
		else if (colorRGBValue == "#FF996666")
		{
			ColorPageBtn8->IsChecked = true;
			ColorPageBtn8Por->IsChecked = true;
		}
	}
}


void AnnotPropertyPhone::AppBarButton_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{
	UIExtensionsManager^ uiMgr = (UIExtensionsManager^)View->ExtensionsManager;
	if (View && uiMgr->CreateTypeOfAnnot == AnnotCreateType::Pencil)
	{
		auto _pAnnotHandler = uiMgr->getAnnotMgr()->GetAnnotHandlerByType(GetAnnotType(AnnotCreateType::Pencil));
		if (_pAnnotHandler)
			_pAnnotHandler->CreateAnnotAfterSelectText();
	}
}


void AnnotPropertyPhone::SetPropertyToolIsEnabled(bool isEnable)
{
	ColorPageBtn1->IsEnabled = isEnable;
	ColorPageBtn2->IsEnabled = isEnable;
	ColorPageBtn3->IsEnabled = isEnable;
	ColorPageBtn4->IsEnabled = isEnable;
	ColorPageBtn5->IsEnabled = isEnable;
	ColorPageBtn6->IsEnabled = isEnable;
	ColorPageBtn7->IsEnabled = isEnable;
	ColorPageBtn8->IsEnabled = isEnable;

	OpacityPageBtn1->IsEnabled = isEnable;
	OpacityPageBtn2->IsEnabled = isEnable;
	OpacityPageBtn3->IsEnabled = isEnable;
	OpacityPageBtn4->IsEnabled = isEnable;
}

void AnnotPropertyPhone::CloseBorderListPopup()
{

}


void AnnotPropertyPhone::SetStampPopupClosed()
{
	stampIconPopup->IsOpen = false;
}


void foxit::rdk::uiextensions::AnnotPropertyPhone::OnValueChanged(Platform::Object ^sender, Windows::UI::Xaml::Controls::Primitives::RangeBaseValueChangedEventArgs ^e)
{
	UIExtensionsManager^ uiMgr = (UIExtensionsManager^)View->ExtensionsManager;
	uiMgr->CreateGraphAnnotBySliderPage();
	tbLiness->Text = sliderLiness->Value.ToString() + "pt";
	IBaseAnnot^ annot = uiMgr->getAnnotMgr()->GetSelectedAnnot();
	double borderWidth = sliderLiness->Value;

	if (annot)
	{
		annot->IsModifyProperty = true;
		annot->BorderWidth = borderWidth;
	}
	uiMgr->getAnnotMgr()->GetAnnotSetting()->SetAnnotDrawBorderWidth((int)borderWidth);
}

void foxit::rdk::uiextensions::AnnotPropertyPhone::OnValuePorChanged(Platform::Object ^sender, Windows::UI::Xaml::Controls::Primitives::RangeBaseValueChangedEventArgs ^e)
{
	UIExtensionsManager^ uiMgr = (UIExtensionsManager^)View->ExtensionsManager;
	uiMgr->CreateGraphAnnotBySliderPage();
	tbLinessPor->Text = sliderLinessPor->Value.ToString() + "pt";
	IBaseAnnot^ annot = uiMgr->getAnnotMgr()->GetSelectedAnnot();
	double borderWidth = sliderLinessPor->Value;

	if (annot)
	{
		annot->IsModifyProperty = true;
		annot->BorderWidth = borderWidth;
	}
	uiMgr->getAnnotMgr()->GetAnnotSetting()->SetAnnotDrawBorderWidth((int)borderWidth);
}


void AnnotPropertyPhone::InitControlSize()
{
	Windows::Foundation::Rect winRect = Windows::UI::Xaml::Window::Current->Bounds;
	mainGrid->Width = winRect.Width;
}

void AnnotPropertyPhone::SetFreeTextPropertyIsShow(bool isShow, Platform::String^ title)
{
	UIExtensionsManager^ uiMgr = (UIExtensionsManager^)View->ExtensionsManager;
	Windows::Foundation::Rect winRect = Windows::UI::Xaml::Window::Current->Bounds;
	if (isShow)
	{
		if (winRect.Width < winRect.Height)
		{
			bitBack->Visibility = Windows::UI::Xaml::Visibility::Visible;
			tb->Text = title;
			colorPanel->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
			opacityPanel->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
		}
		else
		{
			bitBackPor->Visibility = Windows::UI::Xaml::Visibility::Visible;
			tbPor->Text = title;
			colorPanelPor->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
			opacityPanelPor->Visibility = Windows::UI::Xaml::Visibility::Collapsed;

			String^ titleStr = uiMgr->GetResourceString(L"FontNameStrTitle");
			if (titleStr == title)
			{
				scrollPor->Height = 100;
			}
		}
	}
	else
	{
		String^ fillStr = uiMgr->GetResourceString(L"fillTitleTextStr");
		if (winRect.Width < winRect.Height)
		{
			bitBack->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
			tb->Text = fillStr;
			colorPanel->Visibility = Windows::UI::Xaml::Visibility::Visible;
			opacityPanel->Visibility = Windows::UI::Xaml::Visibility::Visible;
			freeTextProperty->HideFontList();
		}
		else
		{
			bitBackPor->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
			tbPor->Text = fillStr;
			colorPanelPor->Visibility = Windows::UI::Xaml::Visibility::Visible;
			opacityPanelPor->Visibility = Windows::UI::Xaml::Visibility::Visible;
			if(freeTextPropertyPor->IsHideFontNameList())			
			{
				scrollPor->Height = 150;
			}
			freeTextPropertyPor->HideFontList();
			scrollPor->ChangeView((double)0.0, (double)0.0, (float)1.0);
		}
	}
}

void foxit::rdk::uiextensions::AnnotPropertyPhone::bitBack_Tapped(Platform::Object^ sender, Windows::UI::Xaml::Input::TappedRoutedEventArgs^ e)
{
	SetFreeTextPropertyIsShow(false, "Fill");
}


NoteStyleInfo::NoteStyleInfo()
{
	_IconIsChecked = Windows::UI::Xaml::Visibility::Collapsed;
}

Platform::String^ NoteStyleInfo::IconName::get()
{
	return _IconName;
}

void NoteStyleInfo::IconName::set(Platform::String^ value)
{
	if (_IconName != value)
	{
		_IconName = value;
		OnPropertyChanged("IconName");
	}
}

Platform::String^ NoteStyleInfo::IconTag::get()
{
	return _IconTag;
}

void NoteStyleInfo::IconTag::set(Platform::String^ value)
{
	if (_IconTag != value)
	{
		_IconTag = value;
		OnPropertyChanged("IconTag");
	}
}


void NoteStyleInfo::IconImage::set(Windows::UI::Xaml::Media::ImageSource^ value)
{
	if (_IconImage != value)
	{
		_IconImage = value;
		OnPropertyChanged("IconImage");
	}
}

Windows::UI::Xaml::Media::ImageSource^ NoteStyleInfo::IconImage::get()
{
	return _IconImage;
}

void NoteStyleInfo::IconIsChecked::set(Windows::UI::Xaml::Visibility value)
{
	if (_IconIsChecked != value)
	{
		_IconIsChecked = value;
		OnPropertyChanged("IconIsChecked");
	}
}

Windows::UI::Xaml::Visibility NoteStyleInfo::IconIsChecked::get()
{
	return _IconIsChecked;
}











