﻿/**
 * Copyright (C) 2003-2018, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit Mobile PDF SDK to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */


#include "pch.h"
#include "AnnotStampUserControl.xaml.h"

using namespace foxit::rdk;
using namespace foxit::rdk::uiextensions;

using namespace Platform;
using namespace Windows::Foundation;
using namespace Windows::Foundation::Collections;
using namespace Windows::UI::Xaml;
using namespace Windows::UI::Xaml::Controls;
using namespace Windows::UI::Xaml::Controls::Primitives;
using namespace Windows::UI::Xaml::Data;
using namespace Windows::UI::Xaml::Input;
using namespace Windows::UI::Xaml::Media;
using namespace Windows::UI::Xaml::Navigation;

// The User Control item template is documented at http://go.microsoft.com/fwlink/?LinkId=234236

AnnotStampUserControl::AnnotStampUserControl()
{
	InitializeComponent();

	
	this->Loaded += ref new Windows::UI::Xaml::RoutedEventHandler(this, &foxit::rdk::uiextensions::AnnotStampUserControl::OnLoaded);
	this->Unloaded += ref new Windows::UI::Xaml::RoutedEventHandler(this, &foxit::rdk::uiextensions::AnnotStampUserControl::OnUnloaded);
}

void AnnotStampUserControl::RelativePageView(PDFViewCtrl^ view)
{
	m_PageViewWeakRef = view;
}

PDFViewCtrl^ AnnotStampUserControl::View::get()
{
	return m_PageViewWeakRef.Resolve<PDFViewCtrl>();
}

void foxit::rdk::uiextensions::AnnotStampUserControl::OnLoaded(Platform::Object ^sender, Windows::UI::Xaml::RoutedEventArgs ^e)
{
	__windowSizeChangedEventRegistrationToken = Window::Current->SizeChanged += ref new Windows::UI::Xaml::WindowSizeChangedEventHandler(this, &foxit::rdk::uiextensions::AnnotStampUserControl::OnSizeChanged);

	m_DefaultStampsVect = ref new Platform::Collections::Vector<Windows::UI::Xaml::Media::ImageSource^>();
	m_SignStampsVect = ref new Platform::Collections::Vector<Windows::UI::Xaml::Media::ImageSource^>();
	m_DynamicStampsVect = ref new Platform::Collections::Vector<Windows::UI::Xaml::Media::ImageSource^>();

	UpdateDefaultStampsList();
	UpdateDynamicStampsList();
	UpdateSignStampsList();	

	if (Windows::System::Profile::AnalyticsInfo::VersionInfo->DeviceFamily == L"Windows.Mobile")
	{
		Windows::Foundation::Rect winRect = Windows::UI::Xaml::Window::Current->Bounds;
		mainGrid->Width = winRect.Width; 
		mainGrid->Height = winRect.Height;
		stampTitleGrid->Width= winRect.Width+3;
		if (winRect.Width < winRect.Height)
		{
		}
		stampTitleGrid->Height = 48;
	}
	else
	{
		mainGrid->BorderThickness = 1;
		mainGrid->BorderBrush = ref new SolidColorBrush(Utility::ConvertFromString("#FFCCCCCC"));
		stampTitleGrid->Height = 0;
	}

	defaultStampsGridView->SelectedIndex = -1;
	customStampsGridView->SelectedIndex = -1;

	UIExtensionsManager^ uiMgr = (UIExtensionsManager^)View->ExtensionsManager;
	int stampIndex = uiMgr->getAnnotMgr()->GetAnnotSetting()->GetStampIndex();
	if (stampIndex < 12)
	{
		defaultStampsGridView->SelectedIndex = stampIndex;
	}
	else
	{
		customStampsGridView->SelectedIndex = stampIndex-12;
	}
}


void foxit::rdk::uiextensions::AnnotStampUserControl::OnUnloaded(Platform::Object ^sender, Windows::UI::Xaml::RoutedEventArgs ^e)
{
	if (__windowSizeChangedEventRegistrationToken.Value)
	{
		Window::Current->SizeChanged -= __windowSizeChangedEventRegistrationToken;
		__windowSizeChangedEventRegistrationToken.Value = 0;
	}
}

void AnnotStampUserControl::UpdateSignStampsList()
{
	m_SignStampsVect->Clear();

	int index = 13;
	Platform::String^ strindex = "13";

	Uri^ uri = ref new Uri("ms-appx:///foxit.rdk.uiextensions/Assets/Annot/stamp_default_13.png");
	while (index <= 17) //TODO determine best resource enum approach
	{
		auto bmi = ref new Windows::UI::Xaml::Media::Imaging::BitmapImage(uri);
		if (bmi)
		{
			Windows::UI::Xaml::Media::Imaging::BitmapImage^ item = bmi;
			m_SignStampsVect->Append(item);
		}
		index++;
		strindex = index.ToString();		
		uri = ref new Uri("ms-appx:///foxit.rdk.uiextensions/Assets/Annot/stamp_default_" + strindex + ".png");
	}
	SignStampsCollectionViewSource->Source = m_SignStampsVect;
}

void AnnotStampUserControl::UpdateDefaultStampsList()
{
	m_DefaultStampsVect->Clear();
	int index = 1;
	Platform::String^ strindex = "01";

	Uri^ uri = ref new Uri("ms-appx:///foxit.rdk.uiextensions/Assets/Annot/stamp_default_01.png");
	while (index < 13) //TODO determine best resource enum approach
	{
		auto bmi = ref new Windows::UI::Xaml::Media::Imaging::BitmapImage(uri);
		if (bmi)
		{
			Windows::UI::Xaml::Media::Imaging::BitmapImage^ item = bmi;
			m_DefaultStampsVect->Append(item);
		}
		index++;
		strindex = index.ToString();
		if (strindex->Length() == 1)
			strindex = "0" + strindex;
		uri = ref new Uri("ms-appx:///foxit.rdk.uiextensions/Assets/Annot/stamp_default_" + strindex + ".png");
	}

	DefaultStampsCollectionViewSource->Source = m_DefaultStampsVect;
}

void AnnotStampUserControl::UpdateDynamicStampsList()
{
	m_DynamicStampsVect->Clear();

	int index = 13;
	Platform::String^ strindex = "13";

	Uri^ uri = ref new Uri("ms-appx:///foxit.rdk.uiextensions/Assets/Annot/stamp_default_13.png");
	while (index <= 17) //TODO determine best resource enum approach
	{
		auto bmi = ref new Windows::UI::Xaml::Media::Imaging::BitmapImage(uri);
		if (bmi)
		{
			Windows::UI::Xaml::Media::Imaging::BitmapImage^ item = bmi;
			m_DynamicStampsVect->Append(item);
		}
		index++;
		strindex = index.ToString();
		uri = ref new Uri("ms-appx:///foxit.rdk.uiextensions/Assets/Annot/stamp_default_" + strindex + ".png");
	}
	DynamicStampsCollectionViewSource->Source = m_DynamicStampsVect;
}

void foxit::rdk::uiextensions::AnnotStampUserControl::btnStandard_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{
	RadioButton^ btn = dynamic_cast<RadioButton^>(sender);
	Platform::String^ tagStr = btn->Tag->ToString();
}


void foxit::rdk::uiextensions::AnnotStampUserControl::defaultStampsGridView_ItemClick(Platform::Object^ sender, Windows::UI::Xaml::Controls::ItemClickEventArgs^ e)
{
	UIExtensionsManager^ uiMgr = (UIExtensionsManager^)View->ExtensionsManager;
	unsigned int index;
	m_DefaultStampsVect->IndexOf(dynamic_cast<Windows::UI::Xaml::Media::ImageSource^>(e->ClickedItem), &index);
	uiMgr->getAnnotMgr()->GetAnnotSetting()->SetStampIndex(index);

	uiMgr->SetAnnotStampClosed();
	uiMgr->InitStampAppBar();
}


void foxit::rdk::uiextensions::AnnotStampUserControl::customStampsGridView_ItemClick(Platform::Object^ sender, Windows::UI::Xaml::Controls::ItemClickEventArgs^ e)
{
	unsigned int index;
	m_SignStampsVect->IndexOf(dynamic_cast<Windows::UI::Xaml::Media::ImageSource^>(e->ClickedItem), &index);
	index = index + 12;
	UIExtensionsManager^ uiMgr = (UIExtensionsManager^)View->ExtensionsManager;
	uiMgr->getAnnotMgr()->GetAnnotSetting()->SetStampIndex(index);

	uiMgr->InitStampAppBar();
	uiMgr->SetAnnotStampClosed();
}


void foxit::rdk::uiextensions::AnnotStampUserControl::dynamicStampsGridView_ItemClick(Platform::Object^ sender, Windows::UI::Xaml::Controls::ItemClickEventArgs^ e)
{
	unsigned int index;
	m_DynamicStampsVect->IndexOf(dynamic_cast<Windows::UI::Xaml::Media::ImageSource^>(e->ClickedItem), &index);
	index = index + 12;
	UIExtensionsManager^ uiMgr = (UIExtensionsManager^)View->ExtensionsManager;
	uiMgr->getAnnotMgr()->GetAnnotSetting()->SetStampIndex(index);

	uiMgr->SetAnnotStampClosed();
	uiMgr->InitStampAppBar();
}


void foxit::rdk::uiextensions::AnnotStampUserControl::OnSizeChanged(Platform::Object ^sender, Windows::UI::Core::WindowSizeChangedEventArgs ^e)
{
	if (Windows::System::Profile::AnalyticsInfo::VersionInfo->DeviceFamily == L"Windows.Mobile")
	{
		Windows::Foundation::Rect winRect = Windows::UI::Xaml::Window::Current->Bounds;
		mainGrid->Width = winRect.Width;
		mainGrid->Height = winRect.Height;
		stampTitleGrid->Width = winRect.Width + 3;
		
		stampTitleGrid->Height = 48;
	}
}
