﻿/**
 * Copyright (C) 2003-2018, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit Mobile PDF SDK to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */


#include "pch.h"
#include "EyeProtectionModeControl.xaml.h"

using namespace foxit::rdk::uiextensions;

using namespace Platform;
using namespace Windows::Foundation;
using namespace Windows::Foundation::Collections;
using namespace Windows::UI::Xaml;
using namespace Windows::UI::Xaml::Controls;
using namespace Windows::UI::Xaml::Controls::Primitives;
using namespace Windows::UI::Xaml::Data;
using namespace Windows::UI::Xaml::Input;
using namespace Windows::UI::Xaml::Media;
using namespace Windows::UI::Xaml::Navigation;

// The User Control item template is documented at http://go.microsoft.com/fwlink/?LinkId=234236

EyeProtectionModeControl::EyeProtectionModeControl()
{
	InitializeComponent();

	OnInitializeControl();
}

void EyeProtectionModeControl::OnInitializeControl()
{
	m_bIsLoaded = false;


	this->Loaded += ref new Windows::UI::Xaml::RoutedEventHandler(this, &foxit::rdk::uiextensions::EyeProtectionModeControl::OnLoaded);
	this->Unloaded += ref new Windows::UI::Xaml::RoutedEventHandler(this, &foxit::rdk::uiextensions::EyeProtectionModeControl::OnUnloaded);
}


void EyeProtectionModeControl::SetPageView(UIExtensionsManager^ view)
{
	m_PageViewWeakRef = view;
}

UIExtensionsManager^ EyeProtectionModeControl::View::get()
{
	return m_PageViewWeakRef.Resolve<UIExtensionsManager>();
}

void foxit::rdk::uiextensions::EyeProtectionModeControl::OnLoaded(Platform::Object ^sender, Windows::UI::Xaml::RoutedEventArgs ^e)
{
	if (View)
	{
		auto bgColor = View->viewControl->PageBgColor;
		auto fgColor = View->viewControl->PageFgColor;

		auto _BtnSize = ColorBtnPanel->Children->Size;
		for (unsigned int i = 0; i < _BtnSize; i++)
		{
			auto _item = ColorBtnPanel->Children->GetAt(i);
			auto _RadioButton = dynamic_cast<Windows::UI::Xaml::Controls::RadioButton^>(_item);
			if (_RadioButton == nullptr)
				continue;

			auto _BackgroundColor = dynamic_cast<Windows::UI::Xaml::Media::SolidColorBrush^>(_RadioButton->Background)->Color;
			auto _ForegroundColor = dynamic_cast<Windows::UI::Xaml::Media::SolidColorBrush^>(_RadioButton->Foreground)->Color;
			if (Utility::IsEqualColor(bgColor, _BackgroundColor) && Utility::IsEqualColor(fgColor, _ForegroundColor))
			{
				auto bIsCheck = _RadioButton->IsChecked->Value;
				if (!bIsCheck)
					_RadioButton->IsChecked = true;
				break;
			}
		}
	}

	/// update the control position
	bool bIsNeedLayoutUpdated = false;
	if (Windows::System::Profile::AnalyticsInfo::VersionInfo->DeviceFamily == L"Windows.Mobile")
	{
		auto pAppView = Windows::UI::ViewManagement::ApplicationView::GetForCurrentView();
		auto _AppOrientation = pAppView->Orientation;
		auto _AppRect = pAppView->VisibleBounds;
		if (_AppOrientation == Windows::UI::ViewManagement::ApplicationViewOrientation::Portrait)
		{
			this->Width = _AppRect.Width;
			bIsNeedLayoutUpdated = true;
		}
	}

	if (bIsNeedLayoutUpdated)
	{
		m_LayoutUpdatedToken = this->LayoutUpdated += ref new Windows::Foundation::EventHandler<Platform::Object ^>([this](Platform::Object^ sender, Platform::Object^ args)
		{
			auto _FlyoutPresenter = GetVisualParent<Windows::UI::Xaml::Controls::FlyoutPresenter^>(this);
			if (_FlyoutPresenter)
			{
				auto _Popup = dynamic_cast<Windows::UI::Xaml::Controls::Primitives::Popup^>(_FlyoutPresenter->Parent);
				if (_Popup)
				{
					_Popup->VerticalOffset += 4;
				}
			}

			this->LayoutUpdated -= m_LayoutUpdatedToken;
			m_LayoutUpdatedToken.Value = 0;
		});
	}
	else
	{
		auto _FlyoutPresenter = GetVisualParent<Windows::UI::Xaml::Controls::FlyoutPresenter^>(this);
		if (_FlyoutPresenter)
		{
			auto _Popup = dynamic_cast<Windows::UI::Xaml::Controls::Primitives::Popup^>(_FlyoutPresenter->Parent);
			if (_Popup)
			{
				_Popup->VerticalOffset = (Windows::System::Profile::AnalyticsInfo::VersionInfo->DeviceFamily == L"Windows.Mobile") ? _Popup->VerticalOffset + 4 : _Popup->VerticalOffset - 4;
			}
		}
	}

	m_bIsLoaded = true;
}


void foxit::rdk::uiextensions::EyeProtectionModeControl::OnUnloaded(Platform::Object ^sender, Windows::UI::Xaml::RoutedEventArgs ^e)
{
	m_bIsLoaded = false;
}


void foxit::rdk::uiextensions::EyeProtectionModeControl::RadioButton_Checked(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{
	if (!m_bIsLoaded)
		return;

	auto _RadioButton = safe_cast<Windows::UI::Xaml::Controls::RadioButton^>(sender);

	if (View)
	{
		auto _BackgroundBrush = dynamic_cast<Windows::UI::Xaml::Media::SolidColorBrush^>(_RadioButton->Background);
		auto _ForegroundBrush = dynamic_cast<Windows::UI::Xaml::Media::SolidColorBrush^>( _RadioButton->Foreground);
		
		View->viewControl->SetViewColor(_BackgroundBrush->Color, _ForegroundBrush->Color);
	}
}


void foxit::rdk::uiextensions::EyeProtectionModeControl::RadioButton_DragStarting(Windows::UI::Xaml::UIElement^ sender, Windows::UI::Xaml::DragStartingEventArgs^ args)
{
	args->Data->SetText(L"");
}


void foxit::rdk::uiextensions::EyeProtectionModeControl::RadioButton_Drop(Platform::Object^ sender, Windows::UI::Xaml::DragEventArgs^ e)
{
	OutputDebugString(L"RadioButton_Drop......\n");
}
