﻿/**
 * Copyright (C) 2003-2018, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit Mobile PDF SDK to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */


#include "pch.h"
#include "NoteStyleUserControl.xaml.h"


using namespace foxit::rdk::uiextensions;
using namespace foxit::rdk;

using namespace Platform;
using namespace Windows::Foundation;
using namespace Windows::Foundation::Collections;
using namespace Windows::UI::Xaml;
using namespace Windows::UI::Xaml::Controls;
using namespace Windows::UI::Xaml::Controls::Primitives;
using namespace Windows::UI::Xaml::Data;
using namespace Windows::UI::Xaml::Input;
using namespace Windows::UI::Xaml::Media;
using namespace Windows::UI::Xaml::Navigation;

// The User Control item template is documented at http://go.microsoft.com/fwlink/?LinkId=234236
namespace foxit
{
	namespace rdk
	{
		namespace uiextensions
		{
			[Windows::Foundation::Metadata::WebHostHidden]
			public ref class ModifyTextAnnotIcon sealed : IUndoItem
			{
			public:
				ModifyTextAnnotIcon(PDFViewCtrl^ view, TextAnnot^ annot);

				virtual void Undo();
				virtual void Redo();
				virtual Platform::String^ GetDescr();
				virtual void ReleaseItem();

			internal:
				property PDFViewCtrl^ View
				{
					PDFViewCtrl^ get();
				}

				property TextAnnot^ AnnotHandle
				{
					TextAnnot^ get();
				}
				void SetIconBeforeModify(AnnotIconNameTextType icon);

			private:
				Platform::WeakReference m_ViewWeakRef;
				Platform::WeakReference m_AnnotWeakRef;

				AnnotIconNameTextType iconTypeOld;
			};
		}
	}
}

ModifyTextAnnotIcon::ModifyTextAnnotIcon(PDFViewCtrl^ view, TextAnnot^ annot)
{
	m_ViewWeakRef = view;
	m_AnnotWeakRef = annot;
}

PDFViewCtrl^ ModifyTextAnnotIcon::View::get()
{
	return m_ViewWeakRef.Resolve<PDFViewCtrl>();
}

TextAnnot^ ModifyTextAnnotIcon::AnnotHandle::get()
{
	return m_AnnotWeakRef.Resolve<TextAnnot>();
}

void ModifyTextAnnotIcon::Undo()
{
	if (View && AnnotHandle)
	{
		AnnotIconNameTextType icon = AnnotHandle->IconType;
		AnnotHandle->IconType = iconTypeOld;
		iconTypeOld = icon;
	}
}

void ModifyTextAnnotIcon::Redo()
{
	if (View && AnnotHandle)
	{
		AnnotIconNameTextType icon = AnnotHandle->IconType;
		AnnotHandle->IconType = iconTypeOld;
		iconTypeOld = icon;
	}
}

Platform::String^ ModifyTextAnnotIcon::GetDescr()
{
	return ref new Platform::String(L"");
}

void ModifyTextAnnotIcon::ReleaseItem()
{
	m_ViewWeakRef = nullptr;
	m_AnnotWeakRef = nullptr;
}

void ModifyTextAnnotIcon::SetIconBeforeModify(AnnotIconNameTextType icon)
{
	iconTypeOld = icon;
}


NoteIcoItem::NoteIcoItem()
{

}

Platform::String^ NoteIcoItem::IconName::get()
{
	return _IconName;
}

void NoteIcoItem::IconName::set(Platform::String^ value)
{
	if (_IconName != value)
	{
		_IconName = value;
		OnPropertyChanged("IconName");
	}
}

Windows::UI::Xaml::Media::ImageSource^  NoteIcoItem::IconBitmap::get()
{
	return _IconBitmap;
}

void NoteIcoItem::IconBitmap::set(Windows::UI::Xaml::Media::ImageSource^ value)
{
	if (_IconBitmap != value)
	{
		_IconBitmap = value;
		OnPropertyChanged("IconBitmap");
	}
}


NoteStyleUserControl::NoteStyleUserControl()
{
	InitializeComponent();

	m_NoteImageVect = ref new Platform::Collections::Vector<NoteIcoItem^>();
	InitNoteImageList();

	if (Windows::System::Profile::AnalyticsInfo::VersionInfo->DeviceFamily == L"Windows.Mobile")
	{
		Windows::Foundation::Rect winRect = Windows::UI::Xaml::Window::Current->Bounds;
		mainGrid->Width = winRect.Width;	
		noteIconGridView->Width = winRect.Width;
		if (winRect.Width > winRect.Height)
		{
			mainGrid->Height = 140;
			ItemsPanelTemplate^ tem = dynamic_cast<ItemsPanelTemplate^>(this->Resources->Lookup("GridViewItemPhoneHorTemplate"));
			noteIconGridView->ItemsPanel = tem;
		}		
	}
	this->Loaded += ref new Windows::UI::Xaml::RoutedEventHandler(this, &foxit::rdk::uiextensions::NoteStyleUserControl::OnLoaded);
	this->Unloaded += ref new Windows::UI::Xaml::RoutedEventHandler(this, &foxit::rdk::uiextensions::NoteStyleUserControl::OnUnloaded);
}

void NoteStyleUserControl::InitNoteImageList()
{
	m_NoteImageVect->Clear();

	Windows::Foundation::Collections::IVector<Platform::String^>^ noteUrlCollections = ref new Platform::Collections::Vector<Platform::String^>();
	noteUrlCollections->Append("NewParagraph");
	noteUrlCollections->Append("Star");
	noteUrlCollections->Append("Comment");
	noteUrlCollections->Append("Note");
	noteUrlCollections->Append("Insert");
	noteUrlCollections->Append("Paragraph");
	noteUrlCollections->Append("Cross");
	noteUrlCollections->Append("Checkmark");
	noteUrlCollections->Append("Circle");
	noteUrlCollections->Append("RightPointer");
	noteUrlCollections->Append("UpLeftArrow");
	noteUrlCollections->Append("UpArrow");
	noteUrlCollections->Append("RightArrow");
	noteUrlCollections->Append("Key");
	noteUrlCollections->Append("Help");

	for (unsigned int i = 0; i < noteUrlCollections->Size; i++)
	{		
		Uri^ url = ref new Uri("ms-appx:///foxit.rdk.uiextensions/Assets/Annot/" + noteUrlCollections->GetAt(i) + ".png");
		if (noteUrlCollections->GetAt(i) == "Note")
		{
			url = ref new Uri("ms-appx:///foxit.rdk.uiextensions/Assets/Annot/" + "note1.png");
		}
		auto bmi = ref new Windows::UI::Xaml::Media::Imaging::BitmapImage(url);
		if (bmi)
		{
			NoteIcoItem^ item = ref new NoteIcoItem();
			item->IconName = noteUrlCollections->GetAt(i);
			item->IconBitmap = bmi;
			
			m_NoteImageVect->Append(item);
		}
	}
	NoteIconCollectionViewSource->Source = m_NoteImageVect;
}


void NoteStyleUserControl::RelateToView(PDFViewCtrl^ View)
{
	m_PageViewWeakRef = View;
}

PDFViewCtrl^ NoteStyleUserControl::View::get()
{
	return m_PageViewWeakRef.Resolve<PDFViewCtrl>();
}

void foxit::rdk::uiextensions::NoteStyleUserControl::StackPanel_Tapped(Platform::Object^ sender, Windows::UI::Xaml::Input::TappedRoutedEventArgs^ e)
{
	StackPanel^ panel = dynamic_cast<StackPanel^>(sender);
	Platform::String^ url = "ms-appx:///foxit.rdk.uiextensions/Assets/Annot/" + panel->Tag->ToString() + ".png";
	UIExtensionsManager^ uiMgr = (UIExtensionsManager^)View->ExtensionsManager;
	if (!uiMgr->getAnnotMgr())
		return;
	IBaseAnnot^ annot = uiMgr->getAnnotMgr()->GetSelectedAnnot();
	if (annot)
	{
		TextAnnot^ textAnnot = dynamic_cast<TextAnnot^>(annot);
		AnnotIconNameTextType icontype = AnnotIconNameTextType::Comment;
		Platform::String^ iconname = panel->Tag->ToString();
		if (iconname == "Checkmark")
		{
			icontype = AnnotIconNameTextType::Checkmark;
		}
		else if (iconname == "Circle")
		{
			icontype = AnnotIconNameTextType::Circle;
		}
		else if (iconname == "Comment")
		{
			icontype = AnnotIconNameTextType::Comment;
		}
		else if (iconname == "Cross")
		{
			icontype = AnnotIconNameTextType::Cross;
		}
		else if (iconname == "Help")
		{
			icontype = AnnotIconNameTextType::Help;
		}
		else if (iconname == "Insert")
		{
			icontype = AnnotIconNameTextType::Insert;
		}
		else if (iconname == "Key")
		{
			icontype = AnnotIconNameTextType::Key;
		}
		else if (iconname == "NewParagraph")
		{
			icontype = AnnotIconNameTextType::NewParagraph;
		}
		else if (iconname == "Note")
		{
			icontype = AnnotIconNameTextType::Note;
		}
		else if (iconname == "Paragraph")
		{
			icontype = AnnotIconNameTextType::Paragraph;
		}
		else if (iconname == "RightArrow")
		{
			icontype = AnnotIconNameTextType::RightArrow;
		}
		else if (iconname == "RightPointer")
		{
			icontype = AnnotIconNameTextType::RightPointer;
		}
		else if (iconname == "Star")
		{
			icontype = AnnotIconNameTextType::Star;
		}
		else if (iconname == "UpArrow")
		{
			icontype = AnnotIconNameTextType::UpArrow;
		}
		else if (iconname == "UpLeftArrow")
		{
			icontype = AnnotIconNameTextType::UpleftArrow;
		}
		textAnnot->IconType = icontype;

		uiMgr->SetNoteStylePageIsActive(false);
		View->Update();

		uiMgr->getAnnotMgr()->GetAnnotSetting()->SetNoteLogoUrl(iconname);
	}
	uiMgr->HideNoteStylePopup();
}


void foxit::rdk::uiextensions::NoteStyleUserControl::listNoteStyle_SelectionChanged(Platform::Object^ sender, Windows::UI::Xaml::Controls::SelectionChangedEventArgs^ e)
{
	
}



void foxit::rdk::uiextensions::NoteStyleUserControl::InitNoteStyleHeight()
{
	
}


void foxit::rdk::uiextensions::NoteStyleUserControl::noteIconGridView_ItemClick(Platform::Object^ sender, Windows::UI::Xaml::Controls::ItemClickEventArgs^ e)
{
	auto item = dynamic_cast<NoteIcoItem^>(e->ClickedItem);
	if (item)
	{
		Platform::String^ iconname = item->IconName;
		UIExtensionsManager^ uiMgr = (UIExtensionsManager^)View->ExtensionsManager;
		uiMgr->getAnnotMgr()->GetAnnotSetting()->SetNoteLogoUrl(iconname);
		IBaseAnnot^ annot = uiMgr->getAnnotMgr()->GetSelectedAnnot();
		if (annot)
		{
			annot->IsModifyProperty = true;
			TextAnnot^ textAnnot = dynamic_cast<TextAnnot^>(annot);
			AnnotIconNameTextType icontype = AnnotIconNameTextType::Comment;
			if (iconname == "Checkmark")
			{
				icontype = AnnotIconNameTextType::Checkmark;
			}
			else if (iconname == "Circle")
			{
				icontype = AnnotIconNameTextType::Circle;
			}
			else if (iconname == "Comment")
			{
				icontype = AnnotIconNameTextType::Comment;
			}
			else if (iconname == "Cross")
			{
				icontype = AnnotIconNameTextType::Cross;
			}
			else if (iconname == "Help")
			{
				icontype = AnnotIconNameTextType::Help;
			}
			else if (iconname == "Insert")
			{
				icontype = AnnotIconNameTextType::Insert;
			}
			else if (iconname == "Key")
			{
				icontype = AnnotIconNameTextType::Key;
			}
			else if (iconname == "NewParagraph")
			{
				icontype = AnnotIconNameTextType::NewParagraph;
			}
			else if (iconname == "Note")
			{
				icontype = AnnotIconNameTextType::Note;
			}
			else if (iconname == "Paragraph")
			{
				icontype = AnnotIconNameTextType::Paragraph;
			}
			else if (iconname == "RightArrow")
			{
				icontype = AnnotIconNameTextType::RightArrow;
			}
			else if (iconname == "RightPointer")
			{
				icontype = AnnotIconNameTextType::RightPointer;
			}
			else if (iconname == "Star")
			{
				icontype = AnnotIconNameTextType::Star;
			}
			else if (iconname == "UpArrow")
			{
				icontype = AnnotIconNameTextType::UpArrow;
			}
			else if (iconname == "UpLeftArrow")
			{
				icontype = AnnotIconNameTextType::UpleftArrow;
			}
			if (textAnnot)
				textAnnot->IconType = icontype;

			uiMgr->SetNoteStylePageIsActive(false);
			View->Update();
		}
		else
		{
			uiMgr->InitCurNoteIcon();
		}		
		uiMgr->HideNoteStylePopup();
	}
}


void foxit::rdk::uiextensions::NoteStyleUserControl::OnLoaded(Platform::Object ^sender, Windows::UI::Xaml::RoutedEventArgs ^e)
{
	__windowSizeChangedEventRegistrationToken = Window::Current->SizeChanged += ref new Windows::UI::Xaml::WindowSizeChangedEventHandler(this, &foxit::rdk::uiextensions::NoteStyleUserControl::OnSizeChanged);
}


void foxit::rdk::uiextensions::NoteStyleUserControl::OnUnloaded(Platform::Object ^sender, Windows::UI::Xaml::RoutedEventArgs ^e)
{
	UIExtensionsManager^ uiMgr = (UIExtensionsManager^)View->ExtensionsManager;
	if (!GetIsDesktopPlatform())
		uiMgr->SetMainAppBarVisible(true);

	if (__windowSizeChangedEventRegistrationToken.Value)
	{
		Window::Current->SizeChanged -= __windowSizeChangedEventRegistrationToken;
		__windowSizeChangedEventRegistrationToken.Value = 0;
	}
}


void foxit::rdk::uiextensions::NoteStyleUserControl::OnSizeChanged(Platform::Object ^sender, Windows::UI::Core::WindowSizeChangedEventArgs ^e)
{
	if (Windows::System::Profile::AnalyticsInfo::VersionInfo->DeviceFamily == L"Windows.Mobile")
	{
		Windows::Foundation::Rect winRect = Windows::UI::Xaml::Window::Current->Bounds;
		mainGrid->Width = winRect.Width;
		noteIconGridView->Width = winRect.Width;
		if (winRect.Width > winRect.Height)
		{
			mainGrid->Height = 140;
			ItemsPanelTemplate^ tem = dynamic_cast<ItemsPanelTemplate^>(this->Resources->Lookup("GridViewItemPhoneHorTemplate"));
			noteIconGridView->ItemsPanel = tem;			
		}
		else
		{
			ItemsPanelTemplate^ tem = dynamic_cast<ItemsPanelTemplate^>(this->Resources->Lookup("GridViewItemPhoneHorTemplatePC"));
			noteIconGridView->ItemsPanel = tem;
			mainGrid->Height = 190;
		}
	}
}
