﻿/**
 * Copyright (C) 2003-2018, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit Mobile PDF SDK to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */


#include "pch.h"
#include "PageNumberControl.xaml.h"
#include <regex>

using namespace foxit::rdk::uiextensions;
using namespace foxit::rdk;

using namespace Platform;
using namespace Windows::Foundation;
using namespace Windows::Foundation::Collections;
using namespace Windows::UI::Xaml;
using namespace Windows::UI::Xaml::Controls;
using namespace Windows::UI::Xaml::Controls::Primitives;
using namespace Windows::UI::Xaml::Data;
using namespace Windows::UI::Xaml::Input;
using namespace Windows::UI::Xaml::Media;
using namespace Windows::UI::Xaml::Navigation;

// The User Control item template is documented at http://go.microsoft.com/fwlink/?LinkId=234236

PageNumberControl::PageNumberControl(UIExtensionsManager^ view)
{
	InitializeComponent();

	m_PageViewWeakRef = view;
}

void foxit::rdk::uiextensions::PageNumberControl::UserControl_Loaded(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{
	__windowSizeChangedEventRegistrationToken = Window::Current->SizeChanged += ref new Windows::UI::Xaml::WindowSizeChangedEventHandler(this, &foxit::rdk::uiextensions::PageNumberControl::OnSizeChanged);

	if (View)
	{
		SetControlLayOut();

		Popup^ m_Popup = safe_cast<Popup^>(this->Parent);
		if (m_Popup && View && GetIsDesktopPlatform())
		{
			auto inputPane = Windows::UI::ViewManagement::InputPane::GetForCurrentView();
			__inputPaneShowingEventRegistrationToken = inputPane->Showing += ref new TypedEventHandler<Windows::UI::ViewManagement::InputPane^, Windows::UI::ViewManagement::InputPaneVisibilityEventArgs^>([=](Windows::UI::ViewManagement::InputPane^ sender, Windows::UI::ViewManagement::InputPaneVisibilityEventArgs^ args)
			{
				args->EnsuredFocusedElementInView = false;

				if (args->OccludedRect.IntersectsWith(Rect((float)m_Popup->HorizontalOffset, (float)m_Popup->VerticalOffset, 320, 150)))
				{
					auto winrc = Windows::UI::Xaml::Window::Current->Bounds;
					auto dbVertocalOffset = m_Popup->VerticalOffset - (234 - (args->OccludedRect.Y - m_Popup->VerticalOffset));
					m_Popup->VerticalOffset = dbVertocalOffset;
					m_Popup->IsLightDismissEnabled = true;
				}
			});
			__inputPaneHidingEventRegistrationToken = inputPane->Hiding += ref new TypedEventHandler<Windows::UI::ViewManagement::InputPane^, Windows::UI::ViewManagement::InputPaneVisibilityEventArgs^>([=](Windows::UI::ViewManagement::InputPane^ sender, Windows::UI::ViewManagement::InputPaneVisibilityEventArgs^ args)
			{
				auto winrc = Windows::UI::Xaml::Window::Current->Bounds;
				auto dbVertocalOffset = winrc.Height / 2 - 234 / 2;
				m_Popup->VerticalOffset = dbVertocalOffset;
			});
		}
	}
}

UIExtensionsManager^ PageNumberControl::View::get()
{
	return m_PageViewWeakRef.Resolve<UIExtensionsManager>();
}


void foxit::rdk::uiextensions::PageNumberControl::btnCancel_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{
	this->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
	View->HideJumpPagePopup();
}


void foxit::rdk::uiextensions::PageNumberControl::btnOk_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{
	JumpToPage();
}

bool PageNumberControl::IsValidString(Platform::String^ str)
{
	return std::regex_match(str->Data(), std::wregex(L"\\d+"));
}

void foxit::rdk::uiextensions::PageNumberControl::UserControl_Unloaded(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{
	auto __inputPane = Windows::UI::ViewManagement::InputPane::GetForCurrentView();
	__inputPane->Showing -= __inputPaneShowingEventRegistrationToken;
	__inputPane->Hiding -= __inputPaneHidingEventRegistrationToken;

	if (__windowSizeChangedEventRegistrationToken.Value)
	{
		Window::Current->SizeChanged -= __windowSizeChangedEventRegistrationToken;
		__windowSizeChangedEventRegistrationToken.Value = 0;
	}
}


void foxit::rdk::uiextensions::PageNumberControl::PageIndicatorText_KeyDown(Platform::Object^ sender, Windows::UI::Xaml::Input::KeyRoutedEventArgs^ e)
{
	auto _virtualKey = e->Key;
	auto _coreWindow = Window::Current->CoreWindow;
	auto _downState = Windows::UI::Core::CoreVirtualKeyStates::Down;
	bool _menuKey = (_coreWindow->GetKeyState(Windows::System::VirtualKey::Menu) & _downState) == _downState;
	bool _controlKey = (_coreWindow->GetKeyState(Windows::System::VirtualKey::Control) & _downState) == _downState;
	bool _shiftKey = (_coreWindow->GetKeyState(Windows::System::VirtualKey::Shift) & _downState) == _downState;
	bool _numberKeyLocktKey = (_coreWindow->GetAsyncKeyState(_virtualKey) & _downState) == _downState;
	bool _noModifiers = !_menuKey && !_controlKey && !_shiftKey;
	bool _onlyAlt = _menuKey && !_controlKey && !_shiftKey;

	if (_noModifiers && _virtualKey == Windows::System::VirtualKey::Enter)
	{
		JumpToPage();
	}
}

void foxit::rdk::uiextensions::PageNumberControl::JumpToPage()
{
	String^ pageNumText = "";
	if (!m_isPhoneHorizontalMode)
	{
		pageNumText = PageIndicatorText->Text;
	}
	else
	{
		pageNumText = PageIndicatorTextPhoneHor->Text;
	}

	if (IsValidString(pageNumText))
	{
		int page = _wtoi(pageNumText->Data());
		if (page > View->viewControl->GetDoc()->GetPageCount())
		{
			page = View->viewControl->GetDoc()->GetPageCount();
		}
		else if (page <= 0)
		{
			page = 1;
		}
		View->JumpPageIndex(page - 1);
		this->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
		View->HideJumpPagePopup();
		auto inputPane = Windows::UI::ViewManagement::InputPane::GetForCurrentView();
		inputPane->TryHide();
	}
	else
	{
		String^ tipStr = View->GetResourceString("JumpPageNumTipStr");

		if (!m_isPhoneHorizontalMode)
		{
			PageIndicatorText->Text = tipStr;
			PageIndicatorText->Foreground = ref new SolidColorBrush(Windows::UI::Colors::Red);
			NonVisibilityBtn->Focus(Windows::UI::Xaml::FocusState::Programmatic);
		}
		else
		{
			PageIndicatorTextPhoneHor->Text = tipStr;
			PageIndicatorTextPhoneHor->Foreground = ref new SolidColorBrush(Windows::UI::Colors::Red);
			NonVisibilityBtn->Focus(Windows::UI::Xaml::FocusState::Programmatic);
		}
	}
}


void foxit::rdk::uiextensions::PageNumberControl::PageIndicatorText_GotFocus(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{
	String^ tipStr = View->GetResourceString("JumpPageNumTipStr");
	if (!m_isPhoneHorizontalMode)
	{
		if (PageIndicatorText->Text == tipStr)
		{
			PageIndicatorText->Text = "";			
		}
		PageIndicatorText->Foreground = ref new SolidColorBrush(Windows::UI::Colors::White);
	}
	else
	{
		if (PageIndicatorTextPhoneHor->Text == tipStr)
		{
			PageIndicatorTextPhoneHor->Text = "";			
		}
		PageIndicatorTextPhoneHor->Foreground = ref new SolidColorBrush(Windows::UI::Colors::White);
	}
}


void foxit::rdk::uiextensions::PageNumberControl::OnSizeChanged(Platform::Object ^sender, Windows::UI::Core::WindowSizeChangedEventArgs ^e)
{
	SetControlLayOut();
}

void foxit::rdk::uiextensions::PageNumberControl::SetControlLayOut()
{
	m_isPhoneHorizontalMode = false;

	if (View == nullptr || View->viewControl->GetDoc() == nullptr)
		return;

	auto _WinRect = Windows::UI::Xaml::Window::Current->Bounds;
	if (View && !GetIsDesktopPlatform())
	{
		mainGrid->Width = _WinRect.Width;
		auto winrc = Windows::UI::Xaml::Window::Current->Bounds;
		mainGrid->Width = winrc.Width;
		if (winrc.Width > winrc.Height)
		{
			m_isPhoneHorizontalMode = true;
			PageNumberGridPhoneHor->Height = _WinRect.Height*0.41;
			btnCancelPhoneHor->Width = _WinRect.Width*0.2;
			btnOkPhoneHor->Width = _WinRect.Width*0.2;
			btnCancelPhoneHor->Height = _WinRect.Height*0.075;
			btnOkPhoneHor->Height = _WinRect.Height*0.075;
			PageIndicatorTextPhoneHor->Width = _WinRect.Width*0.88;
			PageIndicatorTextPhoneHor->Height = _WinRect.Height*0.075;
		}
		else
		{
			btnCancel->Width = winrc.Width*0.43;
			btnOk->Width = winrc.Width*0.43;
		}
	}

	if (!m_isPhoneHorizontalMode)
	{
		PageNumberGrid->Visibility = Windows::UI::Xaml::Visibility::Visible;
		PageNumberGridPhoneHor->Visibility = Windows::UI::Xaml::Visibility::Collapsed;

		PageIndicatorText->Text = (View->GetCurrentPageIndex() + 1).ToString();
		PageIndicatorText->SelectionStart = PageIndicatorText->Text->Length();

		String^ value = "(1-" + View->viewControl->GetDoc()->GetPageCount().ToString() + ")";
		tbNumberSnap->Text = value;

		tbNumberSnap->Focus(Windows::UI::Xaml::FocusState::Programmatic);
	}
	else
	{
		PageNumberGrid->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
		PageNumberGridPhoneHor->Visibility = Windows::UI::Xaml::Visibility::Visible;

		PageIndicatorTextPhoneHor->Text = (View->GetCurrentPageIndex() + 1).ToString();
		PageIndicatorTextPhoneHor->SelectionStart = PageIndicatorTextPhoneHor->Text->Length();

		String^ value = "(1-" + View->viewControl->GetDoc()->GetPageCount().ToString() + ")";
		tbNumberSnapPhoneHor->Text = value;

		tbNumberSnapPhoneHor->Focus(Windows::UI::Xaml::FocusState::Programmatic);

		PageIndicatorTextPhoneHor->Width = _WinRect.Width*0.65;
		btnCancelPhoneHor->Width = _WinRect.Width*0.2;
		btnOkPhoneHor->Width = _WinRect.Width*0.2;
	}
	FocusNumberTextBox();
}

void foxit::rdk::uiextensions::PageNumberControl::FocusNumberTextBox()
{	
	if (PageNumberGrid->Visibility == Windows::UI::Xaml::Visibility::Visible)
	{
		if (PageIndicatorText->FocusState == Windows::UI::Xaml::FocusState::Unfocused)
			PageIndicatorText->Focus(Windows::UI::Xaml::FocusState::Programmatic);
	}
	else if(PageNumberGridPhoneHor->Visibility == Windows::UI::Xaml::Visibility::Visible)
	{
		if (PageIndicatorTextPhoneHor->FocusState == Windows::UI::Xaml::FocusState::Unfocused)
			PageIndicatorTextPhoneHor->Focus(Windows::UI::Xaml::FocusState::Programmatic);		
	}
}
