﻿/**
 * Copyright (C) 2003-2018, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit Mobile PDF SDK to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */


#include "pch.h"
#include "PageNumberShowControl.xaml.h"

using namespace foxit::rdk::uiextensions;
using namespace foxit::rdk;

using namespace Platform;
using namespace Windows::Foundation;
using namespace Windows::Foundation::Collections;
using namespace Windows::UI::Xaml;
using namespace Windows::UI::Xaml::Controls;
using namespace Windows::UI::Xaml::Controls::Primitives;
using namespace Windows::UI::Xaml::Data;
using namespace Windows::UI::Xaml::Input;
using namespace Windows::UI::Xaml::Media;
using namespace Windows::UI::Xaml::Navigation;

// The User Control item template is documented at http://go.microsoft.com/fwlink/?LinkId=234236

PageNumberShowControl::PageNumberShowControl(UIExtensionsManager^ view)
{
	InitializeComponent();

	m_PageViewWeakRef = view;

	this->Loaded += ref new Windows::UI::Xaml::RoutedEventHandler(this, &foxit::rdk::uiextensions::PageNumberShowControl::OnLoaded);
	this->Unloaded += ref new Windows::UI::Xaml::RoutedEventHandler(this, &foxit::rdk::uiextensions::PageNumberShowControl::OnUnloaded);

	InitHideAnimation();
}

void PageNumberShowControl::InitHideAnimation()
{
	m_HideTimer = ref new Windows::UI::Xaml::DispatcherTimer();
	Windows::Foundation::TimeSpan _HideTimerTimeSpan;
	_HideTimerTimeSpan.Duration = 5000000; 
	m_HideTimer->Interval = _HideTimerTimeSpan;
	m_HideTimer->Tick += ref new Windows::Foundation::EventHandler<Platform::Object ^>(this, &foxit::rdk::uiextensions::PageNumberShowControl::OnTick);
}

UIExtensionsManager^ PageNumberShowControl::View::get()
{
	return m_PageViewWeakRef.Resolve<UIExtensionsManager>();
}


void foxit::rdk::uiextensions::PageNumberShowControl::OnLoaded(Platform::Object ^sender, Windows::UI::Xaml::RoutedEventArgs ^e)
{
	if (View)
	{
		tbPageCount->Text = View->viewControl->GetDoc()->GetPageCount().ToString();
		tbPageNumber->Text = (View->GetCurrentPageIndex() + 1).ToString();
	}
}


void foxit::rdk::uiextensions::PageNumberShowControl::OnUnloaded(Platform::Object ^sender, Windows::UI::Xaml::RoutedEventArgs ^e)
{
	m_PageViewWeakRef = nullptr;
	if (m_HideTimer->IsEnabled)
		m_HideTimer->Stop();
}

void foxit::rdk::uiextensions::PageNumberShowControl::InitPageCurIndex(int pageIndex)
{
	tbPageNumber->Text = pageIndex.ToString();
}

void foxit::rdk::uiextensions::PageNumberShowControl::OnTick(Platform::Object ^sender, Platform::Object ^args)
{
	OutputDebugString(L" PageNumberShowControl OnTick\n");
	if (m_HideTimer)
	{
		m_HideTimer->Stop();
	}

	auto hideStoryboard = safe_cast<Windows::UI::Xaml::Media::Animation::Storyboard^>(this->Resources->Lookup("NumberGridHideStoryboard"));
	if (hideStoryboard)
		hideStoryboard->Begin();

	if (View)
		View->HidePageNumberCountPopup();
}

void foxit::rdk::uiextensions::PageNumberShowControl::Activate()
{
	OutputDebugString(L" PageNumberShowControl Activate\n");
	this->Visibility = Windows::UI::Xaml::Visibility::Visible;
	if (m_HideTimer->IsEnabled)
		m_HideTimer->Stop();
	m_HideTimer->Start();
}
