﻿/**
 * Copyright (C) 2003-2018, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit Mobile PDF SDK to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */


#include "pch.h"
#include "PageSlideHorViewControl.xaml.h"

using namespace foxit::rdk::uiextensions;

using namespace Platform;
using namespace Windows::Foundation;
using namespace Windows::Foundation::Collections;
using namespace Windows::UI::Xaml;
using namespace Windows::UI::Xaml::Controls;
using namespace Windows::UI::Xaml::Controls::Primitives;
using namespace Windows::UI::Xaml::Data;
using namespace Windows::UI::Xaml::Input;
using namespace Windows::UI::Xaml::Media;
using namespace Windows::UI::Xaml::Navigation;

// The User Control item template is documented at http://go.microsoft.com/fwlink/?LinkId=234236

PageSlideHorViewControl::PageSlideHorViewControl()
{
	InitializeComponent();
}


void foxit::rdk::uiextensions::PageSlideHorViewControl::Grid_Tapped(Platform::Object^ sender, Windows::UI::Xaml::Input::TappedRoutedEventArgs^ e)
{
	OutputDebugString(L"PageSlideHorViewControl Tapped\n");

	if (!View)
		return;
	bool isDesktopPlatform = true;
	if (Windows::System::Profile::AnalyticsInfo::VersionInfo->DeviceFamily == "Windows.Desktop")
	{
		isDesktopPlatform = true;
	}
	else
	{
		isDesktopPlatform = false;
	}

	if (View && isDesktopPlatform)
	{
		View->ShowPageNumberJumpWindow();
	}
	else
	{
		if (View && View->GetIsCanScrollPageView())
			View->ShowPageNumberJumpWindow();
	}
}

void foxit::rdk::uiextensions::PageSlideHorViewControl::InitPageNumText(int pageIndex)
{
	m_pageIndex = pageIndex;
	PageNum->Text = (pageIndex + 1).ToString();
	m_LayoutGridTransform->TranslateX = m_pageIndex*m_stepOffset;
}

void foxit::rdk::uiextensions::PageSlideHorViewControl::RelatePageView(UIExtensionsManager^ view)
{
	m_PageViewWeakRef = view;
	m_pageCount = view->viewControl->GetDoc()->GetPageCount();
	m_stepOffset = m_SliderViewWidth / m_pageCount;
}

void foxit::rdk::uiextensions::PageSlideHorViewControl::CalculateStepOffset()
{
	if (View)
	{
		auto _winrect = Windows::UI::Xaml::Window::Current->Bounds;
		m_SliderViewWidth = _winrect.Width - m_VerticalOffsetLeft - m_VerticalOffsetRight + m_controlWidth;
		m_pageCount = View->viewControl->GetDoc()->GetPageCount();
		m_stepOffset = m_SliderViewWidth / m_pageCount;
		InitPageNumText(View->viewControl->GetCurrentPage());
	}
}

UIExtensionsManager^ PageSlideHorViewControl::View::get()
{
	return m_PageViewWeakRef.Resolve<UIExtensionsManager>();
}


void foxit::rdk::uiextensions::PageSlideHorViewControl::UserControl_Loaded(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{
	m_controlWidth = 37;

	m_VerticalOffsetLeft = 100;
	m_VerticalOffsetRight = 100;
	m_stepOffset = 0;
	auto _winrect = Windows::UI::Xaml::Window::Current->Bounds;
	m_SliderViewWidth = _winrect.Width - m_VerticalOffsetLeft - m_VerticalOffsetRight + m_controlWidth;

	this->ManipulationStarted += ref new Windows::UI::Xaml::Input::ManipulationStartedEventHandler(this, &foxit::rdk::uiextensions::PageSlideHorViewControl::OnManipulationStarted);
	this->ManipulationDelta += ref new Windows::UI::Xaml::Input::ManipulationDeltaEventHandler(this, &foxit::rdk::uiextensions::PageSlideHorViewControl::OnManipulationDelta);
	this->ManipulationCompleted += ref new Windows::UI::Xaml::Input::ManipulationCompletedEventHandler(this, &foxit::rdk::uiextensions::PageSlideHorViewControl::OnManipulationCompleted);
	this->ManipulationMode = ::ManipulationModes::All;
	this->RenderTransformOrigin = Point(0.5, 0.5);
	this->RenderTransform = m_LayoutGridTransform = ref new CompositeTransform();
	m_LayoutGridTransform->TranslateX = 0;
	m_LayoutGridTransform->TranslateY = 0;
}


void foxit::rdk::uiextensions::PageSlideHorViewControl::OnManipulationDelta(Platform::Object ^sender, Windows::UI::Xaml::Input::ManipulationDeltaRoutedEventArgs ^e)
{
	OutputDebugString(L"PageSlideHorViewControl OnManipulationDelta\n");
	if (View && !View->GetIsCanScrollPageView())
	{
		return;
	}

	bool isNeedGoTOPage = false;
	double transformY = m_LayoutGridTransform->TranslateX;

	double translateX = m_LayoutGridTransform->TranslateX + e->Delta.Translation.X;
	if (translateX < 0)
	{
		m_LayoutGridTransform->TranslateX = 0;
		m_pageIndex = 0;
		PageNum->Text = (m_pageIndex + 1).ToString();
		if (View)
		{
			View->viewControl->GotoPage(m_pageIndex);
			View->ShowPageNumberCountPopup(false);
		}
		return;
	}
	if (translateX > m_SliderViewWidth)
	{
		m_pageIndex = m_pageCount - 1;
		m_LayoutGridTransform->TranslateX = m_SliderViewWidth;

		PageNum->Text = (m_pageIndex + 1).ToString();
		if (View)
		{
			View->viewControl->GotoPage(m_pageIndex);
			View->ShowPageNumberCountPopup(false);
		}
		return;
	}
	m_LayoutGridTransform->TranslateX += e->Delta.Translation.X;

	double xValue = e->Delta.Translation.X;
	int stepSize = (int)abs(e->Delta.Translation.X / m_stepOffset);


	if (e->Delta.Translation.X > 0)
	{
		m_pageIndex = m_pageIndex + stepSize;
	}
	else if (e->Delta.Translation.X < 0)
	{
		m_pageIndex = m_pageIndex - stepSize;
	}

	if (e->Delta.Translation.X > 0 && m_LayoutGridTransform->TranslateX >= (m_pageIndex + 1) * m_stepOffset)
	{
		m_pageIndex++;
		if (m_pageIndex > m_pageCount - 1)
			m_pageIndex = m_pageCount - 1;

		isNeedGoTOPage = true;
	}
	else if (e->Delta.Translation.X < 0 && m_LayoutGridTransform->TranslateX <= m_pageIndex * m_stepOffset)
	{
		m_pageIndex--;

		if (m_pageIndex < 0)
			m_pageIndex = 0;

		isNeedGoTOPage = true;
	}

	if (isNeedGoTOPage)
	{
		PageNum->Text = (m_pageIndex + 1).ToString();
		if (View)
		{
			View->viewControl->GotoPage(m_pageIndex);
		}
	}
	if (View)
		View->ShowPageNumberCountPopup(false);
}


void foxit::rdk::uiextensions::PageSlideHorViewControl::OnManipulationCompleted(Platform::Object ^sender, Windows::UI::Xaml::Input::ManipulationCompletedRoutedEventArgs ^e)
{
	if (View && !View->GetIsCanScrollPageView())
	{
		return;
	}
	if (m_pageIndex == m_pageCount - 1)
		m_LayoutGridTransform->TranslateX = m_SliderViewWidth;
	else if (m_pageIndex == 0)
		m_LayoutGridTransform->TranslateX = 0;

	if (View)
	{
		OutputDebugString(L"PageSlideHorViewControl OnManipulationCompleted\n");
		View->HidePageNumberCountPopup();
		View->SetAppBarHideTimer(false);
	}
}


void foxit::rdk::uiextensions::PageSlideHorViewControl::OnManipulationStarted(Platform::Object ^sender, Windows::UI::Xaml::Input::ManipulationStartedRoutedEventArgs ^e)
{
	if (View && !View->GetIsCanScrollPageView())
	{
		return;
	}
	if (View)
	{
		OutputDebugString(L"PageSlideHorViewControl OnManipulationStarted\n");
		View->ShowPageNumberCountPopup(false);
		View->SetAppBarHideTimer(true);

		View->CreateGraphAnnotBySliderPage();
		View->SetAnnotCanvasVisible(false);
	}
}
