﻿/**
 * Copyright (C) 2003-2018, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit Mobile PDF SDK to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */


#include "pch.h"
#include "PageSliderViewControl.xaml.h"

using namespace foxit::rdk::uiextensions;

using namespace Platform;
using namespace Windows::Foundation;
using namespace Windows::Foundation::Collections;
using namespace Windows::UI::Xaml;
using namespace Windows::UI::Xaml::Controls;
using namespace Windows::UI::Xaml::Controls::Primitives;
using namespace Windows::UI::Xaml::Data;
using namespace Windows::UI::Xaml::Input;
using namespace Windows::UI::Xaml::Media;
using namespace Windows::UI::Xaml::Navigation;

// The User Control item template is documented at http://go.microsoft.com/fwlink/?LinkId=234236

PageSliderViewControl::PageSliderViewControl()
{
	InitializeComponent();
}

void foxit::rdk::uiextensions::PageSliderViewControl::UserControl_Loaded(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{
	m_controlHeight = 37;
	m_AppBarHeight = 60;

	m_VerticalOffsetUp = 100;
	m_VerticalOffsetLower = 100;
	m_stepOffset = 0;
	auto _winrect = Windows::UI::Xaml::Window::Current->Bounds;
	m_SliderViewHeight = _winrect.Height - m_VerticalOffsetLower - m_VerticalOffsetUp+ m_controlHeight;

	this->ManipulationStarted += ref new Windows::UI::Xaml::Input::ManipulationStartedEventHandler(this, &foxit::rdk::uiextensions::PageSliderViewControl::OnManipulationStarted);
	this->ManipulationDelta += ref new Windows::UI::Xaml::Input::ManipulationDeltaEventHandler(this, &foxit::rdk::uiextensions::PageSliderViewControl::OnManipulationDelta);
	this->ManipulationCompleted += ref new Windows::UI::Xaml::Input::ManipulationCompletedEventHandler(this, &foxit::rdk::uiextensions::PageSliderViewControl::OnManipulationCompleted);
	this->ManipulationMode = ::ManipulationModes::All;
	this->RenderTransformOrigin = Point(0.5, 0.5);
	this->RenderTransform = m_LayoutGridTransform = ref new CompositeTransform();
	m_LayoutGridTransform->TranslateX = 0;
	m_LayoutGridTransform->TranslateY = 0;
}

void foxit::rdk::uiextensions::PageSliderViewControl::OnManipulationDelta(Platform::Object ^sender, Windows::UI::Xaml::Input::ManipulationDeltaRoutedEventArgs ^e)
{
	if (View && !View->GetIsCanScrollPageView())
	{
		return;
	}

	bool isNeedGoTOPage = false;
	double transformY = m_LayoutGridTransform->TranslateY;

	double translateY = m_LayoutGridTransform->TranslateY + e->Delta.Translation.Y;
	if (translateY < 0)
	{
		m_LayoutGridTransform->TranslateY = 0;
		m_pageIndex = 0;
		PageNum->Text = (m_pageIndex + 1).ToString();
		if (View)
		{
			View->viewControl->GotoPage(m_pageIndex);
			View->ShowPageNumberCountPopup(false);
		}
		return;
	}
	if (translateY > m_SliderViewHeight)
	{
		auto _winrect = Windows::UI::Xaml::Window::Current->Bounds;
		m_LayoutGridTransform->TranslateY = m_SliderViewHeight;
		m_pageIndex = m_pageCount - 1;
	    PageNum->Text = (m_pageIndex + 1).ToString();
		if (View)
		{
			View->viewControl->GotoPage(m_pageIndex);
			View->ShowPageNumberCountPopup(false);
		}
		return;
	}
	m_LayoutGridTransform->TranslateY += e->Delta.Translation.Y;

	double yValue = e->Delta.Translation.Y;
	int stepSize = (int)abs(e->Delta.Translation.Y / m_stepOffset);


	if (e->Delta.Translation.Y > 0)
	{
		m_pageIndex = m_pageIndex + stepSize;
	}
	else if (e->Delta.Translation.Y < 0)
	{
		m_pageIndex = m_pageIndex - stepSize;
	}
	if (e->Delta.Translation.Y > 0 && m_LayoutGridTransform->TranslateY >= (m_pageIndex + 1) * m_stepOffset)
	{
		m_pageIndex++;
		if (m_pageIndex > m_pageCount - 1)
			m_pageIndex = m_pageCount - 1;

		isNeedGoTOPage = true;
	}
	else if (e->Delta.Translation.Y < 0 && m_LayoutGridTransform->TranslateY <= m_pageIndex * m_stepOffset)
	{
		m_pageIndex--;

		if (m_pageIndex < 0)
			m_pageIndex = 0;

		isNeedGoTOPage = true;
	}

	if (isNeedGoTOPage)
	{
		PageNum->Text = (m_pageIndex + 1).ToString();
		if (View)
		{
			View->viewControl->GotoPage(m_pageIndex);
		}
	}
	if (View)
		View->ShowPageNumberCountPopup(false);
}

void foxit::rdk::uiextensions::PageSliderViewControl::InitPageNumText(int pageIndex)
{
	m_pageIndex = pageIndex;
	PageNum->Text = (pageIndex + 1).ToString();
	m_LayoutGridTransform->TranslateY = m_pageIndex*m_stepOffset;
}

void foxit::rdk::uiextensions::PageSliderViewControl::RelatePageView(UIExtensionsManager^ view)
{
	m_PageViewWeakRef = view;
	m_pageCount = view->viewControl->GetDoc()->GetPageCount();
	m_stepOffset = m_SliderViewHeight / m_pageCount;
}

void foxit::rdk::uiextensions::PageSliderViewControl::CalculateStepOffset()
{
	if (View)
	{
		auto _winrect = Windows::UI::Xaml::Window::Current->Bounds;
		m_SliderViewHeight = _winrect.Height  - m_VerticalOffsetLower - m_VerticalOffsetUp + m_controlHeight;
		m_pageCount = View->viewControl->GetDoc()->GetPageCount();
		m_stepOffset = m_SliderViewHeight / m_pageCount;
		InitPageNumText(View->viewControl->GetCurrentPage());
	}
}

UIExtensionsManager^ PageSliderViewControl::View::get()
{
	return m_PageViewWeakRef.Resolve<UIExtensionsManager>();
}

void foxit::rdk::uiextensions::PageSliderViewControl::OnManipulationCompleted(Platform::Object ^sender, Windows::UI::Xaml::Input::ManipulationCompletedRoutedEventArgs ^e)
{
	if (View && !View->GetIsCanScrollPageView())
	{
		View->HidePageNumberCountPopup();
		return;
	}

	if (m_pageIndex == m_pageCount - 1)
		m_LayoutGridTransform->TranslateY = m_SliderViewHeight;
	else if (m_pageIndex == 0)
		m_LayoutGridTransform->TranslateY = 0;

	if (View)
	{
		View->HidePageNumberCountPopup();
		View->SetAppBarHideTimer(false);
	}
}


void foxit::rdk::uiextensions::PageSliderViewControl::Grid_Tapped(Platform::Object^ sender, Windows::UI::Xaml::Input::TappedRoutedEventArgs^ e)
{
	if (!View)
		return;

	if (View && GetIsDesktopPlatform())
	{
		View->ShowPageNumberJumpWindow();
	}
	else
	{
		if (View)
		{
			if (View->AnnotMgr->GetSelectedAnnot()
				&& View->AnnotMgr->GetSelectedAnnot()->AnnotType == AnnotationType::FreeText && View->GetAnnotCanvasHasChild())
			{
				View->ShowPageNumberJumpWindow();
			}
			else
			{
				if (View->GetIsCanScrollPageView())
				{
					View->ShowPageNumberJumpWindow();
				}
			}
		}
	}
}


void foxit::rdk::uiextensions::PageSliderViewControl::OnManipulationStarted(Platform::Object ^sender, Windows::UI::Xaml::Input::ManipulationStartedRoutedEventArgs ^e)
{
	if (View && !View->GetIsCanScrollPageView())
	{
		e->Complete();
		return;
	}
	if (View)
	{
		View->ShowPageNumberCountPopup(false);

		View->CreateGraphAnnotBySliderPage();
		View->SetAnnotCanvasVisible(false);

		View->SetAppBarHideTimer(true);
	}
}
