﻿/**
 * Copyright (C) 2003-2018, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit Mobile PDF SDK to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */


#include "pch.h"
#include "ReflowViewAppBar.xaml.h"
#include "MainPageAppBar.xaml.h"


using namespace foxit::rdk::uiextensions;

using namespace Platform;
using namespace Windows::Foundation;
using namespace Windows::Foundation::Collections;
using namespace Windows::UI::Xaml;
using namespace Windows::UI::Xaml::Controls;
using namespace Windows::UI::Xaml::Controls::Primitives;
using namespace Windows::UI::Xaml::Data;
using namespace Windows::UI::Xaml::Input;
using namespace Windows::UI::Xaml::Media;
using namespace Windows::UI::Xaml::Navigation;

// The User Control item template is documented at http://go.microsoft.com/fwlink/?LinkId=234236

ReflowViewAppBar::ReflowViewAppBar()
{
	InitializeComponent();

	OnInitializeControl();
}

MainPageAppBar^ ReflowViewAppBar::MainAppBar::get()
{
	return m_MainAppBarWeakRef.Resolve<MainPageAppBar>();
}

bool ReflowViewAppBar::IsMobilePlatform::get()
{
	return m_IsMobilePlatform;
}

void ReflowViewAppBar::OnInitializeControl()
{
	m_IsMobilePlatform = (Windows::System::Profile::AnalyticsInfo::VersionInfo->DeviceFamily == L"Windows.Mobile") ? true : false;

	this->Loaded += ref new Windows::UI::Xaml::RoutedEventHandler(this, &foxit::rdk::uiextensions::ReflowViewAppBar::OnLoaded);
	this->Unloaded += ref new Windows::UI::Xaml::RoutedEventHandler(this, &foxit::rdk::uiextensions::ReflowViewAppBar::OnUnloaded);
}

void ReflowViewAppBar::UpdateControlState()
{
	if (MainAppBar)
	{
		auto pControl = foxit::rdk::uiextensions::GetVisualChild<foxit::rdk::FoxitViewControl^>(MainAppBar->View);
		
		auto _ReflowViewer = dynamic_cast<PDFReflowPagesViewer^>(pControl->PageViewMgr->Viewer);
		if (_ReflowViewer)
		{
			foxit::pdf::PDFDoc ^ pDoc = MainAppBar->View->GetDoc();
			auto bIsHaveBookmark = pDoc->GetRootBookmark() != nullptr && !pDoc->GetRootBookmark()->IsEmpty();
			OutlineBtn->Visibility = bIsHaveBookmark ? Windows::UI::Xaml::Visibility::Visible : Windows::UI::Xaml::Visibility::Collapsed;
			OutlineWpBtn->Visibility = bIsHaveBookmark ? Windows::UI::Xaml::Visibility::Visible : Windows::UI::Xaml::Visibility::Collapsed;
			ShowPageBtn->Visibility = _ReflowViewer->IsShowPicture ? Windows::UI::Xaml::Visibility::Collapsed : Windows::UI::Xaml::Visibility::Visible;
			ShowPageWpBtn->Visibility = _ReflowViewer->IsShowPicture ? Windows::UI::Xaml::Visibility::Collapsed : Windows::UI::Xaml::Visibility::Visible;
			NotShowPageBtn->Visibility = _ReflowViewer->IsShowPicture ? Windows::UI::Xaml::Visibility::Visible : Windows::UI::Xaml::Visibility::Collapsed;
			NotShowPageWpBtn->Visibility = _ReflowViewer->IsShowPicture ? Windows::UI::Xaml::Visibility::Visible : Windows::UI::Xaml::Visibility::Collapsed;			
			
			auto nPages = MainAppBar->View->GetVisiblePages();
			if (nPages->Size == 0)
			{
				if (IsMobilePlatform)
				{
					IncreaseFontSizeWpBtn->IsEnabled = false;
					ReduceFontSizeWpBtn->IsEnabled = false;
					PrevPageWpBtn->IsEnabled = false;
					NextPageWpBtn->IsEnabled = false;
				}
				else
				{
					IncreaseFontSizeBtn->IsEnabled = false;
					ReduceFontSizeBtn->IsEnabled = false;
					PrevPageBtn->IsEnabled = false;
					NextPageBtn->IsEnabled = false;
				}
				return;
			}
		}
	}

	UpdateViewFontSizeBtnState();
	UpdateFlipPageBtnState();
}

void ReflowViewAppBar::UpdateViewFontSizeBtnState()
{
	if (MainAppBar)
	{
		auto pControl = foxit::rdk::uiextensions::GetVisualChild<foxit::rdk::FoxitViewControl^>(MainAppBar->View);

		auto _ReflowViewer = dynamic_cast<PDFReflowPagesViewer^>(pControl->PageViewMgr->Viewer);
		if (_ReflowViewer)
		{
			auto _minScale = _ReflowViewer->GetMinScale();
			auto _maxScale = _ReflowViewer->GetMaxScale();
			auto _Scale = _ReflowViewer->GetScale();

			if (_minScale == _Scale)
			{
				if (IsMobilePlatform)
				{
					IncreaseFontSizeWpBtn->IsEnabled = true;
					ReduceFontSizeWpBtn->IsEnabled = false;
				}
				else
				{
					IncreaseFontSizeBtn->IsEnabled = true;
					ReduceFontSizeBtn->IsEnabled = false;
				}
			}
			else if (_maxScale == _Scale)
			{
				if (IsMobilePlatform)
				{
					IncreaseFontSizeWpBtn->IsEnabled = false;
					ReduceFontSizeWpBtn->IsEnabled = true;
				}
				else
				{
					IncreaseFontSizeBtn->IsEnabled = false;
					ReduceFontSizeBtn->IsEnabled = true;
				}
			}
			else
			{
				if (IsMobilePlatform)
				{
					IncreaseFontSizeWpBtn->IsEnabled = true;
					ReduceFontSizeWpBtn->IsEnabled = true;
				}
				else
				{
					IncreaseFontSizeBtn->IsEnabled = true;
					ReduceFontSizeBtn->IsEnabled = true;
				}
			}
		}
	}
}

void ReflowViewAppBar::UpdateFlipPageBtnState()
{
	if (MainAppBar)
	{
		auto pControl = foxit::rdk::uiextensions::GetVisualChild<foxit::rdk::FoxitViewControl^>(MainAppBar->View);

		auto _ReflowViewer = dynamic_cast<PDFReflowPagesViewer^>(pControl->PageViewMgr->Viewer);
		if (_ReflowViewer)
		{
			auto _bIsCanFlipUp = _ReflowViewer->IsCanFlipUpScreen();
			auto _bIsCanFlipDown = _ReflowViewer->IsCanFlipDownScreen();

			if (IsMobilePlatform)
			{
				PrevPageWpBtn->IsEnabled = _bIsCanFlipUp;
				NextPageWpBtn->IsEnabled = _bIsCanFlipDown;
			}
			else
			{
				PrevPageBtn->IsEnabled = _bIsCanFlipUp;
				NextPageBtn->IsEnabled = _bIsCanFlipDown;
			}
		}
	}
}

void ReflowViewAppBar::SetAppBarIsCompact(bool isCompact)
{
	if (IsMobilePlatform)
	{
		WindowsMobileCommandBar->IsOpen = false;
	}
	else
	{
		WindowsDesktopCommandBar->IsOpen = false;
	}
}

void foxit::rdk::uiextensions::ReflowViewAppBar::OnLoaded(Platform::Object ^sender, Windows::UI::Xaml::RoutedEventArgs ^e)
{
	auto _mainAppbar = foxit::rdk::uiextensions::GetVisualParent<MainPageAppBar^>(this);
	if (_mainAppbar)
		m_MainAppBarWeakRef = _mainAppbar;
	
	UpdateControlState();
}


void foxit::rdk::uiextensions::ReflowViewAppBar::OnUnloaded(Platform::Object ^sender, Windows::UI::Xaml::RoutedEventArgs ^e)
{
	m_MainAppBarWeakRef = nullptr;
}

void foxit::rdk::uiextensions::ReflowViewAppBar::IncreaseFontSizeBtn_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{
	if (MainAppBar)
	{
		auto _ViewMgr = MainAppBar->View;
		double m_scale = _ViewMgr->GetScale();
		double _dbScale = _ViewMgr->GetMaxScale();
		if (m_scale == _dbScale)
		{
			UpdateViewFontSizeBtnState();
			return;
		}

		m_scale = m_scale + 0.5;
		if (m_scale > _dbScale)
			m_scale = _dbScale;

		_ViewMgr->ZoomTo(Point(0,0), m_scale, false, true);

		UpdateViewFontSizeBtnState();
	}
}


void foxit::rdk::uiextensions::ReflowViewAppBar::ReduceFontSizeBtn_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{
	if (MainAppBar)
	{
		auto _ViewMgr = MainAppBar->View;
		double m_scale = _ViewMgr->GetScale();
		double _dbScale = _ViewMgr->GetMinScale();
		if (m_scale == _dbScale)
		{
			UpdateViewFontSizeBtnState();
			return;
		}

		m_scale = m_scale - 0.5;
		if (m_scale < _dbScale)
			m_scale = _dbScale;

		_ViewMgr->ZoomTo(Point(0, 0), m_scale, false, true);

		UpdateViewFontSizeBtnState();
	}
}


void foxit::rdk::uiextensions::ReflowViewAppBar::PrevPageBtn_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{
	if (MainAppBar)
	{
		auto pControl = foxit::rdk::uiextensions::GetVisualChild<foxit::rdk::FoxitViewControl^>(MainAppBar->View);

		auto _ReflowViewer = dynamic_cast<PDFReflowPagesViewer^>(pControl->PageViewMgr->Viewer);
		if (_ReflowViewer)
		{
			_ReflowViewer->PreviousScreen();

			auto _curIndex = MainAppBar->View->GetCurrentPage();

			UpdateFlipPageBtnState();
		}
	}
}


void foxit::rdk::uiextensions::ReflowViewAppBar::NextPageBtn_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{
	if (MainAppBar)
	{
		auto pControl = foxit::rdk::uiextensions::GetVisualChild<foxit::rdk::FoxitViewControl^>(MainAppBar->View);

		auto _ReflowViewer = dynamic_cast<PDFReflowPagesViewer^>(pControl->PageViewMgr->Viewer);
		if (_ReflowViewer)
		{
			_ReflowViewer->NextScreen();
			UpdateFlipPageBtnState();
		}
	}
}


void foxit::rdk::uiextensions::ReflowViewAppBar::OutlineBtn_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{
	auto _mainAppbar = foxit::rdk::uiextensions::GetVisualParent<MainPageAppBar^>(this);
	if (_mainAppbar)
	{
		_mainAppbar->ShowBookmarkPage();
	}
}


void foxit::rdk::uiextensions::ReflowViewAppBar::ShowPageBtn_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{
	if (MainAppBar)
	{
		auto pControl = foxit::rdk::uiextensions::GetVisualChild<foxit::rdk::FoxitViewControl^>(MainAppBar->View);

		auto _ReflowViewer = dynamic_cast<PDFReflowPagesViewer^>(pControl->PageViewMgr->Viewer);
		if (_ReflowViewer)
		{
			if (!_ReflowViewer->IsShowPicture)
			{
				_ReflowViewer->IsShowPicture = true;
				UpdateControlState();
			}
		}
	}
}


void foxit::rdk::uiextensions::ReflowViewAppBar::NotShowPageBtn_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{
	if (MainAppBar)
	{
		auto pControl = foxit::rdk::uiextensions::GetVisualChild<foxit::rdk::FoxitViewControl^>(MainAppBar->View);

		auto _ReflowViewer = dynamic_cast<PDFReflowPagesViewer^>(pControl->PageViewMgr->Viewer);
		if (_ReflowViewer)
		{
			if (_ReflowViewer->IsShowPicture)
			{
				_ReflowViewer->IsShowPicture = false;
				UpdateControlState();
			}
		}
	}
}


void foxit::rdk::uiextensions::ReflowViewAppBar::ExitAppBarBtn_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{
	if (MainAppBar)
	{
		MainAppBar->UIExtMgr->ResetDefaultAppBarButton();
		if (!MainAppBar->UIExtMgr->LoadPrevViewMode())
			MainAppBar->UIExtMgr->LoadViewByViewMode(foxit::rdk::LayoutMode::ContinueMode);
	}
}

