﻿/**
 * Copyright (C) 2003-2018, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit Mobile PDF SDK to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */


#include "pch.h"
#include "ViewToolAppBar.xaml.h"
#include "../MainPageAppBar.xaml.h"

using namespace foxit::rdk::uiextensions;
using namespace foxit::rdk;

using namespace Platform;
using namespace Windows::Foundation;
using namespace Windows::Foundation::Collections;
using namespace Windows::UI::Xaml;
using namespace Windows::UI::Xaml::Controls;
using namespace Windows::UI::Xaml::Controls::Primitives;
using namespace Windows::UI::Xaml::Data;
using namespace Windows::UI::Xaml::Input;
using namespace Windows::UI::Xaml::Media;
using namespace Windows::UI::Xaml::Navigation;

// The User Control item template is documented at http://go.microsoft.com/fwlink/?LinkId=234236

ViewToolAppBar::ViewToolAppBar(foxit::rdk::PDFViewCtrl^ view, MainPageAppBar^ appbar)
{
	InitializeComponent();

	m_PageViewWeakRef = view;
	m_AppBarWeakRef = appbar;
	

	this->Loaded += ref new Windows::UI::Xaml::RoutedEventHandler(this, &foxit::rdk::uiextensions::ViewToolAppBar::OnLoaded);
	this->Unloaded += ref new Windows::UI::Xaml::RoutedEventHandler(this, &foxit::rdk::uiextensions::ViewToolAppBar::OnUnloaded);
}

foxit::rdk::PDFViewCtrl^ ViewToolAppBar::View::get()
{
	return m_PageViewWeakRef.Resolve<foxit::rdk::PDFViewCtrl>();
}

MainPageAppBar^ ViewToolAppBar::MainAppBar::get()
{
	return m_AppBarWeakRef.Resolve<MainPageAppBar>();
}

Windows::UI::Xaml::Controls::Primitives::Popup^ ViewToolAppBar::HostedPopup::get()
{
	return m_HostedPopupWeakRef.Resolve<Windows::UI::Xaml::Controls::Primitives::Popup>();
}

void foxit::rdk::uiextensions::ViewToolAppBar::OnLoaded(Platform::Object ^sender, Windows::UI::Xaml::RoutedEventArgs ^e)
{
	Windows::UI::Xaml::Controls::Primitives::Popup^ parentPopup = dynamic_cast<Windows::UI::Xaml::Controls::Primitives::Popup^>(this->Parent);
	if (parentPopup)
		m_HostedPopupWeakRef = parentPopup;

	OnInitViewToolPanel();

	Windows::Foundation::Collections::IVectorView<String^>^ langus = Windows::Globalization::ApplicationLanguages::Languages;
	String^ lang = langus->GetAt(0);	
	if (langus->Size > 0 && langus->GetAt(0) == "ko")
	{
		viewToolPC->Width = 164;
	}
	else if (langus->Size > 0 && langus->GetAt(0) == "ja")
	{
		viewToolPC->Width = 170;
	}
}

void foxit::rdk::uiextensions::ViewToolAppBar::OnUnloaded(Platform::Object ^sender, Windows::UI::Xaml::RoutedEventArgs ^e)
{

}

void foxit::rdk::uiextensions::ViewToolAppBar::ResetToolSize()
{
	Windows::Foundation::Rect rect = Windows::UI::Xaml::Window::Current->Bounds;
	
	if (rect.Width < rect.Height)
	{
		viewToolPhoneVer->Width = rect.Width;
		viewToolPhoneVer->Visibility = Windows::UI::Xaml::Visibility::Visible;
		viewToolPhoneHor->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
	}
	else
	{
		viewToolPhoneVer->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
		viewToolPhoneHor->Visibility = Windows::UI::Xaml::Visibility::Visible;
		viewToolPhoneHor->Width = rect.Width;
	}
}

void foxit::rdk::uiextensions::ViewToolAppBar::OnInitViewToolPanel()
{
	if (Windows::System::Profile::AnalyticsInfo::VersionInfo->DeviceFamily == "Windows.Desktop")
	{
		viewToolPC->Visibility = Windows::UI::Xaml::Visibility::Visible;
		viewToolPhoneVer->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
		viewToolPhoneHor->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
		if (View->GetPageLayoutMode() == LayoutMode::ContinueMode)
		{
			ContinuousPageCommandPC->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
			SinglePageCommandPC->Visibility = Windows::UI::Xaml::Visibility::Visible;
		}
		else
		{
			ContinuousPageCommandPC->Visibility = Windows::UI::Xaml::Visibility::Visible;
			SinglePageCommandPC->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
		}

		if (View->NightMode)
		{
			DayModeCommandPC->Visibility = Windows::UI::Xaml::Visibility::Visible;
			DarkModeCommandPC->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
		}
		else
		{
			DayModeCommandPC->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
			DarkModeCommandPC->Visibility = Windows::UI::Xaml::Visibility::Visible;
		}
	}
	else
	{
		viewToolPC->Visibility = Windows::UI::Xaml::Visibility::Collapsed;		
		Windows::Foundation::Rect rect = Windows::UI::Xaml::Window::Current->Bounds;
		if (View->GetPageLayoutMode() == LayoutMode::ContinueMode)
		{
			if (rect.Width < rect.Height)
			{
				ContinuousPageCommand->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
				SinglePageCommand->Visibility = Windows::UI::Xaml::Visibility::Visible;
			}
			else
			{
				ContinuousPageCommandHor->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
				SinglePageCommandHor->Visibility = Windows::UI::Xaml::Visibility::Visible;
			}
		}
		else
		{
			if (rect.Width < rect.Height)
			{
				ContinuousPageCommand->Visibility = Windows::UI::Xaml::Visibility::Visible;
				SinglePageCommand->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
			}
			else
			{
				ContinuousPageCommandHor->Visibility = Windows::UI::Xaml::Visibility::Visible;
				SinglePageCommandHor->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
			}
		}

		if (View->NightMode)
		{
			if (rect.Width < rect.Height)
			{
				DayModeCommand->Visibility = Windows::UI::Xaml::Visibility::Visible;
				DarkModeCommand->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
			}
			else
			{
				DayModeCommandHor->Visibility = Windows::UI::Xaml::Visibility::Visible;
				DarkModeCommandHor->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
			}
		}
		else
		{
			if (rect.Width < rect.Height)
			{
				DayModeCommand->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
				DarkModeCommand->Visibility = Windows::UI::Xaml::Visibility::Visible;
			}
			else
			{
				DayModeCommandHor->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
				DarkModeCommandHor->Visibility = Windows::UI::Xaml::Visibility::Visible;
			}
		}
		ResetToolSize();
	}
}


void foxit::rdk::uiextensions::ViewToolAppBar::ContinuousPageCommandPC_Tapped(Platform::Object^ sender, Windows::UI::Xaml::Input::TappedRoutedEventArgs^ e)
{
	OnContinuousPageView();
}


void foxit::rdk::uiextensions::ViewToolAppBar::SinglePageCommandPC_Tapped(Platform::Object^ sender, Windows::UI::Xaml::Input::TappedRoutedEventArgs^ e)
{
	OnSinglePageView();
}


void foxit::rdk::uiextensions::ViewToolAppBar::DayModeCommandPC_Tapped(Platform::Object^ sender, Windows::UI::Xaml::Input::TappedRoutedEventArgs^ e)
{
	OnDayModePageView();
}


void foxit::rdk::uiextensions::ViewToolAppBar::DarkModeCommandPC_Tapped(Platform::Object^ sender, Windows::UI::Xaml::Input::TappedRoutedEventArgs^ e)
{
	OnDarkModePageView();
}


void foxit::rdk::uiextensions::ViewToolAppBar::LeftRotateCommandPC_Tapped(Platform::Object^ sender, Windows::UI::Xaml::Input::TappedRoutedEventArgs^ e)
{
	OnLeftRotatePageView();
}


void foxit::rdk::uiextensions::ViewToolAppBar::RightRotateCommandPC_Tapped(Platform::Object^ sender, Windows::UI::Xaml::Input::TappedRoutedEventArgs^ e)
{
	OnRightRotatePageView();
}


void foxit::rdk::uiextensions::ViewToolAppBar::ReflowPageCommandPC_Tapped(Platform::Object^ sender, Windows::UI::Xaml::Input::TappedRoutedEventArgs^ e)
{
	OnReflowPageView();
}


void foxit::rdk::uiextensions::ViewToolAppBar::ThumbnailPageCommandPC_Tapped(Platform::Object^ sender, Windows::UI::Xaml::Input::TappedRoutedEventArgs^ e)
{
	OnThumbnailPageView();
}


void foxit::rdk::uiextensions::ViewToolAppBar::ContinuousPageCommand_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{
	OnContinuousPageView();
}


void foxit::rdk::uiextensions::ViewToolAppBar::SinglePageCommand_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{
	OnSinglePageView();
}


void foxit::rdk::uiextensions::ViewToolAppBar::DayModeCommand_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{
	OnDayModePageView();
}


void foxit::rdk::uiextensions::ViewToolAppBar::DarkModeCommand_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{
	OnDarkModePageView();
}


void foxit::rdk::uiextensions::ViewToolAppBar::RightRotateCommand_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{
	OnRightRotatePageView();
}


void foxit::rdk::uiextensions::ViewToolAppBar::LeftRotateCommand_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{
	OnLeftRotatePageView();
}


void foxit::rdk::uiextensions::ViewToolAppBar::ReflowPageCommand_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{
	OnReflowPageView();
}


void foxit::rdk::uiextensions::ViewToolAppBar::ThumbnailPageCommand_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{
	OnThumbnailPageView();
}


void foxit::rdk::uiextensions::ViewToolAppBar::EyeProtectionMode_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{
	MainAppBar->ShowEyeProtectionModeControl();

	ClosePopup();
}

void foxit::rdk::uiextensions::ViewToolAppBar::OnContinuousPageView()
{
	UIExtensionsManager^ exMgr = (UIExtensionsManager^)View->ExtensionsManager;
	exMgr->LoadViewByViewMode(LayoutMode::ContinueMode);

	if (Windows::System::Profile::AnalyticsInfo::VersionInfo->DeviceFamily == "Windows.Desktop")
	{
		ContinuousPageCommandPC->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
		SinglePageCommandPC->Visibility = Windows::UI::Xaml::Visibility::Visible;
	}
	else
	{
		Windows::Foundation::Rect rect = Windows::UI::Xaml::Window::Current->Bounds;

		if (rect.Width < rect.Height)
		{
			ContinuousPageCommand->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
			SinglePageCommand->Visibility = Windows::UI::Xaml::Visibility::Visible;
		}
		else
		{
			ContinuousPageCommandHor->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
			SinglePageCommandHor->Visibility = Windows::UI::Xaml::Visibility::Visible;
		}		
	}
}

void foxit::rdk::uiextensions::ViewToolAppBar::OnSinglePageView()
{
	UIExtensionsManager^ exMgr = (UIExtensionsManager^)View->ExtensionsManager;
	exMgr->LoadViewByViewMode(LayoutMode::SingleMode);

	if (Windows::System::Profile::AnalyticsInfo::VersionInfo->DeviceFamily == "Windows.Desktop")
	{
		ContinuousPageCommandPC->Visibility = Windows::UI::Xaml::Visibility::Visible;
		SinglePageCommandPC->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
	}
	else
	{
		Windows::Foundation::Rect rect = Windows::UI::Xaml::Window::Current->Bounds;

		if (rect.Width < rect.Height)
		{
			ContinuousPageCommand->Visibility = Windows::UI::Xaml::Visibility::Visible;
			SinglePageCommand->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
		}
		else
		{
			ContinuousPageCommandHor->Visibility = Windows::UI::Xaml::Visibility::Visible;
			SinglePageCommandHor->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
		}		
	}
}

void foxit::rdk::uiextensions::ViewToolAppBar::OnDayModePageView()
{
	View->NightMode = false;
	View->Update();
	if (Windows::System::Profile::AnalyticsInfo::VersionInfo->DeviceFamily == "Windows.Desktop")
	{
		DayModeCommandPC->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
		DarkModeCommandPC->Visibility = Windows::UI::Xaml::Visibility::Visible;
	}
	else
	{
		Windows::Foundation::Rect rect = Windows::UI::Xaml::Window::Current->Bounds;

		if (rect.Width < rect.Height)
		{
			DayModeCommand->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
			DarkModeCommand->Visibility = Windows::UI::Xaml::Visibility::Visible;
		}
		else
		{
			DayModeCommandHor->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
			DarkModeCommandHor->Visibility = Windows::UI::Xaml::Visibility::Visible;
		}
	}
}

void foxit::rdk::uiextensions::ViewToolAppBar::OnDarkModePageView()
{
	View->NightMode = true;
	if (Windows::System::Profile::AnalyticsInfo::VersionInfo->DeviceFamily == "Windows.Desktop")
	{
		DayModeCommandPC->Visibility = Windows::UI::Xaml::Visibility::Visible;
		DarkModeCommandPC->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
	}
	else
	{
		Windows::Foundation::Rect rect = Windows::UI::Xaml::Window::Current->Bounds;

		if (rect.Width < rect.Height)
		{
			DayModeCommand->Visibility = Windows::UI::Xaml::Visibility::Visible;
			DarkModeCommand->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
		}
		else
		{
			DayModeCommandHor->Visibility = Windows::UI::Xaml::Visibility::Visible;
			DarkModeCommandHor->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
		}
	}
}

void foxit::rdk::uiextensions::ViewToolAppBar::OnLeftRotatePageView()
{
	auto _CurRotate = View->GetRotate();
	if (_CurRotate == 0)
		_CurRotate = 3;
	else
		_CurRotate--;
	View->SetRotate(_CurRotate);
}

void foxit::rdk::uiextensions::ViewToolAppBar::OnRightRotatePageView()
{
	auto _CurRotate = View->GetRotate();
	if (_CurRotate == 3)
		_CurRotate = 0;
	else
		_CurRotate++;
	View->SetRotate(_CurRotate);
}

void foxit::rdk::uiextensions::ViewToolAppBar::OnReflowPageView()
{
	ClosePopup();

	UIExtensionsManager^ exMgr = (UIExtensionsManager^)View->ExtensionsManager;

	exMgr->LoadViewByViewMode(foxit::rdk::LayoutMode::ReflowMode);
	exMgr->LoadAppBarByType(AppBarType::ReflowViewAppBar);

	if (AppConfig::IsAutoReflowMode)
		return;

	if (!AppConfig::IsAutoReflowModeDlgFirstOpen)
		return;
	
	auto _resourceLoader = Windows::ApplicationModel::Resources::ResourceLoader::GetForCurrentView("foxit.rdk.uiextensions/Resources");
	Platform::String^ _saveYes = _resourceLoader->GetString(L"SaveYesText");
	Platform::String^ _saveNo = _resourceLoader->GetString(L"SaveNoText");
	Platform::String^ _SetAutoReflowContentStr = _resourceLoader->GetString(L"MessageBoxAutoReflowSettingContentText");

	auto _MessageBox = ref new Windows::UI::Popups::MessageDialog(_SetAutoReflowContentStr);
	_MessageBox->Commands->Append(ref new Windows::UI::Popups::UICommand(_saveYes, nullptr, PropertyValue::CreateInt32(0)));
	_MessageBox->Commands->Append(ref new Windows::UI::Popups::UICommand(_saveNo, nullptr, PropertyValue::CreateInt32(1)));
	_MessageBox->DefaultCommandIndex = 0;
}

void foxit::rdk::uiextensions::ViewToolAppBar::OnThumbnailPageView()
{
	ClosePopup();

	UIExtensionsManager^ exMgr = (UIExtensionsManager^)View->ExtensionsManager;
	exMgr->EnterThumbnailPage();
}

void ViewToolAppBar::ClosePopup()
{
	if (HostedPopup)
	{
		HostedPopup->IsOpen = false;
	}
}


void foxit::rdk::uiextensions::ViewToolAppBar::CommandPC_PointerEntered(Platform::Object^ sender, Windows::UI::Xaml::Input::PointerRoutedEventArgs^ e)
{
	Canvas^ canvas = dynamic_cast<Canvas^>(sender);
	Windows::UI::Color color = Utility::ConvertFromString("#FFCCCCCC");
	canvas->Background = ref new SolidColorBrush(color);
}


void foxit::rdk::uiextensions::ViewToolAppBar::CommandPC_PointerExited(Platform::Object^ sender, Windows::UI::Xaml::Input::PointerRoutedEventArgs^ e)
{
	Canvas^ canvas = dynamic_cast<Canvas^>(sender);
	Windows::UI::Color color = Utility::ConvertFromString("#FF515151");
	canvas->Background = ref new SolidColorBrush(Windows::UI::Colors::Transparent);
}
