﻿/**
 * Copyright (C) 2003-2018, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit Mobile PDF SDK to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */


#include "pch.h"
#include "FormControlListControl.xaml.h"
using namespace foxit::rdk;
using namespace foxit::rdk::uiextensions;

using namespace Platform;
using namespace Windows::Foundation;
using namespace Windows::Foundation::Collections;
using namespace Windows::UI::Xaml;
using namespace Windows::UI::Xaml::Controls;
using namespace Windows::UI::Xaml::Controls::Primitives;
using namespace Windows::UI::Xaml::Data;
using namespace Windows::UI::Xaml::Input;
using namespace Windows::UI::Xaml::Media;
using namespace Windows::UI::Xaml::Navigation;


FormControlListItem::FormControlListItem(foxit::pdf::interform::ChoiceOption^ choiceOption)
{
	if (choiceOption)
	{
		_Label = choiceOption->option_label;
		if (_Label->Length() == 0)
		{
			_Label = " ";
		}
		_ChoiceOption = choiceOption;
	}
	else
	{
		_ChoiceOption = nullptr;
	}
}

Platform::String^ FormControlListItem::Label::get()
{
	return _Label;
}

bool FormControlListItem::IsSelected::get()
{
	return _ChoiceOption && _ChoiceOption->selected ? true : false;
}

void FormControlListItem::IsSelected::set(bool value)
{
	if (_ChoiceOption)
	{
		_ChoiceOption->selected = value;
	}
}

FormControlListItemGroup::FormControlListItemGroup(foxit::pdf::interform::Form^ form, foxit::pdf::annots::IAnnot^ annot)
{
	_Group = ref new Platform::Collections::Vector<FormControlListItem^>();
	m_DefaultSelectIndex = -1;
	m_SelectIndex = -1;
	m_pSdkForm = form;
	m_pAnnot = ref new foxit::pdf::annots::Widget(dynamic_cast<foxit::pdf::annots::Annot^>(annot));

	foxit::pdf::interform::Control^ _FormControl = ((foxit::pdf::annots::Widget^)m_pAnnot)->GetControl();
	foxit::pdf::interform::Field^ formField = _FormControl->GetField();
	foxit::pdf::interform::Type _Type = formField->GetType();
	if (_Type == foxit::pdf::interform::Type::e_TypeComboBox || _Type == foxit::pdf::interform::Type::e_TypeListBox)
	{
		int _DefaultSelectIndex = -1;
		int _SelectIndex = -1;
		foxit::pdf::interform::ChoiceOptionArray^ _ChoiceOption = formField->GetOptions();
		for (unsigned int i = 0; i < _ChoiceOption->GetSize(); i++)
		{
			auto item = _ChoiceOption->GetAt(i);
			auto defaultSelected = item->default_selected;
			if (defaultSelected)
				_DefaultSelectIndex = i;

			auto selected = item->selected;
			if (selected)
				_SelectIndex = i;

			auto optionLabel = item->option_label;
			auto optionValue = item->option_value;
			_Group->Append(ref new FormControlListItem(item));
		}

		m_DefaultSelectIndex = _DefaultSelectIndex;
		m_SelectIndex = _SelectIndex;
		m_InitializeSelectedIndex = m_SelectIndex;
		m_ChoiceOption = _ChoiceOption;

		OutputDebugString((L"FormControlListItemGroup---m_DefaultSelectIndex: " + m_DefaultSelectIndex + L", m_SelectIndex: " + m_SelectIndex + L", GetHashCode: " + this->GetHashCode() + L", GroupSize: " + _Group->Size + L"\n")->Data());
	}
}

Windows::Foundation::Collections::IVector<FormControlListItem^>^ FormControlListItemGroup::Group::get()
{
	return _Group;
}

int FormControlListItemGroup::DefaultSelectIndex::get()
{
	return m_DefaultSelectIndex;
}

int FormControlListItemGroup::SelectIndex::get()
{
	return m_SelectIndex;
}

int FormControlListItemGroup::GetSelectIndexFromSdk()
{
	int _DefaultSelectIndex = -1;
	int _SelectIndex = -1;
	foxit::pdf::interform::Control^ _FormControl = ((foxit::pdf::annots::Widget^)m_pAnnot)->GetControl();

	foxit::pdf::interform::Field^ formField = _FormControl->GetField();
	foxit::pdf::interform::Type _Type = formField->GetType();

	foxit::pdf::interform::ChoiceOptionArray^ _ChoiceOption = formField->GetOptions();
	for (unsigned int i = 0; i < _ChoiceOption->GetSize(); i++)
	{
		auto item = _ChoiceOption->GetAt(i);
		auto defaultSelected = item->default_selected;
		if (defaultSelected)
			_DefaultSelectIndex = i;

		auto selected = item->selected;
		if (selected)
			_SelectIndex = i;

		auto optionLabel = item->option_label;
		auto optionValue = item->option_value;
	}

	return _SelectIndex;
}

bool FormControlListItemGroup::IsDataChanged()
{
	int selectedIndex = -1;
	auto _GroupSize = _Group->Size;
	for (unsigned int i = 0; i < _GroupSize; i++)
	{
		auto item = _Group->GetAt(i);
		if (item && item->IsSelected)
		{
			selectedIndex = i;
			break;
		}
	}

	return GetSelectIndexFromSdk() != selectedIndex;
}

void FormControlListItemGroup::UpdateToDocument()
{
	FS_RESULT iRet = -1;
	if (m_pSdkForm && m_pAnnot)
	{
		foxit::pdf::interform::Control^ _FormControl = ((foxit::pdf::annots::Widget^)m_pAnnot)->GetControl();
		foxit::pdf::interform::Field^ formField = _FormControl->GetField();
		formField->SetOptions(m_ChoiceOption);
	}
}

void FormControlListItemGroup::ReleaseFormResource()
{
	m_ChoiceOption = nullptr;
}

foxit::pdf::annots::IAnnot^ FormControlListItemGroup::GetSdkAnnot()
{
	return m_pAnnot;
}

// The User Control item template is documented at http://go.microsoft.com/fwlink/?LinkId=234236

FormControlListControl::FormControlListControl()
{
	InitializeComponent();

	OnInitializeControl();
}

void FormControlListControl::OnInitializeControl()
{

	this->Loaded += ref new Windows::UI::Xaml::RoutedEventHandler(this, &foxit::rdk::uiextensions::FormControlListControl::OnLoaded);
	this->Unloaded += ref new Windows::UI::Xaml::RoutedEventHandler(this, &foxit::rdk::uiextensions::FormControlListControl::OnUnloaded);
}

foxit::rdk::PDFViewCtrl^ FormControlListControl::View::get()
{
	return m_PageViewWeakRef.Resolve<foxit::rdk::PDFViewCtrl>();
}

void FormControlListControl::SetPageView(foxit::rdk::PDFViewCtrl^ view)
{
	if (view)
		m_PageViewWeakRef = view;
}

void FormControlListControl::InitDataListSource(FormControlListItemGroup^ data)
{
	m_PrevControlItem = nullptr;
	m_ItemsGroup = data;

	FormControlListView->ItemsSource = m_ItemsGroup->Group;

	int _DefaultSelectIndex = -1;
	int _SelectIndex = -1;

	_DefaultSelectIndex = m_ItemsGroup->DefaultSelectIndex;
	_SelectIndex = m_ItemsGroup->SelectIndex;

	int _NeedScrolltoIndex = _SelectIndex != -1 ? _SelectIndex : _DefaultSelectIndex;

	m_InitializeSelectedIndex = _NeedScrolltoIndex;

	if (_NeedScrolltoIndex != -1 && _NeedScrolltoIndex < (int)m_ItemsGroup->Group->Size)
	{
		FormControlListView->SelectedIndex = _NeedScrolltoIndex;
		FormControlListView->ScrollIntoView(m_ItemsGroup->Group->GetAt(_NeedScrolltoIndex));
	}
}

void FormControlListControl::InitDataList()
{
	if (View == nullptr)
		return;
}

void FormControlListControl::UpdateControlSize()
{
	if (Windows::System::Profile::AnalyticsInfo::VersionInfo->DeviceFamily == L"Windows.Mobile")
	{
		auto pAppView = Windows::UI::ViewManagement::ApplicationView::GetForCurrentView();
		auto _AppViewOrientation = pAppView->Orientation;
		auto _VisibleBounds = pAppView->VisibleBounds;

		auto _FlyoutPresenter = GetVisualParent<Windows::UI::Xaml::Controls::FlyoutPresenter^>(this);
		if (_FlyoutPresenter)
		{
			std::wstringstream ws;
			ws << L"FormControlListControl--MinHeight: " << _FlyoutPresenter->MinHeight;
			ws << L", MaxHeight: " << _FlyoutPresenter->MaxHeight;
			ws << L", ActualWidth: " << _FlyoutPresenter->ActualWidth;
			ws << L", ActualHeight: " << _FlyoutPresenter->ActualHeight;
			ws << L", VisibleBounds.Width: " << _VisibleBounds.Width;
			ws << L", VisibleBounds.Height: " << _VisibleBounds.Height;
			ws << L"\n";
			OutputDebugString(ws.str().c_str());

			if (_FlyoutPresenter->ActualWidth < _VisibleBounds.Width)
			{
				_FlyoutPresenter->MinWidth = _VisibleBounds.Width;
				_FlyoutPresenter->MaxWidth = _VisibleBounds.Width;
				_FlyoutPresenter->Width = _VisibleBounds.Width;
			}

			Windows::UI::Xaml::Controls::Primitives::Popup^ _Popup = dynamic_cast<Windows::UI::Xaml::Controls::Primitives::Popup^>(_FlyoutPresenter->Parent);
			if (_Popup)
			{
				_Popup->IsLightDismissEnabled = true;
			}
		}

		if (RootGrid->ActualWidth < _VisibleBounds.Width)
		{
			RootGrid->MinWidth = _VisibleBounds.Width;
			RootGrid->MaxWidth = _VisibleBounds.Width;
			RootGrid->Width = _VisibleBounds.Width;
		}
	}
}

void foxit::rdk::uiextensions::FormControlListControl::OnLoaded(Platform::Object ^sender, Windows::UI::Xaml::RoutedEventArgs ^e)
{
	m_WindowSizeChangedEventToken = Windows::UI::Xaml::Window::Current->SizeChanged += ref new Windows::UI::Xaml::WindowSizeChangedEventHandler(this, &foxit::rdk::uiextensions::FormControlListControl::OnWindowSizeChanged);

	InitDataList();

	UpdateControlSize();
}


void foxit::rdk::uiextensions::FormControlListControl::OnUnloaded(Platform::Object ^sender, Windows::UI::Xaml::RoutedEventArgs ^e)
{
	if (m_WindowSizeChangedEventToken.Value)
	{
		Windows::UI::Xaml::Window::Current->SizeChanged -= m_WindowSizeChangedEventToken;
		m_WindowSizeChangedEventToken.Value = 0;
	}
}


void foxit::rdk::uiextensions::FormControlListControl::OnWindowSizeChanged(Platform::Object ^sender, Windows::UI::Core::WindowSizeChangedEventArgs ^e)
{
	UpdateControlSize();
}


void foxit::rdk::uiextensions::FormControlListControl::FormControlListView_SelectionChanged(Platform::Object^ sender, Windows::UI::Xaml::Controls::SelectionChangedEventArgs^ e)
{
	auto _AddedItems = e->AddedItems;
	if (_AddedItems->Size > 0)
	{
		if (m_PrevControlItem)
		{
			m_PrevControlItem->IsSelected = false;
		}
		auto _selectItem = dynamic_cast<FormControlListItem^>(FormControlListView->SelectedItem);
		if (_selectItem)
		{
			_selectItem->IsSelected = true;
			m_PrevControlItem = _selectItem;
		}
	}
}
