﻿/**
 * Copyright (C) 2003-2018, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit Mobile PDF SDK to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */


#include "pch.h"
#include "FormControlTextBox.h"

using namespace foxit::rdk::uiextensions;

using namespace Platform;
using namespace Windows::Foundation;
using namespace Windows::Foundation::Collections;
using namespace Windows::UI::Xaml;
using namespace Windows::UI::Xaml::Controls;
using namespace Windows::UI::Xaml::Data;
using namespace Windows::UI::Xaml::Documents;
using namespace Windows::UI::Xaml::Input;
using namespace Windows::UI::Xaml::Interop;
using namespace Windows::UI::Xaml::Media;

// The Templated Control item template is documented at http://go.microsoft.com/fwlink/?LinkId=234235

FormControlTextBox::FormControlTextBox()
{
	DefaultStyleKey = "foxit.rdk.uiextensions.FormControlTextBox";

	if (Windows::ApplicationModel::DesignMode::DesignModeEnabled)
		return;

	OnInitializeControl();
}

void FormControlTextBox::OnInitializeControl()
{
	this->ApplyTemplate();

	this->KeyDown += ref new Windows::UI::Xaml::Input::KeyEventHandler(this, &foxit::rdk::uiextensions::FormControlTextBox::OnControlKeyDown);
	this->TextChanged += ref new Windows::UI::Xaml::Controls::TextChangedEventHandler(this, &foxit::rdk::uiextensions::FormControlTextBox::OnTextChanged);
	this->TextCompositionChanged += ref new Windows::Foundation::TypedEventHandler<Windows::UI::Xaml::Controls::TextBox ^, Windows::UI::Xaml::Controls::TextCompositionChangedEventArgs ^>(this, &foxit::rdk::uiextensions::FormControlTextBox::OnTextCompositionChanged);
	this->Loaded += ref new Windows::UI::Xaml::RoutedEventHandler(this, &foxit::rdk::uiextensions::FormControlTextBox::OnLoaded);
	this->Unloaded += ref new Windows::UI::Xaml::RoutedEventHandler(this, &foxit::rdk::uiextensions::FormControlTextBox::OnUnloaded);
}

void FormControlTextBox::HookUpEvents()
{

}

void FormControlTextBox::UnHookEvents()
{

}

void FormControlTextBox::OnApplyTemplate()
{
	TextBox::OnApplyTemplate();

	UnHookEvents();

	m_PlaceholderTextContentPresenter = safe_cast<Windows::UI::Xaml::Controls::ContentControl^>(GetTemplateChild(L"PlaceholderTextContentPresenter"));

	HookUpEvents();
}

void FormControlTextBox::SetPageView(foxit::rdk::PDFViewCtrl^ view)
{
	m_PageViewWeakRef = view;
}

foxit::rdk::PDFViewCtrl^ FormControlTextBox::View::get()
{
	return m_PageViewWeakRef.Resolve<foxit::rdk::PDFViewCtrl>();
}

void FormControlTextBox::ResetInputText()
{
	m_PrevInputString = ref new Platform::String(L"");
	this->Text = ref new Platform::String(L"");
}

void FormControlTextBox::OnKeyDown(Windows::UI::Xaml::Input::KeyRoutedEventArgs^ e)
{
	auto _virtualKey = e->Key;
	auto _coreWindow = Window::Current->CoreWindow;
	auto _downState = Windows::UI::Core::CoreVirtualKeyStates::Down;
	bool _menuKey = (_coreWindow->GetKeyState(Windows::System::VirtualKey::Menu) & _downState) == _downState;
	bool _controlKey = (_coreWindow->GetKeyState(Windows::System::VirtualKey::Control) & _downState) == _downState;
	bool _shiftKey = (_coreWindow->GetKeyState(Windows::System::VirtualKey::Shift) & _downState) == _downState;
	bool _noModifiers = !_menuKey && !_controlKey && !_shiftKey;
	bool _onlyAlt = _menuKey && !_controlKey && !_shiftKey;
	Windows::UI::Core::CorePhysicalKeyStatus _KeyStatus = e->KeyStatus;

	{
		std::wstringstream ws;
		ws << L"FormControlTextBox::OnKeyDown---_menuKey: " << _menuKey;
		ws << L", _controlKey: " << _controlKey;
		ws << L", _shiftKey: " << _shiftKey;
		ws << L", VirtualKey: " << _virtualKey.ToString()->Data();
		ws << L", OriginalKey: " << e->OriginalKey.ToString()->Data();
		ws << L", ScanCode: " << _KeyStatus.ScanCode;
		ws << L"\n";
		OutputDebugString(ws.str().c_str());
	}
	
	if (View == nullptr || View->GetDoc() == nullptr)
	{
		TextBox::OnKeyDown(e);
		return;
	}

	m_KeyDownVirtualKey = _virtualKey;

	auto pDocument = View->GetDoc();
	auto uiextensions = (foxit::rdk::uiextensions::UIExtensionsManager^)View->ExtensionsManager;
	auto pSelectedAnnot = uiextensions->getAnnotMgr()->GetSelectedAnnot();
	if (pDocument && pSelectedAnnot && pSelectedAnnot->AnnotType == foxit::rdk::uiextensions::AnnotationType::Widget)
	{
		auto _coreWindow = Window::Current->CoreWindow;
		auto _downState = Windows::UI::Core::CoreVirtualKeyStates::Down;
		bool _controlKey = (_coreWindow->GetKeyState(Windows::System::VirtualKey::Control) & _downState) == _downState;
		bool _shiftKey = (_coreWindow->GetKeyState(Windows::System::VirtualKey::Shift) & _downState) == _downState;
		bool _altKey = (_coreWindow->GetKeyState(Windows::System::VirtualKey::Menu) & _downState) == _downState;
		auto _capitalLockKey = (_coreWindow->GetKeyState(Windows::System::VirtualKey::CapitalLock) & Windows::UI::Core::CoreVirtualKeyStates::Locked) == Windows::UI::Core::CoreVirtualKeyStates::Locked;

		bool iRet = false;
		uint32 charDatacode = (FS_DWORD)_virtualKey;
		uint32 charDataflags = 0;

		if (_controlKey)
			charDataflags |= (uint32)foxit::pdf::interform::EventFlags::e_EventFlagControlKey;
		if (_shiftKey)
			charDataflags |= (uint32)foxit::pdf::interform::EventFlags::e_EventFlagShiftKey;
		if (_altKey)
			charDataflags |= (uint32)foxit::pdf::interform::EventFlags::e_EventFlagAltKey;

		uint32 keyDataCode = (FS_DWORD)_virtualKey;
		uint32 keyDataflags = 0;
		if (_controlKey)
			keyDataflags |= (uint32)foxit::pdf::interform::EventFlags::e_EventFlagControlKey;
		if (_shiftKey)
			keyDataflags |= (uint32)foxit::pdf::interform::EventFlags::e_EventFlagShiftKey;
		if (_altKey)
			keyDataflags |= (uint32)foxit::pdf::interform::EventFlags::e_EventFlagAltKey;

		if (_virtualKey >= Windows::System::VirtualKey::A && _virtualKey <= Windows::System::VirtualKey::Z)
		{
			charDatacode = (_capitalLockKey || _shiftKey) ? (FS_DWORD)_virtualKey : (FS_DWORD)_virtualKey + 32;

			iRet = uiextensions->GetFormFiller()->OnKeyDown(keyDataCode, keyDataflags);
		}
		else if (_virtualKey >= Windows::System::VirtualKey::Number0 && _virtualKey <= Windows::System::VirtualKey::Number9)
		{
			charDatacode = (uint32)foxit::pdf::interform::VirtualKeyCode::e_Vkey0 + (int)(_virtualKey - Windows::System::VirtualKey::Number0);
			iRet = uiextensions->GetFormFiller()->OnKeyDown(keyDataCode, keyDataflags);
		}
		else if (_virtualKey >= Windows::System::VirtualKey::NumberPad0 && _virtualKey <= Windows::System::VirtualKey::NumberPad9)
		{
			charDatacode = (uint32)foxit::pdf::interform::VirtualKeyCode::e_Vkey0 + (int)(_virtualKey - Windows::System::VirtualKey::NumberPad0);
			iRet = uiextensions->GetFormFiller()->OnKeyDown(keyDataCode, keyDataflags);
		}
		else if (_virtualKey == Windows::System::VirtualKey::Delete)
		{
			keyDataCode = (uint32)foxit::pdf::interform::VirtualKeyCode::e_VkeyDelete;
			iRet = uiextensions->GetFormFiller()->OnKeyDown(keyDataCode, keyDataflags);
			if (iRet)
				View->SetModified(true);
			return;
		}
		else if (_virtualKey == Windows::System::VirtualKey::Back)
		{
			keyDataCode = (uint32)foxit::pdf::interform::VirtualKeyCode::e_VkeyBack;
			iRet = uiextensions->GetFormFiller()->OnChar(charDatacode, charDataflags);
			if (iRet)
				View->SetModified(true);
			return;
		}
		else if (_virtualKey >= Windows::System::VirtualKey::Left && _virtualKey <= Windows::System::VirtualKey::Down)
		{
			charDatacode = (uint32)foxit::pdf::interform::VirtualKeyCode::e_VkeyLeft + (int)(_virtualKey - Windows::System::VirtualKey::Left);
			iRet = uiextensions->GetFormFiller()->OnKeyDown(keyDataCode, keyDataflags);
		}
		else
		{
			charDatacode = (FS_DWORD)_virtualKey;
			iRet = uiextensions->GetFormFiller()->OnKeyDown(keyDataCode, keyDataflags);
		}
	}

	TextBox::OnKeyDown(e);
}

void FormControlTextBox::OnKeyUp(Windows::UI::Xaml::Input::KeyRoutedEventArgs^ e)
{
	auto _virtualKey = e->Key;
	auto _coreWindow = Window::Current->CoreWindow;
	auto _downState = Windows::UI::Core::CoreVirtualKeyStates::Down;
	bool _menuKey = (_coreWindow->GetKeyState(Windows::System::VirtualKey::Menu) & _downState) == _downState;
	bool _controlKey = (_coreWindow->GetKeyState(Windows::System::VirtualKey::Control) & _downState) == _downState;
	bool _shiftKey = (_coreWindow->GetKeyState(Windows::System::VirtualKey::Shift) & _downState) == _downState;
	bool _noModifiers = !_menuKey && !_controlKey && !_shiftKey;
	bool _onlyAlt = _menuKey && !_controlKey && !_shiftKey;
	Windows::UI::Core::CorePhysicalKeyStatus _KeyStatus = e->KeyStatus;

	{
		std::wstringstream ws;
		ws << L"FormControlTextBox::OnKeyUp---_menuKey: " << _menuKey;
		ws << L", _controlKey: " << _controlKey;
		ws << L", _shiftKey: " << _shiftKey;
		ws << L", VirtualKey: " << _virtualKey.ToString()->Data();
		ws << L", OriginalKey: " << e->OriginalKey.ToString()->Data();
		ws << L", ScanCode: " << _KeyStatus.ScanCode;
		ws << L"\n";
		OutputDebugString(ws.str().c_str());
	}

	if (View == nullptr || View->GetDoc() == nullptr)
	{
		TextBox::OnKeyUp(e);
		return;
	}


	auto pDocument = View->GetDoc();
	auto uiextensions = (foxit::rdk::uiextensions::UIExtensionsManager^)View->ExtensionsManager;
	auto pSelectedAnnot = uiextensions->getAnnotMgr()->GetSelectedAnnot();
	if (pDocument && pSelectedAnnot && pSelectedAnnot->AnnotType == foxit::rdk::uiextensions::AnnotationType::Widget)
	{
		auto _coreWindow = Window::Current->CoreWindow;
		auto _downState = Windows::UI::Core::CoreVirtualKeyStates::Down;
		bool _controlKey = (_coreWindow->GetKeyState(Windows::System::VirtualKey::Control) & _downState) == _downState;
		bool _shiftKey = (_coreWindow->GetKeyState(Windows::System::VirtualKey::Shift) & _downState) == _downState;
		bool _altKey = (_coreWindow->GetKeyState(Windows::System::VirtualKey::Menu) & _downState) == _downState;
		auto _capitalLockKey = (_coreWindow->GetKeyState(Windows::System::VirtualKey::CapitalLock) & Windows::UI::Core::CoreVirtualKeyStates::Locked) == Windows::UI::Core::CoreVirtualKeyStates::Locked;

		bool iRet = false;

		uint32 charDatacode = (FS_DWORD)_virtualKey;
		uint32 charDataflags = 0;

		if (_controlKey)
			charDataflags |= (uint32)foxit::pdf::interform::EventFlags::e_EventFlagControlKey;
		if (_shiftKey)
			charDataflags |= (uint32)foxit::pdf::interform::EventFlags::e_EventFlagShiftKey;
		if (_altKey)
			charDataflags |= (uint32)foxit::pdf::interform::EventFlags::e_EventFlagAltKey;

		uint32 keyDataCode = (FS_DWORD)_KeyStatus.ScanCode;
		uint32 keyDataflags = 0;
		if (_controlKey)
			keyDataflags |= (uint32)foxit::pdf::interform::EventFlags::e_EventFlagControlKey;
		if (_shiftKey)
			keyDataflags |= (uint32)foxit::pdf::interform::EventFlags::e_EventFlagShiftKey;
		if (_altKey)
			keyDataflags |= (uint32)foxit::pdf::interform::EventFlags::e_EventFlagAltKey;

		if ((_virtualKey >= Windows::System::VirtualKey::A && _virtualKey <= Windows::System::VirtualKey::Z) || (_virtualKey >= Windows::System::VirtualKey::Number0 && _virtualKey <= Windows::System::VirtualKey::Number9))
		{
			charDatacode = (FS_DWORD)_virtualKey;
		}
		else if (_virtualKey >= Windows::System::VirtualKey::NumberPad0 && _virtualKey <= Windows::System::VirtualKey::NumberPad9)
		{
			charDatacode = (uint32)foxit::pdf::interform::VirtualKeyCode::e_Vkey0 + (int)(_virtualKey - Windows::System::VirtualKey::NumberPad0);
		}
		else if (_virtualKey == Windows::System::VirtualKey::Delete)
		{
			keyDataCode = (uint32)foxit::pdf::interform::VirtualKeyCode::e_VkeyDelete;
		}
		else if (_virtualKey >= Windows::System::VirtualKey::Left && _virtualKey <= Windows::System::VirtualKey::Down)
		{
			charDatacode = (uint32)foxit::pdf::interform::VirtualKeyCode::e_VkeyLeft + (int)(_virtualKey - Windows::System::VirtualKey::Left);
		}
		else
		{
			charDatacode = (FS_DWORD)_virtualKey;
		}
		iRet = uiextensions->GetFormFiller()->OnKeyUp(keyDataCode, keyDataflags);
	}

	TextBox::OnKeyUp(e);
}

void foxit::rdk::uiextensions::FormControlTextBox::OnLoaded(Platform::Object ^sender, Windows::UI::Xaml::RoutedEventArgs ^e)
{
	auto _PageView = foxit::rdk::uiextensions::GetVisualParent<foxit::rdk::PDFViewCtrl^>(this);
	if (_PageView)
	{
		m_PageViewWeakRef = _PageView;
	}

	HookUpEvents();
}


void foxit::rdk::uiextensions::FormControlTextBox::OnUnloaded(Platform::Object ^sender, Windows::UI::Xaml::RoutedEventArgs ^e)
{
	UnHookEvents();
}


void foxit::rdk::uiextensions::FormControlTextBox::OnControlKeyDown(Platform::Object ^sender, Windows::UI::Xaml::Input::KeyRoutedEventArgs ^e)
{
}


void foxit::rdk::uiextensions::FormControlTextBox::OnTextChanging(Windows::UI::Xaml::Controls::TextBox ^sender, Windows::UI::Xaml::Controls::TextBoxTextChangingEventArgs ^args)
{
	auto _CurString = this->Text;
	
	OutputDebugString((L"FormControlTextBox::OnTextChanging...[" + _CurString + L"]\n")->Data());
}


void foxit::rdk::uiextensions::FormControlTextBox::OnTextChanged(Platform::Object ^sender, Windows::UI::Xaml::Controls::TextChangedEventArgs ^e)
{
	auto _CurString = this->Text;
	auto pDocument = View->GetDoc();
	auto uiextensions = (foxit::rdk::uiextensions::UIExtensionsManager^)View->ExtensionsManager;

	OutputDebugString((L"FormControlTextBox::OnTextChanged...[" + m_PrevInputString + L"],[" + _CurString + L"]\n")->Data());

	if (m_PrevInputString != _CurString)
	{
		auto _prevStrLength = m_PrevInputString->Length();
		auto _curStrLength = _CurString->Length();
		auto _SelectionStart = this->SelectionStart;
		auto _SelectionLength = this->SelectionLength;
		auto _coreWindow = Window::Current->CoreWindow;
		auto _downState = Windows::UI::Core::CoreVirtualKeyStates::Down;
		bool _controlKey = (_coreWindow->GetKeyState(Windows::System::VirtualKey::Control) & _downState) == _downState;
		bool _shiftKey = (_coreWindow->GetKeyState(Windows::System::VirtualKey::Shift) & _downState) == _downState;
		bool _altKey = (_coreWindow->GetKeyState(Windows::System::VirtualKey::Menu) & _downState) == _downState;
		auto _capitalLockKey = (_coreWindow->GetKeyState(Windows::System::VirtualKey::CapitalLock) & Windows::UI::Core::CoreVirtualKeyStates::Locked) == Windows::UI::Core::CoreVirtualKeyStates::Locked;

		wchar_t* pSrc = (wchar_t*)m_PrevInputString->Data();
		wchar_t* pDes = (wchar_t*)_CurString->Data();
		wchar_t* pSrcStart = pSrc;
		wchar_t* pDesStart = pDes;
		wchar_t* pSrcEnd = NULL;
		wchar_t* pDesEnd = NULL;

		wchar_t* p = (wchar_t*)m_PrevInputString->Data();
		wchar_t* q = (wchar_t*)_CurString->Data();
		while (1)
		{
			if (*p == '\0' || *q == '\0')
			{
				pSrcStart = p;
				pDesStart = q;
				break;
			}

			if (*p == *q)
			{
				p++;
				q++;
			}
			else
			{
				pSrcStart = p;
				pDesStart = q;
				break;
			}
		}

		p = (wchar_t*)m_PrevInputString->Data() + _prevStrLength;
		q = (wchar_t*)_CurString->Data() + _curStrLength;
		while (1)
		{
			if (p - pSrcStart == 0 || q - pDesStart == 0)
			{
				if (*p == *q)
				{
					pSrcEnd = p;
					pDesEnd = q;
				}
				else
				{
					pSrcEnd = ++p;
					pDesEnd = ++q;
				}

				break;
			}
			else
			{
				if (*p == *q)
				{
					p--;
					q--;
				}
				else
				{
					pSrcEnd = ++p;
					pDesEnd = ++q;
					break;
				}
			}
		}

		auto _needDeleteCount = pSrcEnd - pSrcStart;
		auto _iLength = pDesEnd - pDesStart;
		auto _iStr = ref new Platform::String(pDesStart, (unsigned int)_iLength);
		p = (wchar_t*)_iStr->Data();

		if (_iLength == 1)
		{
			if (m_KeyDownVirtualKey != Windows::System::VirtualKey::Space && p[0] == ' ')
			{
				return;
			}
		}

		OutputDebugString((L"FormControlTextBox::OnTextChanged...[Delete: " + _needDeleteCount + L"],[Add: " + _iLength + L"]\n")->Data());
		
		bool iRet = false;
		uint32 keyCode = 0;
		uint32 flags = 0;
		if (_controlKey)
			flags |= (uint32)foxit::pdf::interform::EventFlags::e_EventFlagControlKey;
		if (_shiftKey)
			flags |= (uint32)foxit::pdf::interform::EventFlags::e_EventFlagShiftKey;
		if (_altKey)
			flags |= (uint32)foxit::pdf::interform::EventFlags::e_EventFlagAltKey;
		for (int i = 0; i < _needDeleteCount; i++)
		{
			keyCode = (uint32)foxit::pdf::interform::VirtualKeyCode::e_VkeyBack;

			if (pDocument)
				iRet = uiextensions->GetFormFiller()->OnChar(keyCode, flags);
		}

		for (int i = 0; i < _iLength; i++)
		{
			keyCode = (FS_DWORD)p[i];
			iRet = uiextensions->GetFormFiller()->OnChar(keyCode, flags);
		}

		
		m_PrevInputString = ref new Platform::String(Text->Data());
	}
}


void foxit::rdk::uiextensions::FormControlTextBox::OnTextCompositionStarted(Windows::UI::Xaml::Controls::TextBox ^sender, Windows::UI::Xaml::Controls::TextCompositionStartedEventArgs ^args)
{
	auto _CurString = this->Text;
	
	OutputDebugString((L"FormControlTextBox::OnTextCompositionStarted...[" + _CurString + L"], StartIndex: " + args->StartIndex + L", Length: " + args->Length + L"\n")->Data());
}


void foxit::rdk::uiextensions::FormControlTextBox::OnTextCompositionChanged(Windows::UI::Xaml::Controls::TextBox ^sender, Windows::UI::Xaml::Controls::TextCompositionChangedEventArgs ^args)
{
	auto _CurString = this->Text;
	auto pDocument = View->GetDoc();
	auto _StartIndex = args->StartIndex;
	auto _Length = args->Length;

	OutputDebugString((L"FormControlTextBox::OnTextCompositionChanged...[" + m_PrevInputString + L"],[" + _CurString + L"], StartIndex: " + args->StartIndex + L", Length: " + args->Length + L"\n")->Data());
}


void foxit::rdk::uiextensions::FormControlTextBox::OnTextCompositionEnded(Windows::UI::Xaml::Controls::TextBox ^sender, Windows::UI::Xaml::Controls::TextCompositionEndedEventArgs ^args)
{
	auto _CurString = this->Text;
	
	OutputDebugString((L"FormControlTextBox::OnTextCompositionEnded...[" + _CurString + L"], StartIndex: " + args->StartIndex + L", Length: " + args->Length + L"\n")->Data());
}
