﻿/**
 * Copyright (C) 2003-2018, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit Mobile PDF SDK to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */


#include "pch.h"
#include "FormOperationBarControl.xaml.h"
#include "FormControlListControl.xaml.h"

using namespace foxit::rdk;
using namespace foxit::rdk::uiextensions;

using namespace Platform;
using namespace Windows::Foundation;
using namespace Windows::Foundation::Collections;
using namespace Windows::UI::Xaml;
using namespace Windows::UI::Xaml::Controls;
using namespace Windows::UI::Xaml::Controls::Primitives;
using namespace Windows::UI::Xaml::Data;
using namespace Windows::UI::Xaml::Input;
using namespace Windows::UI::Xaml::Media;
using namespace Windows::UI::Xaml::Navigation;

// The User Control item template is documented at http://go.microsoft.com/fwlink/?LinkId=234236

FormOperationBarControl::FormOperationBarControl()
{
	InitializeComponent();

	OnInitializeControl();
}

void FormOperationBarControl::OnInitializeControl()
{
	this->Loaded += ref new Windows::UI::Xaml::RoutedEventHandler(this, &foxit::rdk::uiextensions::FormOperationBarControl::OnLoaded);
	this->Unloaded += ref new Windows::UI::Xaml::RoutedEventHandler(this, &foxit::rdk::uiextensions::FormOperationBarControl::OnUnloaded);
}

foxit::rdk::PDFViewCtrl^ FormOperationBarControl::View::get()
{
	return m_PageViewWeakRef.Resolve<foxit::rdk::PDFViewCtrl>();
}

void FormOperationBarControl::SetPageView(foxit::rdk::PDFViewCtrl^ view)
{
	if (view)
		m_PageViewWeakRef = view;
}

void FormOperationBarControl::OpenFormListControl()
{
	if (View == nullptr)
		return;

	auto uiextensions = (foxit::rdk::uiextensions::UIExtensionsManager^)View->ExtensionsManager;
	auto _pSdkForm = uiextensions->GetSdkForm();
	auto _SelectedAnnot = uiextensions->getAnnotMgr()->GetSelectedAnnot();
	if (_SelectedAnnot == nullptr)
	{
		uiextensions->HideWidgetAnnotPopup();
		return;
	}

	foxit::rdk::uiextensions::PDFAnnotWrapper^ pWrap = (foxit::rdk::uiextensions::PDFAnnotWrapper^)_SelectedAnnot->AnnotHandleWrap->PDFAnnot;
	auto pSdkAnnot = (foxit::pdf::annots::Annot^)pWrap->Pointer;
	foxit::pdf::interform::Control^ _FormControl = dynamic_cast<foxit::pdf::annots::Widget^>(foxit::rdk::FXDocUtil::DynamicCastBaseAnnot(pSdkAnnot))->GetControl();
	foxit::pdf::interform::Field^ _Field = _FormControl->GetField();
	foxit::pdf::interform::Type _Type = _Field->GetType();
	if (_Type == foxit::pdf::interform::Type::e_TypeComboBox || _Type == foxit::pdf::interform::Type::e_TypeListBox)
	{
		HookUpInputPaneEvent();
		uiextensions->LostFocusFormControlHiddenTextBox();

		if (!GetIsDesktopPlatform())
		{
			foxit::pdf::interform::ChoiceOptionArray^ _ChioceOptions = _Field->GetOptions();

			if (m_ItemsGroup)
			{
				m_ItemsGroup->ReleaseFormResource();
				m_ItemsGroup = nullptr;
			}

			if (_ChioceOptions->GetSize() == 0)
			{
				if (ListControlPopup->IsOpen)
					ListControlPopup->IsOpen = false;
				return;
			}

			uiextensions->KillFormfillerFocus();

			m_ItemsGroup = ref new FormControlListItemGroup(_pSdkForm, (foxit::pdf::annots::IAnnot^)pWrap->Pointer);
			uiextensions->SdkFocusFormControl(_SelectedAnnot);

			ListControl->SetPageView(View);
			ListControl->InitDataListSource(m_ItemsGroup);
			if (!ListControlPopup->IsOpen)
				ListControlPopup->IsOpen = true;
			else
			{
				ListControl->UpdateLayout();
				auto _ActualWidth = ListControl->ActualWidth;
				auto _ActualHeight = ListControl->ActualHeight;
				ListControlPopup->VerticalOffset = 0 - _ActualHeight;
			}
		}
	}
	else if (_Type == foxit::pdf::interform::Type::e_TypeTextField)
	{
		FS_INT32 maxLen = _Field->GetMaxLength();
		HookUpInputPaneEvent();
		uiextensions->FocusFormControlHiddenTextBox();
	}
}

void FormOperationBarControl::UpdateControlState()
{
	if (Windows::System::Profile::AnalyticsInfo::VersionInfo->DeviceFamily == L"Windows.Mobile")
	{
		auto pAppView = Windows::UI::ViewManagement::ApplicationView::GetForCurrentView();
		auto _AppViewOrientation = pAppView->Orientation;
		auto _VisibleBounds = pAppView->VisibleBounds;

		if (RootGrid->MaxWidth < _VisibleBounds.Width)
		{
			RootGrid->MaxWidth = _VisibleBounds.Width;
		}

	}

	if (View == nullptr || View->GetDoc() == nullptr)
		return;

	auto uiextensions = (foxit::rdk::uiextensions::UIExtensionsManager^)View->ExtensionsManager;
	auto _pSdkForm = uiextensions->GetSdkForm();
	auto _SelectedAnnot = uiextensions->getAnnotMgr()->GetSelectedAnnot();
	if (_SelectedAnnot == nullptr)
		return;

	foxit::rdk::uiextensions::PDFAnnotWrapper^ pWrap = (foxit::rdk::uiextensions::PDFAnnotWrapper^)_SelectedAnnot->AnnotHandleWrap->PDFAnnot;

	auto pSdkAnnot = (foxit::pdf::annots::Annot^)pWrap->Pointer;
	foxit::pdf::interform::Control^ _FormControl = (dynamic_cast<foxit::pdf::annots::Widget^>(foxit::rdk::FXDocUtil::DynamicCastBaseAnnot(pSdkAnnot)))->GetControl();
	foxit::pdf::interform::Field^ _Field = _FormControl->GetField();
	foxit::pdf::interform::Type _Type = _Field->GetType();
	if (_Type == foxit::pdf::interform::Type::e_TypeComboBox || _Type == foxit::pdf::interform::Type::e_TypeListBox)
	{
		if (ListControlPopup->IsOpen)
		{
			PrevButton->IsEnabled = false;
			NextButton->IsEnabled = false;
		}
		else
		{
			PrevButton->IsEnabled = true;
			NextButton->IsEnabled = true;
		}
	}
	else if (_Type == foxit::pdf::interform::Type::e_TypeTextField)
	{
		uiextensions->FocusFormControlHiddenTextBox();
	}

	UpdateControlPosition();

	FormOperationBarControlClearFieldButton->IsEnabled = (_Type == foxit::pdf::interform::Type::e_TypeTextField || _Type == foxit::pdf::interform::Type::e_TypeCheckBox || _Type == foxit::pdf::interform::Type::e_TypePushButton) ? true : false;
}

void FormOperationBarControl::UpdateControlPosition()
{
	auto _frameworkElement = dynamic_cast<UIElement^>(this);

	if (m_bIsShowInputPane)
	{
		auto _inputPane = Windows::UI::ViewManagement::InputPane::GetForCurrentView();
		auto _inputPaneRect = _inputPane->OccludedRect;
		auto _winRect = Windows::UI::Xaml::Window::Current->Bounds;

		auto _focusElement = dynamic_cast<Windows::UI::Xaml::FrameworkElement^>(Windows::UI::Xaml::Input::FocusManager::GetFocusedElement());
		if (_focusElement == nullptr)
			return;


		if (m_InputPaneTranslateTransform)
		{
			_frameworkElement->RenderTransform = ref new Windows::UI::Xaml::Media::TranslateTransform();
			m_InputPaneTranslateTransform = nullptr;
		}

		Windows::Foundation::Point _screenPt = _focusElement->TransformToVisual(nullptr)->TransformPoint(Point(0, 0));
		auto _focusElementRect = Windows::Foundation::Rect(_screenPt, Windows::Foundation::Size((float)_focusElement->ActualWidth, (float)_focusElement->ActualHeight));

		Windows::Foundation::Point _screenControlPt = this->TransformToVisual(nullptr)->TransformPoint(Point(0, 0));
		auto _controlRect = Windows::Foundation::Rect(_screenControlPt, Windows::Foundation::Size((float)this->ActualWidth, (float)this->ActualHeight));

#if (_DEBUG && 1)
		std::wstringstream ws;
		ws << L"OccludedRect: " << _inputPaneRect.Left << L", " << _inputPaneRect.Top << L", " << _inputPaneRect.Width << L", " << _inputPaneRect.Height << L"\n";
		ws << L"FocusedElement: " << _focusElement->GetType()->FullName->Data() << L", [" << _screenPt.X << L", " << _screenPt.Y << L", " << _focusElement->ActualWidth << L", " << _focusElement->ActualHeight << L"]";
		ws << L"\n";
		OutputDebugString(ws.str().c_str());
#endif

		if (_inputPaneRect.IntersectsWith(_controlRect))
		{
			OutputDebugString(L"IntersectsWith...\n");

			m_InputPaneTranslateTransform = ref new Windows::UI::Xaml::Media::TranslateTransform();
			m_InputPaneTranslateTransform->Y -= _screenControlPt.Y + this->ActualHeight - _inputPaneRect.Top;
			_frameworkElement->RenderTransform = m_InputPaneTranslateTransform;
		}

		_inputPaneRect.Y -= _controlRect.Height;
		_inputPaneRect.Height += _controlRect.Height;
		if (_inputPaneRect.IntersectsWith(_focusElementRect))
		{

		}
	}
	else
	{
		if (m_InputPaneTranslateTransform)
		{
			_frameworkElement->RenderTransform = ref new Windows::UI::Xaml::Media::TranslateTransform();
			m_InputPaneTranslateTransform = nullptr;
		}
	}
}

void FormOperationBarControl::HookUpInputPaneEvent()
{
	auto _InputPane = Windows::UI::ViewManagement::InputPane::GetForCurrentView();

	if (m_InputPaneShowingEventToken.Value == 0)
		m_InputPaneShowingEventToken = _InputPane->Showing += ref new Windows::Foundation::TypedEventHandler<Windows::UI::ViewManagement::InputPane ^, Windows::UI::ViewManagement::InputPaneVisibilityEventArgs ^>(this, &foxit::rdk::uiextensions::FormOperationBarControl::OnShowing);
	if (m_InputPaneHidingEventToken.Value == 0)
		m_InputPaneHidingEventToken = _InputPane->Hiding += ref new Windows::Foundation::TypedEventHandler<Windows::UI::ViewManagement::InputPane ^, Windows::UI::ViewManagement::InputPaneVisibilityEventArgs ^>(this, &foxit::rdk::uiextensions::FormOperationBarControl::OnHiding);
}

void FormOperationBarControl::UnHookInputPaneEvent()
{
	auto _InputPane = Windows::UI::ViewManagement::InputPane::GetForCurrentView();

	if (m_InputPaneShowingEventToken.Value)
	{
		_InputPane->Showing -= m_InputPaneShowingEventToken;
		m_InputPaneShowingEventToken.Value = 0;
	}

	if (m_InputPaneHidingEventToken.Value)
	{
		_InputPane->Hiding -= m_InputPaneHidingEventToken;
		m_InputPaneHidingEventToken.Value = 0;
	}
}

void foxit::rdk::uiextensions::FormOperationBarControl::OnLoaded(Platform::Object ^sender, Windows::UI::Xaml::RoutedEventArgs ^e)
{
	m_WindowSizeChangedEventToken = Windows::UI::Xaml::Window::Current->SizeChanged += ref new Windows::UI::Xaml::WindowSizeChangedEventHandler(this, &foxit::rdk::uiextensions::FormOperationBarControl::OnWindowSizeChanged);
	m_bIsShowInputPane = false;

	UpdateControlState();
	OpenFormListControl();
}


void foxit::rdk::uiextensions::FormOperationBarControl::OnUnloaded(Platform::Object ^sender, Windows::UI::Xaml::RoutedEventArgs ^e)
{
	if (m_WindowSizeChangedEventToken.Value)
	{
		Windows::UI::Xaml::Window::Current->SizeChanged -= m_WindowSizeChangedEventToken;
		m_WindowSizeChangedEventToken.Value = 0;
	}

	UnHookInputPaneEvent();
}


void foxit::rdk::uiextensions::FormOperationBarControl::OnWindowSizeChanged(Platform::Object ^sender, Windows::UI::Core::WindowSizeChangedEventArgs ^e)
{
	UpdateControlState();
}


void foxit::rdk::uiextensions::FormOperationBarControl::PrevButton_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{
	foxit::rdk::uiextensions::UIExtensionsManager^ extMgr = (foxit::rdk::uiextensions::UIExtensionsManager^)View->ExtensionsManager;
	if (extMgr)
	{
		extMgr->FocusPrevFormControl();
		extMgr->ShowWidgetAnnotPopup();
	}
}


void foxit::rdk::uiextensions::FormOperationBarControl::NextButton_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{
	foxit::rdk::uiextensions::UIExtensionsManager^ extMgr = (foxit::rdk::uiextensions::UIExtensionsManager^)View->ExtensionsManager;
	if (extMgr)
	{
		extMgr->FocusNextFormControl();
		extMgr->ShowWidgetAnnotPopup();
	}
}


void foxit::rdk::uiextensions::FormOperationBarControl::FormOperationBarControlClearFieldButton_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{
	foxit::rdk::uiextensions::UIExtensionsManager^ extMgr = (foxit::rdk::uiextensions::UIExtensionsManager^)View->ExtensionsManager;
	if (View == nullptr || View->GetDoc() == nullptr || extMgr->getAnnotMgr() == nullptr)
		return;

	auto _pSdkForm = extMgr->GetSdkForm();
	auto _pSelectedAnnot = extMgr->getAnnotMgr()->GetSelectedAnnot();

	if (_pSdkForm == nullptr || _pSelectedAnnot == nullptr)
		return;

	if (_pSelectedAnnot->AnnotType == foxit::rdk::uiextensions::AnnotationType::Widget)
	{
		foxit::rdk::uiextensions::PDFAnnotWrapper^ pWrap = (foxit::rdk::uiextensions::PDFAnnotWrapper^)_pSelectedAnnot->AnnotHandleWrap->PDFAnnot;
		foxit::pdf::interform::Control^ _FormControl = (dynamic_cast<foxit::pdf::annots::Widget^>(foxit::rdk::FXDocUtil::DynamicCastBaseAnnot((foxit::pdf::annots::Annot^)pWrap->Pointer)))->GetControl();
		if (_FormControl)
		{
			foxit::pdf::interform::Field^ _Field = _FormControl->GetField();
			foxit::pdf::interform::Type _Type = _Field->GetType();
			foxit::pdf::interform::Flags _Flag = (foxit::pdf::interform::Flags)_Field->GetFlags();
			
			String^ _FieldDefaultValue = _Field->GetDefaultValue();
			String^ _FieldValue = _Field->GetValue();

			if (_Type == foxit::pdf::interform::Type::e_TypeCheckBox || _Type == foxit::pdf::interform::Type::e_TypeTextField)
			{
				extMgr->KillFormfillerFocus();
				_Field->Reset();
				if (_FieldValue != L"")
					View->SetModified(true);
				extMgr->SdkFocusFormControl(_pSelectedAnnot);

				View->Redraw();
			}
		}
	}
}


void foxit::rdk::uiextensions::FormOperationBarControl::FormOperationBarControlCompleteButton_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{
	foxit::rdk::uiextensions::UIExtensionsManager^ extMgr = (foxit::rdk::uiextensions::UIExtensionsManager^)View->ExtensionsManager; 
	if (extMgr)
	{
		if (m_ItemsGroup)
		{
			extMgr->KillFormfillerFocus();
			if (m_ItemsGroup->IsDataChanged())
			{
				View->SetModified(true);
				m_ItemsGroup->UpdateToDocument();
			}
			
			m_ItemsGroup->ReleaseFormResource();
			m_ItemsGroup = nullptr;
		}

		extMgr->getAnnotMgr()->SetPdfSelectAnnotNull();
		extMgr->SdkKillFocusFormControl();
		extMgr->LostFocusFormControlHiddenTextBox();
		extMgr->HideWidgetAnnotPopup();
		View->Redraw();
	}
}


void foxit::rdk::uiextensions::FormOperationBarControl::OnShowing(Windows::UI::ViewManagement::InputPane ^sender, Windows::UI::ViewManagement::InputPaneVisibilityEventArgs ^args)
{
	m_PrevInputPaneEnsuredFocusedElementInView = args->EnsuredFocusedElementInView;
	args->EnsuredFocusedElementInView = true;
	m_bIsShowInputPane = true;

	UpdateControlPosition();
}


void foxit::rdk::uiextensions::FormOperationBarControl::OnHiding(Windows::UI::ViewManagement::InputPane ^sender, Windows::UI::ViewManagement::InputPaneVisibilityEventArgs ^args)
{
	args->EnsuredFocusedElementInView = m_PrevInputPaneEnsuredFocusedElementInView;
	m_bIsShowInputPane = false;

	UpdateControlPosition();
}


void foxit::rdk::uiextensions::FormOperationBarControl::ListControlPopup_Opened(Platform::Object^ sender, Platform::Object^ e)
{
	auto _offsetY = ListControlPopup->VerticalOffset;
	auto _ActualWidth = ListControl->ActualWidth;
	auto _ActualHeight = ListControl->ActualHeight;

	ListControlPopup->VerticalOffset = 0 - _ActualHeight;

	UpdateControlState();
}


void foxit::rdk::uiextensions::FormOperationBarControl::BarGrid_Tapped(Platform::Object^ sender, Windows::UI::Xaml::Input::TappedRoutedEventArgs^ e)
{
	auto _OriginalSource = e->OriginalSource;

	if (ListControlPopup->IsOpen)
	{
		if (IsHitElement(e->GetPosition(PrevButton), PrevButton) || IsHitElement(e->GetPosition(NextButton), NextButton))
		{
			return;
		}

		ListControlPopup->IsOpen = false;
	}
}


void foxit::rdk::uiextensions::FormOperationBarControl::ListControlPopup_Closed(Platform::Object^ sender, Platform::Object^ e)
{
	UpdateControlState();
}

bool FormOperationBarControl::IsHitElement(Windows::Foundation::Point pt, Windows::UI::Xaml::FrameworkElement^ frameworkElement)
{
	auto _Rc = Windows::Foundation::Rect(0, 0, (float)frameworkElement->ActualWidth, (float)frameworkElement->ActualHeight);
	
	return _Rc.Contains(pt);
}
