/**
 * Copyright (C) 2003-2018, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit Mobile PDF SDK to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */


#include "pch.h"
#include "PDFFormImplementor.h"

#include <regex>

#define IsLoadFormModule

using namespace foxit::rdk::uiextensions;
using namespace foxit::rdk;

using namespace Platform;
using namespace Windows::Foundation;
using namespace Windows::UI::Xaml;


FSSDK_PDFForm::FSSDK_PDFForm(foxit::pdf::PDFDoc^ pDoc) : m_pDocWeakRef(pDoc)
{

}

foxit::pdf::PDFDoc^ FSSDK_PDFForm::GetDocument()
{
	return m_pDocWeakRef.Resolve<foxit::pdf::PDFDoc>();
}

foxit::pdf::interform::Form^ FSSDK_PDFForm::GetPDFForm()
{
	return m_pPDFForm;
}

foxit::pdf::interform::Filler^ FSSDK_PDFForm::GetFormFiller()
{
	return m_FormFiller;
}

bool FSSDK_PDFForm::Initialize(foxit::rdk::PDFViewCtrl^ viewControl)
{
#ifndef IsLoadFormModule
	return false;
#endif // IsLoadFormModule

	foxit::pdf::PDFDoc^ m_fxDoc = viewControl->GetDoc();
	m_pDocWeakRef = m_fxDoc;
	FS_BOOL hasForm = m_fxDoc->HasForm();
	m_pPDFForm = ref new foxit::pdf::interform::Form(m_fxDoc);

	if (m_pPDFForm == nullptr)
	{
		return false;
	}
	hasForm = m_fxDoc->HasForm();

	m_pFormFillerClientData = new FSSDK_FormFillerClientData(viewControl, GetDocument());
	BeginFormFiller();

	if (AppConfig::IsHighlightFormField)
	{
		SetFormFieldColor(FormFieldType::CHECKBOX, foxit::rdk::uiextensions::Utility::ConvertFromString(L"#80fff160"));
		SetFormFieldColor(FormFieldType::RADIOBUTTON, foxit::rdk::uiextensions::Utility::ConvertFromString(L"#80fff160"));
		SetFormFieldColor(FormFieldType::COMBOBOX, foxit::rdk::uiextensions::Utility::ConvertFromString(L"#80fff160"));
		SetFormFieldColor(FormFieldType::LISTBOX, foxit::rdk::uiextensions::Utility::ConvertFromString(L"#80fff160"));
		SetFormFieldColor(FormFieldType::TEXTFIELD, foxit::rdk::uiextensions::Utility::ConvertFromString(L"#80fff160"));

		SetShowHighlight(true);
	}

	m_fxDoc->DoJSOpenAction();

	m_pPDFForm->SetConstructAppearances(true);
	bool needConstruct = m_pPDFForm->NeedConstructAppearances();
	if (!needConstruct)
		m_pPDFForm->SetConstructAppearances(true);
	
	return true;
}

void FSSDK_PDFForm::Release()
{
	if (m_FormFiller != nullptr)
	{
		delete m_FormFiller;
		m_FormFiller = nullptr;
	}

	if (m_pPDFForm != nullptr)
	{
		delete m_pPDFForm;
		m_pPDFForm = nullptr;
	}
}

void FSSDK_PDFForm::BeginFormFiller()
{
	ActionCallbackImpl^ actionCallback = ref new ActionCallbackImpl(m_pFormFillerClientData);
	foxit::common::Library::SetActionCallback(actionCallback);

	FormFillerAssistCallbackImpl^ assist = ref new FormFillerAssistCallbackImpl(m_pFormFillerClientData);
	m_FormFiller = ref new foxit::pdf::interform::Filler(m_pPDFForm, assist);
}

void FSSDK_PDFForm::EndFormFiller()
{
}

void FSSDK_PDFForm::SetFormFieldColor(FormFieldType field, Windows::UI::Color color)
{
	FS_ARGB rgb = color.A << 24 | color.R << 16 | color.G << 8 | color.B;

	m_FormFiller->SetHighlightColor(rgb);
	
}

void FSSDK_PDFForm::SetShowHighlight(bool bIsShow)
{
	m_FormFiller->HighlightFormFields(bIsShow);
}

PDFFormFillerManagement::PDFFormFillerManagement(foxit::rdk::uiextensions::UIExtensionsManager^ uiMgr)
{
	m_uiManagement = uiMgr;
	m_formRenderToken = uiMgr->viewControl->RenderPageEvent += ref new foxit::rdk::RenderPageEvent(this, &foxit::rdk::uiextensions::PDFFormFillerManagement::OnDrawAnnotEvent);
}

PDFFormFillerManagement::~PDFFormFillerManagement()
{
	m_uiManagement->viewControl->RenderPageEvent -= m_formRenderToken;
}

void foxit::rdk::uiextensions::PDFFormFillerManagement::OnDrawAnnotEvent(foxit::rdk::PDFViewCtrl^ sender, RenderPageEventArgs^ e)
{
	foxit::pdf::PDFDoc^ doc = m_uiManagement->viewControl->GetDoc();
	if (doc)
	{
		foxit::pdf::PDFPage^ pdfPage = doc->GetPage(e->PageIndex);
		m_uiManagement->GetFormFiller()->Render(pdfPage, e->Matrix, e->Render);
	}
}

void PDFFormFillerManagement::Initialize()
{
	m_uiManagement->GetFormFiller();
}

void PDFFormFillerManagement::Finalize()
{
	m_uiManagement->ReleaseFormFiller();
}

void PDFFormFillerManagement::PDFViewControl_PointerPressed(Platform::Object^ sender, Windows::UI::Xaml::Input::PointerRoutedEventArgs^ e)
{
	auto _frameworkelement = safe_cast<FrameworkElement^>(sender);
	bool _hasCapture = (dynamic_cast<PDFViewCanvas^>(sender))->CapturePointer(e->Pointer);

	auto _PointerPt = e->GetCurrentPoint(_frameworkelement);
	Point _Pt = _PointerPt->Position;
	auto _Pageinfo = m_uiManagement->viewControl->ConvertDevicePtToPDFPagePt(_Pt);
	int index = _Pageinfo->PageIndex;
	auto _PagePoint = _Pageinfo->PagePoint;
	if (index == -1)
		return;

	if (!_PointerPt->Properties->IsRightButtonPressed)
	{
		foxit::common::fxcrt::Matrix2D^ matrix = Utility::ConvertToFoxitMatrix(m_uiManagement->viewControl->GetDisplayMatrix(index));

		auto _coreWindow = Window::Current->CoreWindow;
		auto _downState = Windows::UI::Core::CoreVirtualKeyStates::Down;
		bool _controlKey = (_coreWindow->GetKeyState(Windows::System::VirtualKey::Control) & _downState) == _downState;
		bool _shiftKey = (_coreWindow->GetKeyState(Windows::System::VirtualKey::Shift) & _downState) == _downState;
		bool _altKey = (_coreWindow->GetKeyState(Windows::System::VirtualKey::Menu) & _downState) == _downState;

		foxit::common::fxcrt::PointF^ point = ref new foxit::common::fxcrt::PointF((FS_FLOAT)(_Pt.X*m_uiManagement->viewControl->CompositionScaleX),
			(FS_FLOAT)(_Pt.Y*m_uiManagement->viewControl->CompositionScaleY));
		foxit::common::fxcrt::Matrix2D^ reverse = ref new foxit::common::fxcrt::Matrix2D();
		reverse->SetReverse(matrix);
		reverse->TransformPoint(point);

		uint32 flags = 0;
		if (_controlKey)
			flags |= (uint32)foxit::pdf::interform::EventFlags::e_EventFlagControlKey;
		if (_shiftKey)
			flags |= (uint32)foxit::pdf::interform::EventFlags::e_EventFlagShiftKey;
		if (_altKey)
			flags |= (uint32)foxit::pdf::interform::EventFlags::e_EventFlagAltKey;

		foxit::pdf::PDFDoc^ doc = m_uiManagement->viewControl->GetDoc();
		foxit::pdf::PDFPage^ pdfPage = doc->GetPage(index);

		if (_PointerPt->Properties->IsLeftButtonPressed)
			m_uiManagement->GetFormFiller()->OnLButtonDown(pdfPage, point, flags);

		if (_PointerPt->Properties->IsRightButtonPressed)
			m_uiManagement->GetFormFiller()->OnRButtonDown(pdfPage, point, flags);
	}
}
void PDFFormFillerManagement::PDFViewControl_PointerMoved(Platform::Object^ sender, Windows::UI::Xaml::Input::PointerRoutedEventArgs^ e)
{
	auto _frameworkelement = safe_cast<FrameworkElement^>(sender);
	bool _hasCapture = (dynamic_cast<PDFViewCanvas^>(sender))->CapturePointer(e->Pointer);

	auto _PointerPt = e->GetCurrentPoint(_frameworkelement);
	Point _Pt = _PointerPt->Position;
	auto _Pageinfo = m_uiManagement->viewControl->ConvertDevicePtToPDFPagePt(_Pt);
	int index = _Pageinfo->PageIndex;
	auto _PagePoint = _Pageinfo->PagePoint;
	if (index == -1)
		return;

	if (!_PointerPt->Properties->IsRightButtonPressed)
	{
		foxit::common::fxcrt::Matrix2D^ matrix = Utility::ConvertToFoxitMatrix(m_uiManagement->viewControl->GetDisplayMatrix(index));

		auto _coreWindow = Window::Current->CoreWindow;
		auto _downState = Windows::UI::Core::CoreVirtualKeyStates::Down;
		bool _controlKey = (_coreWindow->GetKeyState(Windows::System::VirtualKey::Control) & _downState) == _downState;
		bool _shiftKey = (_coreWindow->GetKeyState(Windows::System::VirtualKey::Shift) & _downState) == _downState;
		bool _altKey = (_coreWindow->GetKeyState(Windows::System::VirtualKey::Menu) & _downState) == _downState;

		foxit::common::fxcrt::PointF^ point = ref new foxit::common::fxcrt::PointF((FS_FLOAT)(_Pt.X*m_uiManagement->viewControl->CompositionScaleX),
			(FS_FLOAT)(_Pt.Y*m_uiManagement->viewControl->CompositionScaleY));
		//Convert to pdf page point.
		foxit::common::fxcrt::Matrix2D^ reverse = ref new foxit::common::fxcrt::Matrix2D();
		reverse->SetReverse(matrix);
		reverse->TransformPoint(point);

		uint32 flags = 0;
		if (_controlKey)
			flags |= (uint32)foxit::pdf::interform::EventFlags::e_EventFlagControlKey;
		if (_shiftKey)
			flags |= (uint32)foxit::pdf::interform::EventFlags::e_EventFlagShiftKey;
		if (_altKey)
			flags |= (uint32)foxit::pdf::interform::EventFlags::e_EventFlagAltKey;

		foxit::pdf::PDFDoc^ doc = m_uiManagement->viewControl->GetDoc();
		foxit::pdf::PDFPage^ pdfPage = doc->GetPage(index);
		m_uiManagement->GetFormFiller()->OnMouseMove(pdfPage, point, flags);
	}
}
void PDFFormFillerManagement::PDFViewControl_PointerReleased(Platform::Object^ sender, Windows::UI::Xaml::Input::PointerRoutedEventArgs^ e)
{
	auto _frameworkelement = safe_cast<FrameworkElement^>(sender);
	bool _hasCapture = (dynamic_cast<PDFViewCanvas^>(sender))->CapturePointer(e->Pointer);

	auto _PointerPt = e->GetCurrentPoint(_frameworkelement);
	Point _Pt = _PointerPt->Position;
	auto _Pageinfo = m_uiManagement->viewControl->ConvertDevicePtToPDFPagePt(_Pt);
	int index = _Pageinfo->PageIndex;
	auto _PagePoint = _Pageinfo->PagePoint;

	if (!_PointerPt->Properties->IsRightButtonPressed)
	{
		foxit::common::fxcrt::Matrix2D^ matrix = Utility::ConvertToFoxitMatrix(m_uiManagement->viewControl->GetDisplayMatrix(index));

		auto _coreWindow = Window::Current->CoreWindow;
		auto _downState = Windows::UI::Core::CoreVirtualKeyStates::Down;
		bool _controlKey = (_coreWindow->GetKeyState(Windows::System::VirtualKey::Control) & _downState) == _downState;
		bool _shiftKey = (_coreWindow->GetKeyState(Windows::System::VirtualKey::Shift) & _downState) == _downState;
		bool _altKey = (_coreWindow->GetKeyState(Windows::System::VirtualKey::Menu) & _downState) == _downState;

		foxit::common::fxcrt::PointF^ point = ref new foxit::common::fxcrt::PointF((FS_FLOAT)(_Pt.X*m_uiManagement->viewControl->CompositionScaleX),
			(FS_FLOAT)(_Pt.Y*m_uiManagement->viewControl->CompositionScaleY));
		//Convert to pdf page point.
		foxit::common::fxcrt::Matrix2D^ reverse = ref new foxit::common::fxcrt::Matrix2D();
		reverse->SetReverse(matrix);
		reverse->TransformPoint(point);

		uint32 flags = 0;
		if (_controlKey)
			flags |= (uint32)foxit::pdf::interform::EventFlags::e_EventFlagControlKey;
		if (_shiftKey)
			flags |= (uint32)foxit::pdf::interform::EventFlags::e_EventFlagShiftKey;
		if (_altKey)
			flags |= (uint32)foxit::pdf::interform::EventFlags::e_EventFlagAltKey;

		foxit::pdf::PDFDoc^ doc = m_uiManagement->viewControl->GetDoc();
		foxit::pdf::PDFPage^ pdfPage = doc->GetPage(index);
		m_uiManagement->GetFormFiller()->OnLButtonUp(pdfPage, point, flags);
	}
}
void PDFFormFillerManagement::OnPointerWheelChanged(Platform::Object ^sender, Windows::UI::Xaml::Input::PointerRoutedEventArgs ^e)
{

}
void PDFFormFillerManagement::SdkFocusFormControl(IBaseAnnot^ pAnnot)
{

}
void PDFFormFillerManagement::SdkKillFocusFormControl()
{

}

foxit::rdk::uiextensions::ActionCallbackImpl::ActionCallbackImpl(FSSDK_FormFillerClientData * clientData)
{
	m_clientData = clientData;
}

foxit::rdk::uiextensions::ActionCallbackImpl::~ActionCallbackImpl()
{
	m_clientData = nullptr;
}

bool foxit::rdk::uiextensions::ActionCallbackImpl::InvalidateRect(foxit::pdf::PDFDoc ^document, int page_index, foxit::common::fxcrt::RectF ^pdf_rect)
{
	if (!m_clientData)
		return false;

	std::wstringstream ws;
	ws << L"InvalidateWindowRect: " << page_index << L", Rect: [" << pdf_rect->left << L", " << pdf_rect->top << L", " << pdf_rect->right << L", " << pdf_rect->bottom << L"]";
	ws << L"\n";
	OutputDebugString(ws.str().c_str());

	auto _Document = m_clientData->Document();
	if (_Document == nullptr)
		return false;

	auto pageView = m_clientData->ViewCtrl();
	if (pageView)
	{
		pageView->Update();
	}

	return true;
}

int foxit::rdk::uiextensions::ActionCallbackImpl::GetCurrentPage(foxit::pdf::PDFDoc ^document)
{
	OutputDebugString((L"JS Callback---GetCurrentPageIndex: " + L"\n")->Data());

	if (m_clientData == nullptr)
		return -1;

	auto _Document = m_clientData->Document();
	if (_Document == nullptr)
		return -1;

	auto pageView = m_clientData->ViewCtrl();
	if (pageView)
	{
		return pageView->GetCurrentPage();
	}

	return -1; 
}

void foxit::rdk::uiextensions::ActionCallbackImpl::SetCurrentPage(foxit::pdf::PDFDoc ^document, int page_index)
{
	OutputDebugString((L"JS Callback---SetCurrentPageIndex: " + page_index + L"\n")->Data());

	if (m_clientData == nullptr) return;

	auto _Document = m_clientData->Document();
	if (_Document == nullptr) return;

	auto pageView = m_clientData->ViewCtrl();
	if (pageView)
	{
		if (page_index >= 0 && page_index < _Document->GetPageCount())
		{
			pageView->GotoPage(page_index);
		}
		else
		{
			return;
		}
	}
}

foxit::common::Rotation foxit::rdk::uiextensions::ActionCallbackImpl::GetPageRotation(foxit::pdf::PDFDoc ^document, int page_index)
{
	return document->GetPage(page_index)->GetRotation();
}

bool foxit::rdk::uiextensions::ActionCallbackImpl::SetPageRotation(foxit::pdf::PDFDoc ^document, int page_index, foxit::common::Rotation rotation)
{
	return false;
}

bool foxit::rdk::uiextensions::ActionCallbackImpl::ExecuteNamedAction(foxit::pdf::PDFDoc ^document, Platform::String ^named_action)
{
	String^ _str = L"ExecuteNamedAction\n\n" + named_action;
	OutputDebugString((L"JS Callback---ExecuteNamedAction: " + _str/*ref new Platform::String(str.c_str())*/ + L"\n")->Data());

	return true;
}

bool foxit::rdk::uiextensions::ActionCallbackImpl::SetDocChangeMark(foxit::pdf::PDFDoc ^document, bool change_mark)
{
	OutputDebugString((L"JS Callback---MySetChangeMark: " + change_mark + L"\n")->Data());

	if (!m_clientData)
		return false;

	auto _Document = m_clientData->Document();
	if (_Document == nullptr)
		return false;

	if (change_mark)
	{
		m_clientData->ViewCtrl()->SetModified(true);
	}

	return true;
}

bool foxit::rdk::uiextensions::ActionCallbackImpl::GetDocChangeMark(foxit::pdf::PDFDoc ^document)
{
	OutputDebugString((L"JS Callback---MyGetChangeMark: " + L"\n")->Data());

	if (m_clientData == nullptr)
		return false;

	auto _Document = m_clientData->Document();
	if (_Document == nullptr)
		return false;

	//return _Document->IsModified();
	return false;
}

int foxit::rdk::uiextensions::ActionCallbackImpl::GetOpenedDocCount()
{
	if (m_clientData == nullptr)
		return 0;

	auto _Document = m_clientData->Document();
	if (_Document == nullptr)
		return 0;
	return 1;
}

foxit::pdf::PDFDoc ^ foxit::rdk::uiextensions::ActionCallbackImpl::GetOpenedDoc(int index)
{
	if (m_clientData == nullptr)
		return nullptr;

	auto _Document = m_clientData->Document();
	if (_Document == nullptr)
		return nullptr;

	return _Document;
}

bool foxit::rdk::uiextensions::ActionCallbackImpl::Beep(int type)
{
	std::wstring funname(L"Beep\n\n");
	std::wstring strType;
	switch (type)
	{
	case 0:
		strType = L"Error";
		break;
	case 1:
		strType = L"Warning";
		break;
	case 2:
		strType = L"Question";
		break;
	case 3:
		strType = L"Status";
		break;
	case 4:
		strType = L"Default";
		break;
	default:
		strType = L"not defined";
	}
	std::wstring str = funname + strType;

	OutputDebugString((L"JS Callback---Beep: " + ref new Platform::String(str.c_str()) + L"\n")->Data());

	return true;
}

Platform::String ^ foxit::rdk::uiextensions::ActionCallbackImpl::Response(Platform::String ^question, Platform::String ^title, Platform::String ^default_value, Platform::String ^label, bool is_password)
{
	String^ response = L"no answers.";

	OutputDebugString((L"JS Callback---Response: " + title + L", Default" + default_value + L", Label" + label + L"\n")->Data());

	return response;
}

Platform::String ^ foxit::rdk::uiextensions::ActionCallbackImpl::GetFilePath(foxit::pdf::PDFDoc ^document)
{
	if (m_clientData == nullptr)
		return nullptr;

	auto _Document = m_clientData->Document();
	if (_Document == nullptr)
		return nullptr;


	OutputDebugString((L"JS Callback---GetFilePath: " + L"\n")->Data());

	return m_clientData->ViewCtrl()->GetStorageFile()->Path;
}

bool foxit::rdk::uiextensions::ActionCallbackImpl::Print(foxit::pdf::PDFDoc ^document, bool is_ui, foxit::common::Range^ page_range, bool is_silent, bool is_shrunk_to_fit, bool is_printed_as_image, bool is_reversed, bool is_to_print_annots)
{
	wchar_t strPrint[1024];
	swprintf_s(strPrint, L"bUI: %d\nrange: from %d to %d\nbSilent: %d\nbShrinkToFit: %d\nbPrintAsImage: %d\nbReverse: %d\nbAnnotations: %d",
		is_ui, page_range->GetSegmentStart(0), page_range->GetSegmentEnd(0), is_silent, is_shrunk_to_fit, is_printed_as_image, is_reversed, is_to_print_annots);

	OutputDebugString((L"JS Callback---Print: " + ref new Platform::String(strPrint) + L"\n")->Data());

	return true;
}

bool foxit::rdk::uiextensions::ActionCallbackImpl::SubmitForm(foxit::pdf::PDFDoc ^document, const Platform::Array<unsigned char> ^form_data, Platform::String ^url)
{
	Platform::String^ _strURL = L"SubmitForm URL\n\n" + url;// ref new Platform::String(strURL.c_str());
	OutputDebugString((L"JS Callback---SubmitForm: " + _strURL + L"\n")->Data());

	return false;
}

bool foxit::rdk::uiextensions::ActionCallbackImpl::LaunchURL(Platform::String ^url)
{
	if (!m_clientData)
		return false;

	auto _Document = m_clientData->Document();
	if (_Document == nullptr)
		return false;

	auto urlStr = url;
	if (urlStr->Length() == 0)
		return true;

	auto uiDelegate = [=]()
	{
		std::wstring wsUrlStr = urlStr->Data();
		std::wregex urlPattern(L"^([A-Za-z]+://){1}[A-Za-z0-9-_]+\\.[A-Za-z0-9-_%&\\?/.=:\"{}]+$");
		if (!std::regex_match(wsUrlStr, urlPattern))
		{
			wsUrlStr = L"http://" + wsUrlStr;
		}
		Platform::String^ _UrlStr = ref new Platform::String(wsUrlStr.c_str());

		if (AppConfig::IsAllowLanchUrlForForm)
		{
			try
			{
				Windows::System::Launcher::LaunchUriAsync(ref new Windows::Foundation::Uri(_UrlStr));
			}
			catch (...)
			{
				OutputDebugString(L"LaunchUriAsync...URL...Exception...\n");
			}
		}
		else
		{
			auto _resourceLoader = Windows::ApplicationModel::Resources::ResourceLoader::GetForCurrentView("foxit.rdk.uiextensions/Resources");
			Platform::String^ saveYes = _resourceLoader->GetString(L"SaveYesText");
			Platform::String^ saveNo = _resourceLoader->GetString(L"SaveNoText");
			Platform::String^ _lanchUrlTitle = _resourceLoader->GetString(L"FormLaunchUrlTitle");
			Platform::String^ _contentFormatStr = _resourceLoader->GetString(L"FormLaunchUrlContentFormat");
			std::wstring wsContent = foxit::rdk::uiextensions::wstring_format(_contentFormatStr->Data(), _UrlStr->Data());

			std::wstring src = L"\\n";
			std::wstring des = L"\n";
			size_t pos = -1;
			size_t curPos = 0;

			while (-1 != (pos = wsContent.find(L"\\n", curPos)))
			{
				wsContent.replace(pos, src.size(), des);
				curPos = pos + des.size();
			}

			auto msgDialog = ref new Windows::UI::Popups::MessageDialog(ref new Platform::String(wsContent.c_str()), _lanchUrlTitle);
			msgDialog->Commands->Append(ref new Windows::UI::Popups::UICommand(saveYes, nullptr, PropertyValue::CreateInt32(0)));
			msgDialog->Commands->Append(ref new Windows::UI::Popups::UICommand(saveNo, nullptr, PropertyValue::CreateInt32(1)));
			msgDialog->DefaultCommandIndex = 0;

			concurrency::create_task(msgDialog->ShowAsync())
				.then([=](Windows::UI::Popups::IUICommand^ command)
			{
				if (command && command->Id != nullptr)
				{
					int const id = safe_cast<IBox<int>^>(command->Id)->Value;
					if (id == 0)
					{
						Windows::System::Launcher::LaunchUriAsync(ref new Windows::Foundation::Uri(_UrlStr));
					}
				}
			})
				.then([=](concurrency::task<void> t)
			{
				try
				{
					t.get();
				}
				catch (...)
				{
					OutputDebugString(L"ShowAsync...URL...Exception...\n");
				}
			});
		}
	};

	auto _Dispatcher = Windows::ApplicationModel::Core::CoreApplication::MainView->CoreWindow->Dispatcher;

	_Dispatcher->RunAsync(Windows::UI::Core::CoreDispatcherPriority::Normal, ref new Windows::UI::Core::DispatchedHandler(uiDelegate));

	return true;;
}

Platform::String ^ foxit::rdk::uiextensions::ActionCallbackImpl::BrowseFile()
{
	OutputDebugString((L"JS Callback---BrowseFile: " + L"\n")->Data());

	return nullptr;
}

foxit::pdf::actions::Language foxit::rdk::uiextensions::ActionCallbackImpl::GetLanguage()
{
	return foxit::pdf::actions::Language();
}

int foxit::rdk::uiextensions::ActionCallbackImpl::Alert(Platform::String ^msg, Platform::String ^title, int type, int icon)
{
	auto msgStr = msg;
	auto _TitleStr = title;

	if (!AppConfig::IsAllowPopupAlertDialog)
		return 0;

	auto uiDelegate = [=]()
	{
		if (msgStr->Length() == 0)
			return;

		Platform::String^ titleStr = _TitleStr;
		if (_TitleStr->Length() == 0)
		{
			auto _resourceLoader = Windows::ApplicationModel::Resources::ResourceLoader::GetForCurrentView("foxit.rdk.uiextensions/Resources");
			titleStr = _resourceLoader->GetString(L"FormAlertTitle");
		}

		auto msgDialog = ref new Windows::UI::Popups::MessageDialog(msgStr, titleStr);
		auto ret = concurrency::create_task(msgDialog->ShowAsync())
			.then([=](concurrency::task<Windows::UI::Popups::IUICommand^> t)
		{
			try
			{
				t.get();
			}
			catch (...)
			{
			}
		});
	};

	auto _Dispatcher = Windows::ApplicationModel::Core::CoreApplication::MainView->CoreWindow->Dispatcher;

	_Dispatcher->RunAsync(Windows::UI::Core::CoreDispatcherPriority::Normal, ref new Windows::UI::Core::DispatchedHandler(uiDelegate));

	return 0;
}

foxit::pdf::actions::IdentityProperties ^ foxit::rdk::uiextensions::ActionCallbackImpl::GetIdentityProperties()
{
	return nullptr;
}

Platform::String ^ foxit::rdk::uiextensions::ActionCallbackImpl::PopupMenu(foxit::pdf::actions::MenuListArray ^menus)
{
	return nullptr;
}

Platform::String ^ foxit::rdk::uiextensions::ActionCallbackImpl::GetAppInfo(foxit::pdf::actions::AppInfoType type)
{
	Platform::String^ csInfo;
	switch (type)
	{
	case foxit::pdf::actions::AppInfoType::e_AppInfoTypeFormsVersion:
		csInfo = L"7.3";
		break;
	case foxit::pdf::actions::AppInfoType::e_AppInfoTypeViewerType:
		csInfo = L"foxit sdk ui demo for windows";
		break;
	case foxit::pdf::actions::AppInfoType::e_AppInfoTypeViewerVariation:
		csInfo = L"full expired";
		break;
	case foxit::pdf::actions::AppInfoType::e_AppInfoTypeViewerVersion:
		csInfo = L"6.5";
		break;
	case foxit::pdf::actions::AppInfoType::e_AppInfoTypeAppVersion:
		csInfo = L"5.6";
		break;
	}

	return csInfo;
}

bool foxit::rdk::uiextensions::ActionCallbackImpl::MailData(Platform::Object ^data, foxit::pdf::actions::MailType data_type, bool is_ui, Platform::String ^to, Platform::String ^subject, Platform::String ^cc, Platform::String ^bcc, Platform::String ^message)
{
	Platform::String^ csMessageType;
	switch (data_type)
	{
	case foxit::pdf::actions::MailType::e_MailTypeDoc:
		csMessageType = L"call MailData() to mail doc";
		break;
	case foxit::pdf::actions::MailType::e_MailTypeForm:
		csMessageType = L"call MailData() to mail form";
		break;
	case foxit::pdf::actions::MailType::e_MailTypeMsg:
		csMessageType = L"call MailData() to mail msg";
		break;
	}

	return true;
}

bool foxit::rdk::uiextensions::ActionCallbackImpl::Release()
{
	return true;
}

foxit::rdk::uiextensions::FormFillerAssistCallbackImpl::FormFillerAssistCallbackImpl(FSSDK_FormFillerClientData * clientData)
{
	m_clientData = clientData;
}

foxit::rdk::uiextensions::FormFillerAssistCallbackImpl::~FormFillerAssistCallbackImpl()
{
	if (m_clientData) {
		delete m_clientData;
		m_clientData = nullptr;
	}
}

void foxit::rdk::uiextensions::FormFillerAssistCallbackImpl::Refresh(foxit::pdf::PDFPage ^page, foxit::common::fxcrt::RectF ^rect)
{
	if (!m_clientData)
		return ;

	auto _Document = m_clientData->Document();
	if (_Document == nullptr)
		return;

	auto pageView = m_clientData->ViewCtrl();
	if (pageView)
	{
		pageView->Redraw();
	}

	return;
}

bool foxit::rdk::uiextensions::FormFillerAssistCallbackImpl::SetTimerCallback(int elapse, foxit::pdf::interform::ITimerCallback ^timer, int* out_timer_id)
{
	auto _Document = m_clientData->Document();
	if (_Document == nullptr)
		return false;

	auto mgr = m_clientData->GetUIViewCtrl();
	if (mgr)
	{
		mgr->CreateTimer(m_clientData, elapse, timer);
	}

	*out_timer_id = (int)(UINT_PTR) m_clientData;
	return true;
}

bool foxit::rdk::uiextensions::FormFillerAssistCallbackImpl::KillTimer(int timer_id)
{
	auto _Document = m_clientData->Document();
	if (_Document == nullptr)
		return false;

	auto mgr = m_clientData->GetUIViewCtrl();
	if (mgr)
	{
		mgr->DestroyTime(timer_id); 
	}

	return true;
}

void foxit::rdk::uiextensions::FormFillerAssistCallbackImpl::FocusGotOnControl(foxit::pdf::interform::Control ^control, Platform::String ^value)
{

}

void foxit::rdk::uiextensions::FormFillerAssistCallbackImpl::FocusLostFromControl(foxit::pdf::interform::Control ^control, Platform::String ^value)
{
	
}

int foxit::rdk::uiextensions::FormFillerAssistCallbackImpl::GetVersion()
{
	return 1;
}

bool foxit::rdk::uiextensions::FormFillerAssistCallbackImpl::Release()
{
	return true;
}
