/**
 * Copyright (C) 2003-2018, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit Mobile PDF SDK to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */


#pragma once

namespace foxit
{
	namespace rdk
	{
		namespace uiextensions
		{
			class FSSDK_FormFillerClientData
			{
			public:
				FSSDK_FormFillerClientData(foxit::rdk::PDFViewCtrl^ viewControl, foxit::pdf::PDFDoc^ doc)
				{
					m_Document = doc;
					m_Control = viewControl;
					m_UIViewMgrWeakRef = viewControl->ExtensionsManager;
				}

				~FSSDK_FormFillerClientData()
				{

				}

				foxit::pdf::PDFDoc^ Document()
				{
					return m_Document.Resolve<foxit::pdf::PDFDoc>();
				}

				foxit::rdk::PDFViewCtrl^ ViewCtrl()
				{
					return m_Control.Resolve<foxit::rdk::PDFViewCtrl>();
				}

				foxit::rdk::uiextensions::UIExtensionsManager^ GetUIViewCtrl()
				{
					return m_UIViewMgrWeakRef.Resolve<foxit::rdk::uiextensions::UIExtensionsManager>();
				}

			private:
				Platform::WeakReference m_Document;
				Platform::WeakReference m_Control;
				Platform::WeakReference m_UIViewMgrWeakRef;
			};

			class PDFFormImplementor
			{
			public:
				PDFFormImplementor() {}
				virtual ~PDFFormImplementor() {}

				virtual bool Initialize(foxit::rdk::PDFViewCtrl^ viewControl) = 0;
				virtual void Release() = 0;
				virtual void BeginFormFiller() = 0;
				virtual void EndFormFiller() = 0;
			};

			class FSSDK_FormFillerClientData;
			class FSSDK_PDFForm : public PDFFormImplementor
			{
			public:
				FSSDK_PDFForm(foxit::pdf::PDFDoc^ pDoc);

				bool Initialize(foxit::rdk::PDFViewCtrl^ viewControl);
				void Release();

				void BeginFormFiller();
				void EndFormFiller();

				void SetFormFieldColor(FormFieldType field, Windows::UI::Color color);
				void SetShowHighlight(bool bIsShow);

				foxit::pdf::PDFDoc^ GetDocument();
				foxit::pdf::interform::Form^ GetPDFForm();
				foxit::pdf::interform::Filler^ GetFormFiller();

			private:
				foxit::pdf::interform::Form^ m_pPDFForm;
				foxit::pdf::interform::Filler^ m_FormFiller;
				foxit::pdf::interform::IFillerAssistCallback^ m_forminterAction;
				FSSDK_FormFillerClientData* m_pFormFillerClientData;
				Platform::WeakReference m_pDocWeakRef;
			};

			interface class IBaseAnnot;
			ref class PDFFormFillerManagement
			{
			internal:
				PDFFormFillerManagement(foxit::rdk::uiextensions::UIExtensionsManager^ uiMgr);
				void Initialize();
				void Finalize();

				void PDFViewControl_PointerPressed(Platform::Object^ sender, Windows::UI::Xaml::Input::PointerRoutedEventArgs^ e);
				void PDFViewControl_PointerMoved(Platform::Object^ sender, Windows::UI::Xaml::Input::PointerRoutedEventArgs^ e);
				void PDFViewControl_PointerReleased(Platform::Object^ sender, Windows::UI::Xaml::Input::PointerRoutedEventArgs^ e);
				void OnPointerWheelChanged(Platform::Object ^sender, Windows::UI::Xaml::Input::PointerRoutedEventArgs ^e);
				void SdkFocusFormControl(IBaseAnnot^ pAnnot);
				void SdkKillFocusFormControl();

				void OnDrawAnnotEvent(PDFViewCtrl^ sender, RenderPageEventArgs^ e);
			private:
				~PDFFormFillerManagement();
			private:
				foxit::rdk::uiextensions::UIExtensionsManager^ m_uiManagement;
				Windows::Foundation::EventRegistrationToken m_formRenderToken;
			};

			ref class ActionCallbackImpl sealed : foxit::pdf::actions::IActionCallback {
			public:
				virtual ~ActionCallbackImpl();
				virtual bool InvalidateRect(foxit::pdf::PDFDoc ^document, int page_index, foxit::common::fxcrt::RectF ^pdf_rect);
				virtual int GetCurrentPage(foxit::pdf::PDFDoc ^document);
				virtual void SetCurrentPage(foxit::pdf::PDFDoc ^document, int page_index);
				virtual foxit::common::Rotation GetPageRotation(foxit::pdf::PDFDoc ^document, int page_index);
				virtual bool SetPageRotation(foxit::pdf::PDFDoc ^document, int page_index, foxit::common::Rotation rotation);
				virtual bool ExecuteNamedAction(foxit::pdf::PDFDoc ^document, Platform::String ^named_action);
				virtual bool SetDocChangeMark(foxit::pdf::PDFDoc ^document, bool change_mark);
				virtual bool GetDocChangeMark(foxit::pdf::PDFDoc ^document);
				virtual int GetOpenedDocCount();
				virtual foxit::pdf::PDFDoc ^ GetOpenedDoc(int index);
				virtual bool Beep(int type);
				virtual Platform::String ^ Response(Platform::String ^question, Platform::String ^title, Platform::String ^default_value, Platform::String ^label, bool is_password);
				virtual Platform::String ^ GetFilePath(foxit::pdf::PDFDoc ^document);
				virtual bool Print(foxit::pdf::PDFDoc ^document, bool is_ui, foxit::common::Range^ page_range, bool is_silent, bool is_shrunk_to_fit, bool is_printed_as_image, bool is_reversed, bool is_to_print_annots);
				virtual bool SubmitForm(foxit::pdf::PDFDoc ^document, const Platform::Array<unsigned char> ^form_data, Platform::String ^url);
				virtual bool LaunchURL(Platform::String ^url);
				virtual Platform::String ^ BrowseFile();
				virtual foxit::pdf::actions::Language GetLanguage();
				virtual int Alert(Platform::String ^msg, Platform::String ^title, int type, int icon);
				virtual foxit::pdf::actions::IdentityProperties ^ GetIdentityProperties();
				virtual Platform::String ^ PopupMenu(foxit::pdf::actions::MenuListArray ^menus);
				virtual Platform::String ^ GetAppInfo(foxit::pdf::actions::AppInfoType type);
				virtual bool MailData(Platform::Object ^data, foxit::pdf::actions::MailType data_type, bool is_ui, Platform::String ^to, Platform::String ^subject, Platform::String ^cc, Platform::String ^bcc, Platform::String ^message);
				virtual bool Release();

			internal:
				ActionCallbackImpl(FSSDK_FormFillerClientData* clientData);
				FSSDK_FormFillerClientData* m_clientData;
			};

			ref class FormFillerAssistCallbackImpl sealed : foxit::pdf::interform::IFillerAssistCallback {
			public:
				virtual ~FormFillerAssistCallbackImpl();
				virtual void Refresh(foxit::pdf::PDFPage ^page, foxit::common::fxcrt::RectF ^rect);
				virtual bool SetTimerCallback(int elapse, foxit::pdf::interform::ITimerCallback ^timer, int* out_timer_id);
				virtual bool KillTimer(int timer_id);
				virtual void FocusGotOnControl(foxit::pdf::interform::Control ^control, Platform::String ^value);
				virtual void FocusLostFromControl(foxit::pdf::interform::Control ^control, Platform::String ^value);
				virtual int GetVersion();
				virtual bool Release();

			internal:
				FormFillerAssistCallbackImpl(FSSDK_FormFillerClientData* clientData);
				FSSDK_FormFillerClientData* m_clientData;
			};
		}
	}
}

