﻿/**
 * Copyright (C) 2003-2018, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit Mobile PDF SDK to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#include "pch.h"
#include "MainPageAppBar.xaml.h"
#include "Bookmark/BookMarkPage.xaml.h"
#include "Text/FindTextControl.xaml.h"
#include "FloatWindow/ViewToolAppBar.xaml.h"
#include "FloatWindow/ReflowViewAppBar.xaml.h"
#include "Annotations/AnnotToolAppBar.xaml.h"
#include "Annotations/FreeTextEventPeer.h"
#include "Common/ToolBarButton.h"
#include "FloatWindow/EyeProtectionModeControl.xaml.h"

using namespace foxit::rdk::uiextensions;

using namespace Platform;
using namespace Windows::Foundation;
using namespace Windows::Foundation::Collections;
using namespace Windows::UI::Xaml;
using namespace Windows::UI::Xaml::Controls;
using namespace Windows::UI::Xaml::Controls::Primitives;
using namespace Windows::UI::Xaml::Data;
using namespace Windows::UI::Xaml::Input;
using namespace Windows::UI::Xaml::Media;
using namespace Windows::UI::Xaml::Navigation;


MainPageAppBar::MainPageAppBar(UIExtensionsManager^ pUIView)
{
	InitializeComponent();

	m_UIViewWeakRef = pUIView;
	m_RdkViewWeakRef = pUIView->viewControl;
	m_curAppbarType = foxit::rdk::uiextensions::AppBarType::None;
	m_isOpen = false;
	m_RotatePopup = ref new Windows::UI::Xaml::Controls::Primitives::Popup();
	m_TipPopup = ref new Windows::UI::Xaml::Controls::Primitives::Popup();
	
	InitTimerClick();
	m_bIsFlyoutShow = false;
	m_isStopTick = false;
	viewtoolPanelPopup = ref new Windows::UI::Xaml::Controls::Primitives::Popup();
	_isCPDFFlyoutOpen = false;

	secondaryCommand->RelateMainAppBar(this);
	AppBarSlowHideStoryboard->Completed += ref new Windows::Foundation::EventHandler<Platform::Object ^>(this, &foxit::rdk::uiextensions::MainPageAppBar::OnHideAppBarAnimationCompleted);
	

	__windowSizeChangedEventRegistrationToken = Window::Current->SizeChanged += ref new Windows::UI::Xaml::WindowSizeChangedEventHandler(this, &foxit::rdk::uiextensions::MainPageAppBar::OnSizeChanged);
}

void MainPageAppBar::InitTimerClick()
{
	m_HideTimer = ref new Windows::UI::Xaml::DispatcherTimer();
	Windows::Foundation::TimeSpan _HideTimerTimeSpan;
	_HideTimerTimeSpan.Duration = 5 * 10000000; 
	m_HideTimer->Interval = _HideTimerTimeSpan;
	m_HideTimer->Tick += ref new Windows::Foundation::EventHandler<Platform::Object ^>(this, &MainPageAppBar::OnTick);
	m_HideTimer->Start();
	OutputDebugString(L"m_HideTimer Start\n");


	this->Unloaded += ref new Windows::UI::Xaml::RoutedEventHandler([this](Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
	{
		if (m_HideTimer && m_HideTimer->IsEnabled)
		{
			m_HideTimer->Stop();
		}
	});
}

PDFViewCtrl^ MainPageAppBar::View::get()
{
	return UIExtMgr->viewControl;
}

UIExtensionsManager^ MainPageAppBar::UIExtMgr::get()
{
	return m_UIViewWeakRef.Resolve<UIExtensionsManager>();
}

foxit::rdk::uiextensions::AppBarType MainPageAppBar::GetCurAppBarType()
{
	return m_curAppbarType;
}

void MainPageAppBar::InitMainAppbar(foxit::rdk::uiextensions::AppBarType appbarType)
{
	if (!m_RdkViewWeakRef)
		return;
	if (!m_RdkViewWeakRef.Resolve<foxit::rdk::PDFViewCtrl>())
		return;

	foxit::rdk::PDFViewCtrl^ view = m_RdkViewWeakRef.Resolve<foxit::rdk::PDFViewCtrl>();
	bool isDocumentCanModify = FSPDFDocUtil::GetIsDocumentCanModify(view->GetDoc());
	bool isDocumentCanModifyAnnotation = FSPDFDocUtil::GetIsDocumentCanModifyAnnotation(view->GetDoc());
	BackCommand->Visibility = Windows::UI::Xaml::Visibility::Collapsed;

	switch (appbarType)
	{
	case foxit::rdk::uiextensions::AppBarType::MainAppBar:
	{
		auto _prevAppBarType = m_curAppbarType;
		m_curAppbarType = foxit::rdk::uiextensions::AppBarType::MainAppBar;

		MainAppBar->Visibility = Windows::UI::Xaml::Visibility::Visible;
		ViewCommand->IsChecked = false;
		if (GetIsDesktopPlatform())
		{
			BackCommand->Visibility = Windows::UI::Xaml::Visibility::Visible;
			RadioUndo->Visibility = Windows::UI::Xaml::Visibility::Visible;
			RadioRedo->Visibility = Windows::UI::Xaml::Visibility::Visible;
			UpdateUndoControlState();
		}
		else
		{
			BackCommand->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
			RadioUndo->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
			RadioRedo->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
			if (_prevAppBarType == foxit::rdk::uiextensions::AppBarType::AnnotAppBar)
			{
				UIExtMgr->HideUndoRedoBarInPhone();
			}
		}
		
		AnnotAppBar->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
		FindTextGrid->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
		ReflowViewGrid->Visibility = Windows::UI::Xaml::Visibility::Collapsed;

		bool bIsOwnerPassword = (view->GetDoc()->GetPasswordType() == foxit::pdf::PasswordType::e_PwdOwner);
		bool bIsCanAssemble = FSPDFDocUtil::GetIsDocumentCanAssemble(view->GetDoc());
		if (isDocumentCanModifyAnnotation || (!isDocumentCanModifyAnnotation && bIsOwnerPassword))
		{
			CommentCommand->Visibility = Windows::UI::Xaml::Visibility::Visible;			
		}	
		if (isDocumentCanModify || bIsOwnerPassword || bIsCanAssemble)
		{
			
		}
	}
	break;
	case foxit::rdk::uiextensions::AppBarType::MainContinueModeAppBar:
	{
		m_curAppbarType = foxit::rdk::uiextensions::AppBarType::MainContinueModeAppBar;
	}
	break;
	case foxit::rdk::uiextensions::AppBarType::MainSingleModeAppBar:
	{
		m_curAppbarType = foxit::rdk::uiextensions::AppBarType::MainSingleModeAppBar;
	}
	break;
	case foxit::rdk::uiextensions::AppBarType::SearchBar:
	{
		m_curAppbarType = foxit::rdk::uiextensions::AppBarType::SearchBar;

		MainAppBar->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
		AnnotAppBar->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
		FindTextGrid->Visibility = Windows::UI::Xaml::Visibility::Visible;
		ReflowViewGrid->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
	}
	break;
	case foxit::rdk::uiextensions::AppBarType::TextMarkBar:
	{
		m_curAppbarType = foxit::rdk::uiextensions::AppBarType::TextMarkBar;
	}
	break;
	case foxit::rdk::uiextensions::AppBarType::AnnotPropertyBar:
	{
		ActivateAppBar();
	}
	break;
	case foxit::rdk::uiextensions::AppBarType::AnnotColorProperty:
	{
		m_curAppbarType = foxit::rdk::uiextensions::AppBarType::AnnotColorProperty;
	}
	break;
	case foxit::rdk::uiextensions::AppBarType::AnnotOpacityProperty:
	{
		
	}
	break;
	case foxit::rdk::uiextensions::AppBarType::ViewModeAppBar:
	{
		m_curAppbarType = foxit::rdk::uiextensions::AppBarType::ViewModeAppBar;
		MainAppBar->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
		
		AnnotAppBar->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
		FindTextGrid->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
		ReflowViewGrid->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
		
	}
	break;
	case foxit::rdk::uiextensions::AppBarType::AnnotAppBar:
	{
		m_curAppbarType = foxit::rdk::uiextensions::AppBarType::AnnotAppBar;
		MainAppBar->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
		BackCommand->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
		AnnotAppBar->Visibility = Windows::UI::Xaml::Visibility::Visible;
		FindTextGrid->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
		ReflowViewGrid->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
		
	}
	break;
	case foxit::rdk::uiextensions::AppBarType::ReflowViewAppBar:
	{
		HideViewToolPanelPopup();
		m_curAppbarType = foxit::rdk::uiextensions::AppBarType::ReflowViewAppBar;
		MainAppBar->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
		
		AnnotAppBar->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
		FindTextGrid->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
		ReflowViewGrid->Visibility = Windows::UI::Xaml::Visibility::Visible;
		reflowViewAppBar->UpdateControlState();
	}
	break;
	case foxit::rdk::uiextensions::AppBarType::FormOperationAppbar:
	{
		m_curAppbarType = foxit::rdk::uiextensions::AppBarType::FormOperationAppbar;
		if (AnnotAppBar->Visibility == Windows::UI::Xaml::Visibility::Collapsed)
		{
			BackCommand->Visibility = GetIsDesktopPlatform() ? Windows::UI::Xaml::Visibility::Visible : Windows::UI::Xaml::Visibility::Collapsed;
		}		
		FindTextGrid->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
		ReflowViewGrid->Visibility = Windows::UI::Xaml::Visibility::Collapsed;

		auto viewControl = m_RdkViewWeakRef.Resolve<foxit::rdk::PDFViewCtrl>();
		foxit::rdk::uiextensions::UIExtensionsManager^ extMgr = (foxit::rdk::uiextensions::UIExtensionsManager^)viewControl->ExtensionsManager;
		if (extMgr)
		{
			if (GetIsDesktopPlatform())
			{
				extMgr->ShowWidgetAnnotPopup();
			}
			else
			{
				if (m_visible == Windows::UI::Xaml::Visibility::Visible)
					HideAppBar(false);

				extMgr->ShowFormOperationBarInPhone();
			}
		}
	}
	break;
	}
}

void MainPageAppBar::UpdateReflowViewAppBarButtonState()
{
	if (m_curAppbarType == foxit::rdk::uiextensions::AppBarType::ReflowViewAppBar)
	{
		reflowViewAppBar->UpdateControlState();
	}
}

void MainPageAppBar::ShowAppBar()
{
	auto _prevState = AppBarControl->Visibility;
	auto _prevThisState = this->Visibility;
	auto _prevOpacity = AppBarControl->Opacity;

	auto hideStoryboard = safe_cast<Windows::UI::Xaml::Media::Animation::Storyboard^>(this->Resources->Lookup("AppBarSlowHideStoryboard"));
	if (hideStoryboard)
		hideStoryboard->Stop();

	AppBarControl->Opacity = 1;
	this->Visibility = Windows::UI::Xaml::Visibility::Visible;
	m_visible = Windows::UI::Xaml::Visibility::Visible;
	if (_prevState == Windows::UI::Xaml::Visibility::Collapsed)
	{
		UIExtMgr->AppBarShow(UIExtMgr, nullptr);
	}

	UIExtMgr->SetPageNumberSliderVisible(true);
	
}

void MainPageAppBar::HideAppBar()
{
	auto hideStoryboard = safe_cast<Windows::UI::Xaml::Media::Animation::Storyboard^>(this->Resources->Lookup("AppBarSlowHideStoryboard"));
	if (hideStoryboard)
		hideStoryboard->Begin();

	if (m_curAppbarType == foxit::rdk::uiextensions::AppBarType::ReflowViewAppBar)
		reflowViewAppBar->SetAppBarIsCompact(true);

	if (m_curAppbarType == foxit::rdk::uiextensions::AppBarType::AnnotAppBar)
		annotToolControl->SetAppBarIsCompact(true);

	SetMainAppBarCompact(true);
	m_visible = Windows::UI::Xaml::Visibility::Collapsed;

	if (UIExtMgr)
		UIExtMgr->SetPageNumberSliderVisible(false);
}

void MainPageAppBar::HideAppBar(bool useTransitions)
{
	if (useTransitions)
	{
		auto hideStoryboard = safe_cast<Windows::UI::Xaml::Media::Animation::Storyboard^>(this->Resources->Lookup("AppBarSlowHideStoryboard"));
		if (hideStoryboard)
			hideStoryboard->Begin();
	}
	else
	{
		this->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
	}

	if (m_curAppbarType == foxit::rdk::uiextensions::AppBarType::ReflowViewAppBar)
		reflowViewAppBar->SetAppBarIsCompact(true);

	if (m_curAppbarType == foxit::rdk::uiextensions::AppBarType::AnnotAppBar)
		annotToolControl->SetAppBarIsCompact(true);

	SetMainAppBarCompact(true);
	m_visible = Windows::UI::Xaml::Visibility::Collapsed;

	if (UIExtMgr)
		UIExtMgr->SetPageNumberSliderVisible(false);
}

void MainPageAppBar::ActivateAppBar()
{
	ShowAppBar();
	SetAppBarCompact();
	HideSecondaryPopup();
	SetMainAppBarCompact(true);
	HideViewToolPanelPopup();
	
	RestartTimeTick();
}

void MainPageAppBar::SetOutlineAppbarButton(bool isHasBookmark)
{
	if (!isHasBookmark)
	{
		OutlineCommand->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
	}
}

void MainPageAppBar::GotoPageIndex(BookMarkGoToData^ goToData)
{
	auto viewControl = m_RdkViewWeakRef.Resolve<foxit::rdk::PDFViewCtrl>();
	if (viewControl)
	{
		auto pageCount = viewControl->GetDoc()->GetPageCount();
		auto pageIndex = goToData->pageIndex;

		
		if (pageIndex < 0 || pageIndex >= pageCount)
			pageIndex = pageCount - 1;

		if (pageIndex >= 0 && pageIndex < pageCount)
		{
			
			viewControl->GotoBookmark(goToData->bookmark);
			UpdateReflowViewAppBarButtonState();
		}
	}
}

void MainPageAppBar::BackCommand_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{
	if (UIExtMgr->IsPopupShow())
		return;

	if (m_curAppbarType == foxit::rdk::uiextensions::AppBarType::MainAppBar)
	{
		HideSecondaryPopup();
		UIExtMgr->ExitFileView();
	}
	else if (m_curAppbarType == foxit::rdk::uiextensions::AppBarType::ReflowViewAppBar)
	{
		HideSecondaryPopup();
		UIExtMgr->ExitFileView();
	}
	else
	{
		ResetAppBarButtonCompact();
		if (m_curAppbarType == foxit::rdk::uiextensions::AppBarType::SearchBar)
		{
			findTextControl->CancleSearch();
		}
		InitMainAppbar(foxit::rdk::uiextensions::AppBarType::MainAppBar);
		
		BackCommand->IsCompact = true;
	}

	HideViewToolPanelPopup();
}

void MainPageAppBar::OutlineCommand_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{
	if (UIExtMgr->IsPopupShow())
		return;

	UIExtMgr->ExitAnnotSelect();
	ShowBookmarkPage();
	SetMainAppBarCompact(true);
}

void MainPageAppBar::ViewCommand_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{
	UIExtensionsManager^ uiMgr = UIExtMgr;
	if (uiMgr->IsPopupShow())
	{
		ViewCommand->IsChecked = false;
		return;
	}

	uiMgr->ExitAnnotSelect();

	SecondaryCommandsPopup->IsOpen = false;
	if (_Flyout)
		_Flyout->Hide();
	if (!viewtoolPanelPopup->IsOpen)
	{
		Windows::UI::Color color = Utility::ConvertFromString("#FF0E90cb");
		

		Windows::Foundation::Point curPt = ViewCommand->TransformToVisual(nullptr)->TransformPoint(Windows::Foundation::Point((float)(ViewCommand->ActualWidth / 2), (float)(ViewCommand->ActualHeight / 2)));
		if (SecondaryCommandsPopup->IsOpen == true)
		{
			
			BackCommand->IsCompact = true;
			OutlineCommand->IsCompact = true;
			ViewCommand->IsCompact = true;
			CommentCommand->IsCompact = true;
			SearchCommand->IsCompact = true;

		}
		HideSecondaryPopup();
		

		viewtoolPanelPopup->Child = nullptr;
		ViewToolAppBar^ viewToolBar = ref new ViewToolAppBar(View, this);
		viewtoolPanelPopup->Child = viewToolBar;

		Windows::Foundation::Rect _winrect = Windows::UI::Xaml::Window::Current->Bounds;
		double height = this->ActualHeight;
		Windows::Foundation::Collections::IVectorView<String^>^ langus = Windows::Globalization::ApplicationLanguages::Languages;
		String^ lang = langus->GetAt(0);
		if (GetIsDesktopPlatform())
		{
			double hoffset = 75;
			if (langus->Size > 0 && lang == "ko")
			{
				hoffset = 82;
			}
			else if (langus->Size > 0 && lang == "ja")
			{
				hoffset = 85;
			}

			viewtoolPanelPopup->HorizontalOffset = curPt.X - hoffset;
			viewtoolPanelPopup->VerticalOffset = height;
		}
		else
		{
			

			double heightvalue = 65;
			if (langus->Size > 0 && (lang == "ko"
				|| lang == "ja"
				))
			{
				if (_winrect.Width < _winrect.Height)
				{
					heightvalue = 140;
				}
				else
				{
					heightvalue = 75;
				}
			}
			else
			{
				if (_winrect.Width < _winrect.Height)
				{
					heightvalue = 130;
				}
				else
				{
					heightvalue = 65;
				}
			}

			viewtoolPanelPopup->VerticalOffset = _winrect.Height - heightvalue;
		}
		viewtoolPanelPopup->IsOpen = true;
		StopHideTimeTick();
	}
	else
	{
		
		viewtoolPanelPopup->IsOpen = false;
	}
}

void MainPageAppBar::OnViewPhoneControlLoaded(Platform::Object ^sender, Windows::UI::Xaml::RoutedEventArgs ^e)
{
	
}

void MainPageAppBar::CommentCommand_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{
	UIExtensionsManager^ extMgr = m_UIViewWeakRef.Resolve<UIExtensionsManager>();
	if (extMgr->IsPopupShow())
		return;

	extMgr->ExitAnnotSelect();
	if (SecondaryCommandsPopup->IsOpen == true)
	{
		SetAppBarCompact();
	}
	SetMainAppBarCompact(true);
	LoadAnnotBar();
	HideViewToolPanelPopup();
}

void MainPageAppBar::LoadAnnotBar()
{
	
	UIExtensionsManager^ extMgr = m_UIViewWeakRef.Resolve<UIExtensionsManager>();
	InitMainAppbar(foxit::rdk::uiextensions::AppBarType::AnnotAppBar);
	annotToolControl->StartHideLabelTick();
	HideSecondaryPopup();

	if (View && !GetIsDesktopPlatform())
	{
		extMgr->ShowUndoRedoBarInPhone();
	}

	auto _AnnotSetting = extMgr->getAnnotMgr()->GetAnnotSetting();
	if (_AnnotSetting->GetIsFirstUserAnnot())
	{
		_AnnotSetting->SetIsFirstUserAnnot(false);
	}
}

void MainPageAppBar::SearchCommand_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{
	if (UIExtMgr->IsPopupShow())
		return;

	UIExtMgr->ExitAnnotSelect();
	if (SecondaryCommandsPopup->IsOpen == true)
	{
		SetAppBarCompact();
	}
	HideSecondaryPopup();
	HideViewToolPanelPopup();
	if (Utility::IsDesktopPlatform())
	{
		InitMainAppbar(foxit::rdk::uiextensions::AppBarType::SearchBar);
		findTextControl->FocusSearchControl();
		HideSecondaryPopup();
		StopHideTimeTick();
	}
	else
	{
		m_curAppbarType = foxit::rdk::uiextensions::AppBarType::SearchBar;
		UIExtMgr->ShowSearchBarInPhone();
		HideAppBar();
	}
	if (UIExtMgr)
		UIExtMgr->SetPageNumberSlideHide();
	SetMainAppBarCompact(true);
}

void MainPageAppBar::appBarSave_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{
	UIExtMgr->SaveDoc();

	ActivateAppBar();
}

void MainPageAppBar::appBarSaveAs_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{
	
	ActivateAppBar();
}

void MainPageAppBar::HideAnnot_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{
	UIExtMgr->IsShowAnnot = false;
	View->Redraw();
	ActivateAppBar();
}

void MainPageAppBar::FileProperty_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{

}

void MainPageAppBar::ExitCommand_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{

}

void MainPageAppBar::Button_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{

}

void MainPageAppBar::ShowRotateFloatWindows()
{

}

void MainPageAppBar::HideRotateFloatWindow()
{

}

void MainPageAppBar::RadioAnnot_Checked(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{
	
	RadioButton^ button = dynamic_cast<RadioButton^>(sender);
	String^ tagStr = button->Tag->ToString();
	
	auto resourceLoader = Windows::ApplicationModel::Resources::ResourceLoader::GetForCurrentView("foxit.rdk.uiextensions/Resources");
	
	auto highlightStr = resourceLoader->GetString(L"appBarHightLightContext.Text");
	auto underlightStr = resourceLoader->GetString(L"appBarUnderLineContext.Text");
	auto strikeoutStr = resourceLoader->GetString(L"appBarStrikeOutContext.Text");
	auto noteStr = resourceLoader->GetString(L"appBarNoteContext.Text");
	if (button->Tag->ToString() == highlightStr)
	{
		UIExtMgr->AddAnnotByType(AnnotCreateType::HightLight);
	}
	else if (button->Tag->ToString() == underlightStr)
	{
		UIExtMgr->AddAnnotByType(AnnotCreateType::UnderLine);
	}
	else if (button->Tag->ToString() == strikeoutStr)
	{
		UIExtMgr->AddAnnotByType(AnnotCreateType::StrikeOut);
	}
	else if (button->Tag->ToString() == noteStr)
	{
		UIExtMgr->AddAnnotByType(AnnotCreateType::Text);
	}
	UIExtMgr->LoadAnnotAppBar(foxit::rdk::uiextensions::AppBarType::AnnotPropertyBar);
}

void MainPageAppBar::imgMore_PointerPressed(Platform::Object^ sender, Windows::UI::Xaml::Input::PointerRoutedEventArgs^ e)
{

}

void MainPageAppBar::ShowAnnot_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{
	ActivateAppBar();
	
	UIExtMgr->IsShowAnnot = true;
	View->Redraw();
}

void MainPageAppBar::OnTick(Platform::Object ^sender, Platform::Object ^args)
{
	if (m_isStopTick)
		return;
	if (m_HideTimer && m_HideTimer->IsEnabled)
	{
		m_HideTimer->Stop();
		HideSecondaryPopup();
		HideAppBar();
		
	}
}

void MainPageAppBar::StopHideTimeTick()
{
	if (m_HideTimer )
	{
		m_HideTimer->Stop();
		
	}
	m_isStopTick = true;
}

void MainPageAppBar::RestartTimeTick()
{
	if (m_HideTimer && m_HideTimer->IsEnabled)
		m_HideTimer->Stop();

	m_HideTimer->Start();
	m_isStopTick = false;
}

Windows::UI::Xaml::Visibility MainPageAppBar::GetMainAppBarVisible()
{
	return m_visible;
}

void MainPageAppBar::imgMore_Tapped(Platform::Object^ sender, Windows::UI::Xaml::Input::TappedRoutedEventArgs^ e)
{
	

	auto size = AnnotAppBar->Children->Size;
	for (unsigned int i = 0; i < size; i++)
	{
		auto _item = AnnotAppBar->Children->GetAt(i);
		auto _ICommandBarElement = dynamic_cast<Windows::UI::Xaml::Controls::ICommandBarElement^>(_item);
		if (_ICommandBarElement)
		{
			auto _IsCompact = _ICommandBarElement->IsCompact;
			_ICommandBarElement->IsCompact = !_ICommandBarElement->IsCompact;
		}
	}
}

void MainPageAppBar::MainAppBar_Opening(Platform::Object^ sender, Platform::Object^ e)
{
	
	BackCommand->IsCompact = false;
	
}

void MainPageAppBar::MainAppBar_Closing(Platform::Object^ sender, Platform::Object^ e)
{
	
	BackCommand->IsCompact = true;
	
}

void MainPageAppBar::AppBarControl_PointerEntered(Platform::Object^ sender, Windows::UI::Xaml::Input::PointerRoutedEventArgs^ e)
{
	StopHideTimeTick();
	ShowAppBar();
}

void MainPageAppBar::AppBarControl_PointerExited(Platform::Object^ sender, Windows::UI::Xaml::Input::PointerRoutedEventArgs^ e)
{
	if (m_curAppbarType != foxit::rdk::uiextensions::AppBarType::SearchBar && !SecondaryCommandsPopup->IsOpen && !viewtoolPanelPopup->IsOpen && !_isCPDFFlyoutOpen && UIExtMgr->CreateTypeOfAnnot != AnnotCreateType::Eraser
		&& UIExtMgr->CreateTypeOfAnnot == AnnotCreateType::None && !annotToolControl->IsPopupIsOpen())
	{
		RestartTimeTick();
	}
}

void MainPageAppBar::imgMoreView_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{
	
}

void MainPageAppBar::imgMoreMain_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{
	UIExtensionsManager^ extMgr = m_UIViewWeakRef.Resolve<UIExtensionsManager>();
	if (extMgr->IsPopupShow())
		return;

	UpdateResetFormControlState();
	SetMainBarCompact();
	extMgr->ExitAnnotSelect();
}

void MainPageAppBar::imgMoreMain_Tapped(Platform::Object^ sender, Windows::UI::Xaml::Input::TappedRoutedEventArgs^ e)
{

}

void MainPageAppBar::Canvas_PointerEntered(Platform::Object^ sender, Windows::UI::Xaml::Input::PointerRoutedEventArgs^ e)
{
	Canvas^ canvas = dynamic_cast<Canvas^>(sender);
	Windows::UI::Color color = Utility::ConvertFromString("#FFCCCCCC");
	canvas->Background = ref new SolidColorBrush(color);
}

void MainPageAppBar::Canvas_PointerExited(Platform::Object^ sender, Windows::UI::Xaml::Input::PointerRoutedEventArgs^ e)
{
	Canvas^ canvas = dynamic_cast<Canvas^>(sender);
	
	canvas->Background = ref new SolidColorBrush(Windows::UI::Colors::Transparent);
}

void MainPageAppBar::HideSecondaryPopup()
{
	if (SecondaryCommandsPopup && SecondaryCommandsPopup->IsOpen)
		SecondaryCommandsPopup->IsOpen = false;
}

void MainPageAppBar::SetSaveBtnUnEnabled()
{
	secondaryCommand->SetSaveBtnUnEnabled();
}

void MainPageAppBar::SetMainBarCompact()
{
	HideViewToolPanelPopup();
	auto size = MainAppBar->Children->Size;
	for (unsigned int i = 0; i < size; i++)
	{
		auto _item = MainAppBar->Children->GetAt(i);
		auto _ICommandBarElement = dynamic_cast<Windows::UI::Xaml::Controls::ICommandBarElement^>(_item);
		if (_ICommandBarElement)
		{
			auto _IsCompact = _ICommandBarElement->IsCompact;
			_ICommandBarElement->IsCompact = !_ICommandBarElement->IsCompact;
		}
	}
	BackCommand->IsCompact = !BackCommand->IsCompact;
	double height = this->ActualHeight;
	if (!BackCommand->IsCompact)
	{
		AppBarControl->Height = 60;

		auto _winrect = Windows::UI::Xaml::Window::Current->Bounds;

		height = this->ActualHeight;

		Windows::Foundation::Collections::IVectorView<String^>^ langus = Windows::Globalization::ApplicationLanguages::Languages;
		secondaryCommand->Loaded += ref new Windows::UI::Xaml::RoutedEventHandler(this, &foxit::rdk::uiextensions::MainPageAppBar::OnSecondaryCommandLoaded);
		if (View && GetIsDesktopPlatform())
		{
			
			SecondaryCommandsPopup->VerticalOffset = 60;
		}
		else
		{
			if (_winrect.Width > _winrect.Height)
			{
				if (langus->Size > 0 && (langus->GetAt(0) == "ko" || langus->GetAt(0) == "ja"))
				{
					secondaryCommand->SetPanelWidth(160);
					SecondaryCommandsPopup->HorizontalOffset = _winrect.Width - 160;
				}
				else
				{
					secondaryCommand->SetPanelWidth(155);
					SecondaryCommandsPopup->HorizontalOffset = _winrect.Width - 155;
				}
			}
			else
			{
				secondaryCommand->SetPanelWidth(_winrect.Width);
				SecondaryCommandsPopup->HorizontalOffset = 0;
			}
		}
		SecondaryCommandsPopup->IsOpen = true;
	}
	else
	{
		AppBarControl->Height = 44;
		SecondaryCommandsPopup->IsOpen = false;
	}
}

void MainPageAppBar::ResetMorePopupLocation()
{
	auto _winrect = Windows::UI::Xaml::Window::Current->Bounds;

	if (View && GetIsDesktopPlatform())
	{
		SecondaryCommandsPopup->HorizontalOffset = _winrect.Width - 150;
		SecondaryCommandsPopup->VerticalOffset = 60;
	}
	else
	{
		if (_winrect.Width > _winrect.Height)
		{
			secondaryCommand->Width = 150;
			SecondaryCommandsPopup->HorizontalOffset = _winrect.Width - 150;
		}
		else
		{
			secondaryCommand->Width = _winrect.Width;
			SecondaryCommandsPopup->HorizontalOffset = 0;
		}
		SecondaryCommandsPopup->VerticalOffset = -200;
	}
}

void MainPageAppBar::HideMorePopup()
{
	if (SecondaryCommandsPopup->IsOpen)
	{
		SecondaryCommandsPopup->IsOpen = false;
		auto size = MainAppBar->Children->Size;
		for (unsigned int i = 0; i < size; i++)
		{
			auto _item = MainAppBar->Children->GetAt(i);
			auto _ICommandBarElement = dynamic_cast<Windows::UI::Xaml::Controls::ICommandBarElement^>(_item);
			if (_ICommandBarElement)
			{
				auto _IsCompact = _ICommandBarElement->IsCompact;
				_ICommandBarElement->IsCompact = !_ICommandBarElement->IsCompact;
			}
		}
		BackCommand->IsCompact = !BackCommand->IsCompact;
		AppBarControl->Height = 44;
	}
}

bool MainPageAppBar::GetMorePopupIsOpen()
{
	bool isOpen = SecondaryCommandsPopup->IsOpen;
	return SecondaryCommandsPopup->IsOpen;
}

void MainPageAppBar::StackPanel_PointerEntered(Platform::Object^ sender, Windows::UI::Xaml::Input::PointerRoutedEventArgs^ e)
{

}

void MainPageAppBar::StackPanel_PointerExited(Platform::Object^ sender, Windows::UI::Xaml::Input::PointerRoutedEventArgs^ e)
{
	RestartTimeTick();
}

void MainPageAppBar::UserControl_Loaded(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{
	double height = this->ActualHeight;
	UpdateUndoControlState();

	auto pUndo = UndoFactory::GetUndoInstance();
	m_AfterUndoEventToken = pUndo->AfterUndo += ref new foxit::rdk::uiextensions::UndoEventHandler(this, &foxit::rdk::uiextensions::MainPageAppBar::OnAfterUndo);
	m_AfterRedoEventToken = pUndo->AfterRedo += ref new foxit::rdk::uiextensions::UndoEventHandler(this, &foxit::rdk::uiextensions::MainPageAppBar::OnAfterRedo);
	m_OperaListChangedUndoRedoEventToken = pUndo->OperaListChanged += ref new foxit::rdk::uiextensions::UndoOperaListChangedEventHandler(this, &foxit::rdk::uiextensions::MainPageAppBar::OnOperaListChanged);

	if (Windows::Foundation::Metadata::ApiInformation::IsTypePresent("Windows.Phone.UI.Input.HardwareButtons"))
	{
		m_BackPresseEventToken = Windows::Phone::UI::Input::HardwareButtons::BackPressed += ref new Windows::Foundation::EventHandler<Windows::Phone::UI::Input::BackPressedEventArgs ^>(this, &MainPageAppBar::OnBackPressed);
	}
	if (Windows::Foundation::Metadata::ApiInformation::IsTypePresent("Windows.UI.Core.SystemNavigationManager"))
	{
		m_SystemNavigationBackRequestedEventToken = Windows::UI::Core::SystemNavigationManager::GetForCurrentView()->BackRequested += ref new Windows::Foundation::EventHandler<Windows::UI::Core::BackRequestedEventArgs ^>(this, &MainPageAppBar::OnBackRequested);
	}
}

void MainPageAppBar::UserControl_Unloaded(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{
	if (Windows::Foundation::Metadata::ApiInformation::IsTypePresent("Windows.Phone.UI.Input.HardwareButtons"))
	{
		Windows::Phone::UI::Input::HardwareButtons::BackPressed -= m_BackPresseEventToken;
		m_BackPresseEventToken.Value = 0;
	}
	if (Windows::Foundation::Metadata::ApiInformation::IsTypePresent("Windows.UI.Core.SystemNavigationManager"))
	{
		Windows::UI::Core::SystemNavigationManager::GetForCurrentView()->BackRequested -= m_SystemNavigationBackRequestedEventToken;
		m_SystemNavigationBackRequestedEventToken.Value = 0;
	}
	if (m_HideTimer->IsEnabled)
		m_HideTimer->Stop();

	auto pUndo = UndoFactory::GetUndoInstance();
	if (m_AfterUndoEventToken.Value)
	{
		pUndo->AfterUndo -= m_AfterUndoEventToken;
		m_AfterUndoEventToken.Value = 0;
	}
	if (m_AfterRedoEventToken.Value)
	{
		pUndo->AfterRedo -= m_AfterRedoEventToken;
		m_AfterRedoEventToken.Value = 0;
	}
	if (m_OperaListChangedUndoRedoEventToken.Value)
	{
		pUndo->OperaListChanged -= m_OperaListChangedUndoRedoEventToken;
		m_OperaListChangedUndoRedoEventToken.Value = 0;
	}

	if (__windowSizeChangedEventRegistrationToken.Value)
	{
		Window::Current->SizeChanged -= __windowSizeChangedEventRegistrationToken;
		__windowSizeChangedEventRegistrationToken.Value = 0;
	}
}

void MainPageAppBar::AppBarControl_Tapped(Platform::Object^ sender, Windows::UI::Xaml::Input::TappedRoutedEventArgs^ e)
{
	UIExtensionsManager^ uiMgr = UIExtMgr;
	if (View)
	{
		if (uiMgr->IsPopupShow())
			return;

		

		
		if (uiMgr->getAnnotMgr() && uiMgr->getAnnotMgr()->GetSelectedAnnot())
		{
			uiMgr->ExitAnnotSelect();
			uiMgr->getAnnotMgr()->SetPdfSelectAnnotNull();
			uiMgr->SetAnnotCanvasVisible(false);
			uiMgr->SetTextMenuPopupHide();
			uiMgr->SetPropertyPoupHide();
			View->Update();
		}
		else if (uiMgr->getTextMgr())
		{
			if (uiMgr->getTextMgr()->SelectedText->Size > 0)
			{
				if (uiMgr->CreateTypeOfAnnot != AnnotCreateType::Caret && uiMgr->CreateTypeOfAnnot != AnnotCreateType::ReplaceText)
				{
					uiMgr->getTextMgr()->ClearTextSelect();
				}
				uiMgr->SetTextMenuPopupHide();
				View->Update();
			}
		}
	}
}

void MainPageAppBar::OnBackPressed(Platform::Object ^sender, Windows::Phone::UI::Input::BackPressedEventArgs ^args)
{
	if (!View)
		return;

	if (!m_bIsFlyoutShow)
	{
		args->Handled = true;

		if (UIExtMgr->IsAnnotAppBarTipShow())
		{
			UIExtMgr->SetAnnotPropertyTipShow(false);
			return;
		}

		if (UIExtMgr->IsInThumbnailMode())
		{
			UIExtMgr->ExitThumbnailPage();
			return;
		}

		if (m_curAppbarType == foxit::rdk::uiextensions::AppBarType::ReflowViewAppBar)
		{
			if (!UIExtMgr->LoadPrevViewMode())
			{
				UIExtMgr->LoadViewByViewMode(LayoutMode::ContinueMode);
				UIExtMgr->ResetDefaultAppBarButton();
			}
			else
			{
				UIExtMgr->ResetDefaultAppBarButton();
			}
			return;
		}

		auto _curBarType = UIExtMgr->GetCurAppBarType();

		if (UIExtMgr->IsPhonePropertyPopupOpen())
		{
			UIExtMgr->SetPhonePropertyPopupClosed();

			if (Windows::System::Profile::AnalyticsInfo::VersionInfo->DeviceFamily == L"Windows.Mobile")
			{
				UIExtMgr->SetUnRedoPhoneAppBarVisible(true);
			}
				return;
		}
		if (UIExtMgr->AnnotMgr && UIExtMgr->AnnotMgr->GetSelectedAnnot() && UIExtMgr->IsSelectAnnotPhonePropertyOpen())
		{
			UIExtMgr->ExitAnnotSelect();
			return;
		}

		if (UIExtMgr->CreateTypeOfAnnot != AnnotCreateType::None)
		{
			if (UIExtMgr->CreateTypeOfAnnot == AnnotCreateType::Pencil)
				UIExtMgr->CreateGraphAnnotBySliderPage();
			if (UIExtMgr->CreateTypeOfAnnot == AnnotCreateType::Eraser)
				UIExtMgr->CollectEraserOpration();
			if (UIExtMgr->CreateTypeOfAnnot == AnnotCreateType::FreeText)
			{
				IBaseAnnotEventPeer^ annotEventPeer = UIExtMgr->getAnnotMgr()->GetAnnotHandlerByType(AnnotationType::FreeText)->AnnotEvent;
				FreeTextEventPeer^ freetextEventPeer = dynamic_cast<FreeTextEventPeer^>(annotEventPeer);

				UIExtMgr->SetAnnotCanvasVisible(false);
				if (!freetextEventPeer->IsTextBoxHasContent())
				{
					if (!UIExtMgr->getAnnotMgr()->GetAnnotSetting()->GetFreeTextIsContinueOpera())
					{
						UIExtMgr->CreateTypeOfAnnot = AnnotCreateType::None;
						UIExtMgr->ExitAnnotPropertyMode();
					}
				}
			}
			else
			{
				UIExtMgr->CreateTypeOfAnnot = AnnotCreateType::None;
				UIExtMgr->ExitAnnotPropertyMode();
			}
			return;
		}

		if (m_curAppbarType != foxit::rdk::uiextensions::AppBarType::MainAppBar
			&& UIExtMgr->GetCurAppBarType() != foxit::rdk::uiextensions::AppBarType::AnnotPropertyBar
			&& UIExtMgr->GetCurAppBarType() != foxit::rdk::uiextensions::AppBarType::SearchBar)
		{
			ResetAppBarButtonCompact();
			InitMainAppbar(foxit::rdk::uiextensions::AppBarType::MainAppBar);
			
			BackCommand->IsCompact = true;
		}
		else if (SecondaryCommandsPopup->IsOpen)
		{
			HideMorePopup();
		}
		else if (UIExtMgr->GetCurAppBarType() == foxit::rdk::uiextensions::AppBarType::SearchBar)
		{
			UIExtMgr->HideFindBar();
		}
		else if (m_curAppbarType == foxit::rdk::uiextensions::AppBarType::MainAppBar && GetViewToolPanelPopupIsOpen())
		{
			HideViewToolPanelPopup();
		}
		else if (m_curAppbarType == foxit::rdk::uiextensions::AppBarType::MainAppBar)
		{
			HideSecondaryPopup();
			UIExtMgr->ExitFileView();
		}
	}
}

void MainPageAppBar::OnBackRequested(Platform::Object ^sender, Windows::UI::Core::BackRequestedEventArgs ^args)
{
	if (!View)
		return;

	if (!m_bIsFlyoutShow)
	{
		args->Handled = true;

		if (m_curAppbarType == foxit::rdk::uiextensions::AppBarType::ReflowViewAppBar)
		{
			HideSecondaryPopup();
			UIExtMgr->ExitFileView();
			return;
		}

		auto _curBarType = UIExtMgr->GetCurAppBarType();

		if (UIExtMgr->IsPhonePropertyPopupOpen())
		{
			UIExtMgr->SetPhonePropertyPopupClosed();
			if (Windows::System::Profile::AnalyticsInfo::VersionInfo->DeviceFamily == L"Windows.Mobile")
			{
				UIExtMgr->SetUnRedoPhoneAppBarVisible(true);
			}
			return;
		}

		if (UIExtMgr->CreateTypeOfAnnot != AnnotCreateType::None)
		{
			if (UIExtMgr->CreateTypeOfAnnot == AnnotCreateType::Pencil)
				UIExtMgr->CreateGraphAnnotBySliderPage();
			if (UIExtMgr->CreateTypeOfAnnot == AnnotCreateType::Eraser)
				UIExtMgr->CollectEraserOpration();
			if (UIExtMgr->CreateTypeOfAnnot == AnnotCreateType::FreeText)
			{
				

				UIExtMgr->SetAnnotCanvasVisible(false);
				
			}
			else
			{
				UIExtMgr->CreateTypeOfAnnot = AnnotCreateType::None;
				UIExtMgr->ExitAnnotPropertyMode();
			}
			return;
		}

		if (m_curAppbarType != foxit::rdk::uiextensions::AppBarType::MainAppBar
			&& UIExtMgr->GetCurAppBarType() != foxit::rdk::uiextensions::AppBarType::AnnotPropertyBar
			&& UIExtMgr->GetCurAppBarType() != foxit::rdk::uiextensions::AppBarType::SearchBar)
		{
			ResetAppBarButtonCompact();
			InitMainAppbar(foxit::rdk::uiextensions::AppBarType::MainAppBar);
			
			BackCommand->IsCompact = true;
		}
		else if (SecondaryCommandsPopup->IsOpen)
		{
			HideMorePopup();
		}
		else if (UIExtMgr->GetCurAppBarType() == foxit::rdk::uiextensions::AppBarType::SearchBar)
		{
			UIExtMgr->HideFindBar();
		}
		else if (m_curAppbarType == foxit::rdk::uiextensions::AppBarType::MainAppBar && GetViewToolPanelPopupIsOpen())
		{
			HideViewToolPanelPopup();
		}
		else if (m_curAppbarType == foxit::rdk::uiextensions::AppBarType::MainAppBar)
		{
			HideSecondaryPopup();
			UIExtMgr->ExitFileView();
		}
	}
}

void MainPageAppBar::OnClosed(Platform::Object ^sender, Platform::Object ^args)
{
	m_bIsFlyoutShow = false;

	Windows::UI::Xaml::Media::TranslateTransform^ _transform = dynamic_cast<Windows::UI::Xaml::Media::TranslateTransform^>(UIExtMgr->FlyoutPlacement->RenderTransform);
	if (_transform)
	{
		if (_transform->Y != 0)
			_transform->Y = 0;
	}

	if (m_WindowSizeChangedEventToken.Value)
	{
		Windows::UI::ViewManagement::ApplicationView::GetForCurrentView()->VisibleBoundsChanged -= m_WindowSizeChangedEventToken;
		m_WindowSizeChangedEventToken.Value = 0;
	}
}

void MainPageAppBar::SetAppBarCompact()
{
	if (m_curAppbarType == foxit::rdk::uiextensions::AppBarType::MainAppBar)
	{
		OutlineCommand->IsCompact = true;
		ViewCommand->IsCompact = true;
		CommentCommand->IsCompact = true;
		SearchCommand->IsCompact = true;
		imgMoreMain->IsCompact = true;
		BackCommand->IsCompact = true;
		RadioUndo->IsCompact = true;
		RadioRedo->IsCompact = true;
		
		

		ViewCommand->IsChecked = false;
	}
}

void MainPageAppBar::ResetAppBarButtonCompact()
{
	if (m_curAppbarType == foxit::rdk::uiextensions::AppBarType::AnnotAppBar)
		annotToolControl->SetAppBarIsCompact(true);
}

void MainPageAppBar::ReflowPageCommand_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{
	
	
}

void MainPageAppBar::ThumbnailPageCommand_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{
	
}

bool MainPageAppBar::GetViewToolPanelPopupIsOpen()
{
	return viewtoolPanelPopup->IsOpen;
}

void MainPageAppBar::HideViewToolPanelPopup()
{
	ViewCommand->IsChecked = false;
	if (viewtoolPanelPopup->IsOpen)
	{
		viewtoolPanelPopup->IsOpen = false;
		RestartTimeTick();
	}
}

void MainPageAppBar::exitAnnotTool_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{
	ResetAppBarButtonCompact();
	InitMainAppbar(foxit::rdk::uiextensions::AppBarType::MainAppBar);
	
}

void MainPageAppBar::ViewCommand_Tapped(Platform::Object^ sender, Windows::UI::Xaml::Input::TappedRoutedEventArgs^ e)
{
	if (UIExtMgr->IsPopupShow())
	{
		ViewCommand->IsChecked = false;
		return;
	}

	UIExtMgr->ExitAnnotSelect();

	SecondaryCommandsPopup->IsOpen = false;
	if (_Flyout)
		_Flyout->Hide();
	if (!viewtoolPanelPopup->IsOpen)
	{
		Windows::UI::Color color = Utility::ConvertFromString("#FF0E90cb");
		

		Windows::Foundation::Point curPt = e->GetPosition(nullptr);
		if (SecondaryCommandsPopup->IsOpen == true)
		{
			
			BackCommand->IsCompact = true;
			OutlineCommand->IsCompact = true;
			ViewCommand->IsCompact = true;
			CommentCommand->IsCompact = true;
			SearchCommand->IsCompact = true;

		}
		HideSecondaryPopup();
		

		viewtoolPanelPopup->Child = nullptr;
		ViewToolAppBar^ viewToolBar = ref new ViewToolAppBar(View, this);
		viewtoolPanelPopup->Child = viewToolBar;

		Windows::Foundation::Rect _winrect = Windows::UI::Xaml::Window::Current->Bounds;
		double height = this->ActualHeight;

		if (GetIsDesktopPlatform())
		{
			Windows::Foundation::Collections::IVectorView<String^>^ langus = Windows::Globalization::ApplicationLanguages::Languages;
			String^ lang = langus->GetAt(0);
			double hoffset = 75;
			if (langus->Size > 0 && langus->GetAt(0) == "en-US" || langus->Size > 0 && langus->GetAt(0) == "zh-Hant-TW")
			{
				hoffset = 82;
			}

			viewtoolPanelPopup->HorizontalOffset = curPt.X - hoffset;
			viewtoolPanelPopup->VerticalOffset = height;
		}
		else
		{
			if (_winrect.Width < _winrect.Height)
			{
				viewtoolPanelPopup->VerticalOffset = _winrect.Height - 130;
			}
			else
			{
				viewtoolPanelPopup->VerticalOffset = _winrect.Height - 65;
			}
		}
		viewtoolPanelPopup->IsOpen = true;
	}
	else
	{
		
		viewtoolPanelPopup->IsOpen = false;
	}
}

void MainPageAppBar::ShowBookmarkPage()
{
	HideSecondaryPopup();
	HideViewToolPanelPopup();

	auto viewControl = UIExtMgr->viewControl;
	auto _Flyout = ref new Windows::UI::Xaml::Controls::Flyout();
	Windows::UI::Xaml::Style^ _FlyoutPresenterStyle;
	_FlyoutPresenterStyle = this->Resources->HasKey(L"FlyoutPresenterStyle") ? dynamic_cast<Windows::UI::Xaml::Style^>(this->Resources->Lookup(L"FlyoutPresenterStyle")) : nullptr;
	_Flyout->FlyoutPresenterStyle = _FlyoutPresenterStyle;
	GoToPageIndexDelegate^ del = ref new GoToPageIndexDelegate(this, &MainPageAppBar::GotoPageIndex);
	_Flyout->Closed += ref new Windows::Foundation::EventHandler<Platform::Object ^>(this, &MainPageAppBar::OnClosed);
	_Flyout->Content = ref new BookMarkPage(viewControl->GetDoc(), del);
	Windows::UI::Xaml::Controls::Flyout::SetAttachedFlyout(UIExtMgr->FlyoutPlacement, _Flyout);
	Windows::UI::Xaml::Controls::Flyout::ShowAttachedFlyout(UIExtMgr->FlyoutPlacement);
	_Flyout->Opened += ref new Windows::Foundation::EventHandler<Platform::Object ^>([this, _Flyout](Platform::Object ^, Platform::Object ^)
	{
		auto viewControl = this->m_RdkViewWeakRef.Resolve<foxit::rdk::PDFViewCtrl>();
		m_bIsFlyoutShow = true;
		HideAppBar();

		auto _VisibleBounds = Windows::UI::ViewManagement::ApplicationView::GetForCurrentView()->VisibleBounds;
		auto _Bounds = Windows::UI::Xaml::Window::Current->Bounds;
		auto _FlyoutPresenter = GetVisualParent<Windows::UI::Xaml::Controls::FlyoutPresenter^>(_Flyout->Content);
		if (_FlyoutPresenter)
		{
			std::wstringstream ws;
			ws << L"MinHeight: " << _FlyoutPresenter->MinHeight;
			ws << L", MaxHeight: " << _FlyoutPresenter->MaxHeight;
			ws << L", ActualWidth: " << _FlyoutPresenter->ActualWidth;
			ws << L", ActualHeight: " << _FlyoutPresenter->ActualHeight;
			ws << L", VisibleBounds.Width: " << _VisibleBounds.Width;
			ws << L", VisibleBounds.Height: " << _VisibleBounds.Height;
			ws << L", _Bounds.Width: " << _Bounds.Width;
			ws << L", _Bounds.Height: " << _Bounds.Height;
			ws << L"\n";
			OutputDebugString(ws.str().c_str());

			auto _height = _VisibleBounds.Height;
			_FlyoutPresenter->MinHeight = _height;
			_FlyoutPresenter->MaxHeight = _height;
			_FlyoutPresenter->Height = _height;
			if (_FlyoutPresenter->ActualWidth > _VisibleBounds.Width)
			{
				_FlyoutPresenter->MinWidth = _VisibleBounds.Width;
				_FlyoutPresenter->MaxWidth = _VisibleBounds.Width;
				_FlyoutPresenter->Width = _VisibleBounds.Width;
			}

			Windows::UI::Xaml::Controls::Primitives::Popup^ _Popup = dynamic_cast<Windows::UI::Xaml::Controls::Primitives::Popup^>(_FlyoutPresenter->Parent);
			if (_Popup)
			{
				_Popup->IsLightDismissEnabled = false;
				double _voffset = _Popup->VerticalOffset;
				if (Windows::System::Profile::AnalyticsInfo::VersionInfo->DeviceFamily == L"Windows.Mobile")
				{
					auto _StatusBar = Windows::UI::ViewManagement::StatusBar::GetForCurrentView();
					_voffset = _StatusBar->OccludedRect.Height;
				}
				else if (Windows::System::Profile::AnalyticsInfo::VersionInfo->DeviceFamily == L"Windows.Desktop")
				{
					auto _screenPt = _FlyoutPresenter->TransformToVisual(nullptr)->TransformPoint(Windows::Foundation::Point(0, 0));
					Windows::UI::Xaml::Media::TranslateTransform^ _transform = dynamic_cast<Windows::UI::Xaml::Media::TranslateTransform^>(UIExtMgr->FlyoutPlacement->RenderTransform);
					if (_transform == nullptr)
					{
						_transform = ref new Windows::UI::Xaml::Media::TranslateTransform();
						UIExtMgr->FlyoutPlacement->RenderTransform = _transform;
					}

					if (_transform->Y != _screenPt.Y)
						_transform->Y = _screenPt.Y;

					_voffset = 0;
				}

				_Popup->VerticalOffset = _voffset;
			}
		}

		m_WindowSizeChangedEventToken = Windows::UI::ViewManagement::ApplicationView::GetForCurrentView()->VisibleBoundsChanged += ref new Windows::Foundation::TypedEventHandler<Windows::UI::ViewManagement::ApplicationView^, Platform::Object^>([this, _Flyout](Windows::UI::ViewManagement::ApplicationView^, Platform::Object^)
		{
			auto viewControl = m_RdkViewWeakRef.Resolve<foxit::rdk::PDFViewCtrl>();
			auto _VisibleBounds = Windows::UI::ViewManagement::ApplicationView::GetForCurrentView()->VisibleBounds;
			auto _Bounds = Windows::UI::Xaml::Window::Current->Bounds;
			auto _FlyoutPresenter = GetVisualParent<Windows::UI::Xaml::Controls::FlyoutPresenter^>(_Flyout->Content);
			if (_FlyoutPresenter)
			{
				std::wstringstream ws;
				ws << L"Window::Current->SizeChanged--MinHeight: " << _FlyoutPresenter->MinHeight;
				ws << L", MaxHeight: " << _FlyoutPresenter->MaxHeight;
				ws << L", ActualWidth: " << _FlyoutPresenter->ActualWidth;
				ws << L", ActualHeight: " << _FlyoutPresenter->ActualHeight;
				ws << L", VisibleBounds.Width: " << _VisibleBounds.Width;
				ws << L", VisibleBounds.Height: " << _VisibleBounds.Height;
				ws << L", _Bounds.Width: " << _Bounds.Width;
				ws << L", _Bounds.Height: " << _Bounds.Height;
				ws << L"\n";
				OutputDebugString(ws.str().c_str());

				auto _height = _VisibleBounds.Height;
				_FlyoutPresenter->MinHeight = _height;
				_FlyoutPresenter->MaxHeight = _height;
				_FlyoutPresenter->Height = _height;
				if (_FlyoutPresenter->ActualWidth > _VisibleBounds.Width)
				{
					_FlyoutPresenter->MinWidth = _VisibleBounds.Width;
					_FlyoutPresenter->MaxWidth = _VisibleBounds.Width;
					_FlyoutPresenter->Width = _VisibleBounds.Width;
				}

				Windows::UI::Xaml::Controls::Primitives::Popup^ _Popup = dynamic_cast<Windows::UI::Xaml::Controls::Primitives::Popup^>(_FlyoutPresenter->Parent);
				if (_Popup)
				{
					_Popup->IsLightDismissEnabled = false;
					double _voffset = _Popup->VerticalOffset;
					if (Windows::System::Profile::AnalyticsInfo::VersionInfo->DeviceFamily == L"Windows.Mobile")
					{
						auto _StatusBar = Windows::UI::ViewManagement::StatusBar::GetForCurrentView();
						_voffset = _StatusBar->OccludedRect.Height;
					}
					else if (Windows::System::Profile::AnalyticsInfo::VersionInfo->DeviceFamily == L"Windows.Desktop")
					{
						auto _screenPt = _FlyoutPresenter->TransformToVisual(nullptr)->TransformPoint(Windows::Foundation::Point(0, 0));
						Windows::UI::Xaml::Media::TranslateTransform^ _transform = dynamic_cast<Windows::UI::Xaml::Media::TranslateTransform^>(UIExtMgr->FlyoutPlacement->RenderTransform);
						if (_transform == nullptr)
						{
							_transform = ref new Windows::UI::Xaml::Media::TranslateTransform();
							UIExtMgr->FlyoutPlacement->RenderTransform = _transform;
						}

						if (_transform->Y != _screenPt.Y)
							_transform->Y = _screenPt.Y;

						_voffset = 0;
					}

					_Popup->VerticalOffset = _voffset;
					_Popup->HorizontalOffset = (_Bounds.Width - _FlyoutPresenter->ActualWidth > 0) ? _Bounds.Width - _FlyoutPresenter->ActualWidth : 0;
				}
			}
		});
	});
}

void MainPageAppBar::UpdateAnnotToolUnRedo()
{
	annotToolControl->UpdateUndoControlState();
}

void foxit::rdk::uiextensions::MainPageAppBar::OnHideAppBarAnimationCompleted(Platform::Object ^sender, Platform::Object ^args)
{
	auto _prevState = AppBarControl->Visibility;
	auto _prevThisState = this->Visibility;
	auto _prevOpacity = AppBarControl->Opacity;

	UIExtMgr->AppBarHide(UIExtMgr, nullptr);
}

void MainPageAppBar::SetAnnotToolStampPopupClosed()
{
	annotToolControl->SetStampPopupClosed();
}

void foxit::rdk::uiextensions::MainPageAppBar::cPDFCommand_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{

}

void foxit::rdk::uiextensions::MainPageAppBar::RadioUndo_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{
	UIExtensionsManager^ uiMgr = UIExtMgr;
	if (uiMgr->IsPopupShow())
		return;
	if (viewtoolPanelPopup->IsOpen)
	{
		ViewCommand->IsChecked = false;
		viewtoolPanelPopup->IsOpen = false;
	}

	if (SecondaryCommandsPopup->IsOpen)
	{
		for (unsigned int i = 0; i < MainAppBar->Children->Size; i++)
		{
			auto _item = MainAppBar->Children->GetAt(i);
			auto _ICommandBarElement = dynamic_cast<Windows::UI::Xaml::Controls::ICommandBarElement^>(_item);
			if (_ICommandBarElement)
			{
				auto _IsCompact = _ICommandBarElement->IsCompact;
				_ICommandBarElement->IsCompact = !_ICommandBarElement->IsCompact;
			}
		}
		BackCommand->IsCompact = !BackCommand->IsCompact;
		AppBarControl->Height = 44;
		SecondaryCommandsPopup->IsOpen = false;
	}

	auto pUndo = UndoFactory::GetUndoInstance();

	uiMgr->ExitAnnotSelect();

	if (View->GetDoc())
		pUndo->Undo(UIExtMgr);
}

void foxit::rdk::uiextensions::MainPageAppBar::RadioRedo_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{
	UIExtensionsManager^ uiMgr = UIExtMgr;
	if (uiMgr->IsPopupShow())
		return;

	if (viewtoolPanelPopup->IsOpen)
	{
		ViewCommand->IsChecked = false;
		viewtoolPanelPopup->IsOpen = false;
	}

	if (SecondaryCommandsPopup->IsOpen)
	{
		for (unsigned int i = 0; i < MainAppBar->Children->Size; i++)
		{
			auto _item = MainAppBar->Children->GetAt(i);
			auto _ICommandBarElement = dynamic_cast<Windows::UI::Xaml::Controls::ICommandBarElement^>(_item);
			if (_ICommandBarElement)
			{
				auto _IsCompact = _ICommandBarElement->IsCompact;
				_ICommandBarElement->IsCompact = !_ICommandBarElement->IsCompact;
			}
		}
		BackCommand->IsCompact = !BackCommand->IsCompact;
		AppBarControl->Height = 44;
		SecondaryCommandsPopup->IsOpen = false;
	}


	auto pUndo = UndoFactory::GetUndoInstance();

	uiMgr->ExitAnnotSelect();
	uiMgr->SetTextMenuPopupHide();

	if (View->GetDoc())
		pUndo->Redo(UIExtMgr);
}

void MainPageAppBar::UpdateUndoControlState()
{
	if (UIExtMgr->CreateTypeOfAnnot != AnnotCreateType::FreeText)
	{
		auto pUndo = UndoFactory::GetUndoInstance();
		if (View && View->GetDoc())
		{
			if (Windows::System::Profile::AnalyticsInfo::VersionInfo->DeviceFamily == L"Windows.Desktop")
			{
				RadioUndo->IsEnabled = pUndo->CanUndo(UIExtMgr);
				RadioRedo->IsEnabled = pUndo->CanRedo(UIExtMgr);
			}
			else
			{
				secondaryCommand->UpdateUnRedoState();
			}
		}
		else
		{
			RadioUndo->IsEnabled = false;
			RadioRedo->IsEnabled = false;
		}
	}
}

void MainPageAppBar::HideNoteStylePopup()
{
	if (m_curAppbarType == foxit::rdk::uiextensions::AppBarType::AnnotAppBar)
		annotToolControl->HideNoteStylePopup();
}

void MainPageAppBar::ResetAnnotToolBG()
{
	annotToolControl->ResetAppBarBG();
}

void foxit::rdk::uiextensions::MainPageAppBar::OnAfterUndo(Platform::Object ^sender, Platform::Object ^args)
{
	UpdateUndoControlState();
}

void foxit::rdk::uiextensions::MainPageAppBar::OnAfterRedo(Platform::Object ^sender, Platform::Object ^args)
{
	UpdateUndoControlState();
}

void foxit::rdk::uiextensions::MainPageAppBar::OnOperaListChanged(Platform::Object ^sender, Platform::Object ^args)
{
	UpdateUndoControlState();
}

void MainPageAppBar::SetAnnotPropertyPopupClosed()
{
	annotToolControl->SetAllPopupClosed();
}

bool MainPageAppBar::IsMobileAnnnotMoreToolOpen()
{
	return annotToolControl->IsMobileAnnnotMoreToolOpen();
}

void MainPageAppBar::InitCurNoteIcon()
{
	annotToolControl->InitCurNoteIcon();
}

void MainPageAppBar::InitStampAppBarSelect()
{
	annotToolControl->InitStampAppBarSelect();
}

bool  MainPageAppBar::IsAnnotPropertyPopupIsOpen()
{
	if (m_curAppbarType == foxit::rdk::uiextensions::AppBarType::AnnotAppBar)
		return annotToolControl->IsPopupIsOpen();
	return false;
}

void MainPageAppBar::SetPropertyCurColor(Windows::UI::Color color)
{
	if (m_curAppbarType == foxit::rdk::uiextensions::AppBarType::AnnotAppBar)
		annotToolControl->SetPropertyCurColor(color);
}

void foxit::rdk::uiextensions::MainPageAppBar::OnCPDFFlyoutOpened(Platform::Object ^sender, Platform::Object ^args)
{
	
}

void foxit::rdk::uiextensions::MainPageAppBar::OnCPDFFlyoutClosed(Platform::Object ^sender, Platform::Object ^args)
{
	_isCPDFFlyoutOpen = false;

	RestartTimeTick();
}

void foxit::rdk::uiextensions::MainPageAppBar::AppBarControl_PointerReleased(Platform::Object^ sender, Windows::UI::Xaml::Input::PointerRoutedEventArgs^ e)
{
	
}

void foxit::rdk::uiextensions::MainPageAppBar::OnSizeChanged(Platform::Object ^sender, Windows::UI::Core::WindowSizeChangedEventArgs ^e)
{
	Windows::Foundation::Rect _winrect = Windows::UI::Xaml::Window::Current->Bounds;
	double height = this->ActualHeight;

	secondaryCommand->ResetScrollViewHeight();
	if (SecondaryCommandsPopup->IsOpen)
	{
		if (!GetIsDesktopPlatform())
		{
			double actualHeight = secondaryCommand->GetActualHeight();
			if (_winrect.Width > _winrect.Height)
			{
				Windows::Foundation::Collections::IVectorView<String^>^ langus = Windows::Globalization::ApplicationLanguages::Languages;
				if (langus->Size > 0 && (langus->GetAt(0) == "ko" || langus->GetAt(0) == "ja"))
				{
					secondaryCommand->SetPanelWidth(160);
					SecondaryCommandsPopup->HorizontalOffset = _winrect.Width - 160;
				}
				else
				{
					secondaryCommand->SetPanelWidth(155);
					SecondaryCommandsPopup->HorizontalOffset = _winrect.Width - 155;
				}

				double height = _winrect.Height - 60;

				if (height > actualHeight)
					height = actualHeight;

				if (_winrect.Height - 60 < height)
				{
					SecondaryCommandsPopup->VerticalOffset = -(_winrect.Height - 60);
				}
				else
				{
					SecondaryCommandsPopup->VerticalOffset = -height;
				}
			}
			else
			{
				secondaryCommand->SetPanelWidth(_winrect.Width);
				SecondaryCommandsPopup->HorizontalOffset = 0;
				SecondaryCommandsPopup->VerticalOffset = -actualHeight;
			}
		}
		else
		{
			SecondaryCommandsPopup->IsOpen = false;
			auto size = MainAppBar->Children->Size;
			for (unsigned int i = 0; i < size; i++)
			{
				auto _item = MainAppBar->Children->GetAt(i);
				auto _ICommandBarElement = dynamic_cast<Windows::UI::Xaml::Controls::ICommandBarElement^>(_item);
				if (_ICommandBarElement)
				{
					auto _IsCompact = _ICommandBarElement->IsCompact;
					_ICommandBarElement->IsCompact = !_ICommandBarElement->IsCompact;
				}
			}
			BackCommand->IsCompact = !BackCommand->IsCompact;
			SetMainAppBarCompact(true);
		}
	}

	if (viewtoolPanelPopup->IsOpen)
	{
		if (GetIsDesktopPlatform())
		{
			
			viewtoolPanelPopup->IsOpen = false;
			ViewCommand->IsChecked = false;
		}
		else
		{
			ViewToolAppBar^ viewTool = dynamic_cast<ViewToolAppBar^>(viewtoolPanelPopup->Child);
			if (viewTool)
				viewTool->ResetToolSize();

			Windows::Foundation::Collections::IVectorView<String^>^ langus = Windows::Globalization::ApplicationLanguages::Languages;
			double heightvalue = 65;
			String^ lang = langus->GetAt(0);
			if (langus->Size > 0 && (lang == "ko"
				|| lang == "ja"
				))
			{
				if (_winrect.Width < _winrect.Height)
				{
					heightvalue = 140;
				}
				else
				{
					heightvalue = 75;
				}
			}
			else
			{
				if (_winrect.Width < _winrect.Height)
				{
					heightvalue = 130;
				}
				else
				{
					heightvalue = 65;
				}
			}

			viewtoolPanelPopup->VerticalOffset = _winrect.Height - heightvalue;
		}
	}
}

void foxit::rdk::uiextensions::MainPageAppBar::SetMainAppBarCompact(bool isCompact)
{
	if (isCompact)
	{
		AppBarControl->Height = 44;
	}
	else
	{
		Windows::Foundation::Collections::IVectorView<String^>^ langus = Windows::Globalization::ApplicationLanguages::Languages;
		String^ lang = langus->GetAt(0);
		if (langus->Size > 0 && langus->GetAt(0) == "en-US" || langus->Size > 0 && langus->GetAt(0) == "zh-Hant-TW")
		{
			AppBarControl->Height = 75;
		}
		else
		{
			AppBarControl->Height = 60;
		}
	}
}

bool MainPageAppBar::IsNoConnectedEnable::get()
{
	return true;
}

void MainPageAppBar::SetFreeTextPropertyShow(bool isShow)
{
	if (m_curAppbarType == foxit::rdk::uiextensions::AppBarType::AnnotAppBar)
		annotToolControl->SetFreeTextPropertyShow(isShow);
}

void MainPageAppBar::SetUnRedoIsEnabled(bool isEnable)
{
	if (!isEnable)
	{
		RadioUndo->IsEnabled = isEnable;
		RadioRedo->IsEnabled = isEnable;
	}
	else
	{
		UpdateUndoControlState();
	}
	annotToolControl->SetUnRedoIsEnabled(isEnable);
}

void MainPageAppBar::ShowEyeProtectionModeControl()
{
	_Flyout = ref new Windows::UI::Xaml::Controls::Flyout();
	Windows::UI::Xaml::Style^ _FlyoutPresenterStyle;
	_FlyoutPresenterStyle = this->Resources->HasKey(L"ButtonFlyoutPresenterStyle") ? dynamic_cast<Windows::UI::Xaml::Style^>(this->Resources->Lookup(L"ButtonFlyoutPresenterStyle")) : nullptr;
	_Flyout->FlyoutPresenterStyle = _FlyoutPresenterStyle;
	EyeProtectionModeControl^ control = ref new EyeProtectionModeControl();
	control->SetPageView(UIExtMgr);
	_Flyout->Content = control;

	_Flyout->Closed += ref new Windows::Foundation::EventHandler<Platform::Object ^>([=](Platform::Object ^, Platform::Object ^)
	{
		ViewCommand->IsChecked = false;

		RestartTimeTick();
	});

	Windows::UI::Xaml::Controls::Flyout::SetAttachedFlyout(ViewCommand, _Flyout);
	Windows::UI::Xaml::Controls::Flyout::ShowAttachedFlyout(ViewCommand);
}

void MainPageAppBar::UpdateResetFormControlState()
{
	secondaryCommand->UpdateBtnState();
}

void foxit::rdk::uiextensions::MainPageAppBar::OnSecondaryCommandLoaded(Platform::Object ^sender, Windows::UI::Xaml::RoutedEventArgs ^e)
{
	auto _winrect = Windows::UI::Xaml::Window::Current->Bounds;
	double actualHeight = secondaryCommand->ActualHeight;
	double actualWidth = secondaryCommand->ActualWidth;
	SecondaryCommandsPopup->VerticalOffset = -actualHeight;
	
	if (View && GetIsDesktopPlatform())
	{
		auto screenPt = imgMoreMain->TransformToVisual(nullptr)->TransformPoint(Windows::Foundation::Point((float)imgMoreMain->ActualWidth, (float)imgMoreMain->ActualHeight));
		SecondaryCommandsPopup->VerticalOffset = screenPt.Y;
		SecondaryCommandsPopup->HorizontalOffset = screenPt.X - secondaryCommand->ActualWidth;
	}
}

void MainPageAppBar::SetFlyoutshowFlag(bool isShow)
{
	m_bIsFlyoutShow = isShow;
}

void foxit::rdk::uiextensions::MainPageAppBar::RadioSignature_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{
	UIExtMgr->ShowDrawSignatureControl();
}


void foxit::rdk::uiextensions::MainPageAppBar::PPOCommand_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{

}
