﻿/**
 * Copyright (C) 2003-2018, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit Mobile PDF SDK to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */


#pragma once

#include "MainPageAppBar.g.h"
#include "MainSecondaryCommandBar.xaml.h"

namespace foxit
{
	namespace rdk
	{
		namespace uiextensions
		{
			ref class BookMarkGoToData;
			ref class RotateFloatControl;
			ref class NoteTipFloatControl;
			ref class ViewToolAppBar;
			[Windows::Foundation::Metadata::WebHostHidden]
			public ref class MainPageAppBar sealed
			{
			public:
				MainPageAppBar(UIExtensionsManager^ pUIView);

			internal:
				property PDFViewCtrl^ View
				{
					PDFViewCtrl^ get();
				}

				property UIExtensionsManager^ UIExtMgr
				{
					UIExtensionsManager^ get();
				}

			public:
				void SetOutlineAppbarButton(bool isHasBookmark);
				void InitMainAppbar(AppBarType appbarType);

				void ShowAppBar();
				void HideAppBar();
				void HideAppBar(bool useTransitions);
				void ActivateAppBar();
				void StopHideTimeTick();
				void RestartTimeTick();

				AppBarType GetCurAppBarType();
				void HideRotateFloatWindow();
				Windows::UI::Xaml::Visibility GetMainAppBarVisible();
				void HideMorePopup();
				bool GetMorePopupIsOpen();
				void SetAppBarCompact();
				void ResetMorePopupLocation();
				void HideViewToolPanelPopup();
				bool GetViewToolPanelPopupIsOpen();

				void ShowBookmarkPage();

				void UpdateReflowViewAppBarButtonState();
				void UpdateAnnotToolUnRedo();

				void SetAnnotToolStampPopupClosed();
				void UpdateUndoControlState();
				void HideNoteStylePopup();
				void ResetAnnotToolBG();
				void SetAnnotPropertyPopupClosed();
				void InitCurNoteIcon();
				void InitStampAppBarSelect();
				bool IsAnnotPropertyPopupIsOpen();
				void SetPropertyCurColor(Windows::UI::Color color);
				void SetSaveBtnUnEnabled();
				void SetMainAppBarCompact(bool isCompact);
				void ShowEyeProtectionModeControl();
				void UpdateResetFormControlState();

				property bool IsNoConnectedEnable
				{
					bool get();
				}
				void SetFreeTextPropertyShow(bool isShow);
				void SetUnRedoIsEnabled(bool isEnable);
				bool IsMobileAnnnotMoreToolOpen();
				void HideSecondaryPopup();
				void SetMainBarCompact();
				void SetFlyoutshowFlag(bool isShow);

			private:
				void GotoPageIndex(BookMarkGoToData^ goToData);
				void ShowRotateFloatWindows();
				void InitTimerClick();
				void LoadAnnotBar();
				void ResetAppBarButtonCompact();

			private:
				Platform::WeakReference m_RdkViewWeakRef;
				Platform::WeakReference m_UIViewWeakRef;
				AppBarType  m_curAppbarType;
				bool m_isOpen;
				Windows::UI::Xaml::Controls::Primitives::Popup^ m_RotatePopup;
				RotateFloatControl^ rotateControl;
				Windows::UI::Xaml::Controls::Primitives::Popup^ m_TipPopup;
				NoteTipFloatControl^ tipControl;
				Windows::UI::Xaml::DispatcherTimer^ m_HideTimer;
				Windows::UI::Xaml::Visibility m_visible;
				Windows::Foundation::EventRegistrationToken			m_BackPresseEventToken, m_SystemNavigationBackRequestedEventToken;
				bool m_bIsFlyoutShow;
				Windows::Foundation::EventRegistrationToken m_WindowSizeChangedEventToken;
				Windows::Foundation::EventRegistrationToken m_CPDFWindowChangedEventToken;
				Windows::Foundation::EventRegistrationToken m_HideAppBarAnimationCompletedEventToken;
				Windows::UI::Xaml::Controls::Primitives::Popup^ viewtoolPanelPopup;
				bool m_isStopTick;
				Windows::Foundation::EventRegistrationToken m_AfterUndoEventToken, m_AfterRedoEventToken, m_OperaListChangedUndoRedoEventToken;
				Windows::UI::Xaml::Controls::Flyout^ _Flyout;
				bool _isCPDFFlyoutOpen;
				Windows::Foundation::EventRegistrationToken __windowSizeChangedEventRegistrationToken;

			private:
				void BackCommand_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e);
				void OutlineCommand_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e);
				void ViewCommand_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e);
				void CommentCommand_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e);
				void SearchCommand_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e);
				void appBarSave_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e);
				void appBarSaveAs_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e);
				void HideAnnot_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e);
				void FileProperty_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e);
				void ExitCommand_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e);
				void Button_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e);
				void RadioAnnot_Checked(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e);
				void imgMore_PointerPressed(Platform::Object^ sender, Windows::UI::Xaml::Input::PointerRoutedEventArgs^ e);
				void ShowAnnot_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e);
				void OnTick(Platform::Object ^sender, Platform::Object ^args);
				void imgMore_Tapped(Platform::Object^ sender, Windows::UI::Xaml::Input::TappedRoutedEventArgs^ e);
				void MainAppBar_Opening(Platform::Object^ sender, Platform::Object^ e);
				void MainAppBar_Closing(Platform::Object^ sender, Platform::Object^ e);
				void AppBarControl_PointerEntered(Platform::Object^ sender, Windows::UI::Xaml::Input::PointerRoutedEventArgs^ e);
				void AppBarControl_PointerExited(Platform::Object^ sender, Windows::UI::Xaml::Input::PointerRoutedEventArgs^ e);
				void imgMoreView_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e);
				void imgMoreMain_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e);
				void imgMoreMain_Tapped(Platform::Object^ sender, Windows::UI::Xaml::Input::TappedRoutedEventArgs^ e);
				void Canvas_PointerEntered(Platform::Object^ sender, Windows::UI::Xaml::Input::PointerRoutedEventArgs^ e);
				void Canvas_PointerExited(Platform::Object^ sender, Windows::UI::Xaml::Input::PointerRoutedEventArgs^ e);
				void StackPanel_PointerEntered(Platform::Object^ sender, Windows::UI::Xaml::Input::PointerRoutedEventArgs^ e);
				void StackPanel_PointerExited(Platform::Object^ sender, Windows::UI::Xaml::Input::PointerRoutedEventArgs^ e);
				void OnBackPressed(Platform::Object ^sender, Windows::Phone::UI::Input::BackPressedEventArgs ^args);
				void UserControl_Loaded(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e);
				void UserControl_Unloaded(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e);
				void AppBarControl_Tapped(Platform::Object^ sender, Windows::UI::Xaml::Input::TappedRoutedEventArgs^ e);
				void OnBackRequested(Platform::Object ^sender, Windows::UI::Core::BackRequestedEventArgs ^args);
				void OnClosed(Platform::Object ^sender, Platform::Object ^args);
				void ReflowPageCommand_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e);
				void ThumbnailPageCommand_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e);
				void exitAnnotTool_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e);
				void ViewCommand_Tapped(Platform::Object^ sender, Windows::UI::Xaml::Input::TappedRoutedEventArgs^ e);
				/*	void OnSizeChanged(Platform::Object ^sender, Windows::UI::Xaml::SizeChangedEventArgs ^e);*/
				void OnHideAppBarAnimationCompleted(Platform::Object ^sender, Platform::Object ^args);
				void cPDFCommand_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e);
				void RadioUndo_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e);
				void RadioRedo_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e);
				void OnAfterUndo(Platform::Object ^sender, Platform::Object ^args);
				void OnAfterRedo(Platform::Object ^sender, Platform::Object ^args);
				void OnOperaListChanged(Platform::Object ^sender, Platform::Object ^args);
				void OnCPDFFlyoutOpened(Platform::Object ^sender, Platform::Object ^args);
				void OnCPDFFlyoutClosed(Platform::Object ^sender, Platform::Object ^args);
				void AppBarControl_PointerReleased(Platform::Object^ sender, Windows::UI::Xaml::Input::PointerRoutedEventArgs^ e);
				void OnSizeChanged(Platform::Object ^sender, Windows::UI::Core::WindowSizeChangedEventArgs ^e);
				void OnViewPhoneControlLoaded(Platform::Object ^sender, Windows::UI::Xaml::RoutedEventArgs ^e);
				void OnSecondaryCommandLoaded(Platform::Object ^sender, Windows::UI::Xaml::RoutedEventArgs ^e);
				void RadioSignature_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e);
				void PPOCommand_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e);
			};
		}
	}
}
