﻿/**
 * Copyright (C) 2003-2018, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit Mobile PDF SDK to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */


#include "pch.h"
#include "MainSecondaryCommandBar.xaml.h"
#include "MainPageAppBar.xaml.h"
#include "PropertyPage.xaml.h"
#include "Annotations\AnnotationsPage.xaml.h"

using namespace foxit::rdk::uiextensions;
using namespace foxit::rdk;

using namespace Platform;
using namespace Windows::Foundation;
using namespace Windows::Foundation::Collections;
using namespace Windows::UI::Xaml;
using namespace Windows::UI::Xaml::Controls;
using namespace Windows::UI::Xaml::Controls::Primitives;
using namespace Windows::UI::Xaml::Data;
using namespace Windows::UI::Xaml::Input;
using namespace Windows::UI::Xaml::Media;
using namespace Windows::UI::Xaml::Navigation;

// The User Control item template is documented at http://go.microsoft.com/fwlink/?LinkId=234236

MainSecondaryCommandBar::MainSecondaryCommandBar()
{
	InitializeComponent();

	ResetScrollViewHeight();
	this->Loaded += ref new Windows::UI::Xaml::RoutedEventHandler(this, &foxit::rdk::uiextensions::MainSecondaryCommandBar::OnLoaded);
}

void MainSecondaryCommandBar::ResetScrollViewHeight()
{
	Windows::Foundation::Rect rect = Windows::UI::Xaml::Window::Current->Bounds;
	if (Windows::System::Profile::AnalyticsInfo::VersionInfo->DeviceFamily == "Windows.Mobile")
	{
		double actualHeight = GetActualHeight();
		double mainAppBarHeight = 60;

		scrollView->VerticalScrollMode = Windows::UI::Xaml::Controls::ScrollMode::Disabled;
		if (rect.Width > rect.Height)
		{
			double height = rect.Height - mainAppBarHeight;
			if (height > actualHeight)
				height = actualHeight;
			scrollView->Height = rect.Height - mainAppBarHeight;
			if (scrollView->Height < actualHeight)
				scrollView->VerticalScrollMode = Windows::UI::Xaml::Controls::ScrollMode::Auto;
			else
				scrollView->Height = height;
		}
		else
		{
			scrollView->Height = actualHeight;
		}
	}
}

double MainSecondaryCommandBar::GetActualHeight()
{
	auto size = secondaryCommand->Children->Size - 1;
	return size * 40;
}

MainPageAppBar^ MainSecondaryCommandBar::MainAppBar::get()
{
	return m_MainAppBarWeakRef.Resolve<MainPageAppBar>();
}

void MainSecondaryCommandBar::RelateMainAppBar(MainPageAppBar^ mainAppBar)
{
	m_MainAppBarWeakRef = mainAppBar;
}

void MainSecondaryCommandBar::SetPanelWidth(double widthValue)
{
	secondaryCommand->Width = widthValue;
}

void MainSecondaryCommandBar::UpdateUnRedoState()
{
	auto pUndo = UndoFactory::GetUndoInstance();
	if (MainAppBar && MainAppBar->View && MainAppBar->View->GetDoc())
	{
		if (pUndo->CanUndo(MainAppBar->UIExtMgr))
		{
			appBarUndo->IsEnabled = true;
		}
		else
		{
			appBarUndo->IsEnabled = false;
		}
		if (pUndo->CanRedo(MainAppBar->UIExtMgr))
		{
			appBarRedo->IsEnabled = true;
		}
		else
		{
			appBarRedo->IsEnabled = false;
		}
	}
}

void foxit::rdk::uiextensions::MainSecondaryCommandBar::secondaryCommand_PointerExited(Platform::Object^ sender, Windows::UI::Xaml::Input::PointerRoutedEventArgs^ e)
{
	if (MainAppBar)
		MainAppBar->RestartTimeTick();
}


void foxit::rdk::uiextensions::MainSecondaryCommandBar::appBarUndo_Tapped(Platform::Object^ sender, Windows::UI::Xaml::Input::TappedRoutedEventArgs^ e)
{
	auto pUndo = UndoFactory::GetUndoInstance();
	if (MainAppBar && MainAppBar->View)
	{
		if (pUndo->CanUndo(MainAppBar->UIExtMgr))
		{
			if (MainAppBar->View->GetDoc())
				pUndo->Undo(MainAppBar->UIExtMgr);
		}
	}
}


void foxit::rdk::uiextensions::MainSecondaryCommandBar::appBarRedo_Tapped(Platform::Object^ sender, Windows::UI::Xaml::Input::TappedRoutedEventArgs^ e)
{
	auto pUndo = UndoFactory::GetUndoInstance();

	if (MainAppBar && MainAppBar->View)
	{
		if (pUndo->CanRedo(MainAppBar->UIExtMgr))
		{
			if (MainAppBar->View->GetDoc())
				pUndo->Redo(MainAppBar->UIExtMgr);
		}
	}
}


void foxit::rdk::uiextensions::MainSecondaryCommandBar::appBarSave_Tapped(Platform::Object^ sender, Windows::UI::Xaml::Input::TappedRoutedEventArgs^ e)
{
	if (MainAppBar && MainAppBar->View)
	{
		if (MainAppBar->View->IsModified())
		{
			MainAppBar->UIExtMgr->SaveDoc();
			MainAppBar->ActivateAppBar();
			MainAppBar->HideSecondaryPopup();
			MainAppBar->SetMainBarCompact();
		}
	}
}


void foxit::rdk::uiextensions::MainSecondaryCommandBar::appBarSaveAs_Tapped(Platform::Object^ sender, Windows::UI::Xaml::Input::TappedRoutedEventArgs^ e)
{
	if (MainAppBar && MainAppBar->View)
	{
		MainAppBar->UIExtMgr->SaveAsFile();
		MainAppBar->ActivateAppBar();
		MainAppBar->HideSecondaryPopup();
		MainAppBar->SetMainBarCompact();
	}
}


void foxit::rdk::uiextensions::MainSecondaryCommandBar::appBarShareContent_Tapped(Platform::Object^ sender, Windows::UI::Xaml::Input::TappedRoutedEventArgs^ e)
{
	try
	{
		Windows::ApplicationModel::DataTransfer::DataTransferManager::GetForCurrentView()->ShowShareUI();
	}
	catch (Platform::Exception^)
	{

	}
}


void foxit::rdk::uiextensions::MainSecondaryCommandBar::HideAnnotBar_Tapped(Platform::Object^ sender, Windows::UI::Xaml::Input::TappedRoutedEventArgs^ e)
{
	HideAnnotBar->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
	ShowAnnotBar->Visibility = Windows::UI::Xaml::Visibility::Visible;
	if (MainAppBar && MainAppBar->View)
	{
		MainAppBar->UIExtMgr->IsShowAnnot = false;
		MainAppBar->View->Redraw();
		MainAppBar->ActivateAppBar();
		MainAppBar->HideSecondaryPopup();
		MainAppBar->SetMainBarCompact();
	}
}


void foxit::rdk::uiextensions::MainSecondaryCommandBar::ShowAnnotBar_Tapped(Platform::Object^ sender, Windows::UI::Xaml::Input::TappedRoutedEventArgs^ e)
{
	ShowAnnotBar->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
	HideAnnotBar->Visibility = Windows::UI::Xaml::Visibility::Visible;
	if (MainAppBar && MainAppBar->View)
	{
		MainAppBar->UIExtMgr->IsShowAnnot = true;
		MainAppBar->View->Redraw();
		MainAppBar->ActivateAppBar();
		MainAppBar->HideSecondaryPopup();
		MainAppBar->SetMainBarCompact();
	}
}


void foxit::rdk::uiextensions::MainSecondaryCommandBar::AnnotationsPage_Tapped(Platform::Object^ sender, Windows::UI::Xaml::Input::TappedRoutedEventArgs^ e)
{
	if (m_AnnotationsPage == nullptr)
		m_AnnotationsPage = ref new foxit::rdk::uiextensions::AnnotationsPage(MainAppBar->UIExtMgr);

	/*if (!m_AnnotationsPage->IsPin)
		m_AnnotationsPage->IsPin = true;*/

	if (m_AnnotationsPage->IsShow)
	{
		MainAppBar->ActivateAppBar();

		m_AnnotationsPage->CloseFlyoutPage();

		MainAppBar->HideSecondaryPopup();
		MainAppBar->SetMainAppBarCompact(true);
	}
	else
		m_AnnotationsPage->ShowFlyoutPage();
}

void foxit::rdk::uiextensions::MainSecondaryCommandBar::FileProperty_Tapped(Platform::Object^ sender, Windows::UI::Xaml::Input::TappedRoutedEventArgs^ e)
{
	if (!MainAppBar || !MainAppBar->View)
	{
		return;
	}

	MainAppBar->ActivateAppBar();
	auto _Flyout = ref new Windows::UI::Xaml::Controls::Flyout();
	Windows::UI::Xaml::Style^ _FlyoutPresenterStyle;
	_FlyoutPresenterStyle = this->Resources->HasKey(L"FlyoutPresenterStyle") ? dynamic_cast<Windows::UI::Xaml::Style^>(this->Resources->Lookup(L"FlyoutPresenterStyle")) : nullptr;
	_Flyout->FlyoutPresenterStyle = _FlyoutPresenterStyle;
	_Flyout->Content = ref new PropertyPage(MainAppBar->View);
	_Flyout->Closed += ref new Windows::Foundation::EventHandler<Platform::Object ^>(this, &foxit::rdk::uiextensions::MainSecondaryCommandBar::OnClosed);
	Windows::UI::Xaml::Controls::Flyout::SetAttachedFlyout(MainAppBar->UIExtMgr->FlyoutPlacement, _Flyout);
	Windows::UI::Xaml::Controls::Flyout::ShowAttachedFlyout(MainAppBar->UIExtMgr->FlyoutPlacement);
	_Flyout->Opened += ref new Windows::Foundation::EventHandler<Platform::Object ^>([this, _Flyout](Platform::Object ^, Platform::Object ^)
	{
		m_bIsFlyoutShow = true;

		MainAppBar->SetFlyoutshowFlag(m_bIsFlyoutShow);
		MainAppBar->HideAppBar();

		auto _VisibleBounds = Windows::UI::ViewManagement::ApplicationView::GetForCurrentView()->VisibleBounds;
		auto _Bounds = Windows::UI::Xaml::Window::Current->Bounds;
		auto _FlyoutPresenter = GetVisualParent<Windows::UI::Xaml::Controls::FlyoutPresenter^>(_Flyout->Content);
		if (_FlyoutPresenter)
		{
			std::wstringstream ws;
			ws << L"Opened--MinHeight: " << _FlyoutPresenter->MinHeight;
			ws << L", MaxHeight: " << _FlyoutPresenter->MaxHeight;
			ws << L", ActualWidth: " << _FlyoutPresenter->ActualWidth;
			ws << L", ActualHeight: " << _FlyoutPresenter->ActualHeight;
			ws << L", VisibleBounds.Width: " << _VisibleBounds.Width;
			ws << L", VisibleBounds.Height: " << _VisibleBounds.Height;
			ws << L", _Bounds.Width: " << _Bounds.Width;
			ws << L", _Bounds.Height: " << _Bounds.Height;
			ws << L"\n";
			OutputDebugString(ws.str().c_str());

			auto _height = _VisibleBounds.Height;
			_FlyoutPresenter->MinHeight = _height;
			_FlyoutPresenter->MaxHeight = _height;
			_FlyoutPresenter->Height = _height;
			if (_FlyoutPresenter->ActualWidth > _VisibleBounds.Width)
			{
				_FlyoutPresenter->MinWidth = _VisibleBounds.Width;
				_FlyoutPresenter->MaxWidth = _VisibleBounds.Width;
				_FlyoutPresenter->Width = _VisibleBounds.Width;
			}

			Windows::UI::Xaml::Controls::Primitives::Popup^ _Popup = dynamic_cast<Windows::UI::Xaml::Controls::Primitives::Popup^>(_FlyoutPresenter->Parent);
			if (_Popup)
			{
				_Popup->IsLightDismissEnabled = false;
				double _voffset = _Popup->VerticalOffset;
				if (Windows::System::Profile::AnalyticsInfo::VersionInfo->DeviceFamily == L"Windows.Mobile")
				{
					auto _StatusBar = Windows::UI::ViewManagement::StatusBar::GetForCurrentView();
					_voffset = _StatusBar->OccludedRect.Height;
				}
				else if (Windows::System::Profile::AnalyticsInfo::VersionInfo->DeviceFamily == L"Windows.Desktop")
				{
					auto _screenPt = _FlyoutPresenter->TransformToVisual(nullptr)->TransformPoint(Windows::Foundation::Point(0, 0));
					Windows::UI::Xaml::Media::TranslateTransform^ _transform = dynamic_cast<Windows::UI::Xaml::Media::TranslateTransform^>(MainAppBar->UIExtMgr->FlyoutPlacement->RenderTransform);
					if (_transform == nullptr)
					{
						_transform = ref new Windows::UI::Xaml::Media::TranslateTransform();
						MainAppBar->UIExtMgr->FlyoutPlacement->RenderTransform = _transform;
					}

					if (_transform->Y != _screenPt.Y)
						_transform->Y = _screenPt.Y;

					_voffset = 0;
				}

				_Popup->VerticalOffset = _voffset;
			}
		}

		m_WindowSizeChangedEventToken = Windows::UI::ViewManagement::ApplicationView::GetForCurrentView()->VisibleBoundsChanged += ref new Windows::Foundation::TypedEventHandler<Windows::UI::ViewManagement::ApplicationView^, Platform::Object^>([this, _Flyout](Windows::UI::ViewManagement::ApplicationView^, Platform::Object^)
		{
			auto _VisibleBounds = Windows::UI::ViewManagement::ApplicationView::GetForCurrentView()->VisibleBounds;
			auto _Bounds = Windows::UI::Xaml::Window::Current->Bounds;
			auto _FlyoutPresenter = GetVisualParent<Windows::UI::Xaml::Controls::FlyoutPresenter^>(_Flyout->Content);
			if (_FlyoutPresenter)
			{
				std::wstringstream ws;
				ws << L"Window::Current->SizeChanged--MinHeight: " << _FlyoutPresenter->MinHeight;
				ws << L", MaxHeight: " << _FlyoutPresenter->MaxHeight;
				ws << L", ActualWidth: " << _FlyoutPresenter->ActualWidth;
				ws << L", ActualHeight: " << _FlyoutPresenter->ActualHeight;
				ws << L", VisibleBounds.Width: " << _VisibleBounds.Width;
				ws << L", VisibleBounds.Height: " << _VisibleBounds.Height;
				ws << L", _Bounds.Width: " << _Bounds.Width;
				ws << L", _Bounds.Height: " << _Bounds.Height;
				ws << L"\n";
				OutputDebugString(ws.str().c_str());

				auto _height = _VisibleBounds.Height;
				_FlyoutPresenter->MinHeight = _height;
				_FlyoutPresenter->MaxHeight = _height;
				_FlyoutPresenter->Height = _height;
				if (_FlyoutPresenter->ActualWidth > _VisibleBounds.Width)
				{
					_FlyoutPresenter->MinWidth = _VisibleBounds.Width;
					_FlyoutPresenter->MaxWidth = _VisibleBounds.Width;
					_FlyoutPresenter->Width = _VisibleBounds.Width;
				}

				Windows::UI::Xaml::Controls::Primitives::Popup^ _Popup = dynamic_cast<Windows::UI::Xaml::Controls::Primitives::Popup^>(_FlyoutPresenter->Parent);
				if (_Popup)
				{
					_Popup->IsLightDismissEnabled = false;
					double _voffset = _Popup->VerticalOffset;
					if (Windows::System::Profile::AnalyticsInfo::VersionInfo->DeviceFamily == L"Windows.Mobile")
					{
						auto _StatusBar = Windows::UI::ViewManagement::StatusBar::GetForCurrentView();
						_voffset = _StatusBar->OccludedRect.Height;
					}
					else if (Windows::System::Profile::AnalyticsInfo::VersionInfo->DeviceFamily == L"Windows.Desktop")
					{
						auto _screenPt = _FlyoutPresenter->TransformToVisual(nullptr)->TransformPoint(Windows::Foundation::Point(0, 0));
						Windows::UI::Xaml::Media::TranslateTransform^ _transform = dynamic_cast<Windows::UI::Xaml::Media::TranslateTransform^>(MainAppBar->UIExtMgr->FlyoutPlacement->RenderTransform);
						if (_transform == nullptr)
						{
							_transform = ref new Windows::UI::Xaml::Media::TranslateTransform();
							MainAppBar->UIExtMgr->FlyoutPlacement->RenderTransform = _transform;
						}

						if (_transform->Y != _screenPt.Y)
							_transform->Y = _screenPt.Y;

						_voffset = 0;
					}

					_Popup->VerticalOffset = _voffset;
					_Popup->HorizontalOffset = (_Bounds.Width - _FlyoutPresenter->ActualWidth > 0) ? _Bounds.Width - _FlyoutPresenter->ActualWidth : 0;
				}
			}
		});
	});

	MainAppBar->HideSecondaryPopup();
	MainAppBar->SetMainAppBarCompact(true);
}


void foxit::rdk::uiextensions::MainSecondaryCommandBar::ResetFormButton_Tapped(Platform::Object^ sender, Windows::UI::Xaml::Input::TappedRoutedEventArgs^ e)
{
	if (!MainAppBar || !MainAppBar->View)
	{
		return;
	}

	auto extensionManager = (UIExtensionsManager^)MainAppBar->View->ExtensionsManager;
	if (!extensionManager) return;
	extensionManager->ResetForm();
	MainAppBar->View->SetModified(true);
	MainAppBar->View->Redraw();
}

void foxit::rdk::uiextensions::MainSecondaryCommandBar::UpdateBtnState()
{
	if (MainAppBar && MainAppBar->View && MainAppBar->UIExtMgr->IsShowAnnot)
	{
		ShowAnnotBar->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
		HideAnnotBar->Visibility = Windows::UI::Xaml::Visibility::Visible;
	}
	else
	{
		HideAnnotBar->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
		ShowAnnotBar->Visibility = Windows::UI::Xaml::Visibility::Visible;
	}

	if (MainAppBar && MainAppBar->View)
	{
		auto pDocument = MainAppBar->View->GetDoc();
		bool bCanChangedForm = FSPDFDocUtil::GetIsDocumentCanFillInForm(pDocument) || (pDocument->GetPasswordType() == foxit::pdf::PasswordType::e_PwdOwner);
		bool _hasForm = pDocument->HasForm();
		if (_hasForm) {
			_hasForm = MainAppBar->UIExtMgr->GetSdkForm()->GetFieldCount(nullptr) > 0;
		}
		ResetFormButton->IsEnabled = bCanChangedForm && _hasForm;
		ImportFormDataButton->IsEnabled = bCanChangedForm && _hasForm;
		ExportFormDataButton->IsEnabled = bCanChangedForm && _hasForm;
		appBarSave->IsEnabled = MainAppBar->View->IsModified() ? true : false;

		UpdateUnRedoState();
	}
}

void foxit::rdk::uiextensions::MainSecondaryCommandBar::OnLoaded(Platform::Object ^sender, Windows::UI::Xaml::RoutedEventArgs ^e)
{

}


void foxit::rdk::uiextensions::MainSecondaryCommandBar::OnClosed(Platform::Object ^sender, Platform::Object ^args)
{
	m_bIsFlyoutShow = false;
	if (MainAppBar)
		MainAppBar->SetFlyoutshowFlag(m_bIsFlyoutShow);

	Windows::UI::Xaml::Media::TranslateTransform^ _transform = dynamic_cast<Windows::UI::Xaml::Media::TranslateTransform^>(MainAppBar->UIExtMgr->FlyoutPlacement->RenderTransform);
	if (_transform)
	{
		if (_transform->Y != 0)
			_transform->Y = 0;
	}

	if (m_WindowSizeChangedEventToken.Value)
	{
		Windows::UI::ViewManagement::ApplicationView::GetForCurrentView()->VisibleBoundsChanged -= m_WindowSizeChangedEventToken;
		m_WindowSizeChangedEventToken.Value = 0;
	}
}

void MainSecondaryCommandBar::SetSaveBtnUnEnabled()
{
	appBarSave->IsEnabled = false;
}

void foxit::rdk::uiextensions::MainSecondaryCommandBar::ImportFormDataButton_Tapped(Platform::Object^ sender, Windows::UI::Xaml::Input::TappedRoutedEventArgs^ e)
{
	if (!MainAppBar || !MainAppBar->View)
		return;

	auto extensionManager = (UIExtensionsManager^)MainAppBar->View->ExtensionsManager;
	if (!extensionManager) return;
	extensionManager->ImportFormDataFromFdfFile();
}


void foxit::rdk::uiextensions::MainSecondaryCommandBar::ExportFormDataButton_Tapped(Platform::Object^ sender, Windows::UI::Xaml::Input::TappedRoutedEventArgs^ e)
{
	if (!MainAppBar || !MainAppBar->View)
		return;

	auto extensionManager = (UIExtensionsManager^)MainAppBar->View->ExtensionsManager;
	if (!extensionManager) return;
	extensionManager->ExportFormDataToFdfFile();
}


void foxit::rdk::uiextensions::MainSecondaryCommandBar::SubscribeButton_Tapped(Platform::Object^ sender, Windows::UI::Xaml::Input::TappedRoutedEventArgs^ e)
{
	if (!MainAppBar || !MainAppBar->View)
		return;

	MainAppBar->HideSecondaryPopup();
	MainAppBar->SetMainAppBarCompact(true);
}


void foxit::rdk::uiextensions::MainSecondaryCommandBar::SubscribeManageButton_Tapped(Platform::Object^ sender, Windows::UI::Xaml::Input::TappedRoutedEventArgs^ e)
{
	if (!MainAppBar || !MainAppBar->View)
		return;

	MainAppBar->HideSecondaryPopup();
	MainAppBar->SetMainAppBarCompact(true);
}
