﻿/**
 * Copyright (C) 2003-2018, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit Mobile PDF SDK to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */


#include "pch.h"
#include "PropertyPage.xaml.h"

using namespace foxit::rdk::uiextensions;
using namespace foxit::rdk;

using namespace Platform;
using namespace Windows::Foundation;
using namespace Windows::Foundation::Collections;
using namespace Windows::UI::Xaml;
using namespace Windows::UI::Xaml::Controls;
using namespace Windows::UI::Xaml::Controls::Primitives;
using namespace Windows::UI::Xaml::Data;
using namespace Windows::UI::Xaml::Input;
using namespace Windows::UI::Xaml::Media;
using namespace Windows::UI::Xaml::Navigation;
using namespace Windows::Storage;
using namespace Windows::Storage::Streams;
using namespace Windows::Storage::FileProperties;
using namespace concurrency;

// The blank page template is documented at http://go.microsoft.com/fwlink/?LinkId=390556

PropertyPage::PropertyPage(foxit::rdk::PDFViewCtrl^ viewcontrol)
{
	InitializeComponent();

	m_Document = viewcontrol->GetDoc();

	OnInitializePage(viewcontrol);
}

void PropertyPage::OnNavigatedTo(NavigationEventArgs^ e)
{

}

void PropertyPage::OnInitializePage(foxit::rdk::PDFViewCtrl^ viewcontrol)
{
	m_pProgressBar = ref new ProgressBarFactory();

	InitFileProperty(viewcontrol, m_Document);

	this->Loaded += ref new Windows::UI::Xaml::RoutedEventHandler(this, &foxit::rdk::uiextensions::PropertyPage::OnLoaded);
	this->Unloaded += ref new Windows::UI::Xaml::RoutedEventHandler(this, &foxit::rdk::uiextensions::PropertyPage::OnUnloaded);
}

void foxit::rdk::uiextensions::PropertyPage::InitFileProperty(foxit::rdk::PDFViewCtrl^ viewcontrol, foxit::pdf::PDFDoc^ doc)
{
	if (doc)
	{
		auto storageFile = viewcontrol->GetStorageFile();
		if (storageFile)
		{
			tbFileName->Text = storageFile->Name;
			foxit::pdf::Metadata^ metaData = ref new foxit::pdf::Metadata(doc);
			
			auto _str = metaData->GetValues(L"Author");
			if (_str && _str->GetSize() > 0)
				tbAuthor->Text = _str->GetAt(0);
			else
				AuthorPanel->Visibility = Windows::UI::Xaml::Visibility::Collapsed;

			auto _ResourceLoader = Windows::ApplicationModel::Resources::ResourceLoader::GetForCurrentView("foxit.rdk.uiextensions/Resources");

			foxit::pdf::PasswordType passwordType = doc->GetPasswordType();
			bool bIsOwnerPassword = (passwordType == foxit::pdf::PasswordType::e_PwdOwner);
			auto docEncryptType = doc->GetEncryptionType();
			Platform::String^ docEncryptTypeString;
			switch (docEncryptType)
			{
			case foxit::pdf::EncryptType::e_EncryptNone:
				docEncryptTypeString = L"PropertyPageSecurityProtectionNoEncryptValue";
				break;
			case foxit::pdf::EncryptType::e_EncryptPassword:
				docEncryptTypeString = L"PropertyPageSecurityProtectionPasswordValue";
				break;
			case foxit::pdf::EncryptType::e_EncryptCertificate:
				docEncryptTypeString = L"PropertyPageSecurityProtectionCertificateValue";
				break;
			case foxit::pdf::EncryptType::e_EncryptFoxitDRM:
				docEncryptTypeString = L"PropertyPageSecurityProtectionFoxitDRMValue";
				break;
			case foxit::pdf::EncryptType::e_EncryptCustom:
				docEncryptTypeString = L"PropertyPageSecurityProtectionCustomValue";
				break;
			case foxit::pdf::EncryptType::e_EncryptRMS:
				docEncryptTypeString = L"PropertyPageSecurityProtectionRMSValue";
				break;
			default:
				docEncryptTypeString = L"PropertyPageSecurityProtectionNoEncryptValue";
				break;
			}
			tbSecurity->Text = _ResourceLoader->GetString(docEncryptTypeString);
			tbPrinting->Text = bIsOwnerPassword || FSPDFDocUtil::GetIsDocumentCanBePrint(doc) ? _ResourceLoader->GetString(L"PropertyPageAuthorityAllowedValue") : _ResourceLoader->GetString(L"PropertyPageAuthorityNotAllowedValue");
			tbFillInForm->Text = bIsOwnerPassword || FSPDFDocUtil::GetIsDocumentCanFillInForm(doc) ? _ResourceLoader->GetString(L"PropertyPageAuthorityAllowedValue") : _ResourceLoader->GetString(L"PropertyPageAuthorityNotAllowedValue");
			tbComment->Text = bIsOwnerPassword || FSPDFDocUtil::GetIsDocumentCanAddComment(doc) ? _ResourceLoader->GetString(L"PropertyPageAuthorityAllowedValue") : _ResourceLoader->GetString(L"PropertyPageAuthorityNotAllowedValue");
			tbManagerpagesandbookmarks->Text = bIsOwnerPassword || FSPDFDocUtil::GetIsDocumentCanAssemble(doc) ? _ResourceLoader->GetString(L"PropertyPageAuthorityAllowedValue") : _ResourceLoader->GetString(L"PropertyPageAuthorityNotAllowedValue");
			tbModifydocument->Text = bIsOwnerPassword || FSPDFDocUtil::GetIsDocumentCanModify(doc) ? _ResourceLoader->GetString(L"PropertyPageAuthorityAllowedValue") : _ResourceLoader->GetString(L"PropertyPageAuthorityNotAllowedValue");
			tbContentcopying->Text = bIsOwnerPassword || FSPDFDocUtil::GetIsDocumentCanBeCopy(doc) ? _ResourceLoader->GetString(L"PropertyPageAuthorityAllowedValue") : _ResourceLoader->GetString(L"PropertyPageAuthorityNotAllowedValue");
			tbExtractthecontents->Text = bIsOwnerPassword || FSPDFDocUtil::GetIsDocumentCanExtra(doc) ? _ResourceLoader->GetString(L"PropertyPageAuthorityAllowedValue") : _ResourceLoader->GetString(L"PropertyPageAuthorityNotAllowedValue");
			tbSigning->Text = bIsOwnerPassword || FSPDFDocUtil::GetIsDocumentCanBeSigning(doc) ? _ResourceLoader->GetString(L"PropertyPageAuthorityAllowedValue") : _ResourceLoader->GetString(L"PropertyPageAuthorityNotAllowedValue");

			concurrency::create_task(storageFile->GetBasicPropertiesAsync())
				.then([=](concurrency::task<Windows::Storage::FileProperties::BasicProperties^> properties)
			{
				try
				{
					auto propertyValue = properties.get();
					auto _LongDateDateTimeFormatter = ref new Windows::Globalization::DateTimeFormatting::DateTimeFormatter(L"shortdate");
					tbModifiedDate->Text = _LongDateDateTimeFormatter->Format(propertyValue->DateModified);
					Platform::String^ datestr = propertyValue->DateModified.ToString();

					auto _FileSizeConverter = ref new FileSizeConverter();
					tbSize->Text = ((Platform::IBox<Platform::String^>^)(_FileSizeConverter->Convert(propertyValue->Size, Windows::UI::Xaml::Controls::TextBlock::typeid, nullptr, nullptr)))->Value;
				}
				catch (...)
				{

				}
			});
		}
	}
}


void foxit::rdk::uiextensions::PropertyPage::OnBackPressed(Platform::Object ^sender, Windows::Phone::UI::Input::BackPressedEventArgs ^args)
{
	auto _FlyoutPresenter = foxit::rdk::uiextensions::GetVisualParent<Windows::UI::Xaml::Controls::FlyoutPresenter^>(this);
	if (_FlyoutPresenter)
	{
		auto _Popup = dynamic_cast<Windows::UI::Xaml::Controls::Primitives::Popup^>(_FlyoutPresenter->Parent);
		if (_Popup)
		{
			_Popup->IsOpen = false;
			args->Handled = true;
		}
	}
}


void foxit::rdk::uiextensions::PropertyPage::OnLoaded(Platform::Object ^sender, Windows::UI::Xaml::RoutedEventArgs ^e)
{
	if (Windows::Foundation::Metadata::ApiInformation::IsTypePresent("Windows.Phone.UI.Input.HardwareButtons"))
	{
		m_HardwareButtonsBackPressed = Windows::Phone::UI::Input::HardwareButtons::BackPressed += ref new Windows::Foundation::EventHandler<Windows::Phone::UI::Input::BackPressedEventArgs ^>(this, &PropertyPage::PropertyPage::OnBackPressed);
	}

	if (Windows::Foundation::Metadata::ApiInformation::IsTypePresent("Windows.UI.Core.SystemNavigationManager"))
	{
		m_SystemNavigationBackRequestedEventToken = Windows::UI::Core::SystemNavigationManager::GetForCurrentView()->BackRequested += ref new Windows::Foundation::EventHandler<Windows::UI::Core::BackRequestedEventArgs ^>(this, &foxit::rdk::uiextensions::PropertyPage::OnBackRequested);
	}
}


void foxit::rdk::uiextensions::PropertyPage::OnUnloaded(Platform::Object ^sender, Windows::UI::Xaml::RoutedEventArgs ^e)
{
	if (Windows::Foundation::Metadata::ApiInformation::IsTypePresent("Windows.Phone.UI.Input.HardwareButtons"))
	{
		Windows::Phone::UI::Input::HardwareButtons::BackPressed -= m_HardwareButtonsBackPressed;
		m_HardwareButtonsBackPressed.Value = 0;
	}

	if (Windows::Foundation::Metadata::ApiInformation::IsTypePresent("Windows.UI.Core.SystemNavigationManager"))
	{
		Windows::UI::Core::SystemNavigationManager::GetForCurrentView()->BackRequested -= m_SystemNavigationBackRequestedEventToken;
		m_SystemNavigationBackRequestedEventToken.Value = 0;
	}

	m_Document = nullptr;
}


void foxit::rdk::uiextensions::PropertyPage::OnBackRequested(Platform::Object ^sender, Windows::UI::Core::BackRequestedEventArgs ^args)
{
	auto _FlyoutPresenter = foxit::rdk::uiextensions::GetVisualParent<Windows::UI::Xaml::Controls::FlyoutPresenter^>(this);
	if (_FlyoutPresenter)
	{
		auto _Popup = dynamic_cast<Windows::UI::Xaml::Controls::Primitives::Popup^>(_FlyoutPresenter->Parent);
		if (_Popup)
		{
			_Popup->IsOpen = false;
			args->Handled = true;
		}
	}
}
