﻿/**
 * Copyright (C) 2003-2018, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit Mobile PDF SDK to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */


#include "pch.h"
#include "FindTextControl.xaml.h"
#include "SearchNoFindMsgControl.xaml.h"
#include "SearchProgressingContorl.xaml.h"
#include "SearchTextResultControl.xaml.h"

#include <regex>

using namespace foxit::rdk::uiextensions;

using namespace Platform;
using namespace Windows::Foundation;
using namespace Windows::Foundation::Collections;
using namespace Windows::UI::Xaml;
using namespace Windows::UI::Xaml::Controls;
using namespace Windows::UI::Xaml::Controls::Primitives;
using namespace Windows::UI::Xaml::Data;
using namespace Windows::UI::Xaml::Input;
using namespace Windows::UI::Xaml::Media;
using namespace Windows::UI::Xaml::Navigation;
using namespace Windows::Storage;
using namespace Windows::UI::Input;
using namespace Windows::System;

// The User Control item template is documented at http://go.microsoft.com/fwlink/?LinkId=234236

FindTextControl::FindTextControl()
{
	InitializeComponent();

	OnInitializeControl();
}

void FindTextControl::OnInitializeControl()
{
	m_MsgBox = ref new MessageDlg();
	m_MsgBox->Background = ref new Windows::UI::Xaml::Media::SolidColorBrush(Windows::UI::Colors::Transparent);
	m_MsgBox->IsStoreAppBarState = true;
	m_SearchProgressBox = ref new MessageDlg();
	m_SearchProgressBox->Background = ref new Windows::UI::Xaml::Media::SolidColorBrush(Windows::UI::Colors::Transparent);
	m_SearchProgressBox->VerticalAlignment = Windows::UI::Xaml::VerticalAlignment::Center;
	m_SearchProgressBox->MessageContent = ref new SearchProgressingContorl();

	this->SizeChanged += ref new Windows::UI::Xaml::SizeChangedEventHandler(this, &foxit::rdk::uiextensions::FindTextControl::OnSizeChanged);
	this->Loaded += ref new Windows::UI::Xaml::RoutedEventHandler(this, &foxit::rdk::uiextensions::FindTextControl::OnLoaded);
	this->Unloaded += ref new Windows::UI::Xaml::RoutedEventHandler(this, &foxit::rdk::uiextensions::FindTextControl::OnUnloaded);
}

void foxit::rdk::uiextensions::FindTextControl::OnLoaded(Platform::Object ^sender, Windows::UI::Xaml::RoutedEventArgs ^e)
{
	foxit::rdk::uiextensions::UIExtensionsManager^ extensions = getUIExtensionsMgr();
	if (extensions && extensions->getTextMgr())
	{
		m_UIViewWeakRef = extensions;
		m_TextManagerWeakRef = extensions->getTextMgr();
		m_SearchResultEventToken = extensions->getTextMgr()->SearchResult += ref new foxit::rdk::uiextensions::SearchResultEventHandler(this, &foxit::rdk::uiextensions::FindTextControl::OnSearchResult);
	}
	
	m_VisibleBoundsChangedEventToken = Windows::UI::ViewManagement::ApplicationView::GetForCurrentView()->VisibleBoundsChanged += ref new Windows::Foundation::TypedEventHandler<Windows::UI::ViewManagement::ApplicationView ^, Platform::Object ^>(this, &foxit::rdk::uiextensions::FindTextControl::OnVisibleBoundsChanged);
	m_QuerySubmittedEventToken = SearchEditBox->QuerySubmitted += ref new Windows::Foundation::TypedEventHandler<Platform::Object ^, foxit::rdk::uiextensions::SearchTextBoxQuerySubmittedEventArgs ^>(this, &foxit::rdk::uiextensions::FindTextControl::OnQuerySubmitted);

	auto _InputPane = Windows::UI::ViewManagement::InputPane::GetForCurrentView();
	__inputPaneShowingEventRegistrationToken = _InputPane->Showing += ref new Windows::Foundation::TypedEventHandler<Windows::UI::ViewManagement::InputPane ^, Windows::UI::ViewManagement::InputPaneVisibilityEventArgs ^>(this, &foxit::rdk::uiextensions::FindTextControl::OnInputPaneShowing);
	__inputPaneHidingEventRegistrationToken = _InputPane->Hiding += ref new Windows::Foundation::TypedEventHandler<Windows::UI::ViewManagement::InputPane ^, Windows::UI::ViewManagement::InputPaneVisibilityEventArgs ^>(this, &foxit::rdk::uiextensions::FindTextControl::OnInputPaneHiding);

	ResetFindBarState();

	UpdateView();

	m_ParentPanel = dynamic_cast<Windows::UI::Xaml::Controls::Panel^>(this->Parent);
	m_ParentContentControl = dynamic_cast<Windows::UI::Xaml::Controls::ContentControl^>(this->Parent);
}


void foxit::rdk::uiextensions::FindTextControl::OnUnloaded(Platform::Object ^sender, Windows::UI::Xaml::RoutedEventArgs ^e)
{
	if (TextManager)
	{
		if (m_SearchResultEventToken.Value != 0)
		{
			TextManager->SearchResult -= m_SearchResultEventToken;
			m_SearchResultEventToken.Value = 0;
		}
	}

	if (m_VisibleBoundsChangedEventToken.Value)
	{
		Windows::UI::ViewManagement::ApplicationView::GetForCurrentView()->VisibleBoundsChanged -= m_VisibleBoundsChangedEventToken;
		m_VisibleBoundsChangedEventToken.Value = 0;
	}

	if (m_QuerySubmittedEventToken.Value)
	{
		SearchEditBox->QuerySubmitted -= m_QuerySubmittedEventToken;
		m_QuerySubmittedEventToken.Value = 0;
	}

	auto _InputPane = Windows::UI::ViewManagement::InputPane::GetForCurrentView();
	if (__inputPaneShowingEventRegistrationToken.Value)
	{
		_InputPane->Showing -= __inputPaneShowingEventRegistrationToken;
		__inputPaneShowingEventRegistrationToken.Value = 0;
	}

	if (__inputPaneHidingEventRegistrationToken.Value)
	{
		_InputPane->Hiding -= __inputPaneHidingEventRegistrationToken;
		__inputPaneHidingEventRegistrationToken.Value = 0;
	}

	m_TextManagerWeakRef = nullptr;
	m_PageViewWeakRef = nullptr;
}


void foxit::rdk::uiextensions::FindTextControl::SearchEditBox_TextChanged(Platform::Object^ sender, Windows::UI::Xaml::Controls::TextChangedEventArgs^ e)
{
	bool bIsChange = false;
	if (m_IsOutSideCaused)
	{
		m_IsOutSideCaused = false;
	}
	else
	{
		if (TextManager)
		{
			auto _textinfo = TextManager->CurSearchTextInfo;
			TextManager->StopSearchString();
			bIsChange = _textinfo != TextManager->CurSearchTextInfo;
		}

		ShowFindButtonsToVisible(false);
	}

	if (bIsChange && UIExtMgr)
		UIExtMgr->viewControl->Update();
	
	m_SearchTextChanged = true;
}


void foxit::rdk::uiextensions::FindTextControl::SearchEditBox_KeyUp(Platform::Object^ sender, Windows::UI::Xaml::Input::KeyRoutedEventArgs^ e)
{
	auto virtualKey = e->Key;
	auto keyStatus = e->KeyStatus;

	if (virtualKey == VirtualKey::Enter)
	{
		auto coreWindow = Window::Current->CoreWindow;
		auto downState = Windows::UI::Core::CoreVirtualKeyStates::Down;
		bool menuKey = (coreWindow->GetKeyState(VirtualKey::Menu) & downState) == downState;
		bool controlKey = (coreWindow->GetKeyState(VirtualKey::Control) & downState) == downState;
		bool shiftKey = (coreWindow->GetKeyState(VirtualKey::Shift) & downState) == downState;
		bool noModifiers = !menuKey && !controlKey && !shiftKey;
		if (noModifiers)
		{
			if (!SearchEditBox->Text->IsEmpty() && m_SearchTextChanged)
			{
				if (TextManager && UIExtMgr)
				{
					auto _FindText = TrimSpace(SearchEditBox->Text);
					if (_FindText->Length() > 0)
					{
						TextManager->StartSearchString((int)UIExtMgr->GetCurrentPageIndex(), _FindText);
						ShowProgressControl();
					}
				}
				e->Handled = true;
			}

			Windows::UI::ViewManagement::InputPane::GetForCurrentView()->TryHide();
		}
	}
}


void foxit::rdk::uiextensions::FindTextControl::SearchEditBox_LostFocus(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{
	
}


void foxit::rdk::uiextensions::FindTextControl::SearchStartBtn_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{
	if (TextManager && m_SearchTextChanged)
	{
		m_SearchTextChanged = false;
		TextManager->StartSearchString((int)UIExtMgr->GetCurrentPageIndex(), SearchEditBox->Text);
		ShowProgressControl();
	}
}


void foxit::rdk::uiextensions::FindTextControl::ReadPageFindPrev_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{
	if (TextManager)
		TextManager->SearchPrev();
}


void foxit::rdk::uiextensions::FindTextControl::ReadPageFindNext_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{
	if (TextManager)
		TextManager->SearchNext();
}


void foxit::rdk::uiextensions::FindTextControl::FindCloseCommand_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{
	bool bIsChange = false;	
	
	if (TextManager)
	{
		auto _textinfo = TextManager->CurSearchTextInfo;
		TextManager->StopSearchString();
		bIsChange = _textinfo != TextManager->CurSearchTextInfo;
		getUIExtensionsMgr()->ResetDefaultAppBarButton();

		auto _controlView = foxit::rdk::uiextensions::GetVisualParent<foxit::rdk::PDFViewCtrl^>(this);
		UIExtMgr->SetPageNumberSliderVisible(true);
		if (bIsChange)
			_controlView->Update();
	}
}

void FindTextControl::ResetFindBarState()
{
	m_SearchTextChanged = SearchEditBox->Text->Length() > 0 ? true : false;

	if (TextManager)
		TextManager->StopSearchString();

	ShowFindButtonsToVisible(false);
	DisableResultsControl();
}

void FindTextControl::UpdateView()
{
	if (!this->Parent)
		return;

	auto _WinBounds = Windows::UI::ViewManagement::ApplicationView::GetForCurrentView()->VisibleBounds;
	auto _ActualWidth = this->ActualWidth;
	
	if (_ActualWidth > 0)
	{
		if (Windows::System::Profile::AnalyticsInfo::VersionInfo->DeviceFamily == L"Windows.Desktop")
		{
			if (_ActualWidth > 550)
			{
				CancelSearchButton->Margin = Windows::UI::Xaml::ThicknessHelper::FromLengths(15, 0, 15, 0);
				SearchEditBox->MinWidth = 350;
				SearchEditBox->MaxWidth = 350;
			}
			else
			{
				CancelSearchButton->Margin = Windows::UI::Xaml::ThicknessHelper::FromLengths(10, 0, 10, 0);
				SearchEditBox->MinWidth = 250;
				SearchEditBox->MaxWidth = 250;

				auto _BorderThickness = SearchEditBox->BorderThickness;
				auto _CanWidth = this->ActualWidth - CancelSearchButton->Margin.Left - CancelSearchButton->Margin.Right - CancelSearchButton->ActualWidth;
				if (_CanWidth < SearchEditBox->MinWidth)
				{
					SearchEditBox->MinWidth = _CanWidth;
					SearchEditBox->Width = _CanWidth;
				}
			}
		}
		else if (Windows::System::Profile::AnalyticsInfo::VersionInfo->DeviceFamily == L"Windows.Mobile")
		{
			auto _CanWidth = this->ActualWidth - CancelSearchButton->Margin.Left - CancelSearchButton->Margin.Right - CancelSearchButton->ActualWidth;
			if (_CanWidth < SearchEditBox->MinWidth)
			{
				SearchEditBox->MinWidth = _CanWidth;
				SearchEditBox->MaxWidth = _CanWidth;
				SearchEditBox->Width = _CanWidth;
			}
		}
	}
}

void FindTextControl::ShowFindButtonsToVisible(bool isShow)
{
	if (isShow)
	{
		if (!m_IsShowFindButtons)
		{
			m_IsShowFindButtons = isShow;
		}
	}
	else
	{
		if (m_IsShowFindButtons)
		{
			m_IsShowFindButtons = isShow;
			m_SearchTextChanged = true;
		}
	}

	if (ReadPageZeroResults->Visibility == Windows::UI::Xaml::Visibility::Visible)
		ReadPageZeroResults->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
}

void FindTextControl::ShowReadPageZeroResults()
{
	if (ReadPageZeroResults->Visibility == Windows::UI::Xaml::Visibility::Collapsed)
	{
		ReadPageZeroResults->Visibility = Windows::UI::Xaml::Visibility::Visible;
	}
}

void FindTextControl::UpdateSliderValue(double value)
{
	if (value == 1.0)
	{
		if (SearchProgressRing->Visibility == ::Visibility::Collapsed)
			SearchProgressRing->Visibility = ::Visibility::Visible;
	}

	if (m_SearchTextResultControl)
	{
		m_SearchTextResultControl->UpdateProgressControl(value);
	}
}

void FindTextControl::HideSliderControl()
{
	if (SearchProgressRing->Visibility == ::Visibility::Visible)
	{
		SearchProgressRing->Visibility = ::Visibility::Collapsed;
	}

	if (m_SearchTextResultControl)
	{
		m_SearchTextResultControl->HideProgressControl();
	}
}

void FindTextControl::ShowProgressControl()
{
	m_SearchProgressBox->Show();
}

void FindTextControl::HideProgressControl()
{
	m_SearchProgressBox->Hide();
}

void FindTextControl::EnableResultsControl()
{
	PrevButton->IsEnabled = true;
	NextButton->IsEnabled = true;
}

void FindTextControl::DisableResultsControl()
{
	PrevButton->IsEnabled = false;
	NextButton->IsEnabled = false;
}

void FindTextControl::ShowSearchResultList()
{
	if (!m_IsResultListOpen)
	{
		if (m_FlyoutHosted == nullptr)
		{
			m_FlyoutHosted = ref new Windows::UI::Xaml::Controls::Flyout();
			Windows::UI::Xaml::Style^ _FlyoutPresenterStyle;
			_FlyoutPresenterStyle = Windows::UI::Xaml::Application::Current->Resources->HasKey(L"FlyoutPresenterStyle") ? dynamic_cast<Windows::UI::Xaml::Style^>(Windows::UI::Xaml::Application::Current->Resources->Lookup(L"FlyoutPresenterStyle")) : nullptr;
			m_FlyoutHosted->FlyoutPresenterStyle = _FlyoutPresenterStyle;
			m_SearchTextResultControl = ref new SearchTextResultControl();
			m_FlyoutHosted->Content = m_SearchTextResultControl;
			m_FlyoutHosted->Opened += ref new Windows::Foundation::EventHandler<Platform::Object ^>(this, &foxit::rdk::uiextensions::FindTextControl::OnSearchResultOpened);
			m_FlyoutHosted->Closed += ref new Windows::Foundation::EventHandler<Platform::Object ^>(this, &foxit::rdk::uiextensions::FindTextControl::OnSearchResultClosed);
		}

		Windows::UI::Xaml::Controls::Flyout::SetAttachedFlyout(SearchEditBox, m_FlyoutHosted);
		if (SearchEditBox->Parent)
			Windows::UI::Xaml::Controls::Flyout::ShowAttachedFlyout(SearchEditBox);
	}
}

void FindTextControl::CloseSearchResultList()
{
	if (m_FlyoutHosted)
	{
		m_FlyoutHosted->Hide();
	}
}

void FindTextControl::StartSearching(Platform::String^ str)
{
	ResetFindBarState();
	
	if (str->Length() > (unsigned int)SearchEditBox->MaxLength)
		SearchEditBox->Text = ref new String(str->Data(), SearchEditBox->MaxLength);
	else
		SearchEditBox->Text = ref new String(str->Data(), str->Length());

	NonVisibilityBtn->Focus(Windows::UI::Xaml::FocusState::Programmatic);

	m_SearchTextChanged = false;
	m_IsOutSideCaused = true;
	if (TextManager && UIExtMgr)
		TextManager->StartSearchString((int)UIExtMgr->GetCurrentPageIndex(), SearchEditBox->Text);
}


void FindTextControl::OnSearchResult(Platform::Object ^sender, ISearchResultEventArgs ^args)
{
	auto _State = args->State;
	auto _Type = args->Type;
	switch (_State)
	{
	case foxit::rdk::uiextensions::SearchState::SEARCHSTART:
		break;
	case foxit::rdk::uiextensions::SearchState::NOFOUND:
		{
			CloseSearchResultList();
			HideProgressControl();
			UpdateSearchPanelPosition();
			if (TextManager && UIExtMgr)
			{
				auto _SearchNoFindMsgControl = ref new SearchNoFindMsgControl();
				_SearchNoFindMsgControl->SetShowString(TextManager->FindTextString);
				m_MsgBox->MessageContent = _SearchNoFindMsgControl;
				m_MsgBox->Show();
				
				SearchEditBox->IsSearching = false;
			}
		}
		break;
	case foxit::rdk::uiextensions::SearchState::FOUND:
		{
			if (_Type == SearchTextType::SingleSearch)
			{
				HideProgressControl();
				ShowFindButtonsToVisible(true);
				UIExtensionsManager^ extMgr = getUIExtensionsMgr();
				PDFTextManager^ textMgr = extMgr->getTextMgr();
				if (textMgr)
				{
					auto _searchOnePage = ref new SingleSearchOnePage(args->OneData);
					_searchOnePage->CurrentIndexInPageData = args->CurrentIndex;
					textMgr->GotoExactlyPage(_searchOnePage);
					SearchEditBox->IsSearching = false;
					UIExtMgr->ShowSearchControl();
				}

				EnableResultsControl();
				UpdateSearchPanelPosition();
			}
			else if (_Type == SearchTextType::ContinousSearch)
			{
				ShowSearchResultList();

				if (args->CurrentIndex == -1)
				{
					UpdateSliderValue(args->OneData->PageIndex + 1);
				}
			}
		}
		break;
	case foxit::rdk::uiextensions::SearchState::FOUNDEND:
		if (m_IsResultListOpen)
		{
			HideSliderControl();
		}
		break;
	case foxit::rdk::uiextensions::SearchState::CLEAR:
		break;
	case foxit::rdk::uiextensions::SearchState::SEARCHSTOP:
		{
			if (_Type == SearchTextType::SingleSearch)
			{
				if (UIExtMgr)
					UIExtMgr->HideSearchControl();

				DisableResultsControl();
				
				if (m_MsgBox)
					m_MsgBox->Hide();
				SearchEditBox->IsSearching = false;
				UpdateSearchPanelPosition();
			}
		}
		break;
	default:
		break;
	}
}

void foxit::rdk::uiextensions::FindTextControl::OnVisibleBoundsChanged(Windows::UI::ViewManagement::ApplicationView ^sender, Platform::Object ^args)
{
	
}

Platform::String^ FindTextControl::TrimSpace(Platform::String^ str)
{
	Platform::String^ _resultstr = ref new Platform::String(L"");
	if (str == nullptr && str->Length() == 0)
		return _resultstr;
	
	wchar_t* pstart = (wchar_t*)str->Data();
	while (*pstart == ' ')
		pstart++;

	wchar_t* p = (wchar_t*)str->Data() + str->Length();
	if (p == pstart)
		return _resultstr;
	p--;
	while (*p == ' ' && p != pstart)
		p--;

	auto length = p - pstart + 1;
	_resultstr = ref new Platform::String(pstart, (unsigned int)length);
	return _resultstr;
}


void foxit::rdk::uiextensions::FindTextControl::OnQuerySubmitted(Platform::Object ^sender, foxit::rdk::uiextensions::SearchTextBoxQuerySubmittedEventArgs ^args)
{
	auto _controlView = foxit::rdk::uiextensions::GetVisualParent<foxit::rdk::PDFViewCtrl^>(this);
	auto _FindText = TrimSpace(SearchEditBox->Text);
	if (_FindText->Length() > 0)
	{
		TextManager->StartSearchString((int)_controlView->GetCurrentPage(), _FindText);

		Windows::UI::ViewManagement::InputPane::GetForCurrentView()->TryHide();
	}
}


void foxit::rdk::uiextensions::FindTextControl::OnSizeChanged(Platform::Object ^sender, Windows::UI::Xaml::SizeChangedEventArgs ^e)
{
	UpdateView();
}

void FindTextControl::UpdateSearchPanelPosition()
{
	auto _frameworkElement = foxit::rdk::uiextensions::GetVisualChildByName<Windows::UI::Xaml::Controls::Panel^>(Windows::UI::Xaml::Window::Current->Content, L"SearchTextPanel");
	if (_frameworkElement == nullptr || _frameworkElement->Visibility == Windows::UI::Xaml::Visibility::Collapsed)
		return;

	if (m_bIsShowInputPane)
	{
		auto _inputPane = Windows::UI::ViewManagement::InputPane::GetForCurrentView();
		auto _inputPaneRect = _inputPane->OccludedRect;

		if (m_TranslateTransform)
		{
			_frameworkElement->RenderTransform = ref new Windows::UI::Xaml::Media::TranslateTransform();
			m_TranslateTransform = nullptr;
		}
		Windows::Foundation::Point _screenPt = _frameworkElement->TransformToVisual(nullptr)->TransformPoint(Point(0, 0));
		auto _docBoxRect = Windows::Foundation::Rect(_screenPt, Windows::Foundation::Size((float)_frameworkElement->ActualWidth, (float)_frameworkElement->ActualHeight));

#if (_DEBUG && 1)
		std::wstringstream ws;
		ws << L"OccludedRect: " << _inputPaneRect.Left << L", " << _inputPaneRect.Top << L", " << _inputPaneRect.Width << L", " << _inputPaneRect.Height << L"\n";
		ws << L"DocPasswordBox Point: " << _screenPt.X << L", " << _screenPt.Y << L", " << _frameworkElement->ActualWidth << L", " << _frameworkElement->ActualHeight;
		ws << L"\n";
		OutputDebugString(ws.str().c_str());

#endif

		if (_inputPaneRect.IntersectsWith(_docBoxRect))
		{
			OutputDebugString(L"IntersectsWith...\n");

			m_TranslateTransform = ref new Windows::UI::Xaml::Media::TranslateTransform();
			m_TranslateTransform->Y -= _screenPt.Y + _frameworkElement->ActualHeight - _inputPaneRect.Top;
			_frameworkElement->RenderTransform = m_TranslateTransform;
		}
	}
	else
	{
		if (m_TranslateTransform)
		{
			_frameworkElement->RenderTransform = ref new Windows::UI::Xaml::Media::TranslateTransform();
			m_TranslateTransform = nullptr;
		}
	}
}

void foxit::rdk::uiextensions::FindTextControl::OnInputPaneShowing(Windows::UI::ViewManagement::InputPane ^sender, Windows::UI::ViewManagement::InputPaneVisibilityEventArgs ^args)
{
	args->EnsuredFocusedElementInView = true;
	m_bIsShowInputPane = true;

	UpdateSearchPanelPosition();
}


void foxit::rdk::uiextensions::FindTextControl::OnInputPaneHiding(Windows::UI::ViewManagement::InputPane ^sender, Windows::UI::ViewManagement::InputPaneVisibilityEventArgs ^args)
{
	args->EnsuredFocusedElementInView = false;
	m_bIsShowInputPane = false;

	UpdateSearchPanelPosition();
}

void foxit::rdk::uiextensions::FindTextControl::CancleSearch()
{
	bool bIsChange = false;

	if (UIExtMgr && TextManager)
	{
		auto _textinfo = TextManager->CurSearchTextInfo;
		TextManager->StopSearchString();
		bIsChange = _textinfo != TextManager->CurSearchTextInfo;
		
		if (bIsChange)
			UIExtMgr->viewControl->Update();
	}
}

void FindTextControl::FocusSearchControl()
{
	this->UpdateLayout();

	if (this->Parent)
	{
		if (SearchEditBox->FocusState == Windows::UI::Xaml::FocusState::Unfocused)
			SearchEditBox->Focus(Windows::UI::Xaml::FocusState::Programmatic);
	}
}

void foxit::rdk::uiextensions::FindTextControl::OnSearchResultOpened(Platform::Object ^sender, Platform::Object ^args)
{
	m_IsResultListOpen = true;

	auto _VisibleBounds = Windows::UI::ViewManagement::ApplicationView::GetForCurrentView()->VisibleBounds;
	auto _Bounds = Windows::UI::Xaml::Window::Current->Bounds;
	auto _FlyoutPresenter = GetVisualParent<Windows::UI::Xaml::Controls::FlyoutPresenter^>(m_FlyoutHosted->Content);
	if (_FlyoutPresenter)
	{
		std::wstringstream ws;
		ws << L"MinHeight: " << _FlyoutPresenter->MinHeight;
		ws << L", MaxHeight: " << _FlyoutPresenter->MaxHeight;
		ws << L", ActualWidth: " << _FlyoutPresenter->ActualWidth;
		ws << L", ActualHeight: " << _FlyoutPresenter->ActualHeight;
		ws << L", VisibleBounds.Width: " << _VisibleBounds.Width;
		ws << L", VisibleBounds.Height: " << _VisibleBounds.Height;
		ws << L", _Bounds.Width: " << _Bounds.Width;
		ws << L", _Bounds.Height: " << _Bounds.Height;
		ws << L"\n";
		OutputDebugString(ws.str().c_str());

		auto _height = _VisibleBounds.Height;
		if (_FlyoutPresenter->ActualWidth > _VisibleBounds.Width)
		{
			_FlyoutPresenter->MinWidth = _VisibleBounds.Width;
			_FlyoutPresenter->MaxWidth = _VisibleBounds.Width;
			_FlyoutPresenter->Width = _VisibleBounds.Width;
		}

		Windows::UI::Xaml::Controls::Primitives::Popup^ _Popup = dynamic_cast<Windows::UI::Xaml::Controls::Primitives::Popup^>(_FlyoutPresenter->Parent);
		if (_Popup)
		{
			_Popup->IsLightDismissEnabled = false;
			double _voffset = _Popup->VerticalOffset;
			if (Windows::System::Profile::AnalyticsInfo::VersionInfo->DeviceFamily == L"Windows.Mobile")
			{
				auto _StatusBar = Windows::UI::ViewManagement::StatusBar::GetForCurrentView();
				_voffset = _StatusBar->OccludedRect.Height;
			}
			else if (Windows::System::Profile::AnalyticsInfo::VersionInfo->DeviceFamily == L"Windows.Desktop")
			{
				auto _screenPt = _FlyoutPresenter->TransformToVisual(nullptr)->TransformPoint(Windows::Foundation::Point(0, 0));
				Windows::UI::Xaml::Media::TranslateTransform^ _transform = dynamic_cast<Windows::UI::Xaml::Media::TranslateTransform^>(SearchEditBox->RenderTransform);
				if (_transform == nullptr)
				{
					_transform = ref new Windows::UI::Xaml::Media::TranslateTransform();
				}

				if (_transform->Y != _screenPt.Y)
					_transform->Y = _screenPt.Y;

				_voffset = _screenPt.Y;
			}

			_Popup->VerticalOffset = _voffset;
		}
	}

	m_FlyoutWindowSizeChangedEventToken = Windows::UI::ViewManagement::ApplicationView::GetForCurrentView()->VisibleBoundsChanged += ref new Windows::Foundation::TypedEventHandler<Windows::UI::ViewManagement::ApplicationView^, Platform::Object^>([this](Windows::UI::ViewManagement::ApplicationView^, Platform::Object^)
	{
		auto _VisibleBounds = Windows::UI::ViewManagement::ApplicationView::GetForCurrentView()->VisibleBounds;
		auto _Bounds = Windows::UI::Xaml::Window::Current->Bounds;
		auto _FlyoutPresenter = GetVisualParent<Windows::UI::Xaml::Controls::FlyoutPresenter^>(m_FlyoutHosted->Content);
		if (_FlyoutPresenter)
		{
			std::wstringstream ws;
			ws << L"Window::Current->SizeChanged--MinHeight: " << _FlyoutPresenter->MinHeight;
			ws << L", MaxHeight: " << _FlyoutPresenter->MaxHeight;
			ws << L", ActualWidth: " << _FlyoutPresenter->ActualWidth;
			ws << L", ActualHeight: " << _FlyoutPresenter->ActualHeight;
			ws << L", VisibleBounds.Width: " << _VisibleBounds.Width;
			ws << L", VisibleBounds.Height: " << _VisibleBounds.Height;
			ws << L", _Bounds.Width: " << _Bounds.Width;
			ws << L", _Bounds.Height: " << _Bounds.Height;
			ws << L"\n";
			OutputDebugString(ws.str().c_str());

			auto _height = _VisibleBounds.Height;
			_FlyoutPresenter->MinHeight = _height;
			_FlyoutPresenter->MaxHeight = _height;
			_FlyoutPresenter->Height = _height;
			if (_FlyoutPresenter->ActualWidth > _VisibleBounds.Width)
			{
				_FlyoutPresenter->MinWidth = _VisibleBounds.Width;
				_FlyoutPresenter->MaxWidth = _VisibleBounds.Width;
				_FlyoutPresenter->Width = _VisibleBounds.Width;
			}

			Windows::UI::Xaml::Controls::Primitives::Popup^ _Popup = dynamic_cast<Windows::UI::Xaml::Controls::Primitives::Popup^>(_FlyoutPresenter->Parent);
			if (_Popup)
			{
				_Popup->IsLightDismissEnabled = false;
				double _voffset = _Popup->VerticalOffset;
				if (Windows::System::Profile::AnalyticsInfo::VersionInfo->DeviceFamily == L"Windows.Mobile")
				{
					auto _StatusBar = Windows::UI::ViewManagement::StatusBar::GetForCurrentView();
					_voffset = _StatusBar->OccludedRect.Height;
				}
				else if (Windows::System::Profile::AnalyticsInfo::VersionInfo->DeviceFamily == L"Windows.Desktop")
				{
					auto _screenPt = _FlyoutPresenter->TransformToVisual(nullptr)->TransformPoint(Windows::Foundation::Point(0, 0));
					Windows::UI::Xaml::Media::TranslateTransform^ _transform = dynamic_cast<Windows::UI::Xaml::Media::TranslateTransform^>(SearchEditBox->RenderTransform);
					if (_transform == nullptr)
					{
						_transform = ref new Windows::UI::Xaml::Media::TranslateTransform();
						SearchEditBox->RenderTransform = _transform;
					}

					if (_transform->Y != _screenPt.Y)
						_transform->Y = _screenPt.Y;

					_voffset = 0;
				}

				_Popup->VerticalOffset = _voffset;
				_Popup->HorizontalOffset = (_Bounds.Width - _FlyoutPresenter->ActualWidth > 0) ? _Bounds.Width - _FlyoutPresenter->ActualWidth : 0;
			}
		}
	});
}


void foxit::rdk::uiextensions::FindTextControl::OnSearchResultClosed(Platform::Object ^sender, Platform::Object ^args)
{
	m_IsResultListOpen = false;

	if (m_FlyoutWindowSizeChangedEventToken.Value)
	{
		Windows::UI::ViewManagement::ApplicationView::GetForCurrentView()->VisibleBoundsChanged -= m_FlyoutWindowSizeChangedEventToken;
		m_FlyoutWindowSizeChangedEventToken.Value = 0;
	}
}

UIExtensionsManager^ FindTextControl::getUIExtensionsMgr()
{
	auto _controlView = foxit::rdk::uiextensions::GetVisualParent<foxit::rdk::PDFViewCtrl^>(this);
	return dynamic_cast<foxit::rdk::uiextensions::UIExtensionsManager^>(_controlView->ExtensionsManager);
}
