﻿/**
 * Copyright (C) 2003-2018, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit Mobile PDF SDK to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */


#pragma once

#include "Text/FindTextControl.g.h"

namespace foxit
{
	namespace rdk
	{
		namespace uiextensions
		{
			ref class SearchTextResultControl;
			[Windows::Foundation::Metadata::WebHostHidden]
			public ref class FindTextControl sealed
			{
			public:
				FindTextControl();

				property IPDFTextManager^ TextManager
				{
					IPDFTextManager^ get()
					{
						return m_TextManagerWeakRef.Resolve<IPDFTextManager>();
					}
				}

				property UIExtensionsManager^ UIExtMgr
				{
					UIExtensionsManager^ get()
					{
						return m_UIViewWeakRef.Resolve<UIExtensionsManager>();
					}
				}

			internal:
				void ShowProgressControl();
				void HideProgressControl();
				void UpdateView();
				MessageDlg^ m_MsgBox;
				void CancleSearch();
				void FocusSearchControl();
				void EnableResultsControl();
				void DisableResultsControl();
				void ShowSearchResultList();
				void CloseSearchResultList();

			private:
				Windows::UI::Xaml::Controls::Panel^ m_ParentPanel;
				Windows::UI::Xaml::Controls::ContentControl^ m_ParentContentControl;

			private:
				void OnInitializeControl();
				void ResetFindBarState();
				void ShowFindButtonsToVisible(bool isShow);
				void ShowReadPageZeroResults();
				void UpdateSliderValue(double value);
				void HideSliderControl();
				void StartSearching(Platform::String^ str);
				Platform::String^ TrimSpace(Platform::String^ str);
				void UpdateSearchPanelPosition();
				UIExtensionsManager^ getUIExtensionsMgr();

			private:
				Platform::WeakReference m_TextManagerWeakRef;
				Platform::WeakReference m_PageViewWeakRef;
				Platform::WeakReference m_UIViewWeakRef;
				Windows::Foundation::EventRegistrationToken m_SearchResultEventToken, m_VisibleBoundsChangedEventToken, m_QuerySubmittedEventToken;

				Windows::Foundation::EventRegistrationToken __inputPaneShowingEventRegistrationToken, __inputPaneHidingEventRegistrationToken;
				bool m_bIsShowInputPane;
				Windows::UI::Xaml::Media::TranslateTransform^ m_TranslateTransform;

				/// Whether the text change variable
				bool m_SearchTextChanged;
				/// Whether the text change caused by the external
				bool m_IsOutSideCaused;
				/// Whether want to display the lookup state
				bool m_IsShowFindButtons;
				MessageDlg^ m_SearchProgressBox;
				bool m_IsResultListOpen;
				Windows::UI::Xaml::Controls::Flyout^ m_FlyoutHosted;
				Windows::Foundation::EventRegistrationToken m_FlyoutWindowSizeChangedEventToken;
				SearchTextResultControl^ m_SearchTextResultControl;

			private:
				void SearchEditBox_TextChanged(Platform::Object^ sender, Windows::UI::Xaml::Controls::TextChangedEventArgs^ e);
				void SearchEditBox_KeyUp(Platform::Object^ sender, Windows::UI::Xaml::Input::KeyRoutedEventArgs^ e);
				void SearchEditBox_LostFocus(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e);
				void SearchStartBtn_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e);
				void ReadPageFindPrev_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e);
				void ReadPageFindNext_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e);
				void FindCloseCommand_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e);
				void OnLoaded(Platform::Object ^sender, Windows::UI::Xaml::RoutedEventArgs ^e);
				void OnUnloaded(Platform::Object ^sender, Windows::UI::Xaml::RoutedEventArgs ^e);
				void OnSearchResult(Platform::Object ^sender, ISearchResultEventArgs ^args);
				void OnVisibleBoundsChanged(Windows::UI::ViewManagement::ApplicationView ^sender, Platform::Object ^args);
				void OnQuerySubmitted(Platform::Object ^sender, foxit::rdk::uiextensions::SearchTextBoxQuerySubmittedEventArgs ^args);
				void OnSizeChanged(Platform::Object ^sender, Windows::UI::Xaml::SizeChangedEventArgs ^e);
				void OnInputPaneShowing(Windows::UI::ViewManagement::InputPane ^sender, Windows::UI::ViewManagement::InputPaneVisibilityEventArgs ^args);
				void OnInputPaneHiding(Windows::UI::ViewManagement::InputPane ^sender, Windows::UI::ViewManagement::InputPaneVisibilityEventArgs ^args);
				void OnSearchResultOpened(Platform::Object ^sender, Platform::Object ^args);
				void OnSearchResultClosed(Platform::Object ^sender, Platform::Object ^args);
			};
		}
	}
}
