/**
 * Copyright (C) 2003-2018, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit Mobile PDF SDK to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */


#pragma once

namespace foxit
{
	namespace rdk
	{
		namespace uiextensions
		{
			interface class ITextSearchDataItem;
			interface class ISearchOnePage;

			/// Search status
			public enum class SearchState
			{
				/// Start finding
				SEARCHSTART,
				/// Not found
				NOFOUND,
				/// Find the result
				FOUND,
				/// Find the end
				FOUNDEND,
				/// The search results were cleared
				CLEAR,
				/// Stop searching
				SEARCHSTOP
			};
			public enum class SearchTextType
			{
				SingleSearch,
				ContinousSearch
			};

			public interface class ISearchResultEventArgs
			{
				property SearchTextType Type
				{
					SearchTextType get();
				}
				property SearchState State
				{
					SearchState get();
				}
				property int CurrentIndex
				{
					int get();
				}
				property ISearchOnePage^ OneData
				{
					ISearchOnePage^ get();
				}
				property Windows::Foundation::Collections::IMapView<int, ISearchOnePage^>^ AllData
				{
					Windows::Foundation::Collections::IMapView<int, ISearchOnePage^>^ get();
				}
			};

			public interface class IPageSelectText
			{
				property int PageIndex
				{
					int get();
				}
				property Windows::Foundation::Collections::IVector<Windows::Foundation::Rect>^ TextRectVect
				{
					Windows::Foundation::Collections::IVector<Windows::Foundation::Rect>^ get();
				}
				property Windows::Foundation::Collections::IVector<int>^ TextRotateVect
				{
					Windows::Foundation::Collections::IVector<int>^ get();
				}
				property int CharStart
				{
					int get();
				}
				property int CharEnd
				{
					int get();
				}
				property int TextRotate
				{
					int get();
					void set(int value);
				}
			};

			public ref class SingleSearchOnePage sealed
			{
			public:
				SingleSearchOnePage()
				{
					_CurrentIndexInPageData = 0;
					_OnePageData = nullptr;
				}
				SingleSearchOnePage(ISearchOnePage^ onepage)
				{
					_CurrentIndexInPageData = 0;
					_OnePageData = onepage;
				}

				property int PageIndex
				{
					int get()
					{
						return _PageIndex;
					}
				}
				property ISearchOnePage^ OnePageData
				{
					ISearchOnePage^ get()
					{
						return _OnePageData;
					}
					void set(ISearchOnePage^ value)
					{
						_OnePageData = value;
					}
				}
				property int CurrentIndexInPageData
				{
					int get()
					{
						return _CurrentIndexInPageData;
					}
					void set(int value)
					{
						_CurrentIndexInPageData = value;
					}
				}

			private:
				int _PageIndex;
				int _CurrentIndexInPageData;
				ISearchOnePage^ _OnePageData;
			};

			public enum class TextSelectChangedType
			{
				Add, Clear
			};

			public interface class ITextSelectChangedEventArgs
			{
				property TextSelectChangedType Type
				{
					TextSelectChangedType get();
				}
				property Windows::Foundation::Collections::IObservableVector<IPageSelectText^>^ SelectedText
				{
					Windows::Foundation::Collections::IObservableVector<IPageSelectText^>^ get();
				}
			};

			public enum class TextSearchFlag
			{
				/** @brief	Whether is it a matching case. */
				Matchcase = 1,
				/** @brief	Whether is it a matching whole word. */
				MatchWholeword = 2,
				/** @brief	Whether matching consecutively (for example, "CC" will match twice in "CCC").*/
				Consecutive = 4,
			};

			public enum class TextSearchDirection
			{
				Prev,
				Next,
			};

			public enum class TextSelectType
			{
				Paragraph,
				Rectbox,
			};

			/// Setting interface for text selection
			public interface class ITextSelectOption
			{
				/// Point selection expansion range
				property double Tolerance;
				property TextSelectType Mode;
				int GetSearchTextMaxLength();
			};

			/// Setting interface for text search
			public interface class ITextSearchOption
			{
				property TextSearchFlag SearchFlag;
			};

			public delegate void SearchResultEventHandler(Platform::Object^ sender, ISearchResultEventArgs^ args);
			public delegate void TextSelectChangedEventHandler(Platform::Object^ sender, ITextSelectChangedEventArgs^ args);

			interface class IPdfDocument;
			ref class foxit::rdk::Render_ScreenOffsetInfo;
			ref class foxit::rdk::PDFViewCanvas;
			[Windows::Foundation::Metadata::WebHostHidden]
			public interface class IPDFTextManager
			{
				property Platform::String^ FindTextString
				{
					Platform::String^ get();
				}
				bool GetIsSingleSearching();
				bool GetIsFoundText();
				void StartSearchString(int pageindex, Platform::String^ pwsFindwhat);
				void StopSearchString();
				void SearchPrev();
				void SearchNext();
				void ClearTextSelect();
				void NotifyByTextSearchDataItem(ITextSearchDataItem^ item);
				property SingleSearchOnePage^ CurSearchTextInfo
				{
					SingleSearchOnePage^ get();
				}
				property Windows::Foundation::Collections::IObservableVector<ITextSearchDataItem^>^ SearchData
				{
					Windows::Foundation::Collections::IObservableVector<ITextSearchDataItem^>^ get();
				}
				event SearchResultEventHandler^ SearchResult;

				bool IsInitText(int pageindex);
				Platform::String^ GetTextByPosition(int pageindex, int start, int count);
				Platform::String^ GetTextByBounded(int pageindex, Windows::Foundation::Rect rect);
				Platform::String^ GetTextByBoundedSegments(int pageindex, Windows::Foundation::Rect rect);
				Platform::String^ GetWordsByPoint(int pageindex, Windows::Foundation::Point point);
				bool IsHitText(int pageindex, Windows::Foundation::Point pt);
				bool IsHitSelectedText(int pageindex, Windows::Foundation::Point pt);
				bool AddSelectText(int startindex, Windows::Foundation::Point startpt, int endindex, Windows::Foundation::Point endpt);

				property Platform::String^ SelectedTextString
				{
					Platform::String^ get();
				}
				property Windows::Foundation::Collections::IObservableVector<IPageSelectText^>^ SelectedText
				{
					Windows::Foundation::Collections::IObservableVector<IPageSelectText^>^ get();
				}
				property double TextSelectTolerance;
				event TextSelectChangedEventHandler^ TextSelectChanged;

				bool OnCreate(foxit::rdk::PDFViewCtrl^ pageview);
				bool OnLoaded();
				void OnDelete();
				bool LoadPdfTextResource(int pageindex);

				property foxit::rdk::PDFViewCtrl^ PDFViewCtrl
				{
					foxit::rdk::PDFViewCtrl^ get();
				}

				Platform::Array<Windows::Foundation::Rect>^ GetPageCoordinateRects(int pageindex);

				/// setting options
				void SetOption(ITextSelectOption^ opt);

				foxit::pdf::PDFPage^ GetPageHandle(int pageindex);

				bool IsResourcesReleased();
				void ReleaseExtraResources();
				void ForceReleaseResources();

				bool GetIsShowSelHandleBtn();
				void SetIsShowSelHandleBtn(bool bIsShow);
				void OnRender(foxit::rdk::PDFViewCtrl^ sender, foxit::rdk::RenderCanvasEventArgs^ e);
				void OnPointerPressed(Platform::Object^ sender, Windows::UI::Xaml::Input::PointerRoutedEventArgs^ e);
				void OnPointerReleased(Platform::Object^ sender, Windows::UI::Xaml::Input::PointerRoutedEventArgs^ e);
				void OnPointerMoved(Platform::Object^ sender, Windows::UI::Xaml::Input::PointerRoutedEventArgs^ e);
				void OnPointerEntered(Platform::Object^ sender, Windows::UI::Xaml::Input::PointerRoutedEventArgs^ e);
				void OnPointerExited(Platform::Object^ sender, Windows::UI::Xaml::Input::PointerRoutedEventArgs^ e);
				void OnTapped(Platform::Object^ sender, Windows::UI::Xaml::Input::TappedRoutedEventArgs^ e);
				void OnHolding(Platform::Object^ sender, Windows::UI::Xaml::Input::HoldingRoutedEventArgs^ e);
				void OnSelHandleClick(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e);

				Windows::Foundation::Rect GetHitTextRect(int pageindex, Windows::Foundation::Point pt);
				IPageSelectText^ GetTextRect(int startindex, Windows::Foundation::Point startpt);

				/// Page editor related call interface
				[Windows::Foundation::Metadata::DefaultOverload]
				void ChangePageIndex(int srcIndex, int dstIndex);
				void ChangePageIndex(Windows::Foundation::Collections::IVector<int>^ srcIndexs, int dstIndex);
				void PageDeleteBefore(Windows::Foundation::Collections::IVector<int>^ pageIndexs);
				void PageDeleteAfter(Windows::Foundation::Collections::IVector<int>^ pageIndexs);
				void CheckPageIndex(int startIndex);
			};
		}
	}
}
