/**
 * Copyright (C) 2003-2018, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit Mobile PDF SDK to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */


#include "pch.h"
#include "PDFTextManager.h"
#include "../Common/VisualTreeHelperEx.h"
#include "../Common/Critsec.h"

#include <map>
#include <regex>
#include <collection.h>

using namespace foxit::rdk::uiextensions;
using namespace foxit::rdk;

using namespace Platform;
using namespace Concurrency;
using namespace Windows::Foundation;
using namespace Windows::UI::Xaml;
using namespace Windows::UI::Xaml::Controls;
using namespace Windows::UI::Xaml::Controls::Primitives;
using namespace Windows::UI::Xaml::Media;
using namespace Windows::ApplicationModel::DataTransfer;

#define DELIMITER '\n'
#define MAXTEXTSELECTTOLERANCE	10
#define MAXSEARCHTEXTLENGTH	128
#define SEARCHPROMPTCHARACTERCOUNTS (45)

namespace foxit
{
	namespace rdk
	{
		namespace uiextensions
		{
			ref class SearchResultEventArgs sealed : public ISearchResultEventArgs
			{
			public:
				SearchResultEventArgs(SearchState state, ISearchOnePage^ onedata, int curindex, SearchTextType type = SearchTextType::SingleSearch, Windows::Foundation::Collections::IMapView<int, ISearchOnePage^>^ allmap = nullptr)
					: _State(state), _Data(onedata), _CurrentIndex(curindex), _AllData(allmap), _Type(type)
				{

				}

				property SearchTextType Type
				{
					virtual SearchTextType get()
					{
						return _Type;
					}
				}

				property SearchState State
				{
					virtual SearchState get()
					{
						return _State;
					}
				}
				property int CurrentIndex
				{
					virtual int get()
					{
						return _CurrentIndex;
					}
				}
				property ISearchOnePage^ OneData
				{
					virtual ISearchOnePage^ get()
					{
						return _Data;
					}
				}

				property Windows::Foundation::Collections::IMapView<int, ISearchOnePage^>^ AllData
				{
					virtual Windows::Foundation::Collections::IMapView<int, ISearchOnePage^>^ get()
					{
						return _AllData;
					}
				}

			private:
				int _CurrentIndex;
				SearchTextType _Type;
				ISearchOnePage^ _Data;
				SearchState _State;
				Windows::Foundation::Collections::IMapView<int, ISearchOnePage^>^  _AllData;
			};

			ref class TextSelectChangedEventArgs sealed : public ITextSelectChangedEventArgs
			{
			public:
				TextSelectChangedEventArgs(TextSelectChangedType type, Windows::Foundation::Collections::IObservableVector<IPageSelectText^>^ text)
					: _Type(type)
				{
					_SelectedText = ref new Platform::Collections::Vector<IPageSelectText^>();
					Platform::Array<IPageSelectText^>^ arry = ref new Platform::Array<IPageSelectText^>(text->Size);
					text->GetMany(0, arry);
					_SelectedText->ReplaceAll(arry);
				}

				property TextSelectChangedType Type
				{
					virtual TextSelectChangedType get()
					{
						return _Type;
					}
				}

				property Windows::Foundation::Collections::IObservableVector<IPageSelectText^>^ SelectedText
				{
					virtual Windows::Foundation::Collections::IObservableVector<IPageSelectText^>^ get()
					{
						return _SelectedText;
					}
				}

			private:
				TextSelectChangedType _Type;
				Platform::Collections::Vector<IPageSelectText^>^ _SelectedText;
			};

			ref class PageSelectText sealed : public IPageSelectText
			{
				friend ref class PDFTextManager;
			public:
				PageSelectText(int index) : _PageIndex(index)
				{
					_TextRectVect = ref new Platform::Collections::Vector<Windows::Foundation::Rect>();
					_TextRotateVect = ref new Platform::Collections::Vector<int>();
				}

				property int PageIndex
				{
					virtual int get()
					{
						return _PageIndex;
					}
				}
				property Windows::Foundation::Collections::IVector<Windows::Foundation::Rect>^ TextRectVect
				{
					virtual Windows::Foundation::Collections::IVector<Windows::Foundation::Rect>^ get()
					{
						return _TextRectVect;
					}
				}
				property Windows::Foundation::Collections::IVector<int>^ TextRotateVect
				{
					virtual Windows::Foundation::Collections::IVector<int>^ get()
					{
						return _TextRotateVect;
					}
				}
				property int CharStart
				{
					virtual int get()
					{
						return _CharStart;
					}
				}
				property int CharEnd
				{
					virtual int get()
					{
						return _CharEnd;
					}
				}

				property int TextRotate
				{
					virtual int get()
					{
						return _rotate;
					}
					virtual void set(int value)
					{
						_rotate = value;
					}
				}

			private:
				int _PageIndex;
				int _CharStart;
				int _CharEnd;
				int _rotate;
				Platform::Collections::Vector<Windows::Foundation::Rect>^ _TextRectVect;
				Platform::Collections::Vector<int>^ _TextRotateVect;
			};

			ref class DefaultTextSelectOption sealed : ITextSelectOption
			{
			public:
				DefaultTextSelectOption()
				{
					_Mode = TextSelectType::Paragraph;
					_Tolerance = MAXTEXTSELECTTOLERANCE;
				}

				property TextSelectType Mode
				{
					virtual TextSelectType get()
					{
						return _Mode;
					}
					virtual void set(TextSelectType value)
					{
						if (_Mode != value)
						{
							_Mode = value;
						}
					}
				}

				property double Tolerance
				{
					virtual double get()
					{
						return _Tolerance;
					}
					virtual void set(double value)
					{
						if ((value > 0 || value <= MAXTEXTSELECTTOLERANCE) && value != _Tolerance)
						{
							_Tolerance = value;
						}
					}
				}

				virtual int GetSearchTextMaxLength()
				{
					return MAXSEARCHTEXTLENGTH;
				}

			private:
				TextSelectType _Mode;
				double _Tolerance;
			};

			ref class DefaultTextSearchOption sealed : ITextSearchOption
			{
			public:
				DefaultTextSearchOption()
				{
					_SearchFlag = TextSearchFlag::MatchWholeword;
				}

				property TextSearchFlag SearchFlag
				{
					virtual TextSearchFlag get()
					{
						return _SearchFlag;
					}
					virtual void set(TextSearchFlag value)
					{
						if (value != _SearchFlag)
						{
							_SearchFlag = value;
						}
					}
				}

			private:
				TextSearchFlag _SearchFlag;
			};
		}
	}
}

TextHandleContext::TextHandleContext(int pageindex, foxit::pdf::PDFDoc^ doc) : m_PageIndex(pageindex), m_DocumentWeakRef(doc)
{
	IsInit = false;
	IsInitTextRect = false;
	IsPageHandleInitByExternalModule = false;
}

bool TextHandleContext::LoadTextPageHandle()
{
	if (IsInit)
		return true;

	auto _Doc = m_DocumentWeakRef.Resolve<foxit::pdf::PDFDoc>();
	if (_Doc == nullptr)
		return false;

	auto _PdfPage = _Doc->GetPage(m_PageIndex);
	if (_PdfPage == nullptr)
		return false;

	pageHandle = _PdfPage;
	if (pageHandle == nullptr)
		return false;

	if (!pageHandle->IsParsed())
	{
		foxit::common::Progressive^ progress = pageHandle->StartParse((unsigned int)foxit::pdf::ParseFlags::e_ParsePageNormal, nullptr, false);
		if (progress != nullptr)
		{
			foxit::common::State state = progress->Continue();
			while (state == foxit::common::State::e_ToBeContinued)
				state = progress->Continue();
		}
	}

	IsInit = true;

	return true;
}

void TextHandleContext::LoadTextPageRects()
{
	if (!LoadTextPageHandle() || IsInitTextRect)
		return;

	foxit::pdf::TextPage^ textSelection = ref new foxit::pdf::TextPage(pageHandle, 0);
	auto charCount = textSelection->GetCharCount();
	if (charCount <= 0)
		return;

	int __RcCount = textSelection->GetTextRectCount(0, charCount);
	if (__RcCount > 0)
	{
		foxit::common::fxcrt::RectF^ __rc;
		auto textarr = ref new Platform::Array<Windows::Foundation::Rect>(__RcCount);
		for (int i = 0; i < __RcCount; i++)
		{
			Rect outrc = FoxitRectConverter::ConvertFromFoxitRectF(textSelection->GetTextRect(i));
			textarr->set(i, outrc);
		}
		TextRect = textarr;
	}

	IsInitTextRect = true;
}

bool TextHandleContext::IsValid()
{
	return m_TextPageRefCount > 0;
}

void TextHandleContext::AddRef()
{
	m_TextPageRefCount++;
}

void TextHandleContext::RemoveRef()
{
	m_TextPageRefCount--;
}

void TextHandleContext::ReleaseTextHandle()
{
	if (textpageHandle)
	{
		auto _Doc = m_DocumentWeakRef.Resolve<foxit::pdf::PDFDoc>();
		if (_Doc == nullptr)
			return;
		auto _PdfPage = _Doc->GetPage(m_PageIndex);
		if (_PdfPage == nullptr)
			return;

		textpageHandle = nullptr;
		IsInit = false;
	}
}

void TextHandleContext::ReleaseTextHandleContext(TextHandleContext* val)
{
	if (val && val->IsInit)
	{
		if (!val->IsPageHandleInitByExternalModule)
		{
			val->pageHandle = nullptr;
		}
		val->ReleaseTextHandle();

#ifdef _DEBUG
		std::wstringstream ws;
		ws << L"ReleaseTextHandleContext: " << val << L"\r\n";
		OutputDebugString(ws.str().c_str());
#endif
	}
}

int TextHandleContext::GetPageIndex()
{
	return m_PageIndex;
}

void TextHandleContext::CheckPageIndex()
{
	if (IsInit)
	{
		m_PageIndex = pageHandle->GetIndex();
	}
}

void TextSelectContext::ResetContext()
{
	SelectMode = 0;
	StartIndex = -1;
	EndIndex = -1;
	StartPoint = Windows::Foundation::Point(0, 0);
	EndPoint = Windows::Foundation::Point(0, 0);
}

PDFTextManager::PDFTextManager(foxit::rdk::PDFViewCtrl^  viewControl)
{
	m_PageViewWeakRef = viewControl;
	m_Critsec = std::make_shared<CritSec>();
	m_MapCritsec = std::make_shared<CritSec>();
	m_PageIndexVectorCritsec = std::make_shared<CritSec>();

	m_LockCount = 0;
	m_IsSearching = false;
	m_IsSingleSearching = false;
	m_SearchMode = 0;
	m_IsForceReleaseResources = false;
	m_IsHadReleasedResources = false;

	m_EventCancelFind = CreateEventEx(NULL, L"CancelFind_Event", 0, EVENT_ALL_ACCESS);
	_SearchData = ref new Platform::Collections::Vector<ITextSearchDataItem^>();
	_SelectedText = ref new Platform::Collections::Vector<IPageSelectText^>();
	_TextSelectTolerance = 5;
	SetOption(ref new DefaultTextSelectOption());
	m_pTextSelectOptContext = ref new TextSelectOptContext();
	m_pTextSelectContext = ref new TextSelectContext();
}

PDFTextManager::~PDFTextManager()
{
	while (m_LockCount)
	{
		m_LockCount = 0;
	}

	if (m_PointerOverTextTimer && m_PointerOverTextTimer->IsEnabled)
		m_PointerOverTextTimer->Stop();

	CloseHandle(m_EventCancelFind);
	if (m_PageHandleVect.size() > 0)
		m_PageHandleVect.clear();
	if (_SearchData->Size > 0)
		_SearchData->Clear();
	if (_SelectedText->Size > 0)
		_SelectedText->Clear();
	if (m_PageHandleVect.size() > 0)
		m_HasSearchedPage.clear();
	if (m_PageHandleVect.size() > 0)
		m_SearchOnePageData.clear();

#ifdef _DEBUG
	std::wstringstream ws;
	ws << L"[########################## PDFTextManager---Delete: " << this->GetHashCode() << L" ##########################]\r\n";
	OutputDebugString(ws.str().c_str());
#endif
}

bool PDFTextManager::OnCreate(foxit::rdk::PDFViewCtrl^ pageview)
{
	if (pageview == nullptr)
		return false;

	m_PageViewWeakRef =  pageview;
	m_WeakfpdfDocument = PDFViewCtrl->GetDoc();
	m_UIViewWeakRef = PDFViewCtrl->ExtensionsManager;

	m_IsPointerOverText = false;
	m_PointerOverTextTimer = ref new Windows::UI::Xaml::DispatcherTimer();
	Windows::Foundation::TimeSpan _HideTimerTimeSpan;
	_HideTimerTimeSpan.Duration = 10000000;
	m_PointerOverTextTimer->Interval = _HideTimerTimeSpan;
	m_PointerOverTextTimer->Tick += ref new Windows::Foundation::EventHandler<Platform::Object ^>(this, &foxit::rdk::uiextensions::PDFTextManager::OnPointerOverTick);

	m_textRenderToken = pageview->RenderCanvasEvent += ref new foxit::rdk::RenderCanvasEvent(
		this,
		&PDFTextManager::OnRender);

	return true;
}

foxit::rdk::PDFViewCtrl^ PDFTextManager::PDFViewCtrl::get()
{
	return m_PageViewWeakRef.Resolve<foxit::rdk::PDFViewCtrl>();
}

UIExtensionsManager^ PDFTextManager::UIExtMgr::get()
{
	return m_UIViewWeakRef.Resolve<UIExtensionsManager>();
}

bool PDFTextManager::OnLoaded()
{
	return true;
}

void PDFTextManager::OnDelete()
{
	PDFViewCtrl->RenderCanvasEvent -= m_textRenderToken;
	m_PageViewWeakRef = nullptr;
}

Windows::Foundation::IAsyncOperationWithProgress<int, SearchOnePage^>^ PDFTextManager::GetSearchResultAsync(Platform::String^ pwsFindwhat)
{
	return Concurrency::create_async([this, pwsFindwhat](progress_reporter<SearchOnePage^> reporter)
	{
		m_IsSearching = true;
		if ((!SearchCheckStringValid(pwsFindwhat)))
			throw ref new Platform::FailureException(L"pwsFindwhat");

		foxit::pdf::PDFDoc^ pDoc = nullptr;
		if (m_WeakfpdfDocument)
		{
			auto pdfdoc = m_WeakfpdfDocument.Resolve<foxit::pdf::PDFDoc>();
			if (pdfdoc)
				pDoc = pdfdoc;
		}

		if (!pDoc)
			return 0;

		return 1;
	});
}

void PDFTextManager::Lock()
{
	m_Critsec->Lock();
	m_LockCount++;
}

void PDFTextManager::Unlock()
{
	m_Critsec->Unlock();
	m_LockCount--;
}

bool PDFTextManager::LoadPageHandle(int pageIndex, foxit::pdf::PDFPage^ pPage, foxit::pdf::TextPage^ pTextPage)
{
	int totalpage = -1;
	foxit::pdf::PDFDoc^ pPDFData = nullptr;

	if (m_WeakfpdfDocument)
	{
		pPDFData = m_WeakfpdfDocument.Resolve<foxit::pdf::PDFDoc>();
		if (!pPDFData)
			return false;
		totalpage = pPDFData->GetPageCount();
	}

	if (pPDFData == nullptr || pageIndex < 0 || pageIndex >= totalpage)
		return false;

	std::shared_ptr<TextHandleContext> pagecontext = nullptr;
	if (!m_PageHandleVect.count(pageIndex))
	{
		pagecontext = std::shared_ptr<TextHandleContext>(new TextHandleContext(pageIndex, pPDFData), &TextHandleContext::ReleaseTextHandleContext);
		m_PageHandleVect.insert(std::make_pair(pageIndex, pagecontext));
	}
	else
	{
		pagecontext = m_PageHandleVect[pageIndex];
		if (!pagecontext)
		{
			pagecontext = std::shared_ptr<TextHandleContext>(new TextHandleContext(pageIndex, pPDFData), &TextHandleContext::ReleaseTextHandleContext);
			m_PageHandleVect[pageIndex] = pagecontext;
		}
	}

	pagecontext->LoadTextPageHandle();
	if (!pagecontext->IsInit)
		return false;
	if (pPage)
		pPage = pagecontext->pageHandle;
	if (pTextPage)
		pTextPage = pagecontext->textpageHandle;

	return true;
}

std::shared_ptr<TextHandleContext> PDFTextManager::LoadPageHandle_Sp(int pageIndex)
{
	int totalpage = -1;
	foxit::pdf::PDFDoc^ pPDFData = nullptr;

	if (m_WeakfpdfDocument)
	{
		pPDFData = m_WeakfpdfDocument.Resolve<foxit::pdf::PDFDoc>();
		if (!pPDFData)
			return nullptr;
		totalpage = pPDFData->GetPageCount();
	}

	if (pPDFData == nullptr || pageIndex < 0 || pageIndex >= totalpage)
		return nullptr;

	std::shared_ptr<TextHandleContext> pagecontext = nullptr;
	pagecontext = std::shared_ptr<TextHandleContext>(new TextHandleContext(pageIndex, pPDFData), &TextHandleContext::ReleaseTextHandleContext);

	Lock();
	do
	{
		pagecontext->IsInit = true;
	} while (0);
	Unlock();

	return pagecontext;
}

bool PDFTextManager::SearchLoadHandle(int pageIndex, foxit::pdf::PDFPage^ pPage, foxit::pdf::TextPage^ pTextPage, foxit::pdf::TextSearch^& pSearchHandle, Platform::String^ pwsFindwhat, int findOption, int findMode)
{
	auto pDoc = m_WeakfpdfDocument.Resolve<foxit::pdf::PDFDoc>();

	pSearchHandle = ref new foxit::pdf::TextSearch(pDoc, nullptr);	
	pSearchHandle->SetPattern(pwsFindwhat);
	pSearchHandle->SetStartPage(pageIndex);
	pSearchHandle->SetEndPage(pageIndex);
	pSearchHandle->SetSearchFlags((int)foxit::pdf::SearchFlags::e_SearchNormal);

	return true;
}

Platform::String^ PDFTextManager::GetShowTextFromPage(int page, int count, int mode)
{
	auto pDoc = m_WeakfpdfDocument.Resolve<foxit::pdf::PDFDoc>();
	foxit::pdf::PDFPage^ pPage = pDoc->GetPage(page);
	auto pTextSelect = ref new foxit::pdf::TextPage(pPage, (int)foxit::pdf::TextParseFlags::e_ParseTextNormal);
	auto nCharCount = pTextSelect->GetCharCount();
	int start = 0;

	if (mode == 0)
	{
		start = (nCharCount - count > 0) ? nCharCount - count : 0;
		count = (nCharCount - count > 0) ? count : nCharCount;
	}
	else if (mode == 1)
	{
		start = 0;
	}
	
	return pTextSelect->GetChars(start, count);
}

bool PDFTextManager::GetSearchItemData(int pageIndex, foxit::pdf::TextPage^ pTextPage, foxit::pdf::TextSearch^ pSeachHandle, TextSearchDataItem^ item, int findMode)
{
	bool bIsMatched = false;
	int iStartCharIndex = -1;
	int totalchar = 0;

	if (findMode == 0)
	{
		bIsMatched = pSeachHandle->FindNext();
	}
	else
	{
		bIsMatched = pSeachHandle->FindPrev();
	}

	if (!bIsMatched)
		return false;
	
	auto pRectArray = pSeachHandle->GetMatchRects();
	auto nRectCount = pRectArray->GetSize();
	for (int j = 0; j < nRectCount; j++)
	{
		foxit::common::fxcrt::RectF^ __rc = pRectArray->GetAt(j);
		Rect outrc = FoxitRectConverter::ConvertFromFoxitRectF(__rc);
		item->RectVect->Append(outrc);
	}

	auto nPageIndex = pSeachHandle->GetMatchPageIndex();
	iStartCharIndex = pSeachHandle->GetMatchStartCharIndex();
	item->StartCharIndex = iStartCharIndex;
	item->Query = pSeachHandle->GetMatchSentence();

	return true;
}

bool PDFTextManager::SearchCheckStringValid(Platform::String^ findstr)
{
	return findstr && findstr->Length() > 0;
}

bool PDFTextManager::GetIsSingleSearching()
{
	return m_IsSingleSearching;
}

bool PDFTextManager::GetIsFoundText()
{
	return m_IsFound;
}

void PDFTextManager::StartSearchString(int pageindex, Platform::String^ pwsFindwhat)
{
	if (!SearchCheckStringValid(pwsFindwhat))
		return;

	if (_SearchData->Size > 0)
		_SearchData->Clear();
	if (m_HasSearchedPage.size() > 0)
		m_HasSearchedPage.clear();
	if (m_SearchOnePageData.size() > 0)
	{
		auto __map = ref new Platform::Collections::Map<int, ISearchOnePage^>(m_SearchOnePageData);
		SearchResultEventArgs^ args = ref new SearchResultEventArgs(SearchState::CLEAR, nullptr, -1, SearchTextType::ContinousSearch, __map->GetView());
		SearchResult(this, args);
		m_SearchOnePageData.clear();
		OnClearTextSelect();
	}
	m_CurSearchOnePage = nullptr;

	if (m_IsSearching || m_IsSingleSearching)
		StopSearchString();

	m_CurSearchIndex = pageindex;
	m_SearchTextString = pwsFindwhat;
	m_IsFound = false;

	if (true)
	{
		std::wstringstream ws;
		ws << L"StartSearchString: " << pageindex << L", FindString: " << pwsFindwhat->Data();
		ws << L"...\n";
		OutputDebugString(ws.str().c_str());
	}

	SearchResultEventArgs^ args = ref new SearchResultEventArgs(SearchState::SEARCHSTART, nullptr, -1, SearchTextType::SingleSearch);
	SearchResult(this, args);

	SearchNext();
}

void PDFTextManager::StopSearchString()
{
	if (m_IsSingleSearching)
		m_SingleSearchCts.cancel();
	if (m_AsyncOperation && m_IsSearching)
		SetEvent(m_EventCancelFind);

	m_SearchTextString = L"";
	m_IsFound = false;

	SearchResultEventArgs^ args = ref new SearchResultEventArgs(SearchState::SEARCHSTOP, nullptr, -1, SearchTextType::SingleSearch);
	SearchResult(this, args);
	m_CurSearchOnePage = nullptr;

	if (_SearchData->Size > 0)
		_SearchData->Clear();
	if (m_HasSearchedPage.size() > 0)
		m_HasSearchedPage.clear();
	if (m_SearchOnePageData.size() > 0)
	{
		auto __map = ref new Platform::Collections::Map<int, ISearchOnePage^>(m_SearchOnePageData);
		SearchResultEventArgs^ args = ref new SearchResultEventArgs(SearchState::CLEAR, nullptr, -1, SearchTextType::ContinousSearch, __map->GetView());
		SearchResult(this, args);
		m_SearchOnePageData.clear();
		OnClearTextSelect();
	}
}

void PDFTextManager::SearchPrev()
{
	if (m_IsSingleSearching || m_SearchTextString->Length() == 0)
		return;

	m_SearchMode = 1;
	m_SingleSearchCts = Concurrency::cancellation_token_source();
	m_IsSingleSearching = true;

	SingleSearchString()
		.then([this](SingleSearchOnePage^ temp)
	{
		if (temp)
		{
			m_IsFound = true;
			SearchResultEventArgs^ args = ref new SearchResultEventArgs(SearchState::FOUND, temp->OnePageData, temp->CurrentIndexInPageData);
			SearchResult(this, args);
		}
		else
		{
			if (m_SingleSearchCts.get_token().is_canceled())
			{
				OutputDebugString(L"SingleSearchString Prev Cancel...\n");
			}
			else
			{
				SearchResultEventArgs^ args = ref new SearchResultEventArgs(SearchState::NOFOUND, nullptr, -1);
				SearchResult(this, args);
				OutputDebugString(L"No Found...\n");
			}
		}
		m_IsSingleSearching = false;
	}, m_SingleSearchCts.get_token(), task_continuation_context::use_current()).then([this](task<void> priorTask)
	{
		try
		{
			priorTask.get();
		}
		catch (const concurrency::task_canceled &)
		{
			OutputDebugString(L"SingleSearchString Prev Cancel...\r\n");
			m_IsSingleSearching = false;
			ReleaseExtraResources();
		};
	});
}

void PDFTextManager::SearchNext()
{
	if (m_IsSingleSearching || m_SearchTextString->Length() == 0)
		return;

	m_SearchMode = 0;
	m_SingleSearchCts = Concurrency::cancellation_token_source();
	m_IsSingleSearching = true;

	SingleSearchString()
		.then([this](SingleSearchOnePage^ temp)
	{
		if (temp)
		{
			m_IsFound = true;
			SearchResultEventArgs^ args = ref new SearchResultEventArgs(SearchState::FOUND, temp->OnePageData, temp->CurrentIndexInPageData);
			SearchResult(this, args);
		}
		else
		{
			if (m_SingleSearchCts.get_token().is_canceled())
			{
				OutputDebugString(L"SingleSearchString Next Cancel...\n");
			}
			else
			{
				SearchResultEventArgs^ args = ref new SearchResultEventArgs(SearchState::NOFOUND, nullptr, -1);
				SearchResult(this, args);
				OutputDebugString(L"No Found...\n");
			}
		}
		m_IsSingleSearching = false;
	}, m_SingleSearchCts.get_token(), task_continuation_context::use_current()).then([this](task<void> priorTask)
	{
		try
		{
			priorTask.get();
		}
		catch (const concurrency::task_canceled &)
		{
			OutputDebugString(L"SingleSearchString Next Cancel...\r\n");
			m_IsSingleSearching = false;
			ReleaseExtraResources();
		}
	});
}

void PDFTextManager::NotifyByTextSearchDataItem(ITextSearchDataItem^ item)
{
	if (item)
	{
		if (m_SearchOnePageData.count(item->PageNo - 1))
		{
			auto _searchonepge = m_SearchOnePageData[item->PageNo - 1];
			auto _vect = _searchonepge->OnePageData;
			unsigned int index = -1;
			if (_vect->IndexOf(item, &index))
			{
				SingleSearchOnePage^ _singlesop = ref new SingleSearchOnePage(_searchonepge);
				_singlesop->CurrentIndexInPageData = index;
				m_CurSearchOnePage = _singlesop;
				NotifyToView(_singlesop);
			}
		}
	}
}

Platform::Array<Windows::Foundation::Rect>^ PDFTextManager::GetPageCoordinateRects(int pageindex)
{
	auto texthandle = m_PageHandleVect[pageindex];
	return texthandle ? texthandle->TextRect : nullptr;
}

bool PDFTextManager::LoadPdfTextResource(int pageindex)
{
	return InitPageTextArry(pageindex);
}

foxit::pdf::PDFPage^ PDFTextManager::GetPageHandle(int pageindex)
{
	return m_PageHandleVect[pageindex]->pageHandle;
}

bool PDFTextManager::IsResourcesReleased()
{
	return (m_IsForceReleaseResources && !m_IsSearching && !m_IsSingleSearching);
}

void PDFTextManager::ReleaseExtraResources()
{
	OutputDebugString(L"TextRecource ReleaseExtraResources\n");
	if (m_IsForceReleaseResources && !m_IsSearching && !m_IsSingleSearching)
	{
		if (m_PageHandleVect.size() > 0)
			m_PageHandleVect.clear();
		if (_SearchData->Size > 0)
			_SearchData->Clear();
		if (_SelectedText->Size > 0)
			_SelectedText->Clear();
		if (m_HasSearchedPage.size() > 0)
			m_HasSearchedPage.clear();
		if (m_SearchOnePageData.size() > 0)
			m_SearchOnePageData.clear();
		m_IsHadReleasedResources = true;
	}
}

void PDFTextManager::ForceReleaseResources()
{
	m_IsForceReleaseResources = true;
	if (m_PointerOverTextTimer && m_PointerOverTextTimer->IsEnabled)
		m_PointerOverTextTimer->Stop();

	if (m_IsSearching || m_IsSingleSearching)
		StopSearchString();
	else
		ReleaseExtraResources();
}

void PDFTextManager::AddTextSelect(Windows::Foundation::Collections::IVector<IPageSelectText^>^ pageTextVect)
{
	Platform::Array<IPageSelectText^>^ selArry = ref new Platform::Array<IPageSelectText^>(pageTextVect->Size);
	pageTextVect->GetMany(0, selArry);
	_SelectedText->ReplaceAll(selArry);
}

bool PDFTextManager::IsInitText(int pageindex)
{
	if (m_PageHandleVect[pageindex])
	{
		if (m_PageHandleVect[pageindex]->IsInit && m_PageHandleVect[pageindex]->IsInitTextRect)
			return true;
	}
	return false;
}

Platform::String^ PDFTextManager::GetTextByPosition(int pageindex, int start, int count)
{
	auto pDoc = m_WeakfpdfDocument.Resolve<foxit::pdf::PDFDoc>();
	foxit::pdf::PDFPage^ pPage = pDoc->GetPage(pageindex);
	auto pTextSelect = ref new foxit::pdf::TextPage(pPage, (int)foxit::pdf::TextParseFlags::e_ParseTextNormal);
	auto nCharCount = pTextSelect->GetCharCount();

	return pTextSelect->GetChars(start, count);
}

Platform::String^ PDFTextManager::GetTextByBounded(int pageindex, Windows::Foundation::Rect rect)
{
	auto pDoc = m_WeakfpdfDocument.Resolve<foxit::pdf::PDFDoc>();
	foxit::pdf::PDFPage^ pPage = pDoc->GetPage(pageindex);
	auto pTextSelect = ref new foxit::pdf::TextPage(pPage, (int)foxit::pdf::TextParseFlags::e_ParseTextNormal);

	return pTextSelect->GetTextInRect(FoxitRectConverter::ConvertToFoxitRectF(rect));
}

Platform::String^ PDFTextManager::GetTextByBoundedSegments(int pageindex, Windows::Foundation::Rect rect)
{
	auto pDoc = m_WeakfpdfDocument.Resolve<foxit::pdf::PDFDoc>();
	foxit::pdf::PDFPage^ pPage = pDoc->GetPage(pageindex);
	auto pTextSelect = ref new foxit::pdf::TextPage(pPage, (int)foxit::pdf::TextParseFlags::e_ParseTextNormal);

	return pTextSelect->GetTextInRect(FoxitRectConverter::ConvertToFoxitRectF(rect));
}

Platform::String^ PDFTextManager::GetWordsByPoint(int pageindex, Windows::Foundation::Point point)
{
	auto pDoc = m_WeakfpdfDocument.Resolve<foxit::pdf::PDFDoc>();
	foxit::pdf::PDFPage^ pPage = pDoc->GetPage(pageindex);
	auto pTextSelect = ref new foxit::pdf::TextPage(pPage, (int)foxit::pdf::TextParseFlags::e_ParseTextNormal);

	int nShartChar = 0;
	int nCharCount = 0;

	foxit::common::Range^ pCharRange = pTextSelect->GetWordAtPos(point.X, point.Y, (float)TextSelectTolerance);
	auto nSegmentCount = pCharRange && !pCharRange->IsEmpty() ? pCharRange->GetSegmentCount() : 0;
	if (pCharRange && nSegmentCount > 0)
		OnClearTextSelect();

	if (nSegmentCount > 0)
	{
		nShartChar = pCharRange->GetSegmentStart(0);
		nCharCount = pCharRange->GetSegmentEnd(0) - nShartChar + 1;
	}
	
	_SelectedTextString = pTextSelect->GetChars(nShartChar, nCharCount);

	PageSelectText^ sel = ref new PageSelectText(pageindex);
	auto __RcCount = pTextSelect->GetTextRectCount(nShartChar, nCharCount);
	for (int m = 0; m < __RcCount; m++)
	{
		int rotate = 0;
		rotate = (int)pTextSelect->GetBaselineRotation(m);
		sel->TextRotateVect->Append(rotate);
		Rect outrc;
		outrc = foxit::rdk::uiextensions::FoxitRectConverter::ConvertFromFoxitRectF(pTextSelect->GetTextRect(m));
		sel->TextRectVect->Append(outrc);
	}
	_SelectedText->Append(sel);

	if (!_SelectedTextString->IsEmpty())
	{
		TextSelectChangedEventArgs^ args = ref new TextSelectChangedEventArgs(TextSelectChangedType::Add, _SelectedText);
		TextSelectChanged(this, args);
	}

	return ref new Platform::String(_SelectedTextString->Data());
}

bool PDFTextManager::IsHitText(int pageindex, Windows::Foundation::Point pt)
{
	if (!IsInitText(pageindex))
	{
		if (!InitPageTextArry(pageindex))
			return false;
	}
	auto texthandle = m_PageHandleVect[pageindex];

	auto textarr = texthandle->TextRect;
	if (textarr == nullptr)
		return false;
	int nsize = textarr->Length;
	for (int i = 0; i < nsize; i++)
	{
		Windows::Foundation::Rect __rc = textarr->get(i);
		if (PointInRectangleByPageCoordiation(pt, __rc))
		{
			return true;
		}
	}
	return false;
}

Windows::Foundation::Rect PDFTextManager::GetHitTextRect(int pageindex, Windows::Foundation::Point pt)
{
	Windows::Foundation::Rect rect = Windows::Foundation::Rect::Empty;
	if (!IsInitText(pageindex))
	{
		if (!InitPageTextArry(pageindex))
			return rect;
	}
	auto texthandle = m_PageHandleVect[pageindex];

	auto textarr = texthandle->TextRect;
	if (textarr == nullptr)
		return rect;
	int nsize = textarr->Length;
	for (int i = 0; i < nsize; i++)
	{
		Windows::Foundation::Rect __rc = textarr->get(i);
		if (PointInRectangleByPageCoordiation(pt, __rc))
		{
			return __rc;
		}
	}
	return rect;
}

bool PDFTextManager::IsHitSelectedText(int pageindex, Windows::Foundation::Point pt)
{
	auto pPDFData = m_WeakfpdfDocument.Resolve<foxit::pdf::PDFDoc>();
	if (!pPDFData)
		return false;

	for each (IPageSelectText^ var in _SelectedText)
	{
		if (var->PageIndex == pageindex)
		{
			for each (Rect gmt in var->TextRectVect)
			{
				if (PointInRectangleByPageCoordiation(pt, gmt))
				{
					return true;
				}
			}
			return false;
		}
	}
	return false;
}

bool PDFTextManager::AddSelectText(int startindex, Windows::Foundation::Point startpt, int endindex, Windows::Foundation::Point endpt)
{
	auto pPDFData = m_WeakfpdfDocument.Resolve<foxit::pdf::PDFDoc>();
	if (!pPDFData)
		return false;
	int nCount = pPDFData->GetPageCount();

	if (startindex < 0 || endindex < 0 || startindex > nCount || endindex > nCount)
	{
		return false;
	}

	int __StartCharIndex = -1;
	int __EndCharIndex = -1;
	int __RcCount = -1;
	std::wstringstream ws;
	float nActualWidth, nActualHeight;
	int tolerance = (int)_TextSelectTolerance;
	int tolerance_size = 0;

	auto pDoc = m_WeakfpdfDocument.Resolve<foxit::pdf::PDFDoc>();

	int nStartChar = 0;
	int nStartCharCount = 0;
	int nStartTotalCount = 0;
	foxit::pdf::PDFPage^ pStartPage = pDoc->GetPage(startindex);
	nActualWidth = pStartPage->GetWidth();
	nActualHeight = pStartPage->GetHeight();
	tolerance_size = (int)max(nActualHeight, nActualWidth);
	
	auto pStartTextSelect = ref new foxit::pdf::TextPage(pStartPage, (int)foxit::pdf::TextParseFlags::e_ParseTextNormal);
	while (true)
	{
		__StartCharIndex = pStartTextSelect->GetIndexAtPos(startpt.X, startpt.Y, (float)tolerance);
		if (__StartCharIndex >= 0)
		{
			break;
		}
		else
		{
			tolerance += 2;
			if (tolerance > tolerance_size)
			{
				return false;
			}
			continue;
		}
	}

	int nEndChar = 0;
	int nEndCharCount = 0;
	int nEndTotalCount = 0;
	foxit::pdf::PDFPage^ pEndPage = pDoc->GetPage(endindex);
	nActualWidth = pEndPage->GetWidth();
	nActualHeight = pEndPage->GetHeight();
	tolerance_size = (int)max(nActualHeight, nActualWidth);
	tolerance = (int)_TextSelectTolerance;

	auto pEndTextSelect = ref new foxit::pdf::TextPage(pEndPage, (int)foxit::pdf::TextParseFlags::e_ParseTextNormal);
	while (true)
	{
		__EndCharIndex = pEndTextSelect->GetIndexAtPos(endpt.X, endpt.Y, (float)tolerance);
		if (__EndCharIndex >= 0)
		{
			break;
		}
		else
		{
			tolerance += 2;
			if (tolerance > tolerance_size)
			{
				return false;
			}
			continue;
		}
	}

	bool bIsStartFixed = (m_pTextSelectOptContext->StartIndex == startindex && m_pTextSelectOptContext->StartCharIndex == __StartCharIndex) ? true : false;
	bool bIsEndFixed = (m_pTextSelectOptContext->EndIndex == endindex && m_pTextSelectOptContext->EndCharIndex == __EndCharIndex) ? true : false;

	m_pTextSelectOptContext->StartIndex = startindex;
	m_pTextSelectOptContext->StartCharIndex = __StartCharIndex;
	m_pTextSelectOptContext->EndIndex = endindex;
	m_pTextSelectOptContext->EndCharIndex = __EndCharIndex;

	if (bIsStartFixed && bIsEndFixed)
		return true;

	OnClearTextSelect();

	int charstart = 0, charend = 0;

	if (startindex < endindex)
	{
		for (int i = startindex; i < endindex + 1; i++)
		{
			PageSelectText^ sel = ref new PageSelectText(i);
			foxit::pdf::PDFPage^ pPage = pDoc->GetPage(i);
			foxit::pdf::TextPage^ pTextSelect = ref new foxit::pdf::TextPage(pPage, (int)foxit::pdf::TextParseFlags::e_ParseTextNormal);
			if (i == startindex)
			{
				charstart = __StartCharIndex;
				charend = pTextSelect->GetCharCount();
				if (charend == -1)
					continue;
				charend -= 1;
			}
			else if (i == endindex)
			{
				charstart = 0;
				charend = __EndCharIndex;
			}
			else
			{
				charstart = 0;
				charend = pTextSelect->GetCharCount();
				if (charend == -1)
					continue;
				charend -= 1;
			}

			Platform::String^ tempstr = pTextSelect->GetChars(charstart, charend - charstart + 1);
			if (tempstr->Length() > 0)
			{
				ws << tempstr->Data();
			}

			__RcCount = pTextSelect->GetTextRectCount(charstart, charend - charstart + 1);
			if (__RcCount == -1)
				continue;
			for (int m = 0; m < __RcCount; m++)
			{
				int rotate = 0;
				rotate = (int)pTextSelect->GetBaselineRotation(m);
				sel->TextRotateVect->Append(rotate);
				Rect outrc;
				outrc = foxit::rdk::uiextensions::FoxitRectConverter::ConvertFromFoxitRectF(pTextSelect->GetTextRect(m));
				sel->TextRectVect->Append(outrc);
			}

			sel->_CharStart = charstart;
			sel->_CharEnd = charend;
			_SelectedText->Append(sel);
		}
	}
	else if (startindex == endindex)
	{
		PageSelectText^ sel = ref new PageSelectText(startindex);

		if (__StartCharIndex <= __EndCharIndex)
		{
			charstart = __StartCharIndex;
			charend = __EndCharIndex;
		}
		else
		{
			if (bIsStartFixed)
			{
				charstart = __EndCharIndex;
				charend = __StartCharIndex - 1;
			}
			else if (bIsEndFixed)
			{
				charstart = __EndCharIndex + 1;
				charend = __StartCharIndex;
			}
			else
			{
				charstart = __EndCharIndex;
				charend = __StartCharIndex;
			}
		}
		
		foxit::pdf::PDFPage^ pPage = pDoc->GetPage(startindex);
		foxit::pdf::TextPage^ pTextSelect = ref new foxit::pdf::TextPage(pPage, (int)foxit::pdf::TextParseFlags::e_ParseTextNormal);
		Platform::String^ tempstr = pTextSelect->GetChars(charstart, charend - charstart + 1);
		if (tempstr->Length() > 0)
		{
			ws << tempstr->Data();
		}

		__RcCount = pTextSelect->GetTextRectCount(charstart, charend - charstart + 1);
		if (__RcCount == -1)
			return false;
		for (int m = 0; m < __RcCount; m++)
		{
			int rotate = 0;
			rotate = (int)pTextSelect->GetBaselineRotation(m);
			sel->TextRotateVect->Append(rotate);
			Rect outrc;
			outrc = foxit::rdk::uiextensions::FoxitRectConverter::ConvertFromFoxitRectF(pTextSelect->GetTextRect(m));
			sel->TextRectVect->Append(outrc);
		}

		sel->_CharStart = charstart;
		sel->_CharEnd = charend;
		_SelectedText->Append(sel);
	}
	else
	{
		for (int i = endindex; i < startindex + 1; i++)
		{
			PageSelectText^ sel = ref new PageSelectText(i);
			foxit::pdf::PDFPage^ pPage = pDoc->GetPage(i);
			foxit::pdf::TextPage^ pTextSelect = ref new foxit::pdf::TextPage(pPage, (int)foxit::pdf::TextParseFlags::e_ParseTextNormal);
			if (i == endindex)
			{
				if (bIsStartFixed)
					charstart = __EndCharIndex;
				else if (bIsEndFixed)
					charstart = __EndCharIndex + 1;
				else
					charstart = __EndCharIndex;
				charend = pTextSelect->GetCharCount();
				if (charend == -1)
					continue;
				charend -= 1;
			}
			else if (i == startindex)
			{
				charstart = 0;

				if (bIsStartFixed)
					charend = __StartCharIndex - 1;
				else if (bIsEndFixed)
					charend = __StartCharIndex;
				else
					charend = __StartCharIndex;
			}
			else
			{
				charstart = 0;
				charend = pTextSelect->GetCharCount();
				if (charend == -1)
					continue;
				charend -= 1;
			}

			Platform::String^ tempstr = pTextSelect->GetChars(charstart, charend - charstart + 1);
			if (tempstr->Length() > 0)
			{
				ws << tempstr->Data();
			}

			__RcCount = pTextSelect->GetTextRectCount(charstart, charend - charstart + 1);
			if (__RcCount == -1)
				continue;
			for (int m = 0; m < __RcCount; m++)
			{
				int rotate = 0;
				rotate = (int)pTextSelect->GetBaselineRotation(m);
				sel->TextRotateVect->Append(rotate);
				Rect outrc;
				outrc = foxit::rdk::uiextensions::FoxitRectConverter::ConvertFromFoxitRectF(pTextSelect->GetTextRect(m));
				sel->TextRectVect->Append(outrc);
			}

			sel->_CharStart = charstart;
			sel->_CharEnd = charend;
			_SelectedText->Append(sel);
		}
	}

	_SelectedTextString = ref new Platform::String(ws.str().c_str());
	if (!_SelectedTextString->IsEmpty())
	{
		TextSelectChangedEventArgs^ args = ref new TextSelectChangedEventArgs(TextSelectChangedType::Add, _SelectedText);
		TextSelectChanged(this, args);
	}
	return true;
}

double PDFTextManager::TextSelectTolerance::get()
{
	return _TextSelectTolerance;
}

void PDFTextManager::TextSelectTolerance::set(double value)
{
	if (value >= 0 && value <= MAXTEXTSELECTTOLERANCE)
		_TextSelectTolerance = value;
}

bool PDFTextManager::InitPageTextArry(int pageindex)
{
	std::shared_ptr<TextHandleContext> __sp;
	if (!m_PageHandleVect.count(pageindex))
	{
		LoadPageHandle(pageindex);
		__sp = m_PageHandleVect[pageindex];
	}
	else
	{
		__sp = m_PageHandleVect[pageindex];
		if (!__sp)
		{
			LoadPageHandle(pageindex);
			__sp = m_PageHandleVect[pageindex];
		}
	}

	if (!__sp)
		return false;

	if (__sp->IsInitTextRect)
		return true;

	__sp->LoadTextPageRects();
	return __sp->IsInitTextRect;
}

bool PDFTextManager::PointInRectangleByPageCoordiation(Windows::Foundation::Point pt, Windows::Foundation::Rect rc)
{
	return (pt.X >= rc.X - _TextSelectTolerance && pt.X <= rc.X + rc.Width + _TextSelectTolerance && pt.Y >= rc.Y - _TextSelectTolerance && pt.Y <= rc.Y + rc.Height + _TextSelectTolerance) ? true : false;
}

bool PDFTextManager::IsHasFinded(int pageindex)
{
	auto iter = std::find_if(std::begin(m_HasSearchedPage), std::end(m_HasSearchedPage), [pageindex](int value)
	{
		return pageindex == value;
	});
	if (iter != std::end(m_HasSearchedPage))
		return true;
	return false;
}

bool PDFTextManager::IsPrevRefPageIndex(int pageindex)
{
	auto iter = std::find_if(std::begin(m_PrevOpraRefPageIndexVector), std::end(m_PrevOpraRefPageIndexVector), [pageindex](int value)
	{
		return pageindex == value;
	});
	if (iter != std::end(m_PrevOpraRefPageIndexVector))
		return true;
	return false;
}

bool PDFTextManager::IsHadRefPageIndex(int pageindex)
{
	m_PageIndexVectorCritsec->Lock();
	auto iter = std::find_if(std::begin(m_RefPageIndexVector), std::end(m_RefPageIndexVector), [pageindex](int value)
	{
		return pageindex == value;
	});
	m_PageIndexVectorCritsec->Unlock();
	if (iter != std::end(m_RefPageIndexVector))
		return true;
	return false;
}

void PDFTextManager::AddPageIndexRef(int pageindex)
{
	if (!IsHadRefPageIndex(pageindex))
	{
		m_PageIndexVectorCritsec->Lock();
		m_RefPageIndexVector.push_back(pageindex);
		m_PageIndexVectorCritsec->Unlock();
		auto _Doc = m_WeakfpdfDocument.Resolve<foxit::pdf::PDFDoc>();
		if (_Doc == nullptr)
			return;
		auto _PdfPage = _Doc->GetPage(pageindex);
		if (_PdfPage == nullptr)
			return;
	}
}

void PDFTextManager::RemovePageIndexRef(int pageindex)
{
	m_PageIndexVectorCritsec->Lock();
	auto _iter = std::find_if(std::begin(m_RefPageIndexVector), std::end(m_RefPageIndexVector), [pageindex](int value)
	{
		return pageindex == value;
	});

	if (_iter != std::end(m_RefPageIndexVector))
	{
		auto _TextHandle = m_PageHandleVect[pageindex];
		if (_TextHandle)
		{
			_TextHandle->RemoveRef();
			if (!_TextHandle->IsValid())
			{
				_TextHandle->ReleaseTextHandle();
				m_RefPageIndexVector.erase(_iter);
			}
		}
	}
	m_PageIndexVectorCritsec->Unlock();
}

void PDFTextManager::AddPageIndexRef(int startindex, int endindex)
{
	int istart = (startindex < endindex) ? startindex : endindex;
	int iend = (startindex < endindex) ? endindex : startindex;

	for (int i = istart; i < iend + 1; i++)
	{
		AddPageIndexRef(i);
	}
}

void PDFTextManager::RemovePageIndexRef(int startindex, int endindex)
{
	int istart = (startindex < endindex) ? startindex : endindex;
	int iend = (startindex < endindex) ? endindex : startindex;

	for (int i = istart; i < iend + 1; i++)
	{
		RemovePageIndexRef(i);
	}
}

void PDFTextManager::NotifyToView(SingleSearchOnePage^ data)
{
	SearchResultEventArgs^ args = ref new SearchResultEventArgs(SearchState::FOUND, data->OnePageData, data->CurrentIndexInPageData);
	SearchResult(this, args);
	OnClearTextSelect();
	PageSelectText^ sel = ref new PageSelectText(data->OnePageData->PageIndex);
	auto rcvect = data->OnePageData->OnePageData->GetAt(data->CurrentIndexInPageData)->RectVect;
	auto selarry = ref new Platform::Array<Windows::Foundation::Rect>(rcvect->Size);
	rcvect->GetMany(0, selarry);
	sel->TextRectVect->ReplaceAll(selarry);
	_SelectedText->Append(sel);
	TextSelectChangedEventArgs^ textselectargs = ref new TextSelectChangedEventArgs(TextSelectChangedType::Add, _SelectedText);
	TextSelectChanged(this, textselectargs);
}

Concurrency::task<SingleSearchOnePage^> PDFTextManager::SingleSearchString()
{
	return Concurrency::create_task([this]()
	{
		auto pDoc = m_WeakfpdfDocument.Resolve<foxit::pdf::PDFDoc>();
		int pagecount = pDoc->GetPageCount();
		int count = 0;
		SingleSearchOnePage^ _singlesearchdata = nullptr;
		Platform::String^ _SearchTextString = ref new Platform::String(m_SearchTextString->Data());

#if _DEBUG
		do
		{
			std::wstringstream ws;
			ws << L"SingleSearchString: Starting From " << m_CurSearchIndex << L".....................\n";
			OutputDebugString(ws.str().c_str());
		} while (0);
#endif

		while (!m_SingleSearchCts.get_token().is_canceled())
		{
			if (m_CurSearchOnePage)
			{
				if (m_SearchMode == 0)
					m_CurSearchOnePage->CurrentIndexInPageData++;
				else
					m_CurSearchOnePage->CurrentIndexInPageData--;
				if (m_CurSearchOnePage->CurrentIndexInPageData >= 0 && m_CurSearchOnePage->CurrentIndexInPageData < (int)m_CurSearchOnePage->OnePageData->OnePageData->Size)
					return m_CurSearchOnePage;
				else
				{
					if (m_SearchMode == 0)
					{
						m_CurSearchIndex++;
						if (m_CurSearchIndex > pagecount - 1)
						{
							m_CurSearchIndex = 0;
						}
					}
					else
					{
						m_CurSearchIndex--;
						if (m_CurSearchIndex < 0)
						{
							m_CurSearchIndex = pagecount - 1;
						}
					}
				}
			}

			if (IsHasFinded(m_CurSearchIndex))
			{
#if _DEBUG
				do
				{
					std::wstringstream ws;
					ws << L"SingleSearchString: " << m_CurSearchIndex << L" Had Find......................\n";
					OutputDebugString(ws.str().c_str());
				} while (0);
#endif

				if (m_SearchOnePageData.count(m_CurSearchIndex))
				{
					auto single = m_SearchOnePageData[m_CurSearchIndex];

					if (single)
					{
						m_CurSearchOnePage = ref new SingleSearchOnePage(single);
						if (m_SearchMode == 0)
							m_CurSearchOnePage->CurrentIndexInPageData = 0;
						else
							m_CurSearchOnePage->CurrentIndexInPageData = single->OnePageData->Size - 1;
						return m_CurSearchOnePage;
					}

				}

				if (m_HasSearchedPage.size() == pagecount && m_SearchOnePageData.size() == 0)
				{
					return _singlesearchdata;
				}
				else
				{
					m_CurSearchOnePage = nullptr;
					if (m_SearchMode == 0)
					{
						m_CurSearchIndex++;
						if (m_CurSearchIndex > pagecount - 1)
						{
							m_CurSearchIndex = 0;
						}
					}
					else
					{
						m_CurSearchIndex--;
						if (m_CurSearchIndex < 0)
						{
							m_CurSearchIndex = pagecount - 1;
						}
					}
				}
				continue;
			}

			if (!SearchLoadHandle(m_CurSearchIndex, m_pCurPageHandle, m_pCurTextPageHandle, m_pCurSeachHandle, _SearchTextString, 0, 0))
			{
				AddPageIndexRef(m_CurSearchIndex);

#if _DEBUG
				do
				{
					std::wstringstream ws;
					ws << L"SingleSearchString: " << m_CurSearchIndex << L" Find Failed......................\n";
					OutputDebugString(ws.str().c_str());
				} while (0);
#endif
				m_HasSearchedPage.push_back(m_CurSearchIndex);

				m_pCurPageHandle = nullptr;
				m_pCurTextPageHandle = nullptr;
				if (nullptr != m_pCurSeachHandle)
				{
					m_pCurSeachHandle = nullptr;
				}
				RemovePageIndexRef(m_CurSearchIndex);

				if (m_SearchMode == 0)
				{
					m_CurSearchIndex++;
					if (m_CurSearchIndex > pagecount - 1)
					{
						m_CurSearchIndex = 0;
					}
				}
				else
				{
					m_CurSearchIndex--;
					if (m_CurSearchIndex < 0)
					{
						m_CurSearchIndex = pagecount - 1;
					}
				}
				continue;
			}
#if _DEBUG && 1
			do
			{
				std::wstringstream ws;
				ws << L"SingleSearchString: " << m_CurSearchIndex << L" ......................\n";
				OutputDebugString(ws.str().c_str());
			} while (0);
#endif
			{
				std::wstringstream ws;
				ws << L"Start Single Release TextPage, PageIndex: " << m_CurSearchIndex;
				ws << L", SearchHandle Ptr: " << m_pCurSeachHandle->GetHashCode();
				ws << L"\n";
				OutputDebugString(ws.str().c_str());
			}
			AddPageIndexRef(m_CurSearchIndex);
			auto onepagedata = ref new SearchOnePage(m_CurSearchIndex);

			while (1)
			{
				auto tmpitem = ref new TextSearchDataItem();
				if (GetSearchItemData(m_CurSearchIndex, m_pCurTextPageHandle, m_pCurSeachHandle, tmpitem, 0))
				{
					onepagedata->OnePageData->Append(tmpitem);
					tmpitem->PageNo = m_CurSearchIndex + 1;
					if (count == 0)
						tmpitem->IsFirstResultInPage = true;
					else
						tmpitem->IsFirstResultInPage = false;

					GetSearchItemContextString(m_CurSearchIndex, m_pCurTextPageHandle, tmpitem);
					count++;
				}
				else
					break;
					
			}
			m_HasSearchedPage.push_back(m_CurSearchIndex);

			if (nullptr != m_pCurSeachHandle)
			{
				RemovePageIndexRef(m_CurSearchIndex);
			}
			{
				std::wstringstream ws;
				ws << L"End Single Release TextPage, PageIndex: " << m_CurSearchIndex;
				ws << L", SearchHandle Ptr: " << m_pCurSeachHandle->GetHashCode();
				ws << L"\n";
				OutputDebugString(ws.str().c_str());
			}

			if (onepagedata->OnePageData->Size > 0)
			{
				_singlesearchdata = ref new SingleSearchOnePage(onepagedata);
				if (!m_SearchOnePageData.count(m_CurSearchIndex))
					m_SearchOnePageData.insert(std::make_pair(m_CurSearchIndex, onepagedata));
			}
			else
			{
				if (m_SearchMode == 0)
				{
					m_CurSearchIndex++;
					if (m_CurSearchIndex > pagecount - 1)
					{
						m_CurSearchIndex = 0;
					}
				}
				else
				{
					m_CurSearchIndex--;
					if (m_CurSearchIndex < 0)
					{
						m_CurSearchIndex = pagecount - 1;
					}
				}
				continue;
			}

			do
			{
				std::wstringstream ws;
				ws << L"SingleSearchString: " << m_CurSearchIndex << L",SearchedSize: " << onepagedata->OnePageData->Size << L" ..................\n";
				OutputDebugString(ws.str().c_str());
			} while (0);

			if (_singlesearchdata)
			{
				m_CurSearchOnePage = _singlesearchdata;
				if (m_SearchMode == 0)
				{
					_singlesearchdata->CurrentIndexInPageData = 0;
				}
				else
				{
					_singlesearchdata->CurrentIndexInPageData = _singlesearchdata->OnePageData->OnePageData->Size - 1;
				}
				return _singlesearchdata;
			}
			else
			{
				m_CurSearchOnePage = nullptr;
				continue;
			}
		}
		return _singlesearchdata;
	});
}

void PDFTextManager::OutputCompositeTransformInfo(Windows::UI::Xaml::FrameworkElement^ framework)
{
	if (!(framework && framework->RenderTransform))
		return;

	OutputDebugString(framework->RenderTransform->ToString()->Data());
	OutputDebugString(L"\n");

	auto _compositeTransform = safe_cast<CompositeTransform^>(framework->RenderTransform);
	if (!_compositeTransform)
		return;
	std::wstringstream ws;
	auto left = safe_cast<Platform::IBox<double>^>(framework->GetValue(Canvas::LeftProperty))->Value;
	auto top = safe_cast<Platform::IBox<double>^>(framework->GetValue(Canvas::TopProperty))->Value;

	ws << L"left: " << left;
	ws << L", top: " << top;
	ws << L", ActualWidth: " << framework->ActualWidth;
	ws << L", ActualHeight: " << framework->ActualHeight;
	ws << L", RenderSize.Width: " << framework->RenderSize.Width;
	ws << L", RenderSize.Height: " << framework->RenderSize.Height;
	ws << L", CenterX: " << _compositeTransform->CenterX;
	ws << L", CenterY: " << _compositeTransform->CenterY;
	ws << L", TranslateX: " << _compositeTransform->TranslateX;
	ws << L", TranslateY: " << _compositeTransform->TranslateY;
	ws << L", ScaleX: " << _compositeTransform->ScaleX;
	ws << L", ScaleY: " << _compositeTransform->ScaleY;
	ws << L", Rotation: " << _compositeTransform->Rotation;
	ws << L", SkewX: " << _compositeTransform->SkewX;
	ws << L", SkewY: " << _compositeTransform->SkewY;

	ws << L"\n";
	OutputDebugString(ws.str().c_str());
}

Platform::String^ PDFTextManager::StandardizationDealWithString(Platform::String^ source)
{
	std::wstring fmt(L" ");
	return ref new String(std::regex_replace(std::regex_replace(source->Begin(), std::wregex(L"\r\n"), fmt), std::wregex(L"[\r\n]"), fmt).c_str());
}

Platform::String^ PDFTextManager::LimitDealWithString(Platform::String^ source)
{
	return source->Length() > (unsigned int)GetOption()->GetSearchTextMaxLength() ? ref new Platform::String(source->Begin(), GetOption()->GetSearchTextMaxLength()) : source;
}

bool PDFTextManager::IsSelectedPageIndex(int pageindex, unsigned int* index)
{
	if (_SelectedText->Size <= 0)
		return false;
	bool _ishas = false;
	auto _iter = std::find_if(begin(_SelectedText), end(_SelectedText), [pageindex](IPageSelectText^ item)
	{
		return item->PageIndex == pageindex ? true : false;
	});
	if (_iter != end(_SelectedText))
	{
		*index = unsigned int(_iter - begin(_SelectedText));
		return true;
	}
	return false;
}

void PDFTextManager::ClearTextSelect()
{
	OnClearTextSelect();
}

void PDFTextManager::OnClearTextSelect()
{
	TextSelectChangedEventArgs^ args = ref new TextSelectChangedEventArgs(TextSelectChangedType::Clear, _SelectedText);
	TextSelectChanged(this, args);
	_SelectedText->Clear();
	if (_SelectedTextString->Length() > 0)
		_SelectedTextString = ref new Platform::String();
}

void PDFTextManager::SetOption(ITextSelectOption^ opt)
{
	if (opt)
	{
		m_Option = opt;
		UpdateTextSelectOption();
	}
}

ITextSelectOption^ PDFTextManager::GetOption()
{
	if (!m_Option)
	{
		m_Option = ref new DefaultTextSelectOption();
		UpdateTextSelectOption();
	}
	return m_Option;
}

void PDFTextManager::UpdateTextSelectOption()
{
	_TextSelectTolerance = m_Option->Tolerance;
}

void PDFTextManager::GetSearchItemContextString(int pageIndex, foxit::pdf::TextPage^ pTextPage, TextSearchDataItem^ item)
{
	int totalcharacterlen = 0;
	String^ prevstr = nullptr;
	String^ nextstr = nullptr;

	int iStartCharIndex = item->StartCharIndex;
	int seachbuflen = item->Query->Length();

	item->ContextTillQuery = prevstr;
	item->ContextPostQuery = nextstr;
}

void PDFTextManager::ClearSelectedText()
{
	OnClearTextSelect();
}

void PDFTextManager::UpdateSelectContext()
{
	auto _SelectText = SelectedText;
	auto _size = SelectedText->Size;
	if (_size == 0)
		return;

	auto _pagerc = _SelectText->GetAt(0)->TextRectVect;
	int _startindex = _SelectText->GetAt(0)->PageIndex;
	auto __rcstart = _pagerc->GetAt(0);
	m_pTextSelectContext->StartIndex = _startindex;
	m_pTextSelectContext->StartPoint = Windows::Foundation::Point(__rcstart.Left, __rcstart.Bottom);
	auto _endpagerc = _SelectText->GetAt(_size - 1)->TextRectVect;
	int _endindex = _SelectText->GetAt(_size - 1)->PageIndex;
	auto _rcend = _endpagerc->Size > 0 ? _endpagerc->GetAt(_endpagerc->Size - 1) : __rcstart;
	m_pTextSelectContext->EndIndex = _endindex;
	m_pTextSelectContext->EndPoint = Windows::Foundation::Point(_rcend.Right, _rcend.Top);
}

bool PDFTextManager::GetIsShowSelHandleBtn()
{
	return m_IsShowSelHandleBtn;
}

void PDFTextManager::SetIsShowSelHandleBtn(bool bIsShow)
{
	m_IsShowSelHandleBtn = bIsShow;
}

void PDFTextManager::OnRender(foxit::rdk::PDFViewCtrl^ sender, RenderCanvasEventArgs^ e)
{
	auto _PageIndex = e->PageIndex;
	auto _SelectedText = SelectedText;
	auto _SelectedTextSize = _SelectedText->Size;
	auto _PageView = PDFViewCtrl;
	auto _Doc = _PageView->GetDoc();

	for (unsigned int i = 0; i < _SelectedTextSize; i++)
	{
		auto _Item = _SelectedText->GetAt(i);
		if (_Item->PageIndex != _PageIndex)
			continue;

		auto _Rects = _Item->TextRectVect;
		auto _Size = _Rects->Size;
		for (unsigned int j = 0; j < _Size; j++)
		{
			auto _Item = _Rects->GetAt(j);
			
			auto _DeviceRc = sender->ConvertPageRectToDeviceRect(_PageIndex, _Item);
			e->Device->FillRectangle(_DeviceRc, Windows::UI::ColorHelper::FromArgb(0x7f, 0x33, 0xcc, 0x00));
		}
	}

	auto _SearchTextInfo = CurSearchTextInfo;
	if (_SearchTextInfo)
	{
		auto _OnePageFind = _SearchTextInfo->OnePageData;
		if (_OnePageFind && _PageIndex == _OnePageFind->PageIndex)
		{
			auto _OnePageFindSize = _OnePageFind->OnePageData->Size;
			unsigned int _index = _SearchTextInfo->CurrentIndexInPageData;
			if (_index >= 0 && _index < _OnePageFindSize)
			{
				auto _TextSearchItem = _OnePageFind->OnePageData->GetAt(_index);
				auto _Rects = _TextSearchItem->RectVect;
				auto _Size = _Rects->Size;
				for (unsigned int j = 0; j < _Size; j++)
				{
					auto _Item = _Rects->GetAt(j);

					auto _DeviceRc = sender->ConvertPageRectToDeviceRect(_PageIndex, _Item);
					e->Device->FillRectangle(_DeviceRc);
				}
			}
		}
	}

	UIExtMgr->ShowTouchSelectHandleButton(SelectedText);
}

void PDFTextManager::OnPointerPressed(Platform::Object^ sender, Windows::UI::Xaml::Input::PointerRoutedEventArgs^ e)
{
	Windows::UI::Xaml::FrameworkElement^ _FrameworkElement = dynamic_cast<Windows::UI::Xaml::FrameworkElement^>(sender);
	auto _PageView = PDFViewCtrl;
	if (!_PageView)
		return;
	if (_FrameworkElement == nullptr || _PageView == nullptr)
		return;
	UIExtensionsManager^ extMgr = UIExtMgr;
	auto _SelectOperate = extMgr->SelectOperate;
	auto _CreateTypeOfAnnot = extMgr->CreateTypeOfAnnot;
	auto _PointerDeviceType = e->Pointer->PointerDeviceType;

	if (_SelectOperate == SelectOperateType::Hand || _SelectOperate == SelectOperateType::SelectText)
	{
		if (_PointerDeviceType == Windows::Devices::Input::PointerDeviceType::Touch)
		{
			if (!m_IsPressed && ((m_IsShowSelHandleBtn && _CreateTypeOfAnnot == AnnotCreateType::None) || _CreateTypeOfAnnot != AnnotCreateType::None))
			{
				Windows::Foundation::Point _Pt;
				auto _PointerPt = e->GetCurrentPoint(_FrameworkElement);
				_Pt = _PointerPt->Position;
				auto _Pageinfo = PDFViewCtrl->ConvertDevicePtToPDFPagePt(_Pt);
				int index = _Pageinfo->PageIndex;
				auto _PagePoint = _Pageinfo->PagePoint;
				m_IsHitText = IsHitText(index, _PagePoint);

				if (m_IsHitText)
				{
					m_IsPressed = true;
					m_pTextSelectContext->StartIndex = index;
					m_pTextSelectContext->StartPoint = _PagePoint;

					if (_CreateTypeOfAnnot == AnnotCreateType::None)
						return;

					PDFViewCtrl->EnableScrollView(false);
				}
				else
				{
					m_IsPressed = false;
				}
			}
		}
		else
		{
			if (_SelectOperate == SelectOperateType::Hand)
			{
				if (!m_IsPointerOverText)
					return;
			}

			m_IsPressed = true;
			Windows::Foundation::Point _Pt;
			auto _PointerPt = e->GetCurrentPoint(_FrameworkElement);
			_Pt = _PointerPt->Position;
			auto _Pageinfo = PDFViewCtrl->ConvertDevicePtToPDFPagePt(_Pt);
			int index = _Pageinfo->PageIndex;
			auto _PagePoint = _Pageinfo->PagePoint;
			m_IsHitText = IsHitText(index, _PagePoint);
			if (m_IsHitText)
			{
				m_IsPressed = true;
				m_pTextSelectContext->StartIndex = index;
				m_pTextSelectContext->StartPoint = _PagePoint;

				PDFViewCtrl->EnableScrollView(false);
				_FrameworkElement->CapturePointer(e->Pointer);
			}
			else
			{
				m_IsPressed = false;
			}
		}
	}
	
}

void PDFTextManager::OnPointerReleased(Platform::Object^ sender, Windows::UI::Xaml::Input::PointerRoutedEventArgs^ e)
{
	Windows::UI::Xaml::FrameworkElement^ _FrameworkElement = dynamic_cast<Windows::UI::Xaml::FrameworkElement^>(sender);
	auto _PageView = m_PageViewWeakRef.Resolve<foxit::rdk::PDFViewCtrl>();
	if (!_PageView)
		return;
	UIExtensionsManager^ extMgr = UIExtMgr;
	if (_FrameworkElement == nullptr || _PageView == nullptr )
		return;
	
	auto _SelectOperate = extMgr->SelectOperate;
	auto _CreateTypeOfAnnot = extMgr->CreateTypeOfAnnot;
	auto _PointerDeviceType = e->Pointer->PointerDeviceType;

	m_IsPressed = false;
	if (m_IsHitText)
	{
		m_IsHitText = false;

		PDFViewCtrl->EnableScrollView(true);
		_FrameworkElement->ReleasePointerCapture(e->Pointer);
		if (_SelectedText->Size > 0 && _SelectedText->GetAt(0)->TextRectVect->Size > 0)
		{
			if (_PageView && extMgr->CreateTypeOfAnnot == AnnotCreateType::None)
				extMgr->ShowTextmarkContextmenu(AnnotContextMenuType::Add, Windows::Foundation::Point(0, 0));
		}
	}

	if (_SelectedText->Size > 0 && _SelectedText->GetAt(0)->TextRectVect->Size > 0)
		m_IsShowSelHandleBtn = _PointerDeviceType == Windows::Devices::Input::PointerDeviceType::Touch ? true : false;

	if (m_IsHitSelHandleBtn)
	{
		m_IsHitSelHandleBtn = false;
		UpdateSelectContext();
		if (extMgr)
			extMgr->ShowTouchSelectHandleButton(SelectedText);

		if (_SelectedText->Size > 0 && _SelectedText->GetAt(0)->TextRectVect->Size > 0)
		{
			if (extMgr && extMgr->CreateTypeOfAnnot == AnnotCreateType::None)
				extMgr->ShowTextmarkContextmenu(AnnotContextMenuType::Add, Windows::Foundation::Point(0, 0));
		}
	}
	
}

void PDFTextManager::OnPointerMoved(Platform::Object^ sender, Windows::UI::Xaml::Input::PointerRoutedEventArgs^ e)
{
	Windows::UI::Xaml::FrameworkElement^ _FrameworkElement = dynamic_cast<Windows::UI::Xaml::FrameworkElement^>(sender);
	auto _PageView = m_PageViewWeakRef.Resolve<foxit::rdk::PDFViewCtrl>();
	if (!_PageView) return;
	UIExtensionsManager^ extMgr = (UIExtensionsManager^)_PageView->ExtensionsManager;
	if (_FrameworkElement == nullptr || _PageView == nullptr)
		return;

	auto _SelectOperate = extMgr->SelectOperate;
	auto _CreateTypeOfAnnot = extMgr->CreateTypeOfAnnot;
	auto _PointerDeviceType = e->Pointer->PointerDeviceType;

	if (_CreateTypeOfAnnot == AnnotCreateType::None)
	{
		if (_PointerDeviceType == Windows::Devices::Input::PointerDeviceType::Touch)
		{
			if (m_IsHitSelHandleBtn)
			{
				Windows::Foundation::Point _Pt;
				auto _PointerPt = e->GetCurrentPoint(_FrameworkElement);
				_Pt = _PointerPt->Position;
				auto _Pageinfo = PDFViewCtrl->ConvertDevicePtToPDFPagePt(_Pt);
				int index = _Pageinfo->PageIndex;
				auto _PagePoint = _Pageinfo->PagePoint;

				if (m_IsHitStartSelHandleBtn)
				{
					m_pTextSelectContext->StartIndex = index;
					m_pTextSelectContext->StartPoint = _PagePoint;
				}
				else
				{
					m_pTextSelectContext->EndIndex = index;
					m_pTextSelectContext->EndPoint = _PagePoint;
				}
				m_IsShowSelHandleBtn = false;

				AddSelectText(m_pTextSelectContext->StartIndex, m_pTextSelectContext->StartPoint, m_pTextSelectContext->EndIndex, m_pTextSelectContext->EndPoint);
				PDFViewCtrl->Update();
			}
		}
		else
		{
			Windows::Foundation::Point _Pt;
			auto _PointerPt = e->GetCurrentPoint(_FrameworkElement);
			_Pt = _PointerPt->Position;
			auto _Pageinfo = PDFViewCtrl->ConvertDevicePtToPDFPagePt(_Pt);
			int index = _Pageinfo->PageIndex;
			auto _PagePoint = _Pageinfo->PagePoint;
			auto _IsOverText = IsHitText(index, _PagePoint);
			auto _pHitAnnot = extMgr->AnnotMgr->IsHitAnnot(index, _PagePoint);
			auto _bIsOverAnnot = _pHitAnnot != nullptr;

			if (m_IsPressed && m_IsHitText)
			{
				m_pTextSelectContext->EndIndex = index;
				m_pTextSelectContext->EndPoint = _PagePoint;

				AddSelectText(m_pTextSelectContext->StartIndex, m_pTextSelectContext->StartPoint, m_pTextSelectContext->EndIndex, m_pTextSelectContext->EndPoint);
				PDFViewCtrl->Update();
			}
			else
			{
				if (_bIsOverAnnot)
				{
					if (_pHitAnnot)
					{
					}
				}
				else if (_IsOverText)
				{
					if (!m_PointerOverTextTimer->IsEnabled)
						m_PointerOverTextTimer->Start();
				}
				else
				{
					if (m_PointerOverTextTimer && m_PointerOverTextTimer->IsEnabled)
						m_PointerOverTextTimer->Stop();
					m_IsPointerOverText = false;
					Windows::UI::Core::CoreWindow::GetForCurrentThread()->PointerCursor = FoxitCursorFactory::Current->GetNormalCursor();
				}
			}
		}
	}
	else
	{
		if (m_IsPressed && (m_IsShowSelHandleBtn || m_IsHitText))
		{
			Windows::Foundation::Point _Pt;
			auto _PointerPt = e->GetCurrentPoint(_FrameworkElement);
			_Pt = _PointerPt->Position;
			auto _Pageinfo = PDFViewCtrl->ConvertDevicePtToPDFPagePt(_Pt);
			int index = _Pageinfo->PageIndex;
			auto _PagePoint = _Pageinfo->PagePoint;

			m_pTextSelectContext->EndIndex = index;
			m_pTextSelectContext->EndPoint = _PagePoint;

			AddSelectText(m_pTextSelectContext->StartIndex, m_pTextSelectContext->StartPoint, m_pTextSelectContext->EndIndex, m_pTextSelectContext->EndPoint);
			PDFViewCtrl->Update();
		}
	}
	
}

void PDFTextManager::OnPointerEntered(Platform::Object^ sender, Windows::UI::Xaml::Input::PointerRoutedEventArgs^ e)
{

}

void PDFTextManager::OnPointerExited(Platform::Object^ sender, Windows::UI::Xaml::Input::PointerRoutedEventArgs^ e)
{
	if (m_PointerOverTextTimer && m_PointerOverTextTimer->IsEnabled)
		m_PointerOverTextTimer->Stop();
	m_IsPointerOverText = false;

	Windows::UI::Core::CoreWindow::GetForCurrentThread()->PointerCursor = FoxitCursorFactory::Current->GetNormalCursor();
}

void PDFTextManager::OnTapped(Platform::Object^ sender, Windows::UI::Xaml::Input::TappedRoutedEventArgs^ e)
{

}

void PDFTextManager::OnHolding(Platform::Object^ sender, Windows::UI::Xaml::Input::HoldingRoutedEventArgs^ e)
{
	Windows::UI::Xaml::FrameworkElement^ _FrameworkElement = dynamic_cast<Windows::UI::Xaml::FrameworkElement^>(sender);
	if (_FrameworkElement == nullptr)
		return;

	Windows::Foundation::Point _Pt;
	_Pt = e->GetPosition(_FrameworkElement);
	if (e->HoldingState == Windows::UI::Input::HoldingState::Started)
	{
		OutputDebugString(L"Windows::UI::Input::HoldingState::Started\n");
		auto _Pageinfo = PDFViewCtrl->ConvertDevicePtToPDFPagePt(_Pt);
		int index = _Pageinfo->PageIndex;
		auto _PagePoint = _Pageinfo->PagePoint;
		bool bIsHitText = IsHitText(index, _PagePoint);
		if (bIsHitText)
		{
			m_IsHitText = true;
			auto _Str = GetWordsByPoint(index, _PagePoint);
			m_IsShowSelHandleBtn = true;
			OutputDebugString(_Str->Data());
			OutputDebugString(L"\n");
			auto _SelectText = SelectedText;
			auto _size = SelectedText->Size;
			if (_size == 0)
				return;

			auto _pagerc = _SelectText->GetAt(0)->TextRectVect;
			int _startindex = _SelectText->GetAt(0)->PageIndex;
			if (_pagerc->Size == 0)
				return;

			auto __rcstart = _pagerc->GetAt(0);
			m_pTextSelectContext->StartIndex = _startindex;
			m_pTextSelectContext->StartPoint = Windows::Foundation::Point(__rcstart.Left, __rcstart.Bottom);
			auto _endpagerc = _SelectText->GetAt(_size - 1)->TextRectVect;
			int _endindex = _SelectText->GetAt(_size - 1)->PageIndex;
			auto _rcend = _endpagerc->Size > 0 ? _endpagerc->GetAt(_endpagerc->Size - 1) : Windows::Foundation::Rect(0, 0, 0, 0);
			m_pTextSelectContext->EndIndex = _endindex;
			m_pTextSelectContext->EndPoint = Windows::Foundation::Point(_rcend.Right, _rcend.Top);
			PDFViewCtrl->Update();
		}
		else
		{
			m_IsShowSelHandleBtn = false;

		}
	}
	else if (e->HoldingState == Windows::UI::Input::HoldingState::Completed)
	{
		OutputDebugString(L"Windows::UI::Input::HoldingState::Completed\n");
	}
	else if (e->HoldingState == Windows::UI::Input::HoldingState::Canceled)
	{
		OutputDebugString(L"Windows::UI::Input::HoldingState::Canceled\n");
	}
	
}

void PDFTextManager::OnSelHandleClick(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{
	Windows::UI::Xaml::FrameworkElement^ _Element = dynamic_cast<Windows::UI::Xaml::FrameworkElement^>(sender);
	m_IsHitStartSelHandleBtn = (_Element->Name == L"StartSelHandle");
	m_IsHitSelHandleBtn = true;

	if (UIExtMgr)
		UIExtMgr->ShowSelHandleButton(false);
}

void foxit::rdk::uiextensions::PDFTextManager::OnPointerOverTick(Platform::Object ^sender, Platform::Object ^args)
{
	m_PointerOverTextTimer->Stop();

	m_IsPointerOverText = true;

	Windows::UI::Core::CoreWindow::GetForCurrentThread()->PointerCursor = FoxitCursorFactory::Current->GetTextSelectCursor();

	OutputDebugString(L"OnPointerOverTick....\n");
}

IPageSelectText^ foxit::rdk::uiextensions::PDFTextManager::GetTextRect(int startindex, Windows::Foundation::Point startpt)
{
	PageSelectText^ selectText = ref new PageSelectText(startindex);

	Windows::Foundation::Rect rect = Windows::Foundation::Rect::Empty;
	bool isPreSepreator = false;
	bool isSepreator = false;

	std::wstringstream ws;

	int _StartCharIndex = -1;
	int _EndCharIndex = -1;
	auto pDoc = m_WeakfpdfDocument.Resolve<foxit::pdf::PDFDoc>();

	OnClearTextSelect();
	PageSelectText^ sel = ref new PageSelectText(startindex);

	foxit::pdf::TextPage^ pTextSelect;
	_StartCharIndex = 0;
	foxit::pdf::PDFPage^ pPage = pDoc->GetPage(startindex);
	pTextSelect = ref new foxit::pdf::TextPage(pPage, (int)foxit::pdf::TextParseFlags::e_ParseTextNormal);
	_EndCharIndex = pTextSelect->GetCharCount();
	if (_EndCharIndex == -1)
		return nullptr;
	_EndCharIndex -= 1;

	Platform::String^ tempstr = pTextSelect->GetChars(_StartCharIndex, _EndCharIndex - _StartCharIndex + 1);
	if (tempstr->Length() > 0)
	{
		ws << tempstr->Data();
	}

	int textrotate = 0;
	auto __RcCount = pTextSelect->GetTextRectCount(_StartCharIndex, _EndCharIndex - _StartCharIndex + 1);
	if (__RcCount == -1)
		return nullptr;
	for (int m = 0; m < __RcCount; m++)
	{
		int rotate = 0;
		rotate = (int)pTextSelect->GetBaselineRotation(m);
		sel->TextRotateVect->Append(rotate);
		Rect outrc;
		outrc = foxit::rdk::uiextensions::FoxitRectConverter::ConvertFromFoxitRectF(pTextSelect->GetTextRect(m));
		sel->TextRectVect->Append(outrc);
	}

	sel->_CharStart = _StartCharIndex;
	sel->_CharEnd = _EndCharIndex;

	if (startpt.X < rect.X + rect.Width / 2)
		isPreSepreator = true;
	else if(startpt.X > rect.X + rect.Width / 2)
		isPreSepreator = false;

	double strHeight = rect.Height;
	double strWidth = rect.Width;
	if (Windows::System::Profile::AnalyticsInfo::VersionInfo->DeviceFamily == "Windows.Desktop")
	{
		if (rect.Height > 15)
		{
			strHeight = strHeight*0.8;
		}
		else if (rect.Height < 6.0)
		{
			strHeight = 6;
		}
		strWidth = strHeight*(2.0 / 3.0);
	}
	else
	{
		if (rect.Height > 20)
		{
			strHeight = strHeight*0.8;
			strWidth = strHeight*(2.0 / 3.0);
		}
		else if (rect.Height <= 10.0)
		{
			if (rect.Height < 6.0)
				strHeight = 6.0;
			strWidth = strHeight*(2.0 / 3.0);
		}
		else if(rect.Height >10.0 && rect.Height<20)
		{			
			strWidth = strHeight*0.85;
		}
	}

	Windows::Foundation::Rect pagerect = Windows::Foundation::Rect((float)(rect.X + rect.Width - strWidth / 2.0), (float)(rect.Y - strHeight / 2.0), (float)strWidth, (float)strHeight);
	if (isPreSepreator)
	{
		pagerect = Windows::Foundation::Rect((float)(rect.X- strWidth / 2.0), (float)(rect.Y - strHeight / 2.0), (float)strWidth, (float)strHeight);
	}

	Rect deviceRect = PDFViewCtrl->ConvertPageRectToDeviceRect(startindex, rect);

	double hOffset =8 ;
	double vOffset = 12 ;

	rect.X = (float)(deviceRect.X + deviceRect.Width - hOffset / 2);
	rect.Y = (float)(deviceRect.Y + deviceRect.Height - deviceRect.Height / 2);
	rect.Width = (float)hOffset;
	rect.Height = (float)vOffset;

	int pageRotate = PDFViewCtrl->GetRotate();

	if (0 == pageRotate || 2 == pageRotate)
	{
		if (isPreSepreator)
		{
			rect.X = (float)(deviceRect.X - hOffset / 2);
			rect.Y = (float)(deviceRect.Y + deviceRect.Height - deviceRect.Height / 2);
		}
		else
		{
			rect.X = (float)(deviceRect.X + deviceRect.Width - (float)hOffset / 2);
			rect.Y = (float)(deviceRect.Y + deviceRect.Height - deviceRect.Height / 2);
		}
	}
	else
	{
		if (isPreSepreator)
		{
			rect.X = deviceRect.X - (float)vOffset / 2;
			rect.Y = deviceRect.Y - (float)hOffset / 2;
			rect.Width = (float)vOffset;
			rect.Height = (float)hOffset;
		}
		else
		{
			rect.X = deviceRect.X - (float)vOffset / 2;
			rect.Y = deviceRect.Y + deviceRect.Height - (float)hOffset / 2;
			rect.Width = (float)vOffset;
			rect.Height = (float)hOffset;
		}
	}
	selectText->TextRectVect->Append(pagerect);
	selectText->TextRotate = textrotate;
	return selectText;
}

bool foxit::rdk::uiextensions::PDFTextManager::IsSpaceCharacter(unsigned short unicode)
{
	if (unicode >= 0 && unicode <= 0x7E)
	{
		if (isspace((int)unicode))
		{
			return true;
		}
	}
	return false;
}

void PDFTextManager::ChangePageIndex(int srcIndex, int dstIndex)
{
	std::vector<std::shared_ptr<TextHandleContext>> _TmpList;
	int nStart = srcIndex < dstIndex ? srcIndex : dstIndex;
	int nEnd = srcIndex < dstIndex ? dstIndex : srcIndex;
	for (int i = nStart; i <= nEnd; i++)
	{
		if (m_PageHandleVect.count(i))
		{
			auto _SwapItem = m_PageHandleVect[i];
			m_PageHandleVect.erase(i);
			if (_SwapItem)
				_TmpList.push_back(_SwapItem);
		}
	}

	auto _TmpListSize = _TmpList.size();
	for (unsigned int i = 0; i < _TmpListSize; i++)
	{
		auto item = _TmpList[i];
		FS_INT32 index = -1;

		item->CheckPageIndex();
		index = item->GetPageIndex();

		m_PageHandleVect.insert(std::make_pair(index, item));
	}
}

void PDFTextManager::ChangePageIndex(Windows::Foundation::Collections::IVector<int>^ srcIndexs, int dstIndex)
{
	std::vector<std::shared_ptr<TextHandleContext>> _TmpList;
	int nStart = dstIndex;
	int nEnd = dstIndex;
	auto srcIndexSize = srcIndexs->Size;
	for (unsigned int i = 0; i < srcIndexSize; i++)
	{
		auto nItem = srcIndexs->GetAt(i);
		if (nStart > nItem)
			nStart = nItem;
		if (nEnd < nItem)
			nEnd = nItem;
	}

	for (int i = nStart; i <= nEnd; i++)
	{
		if (m_PageHandleVect.count(i))
		{
			auto _SwapItem = m_PageHandleVect[i];
			m_PageHandleVect.erase(i);
			if (_SwapItem)
				_TmpList.push_back(_SwapItem);
		}
	}

	auto _TmpListSize = _TmpList.size();
	for (unsigned int i = 0; i < _TmpListSize; i++)
	{
		auto item = _TmpList[i];
		FS_INT32 index = -1;

		item->CheckPageIndex();
		index = item->GetPageIndex();
		
		m_PageHandleVect.insert(std::make_pair(index, item));
	}
}

void PDFTextManager::PageDeleteBefore(Windows::Foundation::Collections::IVector<int>^ pageIndexs)
{
	auto srcIndexSize = pageIndexs->Size;
	for (unsigned int i = 0; i < srcIndexSize; i++)
	{
		auto pageIndex = pageIndexs->GetAt(i);
		if (m_PageHandleVect.count(pageIndex))
		{
			m_PageHandleVect.erase(pageIndex);
		}
	}
}

void PDFTextManager::PageDeleteAfter(Windows::Foundation::Collections::IVector<int>^ pageIndexs)
{
	std::vector<std::shared_ptr<TextHandleContext>> _TmpList;
	int _PageCount = 0;

	if (m_WeakfpdfDocument)
	{
		auto pPDFData = m_WeakfpdfDocument.Resolve<foxit::pdf::PDFDoc>();
		_PageCount = pPDFData->GetPageCount();
	}

	int nStart = _PageCount;
	int nEnd = _PageCount;

	auto srcIndexSize = pageIndexs->Size;
	for (unsigned int i = 0; i < srcIndexSize; i++)
	{
		auto nItem = pageIndexs->GetAt(i);
		if (nStart > nItem)
			nStart = nItem;
	}

	for (int i = nStart; i <= nEnd; i++)
	{
		if (m_PageHandleVect.count(i))
		{
			auto _SwapItem = m_PageHandleVect[i];
			m_PageHandleVect.erase(i);
			if (_SwapItem)
				_TmpList.push_back(_SwapItem);
		}
	}

	auto _TmpListSize = _TmpList.size();
	for (unsigned int i = 0; i < _TmpListSize; i++)
	{
		auto item = _TmpList[i];
		FS_INT32 index = -1;

		item->CheckPageIndex();
		index = item->GetPageIndex();

		m_PageHandleVect.insert(std::make_pair(index, item));
	}
}

void PDFTextManager::CheckPageIndex(int startIndex)
{
	std::vector<std::shared_ptr<TextHandleContext>> _TmpList;
	int _PageCount = 0;

	if (m_WeakfpdfDocument)
	{
		auto pPDFData = m_WeakfpdfDocument.Resolve<foxit::pdf::PDFDoc>();
		_PageCount = pPDFData->GetPageCount();
	}

	int nStart = startIndex;
	int nEnd = _PageCount;
	for (int i = nStart; i < nEnd; i++)
	{
		if (m_PageHandleVect.count(i))
		{
			auto _SwapItem = m_PageHandleVect[i];
			m_PageHandleVect.erase(i);
			if (_SwapItem)
				_TmpList.push_back(_SwapItem);
		}
	}

	auto _TmpListSize = _TmpList.size();
	for (unsigned int i = 0; i < _TmpListSize; i++)
	{
		auto item = _TmpList[i];
		FS_INT32 index = -1;

		item->CheckPageIndex();
		index = item->GetPageIndex();
		
		m_PageHandleVect.insert(std::make_pair(index, item));
	}
}

void PDFTextManager::GotoExactlyPage(SingleSearchOnePage^ pageinfo)
{
	if (pageinfo)
	{
		auto _PageIndex = pageinfo->OnePageData->PageIndex;
		auto _PrevPageIndex = PDFViewCtrl->GetCurrentPage();

		if (_PrevPageIndex != _PageIndex)
		{
			PDFViewCtrl->GotoPage(_PageIndex);
			UIExtMgr->InitSliderPageNumber(_PageIndex);
		}
		auto _vect = pageinfo->OnePageData->OnePageData;
		auto _vectSize = _vect->Size;
		if (pageinfo->CurrentIndexInPageData >= 0 && pageinfo->CurrentIndexInPageData < (int)_vectSize)
		{
			auto _item = _vect->GetAt(pageinfo->CurrentIndexInPageData);
			if (_item == nullptr)
				return;
			if (_item->RectVect->Size <= 0)
				return;
			auto _rc = _item->RectVect->GetAt(0);
			PDFViewCtrl->GotoPagePoint(_PageIndex, Windows::Foundation::Point(_rc.X, _rc.Y));
		}
	}
}
