/**
 * Copyright (C) 2003-2018, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit Mobile PDF SDK to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */


#pragma once

namespace foxit {
	namespace rdk {
		namespace uiextensions {
			class CritSec;
			interface class IPDFTextManager;
			interface class IPDFTextManagerOverrides;
			interface class ISearchOnePage;
			interface class ITextSearchDataItem;
			ref class TextSearchDataItem;
			ref class UIExtensionsManager;
		}
	}
}

namespace foxit
{
	namespace rdk
	{
		namespace uiextensions
		{
			public ref class SearchOnePage sealed : public ISearchOnePage
			{
			public:
				SearchOnePage(int pageindex) : _PageIndex(pageindex)
				{
					_OnePageData = ref new Platform::Collections::Vector<ITextSearchDataItem^>();
				}
				property int PageIndex
				{
					virtual int get()
					{
						return _PageIndex;
					}
				}
				property Windows::Foundation::Collections::IVector<ITextSearchDataItem^>^ OnePageData
				{
					virtual Windows::Foundation::Collections::IVector<ITextSearchDataItem^>^ get()
					{
						return _OnePageData;
					}
				}

			private:
				Platform::Collections::Vector<ITextSearchDataItem^>^ _OnePageData;
				int _PageIndex;
			};

			class TextHandleContext
			{
			public:
				TextHandleContext(int pageindex, foxit::pdf::PDFDoc^ doc);

				bool LoadTextPageHandle();
				void ReleaseTextHandle();
				void LoadTextPageRects();
				bool IsValid();
				void AddRef();
				void RemoveRef();
				static void ReleaseTextHandleContext(TextHandleContext* val);
				bool IsPageHandleInitByExternalModule;
				bool IsInit;
				foxit::pdf::PDFPage^ pageHandle;
				foxit::pdf::TextPage^ textpageHandle;
				bool IsInitTextRect;
				Platform::Array<Windows::Foundation::Rect>^ TextRect;
				int GetPageIndex();
				void CheckPageIndex();

			private:
				int m_PageIndex;
				Platform::WeakReference m_DocumentWeakRef;
				int m_TextPageRefCount;
			};

			public ref class TextSelectContext sealed
			{
			public:
				TextSelectContext() {};

			public:
				property int SelectMode;
				property int StartIndex;
				property Windows::Foundation::Point StartPoint;
				property int EndIndex;
				property Windows::Foundation::Point EndPoint;

				void ResetContext();
			};

			[Windows::Foundation::Metadata::WebHostHidden]
			public ref class PDFTextManager sealed : public IPDFTextManager
			{
				ref class TextSelectOptContext sealed
				{
				public:
					TextSelectOptContext() {};

				public:
					property int StartIndex;
					property int StartCharIndex;
					property int EndIndex;
					property int EndCharIndex;
				};

			public:
				PDFTextManager(foxit::rdk::PDFViewCtrl^  viewControl);
				virtual ~PDFTextManager();

			public:
				property Platform::String^ FindTextString
				{
					virtual Platform::String^ get()
					{
						return m_SearchTextString;
					}
				}
				virtual bool GetIsSingleSearching();
				virtual bool GetIsFoundText();
				virtual void StartSearchString(int pageindex, Platform::String^ pwsFindwhat);
				virtual void StopSearchString();
				virtual void SearchPrev();
				virtual void SearchNext();
				virtual void NotifyByTextSearchDataItem(ITextSearchDataItem^ item);
				property SingleSearchOnePage^ CurSearchTextInfo
				{
					virtual SingleSearchOnePage^ get()
					{
						return m_CurSearchOnePage;
					}
				}
				property Windows::Foundation::Collections::IObservableVector<ITextSearchDataItem^>^ SearchData
				{
					virtual Windows::Foundation::Collections::IObservableVector<ITextSearchDataItem^>^ get()
					{
						return _SearchData;
					}
				}
				virtual event SearchResultEventHandler^ SearchResult;

				virtual bool IsInitText(int pageindex);
				virtual Platform::String^ GetTextByPosition(int pageindex, int start, int count);
				virtual Platform::String^ GetTextByBounded(int pageindex, Windows::Foundation::Rect rect);
				virtual Platform::String^ GetTextByBoundedSegments(int pageindex, Windows::Foundation::Rect rect);
				virtual Platform::String^ GetWordsByPoint(int pageindex, Windows::Foundation::Point point);
				virtual bool IsHitText(int pageindex, Windows::Foundation::Point pt);
				virtual bool IsHitSelectedText(int pageindex, Windows::Foundation::Point pt);
				virtual bool AddSelectText(int startindex, Windows::Foundation::Point startpt, int endindex, Windows::Foundation::Point endpt);
				property Platform::String^ SelectedTextString
				{
					virtual Platform::String^ get()
					{
						return _SelectedTextString;
					}
				}
				property Windows::Foundation::Collections::IObservableVector<IPageSelectText^>^ SelectedText
				{
					virtual Windows::Foundation::Collections::IObservableVector<IPageSelectText^>^ get()
					{
						return _SelectedText;
					}
				}
				property double TextSelectTolerance
				{
					virtual double get();
					virtual void set(double value);
				}
				virtual event TextSelectChangedEventHandler^ TextSelectChanged;

				virtual void SetOption(ITextSelectOption^ opt);
				ITextSelectOption^ GetOption();
				void UpdateTextSelectOption();

				virtual bool OnCreate(foxit::rdk::PDFViewCtrl^ pageview);
				virtual bool OnLoaded();
				virtual void OnDelete();
				virtual property foxit::rdk::PDFViewCtrl^ PDFViewCtrl
				{
					virtual foxit::rdk::PDFViewCtrl^ get();
				}

				virtual bool LoadPdfTextResource(int pageindex);
				virtual Platform::Array<Windows::Foundation::Rect>^ GetPageCoordinateRects(int pageindex);

				virtual foxit::pdf::PDFPage^ GetPageHandle(int pageindex);
				virtual bool IsResourcesReleased();
				virtual void ReleaseExtraResources();
				virtual void ForceReleaseResources();
				virtual void ClearTextSelect();
				virtual Windows::Foundation::Rect GetHitTextRect(int pageindex, Windows::Foundation::Point pt);
				virtual IPageSelectText^ GetTextRect(int startindex, Windows::Foundation::Point startpt);

			private:
				void AddTextSelect(Windows::Foundation::Collections::IVector<IPageSelectText^>^ pageTextVect);
				bool InitPageTextArry(int pageindex);
				bool PointInRectangleByPageCoordiation(Windows::Foundation::Point pt, Windows::Foundation::Rect rc);

			private:
				Windows::Foundation::IAsyncOperationWithProgress<int, SearchOnePage^>^ GetSearchResultAsync(Platform::String^ pwsFindwhat);
				void Lock();
				void Unlock();

				bool LoadPageHandle(int pageIndex, foxit::pdf::PDFPage^ pPage = nullptr, foxit::pdf::TextPage^ pTextPage = nullptr);
				std::shared_ptr<TextHandleContext> LoadPageHandle_Sp(int pageIndex);
				bool SearchLoadHandle(int pageIndex, foxit::pdf::PDFPage^ pPage, foxit::pdf::TextPage^ pTextPage, foxit::pdf::TextSearch^& pSearchHandle, Platform::String^ pwsFindwhat, int findOption = 0, int findMode = 0);
				Platform::String^ GetShowTextFromPage(int page, int count, int mode);
				bool GetSearchItemData(int pageIndex, foxit::pdf::TextPage^ pTextPage, foxit::pdf::TextSearch^ pSeachHandle, TextSearchDataItem^ item, int findMode = 0);
				void GetSearchItemContextString(int pageIndex, foxit::pdf::TextPage^ pTextPage, TextSearchDataItem^ item);
				bool SearchCheckStringValid(Platform::String^ findstr);

				bool IsHasFinded(int pageindex);
				Concurrency::task<SingleSearchOnePage^> SingleSearchString();
				void NotifyToView(SingleSearchOnePage^ data);

			private:
				std::shared_ptr<CritSec> m_Critsec;
				int	m_LockCount;
				bool m_bPopupMenu;
				Platform::WeakReference m_WeakfpdfDocument;
				Platform::WeakReference m_PageViewWeakRef;
				Platform::WeakReference m_UIViewWeakRef;
				std::map<int, std::shared_ptr<TextHandleContext>> m_PageHandleVect;
				HANDLE m_EventCancelFind;
				Windows::Foundation::IAsyncOperationWithProgress<int, SearchOnePage^>^ m_AsyncOperation;

				/// Whether released resources
				bool m_IsHadReleasedResources;
				/// Whether mandatory to release resources
				bool m_IsForceReleaseResources;
				// Whether end of search
				bool m_IsSearching;
				// Pages already searched
				std::vector<int> m_HasSearchedPage;
				std::map<int, ISearchOnePage^> m_SearchOnePageData;
				std::shared_ptr<CritSec> m_MapCritsec;
				SingleSearchOnePage^ m_CurSearchOnePage;

				Platform::String^ m_SearchTextString;
				int m_CurSearchIndex;
				foxit::pdf::PDFPage^ m_pCurPageHandle;
				foxit::pdf::TextPage^ m_pCurTextPageHandle;
				foxit::pdf::TextSearch^ m_pCurSeachHandle;
				bool m_IsSingleSearching;
				int m_SearchMode;
				Concurrency::cancellation_token_source m_SingleSearchCts;
				bool m_IsFound;

				ITextSelectOption^ m_Option;
				std::vector<int> m_RefPageIndexVector;
				std::vector<int> m_PrevOpraRefPageIndexVector;
				std::shared_ptr<CritSec> m_PageIndexVectorCritsec;

				Windows::Foundation::Collections::IObservableVector<ITextSearchDataItem^>^ _SearchData;
				Windows::Foundation::Collections::IObservableVector<IPageSelectText^>^ _SelectedText;
				Platform::String^ _SelectedTextString;
				double _TextSelectTolerance;
				Windows::Foundation::EventRegistrationToken m_textRenderToken;
			private:
				void UpdateSelectContext();

			private:
				bool m_IsHitText;
				bool m_IsPressed;
				TextSelectOptContext^m_pTextSelectOptContext;
				TextSelectContext^ m_pTextSelectContext;
				bool m_IsShowSelHandleBtn;
				bool m_IsHitSelHandleBtn;
				bool m_IsHitStartSelHandleBtn;

				bool m_IsPointerOverText;
				Windows::UI::Xaml::DispatcherTimer^ m_PointerOverTextTimer;

			public:
				virtual void OnRender(foxit::rdk::PDFViewCtrl^ sender, foxit::rdk::RenderCanvasEventArgs^ e);
				virtual void OnPointerPressed(Platform::Object^ sender, Windows::UI::Xaml::Input::PointerRoutedEventArgs^ e);
				virtual void OnPointerReleased(Platform::Object^ sender, Windows::UI::Xaml::Input::PointerRoutedEventArgs^ e);
				virtual void OnPointerMoved(Platform::Object^ sender, Windows::UI::Xaml::Input::PointerRoutedEventArgs^ e);
				virtual void OnPointerEntered(Platform::Object^ sender, Windows::UI::Xaml::Input::PointerRoutedEventArgs^ e);
				virtual void OnPointerExited(Platform::Object^ sender, Windows::UI::Xaml::Input::PointerRoutedEventArgs^ e);
				virtual void OnTapped(Platform::Object^ sender, Windows::UI::Xaml::Input::TappedRoutedEventArgs^ e);
				virtual void OnHolding(Platform::Object^ sender, Windows::UI::Xaml::Input::HoldingRoutedEventArgs^ e);
				virtual void OnSelHandleClick(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e);
				virtual bool GetIsShowSelHandleBtn();
				virtual void SetIsShowSelHandleBtn(bool bIsShow);

			public:
				void ClearSelectedText();
				property UIExtensionsManager^ UIExtMgr
				{
					UIExtensionsManager^ get();
				}

			private:
				void OutputCompositeTransformInfo(Windows::UI::Xaml::FrameworkElement^ framework);
				bool IsSelectedPageIndex(int pageindex, unsigned int* index);
				void OnClearTextSelect();

				bool IsHadRefPageIndex(int pageindex);
				bool IsPrevRefPageIndex(int pageindex);
				void AddPageIndexRef(int pageindex);
				void RemovePageIndexRef(int pageindex);
				void AddPageIndexRef(int startindex, int endindex);
				void RemovePageIndexRef(int startindex, int endindex);

			private:
				Platform::String^ StandardizationDealWithString(Platform::String^ source);
				Platform::String^ LimitDealWithString(Platform::String^ source);
				void OnPointerOverTick(Platform::Object ^sender, Platform::Object ^args);
				bool IsSpaceCharacter(unsigned short unicode);

			public:
				virtual void ChangePageIndex(int srcIndex, int dstIndex);
				virtual void ChangePageIndex(Windows::Foundation::Collections::IVector<int>^ srcIndexs, int dstIndex);
				virtual void PageDeleteBefore(Windows::Foundation::Collections::IVector<int>^ pageIndexs);
				virtual void PageDeleteAfter(Windows::Foundation::Collections::IVector<int>^ pageIndexs);
				virtual void CheckPageIndex(int startIndex);
				void GotoExactlyPage(SingleSearchOnePage^ pageinfo);
			};
		}
	}
}
