﻿/**
 * Copyright (C) 2003-2018, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit Mobile PDF SDK to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */


#include "pch.h"
#include "ExtractPageControl.xaml.h"
#include "ThumbnailPage.xaml.h"

#include <regex>

using namespace foxit::rdk::uiextensions;

using namespace Platform;
using namespace Windows::Foundation;
using namespace Windows::Foundation::Collections;
using namespace Windows::UI::Xaml;
using namespace Windows::UI::Xaml::Controls;
using namespace Windows::UI::Xaml::Controls::Primitives;
using namespace Windows::UI::Xaml::Data;
using namespace Windows::UI::Xaml::Input;
using namespace Windows::UI::Xaml::Media;
using namespace Windows::UI::Xaml::Navigation;

// The User Control item template is documented at http://go.microsoft.com/fwlink/?LinkId=234236

ExtractPageControl::ExtractPageControl()
{
	InitializeComponent();

	OnInitializeControl();
}

void ExtractPageControl::OnInitializeControl()
{
	auto _ObjValue = this->Resources->Lookup(L"LayoutRootGridMinWidth");
	m_LayoutRootGridMinWidth = _ObjValue && _ObjValue->GetType()->FullName == double::typeid->FullName ? dynamic_cast<Platform::IBox<double>^>(_ObjValue)->Value : 400;

	m_ExtractPageIndexVect = ref new Platform::Collections::Vector<int>();

	this->Loaded += ref new Windows::UI::Xaml::RoutedEventHandler(this, &foxit::rdk::uiextensions::ExtractPageControl::OnLoaded);
	this->Unloaded += ref new Windows::UI::Xaml::RoutedEventHandler(this, &foxit::rdk::uiextensions::ExtractPageControl::OnUnloaded);
}

ThumbnailPage^ ExtractPageControl::HostPage::get()
{
	return m_HostPageWeakRef.Resolve<ThumbnailPage>();
}

void ExtractPageControl::SetHostedPage(ThumbnailPage^ page)
{
	if (page)
	{
		m_HostPageWeakRef = page;

		UpdatePageRangeTextBox();
	}
}

void ExtractPageControl::UpdateControlState()
{

}

void ExtractPageControl::UpdatePageRangeTextBox()
{
	if (HostPage == nullptr)
		return;

	auto _pItemsGridView = HostPage->GetItemsGridView();
	auto _SelectedItem = _pItemsGridView->SelectedItems;

	ExtractPageControlPageRangeTextBox->Text = GetPageRangeStringFromHostPage();
	ExtractPageControlPageCountTextBlock->Text = HostPage->Viewer->GetDoc()->GetPageCount().ToString();
	ExtractPageControlFileNameTextBox->Text = L"Extract pages from " + HostPage->Viewer->GetStorageFile()->Name;
}

Platform::String^ ExtractPageControl::GetPageRangeStringFromHostPage()
{
	Platform::String^ _Str = L"";
	if (HostPage == nullptr)
		return _Str;

	auto _pItemsGridView = HostPage->GetItemsGridView();
	auto _SelectedItems = _pItemsGridView->SelectedItems;
	auto _Size = _SelectedItems->Size;
	if (_Size == 0)
		return _Str;

	auto _PageIndexs = ref new Platform::Collections::Vector<int>();
	for (unsigned int i = 0; i < _Size; i++)
	{
		auto srcItem = dynamic_cast<ThumbnailItem^>(_SelectedItems->GetAt(i));
		_PageIndexs->Append(srcItem->PageIndex - 1);
	}

	return GetPageRangeString(_PageIndexs);
}

Platform::String^ ExtractPageControl::GetPageRangeString(Windows::Foundation::Collections::IVector<int>^ pageIndexVect)
{
	Platform::String^ _Str = L"";

	auto _Size = pageIndexVect->Size;
	if (_Size == 0)
		return _Str;

	Platform::Array<int>^ _TmpArray = ref new Platform::Array<int>(pageIndexVect->Size);
	pageIndexVect->GetMany(0, _TmpArray);

	std::vector<int> _pageIndexVect(_TmpArray->begin(), _TmpArray->end());
	std::sort(_pageIndexVect.begin(), _pageIndexVect.end(), std::less<int>());
	auto _pageIndexs = ref new Platform::Collections::Vector<int>(_pageIndexVect);
	auto pageIndexsSize = _pageIndexs->Size;
	int nStartIndex = -1;
	
	return _Str;
}

bool ExtractPageControl::ParsePageRangeString(Platform::String^ pageRangeStr, Windows::Foundation::Collections::IVector<int>^ pageIndexVect)
{
	auto bIsMatch = std::regex_match(pageRangeStr->Data(), std::wregex(L"^[0-9]+[0-9\\-\\,]*"));
	if (!bIsMatch)
		return false;

	auto nPageCount = HostPage->Viewer->GetDoc()->GetPageCount();
	bool bHaveError = false;
	auto dtData = pageRangeStr->Data();
	auto length = wcslen(dtData) + 1;
	wchar_t* data = new wchar_t[length];

	wcscpy_s(data, length, dtData);

	wchar_t* pszContext = NULL;
	wchar_t* pszToken = data;
	wchar_t* pSeps = L",";

	pszToken = wcstok_s(data, pSeps, &pszContext);
	while (pszToken != NULL)
	{
		wchar_t* pszNextContext = NULL;
		auto pNexrToken = wcstok_s(pszToken, L"-", &pszNextContext);
		if (pNexrToken == NULL)
		{
			bHaveError = true;
			break;
		}

		int nStartIndex = _wtoi(pNexrToken);
		int nEndIndex = _wtoi(pszNextContext);

		if (nStartIndex > 0)
		{
			if (nStartIndex > nPageCount || nEndIndex > nPageCount)
			{
				bHaveError = true;
				break;
			}

			if (nEndIndex != 0)
			{
				if (nEndIndex > nStartIndex)
				{
					for (int i = nStartIndex; i <= nEndIndex; i++)
					{
						pageIndexVect->Append(i - 1);
					}
				}
				else
				{
					bHaveError = true;
					break;
				}
			}
			else
			{
				pageIndexVect->Append(nStartIndex - 1);
			}
		}
		else
		{
			bHaveError = true;
			break;
		}

		pszToken = wcstok_s(NULL, pSeps, &pszContext);
	}

	return !bHaveError;
}

bool ExtractPageControl::IsValidPdfFileName(Platform::String^ fileName)
{
	const wchar_t *seperator = wcsrchr(fileName->Data(), L'.');

	if (seperator == nullptr)
	{
		return false;
	}

	_wcslwr_s((wchar_t*)seperator, wcslen(seperator) + 1);

	return wcscmp(seperator, L".pdf") == 0;
}

void ExtractPageControl::UpdateControlPosition()
{
	auto _frameworkElement = dynamic_cast<UIElement^>(this->Parent);
	if (_frameworkElement == nullptr)
		return;

	if (m_bIsShowInputPane)
	{
		auto _inputPane = Windows::UI::ViewManagement::InputPane::GetForCurrentView();
		auto _inputPaneRect = _inputPane->OccludedRect;
		auto _winRect = Windows::UI::Xaml::Window::Current->Bounds;

		if (m_InputPaneTranslateTransform)
		{
			_frameworkElement->RenderTransform = ref new Windows::UI::Xaml::Media::TranslateTransform();
			m_InputPaneTranslateTransform = nullptr;
		}

		auto _focusElement = dynamic_cast<Windows::UI::Xaml::FrameworkElement^>(Windows::UI::Xaml::Input::FocusManager::GetFocusedElement());
		if (_focusElement == nullptr)
			return;

		Windows::Foundation::Point _screenPt = _focusElement->TransformToVisual(nullptr)->TransformPoint(Point(0, 0));
		auto _focusElementRect = Windows::Foundation::Rect(_screenPt, Windows::Foundation::Size((float)_focusElement->ActualWidth, (float)_focusElement->ActualHeight));

#if (_DEBUG && 1)
		std::wstringstream ws;
		ws << L"InputPane OccludedRect: " << _inputPaneRect.Left << L", " << _inputPaneRect.Top << L", " << _inputPaneRect.Width << L", " << _inputPaneRect.Height << L"\n";
		ws << L"FocusElement: " << _focusElement->GetType()->FullName->Data() << L", Point: " << _screenPt.X << L", " << _screenPt.Y << L", " << _focusElement->ActualWidth << L", " << _focusElement->ActualHeight;
		ws << L"\n";
		OutputDebugString(ws.str().c_str());
#endif

		if (_inputPaneRect.IntersectsWith(_focusElementRect))
		{
			OutputDebugString(L"IntersectsWith...\n");
			double _offsetY = _screenPt.Y + _focusElement->ActualHeight - _inputPaneRect.Top;
			auto _ScrollViewer = foxit::rdk::uiextensions::GetVisualParent<Windows::UI::Xaml::Controls::ScrollViewer^>(_focusElement);
			if (_ScrollViewer && _ScrollViewer->VerticalScrollMode != Windows::UI::Xaml::Controls::ScrollMode::Disabled)
			{
				auto _ScrollableHeight = _ScrollViewer->ScrollableHeight;
				auto _CurVerticalOffset = _ScrollViewer->VerticalOffset;
				auto _CanScrollOffsetY = _ScrollViewer->ScrollableHeight - _CurVerticalOffset;
				if (_CanScrollOffsetY >= _offsetY)
				{
					_ScrollViewer->ChangeView(_ScrollViewer->HorizontalOffset, _ScrollViewer->VerticalOffset + _offsetY, _ScrollViewer->ZoomFactor);
					return;
				}
				else
				{
					_ScrollViewer->ChangeView(_ScrollViewer->HorizontalOffset, _ScrollViewer->VerticalOffset + _CanScrollOffsetY, _ScrollViewer->ZoomFactor);
					_offsetY -= _CanScrollOffsetY;
				}
			}

			m_InputPaneTranslateTransform = ref new Windows::UI::Xaml::Media::TranslateTransform();
			m_InputPaneTranslateTransform->Y -= _offsetY;
			_frameworkElement->RenderTransform = m_InputPaneTranslateTransform;
		}
	}
	else
	{
		if (m_InputPaneTranslateTransform)
		{
			_frameworkElement->RenderTransform = ref new Windows::UI::Xaml::Media::TranslateTransform();
			m_InputPaneTranslateTransform = nullptr;
		}
	}
}

void ExtractPageControl::ClosePopup()
{
	auto _PopupDialog = GetParent<ContentHostedDialog^>(this->Parent);
	if (_PopupDialog)
	{
		_PopupDialog->Hide();
	}
}

void ExtractPageControl::ResizeLayoutRoot()
{
	auto _VisibleBounds = Windows::UI::ViewManagement::ApplicationView::GetForCurrentView()->VisibleBounds;
	auto _ActualWidth = this->ActualWidth;

	if (_ActualWidth > _VisibleBounds.Width)
	{
		this->MinWidth = _VisibleBounds.Width;
		this->MaxWidth = _VisibleBounds.Width;
		this->Width = _VisibleBounds.Width;
	}
	else
	{
		if (m_LayoutRootGridMinWidth > _VisibleBounds.Width)
		{
			this->MinWidth = _VisibleBounds.Width;
			this->MaxWidth = _VisibleBounds.Width;
			this->Width = _VisibleBounds.Width;
		}
		else
		{
			this->MinWidth = m_LayoutRootGridMinWidth;
			this->MaxWidth = m_LayoutRootGridMinWidth;
			this->Width = m_LayoutRootGridMinWidth;
		}
	}
}

void ExtractPageControl::ShowNoneErrorTips()
{
	Windows::UI::Xaml::VisualStateManager::GoToState(this, L"NoneErrorTips", true);
}

void ExtractPageControl::ShowInvalidRangeErrorTips()
{
	Windows::UI::Xaml::VisualStateManager::GoToState(this, L"InvalidRangeErrorTips", true);
}

void ExtractPageControl::ShowInvalidFolderErrorTips()
{
	Windows::UI::Xaml::VisualStateManager::GoToState(this, L"InvalidFolderErrorTips", true);
}

void foxit::rdk::uiextensions::ExtractPageControl::OnLoaded(Platform::Object ^sender, Windows::UI::Xaml::RoutedEventArgs ^e)
{
	auto _InputPane = Windows::UI::ViewManagement::InputPane::GetForCurrentView();
	m_InputPaneShowingEventToken = _InputPane->Showing += ref new Windows::Foundation::TypedEventHandler<Windows::UI::ViewManagement::InputPane ^, Windows::UI::ViewManagement::InputPaneVisibilityEventArgs ^>(this, &foxit::rdk::uiextensions::ExtractPageControl::OnInputPaneShowing);
	m_InputPaneHidingEventToken = _InputPane->Hiding += ref new Windows::Foundation::TypedEventHandler<Windows::UI::ViewManagement::InputPane ^, Windows::UI::ViewManagement::InputPaneVisibilityEventArgs ^>(this, &foxit::rdk::uiextensions::ExtractPageControl::OnInputPaneHiding);

	m_WindowSizeChangedEventToken = Windows::UI::Xaml::Window::Current->SizeChanged += ref new Windows::UI::Xaml::WindowSizeChangedEventHandler(this, &foxit::rdk::uiextensions::ExtractPageControl::OnWindowSizeChanged);

	UpdateControlState();

	ResizeLayoutRoot();
}


void foxit::rdk::uiextensions::ExtractPageControl::OnUnloaded(Platform::Object ^sender, Windows::UI::Xaml::RoutedEventArgs ^e)
{
	auto _InputPane = Windows::UI::ViewManagement::InputPane::GetForCurrentView();
	_InputPane->Showing -= m_InputPaneShowingEventToken;
	_InputPane->Hiding -= m_InputPaneHidingEventToken;

	Windows::UI::Xaml::Window::Current->SizeChanged -= m_WindowSizeChangedEventToken;
}

void foxit::rdk::uiextensions::ExtractPageControl::OnInputPaneShowing(Windows::UI::ViewManagement::InputPane ^sender, Windows::UI::ViewManagement::InputPaneVisibilityEventArgs ^args)
{
	m_PrevInputPaneEnsuredFocusedElementInView = args->EnsuredFocusedElementInView;
	args->EnsuredFocusedElementInView = true;
	m_bIsShowInputPane = true;

	UpdateControlPosition();
}


void foxit::rdk::uiextensions::ExtractPageControl::OnInputPaneHiding(Windows::UI::ViewManagement::InputPane ^sender, Windows::UI::ViewManagement::InputPaneVisibilityEventArgs ^args)
{
	args->EnsuredFocusedElementInView = m_PrevInputPaneEnsuredFocusedElementInView;
	m_bIsShowInputPane = false;

	UpdateControlPosition();
}


void foxit::rdk::uiextensions::ExtractPageControl::OnWindowSizeChanged(Platform::Object ^sender, Windows::UI::Core::WindowSizeChangedEventArgs ^e)
{
	ResizeLayoutRoot();
}


void foxit::rdk::uiextensions::ExtractPageControl::ClosePopupButton_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{
	ClosePopup();
}


void foxit::rdk::uiextensions::ExtractPageControl::ExtractPageControlSureButton_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{
	ShowNoneErrorTips();

	Platform::Collections::Vector<int>^ indexVect = ref new Platform::Collections::Vector<int>();
	auto bIsOk = ParsePageRangeString(ExtractPageControlPageRangeTextBox->Text, indexVect);
	auto bIsValid = IsValidPdfFileName(ExtractPageControlFileNameTextBox->Text);
	auto indexVectSize = indexVect->Size;
	if (!bIsOk)
	{
		ShowInvalidRangeErrorTips();
		return;
	}

	m_ExtractPageIndexVect = indexVect;

	if (m_LocateStorageFolder == nullptr)
	{
		ShowInvalidFolderErrorTips();
		return;
	}

	if (!bIsValid)
	{
		ExtractPageControlFileNameTextBox->Text = ExtractPageControlFileNameTextBox->Text + L".pdf";
	}
}


void foxit::rdk::uiextensions::ExtractPageControl::ExtractPageControlCancelButton_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{
	ClosePopup();
}


void foxit::rdk::uiextensions::ExtractPageControl::ExtractPageControlPageRangeTextBox_TextChanged(Platform::Object^ sender, Windows::UI::Xaml::Controls::TextChangedEventArgs^ e)
{
	
}


void foxit::rdk::uiextensions::ExtractPageControl::ExtractPageControlBrowserButton_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{
	Windows::Storage::Pickers::FolderPicker^ _FolderPicker = ref new Windows::Storage::Pickers::FolderPicker();
	auto _SettingsIdentifier = _FolderPicker->SettingsIdentifier;
	_FolderPicker->SuggestedStartLocation = Windows::Storage::Pickers::PickerLocationId::DocumentsLibrary;
	_FolderPicker->ViewMode = Windows::Storage::Pickers::PickerViewMode::Thumbnail;
	_FolderPicker->FileTypeFilter->Append(L".pdf");

	concurrency::create_task(_FolderPicker->PickSingleFolderAsync())
		.then([=](Windows::Storage::StorageFolder^ folder)
	{
		if (folder == nullptr)
			throw ref new Platform::NullReferenceException();

		m_LocateStorageFolder = folder;
		ExtractPageControlOuputFolderPathTextBlock->Text = folder->Path;

		return false;
	})
		.then([=](concurrency::task<bool> t)
	{
		try
		{
			t.get();
		}
		catch (...)
		{

		}
	});
}
