﻿/**
 * Copyright (C) 2003-2018, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit Mobile PDF SDK to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */


#include "pch.h"
#include "InsertPageControl.xaml.h"
#include "ThumbnailPage.xaml.h"

using namespace foxit::rdk::uiextensions;

using namespace Platform;
using namespace Windows::Foundation;
using namespace Windows::Foundation::Collections;
using namespace Windows::UI::Xaml;
using namespace Windows::UI::Xaml::Controls;
using namespace Windows::UI::Xaml::Controls::Primitives;
using namespace Windows::UI::Xaml::Data;
using namespace Windows::UI::Xaml::Input;
using namespace Windows::UI::Xaml::Media;
using namespace Windows::UI::Xaml::Navigation;

// To do...

// The User Control item template is documented at http://go.microsoft.com/fwlink/?LinkId=234236
InsertTypeItem::InsertTypeItem(InsertType type) : m_TypeOfInsert(type)
{

}

Platform::String^ InsertTypeItem::DisplayString::get()
{
	return GetDisplayString(m_TypeOfInsert);
}

InsertType InsertTypeItem::TypeOfInsert::get()
{
	return m_TypeOfInsert;
}

Platform::String^ InsertTypeItem::GetDisplayString(InsertType type)
{
	Platform::String^ _Str = L"";
	auto _ResourceLoader = Windows::ApplicationModel::Resources::ResourceLoader::GetForCurrentView("foxit.rdk.uiextensions/Resources");

	switch (type)
	{
	case foxit::rdk::uiextensions::InsertType::BlankPage:
		_Str = _ResourceLoader->GetString(L"InsertTypeBlankPageText");
		break;
	case foxit::rdk::uiextensions::InsertType::PdfFile:
		_Str = _ResourceLoader->GetString(L"InsertTypePdfFileText");
		break;
	case foxit::rdk::uiextensions::InsertType::ImageFile:
		_Str = _ResourceLoader->GetString(L"InsertTypeImageText");
		break;
	default:
		break;
	}

	return _Str;
}

InsertPositionTypeItem::InsertPositionTypeItem(InsertPositionType type) : m_TypeOfInsertPosition(type)
{

}

Platform::String^ InsertPositionTypeItem::DisplayString::get()
{
	return GetDisplayString(m_TypeOfInsertPosition);
}

InsertPositionType InsertPositionTypeItem::TypeOfInsertPosition::get()
{
	return m_TypeOfInsertPosition;
}

Platform::String^ InsertPositionTypeItem::GetDisplayString(InsertPositionType type)
{
	Platform::String^ _Str = L"";
	auto _ResourceLoader = Windows::ApplicationModel::Resources::ResourceLoader::GetForCurrentView("foxit.rdk.uiextensions/Resources");

	switch (type)
	{
	case foxit::rdk::uiextensions::InsertPositionType::AfterPage:
		_Str = _ResourceLoader->GetString(L"InsertTypeAfterPageText");
		break;
	case foxit::rdk::uiextensions::InsertPositionType::BeforePage:
		_Str = _ResourceLoader->GetString(L"InsertPositionTypeBeforePageText");
		break;
	default:
		break;
	}

	return _Str;
}

InsertPageControl::InsertPageControl()
{
	InitializeComponent();

	OnInitializeControl();
}

void InsertPageControl::OnInitializeControl()
{
	auto _ObjValue = this->Resources->Lookup(L"LayoutRootGridMinWidth");
	m_LayoutRootGridMinWidth = _ObjValue && _ObjValue->GetType()->FullName == double::typeid->FullName ? dynamic_cast<Platform::IBox<double>^>(_ObjValue)->Value : 400;

	m_InsertTypeGroup = ref new Platform::Collections::Vector<InsertTypeItem^>();
	m_InsertTypeGroup->Append(ref new InsertTypeItem(InsertType::BlankPage));
	m_InsertTypeGroup->Append(ref new InsertTypeItem(InsertType::ImageFile));
	m_InsertTypeGroup->Append(ref new InsertTypeItem(InsertType::PdfFile));

	m_InsertPositionTypeGroup = ref new Platform::Collections::Vector<InsertPositionTypeItem^>();
	m_InsertPositionTypeGroup->Append(ref new InsertPositionTypeItem(InsertPositionType::AfterPage));
	m_InsertPositionTypeGroup->Append(ref new InsertPositionTypeItem(InsertPositionType::BeforePage));

	m_SelectedImageFiles = ref new Platform::Collections::Vector<Windows::Storage::StorageFile^>();
	m_SelectedDocFiles = ref new Platform::Collections::Vector<Windows::Storage::StorageFile^>();

	this->Loaded += ref new Windows::UI::Xaml::RoutedEventHandler(this, &foxit::rdk::uiextensions::InsertPageControl::OnLoaded);
	this->Unloaded += ref new Windows::UI::Xaml::RoutedEventHandler(this, &foxit::rdk::uiextensions::InsertPageControl::OnUnloaded);
}

ThumbnailPage^ InsertPageControl::HostPage::get()
{
	return m_HostPageWeakRef.Resolve<ThumbnailPage>();
}

Windows::Foundation::Collections::IVector<InsertTypeItem^>^ InsertPageControl::InsertTypeGroup::get()
{
	return m_InsertTypeGroup;
}

Windows::Foundation::Collections::IVector<InsertPositionTypeItem^>^ InsertPageControl::InsertPositionTypeGroup::get()
{
	return m_InsertPositionTypeGroup;
}

void InsertPageControl::SetHostedPage(ThumbnailPage^ page)
{
	m_HostPageWeakRef = page;
}

void InsertPageControl::SetPageType(InsertType type)
{
	m_CurInsertType = ref new InsertTypeItem(type);
}

void InsertPageControl::SetSelectedFiles(Platform::Collections::Vector<Windows::Storage::StorageFile^>^ files)
{
	if (m_CurInsertType)
	{
		if (m_CurInsertType->TypeOfInsert == InsertType::ImageFile)
		{
			m_SelectedImageFiles = files;
		}
		else if (m_CurInsertType->TypeOfInsert == InsertType::PdfFile)
		{
			m_SelectedDocFiles = files;
		}
	}
}

void InsertPageControl::UpdateControlState()
{
	if (this->Parent == nullptr)
		return;

	if (m_CurInsertType)
	{
		auto _Type = m_CurInsertType->TypeOfInsert;

		switch (_Type)
		{
		case foxit::rdk::uiextensions::InsertType::BlankPage:
		{
			Windows::UI::Xaml::VisualStateManager::GoToState(this, L"InsertBlankPage", false);
			InsertPageControlInsertButton->IsEnabled = true;
			if (m_SelectedDocFiles)
				m_SelectedDocFiles->Clear();
			if (m_SelectedImageFiles)
				m_SelectedImageFiles->Clear();
		}
		break;
		case foxit::rdk::uiextensions::InsertType::PdfFile:
		{
			Windows::UI::Xaml::VisualStateManager::GoToState(this, L"InsertPdfFile", false);

			InsertPageControlInsertButton->IsEnabled = m_SelectedDocFiles && m_SelectedDocFiles->Size > 0;
			if (m_SelectedImageFiles)
				m_SelectedImageFiles->Clear();
		}
		break;
		case foxit::rdk::uiextensions::InsertType::ImageFile:
		{
			Windows::UI::Xaml::VisualStateManager::GoToState(this, L"InsertImageFile", false);

			InsertPageControlInsertButton->IsEnabled = m_SelectedImageFiles && m_SelectedImageFiles->Size > 0;
			if (m_SelectedDocFiles)
				m_SelectedDocFiles->Clear();
		}
		break;
		default:
			break;
		}

		InsertPageControlTitleTextBlock->Text = m_CurInsertType->DisplayString;

		if (HostPage == nullptr)
			return;

		auto pGridView = HostPage->GetItemsGridView();
		InsertPagePositionPanel->Visibility = pGridView->SelectedItems->Size == 0 || pGridView->SelectedItems->Size > 1 ? Windows::UI::Xaml::Visibility::Visible : Windows::UI::Xaml::Visibility::Collapsed;
	}
	else
	{
		InsertTypePanel->Visibility = Windows::UI::Xaml::Visibility::Visible;
	}
}

void InsertPageControl::UpdateControlPosition()
{
	auto _frameworkElement = dynamic_cast<UIElement^>(this->Parent);
	if (_frameworkElement == nullptr)
		return;

	if (m_bIsShowInputPane)
	{
		auto _inputPane = Windows::UI::ViewManagement::InputPane::GetForCurrentView();
		auto _inputPaneRect = _inputPane->OccludedRect;
		auto _winRect = Windows::UI::Xaml::Window::Current->Bounds;

		if (m_InputPaneTranslateTransform)
		{
			_frameworkElement->RenderTransform = ref new Windows::UI::Xaml::Media::TranslateTransform();
			m_InputPaneTranslateTransform = nullptr;
		}

		auto _focusElement = dynamic_cast<Windows::UI::Xaml::FrameworkElement^>(Windows::UI::Xaml::Input::FocusManager::GetFocusedElement());
		if (_focusElement == nullptr)
			return;

		Windows::Foundation::Point _screenPt = _focusElement->TransformToVisual(nullptr)->TransformPoint(Point(0, 0));
		auto _focusElementRect = Windows::Foundation::Rect(_screenPt, Windows::Foundation::Size((float)_focusElement->ActualWidth, (float)_focusElement->ActualHeight));

#if (_DEBUG && 1)
		std::wstringstream ws;
		ws << L"InputPane OccludedRect: " << _inputPaneRect.Left << L", " << _inputPaneRect.Top << L", " << _inputPaneRect.Width << L", " << _inputPaneRect.Height << L"\n";
		ws << L"FocusElement: " << _focusElement->GetType()->FullName->Data() << L", Point: " << _screenPt.X << L", " << _screenPt.Y << L", " << _focusElement->ActualWidth << L", " << _focusElement->ActualHeight;
		ws << L"\n";
		OutputDebugString(ws.str().c_str());
#endif

		if (_inputPaneRect.IntersectsWith(_focusElementRect))
		{
			OutputDebugString(L"IntersectsWith...\n");
			double _offsetY = _screenPt.Y + _focusElement->ActualHeight - _inputPaneRect.Top;
			auto _ScrollViewer = foxit::rdk::uiextensions::GetVisualParent<Windows::UI::Xaml::Controls::ScrollViewer^>(_focusElement);
			if (_ScrollViewer && _ScrollViewer->VerticalScrollMode != Windows::UI::Xaml::Controls::ScrollMode::Disabled)
			{
				auto _ScrollableHeight = _ScrollViewer->ScrollableHeight;
				auto _CurVerticalOffset = _ScrollViewer->VerticalOffset;
				auto _CanScrollOffsetY = _ScrollViewer->ScrollableHeight - _CurVerticalOffset;
				if (_CanScrollOffsetY >= _offsetY)
				{
					_ScrollViewer->ChangeView(_ScrollViewer->HorizontalOffset, _ScrollViewer->VerticalOffset + _offsetY, _ScrollViewer->ZoomFactor);
					return;
				}
				else
				{
					_ScrollViewer->ChangeView(_ScrollViewer->HorizontalOffset, _ScrollViewer->VerticalOffset + _CanScrollOffsetY, _ScrollViewer->ZoomFactor);
					_offsetY -= _CanScrollOffsetY;
				}
			}

			m_InputPaneTranslateTransform = ref new Windows::UI::Xaml::Media::TranslateTransform();
			m_InputPaneTranslateTransform->Y -= _offsetY;
			_frameworkElement->RenderTransform = m_InputPaneTranslateTransform;
		}
	}
	else
	{
		if (m_InputPaneTranslateTransform)
		{
			_frameworkElement->RenderTransform = ref new Windows::UI::Xaml::Media::TranslateTransform();
			m_InputPaneTranslateTransform = nullptr;
		}
	}
}

void InsertPageControl::ClosePopup()
{
	auto _PopupDialog = GetParent<ContentHostedDialog^>(this->Parent);
	if (_PopupDialog)
	{
		_PopupDialog->Hide();
	}
}

void InsertPageControl::ResizeLayoutRoot()
{
	auto _VisibleBounds = Windows::UI::ViewManagement::ApplicationView::GetForCurrentView()->VisibleBounds;
	auto _ActualWidth = this->ActualWidth;

	if (_ActualWidth > _VisibleBounds.Width)
	{
		this->MinWidth = _VisibleBounds.Width;
		this->MaxWidth = _VisibleBounds.Width;
		this->Width = _VisibleBounds.Width;
	}
	else
	{
		if (m_LayoutRootGridMinWidth > _VisibleBounds.Width)
		{
			this->MinWidth = _VisibleBounds.Width;
			this->MaxWidth = _VisibleBounds.Width;
			this->Width = _VisibleBounds.Width;
		}
		else
		{
			this->MinWidth = m_LayoutRootGridMinWidth;
			this->MaxWidth = m_LayoutRootGridMinWidth;
			this->Width = m_LayoutRootGridMinWidth;
		}
	}
}

void InsertPageControl::ShowInvalidPageNumberTipsDialog()
{
	auto _resourceLoader = Windows::ApplicationModel::Resources::ResourceLoader::GetForCurrentView("foxit.rdk.uiextensions/Resources");
	Platform::String^ _InvalidPageNumberTipsString = _resourceLoader->GetString(L"InsertPageControlInvalidPageNumberTipsText");
	auto _MessageDialog = ref new Windows::UI::Popups::MessageDialog(_InvalidPageNumberTipsString);

	concurrency::create_task(_MessageDialog->ShowAsync())
		.then([=](Windows::UI::Popups::IUICommand^ command)
	{
		if (command)
		{
			auto _Id = command->Id;
			auto _Label = command->Label;

			if (command->Id != nullptr)
			{
				int const id = safe_cast<IBox<int>^>(command->Id)->Value;
				if (id == 0)
				{

				}
			}

			PageOfTextBox->Focus(Windows::UI::Xaml::FocusState::Programmatic);
		}
	}, concurrency::task_continuation_context::use_current())
		.then([=](concurrency::task<void> t)
	{
		try
		{
			t.get();
		}
		catch (...)
		{
			OutputDebugString(L"ShowAsync...URL...Exception...\n");
		}
	});
}

void InsertPageControl::ShowInvalidPageCountTipsDialog()
{
	auto _resourceLoader = Windows::ApplicationModel::Resources::ResourceLoader::GetForCurrentView("foxit.rdk.uiextensions/Resources");
	Platform::String^ _InvalidPageNumberTipsString = _resourceLoader->GetString(L"InsertPageControlInvalidInsertCountTipsText");
	auto _MessageDialog = ref new Windows::UI::Popups::MessageDialog(_InvalidPageNumberTipsString);

	concurrency::create_task(_MessageDialog->ShowAsync())
		.then([=](Windows::UI::Popups::IUICommand^ command)
	{
		if (command)
		{
			auto _Id = command->Id;
			auto _Label = command->Label;

			if (command->Id != nullptr)
			{
				int const id = safe_cast<IBox<int>^>(command->Id)->Value;
				if (id == 0)
				{

				}
			}

			PageCountTextBox->Focus(Windows::UI::Xaml::FocusState::Programmatic);
		}
	}, concurrency::task_continuation_context::use_current())
		.then([=](concurrency::task<void> t)
	{
		try
		{
			t.get();
		}
		catch (...)
		{
			OutputDebugString(L"ShowAsync...URL...Exception...\n");
		}
	});
}

void InsertPageControl::ShowProgressRing()
{
	if (ProgressRingPanel->Visibility == Windows::UI::Xaml::Visibility::Collapsed)
		ProgressRingPanel->Visibility = Windows::UI::Xaml::Visibility::Visible;
}

void InsertPageControl::HideProgressRing()
{
	if (ProgressRingPanel->Visibility == Windows::UI::Xaml::Visibility::Visible)
		ProgressRingPanel->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
}

void foxit::rdk::uiextensions::InsertPageControl::OnLoaded(Platform::Object ^sender, Windows::UI::Xaml::RoutedEventArgs ^e)
{
	auto _InputPane = Windows::UI::ViewManagement::InputPane::GetForCurrentView();
	m_InputPaneShowingEventToken = _InputPane->Showing += ref new Windows::Foundation::TypedEventHandler<Windows::UI::ViewManagement::InputPane ^, Windows::UI::ViewManagement::InputPaneVisibilityEventArgs ^>(this, &foxit::rdk::uiextensions::InsertPageControl::OnInputPaneShowing);
	m_InputPaneHidingEventToken = _InputPane->Hiding += ref new Windows::Foundation::TypedEventHandler<Windows::UI::ViewManagement::InputPane ^, Windows::UI::ViewManagement::InputPaneVisibilityEventArgs ^>(this, &foxit::rdk::uiextensions::InsertPageControl::OnInputPaneHiding);

	m_WindowSizeChangedEventToken = Windows::UI::Xaml::Window::Current->SizeChanged += ref new Windows::UI::Xaml::WindowSizeChangedEventHandler(this, &foxit::rdk::uiextensions::InsertPageControl::OnWindowSizeChanged);

	UpdateControlState();

	ResizeLayoutRoot();
}


void foxit::rdk::uiextensions::InsertPageControl::OnUnloaded(Platform::Object ^sender, Windows::UI::Xaml::RoutedEventArgs ^e)
{
	auto _InputPane = Windows::UI::ViewManagement::InputPane::GetForCurrentView();
	_InputPane->Showing -= m_InputPaneShowingEventToken;
	_InputPane->Hiding -= m_InputPaneHidingEventToken;

	Windows::UI::Xaml::Window::Current->SizeChanged -= m_WindowSizeChangedEventToken;
}


void foxit::rdk::uiextensions::InsertPageControl::OnInputPaneShowing(Windows::UI::ViewManagement::InputPane ^sender, Windows::UI::ViewManagement::InputPaneVisibilityEventArgs ^args)
{
	m_PrevInputPaneEnsuredFocusedElementInView = args->EnsuredFocusedElementInView;
	args->EnsuredFocusedElementInView = true;
	m_bIsShowInputPane = true;

	UpdateControlPosition();
}


void foxit::rdk::uiextensions::InsertPageControl::OnInputPaneHiding(Windows::UI::ViewManagement::InputPane ^sender, Windows::UI::ViewManagement::InputPaneVisibilityEventArgs ^args)
{
	args->EnsuredFocusedElementInView = m_PrevInputPaneEnsuredFocusedElementInView;
	m_bIsShowInputPane = false;

	UpdateControlPosition();
}


void foxit::rdk::uiextensions::InsertPageControl::OnWindowSizeChanged(Platform::Object ^sender, Windows::UI::Core::WindowSizeChangedEventArgs ^e)
{
	ResizeLayoutRoot();
}


void foxit::rdk::uiextensions::InsertPageControl::ClosePopup_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{
	ClosePopup();
}


void foxit::rdk::uiextensions::InsertPageControl::InsertPageControlInsert_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{
	auto _Item = m_CurInsertType;
	auto _Type = _Item->TypeOfInsert;

	if (HostPage == nullptr)
		return;

	auto pView = HostPage->Viewer;
	if (pView == nullptr)
		return;

	int startIndex = 0;
	auto pGridView = HostPage->GetItemsGridView();
	auto nTotalCount = pView->GetDoc()->GetPageCount();

	if (pGridView->SelectedItems->Size == 0 || pGridView->SelectedItems->Size > 1)
	{
		if (InsertPageControlFirstPageRadioButton->IsChecked->Value)
		{
			startIndex = 1;
		}
		else if (InsertPageControlLastPageRadioButton->IsChecked->Value)
		{
			startIndex = nTotalCount;
		}
		else if (InsertPageControlPageOfRadioButton->IsChecked->Value)
		{
			startIndex = _wtoi(PageOfTextBox->Text->Data());
			if (startIndex == 0 || startIndex > nTotalCount)
			{
				ShowInvalidPageNumberTipsDialog();
				return;
			}
		}

		startIndex--;
	}
	else if (pGridView->SelectedItems->Size == 1)
	{
		auto srcItem = dynamic_cast<ThumbnailItem^>(pGridView->SelectedItems->GetAt(0));
		startIndex = srcItem->PageIndex - 1;
	}

	auto _PositionSelectedItem = InsertPagePositionTypeComboBox->SelectedItem;
	auto _PositionItem = dynamic_cast<InsertPositionTypeItem^>(_PositionSelectedItem);
	if (_PositionItem->TypeOfInsertPosition == InsertPositionType::AfterPage)
	{
		startIndex++;
	}

	if (_Type == InsertType::BlankPage)
	{
		int pageCount = 1;

		pageCount = _wtoi(PageCountTextBox->Text->Data());
		if (pageCount > foxit::rdk::AppConfig::InsertPageMaxCount)
		{
			ShowInvalidPageCountTipsDialog();
			return;
		}

		ShowProgressRing();

		HostPage->InsertBlankPage(startIndex, pageCount);
		
		HideProgressRing();
		ClosePopup();
	}
	else if (_Type == InsertType::PdfFile)
	{
		if (m_SelectedDocFiles == nullptr || m_SelectedDocFiles->Size == 0)
			return;

		ShowProgressRing();
		HostPage->InsertDocument(startIndex, m_SelectedDocFiles)
			.then([=]()
		{
			HideProgressRing();

			ClosePopup();
		}, concurrency::task_continuation_context::use_current())
			.then([](concurrency::task<void> t)
		{
			try
			{
				t.get();
			}
			catch (...)
			{

			}
		});
	}
	else if (_Type == InsertType::ImageFile)
	{
		if (m_SelectedImageFiles == nullptr || m_SelectedImageFiles->Size == 0)
			return;

		ShowProgressRing();
		HostPage->InsertImage(startIndex, m_SelectedImageFiles)
			.then([=]()
		{
			HideProgressRing();

			ClosePopup();
		}, concurrency::task_continuation_context::use_current())
			.then([](concurrency::task<void> t)
		{
			try
			{
				t.get();
			}
			catch (...)
			{

			}
		});
	}
}


void foxit::rdk::uiextensions::InsertPageControl::InsertPageControlCancel_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{
	ClosePopup();
}


void foxit::rdk::uiextensions::InsertPageControl::InsertTypeComboBox_SelectionChanged(Platform::Object^ sender, Windows::UI::Xaml::Controls::SelectionChangedEventArgs^ e)
{
	auto _SelectedItem = InsertTypeComboBox->SelectedItem;
	auto _Item = dynamic_cast<InsertTypeItem^>(_SelectedItem);
	
	if (InsertTypePanel->Visibility == Windows::UI::Xaml::Visibility::Visible)
	{
		m_CurInsertType = _Item;

		UpdateControlState();
	}
}


void foxit::rdk::uiextensions::InsertPageControl::InsertTypeComboBox_Loaded(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{
	if (InsertTypeGroup->Size > 0)
	{
		InsertTypeComboBox->SelectedIndex = 0;
	}
}


void foxit::rdk::uiextensions::InsertPageControl::InsertPagePositionTypeComboBox_Loaded(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{
	if (InsertPositionTypeGroup->Size > 0)
	{
		InsertPagePositionTypeComboBox->SelectedIndex = 0;
	}
}

void foxit::rdk::uiextensions::InsertPageControl::InsertPageControlSelectImageFile_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{
	Windows::Storage::Pickers::FileOpenPicker^ _FileOpenPicker = ref new Windows::Storage::Pickers::FileOpenPicker();
	_FileOpenPicker->ViewMode = Windows::Storage::Pickers::PickerViewMode::Thumbnail;
	_FileOpenPicker->SuggestedStartLocation = Windows::Storage::Pickers::PickerLocationId::DocumentsLibrary;
	_FileOpenPicker->FileTypeFilter->Append(L".png");
	_FileOpenPicker->FileTypeFilter->Append(L".bmp");
	_FileOpenPicker->FileTypeFilter->Append(L".jpg");

	concurrency::create_task(_FileOpenPicker->PickMultipleFilesAsync())
		.then([this](Windows::Foundation::Collections::IVectorView<Windows::Storage::StorageFile^>^ files)
	{
		if (files == nullptr)
			throw ref new Platform::NullReferenceException();

		Platform::Array<Windows::Storage::StorageFile^>^ _Array = ref new Platform::Array<Windows::Storage::StorageFile^>(files->Size);
		files->GetMany(0, _Array);
		auto pImageFiles = ref new Platform::Collections::Vector<Windows::Storage::StorageFile^>(_Array);

		m_SelectedImageFiles = pImageFiles;
	})
		.then([=](concurrency::task<void> t)
	{
		try
		{
			InsertPageControlInsertButton->IsEnabled = m_SelectedImageFiles && m_SelectedImageFiles->Size > 0;

			t.get();
		}
		catch (...)
		{

		}
	});

}


void foxit::rdk::uiextensions::InsertPageControl::InsertPageControlSelectPdfFile_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{
	Windows::Storage::Pickers::FileOpenPicker^ _FileOpenPicker = ref new Windows::Storage::Pickers::FileOpenPicker();
	_FileOpenPicker->ViewMode = Windows::Storage::Pickers::PickerViewMode::Thumbnail;
	_FileOpenPicker->SuggestedStartLocation = Windows::Storage::Pickers::PickerLocationId::DocumentsLibrary;
	_FileOpenPicker->FileTypeFilter->Append(L".pdf");
	
	concurrency::create_task(_FileOpenPicker->PickSingleFileAsync())
		.then([this](Windows::Storage::StorageFile^ file)
	{
		if (file == nullptr)
			throw ref new Platform::NullReferenceException();

		auto pImageFiles = ref new Platform::Collections::Vector<Windows::Storage::StorageFile^>();

		pImageFiles->Append(file);
		m_SelectedDocFiles = pImageFiles;
	})
		.then([=](concurrency::task<void> t)
	{
		try
		{
			InsertPageControlInsertButton->IsEnabled = m_SelectedDocFiles && m_SelectedDocFiles->Size > 0;

			t.get();
		}
		catch (...)
		{

		}
	});
}


void foxit::rdk::uiextensions::InsertPageControl::LayoutRootGrid_Tapped(Platform::Object^ sender, Windows::UI::Xaml::Input::TappedRoutedEventArgs^ e)
{
	if (e->OriginalSource->GetType()->FullName == Windows::UI::Xaml::Controls::Grid::typeid->FullName)
	{
		auto _InputPane = Windows::UI::ViewManagement::InputPane::GetForCurrentView();
		auto _OccludedRect = _InputPane->OccludedRect;
		if (_OccludedRect.Height != 0)
		{
			InsertPageControlInsertButton->Focus(Windows::UI::Xaml::FocusState::Programmatic);
		}
	}
}


void foxit::rdk::uiextensions::InsertPageControl::PageOfTextBox_KeyDown(Platform::Object^ sender, Windows::UI::Xaml::Input::KeyRoutedEventArgs^ e)
{
	auto _virtualKey = e->Key;
	auto _coreWindow = Window::Current->CoreWindow;
	auto _downState = Windows::UI::Core::CoreVirtualKeyStates::Down;
	bool _menuKey = (_coreWindow->GetKeyState(Windows::System::VirtualKey::Menu) & _downState) == _downState;
	bool _controlKey = (_coreWindow->GetKeyState(Windows::System::VirtualKey::Control) & _downState) == _downState;
	bool _shiftKey = (_coreWindow->GetKeyState(Windows::System::VirtualKey::Shift) & _downState) == _downState;
	bool _numberKeyLocktKey = (_coreWindow->GetAsyncKeyState(_virtualKey) & _downState) == _downState;
	bool _noModifiers = !_menuKey && !_controlKey && !_shiftKey;
	bool _onlyAlt = _menuKey && !_controlKey && !_shiftKey;

	if (_noModifiers && _virtualKey == Windows::System::VirtualKey::Enter)
	{
		PageCountTextBox->Focus(Windows::UI::Xaml::FocusState::Programmatic);

		UpdateControlPosition();

		e->Handled = true;
	}
}


void foxit::rdk::uiextensions::InsertPageControl::PageCountTextBox_KeyDown(Platform::Object^ sender, Windows::UI::Xaml::Input::KeyRoutedEventArgs^ e)
{
	auto _virtualKey = e->Key;
	auto _coreWindow = Window::Current->CoreWindow;
	auto _downState = Windows::UI::Core::CoreVirtualKeyStates::Down;
	bool _menuKey = (_coreWindow->GetKeyState(Windows::System::VirtualKey::Menu) & _downState) == _downState;
	bool _controlKey = (_coreWindow->GetKeyState(Windows::System::VirtualKey::Control) & _downState) == _downState;
	bool _shiftKey = (_coreWindow->GetKeyState(Windows::System::VirtualKey::Shift) & _downState) == _downState;
	bool _numberKeyLocktKey = (_coreWindow->GetAsyncKeyState(_virtualKey) & _downState) == _downState;
	bool _noModifiers = !_menuKey && !_controlKey && !_shiftKey;
	bool _onlyAlt = _menuKey && !_controlKey && !_shiftKey;

	if (_noModifiers && _virtualKey == Windows::System::VirtualKey::Enter)
	{
		InsertPageControlInsertButton->Focus(Windows::UI::Xaml::FocusState::Programmatic);

		e->Handled = true;
	}
}
