﻿/**
 * Copyright (C) 2003-2018, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit Mobile PDF SDK to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */


#pragma once

#include "Thumbnail/InsertPageControl.g.h"

namespace foxit
{
	namespace rdk
	{
		namespace uiextensions
		{
			public enum class InsertType
			{
				BlankPage,
				ImageFile,
				PdfFile,
			};

			public enum class InsertPositionType
			{
				AfterPage,
				BeforePage
			};

			[Windows::UI::Xaml::Data::Bindable]
			public ref class InsertTypeItem sealed : BindableBase
			{
			public:
				property Platform::String^ DisplayString
				{
					Platform::String^ get();
				}

				property InsertType TypeOfInsert
				{
					InsertType get();
				}

			internal:
				InsertTypeItem(InsertType type);

			private:
				Platform::String^ GetDisplayString(InsertType type);

			private:
				InsertType m_TypeOfInsert;
			};

			[Windows::UI::Xaml::Data::Bindable]
			public ref class InsertPositionTypeItem sealed : BindableBase
			{
			public:
				property Platform::String^ DisplayString
				{
					Platform::String^ get();
				}

				property InsertPositionType TypeOfInsertPosition
				{
					InsertPositionType get();
				}

			internal:
				InsertPositionTypeItem(InsertPositionType type);

			private:
				Platform::String^ GetDisplayString(InsertPositionType type);

			private:
				InsertPositionType m_TypeOfInsertPosition;
			};

			ref class ThumbnailPage;
			[Windows::Foundation::Metadata::WebHostHidden]
			public ref class InsertPageControl sealed
			{
			public:
				InsertPageControl();

				property Windows::Foundation::Collections::IVector<InsertTypeItem^>^ InsertTypeGroup
				{
					Windows::Foundation::Collections::IVector<InsertTypeItem^>^ get();
				}

				property Windows::Foundation::Collections::IVector<InsertPositionTypeItem^>^ InsertPositionTypeGroup
				{
					Windows::Foundation::Collections::IVector<InsertPositionTypeItem^>^ get();
				}

			internal:
				property ThumbnailPage^ HostPage
				{
					ThumbnailPage^ get();
				}

				void SetHostedPage(ThumbnailPage^ page);
				void SetPageType(InsertType type);
				void SetSelectedFiles(Platform::Collections::Vector<Windows::Storage::StorageFile^>^ files);

			private:
				bool m_PrevInputPaneEnsuredFocusedElementInView;
				bool m_bIsShowInputPane;
				Windows::UI::Xaml::Media::TranslateTransform^ m_InputPaneTranslateTransform;

				Windows::Foundation::EventRegistrationToken m_InputPaneShowingEventToken, m_InputPaneHidingEventToken;
				Windows::Foundation::EventRegistrationToken m_WindowSizeChangedEventToken;

				Platform::WeakReference m_HostPageWeakRef;

				Platform::Collections::Vector<InsertTypeItem^>^ m_InsertTypeGroup;
				Platform::Collections::Vector<InsertPositionTypeItem^>^ m_InsertPositionTypeGroup;
				Platform::Collections::Vector<Windows::Storage::StorageFile^>^ m_SelectedImageFiles;
				Platform::Collections::Vector<Windows::Storage::StorageFile^>^ m_SelectedDocFiles;

				InsertTypeItem^ m_CurInsertType;

				double m_LayoutRootGridMinWidth;

			private:
				void OnInitializeControl();
				void UpdateControlState();
				void UpdateControlPosition();
				void ClosePopup();
				void ResizeLayoutRoot();
				void ShowInvalidPageNumberTipsDialog();
				void ShowInvalidPageCountTipsDialog();
				void ShowProgressRing();
				void HideProgressRing();

			private:
				void OnLoaded(Platform::Object ^sender, Windows::UI::Xaml::RoutedEventArgs ^e);
				void OnUnloaded(Platform::Object ^sender, Windows::UI::Xaml::RoutedEventArgs ^e);
				void OnInputPaneShowing(Windows::UI::ViewManagement::InputPane ^sender, Windows::UI::ViewManagement::InputPaneVisibilityEventArgs ^args);
				void OnInputPaneHiding(Windows::UI::ViewManagement::InputPane ^sender, Windows::UI::ViewManagement::InputPaneVisibilityEventArgs ^args);
				void OnWindowSizeChanged(Platform::Object ^sender, Windows::UI::Core::WindowSizeChangedEventArgs ^e);
				void ClosePopup_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e);
				void InsertPageControlInsert_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e);
				void InsertPageControlCancel_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e);
				void InsertTypeComboBox_SelectionChanged(Platform::Object^ sender, Windows::UI::Xaml::Controls::SelectionChangedEventArgs^ e);
				void InsertTypeComboBox_Loaded(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e);
				void InsertPagePositionTypeComboBox_Loaded(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e);
				void InsertPageControlSelectImageFile_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e);
				void InsertPageControlSelectPdfFile_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e);
				void LayoutRootGrid_Tapped(Platform::Object^ sender, Windows::UI::Xaml::Input::TappedRoutedEventArgs^ e);
				void PageOfTextBox_KeyDown(Platform::Object^ sender, Windows::UI::Xaml::Input::KeyRoutedEventArgs^ e);
				void PageCountTextBox_KeyDown(Platform::Object^ sender, Windows::UI::Xaml::Input::KeyRoutedEventArgs^ e);
			};
		}
	}
}
