﻿/**
 * Copyright (C) 2003-2018, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit Mobile PDF SDK to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */


#include "pch.h"
#include "ThumbnailItemUserControl.xaml.h"
#include "ThumbnailPage.xaml.h"

using namespace foxit::rdk::uiextensions;
using namespace foxit::rdk;

using namespace Platform;
using namespace Windows::Foundation;
using namespace Windows::Foundation::Collections;
using namespace Windows::UI::Xaml;
using namespace Windows::UI::Xaml::Controls;
using namespace Windows::UI::Xaml::Controls::Primitives;
using namespace Windows::UI::Xaml::Data;
using namespace Windows::UI::Xaml::Input;
using namespace Windows::UI::Xaml::Media;
using namespace Windows::UI::Xaml::Navigation;

// The User Control item template is documented at http://go.microsoft.com/fwlink/?LinkId=234236

ThumbnailItemUserControl::ThumbnailItemUserControl()
{
	InitializeComponent();

	OnInitializeControl();
}

void ThumbnailItemUserControl::OnInitializeControl()
{
	this->Loaded += ref new Windows::UI::Xaml::RoutedEventHandler(this, &foxit::rdk::uiextensions::ThumbnailItemUserControl::OnLoaded);
	this->Unloaded += ref new Windows::UI::Xaml::RoutedEventHandler(this, &foxit::rdk::uiextensions::ThumbnailItemUserControl::OnUnloaded);
}

static Windows::UI::Xaml::DependencyProperty^ _PageNumberProperty =
	Windows::UI::Xaml::DependencyProperty::Register(L"PageNumber", Windows::UI::Xaml::Interop::TypeName(int::typeid), Windows::UI::Xaml::Interop::TypeName(ThumbnailItemUserControl::typeid), nullptr);

Windows::UI::Xaml::DependencyProperty^ ThumbnailItemUserControl::PageNumberProperty::get()
{
	return _PageNumberProperty;
}

int ThumbnailItemUserControl::PageNumber::get()
{
	return (int)GetValue(PageNumberProperty);
}

void ThumbnailItemUserControl::PageNumber::set(int value)
{
	SetValue(PageNumberProperty, value);
}

void foxit::rdk::uiextensions::ThumbnailItemUserControl::OnLoaded(Platform::Object ^sender, Windows::UI::Xaml::RoutedEventArgs ^e)
{
	
}

void foxit::rdk::uiextensions::ThumbnailItemUserControl::OnUnloaded(Platform::Object ^sender, Windows::UI::Xaml::RoutedEventArgs ^e)
{
	ClearData();
}

void ThumbnailItemUserControl::ShowPlaceholder(ThumbnailItem^ item, ThumbnailPage^ page)
{
	m_ThumbnailItem = item;
	m_ThumbnailPageWeakRef = page;
	PageNumber = item->PageIndex;
	PlaceholderImage->Opacity = 1.0;
	ThumbnailImageBorder->Opacity = 0.0;
}

void ThumbnailItemUserControl::ShowImage()
{
	ThumbnailPage^ page = m_ThumbnailPageWeakRef.Resolve<ThumbnailPage>();
	if (m_ThumbnailItem == nullptr)
		return;	
	
	int pageindex = m_ThumbnailItem->PageIndex - 1;

	if (page)
	{
		if (m_Cts.get_token().is_canceled())
		{
			concurrency::cancellation_token_source _newCts;
			m_Cts = _newCts;
		}

		Windows::UI::Xaml::Media::Imaging::WriteableBitmap^ bitmap = nullptr;
		if (page->Viewer)
		{
			bitmap = FSPDFDocUtil::GetThumbImageSync(page->Viewer->GetDoc(), pageindex, 134, 194, 0);
			ThumbnailImage->Source = bitmap;
			ThumbnailImageBorder->Opacity = 1.0;
		}

		PlaceholderImage->Opacity = 0.0;
	}
}

void ThumbnailItemUserControl::ClearData()
{
	if (m_Cts.get_token().is_cancelable() && !m_Cts.get_token().is_canceled())
	{
		m_Cts.cancel();
	}

	ThumbnailImage->ClearValue(Windows::UI::Xaml::Controls::Image::SourceProperty);
}

