﻿/**
 * Copyright (C) 2003-2018, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit Mobile PDF SDK to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */


#include "pch.h"
#include "ThumbnailPage.xaml.h"
#include "ThumbnailItemUserControl.xaml.h"
#include "InsertPageControl.xaml.h"
#include "ExtractPageControl.xaml.h"

using namespace foxit::rdk::uiextensions;

using namespace Platform;
using namespace Windows::Foundation;
using namespace Windows::Foundation::Collections;
using namespace Windows::UI::Xaml;
using namespace Windows::UI::Xaml::Controls;
using namespace Windows::UI::Xaml::Controls::Primitives;
using namespace Windows::UI::Xaml::Data;
using namespace Windows::UI::Xaml::Input;
using namespace Windows::UI::Xaml::Media;
using namespace Windows::UI::Xaml::Navigation;


ThumbnailItem::ThumbnailItem()
{

}

int ThumbnailItem::PageIndex::get()
{
	return _PageIndex;
}

void ThumbnailItem::PageIndex::set(int value)
{
	if (_PageIndex != value)
	{
		_PageIndex = value;
		OnPropertyChanged(L"PageIndex");
	}
}

Windows::UI::Xaml::Media::ImageSource^ ThumbnailItem::ThumbImage::get()
{
	return _Thumb;
}

// The User Control item template is documented at http://go.microsoft.com/fwlink/?LinkId=234238

ThumbnailPage::ThumbnailPage()
{
	InitializeComponent();

	OnInitializePage();
}

void ThumbnailPage::OnInitializePage()
{
	m_ThumbnailVect = ref new Platform::Collections::Vector<ThumbnailItem^>();
	m_IsDirectEntryEditMode = false;

	this->Loaded += ref new Windows::UI::Xaml::RoutedEventHandler(this, &foxit::rdk::uiextensions::ThumbnailPage::OnLoaded);
	this->Unloaded += ref new Windows::UI::Xaml::RoutedEventHandler(this, &foxit::rdk::uiextensions::ThumbnailPage::OnUnloaded);
}

Windows::Foundation::TypedEventHandler<Windows::UI::Xaml::Controls::ListViewBase^, Windows::UI::Xaml::Controls::ContainerContentChangingEventArgs^>^ ThumbnailPage::ContainerContentChangingDelegate::get()
{
	if (_ContainerContentChangingDelegate == nullptr)
	{
		_ContainerContentChangingDelegate = ref new Windows::Foundation::TypedEventHandler<Windows::UI::Xaml::Controls::ListViewBase^, Windows::UI::Xaml::Controls::ContainerContentChangingEventArgs^>(this, &ThumbnailPage::ThumbnailGridView_ContainerContentChanging);
	}

	return _ContainerContentChangingDelegate;
}

foxit::rdk::PDFViewCtrl^ ThumbnailPage::Viewer::get()
{
	return m_PageViewWeakRef.Resolve<foxit::rdk::PDFViewCtrl>();
}

UIExtensionsManager^ ThumbnailPage::UIExtMgr::get()
{
	return m_UIViewWeakRef.Resolve<UIExtensionsManager>();
}

void ThumbnailPage::Show()
{

}

void ThumbnailPage::Hide()
{

}

void ThumbnailPage::SetPageView(Platform::Object^ target)
{
	m_UIViewWeakRef = target;

	foxit::rdk::PDFViewCtrl^ view = UIExtMgr->viewControl;

	if (view)
	{
		m_PageViewWeakRef = view;
		auto count = view->GetDoc()->GetPageCount();
		for (int i = 0; i < count; i++)
		{
			ThumbnailItem^ item = ref new ThumbnailItem();

			item->PageIndex = i + 1;
			m_ThumbnailVect->Append(item);
		}

		ThumbnailCollectionViewSource->Source = m_ThumbnailVect;
	}
}

void ThumbnailPage::SetIsDirectEntryEditMode(bool bIsDirect)
{
	m_IsDirectEntryEditMode = bIsDirect;
}

bool ThumbnailPage::GetIsDirectEntryEditMode()
{
	return m_IsDirectEntryEditMode;
}

void foxit::rdk::uiextensions::ThumbnailPage::OnLoaded(Platform::Object ^sender, Windows::UI::Xaml::RoutedEventArgs ^e)
{
	if (Viewer)
	{
		auto curPageIndex = Viewer->GetCurrentPage();

		OutputDebugString(L"UpdateLayout...Start...\n");
		ThumbnailGridView->UpdateLayout();
		ThumbnailGridView->SelectedIndex = curPageIndex;
		if (curPageIndex >= 0)
			ThumbnailGridView->ScrollIntoView(m_ThumbnailVect->GetAt(curPageIndex));
		OutputDebugString(L"UpdateLayout...End...\n");
	}

	this->UpdateLayout();
	UpdateControlState();
}

void foxit::rdk::uiextensions::ThumbnailPage::OnUnloaded(Platform::Object ^sender, Windows::UI::Xaml::RoutedEventArgs ^e)
{
	m_PageViewWeakRef = nullptr;
}

void ThumbnailPage::UpdateControlState()
{
	if (NarmalPanel->Visibility == Windows::UI::Xaml::Visibility::Visible)
	{
		for each (Windows::UI::Xaml::UIElement^ var in NarmalPanel->Children)
		{
			auto pAppbarBtn = dynamic_cast<Windows::UI::Xaml::Controls::AppBarButton^>(var);
			if (pAppbarBtn == nullptr)
				continue;

			if (pAppbarBtn->IsCompact)
				Windows::UI::Xaml::VisualStateManager::GoToState(pAppbarBtn, L"Compact", true);

			OutputDebugString((L"NarmalPanel---AppbarBtn: " + pAppbarBtn->Name + L", IsCompact: " + pAppbarBtn->IsCompact + L"\n")->Data());
		}
	}

	if (EditButtonsPanel->Visibility == Windows::UI::Xaml::Visibility::Visible)
	{
		for each (Windows::UI::Xaml::UIElement^ var in EditButtonsPanel->Children)
		{
			auto pAppbarBtn = dynamic_cast<Windows::UI::Xaml::Controls::AppBarButton^>(var);
			if (pAppbarBtn == nullptr)
				continue;

			if (pAppbarBtn->IsCompact)
				Windows::UI::Xaml::VisualStateManager::GoToState(pAppbarBtn, L"Compact", true);

			OutputDebugString((L"AppbarBtn: " + pAppbarBtn->Name + L", IsCompact: " + pAppbarBtn->IsCompact + L"\n")->Data());
		}
	}

	bool bIsCanModify = FSPDFDocUtil::GetIsDocumentCanModify(Viewer->GetDoc());
	bool bIsOwnerPassword = (Viewer->GetDoc()->GetPasswordType() == foxit::pdf::PasswordType::e_PwdOwner);
	bool bIsCanAssemble = FSPDFDocUtil::GetIsDocumentCanAssemble(Viewer->GetDoc());
	ThumbnailPageEditModeButton->IsEnabled = bIsCanModify || bIsOwnerPassword || bIsCanAssemble;

	UpdateGridViewState();
	UpdateEditControlState();
}

void ThumbnailPage::UpdateEditControlState()
{
	if (EditPanel->Visibility == Windows::UI::Xaml::Visibility::Collapsed)
		return;

	bool bIsMobile = Utility::IsMobilePlatform();

	if (ThumbnailGridView->SelectedItems->Size > 0)
	{
		if (bIsMobile)
		{
			ThumbnailPageRotateLeftMobileButton->IsEnabled = true;
			ThumbnailPageRotateRightMobileButton->IsEnabled = true;
			ThumbnailPageDeleteMobileButton->IsEnabled = true;
			ThumbnailPageInsertMobileButton->IsEnabled = true;
		}
		else
		{
			ThumbnailPageRotateLeftButton->IsEnabled = true;
			ThumbnailPageRotateRightButton->IsEnabled = true;
			ThumbnailPageDeleteButton->IsEnabled = true;
			ThumbnailPageInsertButton->IsEnabled = true;
		}

		ThumbnailPageSelectAllCheckBox->IsChecked = ThumbnailGridView->SelectedItems->Size == m_ThumbnailVect->Size ? true : false;
	}
	else
	{
		if (bIsMobile)
		{
			ThumbnailPageRotateLeftMobileButton->IsEnabled = false;
			ThumbnailPageRotateRightMobileButton->IsEnabled = false;
			ThumbnailPageDeleteMobileButton->IsEnabled = false;
			ThumbnailPageInsertMobileButton->IsEnabled = true;
		}
		else
		{
			ThumbnailPageRotateLeftButton->IsEnabled = false;
			ThumbnailPageRotateRightButton->IsEnabled = false;
			ThumbnailPageDeleteButton->IsEnabled = false;
			ThumbnailPageInsertButton->IsEnabled = true;
		}

		ThumbnailPageSelectAllCheckBox->IsChecked = false;
	}
}

void ThumbnailPage::UpdateGridViewState()
{
	bool bIsCanModify = FSPDFDocUtil::GetIsDocumentCanModify(Viewer->GetDoc());
	bool bIsOwnerPassword = (Viewer->GetDoc()->GetPasswordType() == foxit::pdf::PasswordType::e_PwdOwner);
	bool bIsCanMove = (bIsCanModify || bIsOwnerPassword);

	ThumbnailGridView->CanDragItems = bIsCanMove;
	ThumbnailGridView->CanReorderItems = bIsCanMove;
}

void ThumbnailPage::PageDelete()
{
	int nStart = (int)m_ThumbnailVect->Size;
	int nEnd = 0;

	Platform::Array<Platform::Object^>^ _Array = ref new Platform::Array<Platform::Object^>(ThumbnailGridView->SelectedItems->Size);

	ThumbnailGridView->SelectedItems->GetMany(0, _Array);
	Platform::Collections::Vector<Platform::Object^>^ _Items = ref new Platform::Collections::Vector<Platform::Object^>(_Array);

	Platform::Collections::Vector<int>^ _SrcIndex = ref new Platform::Collections::Vector<int>();
	auto _Size = _Items->Size;
	for (unsigned int i = 0; i < _Size; i++)
	{
		auto srcItem = dynamic_cast<ThumbnailItem^>(_Items->GetAt(i));
		_SrcIndex->Append(srcItem->PageIndex - 1);
	}


	for each (Platform::Object^ var in _Items)
	{
		auto item = dynamic_cast<ThumbnailItem^>(var);
		
		unsigned int index = -1;
		if (m_ThumbnailVect->IndexOf(item, &index))
		{
			m_ThumbnailVect->RemoveAt(index);

			if ((int)index < nStart)
			{
				nStart = index;
			}
		}
	}

	auto _VectSize = m_ThumbnailVect->Size;
	nEnd = _VectSize;
	for (int i = nStart; i < nEnd; i++)
	{
		auto itemTmp = m_ThumbnailVect->GetAt(i);
		if (itemTmp->PageIndex != i + 1)
		{
			itemTmp->PageIndex = i + 1;
		}

		auto _Containner = ThumbnailGridView->ContainerFromIndex(i);
		if (_Containner)
		{
			auto _Control = foxit::rdk::uiextensions::GetVisualChild<ThumbnailItemUserControl^>(_Containner);
			if (_Control->PageNumber != i + 1)
			{
				_Control->PageNumber = i + 1;
			}
		}
	}

	if (!m_IsDocumentChanged)
		m_IsDocumentChanged = true;
}

void foxit::rdk::uiextensions::ThumbnailPage::ThumbnailGridView_ItemClick(Platform::Object^ sender, Windows::UI::Xaml::Controls::ItemClickEventArgs^ e)
{
	auto _clickItem = dynamic_cast<ThumbnailItem^>(e->ClickedItem);

	if (UIExtMgr)
	{
		UIExtMgr->ExitThumbnailPage();
		if (m_IsDocumentChanged)
		{
			auto pViewerMgr = Viewer;
			auto _CurPageIndex = pViewerMgr->GetCurrentPage();
			auto _CurScale = pViewerMgr->GetScale();
		}

		Viewer->GotoPage(_clickItem->PageIndex - 1);
	}
}

void foxit::rdk::uiextensions::ThumbnailPage::ScrollViewOffset(double voffset)
{
	ScrollViewer^ scrollviewer = foxit::rdk::uiextensions::GetVisualChild<ScrollViewer^>(ThumbnailGridView);
	if (scrollviewer)
	{
		auto _CurVerticalOffset = scrollviewer->VerticalOffset;
		scrollviewer->ChangeView(scrollviewer->HorizontalOffset, _CurVerticalOffset + voffset, scrollviewer->ZoomFactor);
	}
}


void foxit::rdk::uiextensions::ThumbnailPage::ThumbnailGridView_ContainerContentChanging(Windows::UI::Xaml::Controls::ListViewBase^ sender, Windows::UI::Xaml::Controls::ContainerContentChangingEventArgs^ args)
{
	ThumbnailItemUserControl^ control = dynamic_cast<ThumbnailItemUserControl^>(args->ItemContainer->ContentTemplateRoot);
	ThumbnailItem^ item = dynamic_cast<ThumbnailItem^>(args->Item);

	if (args->InRecycleQueue)
	{
		control->ClearData();
	}
	else if (args->Phase == 0)
	{
		control->ShowPlaceholder(item, this);
		args->RegisterUpdateCallback(ContainerContentChangingDelegate);
	}
	else if (args->Phase == 1)
	{
		control->ShowImage();
	}

	args->Handled = true;
}


void foxit::rdk::uiextensions::ThumbnailPage::ExitAppBarBtn_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{
	if (Viewer)
	{
		if (this->Parent && this->Parent->GetType()->FullName == Windows::UI::Xaml::Controls::Grid::typeid->FullName)
		{
			auto pGrid = dynamic_cast<Windows::UI::Xaml::Controls::Grid^>(this->Parent);
			unsigned int index = -1;
			if (pGrid->Children->IndexOf(this, &index))
			{
				pGrid->Children->RemoveAt(index);
			}
		}
	}
}


void foxit::rdk::uiextensions::ThumbnailPage::EditModeButton_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{
	SetIsDirectEntryEditMode(true);

	auto _InternetConnectionProfile = Windows::Networking::Connectivity::NetworkInformation::GetInternetConnectionProfile();
	bool bInternetConnect = (_InternetConnectionProfile && _InternetConnectionProfile->GetNetworkConnectivityLevel() == Windows::Networking::Connectivity::NetworkConnectivityLevel::InternetAccess) ? true : false;

}


void foxit::rdk::uiextensions::ThumbnailPage::ThumbnailPageRotateLeftButton_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{
	if (ThumbnailGridView->SelectedItems->Size == 0)
		return;

	for each (Platform::Object^ var in ThumbnailGridView->SelectedItems)
	{
		auto item = dynamic_cast<ThumbnailItem^>(var);
		auto pageIndex = item->PageIndex - 1;

		unsigned int index = -1;
		if (m_ThumbnailVect->IndexOf(item, &index))
		{
			auto _Containner = ThumbnailGridView->ContainerFromIndex(index);
			if (_Containner)
			{
				auto _Control = foxit::rdk::uiextensions::GetVisualChild<ThumbnailItemUserControl^>(_Containner);
				_Control->ShowImage();
			}
		}
	}

	if (!m_IsDocumentChanged)
		m_IsDocumentChanged = true;
}


void foxit::rdk::uiextensions::ThumbnailPage::ThumbnailPageRotateRightButton_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{
	if (ThumbnailGridView->SelectedItems->Size == 0)
		return;

	for each (Platform::Object^ var in ThumbnailGridView->SelectedItems)
	{
		auto item = dynamic_cast<ThumbnailItem^>(var);
		auto pageIndex = item->PageIndex - 1;

		unsigned int index = -1;
		if (m_ThumbnailVect->IndexOf(item, &index))
		{
			auto _Containner = ThumbnailGridView->ContainerFromIndex(index);
			if (_Containner)
			{
				auto _Control = foxit::rdk::uiextensions::GetVisualChild<ThumbnailItemUserControl^>(_Containner);
				_Control->ShowImage();
			}
		}
	}

	if (!m_IsDocumentChanged)
		m_IsDocumentChanged = true;
}


void foxit::rdk::uiextensions::ThumbnailPage::ThumbnailPageInsertButton_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{
	
}


void foxit::rdk::uiextensions::ThumbnailPage::ThumbnailPageDeleteButton_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{
	if (ThumbnailGridView->SelectedItems->Size > 0)
	{
		auto _resourceLoader = Windows::ApplicationModel::Resources::ResourceLoader::GetForCurrentView("foxit.rdk.uiextensions/Resources");
		Platform::String^ saveYes = _resourceLoader->GetString(L"SaveYesText");
		Platform::String^ saveNo = _resourceLoader->GetString(L"SaveNoText");

		if (ThumbnailGridView->SelectedItems->Size == m_ThumbnailVect->Size)
		{
			Platform::String^ _NoDeleteAllTipsString = _resourceLoader->GetString(L"ThumbnailPageNoDeleteAllTipsText");
			auto _MessageDialog = ref new Windows::UI::Popups::MessageDialog(_NoDeleteAllTipsString);
			concurrency::create_task(_MessageDialog->ShowAsync());
			return;
		}

		Platform::String^ _DeletePageTipsString = _resourceLoader->GetString(L"ThumbnailPageDeletePageTipsText");
		auto _MessageDialog = ref new Windows::UI::Popups::MessageDialog(_DeletePageTipsString);
		_MessageDialog->Commands->Append(ref new Windows::UI::Popups::UICommand(saveYes, nullptr, PropertyValue::CreateInt32(0)));
		_MessageDialog->Commands->Append(ref new Windows::UI::Popups::UICommand(saveNo, nullptr, PropertyValue::CreateInt32(1)));
		_MessageDialog->DefaultCommandIndex = 0;

		concurrency::create_task(_MessageDialog->ShowAsync())
			.then([=](Windows::UI::Popups::IUICommand^ command)
		{
			if (command && command->Id != nullptr)
			{
				int const id = safe_cast<IBox<int>^>(command->Id)->Value;
				if (id == 0)
				{
					PageDelete();
				}
			}
		}, concurrency::task_continuation_context::use_current())
			.then([=](concurrency::task<void> t)
		{
			try
			{
				t.get();
			}
			catch (...)
			{
				OutputDebugString(L"ShowAsync...URL...Exception...\n");
			}
		});
	}
}


void foxit::rdk::uiextensions::ThumbnailPage::ThumbnailPageCompleteButton_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{
	ExitEditMode();
}


void foxit::rdk::uiextensions::ThumbnailPage::ThumbnailGridView_Drop(Platform::Object^ sender, Windows::UI::Xaml::DragEventArgs^ e)
{
	OutputDebugString(L"ThumbnailPage::ThumbnailGridView_Drop...\n");

	if (e->DataView->Contains(Windows::ApplicationModel::DataTransfer::StandardDataFormats::Text))
	{
		// We need to take a Deferral as we won't be able to confirm the end
		// of the operation synchronously
		auto def = e->GetDeferral();
		concurrency::create_task(e->DataView->GetTextAsync()).then([def, this, e](String^ s)
		{
			// Parse the string to add items corresponding to each line
			auto wsText = s->Data();
			while (wsText) {
				auto wsNext = wcschr(wsText, L'\n');
				if (wsNext == nullptr)
				{
					// No more separator
					wsText = wsNext;
				}
				else
				{
					wsText = wsNext + 1;
				}
			}
			e->AcceptedOperation = Windows::ApplicationModel::DataTransfer::DataPackageOperation::Copy;
			def->Complete();
		});
	}
}


void foxit::rdk::uiextensions::ThumbnailPage::ThumbnailGridView_DragOver(Platform::Object^ sender, Windows::UI::Xaml::DragEventArgs^ e)
{
	OutputDebugString(L"ThumbnailPage::ThumbnailGridView_DragOver...\n");
	auto isHave = e->DataView->Contains(L"MyDataFormat");

	e->AcceptedOperation = (e->DataView->Contains(L"MyDataFormat")) ? Windows::ApplicationModel::DataTransfer::DataPackageOperation::Copy : Windows::ApplicationModel::DataTransfer::DataPackageOperation::None;
}


void foxit::rdk::uiextensions::ThumbnailPage::ThumbnailGridView_DragItemsStarting(Platform::Object^ sender, Windows::UI::Xaml::Controls::DragItemsStartingEventArgs^ e)
{
	OutputDebugString((L"ThumbnailPage::ThumbnailGridView_DragItemsStarting..." + e->Items->Size + L", Cancel: " + e->Cancel + L"\n")->Data());
}


void foxit::rdk::uiextensions::ThumbnailPage::ThumbnailGridView_DragItemsCompleted(Windows::UI::Xaml::Controls::ListViewBase^ sender, Windows::UI::Xaml::Controls::DragItemsCompletedEventArgs^ args)
{
	OutputDebugString((L"ThumbnailPage::ThumbnailGridView_DragItemsCompleted..." + args->Items->Size + L", DropResult: " + args->DropResult.ToString() + L"\n")->Data());

	auto _SelectItem = dynamic_cast<ThumbnailItem^>(ThumbnailGridView->SelectedItem);
	int pageindex = -1;

	if (args->Items)
	{
		auto _Size = args->Items->Size;
		if (_Size > 0)
		{
			unsigned int minIndex = m_ThumbnailVect->Size;
			Platform::Collections::Vector<int>^ _SrcIndex = ref new Platform::Collections::Vector<int>();
			for (unsigned int i = 0; i < _Size; i++)
			{
				auto srcItem = dynamic_cast<ThumbnailItem^>(args->Items->GetAt(i));
				unsigned int index;
				if (m_ThumbnailVect->IndexOf(srcItem, &index))
				{
					if ((int)index < minIndex)
					{
						minIndex = index;
					}
				}
				
				_SrcIndex->Append(srcItem->PageIndex - 1);
			}

			pageindex = minIndex > 0 ? m_ThumbnailVect->GetAt(minIndex - 1)->PageIndex - 1 : -1;

			OutputListViewInfo();

			int dstIndex = pageindex;
			int nStart = dstIndex < 0 ? 0 : dstIndex;
			int nEnd = dstIndex;
			auto srcIndexSize = _SrcIndex->Size;
			OutputDebugString(L"SrcIndex:");
			for (unsigned int i = 0; i < srcIndexSize; i++)
			{
				auto nItem = _SrcIndex->GetAt(i);
				OutputDebugString((L" " + nItem)->Data());
				if (nStart > nItem)
					nStart = nItem;
				if (nEnd < nItem)
					nEnd = nItem;
			}
			OutputDebugString(L"\n");

			OutputDebugString((L"ChangePageIndex - UpdateUI: " + pageindex + L",[" + nStart + L"," + nEnd + L"]\n")->Data());
			for (int i = nStart; i <= nEnd; i++)
			{
				auto itemTmp = m_ThumbnailVect->GetAt(i);
				if (itemTmp->PageIndex != i + 1)
				{
					itemTmp->PageIndex = i + 1;
					auto _Containner = ThumbnailGridView->ContainerFromIndex(i);
					if (_Containner)
					{
						auto _Control = foxit::rdk::uiextensions::GetVisualChild<ThumbnailItemUserControl^>(_Containner);
						_Control->PageNumber = i + 1;
						_Control->ShowImage();

						if (!m_IsDocumentChanged)
							m_IsDocumentChanged = true;
					}
				}
			}

			if (_Size > 1)
			{
				ThumbnailGridView->SelectedItem = nullptr;
			}
		}
	}
}

void ThumbnailPage::OutputListViewInfo()
{
	auto _VectSize = m_ThumbnailVect->Size;
	OutputDebugString((L"OutputListViewInfo-" + _VectSize + L"#####################\n")->Data());

	for (unsigned int i = 0; i < _VectSize; i++)
	{
		auto item = m_ThumbnailVect->GetAt(i);
		OutputDebugString((i + L", PageIndex: " + item->PageIndex + L"\n")->Data());
	}

	OutputDebugString((L"OutputListViewInfo-" + _VectSize + L"#####################\n")->Data());

}

void ThumbnailPage::EnterEditMode()
{
	bool bIsCanModify = FSPDFDocUtil::GetIsDocumentCanModify(Viewer->GetDoc());
	foxit::pdf::PasswordType passwordType = Viewer->GetDoc()->GetPasswordType();
	bool bIsOwnerPassword = (passwordType == foxit::pdf::PasswordType::e_PwdOwner);
	bool bIsCanAssemble = FSPDFDocUtil::GetIsDocumentCanAssemble(Viewer->GetDoc());

	bool bIsCanEdit = bIsCanModify || bIsOwnerPassword || bIsCanAssemble;
	if (!bIsCanEdit)
		return;

	VisualStateManager::GoToState(this, L"MultipleSelectionState", true);
	EditButtonsMobilePanel->Visibility = Utility::IsMobilePlatform() ? Windows::UI::Xaml::Visibility::Visible : Windows::UI::Xaml::Visibility::Collapsed;

	this->UpdateLayout();
	UpdateControlState();
}

//To do...
void ThumbnailPage::ExitEditMode()
{
	if (m_IsDirectEntryEditMode)
	{
		if (Viewer == nullptr)
			return;
	}
	else
	{
		ToolBarPanel->UpdateLayout();
		UpdateControlState();

		VisualStateManager::GoToState(this, L"NormalState", true);

		EditButtonsMobilePanel->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
	}
}

void ThumbnailPage::ShowInsertImageFileTooMuchTipsDialog()
{
	auto _resourceLoader = Windows::ApplicationModel::Resources::ResourceLoader::GetForCurrentView("foxit.rdk.uiextensions/Resources");
	Platform::String^ _InvalidPageNumberTipsString = _resourceLoader->GetString(L"ThumbnailPageInsertImageFileTooMuchTipsText");
	auto _MessageDialog = ref new Windows::UI::Popups::MessageDialog(_InvalidPageNumberTipsString);

	concurrency::create_task(_MessageDialog->ShowAsync())
		.then([=](Windows::UI::Popups::IUICommand^ command)
	{
		if (command)
		{
			auto _Id = command->Id;
			auto _Label = command->Label;

			if (command->Id != nullptr)
			{
				int const id = safe_cast<IBox<int>^>(command->Id)->Value;
				if (id == 0)
				{

				}
			}
		}
	}, concurrency::task_continuation_context::use_current())
		.then([=](concurrency::task<void> t)
	{
		try
		{
			t.get();
		}
		catch (...)
		{
			OutputDebugString(L"ShowAsync...URL...Exception...\n");
		}
	});
}

void ThumbnailPage::ShowInsertImageFileTooLargeTipsDialog()
{
	auto _resourceLoader = Windows::ApplicationModel::Resources::ResourceLoader::GetForCurrentView("foxit.rdk.uiextensions/Resources");
	Platform::String^ _InvalidPageNumberTipsString = _resourceLoader->GetString(L"ThumbnailPageInsertImageFileTooLargeTipsText");
	auto _MessageDialog = ref new Windows::UI::Popups::MessageDialog(_InvalidPageNumberTipsString);

	concurrency::create_task(_MessageDialog->ShowAsync())
		.then([=](Windows::UI::Popups::IUICommand^ command)
	{
		if (command)
		{
			auto _Id = command->Id;
			auto _Label = command->Label;

			if (command->Id != nullptr)
			{
				int const id = safe_cast<IBox<int>^>(command->Id)->Value;
				if (id == 0)
				{

				}
			}
		}
	}, concurrency::task_continuation_context::use_current())
		.then([=](concurrency::task<void> t)
	{
		try
		{
			t.get();
		}
		catch (...)
		{
			OutputDebugString(L"ShowAsync...URL...Exception...\n");
		}
	});
}

void ThumbnailPage::ShowInsertPdfFileTooLargeTipsDialog()
{
	auto _resourceLoader = Windows::ApplicationModel::Resources::ResourceLoader::GetForCurrentView("foxit.rdk.uiextensions/Resources");
	Platform::String^ _InvalidPageNumberTipsString = _resourceLoader->GetString(L"ThumbnailPageInsertPdfFileTooLargeTipsText");
	auto _MessageDialog = ref new Windows::UI::Popups::MessageDialog(_InvalidPageNumberTipsString);

	concurrency::create_task(_MessageDialog->ShowAsync())
		.then([=](Windows::UI::Popups::IUICommand^ command)
	{
		if (command)
		{
			auto _Id = command->Id;
			auto _Label = command->Label;

			if (command->Id != nullptr)
			{
				int const id = safe_cast<IBox<int>^>(command->Id)->Value;
				if (id == 0)
				{

				}
			}
		}
	}, concurrency::task_continuation_context::use_current())
		.then([=](concurrency::task<void> t)
	{
		try
		{
			t.get();
		}
		catch (...)
		{
			OutputDebugString(L"ShowAsync...URL...Exception...\n");
		}
	});
}
//To do...
void ThumbnailPage::InsertBlankPage(int startIndex, int insertPageCount)
{
	if (Viewer == nullptr)
		return;

	if (insertPageCount > 0 && !m_IsDocumentChanged)
		m_IsDocumentChanged = true;

	auto _VectSize = m_ThumbnailVect->Size;
	int iStart = startIndex;
	int iEnd = startIndex + insertPageCount;
	for (int i = iStart; i < iEnd; i++)
	{
		ThumbnailItem^ item = ref new ThumbnailItem();

		item->PageIndex = i + 1;
		m_ThumbnailVect->InsertAt(i, item);
	}

	int nEnd = (int)m_ThumbnailVect->Size;
	for (int i = startIndex; i < nEnd; i++)
	{
		auto itemTmp = m_ThumbnailVect->GetAt(i);
		if (itemTmp->PageIndex != i + 1)
		{
			itemTmp->PageIndex = i + 1;
		}
		
		auto _Containner = ThumbnailGridView->ContainerFromIndex(i);
		if (_Containner)
		{
			auto _Control = foxit::rdk::uiextensions::GetVisualChild<ThumbnailItemUserControl^>(_Containner);

			if (_Control->PageNumber != i + 1)
			{
				_Control->PageNumber = i + 1;
			}

			if (i >= startIndex && i < startIndex + insertPageCount)
			{
				_Control->ShowImage();
			}
		}
	}

	UpdateEditControlState();
}

//To do...
concurrency::task<void> ThumbnailPage::InsertDocument(int startIndex, Windows::Foundation::Collections::IVector<Windows::Storage::StorageFile^>^ docFiles)
{
	if (Viewer == nullptr)
		return concurrency::create_task([]() {});

	if (docFiles->Size > 0 && !m_IsDocumentChanged)
		m_IsDocumentChanged = true;
	int nPrevPageCount = Viewer->GetDoc()->GetPageCount();

	return concurrency::create_task([]() {});
}
//To do...
concurrency::task<void> ThumbnailPage::InsertImage(int startIndex, Windows::Foundation::Collections::IVector<Windows::Storage::StorageFile^>^ imageFiles)
{
	if (Viewer == nullptr)
		return concurrency::create_task([]() {});
	
	int nPrevPageCount = Viewer->GetDoc()->GetPageCount();

	return concurrency::create_task([]() {});
}

Windows::UI::Xaml::Controls::GridView^ ThumbnailPage::GetItemsGridView()
{
	return ThumbnailGridView;
}

void foxit::rdk::uiextensions::ThumbnailPage::ThumbnailPageSelectAllCheckBox_Checked(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{
	ThumbnailGridView->SelectAll();
}


void foxit::rdk::uiextensions::ThumbnailPage::ThumbnailPageSelectAllCheckBox_Unchecked(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{
	auto _ItemIndexRange = ref new Windows::UI::Xaml::Data::ItemIndexRange(0, m_ThumbnailVect->Size);
	ThumbnailGridView->DeselectRange(_ItemIndexRange);
}


void foxit::rdk::uiextensions::ThumbnailPage::ThumbnailGridView_SelectionChanged(Platform::Object^ sender, Windows::UI::Xaml::Controls::SelectionChangedEventArgs^ e)
{
	UpdateEditControlState();
}


void foxit::rdk::uiextensions::ThumbnailPage::ThumbnailPageSelectAllCheckBox_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{
	if (ThumbnailPageSelectAllCheckBox->IsChecked->Value)
	{
		ThumbnailGridView->SelectAll();
	}
	else
	{
		auto _ItemIndexRange = ref new Windows::UI::Xaml::Data::ItemIndexRange(0, m_ThumbnailVect->Size);
		ThumbnailGridView->DeselectRange(_ItemIndexRange);
	}
}
//To do...
void ThumbnailPage::OpenDialog(InsertType insertType, Platform::Collections::Vector<Windows::Storage::StorageFile^>^ files /* = ref new Platform::Collections::Vector<Windows::Storage::StorageFile^>() */)
{
	if ((insertType == InsertType::ImageFile || insertType == InsertType::PdfFile) && files && files->Size == 0)
	{
		return;
	}

	auto _Control = ref new InsertPageControl();
	auto _HostedDlg = ref new ContentHostedDialog();

	_Control->SetHostedPage(this);
	_Control->SetPageType(insertType);
	_Control->SetSelectedFiles(files);
	_HostedDlg->Content = _Control;
	_HostedDlg->IsLightDismissEnabled = false;
	_HostedDlg->AutoClose = false;
	_HostedDlg->ShowDialog();
}

concurrency::task<bool> ThumbnailPage::IsImageFileToLarge(Platform::Collections::Vector<Windows::Storage::StorageFile^>^ files)
{
	std::vector<concurrency::task<Windows::Storage::FileProperties::BasicProperties^>> _Tasks;
	for each (Windows::Storage::StorageFile^ var in files)
	{
		_Tasks.push_back(concurrency::create_task(var->GetBasicPropertiesAsync()));
	}

	return concurrency::when_all(std::begin(_Tasks), std::end(_Tasks))
		.then([=](std::vector<Windows::Storage::FileProperties::BasicProperties^> returnValues)
	{
		bool bIsToLarge = false;
		auto retSize = returnValues.size();

		for (size_t i = 0; i < retSize; i++)
		{
			auto filePro = returnValues[i];
			auto _fileSize = filePro->Size;

			unsigned long long _maxFileSize = 50 * 1024 * 1024;
			if (_fileSize > _maxFileSize)
			{
				bIsToLarge = true;
				break;
			}
		}

		return bIsToLarge;
	});
}
//To do...
void foxit::rdk::uiextensions::ThumbnailPage::ThumbnailPageInsertBlankPageButton_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{
	Windows::UI::Xaml::Controls::Primitives::FlyoutBase^ _Flyout = Windows::UI::Xaml::Controls::Flyout::GetAttachedFlyout(ThumbnailPageInsertButton);
	_Flyout = Utility::IsMobilePlatform() ? ThumbnailPageInsertMobileButton->Flyout : ThumbnailPageInsertButton->Flyout;
	if (_Flyout)
	{
		_Flyout->Hide();
	}

	OpenDialog(InsertType::BlankPage);
}

//To do...
void foxit::rdk::uiextensions::ThumbnailPage::ThumbnailPageInsertImageButton_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{
	Windows::Storage::Pickers::FileOpenPicker^ _FileOpenPicker = ref new Windows::Storage::Pickers::FileOpenPicker();
	_FileOpenPicker->ViewMode = Windows::Storage::Pickers::PickerViewMode::Thumbnail;
	_FileOpenPicker->SuggestedStartLocation = Windows::Storage::Pickers::PickerLocationId::DocumentsLibrary;
	_FileOpenPicker->FileTypeFilter->Append(L".png");
	_FileOpenPicker->FileTypeFilter->Append(L".bmp");
	_FileOpenPicker->FileTypeFilter->Append(L".jpg");

	std::shared_ptr<Platform::Collections::Vector<Windows::Storage::StorageFile^>^> _spFiles = std::make_shared<Platform::Collections::Vector<Windows::Storage::StorageFile^>^>(nullptr);

	concurrency::create_task(_FileOpenPicker->PickMultipleFilesAsync())
		.then([this, _spFiles](Windows::Foundation::Collections::IVectorView<Windows::Storage::StorageFile^>^ files)
	{
		if (files == nullptr || files->Size == 0)
			throw ref new Platform::NullReferenceException();

		if (files->Size > 10)
		{
			ShowInsertImageFileTooMuchTipsDialog();
			return concurrency::create_task([=]() { return false; });
		}

		Platform::Array<Windows::Storage::StorageFile^>^ _Array = ref new Platform::Array<Windows::Storage::StorageFile^>(files->Size);
		files->GetMany(0, _Array);
		auto pImageFiles = ref new Platform::Collections::Vector<Windows::Storage::StorageFile^>(_Array);
		*_spFiles = pImageFiles;

		return IsImageFileToLarge(pImageFiles);
	})
		.then([=](bool bIsTooLarge)
	{
		if (bIsTooLarge)
		{
			ShowInsertImageFileTooLargeTipsDialog();
		}
		else
		{
			OpenDialog(InsertType::ImageFile, *_spFiles);
		}
	})
		.then([=](concurrency::task<void> t)
	{
		try
		{
			t.get();
		}
		catch (...)
		{

		}
	});

}


void foxit::rdk::uiextensions::ThumbnailPage::ThumbnailPageInsertPdfFileButton_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{
	Windows::Storage::Pickers::FileOpenPicker^ _FileOpenPicker = ref new Windows::Storage::Pickers::FileOpenPicker();
	_FileOpenPicker->ViewMode = Windows::Storage::Pickers::PickerViewMode::Thumbnail;
	_FileOpenPicker->SuggestedStartLocation = Windows::Storage::Pickers::PickerLocationId::DocumentsLibrary;
	_FileOpenPicker->FileTypeFilter->Append(L".pdf");

	std::shared_ptr<Windows::Storage::StorageFile^> spFile = std::make_shared<Windows::Storage::StorageFile^>(nullptr);

	concurrency::create_task(_FileOpenPicker->PickSingleFileAsync())
		.then([=](Windows::Storage::StorageFile^ file)
	{
		if (file == nullptr)
			throw ref new Platform::NullReferenceException();

		*spFile = file;
		return file->GetBasicPropertiesAsync();
	})
		.then([=](Windows::Storage::FileProperties::BasicProperties^ basicProperties)
	{
		auto _fileSize = basicProperties->Size;
		
		unsigned long long _maxFileSize = 100 * 1024 * 1024;
		if (_fileSize > _maxFileSize)
		{
			ShowInsertPdfFileTooLargeTipsDialog();
			return;
		}

		auto pImageFiles = ref new Platform::Collections::Vector<Windows::Storage::StorageFile^>();
		pImageFiles->Append(*spFile);

		OpenDialog(InsertType::PdfFile, pImageFiles);
	})
		.then([=](concurrency::task<void> t)
	{
		try
		{
			t.get();
		}
		catch (...)
		{

		}
	});
}


void foxit::rdk::uiextensions::ThumbnailPage::ThumbnailPageFlyout_Opened(Platform::Object^ sender, Platform::Object^ e)
{
	auto _Flyout = dynamic_cast<Windows::UI::Xaml::Controls::Flyout^>(sender);
	auto _FlyoutPresenter = GetVisualParent<Windows::UI::Xaml::Controls::FlyoutPresenter^>(_Flyout->Content);
	bool bIsMobile = Utility::IsMobilePlatform();

	if (bIsMobile)
	{
		auto _WinRc = Windows::UI::Xaml::Window::Current->Bounds;
		ThumbnailPageInsertBlankPageButton->Width = _WinRc.Width;
		if (_FlyoutPresenter->MaxWidth < _WinRc.Width)
		{
			_FlyoutPresenter->MaxWidth = _WinRc.Width;
		}
	}

	Windows::UI::Xaml::Controls::Primitives::Popup^ _Popup = dynamic_cast<Windows::UI::Xaml::Controls::Primitives::Popup^>(_FlyoutPresenter->Parent);
	if (_Popup)
	{
		double offset = _Popup->VerticalOffset;
		auto _TargetElement = bIsMobile ? ThumbnailPageInsertMobileButton : ThumbnailPageInsertButton;
		auto _ScreenPt = _TargetElement->TransformToVisual(nullptr)->TransformPoint(Windows::Foundation::Point(0, (float)_TargetElement->ActualHeight));
		if (Utility::IsMobilePlatform())
		{
			_Popup->VerticalOffset = offset + 4;
		}
		else
		{
			_Popup->VerticalOffset = _ScreenPt.Y;
		}
	}
}

//to do...
void foxit::rdk::uiextensions::ThumbnailPage::ThumbnailPageSplitButton_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{
	auto _Control = ref new ExtractPageControl();
	auto _HostedDlg = ref new ContentHostedDialog();

	_Control->SetHostedPage(this);
	_HostedDlg->Content = _Control;
	_HostedDlg->IsLightDismissEnabled = false;
	_HostedDlg->AutoClose = false;
	_HostedDlg->ShowDialog();
	return;

	Windows::Storage::Pickers::FileSavePicker^ _FileSavePicker = ref new Windows::Storage::Pickers::FileSavePicker();
	auto _SettingsIdentifier = _FileSavePicker->SettingsIdentifier;
	_FileSavePicker->SuggestedStartLocation = Windows::Storage::Pickers::PickerLocationId::DocumentsLibrary;
	_FileSavePicker->DefaultFileExtension = L".pdf";
	_FileSavePicker->SuggestedFileName = Viewer->GetStorageFile()->DisplayName;
	auto plainTextExtensions = ref new Platform::Collections::Vector<String^>();
	plainTextExtensions->Append(L".pdf");
	_FileSavePicker->FileTypeChoices->Insert("PDF Files", plainTextExtensions);

}
