﻿/**
 * Copyright (C) 2003-2018, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit Mobile PDF SDK to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */


#pragma once

#include "Thumbnail/ThumbnailPage.g.h"

namespace foxit
{
	namespace rdk
	{
		namespace uiextensions
		{
			[Windows::UI::Xaml::Data::Bindable]
			[Windows::Foundation::Metadata::WebHostHidden]
			public ref class ThumbnailItem sealed : public BindableBase
			{
			public:
				ThumbnailItem();

				property int PageIndex
				{
					int get();
					void set(int value);
				}
				property Windows::UI::Xaml::Media::ImageSource^ ThumbImage
				{
					Windows::UI::Xaml::Media::ImageSource^ get();
				}

			private:
				int _PageIndex;
				Windows::UI::Xaml::Media::Imaging::BitmapImage^ _Thumb;
			};

			enum class InsertType;
			/// <summary>
			/// </summary>
			[Windows::Foundation::Metadata::WebHostHidden]
			public ref class ThumbnailPage sealed : public IThumbnailPage
			{
				friend ref class DefaultFoxitUICustomControlAdapter;
			public:
				ThumbnailPage();

				virtual void Show();
				virtual void Hide();

			internal:
				void SetPageView(Platform::Object^ target);
				void SetIsDirectEntryEditMode(bool bIsDirect);
				bool GetIsDirectEntryEditMode();

			internal:
				property Windows::Foundation::TypedEventHandler<Windows::UI::Xaml::Controls::ListViewBase^, Windows::UI::Xaml::Controls::ContainerContentChangingEventArgs^>^ ContainerContentChangingDelegate
				{
					Windows::Foundation::TypedEventHandler<Windows::UI::Xaml::Controls::ListViewBase^, Windows::UI::Xaml::Controls::ContainerContentChangingEventArgs^>^ get();
				}

				property foxit::rdk::PDFViewCtrl^ Viewer
				{
					foxit::rdk::PDFViewCtrl^ get();
				}

				property UIExtensionsManager^ UIExtMgr
				{
					UIExtensionsManager^ get();
				}

				void EnterEditMode();
				void ExitEditMode();

				void InsertBlankPage(int startIndex, int insertPageCount);
				concurrency::task<void> InsertDocument(int startIndex, Windows::Foundation::Collections::IVector<Windows::Storage::StorageFile^>^ docFiles);
				concurrency::task<void> InsertImage(int startIndex, Windows::Foundation::Collections::IVector<Windows::Storage::StorageFile^>^ imageFiles);

				Windows::UI::Xaml::Controls::GridView^ GetItemsGridView();
				void ScrollViewOffset(double voffset);

			private:
				Platform::Collections::Vector<ThumbnailItem^>^ m_ThumbnailVect;
				Platform::WeakReference m_PageViewWeakRef;
				Platform::WeakReference m_UIViewWeakRef;
				Windows::Foundation::TypedEventHandler<Windows::UI::Xaml::Controls::ListViewBase^, Windows::UI::Xaml::Controls::ContainerContentChangingEventArgs^>^ _ContainerContentChangingDelegate;
				bool m_IsDocumentChanged;

				bool m_IsDirectEntryEditMode;

			private:
				void OnInitializePage();
				void UpdateControlState();
				void UpdateEditControlState();
				void UpdateGridViewState();

				void OutputListViewInfo();

				void PageDelete();

				void OpenDialog(InsertType insertType, Platform::Collections::Vector<Windows::Storage::StorageFile^>^ files = ref new Platform::Collections::Vector<Windows::Storage::StorageFile^>());
				concurrency::task<bool> IsImageFileToLarge(Platform::Collections::Vector<Windows::Storage::StorageFile^>^ files);

				void ShowInsertImageFileTooMuchTipsDialog();
				void ShowInsertImageFileTooLargeTipsDialog();
				void ShowInsertPdfFileTooLargeTipsDialog();
				void EnterEditPageMode();

			private:
				void OnLoaded(Platform::Object ^sender, Windows::UI::Xaml::RoutedEventArgs ^e);
				void OnUnloaded(Platform::Object ^sender, Windows::UI::Xaml::RoutedEventArgs ^e);
				void ThumbnailGridView_ItemClick(Platform::Object^ sender, Windows::UI::Xaml::Controls::ItemClickEventArgs^ e);
				void ThumbnailGridView_ContainerContentChanging(Windows::UI::Xaml::Controls::ListViewBase^ sender, Windows::UI::Xaml::Controls::ContainerContentChangingEventArgs^ args);
				void ExitAppBarBtn_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e);
				void EditModeButton_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e);
				void ThumbnailPageRotateLeftButton_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e);
				void ThumbnailPageRotateRightButton_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e);
				void ThumbnailPageInsertButton_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e);
				void ThumbnailPageDeleteButton_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e);
				void ThumbnailPageCompleteButton_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e);
				void ThumbnailGridView_Drop(Platform::Object^ sender, Windows::UI::Xaml::DragEventArgs^ e);
				void ThumbnailGridView_DragOver(Platform::Object^ sender, Windows::UI::Xaml::DragEventArgs^ e);
				void ThumbnailGridView_DragItemsStarting(Platform::Object^ sender, Windows::UI::Xaml::Controls::DragItemsStartingEventArgs^ e);
				void ThumbnailGridView_DragItemsCompleted(Windows::UI::Xaml::Controls::ListViewBase^ sender, Windows::UI::Xaml::Controls::DragItemsCompletedEventArgs^ args);
				void ThumbnailPageSelectAllCheckBox_Checked(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e);
				void ThumbnailPageSelectAllCheckBox_Unchecked(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e);
				void ThumbnailGridView_SelectionChanged(Platform::Object^ sender, Windows::UI::Xaml::Controls::SelectionChangedEventArgs^ e);
				void ThumbnailPageSelectAllCheckBox_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e);
				void ThumbnailPageInsertBlankPageButton_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e);
				void ThumbnailPageInsertImageButton_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e);
				void ThumbnailPageInsertPdfFileButton_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e);
				void ThumbnailPageFlyout_Opened(Platform::Object^ sender, Platform::Object^ e);
				void ThumbnailPageSplitButton_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e);
			};
		}
	}
}
