﻿/**
 * Copyright (C) 2003-2018, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit Mobile PDF SDK to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */


#include "pch.h"
#include "DefaultProgressingUserControl.xaml.h"

#include <sstream>

using namespace foxit::rdk::uiextensions;

using namespace Platform;
using namespace Windows::Foundation;
using namespace Windows::Foundation::Collections;
using namespace Windows::UI::Xaml;
using namespace Windows::UI::Xaml::Controls;
using namespace Windows::UI::Xaml::Controls::Primitives;
using namespace Windows::UI::Xaml::Data;
using namespace Windows::UI::Xaml::Input;
using namespace Windows::UI::Xaml::Media;
using namespace Windows::UI::Xaml::Navigation;

// The User Control item template is documented at http://go.microsoft.com/fwlink/?LinkId=234236

DefaultProgressingUserControl::DefaultProgressingUserControl()
{
	InitializeComponent();
	
	auto _winRect = Windows::UI::Xaml::Window::Current->Bounds;
	this->Width = _winRect.Width;
	this->Height = _winRect.Height;

	this->Loaded += ref new Windows::UI::Xaml::RoutedEventHandler(this, &DefaultProgressingUserControl::OnLoaded);
	this->Unloaded += ref new Windows::UI::Xaml::RoutedEventHandler(this, &DefaultProgressingUserControl::OnUnloaded);
}

void DefaultProgressingUserControl::OnLoaded(Platform::Object ^sender, Windows::UI::Xaml::RoutedEventArgs ^e)
{
	m_ParentPopup = dynamic_cast<Windows::UI::Xaml::Controls::Primitives::Popup^>(this->Parent);
	AttachWindowSizeChangedHandlers();
}

void DefaultProgressingUserControl::OnUnloaded(Platform::Object ^sender, Windows::UI::Xaml::RoutedEventArgs ^e)
{
	DetachWindowSizeChangedHandlers();
}

void DefaultProgressingUserControl::AttachWindowSizeChangedHandlers()
{
	if (m_WindowSizeChangedEventToken.Value == 0)
		m_WindowSizeChangedEventToken = Windows::UI::Xaml::Window::Current->SizeChanged += ref new Windows::UI::Xaml::WindowSizeChangedEventHandler(this, &DefaultProgressingUserControl::OnWindowSizeChanged);
}

void DefaultProgressingUserControl::DetachWindowSizeChangedHandlers()
{
	if (m_WindowSizeChangedEventToken.Value)
	{
		Windows::UI::Xaml::Window::Current->SizeChanged -= m_WindowSizeChangedEventToken;
		m_WindowSizeChangedEventToken.Value = 0;
	}
}

void DefaultProgressingUserControl::OnWindowSizeChanged(Platform::Object ^sender, Windows::UI::Core::WindowSizeChangedEventArgs ^e)
{
	auto _winRect = Windows::UI::Xaml::Window::Current->Bounds;
	this->Width = _winRect.Width;
	this->Height = _winRect.Height;
	std::wstringstream ws;
	ws << L"OnWindowSizeChanged...";
	if (m_ParentPopup)
	{
		ws << L", Point: (" << m_ParentPopup->HorizontalOffset << L", " << m_ParentPopup->VerticalOffset << L")";
	}
	ws << L", Size: (" << this->Width << L", " << this->Height << L")";
	ws << L"\n";
	OutputDebugString(ws.str().c_str());
}

Platform::Object^ DefaultProgressingUserControl::GetContent()
{
	return this;
}

Windows::UI::Color DefaultProgressingUserControl::GetBackgroundColor()
{
	return dynamic_cast<SolidColorBrush^>(OpenProgressRing->Background)->Color;
}

void DefaultProgressingUserControl::SetBackgroundColor(Windows::UI::Color color)
{
	OpenProgressRing->Background = ref new Windows::UI::Xaml::Media::SolidColorBrush(color);
}

Windows::UI::Color DefaultProgressingUserControl::GetForegroundColor()
{
	return dynamic_cast<SolidColorBrush^>(OpenProgressRing->Foreground)->Color;
}

void DefaultProgressingUserControl::SetForegroundColor(Windows::UI::Color color)
{
	OpenProgressRing->Foreground = ref new Windows::UI::Xaml::Media::SolidColorBrush(color);
}
