/**
 * Copyright (C) 2003-2018, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit Mobile PDF SDK to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */


#include "pch.h"
#include "LogRealize.h"


using namespace foxit::rdk::uiextensions;

using namespace Platform;
using namespace concurrency;
using namespace Windows::Storage;
using namespace Windows::Storage::Streams;
using namespace Windows::ApplicationModel;
using namespace Windows::Storage::FileProperties;
using namespace Windows::Storage::Streams;
using namespace std;


LogRealize* LogRealize::m_instance;
LogRealize* LogRealize::GetInstance()
{
	if (m_instance == NULL)
	{
		m_instance = new LogRealize();	
	}

	return m_instance;
}


LogRealize::LogRealize()
{
	
}

void LogRealize::OutputMsg(int msgType, Platform::String^ msg)
{
	switch (msgType)
	{
	case FOXIT_LOG_LEVEL_DEBUG:
		msg = L"DEBUG: " + msg;
		break;
	case FOXIT_LOG_LEVEL_INFO:
		msg = L"INFO: " + msg;
		break;
	case FOXIT_LOG_LEVEL_WARNING:
		msg = L"WARNING: " + msg;
		break;
	case FOXIT_LOG_LEVEL_ERROR:
		msg = L"ERROR: " + msg;
		break;
	case FOXIT_LOG_LEVEL_FATAL:
		msg = L"FATAL: " + msg;
		break;
	default:
		msg = L"DEBUG: " + msg;
		break;
	}
	OutputDebugString(msg->Data());
}

void LogRealize::WriteMsgToLogFile(Platform::String^ msg)
{	
	Platform::String^ localPath = Windows::Storage::ApplicationData::Current->LocalFolder->Path + "\\logFile.txt";

	FILE* fp;
	char *CStr = ConvertToCharStr(localPath);
	fopen_s(&fp, CStr, "r+");
	DWORD DR = GetLastError();
	if (fp == NULL)
	{
		return;
	}
	else
	{
		fclose(fp);
	}

	fopen_s(&fp, CStr, "a+");
	if (fp == NULL)
	{
		return;
	}
	else
	{
		SYSTEMTIME st;
		GetLocalTime(&st);
		fprintf(fp, "[%d-%d-%d %02u:%02u:%02u:%d]  ", st.wYear, st.wMonth, st.wDay, st.wHour, st.wMinute, st.wSecond, st.wMilliseconds);

		char *logoStr = ConvertToCharStr(msg);
		fprintf(fp, "%s", logoStr);
		fprintf(fp, "%s", "\n");
		DWORD DR1 = GetLastError();
		fclose(fp);
	}
}

void LogRealize::LogFileOutMsgFormat(char* pszFormat, ...)
{
	Platform::String^ localPath = Windows::Storage::ApplicationData::Current->LocalFolder->Path + "\\logFile.txt";

	FILE* fp;
	char *CStr = ConvertToCharStr(localPath);
	fopen_s(&fp, CStr, "r+");
	DWORD DR = GetLastError();
	if (fp == NULL)
	{
		return;
	}
	else
	{
		fclose(fp);
	}

	fopen_s(&fp, CStr, "a+");
	if (fp == NULL)
	{
		return;
	}
	else
	{
		SYSTEMTIME st;
		GetLocalTime(&st);
		fprintf(fp, "[%d-%d-%d %02u:%02u:%02u:%d]  ", st.wYear, st.wMonth, st.wDay, st.wHour, st.wMinute, st.wSecond, st.wMilliseconds);

		va_list args;
		va_start(args, pszFormat);
		int len = vfprintf(fp, pszFormat, args);
		fprintf(fp, "%s", "\n");
		fclose(fp);
	}
	return;
}

char * LogRealize::ConvertToCharStr(Platform::String^ outMsg)
{
	const wchar_t* logoData = outMsg->Data();
	size_t len = wcslen(logoData) + 1;
	size_t converted = 0;
	char *logoStr;
	logoStr = (char*)malloc(len*sizeof(char));
	wcstombs_s(&converted, logoStr, len, logoData, _TRUNCATE);
	return logoStr;
}
