﻿/**
 * Copyright (C) 2003-2018, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit Mobile PDF SDK to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */


#include "pch.h"
#include "MessageDlg.h"

using namespace foxit::rdk::uiextensions;

using namespace Platform;
using namespace Windows::Foundation;
using namespace Windows::Foundation::Collections;
using namespace Windows::UI::Xaml;
using namespace Windows::UI::Xaml::Controls;
using namespace Windows::UI::Xaml::Data;
using namespace Windows::UI::Xaml::Documents;
using namespace Windows::UI::Xaml::Input;
using namespace Windows::UI::Xaml::Interop;
using namespace Windows::UI::Xaml::Media;

// The User Control item template is documented at http://go.microsoft.com/fwlink/?LinkId=234235

MessageDlg::MessageDlg()
{
	DefaultStyleKey = "foxit.rdk.uiextensions.MessageDlg";

	if (Windows::ApplicationModel::DesignMode::DesignModeEnabled)
		return;
	
	m_isGoBack = false;

	if (Windows::System::Profile::AnalyticsInfo::VersionInfo->DeviceFamily == "Windows.Mobile")
	{
		this->VerticalAlignment = Windows::UI::Xaml::VerticalAlignment::Top;
	}
	this->Loaded += ref new Windows::UI::Xaml::RoutedEventHandler(this, &MessageDlg::OnLoaded);
	this->Unloaded += ref new Windows::UI::Xaml::RoutedEventHandler(this, &MessageDlg::OnUnloaded);
}

Windows::UI::Xaml::Controls::Primitives::Popup^ MessageDlg::ChildWindowPopup::get()
{
	return m_ChildWindowPopup;
}

static Windows::UI::Xaml::DependencyProperty^ _IsStoreAppBarStateProperty =
Windows::UI::Xaml::DependencyProperty::Register("UIOperation", TypeName(bool::typeid), TypeName(MessageDlg::typeid),
ref new PropertyMetadata(false, ref new PropertyChangedCallback(&MessageDlg::OnIsStoreAppBarStatePropertyChanged)));

Windows::UI::Xaml::DependencyProperty^ MessageDlg::IsStoreAppBarStateProperty::get()
{
	return _IsStoreAppBarStateProperty;
}

bool MessageDlg::IsStoreAppBarState::get()
{
	return (bool)GetValue(IsStoreAppBarStateProperty);
}

void MessageDlg::IsStoreAppBarState::set(bool value)
{
	SetValue(IsStoreAppBarStateProperty, value);
}

void MessageDlg::OnIsStoreAppBarStatePropertyChanged(Windows::UI::Xaml::DependencyObject^ obj, Windows::UI::Xaml::DependencyPropertyChangedEventArgs^ args)
{
	MessageDlg^ _control = static_cast<MessageDlg^>(obj);

	bool _oldValue = (bool)(args->OldValue);
	bool _newValue = (bool)(args->NewValue);
}

void MessageDlg::OnApplyTemplate()
{
	ContentControl::OnApplyTemplate();

	m_Body = safe_cast<Windows::UI::Xaml::Controls::ContentPresenter^>(GetTemplateChild(L"Body"));
	m_BackgroundRect = safe_cast<Windows::UI::Xaml::Shapes::Rectangle^>(GetTemplateChild(L"BackgroundRect"));

	InitializeMessagePrompt();
}

void MessageDlg::InitializeMessagePrompt()
{
	if (m_Body == nullptr)
	{
		return;
	}

	auto _WinRect = Windows::UI::Xaml::Window::Current->Bounds;
	m_BackgroundRect->Visibility = Windows::UI::Xaml::Visibility::Visible;
	m_Body->Content = MessageContent;
	this->Width = _WinRect.Width;
	this->Height = _WinRect.Height;
}

Windows::UI::Xaml::Controls::Page^ MessageDlg::Page::get()
{
	return foxit::rdk::uiextensions::GetVisualChild<Windows::UI::Xaml::Controls::Page^>(RootVisual);
}

Windows::UI::Xaml::Controls::Frame^ MessageDlg::RootVisual::get()
{
	return Windows::UI::Xaml::Application::Current == nullptr ? nullptr : dynamic_cast<Windows::UI::Xaml::Controls::Frame^>(Windows::UI::Xaml::Window::Current->Content);
}

Platform::Object^ MessageDlg::MessageContent::get()
{
	return this->Content;
}

void MessageDlg::MessageContent::set(Platform::Object^ value)
{
	this->Content = value;
}

void MessageDlg::Hide()
{
	if (ChildWindowPopup != nullptr)
	{
		ChildWindowPopup->IsOpen = false;
	}

	auto bIsTypePresent = Windows::Foundation::Metadata::ApiInformation::IsTypePresent("Windows.Phone.UI.Input.HardwareButtons");
	if (bIsTypePresent)
	{
		Windows::Phone::UI::Input::HardwareButtons::BackPressed -= m_BackPresseEventToken;
		m_BackPresseEventToken.Value = 0;
	}
		
	if (IsStoreAppBarState && Page && Page->BottomAppBar != nullptr)
		Page->BottomAppBar->Visibility = m_AppBarVisual;
}

void MessageDlg::Show()
{
	if (m_ChildWindowPopup == nullptr)
	{
		m_ChildWindowPopup = ref new Windows::UI::Xaml::Controls::Primitives::Popup();
		m_ChildWindowPopup->Child = this;
	}

	if (m_ChildWindowPopup != nullptr && Windows::UI::Xaml::Window::Current->Content != nullptr)
	{
		InitializeMessagePrompt();
		m_ChildWindowPopup->IsOpen = true;
		this->Focus(Windows::UI::Xaml::FocusState::Programmatic);
	}

	auto bIsTypePresent = Windows::Foundation::Metadata::ApiInformation::IsTypePresent("Windows.Phone.UI.Input.HardwareButtons");
	if (bIsTypePresent)
	{
		if (m_BackPresseEventToken.Value == 0)
		{
			m_BackPresseEventToken = Windows::Phone::UI::Input::HardwareButtons::BackPressed += ref new Windows::Foundation::EventHandler<Windows::Phone::UI::Input::BackPressedEventArgs^>(this, &MessageDlg::HardwareButtons_BackPressed);
		}
	}

	if (IsStoreAppBarState && Page != nullptr && Page->BottomAppBar)
	{
		m_AppBarVisual = Page->BottomAppBar->Visibility;
		Page->BottomAppBar->IsSticky = true;
		Page->BottomAppBar->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
	}
	
}

bool MessageDlg::IsOpen::get()
{
	return ChildWindowPopup != nullptr && ChildWindowPopup->IsOpen;
}

void MessageDlg::HardwareButtons_BackPressed(Platform::Object^ sender, Windows::Phone::UI::Input::BackPressedEventArgs^ e)
{
	Windows::UI::Xaml::Controls::Frame^ _frame = dynamic_cast<Windows::UI::Xaml::Controls::Frame^>(Windows::UI::Xaml::Window::Current->Content);
	if (IsOpen)
	{
		e->Handled = true;
		Hide();
	}
	if (_frame == nullptr)
	{
		return;
	}
	else
	{
		if (m_isGoBack)
		{
			_frame->GoBack();
		}
	}
}

void MessageDlg::OnLoaded(Platform::Object ^sender, Windows::UI::Xaml::RoutedEventArgs ^e)
{
	__windowSizeChangedEventRegistrationToken = Windows::UI::Xaml::Window::Current->SizeChanged += ref new Windows::UI::Xaml::WindowSizeChangedEventHandler(this, &MessageDlg::OnWindowSizeChanged);
}


void MessageDlg::OnUnloaded(Platform::Object ^sender, Windows::UI::Xaml::RoutedEventArgs ^e)
{
	Windows::UI::Xaml::Window::Current->SizeChanged -= __windowSizeChangedEventRegistrationToken;
	__windowSizeChangedEventRegistrationToken.Value = 0;
}


void MessageDlg::OnWindowSizeChanged(Platform::Object ^sender, Windows::UI::Core::WindowSizeChangedEventArgs ^e)
{
	auto _WinRect = Windows::UI::Xaml::Window::Current->Bounds;
	this->Width = _WinRect.Width;
	this->Height = _WinRect.Height;
}

void MessageDlg::SetIsBackToPreFrame(bool isGoBack)
{
	m_isGoBack = isGoBack;
}
