/**
 * Copyright (C) 2003-2018, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit Mobile PDF SDK to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */


#include "pch.h"
#include "ProgressBarFactory.h"
#include "DefaultProgressingUserControl.xaml.h"

using namespace foxit::rdk::uiextensions;
using namespace Platform;

ProgressBarFactory::ProgressBarFactory()
{
	m_Popup = ref new Windows::UI::Xaml::Controls::Primitives::Popup();
	auto _bar = ref new DefaultProgressingUserControl();
	m_PragressBar = _bar;
	m_PopupContent = _bar;
	m_Popup->Child = m_PopupContent;
	m_Popup->IsLightDismissEnabled = false;

	m_Popup->Opened += ref new Windows::Foundation::EventHandler<Platform::Object ^>(this, &ProgressBarFactory::OnPopupOpened);
	m_Popup->Closed += ref new Windows::Foundation::EventHandler<Platform::Object ^>(this, &ProgressBarFactory::OnPopupClosed);
}

void ProgressBarFactory::Show()
{
	if (!m_Popup->IsOpen)
		m_Popup->IsOpen = true;
}

void ProgressBarFactory::Close()
{
	if (m_Popup->IsOpen)
		m_Popup->IsOpen = false;
}

IProgressBar^ ProgressBarFactory::GetProgressBar()
{
	return m_PragressBar;
}


void ProgressBarFactory::OnPopupOpened(Platform::Object ^sender, Platform::Object ^args)
{
	if (Windows::UI::Xaml::Application::Current)
	{
		auto _Frame = dynamic_cast<Windows::UI::Xaml::Controls::Frame^>(Windows::UI::Xaml::Window::Current->Content);
		if (_Frame)
		{
			auto _Page = GetVisualChild<Windows::UI::Xaml::Controls::Page^>(_Frame);
			if (_Page && _Page->BottomAppBar)
			{
				m_AppBarVisual = _Page->BottomAppBar->Visibility;
				_Page->BottomAppBar->IsSticky = true;
			}
		}
	}
}


void ProgressBarFactory::OnPopupClosed(Platform::Object ^sender, Platform::Object ^args)
{
	if (Windows::UI::Xaml::Application::Current)
	{
		auto _Frame = dynamic_cast<Windows::UI::Xaml::Controls::Frame^>(Windows::UI::Xaml::Window::Current->Content);
		if (_Frame)
		{
			auto _Page = GetVisualChild<Windows::UI::Xaml::Controls::Page^>(_Frame);
			if (_Page && _Page->BottomAppBar)
			{
			}
		}
	}
}

void ProgressBarFactory::HardwareButtons_BackPressed(Platform::Object^ sender, Windows::Phone::UI::Input::BackPressedEventArgs^ e)
{
	Windows::UI::Xaml::Controls::Frame^ _frame = dynamic_cast<Windows::UI::Xaml::Controls::Frame^>(Windows::UI::Xaml::Window::Current->Content);
	if (_frame == nullptr)
	{
		return;
	}

	if (m_Popup->IsOpen)
	{
		e->Handled = true;
		Close();
	}
}
