/**
 * Copyright (C) 2003-2018, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit Mobile PDF SDK to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */


#include "pch.h"
#include "UtilityToolsFactory.h"
#include "MessageDlg.h"
#include "ProgressBarFactory.h"

#include <sstream>
#include <thread>
#include <mutex>

using namespace foxit::rdk::uiextensions;
using namespace Windows::Foundation;
using namespace Platform;


class SoldTicket
{
public:
	SoldTicket();

public:
	void SoldThreadA();
	void SoldThreadB();
	void SoldThreadC();

	bool Init();

private:
	int m_Tickets;
	std::mutex m_lockmutex;
};

SoldTicket::SoldTicket()
{
	int x = 3;
	decltype(x) y = x;

	auto func = []()
	{
		std::wstringstream ws;
		ws << "Hello world...\n";
		OutputDebugString(ws.str().c_str());
	};
	func(); // now call the function 
}

bool SoldTicket::Init()
{
	m_Tickets = 100;

	std::thread _threadone = std::thread(&SoldTicket::SoldThreadA, this);
	std::thread _threadtwo = std::thread(&SoldTicket::SoldThreadB, this);
	std::thread _threadthree = std::thread(&SoldTicket::SoldThreadC, this);
	_threadone.detach();
	_threadtwo.detach();
	_threadthree.detach();
	return true;
}

template<typename Func>
std::vector<std::string> findMatchingAddresses(Func func)
{
	std::vector<std::string> results;
	for (auto itr = _addresses.begin(), end = _addresses.end(); itr != end; ++itr)
	{
		// call the function passed into findMatchingAddresses and see if it matches 
		if (func(*itr))
		{
			results.push_back(*itr);
		}
	}
	return results;
}

void SoldTicket::SoldThreadA()
{
	while (true)
	{
		m_lockmutex.lock();
		if (m_Tickets > 0)
		{
			Concurrency::wait(3);
			OutputDebugString(L"A Sell ");
			OutputDebugString(m_Tickets.ToString()->Data());
			OutputDebugString(L"\n");
			m_Tickets--;
			m_lockmutex.unlock();
		}
		else
		{
			m_lockmutex.unlock();
			break;
		}
	}
}

void SoldTicket::SoldThreadB()
{
	while (true)
	{
		m_lockmutex.lock();
		if (m_Tickets > 0)
		{
			concurrency::wait(3);
			OutputDebugString(L"B Sell ");
			OutputDebugString(m_Tickets.ToString()->Data());
			OutputDebugString(L"\n");
			m_Tickets--;
			m_lockmutex.unlock();
		}
		else
		{
			m_lockmutex.unlock();
			break;
		}
	}
}

void SoldTicket::SoldThreadC()
{
	while (true)
	{
		m_lockmutex.lock();
		if (m_Tickets > 0)
		{
			concurrency::wait(3);
			OutputDebugString(L"C Sell ");
			OutputDebugString(m_Tickets.ToString()->Data());
			OutputDebugString(L"\n");
			m_Tickets--;
			m_lockmutex.unlock();
		}
		else
		{
			m_lockmutex.unlock();
			break;
		}
	}
}

IProgressBarFactory^ UtilityToolsFactory::GetProgressBarFactory()
{
	return ref new ProgressBarFactory();
}

MessageDlg^ UtilityToolsFactory::GetMessageDlg()
{
	return ref new MessageDlg();
}

void UtilityToolsFactory::Test()
{
	auto _mes = ref new MessageDlg();
	auto _btn = ref new Windows::UI::Xaml::Controls::Button();
	_btn->Content = L"FFFFFFFFFFFFF";
	_mes->MessageContent = _btn;
	_mes->Show();
}

#define		EPS .1e-6

int	ComputationalSegment::dblcmp(double d)
{
	if (fabs(d)<EPS)
		return 0;
	return (d>0) ? 1 : -1;
}

double	ComputationalSegment::det(double x1, double y1, double x2, double y2)
{
	return x1*y2 - x2*y1;
}

double	ComputationalSegment::cross(Point a, Point b, Point c)
{
	return	det(b.X - a.X, b.Y - a.Y, c.X - a.X, c.Y - a.Y);
}

double	ComputationalSegment::dotdet(double x1, double y1, double x2, double y2)
{
	return x1*x2 + y1*y2;
}

double	ComputationalSegment::dot(Point a, Point b, Point c)
{
	return	dotdet(b.X - a.X, b.Y - a.Y, c.X - a.X, c.Y - a.Y);
}

int	ComputationalSegment::betweenCmp(Point a, Point b, Point c)
{
	return dblcmp(dot(a, b, c));
}

/*
	Parameter:
	The two ends of two lines
	A pointer to the point Point
	Return value :
	Disjoint when the return value is 0
	When the return value is 1, the norm intersects, and the intersection point is in r
	Nonstandard intersection when return value is 2
*/
int	ComputationalSegment::LineSegmentCross(Windows::Foundation::Point a, Windows::Foundation::Point b, Windows::Foundation::Point c, Windows::Foundation::Point d)
{
	double	s1, s2, s3, s4;
	int		d1, d2, d3, d4;

	d1 = dblcmp(s1 = cross(a, b, c));
	d2 = dblcmp(s2 = cross(a, b, d));
	d3 = dblcmp(s3 = cross(c, d, a));
	d4 = dblcmp(s4 = cross(c, d, b));

	if ((d1^d2) == -2 && (d3^d4) == -2)
	{
		m_ptIntersection.X = (float)((c.X*s2 - d.X*s1) / (s2 - s1));
		m_ptIntersection.Y = (float)((c.Y*s2 - d.Y*s1) / (s2 - s1));
		return 1;
	}

	if (d1 == 0 && betweenCmp(c, a, b) <= 0 ||
		d2 == 0 && betweenCmp(d, a, b) <= 0 ||
		d3 == 0 && betweenCmp(a, c, d) <= 0 ||
		d4 == 0 && betweenCmp(b, c, d) <= 0)
		return 2;

	return 0;
}

bool ComputationalSegment::IsLineSegmentRectIntersection(Point start_p, Point end_p, Rect rect)
{
	double a = start_p.Y - end_p.Y;
	double b = end_p.X - start_p.X;
	double c = start_p.X* end_p.Y - end_p.X* start_p.Y;

	double left = rect.Left;
	double top = rect.Top;
	double right = rect.Right;
	double bottom = rect.Bottom;

	/// ideas: first look at the straight line does not intersect the rectangle intersection, if need to "F",
	/// if the intersection point is to see whether the line on the same side of the rectangle(i.e.two X(y) coordinates than the small rectangular x(y) coordinates small or large),
	/// if on the same side as "F", otherwise it is the intersection of the situation.
	if ((a* left + b*top + c >= 0 && a* right + b* bottom + c <= 0) ||
		(a* left + b*top + c <= 0 && a* right + b* bottom + c >= 0) ||
		(a* left + b*bottom + c >= 0 && a* right + b* top + c <= 0) ||
		(a* left + b*bottom + c >= 0 && a* right + b* top + c <= 0))
	{
		if (left > right)
		{
			//swap(left, right);
			double temp;
			temp = left;
			left = right;
			right = temp;
			
		}
		if (top < bottom)
		{
			//swap(top, bottom);
			double temp;
			temp = top;
			top = bottom;
			bottom = temp;
		}
		if ((start_p.X < left && end_p.X < left) ||
			(start_p.X > right && end_p.X < left) ||
			(start_p.Y > top && end_p.Y > top) ||
			(start_p.Y < bottom && end_p.Y < bottom))  /// Determine whether the line at the side of a rectangle
		{
			return false;
		}
		else
		{
			return true;
		}
	}
	else
	{
		return false;
	}
}

