/**
 * Copyright (C) 2003-2018, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit Mobile PDF SDK to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */


#pragma once

#include <time.h>

namespace foxit
{
	namespace rdk
	{
		namespace uiextensions
		{
			template<typename T>
			T GetVisualChild(Windows::UI::Xaml::DependencyObject^ parent)
			{
				if (parent == nullptr)
					return nullptr;
				T child = nullptr;
				int numVisuals = Windows::UI::Xaml::Media::VisualTreeHelper::GetChildrenCount(parent);
				for (int i = 0; i < numVisuals; i++)
				{
					Windows::UI::Xaml::DependencyObject ^d = (Windows::UI::Xaml::DependencyObject^)Windows::UI::Xaml::Media::VisualTreeHelper::GetChild(parent, i);
					child = dynamic_cast<T>(d);
					if (!child)
						child = GetVisualChild<T>(d);
					if (child)
						break;
				}
				return child;
			};

			template<typename T>
			T GetVisualParent(Windows::UI::Xaml::DependencyObject^ Child)
			{
#define  ISNEEDDBGMSG 0
				T parent = nullptr;
				if (Child == nullptr)
					return nullptr;
				DependencyObject ^d = (DependencyObject^)Windows::UI::Xaml::Media::VisualTreeHelper::GetParent(Child);
				if (!d)
				{
					auto _framework = dynamic_cast<Windows::UI::Xaml::FrameworkElement^>(Child);
					if (_framework && _framework->Parent)
					{
						parent = dynamic_cast<T>(_framework->Parent);
#if ISNEEDDBGMSG
						if (parent)
						{
							OutputDebugString(parent->ToString()->Data());
							OutputDebugString(L"\r\n");
						}
#endif
						return parent;
					}
					return nullptr;
				}
#if ISNEEDDBGMSG
				OutputDebugString(d->ToString()->Data());
				OutputDebugString(L"\r\n");
#endif
				parent = dynamic_cast<T>(d);
				if (parent == nullptr)
				{
					parent = GetVisualParent<T>(d);
					if (parent != nullptr)
						return parent;
					else
						return nullptr;
				}
				return parent;
			}

			template<typename T>
			T GetVisualParentByName(Windows::UI::Xaml::DependencyObject^ Child, Platform::String^ name)
			{
				if (Child == nullptr)
					return nullptr;
				T parent = nullptr;
				DependencyObject ^d = (DependencyObject^)Windows::UI::Xaml::Media::VisualTreeHelper::GetParent(Child);
				if (!d)
					return nullptr;

				parent = dynamic_cast<T>(d);
				if (parent == nullptr)
				{
					parent = GetVisualParent<T>(d);
					if (parent != nullptr && parent->Name == name)
						return parent;
					else
						return nullptr;
				}
				return parent;
			}

			template<typename T>
			T GetVisualChildByName(Windows::UI::Xaml::DependencyObject^ parent, Platform::String^ name)
			{
				T child = nullptr;
				if (parent == nullptr)
					return nullptr;
				int numVisuals = Windows::UI::Xaml::Media::VisualTreeHelper::GetChildrenCount(parent);
				for (int i = 0; i < numVisuals; i++)
				{
					Windows::UI::Xaml::DependencyObject ^d = (Windows::UI::Xaml::DependencyObject^)Windows::UI::Xaml::Media::VisualTreeHelper::GetChild(parent, i);
					child = dynamic_cast<T>(d);
					if (child == nullptr || child->Name != name)
					{
						child = GetVisualChildByName<T>(d, name);
						if (child != nullptr)
							return child;
					}
					else
						return child;
				}
				return nullptr;
			}

			static Platform::String^ GetTabString(int count)
			{
				if (count == 0)
					return ref new Platform::String();
				Platform::String^ tmp;
				for (int i = 0; i < count; i++)
				{
					tmp = tmp + "\t";
				}
				return tmp;
			}

			static void PrintVisualTreeChildRecursion(Windows::UI::Xaml::DependencyObject ^parent, int depth)
			{
				auto frameelement = safe_cast<Windows::UI::Xaml::FrameworkElement^>(parent);
				if (frameelement)
				{
#ifdef _DEBUG
					OutputDebugString(GetTabString(depth)->Data());
					OutputDebugString(L"Type: ");
					OutputDebugString(frameelement->ToString()->Data());
					OutputDebugString(L", Name: ");
					OutputDebugString(frameelement->Name->Data());
					OutputDebugString(L", ActualWidth: ");
					OutputDebugString(frameelement->ActualWidth.ToString()->Data());
					OutputDebugString(L", ActualHeight: ");
					OutputDebugString(frameelement->ActualHeight.ToString()->Data());
					OutputDebugString(L", IsHitTestVisible: ");
					OutputDebugString(frameelement->IsHitTestVisible.ToString()->Data());
					OutputDebugString(L"\n");
#endif
				}
				else
				{
					return;
				}
				int numVisuals = Windows::UI::Xaml::Media::VisualTreeHelper::GetChildrenCount(parent);
				for (int i = 0; i < numVisuals; i++) {
					Windows::UI::Xaml::DependencyObject ^d = (Windows::UI::Xaml::DependencyObject^) Windows::UI::Xaml::Media::VisualTreeHelper::GetChild(parent, i);

					if (d)
						PrintVisualTreeChildRecursion(d, depth + 1);
				}
				return;
			};

			static void PrintVisualTree(Windows::UI::Xaml::DependencyObject ^parent)
			{
				PrintVisualTreeChildRecursion(parent, 0);
			}

			inline double random(double start, double end)
			{
				static bool issrand = false;
				if (!issrand)
				{
					std::srand((unsigned int)time(NULL));
					issrand = true;
				}
				return start + (end - start)*rand() / (RAND_MAX + 1.0);
			}

			template <class Function>
			__int64 time_call(Function&& f)
			{
				__int64 begin = GetTickCount64();
				f();
				return GetTickCount64() - begin;
			}
		}
	}
}