﻿/**
* Copyright (C) 2003-2018, Foxit Software Inc..
* All Rights Reserved.
*
* http://www.foxitsoftware.com
*
* The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
* distribute any parts of Foxit Mobile PDF SDK to third party or public without permission unless an agreement
* is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
* Review legal.txt for additional license and legal information.
*/

#include "pch.h"
#include "UIExtensions.h"
#include "MainPageAppBar.xaml.h"
#include "FormFiller\PDFFormAbstraction.h"
#include "FormFiller\PDFFormImplementor.h"
#include "FormFiller\FormOperationBarControl.xaml.h"
#include "Thumbnail\ThumbnailPage.xaml.h"
#include "UIHostedContorl.xaml.h"
#include "Annotations\AnnotOperatingOverlayControl.xaml.h"
#include "Annotations\AnnotPropertyAppBar.xaml.h"
#include "FloatWindow\AnnotPropertyPhone.xaml.h"
#include "FloatWindow\DrawSignatureControl.xaml.h"
#include "FloatWindow\DirectControl.xaml.h"
#include "FloatWindow\PageSlideHorViewControl.xaml.h"
#include "FloatWindow\PageSliderViewControl.xaml.h"
#include "FloatWindow\PageNumberControl.xaml.h"
#include "FloatWindow\PageNumberShowControl.xaml.h"
#include "Annotations\EditAnnotUnDoAction.h"
#include "Annotations\ContextMenuUserControl.xaml.h"
#include "Annotations\FreeTextEventPeer.h"
#include "Annotations\AnnotNoteUserControl.xaml.h"
#include "Undo\UndoAppBarPhoneControl.xaml.h"
#include "FloatWindow\UserTipControl.xaml.h"
#include "Annotations\PageViewAnnotMgrPeer.h"
#include "Undo\DocUndo.h"
#include "Text\FindTextControl.xaml.h"

#include <regex>

using namespace foxit::rdk::uiextensions;
using namespace foxit::rdk;

using namespace Platform;
using namespace Windows::Foundation;
using namespace Windows::UI::Xaml::Media;
using namespace Windows::Storage;
using namespace Windows::Storage::Streams;
using namespace Windows::Storage::AccessCache;
using namespace Windows::Storage::Pickers;
using namespace concurrency;

namespace foxit
{
	namespace rdk
	{
		namespace uiextensions
		{
			FormFillTimerWrapper::FormFillTimerWrapper(FS_LPVOID clientData, int uElapse, foxit::pdf::interform::ITimerCallback^ lpTimerFunc)
			{
				m_FormFillClientData = clientData;
				m_lpTimerFunc = lpTimerFunc;
				m_uElapse = uElapse;

				m_FormFillTimer = ref new Windows::UI::Xaml::DispatcherTimer();
				Windows::Foundation::TimeSpan _TimeSpan;
				_TimeSpan.Duration = uElapse * 10000;
				m_FormFillTimer->Interval = _TimeSpan;
				m_FormFillTimer->Tick += ref new Windows::Foundation::EventHandler<Platform::Object ^>(this, &FormFillTimerWrapper::OnTick);
			}

			FormFillTimerWrapper::~FormFillTimerWrapper()
			{
				if (m_FormFillTimer->IsEnabled)
					m_FormFillTimer->Stop();

				m_FormFillClientData = 0;
				m_lpTimerFunc = nullptr;
				m_FormFillTimer = nullptr;
			}

			UINT_PTR FormFillTimerWrapper::TimerId::get()
			{
				return (UINT_PTR)m_FormFillClientData;
			}

			int FormFillTimerWrapper::Elapse::get()
			{
				return m_uElapse;
			}

			void FormFillTimerWrapper::SetTimerElapse(int uElapse)
			{
				if (uElapse == m_uElapse)
					return;

				if (m_FormFillTimer->IsEnabled)
					m_FormFillTimer->Stop();

				Windows::Foundation::TimeSpan _TimeSpan;
				_TimeSpan.Duration = uElapse * 10000;
				m_FormFillTimer->Interval = _TimeSpan;
			}

			void FormFillTimerWrapper::Start()
			{
				m_FormFillTimer->Start();
			}

			void FormFillTimerWrapper::Stop()
			{
				if (m_FormFillTimer->IsEnabled)
				{
					m_FormFillTimer->Stop();
				}
			}

			void FormFillTimerWrapper::OnTick(Platform::Object ^sender, Platform::Object ^args)
			{
				if (m_lpTimerFunc)
				{
					m_lpTimerFunc->OnTimer((int)TimerId);
				}
			}
		}
	}
}

UIExtensionsManager::UIExtensionsManager(foxit::rdk::PDFViewCtrl^ ctrl)
{
	m_control = ctrl;
	IsShowAnnot = true;

	m_control->Loaded += ref new Windows::UI::Xaml::RoutedEventHandler(this, &foxit::rdk::uiextensions::UIExtensionsManager::OnLoaded);
	m_control->Unloaded += ref new Windows::UI::Xaml::RoutedEventHandler(this, &foxit::rdk::uiextensions::UIExtensionsManager::OnUnloaded);
	m_control->PageLoadedEvent += ref new foxit::rdk::PageLoadedEvent(this, &foxit::rdk::uiextensions::UIExtensionsManager::OnPageLoadedEvent);
	m_control->PageUnloadedEvent += ref new foxit::rdk::PageUnloadedEvent(this, &foxit::rdk::uiextensions::UIExtensionsManager::OnPageUnloadedEvent);
	m_control->PageVisibleEvent += ref new foxit::rdk::PageVisibleEvent(this, &foxit::rdk::uiextensions::UIExtensionsManager::OnPageVisibleEvent);
	m_control->PageInvisibleEvent += ref new foxit::rdk::PageInvisibleEvent(this, &foxit::rdk::uiextensions::UIExtensionsManager::OnPageInvisibleEvent);
	m_control->PageChangedEvent += ref new foxit::rdk::PageChangedEvent(this, &foxit::rdk::uiextensions::UIExtensionsManager::OnPageChangedEvent);
	m_control->PageLayoutModeChangedEvent += ref new foxit::rdk::PageLayoutModeChangedEvent(this, &foxit::rdk::uiextensions::UIExtensionsManager::OnPageLayoutModeChangedEvent);
	m_control->RenderPageEvent += ref new foxit::rdk::RenderPageEvent(this, &foxit::rdk::uiextensions::UIExtensionsManager::OnDrawAnnotEvent);
	m_control->OpenPdfFileEvent += ref new foxit::rdk::OpenDocEvent(this, &foxit::rdk::uiextensions::UIExtensionsManager::OnOpenPdfFileEvent);
	m_control->ClosePdfFileEvent += ref new foxit::rdk::CloseDocEvent(this, &foxit::rdk::uiextensions::UIExtensionsManager::OnClosePdfFileEvent);

	m_textManager = nullptr;
	m_isDesktopPlatform = true;

	m_FormFillTimers = ref new Platform::Collections::Vector<FormFillTimerWrapper^>();

	m_scale = 1;
	
	m_textManager = ref new PDFTextManager(m_control);

	m_TappedTimer = ref new Windows::UI::Xaml::DispatcherTimer();
	auto _UISettings = ref new Windows::UI::ViewManagement::UISettings();
	auto _DoubleClickTime = _UISettings->DoubleClickTime;
	Windows::Foundation::TimeSpan _HideTimerTimeSpan;
	_HideTimerTimeSpan.Duration = _DoubleClickTime * 10000000 / 1000; 
	m_TappedTimer->Interval = _HideTimerTimeSpan;
	m_TappedTimer->Tick += ref new Windows::Foundation::EventHandler<Platform::Object ^>(this, &UIExtensionsManager::OnTappedTick);

	m_BottomBarVisibleTimer = ref new Windows::UI::Xaml::DispatcherTimer();
	_HideTimerTimeSpan.Duration = 4 * 10000000; 
	m_BottomBarVisibleTimer->Interval = _HideTimerTimeSpan;
	m_BottomBarVisibleTimer->Tick += ref new Windows::Foundation::EventHandler<Platform::Object ^>(this, &UIExtensionsManager::OnTick);

	m_PointerWheelScaleTimer = ref new Windows::UI::Xaml::DispatcherTimer();
	_HideTimerTimeSpan.Duration = 10000000 / 2; 
	m_PointerWheelScaleTimer->Interval = _HideTimerTimeSpan;
	m_PointerWheelScaleTimer->Tick += ref new Windows::Foundation::EventHandler<Platform::Object ^>(this, &UIExtensionsManager::OnPointerWheelScaleOperationTick);

	Windows::Foundation::TimeSpan _TimeSpan;
	_TimeSpan.Duration = 300000000L; 
	
	if (AppConfig::IsAutoSave)
	{
		m_AutoSaveFileTimer = ref new Windows::UI::Xaml::DispatcherTimer();
		Windows::Foundation::TimeSpan _SaveTimeSpan;
		_SaveTimeSpan.Duration = 5000000000L;
		m_AutoSaveFileTimer->Interval = _SaveTimeSpan;
		m_AutoSaveFileTimer->Tick += ref new Windows::Foundation::EventHandler<Platform::Object ^>(this, &UIExtensionsManager::OnAutoSaveFileTick);
	}

	m_CopyFileTimer = ref new Windows::UI::Xaml::DispatcherTimer();
	Windows::Foundation::TimeSpan _CopyFileTimeSpan;
	_CopyFileTimeSpan.Duration = 5000000L;
	m_CopyFileTimer->Interval = _CopyFileTimeSpan;
	m_CopyFileTimer->Tick += ref new Windows::Foundation::EventHandler<Platform::Object ^>(this, &UIExtensionsManager::OnCopyFileTick);

	m_pProgressBar = ref new ProgressBarFactory();

	m_FormFillTimers = ref new Platform::Collections::Vector<FormFillTimerWrapper^>();

	

	if (Windows::System::Profile::AnalyticsInfo::VersionInfo->DeviceFamily == "Windows.Desktop")
	{
		m_isDesktopPlatform = true;
	}
	else
	{
		m_isDesktopPlatform = false;
		auto statusBar = Windows::UI::ViewManagement::StatusBar::GetForCurrentView();
		statusBar->HideAsync();
	}
	m_ManipulationStart = false;
	m_isPressed = false;
	m_isPressedMove = false;
	positionChangePt = Windows::Foundation::Point(0, 0);
	m_isOpenFileComplete = false;

}

void UIExtensionsManager::ShowProgressRing()
{
	if (m_pHostedControl->GridRing->Visibility == Windows::UI::Xaml::Visibility::Collapsed)
	{
		m_pHostedControl->GridRing->Visibility = Windows::UI::Xaml::Visibility::Visible;
	}
}

void UIExtensionsManager::HideProgressRing()
{
	if (m_pHostedControl->GridRing->Visibility == Windows::UI::Xaml::Visibility::Visible)
	{
		m_pHostedControl->GridRing->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
	}
}

void UIExtensionsManager::OnLoaded(Platform::Object ^sender, Windows::UI::Xaml::RoutedEventArgs ^e)
{
	m_pHostedControl = ref new FoxitUIHostedContorl();
	m_pHostedControl->SetUIExtMgr(this);
	m_control->Layout->Children->Append(m_pHostedControl);
	m_control->Layout->UpdateLayout();

	m_WindowsRect = Windows::UI::Xaml::Window::Current->Bounds;
	m_pHostedControl->GridPopup->Width = m_WindowsRect.Width;
	m_pHostedControl->GridPopup->Height = m_WindowsRect.Height;

	m_AcceleratorKeyActivatedEventToken = Windows::UI::Xaml::Window::Current->CoreWindow->Dispatcher->AcceleratorKeyActivated += ref new Windows::Foundation::TypedEventHandler<Windows::UI::Core::CoreDispatcher ^, Windows::UI::Core::AcceleratorKeyEventArgs ^>(this, &UIExtensionsManager::OnAcceleratorKeyActivated);
	m_KeyDownToken = Windows::UI::Core::CoreWindow::GetForCurrentThread()->KeyDown += ref new Windows::Foundation::TypedEventHandler<Windows::UI::Core::CoreWindow ^, Windows::UI::Core::KeyEventArgs ^>(this, &UIExtensionsManager::OnKeyDown);
	m_KeyUpToken = Windows::UI::Core::CoreWindow::GetForCurrentThread()->KeyUp += ref new Windows::Foundation::TypedEventHandler<Windows::UI::Core::CoreWindow ^, Windows::UI::Core::KeyEventArgs ^>(this, &UIExtensionsManager::OnKeyUp);

	m_PoppupTextMenu = m_pHostedControl->popupTextMenu; 
	m_PoppupNoteWindow = m_pHostedControl->popupNoteWindow; 
	m_NumberPopup = m_pHostedControl->numberPopup; 
	m_ShowNumberPopup = m_pHostedControl->showNumberPopup; 

	m_CurAppBarType = AppBarType::MainAppBar;
	m_AnnotCreateType = AnnotCreateType::None;
	m_isShowAnnot = true;
	m_isSaveAsFile = false;
	m_CtrlKeyDown = false;

	m_pHostedControl->annotOperatingOverlayControl->RelatePageView(m_control);
	m_pHostedControl->direcControl->RelatePageView(this);
	m_pHostedControl->drawSigControl->RelatePageView(this);

	m_IsContinuousOperate = false;
	propertyPopup = ref new Windows::UI::Xaml::Controls::Primitives::Popup();

	m_EraserPageIndex = -1;
	m_IsRightMenuAdd = false;
	m_IsDefaultSignature = true;
	Windows::UI::ViewManagement::ApplicationView::GetForCurrentView()->Consolidated += ref new Windows::Foundation::TypedEventHandler<Windows::UI::ViewManagement::ApplicationView ^, Windows::UI::ViewManagement::ApplicationViewConsolidatedEventArgs ^>([=](Windows::UI::ViewManagement::ApplicationView ^sender, Windows::UI::ViewManagement::ApplicationViewConsolidatedEventArgs ^args)
	{
		auto _IsUserInitiated = args->IsUserInitiated;
		std::wstringstream ws;
		ws << L"Consolidated: " << _IsUserInitiated;
		ws << L"\n";
		OutputDebugString(ws.str().c_str());
	});
	m_IsHandleBackPressed = false;

}

void UIExtensionsManager::OnUnloaded(Platform::Object ^sender, Windows::UI::Xaml::RoutedEventArgs ^e)
{
	if (m_WindowSizeChangedEventToken.Value)
	{
		Windows::UI::Xaml::Window::Current->SizeChanged -= m_WindowSizeChangedEventToken;
		m_WindowSizeChangedEventToken.Value = 0;
	}

	if (m_AcceleratorKeyActivatedEventToken.Value)
	{
		Windows::UI::Xaml::Window::Current->CoreWindow->Dispatcher->AcceleratorKeyActivated -= m_AcceleratorKeyActivatedEventToken;
		m_AcceleratorKeyActivatedEventToken.Value = 0;
	}

	if (m_KeyDownToken.Value)
	{
		Windows::UI::Core::CoreWindow::GetForCurrentThread()->KeyDown -= m_KeyDownToken;
		m_KeyDownToken.Value = 0;
	}

	if (m_KeyUpToken.Value)
	{
		Windows::UI::Core::CoreWindow::GetForCurrentThread()->KeyUp -= m_KeyUpToken;
		m_KeyUpToken.Value = 0;
	}

	if (m_TappedTimer->IsEnabled)
		m_TappedTimer->Stop();

	if (m_BottomBarVisibleTimer->IsEnabled)
		m_BottomBarVisibleTimer->Stop();

	if (m_PointerWheelScaleTimer->IsEnabled)
		m_PointerWheelScaleTimer->Stop();

	if (m_CopyFileTimer && m_CopyFileTimer->IsEnabled)
		m_CopyFileTimer->Stop();

	if (m_AutoSaveFileTimer && m_AutoSaveFileTimer->IsEnabled)
		m_AutoSaveFileTimer->Stop();
}

void UIExtensionsManager::OnAcceleratorKeyActivated(Windows::UI::Core::CoreDispatcher ^sender, Windows::UI::Core::AcceleratorKeyEventArgs ^args)
{
}

void UIExtensionsManager::OnKeyDown(Windows::UI::Core::CoreWindow ^sender, Windows::UI::Core::KeyEventArgs ^args)
{
	auto _virtualKey = args->VirtualKey;
	auto _coreWindow = Windows::UI::Xaml::Window::Current->CoreWindow;
	auto _downState = Windows::UI::Core::CoreVirtualKeyStates::Down;
	bool _menuKey = (_coreWindow->GetKeyState(Windows::System::VirtualKey::Menu) & _downState) == _downState;
	bool _controlKey = (_coreWindow->GetKeyState(Windows::System::VirtualKey::Control) & _downState) == _downState;
	bool _shiftKey = (_coreWindow->GetKeyState(Windows::System::VirtualKey::Shift) & _downState) == _downState;
	bool _noModifiers = !_menuKey && !_controlKey && !_shiftKey;
	bool _onlyAlt = _menuKey && !_controlKey && !_shiftKey;
	Windows::UI::Core::CorePhysicalKeyStatus _KeyStatus = args->KeyStatus;

	{
		std::wstringstream ws;
		ws << L"OnKeyDown---_menuKey: " << _menuKey;
		ws << L", _controlKey: " << _controlKey;
		ws << L", _shiftKey: " << _shiftKey;
		ws << L", VirtualKey: " << _virtualKey.ToString()->Data();
		ws << L", ScanCode: " << _KeyStatus.ScanCode;
		ws << L"\n";
		
	}

	IBaseAnnot^ pSelectedAnnot = nullptr;
	if (AnnotMgr) {
		pSelectedAnnot = AnnotMgr->GetSelectedAnnot();
	}
	if (m_control->GetDoc() && pSelectedAnnot && pSelectedAnnot->AnnotType == AnnotationType::Widget)
	{
		return;
	}

	if (_virtualKey == Windows::System::VirtualKey::PageUp)
	{
		if (m_AnnotCreateType != AnnotCreateType::None)
			return;

		if (!GetIsCanScrollPageView())
			return;

		int pageIndex = GetCurrentPageIndex();
		if (pageIndex > 0)
		{
			pageIndex = pageIndex - 1;
			m_control->GotoPage(pageIndex);
			
		}
	}
	else if (_virtualKey == Windows::System::VirtualKey::PageDown)
	{
		if (m_AnnotCreateType != AnnotCreateType::None)
			return;

		if (!GetIsCanScrollPageView())
			return;

		int pageIndex = GetCurrentPageIndex();
		if (m_control->GetDoc() && pageIndex < m_control->GetDoc()->GetPageCount() - 1)
		{
			pageIndex = pageIndex + 1;
			m_control->GotoPage(pageIndex);
			
		}
	}
	else if (_virtualKey == Windows::System::VirtualKey::Delete)
	{
		if (m_control->GetDoc() && !FSPDFDocUtil::GetIsDocumentCanModifyAnnotation(m_control->GetDoc()) && m_control->GetDoc()->GetPasswordType() != foxit::pdf::PasswordType::e_PwdNoPassword)
		{

		}
		else
		{
			if (m_PoppupNoteWindow->IsOpen)
			{
				AnnotNoteUserControl^ control = dynamic_cast<AnnotNoteUserControl^>(m_PoppupNoteWindow->Child);
				if (control && control->IsSelectText())
				{
					return;
				}
			}

			if (m_AnnotMgr && m_AnnotMgr->GetSelectedAnnot())
			{
				IBaseAnnot^ annot = m_AnnotMgr->GetSelectedAnnot();
				if (annot)
				{
					AnnotMgr->InsertModifyAnnotBeforeDelete();
					if (annot->AnnotType == AnnotationType::FreeText)
					{
						if (GetAnnotCanvasHasChild())
						{
							
							return;
						}
					}

					if (annot->AnnotType == AnnotationType::StrikeOut)
					{
						StrikeOutAnnot^ strikeoutannot = dynamic_cast<StrikeOutAnnot^>(annot);
						if (strikeoutannot->IsHasGroup)
						{
							Windows::Foundation::Rect groupminRect = strikeoutannot->GroupAnnotMinRect;
							bool isGetGroupAnnot = AnnotMgr->IsGetGroupAnnotElement(annot->PageIndex, groupminRect, strikeoutannot->GroupAnnotIndex, AnnotationType::Caret);
							if (isGetGroupAnnot)
							{
								IBaseAnnot^ groupAnnot = AnnotMgr->GetGroupAnnot();

								auto pUndoItem = ref new DeleteReplaceAnnotUndoAction(m_control, groupAnnot, annot);
								UndoFactory::GetUndoInstance()->AddUndoItem(this, pUndoItem);

								if (groupAnnot)
									groupAnnot->OnDelete();
							}
						}
						annot->OnDelete();
					}
					else
					{
						auto pUndoItem = ref new DeleteAnnotUndoAction(m_control, annot);
						UndoFactory::GetUndoInstance()->AddUndoItem(this, pUndoItem);
						foxit::common::fxcrt::RectF^ minRect = FoxitRectConverter::ConvertToFoxitRectF(annot->MinRect);
						String^ rectStr = "[" + minRect->left.ToString() + " " + minRect->top.ToString() + " " + minRect->right.ToString() + " " + minRect->bottom.ToString() + "]";

						Platform::String^ annotTypeStr = GetAnnotTypeString(annot->AnnotType);
						CollectAnnotOperation(annotTypeStr, "Delete", annot->PageIndex.ToString(), rectStr);

						m_control->CancelRenderTask();

						if (annot->AnnotType == AnnotationType::FreeText)
						{
							IBaseAnnotEventPeer^ annotEventPeer = AnnotMgr->GetAnnotHandlerByType(AnnotationType::FreeText)->AnnotEvent;
							FreeTextEventPeer^ freetextEventPeer = dynamic_cast<FreeTextEventPeer^>(annotEventPeer);
							freetextEventPeer->UpdateEditLayout();
						}

						annot->OnDelete();
					}
					SetTextMenuPopupHide();
					if (propertyPopup)
						propertyPopup->IsOpen = false;
					CloseAddNoteWindows();
					SetAnnotCanvasVisible(false);
				}
			}
		}
	}
	else if (_virtualKey == Windows::System::VirtualKey::C)
	{
		foxit::pdf::PDFDoc^ doc = m_control->GetDoc();
		if (_controlKey && !doc->IsEmpty() && doc->GetPasswordType() != foxit::pdf::PasswordType::e_PwdNoPassword && !FSPDFDocUtil::GetIsDocumentCanBeCopy(doc))
		{

		}
		else
		{
			auto _coreWindow = Windows::UI::Xaml::Window::Current->CoreWindow;
			auto _downState = Windows::UI::Core::CoreVirtualKeyStates::Down;
			bool _menuKey = (_coreWindow->GetKeyState(Windows::System::VirtualKey::Menu) & _downState) == _downState;
			bool _controlKey = (_coreWindow->GetKeyState(Windows::System::VirtualKey::Control) & _downState) == _downState;
			bool _shiftKey = (_coreWindow->GetKeyState(Windows::System::VirtualKey::Shift) & _downState) == _downState;
			bool _noModifiers = !_menuKey && !_controlKey && !_shiftKey;
			bool _onlyAlt = _menuKey && !_controlKey && !_shiftKey;
			bool _onlyControl = !_menuKey && _controlKey && !_shiftKey;
			if (_onlyControl)
			{
				if (TextManager)
				{
					auto selString = TextManager->SelectedTextString;
					if (selString->Length() > 0)
					{
						Windows::ApplicationModel::DataTransfer::DataPackage^ _DataPackage = ref new Windows::ApplicationModel::DataTransfer::DataPackage();
						_DataPackage->SetText(selString);
						Windows::ApplicationModel::DataTransfer::Clipboard::SetContent(_DataPackage);
					}
				}
			}
		}
	}
	else if (_virtualKey == Windows::System::VirtualKey::D)
	{
		auto _coreWindow = Windows::UI::Xaml::Window::Current->CoreWindow;
		auto _downState = Windows::UI::Core::CoreVirtualKeyStates::Down;
		bool _menuKey = (_coreWindow->GetKeyState(Windows::System::VirtualKey::Menu) & _downState) == _downState;
		bool _controlKey = (_coreWindow->GetKeyState(Windows::System::VirtualKey::Control) & _downState) == _downState;
		bool _shiftKey = (_coreWindow->GetKeyState(Windows::System::VirtualKey::Shift) & _downState) == _downState;
		bool _noModifiers = !_menuKey && !_controlKey && !_shiftKey;
		bool _onlyAlt = _menuKey && !_controlKey && !_shiftKey;
		bool _onlyControl = !_menuKey && _controlKey && !_shiftKey;
		if (_onlyControl)
		{

		}
	}
}

void UIExtensionsManager::OnKeyUp(Windows::UI::Core::CoreWindow ^sender, Windows::UI::Core::KeyEventArgs ^args)
{
	auto _coreWindow = Windows::UI::Xaml::Window::Current->CoreWindow;
	auto _downState = Windows::UI::Core::CoreVirtualKeyStates::Down;
	bool _shiftKey = (_coreWindow->GetKeyState(Windows::System::VirtualKey::Shift) & _downState) == _downState;
	auto _virtualKey =args->VirtualKey;

	if (_virtualKey == Windows::System::VirtualKey::Tab)
	{
		if(_shiftKey)
			FocusPrevFormControl();
		else
			FocusNextFormControl();
	}
	OutputDebugStringW(L"OnKeyUp\r\n");
}

void UIExtensionsManager::InitMainAppBar()
{
	Windows::UI::Xaml::Controls::Grid^ mainAppBar = m_pHostedControl->AppBarLayout;
	Windows::UI::Xaml::Controls::Grid^ mainAppBarBottom = m_pHostedControl->AppBarBottomLayout;
	m_MainAppBar = ref new MainPageAppBar(this);
	
	m_MainAppBar->InitMainAppbar(AppBarType::MainAppBar);
	if (m_isDesktopPlatform)
	{
		mainAppBar->Visibility = Windows::UI::Xaml::Visibility::Visible;
		mainAppBar->Children->Clear();
		mainAppBar->Children->Append(m_MainAppBar);
		mainAppBar->UpdateLayout();
	}
	else
	{
		mainAppBarBottom->Visibility = Windows::UI::Xaml::Visibility::Visible;
		mainAppBar->Children->Clear();
		mainAppBarBottom->Children->Clear();
		if (m_MainAppBar->VerticalAlignment != Windows::UI::Xaml::VerticalAlignment::Bottom)
			m_MainAppBar->VerticalAlignment = Windows::UI::Xaml::VerticalAlignment::Bottom;

		mainAppBarBottom->Children->Append(m_MainAppBar);
		mainAppBarBottom->UpdateLayout();
	}
}

MainPageAppBar^ UIExtensionsManager::GetMainAppBar()
{
	return m_MainAppBar;
}

Windows::UI::Xaml::Controls::Grid^ UIExtensionsManager::FlyoutPlacement::get()
{
	return m_FlyoutPlacement ? m_FlyoutPlacement : m_pHostedControl->FlyoutPlacement;
}

bool UIExtensionsManager::IsEnableFormModule()
{
	return m_pPDFForm && m_FormFiller ? true : false;
}

bool UIExtensionsManager::IsHaveForm()
{
	bool bIsEnableForm = IsEnableFormModule();
	bool  bIsHaveForm = false;
	if (bIsEnableForm)
	{
		FS_INT32 count = m_pPDFForm->GetFieldCount(nullptr);
		bIsHaveForm = count > 0;
	}

	return bIsHaveForm;
}

foxit::pdf::interform::Form^ UIExtensionsManager::GetSdkForm()
{
	return m_pPDFForm;
}

void UIExtensionsManager::ResetForm()
{
	if (!IsEnableFormModule())
		return;

	FS_RESULT iRet = m_pPDFForm->Reset();
}

void UIExtensionsManager::BeginFormFiller()
{
	if (!IsEnableFormModule())
		return;

	
}

void UIExtensionsManager::EndFormFiller()
{
	if (!IsEnableFormModule())
		return;

	
}

void foxit::rdk::uiextensions::UIExtensionsManager::ImportFormDataFromFdfFile()
{
	Windows::Storage::Pickers::FileOpenPicker^ openPicker = ref new Windows::Storage::Pickers::FileOpenPicker();
	openPicker->ViewMode = Windows::Storage::Pickers::PickerViewMode::Thumbnail;
	openPicker->SuggestedStartLocation = Windows::Storage::Pickers::PickerLocationId::DocumentsLibrary;
	openPicker->FileTypeFilter->Append(".fdf");

	std::shared_ptr<Windows::Storage::StorageFile^> _spStorageFile = std::make_shared<Windows::Storage::StorageFile^>(nullptr);
	concurrency::create_task(openPicker->PickSingleFileAsync())
		.then([=](Windows::Storage::StorageFile^ file)
	{
		if (file == nullptr)
			throw ref new Platform::NullReferenceException(L"File is Null");

		*_spStorageFile = file;
		return UIExtensionsManager::OpenFdfDocumentAsync(file);
	})
		.then([=](foxit::fdf::FDFDoc^ fdfDoc)
	{
		if (viewControl->GetDoc()->ImportFromFDF(fdfDoc, (int)foxit::pdf::DataType::e_Forms, ref new foxit::common::Range())) {
			viewControl->SetModified(true);
		}
		else
			throw ref new Platform::FailureException();
	})
		.then([=](concurrency::task<void> t)
	{
		try
		{
			t.get();

			auto _resourceLoader = ref new Windows::ApplicationModel::Resources::ResourceLoader();
			Platform::String^ _TitleStr = _resourceLoader->GetString(L"ToastImportFormDataTitleText");
			Platform::String^ _ContentStr = _resourceLoader->GetString(L"ToastImportFormDataSuccessContentText");
			
		}
		catch (Platform::Exception^ e)
		{
			auto _msg = e->Message;

			std::wregex pattern(L"File is Null");
			if (std::regex_search(_msg->Data(), pattern))
				return;

			auto _resourceLoader = ref new Windows::ApplicationModel::Resources::ResourceLoader();
			Platform::String^ _TitleStr = _resourceLoader->GetString(L"ToastImportFormDataTitleText");
			Platform::String^ _ContentStr = _resourceLoader->GetString(L"ToastImportFormDataFailedContentText");
			
		}
		catch (...)
		{
			auto _resourceLoader = ref new Windows::ApplicationModel::Resources::ResourceLoader();
			Platform::String^ _TitleStr = _resourceLoader->GetString(L"ToastImportFormDataTitleText");
			Platform::String^ _ContentStr = _resourceLoader->GetString(L"ToastImportFormDataFailedContentText");
			
		}
	});
}

void foxit::rdk::uiextensions::UIExtensionsManager::ExportFormDataToFdfFile()
{
	Windows::Storage::Pickers::FileSavePicker^ savePicker = ref new Windows::Storage::Pickers::FileSavePicker();
	savePicker->SuggestedStartLocation = PickerLocationId::DocumentsLibrary;

	auto plainTextExtensions = ref new Platform::Collections::Vector<String^>();
	plainTextExtensions->Append(".fdf");
	auto _DisplayName = viewControl->GetStorageFile()->DisplayName;
	savePicker->FileTypeChoices->Insert("Plain Text", plainTextExtensions);
	savePicker->SuggestedFileName = _DisplayName;

	std::shared_ptr<Windows::Storage::StorageFile^> _spStorageFile = std::make_shared<Windows::Storage::StorageFile^>(nullptr);
	concurrency::create_task(savePicker->PickSaveFileAsync())
		.then([=](Windows::Storage::StorageFile^ file)
	{
		if (file == nullptr)
			throw ref new Platform::NullReferenceException(L"File is Null");

		*_spStorageFile = file;
		return UIExtensionsManager::OpenNewFdfDocumentAsync();
	})
		.then([=](foxit::fdf::FDFDoc^ fdfDoc)
	{
		if (!viewControl->GetDoc()->ExportToFDF(fdfDoc, (int)foxit::pdf::DataType::e_Forms, ref new foxit::common::Range())) {
			throw ref new Platform::FailureException();
		}

		return fdfDoc->SaveAs(*_spStorageFile);
	})
		.then([=](concurrency::task<bool> t)
	{
		try
		{
			t.get();

			auto _resourceLoader = ref new Windows::ApplicationModel::Resources::ResourceLoader();
			Platform::String^ _TitleStr = _resourceLoader->GetString(L"ToastExportFormDataTitleText");
			Platform::String^ _ContentStr = _resourceLoader->GetString(L"ToastExportFormDataSuccessContentText");
			
		}
		catch (Platform::Exception^ e)
		{
			auto _msg = e->Message;

			std::wregex pattern(L"File is Null");
			if (std::regex_search(_msg->Data(), pattern))
				return;

			auto _resourceLoader = ref new Windows::ApplicationModel::Resources::ResourceLoader();
			Platform::String^ _TitleStr = _resourceLoader->GetString(L"ToastImportFormDataTitleText");
			Platform::String^ _ContentStr = _resourceLoader->GetString(L"ToastImportFormDataFailedContentText");
			
		}
		catch (...)
		{
			auto _resourceLoader = ref new Windows::ApplicationModel::Resources::ResourceLoader();
			Platform::String^ _TitleStr = _resourceLoader->GetString(L"ToastExportFormDataTitleText");
			Platform::String^ _ContentStr = _resourceLoader->GetString(L"ToastExportFormDataFailedContentText");
			
		}
	});
}

Windows::Foundation::IAsyncOperation<foxit::fdf::FDFDoc^>^ foxit::rdk::uiextensions::UIExtensionsManager::OpenNewFdfDocumentAsync()
{
	return concurrency::create_async([=]()
	{
		return concurrency::create_task(Windows::Storage::ApplicationData::Current->LocalFolder->CreateFileAsync(L"NewTemper.fdf", Windows::Storage::CreationCollisionOption::GenerateUniqueName))
			.then([=](Windows::Storage::StorageFile^ file)
		{
			return ref new foxit::fdf::FDFDoc(foxit::fdf::Type::e_FDF);
		});
	});
}

Windows::Foundation::IAsyncOperation<foxit::fdf::FDFDoc^>^ foxit::rdk::uiextensions::UIExtensionsManager::OpenFdfDocumentAsync(Windows::Storage::StorageFile ^ pdfFile)
{
	return concurrency::create_async([=]()
	{
		return concurrency::create_task(pdfFile->GetBasicPropertiesAsync())
			.then([=](Windows::Storage::FileProperties::BasicProperties^ properties)
		{
			return ref new foxit::fdf::FDFDoc(pdfFile, 0, 0, 0, 0);
		});
	});
}

foxit::pdf::interform::Filler^ UIExtensionsManager::GetFormFiller()
{
	
	if (m_FormFiller == nullptr)
	{
		auto pImpl = new FSSDK_PDFForm(m_control->GetDoc());
		m_pFormAbs = PDFFormAbstration::ConcreatePDFFormAbstration(pImpl);
		auto isOk = m_pFormAbs->InitializeModule(m_control);
		m_pPDFForm = pImpl->GetPDFForm();
		m_FormFiller = pImpl->GetFormFiller();
	}

	return m_FormFiller;
}

void UIExtensionsManager::ReleaseFormFiller()
{
	if(m_pFormAbs)
		m_pFormAbs->ReleaseModule();
	m_pFormAbs = nullptr;
	m_FormFiller = nullptr; 
}

bool UIExtensionsManager::IsCreateDrawAnnot()
{
	if (m_AnnotCreateType == AnnotCreateType::Circle || m_AnnotCreateType == AnnotCreateType::Line || m_AnnotCreateType == AnnotCreateType::Arrow
		|| m_AnnotCreateType == AnnotCreateType::Pencil || m_AnnotCreateType == AnnotCreateType::Square
		|| m_AnnotCreateType == AnnotCreateType::Eraser || m_AnnotCreateType == AnnotCreateType::AreaHighLight || m_AnnotCreateType == AnnotCreateType::Text
		|| m_AnnotCreateType == AnnotCreateType::Stamp || m_AnnotCreateType == AnnotCreateType::FreeText)
	{
		return true;
	}
	else if (AnnotMgr->GetSelectedAnnot() && AnnotMgr->GetSelectedAnnot()->AnnotType == AnnotationType::FreeText
		&& GetAnnotCanvasHasChild())
	{
		return true;
	}
	else if (!ExitNotePopupEdit())
	{
		return true;
	}
	else
	{
		return false;
	}
}

void UIExtensionsManager::ManipulationStarted(Platform::Object^ sender, Windows::UI::Xaml::Input::ManipulationStartedRoutedEventArgs^ e)
{
	if (!m_isOpenFileComplete)
		return;
	if (!m_AnnotMgr)
		return;

	if (m_AnnotCreateType != AnnotCreateType::None)
		return;

	
	m_ManipulationStart = true;
	

	if (GetIsCanScrollPageView())
	{
		SetTextMenuPopupHide();
		auto _Pageinfo = m_control->ConvertDevicePtToPDFPagePt(e->Position);
		auto _selectAnnot = m_AnnotMgr->GetSelectedAnnot();
		

		if (!m_isMovingAnnot)
		{
			if (TextManager)
				TextManager->SetIsShowSelHandleBtn(false);
			ShowSelHandleButton(false);
			
			e->Handled = true;
		}
	}
	else
	{
		auto _Pageinfo = m_control->ConvertDevicePtToPDFPagePt(e->Position);
		auto _selectAnnot = m_AnnotMgr->GetSelectedAnnot();
		if (_selectAnnot && _selectAnnot->AnnotType == AnnotationType::Text)
		{
			if (m_AnnotMgr->GetSelectedAnnot()->MinRect.Contains(_Pageinfo->PagePoint))
			{
				m_isMovingAnnot = true;
				m_AnnotMgr->OnManipulationStarted(sender, e);
			}
		}
	}
	SetBottomBarIsVisible(true);
}

void UIExtensionsManager::ManipulationDelta(Platform::Object^ sender, Windows::UI::Xaml::Input::ManipulationDeltaRoutedEventArgs^ e)
{
}

void UIExtensionsManager::ManipulationCompleted(Platform::Object^ sender, Windows::UI::Xaml::Input::ManipulationCompletedRoutedEventArgs^ e)
{
}

void UIExtensionsManager::PointerPressed(Platform::Object^ sender, Windows::UI::Xaml::Input::PointerRoutedEventArgs^ e)
{
	if (!m_control->GetDoc())
		return;

	if (!m_isOpenFileComplete)
		return;

	if (m_NumberPopup && m_NumberPopup->IsOpen)
		return;
	if (IsPhonePropertyPopupOpen())
	{
		return;
	}

	auto _frameworkelement = safe_cast<Windows::UI::Xaml::FrameworkElement^>(sender);

	if (m_control->GetPageLayoutMode() == LayoutMode::ReflowMode)
		return;

	CloseAllPropertyPopup();

	m_isPressed = true;
	bool _hasCapture = (dynamic_cast<PDFViewCanvas^>(sender))->CapturePointer(e->Pointer);

	auto _PointerPt = e->GetCurrentPoint(_frameworkelement);
	Point _Pt = _PointerPt->Position;
	auto _Pageinfo = m_control->ConvertDevicePtToPDFPagePt(_Pt);
	int index = _Pageinfo->PageIndex;
	auto _PagePoint = _Pageinfo->PagePoint;
	if (index == -1)
		return;

	foxit::pdf::PDFDoc^ Document = m_control->GetDoc();
	auto _PageInfo = Document->GetPage(index);
	if (_PageInfo && !IsCreateDrawAnnot() && GetCanModifyForm() && !_PointerPt->Properties->IsRightButtonPressed)
	{
		m_FormFillerMgr->PDFViewControl_PointerPressed(sender, e);
	}

	auto pHitAnnot = m_AnnotMgr->IsHitAnnot(index, _PagePoint);
	if (pHitAnnot && pHitAnnot->AnnotType == AnnotationType::Widget)
	{
		foxit::pdf::interform::Type _FormType = (foxit::pdf::interform::Type)GetFormControlType(pHitAnnot);
		if (_FormType == foxit::pdf::interform::Type::e_TypeComboBox || _FormType == foxit::pdf::interform::Type::e_TypeTextField
			|| _FormType == foxit::pdf::interform::Type::e_TypeListBox)
		{
			_frameworkelement->ManipulationMode = Windows::UI::Xaml::Input::ManipulationModes::System;
		}

	}
	else
	{
		_frameworkelement->ManipulationMode = Windows::UI::Xaml::Input::ManipulationModes::All;
	}

	auto _selectAnnot = (m_AnnotMgr == nullptr) ? nullptr : m_AnnotMgr->GetSelectedAnnot();
	foxit::pdf::annots::IAnnot^ pSdkAnnot = nullptr;
	pSdkAnnot = _PageInfo->GetAnnotAtPoint(ref new foxit::common::fxcrt::PointF(_PagePoint.X, _PagePoint.Y), 10);
	if (pSdkAnnot != nullptr)
	{
		if (_selectAnnot)
		{
			_frameworkelement->ManipulationMode = Windows::UI::Xaml::Input::ManipulationModes::System;
		}
	}
	
	if (_selectAnnot && _selectAnnot->AnnotType == AnnotationType::Widget)
	{
		foxit::pdf::interform::Type _FormType = (foxit::pdf::interform::Type)GetFormControlType(_selectAnnot);
		if (_FormType == foxit::pdf::interform::Type::e_TypeComboBox || _FormType == foxit::pdf::interform::Type::e_TypeTextField
			|| _FormType == foxit::pdf::interform::Type::e_TypeListBox)
		{
			_frameworkelement->ManipulationMode = Windows::UI::Xaml::Input::ManipulationModes::System;
		}
		return;
	}

	if (!_selectAnnot)
	{
		SetBottomBarIsVisible(true);

		if (m_control->GetPageLayoutMode() == LayoutMode::ReflowMode)
		{
			return;
		}

		if (!FSPDFDocUtil::GetIsDocumentCanModifyAnnotation(Document) && Document->GetPasswordType() != foxit::pdf::PasswordType::e_PwdNoPassword
			&& FSPDFDocUtil::GetIsDocumentCanExtra(Document))
		{
			if (TextManager && (m_AnnotCreateType != AnnotCreateType::AreaHighLight && m_AnnotCreateType != AnnotCreateType::Pencil
				&& m_AnnotCreateType != AnnotCreateType::Square && m_AnnotCreateType != AnnotCreateType::Circle && m_AnnotCreateType != AnnotCreateType::Line
				&& m_AnnotCreateType != AnnotCreateType::Arrow && m_AnnotCreateType != AnnotCreateType::Eraser
				&& m_AnnotCreateType != AnnotCreateType::Stamp && m_AnnotCreateType != AnnotCreateType::FreeText))
				TextManager->OnPointerPressed(sender, e);
		}
		else
		{
			if (TextManager && (m_AnnotCreateType != AnnotCreateType::AreaHighLight && m_AnnotCreateType != AnnotCreateType::Pencil
				&& m_AnnotCreateType != AnnotCreateType::Square && m_AnnotCreateType != AnnotCreateType::Circle && m_AnnotCreateType != AnnotCreateType::Line
				&& m_AnnotCreateType != AnnotCreateType::Arrow && m_AnnotCreateType != AnnotCreateType::Eraser
				&& m_AnnotCreateType != AnnotCreateType::Stamp && m_AnnotCreateType != AnnotCreateType::FreeText))
				TextManager->OnPointerPressed(sender, e);
			if (m_AnnotMgr)
				m_AnnotMgr->OnPointerPressed(sender, e);
		}
	}

	if (_PointerPt->Properties->IsRightButtonPressed)
	{
		if (m_AnnotCreateType != AnnotCreateType::None)
			return;

		if (m_NumberPopup && m_NumberPopup->IsOpen)
		{
			return;
		}

		if (m_MainAppBar && m_MainAppBar->GetMainAppBarVisible() == Windows::UI::Xaml::Visibility::Visible)
		{
			if (m_MainAppBar->GetMorePopupIsOpen())
			{
				m_MainAppBar->HideMorePopup();
				return;
			}
			if (m_MainAppBar->GetViewToolPanelPopupIsOpen())
			{
				m_MainAppBar->HideViewToolPanelPopup();
				return;
			}
		}
		else if (m_PoppupTextMenu->IsOpen && menuControl && menuControl->MenuType != AnnotContextMenuType::Note)
		{
			SetTextMenuPopupHide();
			TextManager->ClearTextSelect();
			if (m_AnnotMgr && m_AnnotMgr->GetSelectedAnnot())
			{
				m_AnnotMgr->SetPdfSelectAnnotNull();
				ExitAnnotPropertyMode();
			}
			m_control->Update();
			return;
		}
		if (m_AnnotMgr && m_AnnotMgr->GetSelectedAnnot())
		{
			ExitAnnotSelect();
			
			m_control->Update();
			return;
		}


		if (!FSPDFDocUtil::GetIsDocumentCanModifyAnnotation(Document) && Document->GetPasswordType() != foxit::pdf::PasswordType::e_PwdNoPassword)
		{
			return;
		}

		if (m_MainAppBar && m_MainAppBar->GetCurAppBarType() == AppBarType::SearchBar)
			HideFindBar();

		auto _PointerPt = e->GetCurrentPoint(_frameworkelement);
		Point _Pt = _PointerPt->Position;
		auto _Pageinfo = m_control->ConvertDevicePtToPDFPagePt(_Pt);
		int index = _Pageinfo->PageIndex;
		auto _PagePoint = _Pageinfo->PagePoint;

		if (_PagePoint.X != 0 && _PagePoint.Y != 0)
		{
			_Pageinfo = m_control->ConvertDevicePtToPDFPagePt(_Pt);

			m_holdingPoint = _Pageinfo;
			ShowTextmarkContextmenu(AnnotContextMenuType::Note, _Pt);
		}
	}
}

void UIExtensionsManager::PointerMoved(Platform::Object^ sender, Windows::UI::Xaml::Input::PointerRoutedEventArgs^ e)
{
	if (!m_isOpenFileComplete)
		return;
	foxit::pdf::PDFDoc^ Document = m_control->GetDoc();
	if (!Document)
		return;

	if (m_control->GetPageLayoutMode() == LayoutMode::ReflowMode)
		return;

	auto _selectAnnot = m_AnnotMgr->GetSelectedAnnot();
	if (m_AnnotCreateType == AnnotCreateType::Text || (_selectAnnot && _selectAnnot->AnnotType == AnnotationType::Text)
		|| ((m_AnnotCreateType == AnnotCreateType::Caret || m_AnnotCreateType == AnnotCreateType::ReplaceText) && m_PoppupNoteWindow->IsOpen))
		return;

	auto _frameworkelement = safe_cast<Windows::UI::Xaml::FrameworkElement^>(sender);
	auto _PointerPt = e->GetCurrentPoint(_frameworkelement);
	Point _Pt = _PointerPt->Position;
	auto _Pageinfo = m_control->ConvertDevicePtToPDFPagePt(_Pt);
	int index = _Pageinfo->PageIndex;
	auto _PagePoint = _Pageinfo->PagePoint;
	if (index == -1)
		return;
	auto _PageInfo = Document->GetPage(index);
	if (Document && _PageInfo && !IsCreateDrawAnnot() && GetCanModifyForm())
	{
		m_FormFillerMgr->PDFViewControl_PointerMoved(sender, e);
	}

	if (!FSPDFDocUtil::GetIsDocumentCanModifyAnnotation(Document) && Document->GetPasswordType() != foxit::pdf::PasswordType::e_PwdNoPassword
		&& !FSPDFDocUtil::GetIsDocumentCanExtra(Document))
	{
	}
	else if (!FSPDFDocUtil::GetIsDocumentCanModifyAnnotation(Document) && Document->GetPasswordType() != foxit::pdf::PasswordType::e_PwdNoPassword
		&& FSPDFDocUtil::GetIsDocumentCanExtra(Document))
	{
		if (TextManager)
			TextManager->OnPointerMoved(sender, e);
	}
	else
	{
		if (TextManager)
			TextManager->OnPointerMoved(sender, e);
		if (m_AnnotMgr)
			m_AnnotMgr->OnPointerMoved(sender, e);

		m_isPressedMove = true;
	}
}

void UIExtensionsManager::PointerReleased(Platform::Object^ sender, Windows::UI::Xaml::Input::PointerRoutedEventArgs^ e)
{
	if (!m_isOpenFileComplete)
		return;
	foxit::pdf::PDFDoc^ Document = m_control->GetDoc();
	if (!Document)
		return;

	if (IsPhonePropertyPopupOpen())
	{
		return;
	}

	(dynamic_cast<PDFViewCanvas^>(sender))->ReleasePointerCapture(e->Pointer);

	if (m_control->GetPageLayoutMode() == LayoutMode::ReflowMode)
		return;

	auto _frameworkelement = safe_cast<Windows::UI::Xaml::FrameworkElement^>(sender);
	auto _PointerPt = e->GetCurrentPoint(_frameworkelement);
	Point _Pt = _PointerPt->Position;
	auto _Pageinfo = m_control->ConvertDevicePtToPDFPagePt(_Pt);
	int index = _Pageinfo->PageIndex;
	auto _PagePoint = _Pageinfo->PagePoint;
	if (index == -1)
		return;

	auto _PageInfo = Document->GetPage(index);
	if (_PageInfo && !IsCreateDrawAnnot() && GetCanModifyForm())
	{
		m_FormFillerMgr->PDFViewControl_PointerReleased(sender, e);
	}

	auto _selectAnnot = m_AnnotMgr->GetSelectedAnnot();
	if (_selectAnnot)
		return;

	if (!FSPDFDocUtil::GetIsDocumentCanModifyAnnotation(Document) && Document->GetPasswordType() != foxit::pdf::PasswordType::e_PwdNoPassword
		&& !FSPDFDocUtil::GetIsDocumentCanExtra(Document))
	{
	}
	else if (!FSPDFDocUtil::GetIsDocumentCanModifyAnnotation(Document) && Document->GetPasswordType() != foxit::pdf::PasswordType::e_PwdNoPassword
		&& FSPDFDocUtil::GetIsDocumentCanExtra(Document))
	{
		if (TextManager)
			TextManager->OnPointerReleased(sender, e);
	}
	else
	{
		if (TextManager)
			TextManager->OnPointerReleased(sender, e);
		if (m_AnnotMgr)
			m_AnnotMgr->OnPointerReleased(sender, e);
	}

	m_isPressed = false;
	m_isPressedMove = false;
}

void UIExtensionsManager::PointerEntered(Platform::Object^ sender, Windows::UI::Xaml::Input::PointerRoutedEventArgs^ e)
{
	if (!m_control->GetDoc())
		return;
	if (m_textManager)
		m_textManager->OnPointerEntered(sender, e);
}

void UIExtensionsManager::PointerExited(Platform::Object^ sender, Windows::UI::Xaml::Input::PointerRoutedEventArgs^ e)
{
	if (!m_control->GetDoc())
		return;
	if (m_textManager)
		m_textManager->OnPointerExited(sender, e);
}

void UIExtensionsManager::PointerCaptureLost(Platform::Object^ sender, Windows::UI::Xaml::Input::PointerRoutedEventArgs^ e)
{

}

void UIExtensionsManager::PointerCanceled(Platform::Object^ sender, Windows::UI::Xaml::Input::PointerRoutedEventArgs^ e)
{

}

void UIExtensionsManager::PointerWheelChanged(Platform::Object^ sender, Windows::UI::Xaml::Input::PointerRoutedEventArgs^ e)
{
	if (!GetIsCanScrollPageView())
		return;

	SetTextMenuPopupHide();
	SetPropertyPoupHide();
	if (TextManager && TextManager->SelectedText->Size > 0)
	{
		ShowTextmarkContextmenu(AnnotContextMenuType::Add, Point(0, 0));
	}
}

void UIExtensionsManager::Holding(Platform::Object^ sender, Windows::UI::Xaml::Input::HoldingRoutedEventArgs^ e)
{
	if (!m_control->GetDoc())
		return;
	
	if (m_docViewMode == LayoutMode::ReflowMode)
		return;

	if (m_AnnotCreateType != AnnotCreateType::None)
		return;

	if (m_textManager)
		m_textManager->OnHolding(sender, e);

	Windows::UI::Xaml::FrameworkElement^ _FrameworkElement = dynamic_cast<Windows::UI::Xaml::FrameworkElement^>(sender);
	Windows::Foundation::Point _Pt;
	_Pt = e->GetPosition(nullptr);
	auto _Pageinfo = m_control->ConvertDevicePtToPDFPagePt(_Pt);
	int index = _Pageinfo->PageIndex;
	auto _PagePoint = _Pageinfo->PagePoint;
	if (_PagePoint.X == 0 && _PagePoint.Y == 0)
	{
		return;
	}

	if (TextManager) {
		auto _SelectText = TextManager->SelectedText;

		/*Windows::Foundation::Point pagePt = ViewerMgr->CalculateNotePosition(_Pt, index);
		_Pageinfo = ViewerMgr->GetPDFPagePoint(pagePt);*/
		m_holdingPoint = _Pageinfo;

		if (m_AnnotCreateType == AnnotCreateType::None && TextManager->SelectedText->Size > 0
			&& TextManager->SelectedText->GetAt(0)->TextRectVect->Size > 0)
		{
			Windows::UI::Xaml::Controls::Primitives::Popup^ m_PopUp = ref new Windows::UI::Xaml::Controls::Primitives::Popup();
			ShowTextmarkContextmenu(AnnotContextMenuType::Add, _Pt);
		}
		else
		{
			ShowTextmarkContextmenu(AnnotContextMenuType::Note, _Pt);
		}
	}
}

void UIExtensionsManager::Tapped(Platform::Object^ sender, Windows::UI::Xaml::Input::TappedRoutedEventArgs^ e)
{
	if (!m_control->GetDoc())
		return;

	if (IsPhonePropertyPopupOpen())
	{
		SetPhonePropertyPopupClosed();
		if (m_AnnotCreateType != AnnotCreateType::None && !GetIsDesktopPlatform())
		{
			SetUnRedoPhoneAppBarVisible(true);
		}
		return;
	}

	if (m_TappedTimer->IsEnabled)
		m_TappedTimer->Stop();
	m_TappedTimer->Start();

	if (m_control->GetPageLayoutMode() == LayoutMode::ReflowMode)
	{
		ShowOrHideAppBar();
		return;
	}

	if (m_textManager)
		m_textManager->OnTapped(sender, e);

	if (m_AnnotMgr)
		m_AnnotMgr->OnTapped(sender, e);
}

void UIExtensionsManager::CompositionScaleChanged(Windows::UI::Xaml::Controls::SwapChainPanel^ sender, Platform::Object^ args)
{
	
}

void UIExtensionsManager::SizeChanged(Platform::Object^ sender, Windows::UI::Xaml::SizeChangedEventArgs^ e)
{
	OutputDebugString(L"PageView::OnSizeChanged...\n");
	if (IsInThumbnailMode())
		return;

	if (TextManager) {
		TextManager->ClearTextSelect();
	}

	if (m_docViewMode == LayoutMode::ContinueMode)
	{
		m_pHostedControl->slider->CalculateStepOffset();
	}
	else
	{
		m_pHostedControl->sliderHor->CalculateStepOffset();
	}

	SetNoteCreateOrSave();
	SizeChangedCreateAnnot();

	if (m_AnnotMgr && m_AnnotMgr->GetSelectedAnnot())
	{
		auto pSelectedAnnot = m_AnnotMgr->GetSelectedAnnot();
		if (pSelectedAnnot->AnnotType == AnnotationType::FreeText && GetAnnotCanvasHasChild())
		{
			IBaseAnnotEventPeer^ annotEventPeer = AnnotMgr->GetAnnotHandlerByType(AnnotationType::FreeText)->AnnotEvent;
			FreeTextEventPeer^ freetextEventPeer = dynamic_cast<FreeTextEventPeer^>(annotEventPeer);
			OutputDebugString(L"Window_SizeChanged\r\n");
			freetextEventPeer->OnUpdateFreeTextContent();
		}

		if (pSelectedAnnot->AnnotType == AnnotationType::Widget)
		{
			SdkKillFocusFormControl();
			HideWidgetAnnotPopup();
		}

		SetAnnotCanvasVisible(false);
		m_AnnotMgr->SetPdfSelectAnnotNull();
		ExitAnnotPropertyMode();
	}
}

void UIExtensionsManager::ResetDefaultAppBarButton()
{
	if (m_isDesktopPlatform)
	{
		m_MainAppBar->InitMainAppbar(AppBarType::MainAppBar);
	}
	else
	{
		m_pHostedControl->AppBarLayout->Children->Clear();
		m_MainAppBar->InitMainAppbar(AppBarType::MainAppBar);
		m_MainAppBar->ActivateAppBar();
	}
	m_CurAppBarType = AppBarType::MainAppBar;
}

void UIExtensionsManager::ShowOrHideAppBar()
{
	Windows::UI::Xaml::Controls::Grid^ mainAppBar = m_pHostedControl->AppBarLayout;
	Windows::UI::Xaml::Controls::Grid^ mainAppBarBottom = m_pHostedControl->AppBarBottomLayout;
	if (((mainAppBar->Children->Size > 0 && m_MainAppBar->GetMainAppBarVisible() == Windows::UI::Xaml::Visibility::Visible)
		|| (mainAppBarBottom->Children->Size > 0 && m_MainAppBar->GetMainAppBarVisible() == Windows::UI::Xaml::Visibility::Visible))
		)
	{
		if (m_MainAppBar->GetMorePopupIsOpen())
		{
			m_MainAppBar->HideMorePopup();
			return;
		}
		else if (m_MainAppBar->GetViewToolPanelPopupIsOpen())
		{
			m_MainAppBar->HideViewToolPanelPopup();
			return;
		}
		else if (IsPhonePropertyPopupOpen())
		{
			m_MainAppBar->SetAnnotPropertyPopupClosed();
			return;
		}
	}

	if (m_isDesktopPlatform)
	{
		if (mainAppBar->Children->Size > 0)
		{
			if (m_MainAppBar->GetMainAppBarVisible() == Windows::UI::Xaml::Visibility::Visible)
			{
				
				m_MainAppBar->HideAppBar();
				m_MainAppBar->StopHideTimeTick();
				if (m_MainAppBar->GetCurAppBarType() == AppBarType::SearchBar)
				{
					HideSearchControl();
				}
					
			}
			else
			{
				if (m_MainAppBar->GetCurAppBarType() == AppBarType::SearchBar && m_isDesktopPlatform)
				{
					m_MainAppBar->ActivateAppBar();
					if (TextManager && TextManager->FindTextString && TextManager->FindTextString->Length() > 0 && TextManager->GetIsFoundText())
						ShowSearchControl();
					else
						HideSearchControl();
				}
				else
				{
					
					m_MainAppBar->ActivateAppBar();
				}
			}
		}
		else
		{
			InitMainAppBar();
		}
	}
	else
	{
		if (mainAppBarBottom->Children->Size > 0)
		{
			if (m_MainAppBar->GetMainAppBarVisible() == Windows::UI::Xaml::Visibility::Visible)
			{
				m_MainAppBar->HideAppBar();
				m_MainAppBar->StopHideTimeTick();
				
			}
			else
			{
				if (m_MainAppBar->GetCurAppBarType() == AppBarType::SearchBar
					&& mainAppBar->Visibility == Windows::UI::Xaml::Visibility::Visible)
				{
					HideSearchControl();
					mainAppBar->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
				}
				else if (m_MainAppBar->GetCurAppBarType() == AppBarType::SearchBar
					&& mainAppBar->Visibility == Windows::UI::Xaml::Visibility::Collapsed)
				{
					if (TextManager && TextManager->FindTextString && TextManager->FindTextString->Length() > 0 && TextManager->GetIsFoundText())
						ShowSearchControl();
					else
						HideSearchControl();
					mainAppBar->Visibility = Windows::UI::Xaml::Visibility::Visible;
				}
				else
				{
					
					m_MainAppBar->ActivateAppBar();
				}
			}
		}
	}

}

void UIExtensionsManager::ShowWidgetAnnotPopup()
{
	if (m_pFormOperationBarControl == nullptr)
		m_pFormOperationBarControl = ref new FormOperationBarControl();

	if (GetIsDesktopPlatform())
		m_pFormOperationBarControl->Margin = Windows::UI::Xaml::Thickness(0, 0, 0, 50);

	m_pFormOperationBarControl->SetPageView(m_control);
	if (!m_pHostedControl->FlyoutPopupPanel)
	{
		m_pHostedControl->FlyoutPopupPanel = ref new Windows::UI::Xaml::Controls::Grid();
		m_pHostedControl->FlyoutPopupPanel->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
		m_control->Layout->Children->Append(m_pHostedControl->FlyoutPopupPanel);
		m_control->Layout->UpdateLayout();
	}

	if (m_pHostedControl->FlyoutPopupPanel->Visibility == Windows::UI::Xaml::Visibility::Collapsed)
		m_pHostedControl->FlyoutPopupPanel->Visibility = Windows::UI::Xaml::Visibility::Visible;
	if (m_pHostedControl->FlyoutPopupPanel->VerticalAlignment != Windows::UI::Xaml::VerticalAlignment::Bottom)
		m_pHostedControl->FlyoutPopupPanel->VerticalAlignment = Windows::UI::Xaml::VerticalAlignment::Bottom;

	if (m_pHostedControl->FlyoutPopupPanel->Children->Size > 0)
	{
		if (m_pHostedControl->FlyoutPopupPanel->Children->GetAt(0) == m_pFormOperationBarControl)
		{
			m_pFormOperationBarControl->OpenFormListControl();
			m_pFormOperationBarControl->UpdateControlState();
		}
		else
		{
			m_pHostedControl->FlyoutPopupPanel->Children->Clear();
			m_pHostedControl->FlyoutPopupPanel->Children->Append(m_pFormOperationBarControl);
		}
	}
	else
	{
		m_pHostedControl->FlyoutPopupPanel->Children->Append(m_pFormOperationBarControl);
	}
}

void UIExtensionsManager::HideWidgetAnnotPopup()
{
	if (m_pHostedControl->FlyoutPopupPanel->Visibility == Windows::UI::Xaml::Visibility::Visible)
		m_pHostedControl->FlyoutPopupPanel->Visibility = Windows::UI::Xaml::Visibility::Collapsed;

	m_pHostedControl->FlyoutPopupPanel->Children->Clear();
}

void UIExtensionsManager::ShowFormOperationBarInPhone()
{
	m_CurAppBarType = AppBarType::FormOperationAppbar;

	ShowWidgetAnnotPopup();
}

void UIExtensionsManager::SetBottomBarIsVisible(bool isVisible)
{
	auto _layoutMode = m_control->GetPageLayoutMode();
	if (m_control->GetPageLayoutMode() == LayoutMode::ReflowMode)
	{
		return;
	}

	if (!m_isOpenFileComplete || !m_isDesktopPlatform)
		return;

	if (isVisible)
	{
		auto hideStoryboard = safe_cast<Windows::UI::Xaml::Media::Animation::Storyboard^>(m_pHostedControl->Resources->Lookup("BottomAppBarSlowHideStoryboard"));
		if (hideStoryboard)
			hideStoryboard->Stop();

		auto _prevState = m_pHostedControl->BottomAppBar->Visibility;
		m_pHostedControl->BottomAppBar->Opacity = 0.9;
		m_pHostedControl->BottomAppBar->Visibility = Windows::UI::Xaml::Visibility::Visible;
		

		m_BottomBarVisibleTimer->Start();
		
	}
	else
	{
		if (m_pHostedControl->BottomAppBar->Visibility == Windows::UI::Xaml::Visibility::Visible)
		{
			auto hideStoryboard = safe_cast<Windows::UI::Xaml::Media::Animation::Storyboard^>(m_pHostedControl->Resources->Lookup("BottomAppBarSlowHideStoryboard"));
			if (hideStoryboard)
			{
				
				hideStoryboard->Begin();
			}
		}

		
	}
}

void foxit::rdk::uiextensions::UIExtensionsManager::OnOpenPdfFileEvent(foxit::rdk::PDFViewCtrl ^sender, DocEventArgs^ e)
{
	m_OpenStateCode = e->ErroCode;

	if (e->ErroCode != foxit::common::ErrorCode::e_ErrSuccess)
	{
		if (e->IsCancelByUser)
		{
			ExitFileView();
			return;
		}

		if (e->ErroCode == foxit::common::ErrorCode::e_ErrCertificate || e->ErroCode == foxit::common::ErrorCode::e_ErrPassword
			|| e->ErroCode == foxit::common::ErrorCode::e_ErrSecurityHandler)
		{
			ShowOpenEncryptedDocumentErrorMsgDialog();
		}
		else if (e->ErroCode == foxit::common::ErrorCode::e_ErrFormat || e->ErroCode == foxit::common::ErrorCode::e_ErrUnknown)
		{
			ShowOpenPdfDocumentErrorDialog();
		}
		else
		{
			ShowOpenPdfDocumentErrorDialog();
		}
		return;
	}

	if (m_textManager == nullptr)
		m_textManager = ref new PDFTextManager(m_control);
	m_textManager->OnCreate(m_control);

	m_FormFillerMgr = ref new PDFFormFillerManagement(this);
	m_FormFillerMgr->Initialize();

	m_AnnotMgr = ref new AnnotationManager();
	LoadNoteLogo(m_AnnotMgr->GetAnnotSetting()->GetNoteLogoUrl());
	m_AnnotMgr->OnCreate(ref new PageViewAnnotMgrPeer(this));
	auto visiblePages = m_control->GetVisiblePages();
	for (unsigned int i = 0; i < visiblePages->Size; i++)
		m_AnnotMgr->OnInitAnnot(visiblePages->GetAt(i));
	m_control->HideAnnotType(foxit::pdf::annots::AnnotType::e_Note);

	m_docUndo = ref new DocUndo;

	m_isOpenFileComplete = true;

	InitMainAppBar();

	SetPageNumberSliderVisible(true);
}

void foxit::rdk::uiextensions::UIExtensionsManager::OnClosePdfFileEvent(foxit::rdk::PDFViewCtrl ^sender, DocEventArgs^ e)
{
	m_textManager->ForceReleaseResources();
	m_textManager->OnDelete();
	m_textManager = nullptr;

	if (m_AnnotMgr)
	{
		m_AnnotMgr->ClearResource();
		m_AnnotMgr = nullptr;
	}
	if (m_FormFillerMgr) {
		m_FormFillerMgr->Finalize();
	}
}

void UIExtensionsManager::ShowOpenEncryptedDocumentErrorMsgDialog()
{
	auto resourceLoader = Windows::ApplicationModel::Resources::ResourceLoader::GetForCurrentView("foxit.rdk.uiextensions/Resources");
	Platform::String^ FileNotOpenTip = resourceLoader->GetString(L"FileOpenEncryptedDocumentErrorTipsText");
	Windows::UI::Popups::MessageDialog^ dialog = ref new Windows::UI::Popups::MessageDialog(FileNotOpenTip);

	auto _is_canceled = m_TipsDlgCts.get_token().is_canceled();
	if (m_TipsDlgCts.get_token().is_canceled())
	{
		cancellation_token_source _newCts;
		m_TipsDlgCts = _newCts;
	}

	concurrency::create_task(dialog->ShowAsync(), m_TipsDlgCts.get_token())
		.then([=](Windows::UI::Popups::IUICommand^ command)
	{
		ExitFileView();
	});
}

bool UIExtensionsManager::GetIsCanScrollPageView()
{
	if (m_NumberPopup->IsOpen)
	{
		return false;
	}
	else if (m_AnnotCreateType != AnnotCreateType::None)
	{
		return false;
	}
	else if (m_PoppupNoteWindow->IsOpen)
	{
		return false;
	}
	else
	{
		if (!m_AnnotMgr) return false;
		auto _pSelectedAnnot = m_AnnotMgr->GetSelectedAnnot();
		if (_pSelectedAnnot)
		{
			if (_pSelectedAnnot->AnnotType == AnnotationType::FreeText)
			{
				if (GetAnnotCanvasHasChild())
				{
					return false;
				}
			}

			if (_pSelectedAnnot->AnnotType == AnnotationType::Widget)
			{
				return true;
			}

			m_AnnotMgr->SetPdfSelectAnnotNull();
			ExitAnnotPropertyMode();
		}
		return true;
	}
}


void UIExtensionsManager::ShowOpenPdfDocumentErrorDialog()
{
	auto resourceLoader = Windows::ApplicationModel::Resources::ResourceLoader::GetForCurrentView("foxit.rdk.uiextensions/Resources");
	Platform::String^ FileNotOpenTip = resourceLoader->GetString(L"FileNotOpenText");
	Windows::UI::Popups::MessageDialog^ dialog = ref new Windows::UI::Popups::MessageDialog(FileNotOpenTip);

	auto _is_canceled = m_TipsDlgCts.get_token().is_canceled();
	if (m_TipsDlgCts.get_token().is_canceled())
	{
		cancellation_token_source _newCts;
		m_TipsDlgCts = _newCts;
	}

	concurrency::create_task(dialog->ShowAsync(), m_TipsDlgCts.get_token())
		.then([=](Windows::UI::Popups::IUICommand^ command)
	{
		ExitFileView();
	});
}

bool UIExtensionsManager::IsInThumbnailMode()
{
	auto pGrid = m_pHostedControl->HostedPopupPanel;

	return (pGrid->Visibility == Windows::UI::Xaml::Visibility::Visible && pGrid->Children->Size > 0) ? true : false;
}

void UIExtensionsManager::EnterThumbnailPage()
{
	auto pThumbnailPage = FoxitUICustomControlFactory::Current->ConCreateThumbnailPage();
	if (pThumbnailPage->GetType()->FullName == ThumbnailPage::typeid->FullName)
	{
		auto _Page = dynamic_cast<ThumbnailPage^>(pThumbnailPage);
		_Page->SetPageView(this);
	}
	
	auto _pUIElement = dynamic_cast<Windows::UI::Xaml::UIElement^>(pThumbnailPage);
	if (_pUIElement == nullptr)
	{
		throw ref new Platform::NotImplementedException(L"must be a control");
	}

	m_pHostedControl->HostedPopupPanel->Children->Append(_pUIElement);
}

void UIExtensionsManager::ExitThumbnailPage()
{
	m_pHostedControl->HostedPopupPanel->Children->Clear();
}

void UIExtensionsManager::SetFlyoutPlacement(Windows::UI::Xaml::Controls::Grid^ pPlacementGrid)
{
	if (pPlacementGrid != m_FlyoutPlacement)
		m_FlyoutPlacement = pPlacementGrid;
}

int UIExtensionsManager::GetFormControlType(IBaseAnnot^ pAnnot)
{
	if (pAnnot == nullptr)
		return (int)foxit::pdf::interform::Type::e_TypeUnknown;

	FS_INT32 _Type = (int)foxit::pdf::interform::Type::e_TypeUnknown;
	PDFAnnotWrapper^ pWrap = (PDFAnnotWrapper^)pAnnot->AnnotHandleWrap->PDFAnnot;
	foxit::pdf::annots::Annot^ pSdkAnnot = (foxit::pdf::annots::Annot^)pWrap->Pointer;
	if (pSdkAnnot == nullptr)
		return _Type;
	
	auto _FormField = (dynamic_cast<foxit::pdf::annots::Widget^>(foxit::rdk::FXDocUtil::DynamicCastBaseAnnot(pSdkAnnot)))->GetField();
	
	if (_FormField)
	{
		_Type = (int)_FormField->GetType();
	}

	return _Type;
}

void UIExtensionsManager::SdkFocusFormControl(IBaseAnnot^ pAnnot)
{
	if (pAnnot == nullptr)
		return;

	Windows::Foundation::Point _Pt;

	int index = pAnnot->PageIndex;
	auto _MinRect = pAnnot->MinRect;

	foxit::common::fxcrt::Matrix2D^ matrix = Utility::ConvertToFoxitMatrix(m_control->GetDisplayMatrix(index));	
	foxit::pdf::interform::Type _FormType = (foxit::pdf::interform::Type)GetFormControlType(pAnnot);
	Windows::Foundation::Point _CenterPt = (_FormType == foxit::pdf::interform::Type::e_TypeListBox || _FormType == foxit::pdf::interform::Type::e_TypeCheckBox) ? Windows::Foundation::Point(_MinRect.X, _MinRect.Y) : Windows::Foundation::Point(_MinRect.X + _MinRect.Width / 2, _MinRect.Y + _MinRect.Height / 2);
	auto _DevicePt = m_control->ConvertPagePtToDevicePt(index, _CenterPt);
	auto _PageInfo = m_control->GetDoc()->GetPage(index);
	if (m_FormControlTappedPt.X != 0 || m_FormControlTappedPt.Y != 0)
	{
		if (pAnnot == AnnotMgr->GetSelectedAnnot())
			_Pt = m_FormControlTappedPt;
		else
			_Pt = _DevicePt;
	}
	else
	{
		_Pt = _DevicePt;
	}

	foxit::common::fxcrt::Matrix2D^ deviceToPage = ref new foxit::common::fxcrt::Matrix2D();
	deviceToPage->SetReverse(matrix);
	auto pdfPoint = ref new foxit::common::fxcrt::PointF(_Pt.X*m_control->CompositionScaleX, _Pt.Y*m_control->CompositionScaleY);

	deviceToPage->TransformPoint(pdfPoint);
	
	FS_RESULT iRet = -1;
	if (_PageInfo )
	{
		GetFormFiller()->OnLButtonDown(_PageInfo, pdfPoint, 0);
		
		
		if (_FormType == foxit::pdf::interform::Type::e_TypeTextField)
		{
			GetFormFiller()->OnLButtonUp(_PageInfo, pdfPoint, 0);
			
		}

		if (_FormType == foxit::pdf::interform::Type::e_TypeTextField)
		{
			if (m_pHostedControl->FormControlTextBox->FocusState == Windows::UI::Xaml::FocusState::Unfocused)
				m_pHostedControl->FormControlTextBox->Focus(Windows::UI::Xaml::FocusState::Programmatic);
		}
	}
}

void UIExtensionsManager::SdkKillFocusFormControl()
{
	GetFormFiller()->KillFocus();
}

void UIExtensionsManager::KillFormfillerFocus()
{
	GetFormFiller()->KillFocus();
}

IBaseAnnot^ UIExtensionsManager::GetCanSelectFormControl(Windows::Foundation::Collections::IVector<IBaseAnnot^>^ annotList, int nStartIndex, bool bReverse)
{
	IBaseAnnot^ _NextAnnot = nullptr;
	if (annotList == nullptr)
		return _NextAnnot;

	auto annotSize = annotList->Size;
	if (nStartIndex >= (int)annotSize || nStartIndex < 0)
		return _NextAnnot;

	if (bReverse)
	{
		for (int i = nStartIndex; i >= 0; i--)
		{
			auto item = annotList->GetAt(i);
			if (IsFormControlCanSelect(item))
			{
				_NextAnnot = item;
				break;
			}
		}
	}
	else
	{
		for (int i = nStartIndex; i < (int)annotSize; i++)
		{
			auto item = annotList->GetAt(i);
			if (IsFormControlCanSelect(item))
			{
				_NextAnnot = item;
				break;
			}
		}
	}

	return _NextAnnot;
}

void UIExtensionsManager::FocusPrevFormControl()
{
	auto _pSdkForm = GetSdkForm();
	auto _pSelectedAnnot = AnnotMgr->GetSelectedAnnot();
	if (_pSelectedAnnot && IsFormControlCanSelect(_pSelectedAnnot))
	{
		if (_pSdkForm && _pSelectedAnnot->AnnotType == AnnotationType::Widget)
		{
			auto _SelectedAnnotPageIndex = _pSelectedAnnot->PageIndex;

			auto _AnnotMgr = dynamic_cast<AnnotationManager^>(AnnotMgr);
			IBaseAnnot^ _NextAnnot = nullptr;
			auto _PageIndex = _SelectedAnnotPageIndex;
			auto _PageCount = m_control->GetDoc()->GetPageCount();
			
			bool _bIsCircleOne = false;

			do
			{
				if (_PageIndex < 0)
				{
					_PageIndex = _PageCount - 1;
				}
				

				auto _WidgetAnnots = AnnotMgr->GetPageWidgetAnnots(_PageIndex);

				auto _WidgetAnnotsSize = _WidgetAnnots->Size;
				if (_WidgetAnnotsSize == 0)
				{
					_PageIndex--;
					continue;
				}
				else
				{
					if (_PageIndex != _SelectedAnnotPageIndex || _bIsCircleOne)
					{
						_NextAnnot = GetCanSelectFormControl(_WidgetAnnots, _WidgetAnnotsSize - 1, true);
						if (_NextAnnot)
							break;

						_PageIndex--;
						continue;
					}
				}

				if (!_bIsCircleOne)
					_bIsCircleOne = true;

				int _SelectedIndex = -1;
				for (unsigned int i = 0; i < _WidgetAnnotsSize; i++)
				{
					auto item = _WidgetAnnots->GetAt(i);
					if (item == _pSelectedAnnot)
					{
						_SelectedIndex = i;
						break;
					}
				}

				_SelectedIndex--;
				_NextAnnot = GetCanSelectFormControl(_WidgetAnnots, _SelectedIndex, true);
				if (_NextAnnot)
					break;

				_PageIndex--;
			} while (1);

			
			
			foxit::pdf::interform::Control^ _FormControl;
			PDFAnnotWrapper^ pWrap = (PDFAnnotWrapper^)_NextAnnot->AnnotHandleWrap->PDFAnnot;

			foxit::pdf::annots::Widget^ widget = dynamic_cast<foxit::pdf::annots::Widget^>(foxit::rdk::FXDocUtil::DynamicCastBaseAnnot((foxit::pdf::annots::Annot^)pWrap->Pointer));
			_FormControl = widget->GetControl();
			
			if (_FormControl)
			{
				
				auto _FormField = _FormControl->GetField();
				Platform::String^ _FieldName = _FormField->GetName();
				FS_INT32 _Type = (FS_INT32)_FormField->GetType();
				FS_DWORD _Flag = (FS_DWORD)_FormField->GetFlags();
				
				SetFormControlTappedPoint(Windows::Foundation::Point(0, 0));
				m_control->GotoPagePoint(_NextAnnot->PageIndex, Windows::Foundation::Point(_NextAnnot->MinRect.X, _NextAnnot->MinRect.Y));
				SdkFocusFormControl(_NextAnnot);
				AnnotMgr->SetSelectedAnnot(_NextAnnot);

				OutputDebugString((L"FocusPrevFormControl..." + _FieldName + L", " + _Type + L", Flags: " + _Flag + L"\n")->Data());
			}
		}
	}
}

void UIExtensionsManager::FocusNextFormControl()
{
	auto _pSdkForm = GetSdkForm();
	auto _pSelectedAnnot = AnnotMgr->GetSelectedAnnot();
	if (_pSelectedAnnot && IsFormControlCanSelect(_pSelectedAnnot))
	{
		if (_pSdkForm && _pSelectedAnnot->AnnotType == AnnotationType::Widget)
		{
			auto _SelectedAnnotPageIndex = _pSelectedAnnot->PageIndex;

			auto _AnnotMgr = dynamic_cast<AnnotationManager^>(AnnotMgr);
			IBaseAnnot^ _NextAnnot = nullptr;
			auto _PageIndex = _SelectedAnnotPageIndex;
			auto _PageCount = m_control->GetDoc()->GetPageCount();
			
			bool _bIsCircleOne = false;

			do
			{
				if (_PageIndex >= _PageCount)
				{
					_PageIndex = 0;
				}

				auto _WidgetAnnots = AnnotMgr->GetPageWidgetAnnots(_PageIndex);

				auto _WidgetAnnotsSize = _WidgetAnnots->Size;
				if (_WidgetAnnotsSize == 0)
				{
					_PageIndex++;
					continue;
				}
				else
				{
					if (_PageIndex != _SelectedAnnotPageIndex || _bIsCircleOne)
					{
						_NextAnnot = GetCanSelectFormControl(_WidgetAnnots, 0, false);
						if (_NextAnnot)
							break;

						_PageIndex++;
						continue;
					}

				}

				if (!_bIsCircleOne)
					_bIsCircleOne = true;

				int _SelectedIndex = -1;
				for (unsigned int i = 0; i < _WidgetAnnotsSize; i++)
				{
					auto item = _WidgetAnnots->GetAt(i);
					if (item == _pSelectedAnnot)
					{
						_SelectedIndex = i;
						break;
					}
				}

				_SelectedIndex++;
				_NextAnnot = GetCanSelectFormControl(_WidgetAnnots, _SelectedIndex, false);
				if (_NextAnnot)
					break;

				_PageIndex++;
			} while (1);

			
			
			foxit::pdf::interform::Control^ _FormControl;
			PDFAnnotWrapper^ pWrap = (PDFAnnotWrapper^)_NextAnnot->AnnotHandleWrap->PDFAnnot;

			foxit::pdf::annots::Widget^ widget = dynamic_cast<foxit::pdf::annots::Widget^>(foxit::rdk::FXDocUtil::DynamicCastBaseAnnot((foxit::pdf::annots::Annot^)pWrap->Pointer));
			_FormControl = widget->GetControl();

			
			if (_FormControl)
			{
				SetFormControlTappedPoint(Windows::Foundation::Point(0, 0));
				m_control->GotoPagePoint(_NextAnnot->PageIndex, Windows::Foundation::Point(_NextAnnot->MinRect.X, _NextAnnot->MinRect.Y));
				SdkFocusFormControl(_NextAnnot);
				AnnotMgr->SetSelectedAnnot(_NextAnnot);
			}
		}
	}
}

void UIExtensionsManager::CollectAnnotOperation(Platform::String^ Type,
	Platform::String^ ActionType,
	Platform::String^ PageIndex,
	Platform::String^ Rect)
{
	
}

void UIExtensionsManager::SetAnnotCanvasVisible(bool isVisible)
{
	m_pHostedControl->gridFreeText->Children->Clear();

	if (!isVisible)
	{
		m_pHostedControl->AnnotCanvas->Children->Clear();
		m_pHostedControl->annotOperatingOverlayControl->HideControlView();
	}

	m_pHostedControl->GraphCanvas->ManipulationMode = Windows::UI::Xaml::Input::ManipulationModes::None;
	m_pHostedControl->border->Visibility = Windows::UI::Xaml::Visibility::Collapsed;

	m_LayoutGridTransform = ref new Windows::UI::Xaml::Media::CompositeTransform();
	m_LayoutGridTransform->TranslateX = 0;
	m_LayoutGridTransform->TranslateY = 0;
	m_pHostedControl->GraphCanvas->RenderTransform = m_LayoutGridTransform;

	m_pHostedControl->border->Width = 0;
	m_pHostedControl->border->Height = 0;
	m_pHostedControl->border->Margin = Windows::UI::Xaml::Thickness(0, 0, 0, 0);

}

void UIExtensionsManager::SetTextMenuPopupHide()
{
	if (m_PoppupTextMenu && m_PoppupTextMenu->IsOpen)
	{
		m_PoppupTextMenu->IsOpen = false;
	}
}

void UIExtensionsManager::CloseAddNoteWindows()
{
	if (m_PoppupNoteWindow)
	{
		m_PoppupNoteWindow->IsOpen = false;
		m_control->EnableScrollView(true);
	}
}

void UIExtensionsManager::ExitSearchMode()
{
	m_bIsAppBarInSearchMode = false;

	ResetDefaultAppBarButton();
}

void UIExtensionsManager::UpdateView()
{
	if (m_control == nullptr)
		return;

	m_control->Update();
}

void UIExtensionsManager::HideFindBar()
{
	bool bIsChange = false;
	if (TextManager)
	{
		bIsChange = TextManager->CurSearchTextInfo != nullptr;
		TextManager->StopSearchString();
	}

	ExitSearchMode();
	if (bIsChange)
		UpdateView();
}

int UIExtensionsManager::GetCurrentPageIndex()
{
	return m_control ? m_control->GetCurrentPage() : 0;
}

void UIExtensionsManager::ShowSelHandleButton(bool bIsShow)
{
	auto _Visibility = bIsShow ? Windows::UI::Xaml::Visibility::Visible : Windows::UI::Xaml::Visibility::Collapsed;

	if (m_pHostedControl->StartSelHandle->Visibility != _Visibility)
	{
		m_pHostedControl->StartSelHandle->Visibility = _Visibility;
		m_pHostedControl->StartSelHandle->UpdateLayout();
	}

	if (m_pHostedControl->EndSelHandle->Visibility != _Visibility)
	{
		m_pHostedControl->EndSelHandle->Visibility = _Visibility;
		m_pHostedControl->EndSelHandle->UpdateLayout();
	}
}

void UIExtensionsManager::ShowTouchSelectHandleButton(Windows::Foundation::Collections::IVector<IPageSelectText^>^ e)
{
	if (CreateTypeOfAnnot == AnnotCreateType::None)
	{
		if (!m_textManager->GetIsShowSelHandleBtn())
		{
			ShowSelHandleButton(false);
			return;
		}

		Windows::Foundation::Rect __rcstart, __rcend;

		float selhandleoffset = 0;
		float __startleft = 0, __starttop = 0;
		auto pageseltext = e;
		if (pageseltext->Size <= 0)
		{
			ShowSelHandleButton(false);
			return;
		}

		auto _Rotate = m_control->GetRotate();
		auto pagerc = pageseltext->GetAt(0)->TextRectVect;
		int index = pageseltext->GetAt(0)->PageIndex;

		if (pagerc->Size == 0)
		{
			ShowSelHandleButton(false);
			return;
		}

		auto _CompositionScaleX = m_control->CompositionScaleX;
		auto _CompositionScaleY = m_control->CompositionScaleY;
		Windows::Foundation::Point _Pt;
		Windows::Foundation::Rect _ViewRect(0, 0, (float)m_control->ActualWidth, (float)m_control->ActualHeight);
		__rcstart = pagerc->GetAt(0);
		auto _WrapDevicePt = m_control->ConvertPagePtToDevicePt(index, Windows::Foundation::Point(__rcstart.Left, __rcstart.Top));
		if (_WrapDevicePt != Windows::Foundation::Point())
		{
			_Pt = Windows::Foundation::Point(_WrapDevicePt.X, _WrapDevicePt.Y);
			auto _Transfrom = safe_cast<CompositeTransform^>(m_pHostedControl->StartSelHandle->RenderTransform);
			if (_Transfrom)
			{
				_Transfrom->TranslateX = _Pt.X;
				_Transfrom->TranslateY = _Pt.Y;
				_Transfrom->Rotation = (_Rotate % 4) * 90;
				m_pHostedControl->StartSelHandle->Visibility = _ViewRect.Contains(_Pt) ? Windows::UI::Xaml::Visibility::Visible : Windows::UI::Xaml::Visibility::Collapsed;
			}
		}
		else
		{
			m_pHostedControl->StartSelHandle->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
		}

		pagerc = pageseltext->GetAt(pageseltext->Size - 1)->TextRectVect;
		index = pageseltext->GetAt(pageseltext->Size - 1)->PageIndex;

		if (pagerc->Size == 0)
		{
			ShowSelHandleButton(false);
			return;
		}

		__rcend = pagerc->GetAt(pagerc->Size - 1);
		_WrapDevicePt = m_control->ConvertPagePtToDevicePt(index, Windows::Foundation::Point(__rcend.Right, __rcend.Top));
		if (_WrapDevicePt != Windows::Foundation::Point())
		{
			_Pt = Windows::Foundation::Point(_WrapDevicePt.X, _WrapDevicePt.Y);
			auto _Transfrom = safe_cast<CompositeTransform^>(m_pHostedControl->EndSelHandle->RenderTransform);
			if (_Transfrom)
			{
				_Transfrom->TranslateX = _Pt.X;
				_Transfrom->TranslateY = _Pt.Y;
				_Transfrom->Rotation = (_Rotate % 4) * 90;
				m_pHostedControl->EndSelHandle->Visibility = _ViewRect.Contains(_Pt) ? Windows::UI::Xaml::Visibility::Visible : Windows::UI::Xaml::Visibility::Collapsed;
			}
		}
		else
		{
			m_pHostedControl->EndSelHandle->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
		}
	}
}

void UIExtensionsManager::ShowTextmarkContextmenu(AnnotContextMenuType type, Windows::Foundation::Point viewPoint)
{
	if (m_MainAppBar && m_MainAppBar->GetCurAppBarType() == AppBarType::SearchBar)
		HideFindBar();

	if (m_AnnotCreateType == AnnotCreateType::None)
	{
		
	}

	m_showTextMinRect = Rect(0, 0, 0, 0);
	pageviewRect = Rect(0, 0, 0, 0);

	m_PoppupTextMenu->Child = nullptr;
	if (menuControl)
	{
		if (m_TextPopupLoadedEventToken.Value)
		{
			menuControl->Loaded -= m_TextPopupLoadedEventToken;
			m_TextPopupLoadedEventToken.Value = 0;
		}
	}
	menuControl = ref new ContextMenuUserControl(m_control, type);
	m_PoppupTextMenu->Child = menuControl;
	m_PoppupTextMenu->IsLightDismissEnabled = false;
	bool isPopupOpen = true;

	double popupWidth = m_PoppupTextMenu->ActualWidth;
	if (type == AnnotContextMenuType::Add)
	{
		if (TextManager->SelectedText->Size > 0 && TextManager->SelectedText->GetAt(0)->TextRectVect->Size > 0)
		{
			Windows::Foundation::Rect rect = TextManager->SelectedText->GetAt(0)->TextRectVect->GetAt(0);
			auto deviceRect = m_control->ConvertPageRectToDeviceRect(TextManager->SelectedText->GetAt(0)->PageIndex, rect);
			m_showTextMinRect = deviceRect;

			Windows::Foundation::Rect boundsRect = Windows::Foundation::Rect(0, 0, Windows::UI::Xaml::Window::Current->Bounds.Width, Windows::UI::Xaml::Window::Current->Bounds.Height);

			if (m_showTextMinRect.IsEmpty || !m_showTextMinRect.IntersectsWith(boundsRect))
			{
				if (m_PoppupTextMenu->IsOpen)
				{
					m_PoppupTextMenu->IsOpen = false;
				}

				return;
			}
			if (!FSPDFDocUtil::GetIsDocumentCanBeCopy(viewControl->GetDoc()))
			{
				return;
			}

			pageviewRect = m_control->GetPageViewRect(TextManager->SelectedText->GetAt(0)->PageIndex);
		}
		else
		{
			isPopupOpen = false;
		}
	}
	else if (type == AnnotContextMenuType::Delete || type == AnnotContextMenuType::NotePro)
	{
		Windows::Foundation::Rect pagerect = m_AnnotMgr->GetSelectedAnnot()->MinRect;

		if (m_AnnotMgr->GetSelectedAnnot()->AnnotType == AnnotationType::Text)
		{
			TextAnnot^ textAnnot = dynamic_cast<TextAnnot^>(m_AnnotMgr->GetSelectedAnnot());
			pagerect = textAnnot->DrawRect;
		}
		else if (m_AnnotMgr->GetSelectedAnnot()->AnnotType == AnnotationType::Caret
			|| m_AnnotMgr->GetSelectedAnnot()->AnnotType == AnnotationType::StrikeOut)
		{
			double MarginValue = 3;
			bool _IsNeedRecaulate = false;
			if (m_AnnotMgr->GetSelectedAnnot()->AnnotType == AnnotationType::StrikeOut)
			{
				StrikeOutAnnot^ strikeoutAnnot = dynamic_cast<StrikeOutAnnot^>(m_AnnotMgr->GetSelectedAnnot());
				if (strikeoutAnnot->IsHasGroup)
				{
					MarginValue = 5;
					_IsNeedRecaulate = true;
				}
			}
			else
			{
				_IsNeedRecaulate = true;
			}

			if (_IsNeedRecaulate)
			{
				pagerect.X = (float)(pagerect.X - MarginValue);
				pagerect.Y = (float)(pagerect.Y - MarginValue);
				pagerect.Width = (float)(pagerect.Width + MarginValue * 2);
				pagerect.Height = (float)(pagerect.Height + MarginValue * 2);
			}
		}

		pageviewRect = m_control->GetPageViewRect(m_AnnotMgr->GetSelectedAnnot()->PageIndex);
		auto deviceRect = m_control->ConvertPageRectToDeviceRect(m_AnnotMgr->GetSelectedAnnot()->PageIndex, pagerect);
		m_showTextMinRect = deviceRect;

		if (m_showTextMinRect.IsEmpty)
			return;
	}
	else if (type == AnnotContextMenuType::Note)
	{
		IBaseAnnot^ annot = m_AnnotMgr->GetSelectedAnnot();
		if (annot)
			pageviewRect = m_control->GetPageViewRect(annot->PageIndex);
		else
		{
			pageviewRect = m_control->GetPageViewRect(m_control->GetPageAtPoint(viewPoint));
		}
		m_showTextPoint = viewPoint;
	}
	else if (type == AnnotContextMenuType::NewSign || type == AnnotContextMenuType::ApplySign)
	{
		pageviewRect = m_control->GetPageViewRect(GetCurrentPageIndex());
		m_showTextPoint = viewPoint;
	}
	else
	{
		IBaseAnnot^ annot = m_AnnotMgr->GetSelectedAnnot();
		if (annot)
			pageviewRect = m_control->GetPageViewRect(annot->PageIndex);
		else
			pageviewRect = m_control->GetPageViewRect(m_control->GetPageAtPoint(viewPoint));
		m_showTextPoint = viewPoint;
	}

	if (isPopupOpen)
	{
		m_TextPopupLoadedEventToken = menuControl->Loaded += ref new Windows::UI::Xaml::RoutedEventHandler(this, &UIExtensionsManager::OnMenuControlLoaded);
		m_PoppupTextMenu->IsOpen = true;
	}
}

void UIExtensionsManager::OnMenuControlLoaded(Platform::Object ^sender, Windows::UI::Xaml::RoutedEventArgs ^e)
{
	std::wstringstream ws;

	ws << L"m_PoppupTextMenu [OnLoaded] -- HorizontalOffset: " << m_PoppupTextMenu->HorizontalOffset;
	ws << L", VerticalOffset: " << m_PoppupTextMenu->VerticalOffset;
	ws << L", ActualWidth: " << m_PoppupTextMenu->ActualWidth;
	ws << L", ActualHeight: " << m_PoppupTextMenu->ActualHeight;
	ws << L", TextRect: [" << pageviewRect.X << L"," << pageviewRect.Y << L"," << pageviewRect.Width << L"," << pageviewRect.Height << L"]";
	ws << L", TextRect: [" << m_showTextMinRect.X << L"," << m_showTextMinRect.Y << L"," << m_showTextMinRect.Width << L"," << m_showTextMinRect.Height << L"]";

	ws << L"\n";
	OutputDebugString(ws.str().c_str());

	if (!m_PoppupTextMenu)
		return;

	m_TextMenuTipHoffset = 0;
	double widthValue = menuControl->ActualWidth;
	double heightValue = menuControl->ActualHeight;

	AnnotContextMenuType type = menuControl->MenuType;
	if (type == AnnotContextMenuType::Add)
	{
		m_PoppupTextMenu->HorizontalOffset = m_showTextMinRect.X + m_showTextMinRect.Width / 2 - widthValue / 2;
		m_PoppupTextMenu->VerticalOffset = m_showTextMinRect.Y - heightValue;
	}
	else if (type == AnnotContextMenuType::Delete || type == AnnotContextMenuType::NotePro)
	{
		m_PoppupTextMenu->HorizontalOffset = m_showTextMinRect.X + m_showTextMinRect.Width / 2 - widthValue / 2;
		m_PoppupTextMenu->VerticalOffset = m_showTextMinRect.Y - heightValue;
	}
	else if (type == AnnotContextMenuType::Note)
	{
		m_PoppupTextMenu->HorizontalOffset = m_showTextPoint.X - widthValue / 2;
		m_PoppupTextMenu->VerticalOffset = m_showTextPoint.Y - heightValue;
	}
	else if (type == AnnotContextMenuType::NewSign || type == AnnotContextMenuType::ApplySign)
	{
		m_PoppupTextMenu->HorizontalOffset = m_showTextPoint.X - widthValue / 2;
		m_PoppupTextMenu->VerticalOffset = m_showTextPoint.Y - heightValue;
	}
	else
	{
		m_PoppupTextMenu->HorizontalOffset = m_showTextPoint.X - widthValue / 2;
		m_PoppupTextMenu->VerticalOffset = m_showTextPoint.Y - heightValue;
	}

	double hOffsetValue = 0;

	if (!Windows::UI::Xaml::RectHelper::GetIsEmpty(pageviewRect))
	{
		if (m_PoppupTextMenu->VerticalOffset < 0)
		{
			if (m_MainAppBar && m_MainAppBar->GetMainAppBarVisible() == Windows::UI::Xaml::Visibility::Visible)
			{
				double height = m_MainAppBar->ActualHeight;
				m_PoppupTextMenu->VerticalOffset = m_MainAppBar->ActualHeight;
			}
			else
			{
				m_PoppupTextMenu->VerticalOffset = 0;
			}
		}

		hOffsetValue = m_PoppupTextMenu->HorizontalOffset;
		if (m_PoppupTextMenu->HorizontalOffset < 0 && m_PoppupTextMenu->HorizontalOffset + widthValue < 0)
		{
			m_PoppupTextMenu->IsOpen = false;
			return;
		}
		else if (m_PoppupTextMenu->HorizontalOffset < 0 && m_PoppupTextMenu->HorizontalOffset + widthValue>0)
		{
			m_PoppupTextMenu->HorizontalOffset = 0;
		}
		if (m_PoppupTextMenu->HorizontalOffset < pageviewRect.X)
		{
			m_PoppupTextMenu->HorizontalOffset = pageviewRect.X;
		}
		else if (m_PoppupTextMenu->HorizontalOffset + widthValue > pageviewRect.X + pageviewRect.Width)
		{
			m_PoppupTextMenu->HorizontalOffset = pageviewRect.X + pageviewRect.Width - widthValue;
		}
		else if (pageviewRect.Width > m_WindowsRect.Width && m_PoppupTextMenu->HorizontalOffset + widthValue > m_WindowsRect.Width)
		{
			m_PoppupTextMenu->HorizontalOffset = m_WindowsRect.Width - widthValue;
		}
		if (!m_isDesktopPlatform)
		{
			if (m_PoppupTextMenu->HorizontalOffset + widthValue > m_WindowsRect.Width)
			{
				m_PoppupTextMenu->HorizontalOffset = m_WindowsRect.Width - widthValue;
			}
		}
	}
	hOffsetValue = m_PoppupTextMenu->HorizontalOffset;

	Windows::Foundation::Rect _visibilityFirstRect;
	_visibilityFirstRect = m_showTextMinRect.X > 0 ? Windows::Foundation::Rect(m_showTextMinRect) : Windows::Foundation::Rect(Windows::Foundation::Point(0, m_showTextMinRect.Y), Windows::Foundation::Point(m_showTextMinRect.Right, m_showTextMinRect.Bottom));

	m_TextMenuTipHoffset = (_visibilityFirstRect.X + _visibilityFirstRect.Width / 2) - (hOffsetValue + widthValue / 2);
	//menuControl = ref new ContextMenuUserControl(this->m_control, type);
	ContextMenuUserControl^ control = dynamic_cast<ContextMenuUserControl^>(m_PoppupTextMenu->Child);
	if (m_showTextMinRect != Rect(0, 0, 0, 0) && control && control->MenuType != AnnotContextMenuType::Delete)
	{
		
		control->SetDownLogLocation(m_TextMenuTipHoffset);
	}
}

void UIExtensionsManager::ResetAnnotToolAppBar()
{
	if (m_MainAppBar)
	{
		m_MainAppBar->ActivateAppBar();
		m_MainAppBar->ResetAnnotToolBG();
	}
	if (Windows::System::Profile::AnalyticsInfo::VersionInfo->DeviceFamily == L"Windows.Mobile")
	{
		ShowPhonAnnotPropertyIcon(false);
	}
}

void UIExtensionsManager::SetManipulationStartFlagFalse()
{
	m_ManipulationStart = false;
}

void UIExtensionsManager::SetUnRedoIsEnabled(bool isEnable)
{
	if (Windows::System::Profile::AnalyticsInfo::VersionInfo->DeviceFamily == L"Windows.Desktop")
	{
		if (m_MainAppBar)
		{
			m_MainAppBar->SetUnRedoIsEnabled(isEnable);
		}
	}
	else
	{
		if (m_pHostedControl->mainAppBar->Children->Size > 0)
		{
			UndoAppBarPhoneControl^ control = dynamic_cast<UndoAppBarPhoneControl^>(m_pHostedControl->mainAppBar->Children->GetAt(0));
			if (control)
			{
				control->SetUnRedoIsEnabled(isEnable);
			}
		}
	}
}

void UIExtensionsManager::CloseAllPropertyPopup()
{
	if (Windows::System::Profile::AnalyticsInfo::VersionInfo->DeviceFamily == L"Windows.Desktop")
	{
		if (m_MainAppBar)
		{
			m_MainAppBar->SetAnnotPropertyPopupClosed();
		}
	}
	else
	{
		if (m_pHostedControl->mainAppBar->Children->Size > 0)
		{
			UndoAppBarPhoneControl^ control = dynamic_cast<UndoAppBarPhoneControl^>(m_pHostedControl->mainAppBar->Children->GetAt(0));
			if (control)
			{
				control->SetPropertyPopupColsed();
			}
		}
	}
}

void UIExtensionsManager::InitSliderPageNumber(int pageIndex)
{
	auto docViewMode = viewControl->GetPageLayoutMode();
	m_pHostedControl->slider->RelatePageView(this);
	m_pHostedControl->sliderHor->RelatePageView(this);
	if (docViewMode == LayoutMode::ContinueMode || docViewMode == LayoutMode::ReflowMode)
	{
		if (m_pHostedControl->slider->Visibility == Windows::UI::Xaml::Visibility::Visible)
			m_pHostedControl->slider->InitPageNumText(pageIndex);
	}
	else
	{
		if (m_pHostedControl->sliderHor->Visibility == Windows::UI::Xaml::Visibility::Visible)
		{
			m_pHostedControl->sliderHor->InitPageNumText(pageIndex);
		}
	}
}

void UIExtensionsManager::SetPageNumberSliderVisible(bool isVisible)
{
	if (!m_isOpenFileComplete)
		return;

	if (m_MainAppBar && m_MainAppBar->GetCurAppBarType() == AppBarType::SearchBar)
	{
		return;
	}
	foxit::pdf::PDFDoc^ doc = viewControl->GetDoc();
	if (doc && doc->GetPageCount() <= 5)
		return;

	if (m_AnnotCreateType != AnnotCreateType::None)
		return;

	auto docViewMode = viewControl->GetPageLayoutMode();
	if (isVisible)
	{
		auto hideStoryboard = safe_cast<Windows::UI::Xaml::Media::Animation::Storyboard^>(m_pHostedControl->Resources->Lookup("PageNumberVerSliderHideStoryboard"));
		if (docViewMode == LayoutMode::ContinueMode || docViewMode == LayoutMode::ReflowMode)
		{
			m_pHostedControl->slider->RelatePageView(this);
			m_pHostedControl->slider->InitPageNumText(GetCurrentPageIndex());
			m_pHostedControl->slider->Visibility = Windows::UI::Xaml::Visibility::Visible;
			hideStoryboard = safe_cast<Windows::UI::Xaml::Media::Animation::Storyboard^>(m_pHostedControl->Resources->Lookup("PageNumberVerSliderHideStoryboard"));
		}
		else if (docViewMode == LayoutMode::SingleMode)
		{
			m_pHostedControl->sliderHor->RelatePageView(this);
			m_pHostedControl->sliderHor->InitPageNumText(GetCurrentPageIndex());
			m_pHostedControl->sliderHor->Visibility = Windows::UI::Xaml::Visibility::Visible;
			hideStoryboard = safe_cast<Windows::UI::Xaml::Media::Animation::Storyboard^>(m_pHostedControl->Resources->Lookup("PageNumberHorSliderHideStoryboard"));
		}

		if (hideStoryboard)
			hideStoryboard->Stop();
	}
	else
	{
		if (docViewMode == LayoutMode::ContinueMode || docViewMode == LayoutMode::ReflowMode)
		{
			if (m_pHostedControl->slider->Visibility == Windows::UI::Xaml::Visibility::Visible)
			{
				auto hideStoryboard = safe_cast<Windows::UI::Xaml::Media::Animation::Storyboard^>(m_pHostedControl->Resources->Lookup("PageNumberVerSliderHideStoryboard"));
				if (hideStoryboard)
					hideStoryboard->Begin();
			}
		}
		else if (docViewMode == LayoutMode::SingleMode)
		{
			if (m_pHostedControl->sliderHor->Visibility == Windows::UI::Xaml::Visibility::Visible)
			{
				auto hideStoryboard = safe_cast<Windows::UI::Xaml::Media::Animation::Storyboard^>(m_pHostedControl->Resources->Lookup("PageNumberHorSliderHideStoryboard"));
				if (hideStoryboard)
					hideStoryboard->Begin();
			}
		}
	}
}

void UIExtensionsManager::SetPageNumberSlideHide()
{
	auto docViewMode = viewControl->GetPageLayoutMode();
	if (docViewMode == LayoutMode::ContinueMode || docViewMode == LayoutMode::ReflowMode)
	{
		m_pHostedControl->slider->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
	}
	else if (docViewMode == LayoutMode::SingleMode)
	{
		m_pHostedControl->sliderHor->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
	}
}

void UIExtensionsManager::ShowPageNumberCountPopup(bool isActiveTimer)
{
	OutputDebugString(L"ShowPageNumberCountPopup\n");
	if (!m_ShowNumberPopup->IsOpen)
	{
		PageNumberShowControl^ numberControl = ref new PageNumberShowControl(this);
		m_ShowNumberPopup->Child = nullptr;
		m_ShowNumberPopup->Child = numberControl;
		if (isActiveTimer)
			numberControl->Activate();
		auto _winrect = Windows::UI::Xaml::Window::Current->Bounds;
		double width = 132;
		if (GetIsDesktopPlatform())
		{
			width = 200;
		}

		m_ShowNumberPopup->HorizontalOffset = _winrect.Width / 2 - width / 2;
		m_ShowNumberPopup->VerticalOffset = _winrect.Height / 2 - 132 / 2;

		m_ShowNumberPopup->IsOpen = true;
	}
	else
	{
		PageNumberShowControl^ numberControl = dynamic_cast<PageNumberShowControl^>(m_ShowNumberPopup->Child);
		if (numberControl)
		{
			if (isActiveTimer)
				numberControl->Activate();
			numberControl->InitPageCurIndex(GetCurrentPageIndex() + 1);
		}
	}
}

void UIExtensionsManager::HidePageNumberCountPopup()
{
	if (m_ShowNumberPopup && m_ShowNumberPopup->IsOpen)
	{
		m_ShowNumberPopup->IsOpen = false;
		OutputDebugString(L"HidePageNumberCountPopup\n");
	}
}

void UIExtensionsManager::SetAppBarHideTimer(bool isStop)
{
	if (!m_MainAppBar)
		return;

	m_MainAppBar->HideMorePopup();
	m_MainAppBar->HideViewToolPanelPopup();

	if (isStop)
	{
		m_MainAppBar->StopHideTimeTick();
	}
	else
	{
		m_MainAppBar->RestartTimeTick();
	}
}

void UIExtensionsManager::ShowPageNumberJumpWindow()
{
	OutputDebugString(L"ShowPageNumberJumpWindow\n");

	HidePageNumberCountPopup();
	if (m_AnnotMgr && m_AnnotMgr->GetSelectedAnnot())
	{
		if (m_AnnotMgr->GetSelectedAnnot()->AnnotType == AnnotationType::FreeText)
		{
			if (GetAnnotCanvasHasChild())
			{
				IBaseAnnotEventPeer^ annotEventPeer = AnnotMgr->GetAnnotHandlerByType(AnnotationType::FreeText)->AnnotEvent;
				FreeTextEventPeer^ freetextEventPeer = dynamic_cast<FreeTextEventPeer^>(annotEventPeer);
				freetextEventPeer->OnUpdateFreeTextContent();

				m_AnnotMgr->SetPdfSelectAnnotNull();
				SetAnnotCanvasVisible(false);
			}
		}
		else
		{
			m_AnnotMgr->SetPdfSelectAnnotNull();
			SetAnnotCanvasVisible(false);
			SetTextMenuPopupHide();
			CloseAddNoteWindows();
			UpdateView();
		}
	}
	else if (m_textManager && m_textManager->SelectedText->Size > 0)
	{
		m_textManager->ClearTextSelect();
		SetTextMenuPopupHide();
		UpdateView();
	}
	
	ExitAnnotPropertyMode();
	HideFloatWindow();

	PageNumberControl^ numberControl = ref new PageNumberControl(this);
	m_NumberPopup->Child = nullptr;
	m_NumberPopup->Child = numberControl;
	numberControl->FocusNumberTextBox();
	auto _winrect = Windows::UI::Xaml::Window::Current->Bounds;
	if (m_isDesktopPlatform)
	{
		m_NumberPopup->HorizontalOffset = _winrect.Width / 2 - POPUPWINDOWWIDTH / 2;
		m_NumberPopup->VerticalOffset = _winrect.Height / 2 - POPUPWINDOWHEIGHT / 2;
	}
	else
	{
		m_NumberPopup->HorizontalOffset = 0;
		m_NumberPopup->VerticalOffset = 0;
	}
	numberControl->FocusNumberTextBox();
	m_NumberPopup->IsOpen = true;

	m_MainAppBar->HideAppBar();
}

void UIExtensionsManager::HideJumpPagePopup()
{
	if (m_NumberPopup)
		m_NumberPopup->IsOpen = false;

	m_MainAppBar->ShowAppBar();
}

void UIExtensionsManager::JumpPageIndex(int pageIndex)
{
	if (m_control->GetDoc())
	{
		if (pageIndex != GetCurrentPageIndex())
			m_control->GotoPage(pageIndex);
		InitSliderPageNumber(pageIndex);
	}
}

void UIExtensionsManager::HideFloatWindow()
{
	if (m_MainAppBar)
	{
		m_MainAppBar->HideRotateFloatWindow();
		m_MainAppBar->HideMorePopup();
	}
}

void UIExtensionsManager::ExitAnnotPropertyMode()
{
	if (m_MainAppBar)
	{
		m_CurAppBarType = m_MainAppBar->GetCurAppBarType();
		ResetAnnotToolAppBar();
	}

	SetTextMenuPopupHide();
	if (propertyPopup)
		propertyPopup->IsOpen = false;
	CloseAddNoteWindows();
	m_SelectOperateType = SelectOperateType::Hand;
	m_AnnotCreateType = AnnotCreateType::None;
	SetUnRedoIsEnabled(true);
	SetAnnotCanvasVisible(false);
	CloseAllPropertyPopup();
	SetPageNumberSliderVisible(true);
}

bool UIExtensionsManager::ExitNotePopupEdit()
{ 
	if (m_PoppupNoteWindow->IsOpen == true)
	{
		
		return false;
	}
	return true;
}

Windows::Foundation::Point UIExtensionsManager::GetAnnotPopupOffset()
{
	Point pt(0, 0);
	if (m_PoppupNoteWindow && m_PoppupNoteWindow->IsOpen)
	{
		pt.X = (float)m_PoppupNoteWindow->HorizontalOffset;
		pt.Y = (float)m_PoppupNoteWindow->VerticalOffset;
	}
	return pt;
}

void UIExtensionsManager::SetPropertyPoupHide()
{
	if (propertyPopup && propertyPopup->IsOpen)
		propertyPopup->IsOpen = false;
}

void UIExtensionsManager::ReLocateImageSignature(Windows::Foundation::Rect rect)
{
	m_pHostedControl->imgSignature->SetValue(Windows::UI::Xaml::Controls::Canvas::TopProperty, rect.Y);
	m_pHostedControl->imgSignature->SetValue(Windows::UI::Xaml::Controls::Canvas::LeftProperty, rect.X);

	m_pHostedControl->imgSignature->Width = rect.Width;
	m_pHostedControl->imgSignature->Height = rect.Height;
}

void UIExtensionsManager::LoadNoteStylePage()
{
	
}

void UIExtensionsManager::UpdateFreeTextAnnotOverlayControl(int pageIndex, Windows::Foundation::Rect rect)
{
	SetAnnotCanvasVisible(false);
	auto graphminRect = m_control->ConvertPageRectToDeviceRect(pageIndex, rect);

	int pageRotate = m_control->GetRotate();
	if (foxit::common::Rotation::e_Rotation0 == (foxit::common::Rotation)pageRotate || foxit::common::Rotation::e_Rotation180 == (foxit::common::Rotation)pageRotate)
	{
		if (graphminRect.X < graphAnnotInPageRange.X)
		{
			graphminRect.X = graphAnnotInPageRange.X;
			graphminRect.Width = graphminRect.Width - (graphAnnotInPageRange.X - graphminRect.X);
			if (graphminRect.Width > graphAnnotInPageRange.Width)
				graphminRect.Width = graphAnnotInPageRange.Width;
		}
	}
	else
	{
		if (graphminRect.Y < graphAnnotInPageRange.Y)
		{
			graphminRect.Y = graphAnnotInPageRange.Y;
			graphminRect.Height = graphminRect.Height - (graphAnnotInPageRange.Y - graphminRect.Y);
		}

		if (graphminRect.Y + graphminRect.Height > graphAnnotInPageRange.Y + graphAnnotInPageRange.Height)
		{
			graphminRect.Y = graphAnnotInPageRange.Y;
			graphminRect.Height = graphAnnotInPageRange.Height;
		}
	}



	if (!graphminRect.IsEmpty && isfinite(graphminRect.Width) && isfinite(graphminRect.Height))
	{
		m_pHostedControl->annotOperatingOverlayControl->ShowControlView();
		m_pHostedControl->annotOperatingOverlayControl->CalculateInitialOperatingOverlayPosition(graphminRect, graphAnnotInPageRange);
	}
	else
	{
		m_pHostedControl->annotOperatingOverlayControl->HideControlView();
	}
}

void UIExtensionsManager::CreateGraphAnnotBySliderPage()
{
	IBaseAnnotHandler^ _pAnnotHandler;
	if (m_AnnotCreateType == AnnotCreateType::Pencil)
	{
		_pAnnotHandler = AnnotMgr->GetAnnotHandlerByType(AnnotationType::Ink);
	}
	else if (m_AnnotCreateType == AnnotCreateType::Line)
	{
		_pAnnotHandler = AnnotMgr->GetAnnotHandlerByType(AnnotationType::Line);
	}
	else if (m_AnnotCreateType == AnnotCreateType::Square)
	{
		_pAnnotHandler = AnnotMgr->GetAnnotHandlerByType(AnnotationType::Square);
	}
	else if (m_AnnotCreateType == AnnotCreateType::Circle)
	{
		_pAnnotHandler = AnnotMgr->GetAnnotHandlerByType(AnnotationType::Circle);
	}
	else if (m_AnnotCreateType == AnnotCreateType::Arrow)
	{
		_pAnnotHandler = AnnotMgr->GetAnnotHandlerByType(AnnotationType::Line);
	}
	else if (m_AnnotCreateType == AnnotCreateType::AreaHighLight)
	{
		_pAnnotHandler = AnnotMgr->GetAnnotHandlerByType(AnnotationType::HightLight);
	}
	else if (m_AnnotCreateType == AnnotCreateType::FreeText)
	{
		_pAnnotHandler = AnnotMgr->GetAnnotHandlerByType(AnnotationType::FreeText);
	}
	if (_pAnnotHandler)
		_pAnnotHandler->CreateAnnotAfterSelectText();

	if (m_AnnotCreateType == AnnotCreateType::FreeText && !AnnotMgr->GetAnnotSetting()->GetFreeTextIsContinueOpera())
	{
		m_AnnotCreateType = AnnotCreateType::None;
		
		SetAnnotCanvasVisible(false);
		CloseAllPropertyPopup();
		
	}
}

void UIExtensionsManager::SetPropertyCurColor(Windows::UI::Color color)
{
	if (Windows::System::Profile::AnalyticsInfo::VersionInfo->DeviceFamily == L"Windows.Desktop")
	{
		if (m_MainAppBar)
		{
			m_MainAppBar->SetPropertyCurColor(color);
		}
	}
}

void UIExtensionsManager::HideNoteStylePopup()
{
	if (m_isDesktopPlatform)
	{
		
		if (m_MainAppBar)
			m_MainAppBar->HideNoteStylePopup();
	}
	else
	{
		UndoAppBarPhoneControl^ control = dynamic_cast<UndoAppBarPhoneControl^>(m_pHostedControl->mainAppBar->Children->GetAt(0));
		if (control)
		{
			control->SetPropertyPopupColsed();
		}
	}
}

void UIExtensionsManager::ShowDirecControl(bool isVisible)
{
	
}

void UIExtensionsManager::SetAnnotStampClosed()
{
	if (Windows::System::Profile::AnalyticsInfo::VersionInfo->DeviceFamily == L"Windows.Desktop")
	{
		m_MainAppBar->SetAnnotToolStampPopupClosed();
	}
	else
	{
		m_MainAppBar->SetAnnotToolStampPopupClosed();
		if (m_pHostedControl->mainAppBar->Children->Size > 0)
		{
			UndoAppBarPhoneControl^ control = dynamic_cast<UndoAppBarPhoneControl^>(m_pHostedControl->mainAppBar->Children->GetAt(0));
			if (control)
			{
				control->SetPropertyPopupColsed();
			}
		}
	}
}

foxit::rdk::uiextensions::SelectOperateType UIExtensionsManager::SelectOperate::get()
{
	return m_SelectOperateType;
}

AnnotCreateType UIExtensionsManager::CreateTypeOfAnnot::get()
{
	return m_AnnotCreateType;
}

void UIExtensionsManager::CreateTypeOfAnnot::set(AnnotCreateType value)
{
	if (m_AnnotCreateType != value)
	{
		
		AnnotMgr->EndDrawing();

		m_AnnotCreateType = value;
		if (m_AnnotCreateType == AnnotCreateType::FreeText)
			SetUnRedoIsEnabled(false);
		else
			SetUnRedoIsEnabled(true);
	}
}

void UIExtensionsManager::AddAnnotByType(AnnotCreateType type)
{
	m_AnnotCreateType = type;
	

	m_SelectOperateType = SelectOperateType::SelectText;
	
	SetTextMenuPopupHide();
	if (TextManager->SelectedText && TextManager->SelectedText->Size > 0)
	{
		TextManager->ClearTextSelect();
		UpdateView();
	}
	if (type != AnnotCreateType::None)
		SetPageNumberSlideHide();
}

bool UIExtensionsManager::IsPopupShow() 
{
	if (m_NumberPopup && m_NumberPopup->IsOpen)
		return true;
	return false;
}

void UIExtensionsManager::CollectEraserOpration()
{
	if (m_EraserPageIndex != -1)
	{
		
		
		
		
	}
}

void foxit::rdk::uiextensions::UIExtensionsManager::SizeChangedCreateAnnot()
{
	if (!AnnotMgr) return;
	IBaseAnnotHandler^ _pAnnotHandler;
	bool ishasContent = false;
	if (m_AnnotCreateType == AnnotCreateType::Pencil)
	{
		_pAnnotHandler = AnnotMgr->GetAnnotHandlerByType(AnnotationType::Ink);
	}
	else if (m_AnnotCreateType == AnnotCreateType::Line)
	{
		_pAnnotHandler = AnnotMgr->GetAnnotHandlerByType(AnnotationType::Line);
	}
	else if (m_AnnotCreateType == AnnotCreateType::Square)
	{
		_pAnnotHandler = AnnotMgr->GetAnnotHandlerByType(AnnotationType::Square);
	}
	else if (m_AnnotCreateType == AnnotCreateType::Circle)
	{
		_pAnnotHandler = AnnotMgr->GetAnnotHandlerByType(AnnotationType::Circle);
	}
	else if (m_AnnotCreateType == AnnotCreateType::Arrow)
	{
		_pAnnotHandler = AnnotMgr->GetAnnotHandlerByType(AnnotationType::Line);
	}
	else if (m_AnnotCreateType == AnnotCreateType::AreaHighLight)
	{
		_pAnnotHandler = AnnotMgr->GetAnnotHandlerByType(AnnotationType::HightLight);
	}
	else if (m_AnnotCreateType == AnnotCreateType::FreeText && GetAnnotCanvasHasChild())
	{
		_pAnnotHandler = AnnotMgr->GetAnnotHandlerByType(AnnotationType::FreeText);
		FreeTextEventPeer^ handler = dynamic_cast<FreeTextEventPeer^>(_pAnnotHandler->AnnotEvent);

		if (handler && !handler->IsTextBoxHasContent())
			ishasContent = false;
		else
			ishasContent = true;
	}
	if (_pAnnotHandler)
		_pAnnotHandler->CreateAnnotAfterSelectText();

	if (m_AnnotCreateType == AnnotCreateType::FreeText && !AnnotMgr->GetAnnotSetting()->GetFreeTextIsContinueOpera() && ishasContent)
	{
		m_AnnotCreateType = AnnotCreateType::None;
		SetAnnotCanvasVisible(false);
		CloseAllPropertyPopup();
		ExitAnnotPropertyMode();
	}
}

bool UIExtensionsManager::IsSelectAnnotPhonePropertyOpen()
{
	if (propertyPopup && propertyPopup->IsOpen)
		return true;
	return false;
}


void UIExtensionsManager::ExitAnnotSelect()
{
	SaveOrCreateNoteAnnot();
	IBaseAnnot^ annot = m_AnnotMgr->GetSelectedAnnot();
	if (annot)
	{
		if (annot->AnnotType == AnnotationType::HightLight || annot->AnnotType == AnnotationType::Squiggly
			|| annot->AnnotType == AnnotationType::StrikeOut || annot->AnnotType == AnnotationType::UnderLine
			|| annot->AnnotType == AnnotationType::Caret)
		{
			m_AnnotMgr->SetPdfSelectAnnotNull();
			UpdateView();
		}

		if (annot->AnnotType == AnnotationType::FreeText)
		{
			if (GetAnnotCanvasHasChild())
			{
				IBaseAnnotEventPeer^ annotEventPeer = AnnotMgr->GetAnnotHandlerByType(AnnotationType::FreeText)->AnnotEvent;
				FreeTextEventPeer^ freetextEventPeer = dynamic_cast<FreeTextEventPeer^>(annotEventPeer);
				freetextEventPeer->OnUpdateFreeTextContent();
				SetAnnotCanvasVisible(false);
				UpdateView();
			}
		}

		if (annot->AnnotType == AnnotationType::Widget)
		{
			SdkKillFocusFormControl();
			HideWidgetAnnotPopup();
		}

		m_AnnotMgr->SetPdfSelectAnnotNull();
		SetAnnotCanvasVisible(false);
		

		SetTextMenuPopupHide();
		if (propertyPopup)
			propertyPopup->IsOpen = false;
	}
}

void UIExtensionsManager::SaveOrCreateNoteAnnot()
{
	if (m_PoppupNoteWindow)
	{
		if (m_PoppupNoteWindow->IsOpen && m_PoppupNoteWindow->Child)
		{
			auto control = (AnnotNoteUserControl^)m_PoppupNoteWindow->Child;

			if (m_AnnotMgr && m_AnnotMgr->GetSelectedAnnot())
			{
				control->SaveAnnotNote();
			}
			else
			{
				control->CreateNewNote();
			}
		}

		m_PoppupNoteWindow->IsOpen = false;
	}
}

void UIExtensionsManager::SetUnRedoPhoneAppBarVisible(bool isVisible)
{
	bool visibility = m_pHostedControl->mainAppBar->Visibility == Windows::UI::Xaml::Visibility::Visible ? true : false;

	if (m_pHostedControl->mainAppBar->Children->Size > 0)
	{
		UndoAppBarPhoneControl^ control = dynamic_cast<UndoAppBarPhoneControl^>(m_pHostedControl->mainAppBar->Children->GetAt(0));
		if (control)
		{
			if (isVisible)
			{
				m_pHostedControl->mainAppBar->Visibility = Windows::UI::Xaml::Visibility::Visible;
				control->ShowUnRedoControl();
			}
			else
			{
				control->HideUnRedoControl();
			}
		}
	}
}

void UIExtensionsManager::ShowPhonAnnotPropertyIcon(bool isShow)
{
	if (m_pHostedControl->mainAppBar->Children->Size > 0)
	{
		UndoAppBarPhoneControl^ control = dynamic_cast<UndoAppBarPhoneControl^>(m_pHostedControl->mainAppBar->Children->GetAt(0));
		if (control)
		{
			control->ShowAnnotEditControl(isShow);
		}
	}
}

void UIExtensionsManager::ShowAddNoteWindows(AnnotNoteUserControl^ control, Windows::Foundation::Point curPoint)
{
	if (m_MainAppBar && m_MainAppBar->GetCurAppBarType() == AppBarType::SearchBar)
		HideFindBar();
	m_PoppupNoteWindow->Child = control;

	if (m_isDesktopPlatform)
	{
		double verticalOffset = curPoint.Y + 30;
		double horizontalOffset = curPoint.X;
		if (verticalOffset + POPUPWINDOWHEIGHT > m_WindowsRect.Height)
		{
			verticalOffset = m_WindowsRect.Height - POPUPWINDOWHEIGHT;
		}
		if (horizontalOffset + POPUPWINDOWWIDTH > m_WindowsRect.Width)
		{
			horizontalOffset = m_WindowsRect.Width - POPUPWINDOWWIDTH;
		}
		m_PoppupNoteWindow->HorizontalOffset = horizontalOffset;
		m_PoppupNoteWindow->VerticalOffset = verticalOffset;
	}
	else
	{
		m_PoppupNoteWindow->HorizontalOffset = 0;
		m_PoppupNoteWindow->VerticalOffset = 0;
	}
	m_PoppupNoteWindow->IsOpen = true;
	m_PoppupTextMenu->IsOpen = false;
	m_control->EnableScrollView(false);
	
}

void UIExtensionsManager::CreateNewShapeToCanvas(Windows::UI::Xaml::Shapes::Shape^ shape)
{
	m_pHostedControl->GraphCanvas->ManipulationMode = Windows::UI::Xaml::Input::ManipulationModes::None;
	if (m_AnnotCreateType == AnnotCreateType::Pencil || m_AnnotCreateType == AnnotCreateType::Eraser)
	{
		Windows::UI::Xaml::Shapes::Polyline^ polyline = dynamic_cast<Windows::UI::Xaml::Shapes::Polyline^>(shape);
		if (polyline)
			m_pHostedControl->AnnotCanvas->Children->Append(polyline);
	}
	else if (m_AnnotCreateType == AnnotCreateType::Square)
	{
		Windows::UI::Xaml::Shapes::Rectangle^ rectangle = dynamic_cast<Windows::UI::Xaml::Shapes::Rectangle^>(shape);
		if (rectangle)
		{
			double borderWidth = rectangle->StrokeThickness;
			m_pHostedControl->AnnotCanvas->Children->Append(rectangle);
		}
	}
	else if (m_AnnotCreateType == AnnotCreateType::Circle)
	{
		Windows::UI::Xaml::Shapes::Ellipse^ ellipse = dynamic_cast<Windows::UI::Xaml::Shapes::Ellipse^>(shape);
		if (ellipse)
			m_pHostedControl->AnnotCanvas->Children->Append(ellipse);
	}
	else if (m_AnnotCreateType == AnnotCreateType::Line || m_AnnotCreateType == AnnotCreateType::Arrow)
	{
		Windows::UI::Xaml::Shapes::Line^ line = dynamic_cast<Windows::UI::Xaml::Shapes::Line^>(shape);
		if (line)
			m_pHostedControl->AnnotCanvas->Children->Append(line);
	}
}

void UIExtensionsManager::UpdatePolyLinePoints(Windows::UI::Xaml::Shapes::Shape^ shape, Windows::UI::Xaml::Media::PointCollection^ pointVector)
{
	if (m_AnnotCreateType == AnnotCreateType::Pencil)
	{
		Windows::UI::Xaml::Shapes::Polyline^ polyline = dynamic_cast<Windows::UI::Xaml::Shapes::Polyline^>(shape);
		if (polyline)
			polyline->Points = pointVector;
	}
}

void UIExtensionsManager::CreatePolyLineAnnotToCanvas(int pageIndex, Windows::Foundation::Rect minRect, Windows::Foundation::Collections::IVector<Windows::Foundation::Collections::IVector<Windows::Foundation::Point>^>^ Page_VectPoint, double opacity, double borderWidth, Windows::UI::Color color)
{
	SetAnnotCanvasVisible(false);

	if (Page_VectPoint && Page_VectPoint->Size > 0)
	{
		auto graphminRect = viewControl->ConvertPageRectToDeviceRect(pageIndex, minRect);

		
		
		
		graphAnnotInPageRange = viewControl->GetPageViewRect(pageIndex);

		m_pHostedControl->annotOperatingOverlayControl->ShowControlView();
		m_pHostedControl->annotOperatingOverlayControl->CalculateInitialOperatingOverlayPosition(graphminRect, graphAnnotInPageRange);
	}
}

void UIExtensionsManager::CreateRectangelAnnotToCanvas(int pageIndex, Windows::Foundation::Rect rect, double opacity, double borderWidth, Windows::UI::Color color)
{
	SetAnnotCanvasVisible(false);

	auto graphminRect = viewControl->ConvertPageRectToDeviceRect(pageIndex, rect);

	
	
	
	graphAnnotInPageRange = viewControl->GetPageViewRect(pageIndex);

	int pageRotate = viewControl->GetRotate();
	if (foxit::common::Rotation::e_Rotation0 == (foxit::common::Rotation)pageRotate || foxit::common::Rotation::e_Rotation180 == (foxit::common::Rotation)pageRotate)
	{
		if (graphminRect.X < graphAnnotInPageRange.X)
		{
			graphminRect.X = graphAnnotInPageRange.X;
			graphminRect.Width = graphminRect.Width - (graphAnnotInPageRange.X - graphminRect.X);
			if (graphminRect.Width > graphAnnotInPageRange.Width)
				graphminRect.Width = graphAnnotInPageRange.Width;
		}
		if (graphminRect.Width > graphAnnotInPageRange.Width)
		{
			graphminRect.Width = graphAnnotInPageRange.Width;
			graphminRect.X = graphAnnotInPageRange.X;
		}
	}
	else
	{
		if (graphminRect.Y < graphAnnotInPageRange.Y)
		{
			graphminRect.Y = graphAnnotInPageRange.Y;
			graphminRect.Height = graphminRect.Height - (graphAnnotInPageRange.Y - graphminRect.Y);
		}

		if (graphminRect.Y + graphminRect.Height > graphAnnotInPageRange.Y + graphAnnotInPageRange.Height)
		{
			graphminRect.Y = graphAnnotInPageRange.Y;
			graphminRect.Height = graphAnnotInPageRange.Height;
		}
	}
	

	m_pHostedControl->annotOperatingOverlayControl->ShowControlView();
	m_pHostedControl->annotOperatingOverlayControl->CalculateInitialOperatingOverlayPosition(graphminRect, graphAnnotInPageRange);
}

void UIExtensionsManager::CreateEllipseAnnotToCanvas(int pageIndex, Windows::Foundation::Rect rect, double opacity, double borderWidth, Windows::UI::Color color)
{
	SetAnnotCanvasVisible(false);

	auto graphminRect = viewControl->ConvertPageRectToDeviceRect(pageIndex, rect);

	
	
	
	graphAnnotInPageRange = viewControl->GetPageViewRect(pageIndex); 
	
	m_pHostedControl->annotOperatingOverlayControl->ShowControlView();
	m_pHostedControl->annotOperatingOverlayControl->CalculateInitialOperatingOverlayPosition(graphminRect, graphAnnotInPageRange);
}

void UIExtensionsManager::CreateLineAnnotToCanvas(int pageIndex, Windows::Foundation::Rect rect, Windows::Foundation::Point startPoint, Windows::Foundation::Point endPoint, double opacity, double borderWidth, Windows::UI::Color color)
{
	SetAnnotCanvasVisible(false);

	auto graphminRect = viewControl->ConvertPageRectToDeviceRect(pageIndex, rect);

	
	
	
	graphAnnotInPageRange = viewControl->GetPageViewRect(pageIndex);

	deviceLineStartPt = viewControl->ConvertPagePtToDevicePt(pageIndex, startPoint);
	deviceLineEndPt = viewControl->ConvertPagePtToDevicePt(pageIndex, endPoint);

	m_pHostedControl->annotOperatingOverlayControl->ShowControlView();
	m_pHostedControl->annotOperatingOverlayControl->CalculateInitialOperatingOverlayPosition(graphminRect, graphAnnotInPageRange);
	m_pHostedControl->annotOperatingOverlayControl->SetLineAnnotPoint(deviceLineStartPt, deviceLineEndPt);
}

void UIExtensionsManager::CreateFreeTextBoxToCanvas(Windows::UI::Xaml::Controls::UserControl^ userControl)
{
	m_pHostedControl->GraphCanvas->ManipulationMode = Windows::UI::Xaml::Input::ManipulationModes::None;
	if (userControl)
		m_pHostedControl->AnnotCanvas->Children->Append(userControl);
}

Platform::String^ UIExtensionsManager::GetResourceString(Platform::String^ str)
{
	auto resourceLoader = Windows::ApplicationModel::Resources::ResourceLoader::GetForCurrentView("foxit.rdk.uiextensions/Resources");

	return resourceLoader->GetString(str);
}

Platform::String^ UIExtensionsManager::GetAnnotTypeString(AnnotationType type)
{
	Platform::String^ typestr = nullptr;
	switch (type)
	{
	case AnnotationType::HightLight:
		typestr = "Highlight";
		break;
	case AnnotationType::UnderLine:
		typestr = "Underline";
		break;
	case AnnotationType::StrikeOut:
		typestr = "Strikeout";
		break;
	case AnnotationType::Square:
		typestr = "Rectangle";
		break;
	case AnnotationType::Circle:
		typestr = "Oval";
		break;
	case AnnotationType::Line:
		typestr = "Line";
		break;
	case AnnotationType::Ink:
		typestr = "Pencil";
		break;
	case AnnotationType::Squiggly:
		typestr = "SquigglyUnderline";
		break;
	case AnnotationType::Text:
		typestr = "Note";
		break;
	case AnnotationType::FreeText:
		typestr = "Typewriter";
		break;
	case AnnotationType::Stamp:
		typestr = "Stamp";
		break;
	case AnnotationType::None:
		break;
	default:
		break;
	}

	return typestr;
}

void UIExtensionsManager::LoadAnnotAppBar(AppBarType appbarType)
{
	LoadAnnotAppBarEdit(appbarType);
}

void UIExtensionsManager::LoadAnnotAppBarEdit(AppBarType appbarType)
{
	IBaseAnnot^ annot = AnnotMgr->GetSelectedAnnot();
	if (annot)
	{
		if (Windows::System::Profile::AnalyticsInfo::VersionInfo->DeviceFamily == L"Windows.Desktop")
		{
			if (control)
			{
				if (m_propertyPopupLoadedEventToken.Value)
				{
					control->Loaded -= m_propertyPopupLoadedEventToken;
					m_propertyPopupLoadedEventToken.Value = 0;
				}
			}

			control = ref new AnnotPropertyAppBar(m_control);
			m_propertyPopupLoadedEventToken = control->Loaded += ref new Windows::UI::Xaml::RoutedEventHandler(this, &foxit::rdk::uiextensions::UIExtensionsManager::OnAnnotPropertyAppBarLoaded);
			propertyPopup->Child = control;
		}
		else
		{
			AnnotPropertyPhone^ phoneProperty = ref new AnnotPropertyPhone(m_control);
			propertyPopup->Child = phoneProperty;
			propertyPopup->HorizontalOffset = 0;
			propertyPopup->VerticalOffset = 0;
		}
		propertyPopup->IsOpen = true;
	}
}

void UIExtensionsManager::LoadAppBarByType(AppBarType appbarType)
{
	m_MainAppBar->InitMainAppbar(appbarType);
}

void UIExtensionsManager::ShowDrawSignatureControl()
{

}

Windows::Foundation::Point UIExtensionsManager::GetSignPoint()
{ 
	return sigPt;
}

void UIExtensionsManager::HideSignatureCanvas()
{
	m_pHostedControl->imgSignature->Source = nullptr;
	m_pHostedControl->signImgCanvas->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
	SetTextMenuPopupHide();
	m_pHostedControl->annotOperatingOverlayControl->HideControlView();
	
}

void UIExtensionsManager::HideDrawSignatureControl()
{
	m_pHostedControl->drawSigControl->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
}

void UIExtensionsManager::SetAnnotPropertyToolIsEnable(bool isVisible)
{
	if (m_pHostedControl->mainAppBar->Children->Size > 0)
	{
		AnnotPropertyAppBar^ annotPropertyBar = dynamic_cast<AnnotPropertyAppBar^>(m_pHostedControl->mainAppBar->Children->GetAt(0));
		if (annotPropertyBar)
		{
			annotPropertyBar->SetPropertyToolIsEnabled(isVisible);
		}
	}
}

void UIExtensionsManager::SetEraserPageIndex(int pageIndex)
{
	m_EraserPageIndex = pageIndex;
}

bool UIExtensionsManager::GetAnnotCanvasHasChild() 
{ 
	if (m_pHostedControl->AnnotCanvas->Children->Size > 0)
		return true;
	return false;
}

TypeWriterRange^ UIExtensionsManager::GetTypeWriterContent(int m_pageIndex, Platform::String^ content, double fontsize, Windows::UI::Xaml::Media::FontFamily^ fontfamily, Windows::UI::Text::FontStyle fontstyle, Windows::UI::Text::FontWeight fontweight, double minWidth, double maxWidth)
{
	textStr = "";
	copyStr = "";
	linecount = 0;
	height = 0;

	auto calculateBox = m_pHostedControl->calculateBox;
	calculateBox->FontSize = fontsize;
	calculateBox->FontFamily = fontfamily;
	calculateBox->FontStyle = fontstyle;
	calculateBox->FontWeight = fontweight;
	calculateBox->MinWidth = minWidth;
	calculateBox->MaxWidth = maxWidth;
	calculateBox->MinHeight = 8;

	float totalheight = 0;
	float totalwidth = 0;
	try
	{
		bool isAppendSpace = false;

		double actualHeight = calculateBox->ActualHeight;
		double actualWidth = calculateBox->ActualWidth;

		int length = content->Length();
		const wchar_t* _strptr = content->Data();
		Windows::Foundation::Rect pageViewRect = viewControl->GetPageViewRect(m_pageIndex);

		int pageRotate = viewControl->GetRotate();
		float pageHeight = pageViewRect.Height;
		if (foxit::common::Rotation::e_Rotation90 == (foxit::common::Rotation)pageRotate || foxit::common::Rotation::e_Rotation270 == (foxit::common::Rotation)pageRotate)
		{
			pageHeight = pageViewRect.Width;
		}


		for (int i = 0; i < length; i++)
		{
			if (i - 1 >= 0 && _strptr[i - 1] != '\r' && _strptr[i] == '\n')
			{
				continue;
			}

			copyStr = copyStr + _strptr[i];
			calculateBox->Text = copyStr;
			actualHeight = calculateBox->ActualHeight;
			actualWidth = calculateBox->ActualWidth;

			if (_strptr[i] == '\r')
			{
				textStr = textStr + _strptr[i];
				continue;
			}
			if (_strptr[i] == '\n')
			{
				if (totalheight + height >= pageHeight)
					break;

				totalheight = totalheight + (float)height;

				if (totalwidth < actualWidth)
					totalwidth = (float)actualWidth;
				textStr = textStr + _strptr[i];
				isAppendSpace = true;
				copyStr = "";
				height = 0;
				linecount = 0;
				continue;
			}

			if (actualHeight >= height + 8)
			{
				if (!isAppendSpace)
				{
					if (linecount > 0)
					{
						if (totalheight + height >= pageHeight)
							break;
						textStr = textStr + "\n";
						copyStr = "" + _strptr[i];
						height = height;
						linecount = 1;
						totalheight = totalheight + (float)height;

						if (totalwidth < actualWidth)
							totalwidth = (float)actualWidth;
					}
					else
					{
						height = actualHeight;
						linecount = linecount + 1;
					}
				}
				else
				{
					height = actualHeight;
					linecount = 1;
				}
				textStr = textStr + _strptr[i];
			}
			else
			{
				if (!isAppendSpace)
				{
					textStr = textStr + _strptr[i];
				}
			}
			isAppendSpace = false;
		}

		if (totalheight + height <= pageHeight)
			totalheight = totalheight + (float)height;
		else
			totalheight = pageHeight;

		if (totalwidth < actualWidth)
			totalwidth = (float)actualWidth;
		calculateBox->Text = "";

		return ref new TypeWriterRange(textStr, totalheight, totalwidth);
	}
	catch (...)
	{
		calculateBox->Text = "";
		return ref new TypeWriterRange(textStr, totalheight, totalwidth);
	}
}

void UIExtensionsManager::SetFreeTextPropertyShow(bool isShow, Platform::String^ title)
{
	IBaseAnnot^ annot = getAnnotMgr()->GetSelectedAnnot();
	if (m_isDesktopPlatform)
	{
		if (annot)
		{
			if (propertyPopup->IsOpen)
			{
				AnnotPropertyAppBar^ propertyBar = dynamic_cast<AnnotPropertyAppBar^>(propertyPopup->Child);
				
				if (propertyBar)
					propertyBar->SetFreeTextPropertyShow(isShow);

				String^ titleStr = GetResourceString(L"FontNameStrTitle");
				if (titleStr == title)
					ResetPropertyPopupLocation(!isShow);
			}
		}
		else
		{
			if (m_MainAppBar)
				m_MainAppBar->SetFreeTextPropertyShow(isShow);
		}
	}
	else
	{
		if (annot)
		{
			if (propertyPopup->IsOpen)
			{
				AnnotPropertyPhone^ phoneProperty = dynamic_cast<AnnotPropertyPhone^>(propertyPopup->Child);
				if (phoneProperty)
				{
					phoneProperty->SetFreeTextPropertyIsShow(isShow, title);
				}
			}
		}
		else
		{
			
		}
	}
}

Windows::UI::Xaml::Media::PointCollection^ UIExtensionsManager::RemoveDrawPolyLine()
{
	Windows::UI::Xaml::Media::PointCollection^ ptCollection = ref new Windows::UI::Xaml::Media::PointCollection();
	int size = m_pHostedControl->AnnotCanvas->Children->Size;
	if (size > 0)
	{
		Windows::UI::Xaml::Shapes::Polyline^ polyline = dynamic_cast<Windows::UI::Xaml::Shapes::Polyline^>(m_pHostedControl->AnnotCanvas->Children->GetAt(size - 1));
		if (polyline)
			ptCollection = polyline->Points;

		m_pHostedControl->AnnotCanvas->Children->RemoveAt(size - 1);
	}

	return ptCollection;
}

void UIExtensionsManager::CreateNewPolyLineToCanvas(Windows::UI::Xaml::Shapes::Polyline^ polyLine)
{
	if (polyLine)
		m_pHostedControl->AnnotCanvas->Children->Append(polyLine);
}

Windows::Foundation::Collections::IVector<Windows::UI::Xaml::Media::PointCollection^>^ UIExtensionsManager::GetPolyLinePoints()
{ 
	if ((m_AnnotCreateType == AnnotCreateType::Pencil)
		&& m_pHostedControl->AnnotCanvas->Children->Size > 0)
	{
		if (m_pHostedControl->AnnotCanvas->Children->Size < 1)
			return nullptr;
		Windows::Foundation::Collections::IVector<Windows::UI::Xaml::Media::PointCollection^>^ points = ref new  Platform::Collections::Vector<Windows::UI::Xaml::Media::PointCollection^>();
		auto pCollection = m_pHostedControl->AnnotCanvas->Children;
		for (unsigned int i = 0; i < pCollection->Size; i++)
		{
			Windows::UI::Xaml::Shapes::Polyline^ polyline = dynamic_cast<Windows::UI::Xaml::Shapes::Polyline^>(pCollection->GetAt(i));
			if (polyline)
				points->Append(polyline->Points);
		}
		return points;
	}

	return nullptr;
}

void UIExtensionsManager::UpdateAnnotToolUnReDoControl()
{
	if (m_MainAppBar)
	{
		m_MainAppBar->UpdateUndoControlState();
		m_MainAppBar->UpdateAnnotToolUnRedo();
	}
}

void UIExtensionsManager::SetFormControlTappedPoint(Windows::Foundation::Point pt)
{
	m_FormControlTappedPt = pt;
}

bool UIExtensionsManager::IsWidgetAnnotPopupOpend() 
{ 
	return m_pHostedControl->FlyoutPopupPanel->Visibility == Windows::UI::Xaml::Visibility::Visible && m_pHostedControl->FlyoutPopupPanel->Children->Size > 0 ? true : false;
}

bool UIExtensionsManager::IsFormControlCanSelect(IBaseAnnot^ pAnnot) 
{ 
	if (pAnnot == nullptr)
		return false;

	if (!GetCanModifyForm())
		return false;

	foxit::pdf::interform::Control^ _FormControl;
	foxit::rdk::uiextensions::PDFAnnotWrapper^ pWrap = (PDFAnnotWrapper^)pAnnot->AnnotHandleWrap->PDFAnnot;
	foxit::pdf::annots::Annot^ pSdkAnnot = (foxit::pdf::annots::Annot^)pWrap->Pointer;
	if (pSdkAnnot == nullptr)
		return false;
	
	_FormControl = (dynamic_cast<foxit::pdf::annots::Widget^>(foxit::rdk::FXDocUtil::DynamicCastBaseAnnot(pSdkAnnot)))->GetControl();
	

	if (_FormControl == nullptr)
		return false;

	foxit::pdf::interform::Form^ _pSdkForm = GetSdkForm();
	
	
	auto field = _FormControl->GetField();
	Platform::String^ _FieldName = field->GetName();

	foxit::pdf::interform::Type _FieldType = field->GetType();
	
	if (_FieldType == foxit::pdf::interform::Type::e_TypePushButton)
		return false;

	FS_DWORD _AnnotFlag = pSdkAnnot->GetFlags();
	
	if ((_AnnotFlag & (FS_DWORD)foxit::pdf::annots::Flags::e_FlagInvisible) == (FS_DWORD)foxit::pdf::annots::Flags::e_FlagInvisible)
		return false;

	if ((_AnnotFlag & (FS_DWORD)foxit::pdf::annots::Flags::e_FlagHidden) == (FS_DWORD)foxit::pdf::annots::Flags::e_FlagHidden)
		return false;

	if ((_AnnotFlag &(FS_DWORD)foxit::pdf::annots::Flags::e_FlagNoView) == (FS_DWORD)foxit::pdf::annots::Flags::e_FlagNoView)
		return false;

	FS_DWORD _Flag = (FS_DWORD)field->GetFlags();
	
	return (_Flag & (FS_DWORD)foxit::pdf::interform::Flags::e_FlagReadOnly) == (FS_DWORD)foxit::pdf::interform::Flags::e_FlagReadOnly ? false : true;
}

bool UIExtensionsManager::IsValidFormControlPtr(IBaseAnnot^ pAnnot) 
{ 
	if (pAnnot == nullptr)
		return false;

	foxit::pdf::interform::Control^ _FormControl;
	foxit::rdk::uiextensions::PDFAnnotWrapper^ pWrap = (PDFAnnotWrapper^)pAnnot->AnnotHandleWrap->PDFAnnot;
	foxit::pdf::annots::Annot^ pSdkAnnot = (foxit::pdf::annots::Annot^)pWrap->Pointer;
	if (pSdkAnnot == nullptr)
		return false;

	foxit::pdf::annots::Widget^ widget = dynamic_cast<foxit::pdf::annots::Widget^>(foxit::rdk::FXDocUtil::DynamicCastBaseAnnot(pSdkAnnot));
	if (!widget)
		return false;
	_FormControl = widget->GetControl();
	

	if (_FormControl == nullptr)
		return false;
	return true;
}

void UIExtensionsManager::ExitSearchingModel()
{
	if (m_MainAppBar && m_MainAppBar->GetCurAppBarType() == AppBarType::SearchBar)
		HideFindBar();
}

void UIExtensionsManager::SetMainAppBarVisible(bool isVisible)
{
	if (m_MainAppBar)
	{
		if (isVisible)
			m_MainAppBar->ShowAppBar();
		else
			m_MainAppBar->HideAppBar();
	}
}

void UIExtensionsManager::InitStampAppBar()
{
	if (Windows::System::Profile::AnalyticsInfo::VersionInfo->DeviceFamily == L"Windows.Desktop")
	{
		if (m_MainAppBar)
		{
			m_MainAppBar->InitStampAppBarSelect();
		}
	}
}

void UIExtensionsManager::SetNoteStylePageIsActive(bool isActive)
{

}

void UIExtensionsManager::InitCurNoteIcon()
{
	if (Windows::System::Profile::AnalyticsInfo::VersionInfo->DeviceFamily == L"Windows.Desktop")
	{
		if (m_MainAppBar)
		{
			m_MainAppBar->InitCurNoteIcon();
		}
	}
}

bool foxit::rdk::uiextensions::UIExtensionsManager::IsAnnotAppBarTipShow()
{
	if (m_pHostedControl->annotTipControl->Visibility == Windows::UI::Xaml::Visibility::Visible)
		return true;
	return false;
}

void UIExtensionsManager::SetAnnotPropertyTipShow(bool isShow)
{
	if (isShow)
	{
		m_pHostedControl->AnnotPropertyTipGird->Visibility = Windows::UI::Xaml::Visibility::Visible;
		m_pHostedControl->annotTipControl->Visibility = Windows::UI::Xaml::Visibility::Visible;
	}
	else
	{
		m_pHostedControl->AnnotPropertyTipGird->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
		m_pHostedControl->annotTipControl->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
	}
}

void UIExtensionsManager::HidePopupWindow()
{
	if (m_PoppupTextMenu && m_PoppupTextMenu->IsOpen)
	{
		m_PoppupTextMenu->IsOpen = false;
	}
	if (m_PoppupNoteWindow && m_PoppupNoteWindow->IsOpen)
	{
		m_PoppupNoteWindow->IsOpen = false;
	}
	if (m_NumberPopup && m_NumberPopup->IsOpen)
	{
		m_NumberPopup->IsOpen = false;
	}
	
	HideNoteStylePopup();
}

void UIExtensionsManager::ShowUndoRedoBarInPhone()
{
	UndoAppBarPhoneControl^ _controlBar = ref new UndoAppBarPhoneControl();
	Windows::UI::Xaml::Controls::Grid^ mainAppBar = m_pHostedControl->AppBarLayout;
	mainAppBar->Children->Clear();
	mainAppBar->Children->Append(_controlBar);
}

void UIExtensionsManager::HideUndoRedoBarInPhone()
{
	Windows::UI::Xaml::Controls::Grid^ mainAppBar = m_pHostedControl->AppBarLayout;
	mainAppBar->Children->Clear();
}

void foxit::rdk::uiextensions::UIExtensionsManager::ShowSearchControl()
{
	m_bIsAppBarInSearchMode = true;
}

void foxit::rdk::uiextensions::UIExtensionsManager::HideSearchControl()
{
}

bool foxit::rdk::uiextensions::UIExtensionsManager::IsPhonePropertyPopupOpen()
{
	if (m_MainAppBar->IsAnnotPropertyPopupIsOpen())
		return true;
	auto mainAppBar = m_pHostedControl->AppBarLayout;
	if (!GetIsDesktopPlatform() && mainAppBar->Children->Size > 0)
	{
		UndoAppBarPhoneControl^ control = dynamic_cast<UndoAppBarPhoneControl^>(mainAppBar->Children->GetAt(0));
		if (control && control->ISPropertyPopupIsOpen())
		{
			return true;
		}
	}
	return false;
}

void foxit::rdk::uiextensions::UIExtensionsManager::SetPhonePropertyPopupClosed()
{
	bool isMobileAnnotMoreOpen = false;
	if (m_MainAppBar)
	{
		isMobileAnnotMoreOpen = m_MainAppBar->IsMobileAnnnotMoreToolOpen();
		m_MainAppBar->SetAnnotPropertyPopupClosed();
	}
	auto mainAppBar = m_pHostedControl->AppBarLayout;
	if (!GetIsDesktopPlatform() && mainAppBar->Children->Size > 0)
	{
		UndoAppBarPhoneControl^ control = dynamic_cast<UndoAppBarPhoneControl^>(mainAppBar->Children->GetAt(0));
		if (control->ISPropertyPopupIsOpen())
		{
			control->SetPropertyPopupColsed();
		}
	}
	if (m_AnnotCreateType == AnnotCreateType::FreeText && GetAnnotCanvasHasChild())
	{
		IBaseAnnotEventPeer^ annotEventPeer = AnnotMgr->GetAnnotHandlerByType(AnnotationType::FreeText)->AnnotEvent;
		FreeTextEventPeer^ freetextEventPeer = dynamic_cast<FreeTextEventPeer^>(annotEventPeer);
		if (freetextEventPeer && !isMobileAnnotMoreOpen)
			freetextEventPeer->SetFreeTextBoxGetFocus();
	}
}

void UIExtensionsManager::FocusFormControlHiddenTextBox()
{
	if (!m_pHostedControl->FormControlTextBox->IsHitTestVisible)
		m_pHostedControl->FormControlTextBox->IsHitTestVisible = true;
	if (!m_pHostedControl->FormControlTextBox->IsTabStop)
		m_pHostedControl->FormControlTextBox->IsTabStop = true;

	UpdateFormControlHiddenTextBoxPosition();

	if (m_pHostedControl->FormControlTextBox->FocusState == Windows::UI::Xaml::FocusState::Unfocused)
		m_pHostedControl->FormControlTextBox->Focus(Windows::UI::Xaml::FocusState::Programmatic);
}

void UIExtensionsManager::LostFocusFormControlHiddenTextBox()
{
	if (m_pHostedControl->FormControlTextBox->IsHitTestVisible)
		m_pHostedControl->FormControlTextBox->IsHitTestVisible = false;
	if (m_pHostedControl->FormControlTextBox->IsTabStop)
		m_pHostedControl->FormControlTextBox->IsTabStop = false;

	if (m_pHostedControl->FormControlTextBox->FocusState != Windows::UI::Xaml::FocusState::Unfocused)
		m_pHostedControl->FormControlTextBox->ResetInputText();

	m_pHostedControl->Focus(Windows::UI::Xaml::FocusState::Programmatic);
}

void UIExtensionsManager::UpdateFormControlHiddenTextBoxPosition()
{
	auto pSelectedAnnot = AnnotMgr->GetSelectedAnnot();
	if (pSelectedAnnot && pSelectedAnnot->AnnotType == AnnotationType::Widget)
	{
		auto deviceRect = m_control->ConvertPageRectToDeviceRect(pSelectedAnnot->PageIndex, pSelectedAnnot->MinRect);
		auto _ControlRc = deviceRect;
		Windows::UI::Xaml::Controls::Canvas::SetLeft(m_pHostedControl->FormControlTextBox, _ControlRc.X);
		Windows::UI::Xaml::Controls::Canvas::SetTop(m_pHostedControl->FormControlTextBox, _ControlRc.Y);
		m_pHostedControl->FormControlTextBox->Width = _ControlRc.Width;
		m_pHostedControl->FormControlTextBox->Height = _ControlRc.Height;
		m_pHostedControl->FormControlTextBox->MaxLength;
	}
}

void UIExtensionsManager::OnTappedTick(Platform::Object ^sender, Platform::Object ^args)
{
	m_TappedTimer->Stop();

	if (m_AnnotCreateType != AnnotCreateType::Caret && m_AnnotCreateType != AnnotCreateType::ReplaceText)
	{
		if (m_textManager)
		{
			m_textManager->ClearTextSelect();
		}
	}
	if (m_control)
		m_control->Update();


	if (!m_bIsAppBarInSearchMode)
	{
		
	}
}

void UIExtensionsManager::OnTick(Platform::Object ^sender, Platform::Object ^args)
{
	if (m_BottomBarVisibleTimer)
	{
		m_BottomBarVisibleTimer->Stop();
	}
	SetBottomBarIsVisible(false);
}

void UIExtensionsManager::OnPointerWheelScaleOperationTick(Platform::Object ^sender, Platform::Object ^args)
{
	if (m_PointerWheelScaleTimer)
	{
		m_PointerWheelScaleTimer->Stop();
	}

	
}

void UIExtensionsManager::OnAutoSaveFileTick(Platform::Object ^sender, Platform::Object ^args)
{

}

void UIExtensionsManager::OnCopyFileTick(Platform::Object ^sender, Platform::Object ^args)
{
	auto m_pPDFFile = viewControl->GetStorageFile();
	if (m_pPDFFile == nullptr)
		return;

	m_CopyFileTimer->Stop();

	CopyCurrentFileAsync()
		.then([=](concurrency::task<void> t)
	{
		try
		{
			t.get();
		}
		catch (...)
		{

		}
	});
}


void UIExtensionsManager::OnAnnotPropertyAppBarLoaded(Platform::Object ^sender, Windows::UI::Xaml::RoutedEventArgs ^e)
{
	IBaseAnnot^ annot = getAnnotMgr()->GetSelectedAnnot();
	if (annot)
	{
		double width = control->ActualWidth;
		double height = control->ActualHeight;
		auto _WindowsRect = Windows::UI::Xaml::Window::Current->Bounds;

		Windows::Foundation::Rect pagerect = annot->MinRect;
		if (annot->AnnotType == AnnotationType::FreeText)
		{
			FreeTextAnnot^ freetextAnnot = dynamic_cast<FreeTextAnnot^>(annot);
			pagerect = freetextAnnot->Page_HightLightRect;
		}

		auto rect = viewControl->ConvertPageRectToDeviceRect(annot->PageIndex, pagerect);

		Windows::Foundation::Rect pageViewRect = viewControl->GetPageViewRect(annot->PageIndex);

		double verticalOffset = rect.Y - height - 20;
		double horOffset = rect.X + rect.Width / 2 - width / 2;
		if (rect.X < pageViewRect.X)
		{
			horOffset = pageViewRect.X + pageViewRect.Width / 2 - width / 2;
		}


		if (verticalOffset < 0)
			verticalOffset = 0;
		if (horOffset < 0)
			horOffset = 0;
		else if (horOffset + width > _WindowsRect.Width)
			horOffset = _WindowsRect.Width - width;

		propertyPopup->VerticalOffset = verticalOffset;
		propertyPopup->HorizontalOffset = horOffset;
	}
}

void UIExtensionsManager::ResetPropertyPopupLocation(bool isShowMainBorad)
{
	if (isShowMainBorad)
	{
		propertyPopup->VerticalOffset = propertyPopup->VerticalOffset - 110;
	}
	else
	{
		propertyPopup->VerticalOffset = propertyPopup->VerticalOffset + 110;
	}
}

bool UIExtensionsManager::GetCanModifyForm()
{
	auto pDoc = viewControl->GetDoc();

	return FSPDFDocUtil::GetIsDocumentCanFillInForm(pDoc) || pDoc->GetPasswordType() == foxit::pdf::PasswordType::e_PwdNoPassword;
}

FormFillTimerWrapper^ UIExtensionsManager::GetTimerById(UINT_PTR nTimerID)
{
	FormFillTimerWrapper^ _Timer = nullptr;

	auto _findIter = std::find_if(begin(m_FormFillTimers), end(m_FormFillTimers), [nTimerID](FormFillTimerWrapper^ item)
	{
		return item && item->TimerId == nTimerID;
	});

	if (_findIter != end(m_FormFillTimers))
		_Timer = *_findIter;

	return _Timer;
}

void UIExtensionsManager::CreateTimer(FS_LPVOID clientData, int uElapse, foxit::pdf::interform::ITimerCallback^ lpTimerFunc)
{
	UINT_PTR nTimerID = (UINT_PTR)clientData;
	auto _Timer = GetTimerById(nTimerID);
	if (_Timer == nullptr)
	{
		_Timer = ref new FormFillTimerWrapper(clientData, uElapse, lpTimerFunc);
		m_FormFillTimers->Append(_Timer);
	}
	else
	{
		_Timer->SetTimerElapse(uElapse);
	}

	_Timer->Start();
}

void UIExtensionsManager::DestroyTime(UINT_PTR nTimerID)
{
	auto _Timer = GetTimerById(nTimerID);
	if (_Timer)
	{
		_Timer->Stop();
	}
}

void UIExtensionsManager::OnPageLoadedEvent(foxit::rdk::PDFViewCtrl^ sender, PageLoadedEventArgs^ e)
{
	if (m_AnnotMgr)
	{
		m_AnnotMgr->OnInitAnnot(e->PageIndex);
	}
}

void foxit::rdk::uiextensions::UIExtensionsManager::OnPageUnloadedEvent(foxit::rdk::PDFViewCtrl ^sender, foxit::rdk::PageUnloadedEventArgs ^e)
{
	if (m_AnnotMgr)
	{
		m_AnnotMgr->ClearAnnots(e->PageIndex);
	}
}

void foxit::rdk::uiextensions::UIExtensionsManager::OnPageVisibleEvent(foxit::rdk::PDFViewCtrl ^sender, foxit::rdk::PageVisibleEventArgs ^e)
{
	OutputDebugString((L"OnPageVisibleEvent..." + e->PageIndex + L"....\n")->Data());
}

void foxit::rdk::uiextensions::UIExtensionsManager::OnPageInvisibleEvent(foxit::rdk::PDFViewCtrl ^sender, foxit::rdk::PageInvisibleEventArgs ^e)
{
	OutputDebugString((L"OnPageInvisibleEvent..." + e->PageIndex + L"....\n")->Data());
}

void foxit::rdk::uiextensions::UIExtensionsManager::OnPageChangedEvent(foxit::rdk::PDFViewCtrl ^sender, foxit::rdk::PageChangedEventArgs ^e)
{
	OutputDebugString(L"OnPageChangedEvent......\n");
	InitSliderPageNumber(viewControl->GetCurrentPage());
}

void foxit::rdk::uiextensions::UIExtensionsManager::OnPageLayoutModeChangedEvent(foxit::rdk::PDFViewCtrl ^sender, foxit::rdk::PageLayoutModeChangedEventArgs ^e)
{
	OutputDebugString(L"OnPageLayoutModeChangedEvent......\n");
	InitPageNumberSliderByPageViewMode();
}

Windows::Foundation::Point foxit::rdk::uiextensions::UIExtensionsManager::SetNoteCreateOrSave()
{
	Point movePT(0, 0);
	if (m_PoppupNoteWindow && m_PoppupNoteWindow->IsOpen)
	{
		AnnotNoteUserControl^ noteControl = dynamic_cast<AnnotNoteUserControl^>(m_PoppupNoteWindow->Child);
		if (noteControl)
		{
			auto _CompositionScaleX = m_control->CompositionScaleX;
			auto _CompositionScaleY = m_control->CompositionScaleY;
			if (m_AnnotMgr && m_AnnotMgr->GetSelectedAnnot())
			{
				IBaseAnnot^ annot = m_AnnotMgr->GetSelectedAnnot();
				
				auto _devicePt = m_control->ConvertPagePtToDevicePt(annot->PageIndex, Windows::Foundation::Point(annot->MinRect.X, annot->MinRect.Y));

				auto _OffsetX = _devicePt.X;
				auto _OffsetY = _devicePt.Y;
				movePT = Point(_OffsetX*_CompositionScaleX, _OffsetY*_CompositionScaleY);

				noteControl->SaveAnnotNote();
				CloseAddNoteWindows();
				m_AnnotMgr->SetPdfSelectAnnotNull();
				SetAnnotCanvasVisible(false);
				ExitAnnotPropertyMode();
			}
			else
			{
				movePT = noteControl->CreateAnnotNote();
				auto _devicePt = m_control->ConvertPagePtToDevicePt(GetCurrentPageIndex(), Windows::Foundation::Point(movePT.X, movePT.Y));
				auto _OffsetX = _devicePt.X;
				auto _OffsetY = _devicePt.Y;
				movePT = Point(_OffsetX*_CompositionScaleX, _OffsetY*_CompositionScaleY);
			}
		}
	}
	return movePT;
}

void UIExtensionsManager::OnDrawAnnotEvent(foxit::rdk::PDFViewCtrl^ sender, RenderPageEventArgs^ e)
{

}

void UIExtensionsManager::StartZoomInOutTimer()
{
	if(m_BottomBarVisibleTimer)
		m_BottomBarVisibleTimer->Start();
}

void UIExtensionsManager::StopZoomInOutTimer()
{
	if (m_BottomBarVisibleTimer)
		m_BottomBarVisibleTimer->Stop();
}

static 	void ThrowIfFailed(HRESULT hr)
{
	if (FAILED(hr))
	{
		
		throw Platform::Exception::CreateException(hr);
	}
}

void UIExtensionsManager::LoadNoteLogo(Platform::String^ logoUrl)
{
	notIconCollections = ref new Platform::Collections::Vector<NoteIcoObj^>();
	Windows::Foundation::Collections::IVector<Platform::String^>^ noteUrlCollections = ref new Platform::Collections::Vector<Platform::String^>();
	noteUrlCollections->Append("Comment");
	noteUrlCollections->Append("Circle");
	noteUrlCollections->Append("Cross");
	noteUrlCollections->Append("Help");
	noteUrlCollections->Append("Insert");
	noteUrlCollections->Append("Checkmark");
	noteUrlCollections->Append("Key");
	noteUrlCollections->Append("Paragraph");
	noteUrlCollections->Append("NewParagraph");
	noteUrlCollections->Append("RightArrow");
	noteUrlCollections->Append("RightPointer");
	noteUrlCollections->Append("Star");
	noteUrlCollections->Append("Note");
	noteUrlCollections->Append("UpArrow");
	noteUrlCollections->Append("UpleftArrow");

	for (unsigned int i = 0; i < noteUrlCollections->Size; i++)
	{
		NoteIcoObj^ noteIcon = ref new NoteIcoObj();

		noteIcon->IconName = noteUrlCollections->GetAt(i);
		Uri^ url = ref new Uri("ms-appx:///foxit.rdk.uiextensions/Assets/Annot/" + noteIcon->IconName + ".png");
		if (noteUrlCollections->GetAt(i) == "Note")
		{
			url = ref new Uri("ms-appx:///foxit.rdk.uiextensions/Assets/Annot/" + "note1.png");
		}
		concurrency::create_task(Windows::Storage::StorageFile::GetFileFromApplicationUriAsync(url))
			.then([=](Windows::Storage::StorageFile^ file)
		{
			task<IRandomAccessStream^>(file->OpenAsync(Windows::Storage::FileAccessMode::Read)).then([=](IRandomAccessStream^ readStream)
			{
				Microsoft::WRL::ComPtr<ID2D1DeviceContext> d2dDeviceContext;
				Microsoft::WRL::ComPtr<IWICImagingFactory2> wicFactory;

				ThrowIfFailed(
					CoCreateInstance(
						CLSID_WICImagingFactory2,
						nullptr,
						CLSCTX_INPROC_SERVER,
						IID_PPV_ARGS(&wicFactory)
					)
				);

				Microsoft::WRL::ComPtr<IWICBitmapDecoder> decoder;
				Microsoft::WRL::ComPtr<IWICBitmapFrameDecode> frame;
				Microsoft::WRL::ComPtr<IWICFormatConverter> wicBitmap;
				Microsoft::WRL::ComPtr<ID2D1Bitmap> bitmap;

				Microsoft::WRL::ComPtr<IStream> inputStream;
				CreateStreamOverRandomAccessStream(
					reinterpret_cast<IUnknown*>(readStream),
					IID_PPV_ARGS(&inputStream)
				);

				ThrowIfFailed(
					wicFactory->CreateDecoderFromStream(
						inputStream.Get(),
						nullptr,
						WICDecodeMetadataCacheOnDemand,
						&decoder
					)
				);

				ThrowIfFailed(
					decoder->GetFrame(0, &frame)
				);

				ThrowIfFailed(
					wicFactory->CreateFormatConverter(&wicBitmap)
				);

				ThrowIfFailed(
					wicBitmap->Initialize(
						frame.Get(),
						GUID_WICPixelFormat32bppPBGRA,
						WICBitmapDitherTypeNone,
						nullptr,
						0,
						WICBitmapPaletteTypeCustom
					)
				);
				noteIcon->m_bitmap = wicBitmap;
				notIconCollections->Append(noteIcon);
			});
		});
	}
}

Microsoft::WRL::ComPtr<IWICFormatConverter> UIExtensionsManager::LoadPicSource(Platform::String^ iconName)
{
	if (notIconCollections && notIconCollections->Size > 0)
	{
		for (unsigned int i = 0; i < notIconCollections->Size; i++)
		{
			if (notIconCollections->GetAt(i)->IconName == iconName)
			{
				return notIconCollections->GetAt(i)->m_bitmap;
			}
		}
	}
	return nullptr;
}

void UIExtensionsManager::ExitPageView()
{
	if (viewControl->GetDoc())
		SaveDocumentData();
	SetTextMenuPopupHide();

	auto pFSDK_Document = viewControl->GetDoc();
	if (pFSDK_Document)
	{
		viewControl->CloseDoc();
	}
}

void UIExtensionsManager::ExitFileView()
{
	if (m_SavingFile)
		return;

	Windows::UI::Xaml::Controls::Frame^ _frame = dynamic_cast<Windows::UI::Xaml::Controls::Frame^>(Windows::UI::Xaml::Window::Current->Content);
	if (_frame == nullptr)
		return;

	m_control->CancelRenderTask();

	for (unsigned int i = 0; i < m_FormFillTimers->Size; i++)
	{
		FormFillTimerWrapper^ _Timer = m_FormFillTimers->GetAt(i);
		if (_Timer)
			_Timer->Stop();
	}
	if (TextManager)
	{
		if (TextManager->GetIsSingleSearching())
		{
			TextManager->StopSearchString();
			return;
		}
	}
	HidePopupWindow();

	auto pFSDK_Document = viewControl->GetDoc();
	if (pFSDK_Document && viewControl->IsModified())
	{
		m_SavingFile = true;
		m_isExitPageView = true;
		

		concurrency::create_task(viewControl->SaveDoc())
			.then([=](concurrency::task<void> t)
		{
			try
			{
				t.get();
				ExitPageView();
				m_SavingFile = false;

			}
			catch (...)
			{
				m_SavingFile = false;
				throw;
			}
		});

	}
	else
	{
		m_isExitPageView = true;
		ExitPageView();
		m_SavingFile = false;
	}
	if (m_MainAppBar)
	{
		m_MainAppBar->HideAppBar();
		m_MainAppBar->StopHideTimeTick();
		m_MainAppBar->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
	}

}

void UIExtensionsManager::SaveDocumentData()
{
	if (AnnotMgr) {
		AnnotMgr->GetAnnotSetting()->SavePropertySetting();
	}

	if (m_bIsSaveDocReadStateSetting)
		return;

	ApplicationDataContainer^ localSettings = ApplicationData::Current->LocalSettings;
	auto values = localSettings->Values;
	ApplicationDataCompositeValue^ composite;

	AccessListEntryView^ _entries = StorageApplicationPermissions::MostRecentlyUsedList->Entries;

	auto pDocument = viewControl->GetDoc();
	auto pPDFFile = viewControl->GetStorageFile();
	Platform::String^ path = pPDFFile->Path;

	if (m_OpenStateCode != foxit::common::ErrorCode::e_ErrSuccess)
	{
		m_bIsSaveDocReadStateSetting = true;
		return;
	}

	Platform::String^ token = nullptr;
	auto _iter = _entries->First();
	while (_iter->HasCurrent)
	{
		auto _value = _iter->Current;
		if (_value.Metadata == path)
		{
			token = _value.Token;
			if (values->HasKey(token))
			{
				composite = values->HasKey(token) ? dynamic_cast<ApplicationDataCompositeValue^>(values->Lookup(token)) : ref new ApplicationDataCompositeValue();
				values->Remove(token);
			}

			token = nullptr;
			break;
		}
		_iter->MoveNext();
	}

	if (!token)
	{
		token = StorageApplicationPermissions::MostRecentlyUsedList->Add(pPDFFile, path);
	}

	if (composite == nullptr)
		composite = values->HasKey(token) ? dynamic_cast<ApplicationDataCompositeValue^>(values->Lookup(token)) : ref new ApplicationDataCompositeValue();

	viewControl->SaveState(composite);

	values->Insert(token, composite);
	m_bIsSaveDocReadStateSetting = true;
}

PDFPagePoint^ UIExtensionsManager::GetHoldingPoint()
{
	return m_holdingPoint;
}

void UIExtensionsManager::SaveDoc()
{
	concurrency::create_task(viewControl->SaveDoc())
		.then([=](concurrency::task<void> t)
	{
		try
		{
			m_SavingFile = false;
			viewControl->SetModified(false);

			UndoFactory::GetUndoInstance()->Reset(this);
			UpdateAnnotToolUnReDoControl();
			if (m_MainAppBar)
				m_MainAppBar->SetSaveBtnUnEnabled();

			t.get();
		}
		catch (...)
		{

		}
	});
}

void UIExtensionsManager::SaveAsFile()
{
	SaveAsFileAsync();
}

concurrency::task<void> UIExtensionsManager::SaveFileAsync()
{
	auto m_pPDFFile = viewControl->GetStorageFile();
	auto _Task = concurrency::create_task([]() {});
	if (m_pPDFFile == nullptr)
		return _Task;

	m_SavingFile = true;
	

	if (!GetIsDesktopPlatform() )
	{
		std::shared_ptr<Windows::Storage::StorageFile^> _spTempStorageFile = std::make_shared<Windows::Storage::StorageFile^>(nullptr);

		_Task = create_task(KnownFolders::PicturesLibrary->CreateFolderAsync(L"Foxit", CreationCollisionOption::OpenIfExists))
			.then([this, m_pPDFFile](StorageFolder^ folder)
		{
			return folder->CreateFileAsync(m_pPDFFile->Name, Windows::Storage::CreationCollisionOption::GenerateUniqueName);
		})
			.then([=](Windows::Storage::StorageFile^ file)
		{
			*_spTempStorageFile = file;
			return m_pPDFFile->CopyAndReplaceAsync(*_spTempStorageFile);
		})
			.then([=]()
		{
			
			return foxit::rdk::FXDocUtil::SaveAs(viewControl->GetDoc(), *_spTempStorageFile, (int)foxit::pdf::SaveFlags::e_SaveFlagIncremental);
		})
			.then([=](concurrency::task<bool> t)
		{
			try
			{
				m_SavingFile = false;

				t.get();
				
				viewControl->SetModified(false);
				
			}
			catch (...)
			{
				
				throw;
			}
		});
	}
	else
	{
		_Task = concurrency::create_task([=]()
		{
			return foxit::rdk::FXDocUtil::SaveAs(viewControl->GetDoc(), m_pPDFFile, (int)foxit::pdf::SaveFlags::e_SaveFlagIncremental);
		})
			.then([=](concurrency::task<bool> t)
		{
			try
			{
				m_SavingFile = false;

				t.get();
				
			}
			catch (...)
			{
				
				throw;
			}
		});
	}

	return _Task;
}

concurrency::task<void> UIExtensionsManager::SaveAsFileAsync()
{
	auto m_pPDFFile = viewControl->GetStorageFile();
	auto _Task = concurrency::create_task([]() {});
	if (m_pPDFFile == nullptr)
		return _Task;

	m_isSaveAsFile = true;
	

	Windows::Storage::Pickers::FileSavePicker^ savePicker = ref new Windows::Storage::Pickers::FileSavePicker();
	savePicker->SuggestedStartLocation = Windows::Storage::Pickers::PickerLocationId::DocumentsLibrary;

	auto plainTextExtensions = ref new Platform::Collections::Vector<String^>();
	plainTextExtensions->Append(".pdf");
	savePicker->FileTypeChoices->Insert("PDF Files", plainTextExtensions);

	
	String^ fileName = m_pPDFFile->Name;
	
	savePicker->SuggestedFileName = fileName;

	std::shared_ptr<Windows::Storage::StorageFile^> _spStorageFile = std::make_shared<Windows::Storage::StorageFile^>(nullptr);

	return concurrency::create_task(savePicker->PickSaveFileAsync())
		.then([=](StorageFile^ pickerFile)
	{
		if (pickerFile == nullptr)
			*_spStorageFile = nullptr;
		else {
			if (m_pPDFFile->Path == pickerFile->Path)
			{
				auto messageDialog = ref new Windows::UI::Popups::MessageDialog("Please specifiy a different file name", "error");
				messageDialog->Commands->Append(ref new Windows::UI::Popups::UICommand("OK", nullptr, PropertyValue::CreateInt32(0)));
				messageDialog->DefaultCommandIndex = 0;
				messageDialog->ShowAsync();
				*_spStorageFile = nullptr;
			}
			else
				*_spStorageFile = pickerFile;
		}
		
		return foxit::rdk::FXDocUtil::SaveAs(viewControl->GetDoc(), *_spStorageFile, (int)foxit::pdf::SaveFlags::e_SaveFlagIncremental);
	})
		.then([=](concurrency::task<bool> t)
	{
		try
		{
			m_isSaveAsFile = t.get();
		
		}
		catch (Platform::Exception^ e)
		{
			m_isSaveAsFile = false;
			OutputDebugString(e->Message->Data());
		}
	});
}

concurrency::task<void> UIExtensionsManager::SaveAsFileWithSelectDialogAsync()
{
	Platform::String^ saveTitle = GetResourceString(L"SaveAsFileTipsDialogTitle");
	Platform::String^ saveContent = GetResourceString(L"SaveAsFileTipsDialogContent");
	Platform::String^ saveYes = GetResourceString(L"SaveAsFileYesText");
	Platform::String^ saveNo = GetResourceString(L"SaveAsFileNoText");

	auto messageDialog = ref new Windows::UI::Popups::MessageDialog(saveContent, saveTitle);
	messageDialog->Commands->Append(ref new Windows::UI::Popups::UICommand(saveYes, nullptr, PropertyValue::CreateInt32(0)));
	messageDialog->Commands->Append(ref new Windows::UI::Popups::UICommand(saveNo, nullptr, PropertyValue::CreateInt32(1)));
	messageDialog->DefaultCommandIndex = 0;

	auto _is_canceled = m_TipsDlgCts.get_token().is_canceled();
	if (m_TipsDlgCts.get_token().is_canceled())
	{
		cancellation_token_source _newCts;
		m_TipsDlgCts = _newCts;
	}

	return concurrency::create_task(messageDialog->ShowAsync(), m_TipsDlgCts.get_token())
		.then([=](Windows::UI::Popups::IUICommand^ command)
	{
		auto _Task = concurrency::create_task([]() {});
		if (command && command->Id != nullptr)
		{
			int const id = safe_cast<IBox<int>^>(command->Id)->Value;
			if (id == 0)
			{
				_Task = SaveAsFileAsync();
			}
		}

		return _Task;
	});
}

concurrency::task<void> UIExtensionsManager::CopyFileStreamData(Windows::Storage::StorageFile^ dstFile, Windows::Storage::StorageFile^ srcFile)
{
	OutputDebugString(L"CopyStreamData...\n");
	std::shared_ptr<Windows::Storage::Streams::DataWriter^> _spDataWriter = std::make_shared<Windows::Storage::Streams::DataWriter^>(nullptr);
	std::shared_ptr<Windows::Storage::Streams::DataReader^> _spDataReader = std::make_shared<Windows::Storage::Streams::DataReader^>(nullptr);
	std::shared_ptr<Windows::Storage::StorageFile^> _spSrcFile = std::make_shared<Windows::Storage::StorageFile^>(srcFile);

	return concurrency::create_task(dstFile->OpenAsync(Windows::Storage::FileAccessMode::ReadWrite))
		.then([=](Windows::Storage::Streams::IRandomAccessStream^ stream)
	{
		*_spDataWriter = ref new Windows::Storage::Streams::DataWriter(stream);

		return srcFile->OpenAsync(Windows::Storage::FileAccessMode::Read);
	})
		.then([=](Windows::Storage::Streams::IRandomAccessStream^ stream)
	{
		*_spDataReader = ref new Windows::Storage::Streams::DataReader(stream);

		return srcFile->GetBasicPropertiesAsync();
	})
		.then([=](Windows::Storage::FileProperties::BasicProperties^ basicProperties)
	{
		auto _size = basicProperties->Size;
		
		return (*_spDataReader)->LoadAsync((unsigned int)_size);
	}, concurrency::task_continuation_context::use_arbitrary())
		.then([=](unsigned int size)
	{
		auto _buffer = (*_spDataReader)->ReadBuffer(size);
		auto _buffersize = _buffer->Length;
		(*_spDataWriter)->WriteBuffer(_buffer);

		auto inputStream = (*_spDataReader)->DetachStream();
		delete inputStream;

		return (*_spDataWriter)->StoreAsync();
	})
		.then([=](unsigned int bytesStored)
	{
		return (*_spDataWriter)->FlushAsync();
	})
		.then([=](bool flushSucceeded)
	{
		
		auto outputStream = (*_spDataWriter)->DetachStream();
		delete outputStream;
	});
}

concurrency::task<void> UIExtensionsManager::DeleteTemporaryFile()
{
	return concurrency::create_task(Windows::Storage::ApplicationData::Current->LocalFolder->GetFileAsync(L"temper.pdf"))
		.then([=](Windows::Storage::StorageFile^ storageFile)
	{
		return storageFile->DeleteAsync();
	})
		.then([=](concurrency::task<void> t)
	{
		try
		{
			t.get();
		}
		catch (...)
		{
			OutputDebugString(L"DeleteTemporaryFile Failed...\n");
		}
	});
}

concurrency::task<void> UIExtensionsManager::CopyCurrentFileAsync()
{
	if (m_CopyFileCts.get_token().is_canceled())
	{
		cancellation_token_source _newCts;
		m_CopyFileCts = _newCts;
	}

	std::shared_ptr<Windows::Storage::StorageFile^> _spStorageFile = std::make_shared<Windows::Storage::StorageFile^>(nullptr);

	concurrency::task<Windows::Storage::StorageFile^> _Task;
	if (m_SwapTemperFile)
	{
		_Task = concurrency::create_task([=]() { return m_SwapTemperFile; }, m_CopyFileCts.get_token());
	}
	else
	{
		_Task = DeleteTemporaryFile()
			.then([=]()
		{
			return Windows::Storage::ApplicationData::Current->LocalFolder->CreateFileAsync("temper.pdf", CreationCollisionOption::ReplaceExisting);
		}, m_CopyFileCts.get_token());
	}

	return _Task.then([=](Windows::Storage::StorageFile^ storageFile)
	{
		*_spStorageFile = storageFile;
		auto m_pPDFFile = viewControl->GetStorageFile();

		return CopyFileStreamData(*_spStorageFile, m_pPDFFile);
	})
		.then([=](concurrency::task<void> t)
	{
		try
		{
			t.get();
			m_SwapTemperFile = *_spStorageFile;
			OutputDebugString(L"CopyCurFile Success...\n");
		}
		catch (Platform::Exception^ e)
		{
			auto _ExceptionMsg = e->HResult.ToString() + L", " + e->Message;
			OutputDebugString(L"CopyCurFile Failed...");
			OutputDebugString(_ExceptionMsg->Data());
			OutputDebugString(L"\n");
		}
		catch (...)
		{
			OutputDebugString(L"CopyCurFile Failed...\n");
		}
	});
}

AppBarType UIExtensionsManager::GetCurAppBarType()
{
	return m_CurAppBarType;
}

void UIExtensionsManager::LoadViewByViewMode(foxit::rdk::LayoutMode viewmode)
{
	if (viewControl)
	{
		auto _ViewState = viewControl->GetPageLayoutMode();
		if (_ViewState == LayoutMode::ReflowMode)
		{
			m_bIsSaveDocReadStateSetting = false;
			SaveDocumentData();
			m_bIsSaveDocReadStateSetting = false;
		}

		if (viewmode == LayoutMode::ReflowMode && m_docViewMode != LayoutMode::ReflowMode)
		{
			m_PrevViewMode = m_docViewMode;
		}

		m_docViewMode = viewmode;
		int curPageIndex = viewControl->GetCurrentPage();

		viewControl->SetPageLayoutMode(m_docViewMode);
	}
}

bool UIExtensionsManager::LoadPrevViewMode()
{
	if (m_PrevViewMode == LayoutMode::None)
		return false;

	LoadViewByViewMode(m_PrevViewMode);
	return true;
}

void UIExtensionsManager::ShowSearchBarInPhone()
{
	m_CurAppBarType = AppBarType::SearchBar;

	FindTextControl^ findControl = ref new FindTextControl();
	auto mainAppBar = m_pHostedControl->AppBarLayout;
	mainAppBar->Visibility = Windows::UI::Xaml::Visibility::Visible;
	mainAppBar->Children->Clear();
	mainAppBar->Children->Append(findControl);
	findControl->FocusSearchControl();
}

void UIExtensionsManager::InitPageNumberSliderByPageViewMode()
{
	auto pDoc = viewControl->GetDoc();
	LayoutMode layoutViewMode = viewControl->GetPageLayoutMode();
	m_docViewMode = layoutViewMode;

	if (pDoc && pDoc->GetPageCount() > 5)
	{
		if (layoutViewMode == LayoutMode::ContinueMode || m_docViewMode == LayoutMode::ReflowMode)
		{
			m_pHostedControl->slider->Visibility = Windows::UI::Xaml::Visibility::Visible;
			m_pHostedControl->sliderHor->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
			m_pHostedControl->slider->RelatePageView(this);
			m_pHostedControl->slider->InitPageNumText(GetCurrentPageIndex());
		}
		else
		{
			m_pHostedControl->slider->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
			m_pHostedControl->sliderHor->Visibility = Windows::UI::Xaml::Visibility::Visible;
			m_pHostedControl->sliderHor->RelatePageView(this);
			m_pHostedControl->sliderHor->InitPageNumText(GetCurrentPageIndex());
		}
	}
	else
	{
		m_pHostedControl->slider->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
		m_pHostedControl->sliderHor->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
	}
}

void UIExtensionsManager::AnnotHighlightFlicker(Windows::Foundation::Rect rc)
{
	auto flickerStoryboard = safe_cast<Windows::UI::Xaml::Media::Animation::Storyboard^>(m_pHostedControl->Resources->Lookup("AnnotHighlightFlickerStoryboard"));
	if (flickerStoryboard == nullptr)
		return;
	
	auto pAnnotHighlightFlickerTarget = m_pHostedControl->AnnotHighlightFlickerRectangle;
	if (pAnnotHighlightFlickerTarget->Visibility == Windows::UI::Xaml::Visibility::Visible)
		flickerStoryboard->Stop();

	pAnnotHighlightFlickerTarget->Visibility = Windows::UI::Xaml::Visibility::Visible;
	pAnnotHighlightFlickerTarget->SetValue(Windows::UI::Xaml::Controls::Canvas::LeftProperty, rc.X);
	pAnnotHighlightFlickerTarget->SetValue(Windows::UI::Xaml::Controls::Canvas::TopProperty, rc.Y);
	pAnnotHighlightFlickerTarget->Width = rc.Width;
	pAnnotHighlightFlickerTarget->Height = rc.Height;

	flickerStoryboard->Begin();
}
