﻿/**
 * Copyright (C) 2003-2018, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit Mobile PDF SDK to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */


#pragma once

namespace foxit
{
	namespace rdk
	{
		namespace uiextensions
		{
#define POPUPWINDOWWIDTH 405
#define POPUPWINDOWHEIGHT 234

			public enum class AnnotContextMenuType
			{
				Add,
				Delete,
				Note,
				Edit,
				NotePro,
				NewSign,
				ApplySign
			};

			public enum class AnnotCommandType
			{
				Add,
				Open
			};

			[Windows::Foundation::Metadata::WebHostHidden]
			public ref class AnnotCommandToken sealed
			{
			public:
				property AnnotCommandType Type;
				property int Pageindex;
				property Platform::Object^ Param;
				property Platform::Object^ StrickoutParam;
			};

			/// Timer class packaging for form
			public ref class FormFillTimerWrapper sealed
			{
			public:
				virtual ~FormFillTimerWrapper();

				/// The sign Id of timer
				property UINT_PTR TimerId
				{
					UINT_PTR get();
				}

				/// The interval between milliseconds has been milliseconds
				property int Elapse
				{
					int get();
				}

				void SetTimerElapse(int uElapse);

				void Start();
				void Stop();

			internal:
				FormFillTimerWrapper(FS_LPVOID clientData, int uElapse, foxit::pdf::interform::ITimerCallback^ lpTimerFunc);

			private:
				FS_LPVOID m_FormFillClientData;
				int m_uElapse;
				foxit::pdf::interform::ITimerCallback^ m_lpTimerFunc;
				Windows::UI::Xaml::DispatcherTimer^ m_FormFillTimer;

			private:
				void OnTick(Platform::Object ^sender, Platform::Object ^args);
			};

			public ref class NoteIcoObj sealed
			{
			internal:
				property Platform::String^ IconName;
				Microsoft::WRL::ComPtr<IWICFormatConverter>  m_bitmap;
			};

			ref class FoxitViewControl;
			ref class PDFTextManager;
			ref class MainPageAppBar;
			ref class PDFFormAbstration;
			ref class PDFFormFillerManagement;
			ref class FormOperationBarControl;
			ref class AnnotNoteUserControl;
			ref class FoxitUIHostedContorl;
			ref class ContextMenuUserControl;
			ref class AnnotPropertyAppBar;
			ref class FindTextControl;
			ref class DocUndo;
			[Windows::Foundation::Metadata::WebHostHidden]
			public ref class UIExtensionsManager sealed : public IUIExtensionsEvents
			{
				friend ref class MainPageAppBar;
			public:
				property foxit::rdk::PDFViewCtrl^ viewControl
				{
					foxit::rdk::PDFViewCtrl^ get()
					{
						return m_control;
					}
				}

				property bool IsRightMenuAdd
				{
					virtual bool get() {
						return m_IsRightMenuAdd
							;
					}
					void set(bool value)
					{
						m_IsRightMenuAdd = value;
					}
				}

				property bool IsShowAnnot
				{
					virtual bool get()
					{
						return ((viewControl->GetRenderFlag() & foxit::common::ContentFlag::e_RenderAnnot) == foxit::common::ContentFlag::e_RenderAnnot);
					}
					void set(bool value)
					{
						if (value)
							viewControl->SetRenderFlag(foxit::common::ContentFlag::e_RenderAnnot | foxit::common::ContentFlag::e_RenderPage);
						else
							viewControl->SetRenderFlag(foxit::common::ContentFlag::e_RenderPage);
					}
				}

				property Windows::UI::Xaml::Controls::Grid^ FlyoutPlacement
				{
					Windows::UI::Xaml::Controls::Grid^ get();
				}

				property PDFTextManager^ TextManager
				{
					PDFTextManager^ get()
					{
						return m_textManager;
					}
				}

				property IAnnotManager^ AnnotMgr
				{
					IAnnotManager^ get()
					{
						return m_AnnotMgr;
					}
				}

				/// AppBar display event
				event Windows::Foundation::EventHandler<Platform::Object^>^ AppBarShow;
				/// AppBar hide event
				event Windows::Foundation::EventHandler<Platform::Object^>^ AppBarHide;

			public:
				UIExtensionsManager(foxit::rdk::PDFViewCtrl^ ctrl);

				virtual void ManipulationStarted(Platform::Object^ sender, Windows::UI::Xaml::Input::ManipulationStartedRoutedEventArgs^ e);
				virtual void ManipulationDelta(Platform::Object^ sender, Windows::UI::Xaml::Input::ManipulationDeltaRoutedEventArgs^ e);
				virtual void ManipulationCompleted(Platform::Object^ sender, Windows::UI::Xaml::Input::ManipulationCompletedRoutedEventArgs^ e);
				virtual void PointerPressed(Platform::Object^ sender, Windows::UI::Xaml::Input::PointerRoutedEventArgs^ e);
				virtual void PointerMoved(Platform::Object^ sender, Windows::UI::Xaml::Input::PointerRoutedEventArgs^ e);
				virtual void PointerReleased(Platform::Object^ sender, Windows::UI::Xaml::Input::PointerRoutedEventArgs^ e);
				virtual void PointerEntered(Platform::Object^ sender, Windows::UI::Xaml::Input::PointerRoutedEventArgs^ e);
				virtual void PointerExited(Platform::Object^ sender, Windows::UI::Xaml::Input::PointerRoutedEventArgs^ e);
				virtual void PointerCaptureLost(Platform::Object^ sender, Windows::UI::Xaml::Input::PointerRoutedEventArgs^ e);
				virtual void PointerCanceled(Platform::Object^ sender, Windows::UI::Xaml::Input::PointerRoutedEventArgs^ e);
				virtual void PointerWheelChanged(Platform::Object^ sender, Windows::UI::Xaml::Input::PointerRoutedEventArgs^ e);
				virtual void Holding(Platform::Object^ sender, Windows::UI::Xaml::Input::HoldingRoutedEventArgs^ e);
				virtual void Tapped(Platform::Object^ sender, Windows::UI::Xaml::Input::TappedRoutedEventArgs^ e);
				virtual void CompositionScaleChanged(Windows::UI::Xaml::Controls::SwapChainPanel^ sender, Platform::Object^ args);
				virtual void SizeChanged(Platform::Object^ sender, Windows::UI::Xaml::SizeChangedEventArgs^ e);

				void ResetDefaultAppBarButton();
				void ShowOrHideAppBar();
				void ShowWidgetAnnotPopup();
				void HideWidgetAnnotPopup();
				void ShowFormOperationBarInPhone();
				void SetBottomBarIsVisible(bool isVisible);
				void ExitPageView();
				void ExitFileView();
				bool IsInThumbnailMode();
				void EnterThumbnailPage();
				void ExitThumbnailPage();

				void SetFlyoutPlacement(Windows::UI::Xaml::Controls::Grid^ pPlacementGrid);

			internal:
				void ShowProgressRing();
				void HideProgressRing();
				void OnLoaded(Platform::Object ^sender, Windows::UI::Xaml::RoutedEventArgs ^e);
				void OnUnloaded(Platform::Object ^sender, Windows::UI::Xaml::RoutedEventArgs ^e);
				void OnAcceleratorKeyActivated(Windows::UI::Core::CoreDispatcher ^sender, Windows::UI::Core::AcceleratorKeyEventArgs ^args);
				void OnKeyDown(Windows::UI::Core::CoreWindow ^sender, Windows::UI::Core::KeyEventArgs ^args);
				void OnKeyUp(Windows::UI::Core::CoreWindow ^sender, Windows::UI::Core::KeyEventArgs ^args);
				bool IsCreateDrawAnnot();
				void InitMainAppBar();
				MainPageAppBar^ GetMainAppBar();
				foxit::pdf::interform::Filler^ GetFormFiller();
				void ReleaseFormFiller();
				bool IsEnableFormModule();
				foxit::pdf::interform::Form^ GetSdkForm();
				bool IsHaveForm();
				void ResetForm();
				void BeginFormFiller();
				void EndFormFiller();
				void ImportFormDataFromFdfFile();
				void ExportFormDataToFdfFile();
				static Windows::Foundation::IAsyncOperation<foxit::fdf::FDFDoc^>^ OpenNewFdfDocumentAsync();
				static Windows::Foundation::IAsyncOperation<foxit::fdf::FDFDoc^>^ OpenFdfDocumentAsync(Windows::Storage::StorageFile^ pdfFile);
				PDFTextManager^ getTextMgr() { return m_textManager; }
				AnnotationManager^ getAnnotMgr() { return m_AnnotMgr; }
				void CreateTimer(FS_LPVOID clientData, int uElapse, foxit::pdf::interform::ITimerCallback^ lpTimerFunc);
				void DestroyTime(UINT_PTR nTimerID);
				Microsoft::WRL::ComPtr<IWICFormatConverter> LoadPicSource(Platform::String^ iconName);
				void LoadNoteLogo(Platform::String^ logoUrl);
				void SaveDocumentData();
				PDFPagePoint^ GetHoldingPoint();
				void SaveDoc();
				void SaveAsFile();
				AppBarType GetCurAppBarType();
				void ShowOpenPdfDocumentErrorDialog();
				void ShowOpenEncryptedDocumentErrorMsgDialog();
				bool GetIsCanScrollPageView();
				void ShowSearchBarInPhone();
				concurrency::cancellation_token_source m_TipsDlgCts;
				property SelectOperateType SelectOperate
				{
					SelectOperateType get();
				}
				property AnnotCreateType CreateTypeOfAnnot
				{
					AnnotCreateType get();
					void set(AnnotCreateType value);
				}
				int GetFormControlType(IBaseAnnot^ pAnnot);
				void SdkFocusFormControl(IBaseAnnot^ pAnnot);
				void SdkKillFocusFormControl();
				void KillFormfillerFocus();
				void FocusPrevFormControl();
				void FocusNextFormControl();
				IBaseAnnot^ GetCanSelectFormControl(Windows::Foundation::Collections::IVector<IBaseAnnot^>^ annotList, int nStartIndex, bool bReverse);
				void CollectAnnotOperation(Platform::String^ Type, Platform::String^ ActionType, Platform::String^ PageIndex, Platform::String^ Rect);
				void SetAnnotCanvasVisible(bool isVisible);
				void SetTextMenuPopupHide();
				void CloseAddNoteWindows();
				void ShowSelHandleButton(bool bIsShow);
				void ShowTouchSelectHandleButton(Windows::Foundation::Collections::IVector<IPageSelectText^>^ e);
				void ShowTextmarkContextmenu(AnnotContextMenuType type, Windows::Foundation::Point viewPoint);
				void OnMenuControlLoaded(Platform::Object ^sender, Windows::UI::Xaml::RoutedEventArgs ^e);
				void HideFindBar();
				int GetCurrentPageIndex();
				void ExitSearchMode();
				void UpdateView();
				void ResetAnnotToolAppBar();
				void SetManipulationStartFlagFalse();
				void SetUnRedoIsEnabled(bool isEnable);
				void CloseAllPropertyPopup();
				void InitPageNumberSliderByPageViewMode();
				void InitSliderPageNumber(int pageIndex);
				void SetPageNumberSliderVisible(bool isVisble);
				void SetPageNumberSlideHide();
				void ShowPageNumberCountPopup(bool isActiveTimer);
				void HidePageNumberCountPopup();
				void SetAppBarHideTimer(bool isStop);
				void ShowPageNumberJumpWindow();
				void HideJumpPagePopup();
				void JumpPageIndex(int pageIndex);
				void HideFloatWindow();
				void ExitAnnotPropertyMode();
				bool ExitNotePopupEdit();
				Windows::Foundation::Point GetAnnotPopupOffset();
				void SetPropertyPoupHide();
				void ReLocateImageSignature(Windows::Foundation::Rect rect);
				void LoadNoteStylePage();
				void UpdateFreeTextAnnotOverlayControl(int pageIndex, Windows::Foundation::Rect rect);
				void CreateGraphAnnotBySliderPage();
				void SetPropertyCurColor(Windows::UI::Color color);
				void HideNoteStylePopup();
				void ShowDirecControl(bool isVisible);
				void SetAnnotStampClosed();
				void AddAnnotByType(AnnotCreateType type);
				bool IsPopupShow();
				void CollectEraserOpration();
				void SizeChangedCreateAnnot();
				bool IsSelectAnnotPhonePropertyOpen();
				void ExitAnnotSelect();
				void SaveOrCreateNoteAnnot();
				void SetUnRedoPhoneAppBarVisible(bool isVisible);
				void ShowPhonAnnotPropertyIcon(bool isVisble);
				void ShowAddNoteWindows(AnnotNoteUserControl^ control, Windows::Foundation::Point curPoint);
				void CreateNewShapeToCanvas(Windows::UI::Xaml::Shapes::Shape^ shape);
				void UpdatePolyLinePoints(Windows::UI::Xaml::Shapes::Shape^ shape, Windows::UI::Xaml::Media::PointCollection^ pointVector);
				void CreatePolyLineAnnotToCanvas(int pageIndex, Windows::Foundation::Rect minRect, Windows::Foundation::Collections::IVector<Windows::Foundation::Collections::IVector<Windows::Foundation::Point>^>^ Page_VectPoint, double opacity, double borderWidth, Windows::UI::Color color);
				void CreateRectangelAnnotToCanvas(int pageIndex, Windows::Foundation::Rect rect, double opacity, double borderWidth, Windows::UI::Color color);
				void CreateEllipseAnnotToCanvas(int pageIndex, Windows::Foundation::Rect rect, double opacity, double borderWidth, Windows::UI::Color color);
				void CreateLineAnnotToCanvas(int pageIndex, Windows::Foundation::Rect rect, Windows::Foundation::Point startPoint, Windows::Foundation::Point endPoint, double opacity, double borderWidth, Windows::UI::Color color);
				void CreateFreeTextBoxToCanvas(Windows::UI::Xaml::Controls::UserControl^ userControl);
				Platform::String^ GetResourceString(Platform::String^ str);
				Platform::String^ GetAnnotTypeString(AnnotationType type);
				void LoadAnnotAppBar(AppBarType appbarType);
				void LoadAnnotAppBarEdit(AppBarType appbarType);
				void LoadAppBarByType(AppBarType appbarType);
				void ShowDrawSignatureControl();
				Windows::Foundation::Point GetSignPoint();
				void HideSignatureCanvas();
				void HideDrawSignatureControl();
				void SetAnnotPropertyToolIsEnable(bool isVisible);
				void SetEraserPageIndex(int pageIndex);
				bool GetAnnotCanvasHasChild();
				TypeWriterRange^ GetTypeWriterContent(int m_pageIndex, Platform::String^ content, double fontsize, Windows::UI::Xaml::Media::FontFamily^ fontfamily, Windows::UI::Text::FontStyle fontstyle, Windows::UI::Text::FontWeight fontweight, double minWidth, double maxWidth);
				void SetFreeTextPropertyShow(bool isShow, Platform::String^ title);
				Windows::UI::Xaml::Media::PointCollection^ RemoveDrawPolyLine();
				void CreateNewPolyLineToCanvas(Windows::UI::Xaml::Shapes::Polyline^ polyLine);
				Windows::Foundation::Collections::IVector<Windows::UI::Xaml::Media::PointCollection^>^ GetPolyLinePoints();
				void UpdateAnnotToolUnReDoControl();
				void SetFormControlTappedPoint(Windows::Foundation::Point pt);
				bool IsWidgetAnnotPopupOpend();
				bool IsFormControlCanSelect(IBaseAnnot^ pAnnot);
				bool IsValidFormControlPtr(IBaseAnnot^ pAnnot);
				void ExitSearchingModel();
				void SetMainAppBarVisible(bool isVisible);
				void InitStampAppBar();
				void SetNoteStylePageIsActive(bool isActive);
				void InitCurNoteIcon();
				bool IsAnnotAppBarTipShow();
				void SetAnnotPropertyTipShow(bool isShow);
				void HidePopupWindow();
				void ShowUndoRedoBarInPhone();
				void HideUndoRedoBarInPhone();
				void ShowSearchControl();
				void HideSearchControl();
				bool IsPhonePropertyPopupOpen();
				void SetPhonePropertyPopupClosed();
				void FocusFormControlHiddenTextBox();
				void LostFocusFormControlHiddenTextBox();
				void UpdateFormControlHiddenTextBoxPosition();
				void OnTappedTick(Platform::Object ^sender, Platform::Object ^args);
				void OnTick(Platform::Object ^sender, Platform::Object ^args);
				void OnPointerWheelScaleOperationTick(Platform::Object ^sender, Platform::Object ^args);
				void OnAutoSaveFileTick(Platform::Object ^sender, Platform::Object ^args);
				void OnCopyFileTick(Platform::Object ^sender, Platform::Object ^args);
				void StartZoomInOutTimer();
				void StopZoomInOutTimer();
				void LoadViewByViewMode(foxit::rdk::LayoutMode viewmode);
				bool LoadPrevViewMode();

				void AnnotHighlightFlicker(Windows::Foundation::Rect rc);

			private:
				concurrency::task<void> SaveFileAsync();
				concurrency::task<void> SaveAsFileAsync();
				concurrency::task<void> SaveAsFileWithSelectDialogAsync();
				concurrency::task<void> CopyFileStreamData(Windows::Storage::StorageFile^ dstFile, Windows::Storage::StorageFile^ srcFile);
				concurrency::task<void> DeleteTemporaryFile();
				concurrency::task<void> CopyCurrentFileAsync();
				bool GetCanModifyForm();
				FormFillTimerWrapper^ GetTimerById(UINT_PTR nTimerID);
				void ResetPropertyPopupLocation(bool isShowMainBorad);
				void OnAnnotPropertyAppBarLoaded(Platform::Object ^sender, Windows::UI::Xaml::RoutedEventArgs ^e);
				void OnOpenPdfFileEvent(foxit::rdk::PDFViewCtrl ^sender, foxit::rdk::DocEventArgs^ e);
				void OnClosePdfFileEvent(foxit::rdk::PDFViewCtrl ^sender, foxit::rdk::DocEventArgs^ e);
				void OnPageLoadedEvent(foxit::rdk::PDFViewCtrl^ sender, foxit::rdk::PageLoadedEventArgs^ e);
				void OnPageUnloadedEvent(foxit::rdk::PDFViewCtrl ^sender, foxit::rdk::PageUnloadedEventArgs ^e);
				void OnPageVisibleEvent(foxit::rdk::PDFViewCtrl ^sender, foxit::rdk::PageVisibleEventArgs ^e);
				void OnPageInvisibleEvent(foxit::rdk::PDFViewCtrl ^sender, foxit::rdk::PageInvisibleEventArgs ^e);
				void OnPageChangedEvent(foxit::rdk::PDFViewCtrl ^sender, foxit::rdk::PageChangedEventArgs ^e);
				void OnDrawAnnotEvent(foxit::rdk::PDFViewCtrl^ sender, foxit::rdk::RenderPageEventArgs^ e);
				void OnPageLayoutModeChangedEvent(foxit::rdk::PDFViewCtrl ^sender, foxit::rdk::PageLayoutModeChangedEventArgs ^e);

				Windows::Foundation::Point SetNoteCreateOrSave();

			private:
				Platform::Collections::Vector<FormFillTimerWrapper^>^ m_FormFillTimers;
				FormOperationBarControl^ m_pFormOperationBarControl;
				Windows::Foundation::Point m_FormControlTappedPt;
				FoxitUIHostedContorl^ m_pHostedControl;
				foxit::rdk::PDFViewCtrl^ m_control;
				PDFTextManager^ m_textManager;
				MainPageAppBar^ m_MainAppBar;
				foxit::pdf::interform::Filler^ m_FormFiller;
				foxit::pdf::interform::Form^ m_pPDFForm;
				PDFFormAbstration^ m_pFormAbs;
				PDFFormFillerManagement^ m_FormFillerMgr;
				AnnotationManager^ m_AnnotMgr;
				bool m_isDesktopPlatform;
				Windows::UI::Xaml::DispatcherTimer^ m_BottomBarVisibleTimer;
				ProgressBarFactory^	m_pProgressBar;
				Windows::Foundation::Point sigPt;
				//For typewriter.
				double height;
				int linecount;

				Platform::String^ textStr;
				Platform::String^ copyStr;
				bool m_bIsSaveDocReadStateSetting;
				Windows::UI::Xaml::Shapes::Polyline^ m_curpolyLine;
				bool m_isPressed;
				bool m_isPressedMove;
				Windows::UI::Xaml::Media::CompositeTransform^ m_LayoutGridTransform;
				Windows::Foundation::Rect graphminRect;
				Windows::Foundation::Rect graphAnnotInPageRange;
				Windows::Foundation::Point deviceLineStartPt;
				Windows::Foundation::Point deviceLineEndPt;
				bool  m_isAnnotMoveEnd;
				Windows::Foundation::Point positionChangePt;
				bool m_IsRightMenuAdd;
				bool m_IsDefaultSignature;
				bool m_IsHandleBackPressed;
				Windows::UI::Xaml::Controls::Primitives::Popup^ m_PoppupTextMenu;
				Windows::UI::Xaml::Controls::Primitives::Popup^ m_PoppupNoteWindow;
				Windows::UI::Xaml::Controls::Primitives::Popup^ m_NumberPopup;
				Windows::UI::Xaml::Controls::Primitives::Popup^ m_ShowNumberPopup;
				PDFPagePoint^ m_holdingPoint;
				PixelSource^ m_picSource;
				bool m_isContinueEditAnnot;
				bool m_isSDToLocalFile;
				bool m_isSDFile;
				int m_curRotateValue;
				ContextMenuUserControl^ menuControl;
				Windows::Foundation::Rect m_showTextMinRect;
				Windows::Foundation::Point m_showTextPoint;
				Windows::Foundation::Rect pageviewRect;
				Windows::Foundation::EventRegistrationToken m_TextPopupLoadedEventToken;
				double m_TextMenuTipHoffset;
				Windows::Foundation::EventRegistrationToken m_propertyPopupLoadedEventToken;
				AnnotPropertyAppBar^ control;
				Windows::UI::Xaml::Controls::Primitives::Popup^ propertyPopup;
				FindTextControl^ m_FindTextControl;
				bool m_bIsAppBarInSearchMode;
				int m_EraserPageIndex;
				bool m_SavingFile;
				Windows::Foundation::Collections::IVector<NoteIcoObj^>^ notIconCollections;
				bool m_isMovingAnnot;
				Windows::Storage::StorageFile^ newFile;
				Windows::Storage::StorageFile^ m_FileActivatedFile;
				bool m_isExitPageView;
				bool m_isOpenFileComplete;
				Platform::String^ m_changedFileName;
				Windows::Storage::StorageFile^ m_SwapTemperFile;
				Windows::UI::Xaml::DispatcherTimer^ m_TappedTimer;
				double m_scale;
				bool m_isHasBookmark;
				bool m_isSaveAsFile;
				Windows::Foundation::Rect m_WindowsRect;
				SelectOperateType m_SelectOperateType;
				AnnotCreateType m_AnnotCreateType;
				AppBarType m_CurAppBarType;
				bool m_isShowAnnot;
				bool m_CtrlKeyDown;
				Windows::Foundation::EventRegistrationToken m_WindowSizeChangedEventToken, m_AcceleratorKeyActivatedEventToken;
				Windows::Foundation::EventRegistrationToken m_KeyDownToken, m_KeyUpToken;
				bool m_ManipulationStart;
				Windows::UI::Xaml::DispatcherTimer^ m_PointerWheelScaleTimer;
				Windows::Foundation::Point m_PointerWheelScaleOriPoint;
				int m_PointerPrevMouseWheelDelta;
				bool m_IsContinuousOperate;
				Windows::UI::Xaml::DispatcherTimer^ m_AutoSaveFileTimer;
				Windows::UI::Xaml::DispatcherTimer^ m_CopyFileTimer;
				bool m_IsAutoSaveFileFlag;
				concurrency::cancellation_token_source m_CopyFileCts;
				Windows::Foundation::EventRegistrationToken m_HardwareButtonsBackPressedEventToken, m_SystemNavigationBackRequestedEventToken;
				foxit::rdk::LayoutMode m_docViewMode;
				foxit::rdk::LayoutMode m_PrevViewMode;

				Windows::UI::Xaml::Controls::Grid^ m_FlyoutPlacement;

				foxit::common::ErrorCode m_OpenStateCode;

			internal:
				DocUndo^ m_docUndo;
			};
		}
	}
}
