/**
 * Copyright (C) 2003-2018, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit Mobile PDF SDK to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */


#include "pch.h"
#include "Undo.h"
#include "DocUndo.h"

#include <assert.h>

using namespace foxit::rdk::uiextensions;
using namespace foxit::rdk;
using namespace Platform;

static IUndo^ _Current;

IUndo^ UndoFactory::GetUndoInstance()
{
	if (_Current == nullptr)
	{
		_Current = ref new DefaultUndo();
	}

	return _Current;
}

DefaultUndo::DefaultUndo()
{
	m_bEnabled = false;
	m_UndoItems = ref new Platform::Collections::Vector<IUndoItem^>();
}

bool foxit::rdk::uiextensions::DefaultUndo::IsUndoRedoing(UIExtensionsManager^ pDocument)
{
	DocUndo^ pUndo = dynamic_cast<DocUndo^>(pDocument->m_docUndo);

	return pUndo ? pUndo->IsWorking() : false;
}

bool foxit::rdk::uiextensions::DefaultUndo::CanUndo(UIExtensionsManager^ pDocument)
{
	DocUndo^ pUndo = dynamic_cast<DocUndo^>(pDocument->m_docUndo);

	return pUndo ? pUndo->CanUndo() : false;
}

bool foxit::rdk::uiextensions::DefaultUndo::CanRedo(UIExtensionsManager^ pDocument)
{
	DocUndo^ pUndo = dynamic_cast<DocUndo^>(pDocument->m_docUndo);

	return pUndo ? pUndo->CanRedo() : false;
}

void foxit::rdk::uiextensions::DefaultUndo::Undo(UIExtensionsManager^ pDocument)
{
	DocUndo^ pUndo = dynamic_cast<DocUndo^>(pDocument->m_docUndo);

	if (pUndo && pUndo->CanUndo())
	{
		BeforeUndo(this, pDocument);
		pUndo->Undo();
		AfterUndo(this, pDocument);
	}
}

void foxit::rdk::uiextensions::DefaultUndo::Redo(UIExtensionsManager^ pDocument)
{
	DocUndo^ pUndo = dynamic_cast<DocUndo^>(pDocument->m_docUndo);

	if (pUndo && pUndo->CanRedo())
	{
		BeforeUndo(this, pDocument);
		pUndo->Redo();
		AfterUndo(this, pDocument);
	}
}

Platform::String^ DefaultUndo::GetUndoTitle()
{
	return ref new Platform::String(L"");
}

Platform::String^ DefaultUndo::GetRedoTitle()
{
	return ref new Platform::String(L"");
}

Platform::String^ foxit::rdk::uiextensions::DefaultUndo::GetUndoDescription(UIExtensionsManager^ pDocument)
{
	return ref new Platform::String(L"");
}

Platform::String^ foxit::rdk::uiextensions::DefaultUndo::GetRedoDescription(UIExtensionsManager^ pDocument)
{
	return ref new Platform::String(L"");
}

void DefaultUndo::Enable(bool bEnabled)
{
	m_bEnabled = bEnabled;
}

bool DefaultUndo::IsEnabled()
{
	return m_bEnabled;
}

void foxit::rdk::uiextensions::DefaultUndo::AddUndoItem(UIExtensionsManager^ pDocument, IUndoItem^ pUndoItem)
{
	DocUndo^ pUndo = dynamic_cast<DocUndo^>(pDocument->m_docUndo);

	assert(!pUndo->IsWorking());

	pUndo->AddItem(pUndoItem);

	OperaListChanged(this, pUndoItem);
}

void foxit::rdk::uiextensions::DefaultUndo::DeleteUndoItem(UIExtensionsManager^ pDocument, IUndoItem^ pUndoItem)
{
	DocUndo^ pUndo = dynamic_cast<DocUndo^>(pDocument->m_docUndo);

	assert(!pUndo->IsWorking());

	pUndo->DeleteItem(pUndoItem);

	OperaListChanged(this, pUndoItem);
}

void DefaultUndo::Reset(UIExtensionsManager^ pDocument)
{
	DocUndo^ pUndo = dynamic_cast<DocUndo^>(pDocument->m_docUndo);

	assert(!pUndo->IsWorking());

	pUndo->Reset();

	OperaListChanged(this, nullptr);
}
