﻿/**
 * Copyright (C) 2003-2018, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit Mobile PDF SDK to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */


#include "pch.h"
#include "UndoAppBarPhoneControl.xaml.h"
#include "../MainPageAppBar.xaml.h"
#include "../FloatWindow/AnnotPropertyPhone.xaml.h"
#include "../FloatWindow/AnnotStampUserControl.xaml.h"

using namespace foxit::rdk::uiextensions;
using namespace foxit::rdk;

using namespace Platform;
using namespace Windows::Foundation;
using namespace Windows::Foundation::Collections;
using namespace Windows::UI::Xaml;
using namespace Windows::UI::Xaml::Controls;
using namespace Windows::UI::Xaml::Controls::Primitives;
using namespace Windows::UI::Xaml::Data;
using namespace Windows::UI::Xaml::Input;
using namespace Windows::UI::Xaml::Media;
using namespace Windows::UI::Xaml::Navigation;

// The User Control item template is documented at http://go.microsoft.com/fwlink/?LinkId=234236

UndoAppBarPhoneControl::UndoAppBarPhoneControl()
{
	InitializeComponent();

	this->Loaded += ref new Windows::UI::Xaml::RoutedEventHandler(this, &foxit::rdk::uiextensions::UndoAppBarPhoneControl::OnLoaded);
	this->Unloaded += ref new Windows::UI::Xaml::RoutedEventHandler(this, &foxit::rdk::uiextensions::UndoAppBarPhoneControl::OnUnloaded);
}

MainPageAppBar^ UndoAppBarPhoneControl::MainAppBar::get()
{
	return m_MainAppBarWeakRef.Resolve<MainPageAppBar>();
}

void UndoAppBarPhoneControl::UpdateUndoControlState()
{
	if (MainAppBar->UIExtMgr->CreateTypeOfAnnot != AnnotCreateType::FreeText)
	{
		auto pUndo = UndoFactory::GetUndoInstance();
		if (MainAppBar && MainAppBar->View->GetDoc())
		{
			RadioUndo->IsEnabled = pUndo->CanUndo(MainAppBar->UIExtMgr);
			RadioRedo->IsEnabled = pUndo->CanRedo(MainAppBar->UIExtMgr);
		}
		else
		{
			RadioUndo->IsEnabled = false;
			RadioRedo->IsEnabled = false;
		}
	}
}

void foxit::rdk::uiextensions::UndoAppBarPhoneControl::OnLoaded(Platform::Object ^sender, Windows::UI::Xaml::RoutedEventArgs ^e)
{
	auto _pageView = GetVisualParent<PDFViewCtrl^>(this);
	auto _mainAppbar = GetVisualChild<MainPageAppBar^>(_pageView);
	if (_mainAppbar)
		m_MainAppBarWeakRef = _mainAppbar;

	UIExtensionsManager^ uiMgr = (UIExtensionsManager^)MainAppBar->View->ExtensionsManager;
	if (uiMgr->getAnnotMgr()->GetAnnotSetting()->GetIsFirstUserTip())
	{
		_mainAppbar->StopHideTimeTick();
		uiMgr->SetAnnotPropertyTipShow(true);
		uiMgr->getAnnotMgr()->GetAnnotSetting()->SetIsFirstUserTip(false);
		uiMgr->getAnnotMgr()->GetAnnotSetting()->SavePropertySetting();
	}

	propertyPopup = ref new Windows::UI::Xaml::Controls::Primitives::Popup();
	if (CreateAnnotIsContinueOpera(uiMgr->CreateTypeOfAnnot))
	{
		Windows::UI::Xaml::Media::SolidColorBrush^ selectcolor = ref new Windows::UI::Xaml::Media::SolidColorBrush(Utility::ConvertFromString("#FFD8D8D8"));
		appBarContinueOpera->Background = selectcolor;
	}

	UpdateUndoControlState();

	auto pUndo = UndoFactory::GetUndoInstance();
	m_AfterUndoEventToken = pUndo->AfterUndo += ref new foxit::rdk::uiextensions::UndoEventHandler(this, &foxit::rdk::uiextensions::UndoAppBarPhoneControl::OnAfterUndo);
	m_AfterRedoEventToken = pUndo->AfterRedo += ref new foxit::rdk::uiextensions::UndoEventHandler(this, &foxit::rdk::uiextensions::UndoAppBarPhoneControl::OnAfterRedo);
	m_OperaListChangedEventToken = pUndo->OperaListChanged += ref new foxit::rdk::uiextensions::UndoOperaListChangedEventHandler(this, &foxit::rdk::uiextensions::UndoAppBarPhoneControl::OnOperaListChanged);
}


void foxit::rdk::uiextensions::UndoAppBarPhoneControl::OnUnloaded(Platform::Object ^sender, Windows::UI::Xaml::RoutedEventArgs ^e)
{
	auto pUndo = UndoFactory::GetUndoInstance();
	if (m_AfterUndoEventToken.Value)
	{
		pUndo->AfterUndo -= m_AfterUndoEventToken;
		m_AfterUndoEventToken.Value = 0;
	}
	if (m_AfterRedoEventToken.Value)
	{
		pUndo->AfterRedo -= m_AfterRedoEventToken;
		m_AfterRedoEventToken.Value = 0;
	}
	m_MainAppBarWeakRef = nullptr;
}

void foxit::rdk::uiextensions::UndoAppBarPhoneControl::RadioUndo_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{
	UIExtensionsManager^ uiMgr = (UIExtensionsManager^)MainAppBar->View->ExtensionsManager;
	if (MainAppBar && uiMgr->getAnnotMgr()->GetSelectedAnnot())
	{
		uiMgr->ExitAnnotSelect();
		uiMgr->getAnnotMgr()->SetPdfSelectAnnotNull();
		uiMgr->SetAnnotCanvasVisible(false);
	}

	if (uiMgr->CreateTypeOfAnnot == AnnotCreateType::FreeText)
	{
		uiMgr->CreateGraphAnnotBySliderPage();
		if (!uiMgr->getAnnotMgr()->GetAnnotSetting()->GetFreeTextIsContinueOpera())
			uiMgr->ResetAnnotToolAppBar();
	}
	auto pUndo = UndoFactory::GetUndoInstance();

	if (MainAppBar && MainAppBar->View->GetDoc())
		pUndo->Undo(MainAppBar->UIExtMgr);
}


void foxit::rdk::uiextensions::UndoAppBarPhoneControl::RadioRedo_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{
	UIExtensionsManager^ uiMgr = (UIExtensionsManager^)MainAppBar->View->ExtensionsManager;
	if (MainAppBar && uiMgr->getAnnotMgr()->GetSelectedAnnot())
	{
		uiMgr->getAnnotMgr()->SetPdfSelectAnnotNull();
		uiMgr->SetAnnotCanvasVisible(false);
		uiMgr->HidePopupWindow();
	}

	if (uiMgr->CreateTypeOfAnnot == AnnotCreateType::FreeText)
	{
		uiMgr->CreateGraphAnnotBySliderPage();
		if (!uiMgr->getAnnotMgr()->GetAnnotSetting()->GetFreeTextIsContinueOpera())
			uiMgr->ResetAnnotToolAppBar();
	}
	auto pUndo = UndoFactory::GetUndoInstance();

	if (MainAppBar && MainAppBar->View->GetDoc())
		pUndo->Redo(MainAppBar->UIExtMgr);
}


void foxit::rdk::uiextensions::UndoAppBarPhoneControl::OnAfterUndo(Platform::Object ^sender, Platform::Object ^args)
{
	UpdateUndoControlState();
}


void foxit::rdk::uiextensions::UndoAppBarPhoneControl::OnAfterRedo(Platform::Object ^sender, Platform::Object ^args)
{
	UpdateUndoControlState();
}

void foxit::rdk::uiextensions::UndoAppBarPhoneControl::OnOperaListChanged(Platform::Object ^sender, Platform::Object ^args)
{
	UpdateUndoControlState();
}

void foxit::rdk::uiextensions::UndoAppBarPhoneControl::OnAppBarShow(Platform::Object ^sender, Platform::Object ^args)
{
	if (this->Visibility == Windows::UI::Xaml::Visibility::Collapsed)
	{
		this->Visibility = Windows::UI::Xaml::Visibility::Visible;
	}

	UpdateUndoControlState();
}


void foxit::rdk::uiextensions::UndoAppBarPhoneControl::OnAppBarHide(Platform::Object ^sender, Platform::Object ^args)
{
	if (this->Visibility == Windows::UI::Xaml::Visibility::Visible)
	{
		this->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
	}
}


void foxit::rdk::uiextensions::UndoAppBarPhoneControl::propertyBtn_Tapped(Platform::Object^ sender, Windows::UI::Xaml::Input::TappedRoutedEventArgs^ e)
{
	AnnotPropertyPhone^ propertyControl = ref new AnnotPropertyPhone(MainAppBar->View);
	propertyControl->InitControlSize();
	propertyPopup->Child = propertyControl;
	propertyPopup->HorizontalOffset = 0;
	propertyPopup->VerticalOffset = 0;
	propertyPopup->IsOpen = true;

	MainAppBar->HideAppBar();
}


void foxit::rdk::uiextensions::UndoAppBarPhoneControl::appBarContinueOpera_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{
	AppBarButton^ button = dynamic_cast<AppBarButton^>(sender);
	if (button)
	{
		Windows::UI::Xaml::Media::SolidColorBrush^ selectcolor = ref new Windows::UI::Xaml::Media::SolidColorBrush(Utility::ConvertFromString("#FFD8D8D8"));
		Windows::UI::Xaml::Media::SolidColorBrush^ normalcolor = ref new Windows::UI::Xaml::Media::SolidColorBrush(Windows::UI::Colors::Transparent);

		UIExtensionsManager^ uiMgr = (UIExtensionsManager^)MainAppBar->View->ExtensionsManager;
		bool isContinue = CreateAnnotIsContinueOpera(uiMgr->CreateTypeOfAnnot);
		if (isContinue)
		{
			button->Background = normalcolor;
		}
		else
		{
			button->Background = selectcolor;
		}

		AnnotCreateType type = uiMgr->CreateTypeOfAnnot;
		auto _AnnotSetting = uiMgr->getAnnotMgr()->GetAnnotSetting();

		switch (type)
		{
		case AnnotCreateType::HightLight:
			_AnnotSetting->SetHightLightIsContinueOpera(!isContinue);
			break;
		case AnnotCreateType::AreaHighLight:
			_AnnotSetting->SetAreaHightLightIsContinueOpera(!isContinue);
			break;
		case AnnotCreateType::StrikeOut:
			_AnnotSetting->SetStrikeOutIsContinueOpera(!isContinue);
			break;
		case AnnotCreateType::UnderLine:
			_AnnotSetting->SetUnderLineIsContinueOpera(!isContinue);
			break;
		case AnnotCreateType::Squiggly:
			_AnnotSetting->SetSquigglyIsContinueOpera(!isContinue);
			break;
		case AnnotCreateType::Square:
			_AnnotSetting->SetSquareIsContinueOpera(!isContinue);
			break;
		case AnnotCreateType::Circle:
			_AnnotSetting->SetCircleIsContinueOpera(!isContinue);
			break;
		case AnnotCreateType::Line:
			_AnnotSetting->SetLineIsContinueOpera(!isContinue);
			break;
		case AnnotCreateType::Arrow:
			_AnnotSetting->SetArrowIsContinueOpera(!isContinue);
			break;
		case AnnotCreateType::Stamp:
			_AnnotSetting->SetStampeIsContinueOpera(!isContinue);
			break;
		case AnnotCreateType::Text:
			_AnnotSetting->SetTextIsContinueOpera(!isContinue);
			break;
		case AnnotCreateType::FreeText:
			_AnnotSetting->SetFreeTextIsContinueOpera(!isContinue);
			break;
		case AnnotCreateType::Caret:
			_AnnotSetting->SetInsertTextIsContinueOpera(!isContinue);
			break;
		case AnnotCreateType::ReplaceText:
			_AnnotSetting->SetReplaceTextIsContinueOpera(!isContinue);
			break;
		default:
			break;
		}
	}
}

void UndoAppBarPhoneControl::ShowAnnotEditControl(bool isShow)
{
	Windows::UI::Xaml::Media::SolidColorBrush^ normalcolor = ref new Windows::UI::Xaml::Media::SolidColorBrush(Windows::UI::Colors::Transparent);
	if (isShow)
	{
		Windows::UI::Xaml::Media::SolidColorBrush^ selectcolor = ref new Windows::UI::Xaml::Media::SolidColorBrush(Utility::ConvertFromString("#FFD8D8D8"));
		propertyBtn->IsEnabled = true;
		UIExtensionsManager^ uiMgr = (UIExtensionsManager^)MainAppBar->View->ExtensionsManager;
		if (uiMgr->CreateTypeOfAnnot != AnnotCreateType::Eraser && uiMgr->CreateTypeOfAnnot != AnnotCreateType::Pencil)
		{
			appBarContinueOpera->IsEnabled = true;
			if ((CreateAnnotIsContinueOpera(uiMgr->CreateTypeOfAnnot)))
			{
				appBarContinueOpera->Background = selectcolor;
			}
			else
			{
				appBarContinueOpera->Background = normalcolor;
			}
		}
		else
		{
			appBarContinueOpera->IsEnabled = false;
		}
	}
	else
	{
		propertyBtn->IsEnabled = false;
		appBarContinueOpera->IsEnabled = false;
		appBarContinueOpera->Background = normalcolor;
	}
}

void UndoAppBarPhoneControl::SetPropertyPopupColsed()
{
	if (propertyPopup->IsOpen)
		propertyPopup->IsOpen = false;
}

bool UndoAppBarPhoneControl::CreateAnnotIsContinueOpera(AnnotCreateType type)
{
	bool isContinue = false;
	UIExtensionsManager^ uiMgr = (UIExtensionsManager^)MainAppBar->View->ExtensionsManager;
	auto _AnnotSetting = uiMgr->getAnnotMgr()->GetAnnotSetting();
	switch (type)
	{
	case AnnotCreateType::HightLight:
		isContinue = _AnnotSetting->GetHightLightIsContinueOpera();
		break;
	case AnnotCreateType::AreaHighLight:
		isContinue = _AnnotSetting->GetAreaHightLightIsContinueOpera();
		break;
	case AnnotCreateType::StrikeOut:
		isContinue = _AnnotSetting->GetStrikeOutIsContinueOpera();
		break;
	case AnnotCreateType::UnderLine:
		isContinue = _AnnotSetting->GetUnderLineIsContinueOpera();
		break;
	case AnnotCreateType::Squiggly:
		isContinue = _AnnotSetting->GetSquigglyIsContinueOpera();
		break;
	case AnnotCreateType::Square:
		isContinue = _AnnotSetting->GetSquareIsContinueOpera();
		break;
	case AnnotCreateType::Circle:
		isContinue = _AnnotSetting->GetCircleIsContinueOpera();
		break;
	case AnnotCreateType::Line:
		isContinue = _AnnotSetting->GetLineIsContinueOpera();
		break;
	case AnnotCreateType::Arrow:
		isContinue = _AnnotSetting->GetArrowIsContinueOpera();
		break;
	case AnnotCreateType::Stamp:
		isContinue = _AnnotSetting->GetStampIsContinueOpera();
		break;
	case AnnotCreateType::Text:
		isContinue = _AnnotSetting->GetTextIsContinueOpera();
		break;
	case AnnotCreateType::FreeText:
		isContinue = _AnnotSetting->GetFreeTextIsContinueOpera();
		break;
	case AnnotCreateType::Caret:
		isContinue = _AnnotSetting->GetInsertTextIsContinueOpera();
		break;
	case AnnotCreateType::ReplaceText:
		isContinue = _AnnotSetting->GetReplaceTextIsContinueOpera();
		break;
	default:
		break;
	}
	return isContinue;
}

void foxit::rdk::uiextensions::UndoAppBarPhoneControl::appBarStampIcon_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{
	AnnotPropertyPhone^ propertyControl = ref new AnnotPropertyPhone(MainAppBar->View);
	propertyControl->InitControlSize();
	propertyPopup->Child = propertyControl;
	propertyPopup->HorizontalOffset = 0;
	propertyPopup->VerticalOffset = 0;
	propertyPopup->IsOpen = true;

	MainAppBar->HideAppBar();
}

void UndoAppBarPhoneControl::UpdatePropertyVisible()
{
	UIExtensionsManager^ uiMgr = (UIExtensionsManager^)MainAppBar->View->ExtensionsManager;
	if (uiMgr->CreateTypeOfAnnot == AnnotCreateType::Stamp)
	{
		propertyBtn->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
		appBarStampIcon->Visibility = Windows::UI::Xaml::Visibility::Visible;
	}
	else
	{
		propertyBtn->Visibility = Windows::UI::Xaml::Visibility::Visible;
		appBarStampIcon->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
	}
}

void UndoAppBarPhoneControl::HideUnRedoControl()
{
	gridUnRedoPhone->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
}

void UndoAppBarPhoneControl::ShowUnRedoControl()
{
	gridUnRedoPhone->Visibility = Windows::UI::Xaml::Visibility::Visible;
}

bool UndoAppBarPhoneControl::ISPropertyPopupIsOpen()
{
	return propertyPopup->IsOpen;
}

void UndoAppBarPhoneControl::SetFreeTextPropertyIsShow(bool isShow, Platform::String^ title)
{
	if (propertyPopup->IsOpen)
	{
		AnnotPropertyPhone^ propertyControl = dynamic_cast<AnnotPropertyPhone^>(propertyPopup->Child);
		if (propertyControl)
		{
			propertyControl->SetFreeTextPropertyIsShow(isShow, title);
		}
	}
}

void UndoAppBarPhoneControl::SetUnRedoIsEnabled(bool isEnable)
{
	if (!isEnable)
	{
		RadioUndo->IsEnabled = isEnable;
		RadioRedo->IsEnabled = isEnable;
	}
	else
	{
		UpdateUndoControlState();
	}
}
