/**
 * Copyright (C) 2003-2018, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit Mobile PDF SDK to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */


#pragma once

namespace foxit
{
	namespace rdk
	{
		namespace uiextensions
		{

#define APPSETTINGKEY	L"AppSetting"
#define ANNOTSETTINGKEY L"AnnotProperty"

			public enum class AppBarType
			{
				MainAppBar = 0,
				MainSingleModeAppBar,
				MainContinueModeAppBar,
				TextMarkBar,
				AnnotPropertyBar,
				SearchBar,
				AnnotColorProperty,
				AnnotOpacityProperty,
				ViewModeAppBar,
				AnnotAppBar,
				ReflowViewAppBar,
				FormOperationAppbar,
				None
			};

			public enum class SelectOperateType
			{
				Hand,
				SelectText,
				SelectAnnot
			};

			public enum class AnnotCreateType
			{
				None,
				HightLight,
				AreaHighLight,
				Comment,
				UnderLine,
				StrikeOut,
				Square,
				Circle,
				Line,
				Arrow,
				Pencil,
				Squiggly,
				Text,
				Eraser,
				Stamp,
				FreeText,
				Caret,
				ReplaceText,
				Signature
			};

			public enum class DocumentEncryptType
			{
				NOENCRYPT,
				PASSWORD,
				CERTIFICATE,
				FOXITDRM,
				CUSTOM,
				RMS
			};

			public enum class SignType
			{
				SignIn,
				SignUp
			};

			typedef void* FS_LPVOID;
			typedef const void* FS_LPCVOID;
			typedef bool FS_BOOL;
			typedef unsigned int FS_DWORD;
			typedef int FS_RESULT;
			typedef int FS_INT32;
			typedef float FS_FLOAT;
			typedef unsigned int FS_ARGB;
			typedef unsigned char FS_BYTE;

			/** @brief	Success. */
#define FSCRT_ERRCODE_SUCCESS					0
/** @brief	Common error for any kind. */
#define FSCRT_ERRCODE_ERROR						-1
/** @brief	Invalid Foxit PDF SDK manager. */
#define FSCRT_ERRCODE_INVALIDMANAGER			-2
/** @brief	Invalid module or module uninitialized. */
#define FSCRT_ERRCODE_INVALIDMODULE				-3
/** @brief	The function cannot be recovered. */
#define FSCRT_ERRCODE_UNRECOVERABLE				-4
/** @brief	Memory out of memory. */
#define FSCRT_ERRCODE_OUTOFMEMORY				-5
/** @brief	Incorrect password. */
#define FSCRT_ERRCODE_PASSWORD					-6
/** @brief	Data format error. */
#define FSCRT_ERRCODE_FORMAT					-7
/** @brief	File access error, cannot read or write file data. */
#define FSCRT_ERRCODE_FILE						-8
/** @brief	Parameter error. */
#define FSCRT_ERRCODE_PARAM						-9
/** @brief	License authorization error. */
#define FSCRT_ERRCODE_INVALIDLICENSE			-10
/** @brief	Callback functions in handler are error. */
#define FSCRT_ERRCODE_HANDLER					-11
/** @brief	Unknown security handler or cannot find availably one. */
#define FSCRT_ERRCODE_UNKNOWNSECURITYHANDLER	-12
/** @brief	Incorrect certificate or its data error. */
#define FSCRT_ERRCODE_INVALIDCERTIFICATE		-13
/** @brief	Data cannot be found. */
#define FSCRT_ERRCODE_NOTFOUND					-14
/** @brief	Invalid object type. */
#define FSCRT_ERRCODE_INVALIDTYPE				-15
/** @brief	Some features are not implemented, cannot be used, usually for user-side callback functions. */
#define FSCRT_ERRCODE_UNSUPPORTED				-16
/** @brief	Contents are not parsed. */
#define FSCRT_ERRCODE_NOTPARSED					-17
/** @brief	State is error or unknown. */
#define FSCRT_ERRCODE_UNKNOWNSTATE				-18
/** @brief	Buffer is overflow, need more space. */
#define FSCRT_ERRCODE_BUFFEROVERFLOW			-19
/** @brief	Data or values conflict. */
#define FSCRT_ERRCODE_CONFLICT					-20
/** @brief	Data not ready. */
#define FSCRT_ERRCODE_DATANOTREADY				-21
/** @brief	Memory is rebuilt. */
#define FSCRT_ERRCODE_MEMORYREBUILT				-22
/** @brief	Unknown handler or cannot find availably one. */
#define FSCRT_ERRCODE_UNKNOWNHANDLER			-23
/**
* @brief	To be continued.
*
* @note	It is used under progressive calls, and tells caller to call function ::FSCRT_Progress_Continue again.
*/
#define FSCRT_ERRCODE_TOBECONTINUED				1
/** @brief	Current process has finished, need not do again. */
#define FSCRT_ERRCODE_FINISHED					2
/** @brief	To rollback the application process. */
#define FSCRT_ERRCODE_ROLLBACK					3

/** @brief	Helper macro to test for success on any result value. */
#define FSCRT_SUCCEEDED(result)					((FS_RESULT)(result) >= 0) 
/** @brief	Helper macro to test for failure on any result value. */
#define FSCRT_FAILED(result)					((FS_RESULT)(result) < 0)

		}
	}
}